#include "api_origin_access.h"



/* Set up an AccessControlResponse PDU and sent it to a target */
int send_accessrsp_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error )
{
	DbvAccessControlRsp   *psAccessControlRsp = NULL;
	
	int choice = 0;
	char choice_string[100];

	/* Allocate memory for psAccessControlRsp and fill it in */
        psAccessControlRsp = (DbvAccessControlRsp *) DbvAlloc (sizeof(DbvAccessControlRsp));
        fill_access_response(psAccessControlRsp);
 
	printf("Print AccessControlResponse PDU to screen before sending ? yes (1), no (2): ");
	choice = atoi(gets(choice_string));
	if (choice == 1) {
		printf("-----------------------------------------\n");
		print_accessrsp_fields(stdout, psAccessControlRsp);
		printf("-----------------------------------------\n");
	}

 
        /* Send the AccessControlResponse just filled in to the target */
        if (DbvAccessControlResponse(*assoc_id, psAccessControlRsp, error) == SI_OK) {

		/* Sent PDU to target ok so free up space it used */ 

		SIFreeDbvAccessControlRsp(&psAccessControlRsp);

		return SI_OK;
	} else {
		
		/* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(error);

                SIFreeDbvAccessControlRsp(&psAccessControlRsp);


                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
		return SI_NOTOK;
	}
}



int fill_access_response(DbvAccessControlRsp *psAccessControlRsp)
{

	int choice = 0;
	char choice_string[100];


	char *test_refid = "access control response reference id";
	char *test_simpleform = "access control response simple form";
	char *test_sutrs = "access control response test sutrs";
	char *diag_dbname = "test diag database";
	char *diag_text = "test diag text";
	char *test_otherinfo1 = "first other information test";
	char *test_otherinfo2 = "second other information test";


        /* Compulsory fields... */


       	/* Optional fields... */
 
	/* +1 to allocate space for the \0 */
        psAccessControlRsp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
	fill_dbv_octet_string(psAccessControlRsp->psReferenceId, test_refid);



	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter eSecurityChallengeRspType type: SECT_SimpleForm (1), SECT_ExternallyDefined (2): ");
		choice = (atol(gets(choice_string)));
	}

        psAccessControlRsp->psSecurityChallengeRsp = (DbvSecurityChallengeRsp *) DbvAlloc (sizeof (DbvSecurityChallengeRsp));
			
	switch (choice) {
	case 1:
        	psAccessControlRsp->psSecurityChallengeRsp->eSecurityChallengeRspType = SECT_SimpleForm;
        	psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		fill_dbv_octet_string(psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm, test_simpleform);
		break;
	case 2:
        	psAccessControlRsp->psSecurityChallengeRsp->eSecurityChallengeRspType = SECT_ExternallyDefined;
        	psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		fill_dbv_isosutrs_external(psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo, test_sutrs);

		break;
	}



	/* DbvDiagRec, DbvDefaultDiagFormat V2 */
       	psAccessControlRsp->psDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
	fill_default_diag_rec(psAccessControlRsp->psDiagnostic, ANSI_DANMARC, 655, diag_text, PVIF_Version2, DT_DefaultFormat, NULL);





	/* 2 nodes */
	psAccessControlRsp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);

	psAccessControlRsp->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));

	psAccessControlRsp->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psAccessControlRsp->ppsOtherInformation[2] = NULL;


	fill_dbv_otherinfo_char(psAccessControlRsp->ppsOtherInformation[0], DBV_TRUE, ANSI_UNSTRUCTURED_TEXT_RECORD, 989, test_otherinfo1);
	fill_dbv_otherinfo_char(psAccessControlRsp->ppsOtherInformation[1], DBV_TRUE, ANSI_UNSTRUCTURED_TEXT_RECORD, 999, test_otherinfo2);


}
  

