#include "api_origin_delete.h"


int	send_delreq_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
	 
	DbvDeleteResultSetReq   *psDeleteResultSetReq = NULL;

	int choice = 0;
        char choice_string[100];
 

        /* Allocate memory for psDeleteResultSetReq and fill it in */
        psDeleteResultSetReq = (DbvDeleteResultSetReq *) DbvAlloc (sizeof(DbvDeleteResultSetReq));
        fill_delete_request(psDeleteResultSetReq);
 
 
        printf("Print DeleteResultSetRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_delreq_fields(stdout, psDeleteResultSetReq);
                printf("-----------------------------------------\n");
        }


        /* Send the DeleteResultSetRequest just filled in to the target */
        if (DbvDeleteResultSetRequest(*assoc_id, psDeleteResultSetReq, error) == SI_OK) {
                /* Sent PDU to target ok so free up space it used */

                SIFreeDbvDeleteResultSetReq(&psDeleteResultSetReq);

                return SI_OK;
        } else {
 
                /* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(error);
 
                SIFreeDbvDeleteResultSetReq(&psDeleteResultSetReq);

 
                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
                return SI_NOTOK;
        }
}

	

int fill_delete_request(DbvDeleteResultSetReq *psDeleteResultSetReq)
{
        DbvDeleteFunction test_delfunc = DF_All;
        char *test_reference_id = "delete request reference id";
        int choice = 0;
        char choice_string[100];
 
        /* Compulsory fields */

	/* eDeleteFunction */
	psDeleteResultSetReq->eDeleteFunction = test_delfunc;


	/* Optional fields */

	/* Reference id */
	psDeleteResultSetReq->psReferenceId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
	fill_dbv_octet_string(psDeleteResultSetReq->psReferenceId, test_reference_id);
 
	/* ppsResultSetId  - 3 nodes, 0, 1, \0 */
	psDeleteResultSetReq->ppsResultSetId = (DbvInternationalString **) DbvAlloc (sizeof(DbvInternationalString *) * 3);

	/* Node 0 */
	psDeleteResultSetReq->ppsResultSetId[0] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psDeleteResultSetReq->ppsResultSetId[0], "delete result set result set id 0");

	/* Node 1 */
	psDeleteResultSetReq->ppsResultSetId[1] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psDeleteResultSetReq->ppsResultSetId[1], "delete result set result set id 1");

	psDeleteResultSetReq->ppsResultSetId[2] = NULL;


	/* OtherInformation */
	psDeleteResultSetReq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);
	psDeleteResultSetReq->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psDeleteResultSetReq->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));


	choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter ppsOtherInformation type (for 2 list nodes): char (1), bin (2), ext (3), other (4): ");
                choice = (atol(gets(choice_string)));
        }
 
        switch (choice) {
        case 1:
                fill_deletereq_othinfo_char(psDeleteResultSetReq->ppsOtherInformation[0]);
                fill_deletereq_othinfo_char(psDeleteResultSetReq->ppsOtherInformation[1]);
                break;
        case 2:
                fill_deletereq_othinfo_bin(psDeleteResultSetReq->ppsOtherInformation[0]);
                fill_deletereq_othinfo_bin(psDeleteResultSetReq->ppsOtherInformation[1]) ;
                break;
        case 3:
                fill_deletereq_othinfo_ext(psDeleteResultSetReq->ppsOtherInformation[0]);
                fill_deletereq_othinfo_ext(psDeleteResultSetReq->ppsOtherInformation[1]);
                break;
        case 4:
                fill_deletereq_othinfo_obj(psDeleteResultSetReq->ppsOtherInformation[0]);
                fill_deletereq_othinfo_obj(psDeleteResultSetReq->ppsOtherInformation[1]);
                break;
        }

}



void fill_deletereq_othinfo_char(DbvOtherInformation *psOtherInformation)
{
 
        char oid[100];
        char charinfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 27;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(charinfo, "this is my delete request otherinfo char field");
 
        fill_dbv_otherinfo_char(psOtherInformation, want_infocat, oid, catvalue, charinfo);
}
 
 
 
void fill_deletereq_othinfo_bin(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 33;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my delete request otherinfo bin field");
 
        fill_dbv_otherinfo_bin(psOtherInformation, want_infocat, oid, catvalue, bininfo);
}



void fill_deletereq_othinfo_ext(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
        char sutrs_string[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
        DbvASN1External *external = NULL;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my delete request otherinfo bin field");
        strcpy(sutrs_string, "this is my sutrs string - Delete Request");
 
        fill_dbv_otherinfo_ext(psOtherInformation, want_infocat, oid, catvalue);
 
        /* Set up a sutrs external */
        external = psOtherInformation->uInformation.psExternallyDefinedInfo;
        fill_dbv_sutrs_external(external, sutrs_string);
}
 


void fill_deletereq_othinfo_obj(DbvOtherInformation *psOtherInformation)
{
        char infooid[100];
        char otheroid[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
 
        strcpy(infooid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(otheroid, ANSI_UNSTRUCTURED_TEXT_RECORD);
 
        fill_dbv_otherinfo_obj(psOtherInformation, want_infocat, infooid, catvalue, otheroid);
}
 




