/*  Module Name: %w%
    Version: 2/19/96
    Author: Barry Perryman

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
      This covers the completion of an explain record.

*/

#include "api_origin_explain.h"



/****************************************************/
/* z39.50 explain search */
int
send_srchreq_explain_to_target(DbvAssocId *assoc_id,
		      DbvAssociateState *assoc_state,
		      DbvAssociateResult **assoc_res,
		      DbvReleaseState *rel_state,
		      DbvError **error,
		      char *dbname)
{
	DbvSearchReq    *psSearchReq = NULL;
        int choice = 0;
        char choice_string[100];

        /* Allocate memory for psSearchReq and fill it in */
        psSearchReq = (DbvSearchReq *)DbvAlloc(sizeof(DbvSearchReq));

        fill_search_request_explain(psSearchReq, dbname);

        printf("Print SearchRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_searchreq_fields(stdout, psSearchReq);
                printf("-----------------------------------------\n");
        }

        /* Send the SearchRequest we have just filled in to the target */
        if (DbvSearchRequest(*assoc_id, psSearchReq, error) == SI_OK) {

                SIFreeDbvSearchReq(&psSearchReq);

                return SI_OK;
        } else {
                PrivateErrorHandling(error);

                SIFreeDbvSearchReq(&psSearchReq);


                /* Release buffer, assiearchReq(&psSearchReq);

                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
                return SI_NOTOK;
        }

}


void
fill_search_request_explain(DbvSearchReq *psSearchReq, char *dbname)
{
	Ulong test_ulSmallSetUpperBound = 0;
	Ulong test_ulLargeSetLowerBound = 1;
	Ulong test_ulMediumSetPresentNumber = 0;
	char    *test_psReferenceId = "SearchRequest reference id";
	DbvBool test_fReplaceIndicator = DBV_TRUE;
	DbvObjectIdentifier       test_pcPreferredRecordSyntax = ANSI_EXPLAIN_RECORD;
	char *test_sResultSetName = "default";

	dbname = "IR-Explain-1";

	/* SmallSetUpperBound */
	psSearchReq->ulSmallSetUpperBound = test_ulSmallSetUpperBound;

	/* LargeSetLowerBound */
	psSearchReq->ulLargeSetLowerBound = test_ulLargeSetLowerBound;

	/* MediumSetPresentNumber */
	psSearchReq->ulMediumSetPresentNumber = test_ulMediumSetPresentNumber;

	/* fReplaceIndicator */
	psSearchReq->fReplaceIndicator = test_fReplaceIndicator;

	/* sResultSetName */
	fill_dbv_inter_string(&(psSearchReq->sResultSetName), test_sResultSetName);

	/* fill in the database name */
	psSearchReq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 2);
	psSearchReq->ppsDatabaseNames[0] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psSearchReq->ppsDatabaseNames[0], dbname);
	psSearchReq->ppsDatabaseNames[1] = NULL;

	/* psQuery */
	psSearchReq->psQuery = (DbvQuery *)DbvAlloc(sizeof (DbvQuery));
	fill_srchreq_explain_Query(psSearchReq->psQuery);
}

void
print_pre_search_explain_blurb(void)
{
	printf("\n");
	printf("Please enter an HQL (Hough Query Language) explain query.\n\n");
	printf("Some example queries, as defined in Appendix ATTR, ATR.2:\n");
	printf("(see file api_origin_search_parser.h for a full list of\n");
	printf(" search types)\n");
	printf("\n");
	printf("explaincategory: TargetInfo\n");
	printf("explaincategory: DatabaseInfo\n");
	printf("explaincategory: SchemaInfo\n");
	printf("explaincategory: TagSetInfo\n");
	printf("explaincategory: RecordSyntaxInfo\n");
	printf("explaincategory: AttributeSetInfo\n");
	printf("explaincategory: TermListInfo\n");
	printf("explaincategory: ExtendedServicesInfo\n");
	printf("explaincategory: AttributeDetails\n");
	printf("explaincategory: TermListDetails\n");
	printf("explaincategory: ElementSetDetails\n");
	printf("explaincategory: RetrievalRecordDetails\n");
	printf("explaincategory: SortDetails\n");
	printf("explaincategory: Processing\n");
	printf("explaincategory: VariantSetInfo\n");
	printf("explaincategory: UnitInfo\n");
	printf("explaincategory: CategoryList\n");
	printf("\n");
	printf("proccontext: Access\n");
	printf("proccontext: Search\n");
	printf("\n\n");


}



void
fill_srchreq_explain_Query(DbvQuery *psQuery)
{
	char *rpnoid = ANSI_ATTRIBUTE_SET_EXP_1;
	char hql_query[100];
	DbvRPNStructure *rpnstruct1 = NULL;

	psQuery->eQueryType = QT_Rpn;
	psQuery->uQuery.psRpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
	psQuery->uQuery.psRpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
	memcpy(psQuery->uQuery.psRpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));

	print_pre_search_explain_blurb();

	/* Loop until a valid HQL query is entered */
	while (rpnstruct1 == NULL)
	{
		printf("Enter an HQL query> ");
		/* Get an input line from the user - allow
		 * 99 (max - 1)
		 * characters in the line at most (0-98)
		 */
		if (getline(hql_query, 100))
		{
			rpnstruct1 = form_dbv_rpn(hql_query);
			if (rpnstruct1 == NULL)
			{
				printf("Error in HQL Query\n\n");
			}
		}
	}

	/* At this point there is a valid Dbv RPN query in rpnstruct1 */
	/* Assign structures */
	psQuery->uQuery.psRpnQuery->sRpnStructure = *rpnstruct1;
}



/***********************************************************/
/* CIP style explain search */

int
send_srchreq_explain_cip_to_target(DbvAssocId *assoc_id,
		      DbvAssociateState *assoc_state,
		      DbvAssociateResult **assoc_res,
		      DbvReleaseState *rel_state,
		      DbvError **error,
		      char *dbname)
{
	DbvSearchReq    *psSearchReq = NULL;
        int choice = 0;
        char choice_string[100];

        /* Allocate memory for psSearchReq and fill it in */
        psSearchReq = (DbvSearchReq *)DbvAlloc(sizeof(DbvSearchReq));

        fill_search_request_explain_cip(psSearchReq, dbname);

        printf("Print SearchRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_searchreq_fields(stdout, psSearchReq);
                printf("-----------------------------------------\n");
        }

        /* Send the SearchRequest we have just filled in to the target */
        if (DbvSearchRequest(*assoc_id, psSearchReq, error) == SI_OK) {

                SIFreeDbvSearchReq(&psSearchReq);

                return SI_OK;
        } else {
                PrivateErrorHandling(error);

                SIFreeDbvSearchReq(&psSearchReq);


                /* Release buffer, assiearchReq(&psSearchReq);

                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
                return SI_NOTOK;
        }

}


void
fill_search_request_explain_cip(DbvSearchReq *psSearchReq, char *dbname)
{
	Ulong test_ulSmallSetUpperBound = 0;
	Ulong test_ulLargeSetLowerBound = 1;
	Ulong test_ulMediumSetPresentNumber = 0;
	char    *test_psReferenceId = "SearchRequest reference id";
	DbvBool test_fReplaceIndicator = DBV_TRUE;
	DbvObjectIdentifier       test_pcPreferredRecordSyntax = ANSI_EXPLAIN_RECORD;
	char *test_sResultSetName = "default";

	dbname = "IR-Explain-1";

	/* SmallSetUpperBound */
	psSearchReq->ulSmallSetUpperBound = test_ulSmallSetUpperBound;

	/* LargeSetLowerBound */
	psSearchReq->ulLargeSetLowerBound = test_ulLargeSetLowerBound;

	/* MediumSetPresentNumber */
	psSearchReq->ulMediumSetPresentNumber = test_ulMediumSetPresentNumber;

	/* fReplaceIndicator */
	psSearchReq->fReplaceIndicator = test_fReplaceIndicator;

	/* sResultSetName */
	fill_dbv_inter_string(&(psSearchReq->sResultSetName), test_sResultSetName);

	/* fill in the database name */
	psSearchReq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 2);
	psSearchReq->ppsDatabaseNames[0] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psSearchReq->ppsDatabaseNames[0], dbname);
	psSearchReq->ppsDatabaseNames[1] = NULL;

	/* psQuery */
	psSearchReq->psQuery = (DbvQuery *)DbvAlloc(sizeof (DbvQuery));
	fill_srchreq_explain_cip_Query(psSearchReq->psQuery);
}



void
print_pre_search_explain_cip_blurb(void)
{
	printf("\n");
	printf("Please enter an HQL (Hough Query Language) explain query.\n\n");
	printf("Some examples:\n\n");
	printf("collectionid: image\n");
	printf("\n\n");
}


void
fill_srchreq_explain_cip_Query(DbvQuery *psQuery)
{
	/* All oids are defined in sitypes.h */
	char *rpnoid = ANSI_ATTRIBUTE_SET_CIP_1;
	char hql_query[100];
	DbvRPNStructure *rpnstruct1 = NULL;

	psQuery->eQueryType = QT_Rpn;
	psQuery->uQuery.psRpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
	psQuery->uQuery.psRpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
	memcpy(psQuery->uQuery.psRpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));

	print_pre_search_explain_cip_blurb();

	/* Loop until a valid HQL query is entered */
	while (rpnstruct1 == NULL)
	{
		printf("Enter an HQL query> ");
		/* Get an input line from the user - allow
		 * 99 (max - 1)
		 * characters in the line at most (0-98)
		 */
		if (getline(hql_query, 100))
		{
			rpnstruct1 = form_dbv_rpn(hql_query);
			if (rpnstruct1 == NULL)
			{
				printf("Error in HQL Query\n\n");
			}
		}
	}

	/* At this point there is a valid Dbv RPN query in rpnstruct1 */
	/* Assign structures */
	psQuery->uQuery.psRpnQuery->sRpnStructure = *rpnstruct1;
}



