
#include "api_origin_explain_pres.h"

/* Set up a PresentRequest PDU and sent it to a target */
int send_exppresreq_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *dbname)
{

	DbvPresentReq   * psPresentReq = NULL;

	int choice = 0;
        char choice_string[100];

	/* Allocate memory for psPresentReq and fill it in */
	psPresentReq = (DbvPresentReq * ) DbvAlloc (sizeof(DbvPresentReq));

	fill_explain_pres_request(psPresentReq, dbname);

	printf("Print PresentRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_presreq_fields(stdout, psPresentReq);
                printf("-----------------------------------------\n");
        }
 

        /* Send the PresentRequest we have just filled in to the target */
	if (DbvPresentRequest(*assoc_id, psPresentReq, error) == SI_OK) {


		SIFreeDbvPresentReq(&psPresentReq);

		return SI_OK;
	} else {

		PrivateErrorHandling(error);

		SIFreeDbvPresentReq(&psPresentReq);


		/* Release buffer, assign NULL pointer */
		FreeSIErrorCode(error);
		return SI_NOTOK;
	}
}


int fill_explain_pres_request(DbvPresentReq *psPresentReq, char *dbname)
{
	char	*test_ressetid = "default";

	char	*test_reference_id = "present request reference id";
	char	*test_pref_rec_syntax = ANSI_EXPLAIN_RECORD;

	char	num_recs[1000];
	char	res_start[1000];
        int choice = 0;
        char choice_string[100];


	/* Compulsory fields... */


	/* ResultSetId */
	fill_dbv_inter_string(&(psPresentReq->sResultSetId), test_ressetid);

	/* Result set start point  */
	choice = 0;
	while (choice == 0)
	{
		printf("Starting record, eg 1: ");
		choice = atoi(gets(choice_string));
		psPresentReq->ulResultSetStartPoint = choice;
	}

	/* Number of records requested */
	choice = 0;
	while (choice == 0)
	{
		printf("Number of records, eg 3: ");
		choice = atoi(gets(choice_string));
		psPresentReq->ulNumberOfRecordsRequested = choice;
	}

	/* Optional fields... */

	/* ReferenceId */

	psPresentReq->psReferenceId = NULL;
/*
	psPresentReq->psReferenceId = (DbvOctetString * ) DbvAlloc(sizeof(DbvOctetString));
	fill_dbv_octet_string(psPresentReq->psReferenceId, test_reference_id);
 */

	/* Additional ranges  - create space for ten ranges (0 - 9) */

	psPresentReq->ppsAdditionalRanges = NULL;

	/* Record composititon */

	psPresentReq->psRecordComposition = NULL;	/* V3 feature */

	psPresentReq->psRecordComposition = (DbvRecordComposition * ) DbvAlloc (sizeof(struct DbvRecordComposition));
	fill_dbv_explain_present_reccomp(psPresentReq->psRecordComposition);


	/* Preferred Record Syntax */
	/* +1 to allocate space for the \0 */

	choice = 0;
	while (choice < 1 || choice > 3)
	{
		printf("Preferred Record Syntax: Explain(1), SUTRS(2), None(3): ");
		choice = atoi(gets(choice_string));
		if (choice < 1 || choice > 3)
		{
			printf("Unknown option\n\n");
		}
	}
	if (choice == 1)
	{
		test_pref_rec_syntax = ANSI_EXPLAIN_RECORD;
	}
	else if (choice == 2)
	{
		test_pref_rec_syntax = ANSI_UNSTRUCTURED_TEXT_RECORD;
	}


	if (choice != 3)
	{
		psPresentReq->pcPreferredRecordSyntax = (DbvProtocolVersion) DbvAlloc (strlen(test_pref_rec_syntax) + 1);
		memcpy(psPresentReq->pcPreferredRecordSyntax, test_pref_rec_syntax, strlen(test_pref_rec_syntax));
	}
	else
	{
		psPresentReq->pcPreferredRecordSyntax = NULL;
	}

#ifdef OPTIONAL

	/* Max segment count */
	psPresentReq->pulMaxSegmentCount = (Ulong * ) DbvAlloc (sizeof(Ulong));
	*psPresentReq->pulMaxSegmentCount = 16;

	/* Max record size */
	psPresentReq->pulMaxRecordSize = (Ulong * ) DbvAlloc (sizeof(Ulong));
	*psPresentReq->pulMaxRecordSize = 32;

	/* Max segment size */
	psPresentReq->pulMaxSegmentSize = (Ulong * ) DbvAlloc (sizeof(Ulong));
	*psPresentReq->pulMaxSegmentSize = 48;

	/* OtherInformation */
	psPresentReq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);
	psPresentReq->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psPresentReq->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter ppsOtherInformation type (for 2 list nodes): char (1), bin (2), ext (3), obj (4): ");
                choice = (atol(gets(choice_string)));
        }
	switch (choice) {
	case 1:
		fill_pres_request_othinfo_char(psPresentReq->ppsOtherInformation[0]);
		fill_pres_request_othinfo_char(psPresentReq->ppsOtherInformation[1]);
		break;
	case 2:
		fill_pres_request_othinfo_bin(psPresentReq->ppsOtherInformation[0]);
		fill_pres_request_othinfo_bin(psPresentReq->ppsOtherInformation[1]);
		break;
	case 3:
		fill_pres_request_othinfo_ext(psPresentReq->ppsOtherInformation[0]);
		fill_pres_request_othinfo_ext(psPresentReq->ppsOtherInformation[1]);
		break;
	case 4:
		fill_pres_request_othinfo_obj(psPresentReq->ppsOtherInformation[0]);
		fill_pres_request_othinfo_obj(psPresentReq->ppsOtherInformation[1]);
		break;
	}

#endif /* not OPTIONAL */
}

int
fill_dbv_explain_present_reccomp(DbvRecordComposition *psRecComp)
{
	int choice;
	char s[100];

	while (choice != 1 && choice != 2)
	{
		printf("Do you want to see a Brief Description(1), Full Description (2):");
		choice = atoi(gets(s));
		if (choice != 1 && choice != 2)
		{
			printf("Unknown option\n\n");
		}
	}
	psRecComp->eRecordCompositionType = RCT_ElementSetNames;
	psRecComp->uRecordComposition.psElementSetNames = (DbvElementSetNames *) DbvAlloc(sizeof (DbvElementSetNames));
	psRecComp->uRecordComposition.psElementSetNames->eElementSetType = EST_Generic;
	psRecComp->uRecordComposition.psElementSetNames->uElementSet.psGeneric = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	switch (choice)
	{
		case 1:
			fill_dbv_inter_string(psRecComp->uRecordComposition.psElementSetNames->uElementSet.psGeneric, "b");
			break;
		case 2:
			fill_dbv_inter_string(psRecComp->uRecordComposition.psElementSetNames->uElementSet.psGeneric, "f");
			break;
	}
}
