#include "api_origin_init.h"


/* Set up an InitializeRequest PDU and sent it to a target */
int send_initreq_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, int version)
{
	DbvInitializeReq   *psInitializeReq = NULL;
	
	int choice = 0;
	char choice_string[100];

	/* Allocate memory for psInitializeReq and fill it in */
        psInitializeReq = (DbvInitializeReq *) DbvAlloc (sizeof(DbvInitializeReq));
        fill_init_request(psInitializeReq, version);
 
	printf("Print InitializeRequest PDU to screen before sending ? yes (1), no (2): ");
	choice = atoi(gets(choice_string));
	if (choice == 1) {
		printf("-----------------------------------------\n");
		print_initreq_fields(stdout, psInitializeReq);
		printf("-----------------------------------------\n");
	}

 
        /* Send the InitializeRequest just filled in to the target */
        if (DbvInitializeRequest(*assoc_id, psInitializeReq, error) == SI_OK) {

		/* Sent PDU to target ok so free up space it used */ 

		SIFreeDbvInitializeReq(&psInitializeReq);

		return SI_OK;
	} else {
		
		/* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(error);

                SIFreeDbvInitializeReq(&psInitializeReq);


                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
		return SI_NOTOK;
	}
}



int fill_init_request(DbvInitializeReq *psInitializeReq, int version)
{

	if (version == 2)
		fill_init_request_v2style(psInitializeReq);
	else
		fill_init_request_v3style(psInitializeReq);
}




int fill_init_request_v2style(DbvInitializeReq *psInitializeReq)
{

        char *test_protocol_version = "1100000000000000";

	/* ALLOWED_OPTIONS is a #define in dbv_proto.h */
        char *test_options = "111100011011101000000000";
        /* char *test_options = ALLOWED_OPTIONS;*/

        char *test_reference_id = "init request reference id";
	char *test_open = "open string";
	char test_groupid[100] = "groupid";
	char test_userid[100] = "userid";
	char test_password[100] = "password";

	int choice = 0;
	char choice_string[100];

	char test_impid[100], test_impname[100],  test_impver[100];

	strcpy(test_impid, "DBV-OSI II UNIX API");
	strcpy(test_impname, "Origin Test Program");
	strcpy(test_impver, "Freeware Version 2.0");

 
        /* Compulsory fields... */
 
        /* Protocol Version */

	/* +1 to allocate space for the \0 */

        psInitializeReq->pcProtocolVersion = (DbvProtocolVersion) DbvAlloc (strlen(test_protocol_version) + 1);
        memcpy(psInitializeReq->pcProtocolVersion, test_protocol_version, strlen(test_protocol_version));



        /* Options */
        psInitializeReq->pcOptions = (DbvOptions) DbvAlloc (strlen(test_options) + 1);
        memcpy(psInitializeReq->pcOptions, test_options, strlen(test_options));
 
        /* Prefered message size */


        psInitializeReq->ulPreferredMessageSize = 5200;
 
        /* Maximum message size */
        psInitializeReq->ulMaximumMessageSize = 5200;


       	/* Optional fields... */

	/* ReferenceId */
/*
	psInitializeReq->psReferenceId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
	fill_dbv_octet_string(psInitializeReq->psReferenceId, test_reference_id);
*/

	/* idAuthentication */
       	psInitializeReq->psAuthentication = (DbvAuthentication *) DbvAlloc(sizeof (DbvAuthentication));

	choice = 0;
	while (choice < 1 || choice > 4) {
		printf("Enter psAuthentication type: open (1), userpass (2), anon (3), other (4): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
	
		/* Open */
		fill_init_request_idauth_open(psInitializeReq->psAuthentication, test_open);
		break;
	case 2:
		/* Userpass */

		printf("Enter groupid: ");
		gets(test_groupid);
		printf("Enter userid: ");
		gets(test_userid);
		printf("Enter password: ");
		gets(test_password);

		fill_init_request_idauth_pass(psInitializeReq->psAuthentication, test_groupid, test_userid, test_password);
		break;
	case 3: 
		/* Anonymous */
		fill_init_request_idauth_anon(psInitializeReq->psAuthentication);
		break;
	case 4:
		/* other */
		fill_init_request_idauth_other(psInitializeReq->psAuthentication);
		break;
	}

	


	/* ImplementationId */
	psInitializeReq->psImplementationId = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationId, test_impid);


	/* ImplementationName */
	psInitializeReq->psImplementationName = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationName, test_impname);




	/* ImplementationVersion */
	psInitializeReq->psImplementationVersion = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationVersion, test_impver);


}




int fill_init_request_v3style(DbvInitializeReq *psInitializeReq)
{

        char *test_protocol_version = "1110000000000000";

	/* ALLOWED_OPTIONS is a #define in dbv_proto.h */
        char *test_options = "111100011011101000000000";
        /* char *test_options = ALLOWED_OPTIONS;*/

        char *test_reference_id = "init request reference id";
	char *test_open = "open string";
	char test_groupid[100] = "groupid";
	char test_userid[100] = "userid";
	char test_password[100] = "password";

	int choice = 0;
	char choice_string[100];

	char test_impid[100], test_impname[100],  test_impver[100];


	strcpy(test_impid, "DBV-OSI II UNIX API");
	strcpy(test_impname, "Origin Test Program");
	strcpy(test_impver, "Freeware Version 2.0");

 
        /* Compulsory fields... */
 
        /* Protocol Version */

	/* +1 to allocate space for the \0 */

        psInitializeReq->pcProtocolVersion = (DbvProtocolVersion) DbvAlloc (strlen(test_protocol_version) + 1);
        memcpy(psInitializeReq->pcProtocolVersion, test_protocol_version, strlen(test_protocol_version));



        /* Options */
        psInitializeReq->pcOptions = (DbvOptions) DbvAlloc (strlen(test_options) + 1);
        memcpy(psInitializeReq->pcOptions, test_options, strlen(test_options));
 
        /* Prefered message size */


        psInitializeReq->ulPreferredMessageSize = 5200;
 
        /* Maximum message size */
        psInitializeReq->ulMaximumMessageSize = 5200;


       	/* Optional fields... */

	/* ReferenceId */
/*
	psInitializeReq->psReferenceId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
	fill_dbv_octet_string(psInitializeReq->psReferenceId, test_reference_id);
*/

	/* idAuthentication */
       	psInitializeReq->psAuthentication = (DbvAuthentication *) DbvAlloc(sizeof (DbvAuthentication));

	choice = 0;
	while (choice < 1 || choice > 4) {
		printf("Enter psAuthentication type: open (1), userpass (2), anon (3), other (4): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
	
		/* Open */
		fill_init_request_idauth_open(psInitializeReq->psAuthentication, test_open);
		break;
	case 2:
		/* Userpass */

		printf("Enter groupid: ");
		gets(test_groupid);
		printf("Enter userid: ");
		gets(test_userid);
		printf("Enter password: ");
		gets(test_password);

		fill_init_request_idauth_pass(psInitializeReq->psAuthentication, test_groupid, test_userid, test_password);
		break;
	case 3: 
		/* Anonymous */
		fill_init_request_idauth_anon(psInitializeReq->psAuthentication);
		break;
	case 4:
		/* other */
		fill_init_request_idauth_other(psInitializeReq->psAuthentication);
		break;
	}

	


	/* ImplementationId */
	psInitializeReq->psImplementationId = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationId, test_impid);


	/* ImplementationName */
	psInitializeReq->psImplementationName = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationName, test_impname);




	/* ImplementationVersion */
	psInitializeReq->psImplementationVersion = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	fill_dbv_inter_string(psInitializeReq->psImplementationVersion, test_impver);



}



int fill_init_request_idauth_open(DbvAuthentication *psAuthentication, char *open)
{ 
	psAuthentication->eAuthenticationType = AT_Open;


	/* DbvVisibleString is a typdefed char* */
	psAuthentication->uAuthentication.psOpen = (DbvVisibleString) DbvAlloc(strlen(open) + 1);
	memcpy(psAuthentication->uAuthentication.psOpen, open, strlen(open));

}


int fill_init_request_idauth_pass(DbvAuthentication *psAuthentication, char *groupid, char *userid, char *password)
{

	psAuthentication->eAuthenticationType = AT_IdPassword;
	psAuthentication->uAuthentication.psIdPassword = (DbvIdPassword *) DbvAlloc (sizeof(DbvIdPassword));


	/* Groupid */
	psAuthentication->uAuthentication.psIdPassword->psGroupId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));

	fill_dbv_inter_string(psAuthentication->uAuthentication.psIdPassword->psGroupId, groupid);


	/* Userid */
	psAuthentication->uAuthentication.psIdPassword->psUserId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));

	fill_dbv_inter_string(psAuthentication->uAuthentication.psIdPassword->psUserId, userid);


	/* password */
	psAuthentication->uAuthentication.psIdPassword->psPassword = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psAuthentication->uAuthentication.psIdPassword->psPassword, password);


}



int fill_init_request_idauth_anon(DbvAuthentication *psAuthentication)
{
	psAuthentication->eAuthenticationType = AT_Anonymous;
}



int fill_init_request_idauth_other(DbvAuthentication *psAuthentication)
{

	psAuthentication->eAuthenticationType = AT_Other;
	psAuthentication->uAuthentication.psOther = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));

	/* Lets set up a Sutrs */

	fill_initreq_idauth_other_sutrs(psAuthentication->uAuthentication.psOther);

}



int fill_initreq_idauth_other_sutrs(DbvASN1External *external)
{
        char *sutrs = "this is my idauth sutrs string";
        fill_dbv_isosutrs_external(external, sutrs);
}
 
 




int fill_init_request_othinfo_char(DbvOtherInformation *psOtherInformation)
{

	char oid[100]; 
	char charinfo[100];

	DbvBool want_infocat = DBV_TRUE;
	unsigned long catvalue = 27; 

	strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
	strcpy(charinfo, "this is my otherinfo char field");

	fill_dbv_otherinfo_char(psOtherInformation, want_infocat, oid, catvalue, charinfo);
}
  




int fill_init_request_othinfo_bin(DbvOtherInformation *psOtherInformation)
{
	char oid[100];
	char bininfo[100];

	DbvBool want_infocat = DBV_TRUE;
	unsigned long catvalue = 33; 

	strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
	strcpy(bininfo, "this is my otherinfo bin field");

	fill_dbv_otherinfo_bin(psOtherInformation, want_infocat, oid, catvalue, bininfo);
}





int fill_init_request_othinfo_ext(DbvOtherInformation *psOtherInformation)
{
	char oid[100];
	char bininfo[100];
	char sutrs_string[100];

	DbvBool want_infocat = DBV_TRUE;
	unsigned long catvalue = 47; 
	DbvASN1External *external = NULL;

	strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
	strcpy(bininfo, "this is my otherinfo bin field");
	strcpy(sutrs_string, "this is my sutrs string");

	fill_dbv_otherinfo_ext(psOtherInformation, want_infocat, oid, catvalue);

	/* Set up a sutrs external */

	external = psOtherInformation->uInformation.psExternallyDefinedInfo;
	fill_dbv_sutrs_external(external, sutrs_string);
}





int fill_init_request_othinfo_obj(DbvOtherInformation *psOtherInformation)
{
	char infooid[100];
	char otheroid[100];

	DbvBool want_infocat = DBV_TRUE;
	unsigned long catvalue = 47; 

	strcpy(infooid, ANSI_UNSTRUCTURED_TEXT_RECORD);
	strcpy(otheroid, ISO_AUSMARC);

	fill_dbv_otherinfo_obj(psOtherInformation, want_infocat, infooid, catvalue, otheroid);
}






/* Set up an InitializeRequest contaning a 
   CharacterSetAndLanguageNegotiation external in the 
   UserInformation field */
int send_initreq_charset_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
	DbvInitializeReq   *psInitializeReq = NULL;
	
	int choice = 0;
	char choice_string[100];

	/* Allocate memory for psInitializeReq and fill it in */
        psInitializeReq = (DbvInitializeReq *) DbvAlloc (sizeof(DbvInitializeReq));
	fill_init_request_v3style(psInitializeReq);
 
	/* Now add the CharSetAndLanguage external */
	psInitializeReq->psUserInformationField = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
	fill_DbvCharSetAndLanguage_external(psInitializeReq->psUserInformationField);

 
	printf("Print InitializeRequest PDU to screen before sending ? yes (1), no (2): ");
	choice = atoi(gets(choice_string));
	if (choice == 1) {
		printf("-----------------------------------------\n");
		print_initreq_fields(stdout, psInitializeReq);
		printf("-----------------------------------------\n");
	}


        /* Send the InitializeRequest just filled in to the target */
        if (DbvInitializeRequest(*assoc_id, psInitializeReq, error) == SI_OK) {

		/* Sent PDU to target ok so free up space it used */ 
		SIFreeDbvInitializeReq(&psInitializeReq);
		return SI_OK;
	} else {
		
		/* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(error);
                SIFreeDbvInitializeReq(&psInitializeReq);

                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
		return SI_NOTOK;
	}
}







