#include "api_origin_resrep.h"


int	send_resrepreq_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
	DbvResourceReportReq   *psResourceReportReq = NULL;
 
        int choice = 0;
        char choice_string[100];

        /* Allocate memory for psResourceReportReq and fill it in */
        psResourceReportReq = (DbvResourceReportReq *) DbvAlloc (sizeof(DbvResourceReportReq));
        fill_resrep_request(psResourceReportReq);
 

        printf("Print ResourceReportRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_resrepreq_fields(stdout, psResourceReportReq);
                printf("-----------------------------------------\n");
        }
 

        /* Send the ResourceReportRequest just filled in to the target */
        if (DbvResourceReportRequest(*assoc_id, psResourceReportReq, error) == SI_OK) {
                /* Sent PDU to target ok so free up space it used */

                SIFreeDbvResourceReportReq(&psResourceReportReq);

                return SI_OK;
        } else {
 
                /* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(error);

                SIFreeDbvResourceReportReq(&psResourceReportReq);

 
                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
                return SI_NOTOK;
        }
}



int fill_resrep_request(DbvResourceReportReq *psResourceReportReq)
{
        char *test_reference_id = "resource report request reference id";
        char *test_operation_id = "resource report request operation id";
        char *prefresformat = ANSI_RESOURCE_REPORT_2;
        int choice = 0;
        char choice_string[100];

	/* All optional fields */

        /* Reference id */
        psResourceReportReq->psReferenceId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
        fill_dbv_octet_string(psResourceReportReq->psReferenceId, test_reference_id);

	
        /* psOperationId */
        psResourceReportReq->psOperationId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
        fill_dbv_octet_string(psResourceReportReq->psOperationId, test_operation_id);

	/* pcPrefResourceReportFormat */

	/* DbvObjectIdentifier is a typdefed char*:  +1 for the \0 */
        psResourceReportReq->pcPrefResourceReportFormat = (DbvObjectIdentifier) DbvAlloc (strlen(prefresformat) + 1);
	memcpy(psResourceReportReq->pcPrefResourceReportFormat, prefresformat, strlen(prefresformat));

	
        /* psOtherInformation optional */
        psResourceReportReq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);
        psResourceReportReq->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
        psResourceReportReq->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
 
        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter ppsOtherInformation type (for 2 list nodes): char (1), bin (2), ext (3), other (4): ");
                choice = (atol(gets(choice_string)));
        }
 
        switch (choice) {
        case 1:
                fill_resrepreq_othinfo_char(psResourceReportReq->ppsOtherInformation[0]);
                fill_resrepreq_othinfo_char(psResourceReportReq->ppsOtherInformation[1]);
                break;
        case 2:
                fill_resrepreq_othinfo_bin(psResourceReportReq->ppsOtherInformation[0]);
                fill_resrepreq_othinfo_bin(psResourceReportReq->ppsOtherInformation[1]);
                break;
        case 3:
                fill_resrepreq_othinfo_ext(psResourceReportReq->ppsOtherInformation[0]);
                fill_resrepreq_othinfo_ext(psResourceReportReq->ppsOtherInformation[1]);
                break;
        case 4:
                fill_resrepreq_othinfo_obj(psResourceReportReq->ppsOtherInformation[0]);
                fill_resrepreq_othinfo_obj(psResourceReportReq->ppsOtherInformation[1]);
                break;
        }
}
 


int fill_resrepreq_othinfo_char(DbvOtherInformation *psOtherInformation)
{
 
        char oid[100];
        char charinfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 27;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(charinfo, "this is my resource report request otherinfo char field");
 
        fill_dbv_otherinfo_char(psOtherInformation, want_infocat, oid, catvalue, charinfo);
}



int fill_resrepreq_othinfo_bin(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 33;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my resource report request otherinfo bin field");
 
        fill_dbv_otherinfo_bin(psOtherInformation, want_infocat, oid, catvalue, bininfo);
}



int fill_resrepreq_othinfo_ext(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
        char sutrs_string[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
        DbvASN1External *external = NULL;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my resource report request otherinfo bin field");
        strcpy(sutrs_string, "this is my sutrs string - Resource Report Request");
 
        fill_dbv_otherinfo_ext(psOtherInformation, want_infocat, oid, catvalue);
 
        /* Set up a sutrs external */
        external = psOtherInformation->uInformation.psExternallyDefinedInfo;
        fill_dbv_sutrs_external(external, sutrs_string);
}
 


int fill_resrepreq_othinfo_obj(DbvOtherInformation *psOtherInformation)
{
        char infooid[100];
        char otheroid[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
 
        strcpy(infooid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(otheroid, ANSI_UNSTRUCTURED_TEXT_RECORD);
 
        fill_dbv_otherinfo_obj(psOtherInformation, want_infocat, infooid, catvalue, otheroid);
}




