#include "api_origin_search.h"


/* Set up an SearchRequest PDU and sent it to a target */
int send_srchreq_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *dbname)
{

	DbvSearchReq	*psSearchReq = NULL; 

	int choice = 0;
        char choice_string[100];

	/* Allocate memory for psSearchReq and fill it in */
	psSearchReq = (DbvSearchReq *)DbvAlloc(sizeof(DbvSearchReq));

	fill_search_request(psSearchReq, dbname);

	printf("Print SearchRequest PDU to screen before sending ? yes (1), no (2): ");
        choice = atoi(gets(choice_string));
        if (choice == 1) {
                printf("-----------------------------------------\n");
                print_searchreq_fields(stdout, psSearchReq);
                printf("-----------------------------------------\n");
        }
 


        /* Send the SearchRequest we have just filled in to the target */
        if (DbvSearchRequest(*assoc_id, psSearchReq, error) == SI_OK) {

        	SIFreeDbvSearchReq(&psSearchReq);

		return SI_OK;
	} else {
                PrivateErrorHandling(error);

        	SIFreeDbvSearchReq(&psSearchReq);


                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(error);
		return SI_NOTOK;
        }

}



int fill_search_request(DbvSearchReq *psSearchReq, char *dbname)
{
        if (strlen(dbname) > 0)
                fill_search_request_prepared(psSearchReq, dbname);

        else
                fill_search_request_default(psSearchReq);
}


int fill_search_request_prepared(DbvSearchReq *psSearchReq, char *dbname)
{
  
	char 	*test_psReferenceId = "SearchRequest reference id";

	Ulong   test_ulSmallSetUpperBound = 0;
	Ulong 	test_ulLargeSetLowerBound = 1;
	Ulong 	test_ulMediumSetPresentNumber = 0;
	char databname[1000];


	DbvBool test_fReplaceIndicator = DBV_TRUE;
	char *test_sResultSetName = "default";
	DbvObjectIdentifier       test_pcPreferredRecordSyntax = ANSI_USMARC;


	/* create a search template that is general purpose enough to work on the 
	 * pre-named servers */

	printf("Enter database name (%s): ", dbname);
	gets(databname);
	if (strlen(databname))
		dbname = databname;
		
	

	/* SmallSetUpperBound */
	psSearchReq->ulSmallSetUpperBound = test_ulSmallSetUpperBound;

	/* LargeSetLowerBound */
	psSearchReq->ulLargeSetLowerBound = test_ulLargeSetLowerBound;

	/* MediumSetPresentNumber */
	psSearchReq->ulMediumSetPresentNumber = test_ulMediumSetPresentNumber;


	/* fReplaceIndicator */
	psSearchReq->fReplaceIndicator = test_fReplaceIndicator;

	/* sResultSetName */
	fill_dbv_inter_string(&(psSearchReq->sResultSetName), test_sResultSetName);

	/* DatabaseNames */
	psSearchReq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 2);
	psSearchReq->ppsDatabaseNames[0] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psSearchReq->ppsDatabaseNames[0], dbname);
	psSearchReq->ppsDatabaseNames[1] = NULL;


	/* psQuery */
	psSearchReq->psQuery = (DbvQuery *)DbvAlloc(sizeof(DbvQuery));
	fill_searchreq_Query(psSearchReq->psQuery);
}




int fill_search_request_default(DbvSearchReq *psSearchReq)
{
	int choice = 0;
        char choice_string[100];

        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter search request style: V2 (1), V3 (2): ");
                choice = (atol(gets(choice_string)));
        }
 
        switch (choice) {
        case 1:
        	fill_searchreq_v2style(psSearchReq);
		break;
        case 2:
        	fill_searchreq_v3style(psSearchReq);
		break;
	}
}


	   

/* Generic V2 style */
int fill_searchreq_v2style(DbvSearchReq *psSearchReq)
{
	char 	*test_psReferenceId = "SearchRequest reference id";

	/* the following indicate that there should be no records returned 
	 * with the search response */
	Ulong   test_ulSmallSetUpperBound = 0;
	Ulong 	test_ulLargeSetLowerBound = 1;
	Ulong 	test_ulMediumSetPresentNumber = 0;

	char	dbname[1000];

	DbvBool test_fReplaceIndicator = DBV_TRUE;
	char *test_sResultSetName = "default";
	DbvObjectIdentifier       test_pcPreferredRecordSyntax = ANSI_USMARC;

	/* SmallSetUpperBound */
	psSearchReq->ulSmallSetUpperBound = test_ulSmallSetUpperBound;

	/* LargeSetLowerBound */
	psSearchReq->ulLargeSetLowerBound = test_ulLargeSetLowerBound;

	/* MediumSetPresentNumber */
	psSearchReq->ulMediumSetPresentNumber = test_ulMediumSetPresentNumber;


	/* fReplaceIndicator */
	psSearchReq->fReplaceIndicator = test_fReplaceIndicator;

	/* sResultSetName */
	fill_dbv_inter_string(&(psSearchReq->sResultSetName), test_sResultSetName);

	/* DatabaseNames */
	printf("Enter database name: ");
	gets(dbname);


	psSearchReq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 2);
	psSearchReq->ppsDatabaseNames[0] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psSearchReq->ppsDatabaseNames[0], dbname);
	psSearchReq->ppsDatabaseNames[1] = NULL;


	/* psQuery */
	psSearchReq->psQuery = (DbvQuery *)DbvAlloc(sizeof(DbvQuery));
	fill_searchreq_Query(psSearchReq->psQuery);
}



/* Generic V3 style */
int fill_searchreq_v3style(DbvSearchReq *psSearchReq)
{

	char 	*test_psReferenceId = "SearchRequest reference id";

	/* the following indicate that there should be no records returned 
	 * with the search response */
	Ulong   test_ulSmallSetUpperBound = 0;
	Ulong 	test_ulLargeSetLowerBound = 1;
	Ulong 	test_ulMediumSetPresentNumber = 0;

	DbvBool test_fReplaceIndicator = DBV_TRUE;
	char 	*test_sResultSetName = "default";
	DbvObjectIdentifier       test_pcPreferredRecordSyntax = ANSI_USMARC;
	int 	choice = 0;
        char 	choice_string[100];
	char	dbname[100];


	/* Reference Id Optional */

	psSearchReq->psReferenceId = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
	fill_dbv_octet_string(psSearchReq->psReferenceId, test_psReferenceId);

	/* SmallSetUpperBound */
	psSearchReq->ulSmallSetUpperBound = test_ulSmallSetUpperBound;

	/* LargeSetLowerBound */
	psSearchReq->ulLargeSetLowerBound = test_ulLargeSetLowerBound;

	/* MediumSetPresentNumber */
	psSearchReq->ulMediumSetPresentNumber = test_ulMediumSetPresentNumber;


	/* fReplaceIndicator */
	psSearchReq->fReplaceIndicator = test_fReplaceIndicator;


	/* sResultSetName */
	fill_dbv_inter_string(&(psSearchReq->sResultSetName), test_sResultSetName);

	/* DatabaseNames  - create space for 3 */
	psSearchReq->ppsDatabaseNames = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 3);

	/* the following function shows how to enter three database names */
/*
	fill_searchreq_DatabaseNames(psSearchReq->ppsDatabaseNames);
*/

	/* the following uses a single database name requested from the user */
	printf("Enter database name: ");
	gets(dbname);
	psSearchReq->ppsDatabaseNames[0] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psSearchReq->ppsDatabaseNames[0], dbname);
	psSearchReq->ppsDatabaseNames[1] = NULL;

	/* Optional fields */

	/* SmallSetElementNames optional */
	psSearchReq->psSmallSetElementNames = (DbvElementSetNames *)DbvAlloc(sizeof(DbvElementSetNames));
	fill_searchreq_SmallSetElementNames(psSearchReq->psSmallSetElementNames);

	/* MediumSetElementNames optional */

	 psSearchReq->psMediumSetElementNames = (DbvElementSetNames *)DbvAlloc(sizeof(DbvElementSetNames));

	fill_searchreq_MediumSetElementNames(psSearchReq->psMediumSetElementNames);



	/* pcPreferredRecordSyntax Optional */

	psSearchReq->pcPreferredRecordSyntax = (DbvObjectIdentifier )DbvAlloc(strlen(test_pcPreferredRecordSyntax)+1);
	memcpy(psSearchReq->pcPreferredRecordSyntax, test_pcPreferredRecordSyntax, strlen(test_pcPreferredRecordSyntax));


	/* psQuery */
	psSearchReq->psQuery = (DbvQuery *)DbvAlloc(sizeof(DbvQuery));
	fill_searchreq_Query(psSearchReq->psQuery);


	/* psAdditionalSearchInfo optional */

	psSearchReq->ppsAdditionalSearchInfo = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);
	psSearchReq->ppsAdditionalSearchInfo[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psSearchReq->ppsAdditionalSearchInfo[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));

        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter ppsAdditionalSearchInfo type (for 2 list nodes): char (1), bin (2), ext (3), other (4): ");
                choice = (atol(gets(choice_string)));
        }
 
        switch (choice) {
        case 1:
        	fill_searchreq_othinfo_char(psSearchReq->ppsAdditionalSearchInfo[0]);
        	fill_searchreq_othinfo_char(psSearchReq->ppsAdditionalSearchInfo[1]);
		break;
	case 2:
        	fill_searchreq_othinfo_bin(psSearchReq->ppsAdditionalSearchInfo[0]);
        	fill_searchreq_othinfo_bin(psSearchReq->ppsAdditionalSearchInfo[1]);
		break;
	case 3:
        	fill_searchreq_othinfo_ext(psSearchReq->ppsAdditionalSearchInfo[0]);
        	fill_searchreq_othinfo_ext(psSearchReq->ppsAdditionalSearchInfo[1]);
		break;
	case 4:
        	fill_searchreq_othinfo_obj(psSearchReq->ppsAdditionalSearchInfo[0]);
        	fill_searchreq_othinfo_obj(psSearchReq->ppsAdditionalSearchInfo[1]);
		break;
	}



	/* psOtherInformation optional */
	psSearchReq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);
	psSearchReq->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psSearchReq->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));

        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter ppsOtherInformation type (for 2 list nodes): char (1), bin (2), ext (3), other (4): ");
                choice = (atol(gets(choice_string)));
        }
 
        switch (choice) {
        case 1:
        	fill_searchreq_othinfo_char(psSearchReq->ppsOtherInformation[0]);
        	fill_searchreq_othinfo_char(psSearchReq->ppsOtherInformation[1]);
		break;
	case 2:
        	fill_searchreq_othinfo_bin(psSearchReq->ppsOtherInformation[0]);
        	fill_searchreq_othinfo_bin(psSearchReq->ppsOtherInformation[1]);
		break;
	case 3:
        	fill_searchreq_othinfo_ext(psSearchReq->ppsOtherInformation[0]);
        	fill_searchreq_othinfo_ext(psSearchReq->ppsOtherInformation[1]);
		break;
	case 4:
        	fill_searchreq_othinfo_obj(psSearchReq->ppsOtherInformation[0]);
        	fill_searchreq_othinfo_obj(psSearchReq->ppsOtherInformation[1]);
		break;
	}
}



void fill_searchreq_DatabaseNames(DbvInternationalString **ppsDatabaseNames)
{
	int loop = 0;

	/* 3 list nodes 0, 1, 2 */	

	for (loop=0; loop<3; loop++) 
		ppsDatabaseNames[loop] = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));

	fill_dbv_inter_string(ppsDatabaseNames[0], "XXDEFAULT");
	fill_dbv_inter_string(ppsDatabaseNames[1], "MYDBASE");
	fill_dbv_inter_string(ppsDatabaseNames[2], "YOURDBASE");

	/* Terminate the list */
	ppsDatabaseNames[3] = NULL;
}



void fill_searchreq_SmallSetElementNames(DbvElementSetNames *psSmallSetElementNames)
{
	char *test_Generic = "Generic SmallSetElementNames";
	int loop = 0;
	int choice = 0;
        char choice_string[100];

        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter psSmallSetElementNames eElementSetType type: generic (1), dbspecific (2): ");
                choice = (atol(gets(choice_string)));
        }
 
	switch (choice) {
	case 1:

		psSmallSetElementNames->eElementSetType = EST_Generic;
		psSmallSetElementNames->uElementSet.psGeneric = (DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.psGeneric, test_Generic);


		break;
	case 2:

		psSmallSetElementNames->eElementSetType = EST_DatabaseSpecific;

		/* Create space for 4 list nodes, 0, 1, 2, \0 */
		psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific= (DbvElementSetsDbSpecific **) DbvAlloc (sizeof(DbvElementSetsDbSpecific *) * 4);
		for(loop=0; loop<3;loop++)
		{
			psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]=(DbvElementSetsDbSpecific *)DbvAlloc(sizeof(DbvElementSetsDbSpecific));
			psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]->psDatabaseName=(DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
			psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]->psElementSetName=(DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		}

		/* Terminate the list */
		psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]= NULL;
	
		/* Add data to the list nodes just created */
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[0]->psDatabaseName, "SmallSetElementNames Specific Database Name no. 0");
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[0]->psElementSetName, "SmallSetElementNames Specific Element Set Name no. 0");
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[1]->psDatabaseName, "SmallSetElementNames Specific Database Name no. 1");
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[1]->psElementSetName, "SmallSetElementNames Specific Element Set Name no. 1");
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[2]->psDatabaseName, "SmallSetElementNames Specific Database Name no. 2");
		fill_dbv_inter_string(psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific[2]->psElementSetName, "SmallSetElementNames Specific Element Set Name no. 2");

		break;
	}
	
}



void fill_searchreq_MediumSetElementNames(DbvElementSetNames *psMediumSetElementNames)
{
	char *test_Generic = "Generic MediumSetElementNames";
	int loop = 0;
	int choice = 0;
        char choice_string[100];


        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter psMediumSetElementNames eElementSetType type: generic (1), dbspecific (2): ");
                choice = (atol(gets(choice_string)));
        }


	switch (choice) {
	case 1:

		psMediumSetElementNames->eElementSetType = EST_Generic;
		psMediumSetElementNames->uElementSet.psGeneric = (DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.psGeneric, test_Generic);
		break;
	case 2:
		psMediumSetElementNames->eElementSetType = EST_DatabaseSpecific;

		/* Create 4 list nodes - 0, 1, 2, \0 */
		psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific=(DbvElementSetsDbSpecific **)DbvAlloc(sizeof(DbvElementSetsDbSpecific *) * 4);
		for(loop=0; loop<3;loop++)
		{
			psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]=(DbvElementSetsDbSpecific *)DbvAlloc(sizeof(DbvElementSetsDbSpecific));
			psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]->psDatabaseName=(DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
			psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]->psElementSetName=(DbvInternationalString *)DbvAlloc(sizeof(DbvInternationalString));
		}

		/* Terminate the list */
		psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[loop]= NULL;



		/* Add data to the list nodes created */
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[0]->psDatabaseName, "MediumSetElementNames Specific Database Name no. 0");
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[0]->psElementSetName, "MediumSetElementNames Specific Element Set Name no. 0");
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[1]->psDatabaseName, "MediumSetElementNames Specific Database Name no. 1");
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[1]->psElementSetName, "MediumSetElementNames Specific Element Set Name no. 1");
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[2]->psDatabaseName, "MediumSetElementNames Specific Database Name no. 2");
		fill_dbv_inter_string(psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific[2]->psElementSetName, "MediumSetElementNames Specific Element Set Name no. 2");
		break;
	}
	
}



void fill_searchreq_Query(DbvQuery *psQuery)
{
	char *test_psPrivateQuery="Query - Private Query";

	/* Use Bib-1 for queries */
	char *rpnoid = ANSI_ATTRIBUTE_SET_BIB_1;
	DbvRPNStructure *rpnstruct1 = NULL;
	char hql_query[100];

	char *test_psIso8777Query="Query - Iso8777Query";
	char *test_psZ3958Query="Query - Z3958Query";
	char *test_psNaturalLanguageQuery="Query - NaturalLanguageQuery";

        int choice = 0;
        char choice_string[100];

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter psQuery type: private (1), rpn (2), iso8777 (3), z3958 (4), erpn (5), naturallanguage (6): ");
                choice = (atol(gets(choice_string)));
        }


	switch (choice) {
	case 1:

		psQuery->eQueryType = QT_Private; 
		psQuery->uQuery.psPrivateQuery = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psPrivateQuery, test_psPrivateQuery);
		break;
	case 2:

		psQuery->eQueryType = QT_Rpn; 
		psQuery->uQuery.psRpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		psQuery->uQuery.psRpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
		memcpy(psQuery->uQuery.psRpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));


		print_pre_query_blurb();

		/* Loop until a valid HQL query is entered */
		while (rpnstruct1 == NULL) {
			printf("Enter an HQL query> ");

			/* Get an input line from the user - allow 
			   99 (max - 1) 
		 	   characters in the line at most (0-98) */ 
			if (getline(hql_query, 100)) {
				rpnstruct1 = form_dbv_rpn(hql_query);

				if (rpnstruct1 == NULL)
					printf("Error in HQL Query\n\n");
			}
		}


		/* At this point there is a valid Dbv RPN query in rpnstruct1 */
		/* Assign structures */
		psQuery->uQuery.psRpnQuery->sRpnStructure = *rpnstruct1;

		break;

	case 3:
		psQuery->eQueryType = QT_Iso8777;
		psQuery->uQuery.psIso8777Query = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psIso8777Query, test_psIso8777Query);
		break;
	case 4:
		psQuery->eQueryType = QT_Z3958;
		psQuery->uQuery.psZ3958Query = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psZ3958Query, test_psZ3958Query);
		break;

	case 5:
		psQuery->eQueryType = QT_Erpn;
		psQuery->uQuery.psErpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		psQuery->uQuery.psErpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
		memcpy(psQuery->uQuery.psErpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));


		/* Rather than use HQL fill a DbvRPNStructure structure 'by hand' */
		fill_rpnstructure(&psQuery->uQuery.psErpnQuery->sRpnStructure);
		break;
	

	case 6:
		psQuery->eQueryType = QT_NaturalLanguage;
		psQuery->uQuery.psNaturalLanguageQuery = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psNaturalLanguageQuery, test_psNaturalLanguageQuery);
		break;
	}

}



int print_pre_query_blurb(void)
{
	printf("\n");
	printf("Please enter an HQL (Hough Query Language) query.\n");
	printf("\n");
	printf("Some HQL query examples:\n");
	printf("\n");
	printf("author: Charles Dickens\n");
	printf("title: Little Dorrit\n");
	printf("subject: Lamborghini and ( Diablo SE Jota or Countach 5000 QV )\n");
	printf("subject: /and gates/ and boolean logic\n");
	printf("title,author: Macbeth and William Shakespeare\n");
	printf("\n");


}



int fill_rpnstructure(DbvRPNStructure *psRpnStructure)
{
	/* Large V2 style RPN structures (without prox) were/can be tested using
	HQL (see fill_searchreq_Query(DbvQuery *psQuery) above) */
	 
	int choice = 0;
        char choice_string[100];

	char *test_ressetid = "test resultsetid operand";
	
        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter query type: operand operator operand (1), single operand (2): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psRpnStructure->eRpnStructureType = RST_RpnLopRopOperator;
		psRpnStructure->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *) DbvAlloc (sizeof(DbvRPNLopRopOperator));
		fill_lopropop(psRpnStructure->uRPNStructure.psRpnLopRopOperator);
		break;
	case 2:

		psRpnStructure->eRpnStructureType = RST_RpnOperand;
		psRpnStructure->uRPNStructure.psRpnOperand = (DbvOperand *) DbvAlloc (sizeof(DbvOperand));
		fill_operand(psRpnStructure->uRPNStructure.psRpnOperand);
		break;
	}
		
}



int fill_lopropop(DbvRPNLopRopOperator *psRpnLopRopOperator)
{

	/* All fields compulsory */

	psRpnLopRopOperator->psLeftOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	fill_rpnstructure(psRpnLopRopOperator->psLeftOperand);

	psRpnLopRopOperator->psRightOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	fill_rpnstructure(psRpnLopRopOperator->psRightOperand);

	/* Note that sOperator is a structure */
	fill_operator(&psRpnLopRopOperator->sOperator);

}



int fill_operand(DbvOperand *psRpnOperand)
{
	int choice = 0;
        char choice_string[100];

	char *test_ressetid = "test resultsetid operand";
	
        choice = 0;
        while (choice < 1 || choice > 3) {
                printf("Enter eOperandType type: attplusterm (1), ressetid (2), ressetplusatt (3): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:

		psRpnOperand->eOperandType = OT_AttributePlusTerm; 
		psRpnOperand->uOperand.psAttributesPlusTerm = (DbvAttributesPlusTerm *) DbvAlloc (sizeof(DbvAttributesPlusTerm));
		fill_attsplusterm(psRpnOperand->uOperand.psAttributesPlusTerm);

		break;
	case 2:

		psRpnOperand->eOperandType = OT_ResultSetId; 
		psRpnOperand->uOperand.psResultSetId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psRpnOperand->uOperand.psResultSetId, test_ressetid);  
		break;
	case 3:
		psRpnOperand->eOperandType = OT_ResultSetPlusAttributes; 
		psRpnOperand->uOperand.psResultSetPlusAttributes = (DbvResultSetPlusAttributes *) DbvAlloc (sizeof(DbvResultSetPlusAttributes));
		fill_resplusatts(psRpnOperand->uOperand.psResultSetPlusAttributes);

		break;
	}
}



int fill_resplusatts(DbvResultSetPlusAttributes *psResultSetPlusAttributes)
{
	char *test_ressetid = "result set plus attributes resultset id";

 

	/* Both compulsory */
	fill_dbv_inter_string(&psResultSetPlusAttributes->sResultSetId, test_ressetid);


	printf("Attributes:\n");

	/* Create a ppsAttributeElement list with three nodes - 0, 1, \0 */
	psResultSetPlusAttributes->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof (DbvAttributeElement *) * 3);

	/* Create and fill in node 0 */	

	psResultSetPlusAttributes->ppsAttributeElement[0] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 0: ");
	fill_attelement(psResultSetPlusAttributes->ppsAttributeElement[0]);


	/* Create and fill in node 1 */
	psResultSetPlusAttributes->ppsAttributeElement[1] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 1: ");
	fill_attelement(psResultSetPlusAttributes->ppsAttributeElement[1]);


	/* Terminate the list */
	psResultSetPlusAttributes->ppsAttributeElement[2] = NULL;
	
}



int fill_operator(DbvOperator *psOperator)
{
	int choice = 0;
        char choice_string[100];


	
        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter eOperatorType type: or (1), and (2), andnot (3), prox (4): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psOperator->eOperatorType = OT_Or;
		break;
	case 2:
		psOperator->eOperatorType = OT_And;
		break;
	case 3:
		psOperator->eOperatorType = OT_AndNot;
		break;
	case 4:

		psOperator->eOperatorType = OT_Proximity;
		psOperator->psProximityOperator = (DbvProximityOperator *) DbvAlloc (sizeof(DbvProximityOperator));
		fill_proximity(psOperator->psProximityOperator);
		break;
	}
}



int fill_proximity(DbvProximityOperator *psProximityOperator)
{
	int choice = 0;
        char choice_string[100];


	
	/* optional */


	psProximityOperator->pfExclusion = (DbvBool *) DbvAlloc (sizeof(DbvBool));
	*psProximityOperator->pfExclusion = DBV_TRUE;

	/* All the rest are compulsory */
	psProximityOperator->ulDistance = 888;
	psProximityOperator->fOrdered = DBV_FALSE;

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter eProximityRelationType type: < (1), <= (2), = (3), >= (4), > (5), != (6): ");
                choice = (atol(gets(choice_string)));
        }
	
	switch (choice) {
	case 1:
		psProximityOperator->eProximityRelationType = PRT_LessThan;
		break;
	case 2:
		psProximityOperator->eProximityRelationType = PRT_LessThanOrEqual;
		break;
	case 3:
		psProximityOperator->eProximityRelationType = PRT_Equal;
		break;
	case 4:
		psProximityOperator->eProximityRelationType = PRT_GreaterThanOrEqual;
		break;
	case 5:
		psProximityOperator->eProximityRelationType = PRT_GreaterThan;
		break;
	case 6:
		psProximityOperator->eProximityRelationType = PRT_NotEqual;
		break;
	}
		

	fill_proxunitcode(&psProximityOperator->sProximityUnitCode);
}



int fill_proxunitcode(DbvProximityUnitCode *psProximityUnitCode)
{
	int choice = 0;
        char choice_string[100];

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter eProximityUnitType type: known (1), private (2): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psProximityUnitCode->eProximityUnitType = PUT_Known;
        	choice = 0;
        	while (choice < 1 || choice > 11) {
               		printf("Enter eKnownProximityUnit type: character (1), word (2), sentence (3), para (4), sect (5), chapter (6), doc (7), elem (8), subelem (9), eltype (10), byte (11): ");
                	choice = (atol(gets(choice_string)));
        	}
	
		switch (choice) {
		case 1:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Character;
			break;
		case 2:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Word;
			break;
		case 3:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Sentence;
			break;
		case 4:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Paragraph;
			break;
		case 5:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Section;
			break;
		case 6:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Chapter;
			break;
		case 7:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Document;
			break;
		case 8:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Element;
			break;
		case 9:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_SubElement;
			break;
		case 10:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_ElementType;
			break;
		case 11:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Byte;
			break;
		}

		break;
	case 2:
		psProximityUnitCode->eProximityUnitType = PUT_Private;
		psProximityUnitCode->uProximityUnit.ulPrivate = 112233;
		break;
	}
	
}



void fill_searchreq_othinfo_char(DbvOtherInformation *psOtherInformation)
{
 
        char oid[100];
        char charinfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 27;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(charinfo, "this is my search request otherinfo char field");
 
        fill_dbv_otherinfo_char(psOtherInformation, want_infocat, oid, catvalue, charinfo);
}
 


void fill_searchreq_othinfo_bin(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 33;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my search request otherinfo bin field");
 
        fill_dbv_otherinfo_bin(psOtherInformation, want_infocat, oid, catvalue, bininfo);
}

void fill_searchreq_othinfo_ext(DbvOtherInformation *psOtherInformation)
{
        char oid[100];
        char bininfo[100];
        char sutrs_string[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
        DbvASN1External *external = NULL;
 
        strcpy(oid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(bininfo, "this is my search request otherinfo bin field");
        strcpy(sutrs_string, "this is my sutrs string - Search Request");
 
        fill_dbv_otherinfo_ext(psOtherInformation, want_infocat, oid, catvalue);
 
        /* Set up a sutrs external */
        external = psOtherInformation->uInformation.psExternallyDefinedInfo;
        fill_dbv_sutrs_external(external, sutrs_string);
}

void fill_searchreq_othinfo_obj(DbvOtherInformation *psOtherInformation)
{
        char infooid[100];
        char otheroid[100];
 
        DbvBool want_infocat = DBV_TRUE;
        unsigned long catvalue = 47;
 
        strcpy(infooid, ANSI_UNSTRUCTURED_TEXT_RECORD);
        strcpy(otheroid, ANSI_UNSTRUCTURED_TEXT_RECORD);
 
        fill_dbv_otherinfo_obj(psOtherInformation, want_infocat, infooid, catvalue, otheroid);
}
 

