/*  Module Name: @(#)api_origin_search_parser.c	7.2
    Version: 12/2/96
    Author: John Hough

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description: HQL Parser - your's to freely use and modify. 
		 Chesty's gone, Shakespear's in.

	  	 Processing...
	 	 Infix query -> infix vsl
		 infix vsl -> postfix vsl 
	   	 postfix vsl -> postfix query (original
		 query operators and operands re-assigned).

		This code may be freely used/modified subject to the terms
		and conditions of the GNU general public licence.



/*  here because of another m4 bug - if added after the
   long comment below the directive is left in the file !! */

/*  */



#include "api_origin_search_parser.h"



/******************************************************/
/******************************************************/
/* Global variables */

struct query_type
{
	/* query_identifier corressponds to use attribute */
	char *query_identifier;
	char *query_attributes;
	Querytype query_type;
};

struct query_type hql_supported_queries[] =
{
	/* Bib1 searches */
	{TITLE_SEARCH,		TITLE_ATTRIBUTES,		QT_Title},
	{AUTHOR_SEARCH,		AUTHOR_ATTRIBUTES,		QT_Author},
	{SUBJECT_SEARCH,	SUBJECT_ATTRIBUTES,		QT_Subject},

	/* Explain exp-1 searches - see ATR.2, Appendix 3, ATR */
	{EXPLAINCAT_SEARCH,	EXPLAINCAT_ATTRIBUTES,		QT_Explaincat},
	{HSL_SEARCH,		HSL_ATTRIBUTES,			QT_Hsl},
	{DBASE_SEARCH,		DBASE_ATTRIBUTES,		QT_Dbase},
	{TARGET_SEARCH,		TARGET_ATTRIBUTES,		QT_Target},
	{ATTRSETOID_SEARCH,	ATTRSETOID_ATTRIBUTES,		QT_Attrsetoid},
	{RECSYNOID_SEARCH,	RECSYNOID_ATTRIBUTES,		QT_Recsynoid},
	{TAGETSETOID_SEARCH,	TAGETSETOID_ATTRIBUTES,	        QT_Tagsetoid},
	{EXTSERVICEOID_SEARCH,	EXTSERVICEOID_ATTRIBUTES,	QT_Extservoid},
	{DATEADD_SEARCH,	DATEADD_ATTRIBUTES,		QT_Dateadd},
	{DATECHANGE_SEARCH,	DATECHANGE_ATTRIBUTES,		QT_Datechange},
	{DATEEXPIRES_SEARCH,	DATEEXPIRES_ATTRIBUTES,		QT_Dateexp},
	{ELEMENTSETNAME_SEARCH,	ELEMENTSETNAME_ATTRIBUTES,	QT_Elesetname},
	{PROCCONTEXT_SEARCH,	PROCCONTEXT_ATTRIBUTES,		QT_Proccontext},
	{PROCNAME_SEARCH,	PROCNAME_ATTRIBUTES,		QT_Procname},
	{TERMLISTNAME_SEARCH,	TERMLISTNAME_ATTRIBUTES,	QT_Tlistname},
	{SCHEMAOID_SEARCH,	SCHEMAOID_ATTRIBUTES,		QT_Schemaoid},
	{PRODUCER_SEARCH,	PRODUCER_ATTRIBUTES,		QT_Producer},
	{SUPPLIER_SEARCH,	SUPPLIER_ATTRIBUTES,		QT_Supplier},
	{AVAILABILITY_SEARCH,	AVAILABILITY_ATTRIBUTES,	QT_Avail},
	{PROPRIETY_SEARCH,	PROPRIETY_ATTRIBUTES,		QT_Propriety},
	{USERFEE_SEARCH,	USERFEE_ATTRIBUTES,		QT_Userfee},
	{VARIANTSETOID_SEARCH,	VARIANTSETOID_ATTRIBUTES,	QT_Varsetoid},
	{UNITSYSTEM_SEARCH,	UNITSYSTEM_ATTRIBUTES,		QT_Unitsystem},
	{KEYWORD_SEARCH,	KEYWORD_ATTRIBUTES,		QT_Keyword},
	{EXPLAINDBASE_SEARCH,	EXPLAINDBASE_ATTRIBUTES,	QT_Explaindbase},
	{PROCESSINGOID_SEARCH,	PROCESSINGOID_ATTRIBUTES,	QT_Processoid},

	/* Special for CIP explain .... */
	{COLLECTID_SEARCH,	COLLECTID_ATTRIBUTES,		QT_CipCollectId},

	/* RESULTSETNAME_ATTRIBUTES in the following is a dummy 
	*/

	{RESULTSETNAME_SEARCH,	RESULTSETNAME_ATTRIBUTES,	QT_ResultSetName},

	{DATEADDED_LT_SEARCH,	DATEADDED_LT_ATTRIBUTES,	QT_DateAddedLt},
	{DATEADDED_LE_SEARCH,	DATEADDED_LE_ATTRIBUTES,	QT_DateAddedLe},
	{DATEADDED_EQ_SEARCH,	DATEADDED_EQ_ATTRIBUTES,	QT_DateAddedEq},
	{DATEADDED_GE_SEARCH,	DATEADDED_GE_ATTRIBUTES,	QT_DateAddedGe},
	{DATEADDED_GT_SEARCH,	DATEADDED_GT_ATTRIBUTES,	QT_DateAddedGt},
	{DATEADDED_NE_SEARCH,	DATEADDED_NE_ATTRIBUTES,	QT_DateAddedNe},


	{DATECHANGED_LT_SEARCH,	DATECHANGED_LT_ATTRIBUTES,	QT_DateChangedLt},
	{DATECHANGED_LE_SEARCH,	DATECHANGED_LE_ATTRIBUTES,	QT_DateChangedLe},
	{DATECHANGED_EQ_SEARCH,	DATECHANGED_EQ_ATTRIBUTES,	QT_DateChangedEq},
	{DATECHANGED_GE_SEARCH,	DATECHANGED_GE_ATTRIBUTES,	QT_DateChangedGe},
	{DATECHANGED_GT_SEARCH,	DATECHANGED_GT_ATTRIBUTES,	QT_DateChangedGt},
	{DATECHANGED_NE_SEARCH,	DATECHANGED_NE_ATTRIBUTES,	QT_DateChangedNe},

	{DATEEXPIRES_LT_SEARCH,	DATEEXPIRES_LT_ATTRIBUTES,	QT_DateExpiresLt},
	{DATEEXPIRES_LE_SEARCH,	DATEEXPIRES_LE_ATTRIBUTES,	QT_DateExpiresLe},
	{DATEEXPIRES_EQ_SEARCH,	DATEEXPIRES_EQ_ATTRIBUTES,	QT_DateExpiresEq},
	{DATEEXPIRES_GE_SEARCH,	DATEEXPIRES_GE_ATTRIBUTES,	QT_DateExpiresGe},
	{DATEEXPIRES_GT_SEARCH,	DATEEXPIRES_GT_ATTRIBUTES,	QT_DateExpiresGt},
	{DATEEXPIRES_NE_SEARCH,	DATEEXPIRES_NE_ATTRIBUTES,	QT_DateExpiresNe}
 };


int number_hql_supported_queries = sizeof hql_supported_queries / sizeof(struct query_type);

char *infix_str;
int infix_str_pos;
int infix_str_len;
char postfix_str[200];
int postfix_str_pos;
int buf;


/******************************************************/
/******************************************************/


/* Function which takes an HQL query and returns a corressponding 
   DbvRPNStructure, all filled out  */ 
DbvRPNStructure *form_dbv_rpn(char *query) 
{
	int ch;
	DbvRPNStructure *rpn = NULL;

	/* Type of query */
	Querytype querytype = QT_Unknown;


	/* Array of pointer to char* (index array !) */
	char *index_array[MAXVAL];
	char vsl_string[MAXVAL];
	char lowercase_query[MAXVAL];
	char query_value[MAXVAL];

	/* Array of Querytypes */
	Querytype query_types[MAXVAL];
	int last_query_type_index = 0;
	

	/* Initialize global variables */
	infix_str = NULL;
	infix_str_pos = 0;
	infix_str_len = 0;
	memset(postfix_str, 0, sizeof(postfix_str));
	postfix_str_pos = 0;
	buf = 0;


	/* Initialise (reset) query types array */
	memset(query_types, 0, sizeof(query_types));


	/* Convert HQL query types to lower case (copy first to avoid 
	   potential problem of modifing symbolic constants) */
	strcpy(lowercase_query, query);
	querytypes_to_lower_case(lowercase_query);


	/* Get query type and value 
	   e.g. HQL Query: subject: computers and 
	   	Query Type =	QT_Subject;
		Query Value =	computers and  
	*/ 
		

	get_query_types_and_val(lowercase_query, query_value, query_types, &last_query_type_index);

	/* If something failed */
	if (query_types[0] == QT_Unknown)
		return NULL;

	/* Convert a query string to VSL infix */
	query_val_to_vsl_infix(query_value, vsl_string, index_array);

	infix_str = vsl_string;
	infix_str_len = strlen(infix_str);

	/* Convert VSL infix to VSL postfix */
	if (vsl_infix_to_vsl_postfix() == -1) 
		return NULL;

	rpn = vsl_postfix_to_dbv_rpn(postfix_str, index_array, query_types, last_query_type_index);

	return rpn;
}



int vsl_infix_to_vsl_postfix()
{

	/* m4 macro bug fudge ->  '{' */
	/* (m4 gets confused if the first quote pair it encounters
	   is '{') */

	if (!nextis('{')) {
		printf("HQL Query Error: Query not formed properly\n");
		return -1;
	}

	if (expression() == -1)
		return -1;
	if (!nextis('}')) {
		printf("HQL Query Error: Query not formed properly\n");
		return -1;
	}

	return 0;
}
   


int querytypes_to_lower_case(char *query)
{
	/* Only lower case the query type */
	while ( (*query != '\0') && (*query != QUERY_TYPE_VAL_SEP_CHAR) ){
		*query = tolower(*query);
		query++;
	}
}



/* Routine to get the query type and value */
void get_query_types_and_val(char *input_string, char *query_val, Querytype query_types[], int *num_query_types)
{


	char input_string_copy[MAXVAL];
	char input_string_copy2[MAXVAL];
	char *ptr = NULL;
	int index = 0;
	int i, found;


	/* If nowt */
	if (input_string == NULL) 
		query_types[0] = QT_Unknown;


	/* strcpy to avoid problem with strtoking on symbol tables entries
	   (such strtoking produces segmentation faults !) */

	strcpy(input_string_copy, input_string);
	strcpy(input_string_copy2, input_string);

	ptr = input_string_copy;
	while ((*ptr != QUERY_TYPE_VAL_SEP_CHAR) && (*ptr != 0)) 
		ptr++;

	
	/* If not at end */
	if (*ptr != 0)

		/* Create a new end */
		*ptr = 0;
	else
		query_types[0] = QT_Unknown;

		
	/* Now scan the babe to get the query types */

	/* Get first token */

	ptr = (char *) strtok(input_string_copy, QUERY_TYPE_SEP);

	while (ptr != NULL) {

		found = 0;
		for (i=0; i < number_hql_supported_queries; i++)
		{
			if (strcmp(ptr, hql_supported_queries[i].query_identifier) == 0)
			{
				query_types[index] = hql_supported_queries[i].query_type;
				found = 1;
				break;
			}
		}
		if (!found)
		{
			query_types[0] = QT_Unknown;
			return;
		}

/*		if (strcmp(ptr, AUTHOR_SEARCH) == 0)  
			query_types[index] = QT_Author;

		else if (strcmp(ptr, SUBJECT_SEARCH) == 0)  
			query_types[index] = QT_Subject;

		else if (strcmp(ptr, TITLE_SEARCH) == 0)  
			query_types[index] = QT_Title;

		else if (strcmp(ptr, EXPLAINCAT_SEARCH) == 0)
			query_types[index] = QT_Explain;

		else {
			query_types[0] = QT_Unknown;
			return;

		} */

		ptr = (char *) strtok(NULL, QUERY_TYPE_SEP);
		index++;
	}

	/* last query type index */
	*num_query_types  = index - 1;

	/* Now look at the query */
	ptr = (char *) strtok(input_string_copy2, QUERY_TYPE_VAL_SEP);

	/* Get the query part (if there is one) */
	ptr = (char *) strtok(NULL, "");
	if (ptr == NULL)
		query_types[0] = QT_Unknown;
	else
		strcpy(query_val, ptr);


	/* If we get to this point then the HQL query is probably ok -
	   further checks are made in the rpn generator */
}



/* Convert query value string to VSL */
int query_val_to_vsl_infix(char *input_string, char *vsl_string, char *index_array[])
{
	/* first token -> a, second token -> b etc. */
	/* ( -> (,      ) -> ) */
	/* and -> *, or-> +, andnot -> - */

	char input_string_copy[MAXVAL+1];
	char token_obtained[MAXVAL];
	int insert_pos = 0;
	char *ptr = NULL;

	/* Copy to input_string_copy to avoid seg fault if input_string is
	   a symbolic constant (defined in symbol table) */
	
	strcpy(input_string_copy, input_string);

	/* Remove any leading spaces */
	remove_leading(input_string_copy,  ' ');
	strcat(input_string_copy, TOKEN_SEP);

	/* Start the VSL expression */
	strcpy(vsl_string, "{");

	/* Get first token */
	api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);
	while (strcmp(token_obtained, "NO SUCH TOKEN") != 0) {

		/* If an operator */
		if (operator_check(token_obtained))
			operator_process(token_obtained, vsl_string, input_string_copy);
		else
			/* process an operand */
			operand_process(token_obtained, vsl_string, input_string_copy, &insert_pos, index_array);
	}

	/* End the vsl expression */
	strcat(vsl_string, "}");
}




DbvBool operator_check(char *token_obtained)
{
	if ((strcmp(token_obtained, OPERATOR_AND) == 0) ||
	(strcmp(token_obtained, OPERATOR_OR) == 0) ||
	(strcmp(token_obtained, OPERATOR_ANDNOT) == 0) ||
	(strcmp(token_obtained, OPENING_BRACKET) == 0) ||
	(strcmp(token_obtained, CLOSING_BRACKET) == 0))
		return DBV_TRUE;

	return DBV_FALSE;
}


int operator_process(char *token_obtained, char *vsl_string, char *input_string_copy)
{
	if (strcmp(token_obtained, OPERATOR_AND) == 0) {
		strcat(vsl_string, "*");
		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);

	} else if (strcmp(token_obtained, OPERATOR_OR) == 0) {
		strcat(vsl_string, "+");
		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);

	} else if (strcmp(token_obtained, OPERATOR_ANDNOT) == 0) {
		strcat(vsl_string, "-");
		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);

	} else if (strcmp(token_obtained, OPENING_BRACKET) == 0) {
		strcat(vsl_string, OPENING_BRACKET);
		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);

	} else if (strcmp(token_obtained, CLOSING_BRACKET) == 0) {
		strcat(vsl_string, CLOSING_BRACKET);
		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);
	}

}



int operand_process(char *token_obtained, char *vsl_string, char *input_string_copy, int *insert_pos, char *index_array[])
{
	char operands[MAXVAL];
	char temp[MAXVAL];

	complex_operand_check_and_process(token_obtained, input_string_copy);

	memset(operands, 0, sizeof(operands));

	/* Deal with first operand */
	memcpy(operands, token_obtained, strlen(token_obtained));

	/* Process any remaining operands */ 
	api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);

	/* While not and, or, andnot, (, ) */

	while ((operator_check(token_obtained) != DBV_TRUE) &&
		(strcmp(token_obtained, "NO SUCH TOKEN") != 0)) {

		strcat(operands, TOKEN_SEP);
		strcat(operands, token_obtained);

		api_get_token_from_string(token_obtained, TOKEN_SEP, input_string_copy);
	} 


	/* +1 for the terminating \0 */
	index_array[*insert_pos] = (char *) DbvAlloc (strlen(operands) + 1);
	strcpy(index_array[*insert_pos], operands);

	memset(temp, 0, sizeof(temp));
	sprintf(temp, "%c", '0' + *insert_pos);
	strcat(vsl_string, temp);

	/* Increment the value in insert_pos, not the pointer ! */
	(*insert_pos)++;
}



int complex_operand_check_and_process(char *token_obtained, char *input_string_copy)
{
	/* Complex operand enclosed by / / (can't have / as part of 
	   operand query in HQL) */

	char *ptr = NULL;
	char *token_insert_pos = NULL;
	char *ptr1 = NULL, *ptr2 = NULL;

	ptr = token_obtained;

	/* If the token obtained begins with a / */
	if (*ptr == COMPLEX_SEP) {

		/* Skip past start delimiters */
		while ((*ptr == COMPLEX_SEP) && (*ptr != 0)) 
			ptr++;
		
		strcpy(token_obtained, ptr);
		strcat(token_obtained, TOKEN_SEP);


		ptr1 = token_obtained + strlen(token_obtained);

		/* Position just past token obtained in source string */
		ptr2 = input_string_copy;

		while ((*ptr2 != COMPLEX_SEP) && (*ptr2 != 0)) 
			*ptr1++ = *ptr2++;

		*ptr1 = 0;

		/* skip past multiple delimiters in input_string_copy */
		while ((*ptr2 == COMPLEX_SEP) && (*ptr2 != 0))
			ptr2++;

		/* Remove any leading spaces */
		remove_leading(ptr2,  ' ');
		strcpy(input_string_copy, ptr2);

	}	
}



int remove_leading(char *string, char char_to_remove)
{
	char *ptr = NULL;

	ptr = string;

	while (*ptr == char_to_remove)
		ptr++;

	strcpy(string, ptr);

}




void api_get_token_from_string(char *token_obtained, char *token_delimiter, char *string)
{
 
        char temp[MAXVAL];
        char temp2[MAXVAL];
        char *strptr = NULL;
 
	/* Make 2 copies of string, one for strtok, one for strbrk */
        strcpy(temp, string);
        strcpy(temp2, string);
 
        /* See if a token exists */
        strptr = strtok(temp, token_delimiter);

	/* If one does */
        if (strptr != NULL) {

		/* Copy the token to token_obtained */ 
                strcpy(token_obtained, strptr);
 
		/* Adjust string */

        	/* Check if a delimiter follows the token */
        	strptr = strstr(temp2, token_delimiter);
 
		/* If one does */
		if (strptr != NULL)		

			/* Copy from after the token delimiter */
                	strcpy(string, strptr+strlen(token_delimiter));
		else 
			/* If we have just processed the last token 
			   in the string make the string empty */ 
			strcpy(string, "");
	} else
		strcpy(token_obtained, "NO SUCH TOKEN");
}






/* Get next non space character and check if character is that wanted. 
   If so return 1, if not, store character in buf and return 0 */
int nextis(char chgiven) 
{

	char ch;

	ch = next();
	if (ch != chgiven)
		buf = ch;

	/* Return 0 or 1 */
	return ch==chgiven;
}



/* Get next non space character (from buf (if present) then string) */
int next(void) 
{
	int ch;

	do {
		if (buf) {
			ch = buf;
			buf = 0;
		} else {
			ch = infix_str[infix_str_pos++];
		}
	} while (isspace(ch));
	return ch;
}





/* Convert following VSL expression from infix to postfix */
int expression(void) 
{

	/* Get first digit (digit*digit if present) and add to postfix string 
	   (digit*digit is converted to digit digit * before being added) */
	if (term() == -1)
		return -1;

	for(;;) {

		/* If next input character is + */
		if (nextis('+')) {

			/* get next digit (digit*digit if present) 
			   and add to postfix string 
			   (digit*digit is converted to digit digit * 
			   before being added) */
			if (term() == -1)
				return -1;


			/* then add the + to the postfix string */
			postfix_str[postfix_str_pos++] = '+';
		} else 
			if (nextis('-')) {
				if (term() == -1)
					return -1;
				postfix_str[postfix_str_pos++] = '-';
			} else
				break;
	}

	return 1;
}

    

 
int term(void)
{

	/* Get next digit and add to postfix string being formed  */
	if (factor() == -1)
		return -1;

	/* If the next character is a *, DONT yet add it
	   to the postfix string being formed. We now know though that
	   we have a digit*digit infix expression so ...*/
	while (nextis('*')) {

		/* Get second digit and add to postfix string */
		if (factor() == -1)
			return -1;

		/* NOW add the * found earlier to postfix string */
		postfix_str[postfix_str_pos++] = '*';

	}

	return 1;
}



/* Get next digit from infix string and add to postfix string */
int factor(void)
{
	char ch;

	/* Get next non space character from infix string */
	ch=next();
	if (isdigit(ch)) {

		/* Add digit (e.g. 1) to postfix string */
		postfix_str[postfix_str_pos++] = ch;


	/* If we dont have a next digit then we can have a complete
    	   rpn sub expression in brackets ! */
	} else
		if (ch == '(') {
			if (expression() == -1)
				return -1;
			if (!nextis(')')) {
				printf("HQL Query Error: ')' expected\n");
				return -1;
			}
			
		} else {
			printf("HQL Query Error: Operand or '(' expected\n");
			return -1;
		}


	return 1;
}









/* Bung vsl into Dbv rpn */
DbvRPNStructure *vsl_postfix_to_dbv_rpn(char *postfix_str, char *index_array[], Querytype query_types[], int last_query_type_index)
{

	DbvRPNStructure *formed_rpn = NULL;
	DbvRPNStructure *rpnstruct1 = NULL;
	DbvRPNStructure *rpnstruct2 = NULL;
	DbvRPNStructure *rpnstruct3 = NULL;
	char *temp = NULL;

	/* Query type index */
	int query_type_index = 0;

	int string_pos = 0;

	int operand_position = 0;
	char *operand_string = NULL;
	char operator_str[MAXVAL];


	/* Stack */

	/* Stack pointer */
	int sp = 0;

	/* The stack */
	DbvRPNStructure *val[MAXVAL];  

/*
	if operand (e.g. CAT)
		push to stack

	if operator (e.g. and)
		form rpn from operand and top two on stack
		push rpn back on stack
*/

	memset(val, 0, sizeof(val));
	
	while (string_pos != strlen(postfix_str)) {


		/* If operand (e.g. CAT ) */
		if (operand(postfix_str[string_pos])) {
			rpnstruct1 = (DbvRPNStructure *) DbvAlloc (sizeof(DbvRPNStructure));
			rpnstruct1->eRpnStructureType = RST_RpnOperand;

			/* Get the element out of index_array corressponding
			   to the VSL number */

			operand_position = postfix_str[string_pos] - '0';
			operand_string = index_array[operand_position];
			rpnstruct1->uRPNStructure.psRpnOperand = (DbvOperand *) DbvAlloc (sizeof(DbvOperand));

			/* Fill the operand */

			/* Get query type for operand */
			/* Only advace the query type index if another 
			   query type exists */
			if (operand_position <= last_query_type_index)
				query_type_index = operand_position;
			else
				query_type_index = last_query_type_index;

			fill_dbv_operand(rpnstruct1->uRPNStructure.psRpnOperand, operand_string, query_types[query_type_index]);
			push(rpnstruct1, &sp, val);

		/* If operator (e.g. and) */
		} else {
			rpnstruct1 = pop(&sp, val);
			rpnstruct2 = pop(&sp, val);
			rpnstruct3 = (DbvRPNStructure *) DbvAlloc (sizeof(DbvRPNStructure));

			rpnstruct3->eRpnStructureType = RST_RpnLopRopOperator;
			rpnstruct3->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *) DbvAlloc (sizeof(DbvRPNLopRopOperator));
			rpnstruct3->uRPNStructure.psRpnLopRopOperator->psLeftOperand = rpnstruct2;
			rpnstruct3->uRPNStructure.psRpnLopRopOperator->psRightOperand = rpnstruct1;


			/* VSL back to operator */
			switch (postfix_str[string_pos]) {
			case '*':
				strcpy(operator_str, OPERATOR_AND);
				rpnstruct3->uRPNStructure.psRpnLopRopOperator->sOperator.eOperatorType = OT_And;
				break;
			case '+':
				strcpy(operator_str, OPERATOR_OR);
				rpnstruct3->uRPNStructure.psRpnLopRopOperator->sOperator.eOperatorType = OT_Or;

				break;
			case '-':
				strcpy(operator_str, OPERATOR_ANDNOT);
				rpnstruct3->uRPNStructure.psRpnLopRopOperator->sOperator.eOperatorType = OT_AndNot;
				break;
			}

			
			/* Push it back */
			push(rpnstruct3, &sp, val);
		}
		string_pos++;

	}

	/* At the end of the string so get the final thing off the stack */
	formed_rpn = pop(&sp, val);
	return formed_rpn;
}



int fill_dbv_operand(DbvOperand *psRpnOperand, char *operand_string, Querytype query_type)
{
	if (query_type == QT_ResultSetName) {
		psRpnOperand->eOperandType = OT_ResultSetId;
		psRpnOperand->uOperand.psResultSetId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));

		/* The following function is in apishared.c */
		fill_dbv_inter_string(psRpnOperand->uOperand.psResultSetId, operand_string);
	} else {
		/* Attributes plus term */
		psRpnOperand->eOperandType = OT_AttributePlusTerm;
		psRpnOperand->uOperand.psAttributesPlusTerm = (DbvAttributesPlusTerm *) DbvAlloc ( sizeof(DbvAttributesPlusTerm));
		fill_dbv_attrsplusterm(psRpnOperand->uOperand.psAttributesPlusTerm, operand_string , query_type);
	}
}



int fill_dbv_attrsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm, char *operand_string, Querytype query_type)
{
	int i;

	/* Bib 1 search attribute sets for author, title and subject
	   searches */

	/* author attributes */
	char *author =  AUTHOR_ATTRIBUTES;

	/* title attributes */
	char *title = TITLE_ATTRIBUTES;

	/* subject attributes */
	char *subject = SUBJECT_ATTRIBUTES;

	/* explain attributes */
	char *explain = EXPLAINCAT_ATTRIBUTES;


	/* attributes */

	/* 6 attributes + 1 NULL */
	psAttributesPlusTerm->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * 7);

	/* This used to be a switch statement, but with it now having
	 * about 25 options, it was time for a change...
	 */
	for (i = 0; i < number_hql_supported_queries; i++)
	{
		if (query_type == hql_supported_queries[i].query_type)
		{
			fill_dbv_attterm(psAttributesPlusTerm->ppsAttributeElement, hql_supported_queries[i].query_attributes);
			break;
		}
	}

	/* Now the term */
	psAttributesPlusTerm->sTerm.eTermType = TT_General;
	psAttributesPlusTerm->sTerm.uTerm.psGeneral = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString)); 
	psAttributesPlusTerm->sTerm.uTerm.psGeneral->ulLength = strlen(operand_string);
	psAttributesPlusTerm->sTerm.uTerm.psGeneral->pcValue = (char *) DbvAlloc (strlen(operand_string));
	memcpy(psAttributesPlusTerm->sTerm.uTerm.psGeneral->pcValue, operand_string, strlen(operand_string));
}



int fill_dbv_attterm(DbvAttributeElement **ppsAttributeElement, char *attributes)
{
	int index = 0;
	char *ptr = NULL;
	unsigned long atttype = 0, attval = 0;
	char attributes_copy[MAXVAL];


	strcpy(attributes_copy, attributes);

	ptr = (char *) strtok (attributes_copy, ",");	

	while (ptr != NULL) {
		atttype= atol(ptr);
		attval = atol((char *) strtok(NULL, ":"));

		ppsAttributeElement[index] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));

		/* DONT need an object identifier for each attribute pair */

		ppsAttributeElement[index]->ulAttributeType = atttype;
		ppsAttributeElement[index]->sAttributeValue.eAttributeValueType = AVT_Numeric; 
		ppsAttributeElement[index]->sAttributeValue.uAttributeValue.ulNumeric = attval;  


		ptr = (char *) strtok (NULL, ",");

		/* Set term */
		index++;
	}

	/* Terminate the list */
	ppsAttributeElement[index] = NULL;
}



void push(DbvRPNStructure *rpnstruct, int *sp, DbvRPNStructure *val[])
{
	if (*sp < MAXVAL)

		/* Following increments value in *sp, not sp */
		val[(*sp)++] = rpnstruct;
	else
		printf("HQL Query Error: Internal query parsing error (stack full)\n");
}


		

DbvRPNStructure *pop(int *sp, DbvRPNStructure *val[])
{
	if (*sp > 0)

		/* Following decrements value in *sp, not sp */
		return val[--(*sp)];
	else {
		printf("HQL Query Error: Internal query parsing error (stack empty)\n");
		return NULL;
	}
	
}

		

int operand(char ch) 
{
	if ( (ch != '*') && (ch != '+') && (ch != '-') )
		return 1;
	else
		return 0;

}



