/*  Module Name: @(#)api_origin_search_parser.h	7.2
    Version: 12/2/96
    Author: John Hough

    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description: Query to DbvRPNStructure generator header file.
*/


#ifndef api_origin_search_parser_h
#define api_origin_search_parser_h


#include "api_origin_includes.h"

/* Definitions */
typedef enum Querytype {
        QT_Unknown	= -1,
        QT_Author	= 1,
        QT_Title	= 2,
        QT_Subject	= 3,

        QT_Explaincat	= 4,
	QT_Hsl		= 5,
	QT_Dbase	= 6,
	QT_Target	= 7,
	QT_Attrsetoid	= 8,
	QT_Recsynoid	= 9,
	QT_Tagsetoid	= 10,
	QT_Extservoid	= 11,
	QT_Dateadd	= 12,
	QT_Datechange	= 13,
	QT_Dateexp	= 14,
	QT_Elesetname	= 15,
	QT_Proccontext	= 16,
	QT_Procname	= 17,
	QT_Tlistname	= 18,
	QT_Schemaoid	= 19,
	QT_Producer	= 20,
	QT_Supplier	= 21,
	QT_Avail	= 22,
	QT_Propriety	= 23,
	QT_Userfee	= 24,
	QT_Varsetoid	= 25,
	QT_Unitsystem	= 26,
	QT_Keyword	= 27,
	QT_Explaindbase	= 28,
	QT_Processoid	= 29,

	/* Special for CIP explain */
	QT_CipCollectId	= 30,

	QT_ResultSetName = 31,

	QT_DateAddedLt= 32,	
	QT_DateAddedLe= 33,
	QT_DateAddedEq= 34,
	QT_DateAddedGe= 35,
	QT_DateAddedGt= 36,
	QT_DateAddedNe= 37,

	QT_DateChangedLt= 38,
	QT_DateChangedLe= 39,
	QT_DateChangedEq= 40,
	QT_DateChangedGe= 41,
	QT_DateChangedGt= 42,
	QT_DateChangedNe= 43,

	QT_DateExpiresLt= 44,
	QT_DateExpiresLe= 45,
	QT_DateExpiresEq= 46,
	QT_DateExpiresGe= 47,
	QT_DateExpiresGt= 48,
	QT_DateExpiresNe= 49

        } Querytype;
 
#define MAXVAL 1000

/* Bib-1 searches */ 
#define AUTHOR_SEARCH		"author"
#define SUBJECT_SEARCH		"subject"
#define TITLE_SEARCH		"title"

/* Exp-1 searches */
#define EXPLAINCAT_SEARCH	"explaincategory"
#define HSL_SEARCH		"humanstringlang"
#define DBASE_SEARCH		"dbasename"
#define TARGET_SEARCH		"targetname"
#define ATTRSETOID_SEARCH	"attrsetoid"
#define RECSYNOID_SEARCH	"recsynoid"
#define TAGETSETOID_SEARCH	"tagetsetoid"
#define EXTSERVICEOID_SEARCH	"extserviceoid"
#define DATEADD_SEARCH		"dateadd"
#define DATECHANGE_SEARCH	"datechange"
#define DATEEXPIRES_SEARCH	"dateexpires"
#define ELEMENTSETNAME_SEARCH	"elementsetname"
#define PROCCONTEXT_SEARCH	"proccontext"
#define PROCNAME_SEARCH		"procname"
#define TERMLISTNAME_SEARCH	"termlistname"
#define SCHEMAOID_SEARCH	"schemaoid"
#define PRODUCER_SEARCH		"producer"
#define SUPPLIER_SEARCH		"supplier"
#define AVAILABILITY_SEARCH	"avail"
#define PROPRIETY_SEARCH	"propriety"
#define USERFEE_SEARCH		"userfee"
#define VARIANTSETOID_SEARCH	"variantsetoid"
#define UNITSYSTEM_SEARCH	"unitsystem"
#define KEYWORD_SEARCH		"keyword"
#define EXPLAINDBASE_SEARCH	"explaindbase"
#define PROCESSINGOID_SEARCH	"processingoid"

/* Special for CIP */
#define COLLECTID_SEARCH	"collectionid"

#define RESULTSETNAME_SEARCH	"resultsetname"

#define DATEADDED_LT_SEARCH	"dateaddedlt"
#define DATEADDED_LE_SEARCH	"dateaddedle"
#define DATEADDED_EQ_SEARCH	"dateaddedeq"
#define DATEADDED_GE_SEARCH	"dateaddedge"
#define DATEADDED_GT_SEARCH	"dateaddedgt"
#define DATEADDED_NE_SEARCH	"dateaddedne"

#define DATECHANGED_LT_SEARCH	"datechangedlt"
#define DATECHANGED_LE_SEARCH	"datechangedle"
#define DATECHANGED_EQ_SEARCH	"datechangedeq"
#define DATECHANGED_GE_SEARCH	"datechangedge"
#define DATECHANGED_GT_SEARCH	"datechangedgt"
#define DATECHANGED_NE_SEARCH	"datechangedne"

#define DATEEXPIRES_LT_SEARCH	"dateexpireslt"
#define DATEEXPIRES_LE_SEARCH	"dateexpiresle"
#define DATEEXPIRES_EQ_SEARCH	"dateexpireseq"
#define DATEEXPIRES_GE_SEARCH	"dateexpiresge"
#define DATEEXPIRES_GT_SEARCH	"dateexpiresgt"
#define DATEEXPIRES_NE_SEARCH	"dateexpiresne"



/* Query type - value seperators */
#define QUERY_TYPE_VAL_SEP	":"
#define QUERY_TYPE_VAL_SEP_CHAR	':'

/* Query type seperators */
#define QUERY_TYPE_SEP	","

#define OPERATOR_AND	"and"
#define OPERATOR_OR	"or"
#define OPERATOR_ANDNOT	"andnot"
#define OPENING_BRACKET	"("
#define CLOSING_BRACKET	")"


#define TOKEN_SEP	" "

/* Complex term seperator must be single character */
#define COMPLEX_SEP	'/'

/* Attributes */

/* Bib-1 */
#define TITLE_ATTRIBUTES		"1,4:3,3:4,6:6,1:"
#define AUTHOR_ATTRIBUTES		"1,1003:3,3:4,6:6,1:"
#define SUBJECT_ATTRIBUTES		"1,21:3,3:4,6:6,1:"

/* Exp-1 */
#define EXPLAINCAT_ATTRIBUTES		"1,1:2,3:3,3:4,3:"
#define HSL_ATTRIBUTES			"1,2:2,3:3,3:4,3:"
#define DBASE_ATTRIBUTES		"1,3:2,3:3,3:4,3:"
#define TARGET_ATTRIBUTES		"1,4:2,3:3,3:4,3:"
#define ATTRSETOID_ATTRIBUTES		"1,5:2,3:3,3:4,3:"
#define RECSYNOID_ATTRIBUTES		"1,6:2,3:3,3:4,3:"
#define TAGETSETOID_ATTRIBUTES		"1,7:2,3:3,3:4,3:"
#define EXTSERVICEOID_ATTRIBUTES	"1,8:2,3:3,3:4,3:"
#define DATEADD_ATTRIBUTES		"1,9:2,3:3,3:4,3:"
#define DATECHANGE_ATTRIBUTES		"1,10:2,3:3,3:4,3:"
#define DATEEXPIRES_ATTRIBUTES		"1,11:2,3:3,3:4,3:"
#define ELEMENTSETNAME_ATTRIBUTES	"1,12:2,3:3,3:4,3:"
#define PROCCONTEXT_ATTRIBUTES		"1,13:2,3:3,3:4,3:"
#define PROCNAME_ATTRIBUTES		"1,14:2,3:3,3:4,3:"
#define TERMLISTNAME_ATTRIBUTES		"1,15:2,3:3,3:4,3:"
#define SCHEMAOID_ATTRIBUTES		"1,16:2,3:3,3:4,3:"
#define PRODUCER_ATTRIBUTES		"1,17:2,3:3,3:4,3:"
#define SUPPLIER_ATTRIBUTES		"1,18:2,3:3,3:4,3:"
#define AVAILABILITY_ATTRIBUTES		"1,19:2,3:3,3:4,3:"
#define PROPRIETY_ATTRIBUTES		"1,20:2,3:3,3:4,3:"
#define USERFEE_ATTRIBUTES		"1,21:2,3:3,3:4,3:"
#define VARIANTSETOID_ATTRIBUTES	"1,22:2,3:3,3:4,3:"
#define UNITSYSTEM_ATTRIBUTES		"1,23:2,3:3,3:4,3:"

/* Note - no 1,24 ! */

#define KEYWORD_ATTRIBUTES		"1,25:2,3:3,3:4,3:"
#define EXPLAINDBASE_ATTRIBUTES		"1,26:2,3:3,3:4,3:"
#define PROCESSINGOID_ATTRIBUTES	"1,27:2,3:3,3:4,3:"


/* Special for CIP explain */

/* CIP attributes are defined in Appendix A of the CIP specification
   Release A.

   Format: attribute category, attribute value:

   For COLLECTID_ATTRIBUTES below...
   1, 1: use, collection id
   2, 3: relation, equal
   3, 1: position, first in field
   4, 2: structure, word (the structure attribute value associated with a
	 use attribute (here 2) is indicated in the CIP
	 specification in column 6 'Str' of the use attribute category
         table) 
   5, 4: truncation, do not truncate
   6, 3: completeness, complete field

   In HQL, the search category corresponds to the use attribute value. 
   Hence the search category for COLLECTID_ATTRIBUTES is 

   #define COLLECTID_SEARCH		"collectionid"

   (for BIB1 searches, the "title" category forces use of the bib1 use
   attribute of title)

   Oid are defined in Section 4 of the CIP Specification Release A.
*/
 
#define COLLECTID_ATTRIBUTES		"1,1:2,3:3,1:4,2:5,4:6,3"

/* Dummy attributes for resultset name searches */
#define RESULTSETNAME_ATTRIBUTES	"1,1:"

#define DATEADDED_LT_ATTRIBUTES		"1,9:2,1:3,3:4,3:"
#define DATEADDED_LE_ATTRIBUTES		"1,9:2,2:3,3:4,3:"
#define DATEADDED_EQ_ATTRIBUTES		"1,9:2,3:3,3:4,3:"
#define DATEADDED_GE_ATTRIBUTES		"1,9:2,4:3,3:4,3:"
#define DATEADDED_GT_ATTRIBUTES		"1,9:2,5:3,3:4,3:"
#define DATEADDED_NE_ATTRIBUTES		"1,9:2,6:3,3:4,3:"

#define DATECHANGED_LT_ATTRIBUTES	"1,10:2,1:3,3:4,3:"
#define DATECHANGED_LE_ATTRIBUTES	"1,10:2,2:3,3:4,3:"
#define DATECHANGED_EQ_ATTRIBUTES	"1,10:2,3:3,3:4,3:"
#define DATECHANGED_GE_ATTRIBUTES	"1,10:2,4:3,3:4,3:"
#define DATECHANGED_GT_ATTRIBUTES	"1,10:2,5:3,3:4,3:"
#define DATECHANGED_NE_ATTRIBUTES	"1,10:2,6:3,3:4,3:"

#define DATEEXPIRES_LT_ATTRIBUTES	"1,11:2,1:3,3:4,3:"
#define DATEEXPIRES_LE_ATTRIBUTES	"1,11:2,2:3,3:4,3:"
#define DATEEXPIRES_EQ_ATTRIBUTES	"1,11:2,3:3,3:4,3:"
#define DATEEXPIRES_GE_ATTRIBUTES	"1,11:2,4:3,3:4,3:"
#define DATEEXPIRES_GT_ATTRIBUTES	"1,11:2,5:3,3:4,3:"
#define DATEEXPIRES_NE_ATTRIBUTES	"1,11:2,6:3,3:4,3:"


/*****************************************************
/* Old defunct attributes 

These sets include zero for unwanted attribute tables, as accepted by some
American V2 targets 

#define AUTHOR_ATTRIBUTES	"1,1003:2,0:3,3:4,6:5,0:6,1:"
#define TITLE_ATTRIBUTES	"1,4:2,0:3,3:4,6:5,0:6,1:"
#define SUBJECT_ATTRIBUTES	"1,21:2,0:3,3:4,6:5,0:6,1:"

British library development attributes
#define AUTHOR_ATTRIBUTES	"1,1003:2,3:"
#define TITLE_ATTRIBUTES	"1,4:2,3:"
#define SUBJECT_ATTRIBUTES	"1,5:2,3:"
*****************************************************/




/* Function prototypes */
 
DbvRPNStructure *form_dbv_rpn(char *query); 
int vsl_infix_to_vsl_postfix();
int querytypes_to_lower_case(char *query);
void get_query_types_and_val(char *input_string, char *query_val, Querytype query_types[], int *num_query_types);
int query_val_to_vsl_infix(char *input_string, char *vsl_string, char *index_array[]);
DbvBool operator_check(char *token_obtained);
int operator_process(char *token_obtained, char *vsl_string, char *input_string_copy);
int operand_process(char *token_obtained, char *vsl_string, char *input_string_copy, int *insert_pos, char *index_array[]);
int complex_operand_check_and_process(char *token_obtained, char *input_string_copy);
int remove_leading(char *string, char char_to_remove);
void api_get_token_from_string(char *token_obtained, char *token_delimiter, char *string);
int nextis(char chgiven); 
int next(void); 
int expression(void); 
int term(void);
int factor(void);
DbvRPNStructure *vsl_postfix_to_dbv_rpn(char *postfix_str, char *index_array[], Querytype query_types[], int last_query_type_index);
int fill_dbv_operand(DbvOperand *psRpnOperand, char *operand_string, Querytype query_type);
int fill_dbv_attrsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm, char *operand_string, Querytype query_type);
int fill_dbv_attterm(DbvAttributeElement **ppsAttributeElement, char *attributes);
void push(DbvRPNStructure *rpnstruct, int *sp, DbvRPNStructure *val[]);
DbvRPNStructure *pop(int *sp, DbvRPNStructure *val[]);
int operand(char ch); 

#endif
