#include "api_target_access.h"



/* Set up an AccessControlRequest PDU and sent it to an origin */
int send_accessreq_to_origin(DbvAssocId assoc_id)
{
	DbvError 		*psError = NULL;
	DbvAccessControlReq	*psAccessControlReq = NULL;
	
	int choice = 0;
	char choice_string[100];

	/* Allocate memory for psAccessControlReq and fill it in */
        psAccessControlReq = (DbvAccessControlReq *) DbvAlloc (sizeof(DbvAccessControlReq));
        fill_access_request(psAccessControlReq);
 
	printf("Print AccessControlRequest PDU to screen before sending ? yes (1), no (2): ");
	choice = atoi(gets(choice_string));
	if (choice == 1) {
		printf("-----------------------------------------\n");
		print_accessreq_fields(stdout, psAccessControlReq);
		printf("-----------------------------------------\n");
	}

 
        /* Send the AccessControlRequest just filled in to the origin */
        if (DbvAccessControlRequest(assoc_id, psAccessControlReq, &psError) == SI_OK) {

		/* Sent PDU to target ok so free up space it used */ 

		SIFreeDbvAccessControlReq(&psAccessControlReq);

		return SI_OK;
	} else {
		
		/* The attempt to send the PDU to the target failed */
                PrivateErrorHandling(&psError);

                SIFreeDbvAccessControlReq(&psAccessControlReq);


                /* Release buffer, assign NULL pointer */
                FreeSIErrorCode(&psError);
		return SI_NOTOK;
	}
}



int fill_access_request(DbvAccessControlReq *psAccessControlReq)
{

	int choice = 0;
	char choice_string[100];


	char *test_refid = "access control request reference id";
	char *test_simpleform = "access control request simple form";
	char *test_sutrs = "access control request test sutrs";
	char *diag_dbname = "test diag database";
	char *diag_text = "test diag text";
	char *test_otherinfo1 = "first other information test";
	char *test_otherinfo2 = "second other information test";


        /* Compulsory fields... */


	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter eSecurityChallengeReqType type: SECT_SimpleForm (1), SECT_ExternallyDefined (2): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
        	psAccessControlReq->sSecurityChallengeReq.eSecurityChallengeReqType = SECT_SimpleForm;
        	psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psSimpleForm = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		fill_dbv_octet_string(psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psSimpleForm, test_simpleform);
		break;
	case 2:
        	psAccessControlReq->sSecurityChallengeReq.eSecurityChallengeReqType = SECT_ExternallyDefined;
        	psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psExternallyDefinedInfo = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		fill_DbvPromptObject_external(psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psExternallyDefinedInfo);

		break;
	}




       	/* Optional fields... */
 
	/* +1 to allocate space for the \0 */
        psAccessControlReq->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
	fill_dbv_octet_string(psAccessControlReq->psReferenceId, test_refid);



	/* 2 nodes */
	psAccessControlReq->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc(sizeof(DbvOtherInformation *) * 3);

	psAccessControlReq->ppsOtherInformation[0] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));

	psAccessControlReq->ppsOtherInformation[1] = (DbvOtherInformation *) DbvAlloc(sizeof(DbvOtherInformation));
	psAccessControlReq->ppsOtherInformation[2] = NULL;


	fill_dbv_otherinfo_char(psAccessControlReq->ppsOtherInformation[0], DBV_TRUE, ANSI_UNSTRUCTURED_TEXT_RECORD, 989, test_otherinfo1);
	fill_dbv_otherinfo_char(psAccessControlReq->ppsOtherInformation[1], DBV_TRUE, ANSI_UNSTRUCTURED_TEXT_RECORD, 999, test_otherinfo2);
}
  


/*********************************************************************/

/* From Appendix 7 (ACC) : Access control formats are defined for use 
within the parameters securityChallenge and securityChallengeResponse 
of the
AccessControlRequest and AccessControlResponse APDUs, 
and idAuthentication of the InitializeRequest APDU */

void fill_DbvPromptObject_external(DbvASN1External *psASN1External)
{
	char	*oid = ANSI_ACCESS_PROMPT_1;

	/* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */

	/* +1 to copy the \0 */
	psASN1External->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(psASN1External->pcObjectIdentifier, oid, strlen(oid));

	psASN1External->eValueType = EVT_SingleASN1Type;
	psASN1External->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
	psASN1External->uExternalValue.psSingleASN1Type->eASN1Type = AT_PromptObject;
	psASN1External->uExternalValue.psSingleASN1Type->uASN1.psPromptObject = (struct DbvPromptObject *) DbvAlloc (sizeof (struct DbvPromptObject ));
	fill_DbvPromptObject(psASN1External->uExternalValue.psSingleASN1Type->uASN1.psPromptObject);
}



void fill_DbvPromptObject(DbvPromptObject *psPromptObject) 
{
	int choice = 0;
	char choice_string[100];


	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter ePromptObjectType type: POT_Challenge (1), POT_Response (2): ");
		choice = (atol(gets(choice_string)));
	}
			

	switch (choice) {
	case 1:
		psPromptObject->ePromptObjectType = POT_Challenge;

		/* set up 2 nodes */
		psPromptObject->uPromptObject.ppsChallenge = (DbvChallenge **) DbvAlloc(sizeof(DbvChallenge *) * 3);

		psPromptObject->uPromptObject.ppsChallenge[0] = (DbvChallenge *) DbvAlloc(sizeof(DbvChallenge));
		psPromptObject->uPromptObject.ppsChallenge[1] = (DbvChallenge *) DbvAlloc(sizeof(DbvChallenge));

		psPromptObject->uPromptObject.ppsChallenge[2] = NULL;
	
		/* fill em */
		fill_DbvChallenge(psPromptObject->uPromptObject.ppsChallenge[0], PIIT_Character);
		fill_DbvChallenge(psPromptObject->uPromptObject.ppsChallenge[1], PIIT_Encrypted);
	
		break;
	case 2:
		psPromptObject->ePromptObjectType = POT_Response;

		/* set up 5 nodes (with /0 terminator) */
		psPromptObject->uPromptObject.ppsResponse = (DbvResponse **) DbvAlloc(sizeof(DbvResponse *) * 6);

		psPromptObject->uPromptObject.ppsResponse[0] = (DbvResponse *) DbvAlloc(sizeof(DbvResponse));
		psPromptObject->uPromptObject.ppsResponse[1] = (DbvResponse *) DbvAlloc(sizeof(DbvResponse));
		psPromptObject->uPromptObject.ppsResponse[2] = (DbvResponse *) DbvAlloc(sizeof(DbvResponse));
		psPromptObject->uPromptObject.ppsResponse[3] = (DbvResponse *) DbvAlloc(sizeof(DbvResponse));
		psPromptObject->uPromptObject.ppsResponse[4] = (DbvResponse *) DbvAlloc(sizeof(DbvResponse));

		psPromptObject->uPromptObject.ppsResponse[5] = NULL;


		/* fill em */
		fill_DbvResponse(psPromptObject->uPromptObject.ppsResponse[0], RT_String);
		fill_DbvResponse(psPromptObject->uPromptObject.ppsResponse[1], RT_Accept);
		fill_DbvResponse(psPromptObject->uPromptObject.ppsResponse[2], RT_Acknowledge);
		fill_DbvResponse(psPromptObject->uPromptObject.ppsResponse[3], RT_Diagnostic);
		fill_DbvResponse(psPromptObject->uPromptObject.ppsResponse[4], RT_Encrypted);

		break;
	}
}



void fill_DbvChallenge(DbvChallenge *psChallenge, DbvPromptInfoType ePromptInfoType)
{
	/* c */
	fill_DbvPromptId(&psChallenge->sPromptId, PIT_EnumeratedPrompt);

	/* o */ 
	psChallenge->psDefaultResponse = (DbvInternationalString *)DbvAlloc (sizeof (DbvInternationalString));
	fill_dbv_inter_string(psChallenge->psDefaultResponse, "challenge default response");
	
	/* o */ 
	psChallenge->psPromptInfo = (DbvPromptInfo *)DbvAlloc (sizeof (DbvPromptInfo));
	fill_DbvPromptInfo(psChallenge->psPromptInfo, ePromptInfoType);
	
	/* o */ 
	psChallenge->psRegExpr = (DbvInternationalString *)DbvAlloc (sizeof (DbvInternationalString));
	fill_dbv_inter_string(psChallenge->psRegExpr, "regular expression 45*(6*8)");
	
	/* o  : leave NULL or alloc 1 byte */ 
	psChallenge->pvResponseRequired = DbvAlloc(1);
	
	/* o */ 
	/* set up 2 nodes */
	psChallenge->ppsAllowedValues = (DbvInternationalString **) DbvAlloc(sizeof(DbvInternationalString *) * 3);

	psChallenge->ppsAllowedValues[0] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	psChallenge->ppsAllowedValues[1] = (DbvInternationalString *) DbvAlloc(sizeof(DbvInternationalString));
	psChallenge->ppsAllowedValues[2] = NULL;


	/* fill em */
	fill_dbv_inter_string(psChallenge->ppsAllowedValues[0], "first allowed value");
	fill_dbv_inter_string(psChallenge->ppsAllowedValues[1], "second allowed value");

	
	/* o  : leave NULL or alloc 1 byte */ 
	psChallenge->pvShouldSave = DbvAlloc(1);
	
	/* o */ 
	psChallenge->peDataType = (DbvDataType *)DbvAlloc (sizeof (DbvDataType));
	*psChallenge->peDataType = DTTT_AlphaNumeric;
	
	/* o */ 
	/* The old faithfull ansi sutrs but could be any
	   external */
	psChallenge->psDiagnostic = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
	fill_dbv_sutrs_external(psChallenge->psDiagnostic, "this is a sutrs diagnostic");

}



void fill_DbvPromptId(DbvPromptId *psPromptId, DbvPromptIdType ePromptIdType)
{
	switch (ePromptIdType) {
	case PIT_EnumeratedPrompt:
		psPromptId->ePromptIdType = PIT_EnumeratedPrompt;
		psPromptId->uPromptId.psEnumeratedPrompt = (DbvEnumeratedPrompt *) DbvAlloc (sizeof(DbvEnumeratedPrompt));
		fill_DbvEnumeratedPrompt(psPromptId->uPromptId.psEnumeratedPrompt);
		break;
	case PIT_NoneEnumeratedPrompt:
		psPromptId->ePromptIdType = PIT_NoneEnumeratedPrompt;
		psPromptId->uPromptId.psNonEnumeratedPrompt = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		fill_dbv_inter_string(psPromptId->uPromptId.psNonEnumeratedPrompt, "non enumerated prompt test");
		break;
	}
}




void fill_DbvEnumeratedPrompt(DbvEnumeratedPrompt *psEnumeratedPrompt)
{
	/* c */
	psEnumeratedPrompt->eType = EPIT_Copyright;

	/* o */
	psEnumeratedPrompt->psSuggestedString = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
	fill_dbv_inter_string(psEnumeratedPrompt->psSuggestedString, "suggested string test");
	
}



void fill_DbvPromptInfo(DbvPromptInfo *psPromptInfo, DbvPromptInfoType ePromptInfoType)
{
	switch (ePromptInfoType) {
	case PIIT_Character:
		psPromptInfo->ePromptInfoType = PIIT_Character;
		psPromptInfo->uPromptInfo.psCharacter = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		fill_dbv_inter_string(psPromptInfo->uPromptInfo.psCharacter, "prompt info character test");
		break;
	case PIIT_Encrypted:
		psPromptInfo->ePromptInfoType = PIIT_Encrypted;
		psPromptInfo->uPromptInfo.psEncrypted = (DbvEncryption *) DbvAlloc (sizeof (DbvEncryption));
		fill_DbvEncryption(psPromptInfo->uPromptInfo.psEncrypted);
		break;
	}
}



void fill_DbvEncryption(DbvEncryption *psEncrypted)
{
	/* o */
	psEncrypted->psCryptType = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
	fill_dbv_octet_string(psEncrypted->psCryptType, "encryption crypt type test");

	/* o */
	psEncrypted->psCredential = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
	fill_dbv_octet_string(psEncrypted->psCredential, "encryption credential test");

	/* c */ 
	fill_dbv_octet_string(&psEncrypted->sData, "encryption sdata test");
}



void fill_DbvResponse(DbvResponse *psResponse, DbvResponseType eResponseType)
{
	/* c */
	fill_DbvPromptId(&psResponse->sPromptId, PIT_NoneEnumeratedPrompt);

	/* c */
	fill_DbvPromptResponse(&psResponse->sPromptResponse, eResponseType);
}



void fill_DbvPromptResponse(DbvPromptResponse *psPromptResponse, DbvResponseType eResponseType)
{
	switch (eResponseType) {
	case RT_String:
		psPromptResponse->eResponseType = RT_String;
		psPromptResponse->uResponse.psString = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		fill_dbv_inter_string(psPromptResponse->uResponse.psString, "prompt response string test");
		break;
	
	case RT_Accept:
		psPromptResponse->eResponseType = RT_Accept;
		psPromptResponse->uResponse.fAccept = DBV_TRUE; 
		break;
	
	case RT_Acknowledge:
		psPromptResponse->eResponseType = RT_Acknowledge;

		/* no extra set up required for RT_Acknowledge */
		break;
	
	case RT_Diagnostic:
		psPromptResponse->eResponseType = RT_Diagnostic;
		psPromptResponse->uResponse.psDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
		fill_default_diag_rec(psPromptResponse->uResponse.psDiagnostic, ANSI_DANMARC, 234, "diagnostic error string test", PVIF_Version2, DT_DefaultFormat, NULL); 
		break;

	case RT_Encrypted:
		psPromptResponse->eResponseType = RT_Encrypted;
		psPromptResponse->uResponse.psEncryption = (DbvEncryption *) DbvAlloc (sizeof (DbvEncryption));
		fill_DbvEncryption(psPromptResponse->uResponse.psEncryption);
		break;
	}
}




