#include "apitestorigin.h"


/* State machine off/on variable in API (DBV_FALSE | DBV_TRUE) */
extern int DBVStateMachine;

DbvAssocId      alAssocId[MAX_ASSOCS];
DbvAssocId      current_id = NO_ASSOCIATION_ID_ASSIGNED;


int main()
{
	int index, choice = 0;

	/* TCP/IP address components */
	char host_address[1000];
	int port = 0;

	/* OSI address components */
	char psel[1000];
	char ssel[1000];
	char tsel[1000];
	char nsap[1000];


	/* Database name */
	char dbname[1000];
	AddressType address_type_chosen = AT_tcpip;

	DbvAssocId		assoc_id = NO_ASSOCIATION_ID_ASSIGNED;
	DbvAssociateState	assoc_state = Associate_Null;
	DbvAssociateResult      *assoc_res = NULL;
	DbvReleaseState		rel_state = Release_Null;
	DbvError	       *psError = NULL;

	/* Each association may have its own database name and other details  
	   so create an array of structures to hold such information */ 
	Dbnamestruct 		assoc_dbnames[MAX_ASSOCS];

/* If MALLOC_DEBUG resolves to non zero (MALLOC_DEBUG is a Sun Solaris 1 
   feature only) */
#ifdef MALLOC_DEBUG
	malloc_debug(2);
#endif

	/* Internal state machine toggle (DBV_FALSE | DBV_TRUE) */
	DBVStateMachine = DBV_TRUE;


	/* Default target - Tcp/ip target at host address and port */
	strcpy(host_address, "sun001.sil.com");
	port = 2020;
	address_type_chosen = AT_tcpip;

	/* Clear all elements of the global association array */
	for (index = 0; index < MAX_ASSOCS; index++)
		alAssocId[index] = 0;

	printf("Setting ACSE Configuration file to %s\n", ACSE_CONFIG_FILE);

	if(SISetACSEFile(ACSE_CONFIG_FILE, &psError)==SI_NOTOK)
	{
		if(psError->eErrorClass == SI_Interface_Error &&
			psError->uErrorCode.lIfOsCoError == DBV_FILE)
		{
			printf("Warning: Could not open file: %s\n",
				ACSE_CONFIG_FILE);
		}
	}
			
	
	/* While exit not chosen */ 
	choice = menu_choice();
	while (choice != 30) {
		process_choice(choice, host_address, &port, &assoc_id, &assoc_state, &assoc_res, &rel_state, &psError, assoc_dbnames, dbname, &address_type_chosen, psel, ssel, tsel, nsap);
		choice = menu_choice();
	}
}
 



int menu_choice()
{
	char *result = NULL;
	int operation_choice = 0;
	char operation_choice_str[1000];
	char *menu_header = NULL;

	printf("\n\n\n");


	menu_header = "DBV-OSI II Origin Test Program (c) CSL \n";

	printf("%s", menu_header);
	printf("----------------------------------------\n");

	printf(" 0) About\n");
	printf(" 1) Specify Target Internet Address\n");
	printf(" 2) Specify Target OSI Address\n");
	printf(" 3) Specify Target by Name\n");
	printf(" 4) Connect To Target\n");
	printf(" 5) Send InitializeRequest to Target\n");
	printf(" 6) Send SearchRequest to Target\n");
	printf(" 7) Send PresentRequest to Target\n");
	printf(" 8) Send DeleteResultSetRequest to Target\n");
	printf(" 9) Send ResourceReportRequest to Target\n");
	printf("10) Send ScanRequest to Target\n");
	printf("11) Send SortRequest to Target\n");
	printf("12) Send ExtendedServicesRequest with 'Update' to Target\n");

	printf("13) Send ExtendedServicesRequest with 'ItemOrder,ILL' to Target\n");

	printf("14) Send 'CloseRequest' (Close PDU) to Target\n");
	printf("15) Send 'CloseResponse' (Close PDU) to Target\n");
	printf("16) Release current Association\n");
	printf("17) Abort current Association\n");
	printf("18) Select Current Association\n");
	printf("19) Listen for data on all Associations\n");

	printf("20) Image Retrieval Test Receive (Generic Record SegmentReq/PresentRsp)\n");

	printf("21) Send AccessControlResponse to Target\n");
	printf("22) Send ResourceControlResponse to Target\n");
	printf("23) Send TriggerResourceControlRequest to Target\n");


	printf("24) Send ExtendedServicesRequest with 'Esrin Print' to Target\n");
	printf("25) Send ExtendedServicesRequest with 'Esrin Alert' to Target\n");
	printf("26) Send ExtendedServicesRequest with 'ItemOrder, Esrin COrder' to Target\n");
	printf("27) Send InitializeRequest to Target containing CharSet external \n");

	printf("30) Exit\n");
	printf("Current association id = %d\n",current_id);

	printf("Enter choice: ");

	result = gets(operation_choice_str);
	post_gets_check(result);
	operation_choice = atoi(operation_choice_str);

	/* Return the choosen option */
	return operation_choice;

}



int process_choice(int choice, char *host_address, int *port, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, Dbnamestruct *assoc_dbnames, char *dbname, AddressType *address_type_chosen, char *psel, char *ssel, char *tsel, char *nsap)
{
	int return_value = SI_OK;
	char str[10];
	int choice2;
	static DbvBool done_initial_association = DBV_FALSE;

	/* Perform the users menu choice */
	switch (choice) {
	case 0:
		output_about_message();
		break;
	case 1:
		specify_tcpip_target(host_address, port, dbname);
		*address_type_chosen = AT_tcpip;
		break;

	case 2:
		specify_osi_target(host_address, psel, ssel, tsel, nsap);
		*address_type_chosen = AT_osi;
		break;
	case 3:
		specify_byname_target(host_address);
		*address_type_chosen = AT_byname;
		break;
	case 4:
		connect_to_target(host_address, port, assoc_id, assoc_state, assoc_res, rel_state, error, *address_type_chosen, psel, ssel, tsel, nsap);
		current_id = *assoc_id;

		/* Set up a structure containing details related to the
		   current association */

		/* Allocate + 1 to allocate space for the terminating NULL */
		assoc_dbnames[current_id].dbname = (char *) DbvAlloc (strlen(dbname) + 1);
		memcpy(assoc_dbnames[current_id].dbname, dbname, strlen(dbname));

		/* An association has been done so now PDUs can be sent */ 
		if (! done_initial_association)
			done_initial_association = DBV_TRUE;

		break;
	case 5:
                do_initreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 6:

		choice2 = 0;
                while (choice2 < 1 || choice2 > 3)
                {
                        printf("Search type: Bib-1(1), Explain(2), CIP(3): ");
                        choice2 = atoi(gets(str));
                }
                switch (choice2)
                {
                        case 1:
                                do_srchreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                                break;

                        case 2:
                                do_srchreq_explain(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                                break;
                        case 3:
                                do_srchreq_explain_cip(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                                break;
		}

		break;

        case 7:
                choice2 = 0;
                while (choice2 != 1 && choice2 != 2)
                {
                        printf("Present type: Bib-1(1), Explain(2): ");
                        choice2 = atoi(gets(str));
                }
                switch (choice2)
                {
                        case 1:
                                do_presreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                                break;
                        case 2:
                                do_exppresreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                                break;
                }
                break;

        case 8:
                do_delreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 9:
                do_resrepreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 10:
 
                do_scanreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 11:
                do_sortreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 12:
                do_DbvExtendedServicesReq_DbvUpdateExtendedService(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 13:
                do_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvIllPdu(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;

        case 14:
                do_closereq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;
        case 15:
                do_closersp(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;

	case 16:
		close_connection_to_target(assoc_id, assoc_state, assoc_res, rel_state, error);
		remove_assoc(*assoc_id);
		*assoc_id = current_id = -1;
		break;
	case 17:
		abort_connection_to_target(assoc_id, assoc_state, assoc_res, rel_state, error);
		remove_assoc(*assoc_id);
		*assoc_id = current_id = -1;
		break;
		
	case 18:
		current_id = select_current_assoc();
		*assoc_id = current_id;
		break;
	case 19: wait_for_any_responses(assoc_id, assoc_state, assoc_res, rel_state, error);
		break;


	/* Generic Record image test */
	case 20:
		handle_generic_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		break;


	case 21:
                do_accessrsp(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;

	case 22:
                do_rescntrlrsp(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;

	case 23:
                do_trigrescntrlreq(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;


	case 24:
                do_DbvExtendedServicesReq_DbvPrintExtendedService(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;

	case 25:
                do_DbvExtendedServicesReq_DbvAlertExtendedService(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;

	case 26:
                do_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvCOrder(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
		break;


	case 27:
                do_initreq_charset(done_initial_association, assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbnames[current_id].dbname);
                break;

	case 30:
		close_connection_to_target(assoc_id, assoc_state, assoc_res, rel_state, error);
		break;
	default:
		printf("Unknown option chosen\n");
		break;
	}

}



int output_about_message(void)
{
	char *message = NULL, str[10];

	message = "\n\
This software incorporates the DBV OSI II API package which is a\n\
\"Freeware\" package. The DBV OSI II API software is copyright to \n\
Crossnet Systems Ltd. and to Die Deutsche Bibliothek.\n";

	printf("%s", message);

	printf("Press and key to continue\n");
	gets(str);
}



int specify_tcpip_target(char *host_address, int *port, char *dbname) 
{
	char new_host_address[1000];
	char new_port[1000];
	int choice = 0;
	char choice_string[1000];
	char port_string[1000];
	char *result = NULL;

	/* Clear the dbname */
	strcpy(dbname, "");

	printf("\n\n\n");
	printf("Selection of Target (Internet Details)\n");
	printf("--------------------------------------\n");
	printf("\n");
	printf("--------------------------------------------------------------\n");
	printf("USA Z39.50 Targets 1 (Use If You DO Have A Domain Name Server)\n");
	printf("--------------------------------------------------------------\n");
	printf("1) z3950.bell-labs.com (AT&T Library Server) [port 210]\n");
	printf("2) z3950.silverplatter.com (SilverPlatter server) [port 210]\n");
	printf("--------------------------------------------------------------\n");
	printf("USA Z39.50 Targets 2 (Use If You DONT Have A Domain Name Server)\n");
	printf("----------------------------------------------------------------\n");
	printf("3) (204.178.16.6) [dot notation form of z3950.bell-labs.com] [port 210]\n");
	printf("4) (192.80.71.227) [dot notation form of z3950.silverplatter.com] [port 210]\n");

	printf("--------------------\n");
	printf("Other Z39.50 Targets\n");
	printf("--------------------\n");
	printf("5) other [port 210]\n");
	printf("6) other [not on port 210]\n");
	printf("\n");

	while (choice != 1 && choice != 2 
		&& choice != 3 && choice != 4 
		&& choice != 5 && choice != 6 ) {
		printf("Choice (1/2/3/4/5/6) :");
		result = gets(choice_string);
		post_gets_check(result);
		choice = atoi(choice_string);

		switch (choice) {
		case 1: 
			/* z3950.research.att.com selected */ 
			strcpy(host_address, "z3950.bell-labs.com");
			*port = 210;
			strcpy(dbname, "XXDEFAULT");
			break;
		case 2: 
			/* z3950.silverplatter.com selected */ 
			strcpy(host_address, "z3950.silverplatter.com");
			*port = 210;
			strcpy(dbname, "ML4L");
			break;
		case 3: 
			/* dot form of z3950.research.att.com selected */ 
			strcpy(host_address, "204.178.16.6");
			*port = 210;
			strcpy(dbname, "XXDEFAULT");
			break;
		case 4: 
			/* dot form of z3950.silverplatter.com selected */ 
			strcpy(host_address, "192.80.71.227");
			*port = 210;
			strcpy(dbname, "ML4L");
			break;
		case 5:
			printf("\nEnter target Internet address (port 210 assumed):\n");
			result = gets(new_host_address);
			post_gets_check(result);
			strcpy(host_address, new_host_address); 
			*port = 210;
			break;
		case 6:
			printf("\nEnter target Internet address:\n");
			result = gets(new_host_address);
			post_gets_check(result);
			strcpy(host_address, new_host_address); 
			printf("\nEnter port number:\n");
			result = gets(port_string);
			post_gets_check(result);
			*port = atoi(port_string); 
			break;
		}
	}
}



int specify_osi_target(char *host_address, char *psel, char *ssel, char *tsel, char *nsap) 
{
	char input_string[1000];
	char *result = NULL;

	printf("\nWhat OSI psel ? (Hit return for none) \n");
	result = gets(input_string);
	post_gets_check(result);
	strcpy(psel, input_string); 

	printf("\nWhat OSI ssel ? (Hit return for none)\n");
	result = gets(input_string);
	post_gets_check(result);
	strcpy(ssel, input_string); 

	printf("\nWhat OSI tsel ? (e.g. '0142'H ) \n");
	result = gets(input_string);
	post_gets_check(result);
	strcpy(tsel, input_string); 

	printf("\nWhat OSI nsap ? (e.g. Internet=sun001.sil.com ) \n");
	result = gets(input_string);
	post_gets_check(result);
	strcpy(nsap, input_string); 
}



int specify_byname_target(char *host_address) 
{
	char new_host_address[1000];
	char *result = NULL;

	strcpy(new_host_address, "");

	/* while something not entered */
	while (strcmp(new_host_address, "") == 0) {
		printf("What Target Name (TCP/IP Name in target_names.tbl or OSI name in isoentities)\n");
		result = gets(new_host_address);
		post_gets_check(result);
		strcpy(host_address, new_host_address); 
	}
}



int connect_to_target(char *host_address, int *port, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, AddressType address_type_chosen, char *psel, char *ssel, char *tsel, char *nsap)
{

	DbvTarget targetid;

	/* Trace Files */
	FILE *psTraceFp;
	DbvTraceLevel eTraceLevel = Trace_High;

	int current = 0;
	char buf[20];


	switch (address_type_chosen) {
	case AT_tcpip:
		setup_tcpip_byaddr_target_info(&targetid, host_address, *port);
		break;
	case AT_osi:
		setup_osi_byaddr_target_info(&targetid, psel, ssel, tsel, nsap);
		break;
	case AT_byname:
		setup_byname_target_info(&targetid, host_address);
		break;
	}


	/* DbvAssociateRequest */ 

	/* Second parameter in following call can be CALL_ASYNCHRONOUS, 
	   CALL_BLOCKING or an integer greater than 0 */
	if(DbvAssociateRequest(assoc_id, CALL_BLOCKING, &targetid, 
		assoc_state, assoc_res, error) == SI_OK) {

		if(*assoc_state == Associate_Pending)
		{	
			printf("Association is pending\n");
			while(*assoc_state == Associate_Pending)
			{
				printf("Calling DbvConfirmAssociateRequest\n");
				/* One second delay for confirm */
				if(DbvConfirmAssociateRequest(*assoc_id,
					1, assoc_state, assoc_res, error) ==
					SI_OK)
				{
					if(*assoc_state == Associate_Pending)
						printf("Association still pending\n");
				}
				else
				{
					printf("Error doing DbvConfirmAssociateRequest()\n");
               				PrivateErrorHandling(error);
					exit(1);
				}
			}
		}
		if (*assoc_state == Associate_Accepted)
		{
			printf("Connected OK\n");
			current = get_new_assoc_place();
			alAssocId[current] = *assoc_id;
			printf("Number of associations = %d\n",current);
			print_the_assocreq_result(*assoc_state, *assoc_res);
		}
		else if (*assoc_state == Associate_Rejected)
		{
			printf("Association was rejected\n");
			print_the_assocreq_result(*assoc_state, *assoc_res);
		}
		else 
		{
			printf("Error doing Associate Request\n");
			exit(1);
		}
	}
	else
	{
                PrivateErrorHandling(error);
		return SI_NOTOK;
	}

	if(*assoc_state == Associate_Accepted)
	{
		/* Start Tracing to origin.n.trace. Can only do this once we 
			have a valid lAssocId */
		sprintf(buf, "origin.%d.trace", *assoc_id);
		if((psTraceFp=fopen(buf, "w")) == NULL)
		{
			printf("Couldn't open trace file!!\n");
		}
		else
		{
			/* Don't worry about SI_OK */
			SISetTraceParameter(*assoc_id, psTraceFp,
				eTraceLevel, error);
		}
	}
	return SI_OK;
}



int setup_tcpip_byaddr_target_info(DbvTarget *targetid, char *host_address, int port)
{
        printf("Attempting to connect to host:%s on port %d\n",host_address, port);
	targetid->eTargetNameOrAddress = Target_Address;
	targetid->uNameOrAddress.psTargetInformation = (struct DbvTargetInformation *) DbvAlloc(sizeof(struct DbvTargetInformation));
	targetid->uNameOrAddress.psTargetInformation->eOsiOrInternetAddress = Internet_Address;
	targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psInternetAddress = (struct DbvInternetAddress *) DbvAlloc(sizeof(struct DbvInternetAddress));
	targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psInternetAddress->ulPortNumber = port;
	targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psInternetAddress->pcHostAddress = (char *) DbvAlloc (strlen(host_address) + 1);
	memcpy(targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psInternetAddress->pcHostAddress, host_address, strlen(host_address));
}



int setup_osi_byaddr_target_info(DbvTarget *targetid, char *psel, char *ssel, char *tsel, char *nsap)
{
        printf("Attempting to connect to host: psel:%s  ssel:%s  tsel:%s  nsap:%s\n", psel, ssel, tsel, nsap);
	targetid->eTargetNameOrAddress = Target_Address;
	targetid->uNameOrAddress.psTargetInformation = (struct DbvTargetInformation *) DbvAlloc(sizeof(struct DbvTargetInformation));
	targetid->uNameOrAddress.psTargetInformation->eOsiOrInternetAddress = Osi_Address;
	targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psOsiAddress = (struct DbvOsiAddress *) DbvAlloc(sizeof(struct DbvOsiAddress));
	setup_osi_target_info(targetid->uNameOrAddress.psTargetInformation->uOsiOrInternetAddress.psOsiAddress, psel, ssel, tsel, nsap);
}



int setup_osi_target_info(DbvOsiAddress *psOsiAddress, char *psel, char *ssel, char *tsel, char *nsap)
{

	psOsiAddress->pcPsel = (char *) DbvAlloc(strlen(psel) +1);
	memcpy(psOsiAddress->pcPsel, psel, strlen(psel)); 

	psOsiAddress->pcSsel = (char *) DbvAlloc(strlen(ssel) +1);
	memcpy(psOsiAddress->pcSsel, ssel, strlen(ssel)); 

	psOsiAddress->pcTsel = (char *) DbvAlloc(strlen(tsel) +1);
	memcpy(psOsiAddress->pcTsel, tsel, strlen(tsel)); 

	psOsiAddress->pcNsap = (char *) DbvAlloc(strlen(nsap) +1);
	memcpy(psOsiAddress->pcNsap, nsap, strlen(nsap)); 
}



int setup_byname_target_info(DbvTarget *targetid, char *host_address)
{
        printf("Attempting to connect to host:%s\n",host_address);

	targetid->eTargetNameOrAddress = Target_Name;
	targetid->uNameOrAddress.pcTargetName = (char *) DbvAlloc(strlen(host_address) + 1);
	memcpy(targetid->uNameOrAddress.pcTargetName, host_address, strlen(host_address));
}



void print_the_assocreq_result(DbvAssociateState eAssociateState, 
	DbvAssociateResult *puAssociateResult)
{
	switch(eAssociateState)
	{
		case Associate_Null:
			printf("Associate_Null\n");
			break;
		case Associate_Accepted:
			printf("Associate_Accepted\n");
			PrintApplicationProtocol(puAssociateResult->
				psProtocolAndSyntaxes->peApplicationProtocol);
			PrintAbstractSyntax(puAssociateResult->
				psProtocolAndSyntaxes->peAbstractSyntax);
			PrintppcCommonSyntaxes(puAssociateResult->
				psProtocolAndSyntaxes->ppcCommonSyntaxes);
			break;
		case Associate_Rejected:
			printf("Associate_Rejected\n");
			PrintDbvAssociateRejectSource(puAssociateResult->
				psAssociateReject->peAssociateRejectSource);
			PrintDbvAssociateRejectResult(puAssociateResult->
				psAssociateReject->peAssociateRejectResult);
			PrintDbvAssociateRejectReason(puAssociateResult->
				psAssociateReject->peAssociateRejectReason);
		break;
	}
}

int PrintDbvAssociateRejectSource(DbvAssociateRejectSource 
	*peAssociateRejectSource)
{
        printf("peAssociateRejectSource == ");
        switch(*peAssociateRejectSource)
        {
        case ACSE_User:
                printf("ACSE_User\n");
                break;
        case ACSE_Provider:
                printf("ACSE_Provider\n");
                break;
        default:
                printf("Error in value!!\n");
                break;
        }
 
}



int PrintDbvAssociateRejectResult(DbvAssociateRejectResult 
	*peAssociateRejectResult)
{
        printf("peAssociateRejectResult == ");
        switch(*peAssociateRejectResult)
        {
        case Rejected_Permanent:
                printf("Rejected_Permanent\n");
                break;
        case Rejected_Transient:
                printf("Rejected_Transient\n");
                break;
        default:
                printf("Error in value!!\n");
                break;
        }
 
}



int PrintDbvAssociateRejectReason(DbvAssociateRejectReason 
	*peAssociateRejectReason)
{
 
        printf("peAssociateRejectReason == ");
        switch(*peAssociateRejectReason)
        {
        case No_Reason_Given:
                printf("No_Reason_Given\n");
                break;
        case No_Common_ACSE_Version:
                printf("No_Common_ACSE_Version\n");
                break;
        case Application_Context_Not_Supported:
                printf("Application_Context_Not_Supported\n");
                break;
        default:
                printf("Error in value!!\n");
                break;
        }
}



void PrivateErrorHandling(DbvError **ppsError)
{
	DbvError *psError;

	psError = *ppsError;
	if(psError == NULL)
	{
		printf("Error structure was not filled out!\n");
	}
	else
	{
		printf("psError->eErrorClass == ");
		if(psError->eErrorClass == SI_Communication_Error)
			printf("SI_Communication_Error\n");
		if(psError->eErrorClass == SI_Interface_Error)
			printf("SI_Interface_Error\n");
		if(psError->eErrorClass == SI_OperatingSystem_Error)
			printf("SI_OperatingSystem_Error\n");
		if(psError->eErrorClass == SI_Abort)
		{
			printf("SI_Abort\n");
			printf("psError->uErrorCode.psSIAbort->eAbortSource == ");
			switch(psError->uErrorCode.psAbort->eAbortSource)
			{
			case SI_ACSE_User:
				printf("SI_ACSE_User\n");
				break;
			case SI_ACSE_Provider:
				printf("SI_ACSE_Provider\n");
				break;
			case SI_Presentation_User:
				printf("SI_Presentation_User\n");
				break;
			case SI_Presentation_Provider:
				printf("SI_Presentation_Provider\n");
				break;
			case SI_Transport_Provider:
				printf("SI_Transport_Provider\n");
				break;
			default:
				printf("ERROR: Unknown type!\n");
				break;
			}
			printf("psError->uErrorCode.psSIAbort->lAbortReason == %d\n",psError->uErrorCode.psAbort->lAbortReason);
			printf("->pcDiagnostics == %s\n", psError->uErrorCode.psAbort->pcDiagnostics);
                        printf("psError->eAssociationState == ");

	                if(psError->eAssociationState == Association_Open)
                                printf("Association_Open\n");
                        if(psError->eAssociationState == Association_Closed)
                                printf("Association_Closed\n");
                        if(psError->eAssociationState == Association_Unknown_State)
                                printf("Association_Unknown_State\n");

		}
		else
		{
			printf("psError->uErrorCode.lIfOsCoError == %d\n",psError->uErrorCode.lIfOsCoError);
		}
	}
}
 
 
 



int do_initreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	int	choice;
	int	version;
	char	choice_string[100];
	
	if (done_initial_association) {

		choice = 0;
		while (choice < 1 || choice > 2) {
			printf("Do you want the client to be V2 (1) or V3 (2): ");
			choice = (atol(gets(choice_string)));
		}
		if (choice ==1) version =2;
		if (choice ==2) version =3;
		
		switch (choice) {
		case 1:
		if (send_initreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, version) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("InitReq send failed\n");
		break;

		case 2:
		if (send_initreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, version) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("InitReq send failed\n");
		break;
		}


	} else
		printf("Can't send InitializeRequest - not yet connected to a target\n");
}



int handle_response(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError ** error)
{
	char choice_string[1000];
	char *result = NULL;

	memset(choice_string, 0, sizeof(choice_string));
	printf("Wait for PDU from target (y/n): ");
	result = gets(choice_string);
	post_gets_check(result);
	if ((strcmp(choice_string,"y")) == 0)
		wait_for_and_display_response(assoc_id, assoc_state, assoc_res, rel_state, error);
}



int do_srchreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_srchreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbname) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("SrchReq send failed\n");
	} else
		printf("Can't send SearchRequest - not yet connected to a target\n");
}



int do_presreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{

	if (done_initial_association) {
		if (send_presreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbname) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("PresReq send failed\n");
	} else
		printf("Can't send PresentRequest - not yet connected to a target\n");
}




int do_delreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_delreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("DelReq send failed\n");
	} else
		printf("Can't send DeleteResultSetRequest - not yet connected to a target\n");
}





int do_accessrsp(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_accessrsp_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("AccessRsp send failed\n");
	} else
		printf("Can't send AccessControlResponse - not yet connected to a target\n");
}

int do_rescntrlrsp(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_rescntrlrsp_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ResourceControlRsp send failed\n");
	} else
		printf("Can't send ResourceControlResponse - not yet connected to a target\n");
}

int do_trigrescntrlreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_trigrescntrlreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("TriggerResourceControlReq send failed\n");
	} else
		printf("Can't send TriggerResourceControlRequest - not yet connected to a target\n");
}





int do_resrepreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_resrepreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ResRepReq send failed\n");
	} else
		printf("Can't send ResourceReportRequest - not yet connected to a target\n");
}



int do_scanreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_scanreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ScanReq send failed\n");
	} else
		printf("Can't send ScanRequest - not yet connected to a target\n");
}


int do_sortreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{

	if (done_initial_association) {
		if (send_sortreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("SortReq send failed\n");
	} else
		printf("Can't send SortRequest - not yet connected to a target\n");
}



int do_DbvExtendedServicesReq_DbvUpdateExtendedService(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{

	if (done_initial_association) {
		if (send_DbvExtendedServicesReq_DbvUpdateExtendedService(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ExtendReq send failed\n");
	} else
		printf("Can't send ExtendedServicesRequest - not yet connected to a target\n");
}




int do_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvIllPdu(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{

	if (done_initial_association) {
		if (send_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvIllPdu(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ExtendReq send failed\n");
	} else
		printf("Can't send ExtendedServicesRequest - not yet connected to a target\n");
}




int do_DbvExtendedServicesReq_DbvPrintExtendedService(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_DbvExtendedServicesReq_DbvPrintExtendedService(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ExtendReq send failed\n");
	} else
		printf("Can't send ExtendedServicesRequest - not yet connected to a target\n");
}



int do_DbvExtendedServicesReq_DbvAlertExtendedService(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_DbvExtendedServicesReq_DbvAlertExtendedService(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ExtendReq send failed\n");
	} else
		printf("Can't send ExtendedServicesRequest - not yet connected to a target\n");
}



int do_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvCOrder(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_DbvExtendedServicesReq_DbvItemOrderExtendedService_DbvCOrder(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("ExtendReq send failed\n");
	} else
		printf("Can't send ExtendedServicesRequest - not yet connected to a target\n");
}




int do_closereq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_closereq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("CloseReq send failed\n");
	} else
		printf("Can't send CloseRequest - not yet connected to a target\n");
}



int do_closersp(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
	if (done_initial_association) {
		if (send_closersp_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
			handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
		else 
			printf("CloseRsp send failed\n");
	} else
		printf("Can't send CloseResponse - not yet connected to a target\n");
}



int wait_for_any_responses(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
	
        DbvOriginData *psOriginData = NULL;
        DbvBool finished = DBV_FALSE;
	int place = 0;
 
        DbvAssocId **pplAssocId = NULL;
        DbvEventLocation *psEventLocation = NULL;
        DbvFds **ppulFds = NULL;
 
        int apdu_type, numfields;
        char command[1000];
 

	/* Create a 100 element ppulFds descriptor list */
        ppulFds = (DbvFds **) DbvAlloc (100 * sizeof(DbvFds *));
 
        /* Put nothing in it though - make the first item NULL */
        ppulFds[0] = (DbvFds *) NULL;

	if(get_new_assoc_place() == 0)
	{
		printf("There are currently no active associations\n");
		return -1;
	}

	while(alAssocId[place]!=0)
	{
		printf("%d. Association %d\n", place, alAssocId[place]);
		place++;
	}
	place++;

        pplAssocId = (DbvAssocId **)DbvAlloc(place*sizeof(DbvAssocId *));
	place = 0;
	while(alAssocId[place]!=0)
	{
        	pplAssocId[place] = &alAssocId[place];
		place++;
	}
        pplAssocId[place] = (DbvAssocId *)NULL;
 
        /* Wait for the response from the target */
        printf("Waiting to receive data sent back by target.....\n");
 
        /* Call with a one second time out */
        do{
	if(SIGetEventLocation(pplAssocId, 1, ppulFds, &psEventLocation, error) == SI_OK)
        {
		if(psEventLocation != NULL)
		{
                        /* Receive  a Z39.50 PDU over the communication link from the target */
			place = 0;
			while( psEventLocation->pplProviderEvent[place] != NULL)
			{	
			printf("Got something from Association %d\n",*psEventLocation->pplProviderEvent[place]);
                        if (DbvReceiveDataOrigin(*(psEventLocation->
				pplProviderEvent[place]), CALL_BLOCKING,
				&psOriginData, error) == SI_OK)
                        {
                                finished = DBV_TRUE;
			
			/*********************************************/
			/*********************************************/

			/* At this point a valid PDU has been received
			   from the target and has been entered into
			   the psOriginData structure */
				
        		/* Print to the screen what we have received */
        		print_origin_fields(stdout, psOriginData);

			/* Use the PDU data as appropriate */
 
			/* Free up the PDU data received from the target 
			   (psOriginData space is only allocated
			   if a valid PDU is received) */

			SIFreeDbvOriginData(&psOriginData);


			/*********************************************/
			/*********************************************/
			
                        } 
			else {
                                printf("DbvReceiveDataOrigin returned SI_NOTOK\n");
                                /* Error in DbvReceiveDataOrigin */
                                PrivateErrorHandling(error);
                                FreeSIErrorCode(error);
                                finished = DBV_TRUE;
				remove_assoc(*psEventLocation->
					pplProviderEvent[place]);
 
                        }

			place ++;
			}
                }
                else
                {
                        printf("Timed out with nothing to read\n");
                }
        }
        else
        {
                /* Error in SIGetEventLocation */
                PrivateErrorHandling(error);
                FreeSIErrorCode(error);
                exit(1);
        }
        }
        while(finished == DBV_FALSE);
 

	return SI_OK;
}
 


int FreeSIErrorCode(DbvError **ppsError)
{
	DbvError *psError;
	
	psError = *ppsError;
	if(psError != NULL)
	{
	if ( (psError->eErrorClass != SI_Interface_Error) && 
		(psError->eErrorClass != SI_OperatingSystem_Error) && 
	 	(psError->eErrorClass != SI_Communication_Error) )
	{
		free(psError->uErrorCode.psAbort->pcDiagnostics);
		free(psError->uErrorCode.psAbort);
	}
	free(psError);
	*ppsError = (DbvError *)NULL;
	}
	return 1;
}



int wait_for_and_display_response(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
 
        DbvOriginData *psOriginData = NULL;
	DbvBool finished = DBV_FALSE;

	DbvAssocId **pplAssocId = NULL;
	DbvEventLocation *psEventLocation = NULL;
	DbvFds **ppulFds = NULL;
 
        int apdu_type, numfields;
        char command[1000];
 
	pplAssocId = (DbvAssocId **)DbvAlloc(2*sizeof(DbvAssocId *));
	pplAssocId[0] = assoc_id;
	pplAssocId[1] = (DbvAssocId *)NULL;
	
	/* Create a 100 element ppulFds descriptor list */
	ppulFds = (DbvFds **) DbvAlloc (100 * sizeof(DbvFds *));
	
	/* Put nothing in it though - make the first item NULL */
	ppulFds[0] = (DbvFds *) NULL;

        /* Wait for the response from the target */
 
	printf("Waiting to receive data sent back by target.....\n");

	/* Call with a one second time out */
	do{
	if(SIGetEventLocation(pplAssocId, 1, ppulFds, &psEventLocation, error)
		== SI_OK)
	{
		if(psEventLocation != NULL)
		{

        		/* Receive  a Z39.50 PDU over the communication link from the target */
        		if (DbvReceiveDataOrigin(*pplAssocId[0], CALL_BLOCKING, &psOriginData, error) == SI_OK) 
			{
				finished = DBV_TRUE;
        		} else {
				printf("DbvReceiveDataOrigin returned SI_NOTOK\n");
				/* Error in DbvReceiveDataOrigin */
                		PrivateErrorHandling(error);
                		FreeSIErrorCode(error);
                		exit(1);

		        }
		}
		else
		{
			printf("Timed out with nothing to read\n");
		}
	}
	else
	{
		/* Error in SIGetEventLocation */
                PrivateErrorHandling(error);
                FreeSIErrorCode(error);
                exit(1);
	}
	}
	while(finished == DBV_FALSE);
 
        /* Print to the screen what we have received */
        print_origin_fields(stdout, psOriginData);

	SIFreeDbvOriginData(&psOriginData);


	return SI_OK;
}




int print_origin_fields(FILE *f, DbvOriginData *psOriginData)
{
	fprintf(f, "---------------------------------------\n");
	fprintf(f, "PDU received from target:\n");

        switch (psOriginData->eOriginEvent) {

        	/* InitResponse from target */
                case OR_InitializeConf:
                        print_initrsp_fields(f, psOriginData->uOriginData.psInitializeRsp);
                        break;
		
		/* Search Response from target */
		case OR_SearchConf:
			print_searchrsp_fields(f, psOriginData->uOriginData.psSearchRsp);
			break;
		case OR_PresentConf:
			print_presrsp_fields(f, psOriginData->uOriginData.psPresentRsp);
			break;
		case OR_DeleteResultSetConf:
			print_delrsp_fields(f, psOriginData->uOriginData.psDeleteResultSetRsp);
			break;
		case OR_ScanConf:
			print_scanrsp_fields(f, psOriginData->uOriginData.psScanRsp);
			break;
		case OR_ResourceReportConf:
			print_resreprsp_fields(f, psOriginData->uOriginData.psResourceReportRsp);
			break;
		case OR_SegmentationInd:
			print_segreq_fields(f, psOriginData->uOriginData.psSegmentReq);
			break;
		case OR_SortConf:
			print_sortrsp_fields(f, psOriginData->uOriginData.psSortRsp);
			break;
		case OR_ExtendedServicesConf:
			print_extendrsp_fields(f, psOriginData->uOriginData.psExtendedServicesRsp);
			break;
		case OR_CloseConf:
			fprintf(f, ":::Origin event: OR_CloseConf:::\n");
			print_closereq_fields(f, psOriginData->uOriginData.psCloseReq);
			break;
		case OR_CloseInd:
			fprintf(f, ":::Origin event: OR_CloseInd:::\n");
			print_closereq_fields(f, psOriginData->uOriginData.psCloseReq);
			break;
		case OR_Invalid_APDU:
			print_invalidapdu(f, psOriginData->uOriginData.psInvalidAPDU);
			break;

		case OR_AccessCntrlInd:
			print_accessreq_fields(f, psOriginData->uOriginData.psAccessControlReq);
			break;

		case OR_ResourCntrlInd:
			print_rescntrlreq_fields(f, psOriginData->uOriginData.psResourceControlReq);
			break;

		default:
			break;
        }

	fprintf(f, "---------------------------------------\n");
}



int close_connection_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
	/* second parameter in following call can be CALL_ASYNCHRONOUS 
	   or CALL_BLOCKING */
	if (DbvReleaseRequest(*assoc_id, CALL_BLOCKING, rel_state, error) == SI_OK) 
		process_release_state(assoc_id, rel_state, error);
	else 
                PrivateErrorHandling(error);
}



int process_release_state(DbvAssocId *assoc_id, DbvReleaseState *rel_state, DbvError **error)
{
	if (*rel_state == Release_Accepted)
		printf("DbvReleaseRequest successful\n");

	else if (*rel_state == Release_Rejected)
		printf("DbvReleaseRequest was rejected\n");

	/* rel_state can only be Release_Pending for OSI */
	else if (*rel_state == Release_Pending) {
		printf("DbvReleaseRequest is Pending - attempting DbvConfirmReleaseRequest\n");

		/* Second parameter in following call can be CALL_ASYNCHRONOUS 
		   or CALL_BLOCKING */
		if (DbvConfirmReleaseRequest(*assoc_id, CALL_BLOCKING, rel_state, error) == SI_OK) 
			printf("DbvConfirmReleaseRequest succeeded\n");
		else
			printf("DbvConfirmReleaseRequest failed\n");
	} else
		printf("Error in DbvReleaseRequest\n");
}



int abort_connection_to_target(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{

	if(DbvAbortRequest(*assoc_id, error) == SI_OK)
	{
		printf("DbvAbortRequest was successful\n");
	} else {
		printf("DbvAbortRequest returned SI_NOTOK\n");
                PrivateErrorHandling(error);
	}
}

DbvAssocId select_current_assoc(void)
{
	int place=0;
	int choice = 0;
	char choice_string[1000];
	char *result = NULL;

	if(get_new_assoc_place() == 0)
	{
		printf("There are currently no active associations\n");
		return -1;
	}
	do{
	place = 0;
	while(alAssocId[place]!=0)
	{
		printf("%d. Association %d\n", place, alAssocId[place]);
		place++;
	}
	printf("%d. Invalid Association (for testing)\n",place);
	printf("Select:");
	result = gets(choice_string);
	post_gets_check(result);
	choice = atoi(choice_string);
	}
	while(choice <0 && choice > place);

	if(choice == place)
		return -1;

	return alAssocId[choice];
}

int get_new_assoc_place(void)
{
	int place = 0;
	while(alAssocId[place]!=0)
		place++;
	return place;
}
void remove_assoc(DbvAssocId lAssocId)
{

	int place = 0;
	int loop;

	while((alAssocId[place] != lAssocId) && (place <MAX_ASSOCS))
		place++;
	for(loop=place; loop<MAX_ASSOCS-1; loop++)
		alAssocId[loop]=alAssocId[loop+1];
	alAssocId[MAX_ASSOCS-1]=0;
}

 

/* Explain search */
int do_srchreq_explain(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
        if (done_initial_association)
        {
                if (send_srchreq_explain_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbname) == SI_OK)
                {
                        handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
                }
                else
                {
                        printf("SearchReq send failed\n");
                }
        }
}



/* CIP explain search */
int do_srchreq_explain_cip(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
        if (done_initial_association)
        {
                if (send_srchreq_explain_cip_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbname) == SI_OK)
                {
                        handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
                }
                else
                {
                        printf("SerchReq send failed\n");
                }
        }
}



int do_exppresreq(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
        if (done_initial_association)
        {
                if (send_exppresreq_to_target(assoc_id, assoc_state, assoc_res, rel_state, error, assoc_dbname) == SI_OK)
                {
                        handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
                }
                else
                {
                        printf("ExpPresReq send failed\n");
                }
        }

        else
        {
                printf("Not yet connected\n");
        }
}




int do_initreq_charset(DbvBool done_initial_association, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error, char *assoc_dbname)
{
        if (done_initial_association)
        {
                if (send_initreq_charset_to_target(assoc_id, assoc_state, assoc_res, rel_state, error) == SI_OK)
                {
                        handle_response(assoc_id, assoc_state, assoc_res, rel_state, error);
                }
                else
                {
                        printf("initerq send failed\n");
                }
        }

        else
        {
                printf("Not yet connected\n");
        }
}




