#include "apitesttargetslave.h"


/* State machine off/on toggle variable in API (DBV_FALSE | DBV_TRUE) */
extern int DBVStateMachine;

/* Full pathname will be pre-pended */
#define ACSE_CONFIG_FILE "/ACSE_configuration_file_target.tbl"


int main(int argc, char *argv[])
{

	DbvAssocId lAssocId;
	char *origin_address = NULL;
	DbvError *psError = NULL;
	DbvProtocolAndSyntaxes *psProtocolAndSyntaxes = NULL;
	int file_number = 1;
	char file_name[128];
	char trace_name[128];
	char input_string[12];
	FILE *psTraceFp;
	DbvTraceLevel eTraceLevel = Trace_High;
	DbvAssociateRsp eAssociateRsp;
	char pcDirectory[256];
	int length;
	char *menu_header = NULL;
	

	/* argc will be 3 if invoked from targetd_tcpip */
	if (argc != VALID_ARG_NUM) {
		printf(ARG_ERROR);
		exit(1);
	}
				

	/* Internal state machine toggle (DBV_FALSE | DBV_TRUE) */
        DBVStateMachine = DBV_TRUE; 


	menu_header = "DBV-OSI II Target slave (c) CSL\n";

	printf("%s", menu_header);
	printf("-----------------------------\n");

/* If MALLOC_DEBUG resolves to non zero (MALLOC_DEBUG is a Sun Solaris 1
   feature only) */
#ifdef MALLOC_DEBUG
	malloc_debug(2);
#endif

	/* Comment out the following seven lines to avoid starting a gdb window */


	printf("Debug? (y/n)");
	fflush(stdout);
	fgets(pcDirectory, 50, stdin);
	if(*pcDirectory=='y' || *pcDirectory=='Y')
	{
		setup_gdb(argv);
	}


/* pcDirectory contains the directory of the executable */
	strcpy(pcDirectory, argv[0]);
	length = strlen(pcDirectory);
	length--;
	while(pcDirectory[length]!='/')
		{ length--; }
	pcDirectory[length]='\0';

/* Change to working directory so that targets started by root write to
	the API log file correctly */
	chdir(pcDirectory);

	strcpy(file_name, pcDirectory);
	strcat(file_name, ACSE_CONFIG_FILE);
        printf("Setting ACSE Configuration file to %s\n", file_name);
        if(SISetACSEFile(file_name, &psError)==SI_NOTOK)
        {
                if(psError->eErrorClass == SI_Interface_Error &&
                        psError->uErrorCode.lIfOsCoError == DBV_FILE)
                {
                        printf("Warning: Could not open file: %s\n",
                                file_name);
                }
        }

	DbvTargetInitialize(argc, argv, &psError);

	if(DbvReceiveAssociateRequest(&lAssocId, CALL_BLOCKING, &origin_address, &psProtocolAndSyntaxes, &psError) == SI_NOTOK)
	{
		if(psError->eAssociationState != Association_Open)
		{
			printf("FATAL Error in DbvReceiveAssociateRequest\n");
			/* Look inside the error structure */
			PrivateErrorHandling(&psError);
			printf("Press return to exit\n");
			fgets(input_string,10,stdin);
			exit(1);
		}
		else
			printf("Error in DbvReceiveAssociateRequest\n");

	}

	/* Print out the Protocol and syntaxes structure */
	printf("DbvProtocolAndSyntaxes structure detailed\n");
        PrintApplicationProtocol(psProtocolAndSyntaxes->peApplicationProtocol);
        PrintAbstractSyntax(psProtocolAndSyntaxes->peAbstractSyntax);
        PrintppcCommonSyntaxes(psProtocolAndSyntaxes->ppcCommonSyntaxes);

/* If local development specified */
#ifdef LOCALTRACE
	strcpy(file_name, ".");
#else
	strcpy(file_name, TRACE_DIR);
#endif

	/* Change directory to the location of this program 
	   (so that ones started by (a root owned) tsapd/iaed don't 
	   write their trace files to the root dir  */
        chdir(pcDirectory);

	strcat(file_name, "/target.1.trace");
	while((psTraceFp=fopen(file_name,"r")) != NULL)
	{
		fclose(psTraceFp);
		file_number++;

#ifdef LOCALTRACE
		strcpy(file_name, ".");
#else
		strcpy(file_name, TRACE_DIR);
#endif
		sprintf(trace_name,"/target.%d.trace",file_number);
		strcat(file_name, trace_name);
	}
	printf("File name is %s\n",file_name);
        if((psTraceFp=fopen(file_name, "w")) == NULL)
        {
               printf("Couldn't open trace file!!\n");
        }
        else
        {
                SISetTraceParameter(lAssocId, psTraceFp,
                        eTraceLevel, &psError);
        }

	printf("Do you wish to accept or reject the association?\n");
	printf("1. Accept  2. Reject\n");
	fgets(input_string,10,stdin);
	if((atoi(input_string)) == 1)
		eAssociateRsp = Accepted;
	else
		eAssociateRsp = Rejected_Perm;
	
	if(DbvAssociateResponse(lAssocId, eAssociateRsp, &psError) == SI_NOTOK)
	{
		printf("Error in DbvAssociateResponse\n");
		/* Look inside the error structure */
		PrivateErrorHandling(&psError);
		printf("Press return to exit\n");
		fgets(input_string,10,stdin);
		exit(1);
	}
	else
	{
		if(eAssociateRsp != Accepted)
		{
			printf("Association was rejected\n");
			printf("Press return to exit\n");
			fgets(input_string,10,stdin);
			exit(0);
		}
	}

	printf("Received connection from %s\n", origin_address);

        main_code(lAssocId);
}



int setup_gdb(char **argv)
{
 
	char str[2];
        char gdb_str[100];
        char program[100];
 
#ifdef LOCALTRACE
	strcpy(program, argv[0]);
	strcat(program, " ");
#else
	strcpy(program, argv[0]);
	strcat(program, " ");
#endif


        printf("Invoking gdb to debug %s ... \n", program);
 
        sprintf(gdb_str, "%s%s%s%d%s", XTERM, " -e gdb ", program, getpid(), " &");
        system(gdb_str);

	printf("Press return when gdb has attached\n");
	gets(str);
}



int main_code(int assoc_id)
{
	/* assoc_id holds the id of the association */
	DbvAssocId	lAssocId = assoc_id;
        DbvTargetData	*psTargetData = NULL;
        DbvError	*psError = NULL;
	char		str[128];
	DbvReleaseRsp	*psReleaseRsp = NULL;
	char		*result = NULL;
	int		return_value = SI_OK;
	char		choice_string[1000];
	DbvBool		wait_for_data = DBV_TRUE;
	int		type_of_apdu_to_send = 0;

	
	while (type_of_apdu_to_send != 30) {

		if (wait_for_data == DBV_TRUE) {

			printf("Waiting for data to be sent by the origin...\n");
                	/* Attempt to receive a Z39.50 PDU over the communication link 
		   	from the origin */
                	if (DbvReceiveDataTarget(lAssocId, CALL_BLOCKING, &psTargetData, &psError) != SI_OK) {
                        	PrivateErrorHandling(&psError);
                        	FreeSIErrorCode(&psError);
                        	printf("Receive failed - Hit return to exit\n");
				gets(str);
                        	exit(1);
                	} /* if */


			/* Because of the exit(1) above, this point will only be reached
		   	if a valid PDU has been received from the origin */

			/*******************************************************/
			/*******************************************************/
			/* At this point a valid PDU has been received from the origin
		   	and has been entered into the psTargetData structure */

                	/* Print to the screen the PDU received */
                	print_target_fields(stdout, psTargetData);
 

			/* Use the PDU data as appropriate - e.g. read fields and
		   	access local database etc. */


			/* Free up the PDU data received from the origin (psTargetData
		   	space is only allocated if a valid PDU is received) */


			SIFreeDbvTargetData(&psTargetData);


			/*******************************************************/
			/*******************************************************/
		} /* if */





		/* Create a PDU to send back to the origin */
		type_of_apdu_to_send = 0;
		return_value = SI_NOTOK;
		while (return_value == SI_NOTOK || type_of_apdu_to_send < 1 || type_of_apdu_to_send > 30) {

                	type_of_apdu_to_send = get_apdu_choice();
			switch (type_of_apdu_to_send) {
               		case 1:
                       		return_value = send_initrsp_to_origin(assoc_id, DBV_TRUE);
				break;
               		case 2:
                       		return_value = send_initrsp_to_origin(assoc_id, DBV_FALSE);
				break;
               		case 3:
                       		return_value = send_srchrsp_to_origin(assoc_id);
				break;
               		case 4:
                       		return_value = send_presrsp_to_origin(assoc_id);
				break;
               		case 5:
                       		return_value = send_delrsp_to_origin(assoc_id);
				break;
               		case 6:
                       		return_value = send_resreprsp_to_origin(assoc_id);
				break;
               		case 7:
                       		return_value = send_scanrsp_to_origin(assoc_id);
				break;
               		case 8:
                       		return_value = send_sortrsp_to_origin(assoc_id);
				break;
               		case 9:
				/* A request sent from target to origin */
                       		return_value = send_segreq_to_origin(assoc_id);
				break;
               		case 10:
                       		return_value = send_DbvExtendedServicesRsp_DbvEsTaskPackage_DbvInternationalString(assoc_id);
				break;
	
               		case 11:
                       		return_value = send_DbvExtendedServicesRsp_DbvEsTaskPackage_DbvItemOrderExtendedService_DbvIllPdu(assoc_id);
				break;
               		case 12:
                       		return_value = send_closereq_to_origin(assoc_id);
				break;
               		case 13:
                       		return_value = send_closersp_to_origin(assoc_id);
				break;
			case 14:
				return_value = send_releasersp_to_origin(assoc_id);
				break;
			case 15:
				return_value = abort_connection_to_origin(assoc_id, &psError);

/* Comment layout required for m4 */

/* If this target slave is run within a xterm created by targetd_tcpip, BOTH the xterm and the slave will inherit information about the communication session from targetd_tcpip.  The consequence of this is that an abort will NOT fully abort the session as a cross reference to TCP/IP information will be held by the xterm process - a target slave will only be aborted when all information about a communication session has been destroyed.  

If being run in an xterm, an abort will only be completed if an exit(0) call or equivalent is made immediately after doing an abort.  exit(0) will destroy the xterm together will all information it holds and so force the abort to be completed.  It is possible to run DBV-OSI II target slaves without using xterms. For such slaves calling exit(0) is unnecessary - see DBV-OSI II documentation for further details.  */

                       		/* exit(0); */
				break;

			/* Need to receive data if get out of req-rsp sync (e.g. if
			   sending DbvResourceControlReq and not waiting 
			   for DbvResourceControlRsp */
			case 16:

				/* Just do a receive */
                		if (DbvReceiveDataTarget(lAssocId, CALL_BLOCKING, &psTargetData, &psError) == SI_OK) {
					/* and print what we've received */
                			print_target_fields(stdout, psTargetData);

					SIFreeDbvTargetData(&psTargetData);

				}
				break;


			/* Generic record test */
			case 20:
				send_presrsp_genlite_to_origin(assoc_id);
				break;


			case 21:
                       		return_value = send_accessreq_to_origin(assoc_id);
				break;
			case 22:
                       		return_value = send_rescntrlreq_to_origin(assoc_id, DBV_TRUE);
				break;

			case 23:
                       		return_value = send_rescntrlreq_to_origin(assoc_id, DBV_FALSE);
				break;


			case 24:
                       		return_value = send_DbvExtendedServicesRsp_DbvEsTaskPackage_DbvItemOrderExtendedService_DbvCOrder(assoc_id);
				break;

			case 25:
                       		return_value = send_initrsp_charset_to_origin(assoc_id, DBV_TRUE);
				break;


			case 30:

				/* By convention an exit status code of zero means that 
		   		the process terminated normally and a non zero
		   		code (usually 1) means that an error occurred */
				exit(0);
				break;
			default:
				printf("Unknown option chosen\n");
				break;
			} /* switch */

		} /* while */

		printf("Wait for PDU from origin (y/n): ");
       		result = gets(choice_string);
       		post_gets_check(result);
       		if ((strcmp(choice_string,"y")) == 0) 
			wait_for_data = DBV_TRUE;
		else
			wait_for_data = DBV_FALSE;
	} /* while */
}

 
 
 
int FreeSIErrorCode(DbvError **ppsError)
{
        DbvError *psError;
 
        psError = *ppsError;
	if(psError!=NULL)
	{
        	if(psError->eErrorClass == SI_Abort)
        	{
                	free(psError->uErrorCode.psAbort->pcDiagnostics);
                	free(psError->uErrorCode.psAbort);
        	}
        	free(psError);
        	*ppsError = (DbvError *)NULL;
	}
	return SI_OK;
}
 
 
 
int get_apdu_choice()
{
        int operation_choice;
        char operation_choice_str[100];
	char *result = NULL;
 
	printf(" 1) Send InitializeResponse Accept to connected Origin\n");
	printf(" 2) Send InitializeResponse Reject to connected Origin\n");
	printf(" 3) Send SearchResponse to connected Origin\n");
	printf(" 4) Send PresentResponse to connected Origin\n");
	printf(" 5) Send DeleteResultSetResponse to connected Origin\n");
	printf(" 6) Send ResourceReportResponse to connected Origin\n");
	printf(" 7) Send ScanResponse to connected Origin\n");
	printf(" 8) Send SortResponse to connected Origin\n");
	printf(" 9) Send SegmentRequest to connected Origin\n");
	printf("10) Send ExtendedServicesResponse with 'EsTask,Sutrs' to connected Origin\n");
	printf("11) Send ExtendedServicesResponse with 'EsTask,ItemOrder,ILL' to connected Origin\n");
	printf("12) Send 'CloseRequest' (Close PDU) to connected Origin\n");
	printf("13) Send 'CloseResponse' (Close PDU) to connected Origin\n");
	printf("14) Send ReleaseResponse to connected Origin\n");
	printf("15) Call DbvAbortRequest and exit\n");
        printf("16) Receive data on current association\n");


        printf("20) Image Retrieval Test Send (Generic Record SegmentReq/PresentRsp)\n");

	printf("21) Send AccessControlRequest to connected Origin\n");
	printf("22) Send ResourceControlRequest 'Response Required (TRUE)' to connected Origin\n");
	printf("23) Send ResourceControlRequest 'No Response Required (FALSE)' to connected Origin\n");

	printf("24) Send ExtendedServicesResponse with 'EsTask,ItemOrder, Esrin COrder' to connected Origin\n");
	printf("25) Send InitializeResponse to connected Origin containing CharSet external\n");


        printf("30) Exit\n");

	printf("Enter choice: ");

	result = gets(operation_choice_str);
	post_gets_check(result);
	operation_choice = atoi(operation_choice_str);

	return operation_choice;

}



int print_target_fields(FILE *f, DbvTargetData *psTargetData)
{
        fprintf(f, "---------------------------------------\n");
        fprintf(f, "PDU/event received from origin:\n");

        switch (psTargetData->eTargetEvent) {

        	/* InitRequest from origin  */
                case TA_InitializeInd:
                        print_initreq_fields(f, psTargetData->uTargetData.psInitializeReq);
                        break;
		case TA_SearchInd:
			print_searchreq_fields(f, psTargetData->uTargetData.psSearchReq);
			break;
		case TA_PresentInd:
			print_presreq_fields(f, psTargetData->uTargetData.psPresentReq);
			break;
		case TA_DeleteResultSetInd:
			print_delreq_fields(f, psTargetData->uTargetData.psDeleteResultSetReq);
			break;
		case TA_ScanInd:
			print_scanreq_fields(f, psTargetData->uTargetData.psScanReq);
			break;
		case TA_ResourceReportInd:
			print_resrepreq_fields(f, psTargetData->uTargetData.psResourceReportReq);
			break;
		case TA_SortInd:
			print_sortreq_fields(f, psTargetData->uTargetData.psSortReq);
			break;
		case TA_ExtendedServicesInd:
			print_extendreq_fields(f, psTargetData->uTargetData.psExtendedServicesReq);
			break;
		case TA_CloseInd:
			fprintf(f, ":::Target event: TA_CloseInd:::\n");
			print_closereq_fields(f, psTargetData->uTargetData.psCloseReq);
			break;
		case TA_CloseConf:
			fprintf(f, ":::Target event: TA_CloseConf:::\n");
			print_closereq_fields(f, psTargetData->uTargetData.psCloseReq);
			break;
		case TA_ReleaseInd:
			print_releasereq_fields(f, psTargetData->uTargetData.peReleaseReq);
			break;
		case TA_Invalid_APDU:
			print_invalidapdu(f, psTargetData->uTargetData.psInvalidAPDU);

		case TA_AccessCntrlConf:
			print_accessrsp_fields(f, psTargetData->uTargetData.psAccessControlRsp);
			break;

		case TA_ResourCntrlConf:
			print_rescntrlrsp_fields(f, psTargetData->uTargetData.psResourceControlRsp);
			break;

		case TA_TrigResourCntrlInd:
			print_trigrescntrlreq_fields(f, psTargetData->uTargetData.psTriggerResourceControlReq);
			break;




		default:
			break;
        }

	fprintf(f, "---------------------------------------\n");
}
 
 

int abort_connection_to_origin(DbvAssocId lAssocId, DbvError **ppsError)
{
	int return_value=SI_OK;

	if(DbvAbortRequest(lAssocId, ppsError) == SI_OK)
	{
		printf("DbvAbortRequest was successful\n");
	} else {
		printf("DbvAbortRequest returned SI_NOTOK\n");
		PrivateErrorHandling(ppsError);
		return_value = SI_NOTOK;
	}
	return return_value;
}



int PrivateErrorHandling(DbvError **ppsError)
{
        DbvError *psError;
 
        psError = *ppsError;
        if(psError == NULL)
        {
                printf("Error structure was not filled out!\n");
                return SI_OK;
        }
        else
        {
                printf("psError->eAssociationState == ");
 
                if(psError->eAssociationState == Association_Open)
                        printf("Association_Open\n");
                if(psError->eAssociationState == Association_Closed)
                        printf("Association_Closed\n");
                if(psError->eAssociationState == Association_Unknown_State)
                                printf("Association_Unknown_State\n");

                printf("psError->eErrorClass ==");
                if(psError->eErrorClass == SI_Communication_Error)
                        printf("SI_Communication_Error\n");
                if(psError->eErrorClass == SI_Interface_Error)
                        printf("SI_Interface_Error\n");
                if(psError->eErrorClass == SI_OperatingSystem_Error)
                        printf("SI_OperatingSystem_Error\n");
                if(psError->eErrorClass == SI_Abort)
                {
                        printf("SI_Abort\n");
                        printf("psError->uErrorCode.psSIAbort->eAbortSource == ");
                        switch(psError->uErrorCode.psAbort->eAbortSource)
                        {
                        case SI_ACSE_User:
                                printf("SI_ACSE_User\n");
                                break;
                        case SI_ACSE_Provider:
                                printf("SI_ACSE_Provider\n");
                                break;
                        case SI_Presentation_User:
                                printf("SI_Presentation_User\n");
                                break;
                        case SI_Presentation_Provider:
                                printf("SI_Presentation_Provider\n");
                                break;
                        case SI_Transport_Provider:
                                printf("SI_Transport_Provider\n");
                                break;
                        default:
                                printf("ERROR: Unknown type!\n");
                                break;
                        }
                        printf("psError->uErrorCode.psAbort->lAbortReason == %d\n",psError->uErrorCode.psAbort->lAbortReason);
                        printf("->pcDiagnostics == %s\n", psError->uErrorCode.psAbort->pcDiagnostics);
                }
		else
		{
			printf("psError->uErrorCode.lIfOsCoError == %d\n",psError->uErrorCode.lIfOsCoError);
		}
        }
        return SI_OK;
}
 
 
