/*  Module Name: @(#)error.c	7.5
    Version: 12/6/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: Internal API routine to fill error structure correctly
 
*/

#include "error.h"


/* Complete an error structure from the details given */
void DBVfill_error_structure(  DbvAssocId lAssocId, DbvErrorClass eErrorClass,
			       DbvAbortSource eAbortSource, 
			       DbvAbortReason lAbortReason, char *pcDiagnostics,
			       DbvErrorCode lIfOsCoError, 
			       DbvAssociationState eAssociationState, 
			       DbvError **ppsError)
{
	DbvError *psLocalError;

	/* alocate memory for the structure */
	*ppsError = (struct DbvError *)DbvAlloc(sizeof(struct DbvError));

	psLocalError = *ppsError;

	psLocalError->eErrorClass = eErrorClass;
	psLocalError->eAssociationState = eAssociationState;

	if(eErrorClass == SI_Interface_Error || eErrorClass == 
		SI_OperatingSystem_Error || eErrorClass == 
		SI_Communication_Error)
	{
		psLocalError->uErrorCode.lIfOsCoError=lAbortReason;
	}
	else
	{
		/* An Abort Error */
		psLocalError->uErrorCode.psAbort = (struct DbvAbort *)
			DbvAlloc(sizeof(struct DbvAbort));
		psLocalError->uErrorCode.psAbort->
			eAbortSource = eAbortSource;
		psLocalError->uErrorCode.psAbort->
			lAbortReason = lAbortReason;
		psLocalError->uErrorCode.psAbort->pcDiagnostics = 
			(char *)DbvAlloc(strlen(pcDiagnostics)+1);
		strcpy(psLocalError->uErrorCode.psAbort->pcDiagnostics,
			pcDiagnostics);
	}
	if(lAssocId == 0 || DBVTraceLevel(lAssocId) == Trace_Off)
	/* If 0, then there is no association for this error */
	/* Or No tracefile has been assigned */
		DBVWriteErrorToLog(ppsError, pcDiagnostics);
	else {
		/* Otherwise write errors to the Trace file */
		DBVErrorTrace(lAssocId, ppsError, pcDiagnostics);
	
		/* Could use 
		   printf(pcDiagnostics); printf("\n"); 
	  	   here to output diagnostics to the screen */
	}

}
			
/* Usually called at the beginning of an API function, this function will
	free the error structure if it is not already pointing to NULL */
void DBVCheckError(DbvError **ppsError)
{

	if(*ppsError != NULL)
	{
		if((*ppsError)->eErrorClass == SI_Abort)
		{
			free((*ppsError)->uErrorCode.psAbort->pcDiagnostics);
			free((*ppsError)->uErrorCode.psAbort);
		}
		free(*ppsError);
		*ppsError = (DbvError *)NULL;
	}
}

