/*  Module Name: @(#)originassoc.c	7.3
    Version: 12/2/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: Origin association handling functions Using TCP/OSI Services
*/

#include "originassoc.h"


int DbvAssociateRequest(   DbvAssocId *plAssocId, DbvSecs lSeconds,
                           DbvTarget *psTargetIdentification,
                           DbvAssociateState *peAssociateState,
                           DbvAssociateResult **ppuAssociateResult,
                           DbvError **ppsError )
{

	DbvAssociateState eAssociateStateLocal = Associate_Null;
	DBVAssociateDetails *psAssociateDetails, *psGeneratorAssociateDetails;
	DbvTargetInformation *psTargetInformation = NULL;

	static DbvAssocId IdGenerator = 1;
	int return_value = SI_OK;

	DBVCheckError(ppsError);

	if(DBVAssociateList == (struct SList *)NULL)
		DBVAssociateList = SL_Create( (SL_Free)NULL );

	if(DBVAllocCheck((psAssociateDetails = (DBVAssociateDetails *)
			DBVAlloc(sizeof(DBVAssociateDetails), 
			ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }

	if(psTargetIdentification->eTargetNameOrAddress == Target_Name)
	{
		/* Expand the name if in target_names.tbl file. If in 
			isoentities file leave for osi connect to resolve */	
		return_value = DBVGetAddressByName(psTargetIdentification, 
			&psTargetInformation, ppsError);
	}
	else
		psTargetInformation = psTargetIdentification->uNameOrAddress.
			psTargetInformation;

	if(return_value == SI_OK)
	{
		if(psTargetInformation->eOsiOrInternetAddress == 
			Internet_Address)
		{
			if(lSeconds == CALL_BLOCKING)
			{
			/* Internet Address */
			return_value = DBVTcpConnectToHost(
				  psTargetInformation->uOsiOrInternetAddress.
				  psInternetAddress->pcHostAddress,
				(ushort)psTargetInformation->
				  uOsiOrInternetAddress.
				  psInternetAddress->ulPortNumber,
				&psAssociateDetails->ulTcpSocket,
				&eAssociateStateLocal, psAssociateDetails, 
				ppsError);
			}
			else
			{
			return_value = DBVTcpNonBlockingConnectToHost(
				lSeconds, 
				  psTargetInformation->uOsiOrInternetAddress.
				  psInternetAddress->pcHostAddress,
				(ushort)psTargetInformation->
				  uOsiOrInternetAddress.
				  psInternetAddress->ulPortNumber,
				&psAssociateDetails->ulTcpSocket,
				&eAssociateStateLocal, psAssociateDetails,
				ppsError);
			}

			if((eAssociateStateLocal == Associate_Accepted) && 
				(return_value == SI_OK))
			{
				return_value = DBVFillTcpAssociateResult(
					ppuAssociateResult, ppsError);
			}
		}
		else
		{
			/* OSI Address */
#ifndef NO_OSI
			return_value = DBVOsiConnectToHost(lSeconds, 
				psTargetInformation, &eAssociateStateLocal, 
				psAssociateDetails, ppuAssociateResult,
				ppsError);
#else
			printf("OSI not supported\n");
#endif /* NO_OSI */
/*
		else
		{
			ERROR 
			sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, 
				"Value for psTargetInformation->eOsiOrInternetAddress is invalid");
			DBVfill_error_structure(0, 
				SI_Interface_Error, 0, 
				DBV_INVALIDPARAMS, DBVErrorBuffer, 0,
				Association_Closed, ppsError);
			return_value = SI_NOTOK;
		}
*/
		}
	}

	if(return_value == SI_OK && eAssociateStateLocal != Associate_Rejected)
	{
		/* Generate a unique Association id */
		do {
			/* Just check that it doesn't already exist */
			psGeneratorAssociateDetails = 
				SL_Locate(DBVAssociateList, (SL_LocateFunc)
				DBVLocateAssociate, &IdGenerator);
			if(psGeneratorAssociateDetails == NULL)
				*plAssocId = IdGenerator;
			IdGenerator++;
			if(IdGenerator == LONG_MAX)
				IdGenerator = 0;
		}
		while(psGeneratorAssociateDetails != NULL);
		/* Fill Details of new associate and add to list */
		psAssociateDetails->eOsiOrInternetAddress = 
			psTargetInformation->eOsiOrInternetAddress;
		psAssociateDetails->eAssociateState = eAssociateStateLocal;
		psAssociateDetails->lAssocId = *plAssocId;
		psAssociateDetails->eReleaseState = Release_Null;
		psAssociateDetails->eOriginOrTarget = O;

		SL_Add(DBVAssociateList, psAssociateDetails);

		/* STATE MACHINE - Initialisation */
		if(eAssociateStateLocal == Associate_Accepted)
			DBVSetState(*plAssocId, DBV_OS_ASSOCIATION);
		if(eAssociateStateLocal == Associate_Pending)
			DBVSetState(*plAssocId, DBV_OS_ASSOCREQ_INPROGRESS);
	}
	if(return_value == SI_OK)
	{
		*peAssociateState = eAssociateStateLocal;
	}

	if(psTargetIdentification->eTargetNameOrAddress == Target_Name)
		DBVFreeTargetInformation(psTargetInformation);

	if(return_value == SI_NOTOK || 
		eAssociateStateLocal == Associate_Rejected)
		DBVFree(psAssociateDetails);

	return return_value;

}

int DbvReleaseRequest(     DbvAssocId lAssocId, DbvSecs lSeconds,
                           DbvReleaseState *peReleaseState,
                           DbvError **ppsError )
{
	DbvReleaseState eReleaseStateLocal = Release_Null;
	int return_value = SI_OK;
	DBVAssociateDetails *psAssociateDetails = NULL;
	DbvProtocolStack eProtocolStack;
	
	/* TCP/IP association checking */
	int select_result = 0;
	char buffer[10];
	struct timeval timeout;
	struct fd_set reader, writer;

#ifndef NO_OSI
	/* ISODE */
        struct AcSAPrelease *psAssocControlRelease=NULL;
	struct AcSAPindication *psAssocControlIndication = NULL;
	struct AcSAPabort *psAssocControlAbort = NULL;
#endif

	DBVEnterFuncTrace("DbvReleaseRequest",lAssocId);

	DBVTrace(lAssocId, Trace_High, "\nInput Function Arguments\n");
	Trace_writeDbvAssocId(lAssocId);
	Trace_writeDbvSecs(lSeconds, lAssocId);

        DBVCheckError(ppsError);

#ifndef NO_OSI
	/* Malloc the ISODE structures */
	if(DBVAllocCheck((psAssocControlRelease=(struct AcSAPrelease *)DBVAlloc
		(sizeof(struct AcSAPrelease), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
        if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication),ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort = &psAssocControlIndication->aci_abort;
#endif

	/* STATE MACHINE */
	return_value = DBVChangeState(lAssocId, DBV_OE_IRELREQ, ppsError);

	if(return_value == SI_OK && (SIWhichStack(lAssocId, &eProtocolStack, 
		ppsError) == SI_OK))
	{
		psAssociateDetails = SL_Current(DBVAssociateList);
		if(eProtocolStack == TCP_Stack)
		{
		/* To maintain consistency with an OSI association, the 
			socket is checked before a release is attempted. 
			For an explanation of this bit of code see 
			z3950_send_receive.c */

			FD_ZERO(&reader);
			FD_ZERO(&writer);
			FD_SET(psAssociateDetails->ulTcpSocket, &reader);
			FD_SET(psAssociateDetails->ulTcpSocket, &writer);

			select_result = select(FD_SETSIZE, &reader, &writer,
				NULL, NULL);
			if (FD_ISSET(psAssociateDetails->ulTcpSocket, &reader) 
				&& FD_ISSET(psAssociateDetails->ulTcpSocket, 
				&writer)) {
				if (recv(psAssociateDetails->ulTcpSocket, 
					buffer, 1, MSG_PEEK) == 0)
				{
					/* The association has been lost. We'll
					assume this means an abort */
					if(psAssociateDetails->eOriginOrTarget 
					    = O)
						DBVChangeState(lAssocId,
						  DBV_OE_AABORT_IND, ppsError);
                        		DBVfill_error_structure(lAssocId, 
						SI_Abort, 0, DBV_ABORT, 
						DBV_ABORT_S, 0, 
						Association_Closed, ppsError);
					return_value = SI_NOTOK;
				}
			}

		
			/* For TCP_Stack we are not interested in the value 
				of lSeconds */
			/* Close the socket */
			if(return_value == SI_OK && 
				(close(psAssociateDetails->ulTcpSocket))<0)
			{
				/* ReleaseRequest:Close failed */
				if (errno > 0 && errno < sys_nerr)
					sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, sys_errlist[errno]);
				else
					sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, "");

				DBVfill_error_structure(lAssocId, 
					SI_Communication_Error, 0, 
					DBV_TCP_CLOSEFAILED, DBVErrorBuffer, 0,
					Association_Unknown_State, ppsError);
				return_value = SI_NOTOK;
			}
			else
				eReleaseStateLocal = Release_Accepted;
		}
		else
		{
#ifndef NO_OSI
			if(AcRelRequest(psAssociateDetails->lOSIAssocId, 
				ACF_NORMAL, NULLPEP, 0, lSeconds, 
				psAssocControlRelease, 
				psAssocControlIndication)==NOTOK)
			{
				/* Has the request timed out? */
				if(psAssocControlAbort->aca_reason == ACS_TIMER)
				{
					eReleaseStateLocal = Release_Pending;
					psAssociateDetails->eReleaseState =
						Release_Pending;
				}
				else
				{
					/* Error has occurred */
					DBVTrace(lAssocId, Trace_Low, 
						"A-RELEASE.REQUEST error\n");
					DBVACSEOSICheckISODEError(lAssocId, 
						psAssociateDetails->lOSIAssocId,
						psAssocControlIndication,
						ppsError);
					return_value = DBVChangeState(lAssocId, 
						DBV_OE_AABORT_IND, ppsError);
					return_value = SI_NOTOK;
				}
			}
			else
			{
				if(!psAssocControlRelease->acr_affirmative)
					eReleaseStateLocal = Release_Rejected;
				else
					eReleaseStateLocal = Release_Accepted;
			}
#else
			printf("OSI not supported\n");
#endif /* NO_OSI */
		}
	}
	else
/* SIWhichStack failed! - Error Structure already filled by SIWhichStack*/
		return_value = SI_NOTOK;

	if(return_value == SI_OK)
	{
		*peReleaseState = eReleaseStateLocal;
		DBVTrace(lAssocId, Trace_High, "\nOutput Function Arguments\n");
		Trace_writeDbvReleaseState(peReleaseState, lAssocId);
	}

	if((return_value == SI_OK && eReleaseStateLocal == Release_Accepted) ||
		(return_value == SI_NOTOK && (*ppsError)->eErrorClass == 
			SI_Abort))
	{
		/* Free up space of removed association set by SIWhichStack*/
		psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
		free(psAssociateDetails);
	}

#ifndef NO_OSI
	/* Free the ISODE functions */
	ACRFREE(psAssocControlRelease);
	ACAFREE(psAssocControlAbort);
	DbvFree(psAssocControlRelease);
	DbvFree(psAssocControlIndication);
#endif /* NO_OSI */
	
	DBVLeaveFuncTrace("DbvReleaseRequest",lAssocId, return_value);
	return return_value;
}



int DbvConfirmAssociateRequest(    DbvAssocId lAssocId, DbvSecs lSeconds,
                                   DbvAssociateState *peAssociateState,
                                   DbvAssociateResult **ppuAssociateResult,
                                   DbvError **ppsError )
{
	DbvAssociateState eAssociateStateLocal = Associate_Null;
        DBVAssociateDetails *psAssociateDetails;
	int return_value = SI_OK;
        DbvProtocolStack eProtocolStack;

#ifndef NO_OSI
	struct AcSAPconnect *psAssocControlConnect = NULL;
	struct AcSAPindication *psAssocControlIndication = NULL;
	struct AcSAPabort *psAssocControlAbort = NULL;
#endif /* NO_OSI */



	/* To complete an Osi association you must select (xselect) for 
		writing, then reading */
	fd_set sAssocMask, sSetAssocMask, write, read;
	int select_result = 0;
	int lNfds=1;
	/* Stores the result of calls to xselect() and PSelectMask() */
	long lResult;
	char buffer[10];
	/* We have to know how long to continue waiting, because waiting for
		the completion of an association goes through several stages */
	long lSecondsToWait = 1;
	long lTimeout = 1; /* the xselect commands will pause for a default
				one second. Value is set to 0 if call is
				CALL_ASYNCHRONOUS */
	struct timeval timeout;
	int result;
 
	DBVEnterFuncTrace("DbvConfirmAssociateRequest",lAssocId);

	DBVTrace(lAssocId, Trace_High, "\nInput Function Arguments\n");
	Trace_writeDbvAssocId(lAssocId);
	Trace_writeDbvSecs(lSeconds, lAssocId);

        DBVCheckError(ppsError);

#ifndef NO_OSI
	if(DBVAllocCheck((psAssocControlConnect = (struct AcSAPconnect *)
		DBVAlloc(sizeof(struct AcSAPconnect), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
        if(DBVAllocCheck((psAssocControlIndication = (struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }

	psAssocControlAbort = &psAssocControlIndication->aci_abort;
#endif /* NO_OSI */ 

        if(SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
	{
	psAssociateDetails = SL_Current(DBVAssociateList);
	if(psAssociateDetails->eAssociateState == Associate_Accepted)
	{
		/* Connection is already established */
		sprintf(DBVErrorBuffer, DBV_INVALIDCALL_S, 
			"Association has already been accepted");
		DBVfill_error_structure(lAssocId,SI_Interface_Error, 0, 
			DBV_INVALIDCALL, DBVErrorBuffer,0,Association_Open, 
			ppsError);
		return_value = SI_NOTOK;
	}
	else
	{
		/* Complete the connection */
		if(psAssociateDetails->eOsiOrInternetAddress == 
			Internet_Address)
		{

		signal(SIGPIPE, SIG_IGN);

                do {
                errno = -1;
                if((connect(psAssociateDetails->ulTcpSocket, 
			(struct sockaddr *)&psAssociateDetails->sa, 
			sizeof psAssociateDetails->sa)) < 0)
                {
			if(errno == EISCONN)
				*peAssociateState = Associate_Accepted;

                        else if ((errno == EALREADY) || ( errno == EAGAIN))
                        {
                                FD_ZERO(&write);
                                FD_SET(psAssociateDetails->ulTcpSocket, &write);
				if(lSeconds == CALL_BLOCKING)
				{
					result=select(FD_SETSIZE,  (fd_set *) 0,
						&write, (fd_set *) 0, 
						(struct timeval *) NULL);
				}
				else
				{
                                	timeout.tv_sec = lSeconds;
                                	timeout.tv_usec = 0;
                                	result=select(FD_SETSIZE,  (fd_set *) 0,
						&write, (fd_set *) 0, &timeout);
				}
 
                                if(result==-1)
                                {
                        		if(errno != EINTR)
                        		{
						if (errno > 0 && errno < sys_nerr)
                                			sprintf(DBVErrorBuffer, DBV_SYSERROR_S, "select", sys_errlist[errno]);
						else
                                			sprintf(DBVErrorBuffer, DBV_SYSERROR_S, "select", "");

                                		DBVfill_error_structure(
						   lAssocId,
						   SI_OperatingSystem_Error, 0,
						   DBV_SYSERROR, DBVErrorBuffer,
						   0,Association_Unknown_State, 
						   ppsError);
                                		return_value = SI_NOTOK;
                        		}
                                }
 
                                if((connect(psAssociateDetails->ulTcpSocket, 
				    (struct sockaddr *)&psAssociateDetails->sa, 
				    sizeof psAssociateDetails->sa)) < 0)
                                {
                                        if(errno == EISCONN)
                                                *peAssociateState = Associate_Accepted;
                                        else
                                        {

						/* EAGAIN on Solaris 2 */
                                                if ((errno == EALREADY) || (errno == EAGAIN))
                                                {
                                                        /* Add check here */
                                                        *peAssociateState = Associate_Pending;
                                                }
                                                else
                                                {
							if (errno > 0 && errno < sys_nerr)
                                                        	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, sys_errlist[errno]);
							else
                                                        	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, "");

                                                        DBVfill_error_structure( lAssocId,SI_Communication_Error, 0, DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
                                                        return_value = SI_NOTOK;
                                                }
                                        }
                                }
                		else
                        		*peAssociateState = Associate_Accepted;
                        }
                        else if(errno != EINTR)
                        {
				if (errno > 0 && errno < sys_nerr)
                                	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, sys_errlist[errno]);
				else
                                	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, "");
                                DBVfill_error_structure(lAssocId,
					SI_Communication_Error, 0, 
					DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,
					Association_Closed, ppsError);
                                return_value = SI_NOTOK;
                        }
                }
                else
                        *peAssociateState = Associate_Accepted;

		if(*peAssociateState == Associate_Accepted)
		{
		/* Check that association is still valid. In some cases, TCP/IP
			returns connected when the target has quit. */
		FD_ZERO(&read);
		FD_ZERO(&write);
		FD_SET(psAssociateDetails->ulTcpSocket, &read);
		FD_SET(psAssociateDetails->ulTcpSocket, &write);
 
		select_result = select(FD_SETSIZE, &read, &write,
			NULL, NULL);
		if (FD_ISSET(psAssociateDetails->ulTcpSocket, &read)
			&& FD_ISSET(psAssociateDetails->ulTcpSocket,
			&write)) {
			if (recv(psAssociateDetails->ulTcpSocket,
				buffer, 1, MSG_PEEK) == 0)
			{
			/* The association has been lost. We'll
				assume this means an abort */
				if(psAssociateDetails->eOriginOrTarget
					= O)
					DBVChangeState(lAssocId,
					DBV_OE_AABORT_IND, ppsError);
					DBVfill_error_structure(lAssocId,
					SI_Abort, 0, DBV_ABORT,
					DBV_ABORT_S, 0,
					Association_Closed, ppsError);
				return_value = SI_NOTOK;
				*peAssociateState = Associate_Pending;
			}
		}
		}
                }
                while(errno == EINTR);

		if((*peAssociateState == Associate_Accepted) &&
			(return_value == SI_OK))
		{
			return_value = DBVFillTcpAssociateResult(
				ppuAssociateResult, ppsError);
		}

		signal(SIGPIPE, SIG_DFL);
		}
		else
			/* OSI Connection */
		{
#ifndef NO_OSI
			return_value = DBVOsiCompleteConnectToHost(lAssocId, 
				lSeconds, psAssocControlConnect,
				psAssocControlIndication, psAssociateDetails, 
				ppsError);

			if(return_value == SI_OK && 
				psAssociateDetails->lAssocResult == DONE)
			{
				return_value=DBVOsiCompleteDbvAssociateResult(
					psAssocControlConnect, 
					psAssocControlAbort, ppuAssociateResult,
					peAssociateState, ppsError);
			}
#else
			printf("OSI not supported\n");
#endif /* NO_OSI */ 
		}
	}
	}
	else
		/* Error structure is filled by SIWhichStack */
		return_value = SI_NOTOK;

	if(return_value == SI_OK)
	{
		if(*peAssociateState ==  Associate_Accepted)
		{
			/* STATE MACHINE */
			return_value = DBVChangeState(lAssocId,
				DBV_OE_AASSOCIATERESP_IND_TRUE, ppsError);
			psAssociateDetails->eAssociateState =
				Associate_Accepted;
		}
		/* Remove the association from the list if it was rejected */
		if(*peAssociateState == Associate_Rejected)
		{
			/* Free up space of rejected association set by 
				SIWhichStack*/
			psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
			free(psAssociateDetails);
		}

		if(psAssociateDetails->eAssociateState == Associate_Pending)
			*peAssociateState = Associate_Pending;
	}
#ifndef NO_OSI
/* Free the extra bits ISODE malloc()'ed. */
	ACCFREE(psAssocControlConnect);
	ACAFREE(psAssocControlAbort);
		
/* Free the isode structures */
	DBVFree(psAssocControlConnect);
        DBVFree(psAssocControlIndication);
#endif /* NO_OSI */
	if(return_value == SI_OK)
	{
		DBVTrace(lAssocId, Trace_High, "\nOutput Function Arguments\n");
		Trace_writeDbvAssociateState(peAssociateState, lAssocId);
		Trace_writeDbvAssociateResult(*ppuAssociateResult,
			*peAssociateState, lAssocId);
	}
	DBVLeaveFuncTrace("DbvConfirmAssociateRequest",lAssocId, return_value);

	signal(SIGPIPE, SIG_DFL);
	return return_value;
}
	



int DbvConfirmReleaseRequest(     DbvAssocId lAssocId, DbvSecs lSeconds,
                         	  DbvReleaseState *peReleaseState,
                         	  DbvError **ppsError )
{
	int return_value = SI_OK;
	DbvProtocolStack eProtocolStack;
	DBVAssociateDetails *psAssociateDetails;
	DbvReleaseState eReleaseStateLocal = Release_Pending;

#ifndef NO_OSI
	/* ISODE */
        struct AcSAPrelease *psAssocControlRelease=NULL;
	struct AcSAPindication *psAssocControlIndication = NULL;
	struct AcSAPabort *psAssocControlAbort = NULL;
#endif /* NO_OSI */

	DBVEnterFuncTrace("DbvConfirmReleaseRequest",lAssocId);

	DBVTrace(lAssocId, Trace_High, "\nInput Function Arguments\n");
	Trace_writeDbvAssocId(lAssocId);
	Trace_writeDbvSecs(lSeconds, lAssocId);

        DBVCheckError(ppsError);

#ifndef NO_OSI
	/* Malloc the ISODE structures */
	if(DBVAllocCheck((psAssocControlRelease=(struct AcSAPrelease *)DBVAlloc
		(sizeof(struct AcSAPrelease), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
        if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort = &psAssocControlIndication->aci_abort;
#endif /* NO_OSI */

        if(SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
	{
		psAssociateDetails = SL_Current(DBVAssociateList);
		if(psAssociateDetails->eReleaseState == Release_Pending)
		{
			if(eProtocolStack == TCP_Stack)
			{
				/* This will never happen! */
			}
			else
			{
#ifndef NO_OSI
			if(AcRelRetryRequest(psAssociateDetails->lOSIAssocId, 
				lSeconds, psAssocControlRelease, 
				psAssocControlIndication)==NOTOK)
			{
				/* Has the request timed out? */
				if(psAssocControlAbort->aca_reason == ACS_TIMER)
				{
					eReleaseStateLocal = Release_Pending;
					psAssociateDetails->eReleaseState =
						Release_Pending;
				}
				else
				{
                                        /* Error has occurred */
                                        DBVTrace(lAssocId, Trace_Low,
                                                "A-RELEASE.REQUEST error\n");
                                        DBVACSEOSICheckISODEError(lAssocId,
                                                psAssociateDetails->lOSIAssocId,
                                                psAssocControlIndication,
                                                ppsError);
                                        return_value = DBVChangeState(lAssocId,
                                                DBV_OE_AABORT_IND, ppsError);
                                        return_value = SI_NOTOK;

				}
			}
			else
			{
				if(!psAssocControlRelease->acr_affirmative)
					eReleaseStateLocal = Release_Rejected;
				else
					eReleaseStateLocal = Release_Accepted;
			}
#else
				printf("OSI not supported\n");
#endif /* NO_OSI */
			}
		}
		else
		{
			sprintf(DBVErrorBuffer, DBV_INVALIDCALL_S, "The release state of the supplied association is not Release_Pending");
			DBVfill_error_structure(0,SI_Interface_Error, 0, DBV_INVALIDCALL, DBVErrorBuffer,0,Association_Open, ppsError);
			return_value = SI_NOTOK;
		}
			
	}
	else
		/* Error structure filled by SIWhichStack */
		return_value = SI_NOTOK;

	if(return_value == SI_OK)
	{
		*peReleaseState = eReleaseStateLocal;
		DBVTrace(lAssocId, Trace_High, "\nOutput Function Arguments\n");
		Trace_writeDbvReleaseState(peReleaseState, lAssocId);
	}

	if(return_value == SI_OK && eReleaseStateLocal == Release_Accepted)
	{
		/* Free up space of removed association set by SIWhichStack*/
		psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
		free(psAssociateDetails);
	}

#ifndef NO_OSI
	/* Free the ISODE functions */
	ACRFREE(psAssocControlRelease);
	ACAFREE(psAssocControlAbort);
	free(psAssocControlIndication);
	free(psAssocControlRelease);
#endif

	DBVLeaveFuncTrace("DbvConfirmReleaseRequest",lAssocId, return_value);
	return return_value;
}


/* Connect to a host using TCP/IP */
int DBVTcpConnectToHost(char *host_address, ushort port, DbvFds *socketfd, 
	DbvAssociateState *peAssociateState, DBVAssociateDetails 
	*psAssociateDetails, DbvError **ppsError)
{
	int tempsocketfd;
	int return_value = SI_OK;

	struct sockaddr_in sina;
	struct hostent *host = NULL;


	/* Check if the Internet address is in dot notation (number) form then
	   if it is use the dot notation form, dont call gethostbyname(). 
	   Although most machines allow a dot notation parameter to 
	   gethostbyname(), some dont and return an error 
	   (e.g Siemens-Nixdorf) */

	/* If the Internet address is already in dot notation form */
	sina.sin_addr.s_addr = inet_addr(host_address);
	if (sina.sin_addr.s_addr != -1) {

		/* Store the association socket details */
		bcopy_replacement((char*)&sina.sin_addr, (char *)&psAssociateDetails->sa.sin_addr, sizeof(sina.sin_addr));
		psAssociateDetails->sa.sin_family = AF_INET;
		psAssociateDetails->sa.sin_port = htons(port);

	} else {
		host = gethostbyname(host_address);
		if (host) {

			/* Store the association socket details */

			/* host->h_addr is a #define in /usr/include/netdb.h 
			   for backward compatibility 
			   i.e. #define h_addr  h_addr_list[0] */

			bcopy_replacement((char *)host->h_addr, (char *)&psAssociateDetails->sa.sin_addr, host->h_length);
			psAssociateDetails->sa.sin_family = host->h_addrtype;
			psAssociateDetails->sa.sin_port = htons(port);

		} else {
			sprintf(DBVErrorBuffer, DBV_TCP_INVALIDADDRESS_S, host_address);
			DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_INVALIDADDRESS, DBVErrorBuffer,0,Association_Closed, ppsError);
			return SI_NOTOK;
		}	
	}


	/* Create a socket */
	if((tempsocketfd = socket(psAssociateDetails->sa.sin_family, SOCK_STREAM, 0)) <0)
	{
		if (errno > 0 && errno < sys_nerr)
			sprintf(DBVErrorBuffer, DBV_TCP_SOCKETFAILED_S, sys_errlist[errno]);
		else
			sprintf(DBVErrorBuffer, DBV_TCP_SOCKETFAILED_S, "");

		DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_SOCKETFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
		return_value = SI_NOTOK;
	}

	if(return_value != SI_NOTOK)
	{

		/* Complete connection - wait indefinately */
		do {
		errno = -1;
		if((connect(tempsocketfd, (struct sockaddr *)&psAssociateDetails->sa, sizeof psAssociateDetails->sa)) < 0)
		{
			if(errno != EINTR)
			{
				if (errno > 0 && errno < sys_nerr)
					sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, sys_errlist[errno]);
				else
					sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, "");

				DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
               			return_value = SI_NOTOK;
			}
			else
				printf("Interrupted!\n");
		}
		else
			*peAssociateState = Associate_Accepted;
		}
		while(errno == EINTR);
	}

	if(return_value == SI_OK)
			*socketfd = tempsocketfd;
	else
		close(tempsocketfd);
		/* make sure socket is deleted? */

	return return_value;
}

/* Connect to a host using TCP/IP */
int DBVTcpNonBlockingConnectToHost(DbvSecs lSeconds, char *host_address, 
	ushort port, DbvFds *socketfd, DbvAssociateState *peAssociateState, 
	DBVAssociateDetails *psAssociateDetails, DbvError **ppsError)
{
        int tempsocketfd;
        int return_value = SI_OK;
 
        struct sockaddr_in sina;
        struct hostent *host = NULL;
	
	struct timeval timeout;
	fd_set		write;
	int		result;
	int		conres;
	int		onoff; 


	/* Ignore pipe signals  - a pipe signal can occur on Solaris 2 when
	   the second connect of the sequence is attempted if the target is
	   not running */
	signal(SIGPIPE, SIG_IGN);

 
        /* Check if the Internet address is in dot notation (number) form then
           if it is use the dot notation form, dont call gethostbyname().
           Although most machines allow a dot notation parameter to
           gethostbyname(), some dont and return an error
           (e.g Siemens-Nixdorf) */
 
        /* If the Internet address is already in dot notation form */
        sina.sin_addr.s_addr = inet_addr(host_address);
        if (sina.sin_addr.s_addr != -1) {
 
                /* Store the association socket details */
                bcopy_replacement((char*)&sina.sin_addr, 
			(char *)&psAssociateDetails->sa.sin_addr, 
			sizeof(sina.sin_addr));
                psAssociateDetails->sa.sin_family = AF_INET;
                psAssociateDetails->sa.sin_port = htons(port);
 
        } else {
                host = gethostbyname(host_address);
                if (host) {
 
                        /* Store the association socket details */
 
                        /* host->h_addr is a #define in /usr/include/netdb.h
                           for backward compatibility
                           i.e. #define h_addr  h_addr_list[0] */
 
                        bcopy_replacement((char *)host->h_addr, 
				(char *)&psAssociateDetails->sa.sin_addr, 
				host->h_length);
                        psAssociateDetails->sa.sin_family = host->h_addrtype;
                        psAssociateDetails->sa.sin_port = htons(port);
 
                } else {
                        sprintf(DBVErrorBuffer, DBV_TCP_INVALIDADDRESS_S, 
				host_address);
                        DBVfill_error_structure(0,SI_Communication_Error, 0, 
				DBV_TCP_INVALIDADDRESS, DBVErrorBuffer,0,
				Association_Closed, ppsError);

			/* Default action for pipe signals */
			signal(SIGPIPE, SIG_DFL);
                        return SI_NOTOK;
                }
        }

        /* Create a socket */
        if((tempsocketfd = socket(psAssociateDetails->sa.sin_family, SOCK_STREAM
		, 0)) <0)
        {
		if (errno > 0 && errno < sys_nerr)
                	sprintf(DBVErrorBuffer, DBV_TCP_SOCKETFAILED_S, sys_errlist[errno]);
		else
                	sprintf(DBVErrorBuffer, DBV_TCP_SOCKETFAILED_S, "");

                DBVfill_error_structure(0,SI_Communication_Error, 0, 
			DBV_TCP_SOCKETFAILED, DBVErrorBuffer,0,
			Association_Closed, ppsError);
                return_value = SI_NOTOK;
        }
 

	/* Make socket BSD 4.2 style non-blocking (POSIX and Sys V have
	   different characteristics!) */
	onoff = 1;
	ioctl(tempsocketfd, FIONBIO, (char *)&onoff);

        if(return_value != SI_NOTOK)
        {
 
                /* Complete connection - wait for lSeconds */
                do {
                errno = -1;
                if((connect(tempsocketfd, (struct sockaddr *)&psAssociateDetails
->sa, sizeof psAssociateDetails->sa)) < 0)
                {
			if(errno == EINPROGRESS)
			{
				FD_ZERO(&write);
				FD_SET(tempsocketfd, &write);
				timeout.tv_sec = lSeconds;
				timeout.tv_usec = 0;

				result=select(FD_SETSIZE,  (fd_set *) 0, &write,
					(fd_set *) 0, &timeout);

				if(result==-1)
                        	{
                                	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, " select() error");
                                	DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
                                	return SI_NOTOK;
                        	}
				
				if((connect(tempsocketfd, (struct sockaddr *)&psAssociateDetails->sa, sizeof psAssociateDetails->sa)) < 0)
				{
					if(errno == EISCONN)
					{
						*peAssociateState = Associate_Accepted;
					}
					else
					{

						/* EAGAIN on Solaris 2 */
						if ((errno == EALREADY) || (errno == EAGAIN))
						{
							/* Add check here */
							*peAssociateState = Associate_Pending;
						}
						else
						{
							if (errno > 0 && errno < sys_nerr)
                                				sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, sys_errlist[errno]);
							else
                                				sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, "");

                                			DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
                                			return_value = SI_NOTOK;
						}
					}
				}
				else
                        		*peAssociateState = Associate_Accepted;
		
			}
			else if(errno != EINTR)
                        {
				if (errno > 0 && errno < sys_nerr)
                                	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, sys_errlist[errno]);
				else
                                	sprintf(DBVErrorBuffer, DBV_TCP_CONNECTFAILED_S, "");

                                DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_TCP_CONNECTFAILED, DBVErrorBuffer,0,Association_Closed, ppsError);
                                return_value = SI_NOTOK;
                        }
                        else
                                printf("Interrupted!\n");
                }
                else
                        *peAssociateState = Associate_Accepted;
                }
                while(errno == EINTR);
        }
 
        if(return_value == SI_OK)
                        *socketfd = tempsocketfd;
        else
                close(tempsocketfd);
                /* make sure socket is deleted? */
 
	/* Default action for pipe signals */
	signal(SIGPIPE, SIG_DFL);

        return return_value;
}


#ifndef NO_OSI

/* Submit an AAsociate.Request service primitive. Read the ISODE manuals, 
volumes 9,10,13,16 if you REALLY want to know what is going on here */
int DBVOsiConnectToHost(DbvSecs lSeconds, DbvTargetInformation 
	*psTargetInformation, DbvAssociateState *peAssociateState, 
	DBVAssociateDetails *psAssociateDetails, 
	DbvAssociateResult **ppuAssociateResult, DbvError **ppsError)
{
	int return_value = SI_OK;
	int lRequirements = DBV_SESSION_REQUIREMENTS; /* See sil_defs.h */
	int lCount=0;
	char *pcTargetName = NULL;
	AEI psAppEntityInfo;
	OID psContextOID=NULL;
	OID psDefaultContext=NULL;
	DbvSecs lLocalSeconds;
	char *pcContext=NULL;

	struct AcSAPconnect *psAssocControlConnect = NULL;
	struct AcSAPindication *psAssocControlIndication = NULL;
	struct AcSAPabort *psAssocControlAbort = NULL;

	struct PSAPaddr *psPresentationAddress = NULL;
	struct PSAPctxlist *psPresentationContextList = NULL;

	struct SSAPref sSessionRef;
	struct SSAPref *psSessionRef;

	struct QOStype sQOS;

	/* Allocate memory for the ISODE structures */
	if(DBVAllocCheck((psAssocControlConnect = (struct AcSAPconnect *)
		DBVAlloc(sizeof(struct AcSAPconnect), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	
	psAssocControlAbort = &psAssocControlIndication->aci_abort;
	lLocalSeconds = lSeconds;

	/* Get the desired application context and presentation context
		list from the ACSE configuration file */
	return_value = DBVGetACSEDetails(&pcContext, &psContextOID, 
		&psPresentationContextList, ppsError);
	if(return_value == SI_NOTOK)
		return return_value;

	return_value = DBVConstructOSIAddress(&pcTargetName, 
		psTargetInformation, ppsError);
	
	/* Create the Application Entity Information */
	if((psAppEntityInfo=str2aeinfo(pcTargetName, DBV_SERVICE,0,
        	NULLCP,NULLCP))==NULLAEI)
	{

		sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, "In function DBVOsiConnectToHost(): Cannot create Application Entity Information. Check isoentities file/Address supplied");
		DBVfill_error_structure(0, SI_Interface_Error,
			0, DBV_OSI_ISODE_ERROR, DBVErrorBuffer,
			0, Association_Closed,ppsError);
		return SI_NOTOK;
	}

	if((psPresentationAddress=(struct PSAPaddr *)aei2addr(psAppEntityInfo))
		== NULLPA)
	{
		printf("OSI: Address translation failed\n");
		return_value = SI_NOTOK;
	}
	/* The connection identifier used by the origin */
	if((psSessionRef=addr2ref(PLocalHostName()))==NULL)
	{
		psSessionRef=&sSessionRef;
		(void)bzero((char *)psSessionRef, sizeof *psSessionRef);
	}

	/* Setup Quality of service to be HIGH_QUALITY */
	sQOS.qos_reliability = HIGH_QUALITY;
	sQOS.qos_sversion = NOTOK;
	sQOS.qos_extended = 0;
	sQOS.qos_maxtime = -1;

	/* The ISODE function to call depends on the value of lLocalSeconds */
	if(lLocalSeconds == CALL_BLOCKING)
	{
		psAssociateDetails->lAssocResult=AcAssocRequest(psContextOID, 
			NULLAEI, NULLAEI, NULLPA, psPresentationAddress, 
			psPresentationContextList, psDefaultContext, 0, 
			lRequirements, SERIAL_NONE, 0, psSessionRef, NULLPEP, 0,
			&sQOS, psAssocControlConnect, psAssocControlIndication);
	}
	else
	{
		if(lLocalSeconds == CALL_ASYNCHRONOUS)
			lLocalSeconds = 1;
		psAssociateDetails->lAssocResult=AcAsynAssocRequest(
			psContextOID, NULLAEI, NULLAEI, NULLPA, 
			psPresentationAddress, psPresentationContextList, 
			NULLOID, 0, lRequirements, SERIAL_NONE, 0, psSessionRef,
			NULLPEP, 0, &sQOS, psAssocControlConnect, 
			psAssocControlIndication, lLocalSeconds);
	}

	if(psAssocControlConnect->acc_result != ACS_ACCEPT ||
		psAssociateDetails->lAssocResult == NOTOK)
	{
		/* Go straight to log file, because there is no association */
		DBVLogISODEACSEError("Ac(Asyn)AssocRequest()", 
			psAssocControlAbort);
	}
	if(psAssociateDetails->lAssocResult != NOTOK)
		psAssociateDetails->lOSIAssocId=psAssocControlConnect->acc_sd;

	if(psAssociateDetails->lAssocResult == CONNECTING_1 ||
		psAssociateDetails->lAssocResult == CONNECTING_2)
	{
		*peAssociateState=Associate_Pending;
		/* If the request is specified with a timeout we must call
			DBVOsiCompleteConnectToHost to wait for the 
			appropriate time */
		if(lSeconds > CALL_ASYNCHRONOUS)
			DBVOsiCompleteConnectToHost(0, lSeconds,
        			psAssocControlConnect,
        			psAssocControlIndication, psAssociateDetails, 
				ppsError);
	}

	if(psAssociateDetails->lAssocResult == NOTOK)
	{
		DBVWriteOriginLog("A-ASSOCIATE.REQUEST error");
		sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, "In function DBVOsiConnectToHost(): ISODE function Ac(Asyn)AssocRequest() failed");
		DBVfill_error_structure(0, SI_Interface_Error,
			0, DBV_OSI_ISODE_ERROR, DBVErrorBuffer,
			0, Association_Closed,ppsError);
		return_value = SI_NOTOK;
	}
	else if(psAssociateDetails->lAssocResult == DONE || 
	    (psAssociateDetails->lAssocResult == OK && 
		lSeconds == CALL_BLOCKING))
		return_value=DBVOsiCompleteDbvAssociateResult(
			psAssocControlConnect, psAssocControlAbort, 
			ppuAssociateResult, peAssociateState, ppsError);

	if(psAssociateDetails->lAssocResult == OK)
		ACCFREE(psAssocControlConnect);
	ACAFREE(psAssocControlAbort);
	AEIFREE(psAppEntityInfo);
	/*free(psAssocControlConnect);*/
	free(psAssocControlIndication);
	free(psPresentationContextList);

	return return_value;
}

int DBVOsiCompleteConnectToHost(DbvAssocId lAssocId, DbvSecs lSeconds, 
        struct AcSAPconnect *psAssocControlConnect,
	struct AcSAPindication *psAssocControlIndication, DBVAssociateDetails 
	*psAssociateDetails, DbvError **ppsError)
{ 
        struct PSAPindication *psPresentationIndication = NULL;
        struct AcSAPabort *psAssocControlAbort = NULL;
 
        /* To complete an Osi association you must select (xselect) for
                writing, then reading */
        fd_set sAssocMask, sSetAssocMask, write;
        int lNfds=1;
        /* Stores the result of calls to xselect() and PSelectMask() */
        long lResult;
        /* We have to know how long to continue waiting, because waiting for
                the completion of an association goes through several stages */
        long lSecondsToWait = 1;
        long lTimeout = 1; /* the xselect commands will pause for a default
                                one second. Value is set to 0 if call is
                                CALL_ASYNCHRONOUS */
        struct timeval timeout;
        int result;
	int return_value = SI_OK;
 
	if(DBVAllocCheck((psPresentationIndication = (struct PSAPindication *)
		DBVAlloc(sizeof(struct PSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }

        psAssocControlAbort = &psAssocControlIndication->aci_abort;

        if(lSeconds == CALL_ASYNCHRONOUS)
                lTimeout = OK;
        if(lSeconds >= 1)
                lSecondsToWait=lSeconds;

        /*lResult = psAssociateDetails->lAssocResult;*/
        /* Place the association id in the mask */
        lResult=PSelectMask(psAssociateDetails->lOSIAssocId,
                &sSetAssocMask, &lNfds, psPresentationIndication);
        if(lResult == NOTOK)
        {
                DBVTrace(lAssocId, Trace_Low,
                        "PSelectMask() error");
                DBVPOSICheckISODEError(lAssocId,
                        psAssociateDetails->lOSIAssocId,
                        psPresentationIndication, ppsError);
                return_value = SI_NOTOK;
        }
        /* Do the connection logic */
        while( return_value == SI_OK && lResult != NOTOK && lSecondsToWait > 0 
		&& psAssociateDetails->lAssocResult != DONE)
        {
                lResult = OK;
                sAssocMask = sSetAssocMask;
                if(psAssociateDetails->lAssocResult ==
                        CONNECTING_1)
                        lResult=xselect(lNfds, (fd_set *)NULL,
                                &sAssocMask, (fd_set *)NULL,
                                lTimeout);
                if(psAssociateDetails->lAssocResult ==
                        CONNECTING_2)
                lResult=xselect(lNfds, &sAssocMask,
                        (fd_set *)NULL, (fd_set *)NULL,
                        lTimeout);
                /* Something's Happened */
                if(lResult != OK && lResult != NOTOK)
                        psAssociateDetails->lAssocResult =
                        AcAsynRetryRequest(psAssociateDetails->
                        lOSIAssocId, psAssocControlConnect,
                        psAssocControlIndication);
                if(psAssociateDetails->lAssocResult == NOTOK)
                {
                        printf("A-ASSOCIATE.REQUEST: %s\n",AcErrString(psAssocControlAbort->aca_reason));
                        DBVTrace(lAssocId, Trace_Low,
                                "A-RELEASE.REQUEST error\n");
                        DBVACSEOSICheckISODEError(lAssocId,
                                psAssociateDetails->lOSIAssocId,
                                psAssocControlIndication,
                                ppsError);
                        return_value = DBVChangeState(lAssocId,
                                DBV_OE_AABORT_IND, ppsError);
                        /* *peAssociateState=Associate_Null;*/
                        return_value = SI_NOTOK;
                }
                if(lResult == NOTOK)
                {
                    printf("DBVOsiCompleteConnectToHost: error in xselect\n");
                    /* *peAssociateState=Associate_Null;*/
                }

                if(lSeconds != CALL_BLOCKING)
                        lSecondsToWait-=lTimeout;
                if(lSeconds == CALL_ASYNCHRONOUS)
                        lSecondsToWait = 0;
	}

	PAFREE(&(psPresentationIndication->pi_un.pi_un_abort));
	DBVFree(psPresentationIndication);

	return return_value;
} 

/* Create a string (for the osi function str2aeinfo) from the address 
	supplied */
int DBVConstructOSIAddress(char **ppcTargetName, DbvTargetInformation 
	*psTargetInformation, DbvError **ppsError)
{
	long lCount = 0;
	int return_value = SI_OK;

	if(psTargetInformation->eOsiOrInternetAddress == DBV_OSI_NAME)
	{
		/* The address given is a name in the isoentities file */
		*ppcTargetName = (char *)DbvAlloc(strlen(psTargetInformation->
                        uOsiOrInternetAddress.psOsiAddress->pcPsel)+1);

		strcpy(*ppcTargetName, psTargetInformation->
			uOsiOrInternetAddress.psOsiAddress->pcPsel);

		psTargetInformation->eOsiOrInternetAddress = Osi_Address;
		/* Just return straight away */
		return SI_OK;
	}

	/* Check that the osi address is completed */
	if(psTargetInformation->uOsiOrInternetAddress.psOsiAddress->
		pcPsel==NULL ||
	   psTargetInformation->uOsiOrInternetAddress.psOsiAddress->
		pcSsel==NULL ||
	   psTargetInformation->uOsiOrInternetAddress.psOsiAddress->
		pcTsel==NULL ||
	   psTargetInformation->uOsiOrInternetAddress.psOsiAddress->
		pcNsap==NULL)
	{
		sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, 
			"->pcOsiAddress is incomplete");
		DBVfill_error_structure(0, SI_Interface_Error, 0, 
			DBV_INVALIDPARAMS, DBVErrorBuffer, 0,
			Association_Closed, ppsError);

	/* Return straight away here */
		return SI_NOTOK;
	}
		
	/* Count the characters in the OSI address fields (Add three for the 
		slashes */
	lCount = strlen(psTargetInformation->uOsiOrInternetAddress.
		psOsiAddress->pcPsel) +
	strlen(psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcSsel)+
	strlen(psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcTsel)+
	strlen(psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcNsap)+
	4;

	if(DBVAllocCheck((*ppcTargetName=(char *)DBVAlloc(lCount,ppsError)))
		==SI_NOTOK)
		{ return SI_NOTOK; }
	
	/* Convert the address to string format */
	strcpy(*ppcTargetName, psTargetInformation->uOsiOrInternetAddress.
		psOsiAddress->pcPsel);
	strcat(*ppcTargetName, "/");
	strcat(*ppcTargetName,psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcSsel);
	strcat(*ppcTargetName, "/");
	strcat(*ppcTargetName,psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcTsel);
	strcat(*ppcTargetName, "/");
	strcat(*ppcTargetName,psTargetInformation->uOsiOrInternetAddress.psOsiAddress->pcNsap);

	return return_value;
}


/* Fills the DBV union ppuAssociateResult with data obtained from the 
	psAssocControlConnect ISODE structure. Only called if request
	accepted/rejected, not if still connecting. */
int DBVOsiCompleteDbvAssociateResult(struct AcSAPconnect *psAssocControlConnect,
	struct AcSAPabort *psAssocControlAbort, DbvAssociateResult 
	**ppuAssociateResult, DbvAssociateState *peAssociateState,
	DbvError **ppsError)
{
	int lIndex=0;
	int lAccepted=0;
	int return_value = SI_OK;
	long lPCsWhichAreRejectedButAreNotACNs = 0;
	DbvAssociateResult *puAssociateResult; 
	char *pcNegotiatedContext=NULL;

	/* First determine if the association was accepted or rejected */
	/* Call was accepted */
	if(psAssocControlConnect->acc_result == ACS_ACCEPT)
	{
		*peAssociateState = Associate_Accepted;
		if(DBVAllocCheck((*ppuAssociateResult = 
			(union DbvAssociateResult *)DBVAlloc
			(sizeof(DbvAssociateResult),ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }

		/* Just so that I don't have to do so much de-referencing */
		puAssociateResult = *ppuAssociateResult;

		if(DBVAllocCheck((puAssociateResult->psProtocolAndSyntaxes = 
			(struct DbvProtocolAndSyntaxes *)DBVAlloc(sizeof
			(struct DbvProtocolAndSyntaxes), ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }

		if(DBVAllocCheck((puAssociateResult->psProtocolAndSyntaxes->
			peApplicationProtocol = (DbvApplicationProtocol *)
			DBVAlloc(sizeof(DbvApplicationProtocol),ppsError)))==
			  SI_NOTOK)
			{ return SI_NOTOK; }

		/* See file sil_defs.h for OID definitions*/
		pcNegotiatedContext=sprintoid(psAssocControlConnect->
			acc_context);
		if((strcmp(BASIC_Z3950_OID, pcNegotiatedContext))==0)
		{
 			*puAssociateResult->psProtocolAndSyntaxes->
				peApplicationProtocol = Z3950_Protocol;
		}
		else if((strcmp(BASIC_SR_OID, pcNegotiatedContext))==0)
		{
 			*puAssociateResult->psProtocolAndSyntaxes->
				peApplicationProtocol = SR_Protocol;
		}
		else
 			*puAssociateResult->psProtocolAndSyntaxes->
				peApplicationProtocol = Unidentified_Protocol;

		if(DBVAllocCheck((puAssociateResult->psProtocolAndSyntaxes->
			peAbstractSyntax = (DbvAbstractSyntax *)DBVAlloc
			(sizeof(DbvAbstractSyntax),ppsError)))==SI_NOTOK)
			{ return SI_NOTOK; }


		/* Now go through the negotiated presentation context list
		and compare them with the requested ones. Fill out the 
		ppcCommonSyntaxes array accordingly  */
		
		/* No contexts were provided */
		if(psAssocControlConnect->acc_connect.pc_ctxlist.pc_nctx == 0)
		{
 			*puAssociateResult->psProtocolAndSyntaxes->
				peAbstractSyntax = No_Syntaxes_Provided;
		}
		else
		/* We need to go through the ppcDBVProposedPresContexts array
		and place the accepted ones into the ppcCommonSyntaxes array */
		{
		/* First, see how many contexts were accepted and DBVAlloc the 
		memory accordingly */
			for(lIndex=0;lIndex<psAssocControlConnect->acc_connect.
				pc_ctxlist.pc_nctx; lIndex++)
			{

				if((strcmp(BASIC_Z3950_OID,
                                  ppcDBVProposedPresContexts[lIndex]) != 0) &&
                                  (strcmp(BASIC_SR_OID,
                                  ppcDBVProposedPresContexts[lIndex])!=0))
				{
					/* The PC is not an ACN */

					if(psAssocControlConnect->acc_connect.
                                  	  pc_ctxlist.pc_ctx[lIndex].pc_result ==
                                 	  PC_ACCEPT)
						lAccepted++;

					if(psAssocControlConnect->acc_connect.
                                  	  pc_ctxlist.pc_ctx[lIndex].pc_result ==
                                 	  PC_REJECTED)
					    lPCsWhichAreRejectedButAreNotACNs++;
				}
			}
			/* There were no common (agreed) syntaxes */
			if(lAccepted == 0 && 
				lPCsWhichAreRejectedButAreNotACNs > 0)
			{
 				*puAssociateResult->psProtocolAndSyntaxes->
					peAbstractSyntax = No_Common_Syntax;
			}
			else if( lAccepted == 0 &&
				lPCsWhichAreRejectedButAreNotACNs == 0)
			{
				*puAssociateResult->psProtocolAndSyntaxes->
					peAbstractSyntax = No_Syntaxes_Provided;
			}
			else
			/* There were some common syntaxes! */
			{
				lAccepted++; /* for terminating NULL */
				return_value = DBVCompleteCommonSyntaxes(
					psAssocControlConnect,
					puAssociateResult, lAccepted, ppsError);
			}
		}
	}
	else
	/* Call was rejected for some reason */
	{
		*peAssociateState = Associate_Rejected;
		if(DBVAllocCheck((*ppuAssociateResult = (union 
			DbvAssociateResult *)DBVAlloc(sizeof(union 
			DbvAssociateResult), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }

		/* Just so that I don;t have to do so much de-referencing */
		puAssociateResult = *ppuAssociateResult;

		if(DBVAllocCheck((puAssociateResult->psAssociateReject = 
			(struct DbvAssociateReject *)
			DBVAlloc(sizeof(struct DbvAssociateReject),ppsError)))
			==SI_NOTOK)
		{ return SI_NOTOK; }

		if(DBVAllocCheck((puAssociateResult->psAssociateReject->
			peAssociateRejectSource = (DbvAssociateRejectSource *)
			DBVAlloc(sizeof(DbvAssociateRejectSource), ppsError)))
			==SI_NOTOK)
			{ return SI_NOTOK; }
		if(DBVAllocCheck((puAssociateResult->psAssociateReject->
			peAssociateRejectResult = (DbvAssociateRejectResult *)
			DBVAlloc(sizeof(DbvAssociateRejectResult), ppsError)))
			==SI_NOTOK)
			{ return SI_NOTOK; }
		if(DBVAllocCheck((puAssociateResult->psAssociateReject->
			peAssociateRejectReason = (DbvAssociateRejectReason *)
			DBVAlloc(sizeof(DbvAssociateRejectReason), ppsError)))
			==SI_NOTOK)
			{ return SI_NOTOK; }
	
		/* Rejected by target user */
/* NOTE: Write reasons to log */
		if(psAssocControlConnect->acc_result == ACS_PERMANENT ||
			psAssocControlConnect->acc_result == ACS_TRANSIENT)
		{
			if(psAssocControlConnect->acc_result == ACS_PERMANENT)
				*puAssociateResult->psAssociateReject->
                        		peAssociateRejectResult =	
					Rejected_Permanent;

			if(psAssocControlConnect->acc_result == ACS_TRANSIENT)
				*puAssociateResult->psAssociateReject->
                        		peAssociateRejectResult =	
					Rejected_Transient;

			*puAssociateResult->psAssociateReject->
				peAssociateRejectSource =
				ACSE_User;
		}
		else
		/* Rejected by service provider */
		{
			*puAssociateResult->psAssociateReject->
                        	peAssociateRejectResult =	
				Rejected_Permanent;

			*puAssociateResult->psAssociateReject->
				peAssociateRejectSource =
				ACSE_Provider;
		}

		/* Fill out reason for rejection */
		switch(psAssocControlConnect->acc_diagnostic)
		{
		case ACS_CONTEXT:
			*puAssociateResult->psAssociateReject->
				peAssociateRejectReason = 
				Application_Context_Not_Supported;
			break;
		case ACS_VERSION:
			*puAssociateResult->psAssociateReject->
				peAssociateRejectReason = 
				No_Common_ACSE_Version;
			break;
		default:
			*puAssociateResult->psAssociateReject->
				peAssociateRejectReason = 
				No_Reason_Given;
			break;
		}
	}

	return return_value;
}


/* Retrieve Application Context and Presentation Context definition list from
	the ACSE configuration file. Also places the desired presentation
	contexts into the ppcDBVProposedPresContexts array for comparison
	with the negotiated list returned in the AAsociate.Confirmation
	service primitive */
int DBVGetACSEDetails(char **ppcContext, 
	OID *ppsContextOID, struct PSAPctxlist **ppsPresentationContextList, 
	DbvError **ppsError)
{
	int return_value = SI_OK;
	int lIndex=0;
	char *pcErrorString;
	struct PSAPctxlist *psPresentationContextList;

	if(DBVAllocCheck((*ppsPresentationContextList = (struct PSAPctxlist *)
		DBVAlloc(sizeof(struct PSAPctxlist), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psPresentationContextList = *ppsPresentationContextList;

	return_value=DBVGetACN(ppcContext, Association_Closed, O, ppsError);

	if(return_value == SI_OK)
	{
		return_value=DBVGetPCDL(psPresentationContextList, 
			Association_Closed, ppsError);

		if((*ppsContextOID=str2oid(*ppcContext))==NULLOID)
		{
			pcErrorString=(char *)DbvAlloc(strlen(
				"In function DBVGetACSEDetails(): ISODE function str2oid()  : invalid object descriptor")+
				strlen(*ppcContext)+1);
			sprintf(pcErrorString, "In function DBVGetACSEDetails(): ISODE function str2oid() %s: invalid object descriptor", *ppcContext);
			sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S,
				pcErrorString);
			DBVfill_error_structure(0, SI_Interface_Error,
				0, DBV_OSI_ISODE_ERROR, DBVErrorBuffer,
				0, Association_Closed,ppsError);
				DbvFree(pcErrorString);
			return_value=SI_NOTOK;

		}
		else if((*ppsContextOID=oid_cpy(*ppsContextOID))==NULLOID)
		{
			sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, "In function DBVGetACSEDetails(): ISODE function oid_cpy() failed");
			DBVfill_error_structure(0, SI_Interface_Error,
				0, DBV_OSI_ISODE_ERROR, DBVErrorBuffer,
				0, Association_Closed,ppsError);
			return_value=SI_NOTOK;
		}
	}

	return return_value;
}

#endif /* NO_OSI */

/* Get the address of the target from the ACSE configuration file and
	place it in ppsTargetInformation. Uses routines in the file 
	address_by_name.[ch] */
int DBVGetAddressByName(DbvTarget *psTargetIdentification, 
			DbvTargetInformation **ppsTargetInformation, 
			DbvError **ppsError)
{

	return DBVConvertNameToAddress(psTargetIdentification->
		uNameOrAddress.pcTargetName, ppsTargetInformation, 
		ppsError);
}



int DBVFillTcpAssociateResult(DbvAssociateResult **ppuAssociateResult, 
	DbvError **ppsError)
{
	DbvAssociateResult *puAssociateResult;
	int return_value = SI_OK;

	if(DBVAllocCheck((*ppuAssociateResult=(
		DbvAssociateResult *)DBVAlloc
		(sizeof(DbvAssociateResult),
		ppsError)))==SI_NOTOK)
	{ return SI_NOTOK; }

		puAssociateResult = *ppuAssociateResult;

	if(DBVAllocCheck((puAssociateResult->
		psProtocolAndSyntaxes = 
		(DbvProtocolAndSyntaxes *)
		DBVAlloc(sizeof( 
		DbvProtocolAndSyntaxes), 
		ppsError)))==SI_NOTOK)
	{ return SI_NOTOK; }

	if(DBVAllocCheck((puAssociateResult->
		psProtocolAndSyntaxes->
		peApplicationProtocol = 
		(DbvApplicationProtocol *)
		DBVAlloc(sizeof(DbvApplicationProtocol),
		ppsError)))==SI_NOTOK)
	{ return SI_NOTOK; }

		*puAssociateResult->psProtocolAndSyntaxes->
		peApplicationProtocol = Unknown_Protocol;

	if(DBVAllocCheck((puAssociateResult->
		psProtocolAndSyntaxes->
		peAbstractSyntax = (DbvAbstractSyntax *)
		DBVAlloc(sizeof(DbvAbstractSyntax), 
		ppsError)))==SI_NOTOK)
	{ return SI_NOTOK; }

		*puAssociateResult->psProtocolAndSyntaxes->
		peAbstractSyntax = No_Syntaxes_Provided;

	/* Already set to NULL by DbvAlloc 
		puAssociateResult->psProtocolAndSyntaxes->
		ppcCommonSyntaxes = (char **)NULL; */

	return return_value;
}

#ifndef NO_OSI

int DBVCompleteCommonSyntaxes(struct AcSAPconnect *psAssocControlConnect,
	DbvAssociateResult *puAssociateResult, int lAccepted, 
	DbvError **ppsError)
{
	int lIndex = 0;

	if(DBVAllocCheck((puAssociateResult->
		psProtocolAndSyntaxes->
		ppcCommonSyntaxes=(char **)DBVAlloc
		(lAccepted*sizeof(char *),ppsError)))
		== SI_NOTOK)
		{ return SI_NOTOK; }

	lAccepted=0; /* Used as an index to
		ppcCommonSyntaxes */
	for(lIndex=0;lIndex<psAssocControlConnect->
		acc_connect.pc_ctxlist.pc_nctx;
		lIndex++)
	{
		if(psAssocControlConnect->acc_connect.pc_ctxlist.pc_ctx[lIndex].
			pc_result == PC_ACCEPT && (strcmp(BASIC_Z3950_OID,
			ppcDBVProposedPresContexts[lIndex]) != 0) && (strcmp(
			BASIC_SR_OID, ppcDBVProposedPresContexts[lIndex])!=0))
		{
			if(DBVAllocCheck((puAssociateResult->
				psProtocolAndSyntaxes->
				ppcCommonSyntaxes[lAccepted]=(char *)
				DBVAlloc(1+strlen(
				ppcDBVProposedPresContexts[lIndex]),
				ppsError)))==SI_NOTOK)
				{ return SI_NOTOK; }
 
			strcpy(puAssociateResult->
				psProtocolAndSyntaxes->
				ppcCommonSyntaxes[lAccepted],
				ppcDBVProposedPresContexts[lIndex]);
 
			lAccepted++;
		}
	}
 
	/* End list with a NULL pointer */
	puAssociateResult->psProtocolAndSyntaxes->
		ppcCommonSyntaxes[lAccepted]=NULL;
 
	*puAssociateResult->psProtocolAndSyntaxes->
		peAbstractSyntax = Common_Syntaxes;
	
	return SI_OK;

}
#endif

void DBVFreeTargetInformation(DbvTargetInformation *psTargetInformation)
{
	DbvOsiAddress *psOsiAddress = NULL;

	if(psTargetInformation != NULL)
	{
		if(psTargetInformation->peTargetSupportedProtocol != NULL)
			DbvFree(psTargetInformation->peTargetSupportedProtocol);

		if(psTargetInformation->eOsiOrInternetAddress == 
			Internet_Address)
		{
                       DbvFree(psTargetInformation->uOsiOrInternetAddress.
				psInternetAddress->pcHostAddress);

                        DbvFree(psTargetInformation->uOsiOrInternetAddress.
				psInternetAddress);
		}
		else
		{
			psOsiAddress = psTargetInformation->
				uOsiOrInternetAddress.psOsiAddress;
			if(psOsiAddress->pcPsel != NULL)
                        	DbvFree(psOsiAddress->pcPsel);
			if(psOsiAddress->pcSsel != NULL)
                        	DbvFree(psOsiAddress->pcSsel);
			if(psOsiAddress->pcTsel != NULL)
                        	DbvFree(psOsiAddress->pcTsel);
			if(psOsiAddress->pcNsap != NULL)
                        	DbvFree(psOsiAddress->pcNsap);
                        DbvFree(psOsiAddress);
                }
	DbvFree(psTargetInformation);
	}
}

