
#include "osendbas.h"



int handle_generic_response(DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError ** error)
{
	DbvOriginData *psOriginData = NULL;
	DbvOriginEvent eOriginEvent;

	printf("Getting back image data\n");

	wait_for_and_display_generic_response(&psOriginData, assoc_id, assoc_state, assoc_res, rel_state, error);

	/* Extract image data from the recived data and output it to a file */
	extract_imagedata(psOriginData);

	eOriginEvent = psOriginData->eOriginEvent;


	/* While not got a PresentResponse (level-2 response terminator) */
	while (eOriginEvent != OR_PresentConf) {
		wait_for_and_display_generic_response(&psOriginData, assoc_id, assoc_state, assoc_res, rel_state, error);
		extract_imagedata(psOriginData);
		eOriginEvent = psOriginData->eOriginEvent;
	}
	
}




int wait_for_and_display_generic_response(DbvOriginData **ppsOriginData, DbvAssocId *assoc_id, DbvAssociateState *assoc_state, DbvAssociateResult **assoc_res, DbvReleaseState *rel_state, DbvError **error)
{
 
	DbvBool finished = DBV_FALSE;

	DbvAssocId **pplAssocId = NULL;
	DbvEventLocation *psEventLocation = NULL;
	DbvFds **ppulFds = NULL;
 
        int apdu_type, numfields;
        char command[1000];
 
	pplAssocId = (DbvAssocId **)DbvAlloc(2*sizeof(DbvAssocId *));
	pplAssocId[0] = assoc_id;
	pplAssocId[1] = (DbvAssocId *)NULL;
	
	/* Create a 100 element ppulFds descriptor list */
	ppulFds = (DbvFds **) DbvAlloc (100 * sizeof(DbvFds *));
	
	/* Put nothing in it though - make the first item NULL */
	ppulFds[0] = (DbvFds *) NULL;

        /* Wait for the response from the target */
 
	printf("Waiting to receive data sent back by target.....\n");

	/* Call with a one second time out */
	do {
		if (SIGetEventLocation(pplAssocId, 1, ppulFds, &psEventLocation, error) == SI_OK) {
			if (psEventLocation != NULL) {

       				if (DbvReceiveDataOrigin(*pplAssocId[0], CALL_BLOCKING, ppsOriginData, error) == SI_OK) 
					finished = DBV_TRUE;
       				else {
					printf("DbvReceiveDataOrigin returned SI_NOTOK\n");
               				PrivateErrorHandling(error);
               				FreeSIErrorCode(error);
               				exit(1);
	        		}
			} else
				printf("Timed out with nothing to read\n");
		} else {
               		PrivateErrorHandling(error);
               		FreeSIErrorCode(error);
               		exit(1);
		}

	} while (finished == DBV_FALSE);
 
	return SI_OK;
}



int extract_imagedata(DbvOriginData *psOriginData)
{
	switch (psOriginData->eOriginEvent) {

	/* Extraction assumes that data is always present */


	/* Assume one name plus record only */

	/* Final fragement */
	case OR_PresentConf:
		printf("Outputting PresentResponse...\n");
		output_data_to_file(psOriginData->uOriginData.psPresentRsp->psRecords->uRecord.ppsResponseRecord[0]);
		break;

	/* Start of Intermediate fragment */
	case OR_SegmentationInd:
		printf("Outputting SegmentRequest...\n");
		output_data_to_file(psOriginData->uOriginData.psSegmentReq->ppsSegmentRecords[0]);
		break;
	}
}


int output_data_to_file(DbvNamePlusRecord *psResponseRecord)
{

	/* Assume either starting fragment, intermediate fragment or 
	   final fragment */
	switch (psResponseRecord->eDbRecordType) {

	/* SegmentReq returns */
	case DRT_StartingFragment:
		printf("Outputting Starting Fragment...\n");
		output_genericrecord(psResponseRecord->uDbRecord.psStartingFragment->uFragmentSyntax.psExternallyTagged->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, DRT_StartingFragment);

		break;
	case DRT_IntermediateFragment:
		printf("Outputting Intermediate Fragment...\n");
		output_genericrecord(psResponseRecord->uDbRecord.psIntermediateFragment->uFragmentSyntax.psExternallyTagged->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, DRT_IntermediateFragment);

		break;


	/* PresentReq returns */
	case DRT_FinalFragment:
		printf("Outputting Final Fragment...\n");
		output_genericrecord(psResponseRecord->uDbRecord.psFinalFragment->uFragmentSyntax.psExternallyTagged->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, DRT_FinalFragment);
		break;
	}
		
}


int output_genericrecord(DbvGenericRecord *psGenericRecord, DbvDbRecordType current_fragment)
{
	char *filename_received = NULL;
	unsigned long filename_len_received = 0;
	char *file_to_create= NULL;

	char *received_data = NULL;
	int received_data_length = 0;
	char *data_to_output = NULL;

	FILE *file_stream = NULL;
	
	/* Assume Externally defined fragments using Generic Record!! */
	
	/* Assume one DbvTaggedElement */


	/* Get the filename */
	filename_received = psGenericRecord->ppsTaggedElement[0]->sTagValue.uStringOrNumeric.psString->pcValue;
	filename_len_received = psGenericRecord->ppsTaggedElement[0]->sTagValue.uStringOrNumeric.psString->ulLength;

	file_to_create = (char *) DbvAlloc (strlen(PRETAG) + filename_len_received + 1);

	memcpy(file_to_create, PRETAG, strlen(PRETAG));
	strncat(file_to_create, filename_received, filename_len_received);


	/* and the binary data */
	received_data = psGenericRecord->ppsTaggedElement[0]->sContent.uElementDataG.psOctets->pcValue;
	received_data_length = psGenericRecord->ppsTaggedElement[0]->sContent.uElementDataG.psOctets->ulLength;


	/* If a starting fragment */
	if (current_fragment == DRT_StartingFragment)

		/* If file already exists then delete it ! (ok for a demo,
		   better to rename as a backup though) */
		if (fopen(file_to_create, "r") != NULL) {
			printf("File %s already exists so deleting it !!\n", file_to_create);
			remove(file_to_create);
		} else
			printf("File %s doesn't already exist\n", file_to_create);
	

	/* And now for all data fragments .... */

	/* open the file with append */

	printf("Opening file %s for append\n", file_to_create);
	if ((file_stream = fopen(file_to_create, "a")) == NULL)
		printf("Failed to open file %s\n", file_to_create);


	/* Now write out data, appending to the file  */
	printf("Appending %lu bytes of binary data\n", received_data_length); 
	if (fwrite(received_data, sizeof(char), received_data_length, file_stream) != received_data_length)
		printf("Failed to write all the data\n");

	/* And close the file */
	if (fclose(file_stream) == EOF)
		printf("Error closing file\n");

	

	
}




