/*  Module Name: @(#)targetassoc.c	7.3
    Version: 12/2/96
    Author: Chris Attewell
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        Target association handling functions Using TCP/OSI Services
 
 
*/

#include "targetassoc.h"

int DBVsocket; /* Used to store the incoming socket file desriptor.*/
DbvOsiOrInternetAddress DBVOsiOrInternetAddress;

#ifndef NO_OSI
OID psDBVContextSupported; /* Stores the context to send back to the origin */
#endif

DbvBool fDBVACSEError;
DbvBool fDBVAcceptAllowed; /* If the PCDL was specified and there is an error
	the association must be aborted */
 
int DbvTargetInitialize(int argc, char **argv, DbvError **ppsError)
{
	int return_value = SI_OK;

	/* ISODE structures */
#ifndef NO_OSI
	struct AcSAPindication *psAssocControlIndication=NULL;
	struct AcSAPabort *psAssocControlAbort = NULL;
#endif

        DBVCheckError(ppsError);

#ifndef NO_OSI
	if(DBVAllocCheck((DBVpsAssocControlStart=(struct AcSAPstart *)
		DBVAlloc(sizeof(struct AcSAPstart), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort = &psAssocControlIndication->aci_abort;
#endif

	/* By default Accepting the association is allowed */
	fDBVAcceptAllowed = DBV_TRUE;
	/* There's no error yet */
	fDBVACSEError = DBV_FALSE;

	if((strcmp(argv[1],"TCP")) == 0)
	{
        	/* Get values passed down by the server master */
		/* Better do some checking, here really or trouble 
			could result */
        	DBVsocket = atoi(argv[2]);
		/* Although only one association is ever used in a target, 
		a linked list is used to maintain compatibility with the 
		utility functions */
        	if(DBVAssociateList == (struct SList *)NULL)
                	DBVAssociateList = SL_Create( (SL_Free)NULL );
		DBVOsiOrInternetAddress = Internet_Address;
	}
	else
	{
#ifndef NO_OSI
        	if(DBVAssociateList == (struct SList *)NULL)
                	DBVAssociateList = SL_Create( (SL_Free)NULL );
		if(AcInit(argc, argv, DBVpsAssocControlStart, psAssocControlIndication) == NOTOK)
		{
			/* Go straight to log file, because there is no 
				association */
                	DBVLogISODEACSEError("AcInit()", psAssocControlAbort);
			sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, 
				"In function DbvTargetInitialize(): ISODE function AcInit() failed");
			DBVfill_error_structure(0, SI_Interface_Error,
				0, DBV_OSI_ISODE_ERROR, DBVErrorBuffer,
				0, Association_Closed,ppsError);
			return_value = SI_NOTOK;
		}
		DBVOsiOrInternetAddress = Osi_Address;
#else
		printf("OSI not supported\n");
#endif
	}

#ifndef NO_OSI
	ACAFREE(psAssocControlAbort);
	free(psAssocControlIndication);
#endif 

	return return_value;
}

int DbvReceiveAssociateRequest( DbvAssocId *plAssocId, DbvSecs lSeconds,
                                char **ppcOriginAddress,
				DbvProtocolAndSyntaxes **ppsProtocolAndSyntaxes,
                                DbvError **ppsError )
{
        DBVAssociateDetails *psAssociateDetails;
	static DbvAssocId IdGenerator = 0;
	struct sockaddr_in Name;
	int Namelen = sizeof Name;
	int loop = 0;
	char address[64];
	int return_value = SI_OK;
	int ACN_return_value = SI_OK;
	DbvError *psACNError = NULL;
	DbvProtocolAndSyntaxes *psLocalProtocolAndSyntaxes=NULL;
/*	DbvProtocolAndSyntaxes **ppsProtocolAndSyntaxes=NULL;*/
#ifndef NO_OSI
        struct PSAPctxlist *psPresentationContextList=NULL;
#endif

        DBVCheckError(ppsError);

	if(IdGenerator != 0)
	{
		sprintf(DBVErrorBuffer, DBV_INVALIDCALL_S, "DbvReceiveAssociateRequest can only be called once");
		DBVfill_error_structure(0,SI_Communication_Error, 0, DBV_INVALIDCALL, DBVErrorBuffer,0,Association_Open, ppsError);
                return_value = SI_NOTOK;
        }

        /* Allocate memory for new Association */
        psAssociateDetails = (DBVAssociateDetails *)
        	DbvAlloc(sizeof(DBVAssociateDetails));

	psLocalProtocolAndSyntaxes = (DbvProtocolAndSyntaxes *)DbvAlloc
		(sizeof(DbvProtocolAndSyntaxes));

	*plAssocId = ++IdGenerator;
	if(DBVOsiOrInternetAddress == Internet_Address)
	{
		psLocalProtocolAndSyntaxes->peApplicationProtocol=
			(DbvApplicationProtocol *)DbvAlloc
			(sizeof(DbvApplicationProtocol));
		*psLocalProtocolAndSyntaxes->peApplicationProtocol = 
			Unknown_Protocol;

		psLocalProtocolAndSyntaxes->peAbstractSyntax=
			(DbvAbstractSyntax *)DbvAlloc
			(sizeof(DbvAbstractSyntax));
		*psLocalProtocolAndSyntaxes->peAbstractSyntax = No_Syntaxes_Provided;

/*		*peApplicationProtocol = Unknown_Protocol;*/

        	/* Fill Details of new associate and add to list */
        	psAssociateDetails->eOsiOrInternetAddress = Internet_Address;
        	psAssociateDetails->ulTcpSocket = DBVsocket;

		/* Get the address of the connected peer */
		if(getpeername(DBVsocket,(struct sockaddr *)&Name, &Namelen)<0)
		{
			if (errno > 0 && errno < sys_nerr)
				sprintf(DBVErrorBuffer, DBV_SYSERROR_S, "getpeername()", sys_errlist[errno]);
			else
				sprintf(DBVErrorBuffer, DBV_SYSERROR_S, "getpeername()", "");

                	DBVfill_error_structure(0,SI_OperatingSystem_Error, 0, 
				DBV_SYSERROR, DBVErrorBuffer,0,
				Association_Open, ppsError);
                	return_value = SI_NOTOK;
        	}
		else
		{
			strcpy(address,
				inet_ntoa(Name.sin_addr));
			*ppcOriginAddress = (char *)DbvAlloc(sizeof(address));
			strcpy(*ppcOriginAddress, address);
		}

        	psAssociateDetails->eAssociateState = Associate_Accepted;
		psAssociateDetails->lAssocId = *plAssocId;
	}
	else
	{
#ifndef NO_OSI
        	/* Fill Details of new associate and add to list */
        	psAssociateDetails->eOsiOrInternetAddress = Osi_Address;
        	psAssociateDetails->lOSIAssocId=DBVpsAssocControlStart->acs_sd;
		psAssociateDetails->lAssocId = *plAssocId;
        	psAssociateDetails->eAssociateState=Associate_Accepted;

		/*Find out what the requested application protocol is*/
		ACN_return_value=DBVCheckApplicationContext(&psAssociateDetails
			->eApplicationProtocol, DBVpsAssocControlStart->
			acs_context, &psACNError);
		if(return_value == SI_OK || fDBVACSEError == DBV_TRUE)
		{
			psLocalProtocolAndSyntaxes->peApplicationProtocol=
				(DbvApplicationProtocol *)DbvAlloc
				(sizeof(DbvApplicationProtocol));
			*psLocalProtocolAndSyntaxes->peApplicationProtocol = 
				psAssociateDetails->eApplicationProtocol;
			/* Place the connecting origin's address in the 
				OriginAddress structure */
			*ppcOriginAddress = paddr2str(&(DBVpsAssocControlStart->
				acs_start.ps_calling),NULLNA);
		}
		/* Reset the error variable for the presentation context code */
		fDBVACSEError = DBV_FALSE;
		/* Accept or reject the applications contexts and complete the
			ppcCommonSyntaxes part of the protocol and syntaxes 
			structure */
		psPresentationContextList=&(DBVpsAssocControlStart->
			acs_start.ps_ctxlist);
		return_value = DBVAcceptOrRejectPresentationContexts(
			psLocalProtocolAndSyntaxes,
			psAssociateDetails->eApplicationProtocol,
			psPresentationContextList, ppsError);
#else
		printf("OSI not supported\n");
#endif
	}

	if(return_value == SI_OK || fDBVACSEError == DBV_TRUE)
	{
		/* Everything's Ok, or there was an error in the ACSE file */
		psAssociateDetails->eOriginOrTarget = T;
        	SL_Add(DBVAssociateList, psAssociateDetails);
		DBVSetState(*plAssocId, DBV_TS_ASSOC_INPROGRESS);
		*ppsProtocolAndSyntaxes = psLocalProtocolAndSyntaxes;
	}
	else
	{
		/* We're not ok for some other reason, so free everything */
		DbvFree(psLocalProtocolAndSyntaxes->peApplicationProtocol);
		DbvFree(psLocalProtocolAndSyntaxes->peAbstractSyntax);
		while(psLocalProtocolAndSyntaxes->ppcCommonSyntaxes != NULL && 
			psLocalProtocolAndSyntaxes->ppcCommonSyntaxes[loop] != 
			NULL)
		{
			DbvFree(psLocalProtocolAndSyntaxes->
				ppcCommonSyntaxes[loop]);
			loop ++;
		}
	}
	if(ACN_return_value == SI_NOTOK)
	{
        	DBVCheckError(ppsError);
		*ppsError = psACNError;
		return_value = SI_NOTOK;
	}

	return return_value; 
}

int DbvAssociateResponse(  DbvAssocId lAssocId, DbvAssociateRsp eAssociateRsp,
                           DbvError **ppsError )
{
        DBVAssociateDetails *psAssociateDetails;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_OK;
	int reject_error = SI_OK;

#ifndef NO_OSI
	/* ISODE */
	long lReason = ACS_USER_NULL; /* Reason for rejection */
	long lAcceptOrReject = ACS_ACCEPT;
	struct AcSAPindication *psAssocControlIndication=NULL;
	struct AcSAPabort *psAssocControlAbort;
	struct PSAPctxlist *psPresentationContextList=NULL;
	struct PSAPstart *psPresentationStart=NULL;
#endif

        DBVEnterFuncTrace("DbvAssociateResponse",lAssocId);
	
	DBVTrace(lAssocId, Trace_High, "\nInput Function Arguments\n");
	Trace_writeDbvAssocId(lAssocId);
	Trace_writeDbvAssociateRsp(eAssociateRsp, lAssocId);

        DBVCheckError(ppsError);

#ifndef NO_OSI
	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort=&psAssocControlIndication->aci_abort;
#endif

	/* Make sure that the Response is valid */
	switch(eAssociateRsp)
	{
	case 	Accepted:
	case	Rejected_Perm:
	case	Rejected_Tran:
		break;
	default:
		/* Fill out error structure */
		/* invalid eAssociateRsp type */
        	sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, 
			"Variable eAssociateRsp was of invalid type");
                DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,
			DBV_INVALIDPARAMS, DBVErrorBuffer, 0, 
			Association_Open, ppsError);
		return SI_NOTOK;
		break;
	}

	/* First of all check the association may be accepted */
	if(eAssociateRsp == Accepted && fDBVAcceptAllowed == DBV_FALSE)
	{
		/* The Association must be rejected */
		eAssociateRsp = Rejected_Tran;
		sprintf(DBVErrorBuffer, DBV_ACSE_FORMATERROR_S, "An error reported by DbvReceiveAssociateRequest() means that you may not Accept the Association.");
                DBVfill_error_structure(0, SI_Interface_Error, 0,
                        DBV_ACSE_FORMATERROR, DBVErrorBuffer, 0,
                        Association_Closed, ppsError);
		reject_error = SI_NOTOK;

		
	}
	if(return_value == SI_OK &&
		SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK)
	{
	/* If the call should be accepted then do nothing */
	psAssociateDetails=SL_Current(DBVAssociateList);
	 if(psAssociateDetails->eOsiOrInternetAddress == Internet_Address)
	 {
	  if(eAssociateRsp != Accepted)
	  {
		/* Close the socket */
		if(close(psAssociateDetails->ulTcpSocket) == -1)
		{
			/* Fill out error structure */
			/* Error on close */
			if (errno > 0 && errno < sys_nerr)
				sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, sys_errlist[errno]);
			else
				sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, "");

			DBVfill_error_structure(lAssocId, 
				SI_Communication_Error, 0, DBV_TCP_CLOSEFAILED,
				DBVErrorBuffer, 0, Association_Unknown_State, 
				ppsError);
			return_value = SI_NOTOK;
		}
		else
		{
			/* STATE MACHINE */
			return_value = DBVChangeState(lAssocId, 
				DBV_TE_IASSOCRESP_FALSE, ppsError);

	                psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                 	free(psAssociateDetails);
		}
	  }
	  else
	  {
		if(eAssociateRsp == Accepted)
		{	
			/* STATE MACHINE */
			return_value = DBVChangeState(lAssocId, 
				DBV_TE_IASSOCRESP_TRUE, ppsError);

			;
		}	
		
	  }
	 }
	 else
	 {
#ifndef NO_OSI
	  /* OSI */
	  psPresentationStart=&DBVpsAssocControlStart->acs_start;
	  if(eAssociateRsp != Accepted)
	  {
		lReason = ACS_USER_NOREASON;
		switch(eAssociateRsp)
		{
		case Rejected_Tran:
			lAcceptOrReject = ACS_TRANSIENT;
			break;
		case Rejected_Perm:
			lAcceptOrReject = ACS_PERMANENT;
			break;
		default:
			break;
		}
		if(psAssociateDetails->eApplicationProtocol == 
			Unidentified_Protocol)
		{ lReason = ACS_CONTEXT; }
		/* STATE MACHINE */
		return_value = DBVChangeState(lAssocId, DBV_TE_IASSOCRESP_FALSE,
			ppsError);
	  }
	  else
	  {
		/* Accepted by API user */
		/* STATE MACHINE */
		return_value = DBVChangeState(lAssocId, DBV_TE_IASSOCRESP_TRUE, 
			ppsError);
	  }
	  if(return_value == SI_OK || fDBVAcceptAllowed == DBV_FALSE)
	  {
		if(AcAssocResponse(psAssociateDetails->lOSIAssocId,
			lAcceptOrReject, lReason, psDBVContextSupported,
			NULLAEI,&psPresentationStart->ps_called,
			&psPresentationStart->ps_ctxlist,
			psPresentationStart->ps_defctxresult,
			psPresentationStart->ps_prequirements,
			psPresentationStart->ps_srequirements, SERIAL_NONE,
			psPresentationStart->ps_settings,
			&psPresentationStart->ps_connect, NULLPEP, 0,
			psAssocControlIndication)==NOTOK)
		{
			DBVTrace(lAssocId, Trace_Low,
				"A-ASSOCIATE.RESPONSE error");
			DBVACSEOSICheckISODEError(lAssocId,
				psAssociateDetails->lOSIAssocId,
				psAssocControlIndication, ppsError);
			

			return_value = SI_NOTOK;
        	}
	  }
#else
	printf("OSI not supported\n");
#endif
	 }
	}
	else
		/* Error structure completed by SIWhichStack() */
		return_value = SI_NOTOK;

        if(return_value == SI_OK && eAssociateRsp != Accepted)
        {
                /* Free up space of removed association set by SIWhichStack*/
                psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                free(psAssociateDetails);
        }

	if( reject_error == SI_NOTOK)
		return_value = SI_NOTOK;

#ifndef NO_OSI
	/* Free the ISODE structures */
        ACSFREE(DBVpsAssocControlStart);
	DBVFree(DBVpsAssocControlStart);
        ACAFREE(psAssocControlAbort);
        DBVFree(psAssocControlIndication);
#endif

        DBVLeaveFuncTrace("DbvAssociateResponse",lAssocId, return_value);
	return return_value;

}

int DbvReleaseResponse(    DbvAssocId lAssocId, DbvReleaseRsp *psReleaseRsp,
                           DbvError **ppsError )
{
        DBVAssociateDetails *psAssociateDetails;
        int return_value = SI_OK;
	DbvProtocolStack eProtocolStack;

#ifndef NO_OSI
	/* ISODE structures */
	int dStatus = ACR_NORMAL;
	int dReason = ACS_ACCEPT;
	struct AcSAPindication *psAssocControlIndication=NULL;
	struct AcSAPabort *psAssocControlAbort=NULL;
#endif
 
        DBVEnterFuncTrace("DbvReleaseResponse",lAssocId);

        DBVTrace(lAssocId, Trace_High, "\nInput Function Arguments\n");
        Trace_writeDbvAssocId(lAssocId);
	Trace_writeDbvReleaseRsp(psReleaseRsp, lAssocId);

        DBVCheckError(ppsError);

#ifndef NO_OSI
	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psAssocControlAbort = &psAssocControlIndication->aci_abort;
#endif

	/* Rejection error */
	/* Under SR/Z39.50 A Release Request may not be rejected */
	if(psReleaseRsp != NULL && psReleaseRsp->eReleaseRspResult == Negative)
	{
		sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, 
			"Under SR/Z39.50 A Release Request may not be rejected");
		DBVfill_error_structure(lAssocId, 
			SI_Interface_Error, 0, 
			DBV_INVALIDPARAMS, DBVErrorBuffer, 0, 
			Association_Open, ppsError);
		return_value = SI_NOTOK;
	}
	else
		return_value = 
			DBVChangeState(lAssocId, DBV_TE_IRELRESP, ppsError);

        if(return_value == SI_OK && (SIWhichStack(lAssocId, &eProtocolStack, 
		ppsError) == SI_OK))
	{
        	psAssociateDetails = SL_Current(DBVAssociateList);
		if(eProtocolStack == TCP_Stack)
		{
        		if((close(psAssociateDetails->ulTcpSocket)) < 0)
			{
				/* Error on close */
				if (errno > 0 && errno < sys_nerr)
					sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, sys_errlist[errno]);
				else
					sprintf(DBVErrorBuffer, DBV_TCP_CLOSEFAILED_S, lAssocId, "");

				DBVfill_error_structure(lAssocId, 
					SI_Communication_Error, 0, 
					DBV_TCP_CLOSEFAILED, DBVErrorBuffer, 0, 
					Association_Unknown_State, ppsError);
				return_value = SI_NOTOK;
			}
		}
		else
                {
                        if(eProtocolStack == OSI_Stack)
                        {
#ifndef NO_OSI
			    if(psReleaseRsp != NULL)
			    {
				/* Map API enums onto ISODE equivalents */
				switch(psReleaseRsp->eReleaseRspResult)
				{
				case Affirmative:
					dReason=ACS_ACCEPT;
					break;
				case Negative:
					dReason=ACS_REJECT;
					break;
				default:
					/* Error ! */
					sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, "Value for psReleaseRsp->eReleaseRspResult is invalid");
					DBVfill_error_structure(0, 
						SI_Interface_Error, 0, 
						DBV_INVALIDPARAMS, 
						DBVErrorBuffer, 0, 
						Association_Open, ppsError);
					return_value = SI_NOTOK;
					break;
				}
				if(psReleaseRsp->peReleaseRspReason != 
					NULL)
				{
					switch(*psReleaseRsp->
							peReleaseRspReason)
					{
					case Normal:
						dStatus=ACR_NORMAL;
						break;
					case Not_Finished:
						dStatus=ACR_NOTFINISHED;
						break;
					default:
						/* Error ! */
						sprintf(DBVErrorBuffer, DBV_INVALIDPARAMS_S, "Value for psReleaseRsp->peReleaseRspReason is invalid");
						DBVfill_error_structure(0, SI_Interface_Error, 0, DBV_INVALIDPARAMS, DBVErrorBuffer, 0, Association_Open, ppsError);
						return_value = SI_NOTOK;
						break;
					}
				}
			    }
				if(return_value==SI_OK &&
					(AcRelResponse(psAssociateDetails->
						lOSIAssocId, dReason, 
						dStatus, NULLPEP, 0, 
						psAssocControlIndication)
						== NOTOK))
				{
			
					DBVTrace(lAssocId, Trace_Low,
						"A-RELEASE.RESPONSE error");
					DBVACSEOSICheckISODEError(lAssocId,
						psAssociateDetails->lOSIAssocId,
						psAssocControlIndication, 
						ppsError);
					return_value = SI_NOTOK;
				}
#else
				printf("OSI not supported\n");
#endif

                        }
                        else
                        {
                                /* Fill out error structure */
                                printf("ReleaseResponse: Invalid stack type!\n");
                        }
                }
        }
        else
        /* Structure already filled by SIWhichStack*/
                return_value = SI_NOTOK;

        if(return_value == SI_OK)
        {
                /* Free up space of removed association set by SIWhichStack*/
                psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                free(psAssociateDetails);
        }

#ifndef NO_OSI
	ACAFREE(psAssocControlAbort);
	free(psAssocControlIndication);
#endif

        DBVLeaveFuncTrace("DbvReleaseResponse",lAssocId, return_value);
	return return_value;
}



#ifndef NO_OSI
/* Convert the requested application context into the peApplicationProtocol
	enumerated type */
int DBVCheckApplicationContext(DbvApplicationProtocol *peApplicationProtocol, 
	OID sContext, DbvError **ppsError)
{
	int return_value=SI_OK;
	char *pcOID=NULL;
	char *pcSupportedContext=NULL;
	DbvBool bSR, bZ3950;

	fDBVACSEError = DBV_FALSE;
	pcOID=sprintoid(DBVpsAssocControlStart->acs_context);

	/* Logic: If target app. supports both SR & Z3950 services then
		peApplicationProtocol is set to the equivalent value of the 
		requested app. context 
		
		If the target app. supports only one protocol, then 
		peApplicationProtocol is set to the one which the app. supports
	*/

	/* Get the supported context from the ACSE config file. (and set 
		variables ready for a call to DBVSupportedContexts() )*/
	return_value=DBVGetACNAtTarget(&pcSupportedContext, Association_Open, 
		ppsError);
	if(return_value == SI_NOTOK)
	{
		/* set fDBVACSEError so that we know there's been an error */
		fDBVACSEError = DBV_TRUE;
	}	
	sprintf(DBVErrorBuffer,"Requested Application Context: %s", pcOID);
	DBVWriteTargetLog(DBVErrorBuffer);
	
	psDBVContextSupported=DBVpsAssocControlStart->
		acs_context;
	DBVGetSupportedContexts(&bSR, &bZ3950);
	if(bSR == DBV_TRUE && bZ3950==DBV_TRUE)
	{
		/* Compare the application context with the supported ones */
		if(strcmp(BASIC_Z3950_OID, pcOID)==0)
			{ *peApplicationProtocol=Z3950_Protocol; }
		if(strcmp(BASIC_SR_OID, pcOID)==0)
			{ *peApplicationProtocol=SR_Protocol; }
	}
	/* else if(return_value == SI_OK) */
	else 
	{
		if(bSR == DBV_TRUE)
		{
			psDBVContextSupported=str2oid(pcSupportedContext);
			psDBVContextSupported=oid_cpy(psDBVContextSupported);
			*peApplicationProtocol=SR_Protocol;
		}
		else if(bZ3950 == DBV_TRUE)
		{
			psDBVContextSupported=str2oid(pcSupportedContext);
			psDBVContextSupported=oid_cpy(psDBVContextSupported);
			*peApplicationProtocol=Z3950_Protocol;
		}
		else
		{
			if(fDBVACSEError == DBV_TRUE)
			{
				psDBVContextSupported=str2oid(BASIC_SR_OID);
				psDBVContextSupported=oid_cpy(
					psDBVContextSupported);
				*peApplicationProtocol=SR_Protocol;
			}
			else
				*peApplicationProtocol = Unidentified_Protocol;
		}
	}
	
	return return_value;
}

/* Run through the proposed presentation contexts and see if they are supported
	by the application (by consulting the ACSE configuration file). 
	Place the accepted ones in the DbvProtocolAndSyntaxes structure */
int DBVAcceptOrRejectPresentationContexts(DbvProtocolAndSyntaxes 
	*psProtocolAndSyntaxes, DbvApplicationProtocol eApplicationProtocol, 
	struct PSAPctxlist *psPresentationContextList, DbvError **ppsError)
{
	int return_value = SI_OK;
	long lLoop=0;
	long lCommonLoop=0;
	/* And the prize for best variable name goes to .... */
	long lPCsWhichAreRejectedButAreNotACNs = 0;
	char *pcOIDString;
	DbvBool bSR, bZ3950;
	char pcSupportedApplicationContext[32]=BASIC_SR_OID;
	/* NPCTX - See sil_defs.h */
	char *apcSupportedPresentationContexts[NPCTX];
	DbvBool bFoundMatch = DBV_FALSE;
	DbvBool bJustACNs = DBV_TRUE;

	/* Work out which Application context to support so that when one
		is read in, we know whether to accept or reject it. Defaults
		to SR. (If the value of eApplicationProtocol is 
		Unknown_Protocol or Unidentified_Protocol the association
		will be rejected) */
	if(eApplicationProtocol==Z3950_Protocol)
		strcpy(pcSupportedApplicationContext, BASIC_Z3950_OID);
	/* Get the supported application contexts */
	DBVGetSupportedContexts(&bSR, &bZ3950);
	
	lLoop=0;
	if(psPresentationContextList->pc_nctx == 1)
	{
		/* The only Presentation Context sent is the ACSE */
		/* which is handled automatically (I believe) */
	}
	else
	{
	while(fDBVACSEError == DBV_FALSE && return_value == SI_OK && 
		lLoop<(psPresentationContextList->pc_nctx-1))
	/* pc_nctx-1 becuase we're not interested in the ASCE pci version 
		and it's always last on the list */
        {
                pcOIDString=sprintoid(psPresentationContextList->
			pc_ctx[lLoop].pc_asn);
			
		return_value=DBVMatchPresentationContext(pcOIDString,
			&bFoundMatch, ppsError);
		if(return_value == SI_NOTOK)
		{
			fDBVACSEError = DBV_TRUE;
		}
		if(return_value == SI_OK && bFoundMatch==DBV_TRUE)
		{
			/* The propsed presentation context is supported */
			psPresentationContextList->pc_ctx[lLoop].pc_result=
				PC_ACCEPT;
			apcSupportedPresentationContexts[lCommonLoop] = 
				(char *)DbvAlloc(strlen(pcOIDString)+1);
			strcpy(apcSupportedPresentationContexts[lCommonLoop],
				pcOIDString);
			lCommonLoop ++;
			bJustACNs = DBV_FALSE;
		}
		if(return_value == SI_OK && bFoundMatch==DBV_FALSE)
		{
			if((strcmp(pcOIDString, BASIC_SR_OID) == 0) || 
				(strcmp(pcOIDString, BASIC_Z3950_OID) == 0))
			{
				/* The OID is a supported Application 
					Context so accept it */
				psPresentationContextList->pc_ctx[lLoop].
					pc_result=PC_ACCEPT;
			/* Accept them but don't place them in the list! */
			}
			else
			{
				/* The propsed presentation context is not 
					supported */
				psPresentationContextList->pc_ctx[lLoop]
					.pc_result=PC_REJECTED;
				lPCsWhichAreRejectedButAreNotACNs ++;
				bJustACNs = DBV_FALSE;
			}
		}
		lLoop++;
        } /* End while */
	/* If fDBVACSEError == DBV_TRUE an error has occurred, so we must
		reject all Presentation Contexts */
	if(fDBVACSEError == DBV_TRUE)
	{
		lCommonLoop = 0;
		DBVRejectPresentationContexts(psPresentationContextList);
	} /* End if */
	
	} /* End if */

	psProtocolAndSyntaxes->peAbstractSyntax = (DbvAbstractSyntax *)
		DbvAlloc(sizeof(DbvAbstractSyntax));
	if(bJustACNs == DBV_TRUE && fDBVACSEError == DBV_FALSE)
	{
		/* No syntaxes provided No_Syntaxes_Provided*/
		*psProtocolAndSyntaxes->peAbstractSyntax = No_Syntaxes_Provided;
	}
	else if(lCommonLoop > 0)
	{
		*psProtocolAndSyntaxes->peAbstractSyntax = Common_Syntaxes;
		lLoop = 0;
		/* There are some common syntaxes Common_Syntaxes */
		psProtocolAndSyntaxes->ppcCommonSyntaxes = (char **)DbvAlloc
			(sizeof(char *)*(lCommonLoop+1));
		for(lLoop = 0; lLoop < lCommonLoop; lLoop ++)
			psProtocolAndSyntaxes->ppcCommonSyntaxes[lLoop] = 
				apcSupportedPresentationContexts[lLoop];
	}
	else
	{
		*psProtocolAndSyntaxes->peAbstractSyntax = No_Common_Syntax;
		/* No common syntaxes No_Common_Syntax */
	}
	
	return return_value;
}

void DBVRejectPresentationContexts(struct PSAPctxlist 
	*psPresentationContextList)
{
	int lLoop = 0;
	int lRequestedContexts = 0;
	char *pcOIDString;

	/* Reject all in the list (except the included ACNs) and make sure that 
		the association is rejected */
	for(lLoop = 0; lLoop < (psPresentationContextList->pc_nctx-1);lLoop ++)
	{
                pcOIDString=sprintoid(psPresentationContextList->
			pc_ctx[lLoop].pc_asn);

		/* If the context is not an application context count it as
		as a requested context */
		if((strcmp(pcOIDString, BASIC_SR_OID) != 0) &&
		    (strcmp(pcOIDString, BASIC_Z3950_OID) !=0 ))
		{
			lRequestedContexts++;
			psPresentationContextList->pc_ctx[lLoop].pc_result = 
				PC_REJECTED;
		}
		else
		{
			psPresentationContextList->pc_ctx[lLoop].pc_result = 
				PC_ACCEPT;
		}
	}

	/* Are there any requested presentation contexts? If so we must 
	make sure that the service-user rejects the association */
	if(lRequestedContexts > 0)
		fDBVAcceptAllowed = DBV_FALSE;

}

#endif

