/*  Module Name: @(#)targetd_tcpip.c	7.2
    Version: 12/2/96
    Author: Chris Attewell/John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
	Target daemon - Origins connect here and target slaves are spawned	
 
 
*/



#include "z3950_includes.h"

#if !defined(TRUE)
#define TRUE  1
#define FALSE 0
#endif

void print_cmdline_options(void);

/* Function called to kill children which have changed state */
void reaper();
void killer();
void get_origin_address(struct sockaddr_in sin,int sinlen,char *originaddress);
void parse_commandline(int argc, char **argv, char **xterm_path);


char slave_store[128];

/* Full Path of slave program (e.g. /home/dbvosi/src/api/apitesttargetslave) */
char *slave_program = NULL;


int socket_fd;
char server_socket[8] = TARGET_PORT;
int xtermbased_slaves = TRUE;

main(int argc, char **argv)
{
	/* xterm path */
	char *xterm_path = XTERM;
	
	/* xterm start options */
	char *xterm_start_options = "-sl 1000";

	char *originaddress;

	char choice_string[100];
	char sock_buf[10];
	int new_socket_fd;
	int slave_pid = 0;
	int repeat;
	struct hostent *hp;

        struct sockaddr_in sin; /* client's address placed in here */
        int sinlen; /* length of client's address */

	int fd, newprog;
        char sockfd[10];
	char *header = NULL;
	
	sinlen = sizeof(sin);
 
	signal(SIGINT, killer);

	/* Set up a default slave_program (the -e command line switch
	   can be used to change the path of the slave program) */
	strcpy(slave_store, "../libtest/apitesttargetslave");

	/* If a slave program name has been given as an argument
	if (argc == 2)
		strcpy(slave_program, argv[1]); */
	slave_program = slave_store;

	parse_commandline(argc, argv, &xterm_path);
 
        printf("\n\n\n");

	header = "DBV-OSI II Target Daemon (c) CSL\n";

	printf("%s", header);
        printf("--------------------------------\n");
        printf("\n");

	printf("Slave program:%s\n", slave_program);
	if (xtermbased_slaves) {
		printf("xterm path:%s\n", xterm_path);
		printf("xterm start options:%s\n", xterm_start_options);
	}




/* Set up a signal trap function for any child processes which change state */
	signal(SIGCHLD, reaper);

	if((socket_fd = initialise_socket(server_socket, "tcp", 5)) == -1)
	{
		printf("targetd: Could not create socket\n");
		exit(-1);
	}
	printf("Configured for port:%s\n",server_socket);
	printf("target daemon listening\n");

	do
	{
		do{
			repeat = FALSE;
			new_socket_fd = accept(socket_fd, (struct sockaddr *)&sin, &sinlen);
			if(new_socket_fd <0)
			{
				if(errno == EINTR)
					repeat = TRUE;
				else
					printf("Error accepting new connection\n");
			}
			else
			{

				/* Get the Internet address of the client 
				   which has connected */

				/* Attempt to get the domain name form */
				hp = gethostbyaddr((char *) &sin.sin_addr, sizeof(struct in_addr), sin.sin_family);

				/* If couldnt get domain name form */
				if (hp == NULL)

					/* Get dot notation form */
					originaddress = inet_ntoa(sin.sin_addr);
				else
					originaddress = (char *) hp->h_name;

				printf("New connection. Origin address=%s\n",originaddress);
				switch(slave_pid = fork())
				{
				case 0: /* Child */
					(void)close(socket_fd);
					sprintf(sock_buf,"%d",new_socket_fd);

					if (xtermbased_slaves) 
						execlp(xterm_path, "xterm", "-sl", "1000", "-e", slave_program, "TCP", sock_buf, NULL);
					else {
						/* anything accepted for 
						   second argument 
						   (arg0 program name) */
						execlp(slave_program, slave_program, "TCP", sock_buf, NULL);
					}

					printf("Error in execlp!\n");
					exit(-1);
				default: /* Parent Process */
					close(new_socket_fd);
					break;
				case -1:
					printf("Fork error!\n");
					break;
				}
			}
		}
		while(repeat == TRUE);
	}
	while(1);

}
/* Initialise a socket (bind and listen) */
int initialise_socket( char *service, char *protocol, int qlen )
{
        struct servent  *pse;   /* pointer to service information entry */
        struct protoent *ppe;   /* pointer to protocol information entry*/
        struct sockaddr_in sin; /* an Internet endpoint address         */
        int     s, type;        /* socket descriptor and socket type    */
 
 
        static u_short  portbase = 0;
 
        bzero_replacement((char *)&sin, sizeof(sin));
        sin.sin_family = AF_INET;
        sin.sin_addr.s_addr = htonl(INADDR_ANY);
 
        /* Map service name to port number if possible */
        if ( pse = getservbyname(service, protocol) )
                sin.sin_port = htons(ntohs((u_short)pse->s_port)
                        + portbase);
        else if ( (sin.sin_port = htons((u_short)atoi(service))) == 0 ) {
                printf("initialise_socket:can't get \"%s\" service entry\n", service);
                return -1;
        }
        /* Map protocol name to protocol number */
        if ( (ppe = getprotobyname(protocol)) == 0) {
                printf("initialise_socket:can't get \"%s\" protocol entry\n", protocol);
                return -1;
        }
 
        type = SOCK_STREAM;
        /* Allocate a socket */
        s = socket(PF_INET, type, ppe->p_proto);
        if (s < 0){
		if (errno > 0 && errno < sys_nerr)
                	printf("initialise_socket:can't create socket: %s\n", sys_errlist[errno]);
		else
                	printf("initialise_socket:can't create socket: %s\n", "");

                return -1;
        }
 
        /* Bind the socket */
        if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0){
		if (errno > 0 && errno < sys_nerr)
                	printf("initialise_socket:can't bind to %s port: %s\n", service, sys_errlist[errno]);
		else
                	printf("initialise_socket:can't bind to %s port: %s\n", service, "");

                return -1;
        }
        if (type == SOCK_STREAM && listen(s, qlen) < 0){
		if (errno > 0 && errno < sys_nerr)
                	printf("initialise_socket:can't listen on %s port: %s\n", service, sys_errlist[errno]);
		else
                	printf("initialise_socket:can't listen on %s port: %s\n", service, "");

                return -1;
        }
        return s;
}

/* Parse command line arguments */
void parse_commandline(int argc, char **argv, char **xterm_path)
{
	int place=1;
	int parsed;
	long port;	
	FILE *fd;
	char *temp = NULL;
	
	/* Interactive (xterm based) slaves by default */
	xtermbased_slaves = TRUE;

	while(argv[place])
	{
		parsed = 0;

		/* help */
		if(strcmp(argv[place],"-h")==0)
		{
			print_cmdline_options();
			exit(0);
		}

		/* port change */
		if(strcmp(argv[place],"-p")==0)
		{
			place++;
			if(strlen(server_socket)>8 || atoi(argv[place])<=0)
			{
				printf("Invalid port number \"%s\"\n",argv[place]);
				exit(0);
			}
			strcpy( server_socket ,argv[place]);
			parsed=1;
		}

		/* Alternative target slave path */
		if(strcmp(argv[place] ,"-e") == 0)
		{
			slave_program = argv[++place];
			parsed=1;
		/* check it exists */
			if((fd = fopen(argv[place],"rb")) == NULL)
			{
				printf("Program %s does not exist\n", argv[place]);
				exit(0);
			}
			else
				close(fd);
		}		


		/* Alternate xterm path */
		if(strcmp(argv[place] ,"-xp") == 0)
		{
			*xterm_path = argv[++place];
			parsed=1;
		}		


		/* Non interactive slaves */
		if(strcmp(argv[place] ,"-n") == 0)
		{
			xtermbased_slaves = FALSE;
			parsed=1;
		}		


		if(parsed == 0)
		{
			printf("Invalid option:%s\n",argv[place]);
			exit(0);
		}
		
		place++;
	}
}
		


void print_cmdline_options(void)
{
	printf("\nUsage:\n\ttargetd_tcpip [-options ...]\n\n"); 
	printf("where options include:\n");
	printf("\t-h\t\tprint out this message\n");
	printf("\t-n\t\tgenerate non-interactive slaves (no xterms)\n");
	printf("\t-xp path\tspecify full xterm path (only necessary if the\n\t\t\t configured xterm path setting is wrong)\n");
	printf("\t-p port\t\tspecify port used to listen for incoming\n\t\t\t origin connections\n");
	printf("\t-e path\t\tspecify full slave program path\n");
	printf("\nExamples:\n\n");
	printf("targetd_tcpip -p 2055\n");
	printf("targetd_tcpip -p 2034 -e /home/users/mike/apitesttargetslave\n");
	printf("targetd_tcpip -n -p 2066\n");
	printf("targetd_tcpip -xp /home/users/mike/xterm\n\n");
	
}



/**********************************************************************
 reaper - routine to get the status of then kill any potential zombie children
 
            In params:  None
 
            Out params: None
 
            Returned: None. Execution is returned to the point at which
                      it was interrupted - interrupted functions
                      have return value of EINTR
*********************************************************************/

void reaper(int i)
{

/* The following works on BSD and System V systems: */
        int wpid, status;
 
	wpid = wait(&status);
        printf("Reaped and killed process %d\n", wpid);
 

	/* Re-arm the signal - needed on most System V systems (e.g. HP) */
	signal(SIGCHLD, reaper);
}

void killer(int i)
{
	close(socket_fd);
	printf("\nArg!\n");
	exit(-1);
}
