#include "tsendbas.h"




int send_presrsp_genlite_to_origin(int associd)
{
	char *filedata = NULL;
	char *image_file="jupiter.bmp";
	int filesize = 0;

	/* Fragment size */
	int fragsize = FRAGSIZE;

	/* load in image into filedata */
	if (get_image(&filedata, image_file, &filesize))
		printf("Got image ok: filesize = %d\n", filesize);
	else {
		printf("Failed to get image\n"); 
		return;
	}

	/* Split the image into fragments */
	split_into_frags(image_file, filedata, filesize, fragsize, associd);

}



DbvBool get_image(char **filedata, char *image_file, int *filesize)
{ 
	int fd = 0;
	char *filed;
	
	printf("Image file: %s\n", image_file);

	/* Open the image file */
	if (open_image_file(image_file, &fd, filesize))
		printf("Image file ok\n");
	else {
		printf("Error with image file\n");
		return DBV_FALSE;
	}
 

	/* Malloc space for all the data in the file */

	if ( (*filedata = (char *) malloc (*filesize)) == NULL ) {
		printf("Error allocating space\n");
		return DBV_FALSE;
	}

	/* Read in the file */
	if (read_in_filedata(fd, *filedata, *filesize))
		printf("Read in file data ok\n");
	else {
		printf("Error reading in file data\n");
		return DBV_FALSE;
	}


	return DBV_TRUE;
}



DbvBool open_image_file(char *image_file, int *fd, int *size)
{
	struct stat sbuf;

	*fd = open(image_file, O_RDONLY, 0);

	/* If cant open file */
	if (*fd < 0 ) {
		perror("Error opening image file");
		return DBV_FALSE;
	}   

	/* If cant get structure containig size of file */
	if (fstat(*fd, &sbuf) < 0) {
        	perror("Error getting file size");
		return DBV_FALSE;
	}
	
	*size = sbuf.st_size;
	printf("Size of image file = %d\n", *size);

	return DBV_TRUE;
}



DbvBool read_in_filedata(int fd, char *filedata, int filesize)
{
	if ( read(fd, filedata, filesize) != filesize) {
		perror("Error reading in file data");
		return DBV_FALSE;
	}
	return DBV_TRUE;
}




DbvBool split_into_frags(char *image_file, char *filedata, int filesize, int fragsize, DbvAssocId associd)
{
	int datadone = 0;
	int fragment = 0;

	static DbvDbRecordType current_frag = NOT_YET_SET; 

	/* While not processed whole file */
	while (datadone < filesize) {

		if (datadone + fragsize > filesize) {
			fragment = filesize - datadone;
			current_frag = DRT_FinalFragment; 
		} else {
			fragment = fragsize;
			if (current_frag == NOT_YET_SET)
				current_frag = DRT_StartingFragment;
			else
				current_frag = DRT_IntermediateFragment;
		}


		printf("Fragment: Type = %d, size = %d\n", current_frag, fragment);
		if (datadone < filesize) 
			send_frag(image_file, filedata+datadone, fragment, current_frag, associd);

		datadone += fragment;


	}

	printf("data done = %d\n", datadone);
}


int send_frag(char *imagename, char *imagedata, int imagedatasize, DbvDbRecordType current_frag, DbvAssocId associd)
{
	DbvAssocId 	 lAssocId = associd;
	DbvPresentRsp	*psPresentRsp = NULL;
	DbvSegmentReq   *psSegmentReq = NULL;
	DbvError	*psError = NULL;
 

	switch (current_frag) {

	case DRT_StartingFragment:
	case DRT_IntermediateFragment:

		/* Allocate memory for psSegmentReq and fill it in */
		psSegmentReq = (DbvSegmentReq *) DbvAlloc (sizeof(DbvSegmentReq));
		fill_geseg_request(psSegmentReq, imagename, imagedata, imagedatasize, current_frag);
		printf("sending segment request frag ..\n");

		if (DbvSegmentRequest(lAssocId, psSegmentReq, &psError) == SI_NOTOK)
			printf("Failed to send SegmentRequest - check trace file for diagnostic\n");

		break;

	case DRT_FinalFragment:

		/* Allocate memory for psPresentRsp and fill it in */
		psPresentRsp = (DbvPresentRsp *) DbvAlloc (sizeof(DbvPresentRsp));
		fill_gepres_response(psPresentRsp, imagename, imagedata, imagedatasize, current_frag);
		printf("sending present response frag ..\n");

		/* Send PresentResponse to client */
		if (DbvPresentResponse(lAssocId, psPresentRsp, &psError) == SI_NOTOK)
			printf("Failed to send PresentResponse - check trace file for diagnostic\n");
		break;
	}
}



/* The only request which goes from target to origin */
int fill_geseg_request(DbvSegmentReq *psSegmentReq, char *imagename, char *imagedata, int imagedatasize, DbvDbRecordType current_frag)
{

	/* Compulsory fields */

	/* ulNumberOfRecordsReturned */
	psSegmentReq->ulNumberOfRecordsReturned = 1;

	/* optional */
	
	/* Records - create 1: 0, \0*/
	psSegmentReq->ppsSegmentRecords = (DbvNamePlusRecord **) DbvAlloc (sizeof(DbvNamePlusRecord *) * 2);
	fill_segment_records_genlite(psSegmentReq->ppsSegmentRecords, imagename, imagedata, imagedatasize, current_frag);

}
 


 
int fill_segment_records_genlite(DbvNamePlusRecord **ppsResponseRecord, char *imagename, char *imagedata, int imagedatasize, DbvDbRecordType current_frag)
{

	/* A segment request can only hold start or intermediate fragments */
	DbvASN1External *external0 = NULL;

	ppsResponseRecord[0] = (DbvNamePlusRecord *) DbvAlloc (sizeof (DbvNamePlusRecord));

	/* Put the next fragment of the image into
	   a generic record external */
	external0 = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
	fill_external_genlite(external0, imagename, imagedata, imagedatasize);


	/* No databasename */
	
	switch (current_frag) {
	case DRT_StartingFragment:
		ppsResponseRecord[0]->eDbRecordType = DRT_StartingFragment;
		ppsResponseRecord[0]->uDbRecord.psStartingFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof (DbvFragmentSyntax));

		ppsResponseRecord[0]->uDbRecord.psStartingFragment->eFragmentSyntaxType = FST_ExternallyTagged;
		ppsResponseRecord[0]->uDbRecord.psStartingFragment->uFragmentSyntax.psExternallyTagged = external0; 

		
		break;
	case DRT_IntermediateFragment:
		ppsResponseRecord[0]->eDbRecordType = DRT_IntermediateFragment;
		ppsResponseRecord[0]->uDbRecord.psIntermediateFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof (DbvFragmentSyntax));

		ppsResponseRecord[0]->uDbRecord.psIntermediateFragment->eFragmentSyntaxType = FST_ExternallyTagged;
		ppsResponseRecord[0]->uDbRecord.psIntermediateFragment->uFragmentSyntax.psExternallyTagged = external0; 

		break;
	}

}



int fill_external_genlite(DbvASN1External *external, char *imagename, char *imagedata, int imagedatasize)
{

	char *oid = ANSI_GENERIC_RECORD;

	/* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */
	/* +1 to copy the \0 */
	external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);

	memcpy(external->pcObjectIdentifier, oid, strlen(oid));

	external->eValueType = EVT_SingleASN1Type;
	external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
	external->uExternalValue.psSingleASN1Type->eASN1Type = AT_GenericRecord;
	external->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord = (struct DbvGenericRecord *) DbvAlloc (sizeof (struct DbvGenericRecord));

	fill_the_generic(external->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, imagename, imagedata, imagedatasize);

}



int fill_the_generic(struct DbvGenericRecord *psGenericRecord, char *imagename, char *imagedata, int imagedatasize)
{
	
	psGenericRecord->ppsTaggedElement = (DbvTaggedElement **) DbvAlloc (sizeof(DbvTaggedElement *) * 2);

	psGenericRecord->ppsTaggedElement[0] = (DbvTaggedElement *) DbvAlloc (sizeof(DbvTaggedElement));
	fill_tagged_element(psGenericRecord->ppsTaggedElement[0], imagename, imagedata, imagedatasize);

	/* Terminater */
	psGenericRecord->ppsTaggedElement[1] = NULL;
}



int fill_tagged_element(DbvTaggedElement *psDbvTaggedElement, char *imagename, char *imagedata, int imagedatasize)
{
	/* image name */
	psDbvTaggedElement->sTagValue.eStringOrNumeric = SNT_String;
	psDbvTaggedElement->sTagValue.uStringOrNumeric.psString = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(psDbvTaggedElement->sTagValue.uStringOrNumeric.psString, imagename);



	/* image data fragement */
	fill_content(&psDbvTaggedElement->sContent, imagedata, imagedatasize);	
}



int fill_content(DbvElementDataG *psContent, char *imagedata, int imagedatasize)
{

	psContent->eElementDataGType = EDT_Octets;
	psContent->uElementDataG.psOctets = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));

	psContent->uElementDataG.psOctets->pcValue = (char *) DbvAlloc (imagedatasize);
	psContent->uElementDataG.psOctets->ulLength = imagedatasize;
	memcpy(psContent->uElementDataG.psOctets->pcValue, imagedata, imagedatasize);
}




int fill_gepres_response(DbvPresentRsp *psPresentRsp, char *imagename, char *imagedata, int imagedatasize, DbvDbRecordType current_frag)
{

	/* Compulsory fields */

	/* ulNumberOfRecordsReturned */
	psPresentRsp->ulNumberOfRecordsReturned = 1;

	/* ulNextResultSetPosition */
	psPresentRsp->ulNextResultSetPosition = 2;

	/* ePresentStatus */
	psPresentRsp->ePresentStatus = PS_Success;

	/* optional */
	
	/* Records */
	psPresentRsp->psRecords = (DbvRecords *) DbvAlloc (sizeof(DbvRecords));
	fill_generic_records(psPresentRsp->psRecords, imagename, imagedata, imagedatasize, current_frag);

}
 


int fill_generic_records(DbvRecords *psRecords, char *imagename, char *imagedata, int imagedatasize, DbvDbRecordType current_frag)
{

	/* A PresentResponse can only hold a final fragment */

	DbvASN1External *external0 = NULL;

	/* 1 Response Record - NamePlusRecord: 1 GenericRecord  */
	psRecords->eRecordType = RT_Response;

	/* Create 1 records, 0 \0 */
	psRecords->uRecord.ppsResponseRecord = (DbvNamePlusRecord **) DbvAlloc (sizeof(DbvNamePlusRecord *) * 2);
	psRecords->uRecord.ppsResponseRecord[0] = (DbvNamePlusRecord *) DbvAlloc (sizeof(DbvNamePlusRecord));

	external0 = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
	fill_external_genlite(external0, imagename, imagedata, imagedatasize);


	/* No databasename */
	
	psRecords->uRecord.ppsResponseRecord[0]->eDbRecordType = DRT_FinalFragment;
	psRecords->uRecord.ppsResponseRecord[0]->uDbRecord.psFinalFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof (DbvFragmentSyntax));

	psRecords->uRecord.ppsResponseRecord[0]->uDbRecord.psFinalFragment->eFragmentSyntaxType = FST_ExternallyTagged;
	psRecords->uRecord.ppsResponseRecord[0]->uDbRecord.psFinalFragment->uFragmentSyntax.psExternallyTagged = external0; 
}



