/*  Module Name: @(#)z3950_externals4.c	7.18
    Version: 12/6/96
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: More DBV-OSI II externals setup/get code
*/




#include "z3950_externals4.h"


/* AT_AccessControlPrompt1 */
/* typedef struct PromptObject */
/* externals e */


/* dbv -> snacc ansi DbvPromptObject external */

int DBVdbv_to_snacc_DbvPromptObject_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static PromptObject promptobj_external[STRING_LEN];
 
        PromptObject *promptobj_ptr;

	/* Set up the PromptObject external */
	promptobj_ptr = &promptobj_external[current_promptobj];

	/* Clear the location (may have been used by the last PDU) */
	memset(promptobj_ptr, 0, sizeof(PromptObject));

	DBVdbv_to_snacc_DbvPromptObject_main(promptobj_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psPromptObject, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &promptobj_external[current_promptobj];

	/* Only if not at end of array */
	if (current_promptobj < (STRING_LEN - 1)) {

		/* Increment current_promptobj for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_promptobj++;
	}
}



/* dbv -> snacc iso DbvPromptObject external */

int DBVdbv_to_snacc_isoDbvPromptObject_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoPromptObject isopromptobj_external[STRING_LEN];
        IsoPromptObject *isopromptobj_ptr;

	isopromptobj_ptr = &isopromptobj_external[current_isopromptobj];
	memset(isopromptobj_ptr, 0, sizeof(IsoPromptObject));
	DBVdbv_to_snacc_DbvPromptObject_main(isopromptobj_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psPromptObject, env);
	any_type->value = (void *) &isopromptobj_external[current_isopromptobj];

	if (current_isopromptobj < (STRING_LEN - 1)) {
		current_isopromptobj++;
	}
}




/* WARNING: pointers to functions, pointers to functions !! */

int DBVdbv_to_snacc_DbvPromptObject_main(PromptObject *promptobj_ptr, struct DbvPromptObject *psPromptObject, jmp_buf env)
{

	switch (psPromptObject->ePromptObjectType) {
	case POT_Challenge:
		promptobj_ptr->choiceId = PROMPTOBJECT_CHALLENGE;

		DBVdbv_to_snacc_list_opt(&promptobj_ptr->a.challenge, (void **) psPromptObject->uPromptObject.ppsChallenge, sizeof(ChallengeSeq), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvChallenge));

		break;
	case POT_Response:
		promptobj_ptr->choiceId = PROMPTOBJECT_RESPONSE;

		DBVdbv_to_snacc_list_opt(&promptobj_ptr->a.response, (void **) psPromptObject->uPromptObject.ppsResponse, sizeof(ResponseSeq), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvResponse));

		break;
	}
}



void DBVdbv_to_snacc_list_opt(AsnList **snacclist, void **dbv_list, long snaccnodesize, jmp_buf env, 
void (*node_func) (void *, void *, jmp_buf)
)
{
        if (dbv_list) {
		*snacclist = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*snacclist, env);
		DBVdbv_to_snacc_list(*snacclist, dbv_list, snaccnodesize, env,  node_func);
	}
}



void DBVdbv_to_snacc_list_comp(AsnList **snacclist, void **dbv_list, long snaccnodesize, jmp_buf env, 
void (*node_func) (void *, void *, jmp_buf)
)
{
	DBVdbv_to_snacc_list_opt(snacclist, dbv_list, snaccnodesize, env, node_func);
}








void DBVdbv_to_snacc_list(AsnList *snacclist, void **dbv_list, long snaccnodesize, jmp_buf env, 
void (*node_func) (void *, void *, jmp_buf)
)
{
	void *data_ptr = NULL;
	void **data_ptr_address = NULL;
 
	int current_listnode = 0;

	while (dbv_list[current_listnode] != NULL) {
		data_ptr_address = AsnListAppend(snacclist);
		CheckAsn1Alloc(data_ptr_address, env);
   	 
		*data_ptr_address = Asn1Alloc(snaccnodesize);
		CheckAsn1Alloc(*data_ptr_address, env);

		data_ptr = *data_ptr_address;

		(*node_func) (data_ptr, dbv_list[current_listnode], env);
		current_listnode++;
       	}
}



void DBVdbv_to_snacc_DbvChallenge(ChallengeSeq *challenge, DbvChallenge *psChallenge, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvPromptId_comp(&challenge->promptId, &psChallenge->sPromptId, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&challenge->defaultResponse, psChallenge->psDefaultResponse, env);

	/* o */
	DBVdbv_to_snacc_DbvPromptInfo_opt(&challenge->promptInfo, psChallenge->psPromptInfo, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&challenge->regExpr, psChallenge->psRegExpr, env);

	/* o */
	DBVdbv_to_snacc_exp_nullpointer_opt(&challenge->responseRequired, psChallenge->pvResponseRequired, env);


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&challenge->allowedValues, psChallenge->ppsAllowedValues, env);


	/* o */
	DBVdbv_to_snacc_exp_nullpointer_opt(&challenge->shouldSave, psChallenge->pvShouldSave, env);


	/* o */
	DBVdbv_to_snacc_DbvDataType_opt(&challenge->dataType, psChallenge->peDataType, env);

	/* o */
	DBVdbv_to_snacc_external_opt(&challenge->diagnostic, psChallenge->psDiagnostic, env);

}



/* compulsory which in snacc terms is a pointer ! */
void DBVdbv_to_snacc_DbvPromptId_comp(struct PromptId **promptId, DbvPromptId *psPromptId, jmp_buf env)
{
	*promptId = (struct PromptId*) Asn1Alloc(sizeof (struct PromptId));
	CheckAsn1Alloc(*promptId, env);
	DBVdbv_to_snacc_DbvPromptId(*promptId, psPromptId, env);
}



void DBVdbv_to_snacc_DbvPromptId(struct PromptId *promptId, DbvPromptId *psPromptId, jmp_buf env)
{
	switch (psPromptId->ePromptIdType) {
	case PIT_EnumeratedPrompt:
		promptId->choiceId = PROMPTID_ENUMMERATEDPROMPT;
		DBVdbv_to_snacc_DbvEnumeratedPrompt_comp(&promptId->a.enummeratedPrompt, psPromptId->uPromptId.psEnumeratedPrompt, env);
		break;

	case PIT_NoneEnumeratedPrompt:
		promptId->choiceId = PROMPTID_NONENUMERATEDPROMPT;
		DBVdbv_to_snacc_interstring_comp(&promptId->a.nonEnumeratedPrompt, psPromptId->uPromptId.psNonEnumeratedPrompt, env);
		break;
	}
}



void DBVdbv_to_snacc_DbvEnumeratedPrompt_comp(struct PromptIdSeq** enummeratedPrompt, DbvEnumeratedPrompt *psEnumeratedPrompt, jmp_buf env)
{
	*enummeratedPrompt = (struct PromptIdSeq*) Asn1Alloc (sizeof (struct PromptIdSeq));
	CheckAsn1Alloc(*enummeratedPrompt, env);
	DBVdbv_to_snacc_DbvEnumeratedPrompt(*enummeratedPrompt, psEnumeratedPrompt, env);
}


void DBVdbv_to_snacc_DbvEnumeratedPrompt(struct PromptIdSeq* enummeratedPrompt, DbvEnumeratedPrompt *psEnumeratedPrompt, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvEPromptIdType(&enummeratedPrompt->type, &psEnumeratedPrompt->eType, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&enummeratedPrompt->suggestedString, psEnumeratedPrompt->psSuggestedString, env);
}



void DBVdbv_to_snacc_DbvEPromptIdType(PromptIdSeqInt *type, DbvEPromptIdType *peType, jmp_buf env)
{
	switch (*peType) {
	case EPIT_GroupId:
		*type = GROUPID;
		break;
	case EPIT_UserId:
		*type = USERID;
		break;
	case EPIT_Password:
		*type = PASSWORD;
		break;
	case EPIT_NewPassword:
		*type = NEWPASSWORD;
		break;
	case EPIT_Copyright:
		*type = COPYRIGHT;
		break;
	case EPIT_SessionId:
		*type = SESSIONID;
		break;
	}
}


void DBVdbv_to_snacc_DbvPromptInfo_opt(struct ChallengeSeqChoice **promptInfo, DbvPromptInfo *psPromptInfo, jmp_buf env)
{
	if (psPromptInfo) {
		*promptInfo = (struct ChallengeSeqChoice*) Asn1Alloc(sizeof (struct ChallengeSeqChoice));
		CheckAsn1Alloc(*promptInfo, env);
		DBVdbv_to_snacc_DbvPromptInfo(*promptInfo, psPromptInfo, env);
	}
}



void DBVdbv_to_snacc_DbvPromptInfo(struct ChallengeSeqChoice *promptInfo, DbvPromptInfo *psPromptInfo, jmp_buf env)
{
	switch (psPromptInfo->ePromptInfoType) {
	case PIIT_Character:
		promptInfo->choiceId = CHALLENGESEQCHOICE_CHARACTER;
		DBVdbv_to_snacc_interstring_comp(&promptInfo->a.character, psPromptInfo->uPromptInfo.psCharacter, env);
		break;
	case PIIT_Encrypted:
		promptInfo->choiceId = CHALLENGESEQCHOICE_ENCRYPTED;
		DBVdbv_to_snacc_DbvEncryption_comp(&promptInfo->a.encrypted, psPromptInfo->uPromptInfo.psEncrypted, env);
		break;
	}
}



void DBVdbv_to_snacc_DbvEncryption_comp(struct Encryption** encrypted, DbvEncryption *psEncryption, jmp_buf env)
{
	*encrypted = (struct Encryption*) Asn1Alloc(sizeof (struct Encryption));
	CheckAsn1Alloc(*encrypted, env);
	DBVdbv_to_snacc_DbvEncryption(*encrypted, psEncryption, env);
}



void DBVdbv_to_snacc_DbvEncryption(struct Encryption* encrypted, DbvEncryption *psEncrypted, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_octet_string(&encrypted->cryptType, psEncrypted->psCryptType, env);

	/* o */
	DBVdbv_to_snacc_octet_string(&encrypted->credential, psEncrypted->psCredential, env);

	/* c */
	DBVdbv_to_snacc_octet_string(&encrypted->data, &psEncrypted->sData, env);
}



void DBVdbv_to_snacc_DbvDataType_opt(ChallengeSeqInt **dataType, DbvDataType *peDataType, jmp_buf env)
{
	if (peDataType) {
		*dataType = (ChallengeSeqInt*) Asn1Alloc(sizeof (ChallengeSeqInt));
		CheckAsn1Alloc(*dataType, env);
		DBVdbv_to_snacc_DbvDataType(*dataType, peDataType, env);
	}
}



void DBVdbv_to_snacc_DbvDataType(ChallengeSeqInt *dataType, DbvDataType *peDataType, jmp_buf env)
{
	switch (*peDataType) {
	case DTTT_Integer:
		*dataType = INTEGER;
		break;
	case DTTT_Date1:
		*dataType = DATE1;
		break;
	case DTTT_Float:
		*dataType = FLOAT;
		break;
	case DTTT_AlphaNumeric:
		*dataType = ALPHANUMERIC;
		break;
	case DTTT_UrlUrn:
		*dataType = URL_URN;
		break;
	case DTTT_Boolean:
		*dataType = BOOLEAN;
		break;
	}
}



void DBVdbv_to_snacc_DbvResponse(ResponseSeq *response, DbvResponse *psResponse, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvPromptId_comp(&response->promptId, &psResponse->sPromptId, env);

	/* c */
	DBVdbv_to_snacc_DbvPromptResponse_comp(&response->promptResponse, &psResponse->sPromptResponse, env);
}



void DBVdbv_to_snacc_DbvPromptResponse_comp(struct ResponseSeqChoice **promptResponse, DbvPromptResponse *psPromptResponse, jmp_buf env)
{
	*promptResponse = (struct ResponseSeqChoice*) Asn1Alloc(sizeof (struct ResponseSeqChoice));
	CheckAsn1Alloc(*promptResponse, env);
	DBVdbv_to_snacc_DbvPromptResponse(*promptResponse, psPromptResponse, env);
}



void DBVdbv_to_snacc_DbvPromptResponse(struct ResponseSeqChoice *promptResponse, DbvPromptResponse *psPromptResponse, jmp_buf env)
{
	switch (psPromptResponse->eResponseType) {
	case RT_String:
		promptResponse->choiceId = RESPONSESEQCHOICE_STRING;
		DBVdbv_to_snacc_interstring_comp(&promptResponse->a.string, psPromptResponse->uResponse.psString, env);
		break;
	case RT_Accept:
		promptResponse->choiceId = RESPONSESEQCHOICE_ACCEPT;
		DBVdbv_to_snacc_bool(&promptResponse->a.accept, &psPromptResponse->uResponse.fAccept, env);
		break;
	case RT_Acknowledge:
		promptResponse->choiceId = RESPONSESEQCHOICE_ACKNOWLEDGE;

		/* No Dbv structure for compulsory AsnNull */

		break;
	case RT_Diagnostic:
		promptResponse->choiceId = RESPONSESEQCHOICE_DIAGNOSTIC;
		DBVdbv_to_snacc_DbvDiagRec_comp(&promptResponse->a.diagnostic, psPromptResponse->uResponse.psDiagnostic, env);
		break;
	case RT_Encrypted:
		promptResponse->choiceId = RESPONSESEQCHOICE_ENCRYPTED;
		DBVdbv_to_snacc_DbvEncryption_comp(&promptResponse->a.encrypted, psPromptResponse->uResponse.psEncryption, env);
		break;
	}
}


/****************************************************************/


/* snacc -> dbv ansi DbvPromptObject external */

void DBVsnacc_to_dbv_DbvPromptObject_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{

	PromptObject *promptobj_ptr;

	promptobj_ptr = (PromptObject *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_PromptObject;
	psSingleASN1Type->uASN1.psPromptObject = (struct DbvPromptObject *) DbvAlloc (sizeof(struct DbvPromptObject));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psPromptObject, env);

	DBVsnacc_to_dbv_DbvPromptObject_main(psSingleASN1Type->uASN1.psPromptObject, promptobj_ptr, env);

}



/* snacc -> dbv iso DbvPromptObject external */

void DBVsnacc_to_dbv_isoDbvPromptObject_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoPromptObject *isopromptobj_ptr;

	isopromptobj_ptr = (IsoPromptObject *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_PromptObject;
	psSingleASN1Type->uASN1.psPromptObject = (struct DbvPromptObject *) DbvAlloc (sizeof(struct DbvPromptObject));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psPromptObject, env);

	DBVsnacc_to_dbv_DbvPromptObject_main(psSingleASN1Type->uASN1.psPromptObject, isopromptobj_ptr, env);

}



void DBVsnacc_to_dbv_DbvPromptObject_main(struct DbvPromptObject *psPromptObject, PromptObject *promptobj_ptr, jmp_buf env)
{

	switch (promptobj_ptr->choiceId) {
	case PROMPTOBJECT_CHALLENGE:
		psPromptObject->ePromptObjectType = POT_Challenge;
		DBVsnacc_to_dbv_DbvChallenge_list_comp(&psPromptObject->uPromptObject.ppsChallenge, promptobj_ptr->a.challenge, env);
		break;
	case PROMPTOBJECT_RESPONSE:
		psPromptObject->ePromptObjectType = POT_Response;
		DBVsnacc_to_dbv_DbvResponse_list_comp(&psPromptObject->uPromptObject.ppsResponse, promptobj_ptr->a.response, env);
		break;
	default:
		psPromptObject->ePromptObjectType = POT_Invalid;
		break;
	}
}


void DBVsnacc_to_dbv_DbvChallenge_list_comp(DbvChallenge ***pppsChallenge, Challenge *challenge, jmp_buf env)
{
	int num_elements = 0;

        /* c */
	num_elements = AsnListCount(challenge);
	*pppsChallenge = (DbvChallenge **) DbvAlloc (sizeof (DbvChallenge *) * (num_elements + 1));
	CheckDbvAlloc(*pppsChallenge, env);
	DBVsnacc_to_dbv_DbvChallenge_list(*pppsChallenge, challenge, env);
}



void DBVsnacc_to_dbv_DbvChallenge_list(DbvChallenge **ppsChallenge, Challenge *challenge, jmp_buf env)
{
	int index = 0;

	ChallengeSeq *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, challenge) {
		ppsChallenge[index] = (DbvChallenge *) DbvAlloc (sizeof(DbvChallenge));
		CheckDbvAlloc(ppsChallenge[index], env);
		DBVsnacc_to_dbv_DbvChallenge(ppsChallenge[index], data_ptr, env);
		index++;
	}
	ppsChallenge[index] = NULL;
}



void DBVsnacc_to_dbv_DbvChallenge(DbvChallenge *psChallenge, ChallengeSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvPromptId(&psChallenge->sPromptId, data_ptr->promptId, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psChallenge->psDefaultResponse, &data_ptr->defaultResponse, env);

	/* o */
	DBVsnacc_to_dbv_DbvPromptInfo_opt(&psChallenge->psPromptInfo, data_ptr->promptInfo, env);
	
	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psChallenge->psRegExpr, &data_ptr->regExpr, env);
	
	/* o */
	DBVsnacc_to_dbv_nullpointer_opt(&psChallenge->pvResponseRequired, data_ptr->responseRequired, env);
	
	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psChallenge->ppsAllowedValues, data_ptr->allowedValues, env);
	
	/* o */
	DBVsnacc_to_dbv_nullpointer_opt(&psChallenge->pvShouldSave, data_ptr->shouldSave, env);
	
	/* o */
	DBVsnacc_to_dbv_DbvDataType_opt(&psChallenge->peDataType, data_ptr->dataType, env);
	
	/* o */
	DBVsnacc_to_dbv_external_opt(&psChallenge->psDiagnostic, data_ptr->diagnostic, env);
	
}


void DBVsnacc_to_dbv_DbvPromptId(DbvPromptId *psPromptId, struct PromptId *promptId, jmp_buf env)
{
	/* c */
	switch (promptId->choiceId) {
	case PROMPTID_ENUMMERATEDPROMPT:
		psPromptId->ePromptIdType = PIT_EnumeratedPrompt;
		DBVsnacc_to_dbv_DbvEnumeratedPrompt_comp(&psPromptId->uPromptId.psEnumeratedPrompt, promptId->a.enummeratedPrompt, env);
		
		break;
	case PROMPTID_NONENUMERATEDPROMPT:
		psPromptId->ePromptIdType = PIT_NoneEnumeratedPrompt;
		DBVsnacc_to_dbv_interstring_comp(&psPromptId->uPromptId.psNonEnumeratedPrompt, promptId->a.nonEnumeratedPrompt, env);
		break;
	default:
		psPromptId->ePromptIdType = PIT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvEnumeratedPrompt_comp(DbvEnumeratedPrompt **ppsEnumeratedPrompt, struct PromptIdSeq* enummeratedPrompt, jmp_buf env)
{
	*ppsEnumeratedPrompt = (DbvEnumeratedPrompt *) DbvAlloc(sizeof (DbvEnumeratedPrompt));
	CheckDbvAlloc(*ppsEnumeratedPrompt, env);
	DBVsnacc_to_dbv_DbvEnumeratedPrompt(*ppsEnumeratedPrompt, enummeratedPrompt, env);
}



void DBVsnacc_to_dbv_DbvEnumeratedPrompt(DbvEnumeratedPrompt *psEnumeratedPrompt, struct PromptIdSeq* enummeratedPrompt, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvEPromptIdType(&psEnumeratedPrompt->eType, &enummeratedPrompt->type, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psEnumeratedPrompt->psSuggestedString, &enummeratedPrompt->suggestedString, env);
}



void DBVsnacc_to_dbv_DbvEPromptIdType(DbvEPromptIdType *peType, PromptIdSeqInt *type, jmp_buf env)
{
	switch (*type) {
	case GROUPID:
		*peType = EPIT_GroupId;
		break;
	case USERID:
		*peType = EPIT_UserId;
		break;
	case PASSWORD:
		*peType = EPIT_Password;
		break;
	case NEWPASSWORD:
		*peType = EPIT_NewPassword;
		break;
	case COPYRIGHT:
		*peType = EPIT_Copyright;
		break;
	case SESSIONID:
		*peType = EPIT_SessionId;
		break;
	default:
		*peType = EPIT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvPromptInfo_opt(DbvPromptInfo **ppsPromptInfo, struct ChallengeSeqChoice* promptInfo, jmp_buf env)
{
	if (promptInfo) {
		*ppsPromptInfo = (DbvPromptInfo *) DbvAlloc(sizeof (DbvPromptInfo));
		CheckDbvAlloc(*ppsPromptInfo, env);
		DBVsnacc_to_dbv_DbvPromptInfo(*ppsPromptInfo, promptInfo, env);
	}
}



void DBVsnacc_to_dbv_DbvPromptInfo(DbvPromptInfo *psPromptInfo, struct ChallengeSeqChoice* promptInfo, jmp_buf env)
{
	switch (promptInfo->choiceId) {
	case CHALLENGESEQCHOICE_CHARACTER:
		psPromptInfo->ePromptInfoType = PIIT_Character;	
		DBVsnacc_to_dbv_interstring_comp(&psPromptInfo->uPromptInfo.psCharacter, promptInfo->a.character, env);
		break;
	case CHALLENGESEQCHOICE_ENCRYPTED:
		psPromptInfo->ePromptInfoType = PIIT_Encrypted;	
		DBVsnacc_to_dbv_DbvEncryption_comp(&psPromptInfo->uPromptInfo.psEncrypted, promptInfo->a.encrypted, env);
		break;
	default:
		psPromptInfo->ePromptInfoType = PIIT_Invalid;	
		break;
	}
}



void DBVsnacc_to_dbv_DbvEncryption_comp(DbvEncryption **ppsEncrypted, struct Encryption* encrypted, jmp_buf env)
{
	*ppsEncrypted = (DbvEncryption *) DbvAlloc(sizeof (DbvEncryption));
	CheckDbvAlloc(*ppsEncrypted, env);
	DBVsnacc_to_dbv_DbvEncryption(*ppsEncrypted, encrypted, env);
}



void DBVsnacc_to_dbv_DbvEncryption(DbvEncryption *psEncrypted, struct Encryption*encrypted, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvOctetString_opt(&psEncrypted->psCryptType, &encrypted->cryptType, env);

	/* o */
	DBVsnacc_to_dbv_DbvOctetString_opt(&psEncrypted->psCredential, &encrypted->credential, env);

	/* c */
	DBVsnacc_to_dbv_octet_string(&psEncrypted->sData, &encrypted->data, env);
}


void DBVsnacc_to_dbv_DbvDataType_opt(DbvDataType **ppeDataType, ChallengeSeqInt *dataType, jmp_buf env)
{
	if (dataType) {
		*ppeDataType = (DbvDataType *) DbvAlloc(sizeof (DbvDataType));
		CheckDbvAlloc(*ppeDataType, env);
		DBVsnacc_to_dbv_DbvDataType(*ppeDataType, dataType, env);
	}
}



void DBVsnacc_to_dbv_DbvDataType(DbvDataType *peDataType, ChallengeSeqInt *dataType, jmp_buf env)
{
	switch (*dataType) {
	case INTEGER:
		*peDataType = DTTT_Integer;
		break;
	case DATE1:
		*peDataType = DTTT_Date1;
		break;
	case FLOAT:
		*peDataType = DTTT_Float;
		break;
	case ALPHANUMERIC:
		*peDataType = DTTT_AlphaNumeric;
		break;
	case URL_URN:
		*peDataType = DTTT_UrlUrn;
		break;
	case BOOLEAN:
		*peDataType = DTTT_Boolean;
		break;
	default:
		*peDataType = DTTT_Invalid;
		break;
	}


}


void DBVsnacc_to_dbv_DbvResponse_list_comp(DbvResponse ***pppsResponse, Response *response, jmp_buf env)
{
	int num_elements = 0;

        /* c */
	num_elements = AsnListCount(response);
	*pppsResponse = (DbvResponse **) DbvAlloc (sizeof (DbvResponse *) * (num_elements + 1));
	CheckDbvAlloc(*pppsResponse, env);
	DBVsnacc_to_dbv_DbvResponse_list(*pppsResponse, response, env);
}



void DBVsnacc_to_dbv_DbvResponse_list(DbvResponse **ppsResponse, Response *response, jmp_buf env)
{
	int index = 0;

	ResponseSeq *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, response) {
		ppsResponse[index] = (DbvResponse *) DbvAlloc (sizeof(DbvResponse));
		CheckDbvAlloc(ppsResponse[index], env);
		DBVsnacc_to_dbv_DbvResponse(ppsResponse[index], data_ptr, env);
		index++;
	}
	ppsResponse[index] = NULL;
}



void DBVsnacc_to_dbv_DbvResponse(DbvResponse *psResponse, ResponseSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvPromptId(&psResponse->sPromptId, data_ptr->promptId, env);

	/* c */
	DBVsnacc_to_dbv_DbvPromptResponse(&psResponse->sPromptResponse, data_ptr->promptResponse, env);
}



void DBVsnacc_to_dbv_DbvPromptResponse(DbvPromptResponse *psPromptResponse, struct ResponseSeqChoice* promptResponse, jmp_buf env)
{
	switch (promptResponse->choiceId) {
	case RESPONSESEQCHOICE_STRING:
		psPromptResponse->eResponseType = RT_String;
		DBVsnacc_to_dbv_interstring_comp(&psPromptResponse->uResponse.psString, promptResponse->a.string, env);
		break;

	case RESPONSESEQCHOICE_ACCEPT:
		psPromptResponse->eResponseType = RT_Accept;
		DBVsnacc_to_dbv_bool(&psPromptResponse->uResponse.fAccept, &promptResponse->a.accept, env);
		break;

	case RESPONSESEQCHOICE_ACKNOWLEDGE:
		psPromptResponse->eResponseType = RT_Acknowledge;
		break;

	case RESPONSESEQCHOICE_DIAGNOSTIC:
		psPromptResponse->eResponseType = RT_Diagnostic;
		DBVsnacc_to_dbv_DbvDiagRec_comp(&psPromptResponse->uResponse.psDiagnostic, promptResponse->a.diagnostic, env);
		break;

	case RESPONSESEQCHOICE_ENCRYPTED:
		psPromptResponse->eResponseType = RT_Encrypted;
		DBVsnacc_to_dbv_DbvEncryption_comp(&psPromptResponse->uResponse.psEncryption, promptResponse->a.encrypted, env);
		break;

	default:
		psPromptResponse->eResponseType = RT_Invalid;
		break;
	}
}





/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/
/****************************************************************/

/* DbvPrintExtendedService */


/* dbv -> snacc ansi DbvPrintExtendedService external */

int DBVdbv_to_snacc_DbvPrintExtendedService_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static Print print_external[STRING_LEN];
 
        Print *print_ptr;

	/* Set up the DbvPrintExtendedService external */
	print_ptr = &print_external[current_print];

	/* Clear the location (may have been used by the last PDU) */
	memset(print_ptr, 0, sizeof(Print));

	DBVdbv_to_snacc_DbvPrintExtendedService_main(print_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psPrintExtendedService, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &print_external[current_print];

	/* Only if not at end of array */
	if (current_print < (STRING_LEN - 1)) {

		/* Increment current_print for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_print++;
	}
}



/* dbv -> snacc iso DbvPrintExtendedService external */

/* An iso oid for a private ANSI z39.50 external - don't be silly */





void DBVdbv_to_snacc_DbvPrintExtendedService_main(Print *print_ptr, DbvPrintExtendedService *psPrintExtendedService, jmp_buf env)
{

	switch (psPrintExtendedService->ePrintReqRsp) {
	case PROR_Request:
		print_ptr->choiceId = PRINT_ESREQUEST;
		DBVdbv_to_snacc_DbvPrintRequestPackage_comp(&print_ptr->a.esrequest, psPrintExtendedService->uPrintReqOrRsp.psPrintRequest, env);

		break;
	case PROR_Response:
		print_ptr->choiceId = PRINT_TASKPACKAGE;
		DBVdbv_to_snacc_DbvPrintTaskPackage_comp(&print_ptr->a.taskPackage, psPrintExtendedService->uPrintReqOrRsp.psPrintResponse, env);

		break;
	}
}


void DBVdbv_to_snacc_DbvPrintRequestPackage_comp(struct PrintSeq **esrequest, DbvPrintRequestPackage *psPrintRequest, jmp_buf env)
{
	*esrequest = (struct PrintSeq*) Asn1Alloc (sizeof (struct PrintSeq));
	CheckAsn1Alloc(*esrequest, env);
	DBVdbv_to_snacc_DbvPrintRequestPackage(*esrequest, psPrintRequest, env);
}



void DBVdbv_to_snacc_DbvPrintRequestPackage(struct PrintSeq* esrequest, DbvPrintRequestPackage *psPrintRequest, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvPrintOriginPartToKeep_opt(&esrequest->tokeep, psPrintRequest->psOriginPartToKeep, env);
}



void DBVdbv_to_snacc_DbvPrintOriginPartToKeep_opt(struct OriginPartToKeep9** tokeep, DbvPrintOriginPartToKeep *psOriginPartToKeep, jmp_buf env)
{
	if (psOriginPartToKeep) {
		*tokeep = (struct OriginPartToKeep9*) Asn1Alloc (sizeof (struct OriginPartToKeep9));
		CheckAsn1Alloc(*tokeep, env);
		DBVdbv_to_snacc_DbvPrintOriginPartToKeep(*tokeep, psOriginPartToKeep, env);
	}
}



void DBVdbv_to_snacc_DbvPrintOriginPartToKeep(struct OriginPartToKeep9* tokeep, DbvPrintOriginPartToKeep *psOriginPartToKeep, jmp_buf env)
{
	switch (psOriginPartToKeep->ePrintOriginPartToKeepType) {
	case POPKT_Create:
		tokeep->choiceId = ORIGINPARTTOKEEP9_CREATEOPTION;
		DBVdbv_to_snacc_DbvCreateOption_comp(&tokeep->a.createOption, psOriginPartToKeep->uPrintOriginPartToKeep.psCreateOption, env);
		break;

	case POPKT_Delete:
		tokeep->choiceId = ORIGINPARTTOKEEP9_DELETEOPTION;
		DBVdbv_to_snacc_DbvDeleteOption_comp(&tokeep->a.deleteOption, psOriginPartToKeep->uPrintOriginPartToKeep.psDeleteOption, env);

		break;
	}
	
}



void DBVdbv_to_snacc_DbvCreateOption_comp(struct CreateOption** createOption, DbvCreateOption *psCreateOption, jmp_buf env)
{
	*createOption = (struct CreateOption*) Asn1Alloc (sizeof (struct CreateOption));
	CheckAsn1Alloc(*createOption, env);
	DBVdbv_to_snacc_DbvCreateOption(*createOption, psCreateOption, env);
}



void DBVdbv_to_snacc_DbvCreateOption(struct CreateOption* createOption, DbvCreateOption *psCreateOption, jmp_buf env)
{
	switch (psCreateOption->eCreateOptionType) {
	case COPT_ResultSetReference:
		createOption->choiceId = CREATEOPTION_RESULTSETREFERENCE;
		DBVdbv_to_snacc_DbvResultSetReference_comp(&createOption->a.resultSetReference, psCreateOption->uCreateOption.psResultSetReference, env);
		break;

	case COPT_AccessionReference:
		createOption->choiceId = CREATEOPTION_ACCESSIONREFERENCE;
		DBVdbv_to_snacc_DbvAccessionReference_comp(&createOption->a.accessionReference, psCreateOption->uCreateOption.psAccessionReference, env);

		break;
	}
}
	
	

void DBVdbv_to_snacc_DbvResultSetReference_comp(struct ResultSetReference** resultSetReference, DbvResultSetReference *psResultSetReference, jmp_buf env)
{
	*resultSetReference = (struct ResultSetReference*) Asn1Alloc (sizeof (struct ResultSetReference));
	CheckAsn1Alloc(*resultSetReference, env);
	DBVdbv_to_snacc_DbvResultSetReference(*resultSetReference, psResultSetReference, env);
}



void DBVdbv_to_snacc_DbvResultSetReference(struct ResultSetReference* resultSetReference, DbvResultSetReference *psResultSetReference, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&resultSetReference->resultSetId, &psResultSetReference->sResultSetId, env);

	/* c */
	resultSetReference->resultSetStartPoint = psResultSetReference->ulResultSetStartPoint;
	/* c */
	resultSetReference->numberOfRecordsRequested = psResultSetReference->ulNumberOfRecordsRequested;

	/* o */
	DBVdbv_to_snacc_list_opt(&resultSetReference->additionalRanges, (void **) psResultSetReference->ppsAdditionalRanges, sizeof(Range), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvRange));

	/* o */
	DBVdbv_to_snacc_DbvRecordComposition_opt(&resultSetReference->recordComposition, psResultSetReference->psRecordComposition, env);

	/* c */
	DBVdbv_to_snacc_DbvAddress_comp(&resultSetReference->address, &psResultSetReference->sAddress, env);
}



void DBVdbv_to_snacc_DbvRange(Range *psRange, DbvRange *psDbvRange, jmp_buf env)
{
	/* c, c */
	psRange->startingPosition = psDbvRange->ulStartingPosition;
	psRange->numberOfRecords = psDbvRange->ulNumberOfRecords;

}



void DBVdbv_to_snacc_DbvRecordComposition_opt(struct ResultSetReferenceChoice** recordComposition, DbvRecordComposition *psRecordComposition, jmp_buf env)
{
	if (psRecordComposition) {
		*recordComposition = (struct ResultSetReferenceChoice*) Asn1Alloc (sizeof (struct ResultSetReferenceChoice));
		CheckAsn1Alloc(*recordComposition, env);
		DBVdbv_to_snacc_DbvRecordComposition((struct PresentRequestChoice*) *recordComposition, psRecordComposition, env);
	}
}



void DBVdbv_to_snacc_DbvAddress_comp(struct Address** address, DbvAddress *psAddress, jmp_buf env)
{
	*address = (struct Address*) Asn1Alloc (sizeof (struct Address));
	CheckAsn1Alloc(*address, env);
	DBVdbv_to_snacc_DbvAddress(*address, psAddress, env);
}



void DBVdbv_to_snacc_DbvAddress(struct Address* address, DbvAddress *psAddress, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&address->postalAddress, &psAddress->sPostalAddress, env);

	/* o */
	DBVdbv_to_snacc_DbvInternationalString(&address->emailAddress, psAddress->psEmailAddress, env);

}



void DBVdbv_to_snacc_DbvAccessionReference_comp(struct AccessionReference** accessionReference, DbvAccessionReference *psAccessionReference, jmp_buf env)
{
	*accessionReference = (struct AccessionReference*) Asn1Alloc (sizeof (struct AccessionReference));
	CheckAsn1Alloc(*accessionReference, env);
	DBVdbv_to_snacc_DbvAccessionReference(*accessionReference, psAccessionReference, env);
}



void DBVdbv_to_snacc_DbvAccessionReference(struct AccessionReference* accessionReference, DbvAccessionReference *psAccessionReference, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&accessionReference->referenceid, &psAccessionReference->sReferenceId, env);

	/* o */
	DBVdbv_to_snacc_DbvRecordComposition_opt((struct ResultSetReferenceChoice**) &accessionReference->recordComposition, psAccessionReference->psRecordComposition, env);

	/* c */
	DBVdbv_to_snacc_DbvAddress_comp(&accessionReference->address, &psAccessionReference->sAddress, env);
	
}



void DBVdbv_to_snacc_DbvDeleteOption_comp(DeleteOption** deleteOption, DbvDeleteOption *psDeleteOption, jmp_buf env)
{
	*deleteOption = (struct DeleteOption *) Asn1Alloc (sizeof (struct DeleteOption));
	CheckAsn1Alloc(*deleteOption, env);
	DBVdbv_to_snacc_DbvDeleteOption(*deleteOption, psDeleteOption, env);
}



void DBVdbv_to_snacc_DbvDeleteOption(DeleteOption* deleteOption, DbvDeleteOption *psDeleteOption, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&deleteOption->jobReference, &psDeleteOption->sJobReference, env);
}



void DBVdbv_to_snacc_DbvPrintTaskPackage_comp(struct PrintSeq1 **taskPackage, DbvPrintTaskPackage *psPrintResponse, jmp_buf env)
{
	*taskPackage = (struct PrintSeq1*) Asn1Alloc (sizeof (struct PrintSeq1));
	CheckAsn1Alloc(*taskPackage, env);
	DBVdbv_to_snacc_DbvPrintTaskPackage(*taskPackage, psPrintResponse, env);
}



void DBVdbv_to_snacc_DbvPrintTaskPackage(struct PrintSeq1 *taskPackage, DbvPrintTaskPackage *psPrintResponse, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvPrintOriginPartToKeep_opt(&taskPackage->originPart, psPrintResponse->psOriginPartToKeep, env);
	
	/* c */
	DBVdbv_to_snacc_DbvPrintTargetPartToKeep_comp(&taskPackage->targetPart, &psPrintResponse->sTargetPart, env);
}



void DBVdbv_to_snacc_DbvPrintTargetPartToKeep_comp(struct TargetPart9** targetPart, DbvPrintTargetPartToKeep *psTargetPart, jmp_buf env)
{
	*targetPart = (struct TargetPart9*) Asn1Alloc (sizeof (struct TargetPart9));
	CheckAsn1Alloc(*targetPart, env);
	DBVdbv_to_snacc_DbvPrintTargetPartToKeep(*targetPart, psTargetPart, env);
}



void DBVdbv_to_snacc_DbvPrintTargetPartToKeep(struct TargetPart9* targetPart, DbvPrintTargetPartToKeep *psTargetPart, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&targetPart->jobReference, &psTargetPart->sJobReference, env);
}



/****************************************************************/


/* snacc -> dbv ansi DbvPrintExtendedService external */

void DBVsnacc_to_dbv_DbvPrintExtendedService_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Print *print_ptr;

	print_ptr = (Print *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_Print;
	psSingleASN1Type->uASN1.psPrintExtendedService = (struct DbvPrintExtendedService *) DbvAlloc (sizeof(struct DbvPrintExtendedService));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psPrintExtendedService, env);
	DBVsnacc_to_dbv_DbvPrintExtendedService_main(psSingleASN1Type->uASN1.psPrintExtendedService, print_ptr, env);

}




/* snacc -> dbv iso DbvPrintExtendedService external */

/* there aint one */



void DBVsnacc_to_dbv_DbvPrintExtendedService_main(struct DbvPrintExtendedService *psPrintExtendedService, Print *print_ptr, jmp_buf env)
{
	switch (print_ptr->choiceId) {
	case PRINT_ESREQUEST:
		psPrintExtendedService->ePrintReqRsp = PROR_Request;
		DBVsnacc_to_dbv_DbvPrintRequestPackage_comp(&psPrintExtendedService->uPrintReqOrRsp.psPrintRequest, print_ptr->a.esrequest, env);
		break;

	case PRINT_TASKPACKAGE:
		psPrintExtendedService->ePrintReqRsp = PROR_Response;
		DBVsnacc_to_dbv_DbvPrintTaskPackage_comp(&psPrintExtendedService->uPrintReqOrRsp.psPrintResponse, print_ptr->a.taskPackage, env);
		break;
	default:
		psPrintExtendedService->ePrintReqRsp = PROR_Response;
		break;
	}
}



void DBVsnacc_to_dbv_DbvPrintRequestPackage_comp(DbvPrintRequestPackage **ppsPrintRequest, struct PrintSeq* esrequest, jmp_buf env)
{
	*ppsPrintRequest = (DbvPrintRequestPackage *) DbvAlloc (sizeof (DbvPrintRequestPackage));
	CheckDbvAlloc(*ppsPrintRequest, env);
	DBVsnacc_to_dbv_DbvPrintRequestPackage(*ppsPrintRequest, esrequest, env);
}



void DBVsnacc_to_dbv_DbvPrintRequestPackage(DbvPrintRequestPackage *psPrintRequest, struct PrintSeq* esrequest, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvPrintOriginPartToKeep_opt(&psPrintRequest->psOriginPartToKeep, esrequest->tokeep, env);
}



void DBVsnacc_to_dbv_DbvPrintOriginPartToKeep_opt(DbvPrintOriginPartToKeep **ppsOriginPartToKeep, struct OriginPartToKeep9* tokeep, jmp_buf env)
{
	if (tokeep) {
		*ppsOriginPartToKeep = (DbvPrintOriginPartToKeep *) DbvAlloc (sizeof (DbvPrintOriginPartToKeep));
		CheckDbvAlloc(*ppsOriginPartToKeep, env);
		DBVsnacc_to_dbv_DbvPrintOriginPartToKeep(*ppsOriginPartToKeep, tokeep, env);
	}
}



void DBVsnacc_to_dbv_DbvPrintOriginPartToKeep(DbvPrintOriginPartToKeep *psOriginPartToKeep, struct OriginPartToKeep9* tokeep, jmp_buf env)
{
	switch (tokeep->choiceId) {
	case ORIGINPARTTOKEEP9_CREATEOPTION:
		psOriginPartToKeep->ePrintOriginPartToKeepType = POPKT_Create;
		DBVsnacc_to_dbv_DbvCreateOption_comp(&psOriginPartToKeep->uPrintOriginPartToKeep.psCreateOption, tokeep->a.createOption, env);
		break;

	case ORIGINPARTTOKEEP9_DELETEOPTION:
		psOriginPartToKeep->ePrintOriginPartToKeepType = POPKT_Delete;
		DBVsnacc_to_dbv_DbvDeleteOption_comp(&psOriginPartToKeep->uPrintOriginPartToKeep.psDeleteOption, tokeep->a.deleteOption, env);
		break;
	default:
		psOriginPartToKeep->ePrintOriginPartToKeepType = POPKT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvCreateOption_comp(DbvCreateOption **ppsCreateOption, struct CreateOption* createOption, jmp_buf env)
{
	*ppsCreateOption = (DbvCreateOption *) DbvAlloc (sizeof (DbvCreateOption));
	CheckDbvAlloc(*ppsCreateOption, env);
	DBVsnacc_to_dbv_DbvCreateOption(*ppsCreateOption, createOption, env);
}



void DBVsnacc_to_dbv_DbvCreateOption(DbvCreateOption *psCreateOption, struct CreateOption* createOption, jmp_buf env)
{
	switch (createOption->choiceId) {
	case CREATEOPTION_RESULTSETREFERENCE:
		psCreateOption->eCreateOptionType = COPT_ResultSetReference;
		DBVsnacc_to_dbv_DbvResultSetReference_comp(&psCreateOption->uCreateOption.psResultSetReference, createOption->a.resultSetReference, env);
		break;

	case CREATEOPTION_ACCESSIONREFERENCE:
		psCreateOption->eCreateOptionType = COPT_AccessionReference;
		DBVsnacc_to_dbv_DbvAccessionReference_comp(&psCreateOption->uCreateOption.psAccessionReference, createOption->a.accessionReference, env);
		break;
	default:
		psCreateOption->eCreateOptionType = COPT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvResultSetReference_comp(DbvResultSetReference **ppsResultSetReference, struct ResultSetReference* resultSetReference, jmp_buf env)
{
	*ppsResultSetReference = (DbvResultSetReference *) DbvAlloc (sizeof (DbvResultSetReference));
	CheckDbvAlloc(*ppsResultSetReference, env);
	DBVsnacc_to_dbv_DbvResultSetReference(*ppsResultSetReference, resultSetReference, env);
}
	
	

void DBVsnacc_to_dbv_DbvResultSetReference(DbvResultSetReference *psResultSetReference, struct ResultSetReference* resultSetReference, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psResultSetReference->sResultSetId, &resultSetReference->resultSetId, env);

	/* c */
	psResultSetReference->ulResultSetStartPoint = resultSetReference->resultSetStartPoint;
	/* c */
	psResultSetReference->ulNumberOfRecordsRequested = resultSetReference->numberOfRecordsRequested;

	/* o */
	DBVsnacc_to_dbv_list_opt((void ***) &psResultSetReference->ppsAdditionalRanges, resultSetReference->additionalRanges, sizeof(DbvRange), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_DbvRange));

	/* o */
	DBVsnacc_to_dbv_DbvRecordComposition_opt(&psResultSetReference->psRecordComposition, resultSetReference->recordComposition, env);


	/* c */
	DBVsnacc_to_dbv_DbvAddress(&psResultSetReference->sAddress, resultSetReference->address, env);
	
}



void DBVsnacc_to_dbv_list_opt(void ***dbv_list, AsnList *snacclist, long dbvnodesize, jmp_buf env, void (*node_func) (void *, void *, jmp_buf))
{
	int num_elements = 0;

        /* o */
	if (snacclist) {

		/* Remember that we can have 0 length lists */ 
		num_elements = AsnListCount(snacclist);

		*dbv_list = DbvAlloc (sizeof (void *) * (num_elements + 1));
		CheckDbvAlloc(*dbv_list, env);
		DBVsnacc_to_dbv_list(*dbv_list, snacclist, dbvnodesize, env, node_func);
	}
}



void DBVsnacc_to_dbv_list(void **dbv_list, AsnList *snacclist, long dbvnodesize, jmp_buf env, void (*node_func) (void *, void *, jmp_buf)
)
{
	int index = 0;

	void *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, snacclist) {
		dbv_list[index] = (void *) DbvAlloc (dbvnodesize);
		CheckDbvAlloc(dbv_list[index], env);
		(*node_func) (dbv_list[index], data_ptr, env);
		index++;
	}
	dbv_list[index] = NULL;
}




void DBVsnacc_to_dbv_DbvRange(DbvRange *psAdditionalRanges, Range *data_ptr, jmp_buf env)
{
	/* c, c */
	psAdditionalRanges->ulStartingPosition = data_ptr->startingPosition;
	psAdditionalRanges->ulNumberOfRecords = data_ptr->numberOfRecords;
}



void DBVsnacc_to_dbv_DbvRecordComposition_opt(DbvRecordComposition **ppsRecordComposition, struct ResultSetReferenceChoice* recordComposition, jmp_buf env)
{
	if (recordComposition) {
		*ppsRecordComposition = (DbvRecordComposition *) DbvAlloc (sizeof (DbvRecordComposition));
		CheckDbvAlloc(*ppsRecordComposition, env);
		DBVsnacc_to_dbv_DbvRecordComposition(*ppsRecordComposition, (struct PresentRequestChoice*) recordComposition, env);
	}
}



void DBVsnacc_to_dbv_DbvAddress(DbvAddress *psAddress, struct Address* address, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psAddress->sPostalAddress, &address->postalAddress, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psAddress->psEmailAddress, &address->emailAddress, env);
}




void DBVsnacc_to_dbv_DbvAccessionReference_comp(DbvAccessionReference **ppsAccessionReference, struct AccessionReference* accessionReference, jmp_buf env)
{
	*ppsAccessionReference = (DbvAccessionReference *) DbvAlloc (sizeof (DbvAccessionReference));
	CheckDbvAlloc(*ppsAccessionReference, env);
	DBVsnacc_to_dbv_DbvAccessionReference(*ppsAccessionReference, accessionReference, env);
}



void DBVsnacc_to_dbv_DbvAccessionReference(DbvAccessionReference *psAccessionReference, struct AccessionReference* accessionReference, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psAccessionReference->sReferenceId, &accessionReference->referenceid, env);

	/* o */
	DBVsnacc_to_dbv_DbvRecordComposition_opt(&psAccessionReference->psRecordComposition, (struct ResultSetReferenceChoice*) accessionReference->recordComposition, env);

	/* c */
	DBVsnacc_to_dbv_DbvAddress(&psAccessionReference->sAddress, accessionReference->address, env);
}




void DBVsnacc_to_dbv_DbvDeleteOption_comp(DbvDeleteOption **ppsDeleteOption, struct DeleteOption* deleteOption, jmp_buf env)
{
	*ppsDeleteOption = (DbvDeleteOption *) DbvAlloc (sizeof (DbvDeleteOption));
	CheckDbvAlloc(*ppsDeleteOption, env);
	DBVsnacc_to_dbv_DbvDeleteOption(*ppsDeleteOption, deleteOption, env);
}



void DBVsnacc_to_dbv_DbvDeleteOption(DbvDeleteOption *psDeleteOption, struct DeleteOption* deleteOption, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psDeleteOption->sJobReference, &deleteOption->jobReference, env);

}



void DBVsnacc_to_dbv_DbvPrintTaskPackage_comp(DbvPrintTaskPackage **ppsPrintResponse, struct PrintSeq1* taskPackage, jmp_buf env)
{
	*ppsPrintResponse = (DbvPrintTaskPackage *) DbvAlloc (sizeof (DbvPrintTaskPackage));
	CheckDbvAlloc(*ppsPrintResponse, env);
	DBVsnacc_to_dbv_DbvPrintTaskPackage(*ppsPrintResponse, taskPackage, env);
}


void DBVsnacc_to_dbv_DbvPrintTaskPackage(DbvPrintTaskPackage *psPrintResponse, struct PrintSeq1* taskPackage, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvPrintOriginPartToKeep_opt(&psPrintResponse->psOriginPartToKeep, taskPackage->originPart, env);

	/* c */
	DBVsnacc_to_dbv_DbvPrintTargetPartToKeep(&psPrintResponse->sTargetPart, taskPackage->targetPart, env);
}



void DBVsnacc_to_dbv_DbvPrintTargetPartToKeep(DbvPrintTargetPartToKeep *psTargetPart, struct TargetPart9* targetPart, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psTargetPart->sJobReference, &targetPart->jobReference, env);

}



/****************************************************************/
/****************************************************************/


/* DbvAlertExtendedService */


/* dbv -> snacc ansi DbvAlertExtendedService external */

int DBVdbv_to_snacc_DbvAlertExtendedService_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{

	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static Alert alert_external[STRING_LEN];
 
        Alert *alert_ptr;

	/* Set up the DbvAlertExtendedService external */
	alert_ptr = &alert_external[current_alert];

	/* Clear the location (may have been used by the last PDU) */
	memset(alert_ptr, 0, sizeof(Alert));

	DBVdbv_to_snacc_DbvAlertExtendedService_main(alert_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psAlertExtendedService, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &alert_external[current_alert];

	/* Only if not at end of array */
	if (current_alert < (STRING_LEN - 1)) {

		/* Increment current_alert for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_alert++;
	}
}



/* dbv -> snacc iso DbvAlertExtendedService external */

/* An iso oid for a private ANSI z39.50 external - don't be silly */


void DBVdbv_to_snacc_DbvAlertExtendedService_main(Alert *alert_ptr, DbvAlertExtendedService *psAlertExtendedService, jmp_buf env)
{
	switch (psAlertExtendedService->eAlertReqRsp) {
	case AROR_Request:
		alert_ptr->choiceId = ALERT_ESREQUEST;
		DBVdbv_to_snacc_DbvAlertRequestPackage_comp(&alert_ptr->a.esrequest, psAlertExtendedService->uAlertReqOrRsp.psAlertRequest, env);
		break;

	case AROR_Response:
		alert_ptr->choiceId = ALERT_TASKPACKAGE;
		DBVdbv_to_snacc_DbvAlertTaskPackage_comp(&alert_ptr->a.taskPackage, psAlertExtendedService->uAlertReqOrRsp.psAlertResponse, env);
		break;
	}
}



void DBVdbv_to_snacc_DbvAlertRequestPackage_comp(struct AlertSeq **esrequest, DbvAlertRequestPackage *psAlertRequest, jmp_buf env)
{
	*esrequest = (struct AlertSeq*) Asn1Alloc (sizeof (struct AlertSeq));
	CheckAsn1Alloc(*esrequest, env);
	DBVdbv_to_snacc_DbvAlertRequestPackage(*esrequest, psAlertRequest, env);
}



void DBVdbv_to_snacc_DbvAlertRequestPackage(struct AlertSeq* esrequest, DbvAlertRequestPackage *psAlertRequest, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvAlertOriginPartToKeep_opt(&esrequest->tokeep, psAlertRequest->psOriginPartToKeep, env);
}



void DBVdbv_to_snacc_DbvAlertOriginPartToKeep_opt(struct OriginPartToKeep10** originPart, DbvAlertOriginPartToKeep *psOriginPartToKeep, jmp_buf env)
{
	if (psOriginPartToKeep) {
		*originPart = (struct OriginPartToKeep10*) Asn1Alloc (sizeof (struct OriginPartToKeep10));
		CheckAsn1Alloc(*originPart, env);
		DBVdbv_to_snacc_DbvAlertOriginPartToKeep(*originPart, psOriginPartToKeep, env);
	}
}



void DBVdbv_to_snacc_DbvAlertOriginPartToKeep(struct OriginPartToKeep10* originPart, DbvAlertOriginPartToKeep *psOriginPartToKeep, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvInternationalString(&originPart->alertName, &psOriginPartToKeep->sAlertName, env);

	/* o */
	DBVdbv_to_snacc_DbvAddress_opt(&originPart->address, psOriginPartToKeep->psAddress, env);
}



void DBVdbv_to_snacc_DbvAddress_opt(struct Address** address, DbvAddress *psAddress, jmp_buf env)
{
	if (psAddress) 
		DBVdbv_to_snacc_DbvAddress_comp(address, psAddress, env);
}



void DBVdbv_to_snacc_DbvAlertTaskPackage_comp(struct AlertSeq1 **taskPackage, DbvAlertTaskPackage *psAlertResponse, jmp_buf env)

{
	*taskPackage = (struct AlertSeq1*) Asn1Alloc (sizeof (struct AlertSeq1));
	CheckAsn1Alloc(*taskPackage, env);
	DBVdbv_to_snacc_DbvAlertTaskPackage(*taskPackage, psAlertResponse, env);
}



void DBVdbv_to_snacc_DbvAlertTaskPackage(struct AlertSeq1* taskPackage, DbvAlertTaskPackage *psAlertResponse, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvAlertOriginPartToKeep_opt(&taskPackage->originPart, psAlertResponse->psOriginPartToKeep, env);

}




/****************************************************************/


/* snacc -> dbv ansi DbvAlertExtendedService external */

void DBVsnacc_to_dbv_DbvAlertExtendedService_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Alert *alert_ptr;

	alert_ptr = (Alert *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_Alert;
	psSingleASN1Type->uASN1.psAlertExtendedService = (struct DbvAlertExtendedService *) DbvAlloc (sizeof(struct DbvAlertExtendedService));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psAlertExtendedService, env);
	DBVsnacc_to_dbv_DbvAlertExtendedService_main(psSingleASN1Type->uASN1.psAlertExtendedService, alert_ptr, env);

}




/* snacc -> dbv iso DbvAlertExtendedService external */

/* there aint one */



void DBVsnacc_to_dbv_DbvAlertExtendedService_main(DbvAlertExtendedService *psAlertExtendedService, Alert *alert_ptr, jmp_buf env)
{
	switch (alert_ptr->choiceId) {
	case ALERT_ESREQUEST:
		psAlertExtendedService->eAlertReqRsp = AROR_Request;
		DBVsnacc_to_dbv_DbvAlertRequestPackage_comp(&psAlertExtendedService->uAlertReqOrRsp.psAlertRequest, alert_ptr->a.esrequest, env);
		break;

	case ALERT_TASKPACKAGE:
		psAlertExtendedService->eAlertReqRsp = AROR_Response;
		DBVsnacc_to_dbv_DbvAlertTaskPackage_comp(&psAlertExtendedService->uAlertReqOrRsp.psAlertResponse, alert_ptr->a.taskPackage, env);
		break;
	default:
		psAlertExtendedService->eAlertReqRsp = AROR_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvAlertRequestPackage_comp(DbvAlertRequestPackage **ppsAlertRequest, struct AlertSeq* esrequest, jmp_buf env)
{
	*ppsAlertRequest = (DbvAlertRequestPackage *) DbvAlloc (sizeof (DbvAlertRequestPackage));
	CheckDbvAlloc(*ppsAlertRequest, env);
	DBVsnacc_to_dbv_DbvAlertRequestPackage(*ppsAlertRequest, esrequest, env);
}



void DBVsnacc_to_dbv_DbvAlertRequestPackage(DbvAlertRequestPackage *psAlertRequest, struct AlertSeq* esrequest, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvAlertOriginPartToKeep_opt(&psAlertRequest->psOriginPartToKeep, esrequest->tokeep, env);
	
}



void DBVsnacc_to_dbv_DbvAlertOriginPartToKeep_opt(DbvAlertOriginPartToKeep **ppsOriginPartToKeep, struct OriginPartToKeep10* esrequest, jmp_buf env)
{
	if (esrequest) {
		*ppsOriginPartToKeep = (DbvAlertOriginPartToKeep *) DbvAlloc (sizeof (DbvAlertOriginPartToKeep));
		CheckDbvAlloc(*ppsOriginPartToKeep, env);
		DBVsnacc_to_dbv_DbvAlertOriginPartToKeep(*ppsOriginPartToKeep, esrequest, env);
	}
}



void DBVsnacc_to_dbv_DbvAlertOriginPartToKeep(DbvAlertOriginPartToKeep *psOriginPartToKeep, struct OriginPartToKeep10* originPart, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvInternationalString(&psOriginPartToKeep->sAlertName, &originPart->alertName, env);

	/* o */
	DBVsnacc_to_dbv_DbvAddress_opt(&psOriginPartToKeep->psAddress, originPart->address, env);
}



void DBVsnacc_to_dbv_DbvAddress_opt(DbvAddress **ppsAddress, struct Address* address, jmp_buf env)
{
	if (address) {
		*ppsAddress = (DbvAddress *) DbvAlloc (sizeof (DbvAddress));
		CheckDbvAlloc(*ppsAddress, env);
		DBVsnacc_to_dbv_DbvAddress(*ppsAddress, address, env);
	}
}



void DBVsnacc_to_dbv_DbvAlertTaskPackage_comp(DbvAlertTaskPackage **ppsAlertResponse, struct AlertSeq1* taskPackage, jmp_buf env)
{
	*ppsAlertResponse = (DbvAlertTaskPackage *) DbvAlloc (sizeof (DbvAlertTaskPackage));
	CheckDbvAlloc(*ppsAlertResponse, env);
	DBVsnacc_to_dbv_DbvAlertTaskPackage(*ppsAlertResponse, taskPackage, env);
}



void DBVsnacc_to_dbv_DbvAlertTaskPackage(DbvAlertTaskPackage *psAlertResponse, struct AlertSeq1* taskPackage, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvAlertOriginPartToKeep_opt(&psAlertResponse->psOriginPartToKeep, taskPackage->originPart, env);
}




/****************************************************************/
/****************************************************************/


/* DbvCOrder */


/* dbv -> snacc ansi DbvCOrder external (this is an external to
 be carried in the ItemOrder extended service, just like ILL) */

int DBVdbv_to_snacc_DbvCOrder_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{

	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static Corder corder_external[STRING_LEN];
 
        Corder *corder_ptr;

	/* Set up the DbvAlertExtendedService external */
	corder_ptr = &corder_external[current_corder];

	/* Clear the location (may have been used by the last PDU) */
	memset(corder_ptr, 0, sizeof(Corder));

	DBVdbv_to_snacc_DbvCOrder_main(corder_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psCOrder, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &corder_external[current_corder];

	/* Only if not at end of array */
	if (current_corder < (STRING_LEN - 1)) {

		/* Increment current_corder for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_corder++;
	}
}



/* dbv -> snacc iso DbvCOrder external */

/* An iso oid for a private ANSI z39.50 external - don't be silly */



void DBVdbv_to_snacc_DbvCOrder_main(Corder *corder_ptr, DbvCOrder *psCOrder, jmp_buf env)
{
	switch (psCOrder->eCOrderType) {
	case COT_OrderRequest:
		corder_ptr->choiceId = CORDER_ORDERREQUEST;
		DBVdbv_to_snacc_DbvOrderRequest_comp(&corder_ptr->a.orderRequest, psCOrder->uCOrder.psOrderRequest, env);
		break;

	case COT_OrderResponse:
		corder_ptr->choiceId = CORDER_ORDERRESPONSE;
		DBVdbv_to_snacc_interstring_list_opt(&corder_ptr->a.orderResponse, psCOrder->uCOrder.ppsOrderResponse, env);
		break;
	}
}



void DBVdbv_to_snacc_DbvOrderRequest_comp(struct OrderRequest **orderRequest, DbvOrderRequest *psOrderRequest, jmp_buf env)
{
	*orderRequest = (struct OrderRequest*) Asn1Alloc (sizeof (struct OrderRequest));
	CheckAsn1Alloc(*orderRequest, env);
	DBVdbv_to_snacc_DbvOrderRequest(*orderRequest, psOrderRequest, env);
}



void DBVdbv_to_snacc_DbvOrderRequest(struct OrderRequest *orderRequest, DbvOrderRequest *psOrderRequest, jmp_buf env)
{
	switch (psOrderRequest->eOrderRequestType) {
	case ORT_NormalOrder:
		orderRequest->choiceId = ORDERREQUEST_NORMALORDER;
		DBVdbv_to_snacc_DbvNormalOrder_comp(&orderRequest->a.normalOrder, psOrderRequest->uOrderRequest.psNormalOrder, env);
		break;

	case ORT_OrderRefs:
		orderRequest->choiceId = ORDERREQUEST_ORDERREFS;
		DBVdbv_to_snacc_interstring_list_opt(&orderRequest->a.orderRefs, psOrderRequest->uOrderRequest.ppsOrderRefs, env);
		break;
	}
}



void DBVdbv_to_snacc_DbvNormalOrder_comp(struct NormalOrder** normalOrder, DbvNormalOrder *psNormalOrder, jmp_buf env)
{
	*normalOrder = (struct NormalOrder*) Asn1Alloc (sizeof (struct NormalOrder));
	CheckAsn1Alloc(*normalOrder, env);
	DBVdbv_to_snacc_DbvNormalOrder(*normalOrder, psNormalOrder, env);

}



void DBVdbv_to_snacc_DbvNormalOrder(struct NormalOrder* normalOrder, DbvNormalOrder *psNormalOrder, jmp_buf env)
{
	/* c, c */
	DBVdbv_to_snacc_list_opt(&normalOrder->orderRanges, (void **) psNormalOrder->ppsOrderRanges, sizeof(Range), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvRange));
	
	normalOrder->quantity = psNormalOrder->ulQuantity;
}



/****************************************************************/


/* snacc -> dbv ansi DbvCOrder external */

void DBVsnacc_to_dbv_DbvCOrder_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Corder *corder_ptr;

	corder_ptr = (Corder *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_Corder;
	psSingleASN1Type->uASN1.psCOrder = (struct DbvCOrder *) DbvAlloc (sizeof(struct DbvCOrder));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psCOrder, env);
	DBVsnacc_to_dbv_DbvCOrder_main(psSingleASN1Type->uASN1.psCOrder, corder_ptr, env);

}




/* snacc -> dbv iso DbvCOrder external */

/* there aint one */



void DBVsnacc_to_dbv_DbvCOrder_main(DbvCOrder *psCOrder, Corder *corder_ptr, jmp_buf env)
{
	switch (corder_ptr->choiceId) {
	case CORDER_ORDERREQUEST:
		psCOrder->eCOrderType = COT_OrderRequest;
		DBVsnacc_to_dbv_DbvOrderRequest_comp(&psCOrder->uCOrder.psOrderRequest, corder_ptr->a.orderRequest, env);
		break;

	case CORDER_ORDERRESPONSE:
		psCOrder->eCOrderType = COT_OrderResponse;
		DBVsnacc_to_dbv_interstring_list_opt(&psCOrder->uCOrder.ppsOrderResponse, corder_ptr->a.orderResponse, env);
		break;
	default:
		psCOrder->eCOrderType = COT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvOrderRequest_comp(DbvOrderRequest **ppsOrderRequest, struct OrderRequest* orderRequest, jmp_buf env)
{
	*ppsOrderRequest = (DbvOrderRequest *) DbvAlloc (sizeof (DbvOrderRequest));
	CheckDbvAlloc(*ppsOrderRequest, env);
	DBVsnacc_to_dbv_DbvOrderRequest(*ppsOrderRequest, orderRequest, env);
}



void DBVsnacc_to_dbv_DbvOrderRequest(DbvOrderRequest *psOrderRequest, struct OrderRequest* orderRequest, jmp_buf env)
{
	switch (orderRequest->choiceId) {
	case ORDERREQUEST_NORMALORDER:
		psOrderRequest->eOrderRequestType = ORT_NormalOrder;
		DBVsnacc_to_dbv_DbvNormalOrder_comp(&psOrderRequest->uOrderRequest.psNormalOrder, orderRequest->a.normalOrder, env);
		break;

	case ORDERREQUEST_ORDERREFS:
		psOrderRequest->eOrderRequestType = ORT_OrderRefs;
		DBVsnacc_to_dbv_interstring_list_opt(&psOrderRequest->uOrderRequest.ppsOrderRefs, orderRequest->a.orderRefs, env);
		break;
	default:
		psOrderRequest->eOrderRequestType = ORT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvNormalOrder_comp(DbvNormalOrder **ppsNormalOrder, struct NormalOrder* normalOrder, jmp_buf env)
{
	*ppsNormalOrder = (DbvNormalOrder *) DbvAlloc (sizeof (DbvNormalOrder));
	CheckDbvAlloc(*ppsNormalOrder, env);
	DBVsnacc_to_dbv_DbvNormalOrder(*ppsNormalOrder, normalOrder, env);
}



void DBVsnacc_to_dbv_DbvNormalOrder(DbvNormalOrder *psNormalOrder, struct NormalOrder* normalOrder, jmp_buf env)
{
	/* c, c */
	DBVsnacc_to_dbv_list_comp((void ***) &psNormalOrder->ppsOrderRanges, normalOrder->orderRanges, sizeof(DbvRange), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_DbvRange));

	psNormalOrder->ulQuantity = normalOrder->quantity;
}



void DBVsnacc_to_dbv_list_comp(void ***dbv_list, AsnList *snacclist, long dbvnodesize, jmp_buf env, void (*node_func) (void *, void *, jmp_buf))
{
	int num_elements = 0;

	num_elements = AsnListCount(snacclist);

	*dbv_list = DbvAlloc (sizeof (void *) * (num_elements + 1));
	CheckDbvAlloc(*dbv_list, env);
	DBVsnacc_to_dbv_list(*dbv_list, snacclist, dbvnodesize, env, node_func);
}





/* dbv -> snacc ansi DbvCharSetAndLanguage external */

int DBVdbv_to_snacc_DbvCharSetAndLanguage_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static CharSetandLanguageNegotiation charsetandlang_external[STRING_LEN];
 
        CharSetandLanguageNegotiation *charsetandlang_ptr;

	/* Set up the PromptObject external */
	charsetandlang_ptr = &charsetandlang_external[current_charsetandlang];

	/* Clear the location (may have been used by the last PDU) */
	memset(charsetandlang_ptr, 0, sizeof(CharSetandLanguageNegotiation));

	DBVdbv_to_snacc_DbvCharSetAndLanguage_main(charsetandlang_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psCharSetAndLanguage, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &charsetandlang_external[current_charsetandlang];

	/* Only if not at end of array */
	if (current_charsetandlang < (STRING_LEN - 1)) {

		/* Increment current_charsetandlang for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_charsetandlang++;
	}
}



/* dbv -> snacc iso DbvPromptObject external */

int DBVdbv_to_snacc_isoDbvCharSetAndLanguage_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoCharSetandLanguageNegotiation isocharsetandlang_external[STRING_LEN];
        IsoCharSetandLanguageNegotiation *isocharsetandlang_ptr;

	isocharsetandlang_ptr = &isocharsetandlang_external[current_isocharsetandlang];
	memset(isocharsetandlang_ptr, 0, sizeof(IsoCharSetandLanguageNegotiation));
	DBVdbv_to_snacc_DbvCharSetAndLanguage_main(isocharsetandlang_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psCharSetAndLanguage, env);
	any_type->value = (void *) &isocharsetandlang_external[current_isocharsetandlang];

	if (current_isocharsetandlang < (STRING_LEN - 1)) {
		current_isocharsetandlang++;
	}
}



void DBVdbv_to_snacc_DbvCharSetAndLanguage_main(CharSetandLanguageNegotiation *charsetandlang_ptr, struct DbvCharSetAndLanguage *psCharSetAndLanguage, jmp_buf env)
{
	switch (psCharSetAndLanguage->eCharSetAndLanguageType) {
	case CSALT_Proposal:
		charsetandlang_ptr->choiceId = CHARSETANDLANGUAGENEGOTIATION_PROPOSAL;
		DBVdbv_to_snacc_DbvOriginProposal_comp(&charsetandlang_ptr->a.proposal, psCharSetAndLanguage->uCharSetAndLanguage.psProposal, env);

		break;
	case CSALT_Response:
		charsetandlang_ptr->choiceId = CHARSETANDLANGUAGENEGOTIATION_RESPONSE;
		DBVdbv_to_snacc_DbvTargetResponse_comp(&charsetandlang_ptr->a.response, psCharSetAndLanguage->uCharSetAndLanguage.psResponse, env);

		break;
	}
}



void DBVdbv_to_snacc_DbvOriginProposal_comp(struct OriginProposal** proposal, DbvOriginProposal *psProposal, jmp_buf env)
{
	*proposal = (struct OriginProposal*) Asn1Alloc (sizeof (struct OriginProposal));
	CheckAsn1Alloc(*proposal, env);
	DBVdbv_to_snacc_DbvOriginProposal(*proposal, psProposal, env);


}



void DBVdbv_to_snacc_DbvOriginProposal(struct OriginProposal* proposal, DbvOriginProposal *psProposal, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_list_opt(&proposal->proposedCharSets, (void **) psProposal->ppsProposedCharSets, sizeof(OriginProposalSeqOfChoice), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvProposedCharSet));

	/* o */
	DBVdbv_to_snacc_list_opt(&proposal->proposedlanguages, (void **) psProposal->ppsProposedLanguages, sizeof(LanguageCode), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvLanguageCode));
	
}



void DBVdbv_to_snacc_DbvProposedCharSet(OriginProposalSeqOfChoice *proposedCharSets, DbvProposedCharSet *psProposedCharSets, jmp_buf env)
{
	switch (psProposedCharSets->eProposedCharSetType) {
	case PRCST_Iso2022:
		proposedCharSets->choiceId = ORIGINPROPOSALSEQOFCHOICE_ISO2022;
		DBVdbv_to_snacc_DbvIso2022_comp(&proposedCharSets->a.iso2022, psProposedCharSets->uProposedCharSet.psIso2022, env);

		break;
	case PRCST_Iso10646:
		proposedCharSets->choiceId = ORIGINPROPOSALSEQOFCHOICE_ISO10646;
		DBVdbv_to_snacc_DbvIso10646_comp(&proposedCharSets->a.iso10646, psProposedCharSets->uProposedCharSet.psIso10646, env);

		break;
	case PRCST_Private:
		proposedCharSets->choiceId = ORIGINPROPOSALSEQOFCHOICE_PRIVATE;
		DBVdbv_to_snacc_DbvPrivateCharacterSet_comp(&proposedCharSets->a.private, psProposedCharSets->uProposedCharSet.psPrivate, env);

		break;
	}
}



void DBVdbv_to_snacc_DbvIso2022_comp(struct Iso2022** iso2022, DbvIso2022 *psIso2022, jmp_buf env)
{
	*iso2022 = (struct Iso2022*) Asn1Alloc (sizeof (struct Iso2022));
	CheckAsn1Alloc(*iso2022, env);
	DBVdbv_to_snacc_DbvIso2022(*iso2022, psIso2022, env);

}



void DBVdbv_to_snacc_DbvIso2022(struct Iso2022* iso2022, DbvIso2022 *psIso2022, jmp_buf env)
{
	switch (psIso2022->eIso2022Type) {
	case I2022T_OriginProposal:
		iso2022->choiceId = ISO2022_ORIGINPROPOSAL;
		DBVdbv_to_snacc_DbvIso2022Proposal_comp(&iso2022->a.originProposal, psIso2022->uIso2022Type.psOriginProposal, env);

		break;
	case I2022T_TargetResponse:
		iso2022->choiceId = ISO2022_TARGETRESPONSE;
		DBVdbv_to_snacc_DbvIso2022Response_comp(&iso2022->a.targetResponse, psIso2022->uIso2022Type.psTargetResponse, env);

		break;
	}
}



void DBVdbv_to_snacc_DbvIso2022Proposal_comp(struct Iso2022Seq** originProposal, DbvIso2022Proposal *psOriginProposal, jmp_buf env)
{
	*originProposal = (struct Iso2022Seq*) Asn1Alloc (sizeof (struct Iso2022Seq));
	CheckAsn1Alloc(*originProposal, env);
	DBVdbv_to_snacc_DbvIso2022Proposal(*originProposal, psOriginProposal, env);
}



void DBVdbv_to_snacc_DbvIso2022Proposal(struct Iso2022Seq* originProposal, DbvIso2022Proposal *psOriginProposal, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvEnvironment_opt(&originProposal->proposedEnvironment, psOriginProposal->psProposedEnvironment, env);

	/* c */
	DBVdbv_to_snacc_list_opt(&originProposal->proposedSets, (void **) psOriginProposal->ppulProposedSets, sizeof(AsnInt), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_Ulong));

	/* c */
	DBVdbv_to_snacc_list_opt(&originProposal->proposedInitialSets, (void **) psOriginProposal->ppsProposedInitialSets, sizeof(InitialSet), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvInitialSet));

	/* c */
	DBVdbv_to_snacc_DbvLeftAndRight_comp(&originProposal->proposedLeftAndRight, psOriginProposal->psProposedLeftAndRight, env);
}



void DBVdbv_to_snacc_DbvEnvironment_opt(struct Environment** proposedEnvironment, DbvEnvironment *psProposedEnvironment, jmp_buf env)
{
	if (psProposedEnvironment) {
		*proposedEnvironment = (struct Environment*) Asn1Alloc (sizeof (struct Environment));
		CheckAsn1Alloc(*proposedEnvironment, env);
		DBVdbv_to_snacc_DbvEnvironment(*proposedEnvironment, psProposedEnvironment, env);
	}
}




void DBVdbv_to_snacc_DbvEnvironment(struct Environment* proposedEnvironment, DbvEnvironment *psEnvironment, jmp_buf env)
{
	switch (psEnvironment->eEnvironmentType) {
	case ENT_SevenBit:
		proposedEnvironment->choiceId = ENVIRONMENT_SEVENBIT;
		break;
	case ENT_EightBit:
		proposedEnvironment->choiceId = ENVIRONMENT_EIGHTBIT;
		break;
	}
}



void DBVdbv_to_snacc_DbvInitialSet(struct InitialSet* proposedInitialSets, DbvInitialSet *psProposedInitialSets, jmp_buf env)
{
	/* 6 c */
	proposedInitialSets->g0 = psProposedInitialSets->ulG0;
	proposedInitialSets->g1 = psProposedInitialSets->ulG1;
	proposedInitialSets->g2 = psProposedInitialSets->ulG2;
	proposedInitialSets->g3 = psProposedInitialSets->ulG3;
	proposedInitialSets->c0 = psProposedInitialSets->ulC0;
	proposedInitialSets->c1 = psProposedInitialSets->ulC1;
}



void DBVdbv_to_snacc_DbvLeftAndRight_comp(struct LeftAndRight** proposedLeftAndRight, DbvLeftAndRight *psProposedLeftAndRight, jmp_buf env)
{
	*proposedLeftAndRight = (struct LeftAndRight*) Asn1Alloc (sizeof (struct LeftAndRight));
	CheckAsn1Alloc(*proposedLeftAndRight, env);
	DBVdbv_to_snacc_DbvLeftAndRight(*proposedLeftAndRight, psProposedLeftAndRight, env);
}




void DBVdbv_to_snacc_DbvLeftAndRight(struct LeftAndRight* proposedLeftAndRight, DbvLeftAndRight *psProposedLeftAndRight, jmp_buf env)
{
	/* c */
	proposedLeftAndRight->gLeft = psProposedLeftAndRight->ulGLeft;


	/* c */
	proposedLeftAndRight->gRight = psProposedLeftAndRight->ulGRight;
}



void DBVdbv_to_snacc_DbvIso2022Response_comp(struct Iso2022Seq1** targetResponse, DbvIso2022Response *psTargetResponse, jmp_buf env)
{
	*targetResponse = (struct Iso2022Seq1*) Asn1Alloc (sizeof (struct Iso2022Seq1));
	CheckAsn1Alloc(*targetResponse, env);
	DBVdbv_to_snacc_DbvIso2022Response(*targetResponse, psTargetResponse, env);
}




void DBVdbv_to_snacc_DbvIso2022Response(struct Iso2022Seq1* targetResponse, DbvIso2022Response *psTargetResponse, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvEnvironment_comp(&targetResponse->selectedEnvironment, psTargetResponse->psSelectedEnvironment, env);

	/* c */
	DBVdbv_to_snacc_list_opt(&targetResponse->selectedSets, (void **) psTargetResponse->ppulSelectedSets, sizeof(AsnInt), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_Ulong));

	/* c */
	DBVdbv_to_snacc_DbvInitialSet_comp(&targetResponse->selectedinitialSet, psTargetResponse->psSelectedInitialSet, env);

	/* c */
	DBVdbv_to_snacc_DbvLeftAndRight_comp(&targetResponse->selectedLeftAndRight, psTargetResponse->psSelectedLeftAndRight, env);
}



void DBVdbv_to_snacc_DbvEnvironment_comp(struct Environment** selectedEnvironment, DbvEnvironment *psSelectedEnvironment, jmp_buf env)
{
	DBVdbv_to_snacc_DbvEnvironment_opt(selectedEnvironment, psSelectedEnvironment, env);
}



void DBVdbv_to_snacc_DbvInitialSet_comp(struct InitialSet** selectedinitialSet, DbvInitialSet *psSelectedInitialSet, jmp_buf env)
{
	*selectedinitialSet = (struct InitialSet*) Asn1Alloc (sizeof (struct InitialSet));
	CheckAsn1Alloc(*selectedinitialSet, env);
	DBVdbv_to_snacc_DbvInitialSet(*selectedinitialSet, psSelectedInitialSet, env);

}



void DBVdbv_to_snacc_DbvIso10646_comp(struct Iso10646** iso10646, DbvIso10646 *psIso10646, jmp_buf env)
{
	*iso10646 = (struct Iso10646*) Asn1Alloc (sizeof (struct Iso10646));
	CheckAsn1Alloc(*iso10646, env);
	DBVdbv_to_snacc_DbvIso10646(*iso10646, psIso10646, env);

}



void DBVdbv_to_snacc_DbvIso10646(struct Iso10646* iso10646, DbvIso10646 *psIso10646, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_DbvObjectIdentifier(&iso10646->collections, psIso10646->pcCollections, env);

	/* c */
	DBVdbv_to_snacc_DbvObjectIdentifier(&iso10646->encodingLevel, psIso10646->pcEncodingLevel, env);
}



void DBVdbv_to_snacc_DbvPrivateCharacterSet_comp(struct PrivateCharacterSet** private, DbvPrivateCharacterSet *psPrivate, jmp_buf env)
{
	*private = (struct PrivateCharacterSet*) Asn1Alloc (sizeof (struct PrivateCharacterSet));
	CheckAsn1Alloc(*private, env);
	DBVdbv_to_snacc_DbvPrivateCharacterSet(*private, psPrivate, env);
}



void DBVdbv_to_snacc_DbvPrivateCharacterSet(struct PrivateCharacterSet* private, DbvPrivateCharacterSet *psPrivate, jmp_buf env)
{
	switch (psPrivate->ePrivateCharacterSetType) {
	case PCST_ViaOid:
		private->choiceId = PRIVATECHARACTERSET_VIAOID_SNACC;
		DBVdbv_to_snacc_list_comp(&private->a.viaOid, (void **) psPrivate->uPrivateCharacterSet.ppcViaOid, sizeof(AsnOid), env, (void (*) (void*, void*, jmp_buf)) (DBVdbv_to_snacc_DbvObjectIdentifier));
		break;
	case PCST_ExternallySpecified:
		private->choiceId = PRIVATECHARACTERSET_EXTERNALLYSPECIFIED;
		DBVdbv_to_snacc_DbvASN1External_comp(&private->a.externallySpecified, psPrivate->uPrivateCharacterSet.psExternallySpecified, env);

		break;
	case PCST_PreviouslyAgreedUpon:
		private->choiceId = PRIVATECHARACTERSET_PREVIOUSLYAGREEDUPON;
		/* Nothing required here */

		break;
	}
}



void DBVdbv_to_snacc_DbvLanguageCode(LanguageCode *proposedlanguages, DbvLanguageCode *psProposedLangauages, jmp_buf env)
{
	DBVdbv_to_snacc_DbvInternationalString(proposedlanguages, psProposedLangauages, env);
}





void DBVdbv_to_snacc_DbvTargetResponse_comp(struct TargetResponse** response, DbvTargetResponse *psResponse, jmp_buf env)
{
	*response = (struct TargetResponse*) Asn1Alloc (sizeof (struct TargetResponse));
	CheckAsn1Alloc(*response, env);
	DBVdbv_to_snacc_DbvTargetResponse(*response, psResponse, env);

}



void DBVdbv_to_snacc_DbvTargetResponse(struct TargetResponse* response, DbvTargetResponse *psResponse, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_DbvSelectedCharSet_opt(&response->selectedCharSets, psResponse->psSelectedCharSets, env);

	/* o */
	DBVdbv_to_snacc_DbvGeneralString_opt(&response->selectedLanguage, psResponse->psSelectedLanguage, env);
}



void DBVdbv_to_snacc_DbvSelectedCharSet_opt(struct TargetResponseChoice** selectedCharSets, DbvSelectedCharSet *psSelectedCharSets, jmp_buf env)
{
	if (psSelectedCharSets) {
		*selectedCharSets = (struct TargetResponseChoice*) Asn1Alloc (sizeof (struct TargetResponseChoice));
		CheckAsn1Alloc(*selectedCharSets, env);
		DBVdbv_to_snacc_DbvSelectedCharSet(*selectedCharSets, psSelectedCharSets, env);
	}
}



void DBVdbv_to_snacc_DbvSelectedCharSet(struct TargetResponseChoice* selectedCharSets, DbvSelectedCharSet *psSelectedCharSets, jmp_buf env)
{
	switch (psSelectedCharSets->eSelectedCharSetType) {
	case SCST_Iso2022:
		selectedCharSets->choiceId = TARGETRESPONSECHOICE_ISO2022;
		DBVdbv_to_snacc_DbvIso2022_comp(&selectedCharSets->a.iso2022, psSelectedCharSets->uSelectedCharSet.psIso2022, env);
		break;
	case SCST_Iso10646:
		selectedCharSets->choiceId = TARGETRESPONSECHOICE_ISO10646;
		DBVdbv_to_snacc_DbvIso10646_comp(&selectedCharSets->a.iso10646, psSelectedCharSets->uSelectedCharSet.psIso10646, env);

		break;
	case SCST_Private:
		selectedCharSets->choiceId = TARGETRESPONSECHOICE_PRIVATE;
		DBVdbv_to_snacc_DbvPrivateCharacterSet_comp(&selectedCharSets->a.private, psSelectedCharSets->uSelectedCharSet.psPrivate, env);
		break;
	case SCST_None:
		selectedCharSets->choiceId = TARGETRESPONSECHOICE_NONE;

		/* nothing else need to be done here */
		break;
	}
}




/****************************************************************/


/* snacc -> dbv ansi DbvCharSetAndLanguage external */

void DBVsnacc_to_dbv_DbvCharSetAndLanguage_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	CharSetandLanguageNegotiation *charsetandlang_ptr;

	charsetandlang_ptr = (CharSetandLanguageNegotiation *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_CharSetAndLanguage;
	psSingleASN1Type->uASN1.psCharSetAndLanguage = (struct DbvCharSetAndLanguage *) DbvAlloc (sizeof(struct DbvCharSetAndLanguage));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psCharSetAndLanguage, env);

	DBVsnacc_to_dbv_DbvCharSetAndLanguage_main(psSingleASN1Type->uASN1.psCharSetAndLanguage, charsetandlang_ptr, env);

}



/* snacc -> dbv iso DbvCharSetAndLanguage external */

void DBVsnacc_to_dbv_isoDbvCharSetAndLanguage_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoCharSetandLanguageNegotiation *isocharsetandlang_ptr;

	isocharsetandlang_ptr = (IsoCharSetandLanguageNegotiation *) any_type->value;

	psSingleASN1Type->eASN1Type = AT_CharSetAndLanguage;
	psSingleASN1Type->uASN1.psCharSetAndLanguage = (struct DbvCharSetAndLanguage *) DbvAlloc (sizeof(struct DbvCharSetAndLanguage));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psCharSetAndLanguage, env);

	DBVsnacc_to_dbv_DbvCharSetAndLanguage_main(psSingleASN1Type->uASN1.psCharSetAndLanguage, isocharsetandlang_ptr, env);

}



void DBVsnacc_to_dbv_DbvCharSetAndLanguage_main(struct DbvCharSetAndLanguage *psCharSetAndLanguage, CharSetandLanguageNegotiation *charsetandlang_ptr, jmp_buf env)
{
	switch (charsetandlang_ptr->choiceId) {
	case CHARSETANDLANGUAGENEGOTIATION_PROPOSAL:
		psCharSetAndLanguage->eCharSetAndLanguageType = CSALT_Proposal;
		DBVsnacc_to_dbv_DbvOriginProposal_comp(&psCharSetAndLanguage->uCharSetAndLanguage.psProposal, charsetandlang_ptr->a.proposal, env);
		break;
	case CHARSETANDLANGUAGENEGOTIATION_RESPONSE:
		psCharSetAndLanguage->eCharSetAndLanguageType = CSALT_Response;
		DBVsnacc_to_dbv_DbvTargetResponse_comp(&psCharSetAndLanguage->uCharSetAndLanguage.psResponse, charsetandlang_ptr->a.response, env);
		break;
	default:
		psCharSetAndLanguage->eCharSetAndLanguageType = CSALT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvOriginProposal_comp(DbvOriginProposal **ppsProposal, struct OriginProposal* proposal, jmp_buf env)
{
	*ppsProposal = (DbvOriginProposal *) DbvAlloc(sizeof (DbvOriginProposal));
	CheckDbvAlloc(*ppsProposal, env);
	DBVsnacc_to_dbv_DbvOriginProposal(*ppsProposal, proposal, env);
}



void DBVsnacc_to_dbv_DbvOriginProposal(DbvOriginProposal *psProposal, struct OriginProposal* proposal, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_list_opt((void ***) &psProposal->ppsProposedCharSets, proposal->proposedCharSets, sizeof(DbvProposedCharSet), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_DbvProposedCharSet));

	/* o */
	DBVsnacc_to_dbv_list_opt((void ***) &psProposal->ppsProposedLanguages, proposal->proposedlanguages, sizeof(DbvLanguageCode), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_DbvLanguageCode));
}



void DBVsnacc_to_dbv_DbvProposedCharSet(DbvProposedCharSet *psProposedCharSets, struct OriginProposalSeqOfChoice* proposal, jmp_buf env)
{
	switch (proposal->choiceId) {
	case ORIGINPROPOSALSEQOFCHOICE_ISO2022:
		psProposedCharSets->eProposedCharSetType = PRCST_Iso2022;
		DBVsnacc_to_dbv_DbvIso2022_comp(&psProposedCharSets->uProposedCharSet.psIso2022, proposal->a.iso2022, env);
		break;
	case ORIGINPROPOSALSEQOFCHOICE_ISO10646:
		psProposedCharSets->eProposedCharSetType = PRCST_Iso10646;
		DBVsnacc_to_dbv_DbvIso10646_comp(&psProposedCharSets->uProposedCharSet.psIso10646, proposal->a.iso10646, env);
		break;
	case ORIGINPROPOSALSEQOFCHOICE_PRIVATE:
		psProposedCharSets->eProposedCharSetType = PRCST_Private;
		DBVsnacc_to_dbv_DbvPrivateCharacterSet_comp(&psProposedCharSets->uProposedCharSet.psPrivate, proposal->a.private, env);
		break;
	default:
		psProposedCharSets->eProposedCharSetType = PRCST_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvIso2022_comp(DbvIso2022 **ppsIso2022, struct Iso2022* iso2022, jmp_buf env)
{
	*ppsIso2022 = (DbvIso2022 *) DbvAlloc (sizeof (DbvIso2022));
	CheckDbvAlloc(*ppsIso2022, env);
	DBVsnacc_to_dbv_DbvIso2022(*ppsIso2022, iso2022, env);
}



void DBVsnacc_to_dbv_DbvIso2022(DbvIso2022 *psIso2022, struct Iso2022* iso2022, jmp_buf env)
{
	switch (iso2022->choiceId) {
	case ISO2022_ORIGINPROPOSAL:
		psIso2022->eIso2022Type = I2022T_OriginProposal;
		DBVsnacc_to_dbv_DbvIso2022Proposal_comp(&psIso2022->uIso2022Type.psOriginProposal, iso2022->a.originProposal, env);
		break;
	case ISO2022_TARGETRESPONSE:
		psIso2022->eIso2022Type = I2022T_TargetResponse;
		DBVsnacc_to_dbv_DbvIso2022Response_comp(&psIso2022->uIso2022Type.psTargetResponse, iso2022->a.targetResponse, env);
		break;
	default:
		psIso2022->eIso2022Type = I2022T_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvIso2022Proposal_comp(DbvIso2022Proposal **ppsOriginProposal, struct Iso2022Seq* originProposal, jmp_buf env)
{
	*ppsOriginProposal = (DbvIso2022Proposal *) DbvAlloc (sizeof (DbvIso2022Proposal));
	CheckDbvAlloc(*ppsOriginProposal, env);
	DBVsnacc_to_dbv_DbvIso2022Proposal(*ppsOriginProposal, originProposal, env);
}



void DBVsnacc_to_dbv_DbvIso2022Proposal(DbvIso2022Proposal *psOriginProposal, struct Iso2022Seq* originProposal, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvEnvironment_opt(&psOriginProposal->psProposedEnvironment, originProposal->proposedEnvironment, env);
	
	/* c */
	DBVsnacc_to_dbv_list_comp((void ***) &psOriginProposal->ppulProposedSets, originProposal->proposedSets, sizeof(Ulong), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_Ulong));

	/* c */
	DBVsnacc_to_dbv_list_comp((void ***) &psOriginProposal->ppsProposedInitialSets, originProposal->proposedInitialSets, sizeof(DbvInitialSet), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_DbvInitialSet));
	
	/* c */
	DBVsnacc_to_dbv_DbvLeftAndRight_comp(&psOriginProposal->psProposedLeftAndRight, originProposal->proposedLeftAndRight, env);
	
}




void DBVsnacc_to_dbv_DbvEnvironment_opt(DbvEnvironment **ppsProposedEnvironment, struct Environment* proposedEnvironment, jmp_buf env)
{
	if (proposedEnvironment) {
		*ppsProposedEnvironment = (DbvEnvironment *) DbvAlloc (sizeof (DbvEnvironment));
		CheckDbvAlloc(*ppsProposedEnvironment, env);
		DBVsnacc_to_dbv_DbvEnvironment(*ppsProposedEnvironment, proposedEnvironment, env);
	}
}



void DBVsnacc_to_dbv_DbvEnvironment(DbvEnvironment *psProposedEnvironment, struct Environment* proposedEnvironment, jmp_buf env)
{
	switch (proposedEnvironment->choiceId) {
	case ENVIRONMENT_SEVENBIT:
		psProposedEnvironment->eEnvironmentType = ENT_SevenBit;
		break;
	case ENVIRONMENT_EIGHTBIT:
		psProposedEnvironment->eEnvironmentType = ENT_EightBit;
		break;
	default:
		psProposedEnvironment->eEnvironmentType = ENT_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvInitialSet(DbvInitialSet *psProposedInitialSets, struct InitialSet* proposedInitialSets, jmp_buf env)
{
	/* 6 c */
	psProposedInitialSets->ulG0 = proposedInitialSets->g0;
	psProposedInitialSets->ulG1 = proposedInitialSets->g1;
	psProposedInitialSets->ulG2 = proposedInitialSets->g2;
	psProposedInitialSets->ulG3 = proposedInitialSets->g3;
	psProposedInitialSets->ulC0 = proposedInitialSets->c0;
	psProposedInitialSets->ulC1 = proposedInitialSets->c1;
}



void DBVsnacc_to_dbv_DbvLeftAndRight_comp(DbvLeftAndRight **ppsProposedLeftAndRight, struct LeftAndRight* proposedLeftAndRight, jmp_buf env)
{
	*ppsProposedLeftAndRight = (DbvLeftAndRight *) DbvAlloc (sizeof (DbvLeftAndRight));
	CheckDbvAlloc(*ppsProposedLeftAndRight, env);
	DBVsnacc_to_dbv_DbvLeftAndRight(*ppsProposedLeftAndRight, proposedLeftAndRight, env);
}



void DBVsnacc_to_dbv_DbvLeftAndRight(DbvLeftAndRight *psProposedLeftAndRight, struct LeftAndRight* proposedLeftAndRight, jmp_buf env)
{
	/* c */
	psProposedLeftAndRight->ulGLeft = proposedLeftAndRight->gLeft;

	/* c */
	psProposedLeftAndRight->ulGRight = proposedLeftAndRight->gRight;
}



void DBVsnacc_to_dbv_DbvIso2022Response_comp(DbvIso2022Response **ppsTargetResponse, struct Iso2022Seq1* targetResponse, jmp_buf env)
{
	*ppsTargetResponse = (DbvIso2022Response *) DbvAlloc (sizeof (DbvIso2022Response));
	CheckDbvAlloc(*ppsTargetResponse, env);
	DBVsnacc_to_dbv_DbvIso2022Response(*ppsTargetResponse, targetResponse, env);
}



void DBVsnacc_to_dbv_DbvIso2022Response(DbvIso2022Response *psTargetResponse, struct Iso2022Seq1* targetResponse, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvEnvironment_comp(&psTargetResponse->psSelectedEnvironment, targetResponse->selectedEnvironment, env);
	
	/* c */
	DBVsnacc_to_dbv_list_comp((void ***) &psTargetResponse->ppulSelectedSets, targetResponse->selectedSets, sizeof(Ulong), env, (void (*) (void*, void*, jmp_buf)) (DBVsnacc_to_dbv_Ulong));

	/* c */
	DBVsnacc_to_dbv_DbvInitialSet_comp(&psTargetResponse->psSelectedInitialSet, targetResponse->selectedinitialSet, env);
	
	/* c */
	DBVsnacc_to_dbv_DbvLeftAndRight_comp(&psTargetResponse->psSelectedLeftAndRight, targetResponse->selectedLeftAndRight, env);
}



void DBVsnacc_to_dbv_DbvEnvironment_comp(DbvEnvironment **ppSelectedEnvironment, struct Environment* selectedEnvironment, jmp_buf env)
{
	DBVsnacc_to_dbv_DbvEnvironment_opt(ppSelectedEnvironment, selectedEnvironment, env);
}



void DBVsnacc_to_dbv_DbvInitialSet_comp(DbvInitialSet **ppsSelectedInitialSet, struct InitialSet* selectedinitialSet, jmp_buf env)
{
	*ppsSelectedInitialSet = (DbvInitialSet *) DbvAlloc (sizeof (DbvInitialSet));
	CheckDbvAlloc(*ppsSelectedInitialSet, env);
	DBVsnacc_to_dbv_DbvInitialSet(*ppsSelectedInitialSet, selectedinitialSet, env);
}



void DBVsnacc_to_dbv_DbvIso10646_comp(DbvIso10646 **ppsIso10646, struct Iso10646* iso10646, jmp_buf env)
{
	*ppsIso10646 = (DbvIso10646 *) DbvAlloc (sizeof (DbvIso10646));
	CheckDbvAlloc(*ppsIso10646, env);
	DBVsnacc_to_dbv_DbvIso10646(*ppsIso10646, iso10646, env);
}



void DBVsnacc_to_dbv_DbvIso10646(DbvIso10646 *psIso10646, struct Iso10646* iso10646, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_DbvObjectIdentifier_comp(&psIso10646->pcCollections, &iso10646->collections, env);

	/* c */
	DBVsnacc_to_dbv_DbvObjectIdentifier_comp(&psIso10646->pcEncodingLevel, &iso10646->encodingLevel, env);

}



void DBVsnacc_to_dbv_DbvPrivateCharacterSet_comp(DbvPrivateCharacterSet **ppsPrivate, struct PrivateCharacterSet* private, jmp_buf env)
{
	*ppsPrivate = (DbvPrivateCharacterSet *) DbvAlloc (sizeof (DbvPrivateCharacterSet));
	CheckDbvAlloc(*ppsPrivate, env);
	DBVsnacc_to_dbv_DbvPrivateCharacterSet(*ppsPrivate, private, env);
}



void DBVsnacc_to_dbv_DbvPrivateCharacterSet(DbvPrivateCharacterSet *psPrivate, struct PrivateCharacterSet* private, jmp_buf env)
{
	switch (private->choiceId) {
	case PRIVATECHARACTERSET_VIAOID_SNACC:
		psPrivate->ePrivateCharacterSetType = PCST_ViaOid;

		/* A special list set up !! */
		DBVsnacc_to_dbv_DbvObjectIdentifier_list_comp(&psPrivate->uPrivateCharacterSet.ppcViaOid, private->a.viaOid, env);
		break;
	case PRIVATECHARACTERSET_EXTERNALLYSPECIFIED:
		psPrivate->ePrivateCharacterSetType = PCST_ExternallySpecified;
		DBVsnacc_to_dbv_DbvASN1External_comp(&psPrivate->uPrivateCharacterSet.psExternallySpecified, private->a.externallySpecified, env);
		break;
	case PRIVATECHARACTERSET_PREVIOUSLYAGREEDUPON:
		psPrivate->ePrivateCharacterSetType = PCST_PreviouslyAgreedUpon;
		/* Nothing else to do for this */
		break;

	default:
		psPrivate->ePrivateCharacterSetType = PCST_Invalid;
		break;
	}
}



void DBVsnacc_to_dbv_DbvLanguageCode(DbvLanguageCode *psProposedLangauages, LanguageCode* proposedlanguages, jmp_buf env)
{
	DBVsnacc_to_dbv_DbvInternationalString(psProposedLangauages, proposedlanguages, env);
}



void DBVsnacc_to_dbv_DbvTargetResponse_comp(DbvTargetResponse **ppsResponse, struct TargetResponse* response, jmp_buf env)
{
	*ppsResponse = (DbvTargetResponse *) DbvAlloc(sizeof (DbvTargetResponse));
	CheckDbvAlloc(*ppsResponse, env);
	DBVsnacc_to_dbv_DbvTargetResponse(*ppsResponse, response, env);
}



void DBVsnacc_to_dbv_DbvTargetResponse(DbvTargetResponse *psResponse, struct TargetResponse* response, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_DbvSelectedCharSet_opt(&psResponse->psSelectedCharSets, response->selectedCharSets, env);
	
	/* o */
	DBVsnacc_to_dbv_DbvGeneralString_opt(&psResponse->psSelectedLanguage, &response->selectedLanguage, env);
}



void DBVsnacc_to_dbv_DbvSelectedCharSet_opt(DbvSelectedCharSet **ppsSelectedCharSets, struct TargetResponseChoice* selectedCharSets, jmp_buf env)
{
	if (selectedCharSets) {
		*ppsSelectedCharSets = (DbvSelectedCharSet *) DbvAlloc(sizeof (DbvSelectedCharSet));
		CheckDbvAlloc(*ppsSelectedCharSets, env);
		DBVsnacc_to_dbv_DbvSelectedCharSet(*ppsSelectedCharSets, selectedCharSets, env);
	}
}



void DBVsnacc_to_dbv_DbvSelectedCharSet(DbvSelectedCharSet *psSelectedCharSets, struct TargetResponseChoice* selectedCharSets, jmp_buf env)
{
	switch (selectedCharSets->choiceId) {
	case TARGETRESPONSECHOICE_ISO2022:
		psSelectedCharSets->eSelectedCharSetType = SCST_Iso2022;
		DBVsnacc_to_dbv_DbvIso2022_comp(&psSelectedCharSets->uSelectedCharSet.psIso2022, selectedCharSets->a.iso2022, env);
		break;
	case TARGETRESPONSECHOICE_ISO10646:
		psSelectedCharSets->eSelectedCharSetType = SCST_Iso10646;
		DBVsnacc_to_dbv_DbvIso10646_comp(&psSelectedCharSets->uSelectedCharSet.psIso10646, selectedCharSets->a.iso10646, env);
		break;
	case TARGETRESPONSECHOICE_PRIVATE:
		psSelectedCharSets->eSelectedCharSetType = SCST_Private;
		DBVsnacc_to_dbv_DbvPrivateCharacterSet_comp(&psSelectedCharSets->uSelectedCharSet.psPrivate, selectedCharSets->a.private, env);
		break;
	case TARGETRESPONSECHOICE_NONE:
		psSelectedCharSets->eSelectedCharSetType = SCST_None;
		/* Nothing else to do */
		break;
	default:
		psSelectedCharSets->eSelectedCharSetType = SCST_Invalid;
		break;
	}
}







/*******************************************************************/
/* Structure checking routines */
    

  
/* Check the DBV prompt object external structure for validity */
        
int DBVcheck_at_DbvPromptObject(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{       
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ANSI_ACCESS_PROMPT_1) != 0) && (strcmp(oid, ISO_ACCESS_PROMPT_1) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


	DBVsa_strcat(local_error_buffer, "uASN1.psPromptObject");
	if (psSingleASN1Type->uASN1.psPromptObject == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvPromptObject(lAssocId, psSingleASN1Type->uASN1.psPromptObject, local_error_buffer, ppsSIError));

	return SI_OK;

}
        


int DBVcheck_DbvPromptObject(DbvAssocId lAssocId, struct DbvPromptObject *psPromptObject, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPromptObject->ePromptObjectType) {
	case POT_Challenge:
		DBVsa_strcat(local_error_buffer, "uPromptObject.ppsChallenge");
		if (psPromptObject->uPromptObject.ppsChallenge == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uPromptObject.");
		if (DBVcheck_list_opt(lAssocId, (void **) psPromptObject->uPromptObject.ppsChallenge, local_error_buffer, "ppsChallenge", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvChallenge)) == SI_NOTOK)
                	return SI_NOTOK;
		break;

	case POT_Response:
		DBVsa_strcat(local_error_buffer, "uPromptObject.ppsResponse");
		if (psPromptObject->uPromptObject.ppsResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uPromptObject.");
		if (DBVcheck_list_opt(lAssocId, (void **) psPromptObject->uPromptObject.ppsResponse, local_error_buffer, "ppsResponse", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvResponse)) == SI_NOTOK)
                	return SI_NOTOK;
		break;
	default:
		DBVsa_strcat(local_error_buffer, "ePromptObjectType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvChallenge(DbvAssocId lAssocId, DbvChallenge *psChallenge, char *error_buffer, DbvError **ppsSIError)
{ 
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
        DBVsa_strcat(local_error_buffer, "sPromptId.");
	if (DBVcheck_DbvPromptId(lAssocId, &psChallenge->sPromptId, local_error_buffer, ppsSIError) == SI_NOTOK)   
		return SI_NOTOK;

	/* o  psDefaultResponse - no need to check */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psPromptInfo->");
	if (DBVcheck_DbvPromptInfo_opt(lAssocId, psChallenge->psPromptInfo, local_error_buffer, ppsSIError) == SI_NOTOK)   
		return SI_NOTOK;

	/* o psRegExpr - no need to check */

	/* o pvResponseRequired - no need to check */

	/* o ppsAllowedValues - no need to check */

	/* o pvShouldSave - no need to check */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "peDataType");
	if (DBVcheck_DbvDataType_opt(lAssocId, psChallenge->peDataType, local_error_buffer, ppsSIError) == SI_NOTOK)   
		return SI_NOTOK;



	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psDiagnostic->");
	if (DBVcheck_DbvASN1External_opt(lAssocId, psChallenge->psDiagnostic, local_error_buffer, ppsSIError) == SI_NOTOK)   
		return SI_NOTOK;

	
	return SI_OK;
}



int DBVcheck_DbvPromptId(DbvAssocId lAssocId, DbvPromptId *psPromptId, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPromptId->ePromptIdType) {
	case PIT_EnumeratedPrompt:
		DBVsa_strcat(local_error_buffer, "uPromptId.psEnumeratedPrompt");
		if (psPromptId->uPromptId.psEnumeratedPrompt == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvEnumeratedPrompt(lAssocId, psPromptId->uPromptId.psEnumeratedPrompt, local_error_buffer, ppsSIError));

		break;

	case PIT_NoneEnumeratedPrompt:
		DBVsa_strcat(local_error_buffer, "uPromptId.psNonEnumeratedPrompt");
		if (psPromptId->uPromptId.psNonEnumeratedPrompt == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* nought else to check */
		break;
	default:
		DBVsa_strcat(local_error_buffer, "ePromptIdType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvEnumeratedPrompt(DbvAssocId lAssocId, DbvEnumeratedPrompt *psEnumeratedPrompt, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);
	
	/* c */
	if (DBVcheck_dbv_enum(psEnumeratedPrompt->eType, EPIT_GroupId, EPIT_SessionId) == SI_NOTOK) {
                DBVsa_strcat(local_error_buffer, "eType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));    
        }       

	/* c psSuggestedString - no need to check */

	return SI_OK;
}


int DBVcheck_DbvPromptInfo_opt(DbvAssocId lAssocId, DbvPromptInfo *psPromptInfo, char *error_buffer, DbvError **ppsSIError)
{
	if (psPromptInfo)
		return(DBVcheck_DbvPromptInfo(lAssocId, psPromptInfo, error_buffer, ppsSIError));

	return SI_OK;
	
}


int DBVcheck_DbvPromptInfo(DbvAssocId lAssocId, DbvPromptInfo *psPromptInfo, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPromptInfo->ePromptInfoType) {
	case PIIT_Character:
		DBVsa_strcat(local_error_buffer, "uPromptInfo.psCharacter");
		if (psPromptInfo->uPromptInfo.psCharacter == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* aint nowt more ta check */

		break;

	case PIIT_Encrypted:
		DBVsa_strcat(local_error_buffer, "uPromptInfo.psEncrypted");
		if (psPromptInfo->uPromptInfo.psEncrypted == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvEncryption(lAssocId, psPromptInfo->uPromptInfo.psEncrypted, local_error_buffer, ppsSIError));

		break;
	default:
		DBVsa_strcat(local_error_buffer, "ePromptInfoType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
	
}



int DBVcheck_DbvEncryption(DbvAssocId lAssocId, DbvEncryption *psEncrypted, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o psCryptType - dont have to check */

	/* o psCredential - dont have to check */

	/* o sData - dont have to check */

	return SI_OK;
}


int DBVcheck_DbvDataType_opt(DbvAssocId lAssocId, DbvDataType *peDataType, char *error_buffer, DbvError **ppsSIError)
{
	if (peDataType)
		return(DBVcheck_DbvDataType(lAssocId, peDataType, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvDataType(DbvAssocId lAssocId, DbvDataType *peDataType, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (DBVcheck_dbv_enum(*peDataType, DTTT_Integer, DTTT_Boolean) == SI_NOTOK) 
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));    
	
	return SI_OK;
}



int DBVcheck_DbvResponse(DbvAssocId lAssocId, DbvResponse *psResponse, char *error_buffer, DbvError **ppsSIError)
{ 
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	DBVsa_strcat(local_error_buffer, "sPromptId.");
	if (DBVcheck_DbvPromptId(lAssocId, &psResponse->sPromptId, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "sPromptResponse.");
	if (DBVcheck_DbvPromptResponse(lAssocId, &psResponse->sPromptResponse, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvPromptResponse(DbvAssocId lAssocId, DbvPromptResponse *psPromptResponse, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPromptResponse->eResponseType) {
	case RT_String:
		DBVsa_strcat(local_error_buffer, "uResponse.psCharacter");
		if (psPromptResponse->uResponse.psString == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* aint nowt more ta check */

		break;

	case RT_Accept:
		DBVsa_strcat(local_error_buffer, "uResponse.fAccept");
		return(DBVcheck_DbvBool(lAssocId, &psPromptResponse->uResponse.fAccept, local_error_buffer, ppsSIError));
		break;

	case RT_Acknowledge:

		/* check a NULL ? - no need my son */
		break;
	case RT_Diagnostic:
		DBVsa_strcat(local_error_buffer, "uResponse.psDiagnostic");
		if (psPromptResponse->uResponse.psDiagnostic == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_surrdiag(lAssocId, psPromptResponse->uResponse.psDiagnostic, local_error_buffer, ppsSIError));

		break;
	case RT_Encrypted:
		DBVsa_strcat(local_error_buffer, "uResponse.psEncryption");
		if (psPromptResponse->uResponse.psEncryption == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvEncryption(lAssocId, psPromptResponse->uResponse.psEncryption, local_error_buffer, ppsSIError));

		break;
	default:
		DBVsa_strcat(local_error_buffer, "eResponseType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}





/********************************************************/
/********************************************************/


/* Check the DbvPrintExtendedService external */

int DBVcheck_at_DbvPrintExtendedService(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{       
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ESRIN_PRINT) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


	DBVsa_strcat(local_error_buffer, "uASN1.psPrintExtendedService");
	if (psSingleASN1Type->uASN1.psPrintExtendedService == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvPrintExtendedService(lAssocId, psSingleASN1Type->uASN1.psPrintExtendedService, local_error_buffer, ppsSIError));

	return SI_OK;

}



int DBVcheck_DbvPrintExtendedService(DbvAssocId lAssocId, DbvPrintExtendedService *psPrintExtendedService, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPrintExtendedService->ePrintReqRsp) {
	case PROR_Request:
		DBVsa_strcat(local_error_buffer, "uPrintReqOrRsp.psPrintRequest");
		if (psPrintExtendedService->uPrintReqOrRsp.psPrintRequest == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvPrintRequestPackage(lAssocId, psPrintExtendedService->uPrintReqOrRsp.psPrintRequest, local_error_buffer, ppsSIError));

		break;

	case PROR_Response:
		DBVsa_strcat(local_error_buffer, "uPrintReqOrRsp.psPrintResponse");
		if (psPrintExtendedService->uPrintReqOrRsp.psPrintResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvPrintTaskPackage(lAssocId, psPrintExtendedService->uPrintReqOrRsp.psPrintResponse, local_error_buffer, ppsSIError));
		break;

	default:
		DBVsa_strcat(local_error_buffer, "ePrintReqRsp");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvPrintRequestPackage(DbvAssocId lAssocId, DbvPrintRequestPackage *psPrintRequest, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psOriginPartToKeep");
	if (DBVcheck_DbvPrintOriginPartToKeep_opt(lAssocId, psPrintRequest->psOriginPartToKeep, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvPrintOriginPartToKeep_opt(DbvAssocId lAssocId, DbvPrintOriginPartToKeep *psOriginPartToKeep, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	if (psOriginPartToKeep) {
		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvPrintOriginPartToKeep(lAssocId, psOriginPartToKeep, local_error_buffer, ppsSIError));
	}
		

	return SI_OK;
}



int DBVcheck_DbvPrintOriginPartToKeep(DbvAssocId lAssocId, DbvPrintOriginPartToKeep *psPrintOriginPartToKeep, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPrintOriginPartToKeep->ePrintOriginPartToKeepType) {
	case POPKT_Create:
		DBVsa_strcat(local_error_buffer, "uPrintOriginPartToKeep.psCreateOption");
		if (psPrintOriginPartToKeep->uPrintOriginPartToKeep.psCreateOption == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvCreateOption(lAssocId, psPrintOriginPartToKeep->uPrintOriginPartToKeep.psCreateOption, local_error_buffer, ppsSIError));

		break;

	case POPKT_Delete:
		DBVsa_strcat(local_error_buffer, "uPrintOriginPartToKeep.psDeleteOption");
		if (psPrintOriginPartToKeep->uPrintOriginPartToKeep.psDeleteOption == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvDeleteOption(lAssocId, psPrintOriginPartToKeep->uPrintOriginPartToKeep.psDeleteOption, local_error_buffer, ppsSIError));
		break;

	default:
		DBVsa_strcat(local_error_buffer, "ePrintOriginPartToKeepType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvCreateOption(DbvAssocId lAssocId, DbvCreateOption *psCreateOption, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psCreateOption->eCreateOptionType) {
	case COPT_ResultSetReference:
		DBVsa_strcat(local_error_buffer, "uCreateOption.psResultSetReference");
		if (psCreateOption->uCreateOption.psResultSetReference == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvResultSetReference(lAssocId, psCreateOption->uCreateOption.psResultSetReference, local_error_buffer, ppsSIError));

		break;

	case COPT_AccessionReference:
		DBVsa_strcat(local_error_buffer, "uCreateOption.psAccessionReference");
		if (psCreateOption->uCreateOption.psAccessionReference == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvAccessionReference(lAssocId, psCreateOption->uCreateOption.psAccessionReference, local_error_buffer, ppsSIError));
		break;

	default:
		DBVsa_strcat(local_error_buffer, "eCreateOptionType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvResultSetReference(DbvAssocId lAssocId, DbvResultSetReference *psResultSetReference, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	/* c */
	/* o  but no need to check*/

	/* o */
        DBVsa_strcat(local_error_buffer, "psRecordComposition->");
	if (DBVcheck_DbvRecordComposition_opt(lAssocId, psResultSetReference->psRecordComposition, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvDeleteOption(DbvAssocId lAssocId, DbvDeleteOption *psDeleteOption, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* nothing to check */

	return SI_OK;
}




int DBVcheck_DbvAccessionReference(DbvAssocId lAssocId, DbvAccessionReference *psAccessionReference, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c - no need to check */

	/* o */
        DBVsa_strcat(local_error_buffer, "psRecordComposition->");
	if (DBVcheck_DbvRecordComposition_opt(lAssocId, psAccessionReference->psRecordComposition, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "sAddress.");
	if (DBVcheck_DbvAddress(lAssocId, &psAccessionReference->sAddress, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvAddress(DbvAssocId lAssocId, DbvAddress *psAddress, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c - no need to check */

	/* o - no need to check */

	return SI_OK;
}




int DBVcheck_DbvPrintTaskPackage(DbvAssocId lAssocId, DbvPrintTaskPackage *psPrintResponse, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psOriginPartToKeep");
	if (DBVcheck_DbvPrintOriginPartToKeep_opt(lAssocId, psPrintResponse->psOriginPartToKeep, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c */
	/* no need to check */

	return SI_OK;
}



/********************************************************/
/********************************************************/


/* Check the DbvAlertExtendedService external */

int DBVcheck_at_DbvAlertExtendedService(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{       
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ESRIN_ALERT) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


	DBVsa_strcat(local_error_buffer, "uASN1.psAlertExtendedService");
	if (psSingleASN1Type->uASN1.psAlertExtendedService == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvAlertExtendedService(lAssocId, psSingleASN1Type->uASN1.psAlertExtendedService, local_error_buffer, ppsSIError));

	return SI_OK;

}



int DBVcheck_DbvAlertExtendedService(DbvAssocId lAssocId, DbvAlertExtendedService *psAlertExtendedService, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psAlertExtendedService->eAlertReqRsp) {
	case AROR_Request:
		DBVsa_strcat(local_error_buffer, "uAlertReqOrRsp.psAlertRequest");
		if (psAlertExtendedService->uAlertReqOrRsp.psAlertRequest == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvAlertRequestPackage(lAssocId, psAlertExtendedService->uAlertReqOrRsp.psAlertRequest, local_error_buffer, ppsSIError));

		break;

	case AROR_Response:
		DBVsa_strcat(local_error_buffer, "uAlertReqOrRsp.psAlertResponse");
		if (psAlertExtendedService->uAlertReqOrRsp.psAlertResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvAlertTaskPackage(lAssocId, psAlertExtendedService->uAlertReqOrRsp.psAlertResponse, local_error_buffer, ppsSIError));
		break;

	default:
		DBVsa_strcat(local_error_buffer, "eAlertReqRsp");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvAlertRequestPackage(DbvAssocId lAssocId, DbvAlertRequestPackage *psAlertRequest, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "psOriginPartToKeep->");
	if (DBVcheck_DbvAlertOriginPartToKeep_opt(lAssocId, psAlertRequest->psOriginPartToKeep, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvAlertOriginPartToKeep_opt(DbvAssocId lAssocId, DbvAlertOriginPartToKeep *psOriginPartToKeep, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psOriginPartToKeep)
		if (DBVcheck_DbvAlertOriginPartToKeep(lAssocId, psOriginPartToKeep, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvAlertOriginPartToKeep(DbvAssocId lAssocId, DbvAlertOriginPartToKeep *psOriginPartToKeep, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */

	/* o */
        DBVsa_strcat(local_error_buffer, "psAddress->");
	if (DBVcheck_DbvAddress_opt(lAssocId, psOriginPartToKeep->psAddress, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvAddress_opt(DbvAssocId lAssocId, DbvAddress *psAddress, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psAddress)
		if (DBVcheck_DbvAddress(lAssocId, psAddress, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvAlertTaskPackage(DbvAssocId lAssocId, DbvAlertTaskPackage *psAlertResponse, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "psOriginPartToKeep->");
	if (DBVcheck_DbvAlertOriginPartToKeep_opt(lAssocId, psAlertResponse->psOriginPartToKeep, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



/********************************************************/
/********************************************************/


/* Check the DbvCOrder external */

int DBVcheck_at_DbvCOrder(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{       
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ESRIN_ORDER) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


	DBVsa_strcat(local_error_buffer, "uASN1.psCOrder");
	if (psSingleASN1Type->uASN1.psCOrder == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvCOrder(lAssocId, psSingleASN1Type->uASN1.psCOrder, local_error_buffer, ppsSIError));

	return SI_OK;

}



int DBVcheck_DbvCOrder(DbvAssocId lAssocId, DbvCOrder *psCOrder, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psCOrder->eCOrderType) {
	case COT_OrderRequest:
		DBVsa_strcat(local_error_buffer, "uCOrder.psOrderRequest");
		if (psCOrder->uCOrder.psOrderRequest == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvOrderRequest(lAssocId, psCOrder->uCOrder.psOrderRequest, local_error_buffer, ppsSIError));

		break;

	case COT_OrderResponse:
		DBVsa_strcat(local_error_buffer, "uCOrder.ppsOrderResponse");
		if (psCOrder->uCOrder.ppsOrderResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	
		/* no need to check rest */

		break;

	default:
		DBVsa_strcat(local_error_buffer, "eCOrderType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvOrderRequest(DbvAssocId lAssocId, DbvOrderRequest *psOrderRequest, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psOrderRequest->eOrderRequestType) {
	case ORT_NormalOrder:
		DBVsa_strcat(local_error_buffer, "uOrderRequest.psNormalOrder");
		if (psOrderRequest->uOrderRequest.psNormalOrder == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvNormalOrder(lAssocId, psOrderRequest->uOrderRequest.psNormalOrder, local_error_buffer, ppsSIError));
		break;

	case ORT_OrderRefs:
		DBVsa_strcat(local_error_buffer, "uOrderRequest.ppsOrderRefs");
		if (psOrderRequest->uOrderRequest.ppsOrderRefs == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	
		/* no need to check rest */

		break;

	default:
		DBVsa_strcat(local_error_buffer, "eOrderRequestType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvNormalOrder(DbvAssocId lAssocId, DbvNormalOrder *psNormalOrder, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	DBVsa_strcat(local_error_buffer, "ppsOrderRanges");
	if (psNormalOrder->ppsOrderRanges == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	/* c ulQuantity but no need to check */

	return SI_OK;
}






/************************************************************/
  

/* Check the DbvCharSetAndLanguage external structure for validity */
        
int DBVcheck_at_DbvCharSetAndLanguage(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{       
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION) != 0) && (strcmp(oid, ISO_CHARSET_AND_LANGUAGE_NEGOTIATION) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


	DBVsa_strcat(local_error_buffer, "uASN1.psCharSetAndLanguage");
	if (psSingleASN1Type->uASN1.psCharSetAndLanguage == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvCharSetAndLanguage(lAssocId, psSingleASN1Type->uASN1.psCharSetAndLanguage, local_error_buffer, ppsSIError));

	return SI_OK;

}
        


int DBVcheck_DbvCharSetAndLanguage(DbvAssocId lAssocId, struct DbvCharSetAndLanguage *psCharSetAndLanguage, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psCharSetAndLanguage->eCharSetAndLanguageType) {
	case CSALT_Proposal:
		DBVsa_strcat(local_error_buffer, "uCharSetAndLanguage.psProposal");
		if (psCharSetAndLanguage->uCharSetAndLanguage.psProposal == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvOriginProposal(lAssocId, psCharSetAndLanguage->uCharSetAndLanguage.psProposal, local_error_buffer, ppsSIError));
		break;

	case CSALT_Response:
		DBVsa_strcat(local_error_buffer, "uCharSetAndLanguage.psResponse");
		if (psCharSetAndLanguage->uCharSetAndLanguage.psResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvTargetResponse(lAssocId, psCharSetAndLanguage->uCharSetAndLanguage.psResponse, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eCharSetAndLanguageType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvOriginProposal(DbvAssocId lAssocId, DbvOriginProposal *psProposal, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psProposal->ppsProposedCharSets, local_error_buffer, "ppsProposedCharSets", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvProposedCharSet)) == SI_NOTOK)
              	return SI_NOTOK;

	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psProposal->ppsProposedLanguages, local_error_buffer, "ppsProposedLangauages", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvLanguageCode)) == SI_NOTOK)
              	return SI_NOTOK;

	return SI_OK;
	
}



int DBVcheck_DbvProposedCharSet(DbvAssocId lAssocId, DbvProposedCharSet *psProposedCharSets, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psProposedCharSets->eProposedCharSetType) {
	case PRCST_Iso2022:
		DBVsa_strcat(local_error_buffer, "uProposedCharSet.psIso2022");
		if (psProposedCharSets->uProposedCharSet.psIso2022 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso2022(lAssocId, psProposedCharSets->uProposedCharSet.psIso2022, local_error_buffer, ppsSIError));
		break;

	case PRCST_Iso10646:
		DBVsa_strcat(local_error_buffer, "uProposedCharSet.psIso10646");
		if (psProposedCharSets->uProposedCharSet.psIso10646 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso10646(lAssocId, psProposedCharSets->uProposedCharSet.psIso10646, local_error_buffer, ppsSIError));
		break;
	case PRCST_Private:
		DBVsa_strcat(local_error_buffer, "uProposedCharSet.psPrivate");
		if (psProposedCharSets->uProposedCharSet.psPrivate == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvPrivateCharacterSet(lAssocId, psProposedCharSets->uProposedCharSet.psPrivate, local_error_buffer, ppsSIError));
		break;

	default:
		DBVsa_strcat(local_error_buffer, "eProposedCharSetType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}


int DBVcheck_DbvIso2022(DbvAssocId lAssocId, DbvIso2022 *psIso2022, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psIso2022->eIso2022Type) {
	case I2022T_OriginProposal:
		DBVsa_strcat(local_error_buffer, "uIso2022Type.psOriginProposal");
		if (psIso2022->uIso2022Type.psOriginProposal == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso2022Proposal(lAssocId, psIso2022->uIso2022Type.psOriginProposal, local_error_buffer, ppsSIError));
		break;

	case I2022T_TargetResponse:
		DBVsa_strcat(local_error_buffer, "uIso2022Type.psTargetResponse");
		if (psIso2022->uIso2022Type.psTargetResponse == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso2022Response(lAssocId, psIso2022->uIso2022Type.psTargetResponse, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eIso2022Type");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvIso2022Proposal(DbvAssocId lAssocId, DbvIso2022Proposal *psOriginProposal, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psProposedEnvironment->");
	if (psOriginProposal->psProposedEnvironment) 
		if (DBVcheck_DbvEnvironment(lAssocId, psOriginProposal->psProposedEnvironment, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;


	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "ppulProposedSets");
	if (psOriginProposal->ppulProposedSets == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "ppsProposedInitialSets");
	if (psOriginProposal->ppsProposedInitialSets == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psProposedLeftAndRight");
	if (psOriginProposal->psProposedLeftAndRight == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	if (DBVcheck_DbvLeftAndRight(lAssocId, psOriginProposal->psProposedLeftAndRight, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvEnvironment(DbvAssocId lAssocId, DbvEnvironment *psProposedEnvironment, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psProposedEnvironment->eEnvironmentType) {
	case ENT_SevenBit:
	case ENT_EightBit:
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eEnvironmentType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvLeftAndRight(DbvAssocId lAssocId, DbvLeftAndRight *psProposedLeftAndRight, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);


	/* Nothing to check here */

	return SI_OK;
}



int DBVcheck_DbvIso2022Response(DbvAssocId lAssocId, DbvIso2022Response *psTargetResponse, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	DBVsa_strcat(local_error_buffer, "psSelectedEnvironment");
	if (psTargetResponse->psSelectedEnvironment == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	if (DBVcheck_DbvEnvironment(lAssocId, psTargetResponse->psSelectedEnvironment, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "ppulSelectedSets");
	if (psTargetResponse->ppulSelectedSets == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psSelectedInitialSet");
	if (psTargetResponse->psSelectedInitialSet == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psSelectedLeftAndRight");
	if (psTargetResponse->psSelectedLeftAndRight == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	if (DBVcheck_DbvLeftAndRight(lAssocId, psTargetResponse->psSelectedLeftAndRight, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvIso10646(DbvAssocId lAssocId, DbvIso10646 *psIso10646, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	
	
	/* c */
        DBVsa_strcat(local_error_buffer, "pcCollections");
        if (psIso10646->pcCollections == NULL)
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));   

        if (DBVcheck_DbvObjectIdentifier(lAssocId, psIso10646->pcCollections, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcEncodingLevel");
        if (psIso10646->pcEncodingLevel == NULL)
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));   

        if (DBVcheck_DbvObjectIdentifier(lAssocId, psIso10646->pcEncodingLevel, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvPrivateCharacterSet(DbvAssocId lAssocId, DbvPrivateCharacterSet *psPrivate, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psPrivate->ePrivateCharacterSetType) {
	case PCST_ViaOid:
		DBVsa_strcat(local_error_buffer, "uPrivateCharacterSet.ppcViaOid");
		if (psPrivate->uPrivateCharacterSet.ppcViaOid == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uPrivateCharacterSet.");

		/* DBV_FALSE because no sub structures */
		if (DBVcheck_list(lAssocId, (void **) psPrivate->uPrivateCharacterSet.ppcViaOid, local_error_buffer, "ppcViaOid", ppsSIError, DBV_FALSE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)) == SI_NOTOK)
              		return SI_NOTOK;

		break;

	case PCST_ExternallySpecified:
		DBVsa_strcat(local_error_buffer, "uPrivateCharacterSet.psExternallySpecified");
		if (psPrivate->uPrivateCharacterSet.psExternallySpecified == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvASN1External(lAssocId, psPrivate->uPrivateCharacterSet.psExternallySpecified, local_error_buffer, ppsSIError));
		break;

	case PCST_PreviouslyAgreedUpon:
		break;

	default:
		DBVsa_strcat(local_error_buffer, "ePrivateCharacterSetType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}




int DBVcheck_DbvLanguageCode(DbvAssocId lAssocId, DbvLanguageCode *psProposedLangauages, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* no need to check - its a DbvInternationalString !! */

	return SI_OK;
}



int DBVcheck_DbvTargetResponse(DbvAssocId lAssocId, DbvTargetResponse *psResponse, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	
	/* o */
	if (DBVcheck_DbvSelectedCharSet_opt(lAssocId, psResponse->psSelectedCharSets, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;
	
	/* o */
	/* Nothing else to check */
	


	return SI_OK;
}



int DBVcheck_DbvSelectedCharSet_opt(DbvAssocId lAssocId, DbvSelectedCharSet *psSelectedCharSets, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	
	if (psSelectedCharSets) {
		DBVsa_strcat(local_error_buffer, "psSelectedCharSets->");
		if (DBVcheck_DbvSelectedCharSet(lAssocId, psSelectedCharSets, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}

	return SI_OK;
}



int DBVcheck_DbvSelectedCharSet(DbvAssocId lAssocId, DbvSelectedCharSet *psSelectedCharSets, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psSelectedCharSets->eSelectedCharSetType) {
	case SCST_Iso2022:
		DBVsa_strcat(local_error_buffer, "uSelectedCharSet.psIso2022");
		if (psSelectedCharSets->uSelectedCharSet.psIso2022 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso2022(lAssocId, psSelectedCharSets->uSelectedCharSet.psIso2022, local_error_buffer, ppsSIError));
		break;

	case SCST_Iso10646:
		DBVsa_strcat(local_error_buffer, "uSelectedCharSet.psIso10646");
		if (psSelectedCharSets->uSelectedCharSet.psIso10646 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIso10646(lAssocId, psSelectedCharSets->uSelectedCharSet.psIso10646, local_error_buffer, ppsSIError));
		break;
	case PRCST_Private:
		DBVsa_strcat(local_error_buffer, "uSelectedCharSet.psPrivate");
		if (psSelectedCharSets->uSelectedCharSet.psPrivate == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvPrivateCharacterSet(lAssocId, psSelectedCharSets->uSelectedCharSet.psPrivate, local_error_buffer, ppsSIError));
		break;
	case SCST_None:

		/* Nothing else to check */
		break;

	default:
		DBVsa_strcat(local_error_buffer, "eSelectedCharSetType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError)); 
		break;
	}

	return SI_OK;
}







