/*  Module Name: @(#)z3950_send_receive.c	7.3
    Version: 1/15/97
    Author: John Hough/Chris Attewell
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: DBV-OSI II TCP/IP and OSI BER data send and receive routines 

*/

#include "z3950_send_receive.h"

int DBVencode_and_send(PDU *PDU_ptr, DbvAssocId lAssocId, DbvError **ppsError)
{
	char *encoded_pdu = NULL;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_OK;
	int encoded_pdu_length = 0;
	DBVAssociateDetails *psAssociateDetails;
	DBVTraceDetails *psTraceInformation;

#ifndef NO_OSI
	/* ISODE structures */
	PE psPresentationElement;
	int pci;
	struct AcSAPindication *psAssocControlIndication;
	struct PSAPindication  *psPresentationIndication=NULL;
	struct PSAPabort *psPresentationAbort;
#endif
        DBVEnterFuncTrace("DBVencode_and_send",lAssocId);

#ifndef NO_OSI
	if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
		DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	if(DBVAllocCheck((psPresentationIndication=(struct PSAPindication *)
		DBVAlloc(sizeof(struct PSAPindication), ppsError)))==SI_NOTOK)
		{ return SI_NOTOK; }
	psPresentationAbort=&psPresentationIndication->pi_abort;
#endif

	/* Reset all singleASN1Type external counters to 0 */
	DBV_reset_external_counters();
	
	if(DBVTraceLevel(lAssocId) == Trace_High)
		DBVTrace(lAssocId, Trace_High, "Encoding and sending PDU\n");

        if (DBVencode_pdu_to_bytes(lAssocId, PDU_ptr, &encoded_pdu, 
		&encoded_pdu_length, ppsError) == SI_NOTOK) 
	{
                /* Encoding failed\n */
                return_value = SI_NOTOK;
        }
 
	if(return_value == SI_OK)
	{
	if((SIWhichStack(lAssocId, &eProtocolStack, ppsError)) == SI_OK)
	{
        psAssociateDetails = SL_Current(DBVAssociateList);
	if(eProtocolStack == TCP_Stack)
	{
		/* Sending PDU via TCP STACK */
		if (DBVsend_data_over_socket(psAssociateDetails->ulTcpSocket, 
			encoded_pdu, encoded_pdu_length) <= 0) 
		{
			/* printf("send failed\n"); */

			/* A send failure is an abort request in both
			   origin and target */
			if(psAssociateDetails->eOriginOrTarget == O)
				DBVChangeState(lAssocId,DBV_OE_AABORT_IND, ppsError);
			else
				DBVChangeState(lAssocId,DBV_TE_AABORT_IND, ppsError); 

			DBVfill_error_structure(lAssocId, SI_Abort, 0, DBV_ABORT
, DBV_ABORT_S, 0, Association_Closed, ppsError);
			return_value = SI_NOTOK;
		} 	 
	}
	else
	{
#ifndef NO_OSI
		psPresentationElement=oct2prim(encoded_pdu,encoded_pdu_length);
		AcFindPCI(psAssociateDetails->lOSIAssocId, &pci, psAssocControlIndication);
		psPresentationElement->pe_context=pci;
		if(PDataRequest(psAssociateDetails->lOSIAssocId, 
			&psPresentationElement, 1, psPresentationIndication) 
			== NOTOK)
        	{
			return_value = DBVPOSICheckISODEError(lAssocId,
				psAssociateDetails->lOSIAssocId,
				psPresentationIndication, ppsError);
			if(return_value >= DBV_RELEASE_REQUEST)
			{
			if(psAssociateDetails->eOriginOrTarget == O)
				return_value = DBVChangeState(lAssocId,DBV_TE_AREL_IND, ppsError); 
			else
				return_value = DBVChangeState(lAssocId,DBV_OE_ARELREQ_IND, ppsError); 
			}
			/*
                	printf("P-DATA.REQUEST: %s\n", PErrString(psPresentationAbort->pa_reason));
                	printf("P-DATA.REQUEST: %s\n", psPresentationIndication->pi_un.pi_un_abort.pa_data);
			PAFREE(psPresentationAbort);
			*/
			return_value = SI_NOTOK;
		}
		pe_free(psPresentationElement);
#else
		printf("OSI not supported\n");
#endif /* NO_OSI */
	}
	}
	else
	{
		return_value = SI_NOTOK;
	}
	}

	if(return_value == SI_OK && DBVTraceLevel(lAssocId) == Trace_High)
		DBVTrace(lAssocId, Trace_High, "Send was successful\n");

        DBVLeaveFuncTrace("DBVencode_and_send",lAssocId, return_value);

        if(return_value == SI_NOTOK)
                if((*ppsError)->eErrorClass == SI_Abort)
                {
		/* Remove the association if it's finished */
			if(DBVTraceList != NULL)
                        {
                        psTraceInformation = SL_Locate(DBVTraceList,
                            (SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);
                        if(psTraceInformation != NULL)
                        {
                                psTraceInformation = SL_RemoveCurrent(DBVTraceList);
                                if(psTraceInformation != NULL)
                                        free(psTraceInformation);
 
                        }
			}
                        psAssociateDetails = SL_RemoveCurrent(DBVAssociateList);
                        free(psAssociateDetails);

                }


	/* Cant free encoded_pdu here because it is a pointer to a static
	   in DBVencode_pdu_to_bytes() */ 

        return return_value;
}



int DBVsend_data_over_socket(int socket, char *sendbuffer, int len)
{
        long amount_to_send = len;
        long total = 0;
        long sent = 0;
	struct timeval timeout;
	struct fd_set reader, writer;
	int result;
	char buffer[100];


	FD_ZERO(&reader);
	FD_ZERO(&writer);
	FD_SET(socket, &reader);
	FD_SET(socket, &writer);

	result = select(FD_SETSIZE, &reader, &writer, NULL, NULL);
	
	/* On most systems (Solaris1, Solaris 2, HP-UX, Linux) result will
	   be 1 (writer fd_set) if other end is up, 2 if other end
	   is down */
	if (result == 2) {

		/* Here, recv() MSG_PEEK always returns instantly
		   (for blocking and BSD nonblocking sockets) with
		   1 if data is pending to be received 
		   0 if other end is down on 

		   There could be a possiblility of -1, EWOULDBLOCK for
		   non blocking data pending but this is ok - it won't 
		   cause a -1 return.
		*/

		if (recv(socket, buffer, 1, MSG_PEEK) == 0)
			return -1;	
	}


	/* On some bugged systems (e.g. Siemens-Nixdorf) we need to check
	   the fd_sets cos result will always be 1 - oh for 
	   UNIX interoperatibility....  */
	if (FD_ISSET(socket, &reader) && FD_ISSET(socket, &writer)) {

		if (recv(socket, buffer, 1, MSG_PEEK) == 0)  
			return -1;
	}


	/* This wrapper around send() is needed to ensure that non 
	   blocking sockets always send an entire data stream */
        while (total != amount_to_send) {


		/* Can NEVER send 0 bytes, even if BSD non blocking. If no 
		   bytes can be sent when using a non blocking socket,
		   send() returns -1 and sets errno to EWOULDBLOCK */
                sent = send(socket, sendbuffer, len, 0);

		if (sent == -1)
			if (errno != EWOULDBLOCK) 
				return -1;

		if (sent > 0) {
                	total += sent;
                	sendbuffer += sent;
                	len -= sent;
		}
        }

	/* If got to this point then the entire data stream has been sent */
        return total;
}



int DBVtestsend(int socket, char *sendbuffer, int len)
{
	int sent = 0;

        if (len >= 10) {
                sent = send(socket, sendbuffer, 10, 0);
		printf("sent 10\n");
                return 10;
	}

	sent = send(socket, sendbuffer, len, 0);
	printf("sent %d\n", sent);
        return sent;
}



int DBVreceive_data_over_socket(DbvAssocId lAssocId, PDU *PDU_ptr, int socket, DbvSecs lSeconds, char **ber_data, unsigned long *ber_data_length, DbvError **ppsError)
{

	AsnLen decodedLen;

	long remainder = 0;
	long length = 0;
	long code = 0;
	char *ber_record = NULL;
	int nbytesRead = 0;

	/* Maximum size of each recv() */
	char recv_buffer[MAXMESSAGESIZE];

	int socketFd;
	int return_value = SI_OK;

	/* Structures used by select statement */
        int select_result;
        fd_set read_template;
	struct timeval TimeOut;
	struct timeval *pTimeOut = NULL;
	unsigned int count = 0;
	unsigned int read_all = 0;
	unsigned int total_bytes = 0;
	unsigned long recv_size = 0;
 

	socketFd = socket;
	memset(recv_buffer, 0, sizeof(recv_buffer));
        FD_ZERO(&read_template);
        FD_SET(socketFd, &read_template);

	if(lSeconds == CALL_ASYNCHRONOUS)
	{
		TimeOut.tv_usec=1000;
		TimeOut.tv_sec = 0;
		pTimeOut = &TimeOut;
	}
	if(lSeconds > 0)
	{
		TimeOut.tv_usec = 0;
		TimeOut.tv_sec = lSeconds;
		pTimeOut = &TimeOut;
	}
        select_result = select(FD_SETSIZE, &read_template, (fd_set *)0, 
		(fd_set *)0, pTimeOut);
 
	if (select_result < 0) 
	{
                /* Error occurred in select */
		if (errno > 0 && errno < sys_nerr)
			sprintf(DBVErrorBuffer,DBV_SYSERROR_S, "select() ", sys_errlist[errno]);
		else
			sprintf(DBVErrorBuffer,DBV_SYSERROR_S, "select() ", "");

		DBVfill_error_structure(0,SI_Communication_Error, 0, 
			DBV_SYSERROR, DBVErrorBuffer,0,
			Association_Unknown_State, ppsError);
                return_value = SI_NOTOK;
        }

	if(select_result == 0)
		/* Signifies a time out on a select! */
		return_value = TIMED_OUT;


	/* If there is something ready to read on socketFd */
	if (FD_ISSET(socketFd, &read_template) && return_value == SI_OK) 
	{

		/* PDU receive code which caters for both definite
		  and indefinite length encodings */

		ber_record = (char *) malloc (1);
		if (ber_record == NULL) {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                      	return SI_NOTOK;
                }

		read_all = 0;
		count = 0;
		total_bytes = 0;


		/* Care must be taken when receiving PDU data as
		   different targets/origins may send such data 
		   differently (the z3950.dra.com
		   target for example sends back a PDU in 512 byte 
		   packets) */ 
		  
		/* Read in 1 byte initially */
		recv_size = 1;

		while (!read_all) {

			nbytesRead = recv(socketFd, recv_buffer, recv_size, 0);


			/* nbytesRead should never be zero:
			   Blocking sockets return >0 or -1 if an error occurs.
			   BSD style non blocking sockets return >0 or 
			   -1 if an error occurs. 
			   If data is not available when a non blocking recv()
			   is attempted, recv() returns immediately with -1 and
			   sets errno to EWOULDBLOCK.
			   Non blocking sockets can never recv() zero bytes !!

			   NOTE HOWEVER:
			   If an abort (e.g. [CNTRL+C]) occurs at the other 
		  	   end of the virtual circuit whilst waiting in a 
			   select() with a socket in a read fd_set, 
			   the select() will return with 1, and all
			   subsequent recv()s will return instantly with
			   zero, irrespective of the socket being
			   blocking or non blocking */ 

			if (nbytesRead == 0) {

				if (ber_record != NULL)
					free(ber_record);		

				return DBVtcp_abort_or_release(lAssocId, 
					ppsError);
			}

			/* nbytesBytes will only be less than zero with error
				EWOULDBLOCK for a non blocking recv(). */
			if (nbytesRead < 0)
			{
				if(errno != EWOULDBLOCK)
				{
					DBVfill_error_structure(lAssocId,SI_Communication_Error, 0, DBV_ASSOCDEAD, DBV_ASSOCDEAD_S, 0, Association_Closed, ppsError);

					if (ber_record != NULL)
						free(ber_record);		
					return SI_NOTOK;
				}
			}

			if(nbytesRead >= 1)
			{
				total_bytes += nbytesRead;
				ber_record = (char *) realloc (ber_record, 
					total_bytes);
				if (ber_record == NULL) {
					DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                        		return SI_NOTOK;
                		}
	
				memcpy(ber_record + count, recv_buffer, 
					nbytesRead);  
				count += nbytesRead;
	
				if (IsCompleteBER(ber_record, count, 
					&remainder))
					read_all = 1;

				if (remainder < MAX_RECV)

					/* remainder will be 0 for definite 
					   lengths before the length field
					   has been received and always 0
					   for indefinite lengths */
					recv_size = remainder?remainder:1;
				else
					recv_size = MAX_RECV;
			}
		}


		/* Save the PDU BER received - it may be the BER of a PDU
                   not available in DBVOSI */
                *ber_data = (char *) malloc (count);
                if (*ber_data == NULL) {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0,
 DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                        return SI_NOTOK;
                }
                memcpy(*ber_data, ber_record, count);
                *ber_data_length = count;
 

		if (DBVdecode_bytes_to_pdu(lAssocId, ber_record, count, 
			PDU_ptr, ppsError) == SI_OK) {

			if (ber_record != NULL)
				free(ber_record);		
			return_value = SI_OK;
		} else {
			if (ber_record != NULL)
				free(ber_record);		
			return_value = SI_NOTOK;
			
		}
	} 

	return return_value;
}



int DBVtcp_abort_or_release(DbvAssocId lAssocId, DbvError **ppsError)
{
	DbvProtocolStack eProtocolStack;
	long lTargetState;
	DBVAssociateDetails *psAssociateDetails;


	if (SIWhichStack(lAssocId, &eProtocolStack, ppsError) == SI_OK) {
		psAssociateDetails = SL_Current(DBVAssociateList);

		/* If origin */
		if (psAssociateDetails->eOriginOrTarget == O) {
			/* Set up an abort error message */ 
			DBVfill_error_structure(lAssocId, SI_Abort, 0, DBV_ABORT, DBV_ABORT_S, 0, Association_Closed, ppsError);
			return SI_NOTOK;
			
		}
		/* else */
		else
		{
			lTargetState = DBVGetTargetState(lAssocId);
			if(lTargetState == DBV_TS_ASSOCIATION || 
				lTargetState == DBV_TS_OPEN)
			{
				/* Set up a release error message */
				return DBV_RELEASE_REQUEST;
			}
			else
			{
				DBVfill_error_structure(lAssocId, SI_Abort, 0, DBV_ABORT, DBV_ABORT_S, 0, Association_Closed, ppsError);
				return SI_NOTOK;

			}
		}
	}

	
	return SI_NOTOK;
}



#ifndef NO_OSI
int DBVreceive_data_over_OSI(DbvAssocId lAssocId, PDU *PDU_ptr, int lOSIAssocId, DbvSecs lSeconds, char **ber_data, unsigned long *ber_data_length, DbvError **ppsError)
{
	int return_value=SI_OK;

	long remainder = 0;
	int nfds;
	char *ber_record=NULL;
	long nbytesRead=0;
	fd_set read_template;
	int lResult;
	int lTimeout=OK;
	unsigned int count = 0;
	unsigned int read_all = 0;
	unsigned int total_bytes = 0;

	/* ISODE structures */
	struct AcSAPindication  *psAssocControlIndication=NULL;
	struct AcSAPabort *psAssocControlAbort;

	struct PSAPindication  *psPresentationIndication=NULL;
	struct PSAPabort *psPresentationAbort;
	struct PSAPdata *psPresentationData=NULL;

        DBVEnterFuncTrace("DBVreceive_data_over_OSI",lAssocId);


        if(DBVAllocCheck((psAssocControlIndication=(struct AcSAPindication *)
                DBVAlloc(sizeof(struct AcSAPindication), ppsError)))==SI_NOTOK)
                { return SI_NOTOK; }
        if(DBVAllocCheck((psPresentationIndication=(struct PSAPindication *)
                DBVAlloc(sizeof(struct PSAPindication), ppsError)))==SI_NOTOK)
                { return SI_NOTOK; }
        if(DBVAllocCheck((psPresentationData=(struct PSAPdata *)
                DBVAlloc(sizeof(struct PSAPdata), ppsError)))==SI_NOTOK)
                { return SI_NOTOK; }
        psPresentationAbort=&psPresentationIndication->pi_abort;
	psAssocControlAbort=&psAssocControlIndication->aci_abort;

	FD_ZERO(&read_template);

	/* Add error checking */
	lResult=PSelectMask(lOSIAssocId, &read_template, &nfds, 
		psPresentationIndication);
	
	if(lSeconds == CALL_ASYNCHRONOUS)
		lTimeout = OK;
	else if(lSeconds == CALL_BLOCKING)
		lTimeout = NOTOK;
	else
		lTimeout = lSeconds;

	lResult=xselect(nfds, &read_template, (fd_set *)NULL, (fd_set *)NULL,
		lTimeout);
	if(lResult==NOTOK)
	{
		/* xselect failed!! Most probably due to a signal */
		return_value = SI_NOTOK;
		sprintf(DBVErrorBuffer, DBV_OSI_ISODE_ERROR_S, "In function DBVreceive_data_over_OSI(): ISODE function xselect() failed\n");
		DBVfill_error_structure(lAssocId, SI_Communication_Error,
                        SI_ACSE_Provider, DBV_OSI_ISODE_ERROR,
                        DBVErrorBuffer, 0, Association_Open,ppsError);
	}
	if(lResult==OK)
		/* Signifies a time out on an xselect */
		return_value=TIMED_OUT;

	if(FD_ISSET(lOSIAssocId, &read_template) && return_value == SI_OK)
	{
		/* PDU receive code which caters for both definite
                  and indefinite length encodings */

		ber_record = (char *)malloc(1);
		if (ber_record == NULL) {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                      	return SI_NOTOK;
                }

		read_all=0;
		count=0;
		total_bytes=0;

		while(!read_all && return_value == SI_OK)
		{
		
			if((lResult=PReadRequest(lOSIAssocId,psPresentationData,
				NOTOK, psPresentationIndication)) != OK)
			{
				/* Check for Release Request */
				return_value = DBVPOSICheckISODEError(lAssocId,
					lOSIAssocId, psPresentationIndication, 
					ppsError);
				if(ber_record != NULL)
					free(ber_record);
			}
			else
			{
		/*
				printf("The Data is:%s\n", psPresentationData->px_info[0]->pe_un1.un_pe_prim);
				printf("The number of elements is %d\n", psPresentationData->px_ninfo);
		*/
				nbytesRead=psPresentationData->px_info[0]->
					pe_len;
				total_bytes += nbytesRead;
				ber_record = (char *)realloc(ber_record, 
					total_bytes);

				if (ber_record == NULL) {
		                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                        		return SI_NOTOK;
                		}


				memcpy(ber_record+count, psPresentationData->
					px_info[0]->pe_un1.un_pe_prim, 
					nbytesRead);
				count += nbytesRead;
			
				if(IsCompleteBER(ber_record, count, &remainder))
					read_all = 1;
			}
			PXFREE(psPresentationData);
		}


                /* Save the PDU BER received - it may be the BER of a PDU
                   not available in DBVOSI */
                *ber_data = (char *) malloc (count);
		if (*ber_data == NULL) {
			DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsError);
                       	return SI_NOTOK;
                }
                memcpy(*ber_data, ber_record, count);
                *ber_data_length = count;
 
		
		if(return_value == SI_OK)
		{
		  if (DBVdecode_bytes_to_pdu(lAssocId, ber_record, count,
				PDU_ptr, ppsError) == SI_OK) {

			if (ber_record != NULL)
				free(ber_record);
			return_value = SI_OK;
		  } else {
			if (ber_record != NULL)
				free(ber_record);
			return_value = SI_NOTOK;
		  }
		}
	}
	PAFREE(psPresentationAbort);
	ACAFREE(psAssocControlAbort);
	DBVFree(psAssocControlIndication);
	DBVFree(psPresentationIndication);
	DBVFree(psPresentationData);

        DBVLeaveFuncTrace("DBVreceive_data_over_OSI",lAssocId, return_value);

	return return_value;
}
#endif

