/*  Module Name: @(#)z3950_shared.c	7.6
    Version: 12/6/96
    Author: John Hough
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Some Useful DBV-OSI II Routines Shared By Origin And
		Target 

*/

#include "z3950_shared.h"

/* Convert all characters in a string to upper case */
void DBVstring_to_up_case(char *input)
{
 
        /* Convert all but \0 char */
        while (*input != '\0'){
                *input = toupper(*input);
                input++;
        }
 
 
}



/* DBVget_token_from_string -
   This function looks for the next delimited token in a string. If one exists
   the token and its following delimiter are removed from the string and both
   the token and the updated string are returned.
	
   If no token is found, NO SUCH TOKEN is returned as the token and the
   original string is returned.
 
   The function assumes that the length of the input string is <= STRING_LEN.
 
   Parameters:
   Input  token_delmimiter   token separator
          string             string containing tokens and delimiters. The
			     string must end in a delimiter or a segmentation
			     fault will occur.
 
   Output  token_obtained    next token found or NO SUCH TOKEN
           string            string updated (token and delimiter removed) only 				     if token is found

*/

void DBVget_token_from_string(char *token_obtained, char *token_delimiter, char *string)
{
 
        char temp[STRING_LEN];
        char temp2[STRING_LEN];
        char *strptr = NULL;
 
	/* Make 2 copies of string, one for strtok, one for strbrk */
        strcpy(temp, string);
        strcpy(temp2, string);
 
        /* See if a token exists */
        strptr = strtok(temp, token_delimiter);

	/* If one does */
        if (strptr != NULL) {

		/* Copy the token to token_obtained */ 
                strcpy(token_obtained, strptr);
 
		/* Adjust string */

        	/* Check if a delimiter follows the token */
        	strptr = strstr(temp2, token_delimiter);
 
		/* If one does */
		if (strptr != NULL)		

			/* Copy from after the token delimiter */
                	strcpy(string, strptr+strlen(token_delimiter));
		else 
			/* If we have just processed the last token 
			   in the string make the string empty */ 
			strcpy(string, "");
	} else
		strcpy(token_obtained, "NO SUCH TOKEN");
}



void DBVdbv_to_snacc_DbvOctetString(AsnOcts *snacc_asnocts, DbvOctetString *dbv_octet_string, jmp_buf env)
{
	DBVdbv_to_snacc_octet_string(snacc_asnocts, dbv_octet_string, env);
}



/* Set up snacc Asnocts from DBV OctetString */
void DBVdbv_to_snacc_octet_string(AsnOcts *snacc_asnocts, DbvOctetString *dbv_octet_string, jmp_buf env)
{
	snacc_asnocts->octs = (char *) Asn1Alloc (1);
	CheckAsn1Alloc(snacc_asnocts->octs, env);

	/* If the dbv string is NULL return without doing anything */
	if (dbv_octet_string == NULL) 
		return;

	/* Set nothing if the dbv string has a zero ulLength or 
	   NULL pcValue */
	 if ((dbv_octet_string->ulLength == 0) || (dbv_octet_string->pcValue == NULL))
		return;

	snacc_asnocts->octetLen = dbv_octet_string->ulLength;
	DbvFree(snacc_asnocts->octs);
	snacc_asnocts->octs = (char *) Asn1Alloc (dbv_octet_string->ulLength);
	CheckAsn1Alloc(snacc_asnocts->octs, env);
	memcpy(snacc_asnocts->octs,
		dbv_octet_string->pcValue,
		dbv_octet_string->ulLength);
}
 


int DBVsnacc_to_dbv_DbvOctetString(DbvOctetString *dbv_octet_string, AsnOcts *snacc_asnocts, jmp_buf env)
{
	DBVsnacc_to_dbv_octet_string(dbv_octet_string, snacc_asnocts, env);
}



/* Set DBV OctetString from snacc AsnOcts */
int DBVsnacc_to_dbv_octet_string(DbvOctetString *dbv_octet_string, AsnOcts *snacc_asnocts, jmp_buf env)
{
	dbv_octet_string->ulLength = snacc_asnocts->octetLen;
	dbv_octet_string->pcValue = (char *) DbvAlloc (snacc_asnocts->octetLen);
	CheckDbvAlloc(dbv_octet_string->pcValue, env);
	memcpy(dbv_octet_string->pcValue, snacc_asnocts->octs, snacc_asnocts->octetLen);
}
 


void DBVdbv_to_snacc_DbvInternationalString(InternationalString *snacc_interstr, DbvInternationalString *dbv_interstr, jmp_buf env)
{
	DBVdbv_to_snacc_inter_string(snacc_interstr, dbv_interstr, env);
}



/* Set up snacc InternationalString from DBV DbvInternationalString */
void DBVdbv_to_snacc_inter_string(InternationalString *snacc_interstr, DbvInternationalString *dbv_interstr, jmp_buf env)
{

	/* If the dbv string is NULL return without doing anything */
	if (dbv_interstr == NULL)
		return;

	/* Set nothing if the dbv string has a zero ulLength or 
	   NULL pcValue */
	 if ((dbv_interstr->ulLength == 0) || (dbv_interstr->pcValue == NULL))
		return;

	snacc_interstr->octetLen = dbv_interstr->ulLength;
	snacc_interstr->octs = (char *)Asn1Alloc(dbv_interstr->ulLength);
	CheckAsn1Alloc(snacc_interstr->octs, env);
	memcpy(snacc_interstr->octs,
		dbv_interstr->pcValue,
		dbv_interstr->ulLength);

}



int DBVsnacc_to_dbv_DbvInternationalString(DbvInternationalString *dbv_interstr, InternationalString *snacc_interstr, jmp_buf env)
{
	DBVsnacc_to_dbv_inter_string(dbv_interstr, snacc_interstr, env);
}



/* Set DBV DbvInternationalString from snacc InternationalString */
int DBVsnacc_to_dbv_inter_string(DbvInternationalString *dbv_interstr, InternationalString *snacc_interstr, jmp_buf env)

{
	dbv_interstr->ulLength = snacc_interstr->octetLen;
	dbv_interstr->pcValue = (char *) DbvAlloc (snacc_interstr->octetLen);
	CheckDbvAlloc(dbv_interstr->pcValue, env);

	memcpy(dbv_interstr->pcValue, snacc_interstr->octs, snacc_interstr->octetLen);
}
 


void DBVdbv_to_snacc_DbvVisibleString(VisibleString *snacc_visiblestr, DbvVisibleString dbv_visiblestr, jmp_buf env)
{
	DBVdbv_to_snacc_vis_string(snacc_visiblestr, dbv_visiblestr, env);
}



/* Set up snacc VisibleString from DBV DbvVisibleString */
void DBVdbv_to_snacc_vis_string(VisibleString *snacc_visiblestr, DbvVisibleString dbv_visiblestr, jmp_buf env)
{
	char *string = NULL;
	unsigned long length = 0;

	/* Set nothing if the dbv string is NULL */
	 if (dbv_visiblestr == NULL)
		return;

	/* Cant do inter and octet style ->length and ->pcValue checking
	   here because DbvVisibleString is a typedefed char * ! */

	string = dbv_visiblestr;
	length = (strlen(dbv_visiblestr));

	snacc_visiblestr->octetLen = length;
	snacc_visiblestr->octs = (char *)Asn1Alloc(length + 1);
	CheckAsn1Alloc(snacc_visiblestr->octs, env);
	memcpy(snacc_visiblestr->octs, dbv_visiblestr, length);

}



int DBVsnacc_to_dbv_DbvVisibleString(char **dbv_visiblestr, VisibleString *snacc_visiblestr, jmp_buf env)
{
	DBVsnacc_to_dbv_vis_string(dbv_visiblestr, snacc_visiblestr, env);
}



/* Set DBV DbvVisibleString from snacc VisibleString */
int DBVsnacc_to_dbv_vis_string(char **dbv_visiblestr, VisibleString *snacc_visiblestr, jmp_buf env)
{

	char *string = NULL;
	unsigned long length = 0;

	string = snacc_visiblestr->octs;
	length = snacc_visiblestr->octetLen;

	*dbv_visiblestr = (char *) DbvAlloc(length + 1);
	CheckDbvAlloc(*dbv_visiblestr, env);

	memcpy(*dbv_visiblestr, string, length);
}




/* A DBV bit string must only contains 0s or 1s, anything else in the
   string is invalid.
   Example valid bit string ( char *):
   "010001000"
*/
int DBVcheck_bit_string(char  *bit_string)
{
	int i;
 
	if (bit_string == NULL)
		return SI_NOTOK;

	for (i = 0; i < strlen(bit_string); i++)
		if ( (bit_string[i] != '0') && (bit_string[i] != '1') )
			return SI_NOTOK;
 
	return SI_OK;
}



/* Check that the InitRequest Options field specified by the user 
   only contains options allowed by the
   API. Allowed options are specified by the ALLOWED_OPTIONS #define
   in ../headers/dbv_proto.h */
int DBVcheck_chosen_options(char  *bit_string)
{
	int i;
	char *allowed_opts = ALLOWED_OPTIONS;
/* Old code  
	for (i = 0; i < strlen(allowed_opts); i++) {

		** If not at end of InitRequest options bit string **
		if (i < (strlen(bit_string)) )

			if ( (bit_string[i] == '1') && (allowed_opts[i] != '1') )
				return SI_NOTOK;
	}
*/
/* New code rejects any 1's after the last bit specified by ALLOWED_OPTIONS */
	for (i = 0; i < strlen(bit_string); i++) {

		if( i < (strlen(allowed_opts)) ) {
			if( (bit_string[i] == '1') && (allowed_opts[i] != '1'))
				return SI_NOTOK;
		} else {
			if( (bit_string[i] == '1') )
				return SI_NOTOK;
		}
		
	}
	return SI_OK;
}







/* Set snacc ElementSetNames structure from DBV DbvElementSetNames structure  */
int DBVdbv_to_snacc_el_set_names(struct ElementSetNames* simple, DbvElementSetNames *psElementSetNames, jmp_buf env)
{
	switch(psElementSetNames->eElementSetType) {
		case EST_Generic:
			DBVdbv_to_snacc_el_generic(simple, psElementSetNames, env);
			break;
		case EST_DatabaseSpecific:
			DBVdbv_to_snacc_el_specific(simple, psElementSetNames, env);
			break;
	}
}



int DBVdbv_to_snacc_el_generic(struct ElementSetNames* simple, DbvElementSetNames *psElementSetNames, jmp_buf env)
{
	simple->choiceId = ELEMENTSETNAMES_GENERICELEMENTSETNAME;
	simple->a.genericElementSetName = (InternationalString *) Asn1Alloc (sizeof (InternationalString));
	CheckAsn1Alloc(simple->a.genericElementSetName, env);

	DBVdbv_to_snacc_inter_string(simple->a.genericElementSetName, psElementSetNames->uElementSet.psGeneric, env);
}



int DBVdbv_to_snacc_el_specific(struct ElementSetNames* simple, DbvElementSetNames *psElementSetNames, jmp_buf env)
{
        ElementSetNamesSeqOfSeq *data_ptr;
        ElementSetNamesSeqOfSeq**data_ptr_address;
        int current_listnode = 0;

	simple->choiceId = ELEMENTSETNAMES_DATABASESPECIFIC;
		
        /* If the Dbv list is not NULL (the field is optional) */
     	if (psElementSetNames->uElementSet.ppsElementSetsDbSpecific != NULL) {
 
		/* Create a snacc list */
	
               	simple->a.databaseSpecific = AsnListNew(sizeof( void *));
		CheckAsn1Alloc(simple->a.databaseSpecific, env);
 
               	/* while not at end of Dbv List */
               	while (psElementSetNames->uElementSet.ppsElementSetsDbSpecific[current_listnode] != NULL) {
			/* create a snacc list node */
			data_ptr_address = AsnListAppend(simple->a.databaseSpecific);
			CheckAsn1Alloc(data_ptr_address, env);

               		*data_ptr_address = Asn1Alloc(sizeof(struct ElementSetNamesSeqOfSeq));
			CheckAsn1Alloc(*data_ptr_address, env);

               		data_ptr = *data_ptr_address;

			DBVdbv_to_snacc_inter_string(&data_ptr->dbName, psElementSetNames->uElementSet.ppsElementSetsDbSpecific[current_listnode]->psDatabaseName, env); 
			DBVdbv_to_snacc_inter_string(&data_ptr->esn, psElementSetNames->uElementSet.ppsElementSetsDbSpecific[current_listnode]->psElementSetName, env); 
			current_listnode++;
		}
	}
}



int DBVdbv_to_snacc_records(struct Records* records, DbvRecords *psRecords, jmp_buf env)
{
	switch (psRecords->eRecordType) {
	case RT_Response:
		records->choiceId = RECORDS_RESPONSERECORDS;
		records->a.responseRecords = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(records->a.responseRecords, env);

		DBVdbv_to_snacc_resprec(records->a.responseRecords, psRecords->uRecord.ppsResponseRecord, env);
		break;
	case RT_NonSurrDiagnostics:
		records->choiceId = RECORDS_NONSURROGATEDIAGNOSTIC;
		records->a.nonSurrogateDiagnostic = (struct DefaultDiagFormat*) Asn1Alloc (sizeof(struct DefaultDiagFormat));
		CheckAsn1Alloc(records->a.nonSurrogateDiagnostic, env);

		DBVdbv_to_snacc_defaultformat(records->a.nonSurrogateDiagnostic, psRecords->uRecord.psNonSurrogateDiag, env);
		
		break;
	case RT_MultipleNonSurrDiagnostics:
		records->choiceId = RECORDS_MULTIPLENONSURDIAGNOSTICS;
		records->a.multipleNonSurDiagnostics = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(records->a.multipleNonSurDiagnostics, env);

		DBVdbv_to_snacc_multdiag(records->a.multipleNonSurDiagnostics, psRecords->uRecord.ppsMultipleNonSurrogateDiag, env);
		break;
	}
}



int DBVdbv_to_snacc_resprec(RecordsSeqOf *responseRecords, DbvNamePlusRecord **ppsResponseRecord, jmp_buf env)
{
	
        NamePlusRecord *data_ptr;
        NamePlusRecord **data_ptr_address;
 
        int current_listnode = 0;
 
        while(ppsResponseRecord[current_listnode] != NULL)
        {
                data_ptr_address = AsnListAppend(responseRecords);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(NamePlusRecord));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_nameplusrec(data_ptr, ppsResponseRecord[current_listnode], env);
                current_listnode++;
        }
 

}



int DBVdbv_to_snacc_nameplusrec(NamePlusRecord *data_ptr, DbvNamePlusRecord *psResponseRecord, jmp_buf env)
{
	
	/* name is optional */
	if (psResponseRecord->psDatabaseName != NULL)
		DBVdbv_to_snacc_inter_string(& (data_ptr->name), psResponseRecord->psDatabaseName, env);
	
	/* Extra snacc structure here */
	data_ptr->record = (struct NamePlusRecordChoice*) Asn1Alloc (sizeof(struct NamePlusRecordChoice));
	CheckAsn1Alloc(data_ptr->record, env);
	

	/* record is compulsory */
	switch (psResponseRecord->eDbRecordType) {
	case DRT_Retrieval:
		data_ptr->record->choiceId = NAMEPLUSRECORDCHOICE_RETRIEVALRECORD;
		data_ptr->record->a.retrievalRecord = (struct NEWEXTERNAL*) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(data_ptr->record->a.retrievalRecord, env);

		DBVdbv_to_snacc_external(data_ptr->record->a.retrievalRecord, psResponseRecord->uDbRecord.psRetrievalRecord, env);
		break;
	case DRT_SurrogateDiagnostics:
		data_ptr->record->choiceId = NAMEPLUSRECORDCHOICE_SURROGATEDIAGNOSTIC;
		data_ptr->record->a.surrogateDiagnostic = (struct DiagRec*) Asn1Alloc (sizeof(struct DiagRec));
		CheckAsn1Alloc(data_ptr->record->a.surrogateDiagnostic, env);

		DBVdbv_to_snacc_diag_rec(data_ptr->record->a.surrogateDiagnostic, psResponseRecord->uDbRecord.psSurrogateDiagnostic, env);
		break;
	case DRT_StartingFragment:
		data_ptr->record->choiceId = NAMEPLUSRECORDCHOICE_STARTINGFRAGMENT;
		data_ptr->record->a.startingFragment = (struct FragmentSyntax*) Asn1Alloc (sizeof(struct FragmentSyntax));
		CheckAsn1Alloc(data_ptr->record->a.startingFragment, env);

		DBVdbv_to_snacc_fragsyntax(data_ptr->record->a.startingFragment, psResponseRecord->uDbRecord.psStartingFragment, env); 
		break;
	case DRT_IntermediateFragment:
		data_ptr->record->choiceId = NAMEPLUSRECORDCHOICE_INTERMEDIATEFRAGMENT;
		data_ptr->record->a.intermediateFragment = (struct FragmentSyntax*) Asn1Alloc (sizeof(struct FragmentSyntax));
		CheckAsn1Alloc(data_ptr->record->a.intermediateFragment, env);

		DBVdbv_to_snacc_fragsyntax(data_ptr->record->a.intermediateFragment, psResponseRecord->uDbRecord.psIntermediateFragment, env); 
		break;
	case DRT_FinalFragment:
		data_ptr->record->choiceId = NAMEPLUSRECORDCHOICE_FINALFRAGMENT;
		data_ptr->record->a.finalFragment = (struct FragmentSyntax*) Asn1Alloc (sizeof(struct FragmentSyntax));
		CheckAsn1Alloc(data_ptr->record->a.finalFragment, env);

		DBVdbv_to_snacc_fragsyntax(data_ptr->record->a.finalFragment, psResponseRecord->uDbRecord.psFinalFragment, env); 
		break;
	}
}



int DBVdbv_to_snacc_diag_rec(struct DiagRec *surrogateDiagnostic, DbvDiagRec *psSurrogateDiagnostic, jmp_buf env)
{
	switch (psSurrogateDiagnostic->eDiagnosticsType) {
	case DT_DefaultFormat:
		surrogateDiagnostic->choiceId = DIAGREC_DEFAULTFORMAT;
		surrogateDiagnostic->a.defaultFormat = (struct DefaultDiagFormat*) Asn1Alloc (sizeof(struct DefaultDiagFormat));
		CheckAsn1Alloc(surrogateDiagnostic->a.defaultFormat, env);

		DBVdbv_to_snacc_defaultformat(surrogateDiagnostic->a.defaultFormat, psSurrogateDiagnostic->uDiagnostics.psDefault, env);
		break;
	case DT_ExternallyDefined:
		surrogateDiagnostic->choiceId = DIAGREC_EXTERNALLYDEFINED;
		surrogateDiagnostic->a.externallyDefined = (struct NEWEXTERNAL*) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(surrogateDiagnostic->a.externallyDefined, env);

		DBVdbv_to_snacc_external(surrogateDiagnostic->a.externallyDefined, psSurrogateDiagnostic->uDiagnostics.psExternallyDefined, env);
		
		break;
	}
}



int DBVdbv_to_snacc_defaultformat(struct DefaultDiagFormat* defaultFormat, DbvDefaultDiagFormat *psDefault, jmp_buf env)
{

	ASN_MakeOid((char *) psDefault->pcDiagnosticSetId, &defaultFormat->diagnosticSetId, env);

	defaultFormat->condition = psDefault->ulCondition;

	defaultFormat->addinfo = (struct DefaultDiagFormatChoice*) Asn1Alloc (sizeof (struct DefaultDiagFormatChoice));
	CheckAsn1Alloc(defaultFormat->addinfo, env);


	DBVdbv_to_snacc_addinfo(defaultFormat->addinfo, psDefault->sAdditionalInfo, env);
}



int DBVdbv_to_snacc_addinfo(struct DefaultDiagFormatChoice* addinfo, DbvAdditionalInfo sAdditionalInfo, jmp_buf env)
{

	switch (sAdditionalInfo.eProtocolVersionInForce) {
	case PVIF_Version2:
		addinfo->choiceId = DEFAULTDIAGFORMATCHOICE_V2ADDINFO;
		addinfo->a.v2Addinfo = (VisibleString *) Asn1Alloc (sizeof(VisibleString));
		CheckAsn1Alloc(addinfo->a.v2Addinfo, env);

		/* Cant use DBVdbv_to_snacc_octet_string here becuase 
		   sAdditionalInfo.uAdditionalInfo.pcV2 is a char*, 
		   not an octet string */
		addinfo->a.v2Addinfo->octetLen = strlen(sAdditionalInfo.uAdditionalInfo.pcV2);
		addinfo->a.v2Addinfo->octs = (char *) Asn1Alloc (strlen(sAdditionalInfo.uAdditionalInfo.pcV2));
		CheckAsn1Alloc(addinfo->a.v2Addinfo->octs, env);

		memcpy(addinfo->a.v2Addinfo->octs, sAdditionalInfo.uAdditionalInfo.pcV2, strlen(sAdditionalInfo.uAdditionalInfo.pcV2));
		break;
	case PVIF_Version3:
		addinfo->choiceId = DEFAULTDIAGFORMATCHOICE_V3ADDINFO;
		addinfo->a.v3Addinfo = (InternationalString *) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(addinfo->a.v3Addinfo, env);

		DBVdbv_to_snacc_inter_string(addinfo->a.v3Addinfo, sAdditionalInfo.uAdditionalInfo.psV3, env);
		
		break;
	}
}



int DBVdbv_to_snacc_fragsyntax(struct FragmentSyntax *snaccfragment, DbvFragmentSyntax *dbvfragment, jmp_buf env)
{
	switch (dbvfragment->eFragmentSyntaxType) {
	case FST_ExternallyTagged:
		snaccfragment->choiceId = FRAGMENTSYNTAX_EXTERNALLYTAGGED;
		snaccfragment->a.externallyTagged = (struct NEWEXTERNAL *) Asn1Alloc (sizeof (struct NEWEXTERNAL));
		CheckAsn1Alloc(snaccfragment->a.externallyTagged, env);

		DBVdbv_to_snacc_external(snaccfragment->a.externallyTagged, dbvfragment->uFragmentSyntax.psExternallyTagged, env);
		break;
	case FST_NotExternallyTagged:
		snaccfragment->choiceId = FRAGMENTSYNTAX_NOTEXTERNALLYTAGGED;
		snaccfragment->a.notExternallyTagged = (AsnOcts *) Asn1Alloc (sizeof (AsnOcts));
		CheckAsn1Alloc(snaccfragment->a.notExternallyTagged, env);

		DBVdbv_to_snacc_octet_string(snaccfragment->a.notExternallyTagged, dbvfragment->uFragmentSyntax.psNotExternallyTagged, env);

		break;
	}
}



int DBVdbv_to_snacc_multdiag(RecordsSeqOf1 *multipleNonSurDiagnostics, DbvDiagRec **ppsMultipleNonSurrogateDiag, jmp_buf env)
{
        DiagRec *data_ptr;
        DiagRec **data_ptr_address;
 
        int current_listnode = 0;
 
        while(ppsMultipleNonSurrogateDiag[current_listnode] != NULL)
        {
                data_ptr_address = AsnListAppend(multipleNonSurDiagnostics);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(DiagRec));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
		DBVdbv_to_snacc_diag_rec(data_ptr, ppsMultipleNonSurrogateDiag[current_listnode], env);
                current_listnode++;
        }
}



int DBVsnacc_to_dbv_records(DbvRecords *psRecords, struct Records* records, jmp_buf env)
{
	int num_elmnts = 0;

	switch (records->choiceId) {
	case RECORDS_RESPONSERECORDS:
		psRecords->eRecordType = RT_Response;
		num_elmnts = AsnListCount(records->a.responseRecords);
		psRecords->uRecord.ppsResponseRecord = (DbvNamePlusRecord **) DbvAlloc (sizeof(DbvNamePlusRecord *) * (num_elmnts + 1));
		CheckDbvAlloc(psRecords->uRecord.ppsResponseRecord, env);

		DBVsnacc_to_dbv_resprec(psRecords->uRecord.ppsResponseRecord, records->a.responseRecords, env);
		break;
	case RECORDS_NONSURROGATEDIAGNOSTIC:
		psRecords->eRecordType = RT_NonSurrDiagnostics;
		psRecords->uRecord.psNonSurrogateDiag = (DbvDefaultDiagFormat *) DbvAlloc (sizeof(struct DbvDefaultDiagFormat)); 
		CheckDbvAlloc(psRecords->uRecord.psNonSurrogateDiag, env);

		DBVsnacc_to_dbv_defaultformat(psRecords->uRecord.psNonSurrogateDiag, records->a.nonSurrogateDiagnostic, env);
		break;
	case RECORDS_MULTIPLENONSURDIAGNOSTICS:
		psRecords->eRecordType = RT_MultipleNonSurrDiagnostics;
		num_elmnts = AsnListCount(records->a.multipleNonSurDiagnostics);
		psRecords->uRecord.ppsMultipleNonSurrogateDiag = (DbvDiagRec **) DbvAlloc (sizeof(DbvDiagRec *) * (num_elmnts + 1));
		CheckDbvAlloc(psRecords->uRecord.ppsMultipleNonSurrogateDiag, env);

		DBVsnacc_to_dbv_multdiag(psRecords->uRecord.ppsMultipleNonSurrogateDiag, records->a.multipleNonSurDiagnostics, env);
		break;
	}
}



int DBVsnacc_to_dbv_resprec(DbvNamePlusRecord **ppsResponseRecord, RecordsSeqOf *responseRecords, jmp_buf env)
{
        NamePlusRecord *data_ptr;
	int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, responseRecords)
        {
                ppsResponseRecord[index] = (DbvNamePlusRecord *) DbvAlloc (sizeof(DbvNamePlusRecord));
		CheckDbvAlloc(ppsResponseRecord[index], env);

		DBVsnacc_to_dbv_nameplusrec(ppsResponseRecord[index], data_ptr, env);
                index++;
        }
        ppsResponseRecord[index]=NULL;

}



int DBVsnacc_to_dbv_nameplusrec(DbvNamePlusRecord *psResponseRecord, NamePlusRecord *data_ptr, jmp_buf env)
{
	
	/* psDatabaseName is optional */
	if (data_ptr->name.octetLen != 0) {
		psResponseRecord->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof(struct DbvInternationalString));
		CheckDbvAlloc(psResponseRecord->psDatabaseName, env);

		DBVsnacc_to_dbv_inter_string(psResponseRecord->psDatabaseName, & (data_ptr->name), env);
	}
	

	/* record is compulsory */
	switch (data_ptr->record->choiceId) {
	case NAMEPLUSRECORDCHOICE_RETRIEVALRECORD:
		psResponseRecord->eDbRecordType = DRT_Retrieval;
		psResponseRecord->uDbRecord.psRetrievalRecord = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(psResponseRecord->uDbRecord.psRetrievalRecord, env);
		DBVsnacc_to_dbv_external(psResponseRecord->uDbRecord.psRetrievalRecord, data_ptr->record->a.retrievalRecord, env);
		break;
	case NAMEPLUSRECORDCHOICE_SURROGATEDIAGNOSTIC:
		psResponseRecord->eDbRecordType = DRT_SurrogateDiagnostics;
		psResponseRecord->uDbRecord.psSurrogateDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof(DbvDiagRec));
		CheckDbvAlloc(psResponseRecord->uDbRecord.psSurrogateDiagnostic, env);

		DBVsnacc_to_dbv_diag_rec(psResponseRecord->uDbRecord.psSurrogateDiagnostic, data_ptr->record->a.surrogateDiagnostic, env);
		break;
	case NAMEPLUSRECORDCHOICE_STARTINGFRAGMENT:
		psResponseRecord->eDbRecordType = DRT_StartingFragment;
		psResponseRecord->uDbRecord.psStartingFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof(DbvFragmentSyntax));
		CheckDbvAlloc(psResponseRecord->uDbRecord.psStartingFragment, env);

		DBVsnacc_to_dbv_fragsyntax(psResponseRecord->uDbRecord.psStartingFragment, data_ptr->record->a.startingFragment, env);
		break;
	case NAMEPLUSRECORDCHOICE_INTERMEDIATEFRAGMENT:
		psResponseRecord->eDbRecordType = DRT_IntermediateFragment;
		psResponseRecord->uDbRecord.psIntermediateFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof(DbvFragmentSyntax));
		CheckDbvAlloc(psResponseRecord->uDbRecord.psIntermediateFragment, env);

		DBVsnacc_to_dbv_fragsyntax(psResponseRecord->uDbRecord.psIntermediateFragment, data_ptr->record->a.intermediateFragment, env);
		break;
	case NAMEPLUSRECORDCHOICE_FINALFRAGMENT:
		psResponseRecord->eDbRecordType = DRT_FinalFragment;
		psResponseRecord->uDbRecord.psFinalFragment = (DbvFragmentSyntax *) DbvAlloc (sizeof(DbvFragmentSyntax));
		CheckDbvAlloc(psResponseRecord->uDbRecord.psFinalFragment, env);

		DBVsnacc_to_dbv_fragsyntax(psResponseRecord->uDbRecord.psFinalFragment, data_ptr->record->a.finalFragment, env);
		break;
	}
}



int DBVsnacc_to_dbv_diag_rec(DbvDiagRec *psSurrogateDiagnostic, struct DiagRec *surrogateDiagnostic, jmp_buf env)
{

	switch (surrogateDiagnostic->choiceId) {
	case DIAGREC_DEFAULTFORMAT:
		psSurrogateDiagnostic->eDiagnosticsType = DT_DefaultFormat;
		psSurrogateDiagnostic->uDiagnostics.psDefault = (DbvDefaultDiagFormat *) DbvAlloc (sizeof(DbvDefaultDiagFormat));
		CheckDbvAlloc(psSurrogateDiagnostic->uDiagnostics.psDefault, env);

		DBVsnacc_to_dbv_defaultformat(psSurrogateDiagnostic->uDiagnostics.psDefault, surrogateDiagnostic->a.defaultFormat, env);
		break;
	case DIAGREC_EXTERNALLYDEFINED:
		psSurrogateDiagnostic->eDiagnosticsType = DT_ExternallyDefined;
		psSurrogateDiagnostic->uDiagnostics.psExternallyDefined = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(psSurrogateDiagnostic->uDiagnostics.psExternallyDefined, env);

		DBVsnacc_to_dbv_external(psSurrogateDiagnostic->uDiagnostics.psExternallyDefined, surrogateDiagnostic->a.externallyDefined, env);
		break;
	}
}



int DBVsnacc_to_dbv_defaultformat(DbvDefaultDiagFormat *psDefault, struct DefaultDiagFormat* defaultFormat, jmp_buf env)
{
	char *oid;

	oid = (char *) ASN_EncodedToString(&defaultFormat->diagnosticSetId);
        psDefault->pcDiagnosticSetId = (DbvObjectIdentifier) DbvAlloc (strlen(oid) +1);;
	CheckDbvAlloc(psDefault->pcDiagnosticSetId, env);

	memcpy(psDefault->pcDiagnosticSetId, oid, strlen(oid));
        psDefault->ulCondition = defaultFormat->condition;
 
        DBVsnacc_to_dbv_addinfo(&psDefault->sAdditionalInfo, defaultFormat->addinfo, env);

}



int DBVsnacc_to_dbv_addinfo(DbvAdditionalInfo *psAdditionalInfo, struct DefaultDiagFormatChoice* addinfo, jmp_buf env)
{
 
        switch (addinfo->choiceId) {
	case DEFAULTDIAGFORMATCHOICE_V2ADDINFO:
		psAdditionalInfo->eProtocolVersionInForce = PVIF_Version2;
		psAdditionalInfo->uAdditionalInfo.pcV2 = (DbvVisibleString) DbvAlloc ((addinfo->a.v2Addinfo->octetLen) + 1);
		CheckDbvAlloc(psAdditionalInfo->uAdditionalInfo.pcV2, env);

		memcpy(psAdditionalInfo->uAdditionalInfo.pcV2, addinfo->a.v2Addinfo->octs, addinfo->a.v2Addinfo->octetLen);
		break;
	case DEFAULTDIAGFORMATCHOICE_V3ADDINFO:
		psAdditionalInfo->eProtocolVersionInForce = PVIF_Version3;
		psAdditionalInfo->uAdditionalInfo.psV3 = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psAdditionalInfo->uAdditionalInfo.psV3, env);

		DBVsnacc_to_dbv_inter_string(psAdditionalInfo->uAdditionalInfo.psV3, addinfo->a.v3Addinfo, env);
                break;
        }
}



int DBVsnacc_to_dbv_fragsyntax(DbvFragmentSyntax *dbvfragment, struct FragmentSyntax* snaccfragment, jmp_buf env)
{
	switch (snaccfragment->choiceId) {
	case FRAGMENTSYNTAX_EXTERNALLYTAGGED:
		dbvfragment->eFragmentSyntaxType = FST_ExternallyTagged;
		dbvfragment->uFragmentSyntax.psExternallyTagged = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(dbvfragment->uFragmentSyntax.psExternallyTagged, env);

		DBVsnacc_to_dbv_external(dbvfragment->uFragmentSyntax.psExternallyTagged, snaccfragment->a.externallyTagged, env);
		break;
	case FRAGMENTSYNTAX_NOTEXTERNALLYTAGGED:
		dbvfragment->eFragmentSyntaxType = FST_NotExternallyTagged;
		dbvfragment->uFragmentSyntax.psNotExternallyTagged = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(dbvfragment->uFragmentSyntax.psNotExternallyTagged, env);

		DBVsnacc_to_dbv_octet_string(dbvfragment->uFragmentSyntax.psNotExternallyTagged, snaccfragment->a.notExternallyTagged, env);
		break;
	}
}



int DBVsnacc_to_dbv_multdiag(DbvDiagRec **ppsMultipleNonSurrogateDiag, RecordsSeqOf1 *multipleNonSurDiagnostics, jmp_buf env)
{
        DiagRec *data_ptr;
	int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, multipleNonSurDiagnostics)
        {
                ppsMultipleNonSurrogateDiag[index] = (DbvDiagRec *) DbvAlloc (sizeof(DbvDiagRec));
		CheckDbvAlloc(ppsMultipleNonSurrogateDiag[index], env);
		DBVsnacc_to_dbv_diag_rec(ppsMultipleNonSurrogateDiag[index], data_ptr, env);
                index++;
        }
        ppsMultipleNonSurrogateDiag[index]=NULL;
}



int DBVcheck_dbv_enum(unsigned long field, unsigned long bottom_val, unsigned long top_val)
{

	/* If field is in range bottom_val -> top_val inclusive */ 
	if ((field >= bottom_val) && (field <= top_val) )
		return SI_OK;
	else
		return SI_NOTOK;
}



int DBVdbv_field_missing_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError)
{

	/* Following requires DBV_PDU_NULLFIELD_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVsingle_field_error(lAssocId, field_in_error, ppsSIError, DBV_PDU_NULLFIELD, DBV_PDU_NULLFIELD_S));
}



int DBVsingle_field_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError, int error_code, char *error_diag)
{
	char field_buffer[SA_ARRAY_SIZE];
	unsigned long room = 0;

	DBVsa_strcpy(field_buffer, field_in_error);

	room = SA_ARRAY_SIZE - strlen(error_diag);

	/* If cant fit in error buffer */
	if (strlen(field_buffer) >= room) 
		DBVsa_strcpy(field_buffer, field_in_error + (strlen(field_buffer) - room));

	sprintf(DBVErrorBuffer, error_diag, field_buffer);
        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, error_code,  DBVErrorBuffer, 0, Association_Open, ppsSIError);
        return SI_NOTOK;
}



int DBVdbv_bit_string_error(DbvAssocId lAssocId, char *field_in_error, char *value, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_INVALIDBITSTRING_S always to be shorter than SA_ARRAY_SIZE 
	*/

	return(DBVtwo_field_error(lAssocId, field_in_error, value, ppsSIError, DBV_PDU_INVALIDBITSTRING, DBV_PDU_INVALIDBITSTRING_S));
}



int DBVtwo_field_error(DbvAssocId lAssocId, char *field1, char *field2, DbvError **ppsSIError, int error_code, char *error_diag)
{
	char field1_buffer[SA_ARRAY_SIZE];
	char field2_buffer[SA_ARRAY_SIZE];
	unsigned long room = 0;

	DBVsa_strcpy(field1_buffer, field1);
	DBVsa_strcpy(field2_buffer, field2);

	room = SA_ARRAY_SIZE - strlen(error_diag); 

	/* If can't fit in error buffer */
	if ( (strlen(field1_buffer) + strlen(field2_buffer)) >= room) {

		/* Just clear the arrays */
		DBVsa_clear_array(field1_buffer);
		DBVsa_clear_array(field2_buffer);
	}

        sprintf(DBVErrorBuffer, error_diag, field1_buffer, field2_buffer);
        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, error_code,  DBVErrorBuffer, 0, Association_Open, ppsSIError);
        return SI_NOTOK;
}



int DBVdbv_options_chosen_error(DbvAssocId lAssocId, char *field_in_error, char *value, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_INVALIDOPTIONS_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVtwo_field_error(lAssocId, field_in_error, value, ppsSIError, DBV_PDU_INVALIDOPTIONS, DBV_PDU_INVALIDOPTIONS_S));
}



int DBVdbv_invalid_enum_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_OUTOFRANGE_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVsingle_field_error(lAssocId, field_in_error, ppsSIError, DBV_PDU_OUTOFRANGE, DBV_PDU_OUTOFRANGE_S));
}



int DBVdbv_bool_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_INVALIDBOOL_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/
	return(DBVsingle_field_error(lAssocId, field_in_error, ppsSIError, DBV_PDU_INVALIDBOOL, DBV_PDU_INVALIDBOOL_S));
}



int DBVdbv_oid_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_INVALIDOID_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVsingle_field_error(lAssocId, field_in_error, ppsSIError, DBV_PDU_INVALIDOID, DBV_PDU_INVALIDOID_S));
}



/* Used for mismatch between external and oid */
int DBVdbv_external_oid_error(DbvAssocId lAssocId, char *oid, char *field_in_error, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_EXTERNALOID_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVtwo_field_error(lAssocId, oid, field_in_error, ppsSIError, DBV_PDU_EXTERNALOID, DBV_PDU_EXTERNALOID_S));
}



/* general error */
int DBVdbv_general_error(DbvAssocId lAssocId, char *field_in_error, DbvError **ppsSIError)
{
	/* Following requires DBV_PDU_GENERAL_S always to be shorter than
	   SA_ARRAY_SIZE 
	*/

	return(DBVsingle_field_error(lAssocId, field_in_error, ppsSIError, DBV_PDU_GENERAL, DBV_PDU_GENERAL_S));
}




int DBVcheck_DbvBool_opt(DbvAssocId lAssocId, DbvBool *pfIncreasing, char *error_buffer, DbvError **ppsSIError)
{
	if (pfIncreasing) 
		return(DBVcheck_DbvBool(lAssocId, pfIncreasing, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvBool(DbvAssocId lAssocId, DbvBool *pfIncreasing, char *error_buffer, DbvError **ppsSIError)
{

	if (DBVcheck_dbv_bool(*pfIncreasing) == SI_NOTOK) 
		return(DBVdbv_bool_error(lAssocId, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_bool(DbvBool boolval)
{
	if ( (boolval != DBV_FALSE) &&
	     (boolval != DBV_TRUE) ) 
		return SI_NOTOK;

	return SI_OK;
}



/* Check if oid contains a valid DBV record syntax oid */
int DBVcheck_dbv_recsyn(char *oid)
{

	if (oid == NULL)
		return SI_NOTOK;

	/* Only check that that its got numbers and dots ! */
	if (DBVcheck_oidformat(oid) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



/* Check if oid contains a valid DBV external oid which is not a
   record syntax. This code is used when checking an external prior
   to an encoding */
int DBVcheck_dbv_allowed_external(char *oid)
{

	if (oid == NULL)
		return SI_NOTOK;

	if (DBVunknown_external(oid))
		return SI_NOTOK;

	return SI_OK;
}



/* Check if oid contains a valid DBV attribute set oid */
int DBVcheck_dbv_attsetoid(char *oid)
{
	if (oid == NULL)
		return SI_NOTOK;

	/* Only check that that its got numbers and dots ! */
	if (DBVcheck_oidformat(oid) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
} 



/* Check if oid contains a valid DBV diagnostic set oid */
int DBVcheck_dbv_diagsetoid(char *oid)
{
	if (oid == NULL)
		return SI_NOTOK;

	/* Only check that that its got numbers and dots ! */
	if (DBVcheck_oidformat(oid) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
} 



/* Check if oid contains a valid DBV resource report oid */
int DBVcheck_dbv_resrepoid(char *oid)
{
	if (oid == NULL)
		return SI_NOTOK;

	/* Only check that that its got numbers and dots ! */
	if (DBVcheck_oidformat(oid) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
} 



/* Check if oid contains a valid DBV update extended service report oid */
int DBVcheck_dbv_updateoid(char *oid)
{
	if (oid == NULL)
		return SI_NOTOK;

	/* Only check that that its got numbers and dots ! */
	if (DBVcheck_oidformat(oid) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
} 



/* Check if any valid DBV oid */
int DBVcheck_dbv_validoid(char *oid)
{
	if (oid == NULL)
		return SI_NOTOK;

	if ( (DBVcheck_dbv_recsyn(oid) != SI_OK) && 
		(DBVcheck_dbv_attsetoid(oid) != SI_OK) && 
		(DBVcheck_dbv_diagsetoid(oid) != SI_OK) && 
		(DBVcheck_dbv_resrepoid(oid) != SI_OK) && 
		(DBVcheck_dbv_updateoid(oid) != SI_OK))
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbspecific(DbvAssocId lAssocId, DbvElementSetsDbSpecific **ppsElementSetsDbSpecific, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	
	DBVsa_strcpy(local_error_buffer, error_buffer);


	return SI_OK;
}



int DBVcheck_dbv_records(DbvAssocId lAssocId, DbvRecords *psRecords, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psRecords->eRecordType) {
	case RT_Response:

		/* If the choice dosent exist */
		if (psRecords->uRecord.ppsResponseRecord == NULL) {
			DBVsa_strcat(local_error_buffer, "uRecord.ppsResponseRecord");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}



		DBVsa_strcat(local_error_buffer, "uRecord.ppsResponseRecord");
		return(DBVcheck_response_rec(lAssocId, psRecords->uRecord.ppsResponseRecord, local_error_buffer, ppsSIError));

	
		break;
	case RT_NonSurrDiagnostics:

		DBVsa_strcat(local_error_buffer, "uRecord.psNonSurrogateDiag");

		/* If the choice dosent exist */
		if (psRecords->uRecord.psNonSurrogateDiag == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_default(lAssocId, psRecords->uRecord.psNonSurrogateDiag, local_error_buffer, ppsSIError));
		break;
	case RT_MultipleNonSurrDiagnostics:

		DBVsa_strcat(local_error_buffer, "uRecord.ppsMultipleNonSurrogateDiag");

		/* If the choice dosent exist */
		if (psRecords->uRecord.ppsMultipleNonSurrogateDiag == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


 

		return(DBVcheck_dbv_multdiag(lAssocId, psRecords->uRecord.ppsMultipleNonSurrogateDiag, local_error_buffer, ppsSIError));

		break;
	default:

		DBVsa_strcat(local_error_buffer, "eRecordType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
	}

	return SI_OK;


}



int DBVcheck_response_rec(DbvAssocId lAssocId, DbvNamePlusRecord **ppsResponseRecord, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];


	DBVsa_strcpy(local_error_buffer, error_buffer);

	while (ppsResponseRecord[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");

		if (DBVcheck_dbv_nameplusrec(lAssocId, ppsResponseRecord[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		index++;
	}


	return SI_OK;
}



int DBVcheck_dbv_nameplusrec(DbvAssocId lAssocId, DbvNamePlusRecord *psResponseRecord, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);
	
	switch (psResponseRecord->eDbRecordType) {
	case DRT_Retrieval:

		DBVsa_strcat(local_error_buffer, "uDbRecord.psRetrievalRecord");

		/* If the choice dosent exist */
		if (psResponseRecord->uDbRecord.psRetrievalRecord == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");

		return(DBVcheck_dbv_external(lAssocId, psResponseRecord->uDbRecord.psRetrievalRecord, local_error_buffer, ppsSIError));
		break;
	case DRT_SurrogateDiagnostics:

		DBVsa_strcat(local_error_buffer, "uDbRecord.psSurrogateDiagnostic");

		/* If the choice dosent exist */
		if (psResponseRecord->uDbRecord.psSurrogateDiagnostic == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_surrdiag(lAssocId, psResponseRecord->uDbRecord.psSurrogateDiagnostic, local_error_buffer, ppsSIError));
		
		break;
	case DRT_StartingFragment:

		DBVsa_strcat(local_error_buffer, "uDbRecord.psStartingFragment");

		/* If the choice dosent exist */
		if (psResponseRecord->uDbRecord.psStartingFragment == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");	
		return(DBVcheck_dbv_fragment(lAssocId, psResponseRecord->uDbRecord.psStartingFragment, local_error_buffer, ppsSIError));

		break;
	case DRT_IntermediateFragment:

		DBVsa_strcat(local_error_buffer, "uDbRecord.psIntermediateFragment");

		/* If the choice dosent exist */
		if (psResponseRecord->uDbRecord.psIntermediateFragment == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_fragment(lAssocId, psResponseRecord->uDbRecord.psIntermediateFragment, local_error_buffer, ppsSIError));

		break;
	case DRT_FinalFragment:

		DBVsa_strcat(local_error_buffer, "uDbRecord.psFinalFragment");

		/* If the choice dosent exist */
		if (psResponseRecord->uDbRecord.psFinalFragment == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_fragment(lAssocId, psResponseRecord->uDbRecord.psFinalFragment, local_error_buffer, ppsSIError));

		break;
	default:
		strcat(local_error_buffer, "eDbRecordType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
	}

	return SI_OK;

}



int DBVcheck_dbv_surrdiag(DbvAssocId lAssocId, DbvDiagRec *psSurrogateDiagnostic, char *error_buffer, DbvError **ppsSIError)
{
	
	char local_error_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);

	switch (psSurrogateDiagnostic->eDiagnosticsType) {
	case DT_DefaultFormat:

		DBVsa_strcat(local_error_buffer, "uDiagnostics.psDefault");

		/* If the choice dosent exist */
		if (psSurrogateDiagnostic->uDiagnostics.psDefault == NULL) {

			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_default(lAssocId, psSurrogateDiagnostic->uDiagnostics.psDefault, local_error_buffer, ppsSIError));
		
		break;
	case DT_ExternallyDefined:

		DBVsa_strcat(local_error_buffer, "uDiagnostics.psExternallyDefined");

		/* If the choice dosent exist */
		if (psSurrogateDiagnostic->uDiagnostics.psExternallyDefined == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");	
		return(DBVcheck_dbv_external(lAssocId, psSurrogateDiagnostic->uDiagnostics.psExternallyDefined, local_error_buffer, ppsSIError));

		break;
	default:
		DBVsa_strcat(local_error_buffer, "eDiagnosticsType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		
		break;
	}

	return SI_OK;

}



int DBVcheck_dbv_default(DbvAssocId lAssocId, DbvDefaultDiagFormat *psDefault, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	strcpy(local_error_buffer, error_buffer);

	if (DBVcheck_dbv_validoid(psDefault->pcDiagnosticSetId) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "pcDiagnosticSetId");
		return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* Dont check ulCondition - allow anything */


	switch (psDefault->sAdditionalInfo.eProtocolVersionInForce) {
	case PVIF_Version2:
		if (psDefault->sAdditionalInfo.uAdditionalInfo.pcV2 == NULL) {
                        DBVsa_strcat(local_error_buffer, "sAdditionalInfo.uAdditionalInfo.pcV2");
                        return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}
			
		break;
	case PVIF_Version3:

		/* If the choice dosent exist */
		if (psDefault->sAdditionalInfo.uAdditionalInfo.psV3 == NULL) {
			DBVsa_strcat(local_error_buffer, "sAdditionalInfo.uAdditionalInfo.psV3");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		
		break;	
	default:
		DBVsa_strcat(local_error_buffer, "sAdditionalInfo.eProtocolVersionInForce");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		
		break;

	}
	return SI_OK;
}



int DBVcheck_dbv_fragment(DbvAssocId lAssocId, DbvFragmentSyntax *psFragment, char *error_buffer, DbvError **ppsSIError)
{
	
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	
	switch (psFragment->eFragmentSyntaxType) {
	case FST_ExternallyTagged:

		DBVsa_strcat(local_error_buffer, "uFragmentSyntax.psExternallyTagged");
		/* If the choice dosent exist */
		if (psFragment->uFragmentSyntax.psExternallyTagged == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_external(lAssocId, psFragment->uFragmentSyntax.psExternallyTagged, local_error_buffer, ppsSIError));

		break;
	case FST_NotExternallyTagged:

		DBVsa_strcat(local_error_buffer, "uFragmentSyntax.psNotExternallyTagged");
		/* If the choice dosent exist */
		if (psFragment->uFragmentSyntax.psNotExternallyTagged == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		break;
	default:
		DBVsa_strcat(local_error_buffer, "eFragmentSyntaxType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

		
	return SI_OK;
}



int DBVcheck_dbv_multdiag(DbvAssocId lAssocId, DbvDiagRec **ppsMultipleNonSurrogateDiag, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	while (ppsMultipleNonSurrogateDiag[index] != NULL) {

		/* Cant use ppsMultipleNonSurrogateDiag in temp_error_buffer
		   construction cos different fields in different PDUs
		   have different names */

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");
 

		if (DBVcheck_dbv_surrdiag(lAssocId, ppsMultipleNonSurrogateDiag[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		index++;
	}

	return SI_OK;

}



int DBVdbv_to_snacc_strornum(StringOrNumeric *data_ptr, DbvStringOrNumeric *psAttributeElement, jmp_buf env)
{
        switch (psAttributeElement->eStringOrNumeric) {
        case SNT_String:
                data_ptr->choiceId = STRINGORNUMERIC_STRING;
                data_ptr->a.string = (InternationalString*) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(data_ptr->a.string, env);

                DBVdbv_to_snacc_inter_string(data_ptr->a.string, psAttributeElement->uStringOrNumeric.psString, env);
                break;
        case SNT_Numeric:
                data_ptr->choiceId = STRINGORNUMERIC_NUMERIC;
                data_ptr->a.numeric = psAttributeElement->uStringOrNumeric.ulNumeric;
                break;
        }
}



int DBVdbv_to_snacc_intunit(struct IntUnit* integerAndUnit, DbvIntUnit *psIntegerAndUnit, jmp_buf env)
{
 
        /* Both compulsory */
        integerAndUnit->value = psIntegerAndUnit->ulValue;
        integerAndUnit->unitUsed = (struct Unit*) Asn1Alloc (sizeof(struct Unit)
);
	CheckAsn1Alloc(integerAndUnit->unitUsed, env);

        DBVdbv_to_snacc_unit(integerAndUnit->unitUsed, &psIntegerAndUnit->sUnitUsed, env);
 
}



int DBVdbv_to_snacc_unit(struct Unit* unitUsed, DbvUnit *psUnitUsed, jmp_buf env)
{
        /* All the following are optional */
 
        if (psUnitUsed->psUnitSystem != NULL)
 
                /* No International String alloc needed here */
                DBVdbv_to_snacc_inter_string(&unitUsed->unitSystem, psUnitUsed->psUnitSystem, env);
 
 
        if (psUnitUsed->psUnitType != NULL) {
                unitUsed->unitType = (struct StringOrNumeric*) Asn1Alloc (sizeof(struct StringOrNumeric));
		CheckAsn1Alloc(unitUsed->unitType, env);
                DBVdbv_to_snacc_strornum(unitUsed->unitType, psUnitUsed->psUnitType, env);
        }
 
 
        if (psUnitUsed->psUnit != NULL) {
 
                unitUsed->unit = (struct StringOrNumeric*) Asn1Alloc (sizeof (struct StringOrNumeric));
		CheckAsn1Alloc(unitUsed->unit, env);

                DBVdbv_to_snacc_strornum(unitUsed->unit, psUnitUsed->psUnit, env);
        }
 
 
        if (psUnitUsed->pulScaleFactor != NULL) {
                unitUsed->scaleFactor = (AsnInt*) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(unitUsed->scaleFactor, env);

                *unitUsed->scaleFactor = *psUnitUsed->pulScaleFactor;
        }
 
}


int DBVsnacc_to_dbv_strornum(DbvStringOrNumeric *psList, StringOrNumeric *data_ptr, jmp_buf env)
{
        switch (data_ptr->choiceId) {
        case STRINGORNUMERIC_STRING:
                psList->eStringOrNumeric = SNT_String;
                psList->uStringOrNumeric.psString = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psList->uStringOrNumeric.psString, env);

                DBVsnacc_to_dbv_inter_string(psList->uStringOrNumeric.psString,data_ptr->a.string, env);
                break;
        case STRINGORNUMERIC_NUMERIC:
                psList->eStringOrNumeric = SNT_Numeric;
                psList->uStringOrNumeric.ulNumeric = data_ptr->a.numeric;
                break;
        }
}



int DBVsnacc_to_dbv_intunit(DbvIntUnit *psIntegerAndUnit, struct IntUnit* integerAndUnit, jmp_buf env)
{
        /* Both compulsory */
        psIntegerAndUnit->ulValue = integerAndUnit->value;
 
        DBVsnacc_to_dbv_unit(&psIntegerAndUnit->sUnitUsed, integerAndUnit->unitUsed, env);
}



int DBVsnacc_to_dbv_unit(DbvUnit *psUnitUsed, struct Unit* unitUsed, jmp_buf env)
{
        /* Optional */
        if (unitUsed->unitSystem.octetLen != 0) {
                psUnitUsed->psUnitSystem = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psUnitUsed->psUnitSystem, env);

                DBVsnacc_to_dbv_inter_string(psUnitUsed->psUnitSystem, &unitUsed
->unitSystem, env);
        }
 
 
        if (unitUsed->unitType != NULL) {
                psUnitUsed->psUnitType = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
		CheckDbvAlloc(psUnitUsed->psUnitType, env);

                DBVsnacc_to_dbv_strornum(psUnitUsed->psUnitType, unitUsed->unitType, env);
        }
 
 
 
        if (unitUsed->unit != NULL) {
                psUnitUsed->psUnit = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
		CheckDbvAlloc(psUnitUsed->psUnit, env);
 
               DBVsnacc_to_dbv_strornum(psUnitUsed->psUnit, unitUsed->unit, env);
        }
 
 
 
        if (unitUsed->scaleFactor != NULL) {
                psUnitUsed->pulScaleFactor = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(psUnitUsed->pulScaleFactor, env);
                *psUnitUsed->pulScaleFactor = *unitUsed->scaleFactor;
        }
 
}




int DBVdbv_to_snacc_attrterm(struct AttributesPlusTerm* attrTerm, DbvAttributesPlusTerm *psAttributesPlusTerm, jmp_buf env)
{
 
        /* Attributes */
        if (psAttributesPlusTerm->ppsAttributeElement != NULL) {
                attrTerm->attributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(attrTerm->attributes, env);

                DBVdbv_to_snacc_attribute_list(attrTerm->attributes, psAttributesPlusTerm->ppsAttributeElement, env);
        }
 
        /* Term */
        attrTerm->term = (struct Term*) Asn1Alloc (sizeof(struct Term));
	CheckAsn1Alloc(attrTerm->term, env);

        DBVdbv_to_snacc_term(attrTerm->term, &psAttributesPlusTerm->sTerm, env);
}



int DBVdbv_to_snacc_attribute_list(AttributeList* attributes, DbvAttributeElement **ppsAttributeElement, jmp_buf env)
{
 
        int index = 0;
 
        AttributeElement *data_ptr;
        AttributeElement **data_ptr_address;
 
 
        /* Attributes */
        if (ppsAttributeElement != NULL) {
                while (ppsAttributeElement[index] != NULL){
                        data_ptr_address = AsnListAppend(attributes);
			CheckAsn1Alloc(data_ptr_address, env);

                        *data_ptr_address = Asn1Alloc(sizeof(AttributeElement));
			CheckAsn1Alloc(*data_ptr_address, env);

                        data_ptr = *data_ptr_address;
 
                        DBVdbv_to_snacc_attributes(data_ptr, ppsAttributeElement[index], env);
                        index++;
                }
        }
 
 
}
 
 
 
int DBVdbv_to_snacc_attributes(AttributeElement *data_ptr, DbvAttributeElement *psAttributeElement, jmp_buf env)
{
 
        /* Optional */
        if (psAttributeElement->pcAttributeSetId != NULL)
                ASN_MakeOid((char *) psAttributeElement->pcAttributeSetId, &data_ptr->attributeSet, env);
 
 
 
        /* Compulsory */
        data_ptr->attributeType = psAttributeElement->ulAttributeType;
 
        /* Compulsory */
        data_ptr->attributeValue = (struct AttributeElementChoice*) Asn1Alloc (sizeof(struct AttributeElementChoice));
	CheckAsn1Alloc(data_ptr->attributeValue, env);

        DBVdbv_to_snacc_attval(data_ptr->attributeValue, psAttributeElement->sAttributeValue, env);
}
 


int DBVdbv_to_snacc_attval(struct AttributeElementChoice* attributeValue, DbvAttributeValue sAttributeValue, jmp_buf env)
{
        switch (sAttributeValue.eAttributeValueType) {
        case AVT_Numeric:
                attributeValue->choiceId = ATTRIBUTEELEMENTCHOICE_NUMERIC;
                attributeValue->a.numeric = sAttributeValue.uAttributeValue.ulNumeric;
                break;
        case AVT_Complex:
                attributeValue->choiceId = ATTRIBUTEELEMENTCHOICE_COMPLEX;
                attributeValue->a.complex = (struct AttributeElementChoiceSeq*) Asn1Alloc (sizeof(struct AttributeElementChoiceSeq));
		CheckAsn1Alloc(attributeValue->a.complex, env);

                DBVdbv_to_snacc_attcomplex(attributeValue->a.complex, sAttributeValue.uAttributeValue.psComplexValue, env);
                break;
        }
}
 


int DBVdbv_to_snacc_attcomplex(struct AttributeElementChoiceSeq* complex, DbvComplexValue *psComplexValue, jmp_buf env)
{
        /* Compulsory */
        if (psComplexValue->ppsList != NULL) {
                complex->list = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(complex->list, env);

                DBVdbv_to_snacc_attcomplist(complex->list, psComplexValue->ppsList, env);
        }
 
        /* Optional */
        if (psComplexValue->ppulSemanticAction != NULL) {
                complex->semanticAction = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(complex->semanticAction, env);

                DBVdbv_to_snacc_attcompsem(complex->semanticAction, psComplexValue->ppulSemanticAction, env);
        }
}
 
 
 
int DBVdbv_to_snacc_attcomplist(AttributeElementChoiceSeqSeqOf* list, DbvStringOrNumeric **ppsList, jmp_buf env)
{
        int index = 0;
 
        StringOrNumeric *data_ptr;
        StringOrNumeric **data_ptr_address;
 
        while (ppsList[index] != NULL){
                data_ptr_address = AsnListAppend(list);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(StringOrNumeric));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_strornum(data_ptr, ppsList[index], env);
                index++;
        }
 
}
 
 
 
int DBVdbv_to_snacc_attcompsem(AttributeElementChoiceSeqSeqOf1* semanticAction, Ulong **ppulSemanticAction, jmp_buf env)
{
        int index = 0;
 
        AsnInt *data_ptr;
        AsnInt **data_ptr_address;
 
        while (ppulSemanticAction[index] != NULL){
                data_ptr_address = AsnListAppend(semanticAction);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AsnInt));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                *data_ptr = *ppulSemanticAction[index];
                index++;
        }
 
 
}
 
 

/* Term */
int DBVdbv_to_snacc_term(struct Term* term, DbvTerm *psTerm, jmp_buf env)
{
 
        char *string = NULL;
        unsigned long length = 0;
 
        switch (psTerm->eTermType) {
        case TT_General:
                term->choiceId = TERM_GENERAL;
                term->a.general = (AsnOcts*) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(term->a.general, env);

                DBVdbv_to_snacc_octet_string(term->a.general, psTerm->uTerm.psGeneral, env);
                break;
        case TT_Numeric:
                term->choiceId = TERM_NUMERIC;
                term->a.numeric = psTerm->uTerm.ulInteger;
                break;
        case TT_SimpleCharacter:
                term->choiceId = TERM_CHARACTERSTRING;
                term->a.characterString = (InternationalString*) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(term->a.characterString, env);

                DBVdbv_to_snacc_inter_string(term->a.characterString, psTerm->uTerm.psSimpleCharacter, env);
                break;
        case TT_ObjectIdentifier:
                term->choiceId = TERM_OID_SNACC;
                term->a.oid = (AsnOid*) Asn1Alloc (sizeof(AsnOid));
		CheckAsn1Alloc(term->a.oid, env);

                ASN_MakeOid((char *) psTerm->uTerm.pcObjectIdentifier, term->a.oid, env);
                break;
        case TT_DateTime:
                term->choiceId = TERM_DATETIME;
 
                /* Generalized Time - just an octet string which MUST
                   have a predefined format (see the ASN.1 spec!!) */
 
                term->a.dateTime = (GeneralizedTime*) Asn1Alloc (sizeof(GeneralizedTime));
		CheckAsn1Alloc(term->a.dateTime, env);
 
                /* The Dbv pcDateTime is just a char */
 
                length = strlen(psTerm->uTerm.pcDateTime);
                string = psTerm->uTerm.pcDateTime;
 
                term->a.dateTime->octetLen = length;
                term->a.dateTime->octs = (char *) Asn1Alloc (length);
		CheckAsn1Alloc(term->a.dateTime->octs, env);
 
                memcpy(term->a.dateTime->octs, string, length);
                break;
        case TT_External:
                term->choiceId = TERM_EXTERNAL;
                term->a.external = (NEWEXTERNAL*) Asn1Alloc (sizeof(NEWEXTERNAL));
		CheckAsn1Alloc(term->a.external, env);

                DBVdbv_to_snacc_external(term->a.external, psTerm->uTerm.psExternal, env);
 
                break;
        case TT_IntegerAndUnit:
                term->choiceId = TERM_INTEGERANDUNIT;
                term->a.integerAndUnit = (struct IntUnit*) Asn1Alloc (sizeof(struct IntUnit));
		CheckAsn1Alloc(term->a.integerAndUnit, env);

                DBVdbv_to_snacc_intunit(term->a.integerAndUnit, psTerm->uTerm.psIntegerAndUnit, env);
 
                break;
        case TT_Null:
                term->choiceId = TERM_NULL;
                break;
        }
}
 
 



int DBVsnacc_to_dbv_attrterm(DbvAttributesPlusTerm *psAttributesPlusTerm, struct AttributesPlusTerm* attrTerm, jmp_buf env)
{
        AttributeElement *data_ptr;
        int num_elements = 0;
        int index = 0;
 
 
        /* attributes */
        num_elements = AsnListCount(attrTerm->attributes);
 
        /* +1 to allocate space for the list terminator NULL */
        psAttributesPlusTerm->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc(sizeof(DbvAttributeElement *) * (num_elements+1));
	CheckDbvAlloc(psAttributesPlusTerm->ppsAttributeElement, env);
 
        FOR_EACH_LIST_ELMT(data_ptr, attrTerm->attributes)
        {
                psAttributesPlusTerm->ppsAttributeElement[index]=(DbvAttributeElement *)DbvAlloc(sizeof(DbvAttributeElement));
		CheckDbvAlloc(psAttributesPlusTerm->ppsAttributeElement[index], env);

                DBVsnacc_to_dbv_attributes(psAttributesPlusTerm->ppsAttributeElement[index], data_ptr, env);
                index++;
        }
        psAttributesPlusTerm->ppsAttributeElement[index]=NULL;
 
 
        /* Term */
        DBVsnacc_to_dbv_term(&psAttributesPlusTerm->sTerm, attrTerm->term, env);
}



int DBVsnacc_to_dbv_attributes(DbvAttributeElement *psAttributeElement, AttributeElement *data_ptr, jmp_buf env)
{
        char *oid = NULL;
 
        /* pcAttributeSetId - optional */
        if (data_ptr->attributeSet.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&data_ptr->attributeSet);
                psAttributeElement->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(psAttributeElement->pcAttributeSetId, env);

                memcpy(psAttributeElement->pcAttributeSetId, oid, strlen(oid));
        }
 
        /* ulAttributeType - compulsory */
        psAttributeElement->ulAttributeType = data_ptr->attributeType;
 
        /* sAttributeValue - compulsory */
        DBVsnacc_to_dbv_attvalue(&psAttributeElement->sAttributeValue, data_ptr->attributeValue, env);
}
 


int DBVsnacc_to_dbv_attvalue(DbvAttributeValue *psAttributeValue, struct AttributeElementChoice* attributeValue, jmp_buf env)
{
        switch (attributeValue->choiceId) {
        case ATTRIBUTEELEMENTCHOICE_NUMERIC:
                psAttributeValue->eAttributeValueType = AVT_Numeric;
                psAttributeValue->uAttributeValue.ulNumeric = attributeValue->a.numeric;
                break;
        case ATTRIBUTEELEMENTCHOICE_COMPLEX:
                psAttributeValue->eAttributeValueType = AVT_Complex;
                psAttributeValue->uAttributeValue.psComplexValue = (DbvComplexValue *) DbvAlloc (sizeof(DbvComplexValue));
		CheckDbvAlloc(psAttributeValue->uAttributeValue.psComplexValue, env);

                DBVsnacc_to_dbv_complex(psAttributeValue->uAttributeValue.psComplexValue, attributeValue->a.complex, env);
                break;
        }
}
 


int DBVsnacc_to_dbv_complex(DbvComplexValue *psComplexValue, struct AttributeElementChoiceSeq* complex, jmp_buf env)
{
        int num_elmnts = 0;
 
        /* compulsory */
 
        num_elmnts = AsnListCount(complex->list);
 
        /* +1 to allocate space for a list terminating NULL */
        psComplexValue->ppsList = (DbvStringOrNumeric **) DbvAlloc (sizeof(DbvStringOrNumeric *) * (num_elmnts + 1));
	CheckDbvAlloc(psComplexValue->ppsList, env);

        DBVsnacc_to_dbv_DbvStringOrNumeric_list(psComplexValue->ppsList, complex->list, env);
 
 
        /* optional */
 
        if (complex->semanticAction != NULL) {
                num_elmnts = AsnListCount(complex->semanticAction);
                psComplexValue->ppulSemanticAction = (Ulong **) DbvAlloc (sizeof(Ulong *) * (num_elmnts + 1));
		CheckDbvAlloc(psComplexValue->ppulSemanticAction, env);

                DBVsnacc_to_dbv_semantic(psComplexValue->ppulSemanticAction, complex->semanticAction, env);
        }
}
 


int DBVsnacc_to_dbv_DbvStringOrNumeric_list(DbvStringOrNumeric **ppsList, AttributeElementChoiceSeqSeqOf* list, jmp_buf env)
{
        StringOrNumeric *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, list)
        {
                ppsList[index]=(DbvStringOrNumeric *)DbvAlloc(sizeof(DbvStringOrNumeric));
		CheckDbvAlloc(ppsList[index], env);

                DBVsnacc_to_dbv_strornum(ppsList[index], data_ptr, env);
                index++;
        }
        ppsList[index]=NULL;
}
 
 

int DBVsnacc_to_dbv_semantic(Ulong **ppulSemanticAction, AttributeElementChoiceSeqSeqOf1* semanticAction, jmp_buf env)
{
        AsnInt *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, semanticAction)
        {
                ppulSemanticAction[index] = (Ulong *)DbvAlloc(sizeof(Ulong));
		CheckDbvAlloc(ppulSemanticAction[index], env);

                *ppulSemanticAction[index] = *data_ptr;
                index++;
        }
        ppulSemanticAction[index]=NULL;
}
 
 

int DBVsnacc_to_dbv_term(DbvTerm *psTerm, struct Term* term, jmp_buf env)
{
        char *oid = NULL;
        char *string = NULL;
        unsigned long length = 0;
 
 
        switch (term->choiceId) {
        case TERM_GENERAL:
        psTerm->eTermType = TT_General;
                psTerm->uTerm.psGeneral = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(psTerm->uTerm.psGeneral, env);

                DBVsnacc_to_dbv_octet_string(psTerm->uTerm.psGeneral, term->a.general, env);
                break;
        case TERM_NUMERIC:
                psTerm->eTermType = TT_Numeric;
                psTerm->uTerm.ulInteger = term->a.numeric;
                break;
        case TERM_CHARACTERSTRING:
                psTerm->eTermType = TT_SimpleCharacter;
                psTerm->uTerm.psSimpleCharacter = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psTerm->uTerm.psSimpleCharacter, env);
 
                DBVsnacc_to_dbv_inter_string(psTerm->uTerm.psSimpleCharacter, term->a.characterString, env);
                break;
        case TERM_OID_SNACC:
                psTerm->eTermType = TT_ObjectIdentifier;
                oid = (char *) ASN_EncodedToString(term->a.oid);
 
                /* +1 for the \0 */
                psTerm->uTerm.pcObjectIdentifier = (DbvObjectIdentifier) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(psTerm->uTerm.pcObjectIdentifier , env);

                memcpy(psTerm->uTerm.pcObjectIdentifier, oid, strlen(oid));
                break;
        case TERM_DATETIME:
                psTerm->eTermType = TT_DateTime;
                string = term->a.dateTime->octs;
                length = term->a.dateTime->octetLen;
 
                /* +1 for the \0 */
                psTerm->uTerm.pcDateTime = (DbvGeneralizedTime) DbvAlloc (length+ 1);
		CheckDbvAlloc(psTerm->uTerm.pcDateTime, env);

                memcpy(psTerm->uTerm.pcDateTime, string, length);
                break;
        case TERM_EXTERNAL:
                psTerm->eTermType = TT_External;
                psTerm->uTerm.psExternal = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(psTerm->uTerm.psExternal, env);

                DBVsnacc_to_dbv_external(psTerm->uTerm.psExternal, term->a.external, env);
                break;
        case TERM_INTEGERANDUNIT:
                psTerm->eTermType = TT_IntegerAndUnit;
                psTerm->uTerm.psIntegerAndUnit = (DbvIntUnit *) DbvAlloc (sizeof (DbvIntUnit));
		CheckDbvAlloc(psTerm->uTerm.psIntegerAndUnit, env);

                DBVsnacc_to_dbv_intunit(psTerm->uTerm.psIntegerAndUnit, term->a.integerAndUnit, env);
                break;
        case TERM_NULL:
                psTerm->eTermType = TT_Null;
                break;
        }
}





int DBVsnacc_to_dbv_attribute_list(DbvAttributeElement **ppsAttributeElement, AttributeList *attributes, jmp_buf env)
{
 
        AttributeElement *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, attributes) {
                ppsAttributeElement[index] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
		CheckDbvAlloc(ppsAttributeElement[index], env);

                DBVsnacc_to_dbv_attributes(ppsAttributeElement[index], data_ptr, env);
                index++;
        }
        ppsAttributeElement[index] = NULL;
 
}



int DBVdbv_to_snacc_specification(struct Specification* generic, DbvSpecification *psGeneric, jmp_buf env)
{
        /* schema */
        if (psGeneric->pcSchema != NULL)
                ASN_MakeOid((char *) psGeneric->pcSchema, &generic->schema, env);
	DBVdbv_to_snacc_DbvElementSpec_opt(&generic->elementSpec, psGeneric->psElementSpec, env);
}



void DBVdbv_to_snacc_DbvElementSpec_opt(struct SpecificationChoice** elementSpec, DbvElementSpec *psElementSpec, jmp_buf env)
{
	/* o */
	if (psElementSpec) {
		*elementSpec = (struct SpecificationChoice*) DbvAlloc (sizeof (struct SpecificationChoice));
		CheckAsn1Alloc(*elementSpec, env);
		DBVdbv_to_snacc_DbvElementSpec(*elementSpec, psElementSpec, env);
	} 
}



void DBVdbv_to_snacc_DbvElementSpec(struct SpecificationChoice* elementSpec, DbvElementSpec *psElementSpec, jmp_buf env)
{
        /* elementspectype */
        switch (psElementSpec->eElementSpecType) {
        case EST_Primitive:
                elementSpec->choiceId = SPECIFICATIONCHOICE_ELEMENTSETNAME;
                elementSpec->a.elementSetName = (InternationalString *) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(elementSpec->a.elementSetName, env);

                DBVdbv_to_snacc_inter_string(elementSpec->a.elementSetName, psElementSpec->uElementSpec.psPrimitive, env);
                break;
        case EST_External:
                elementSpec->choiceId = SPECIFICATIONCHOICE_EXTERNALESPEC;
                elementSpec->a.externalEspec = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
		CheckAsn1Alloc(elementSpec->a.externalEspec, env);
 
                DBVdbv_to_snacc_external(elementSpec->a.externalEspec, psElementSpec->uElementSpec.psExternal, env);
                break;
        }
}



int DBVsnacc_to_dbv_specification(DbvSpecification *psGeneric, struct Specification* generic, jmp_buf env)
{
        char *oid = NULL;
 
        /* schema */
        if (generic->schema.octetLen != 0) {
                oid = (char *) ASN_EncodedToString(&(generic->schema));
 
                /* ObjectIdentifier is a char * */
                psGeneric->pcSchema = (DbvObjectIdentifier ) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(psGeneric->pcSchema, env);
                memcpy(psGeneric->pcSchema, oid, strlen(oid));
        }
 
	/* o */
	DBVsnacc_to_dbv_DbvElementSpec_opt(&psGeneric->psElementSpec, generic->elementSpec, env);
}



void DBVsnacc_to_dbv_DbvElementSpec_opt(DbvElementSpec **ppsElementSpec, struct SpecificationChoice* elementSpec, jmp_buf env)
{
 
	if (elementSpec) {
		*ppsElementSpec = (DbvElementSpec *) DbvAlloc (sizeof (DbvElementSpec));
		CheckDbvAlloc(*ppsElementSpec, env);
		DBVsnacc_to_dbv_DbvElementSpec(*ppsElementSpec, elementSpec, env);
	}
}
 


void DBVsnacc_to_dbv_DbvElementSpec(DbvElementSpec *psElementSpec, struct SpecificationChoice* elementSpec, jmp_buf env)
{
        switch (elementSpec->choiceId) {
        case SPECIFICATIONCHOICE_ELEMENTSETNAME:
 
                psElementSpec->eElementSpecType = EST_Primitive;
                psElementSpec->uElementSpec.psPrimitive = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psElementSpec->uElementSpec.psPrimitive, env);

                DBVsnacc_to_dbv_inter_string(psElementSpec->uElementSpec.psPrimitive , elementSpec->a.elementSetName, env);
                break;
        case SPECIFICATIONCHOICE_EXTERNALESPEC:
                psElementSpec->eElementSpecType = EST_External;
                psElementSpec->uElementSpec.psExternal = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(psElementSpec->uElementSpec.psExternal, env);
 
                /* get the external from the snacc structure */
                DBVsnacc_to_dbv_external(psElementSpec->uElementSpec.psExternal, elementSpec->a.externalEspec, env);
                break;
        }
}




/* Routine to malloc all remaining space then return the NULL returned
   by the last malloc - this
   routine is used to check the functionality of the CheckDbvAlloc
   macro */
char *DBVgobble_up_memory()
{
	char *test = NULL;
        long size = 0;
 
        test = (char *) malloc (10);
 
        while (test != NULL) {
                size+= 100000000;
                test = (char *) realloc (test, size);
        }

	return test;
}



int DBVcheck_dbv_attsplusterm(DbvAssocId lAssocId, DbvAttributesPlusTerm *psAttributesPlusTerm, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];
 
	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* Both compulsory */
	
        /* if no list nodes */
        if (psAttributesPlusTerm->ppsAttributeElement == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsAttributeElement");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}
 

 
        /* Check all items in the list */
        while (psAttributesPlusTerm->ppsAttributeElement[index] != NULL) {

                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsAttributeElement[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");
 
                if (DBVcheck_dbv_attelement(lAssocId, psAttributesPlusTerm->ppsAttributeElement[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
 
        	index++;
	}
        


	DBVsa_strcat(local_error_buffer, "sTerm.");
	return(DBVcheck_dbv_term(lAssocId, &psAttributesPlusTerm->sTerm, local_error_buffer, ppsSIError));


        return SI_OK;
	
}



int DBVcheck_dbv_attelement(DbvAssocId lAssocId, DbvAttributeElement *psAttributeElement, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];
	
	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* pcAttributeSetId is optional */

	if (psAttributeElement->pcAttributeSetId != NULL) 
		if (DBVcheck_dbv_validoid(psAttributeElement->pcAttributeSetId) == SI_NOTOK) {
			DBVsa_strcat(local_error_buffer, "pcAttributeSetId");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}
	

	/* Dont check ulAttributeType */

	DBVsa_strcat(local_error_buffer, "sAttributeValue.");
	return(DBVcheck_dbv_attval(lAssocId, &psAttributeElement->sAttributeValue, local_error_buffer, ppsSIError));  

	return SI_OK;
	
}



int DBVcheck_dbv_attval(DbvAssocId lAssocId, DbvAttributeValue *psAttributeValue, char *error_buffer, DbvError **ppsSIError)  
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psAttributeValue->eAttributeValueType) {
	case AVT_Numeric:

		/* Dont check numeric */
		break;
	case AVT_Complex:

		DBVsa_strcat(local_error_buffer, "uAttributeValue.psComplexValue");
		if (psAttributeValue->uAttributeValue.psComplexValue == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_complexval(lAssocId, psAttributeValue->uAttributeValue.psComplexValue, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eAttributeValueType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}
	return SI_OK;
}



int DBVcheck_dbv_complexval(DbvAssocId lAssocId, DbvComplexValue *psComplexValue, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
 
	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* First compulsory */
	
	/* If no list nodes */
        if (psComplexValue->ppsList == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsList");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}
 


        /* Check all items in the list */
        while (psComplexValue->ppsList[index] != NULL) {

                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsList[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->");

                if (DBVcheck_DbvStringOrNumeric(lAssocId, psComplexValue->ppsList[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
                        return SI_NOTOK;
 
        	index++;
        }


	/* Second optional */

	if (psComplexValue->ppulSemanticAction == NULL)
		return SI_OK;


	/* Dont check the ppulSemanticAction list cos its a list of longs */

	return SI_OK;
}




int DBVcheck_DbvStringOrNumeric_opt(DbvAssocId lAssocId, DbvStringOrNumeric *psDefaultValue, char *error_buffer, DbvError **ppsSIError)
{  
        if (psDefaultValue)
                return(DBVcheck_DbvStringOrNumeric(lAssocId, psDefaultValue, error_buffer, ppsSIError));
   
        return SI_OK;
}  



int DBVcheck_DbvStringOrNumeric(DbvAssocId lAssocId, DbvStringOrNumeric *psList, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];
	
	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psList->eStringOrNumeric) {
	case SNT_String:

        	if (psList->uStringOrNumeric.psString == NULL)  {
			DBVsa_strcat(local_error_buffer, "uStringOrNumeric.psString");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
	case SNT_Numeric:

		/* Dont check the numeric */
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eStringOrNumeric");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_term(DbvAssocId lAssocId, DbvTerm *psTerm, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psTerm->eTermType) {
	case TT_General:
        	if (psTerm->uTerm.psGeneral == NULL) {
			DBVsa_strcat(local_error_buffer, "uTerm.psGeneral");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}
		break;
	case TT_Numeric:

		/* No need to check numeric */
		break;
	case TT_SimpleCharacter:
        	if (psTerm->uTerm.psSimpleCharacter == NULL) {
			DBVsa_strcat(local_error_buffer, "uTerm.psSimpleCharacter");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
	case TT_ObjectIdentifier:
		if (DBVcheck_dbv_validoid(psTerm->uTerm.pcObjectIdentifier) == SI_NOTOK) {
			DBVsa_strcat(local_error_buffer, "uTerm.pcObjectIdentifier");
			return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
		}
		
		break;
	case TT_DateTime:

		/* Just check that pcDateTime isnt NULL */
        	if (psTerm->uTerm.pcDateTime == NULL) {
			DBVsa_strcat(local_error_buffer, "uTerm.pcDateTime");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
	case TT_External:

		DBVsa_strcat(local_error_buffer, "uTerm.psExternal");
	
        	if (psTerm->uTerm.psExternal == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_external(lAssocId, psTerm->uTerm.psExternal, local_error_buffer, ppsSIError));
		break;
	case TT_IntegerAndUnit:

		DBVsa_strcat(local_error_buffer, "uTerm.psIntegerAndUnit");

        	if (psTerm->uTerm.psIntegerAndUnit == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvIntUnit(lAssocId, psTerm->uTerm.psIntegerAndUnit, local_error_buffer, ppsSIError));
		break;
	case TT_Null:
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eTermType");
		
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;
}



int DBVcheck_DbvIntUnit(DbvAssocId lAssocId, DbvIntUnit *psIntegerAndUnit, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* All optional */

	/* Dont check ulValue */

	/* check the sUnitUsed value */
	DBVsa_strcat(local_error_buffer, "sUnitUsed.");
	if (DBVcheck_DbvUnit(lAssocId, &psIntegerAndUnit->sUnitUsed, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvUnit(DbvAssocId lAssocId, DbvUnit *psUnitUsed, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);


	/* Can be NULL because optional */
	if (psUnitUsed->psUnitType != NULL) {
		DBVsa_strcat(local_error_buffer, "psUnitType->");

		if (DBVcheck_DbvStringOrNumeric(lAssocId, psUnitUsed->psUnitType, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


	if (psUnitUsed->psUnit != NULL)  {
		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "psUnit->");

		if (DBVcheck_DbvStringOrNumeric(lAssocId, psUnitUsed->psUnit, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
	


	/* Dont check psUnitUsed->pulScaleFactor */

	return SI_OK;
}



int DBVcheck_DbvSpecification(DbvAssocId lAssocId, DbvSpecification *psGeneric, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* pcSchema optional so can be null */
	if (psGeneric->pcSchema != NULL)
		if (DBVcheck_dbv_recsyn(psGeneric->pcSchema) == SI_NOTOK) {
			DBVsa_strcat(local_error_buffer, "pcSchema");
			return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
		}


	/* o */
	DBVsa_strcat(local_error_buffer, "psElementSpec->");
	if (DBVcheck_DbvElementSpec_opt(lAssocId, psGeneric->psElementSpec, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_DbvElementSpec_opt(DbvAssocId lAssocId, DbvElementSpec *psElementSpec, char *error_buffer, DbvError **ppsSIError)
{
	if (psElementSpec)
		return(DBVcheck_DbvElementSpec(lAssocId, psElementSpec, error_buffer, ppsSIError));

	return SI_OK;

}



int DBVcheck_DbvElementSpec(DbvAssocId lAssocId, DbvElementSpec *psElementSpec, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psElementSpec->eElementSpecType) {
	case EST_Primitive:
		if (psElementSpec->uElementSpec.psPrimitive == NULL) {
			DBVsa_strcat(local_error_buffer, "uElementSpec.psPrimitive");
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}
		break;
	case EST_External:
		
		DBVsa_strcat(local_error_buffer, "uElementSpec.psExternal");
		
		if (psElementSpec->uElementSpec.psExternal == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		
		DBVsa_strcat(local_error_buffer, "->");
	        return(DBVcheck_dbv_external(lAssocId, psElementSpec->uElementSpec.psExternal, local_error_buffer, ppsSIError));
		
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eElementSpecType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_allowfuncs(DbvAssocId lAssocId, DbvPermAllowableFunctions **ppeAllowableFunctions, char *error_buffer, DbvError **ppsSIError)
{
        int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];
 
	DBVsa_strcpy(local_error_buffer, error_buffer);

        /* Cant be NULL because compulsory */
        if (ppeAllowableFunctions == NULL) {
		DBVsa_strcat(local_error_buffer, "ppeAllowableFunctions");
                return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

 
 
        while (ppeAllowableFunctions[index] != NULL) {
                if (DBVcheck_dbv_enum(*(ppeAllowableFunctions[index]), PAF_Delete, PAF_Invoke) == SI_NOTOK) {

                	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                	DBVsa_strcat(temp_error_buffer, "ppeAllowableFunctions[");
			if (index < MAX_INDEX)
                		sprintf(index_buffer, "%d", index);
                	DBVsa_strcat(temp_error_buffer, index_buffer);
                	DBVsa_strcat(temp_error_buffer, "]");

                        return(DBVdbv_invalid_enum_error(lAssocId, temp_error_buffer, ppsSIError));
                }
		index++;
        }
 
        return SI_OK;
}
 


/* Safe array routines */

int DBVsa_clear_array(char *array)
{
        memset(array, 0, SA_ARRAY_SIZE);
}
 
 
int DBVsa_strcat(char *dest, char *src)
{
        strncat(dest, src, ( (SA_ARRAY_SIZE - 1)  - (strlen(dest)) ) );
}
 
 
 
int DBVsa_strcpy(char *dest, char *src)
{
        strncpy(dest, src, (SA_ARRAY_SIZE-1));
}




int DBVcheck_oidformat(char *oid)
{

	/* Cannot have 0 at start of any object identifer component number.
 	   i.e. 204 is treated as valid but 0204 is treated as invalid */

	char token_obtained[STRING_LEN];
	char oidcopy[STRING_LEN];
	int current_pos = 0;

	memset(oidcopy, 0, sizeof(oidcopy));

	if (strlen(oid) > STRING_LEN - 2)
		return SI_NOTOK;
	else
		strcpy(oidcopy, oid);

	/* To be used in DBVget_token_from_string() string must end in
	   a delimiter */
	strcat(oidcopy, OID_SEP);

	memset(token_obtained, 0, sizeof(token_obtained));

	while (strcmp(token_obtained, "NO SUCH TOKEN") != 0) {
		DBVget_token_from_string(token_obtained, OID_SEP, oidcopy); 

		/* If got a token */
		if (strcmp(token_obtained, "NO SUCH TOKEN") != 0) {

			current_pos += (strlen(token_obtained) + strlen(OID_SEP));
			if (DBVcheck_if_number(token_obtained) == SI_NOTOK)
				return SI_NOTOK;
		 } else {

			/* If not at end of list (to catch .. in oids) */
			/* [ +1 because of the strcated OID_SEP */
			if (current_pos != strlen(oid) + 1)
				return SI_NOTOK;
		}
	}
		

	return SI_OK;

}



int DBVcheck_if_number(char *strtok_ptr)
{
	unsigned long longval = 0;
	char convert[STRING_LEN];

	longval = atol(strtok_ptr);

	/* if not zero then probabably ok */
	if (longval != 0) {
		sprintf(convert, "%lu" , longval);

		/* but check that the value produced is the same as 
		   the string passed in (to cater for cases like 66,66) */
		if (strcmp(strtok_ptr, convert) != 0)
			return SI_NOTOK;
		else
			return SI_OK;
	}

	/* If not one character in length or first characetr not 0 */
	if ( (strlen(strtok_ptr) != 1) || (strtok_ptr[0] != '0') )
		return SI_NOTOK;
	
	return SI_OK;
}



int DBVsnacc_to_dbv_presstat(DbvPresentStatus *ePresentStatus, PresentStatus presentStatus)
{
        /* Switch on the snacc value */
        switch (presentStatus) {
        case SUCCESS:
                *ePresentStatus = PS_Success;
                break;
        case PARTIAL_1:
                *ePresentStatus = PS_Partial_1;
                break;
        case PARTIAL_2:
                *ePresentStatus = PS_Partial_2;
                break;
        case PARTIAL_3:
                *ePresentStatus = PS_Partial_3;
                break;
        case PARTIAL_4:
                *ePresentStatus = PS_Partial_4;
                break;
        case FAILURE:
                *ePresentStatus = PS_Failure;
                break;
        default:
                *ePresentStatus = PS_Invalid;
                break;
        }
}
 


int DBVdbv_to_snacc_presstat(PresentStatus *presentStatus, DbvPresentStatus ePresentStatus)
{

        switch (ePresentStatus) {
        case PS_Success:
                *presentStatus = SUCCESS;
                break;
        case PS_Partial_1:
                *presentStatus = PARTIAL_1;
                break;
        case PS_Partial_2:
                *presentStatus = PARTIAL_2;
                break;
        case PS_Partial_3:
                *presentStatus = PARTIAL_3;
                break;
        case PS_Partial_4:
                *presentStatus = PARTIAL_4;
                break;
        case PS_Failure:
                *presentStatus = FAILURE;
               break;
        }
}
 

int DBVdbv_to_snacc_bool(AsnBool *snacc_bool, DbvBool *dbv_bool, jmp_buf env)
{
        if (*dbv_bool == DBV_TRUE)
                *snacc_bool = TRUE;
        else
                *snacc_bool = FALSE;
}


int DBVdbv_to_snacc_bool_opt(AsnBool **snacc_bool, DbvBool *dbv_bool, jmp_buf env)
{
	if (dbv_bool) {
		*snacc_bool = (AsnBool *) Asn1Alloc(sizeof (AsnBool));
		CheckAsn1Alloc(*snacc_bool, env);

		DBVdbv_to_snacc_bool(*snacc_bool, dbv_bool, env);
	}

}



int DBVsnacc_to_dbv_bool(DbvBool *dbv_bool, AsnBool *snacc_bool, jmp_buf env)
{
        if (*snacc_bool == TRUE)
                *dbv_bool = DBV_TRUE;
        else
                *dbv_bool = DBV_FALSE;
}



int DBVsnacc_to_dbv_bool_opt(DbvBool **dbv_bool, AsnBool *snacc_bool, jmp_buf env)
{
        if (snacc_bool) {
		*dbv_bool = (DbvBool *) DbvAlloc (sizeof (DbvBool));
		CheckDbvAlloc(*dbv_bool, env);
		DBVsnacc_to_dbv_bool(*dbv_bool, snacc_bool, env);
	}

}




/* Additions made when explain was added - optional field malloc wrappers.
   Useful to explain and likely to be useful to other setups */

int DBVdbv_to_snacc_int_opt(AsnInt **maxResultSets, Ulong *pulMaxResultSets, jmp_buf env)
{
        if (pulMaxResultSets != NULL) {
                *maxResultSets = (AsnInt*) Asn1Alloc (sizeof (AsnInt));
                CheckAsn1Alloc(*maxResultSets, env);
                **maxResultSets = *pulMaxResultSets;
        }
}



int DBVdbv_to_snacc_intunit_opt(struct IntUnit** timeoutInterval, DbvIntUnit *psTimeoutInterval, jmp_buf env)
{
	if (psTimeoutInterval != NULL) {
		*timeoutInterval = (struct IntUnit*) Asn1Alloc (sizeof (struct IntUnit));
		CheckAsn1Alloc(*timeoutInterval, env);
		DBVdbv_to_snacc_intunit(*timeoutInterval, psTimeoutInterval, env);
	}
}



int DBVdbv_to_snacc_interstring_list(TargetInfoSeqOf *nicknames, DbvInternationalString **ppsNicknames, jmp_buf env)
{
        InternationalString *data_ptr;
        InternationalString **data_ptr_address;

        int current_listnode = 0;

        while (ppsNicknames[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(nicknames);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(InternationalString));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_inter_string(data_ptr, ppsNicknames[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_interstring_list_opt(TargetInfoSeqOf **nicknames, DbvInternationalString **ppsNicknames, jmp_buf env)
{

	if (ppsNicknames != NULL) {
		*nicknames = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*nicknames, env);
		DBVdbv_to_snacc_interstring_list(*nicknames, ppsNicknames, env);
	}
}



/* DbvGeneralizedTime is a typedefed char * in sitypes.h */
int DBVdbv_to_snacc_gentime_opt(GeneralizedTime *dateAdded, DbvGeneralizedTime pcDateAdded, jmp_buf env)
{
	if (pcDateAdded != NULL) 
		DBVdbv_to_snacc_gentime(dateAdded, pcDateAdded, env);
}



int DBVdbv_to_snacc_gentime(GeneralizedTime *dateAdded, DbvGeneralizedTime pcDateAdded, jmp_buf env)
{
        char *string = NULL;
        unsigned long length = 0;

	length = strlen(pcDateAdded);
	string = pcDateAdded;

	dateAdded->octetLen = length;
	dateAdded->octs = (char *) Asn1Alloc (length + 1);
	CheckAsn1Alloc(dateAdded->octs, env);
	memcpy(dateAdded->octs, string, length);
	 
}


int DBVdbv_to_snacc_otherinfo_opt(OtherInformation **otherInfo, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{

	if (ppsOtherInformation != NULL) {
		*otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*otherInfo, env);
		DBVdbv_to_snacc_otherinfo(*otherInfo, ppsOtherInformation, env);
	}

}



void DBVdbv_to_snacc_DbvInternationalString_comp(InternationalString **ianaType, DbvInternationalString *psIanaType, jmp_buf env)
{
	DBVdbv_to_snacc_interstring_comp(ianaType, psIanaType, env);
}



int DBVdbv_to_snacc_interstring_comp(InternationalString **ianaType, DbvInternationalString *psIanaType, jmp_buf env)
{

	*ianaType = (InternationalString *) DbvAlloc(sizeof(InternationalString));
	CheckAsn1Alloc(*ianaType, env);
	DBVdbv_to_snacc_inter_string(*ianaType, psIanaType, env);
}



void DBVdbv_to_snacc_DbvObjectIdentifier_opt(AsnOid *tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{
	DBVdbv_to_snacc_objectident_opt(tagSet, pcTagSet, env);
}



int DBVdbv_to_snacc_objectident_opt(AsnOid *tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{
        if (pcTagSet != NULL)
		DBVdbv_to_snacc_objectident(tagSet, pcTagSet, env);
}



void DBVdbv_to_snacc_DbvObjectIdentifier_comp(AsnOid **tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{
	DBVdbv_to_snacc_objectident_comp(tagSet, pcTagSet, env);
}



int DBVdbv_to_snacc_objectident_comp(AsnOid **tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{

        if (pcTagSet != NULL) {
		*tagSet = (AsnOid *) Asn1Alloc(sizeof (AsnOid));
		CheckAsn1Alloc(*tagSet, env);
		DBVdbv_to_snacc_objectident(*tagSet, pcTagSet, env);
	}
}



void DBVdbv_to_snacc_DbvObjectIdentifier(AsnOid *tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{
	DBVdbv_to_snacc_objectident(tagSet, pcTagSet, env);
}



int DBVdbv_to_snacc_objectident(AsnOid *tagSet, DbvObjectIdentifier pcTagSet, jmp_buf env)
{
        /* Careful - note 'reverse' parameters !! */
	ASN_MakeOid((char *) pcTagSet, tagSet, env);
}



int DBVdbv_to_snacc_strornum_comp(struct StringOrNumeric** tagValue, DbvStringOrNumeric *psTagValue, jmp_buf env)
{
	*tagValue = (struct StringOrNumeric*) Asn1Alloc (sizeof (struct StringOrNumeric));
	CheckAsn1Alloc(*tagValue, env);
	DBVdbv_to_snacc_strornum(*tagValue, psTagValue, env);
	
}



int DBVdbv_to_snacc_strornum_opt(struct StringOrNumeric** tagValue, DbvStringOrNumeric *psTagValue, jmp_buf env)
{
	if (psTagValue != NULL) 
		DBVdbv_to_snacc_strornum_comp(tagValue, psTagValue, env);
	
}



int DBVdbv_to_snacc_octetstring_comp(AsnOcts **octets, DbvOctetString *psOctets, jmp_buf env)
{
	*octets = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	CheckAsn1Alloc(*octets, env);
	DBVdbv_to_snacc_octet_string(*octets, psOctets, env);
}





int DBVsnacc_to_dbv_int_opt(Ulong **ppulMaxResultSets, AsnInt *maxResultSets, jmp_buf env)
{
        if (maxResultSets != NULL) {
                *ppulMaxResultSets  = (Ulong *) DbvAlloc(sizeof(Ulong));
                CheckDbvAlloc(*ppulMaxResultSets, env);

                **ppulMaxResultSets = *maxResultSets;
        }

}



int DBVsnacc_to_dbv_intunit_opt(DbvIntUnit **ppsTimeoutInterval, struct IntUnit *timeoutInterval, jmp_buf env)
{

	if (timeoutInterval != NULL) {
		*ppsTimeoutInterval = (DbvIntUnit *) DbvAlloc (sizeof (DbvIntUnit));
		DBVsnacc_to_dbv_intunit(*ppsTimeoutInterval, timeoutInterval, env);
	}
}



int DBVsnacc_to_dbv_gentime_opt(DbvGeneralizedTime *ppcDateAdded, GeneralizedTime dateAdded, jmp_buf env)
{
	char *string = NULL;
        unsigned long length = 0;

	if (dateAdded.octs) {
		string = dateAdded.octs;
		length = dateAdded.octetLen;
		*ppcDateAdded = (DbvGeneralizedTime) DbvAlloc (length + 1);
		CheckDbvAlloc(*ppcDateAdded, env);
		memcpy(*ppcDateAdded, string, length);
	}
}



int DBVsnacc_to_dbv_otherinfo_opt(DbvOtherInformation ***pppsOtherInformation, OtherInformation *otherInfo, jmp_buf env)
{

        int num_elements = 0;

        if (otherInfo != NULL) {
                num_elements = AsnListCount(otherInfo);
                *pppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
                CheckDbvAlloc(*pppsOtherInformation, env);
                DBVsnacc_to_dbv_otherinfo(*pppsOtherInformation, otherInfo, env);
        }
}



int DBVsnacc_to_dbv_interstring_comp(DbvInternationalString **ppsIanaType, InternationalString *ianaType, jmp_buf env)
{

	*ppsIanaType = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
	CheckDbvAlloc(*ppsIanaType, env);
	DBVsnacc_to_dbv_inter_string(*ppsIanaType, ianaType, env);
}



int DBVsnacc_to_dbv_interstring_opt(DbvInternationalString **ppsName, InternationalString *name, jmp_buf env)
{
        if (name->octs != NULL) {
                *ppsName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
                CheckDbvAlloc(*ppsName, env);
                DBVsnacc_to_dbv_inter_string(*ppsName, name, env);
        }
}



int DBVsnacc_to_dbv_interstring_list_opt(DbvInternationalString ***ppsNicknames, TargetInfoSeqOf *nicknames, jmp_buf env)
{
        int num_elements = 0;

        if (nicknames != NULL) {
                num_elements = AsnListCount(nicknames);
                *ppsNicknames = (DbvInternationalString **) DbvAlloc (sizeof (DbvInternationalString *) * (num_elements + 1));
                CheckDbvAlloc(*ppsNicknames, env);
                DBVsnacc_to_dbv_inter_list(*ppsNicknames, nicknames, env);
        }
}



int DBVsnacc_to_dbv_inter_list(DbvInternationalString **ppsNicknames, TargetInfoSeqOf *nicknames, jmp_buf env)
{
        InternationalString *data_ptr;
        int index = 0;

        FOR_EACH_LIST_ELMT(data_ptr, nicknames) {
                ppsNicknames[index] = (DbvInternationalString*) DbvAlloc (sizeof (DbvInternationalString));
                CheckDbvAlloc(ppsNicknames[index], env);
                DBVsnacc_to_dbv_inter_string(ppsNicknames[index], data_ptr, env)
;
                index++;
        }
        ppsNicknames[index] = NULL;
}


int DBVsnacc_to_dbv_ulong_list_opt(Ulong ***pppulOperators, RpnCapabilitiesSeqOf *operators, jmp_buf env)
{
        int num_elements = 0;

        if (operators != NULL) {
                num_elements = AsnListCount(operators);
                *pppulOperators = (Ulong **) DbvAlloc (sizeof (Ulong *) * (num_elements + 1));
                CheckDbvAlloc(*pppulOperators, env);
                DBVsnacc_to_dbv_exp_ullist(*pppulOperators, operators, env);

        }
}


int DBVsnacc_to_dbv_exp_ullist(Ulong **ppulOperators, RpnCapabilitiesSeqOf *operators, jmp_buf env)
{
        AsnInt *data_ptr;
        int index = 0;

        FOR_EACH_LIST_ELMT(data_ptr, operators) {
                ppulOperators[index] = (Ulong*) DbvAlloc (sizeof (Ulong));
                CheckDbvAlloc(ppulOperators[index], env);
                *ppulOperators[index] = *data_ptr;
                index++;
        }
        ppulOperators[index] = NULL;
}



/* moved here from target_z3950_present.c because needed in explain */

int DBVsnacc_to_dbv_elset_name(DbvElementSetNames *psElementSetNames, struct ElementSetNames* simple, jmp_buf env)
{
        switch(simple->choiceId) {
                case ELEMENTSETNAMES_GENERICELEMENTSETNAME:
                        DBVsnacc_to_dbv_el_generic(psElementSetNames, simple, env);
                        break;
                case ELEMENTSETNAMES_DATABASESPECIFIC:
                        DBVsnacc_to_dbv_el_specific(psElementSetNames, simple, env);
                        break;
        }

}


int DBVsnacc_to_dbv_el_generic(DbvElementSetNames *psElementSetNames, struct ElementSetNames* simple, jmp_buf env)
{
        psElementSetNames->eElementSetType = EST_Generic;
        psElementSetNames->uElementSet.psGeneric = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	CheckDbvAlloc(psElementSetNames->uElementSet.psGeneric, env);

        DBVsnacc_to_dbv_inter_string(psElementSetNames->uElementSet.psGeneric, simple->a.genericElementSetName, env);
}


int DBVsnacc_to_dbv_el_specific(DbvElementSetNames *psElementSetNames, struct ElementSetNames* simple, jmp_buf env)
{
        ElementSetNamesSeqOfSeq *data_ptr;
        int num_elements = 0;
        int index = 0;


        if (simple->a.databaseSpecific != NULL) {

                psElementSetNames->eElementSetType = EST_DatabaseSpecific;

                /* Find the number of elements in the AsnList so that we can
                malloc a correct size for ppsAdditionalRanges */
                num_elements = AsnListCount(simple->a.databaseSpecific);

                /* +1 to allocate space for the final NULL */
                psElementSetNames->uElementSet.ppsElementSetsDbSpecific = (DbvElementSetsDbSpecific **) DbvAlloc (sizeof(DbvElementSetsDbSpecific *) * (num_elements + 1));
                CheckDbvAlloc(psElementSetNames->uElementSet.ppsElementSetsDbSpecific, env);

                FOR_EACH_LIST_ELMT(data_ptr, simple->a.databaseSpecific) {

                        /* Create the dbv array of pointers */
                        psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index] = (DbvElementSetsDbSpecific *) DbvAlloc(sizeof (DbvElementSetsDbSpecific));
                        CheckDbvAlloc(psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index], env);

                        /* Alloc space for the Dbv psDatabaseName and
                        psElementSetName because they are pointers */


                        /* psDatabaseName */
                        psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
                        CheckDbvAlloc(psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psDatabaseName, env);
                        DBVsnacc_to_dbv_inter_string(psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psDatabaseName, &data_ptr->dbName, env);


                        /* psElementSetName  */
                        psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psElementSetName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
                        CheckDbvAlloc(psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psElementSetName, env);

                        DBVsnacc_to_dbv_inter_string(psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index]->psElementSetName, &data_ptr->esn, env);


                        index++;
                }

                psElementSetNames->uElementSet.ppsElementSetsDbSpecific[index] =
 NULL;
        }

}



int DBVsnacc_to_dbv_DbvObjectIdentifier_comp(DbvObjectIdentifier *ppcSchema, AsnOid *schema, jmp_buf env)
{
	DBVsnacc_to_dbv_objectident_comp(ppcSchema, schema, env);
}



int DBVsnacc_to_dbv_objectident_comp(DbvObjectIdentifier *ppcSchema, AsnOid *schema, jmp_buf env)
{
	char *oid = NULL;

	oid = (char *) ASN_EncodedToString(schema);

	*ppcSchema = (DbvObjectIdentifier) DbvAlloc (strlen(oid) +1);
        CheckDbvAlloc(*ppcSchema, env);
        memcpy(*ppcSchema, oid, strlen(oid));
}



int DBVsnacc_to_dbv_DbvObjectIdentifier_opt(DbvObjectIdentifier *ppcSchema, AsnOid *schema, jmp_buf env)
{
	DBVsnacc_to_dbv_objectident_opt(ppcSchema, schema, env);
}



int DBVsnacc_to_dbv_objectident_opt(DbvObjectIdentifier *ppcSchema, AsnOid *schema, jmp_buf env)
{
	if (schema != NULL) 
		if (schema->octetLen != 0)
			DBVsnacc_to_dbv_objectident_comp(ppcSchema, schema, env);
}


int DBVsnacc_to_dbv_nullpointer_opt(DbvNull *ppvDefaultTagType, AsnNull *defaultTagType, jmp_buf env)
{

	if (defaultTagType != NULL) {
		*ppvDefaultTagType = (char *) DbvAlloc(1);
		CheckDbvAlloc(*ppvDefaultTagType, env);
	}
}



int DBVsnacc_to_dbv_strornum_opt(DbvStringOrNumeric **ppsDefaultValue, struct StringOrNumeric *defaultValue, jmp_buf env)
{

	if (defaultValue != NULL) {
		*ppsDefaultValue = (DbvStringOrNumeric *) DbvAlloc (sizeof (DbvStringOrNumeric));
		CheckDbvAlloc(*ppsDefaultValue, env);
		DBVsnacc_to_dbv_strornum(*ppsDefaultValue, defaultValue, env);
	}
}


int DBVsnacc_to_dbv_octetstring_comp(DbvOctetString **ppsOctets, AsnOcts *octets, jmp_buf env)
{
	*ppsOctets = (DbvOctetString *) DbvAlloc(sizeof(DbvOctetString));
	CheckDbvAlloc(*ppsOctets, env);
	DBVsnacc_to_dbv_octet_string(*ppsOctets, octets, env);
}



/* shared checking functions */

int DBVcheck_DbvOtherInformation_opt(DbvAssocId lAssocId, DbvOtherInformation **ppsOtherInformation, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	if (ppsOtherInformation) 
                return(DBVcheck_dbv_otherinfo(lAssocId, ppsOtherInformation, local_error_buffer, ppsSIError));
        

	return SI_OK;
}



int DBVcheck_DbvIntUnit_opt(DbvAssocId lAssocId, DbvIntUnit *psTimeoutInterval, 
char *error_buffer, DbvError **ppsSIError) 
{
        char local_error_buffer[SA_ARRAY_SIZE];
                
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
        if (psTimeoutInterval)
                return(DBVcheck_DbvIntUnit(lAssocId, psTimeoutInterval, local_error_buffer, ppsSIError));


        return SI_OK;
}



int DBVcheck_DbvUnit_opt(DbvAssocId lAssocId, DbvUnit *psUnitUsed, char *error_buffer, DbvError **ppsSIError)
{

        char local_error_buffer[SA_ARRAY_SIZE];
                
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
        if (psUnitUsed)
		return(DBVcheck_DbvUnit(lAssocId, psUnitUsed, local_error_buffer, ppsSIError));

        return SI_OK;
}




int DBVsnacc_to_dbv_DbvOctetString_opt(DbvOctetString **ppsReferenceId, AsnOcts 
*referenceId, jmp_buf env)
{
	/* o */
	if (referenceId->octetLen != 0) {
		*ppsReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));                
		CheckDbvAlloc(*ppsReferenceId, env);
		DBVsnacc_to_dbv_octet_string(*ppsReferenceId, referenceId, env);
	}
}



int DBVdbv_to_snacc_DbvOctetString_opt(AsnOcts *referenceId, DbvOctetString *psReferenceId, jmp_buf env)
{
	if (psReferenceId) 
		DBVdbv_to_snacc_octet_string(referenceId, psReferenceId, env);
}


int DBVdbv_to_snacc_DbvBool_opt(AsnBool **resultSetWanted, DbvBool *pfResultSetWanted, jmp_buf env)
{
	/* o */
	if (pfResultSetWanted) {
		*resultSetWanted = (AsnBool *) DbvAlloc (sizeof (AsnBool));
		CheckAsn1Alloc(*resultSetWanted, env);
		DBVdbv_to_snacc_bool(*resultSetWanted, pfResultSetWanted, env);
	}
}



int DBVsnacc_to_dbv_DbvBool_opt(DbvBool **ppfSuspendedFlag, AsnBool* suspendedFlag, jmp_buf env)
{ 
	if (suspendedFlag) {
		*ppfSuspendedFlag = (DbvBool *) DbvAlloc (sizeof (DbvBool));
		CheckDbvAlloc(*ppfSuspendedFlag, env);
		DBVsnacc_to_dbv_bool(*ppfSuspendedFlag, suspendedFlag, env);
	}
}



int DBVdbv_to_snacc_DbvDiagRec_opt(DiagRec** diagnostic, DbvDiagRec *psDiagnostic, jmp_buf env)
{
	if (psDiagnostic) {
		*diagnostic = (DiagRec *) Asn1Alloc(sizeof (DiagRec));
		CheckAsn1Alloc(*diagnostic, env);	
		DBVdbv_to_snacc_diag_rec(*diagnostic, psDiagnostic, env);
	}
}



void DBVdbv_to_snacc_DbvDiagRec_comp(struct DiagRec** diagnostic, DbvDiagRec *psDiagnostic, jmp_buf env)
{
	DBVdbv_to_snacc_DbvDiagRec_opt(diagnostic, psDiagnostic, env);
}



void DBVsnacc_to_dbv_DbvDiagRec_opt(DbvDiagRec **ppsDiagnostic, DiagRec *diagnostic, jmp_buf env)
{  
	if (diagnostic) {
		*ppsDiagnostic = (struct DbvDiagRec*) DbvAlloc (sizeof (struct DbvDiagRec));
		CheckDbvAlloc(*ppsDiagnostic, env);
		DBVsnacc_to_dbv_diag_rec(*ppsDiagnostic, diagnostic, env);
	}
}



void DBVsnacc_to_dbv_DbvDiagRec_comp(DbvDiagRec **ppsDiagnostic, struct DiagRec *diagnostic, jmp_buf env)
{
	DBVsnacc_to_dbv_DbvDiagRec_opt(ppsDiagnostic, diagnostic, env);
}



int DBVcheck_list_opt(DbvAssocId lAssocId, void **listptr, char *error_buffer, char *list_name, DbvError **ppsSIError, 
int (*func) (DbvAssocId, void *, char *, DbvError **)
)
{
	if (listptr)
		return(DBVcheck_list(lAssocId, (void **) listptr, error_buffer, list_name, ppsSIError, DBV_TRUE, func));

	return SI_OK;
}



/* fSubstr - DBV_FALSE | DBV_TRUE if sub structures */
int DBVcheck_list(DbvAssocId lAssocId, void **listptr, char *error_buffer, char *list_name, DbvError **ppsSIError, DbvBool fSubstr,
int (*func) (DbvAssocId, void *, char *, DbvError **)
)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	while (listptr[index] != NULL) {
        
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, list_name);
		DBVsa_strcat(temp_error_buffer, "[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);

		if (fSubstr == DBV_TRUE)
			DBVsa_strcat(temp_error_buffer, "]->");
		else
			DBVsa_strcat(temp_error_buffer, "]");
	
		if ( (*func) (lAssocId, listptr[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		index++;
	}

	return SI_OK;
}



/* function for no sub structures (no -> deferenceing) */
int DBVcheck_list_opt_nosub(DbvAssocId lAssocId, void **listptr, char *error_buffer, char *list_name, DbvError **ppsSIError, 
int (*func) (DbvAssocId, void *, char *, DbvError **)
)
{
	if (listptr)
		return(DBVcheck_list(lAssocId, (void **) listptr, error_buffer, list_name, ppsSIError, DBV_FALSE, func));

	return SI_OK;
}



int DBVcheck_DbvObjectIdentifier_opt(DbvAssocId lAssocId, DbvObjectIdentifier pcPreferredResourceReportFormat, char *error_buffer, DbvError **ppsSIError)
{

        if (pcPreferredResourceReportFormat)
		return(DBVcheck_DbvObjectIdentifier(lAssocId, pcPreferredResourceReportFormat, error_buffer, ppsSIError));

        return SI_OK;
}




/*************************************************************************/
/* Special version for DBV-OSI II only originally in tbl.c */


Bool_	TBL_GetRecordDbv(TBL Table, DbvError **ppsError)
{
	tbl_TokenPtr    AToken;
	Bool_	    SyntaxError = FALSE;

	tbl_ClearBuffer(Table);
	if (tbl_ParseFieldIntoTree(Table) == TRUE) {
		AToken = tbl_Token(Table); /* Get field or record sep */
		if ( !( (AToken->Type == tbl_FieldSep) || 
		    (AToken->Type == tbl_RecTerm) ) ) {
			/* SyntaxError */
			return(FALSE);
		} else {
			while ( (AToken->Type == tbl_FieldSep) && (SyntaxError == FALSE) ) {
				if (tbl_ParseFieldIntoTree(Table) == TRUE) {
					tbl_ThrowToken(AToken);
					AToken = tbl_Token(Table);  /* get field or record sep */
					if ( !( (AToken->Type == tbl_FieldSep) || 
					    (AToken->Type == tbl_RecTerm) ) ) {
						SyntaxError = TRUE;
					}
				} else {
					SyntaxError = TRUE;
				}
			};
			if (AToken->Type == tbl_RecTerm && !SyntaxError) {
				tbl_ThrowToken(AToken);
				return(TRUE);
			} else {
				tbl_ThrowToken(AToken);
				/* There's something wrong in the TBL file! */
				DBVfill_error_structure(0, SI_Interface_Error, 
					0, DBV_FILE_ERROR, DBV_FILE_ERROR_S, 0,
					Association_Closed, ppsError);

				return(FALSE);
			}
		}
	} else {
		return (FALSE);
	}
}



/*********************************************************************/
/* Functions for DBV-OSI II only originally in memcheck.c */


/* Free a structure (NOT a PDU!!) */
void DBVFree(void *psItemToFree)
{
	free((char *)psItemToFree);
}



void *DBVAlloc(size_t sBytes, DbvError **ppsError)
{

	void *pSpace=NULL;

	pSpace = DbvAlloc(sBytes);
	if(pSpace==NULL)
	{
		/* Error */
		if (errno > 0 && errno < sys_nerr)
			sprintf(DBVErrorBuffer, DBV_MALLOC_S, sys_errlist[errno]);
		else
			sprintf(DBVErrorBuffer, DBV_MALLOC_S, "");

		DBVfill_error_structure(0,SI_OperatingSystem_Error, 0,
		DBV_MALLOC, DBVErrorBuffer,errno,
		Association_Open, ppsError);
		return NULL;
	}
	return pSpace;
}



int DBVAllocCheck(void *pStructure)
{
	if(pStructure == NULL)
		return SI_NOTOK;

        return SI_OK;
}



void DBVdbv_to_snacc_DbvRecordComposition(struct PresentRequestChoice* recordComposition, DbvRecordComposition *psRecordComposition, jmp_buf env)
{

        switch (psRecordComposition->eRecordCompositionType) {
        case RCT_ElementSetNames:
                recordComposition->choiceId = PRESENTREQUESTCHOICE_SIMPLE;
                recordComposition->a.simple = (struct ElementSetNames *) Asn1Alloc (sizeof (struct ElementSetNames));
                CheckAsn1Alloc(recordComposition->a.simple, env);
                DBVdbv_to_snacc_el_set_names(recordComposition->a.simple, psRecordComposition->uRecordComposition.psElementSetNames, env);
                break;

        case RCT_CompSpec:
                recordComposition->choiceId = PRESENTREQUESTCHOICE_COMPLEX;
                recordComposition->a.complex = (struct CompSpec *) Asn1Alloc (sizeof (struct CompSpec));
                CheckAsn1Alloc(recordComposition->a.complex, env);
                DBVdbv_to_snacc_DbvCompSpec(recordComposition->a.complex, psRecordComposition->uRecordComposition.psCompSpec, env);
                break;
        }
}



int DBVdbv_to_snacc_DbvCompSpec(struct CompSpec* complex, DbvCompSpec *psCompSpec, jmp_buf env)
{

	/* selectAlternativeSyntax */
	if (psCompSpec->fSelectAlternativeSyntax == DBV_TRUE)
		complex->selectAlternativeSyntax = TRUE;
	else
		complex->selectAlternativeSyntax = FALSE;

	/* generic */
	if (psCompSpec->psGeneric != NULL) {
		complex->generic = (struct Specification* ) Asn1Alloc (sizeof(struct Specification));
		CheckAsn1Alloc(complex->generic, env);

		DBVdbv_to_snacc_specification(complex->generic, psCompSpec->psGeneric, env);
	}


	/* dbSpecific */
	if (psCompSpec->ppsDbSpecific != NULL) {

		/* Create a snacc list */
		complex->dbSpecific = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(complex->dbSpecific, env);
		
		DBVdbv_to_snacc_DbvDbSpecificComp_list(complex->dbSpecific, psCompSpec->ppsDbSpecific, env);
	}


	/* recordSyntax */
	if (psCompSpec->ppcRecordSyntax != NULL) {

		complex->recordSyntax = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(complex->recordSyntax, env);

		DBVdbv_to_snacc_DbvObjectIdentifier_list(complex->recordSyntax, psCompSpec->ppcRecordSyntax, env);
	}
	
}

		

int DBVdbv_to_snacc_DbvDbSpecificComp_list(CompSpecSeqOf *dbSpecific, DbvDbSpecificComp **ppsDbSpecific, jmp_buf env)
{
	CompSpecSeqOfSeq *data_ptr;
	CompSpecSeqOfSeq **data_ptr_address;
	int current_listnode = 0;


	/* while not at end of Dbv List */
	while (ppsDbSpecific[current_listnode] != NULL) {

		/* create a snacc list node */
		data_ptr_address = AsnListAppend(dbSpecific);
		CheckAsn1Alloc(data_ptr_address, env);

               	*data_ptr_address = Asn1Alloc(sizeof(struct CompSpecSeqOfSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

               	data_ptr = *data_ptr_address;

		/* Now add the Dbv list item to the snacc list node */
		DBVdbv_to_snacc_inter_string(&data_ptr->db, &ppsDbSpecific[current_listnode]->sDatabaseName, env); 


		/* Another specification */
		data_ptr->spec = (struct Specification *) Asn1Alloc (sizeof(struct Specification));
		CheckAsn1Alloc(data_ptr->spec, env);

		DBVdbv_to_snacc_specification(data_ptr->spec, &ppsDbSpecific[current_listnode]->sSpecification, env);
		current_listnode++;
	}
}



int DBVdbv_to_snacc_DbvObjectIdentifier_list(CompSpecSeqOf1* recordSyntax,  DbvObjectIdentifier *ppcRecordSyntax, jmp_buf env)
{

	/* We have to set up and use a list of object identifiers as that it is
	   the Z39.50 protocol. In Dbv we only have one oid to put in
	   the list however */
	
	AsnOid *data_ptr;
	AsnOid **data_ptr_address;
	int current_listnode = 0;

	/* while not at end of Dbv List */
	while (ppcRecordSyntax[current_listnode] != NULL) {

		/* create a snacc list node */
		data_ptr_address = AsnListAppend(recordSyntax);
		CheckAsn1Alloc(data_ptr_address, env);

       		*data_ptr_address = Asn1Alloc(sizeof(AsnOid));
		CheckAsn1Alloc(*data_ptr_address, env);

        	data_ptr = *data_ptr_address;
	
		/* Put char* first parameter into second parameter */ 
		ASN_MakeOid((char *)ppcRecordSyntax[current_listnode], data_ptr, env);
		current_listnode++;
	}
}



void DBVsnacc_to_dbv_DbvRecordComposition(DbvRecordComposition *psRecordComposition, struct PresentRequestChoice* recordComposition, jmp_buf env)
{
	switch (recordComposition->choiceId) {
	case PRESENTREQUESTCHOICE_SIMPLE:
		psRecordComposition->eRecordCompositionType = RCT_ElementSetNames;
		psRecordComposition->uRecordComposition.psElementSetNames = (DbvElementSetNames *) DbvAlloc (sizeof(struct DbvElementSetNames));
		CheckDbvAlloc(psRecordComposition->uRecordComposition.psElementSetNames, env);

		DBVsnacc_to_dbv_elset_name(psRecordComposition->uRecordComposition.psElementSetNames, recordComposition->a.simple, env);
		break;
	case PRESENTREQUESTCHOICE_COMPLEX:
		psRecordComposition->eRecordCompositionType = RCT_CompSpec;
		psRecordComposition->uRecordComposition.psCompSpec = (DbvCompSpec *) DbvAlloc (sizeof(struct DbvCompSpec));
		CheckDbvAlloc(psRecordComposition->uRecordComposition.psCompSpec, env);

		DBVsnacc_to_dbv_DbvCompSpec(psRecordComposition->uRecordComposition.psCompSpec, recordComposition->a.complex, env);
		break;
	}
}

			

int DBVsnacc_to_dbv_DbvCompSpec(DbvCompSpec *psCompSpec, struct CompSpec* complexy, jmp_buf env)
{
	int num_elements = 0;

	/* selectAlternativeSyntax */
	if (complexy->selectAlternativeSyntax == TRUE)
		psCompSpec->fSelectAlternativeSyntax = DBV_TRUE;
	else
		psCompSpec->fSelectAlternativeSyntax = DBV_FALSE;

	/* generic */
	if (complexy->generic != NULL) {
		psCompSpec->psGeneric = (DbvSpecification *) DbvAlloc (sizeof (DbvSpecification));
		CheckDbvAlloc(psCompSpec->psGeneric, env);
		
		DBVsnacc_to_dbv_specification(psCompSpec->psGeneric, complexy->generic, env);
	}
		
	
	/* dbSpecific */
	if (complexy->dbSpecific != NULL) {

		/* Create a dbv list */

		/* +1 to allocate space for the final NULL */

		num_elements = AsnListCount(complexy->dbSpecific);
		psCompSpec->ppsDbSpecific = (DbvDbSpecificComp **) DbvAlloc (sizeof(DbvDbSpecificComp *) * (num_elements + 1));
		CheckDbvAlloc(psCompSpec->ppsDbSpecific, env);

		DBVsnacc_to_dbv_DbvDbSpecificComp(psCompSpec->ppsDbSpecific, complexy->dbSpecific, env);
	}


	/* recordSyntax */
	if (complexy->recordSyntax != NULL) {
		num_elements = AsnListCount(complexy->recordSyntax);

		/* DbvObjectIdentifier is a "char *" so "DbvObjectIdentifier *"
		   equates to "char **" */
		psCompSpec->ppcRecordSyntax = (DbvObjectIdentifier *) DbvAlloc (sizeof(DbvObjectIdentifier ) * (num_elements + 1));
		CheckDbvAlloc(psCompSpec->ppcRecordSyntax , env);

		DBVsnacc_to_dbv_DbvObjectIdentifier_list(psCompSpec->ppcRecordSyntax, complexy->recordSyntax, env);
	}
}



int DBVsnacc_to_dbv_DbvDbSpecificComp(DbvDbSpecificComp **ppsDbSpecific, CompSpecSeqOf *dbSpecific, jmp_buf env)
{

	CompSpecSeqOfSeq *data_ptr;
	int index = 0;
 

	/* If an additional ranges list exists */
	if (dbSpecific != NULL) {
		FOR_EACH_LIST_ELMT(data_ptr, dbSpecific) {
			ppsDbSpecific[index] = (DbvDbSpecificComp *) DbvAlloc(sizeof (DbvDbSpecificComp));
			CheckDbvAlloc(ppsDbSpecific[index], env);

			DBVsnacc_to_dbv_inter_string(& (ppsDbSpecific[index]->sDatabaseName), &data_ptr->db, env);
			DBVsnacc_to_dbv_specification(& (ppsDbSpecific[index]->sSpecification), data_ptr->spec, env);
			index++;
		}

		ppsDbSpecific[index] = NULL;
	}
}



/* Weve got a char *** here because of the typedef */
void DBVsnacc_to_dbv_DbvObjectIdentifier_list_comp(DbvObjectIdentifier **pppcRecordSyntax, CompSpecSeqOf1* recordSyntax, jmp_buf env)
{
        int num_elements = 0;

	
	num_elements = AsnListCount(recordSyntax);
	*pppcRecordSyntax = (DbvObjectIdentifier *) DbvAlloc (sizeof(DbvObjectIdentifier) * (num_elements + 1));  
	CheckDbvAlloc(*pppcRecordSyntax, env);
	DBVsnacc_to_dbv_DbvObjectIdentifier_list(*pppcRecordSyntax, recordSyntax, env);
}



/* Weve got a char *** here because of the typedef */
void DBVsnacc_to_dbv_DbvObjectIdentifier_list_opt(DbvObjectIdentifier **pppcRecordSyntax, CompSpecSeqOf1* recordSyntax, jmp_buf env)
{
	if (recordSyntax)
		DBVsnacc_to_dbv_DbvObjectIdentifier_list_comp(pppcRecordSyntax, recordSyntax, env);
}



int DBVsnacc_to_dbv_DbvObjectIdentifier_list(DbvObjectIdentifier *ppcRecordSyntax, CompSpecSeqOf1* recordSyntax, jmp_buf env)
{
	AsnOid *data_ptr;
	int num_elements = 0;
	int index = 0;
	char *oid;
 
	/* If an additional ranges list exists */
	if (recordSyntax != NULL) {

		FOR_EACH_LIST_ELMT(data_ptr, recordSyntax) {

                	oid = (char *) ASN_EncodedToString(data_ptr);

			/* Remember that DbvObjectIdentifier is a typdefed
			   char * */
			ppcRecordSyntax[index] = (DbvObjectIdentifier ) DbvAlloc(strlen(oid) + 1);
			CheckDbvAlloc(ppcRecordSyntax[index], env);
			memcpy(ppcRecordSyntax[index], oid, strlen(oid));
			index++;
		}
	}
}
	


/* Weve got a char *** here because of the typedef */
int DBVsnacc_to_dbv_objectident_list_opt(DbvObjectIdentifier **pppcDiagnosticsSets, AccessInfoSeqOf1 *diagnosticsSets, jmp_buf env)
{

        int num_elements = 0;

        if (diagnosticsSets != NULL) {
                num_elements = AsnListCount(diagnosticsSets);
                *pppcDiagnosticsSets = (DbvObjectIdentifier *) DbvAlloc (sizeof (DbvObjectIdentifier) * (num_elements + 1));  
                CheckDbvAlloc(*pppcDiagnosticsSets, env);
                DBVsnacc_to_dbv_exp_objectident_list(*pppcDiagnosticsSets, diagnosticsSets, env);
        }
}



int DBVcheck_DbvRecordComposition_opt(DbvAssocId lAssocId, DbvRecordComposition *psRecordComposition, char *error_buffer, DbvError **ppsSIError) 
{
	char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psRecordComposition)
		if (DBVcheck_DbvRecordComposition(lAssocId, psRecordComposition, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK; 
	
	return SI_OK;
}



int DBVcheck_DbvRecordComposition(DbvAssocId lAssocId, DbvRecordComposition *psRecordComposition, char *error_buffer, DbvError **ppsSIError) 
{
	char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psRecordComposition->eRecordCompositionType) {
	case RCT_ElementSetNames:
		DBVsa_strcat(local_error_buffer, "uRecordComposition.psElementSetNames");
        	if (psRecordComposition->uRecordComposition.psElementSetNames == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
 
		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvElementSetNames(lAssocId, psRecordComposition->uRecordComposition.psElementSetNames, local_error_buffer, ppsSIError));
		break;

	case RCT_CompSpec:
		DBVsa_strcat(local_error_buffer, "uRecordComposition.psCompSpec");
        	if (psRecordComposition->uRecordComposition.psCompSpec == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvCompSpec(lAssocId, psRecordComposition->uRecordComposition.psCompSpec, local_error_buffer, ppsSIError));
		break;
	default:
	   	DBVsa_strcat(local_error_buffer, "eRecordCompositionType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
	}

	return SI_OK;
}




int DBVcheck_DbvElementSetNames(DbvAssocId lAssocId, DbvElementSetNames *psElementSetNames, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

        switch (psElementSetNames->eElementSetType) {
        case EST_Generic:

		DBVsa_strcat(local_error_buffer, "uElementSet.psGeneric");
		if (psElementSetNames->uElementSet.psGeneric == NULL)
                        return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

                break;
        case EST_DatabaseSpecific:

		DBVsa_strcat(local_error_buffer, "uElementSet.");
		if (psElementSetNames->uElementSet.ppsElementSetsDbSpecific == NULL) {
			DBVsa_strcat(local_error_buffer, "ppsElementSetsDbSpecific");
                        return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}


                return(DBVcheck_dbspecific(lAssocId, psElementSetNames->uElementSet.ppsElementSetsDbSpecific, local_error_buffer, ppsSIError));
                break;
        default:
		DBVsa_strcat(local_error_buffer, "eElementSetType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvCompSpec(DbvAssocId lAssocId, DbvCompSpec *psCompSpec, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* fSelectAlternativeSyntax is compulsory */
	if (DBVcheck_dbv_bool(psCompSpec->fSelectAlternativeSyntax) == SI_NOTOK){
		DBVsa_strcat(local_error_buffer, "fSelectAlternativeSyntax");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}

	
	/* can be null because optional */
        if (psCompSpec->psGeneric != NULL) {
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "psGeneric->");
		if (DBVcheck_DbvSpecification(lAssocId, psCompSpec->psGeneric, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
		

	if (psCompSpec->ppsDbSpecific != NULL) 
		if (DBVcheck_DbvDbSpecificComp(lAssocId, psCompSpec->ppsDbSpecific, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		

	/* List of object identifiers */
	if (psCompSpec->ppcRecordSyntax != NULL) 
		if (DBVcheck_DbvObjectIdentifier_list(lAssocId, psCompSpec->ppcRecordSyntax, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	

	return SI_OK;
}




int DBVcheck_DbvDbSpecificComp(DbvAssocId lAssocId, DbvDbSpecificComp **ppsDbSpecific, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);


	/* Check all items in the list */
	while (ppsDbSpecific[index] != NULL) {

		/* Dont have to check sDatabaseName */
		
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppsDbSpecific[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]->sSpecification.");

		if (DBVcheck_DbvSpecification(lAssocId, &ppsDbSpecific[index]->sSpecification, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

	index++;
	}

        return SI_OK;
		
}


int DBVcheck_DbvObjectIdentifier_list(DbvAssocId lAssocId, DbvObjectIdentifier *ppcRecordSyntax, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

 
	/* Check all items in the list */
	while (ppcRecordSyntax[index] != NULL) {

                DBVsa_strcpy(temp_error_buffer, local_error_buffer);
                DBVsa_strcat(temp_error_buffer, "ppcRecordSyntax[");

		if (index < MAX_INDEX)
                	sprintf(index_buffer, "%d", index);
                DBVsa_strcat(temp_error_buffer, index_buffer);
                DBVsa_strcat(temp_error_buffer, "]");

		if (DBVcheck_dbv_recsyn(ppcRecordSyntax[index]) == SI_NOTOK)
			return(DBVdbv_oid_error(lAssocId, temp_error_buffer, ppsSIError));

		index++;
	}


        return SI_OK;
		
}



void DBVdbv_to_snacc_Ulong(AsnInt* proposedSets, Ulong *pulProposedSets, jmp_buf env)
{
        *proposedSets = *pulProposedSets;
}   



void DBVsnacc_to_dbv_Ulong(Ulong *pulProposedSets, AsnInt* proposedSets, jmp_buf env)
{
        *pulProposedSets = *proposedSets;
}   



/* A GeneralString is an octet string as far as snacc is concerned */
/* typedef AsnOcts GeneralString; */ /* [UNIVERSAL 27] IMPLICIT OCTET STRING */
/* asn_useful.h */

void DBVdbv_to_snacc_DbvGeneralString_opt(GeneralString *selectedLanguage, DbvGeneralString *psSelectedLanguage, jmp_buf env)
{
	DBVdbv_to_snacc_DbvOctetString_opt(selectedLanguage, (DbvOctetString *) psSelectedLanguage, env);
}



void DBVsnacc_to_dbv_DbvGeneralString_opt(DbvGeneralString** psSelectedLanguage, GeneralString *selectedLanguage, jmp_buf env)
{
	DBVsnacc_to_dbv_DbvOctetString_opt((DbvOctetString **) psSelectedLanguage, selectedLanguage, env);
}



