/* -----------  crxtypes.h -------------------------------------------*/


/* When using this file with other DBV-OSI II includes, include as follows

#include "sidefs.h"
#include "sitypes.h"
#include "sitypes2.h"
#include "dbvproto.h"
#include "iord_ext.h"
#include "itemordr.h"
*/

/*
#include "crxtypes.h"
*/






/*  Module Name: @(#)crxtypes.h	7.5
    Version: 12/6/96
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:    
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
    
 
*/


/* The Extended-Services service allows an origin to 
   create, modify or delete a task package at a target.

The origin sends an Extended Services Request PDU 
to the target requesting execution of the task.
This PDU contains an external defined in appendix EXT (such as
ESFormat-PersistentResultSet) containing a completed esRequest 
structure.

EXT external esRequest toKeep - parameters to be kept in the task package
EXT external esRequest notToKeep - parameters not to be kept in the task package


The target sends an Extended Service Response PDU to the
origin indicating that the request was
accepted or supplying an indication of the reason the request was
rejected. 
If accepted, this PDU contains the Task Package external defined in 
appendix REC. This external in turn contains an external defined in
appendix EXT (such as
ESFormat-PersistentResultSet) containing a completed taskPackage 
structure.

EXT external taskPackage originPart - parameters supplied by origin
EXT external taskPackage targetPart - parameters supplied by target  


It is possible to carry an external in an EXT external 
(e.g. the ILL external in the Item Order extended service
external).

e.g.

Origin sends:

Extended Service Request PDU containing
Item Order with esRequest set


Target Receives and responds with:

Extended Service Response PDU containing
ESTaskPackage record containing
Item Order with taskPackage set


*/





#if !defined(CRXTYPES_H)
#define CRXTYPES_H

/* --------------  DbvPrintExtendedService  ----------------------- */

/* This is an extended service external */ 

/* Proposed oid of this extended service: 1.2.840.10003.9.1000.70.1 */

typedef enum DbvPrintOriginPartToKeepType {
                    POPKT_Create = 1,
                    POPKT_Delete = 2,
                    POPKT_Invalid = -1
} DbvPrintOriginPartToKeepType;



typedef enum DbvCreateOptionType {
                    COPT_ResultSetReference = 1,
                    COPT_AccessionReference  = 2,
                    COPT_Invalid             = -1
} DbvCreateOptionType;


typedef struct DbvAddress {
DbvInternationalString      sPostalAddress;
DbvInternationalString     *psEmailAddress;
} DbvAddress;


typedef struct DbvResultSetReference {
DbvInternationalString      sResultSetId;
Ulong                       ulResultSetStartPoint;
Ulong                       ulNumberOfRecordsRequested;
DbvRange                  **ppsAdditionalRanges;
DbvRecordComposition       *psRecordComposition;
DbvAddress                  sAddress; 
} DbvResultSetReference;


typedef struct DbvAccessionReference {
DbvInternationalString      sReferenceId;
DbvRecordComposition       *psRecordComposition;
DbvAddress                  sAddress; 
} DbvAccessionReference;


typedef struct DbvCreateOption {
DbvCreateOptionType     eCreateOptionType;
union {
    DbvResultSetReference  *psResultSetReference;
    DbvAccessionReference  *psAccessionReference;
  }                uCreateOption;
} DbvCreateOption;


typedef struct DbvDeleteOption {
DbvInternationalString      sJobReference;
} DbvDeleteOption;


typedef struct DbvPrintOriginPartToKeep {
DbvPrintOriginPartToKeepType     ePrintOriginPartToKeepType;
union {
    DbvCreateOption  *psCreateOption;
    DbvDeleteOption  *psDeleteOption;
  }                uPrintOriginPartToKeep;
} DbvPrintOriginPartToKeep;


typedef struct DbvPrintRequestPackage {
DbvPrintOriginPartToKeep    *psOriginPartToKeep;
/* No OrignPartNotToKeep */
} DbvPrintRequestPackage;



typedef struct DbvPrintTargetPartToKeep {
DbvInternationalString      sJobReference;
} DbvPrintTargetPartToKeep;


typedef struct DbvPrintTaskPackage {
DbvPrintOriginPartToKeep      *psOriginPartToKeep;
DbvPrintTargetPartToKeep      sTargetPart;
} DbvPrintTaskPackage;




typedef enum DbvPrintReqOrRsp {
                    PROR_Request  = 1,
                    PROR_Response = 2,
                    PROR_Invalid  = -1 
} DbvPrintReqOrRsp;


typedef struct DbvPrintExtendedService {
DbvPrintReqOrRsp     ePrintReqRsp;
union {
    DbvPrintRequestPackage  *psPrintRequest;
    DbvPrintTaskPackage     *psPrintResponse;
  }                uPrintReqOrRsp;
} DbvPrintExtendedService;



/* --------------  DbvAlertExtendedService  ----------------------- */

/* This is an extended service external */ 

/* Proposed oid of this extended service: 1.2.840.10003.9.1000.70.2 */


typedef struct DbvAlertOriginPartToKeep {
DbvInternationalString     sAlertName; 
DbvAddress                *psAddress;
} DbvAlertOriginPartToKeep;


typedef struct DbvAlertRequestPackage {
DbvAlertOriginPartToKeep      *psOriginPartToKeep;
/* No OrignPartNotToKeep */
} DbvAlertRequestPackage;


typedef struct DbvAlertTaskPackage {
DbvAlertOriginPartToKeep      *psOriginPartToKeep;
/* No TargetPart */
} DbvAlertTaskPackage;


typedef enum DbvAlertReqOrRsp {
                    AROR_Request  = 1,
                    AROR_Response = 2,
                    AROR_Invalid  = -1
} DbvAlertReqOrRsp;


typedef struct DbvAlertExtendedService {
DbvAlertReqOrRsp     eAlertReqRsp;
union {
    DbvAlertRequestPackage  *psAlertRequest;
    DbvAlertTaskPackage     *psAlertResponse;
  }                uAlertReqOrRsp;
} DbvAlertExtendedService;




/* -------------- DbvCOrder  ----------------------- */

/* This is an external to be carried on an extended service external
 (analogous to the ILL external carried on the Item Order extended
  service external)
*/
 
/* Proposed oid of this external: 1.2.840.10003.9.4.1000.70.1 */

typedef struct DbvNormalOrder {
DbvRange             **ppsOrderRanges;
Ulong                  ulQuantity;
} DbvNormalOrder;


typedef enum DbvOrderRequestType {
                     ORT_NormalOrder = 1,
                     ORT_OrderRefs   = 2,
                     ORT_Invalid     = -1
} DbvOrderRequestType;


typedef struct DbvOrderRequest {
DbvOrderRequestType          eOrderRequestType;
union {
    DbvNormalOrder              *psNormalOrder;
    DbvInternationalString     **ppsOrderRefs;
  }               uOrderRequest;
} DbvOrderRequest;


typedef enum DbvCOrderType {
                     COT_OrderRequest  = 1,
                     COT_OrderResponse = 2,
                     COT_Invalid       = -1
} DbvCOrderType;


typedef struct DbvCOrder {
DbvCOrderType          eCOrderType;
union {
    DbvOrderRequest              *psOrderRequest;
    DbvInternationalString      **ppsOrderResponse;
  }               uCOrder;
} DbvCOrder;





#endif


