/*  Module Name: @(#)free_code.c	7.1
    Version: 11/15/96
    Author: 
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
	Routines to free DBV PDU structures
        Used by the origin and target.
*/


/* 

After the SI functions ...

(remove #includes when including, and top two funcs of apifree.c) 

../testtools/apifree.c:
../testtools/FreeC_DbvFuncs.c:
../testtools/FreeN_DbvFuncs.c:
../testtools/Free_DbvFuncs.c:
../testtools/Free_DbvFuncs2.c:
../testtools/Free_DbvFuncs3.c:
../testtools/Free_DbvFuncs4.c:

<CNTRL+G>
<current line>,$s/FreeDbv/DBVFreeDbv/g
<current line>,$s/Free_/DBVFree_/g
<current line>,$s/DEBUG/DEBUGZ/g



*/




#include "free_code.h" 



/* Origin end freeing function for structure returned by
   DbvReceiveDataOrigin() */
int SIFreeDbvOriginData(DbvOriginData **ppsOriginData)
{

	if (ppsOriginData == NULL)
		return SI_OK;

	if (*ppsOriginData == NULL) 
		return SI_OK;

	switch((*ppsOriginData)->eOriginEvent)
	{
	case OR_InitializeConf:
		DBVFreeDbvInitializeRsp((*ppsOriginData)->uOriginData.psInitializeRsp);
		break;
	case OR_SearchConf:
		DBVFree_DbvSearchRsp((*ppsOriginData)->uOriginData.psSearchRsp);
		break;
	case OR_PresentConf:
		DBVFree_DbvPresentRsp((*ppsOriginData)->uOriginData.psPresentRsp);
		break;
	case OR_DeleteResultSetConf:
		DBVFree_DbvDeleteResultSetRsp((*ppsOriginData)->uOriginData.
			psDeleteResultSetRsp);
		break;
	case OR_ScanConf:
		DBVFree_DbvScanRsp((*ppsOriginData)->uOriginData.psScanRsp);
		break;
	case OR_ResourceReportConf:
		DBVFree_DbvResourceReportRsp((*ppsOriginData)->uOriginData.
			psResourceReportRsp);
		break;
	case OR_SegmentationInd:
		DBVFree_DbvSegmentReq((*ppsOriginData)->uOriginData.psSegmentReq);
		break;
	case OR_SortConf:
		DBVFree_DbvSortRsp((*ppsOriginData)->uOriginData.psSortRsp);
		break;
	case OR_ExtendedServicesConf:
		DBVFree_DbvExtendedServicesRsp((*ppsOriginData)->uOriginData.
			psExtendedServicesRsp);
		break;
	case OR_CloseConf:
		DBVFree_DbvCloseReq((*ppsOriginData)->uOriginData.psCloseReq);
		break;
	case OR_CloseInd:
		DBVFree_DbvCloseReq((*ppsOriginData)->uOriginData.psCloseReq);
		break;
	case OR_AccessCntrlInd:
		DBVFree_DbvAccessControlReq((*ppsOriginData)->uOriginData.psAccessControlReq);
		break;
	case OR_ResourCntrlInd:
		DBVFree_DbvResourceControlReq((*ppsOriginData)->uOriginData.psResourceControlReq);
		break;
	default:
		break;
	}

	DbvFree(*ppsOriginData);
        *ppsOriginData = NULL;

	return SI_OK;
	
}



/* Target end freeing function for structure returned by
   DbvReceiveDataTarget() */
int SIFreeDbvTargetData(DbvTargetData **ppsTargetData)
{
	if (ppsTargetData == NULL)
		return SI_OK;

	if (*ppsTargetData == NULL)
		return SI_OK;

	switch((*ppsTargetData)->eTargetEvent)
	{
	case TA_InitializeInd:
		DBVFreeDbvInitializeReq((*ppsTargetData)->uTargetData.psInitializeReq);
		break;
	case TA_SearchInd:
		DBVFree_DbvSearchReq((*ppsTargetData)->uTargetData.psSearchReq);
		break;
	case TA_PresentInd: 
		DBVFree_DbvPresentReq((*ppsTargetData)->uTargetData.psPresentReq);
		break;
	case TA_DeleteResultSetInd: 
		DBVFree_DbvDeleteResultSetReq((*ppsTargetData)->uTargetData.
			psDeleteResultSetReq);
		break;
	case TA_ScanInd:
		DBVFree_DbvScanReq((*ppsTargetData)->uTargetData.psScanReq);
		break;
	case TA_ResourceReportInd: 
		DBVFree_DbvResourceReportReq((*ppsTargetData)->uTargetData.
			psResourceReportReq);
		break;
	case TA_SortInd: 
		DBVFree_DbvSortReq((*ppsTargetData)->uTargetData.psSortReq);
		break;
	case TA_ExtendedServicesInd: 
		DBVFree_DbvExtendedServicesReq((*ppsTargetData)->uTargetData.
			psExtendedServicesReq);
		break;
	case TA_CloseInd:
                DBVFree_DbvCloseReq((*ppsTargetData)->uTargetData.psCloseReq);
		break;
	case TA_CloseConf:
                DBVFree_DbvCloseReq((*ppsTargetData)->uTargetData.psCloseReq);
		break;
	case TA_ReleaseInd:
		DbvFree((*ppsTargetData)->uTargetData.peReleaseReq);
		break;
	case TA_AccessCntrlConf:
		DBVFree_DbvAccessControlRsp((*ppsTargetData)->uTargetData.psAccessControlRsp);
		break;
	case TA_ResourCntrlConf:
		DBVFree_DbvResourceControlRsp((*ppsTargetData)->uTargetData.psResourceControlRsp);
		break;
	case TA_TrigResourCntrlInd:
		DBVFree_DbvTriggerResourceControlReq((*ppsTargetData)->uTargetData.psTriggerResourceControlReq);
		break;
	default:
		break;
	}

	DbvFree(*ppsTargetData);
        *ppsTargetData = NULL;


	return SI_OK;
	
}



/************************************************************/
/* Origin end freeing functions for PDUs sent to a target */
int SIFreeDbvInitializeReq(DbvInitializeReq **ppsInitializeReq)
{
	if (ppsInitializeReq == NULL)
		return SI_OK;

	if (*ppsInitializeReq == NULL)
		return SI_OK;

	DBVFreeDbvInitializeReq(*ppsInitializeReq);

	*ppsInitializeReq = NULL;

	return SI_OK;
	
}



int SIFreeDbvSearchReq(DbvSearchReq **ppsSearchReq)
{
	if (ppsSearchReq == NULL)
		return SI_OK;

	if (*ppsSearchReq == NULL)
		return SI_OK;

	DBVFree_DbvSearchReq(*ppsSearchReq);

	*ppsSearchReq = NULL;

	return SI_OK;
	
}



int SIFreeDbvPresentReq(DbvPresentReq **ppsPresentReq)
{
	if (ppsPresentReq == NULL)
		return SI_OK;

	if (*ppsPresentReq == NULL)
		return SI_OK;

	DBVFree_DbvPresentReq(*ppsPresentReq);

	*ppsPresentReq = NULL;

	return SI_OK;
}



int SIFreeDbvDeleteResultSetReq(DbvDeleteResultSetReq **ppsDeleteResultSetReq)
{
	if (ppsDeleteResultSetReq == NULL)
		return SI_OK;

	if (*ppsDeleteResultSetReq == NULL)
		return SI_OK;

	DBVFree_DbvDeleteResultSetReq(*ppsDeleteResultSetReq);

	*ppsDeleteResultSetReq = NULL;

	return SI_OK;
}



int SIFreeDbvScanReq(DbvScanReq **ppsScanReq)
{
	if (ppsScanReq == NULL)
		return SI_OK;

	if (*ppsScanReq == NULL)
		return SI_OK;

	DBVFree_DbvScanReq(*ppsScanReq);

	*ppsScanReq = NULL;

	return SI_OK;
}



int SIFreeDbvResourceReportReq(DbvResourceReportReq **ppsResourceReportReq)
{
	if (ppsResourceReportReq == NULL)
		return SI_OK;

	if (*ppsResourceReportReq == NULL)
		return SI_OK;

	DBVFree_DbvResourceReportReq(*ppsResourceReportReq);

	*ppsResourceReportReq = NULL;

	return SI_OK;
}




int SIFreeDbvSortReq(DbvSortReq **ppsSortReq)
{
	if (ppsSortReq == NULL)
		return SI_OK;

	if (*ppsSortReq == NULL)
		return SI_OK;

	DBVFree_DbvSortReq(*ppsSortReq);

	*ppsSortReq = NULL;

	return SI_OK;
}



int SIFreeDbvExtendedServicesReq(DbvExtendedServicesReq **ppsExtendedServicesReq)
{
	if (ppsExtendedServicesReq == NULL)
		return SI_OK;

	if (*ppsExtendedServicesReq == NULL)
		return SI_OK;

	DBVFree_DbvExtendedServicesReq(*ppsExtendedServicesReq);

	*ppsExtendedServicesReq = NULL;

	return SI_OK;
}




int SIFreeDbvCloseReq(DbvCloseReq **ppsCloseReq)
{
	if (ppsCloseReq == NULL)
		return SI_OK;

	if (*ppsCloseReq == NULL)
		return SI_OK;

	DBVFree_DbvCloseReq(*ppsCloseReq);

	*ppsCloseReq = NULL;

	return SI_OK;
}



int SIFreeDbvOctetString(DbvOctetString **ppsOctetString)
{
	if (ppsOctetString == NULL)
		return SI_OK;

	if (*ppsOctetString == NULL)
		return SI_OK;

	DBVFreeDbvOctetString(*ppsOctetString);

	*ppsOctetString = NULL;

	return SI_OK;
}






int SIFreeDbvAccessControlRsp(DbvAccessControlRsp **ppsAccessControlRsp)
{
	if (ppsAccessControlRsp == NULL)
		return SI_OK;

	if (*ppsAccessControlRsp == NULL)
		return SI_OK;

	DBVFree_DbvAccessControlRsp(*ppsAccessControlRsp);

	*ppsAccessControlRsp = NULL;

	return SI_OK;
}




int SIFreeDbvResourceControlRsp(DbvResourceControlRsp **ppsResourceControlRsp)
{
	if (ppsResourceControlRsp == NULL)
		return SI_OK;

	if (*ppsResourceControlRsp == NULL)
		return SI_OK;

	DBVFree_DbvResourceControlRsp(*ppsResourceControlRsp);

	*ppsResourceControlRsp = NULL;

	return SI_OK;
}



int SIFreeDbvTriggerResourceControlReq(DbvTriggerResourceControlReq **ppsTriggerResourceControlReq)
{
	if (ppsTriggerResourceControlReq == NULL)
		return SI_OK;

	if (*ppsTriggerResourceControlReq == NULL)
		return SI_OK;

	DBVFree_DbvTriggerResourceControlReq(*ppsTriggerResourceControlReq);

	*ppsTriggerResourceControlReq = NULL;

	return SI_OK;
}



/***************************************************************/
/* Target end freeing functions for PDUs sent to an origin */
int SIFreeDbvInitializeRsp(DbvInitializeRsp **ppsInitializeRsp)
{
	if (ppsInitializeRsp == NULL)
		return SI_OK;

	if (*ppsInitializeRsp == NULL)
		return SI_OK;

	DBVFreeDbvInitializeRsp(*ppsInitializeRsp);

	*ppsInitializeRsp = NULL;

	return SI_OK;
}



int SIFreeDbvSearchRsp(DbvSearchRsp **ppsSearchRsp)
{
	if (ppsSearchRsp == NULL)
		return SI_OK;

	if (*ppsSearchRsp == NULL)
		return SI_OK;

	DBVFree_DbvSearchRsp(*ppsSearchRsp);

	*ppsSearchRsp = NULL;

	return SI_OK;
}



int SIFreeDbvPresentRsp(DbvPresentRsp **ppsPresentRsp)
{
	if (ppsPresentRsp == NULL)
		return SI_OK;

	if (*ppsPresentRsp == NULL)
		return SI_OK;

	DBVFree_DbvPresentRsp(*ppsPresentRsp);

	*ppsPresentRsp = NULL;

	return SI_OK;
}



int SIFreeDbvDeleteResultSetRsp(DbvDeleteResultSetRsp **ppsDeleteResultSetRsp)
{
	if (ppsDeleteResultSetRsp == NULL)
		return SI_OK;

	if (*ppsDeleteResultSetRsp == NULL)
		return SI_OK;

	DBVFree_DbvDeleteResultSetRsp(*ppsDeleteResultSetRsp);

	*ppsDeleteResultSetRsp = NULL;

	return SI_OK;
}



int SIFreeDbvScanRsp(DbvScanRsp **ppsScanRsp)
{
	if (ppsScanRsp == NULL)
		return SI_OK;

	if (*ppsScanRsp == NULL)
		return SI_OK;

	DBVFree_DbvScanRsp(*ppsScanRsp);

	*ppsScanRsp = NULL;

	return SI_OK;
}



int SIFreeDbvResourceReportRsp(DbvResourceReportRsp **ppsResourceReportRsp)
{
	if (ppsResourceReportRsp == NULL)
		return SI_OK;

	if (*ppsResourceReportRsp == NULL)
		return SI_OK;

	DBVFree_DbvResourceReportRsp(*ppsResourceReportRsp);

	*ppsResourceReportRsp = NULL;

	return SI_OK;
}



int SIFreeDbvSegmentReq(DbvSegmentReq **ppsSegmentReq)
{
	if (ppsSegmentReq == NULL)
		return SI_OK;

	if (*ppsSegmentReq == NULL)
		return SI_OK;

	DBVFree_DbvSegmentReq(*ppsSegmentReq);

	*ppsSegmentReq = NULL;

	return SI_OK;
}



int SIFreeDbvSortRsp(DbvSortRsp **ppsSortRsp)
{
	if (ppsSortRsp == NULL)
		return SI_OK;

	if (*ppsSortRsp == NULL)
		return SI_OK;

	DBVFree_DbvSortRsp(*ppsSortRsp);

	*ppsSortRsp = NULL;

	return SI_OK;
}



int SIFreeDbvExtendedServicesRsp(DbvExtendedServicesRsp **ppsExtendedServicesRsp)
{
	if (ppsExtendedServicesRsp == NULL)
		return SI_OK;

	if (*ppsExtendedServicesRsp == NULL)
		return SI_OK;

	DBVFree_DbvExtendedServicesRsp(*ppsExtendedServicesRsp);

	*ppsExtendedServicesRsp = NULL;

	return SI_OK;
}



int SIFreeDbvAccessControlReq(DbvAccessControlReq **ppsAccessControlReq)
{
	if (ppsAccessControlReq == NULL)
		return SI_OK;

	if (*ppsAccessControlReq == NULL)
		return SI_OK;

	DBVFree_DbvAccessControlReq(*ppsAccessControlReq);

	*ppsAccessControlReq = NULL;

	return SI_OK;
}



int SIFreeDbvResourceControlReq(DbvResourceControlReq **ppsResourceControlReq)
{
	if (ppsResourceControlReq == NULL)
		return SI_OK;

	if (*ppsResourceControlReq == NULL)
		return SI_OK;

	DBVFree_DbvResourceControlReq(*ppsResourceControlReq);

	*ppsResourceControlReq = NULL;

	return SI_OK;
}




/* Now for the imported testtool stuff ... */
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/



void DBVFreeDbvOriginData(DbvOriginData *psOriginData)
{


	if(psOriginData == NULL)
		return;

	switch(psOriginData->eOriginEvent)
	{
	case OR_InitializeConf:
		DBVFreeDbvInitializeRsp(psOriginData->uOriginData.psInitializeRsp);
		break;
	case OR_SearchConf:
		DBVFree_DbvSearchRsp(psOriginData->uOriginData.psSearchRsp);
		break;
	case OR_PresentConf:
		DBVFree_DbvPresentRsp(psOriginData->uOriginData.psPresentRsp);
		break;
	case OR_DeleteResultSetConf:
		DBVFree_DbvDeleteResultSetRsp(psOriginData->uOriginData.
			psDeleteResultSetRsp);
		break;
	case OR_ScanConf:
		DBVFree_DbvScanRsp(psOriginData->uOriginData.psScanRsp);
		break;
	case OR_ResourceReportConf:
		DBVFree_DbvResourceReportRsp(psOriginData->uOriginData.
			psResourceReportRsp);
		break;
	case OR_SegmentationInd:
		DBVFree_DbvSegmentReq(psOriginData->uOriginData.psSegmentReq);
		break;
	case OR_SortConf:
		DBVFree_DbvSortRsp(psOriginData->uOriginData.psSortRsp);
		break;
	case OR_ExtendedServicesConf:
		DBVFree_DbvExtendedServicesRsp(psOriginData->uOriginData.
			psExtendedServicesRsp);
		break;
	case OR_CloseConf:
		DBVFree_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_CloseInd:
		DBVFree_DbvCloseReq(psOriginData->uOriginData.psCloseReq);
		break;
	case OR_AccessCntrlInd:
		DBVFree_DbvAccessControlReq(psOriginData->uOriginData.psAccessControlReq);
		break;
	case OR_ResourCntrlInd:
		DBVFree_DbvResourceControlReq(psOriginData->uOriginData.psResourceControlReq);
		break;
	default:

		break;
	}


	
}

void DBVFreeDbvTargetData(DbvTargetData *psTargetData)
{

	if(psTargetData == NULL)
		return;

	switch(psTargetData->eTargetEvent)
	{
	case TA_InitializeInd:
		DBVFreeDbvInitializeReq(psTargetData->uTargetData.psInitializeReq);
		break;
	case TA_SearchInd:
		DBVFree_DbvSearchReq(psTargetData->uTargetData.psSearchReq);
		break;
	case TA_PresentInd: 
		DBVFree_DbvPresentReq(psTargetData->uTargetData.psPresentReq);
		break;
	case TA_DeleteResultSetInd: 
		DBVFree_DbvDeleteResultSetReq(psTargetData->uTargetData.
			psDeleteResultSetReq);
		break;
	case TA_ScanInd:
		DBVFree_DbvScanReq(psTargetData->uTargetData.psScanReq);
		break;
	case TA_ResourceReportInd: 
		DBVFree_DbvResourceReportReq(psTargetData->uTargetData.
			psResourceReportReq);
		break;
	case TA_SortInd: 
		DBVFree_DbvSortReq(psTargetData->uTargetData.psSortReq);
		break;
	case TA_ExtendedServicesInd: 
		DBVFree_DbvExtendedServicesReq(psTargetData->uTargetData.
			psExtendedServicesReq);
		break;
	case TA_CloseInd:
                DBVFree_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_CloseConf:
                DBVFree_DbvCloseReq(psTargetData->uTargetData.psCloseReq);
		break;
	case TA_ReleaseInd:
		DbvFree(psTargetData->uTargetData.peReleaseReq);
		break;
	case TA_AccessCntrlConf:
		DBVFree_DbvAccessControlRsp(psTargetData->uTargetData.psAccessControlRsp);
		break;
	case TA_ResourCntrlConf:
		DBVFree_DbvResourceControlRsp(psTargetData->uTargetData.psResourceControlRsp);
		break;
	case TA_TrigResourCntrlInd:
		DBVFree_DbvTriggerResourceControlReq(psTargetData->uTargetData.psTriggerResourceControlReq);
		break;
	default:

		break;
	}
	

}

void DBVFreeDbvInitializeRsp(DbvInitializeRsp *psInitializeRsp)
{
	if(psInitializeRsp->psReferenceId != NULL)
		DBVFreeDbvOctetString(psInitializeRsp->psReferenceId);
	DBVFreeDbvProtocolVersion(psInitializeRsp->pcProtocolVersion);
	DBVFreeDbvOptions(psInitializeRsp->pcOptions);
	if(psInitializeRsp->psImplementationId != NULL)
		DBVFreeDbvInternationalString(psInitializeRsp->psImplementationId);
	if(psInitializeRsp->psImplementationName != NULL)
		DBVFreeDbvInternationalString(psInitializeRsp->psImplementationName);
	if(psInitializeRsp->psImplementationVersion != NULL)
		DBVFreeDbvInternationalString(psInitializeRsp->psImplementationVersion);
	if(psInitializeRsp->psUserInformationField != NULL)
		DBVFreeDbvASN1External(psInitializeRsp->psUserInformationField);
	if(psInitializeRsp->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeRsp->ppsOtherInformation);
        DbvFree(psInitializeRsp);
}

void DBVFreeDbvInitializeReq(DbvInitializeReq *psInitializeReq)
{
	if(psInitializeReq->psReferenceId != NULL)
		DBVFreeDbvOctetString(psInitializeReq->psReferenceId);
	DBVFreeDbvProtocolVersion(psInitializeReq->pcProtocolVersion);
	DBVFreeDbvOptions(psInitializeReq->pcOptions);
	if(psInitializeReq->psAuthentication != NULL)
		DBVFreeDbvAuthentication(psInitializeReq->psAuthentication);
	if(psInitializeReq->psImplementationId != NULL)
		DBVFreeDbvInternationalString(psInitializeReq->psImplementationId);
	if(psInitializeReq->psImplementationName != NULL)
		DBVFreeDbvInternationalString(psInitializeReq->psImplementationName);
	if(psInitializeReq->psImplementationVersion != NULL)
		DBVFreeDbvInternationalString(psInitializeReq->psImplementationVersion);
	if(psInitializeReq->psUserInformationField != NULL)
		DBVFreeDbvASN1External(psInitializeReq->psUserInformationField);
	if(psInitializeReq->ppsOtherInformation != NULL)
		FreeN_DbvOtherInformation(psInitializeReq->ppsOtherInformation);
	DbvFree(psInitializeReq);
}

void DBVFreeDbvOctetString(DbvOctetString *psOctetString)
{
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
		DbvFree(psOctetString);
	}
}



void DBVFree_compulsory_DbvOctetString(DbvOctetString *psOctetString)
{   
	if(psOctetString != NULL)
	{
		if(psOctetString->pcValue != NULL)
			DbvFree(psOctetString->pcValue);
	}
}



void DBVFreeDbvProtocolVersion(DbvProtocolVersion pcProtocolVersion)
{
	if(pcProtocolVersion != NULL)
		DbvFree(pcProtocolVersion);
}

void DBVFreeDbvOptions(DbvOptions pcOptions)
{
	if(pcOptions != NULL)
		DbvFree(pcOptions);
}

void DBVFreeDbvInternationalString(DbvInternationalString *psInternationalString)
{
	if(psInternationalString != NULL)
	{
		if(psInternationalString->pcValue != NULL)
			DbvFree(psInternationalString->pcValue);
		DbvFree(psInternationalString);
	}
}

void DBVFreeDbvObjectIdentifier(DbvObjectIdentifier pcObjectIdentifier)
{
	if(pcObjectIdentifier != NULL)
		DbvFree(pcObjectIdentifier);
}

void DBVFreeDbvOtherInformation(DbvOtherInformation *psOtherInformation)
{
	if(psOtherInformation == NULL)
		return;

	switch(psOtherInformation->eInformationType)
	{
	case IT_CharacterInfo:
		DBVFreeDbvInternationalString(psOtherInformation->uInformation.psCharacterInfo);
		break;
	case IT_BinaryInfo:
		DBVFreeDbvOctetString(psOtherInformation->uInformation.psBinaryInfo);
		break;
	case IT_ExternalInfo:
		DBVFreeDbvASN1External(psOtherInformation->uInformation.psExternallyDefinedInfo);
		break;
	case IT_ObjectIdentifier:
		DBVFreeDbvObjectIdentifier(psOtherInformation->uInformation.pcObjectIdentifier);
		break;
	default:

		break;
	}
	if(psOtherInformation->psInfoCategory != NULL)
		DBVFreeDbvInfoCategory(psOtherInformation->psInfoCategory);
	DbvFree(psOtherInformation);
}

void DBVFreeDbvInfoCategory(DbvInfoCategory *psInfoCategory)
{
	DBVFreeDbvObjectIdentifier(psInfoCategory->pcCategoryTypeId);
	DbvFree(psInfoCategory);
}

void DBVFreeDbvIdPassword(DbvIdPassword *psIdPassword)
{
	if(psIdPassword != NULL)
	{
		DBVFreeDbvInternationalString(psIdPassword->psGroupId);
		DBVFreeDbvInternationalString(psIdPassword->psUserId);
		DBVFreeDbvInternationalString(psIdPassword->psPassword);
	}
}

void DBVFreeDbvAuthentication(DbvAuthentication *psAuthentication)
{
	if(psAuthentication == NULL)
		return;

	switch(psAuthentication->eAuthenticationType)
	{
	case AT_Open:
		DbvFree(psAuthentication->uAuthentication.psOpen);
		break;
	case AT_IdPassword:
		DBVFreeDbvIdPassword(psAuthentication->uAuthentication.psIdPassword);
		break;
	case AT_Anonymous:
		break;
	case  AT_Other:
		DBVFreeDbvASN1External(psAuthentication->uAuthentication.psOther);
		break;
	default:

		break;
	}

	DbvFree(psAuthentication);
}


void DBVFree_compulsory_DbvAdditionalInfo( DbvAdditionalInfo *psAdditionalInfo )
   {

   if ( psAdditionalInfo == NULL )
      return;

   switch(  psAdditionalInfo->eProtocolVersionInForce )
      {
      case PVIF_Version2:
         DBVFree_DbvVisibleString( psAdditionalInfo->uAdditionalInfo.pcV2 );
	 break;

      case PVIF_Version3:
         DBVFreeDbvInternationalString( psAdditionalInfo->uAdditionalInfo.psV3 );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End DBVFree_compulsory_DbvAdditionalInfo() */ 

void DBVFree_compulsory_DbvAttributeValue( DbvAttributeValue *psAttributeValue )
   {

   if ( psAttributeValue == NULL )
      return;

   switch(  psAttributeValue->eAttributeValueType )
      {
      case AVT_Numeric:
	 /* No space allocated */
	 break;

      case AVT_Complex:
         DBVFree_DbvComplexValue( psAttributeValue->
			       uAttributeValue.psComplexValue );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End DBVFree_compulsory_DbvAttributeValue() */ 

void DBVFree_compulsory_DbvHoldingsAndCircData( DbvHoldingsAndCircData 
					*psHoldingsAndCircData )
   {

   /* psTypeOfRecord */

   if ( psHoldingsAndCircData->psTypeOfRecord != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psTypeOfRecord );

   /* psEncodingLevel */

   if ( psHoldingsAndCircData->psEncodingLevel != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psEncodingLevel );

   /* psFormat */

   if ( psHoldingsAndCircData->psFormat != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psFormat );

   /* psReceiptAcqStatus */

   if ( psHoldingsAndCircData->psReceiptAcqStatus != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psReceiptAcqStatus );

   /* psGeneralRetention */

   if ( psHoldingsAndCircData->psGeneralRetention != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psGeneralRetention );

   /* psCompleteness */

   if ( psHoldingsAndCircData->psCompleteness != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psCompleteness );

   /* psDateOfReport */

   if ( psHoldingsAndCircData->psDateOfReport != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psDateOfReport );

   /* psNucCode */

   if ( psHoldingsAndCircData->psNucCode != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psNucCode );

   /* psLocalLocation */

   if ( psHoldingsAndCircData->psLocalLocation != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psLocalLocation );

   /* psShelvingLocation */

   if ( psHoldingsAndCircData->psShelvingLocation != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psShelvingLocation );

   /* psCallNumber */

   if ( psHoldingsAndCircData->psCallNumber != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psCallNumber );

   /* psShelvingData */

   if ( psHoldingsAndCircData->psShelvingData != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psShelvingData );

   /* psCopyNumber */

   if ( psHoldingsAndCircData->psCopyNumber != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psCopyNumber );

   /* psPublicNote */

   if ( psHoldingsAndCircData->psPublicNote != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psPublicNote );

   /* psReproductionNote */

   if ( psHoldingsAndCircData->psReproductionNote != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psReproductionNote );

   /* psTermsUseRepro */

   if ( psHoldingsAndCircData->psTermsUseRepro != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psTermsUseRepro );

   /* psEnumAndChron */

   if ( psHoldingsAndCircData->psEnumAndChron != NULL )
      DBVFreeDbvInternationalString( psHoldingsAndCircData->psEnumAndChron );

   /* ppsVolume */

   FreeN_DbvVolume( psHoldingsAndCircData->ppsVolume );

   /* ppsCircRecord*/

   FreeN_DbvCircRecord( psHoldingsAndCircData->ppsCircRecord );

   /* Note No final free as compulsory */

   } /* End DBVFree_DbvHoldingsAndCircData() */ 

void DBVFree_compulsory_DbvInternationalString( DbvInternationalString 
					*psInternationalString)
   {
   DbvFree(psInternationalString->pcValue);
   } /* DBVFree_compulsory_DbvInternationalString() */

void DBVFree_compulsory_DbvOriginPartToKeep( DbvOriginPartToKeep 
						*psOriginPartToKeep )
   {

   if ( psOriginPartToKeep == NULL )
      return;

   /* sDatabaseName */

   DBVFree_compulsory_DbvInternationalString( &psOriginPartToKeep->sDatabaseName ); 
   /* pcSchema */

   if ( psOriginPartToKeep->pcSchema != NULL )
      DBVFreeDbvObjectIdentifier( psOriginPartToKeep->pcSchema );

   /* psElementSetName */

   if ( psOriginPartToKeep->psElementSetName != NULL )
      DBVFreeDbvInternationalString( psOriginPartToKeep->psElementSetName ); 


   } /* End DBVFree_compulsory_DbvOriginPartToKeep() */ 

void DBVFree_compulsory_DbvTerm( DbvTerm *psTerm )
   {

   if ( psTerm == NULL )
      return;

   switch( psTerm->eTermType )
      {
      case TT_General:
         if ( psTerm->uTerm.psGeneral != NULL )
	    DBVFreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
         /* No Space allocated */
	 break;

      case TT_SimpleCharacter:
         if ( psTerm->uTerm.psSimpleCharacter != NULL )
            DBVFreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter ); 
	 break;

      case TT_ObjectIdentifier:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            DBVFreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
         if ( psTerm->uTerm.pcObjectIdentifier != NULL )
            DBVFree_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
         DBVFreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 DBVFree_DbvIntUnit(  psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVFree_compulsory_DbvTerm() */ 

void DBVFree_compulsory_DbvUnit( DbvUnit *psUnit )
   {

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */

   if ( psUnit->psUnitSystem != NULL )
      DBVFreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */

   if ( psUnit->psUnitType != NULL )
      DBVFree_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */

   if ( psUnit->psUnit != NULL )
      DBVFree_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */

   if ( psUnit->pulScaleFactor != NULL )
      DBVFree_Ulong( psUnit->pulScaleFactor );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End DBVFree_compulsory_DbvUnit() */

void DBVFree_compulsory_DbvUpdateTargetPart( DbvUpdateTargetPart 
						*psUpdateTargetPart )
   {

   if ( psUpdateTargetPart == NULL )
      return;

   /* ppsGlobalDiagnostics */

   FreeN_DbvDiagRec( psUpdateTargetPart->ppsGlobalDiagnostics );

   /* ppsTaskPackageRecord */

   FreeN_DbvTaskPackageRecord( psUpdateTargetPart->ppsTaskPackageRecord );

   /* Note : No final free as compulsory , therefore embedded structure */

   } /* End DBVFree_compulsory_DbvUpdateTargetPart() */



void DBVFree_compulsory_DbvSpecification(DbvSpecification *psSpecification)
{
	DBVFreeDbvObjectIdentifier(psSpecification->pcSchema);

	/* optional */
	DBVFree_DbvElementSpec(psSpecification->psElementSpec);
} /* End DBVFree_compulsory_DbvSpecification */


/* 12-2-96 The following function where added by Dan Maskell */

void DBVFree_compulsory_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      do_DBVFree_DbvEsIoTargetPart( psEsIoTargetPart );

   } /* End DBVFree_compulsory_DbvEsIoTargetPart() */ 

void DBVFree_compulsory_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      do_DBVFree_DbvIllString( psIllString );

   } /* End DBVFree_compulsory_DbvIllString() */ 

void DBVFree_compulsory_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      do_DBVFree_DbvDateTime( psDateTime );

   } /* End DBVFree_compulsory_DbvDateTime() */ 


void DBVFree_compulsory_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      do_DBVFree_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );

   } /* End DBVFree_compulsory_DbvUserOrProviderErrorReport() */ 

void DBVFree_compulsory_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      do_DBVFree_DbvHistoryReport( psHistoryReport );

   } /* End DBVFree_compulsory_DbvHistoryReport() */ 

void DBVFree_compulsory_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      do_DBVFree_DbvItemId( psItemId );

   } /* End DBVFree_compulsory_DbvItemId() */ 

void DBVFree_compulsory_DbvElDeliveryDetails( 
		DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      do_DBVFree_DbvElDeliveryDetails( psElDeliveryDetails );

   } /* End DBVFree_compulsory_DbvElDeliveryDetails() */ 

void DBVFree_compulsory_DbvSystemId( DbvSystemId *psSystemId )
   {

   if ( psSystemId != NULL )
      do_DBVFree_DbvSystemId( psSystemId );

   } /* End DBVFree_compulsory_DbvSystemId() */ 

void DBVFree_compulsory_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      do_DBVFree_DbvTransactionId( psTransactionId );

   } /* End DBVFree_compulsory_DbvTransactionId() */ 

void DBVFree_compulsory_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      do_DBVFree_DbvServiceDateTime( psServiceDateTime );

   } /* End DBVFree_compulsory_DbvServiceDateTime() */ 


void DBVFree_compulsory_DbvEsIoPaymentMethod( 
		DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod != NULL )
      do_DBVFree_DbvEsIoPaymentMethod( psEsIoPaymentMethod );

   } /* End DBVFree_compulsory_DbvEsIoPaymentMethod() */ 

void DBVFree_compulsory_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      do_DBVFree_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );

   } /* End DBVFree_compulsory_DbvEsIoOriginPartNotToKeep() */ 





/* Explain Record Support */


void DBVFree_compulsory_DbvCharge( DbvCharge *psCharge )
   {

   if ( psCharge != NULL )
      do_DBVFree_DbvCharge( psCharge );

   } /* End DBVFree_compulsory_DbvCharge() */ 

void DBVFree_compulsory_DbvBodyType( DbvBodyType *psBodyType )
   {

   if ( psBodyType != NULL )
      do_DBVFree_DbvBodyType( psBodyType );

   } /* End DBVFree_compulsory_DbvBodyType() */ 

void DBVFree_compulsory_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {

   if ( psAttributeOccurrenceChoice != NULL )
      do_DBVFree_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );

   } /* End DBVFree_compulsory_DbvAttributeOccurrenceChoice() */ 


#ifdef NOT_YET_IMPLEMENTED

/* Generic Record Support */

void DBVFree_compulsory_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

   if ( psElementDataG != NULL )
      do_DBVFree_DbvElementDataG( psElementDataG );

   } /* End DBVFree_compulsory_DbvElementDataG() */ 


#endif 



/*******************************************************/
/* Access, Resource and Trigger Control */


void DBVFree_compulsory_DbvSecurityChallengeReq( DbvSecurityChallengeReq *psSecurityChallengeReq )
   {
#ifdef DEBUGZ

#endif

   if ( psSecurityChallengeReq == NULL )
      return;

   /* uSecurityChallengeReq */

   switch( psSecurityChallengeReq->eSecurityChallengeReqType ) 
      {
      case SECT_SimpleForm:
   	 DBVFreeDbvOctetString( psSecurityChallengeReq->uSecurityChallengeReq.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 DBVFreeDbvASN1External( psSecurityChallengeReq->uSecurityChallengeReq.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End DBVFree_compulsory_DbvSecurityChallengeReq */



void DBVFree_compulsory_DbvPromptId( DbvPromptId *psPromptId )
   {
#ifdef DEBUGZ

#endif

   if ( psPromptId == NULL )
      return;


   /* uPromptId */

   switch( psPromptId->ePromptIdType ) 
      {
      case PIT_EnumeratedPrompt:
   	 DBVFree_DbvEnumeratedPrompt( psPromptId->uPromptId.psEnumeratedPrompt );
	 break;

      case PIT_NoneEnumeratedPrompt:
         DBVFreeDbvInternationalString( psPromptId->uPromptId.psNonEnumeratedPrompt );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End DBVFree_compulsory_DbvPromptId */



void DBVFree_compulsory_DbvPromptResponse( DbvPromptResponse *psPromptResponse )
   {
#ifdef DEBUGZ

#endif

   if ( psPromptResponse == NULL )
      return;


   /* uResponse */

   switch( psPromptResponse->eResponseType ) 
      {
      case RT_String:
   	 DBVFreeDbvInternationalString( psPromptResponse->uResponse.psString );
	 break;

      case RT_Accept:
	 break;

      case RT_Acknowledge:
	 break;

      case RT_Diagnostic:
   	 DBVFree_DbvDiagRec( psPromptResponse->uResponse.psDiagnostic );
	 break;

      case RT_Encrypted:
   	 DBVFree_DbvEncryption( psPromptResponse->uResponse.psEncryption );
	 break;

      default:
	 break;
      } /* End switch */

   /* Note: No final free */

   } /* End DBVFree_compulsory_DbvPromptResponse() */


void FreeN_DbvAttributeElement( DbvAttributeElement **ppsAttributeElement )
   {

   DbvAttributeElement **pps_base;

   if ( ppsAttributeElement == NULL )
      return;

   pps_base = ppsAttributeElement;

   while( *ppsAttributeElement )
      {
      DBVFree_DbvAttributeElement( *ppsAttributeElement );
      ppsAttributeElement++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeElement() */ 

void FreeN_DbvAttributesPlusTerm( DbvAttributesPlusTerm 
					**ppsAttributesPlusTerm )
   {

   DbvAttributesPlusTerm **pps_base;

   if ( ppsAttributesPlusTerm == NULL )
      return;

   pps_base = ppsAttributesPlusTerm;

   while( *ppsAttributesPlusTerm )
      {
      DBVFree_DbvAttributesPlusTerm( *ppsAttributesPlusTerm );
      ppsAttributesPlusTerm++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesPlusTerm() */ 

void FreeN_DbvCircRecord( DbvCircRecord **ppsCircRecord )
   {

   DbvCircRecord **pps_base;

   if ( ppsCircRecord == NULL )
      return;

   pps_base = ppsCircRecord;

   while( *ppsCircRecord )
      {
      DBVFree_DbvCircRecord( *ppsCircRecord );
      ppsCircRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvCircRecord() */ 

void FreeN_DbvDiagRec( DbvDiagRec **ppsDiagRec )
   {

   DbvDiagRec **pps_base;

   if ( ppsDiagRec == NULL )
      return;

   pps_base = ppsDiagRec;

   while( *ppsDiagRec )
      {
      DBVFree_DbvDiagRec( *ppsDiagRec );
      ppsDiagRec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDiagRec() */ 

void FreeN_DbvElementSetsDbSpecific( DbvElementSetsDbSpecific 
					**ppsElementSetsDbSpecific )
   {

   DbvElementSetsDbSpecific **pps_base;

   if ( ppsElementSetsDbSpecific == NULL )
      return;

   pps_base = ppsElementSetsDbSpecific;

   while( *ppsElementSetsDbSpecific )
      {
      DBVFree_DbvElementSetsDbSpecific( *ppsElementSetsDbSpecific );
      ppsElementSetsDbSpecific++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementSetsDbSpecific() */ 

void FreeN_DbvEntry( DbvEntry **ppsEntry )
   {

   DbvEntry **pps_base;

   if ( ppsEntry == NULL )
      return;

   pps_base = ppsEntry;

   while( *ppsEntry )
      {
      DBVFree_DbvEntry( *ppsEntry );
      ppsEntry++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEntry() */ 

void FreeN_DbvEstimate1( DbvEstimate1 **ppsEstimate1 )
   {

   DbvEstimate1 **pps_base;

   if ( ppsEstimate1 == NULL )
      return;

   pps_base = ppsEstimate1;

   while( *ppsEstimate1 )
      {
      DBVFree_DbvEstimate1( *ppsEstimate1 );
      ppsEstimate1++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEstimate1() */ 

void FreeN_DbvEstimate2( DbvEstimate2 **ppsEstimate2 )
   {

   DbvEstimate2 **pps_base;

   if ( ppsEstimate2 == NULL )
      return;

   pps_base = ppsEstimate2;

   while( *ppsEstimate2 )
      {
      DBVFree_DbvEstimate2( *ppsEstimate2 );
      ppsEstimate2++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvEstimate2() */ 

void FreeN_DbvFormatSpec( DbvFormatSpec **ppsFormatSpec )
   {

   DbvFormatSpec **pps_base;

   if ( ppsFormatSpec == NULL )
      return;

   pps_base = ppsFormatSpec;

   while( *ppsFormatSpec )
      {
      DBVFree_DbvFormatSpec( *ppsFormatSpec );
      ppsFormatSpec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvFormatSpec() */ 

void FreeN_DbvHoldingsRecord( DbvHoldingsRecord **ppsHoldingsRecord )
   {

   DbvHoldingsRecord **pps_base;

   if ( ppsHoldingsRecord == NULL )
      return;

   pps_base = ppsHoldingsRecord;

   while( *ppsHoldingsRecord )
      {
      DBVFree_DbvHoldingsRecord( *ppsHoldingsRecord );
      ppsHoldingsRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHoldingsRecord() */ 

void FreeN_DbvInternationalString( DbvInternationalString 
					**ppsInternationalString )
   {

   DbvInternationalString **pps_base;

   if ( ppsInternationalString == NULL )
      return;

   pps_base = ppsInternationalString;

   while( *ppsInternationalString )
      {
      DBVFreeDbvInternationalString( *ppsInternationalString );
      ppsInternationalString++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvInternationalString() */ 

void FreeN_DbvListStatus( DbvListStatus **ppsListStatus )
   {

   DbvListStatus **pps_base;

   if ( ppsListStatus == NULL )
      return;

   pps_base = ppsListStatus;

   while( *ppsListStatus )
      {
      DBVFree_DbvListStatus( *ppsListStatus );
      ppsListStatus++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvListStatus() */ 

void FreeN_DbvNamePlusRecord( DbvNamePlusRecord **ppsNamePlusRecord )
   {

   DbvNamePlusRecord **pps_base;

   if ( ppsNamePlusRecord == NULL )
      return;

   pps_base = ppsNamePlusRecord;

   while( *ppsNamePlusRecord )
      {
      DBVFree_DbvNamePlusRecord( *ppsNamePlusRecord );
      ppsNamePlusRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvNamePlusRecord() */ 

void FreeN_DbvOccurrenceByDb( DbvOccurrenceByDb **ppsOccurrenceByDb )
   {

   DbvOccurrenceByDb **pps_base;

   if ( ppsOccurrenceByDb == NULL )
      return;

   pps_base = ppsOccurrenceByDb;

   while( *ppsOccurrenceByDb )
      {
      DBVFree_DbvOccurrenceByDb( *ppsOccurrenceByDb );
      ppsOccurrenceByDb++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOccurrenceByDb() */ 

void FreeN_DbvPermAllowableFunctions( DbvPermAllowableFunctions 
					**ppePermAllowableFunctions )
   {

   DbvPermAllowableFunctions **pps_base;

   if ( ppePermAllowableFunctions == NULL )
      return;

   pps_base = ppePermAllowableFunctions;

   while( *ppePermAllowableFunctions )
      {
      DBVFree_DbvPermAllowableFunctions( *ppePermAllowableFunctions );
      ppePermAllowableFunctions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPermAllowableFunctions() */ 

void FreeN_DbvPermissions( DbvPermissions **ppsPermissions )
   {

   DbvPermissions **pps_base;

   if ( ppsPermissions == NULL )
      return;

   pps_base = ppsPermissions;

   while( *ppsPermissions )
      {
      DBVFree_DbvPermissions( *ppsPermissions );
      ppsPermissions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPermissions() */ 

void FreeN_DbvRange( DbvRange **ppsRange )
   {

   DbvRange **pps_base;

   if ( ppsRange == NULL )
      return;

   pps_base = ppsRange;

   while( *ppsRange )
      {
      DBVFree_DbvRange( *ppsRange );
      ppsRange++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvRange() */ 

void FreeN_DbvSortKeySpec( DbvSortKeySpec **ppsSortKeySpec )
   {

   DbvSortKeySpec **pps_base;

   if ( ppsSortKeySpec == NULL )
      return;

   pps_base = ppsSortKeySpec;

   while( *ppsSortKeySpec )
      {
      DBVFree_DbvSortKeySpec( *ppsSortKeySpec );
      ppsSortKeySpec++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSortKeySpec() */ 

void FreeN_DbvStringOrNumeric( DbvStringOrNumeric **ppsStringOrNumeric )
   {

   DbvStringOrNumeric **pps_base;

   if ( ppsStringOrNumeric == NULL )
      return;

   pps_base = ppsStringOrNumeric;

   while( *ppsStringOrNumeric )
      {
      DBVFree_DbvStringOrNumeric( *ppsStringOrNumeric );
      ppsStringOrNumeric++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvStringOrNumeric() */ 

void FreeN_DbvSuppliedRecord( DbvSuppliedRecord **ppsSuppliedRecord )
   {

   DbvSuppliedRecord **pps_base;

   if ( ppsSuppliedRecord == NULL )
      return;

   pps_base = ppsSuppliedRecord;

   while( *ppsSuppliedRecord )
      {
      DBVFree_DbvSuppliedRecord( *ppsSuppliedRecord );
      ppsSuppliedRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSuppliedRecord() */ 

void FreeN_DbvTaskPackageRecord( DbvTaskPackageRecord **ppsTaskPackageRecord )
   {

   DbvTaskPackageRecord **pps_base;

   if ( ppsTaskPackageRecord == NULL )
      return;

   pps_base = ppsTaskPackageRecord;

   while( *ppsTaskPackageRecord )
      {
      DBVFree_DbvTaskPackageRecord( *ppsTaskPackageRecord );
      ppsTaskPackageRecord++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTaskPackageRecord() */ 

void FreeN_DbvVolume( DbvVolume **ppsVolume )
   {

   DbvVolume **pps_base;

   if ( ppsVolume == NULL )
      return;

   pps_base = ppsVolume;

   while( *ppsVolume )
      {
      DBVFree_DbvVolume( *ppsVolume );
      ppsVolume++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVolume() */ 

void FreeN_Ulong( Ulong **ppUlong )
   {

   Ulong **pp_base;

   if ( ppUlong == NULL )
      return;

   pp_base = ppUlong;

   while( *ppUlong )
      {
      DBVFree_Ulong( *ppUlong );
      ppUlong++;
      } /* End while */

   DbvFree( pp_base );
   } /* End FreeN_Ulong() */ 

void FreeN_DbvOtherInformation( DbvOtherInformation **ppsOtherInformation )
   {
 
   DbvOtherInformation **pps_base;
 
   if ( ppsOtherInformation == NULL )
      return;
 
   pps_base = ppsOtherInformation;
 
   while( *ppsOtherInformation )
      {
      DBVFreeDbvOtherInformation( *ppsOtherInformation );
      ppsOtherInformation++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvOtherInformation() */
 
void FreeN_DbvOccurrenceByAttributes(DbvOccurrenceByAttributes 
	**ppsByAttributes )
   {
 
   DbvOccurrenceByAttributes **pps_base;
 
   if ( ppsByAttributes == NULL )
      return;
 
   pps_base = ppsByAttributes;
 
   while( *ppsByAttributes )
      {
      DBVFree_DbvOccurrenceByAttributes( *ppsByAttributes );
      ppsByAttributes++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvOccurrenceByAttributes() */
 
void FreeN_DbvDbSpecificComp(DbvDbSpecificComp **ppsDbSpecific )
   {
 
   DbvDbSpecificComp **pps_base;
 
   if ( ppsDbSpecific == NULL )
      return;
 
   pps_base = ppsDbSpecific;
 
   while( *ppsDbSpecific )
      {
      DBVFree_DbvDbSpecificComp( *ppsDbSpecific );
      ppsDbSpecific++;
      } /* End while */
 
   DbvFree( pps_base );
   } /* End FreeN_DbvDbSpecificComp() */

void FreeN_DbvObjectIdentifier(DbvObjectIdentifier *ppcObjectIdentifier)
   {
 
   DbvObjectIdentifier *ppc_base;
 
   if ( ppcObjectIdentifier == NULL )
      return;
 
   ppc_base = ppcObjectIdentifier;
 
   while( *ppcObjectIdentifier )
      {
      DBVFreeDbvObjectIdentifier( *ppcObjectIdentifier );
      ppcObjectIdentifier++;
      } /* End while */
 
   DbvFree( ppc_base );
   } /* End FreeN_DbvObjectIdentifier() */

/* 5-2-96 The following function where added by Dan Maskell  */

void FreeN_DbvSystemId( DbvSystemId **ppsSystemId )
   {

   DbvSystemId **pps_base;

   if ( ppsSystemId == NULL )
      return;

   pps_base = ppsSystemId;

   while( *ppsSystemId )
      {
      DBVFree_DbvSystemId( *ppsSystemId );
      ppsSystemId++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSystemId() */ 

void FreeN_DbvElectronicDeliveryService( 
	DbvElectronicDeliveryService **ppsElectronicDeliveryService )
   {

   DbvElectronicDeliveryService **pps_base;

   if ( ppsElectronicDeliveryService == NULL )
      return;

   pps_base = ppsElectronicDeliveryService;

   while( *ppsElectronicDeliveryService )
      {
      DBVFree_DbvElectronicDeliveryService( *ppsElectronicDeliveryService );
      ppsElectronicDeliveryService++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElectronicDeliveryService() */ 

void FreeN_DbvSupplyMediumInfo( DbvSupplyMediumInfo **ppsSupplyMediumInfo )
   {

   DbvSupplyMediumInfo **pps_base;

   if ( ppsSupplyMediumInfo == NULL )
      return;

   pps_base = ppsSupplyMediumInfo;

   while( *ppsSupplyMediumInfo )
      {
      DBVFree_DbvSupplyMediumInfo( *ppsSupplyMediumInfo );
      ppsSupplyMediumInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSupplyMediumInfo() */ 

void FreeN_DbvSendToList( DbvSendToList **ppsSendToList )
   {

   DbvSendToList **pps_base;

   if ( ppsSendToList == NULL )
      return;

   pps_base = ppsSendToList;

   while( *ppsSendToList )
      {
      DBVFree_DbvSendToList( *ppsSendToList );
      ppsSendToList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSendToList() */ 

void FreeN_DbvExtension( DbvExtension **ppsExtension )
   {

   DbvExtension **pps_base;

   if ( ppsExtension == NULL )
      return;

   pps_base = ppsExtension;

   while( *ppsExtension )
      {
      DBVFree_DbvExtension( *ppsExtension );
      ppsExtension++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvExtension() */ 

void FreeN_DbvServiceType( DbvServiceType **ppsServiceType )
   {

   DbvServiceType **pps_base;

   if ( ppsServiceType == NULL )
      return;

   pps_base = ppsServiceType;

   while( *ppsServiceType )
      {
      DBVFree_DbvServiceType( *ppsServiceType );
      ppsServiceType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvServiceType() */ 

void FreeN_DbvASN1External( DbvASN1External **ppsASN1External )
   {

   DbvASN1External **pps_base;

   if ( ppsASN1External == NULL )
      return;

   pps_base = ppsASN1External;

   while( *ppsASN1External )
      {
      DBVFreeDbvASN1External( *ppsASN1External );
      ppsASN1External++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvASN1External() */ 


/* Explain Record Support */


void FreeN_DbvHumanString( DbvHumanString **ppsHumanString )
   {
   DbvHumanString **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsHumanString == NULL )
      return;

   pps_base = ppsHumanString;

   while( *ppsHumanString )
      {
      DBVFree_DbvHumanString( *ppsHumanString );
      ppsHumanString++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHumanString() */ 

void FreeN_DbvIconObject( DbvIconObject **ppsIconObject )
   {
   DbvIconObject **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsIconObject == NULL )
      return;

   pps_base = ppsIconObject;

   while( *ppsIconObject )
      {
      DBVFree_DbvIconObject( *ppsIconObject );
      ppsIconObject++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvIconObject() */ 

void FreeN_DbvDatabaseList( DbvDatabaseList **ppsDatabaseList )
   {
   DbvDatabaseList **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsDatabaseList == NULL )
      return;

   pps_base = ppsDatabaseList;

   while( *ppsDatabaseList )
      {
      DBVFree_DbvDatabaseList( *ppsDatabaseList );
      ppsDatabaseList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDatabaseList() */ 

void FreeN_DbvNetworkAddress( DbvNetworkAddress **ppsNetworkAddress )
   {
   DbvNetworkAddress **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsNetworkAddress == NULL )
      return;

   pps_base = ppsNetworkAddress;

   while( *ppsNetworkAddress )
      {
      DBVFree_DbvNetworkAddress( *ppsNetworkAddress );
      ppsNetworkAddress++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvNetworkAddress() */ 

void FreeN_DbvOperators( DbvOperators **ppsOperators )
   {
   DbvOperators **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsOperators == NULL )
      return;

   pps_base = ppsOperators;

   while( *ppsOperators )
      {
      DBVFree_DbvOperators( *ppsOperators );
      ppsOperators++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOperators() */ 

void FreeN_DbvSearchKey( DbvSearchKey **ppsSearchKey )
   {
   DbvSearchKey **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsSearchKey == NULL )
      return;

   pps_base = ppsSearchKey;

   while( *ppsSearchKey )
      {
      DBVFree_DbvSearchKey( *ppsSearchKey );
      ppsSearchKey++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSearchKey() */ 


void FreeN_DbvHumanStringSeq( DbvHumanStringSeq **ppsDescription )
   {
   DbvHumanStringSeq **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsDescription == NULL )
      return;

   pps_base = ppsDescription;

   while( *ppsDescription )
      {
      DBVFree_DbvHumanStringSeq( *ppsDescription );
      ppsDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHumanStringSeq() */ 



void FreeN_DbvUnitType( DbvUnitType **ppsUnitType )
   {
   DbvUnitType **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsUnitType == NULL )
      return;

   pps_base = ppsUnitType;

   while( *ppsUnitType )
      {
      DBVFree_DbvUnitType( *ppsUnitType );
      ppsUnitType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnitType() */ 

void FreeN_DbvUnitsSupported( DbvUnitsSupported **ppsUnitsSupported )
   {
   DbvUnitsSupported **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsUnitsSupported == NULL )
      return;

   pps_base = ppsUnitsSupported;

   while( *ppsUnitsSupported )
      {
      DBVFree_DbvUnitsSupported( *ppsUnitsSupported );
      ppsUnitsSupported++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnitsSupported() */ 

void FreeN_DbvQueryTypeDetails( DbvQueryTypeDetails **ppsQueryTypeDetails )
   {
   DbvQueryTypeDetails **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsQueryTypeDetails == NULL )
      return;

   pps_base = ppsQueryTypeDetails;

   while( *ppsQueryTypeDetails )
      {
      DBVFree_DbvQueryTypeDetails( *ppsQueryTypeDetails );
      ppsQueryTypeDetails++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvQueryTypeDetails() */ 

void FreeN_DbvOtherCharge( DbvOtherCharge **ppsOtherCharge )
   {
   DbvOtherCharge **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsOtherCharge == NULL )
      return;

   pps_base = ppsOtherCharge;

   while( *ppsOtherCharge )
      {
      DBVFree_DbvOtherCharge( *ppsOtherCharge );
      ppsOtherCharge++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvOtherCharge() */ 

void FreeN_DbvElementSetNames( DbvElementSetNames **ppsElementSetNames )
   {
   DbvElementSetNames **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsElementSetNames == NULL )
      return;

   pps_base = ppsElementSetNames;

   while( *ppsElementSetNames )
      {
      DBVFree_DbvElementSetNames( *ppsElementSetNames );
      ppsElementSetNames++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementSetNames() */ 

void FreeN_DbvAccessRestrictions( 
			DbvAccessRestrictions **ppsAccessRestrictions )
   {
   DbvAccessRestrictions **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAccessRestrictions == NULL )
      return;

   pps_base = ppsAccessRestrictions;

   while( *ppsAccessRestrictions )
      {
      DBVFree_DbvAccessRestrictions( *ppsAccessRestrictions );
      ppsAccessRestrictions++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAccessRestrictions() */ 


void FreeN_DbvTagTypeMapping( DbvTagTypeMapping **ppsTagTypeMapping )
   {
   DbvTagTypeMapping **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTagTypeMapping == NULL )
      return;

   pps_base = ppsTagTypeMapping;

   while( *ppsTagTypeMapping )
      {
      DBVFree_DbvTagTypeMapping( *ppsTagTypeMapping );
      ppsTagTypeMapping++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTagTypeMapping() */ 

void FreeN_DbvPath( DbvPath **ppsPath )
   {
   DbvPath **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsPath == NULL )
      return;

   pps_base = ppsPath;

   while( *ppsPath )
      {
      DBVFree_DbvPath( *ppsPath );
      ppsPath++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPath() */ 

void FreeN_DbvPathA( DbvPathA **ppsPathA )
   {
   DbvPathA **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsPathA == NULL )
      return;

   pps_base = ppsPathA;

   while( *ppsPathA )
      {
      DBVFree_DbvPathA( *ppsPathA );
      ppsPathA++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvPathA() */ 

void FreeN_DbvElementInfo( DbvElementInfo **ppsElementInfo )
   {
   DbvElementInfo **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsElementInfo == NULL )
      return;

   pps_base = ppsElementInfo;

   while( *ppsElementInfo )
      {
      DBVFree_DbvElementInfo( *ppsElementInfo );
      ppsElementInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElementInfo() */ 

void FreeN_DbvElements( DbvElements **ppsElements )
   {
   DbvElements **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsElements == NULL )
      return;

   pps_base = ppsElements;

   while( *ppsElements )
      {
      DBVFree_DbvElements( *ppsElements );
      ppsElements++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvElements() */ 

void FreeN_DbvAttributeDescription( 
		DbvAttributeDescription **ppsAttributeDescription )
   {
   DbvAttributeDescription **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeDescription == NULL )
      return;

   pps_base = ppsAttributeDescription;

   while( *ppsAttributeDescription )
      {
      DBVFree_DbvAttributeDescription( *ppsAttributeDescription );
      ppsAttributeDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeDescription() */ 

void FreeN_DbvAttributeType( DbvAttributeType **ppsAttributeType )
   {
   DbvAttributeType **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeType == NULL )
      return;

   pps_base = ppsAttributeType;

   while( *ppsAttributeType )
      {
      DBVFree_DbvAttributeType( *ppsAttributeType );
      ppsAttributeType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeType() */ 

void FreeN_DbvTermList( DbvTermList **ppsTermList )
   {
   DbvTermList **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTermList == NULL )
      return;

   pps_base = ppsTermList;

   while( *ppsTermList )
      {
      DBVFree_DbvTermList( *ppsTermList );
      ppsTermList++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTermList() */ 

void FreeN_DbvValueDescription( DbvValueDescription **ppsValueDescription )
   {
   DbvValueDescription **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsValueDescription == NULL )
      return;

   pps_base = ppsValueDescription;

   while( *ppsValueDescription )
      {
      DBVFree_DbvValueDescription( *ppsValueDescription );
      ppsValueDescription++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvValueDescription() */ 

void FreeN_DbvAttributeValues( DbvAttributeValues **ppsAttributeValues )
   {
   DbvAttributeValues **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeValues == NULL )
      return;

   pps_base = ppsAttributeValues;

   while( *ppsAttributeValues )
      {
      DBVFree_DbvAttributeValues( *ppsAttributeValues );
      ppsAttributeValues++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeValues() */ 

void FreeN_DbvAttributesByType( DbvAttributesByType **ppsAttributesByType )
   {
   DbvAttributesByType **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributesByType == NULL )
      return;

   pps_base = ppsAttributesByType;

   while( *ppsAttributesByType )
      {
      DBVFree_DbvAttributesByType( *ppsAttributesByType );
      ppsAttributesByType++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesByType() */ 

void FreeN_DbvAttributesBySet( DbvAttributesBySet **ppsAttributesBySet )
   {
   DbvAttributesBySet **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributesBySet == NULL )
      return;

   pps_base = ppsAttributesBySet;

   while( *ppsAttributesBySet )
      {
      DBVFree_DbvAttributesBySet( *ppsAttributesBySet );
      ppsAttributesBySet++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributesBySet() */ 

void FreeN_DbvAttributeDetails( DbvAttributeDetails **ppsAttributeDetails )
   {
   DbvAttributeDetails **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeDetails == NULL )
      return;

   pps_base = ppsAttributeDetails;

   while( *ppsAttributeDetails )
      {
      DBVFree_DbvAttributeDetails( *ppsAttributeDetails );
      ppsAttributeDetails++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeDetails() */ 

void FreeN_DbvTerm( DbvTerm **ppsTerm )
   {
   DbvTerm **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTerm == NULL )
      return;

   pps_base = ppsTerm;

   while( *ppsTerm )
      {
      DBVFree_DbvTerm( *ppsTerm );
      ppsTerm++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTerm() */ 

void FreeN_DbvDetailsPerElement( DbvDetailsPerElement **ppsDetailsPerElement )
   {
   DbvDetailsPerElement **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsDetailsPerElement == NULL )
      return;

   pps_base = ppsDetailsPerElement;

   while( *ppsDetailsPerElement )
      {
      DBVFree_DbvDetailsPerElement( *ppsDetailsPerElement );
      ppsDetailsPerElement++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDetailsPerElement() */ 

void FreeN_DbvSortKeys( DbvSortKeys **ppsSortKeys )
   {
   DbvSortKeys **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsSortKeys == NULL )
      return;

   pps_base = ppsSortKeys;

   while( *ppsSortKeys )
      {
      DBVFree_DbvSortKeys( *ppsSortKeys );
      ppsSortKeys++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSortKeys() */ 

void FreeN_DbvVariantTypes( DbvVariantTypes **ppsVariantTypes )
   {
   DbvVariantTypes **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsVariantTypes == NULL )
      return;

   pps_base = ppsVariantTypes;

   while( *ppsVariantTypes )
      {
      DBVFree_DbvVariantTypes( *ppsVariantTypes );
      ppsVariantTypes++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVariantTypes() */ 

void FreeN_DbvVariantClass( DbvVariantClass **ppsVariantClass )
   {
   DbvVariantClass **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsVariantClass == NULL )
      return;

   pps_base = ppsVariantClass;

   while( *ppsVariantClass )
      {
      DBVFree_DbvVariantClass( *ppsVariantClass );
      ppsVariantClass++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVariantClass() */ 

void FreeN_DbvUnits( DbvUnits **ppsUnits )
   {
   DbvUnits **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsUnits == NULL )
      return;

   pps_base = ppsUnits;

   while( *ppsUnits )
      {
      DBVFree_DbvUnits( *ppsUnits );
      ppsUnits++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnits() */ 

void FreeN_DbvUnit( DbvUnit **ppsUnit )
   {
   DbvUnit **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsUnit == NULL )
      return;

   pps_base = ppsUnit;

   while( *ppsUnit )
      {
      DBVFree_DbvUnit( *ppsUnit );
      ppsUnit++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvUnit() */ 

void FreeN_DbvCategoryInfo( DbvCategoryInfo **ppsCategoryInfo )
   {
   DbvCategoryInfo **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsCategoryInfo == NULL )
      return;

   pps_base = ppsCategoryInfo;

   while( *ppsCategoryInfo )
      {
      DBVFree_DbvCategoryInfo( *ppsCategoryInfo );
      ppsCategoryInfo++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvCategoryInfo() */ 

void FreeN_DbvAttributeOccurrence( 
			DbvAttributeOccurrence **ppsAttributeOccurrence )
   {
   DbvAttributeOccurrence **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeOccurrence == NULL )
      return;

   pps_base = ppsAttributeOccurrence;

   while( *ppsAttributeOccurrence )
      {
      DBVFree_DbvAttributeOccurrence( *ppsAttributeOccurrence );
      ppsAttributeOccurrence++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeOccurrence() */ 

void FreeN_DbvAttributeCombination( 
		DbvAttributeCombination **ppsAttributeCombination )
   {
   DbvAttributeCombination **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeCombination == NULL )
      return;

   pps_base = ppsAttributeCombination;

   while( *ppsAttributeCombination )
      {
      DBVFree_DbvAttributeCombination( *ppsAttributeCombination );
      ppsAttributeCombination++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeCombination() */ 

void FreeN_DbvAttributeCombinations( 
			DbvAttributeCombinations **ppsAttributeCombinations )
   {
   DbvAttributeCombinations **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsAttributeCombinations == NULL )
      return;

   pps_base = ppsAttributeCombinations;

   while( *ppsAttributeCombinations )
      {
      DBVFree_DbvAttributeCombinations( *ppsAttributeCombinations );
      ppsAttributeCombinations++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvAttributeCombinations() */ 

void FreeN_DbvSpecification( DbvSpecification **ppsSpecification )
   {
   DbvSpecification **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsSpecification == NULL )
      return;

   pps_base = ppsSpecification;

   while( *ppsSpecification )
      {
      DBVFree_DbvSpecification( *ppsSpecification );
      ppsSpecification++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvSpecification() */ 


#ifdef NOT_YET_IMPLEMENTED

/* Generic Record Support */

void FreeN_DbvValue( DbvValue **ppsValue )
   {
   DbvValue **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsValue == NULL )
      return;

   pps_base = ppsValue;

   while( *ppsValue )
      {
      DBVFree_DbvValue( *ppsValue );
      ppsValue++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvValue() */ 



void FreeN_DbvTriples( DbvTriples **ppsTriples )
   {
   DbvTriples **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTriples == NULL )
      return;

   pps_base = ppsTriples;

   while( *ppsTriples )
      {
      DBVFree_DbvTriples( *ppsTriples );
      ppsTriples++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTriples() */ 



void FreeN_DbvTagPath( DbvTagPath **ppsTagPath )
   {
   DbvTagPath **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTagPath == NULL )
      return;

   pps_base = ppsTagPath;

   while( *ppsTagPath )
      {
      DBVFree_DbvTagPath( *ppsTagPath );
      ppsTagPath++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTagPath() */ 

void FreeN_DbvTaggedElement( DbvTaggedElement **ppsTaggedElement )
   {
   DbvTaggedElement **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsTaggedElement == NULL )
      return;

   pps_base = ppsTaggedElement;

   while( *ppsTaggedElement )
      {
      DBVFree_DbvTaggedElement( *ppsTaggedElement );
      ppsTaggedElement++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvTaggedElement() */ 



void FreeN_DbvVariant( DbvVariant **ppsVariant )
   {
   DbvVariant **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsVariant == NULL )
      return;

   pps_base = ppsVariant;

   while( *ppsVariant )
      {
      DBVFree_DbvVariant( *ppsVariant );
      ppsVariant++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvVariant() */ 

void FreeN_DbvHits( DbvHits **ppsHits )
   {
   DbvHits **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsHits == NULL )
      return;

   pps_base = ppsHits;

   while( *ppsHits )
      {
      DBVFree_DbvHits( *ppsHits );
      ppsHits++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvHits() */ 


#endif 



void FreeN_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq **ppsDbvDataBaseInfoSeq )
   {
   DbvDataBaseInfoSeq **pps_base;

#ifdef DEBUGZ

#endif

   if ( ppsDbvDataBaseInfoSeq == NULL )
      return;

   pps_base = ppsDbvDataBaseInfoSeq;

   while( *ppsDbvDataBaseInfoSeq )
      {
      DBVFree_DbvDataBaseInfoSeq( *ppsDbvDataBaseInfoSeq );
      ppsDbvDataBaseInfoSeq++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvDataBaseInfoSeq() */ 




/*******************************************************/
/* Access, Resource and Trigger Control */


void FreeN_DbvChallenge( DbvChallenge **ppsChallenge )
   {
   DbvChallenge **pps_base;

   if ( ppsChallenge == NULL )
      return;

   pps_base = ppsChallenge;

   while( *ppsChallenge )
      {
      DBVFree_DbvChallenge( *ppsChallenge );
      ppsChallenge++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvChallenge() */



void FreeN_DbvResponse( DbvResponse **ppsResponse )
   {
   DbvResponse **pps_base;

   if ( ppsResponse == NULL )
      return;

   pps_base = ppsResponse;

   while( *ppsResponse )
      {
      DBVFree_DbvResponse( *ppsResponse );
      ppsResponse++;
      } /* End while */

   DbvFree( pps_base );
   } /* End FreeN_DbvResponse() */


void DBVFree_DbvAttributeElement( DbvAttributeElement *psAttributeElement )
   {

   if ( psAttributeElement == NULL )
      return;

   /* pcAttributeSetId */

   if ( psAttributeElement->pcAttributeSetId != NULL )
      DBVFreeDbvObjectIdentifier( psAttributeElement->pcAttributeSetId );

   /* sAttributeValue */

   DBVFree_compulsory_DbvAttributeValue( &psAttributeElement->sAttributeValue );

   DbvFree( psAttributeElement );
   } /* End DBVFree_DbvAttributeElement() */ 



void DBVFree_DbvAttributesPlusTerm( DbvAttributesPlusTerm *psAttributesPlusTerm )
   {

   if ( psAttributesPlusTerm == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psAttributesPlusTerm->ppsAttributeElement );

   /* sTerm */

   DBVFree_compulsory_DbvTerm( &psAttributesPlusTerm->sTerm );

   DbvFree( psAttributesPlusTerm );
   } /* End DBVFree_DbvAttributesPlusTerm() */ 



void DBVFree_DbvCircRecord( DbvCircRecord *psCircRecord )
   {

   if ( psCircRecord == NULL )
      return;

   /* psAvailabilityDate */

   if ( psCircRecord->psAvailabilityDate != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psAvailabilityDate );

   /* psAvailableThru */

   if ( psCircRecord->psAvailableThru != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psAvailableThru );

   /* psRestrictions */

   if ( psCircRecord->psRestrictions != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psRestrictions );

   /* psItemId */

   if ( psCircRecord->psItemId != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psItemId );

   /* psEnumAndChron */

   if ( psCircRecord->psEnumAndChron != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psEnumAndChron );

   /* psMidspine */

   if ( psCircRecord->psMidspine != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psMidspine );

   /* psTemporaryLocation */

   if ( psCircRecord->psTemporaryLocation != NULL )
      DBVFreeDbvInternationalString( psCircRecord->psTemporaryLocation );

   DbvFree( psCircRecord );
   } /* End DBVFree_DbvCircRecord() */ 


void DBVFree_DbvCloseReq( DbvCloseReq *psCloseReq )
   {

   /* psReferenceId */

   if( psCloseReq->psReferenceId != NULL)
      DBVFreeDbvOctetString( psCloseReq->psReferenceId );

   /* psDiagnosticInformation */

   if( psCloseReq->psDiagnosticInformation != NULL)
      DBVFreeDbvInternationalString( psCloseReq->psDiagnosticInformation );

   /* pcResourceReportFormat */

   if( psCloseReq->pcResourceReportFormat != NULL)
      DBVFreeDbvObjectIdentifier( psCloseReq->pcResourceReportFormat );

   /* psResourceReport */

   DBVFreeDbvASN1External( psCloseReq->psResourceReport );

   /* ppsOtherInformation */

   if( psCloseReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psCloseReq->ppsOtherInformation );

   DbvFree( psCloseReq );
   } /* End DBVFree_DbvCloseReq() */ 


void DBVFree_DbvComplexValue( DbvComplexValue *psComplexValue )
   {

   if ( psComplexValue == NULL )
      return;

   /* ppsList */

   FreeN_DbvStringOrNumeric( psComplexValue->ppsList );

   /* ppulSemanticAction */

   FreeN_Ulong( psComplexValue->ppulSemanticAction );

   DbvFree( psComplexValue );
   } /* End DBVFree_DbvComplexValue() */ 

void DBVFree_DbvCorrelationInfo( DbvCorrelationInfo *psCorrelationInfo )
   {

   if ( psCorrelationInfo == NULL )
      return;

   /* psNote */

   if ( psCorrelationInfo->psNote != NULL )
      DBVFreeDbvInternationalString( psCorrelationInfo->psNote );

   /* pulId */

   if ( psCorrelationInfo->pulId != NULL )
      DBVFree_Ulong( psCorrelationInfo->pulId );

   DbvFree( psCorrelationInfo );
   } /* End DBVFree_DbvCorrelationInfo() */ 

void DBVFree_DbvDefaultDiagFormat( DbvDefaultDiagFormat *psDefaultDiagFormat )
   {

   if ( psDefaultDiagFormat == NULL )
      return;

   if ( psDefaultDiagFormat->pcDiagnosticSetId != NULL )
      DBVFreeDbvObjectIdentifier( psDefaultDiagFormat->pcDiagnosticSetId );

   DBVFree_compulsory_DbvAdditionalInfo( &psDefaultDiagFormat->sAdditionalInfo );

   DbvFree( psDefaultDiagFormat );
   } /* End DBVFree_DbvDefaultDiagFormat() */ 

void DBVFree_DbvDeleteResultSetReq( DbvDeleteResultSetReq *psDeleteResultSetReq )
   {

   /* psReferenceId */
   if( psDeleteResultSetReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psDeleteResultSetReq->psReferenceId);
			 
   /* ppsResultSetId */

   FreeN_DbvInternationalString( psDeleteResultSetReq->ppsResultSetId );

   /* ppsOtherInformation */

   if( psDeleteResultSetReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetReq->ppsOtherInformation );

   DbvFree( psDeleteResultSetReq );
   } /* End DBVFree_DbvDeleteResultSetReq() */ 

void DBVFree_DbvDeleteResultSetRsp( DbvDeleteResultSetRsp *psDeleteResultSetRsp )
   {

   /* psReferenceId */

   if( psDeleteResultSetRsp->psReferenceId != NULL)
      DBVFreeDbvOctetString(psDeleteResultSetRsp->psReferenceId);
			 
   /* ppsDeleteListStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsDeleteListStatuses );

   /* pulNumberNotDeleted */

   if ( psDeleteResultSetRsp->pulNumberNotDeleted != NULL )
      DBVFree_Ulong( psDeleteResultSetRsp->pulNumberNotDeleted );

   /* ppsBulkStatuses */

   FreeN_DbvListStatus( psDeleteResultSetRsp->ppsBulkStatuses );

   /* psDeleteMessage */

   if ( psDeleteResultSetRsp->psDeleteMessage != NULL )
      DBVFreeDbvInternationalString( psDeleteResultSetRsp->psDeleteMessage );

   /* ppsOtherInformation */
   if( psDeleteResultSetRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psDeleteResultSetRsp->ppsOtherInformation );

   DbvFree( psDeleteResultSetRsp );
   } /* End DBVFree_DbvDeleteResultSetRsp() */ 

void DBVFree_DbvDiagRec( DbvDiagRec *psDiagRec )
   {

   if ( psDiagRec == NULL )
      return;

   switch( psDiagRec->eDiagnosticsType )
      {
      case DT_DefaultFormat:
	 DBVFree_DbvDefaultDiagFormat( psDiagRec->uDiagnostics.psDefault );
	 break;

      case DT_ExternallyDefined:
	 DBVFreeDbvASN1External( psDiagRec->uDiagnostics.psExternallyDefined );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDiagRec );
   } /* End DBVFree_DbvDiagRec() */ 

void DBVFree_DbvElementSetNames( DbvElementSetNames *psElementSetNames )
   {

   if ( psElementSetNames == NULL )
      return;

   switch( psElementSetNames->eElementSetType )
      {
      case EST_Generic:
	 DBVFreeDbvInternationalString( psElementSetNames->uElementSet.psGeneric );
	 break;

      case EST_DatabaseSpecific:
	 FreeN_DbvElementSetsDbSpecific( psElementSetNames->
					uElementSet.ppsElementSetsDbSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementSetNames );
   } /* End DBVFree_DbvElementSetNames() */ 

void DBVFree_DbvElementSetsDbSpecific( DbvElementSetsDbSpecific 
						*psElementSetsDbSpecific )
   {

   /* psDatabaseName */

   if ( psElementSetsDbSpecific->psDatabaseName != NULL )
      DBVFreeDbvInternationalString( psElementSetsDbSpecific->psDatabaseName );

   /* psElementSetName */

   if ( psElementSetsDbSpecific->psElementSetName != NULL )
      DBVFreeDbvInternationalString( psElementSetsDbSpecific->psElementSetName );

   DbvFree( psElementSetsDbSpecific );
   } /* End DBVFree_DbvElementSetsDbSpecific() */ 

void DBVFree_DbvEntry( DbvEntry *psEntry )
   {

   if ( psEntry == NULL )
      return;

   switch( psEntry->eEntryType )
      {
      case ET_TermInfo:
	 DBVFree_DbvTermInfo( psEntry->uEntry.psTermInfo );
	 break;

      case ET_SurrogateDiagnostic:
	 DBVFree_DbvDiagRec( psEntry->uEntry.psSurrogateDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psEntry );
   } /* End DBVFree_DbvEntry() */ 

void DBVFree_DbvEsTaskPackage( DbvEsTaskPackage *psEsTaskPackage )
   {

   if ( psEsTaskPackage == NULL )
      return;

   /* pcPackageType */

   DBVFreeDbvObjectIdentifier( psEsTaskPackage->pcPackageType );

   /* psPackageName */

   if ( psEsTaskPackage->psPackageName != NULL )
      DBVFreeDbvInternationalString( psEsTaskPackage->psPackageName );

   /* psUserId */

   if ( psEsTaskPackage->psUserId != NULL )
      DBVFreeDbvInternationalString( psEsTaskPackage->psUserId );

   /* psRetentionTime */

   DBVFree_DbvIntUnit( psEsTaskPackage->psRetentionTime );

   /* ppsPermissions */

   FreeN_DbvPermissions( psEsTaskPackage->ppsPermissions );

   /* psDescription */

   if ( psEsTaskPackage->psDescription != NULL )
      DBVFreeDbvInternationalString( psEsTaskPackage->psDescription );

   /* psTargetReference */

   if( psEsTaskPackage->psTargetReference != NULL)
      DBVFreeDbvOctetString( psEsTaskPackage->psTargetReference );

   /* pcCreationDateTime */

   DBVFree_DbvGeneralizedTime( psEsTaskPackage->pcCreationDateTime );

   /* ppsPackageDiagnostics */

   FreeN_DbvDiagRec( psEsTaskPackage->ppsPackageDiagnostics );

   /* sTaskSpecificParameters */

   DBVFree_compulsory_DbvASN1External( &psEsTaskPackage->sTaskSpecificParameters );

   DbvFree( psEsTaskPackage );
   } /* End DBVFree_DbvEsTaskPackage() */ 


void DBVFree_DbvEstimate1( DbvEstimate1 *psEstimate1 )
   {

   if ( psEstimate1 == NULL )
	 return;

   /* pulCurrencyCode */

   if ( psEstimate1->pulCurrencyCode != NULL )
      DBVFree_Ulong( psEstimate1->pulCurrencyCode );
	   
   DbvFree( psEstimate1 );
   } /* End DBVFree_DbvEstimate1() */ 


void DBVFree_DbvEstimate2( DbvEstimate2 *psEstimate2 )
   {

   if ( psEstimate2 == NULL )
	 return;

   DBVFree_compulsory_DbvStringOrNumeric(&psEstimate2->sType);
   DBVFree_compulsory_DbvIntUnit(&psEstimate2->sValue);

   DbvFree( psEstimate2 );
   } /* End DBVFree_DbvEstimate2() */ 

void DBVFree_DbvFormatSpec( DbvFormatSpec *psFormatSpec )
   {

   if ( psFormatSpec == NULL )
	 return;

   DBVFree_compulsory_DbvInternationalString( &psFormatSpec->sType );

   /* pulSize */

   if ( psFormatSpec->pulSize != NULL )
      DBVFree_Ulong( psFormatSpec->pulSize );
	   
   /* pulBestPosn */

   if ( psFormatSpec->pulBestPosn != NULL )
      DBVFree_Ulong( psFormatSpec->pulBestPosn );
	   
   DbvFree( psFormatSpec );
   } /* End DBVFree_DbvFormatSpec() */ 

void DBVFree_DbvFragmentSyntax( DbvFragmentSyntax *psFragmentSyntax )
   {

   if ( psFragmentSyntax == NULL )
      return;

   switch( psFragmentSyntax->eFragmentSyntaxType )
      {
      case FST_ExternallyTagged:
	 DBVFreeDbvASN1External( psFragmentSyntax->
			      uFragmentSyntax.psExternallyTagged );
	 break;

      case FST_NotExternallyTagged:
	 if ( psFragmentSyntax-> uFragmentSyntax.psNotExternallyTagged != NULL )
	    DBVFreeDbvOctetString( psFragmentSyntax->
			     uFragmentSyntax.psNotExternallyTagged );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psFragmentSyntax );
   } /* End DBVFree_DbvFragmentSyntax() */ 

void DBVFree_DbvGeneralizedTime( DbvGeneralizedTime psGeneralizedTime )
   {

   if ( psGeneralizedTime == NULL )
      return;

   DbvFree( psGeneralizedTime );
   } /* End DBVFree_DbvGeneralizedTime() */ 

void DBVFree_DbvHoldingsRecord( DbvHoldingsRecord *psHoldingsRecord )
   {

   if ( psHoldingsRecord == NULL )
      return;

   switch( psHoldingsRecord->eHoldingsRecType )
      {
      case HRT_MarcHoldingsRecord:
         DBVFree_compulsory_DbvASN1External( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sMarcHoldingsRecord );
	 break;

      case HRT_HoldingsAndCirc:
         DBVFree_compulsory_DbvHoldingsAndCircData( &psHoldingsRecord->
				uMarcOrHoldAndCirc.sHoldingsAndCirc );
	 break;
      default:
	 break;
      } /* End switch */

   DbvFree( psHoldingsRecord );
   } /* End DBVFree_DbvHoldingsRecord() */ 

void DBVFree_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      {
      do_DBVFree_DbvIntUnit( psIntUnit );
      DbvFree( psIntUnit );
      } /* End if */

   } /* End DBVFree_DbvIntUnit() */

void DBVFree_compulsory_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   if ( psIntUnit != NULL )
      do_DBVFree_DbvIntUnit( psIntUnit );

   } /* End DBVFree_compulsory_DbvIntUnit() */


void do_DBVFree_DbvIntUnit( DbvIntUnit *psIntUnit )
   {

   DBVFree_compulsory_DbvUnit( &psIntUnit->sUnitUsed );
   } /* End DBVFree_DbvIntUnit() */ 

void DBVFree_DbvListEntries( DbvListEntries *psListEntries )
   {

   if ( psListEntries == NULL )
      return;

   FreeN_DbvEntry( psListEntries->ppsEntry );
   FreeN_DbvDiagRec( psListEntries->ppsNonSurrogateDiagnostics );

   DbvFree( psListEntries );
   } /* End DBVFree_DbvListEntries() */ 

void DBVFree_DbvListStatus( DbvListStatus *psListStatus )
   {
   DBVFree_compulsory_DbvInternationalString( &psListStatus->sResultSetId );
   DbvFree( psListStatus );
   } /* End DBVFree_DbvListStatus() */ 

void DBVFree_DbvNamePlusRecord( DbvNamePlusRecord *psNamePlusRecord )
   {

   if ( psNamePlusRecord == NULL )
      return;

   if ( psNamePlusRecord->psDatabaseName != NULL )
      DBVFreeDbvInternationalString( psNamePlusRecord->psDatabaseName );

   switch( psNamePlusRecord->eDbRecordType )
      {
      case DRT_Retrieval:
	 DBVFreeDbvASN1External( psNamePlusRecord->uDbRecord.psRetrievalRecord );
	 break;

      case DRT_SurrogateDiagnostics:
	 DBVFree_DbvDiagRec( psNamePlusRecord->uDbRecord.psSurrogateDiagnostic );
	 break;

      case DRT_StartingFragment:
	 DBVFree_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psStartingFragment );
	 break;

      case DRT_IntermediateFragment:
	 DBVFree_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psIntermediateFragment );
	 break;

      case DRT_FinalFragment:
	 DBVFree_DbvFragmentSyntax( psNamePlusRecord->
				 uDbRecord.psFinalFragment);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNamePlusRecord );
   } /* End DBVFree_DbvNamePlusRecord() */ 

void DBVFree_DbvOccurrence( DbvOccurrence *psOccurrence )
   {

   if ( psOccurrence == NULL )
      return;

   switch( psOccurrence->eOccurrenceType )
      {
      case OT_Global:
	 /* No space allocated */
	 break;

      case OT_ByDatabase:
	 FreeN_DbvOccurrenceByDb( psOccurrence->uOccurence.ppsOccurrenceByDb );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psOccurrence );
   } /* End DBVFree_DbvOccurrence() */ 


void DBVFree_DbvOccurrenceByAttributes( DbvOccurrenceByAttributes 
			*psOccurrenceByAttributes )
   {

   if ( psOccurrenceByAttributes == NULL )
      return;

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psOccurrenceByAttributes->ppsAttributeElement );

   /* psOccurrence */

   DBVFree_DbvOccurrence( psOccurrenceByAttributes->psOccurrence );

   /* ppsOtherInformation */
   if( psOccurrenceByAttributes->ppsOtherInformation != NULL)
      FreeN_DbvOtherInformation(psOccurrenceByAttributes->ppsOtherInformation );


   DbvFree( psOccurrenceByAttributes );
   } /* End DBVFree_DbvOccurrenceByAttributes() */ 


void DBVFree_DbvOccurrenceByDb( DbvOccurrenceByDb *psOccurrenceByDb )
   {

   if ( psOccurrenceByDb == NULL )
      return;

   /* sDatabaseName */

   DBVFree_compulsory_DbvInternationalString( &psOccurrenceByDb->sDatabaseName );

   /* pulNumber */

   if ( psOccurrenceByDb->pulNumber ) 
      DBVFree_Ulong( psOccurrenceByDb->pulNumber );

   /* ppsOtherInformation */
   if ( psOccurrenceByDb->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psOccurrenceByDb->ppsOtherInformation );

   DbvFree( psOccurrenceByDb );
   } /* End DBVFree_DbvOccurrenceByDb() */ 


void DBVFree_DbvOpacRecord( DbvOpacRecord *psOpacRecord )
   {

   if ( psOpacRecord == NULL )
      return;

   DBVFreeDbvASN1External( psOpacRecord->psBibliographicRecord );
   FreeN_DbvHoldingsRecord( psOpacRecord->ppsHoldingsData );

   DbvFree( psOpacRecord );
   } /* End DBVFree_DbvOpacRecord() */ 

void DBVFree_DbvPermAllowableFunctions( DbvPermAllowableFunctions 
					*psPermAllowableFunctions )
   {

   if ( psPermAllowableFunctions == NULL )
      return;

   DbvFree( psPermAllowableFunctions );
   } /* End DBVFree_DbvPermAllowableFunctions() */ 

void DBVFree_DbvPermissions( DbvPermissions *psPermissions )
   {

   if ( psPermissions == NULL )
      return;

   DBVFree_compulsory_DbvInternationalString( &psPermissions->sUserId );
   FreeN_DbvPermAllowableFunctions( psPermissions->ppeAllowableFunctions );

   DbvFree( psPermissions );
   } /* End DBVFree_DbvPermissions() */ 

void DBVFree_DbvPresentReq( DbvPresentReq *psPresentReq )
   {
   /* psReferenceId */
   if( psPresentReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psPresentReq->psReferenceId);

   /* sResultSetId */

   DBVFree_compulsory_DbvInternationalString( &psPresentReq->sResultSetId );

   /* ppsAdditionalRanges */

   FreeN_DbvRange( psPresentReq->ppsAdditionalRanges );

   /* DbvRecordComposition */

   if( psPresentReq->psRecordComposition != NULL)
      DBVFree_DbvRecordComposition( psPresentReq->psRecordComposition );

   /* pcPreferredRecordSyntax */

   if( psPresentReq->pcPreferredRecordSyntax != NULL)
      DBVFreeDbvObjectIdentifier( psPresentReq->pcPreferredRecordSyntax );

   /* pulMaxSegmentCount */

   if( psPresentReq->pulMaxSegmentCount != NULL)
      DBVFree_Ulong( psPresentReq->pulMaxSegmentCount );

   /* pulMaxRecordSize */

   if( psPresentReq->pulMaxRecordSize!= NULL)
      DBVFree_Ulong( psPresentReq->pulMaxRecordSize );

   /* pulMaxSegmentSize */

   if( psPresentReq->pulMaxSegmentSize != NULL)
      DBVFree_Ulong( psPresentReq->pulMaxSegmentSize );

   /* ppsOtherInformation */

   if( psPresentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentReq->ppsOtherInformation );

   DbvFree( psPresentReq );
   } /* End DBVFree_DbvPresentReq() */ 

void DBVFree_DbvPresentRsp( DbvPresentRsp *psPresentRsp )
   {
   /* psReferenceId */
   if( psPresentRsp->psReferenceId != NULL)
      DBVFreeDbvOctetString( psPresentRsp->psReferenceId );

   /* psRecords */

   DBVFree_DbvRecords( psPresentRsp->psRecords );

   /* ppsOtherInformation */

   if( psPresentRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psPresentRsp->ppsOtherInformation );

   DbvFree( psPresentRsp );
   } /* End DBVFree_DbvPresentRsp() */ 

void DBVFree_DbvPresentStatus( DbvPresentStatus *pePresentStatus )
   {

   if ( pePresentStatus == NULL )
      return;

   DbvFree( pePresentStatus );
   } /* End DBVFree_DbvPresentStatus() */ 

void DBVFree_DbvQuery( DbvQuery *psQuery )
   {

   if ( psQuery == NULL )
      return;

   switch( psQuery->eQueryType )
      {
      case QT_Private:
	 if ( psQuery->uQuery.psPrivateQuery )
	    DBVFreeDbvOctetString( psQuery->uQuery.psPrivateQuery);
	 break;

      case QT_Iso8777:
	 if ( psQuery->uQuery.psIso8777Query )
	    DBVFreeDbvOctetString( psQuery->uQuery.psIso8777Query);
	 break;

      case QT_Z3958:
	 if ( psQuery->uQuery.psZ3958Query )
	    DBVFreeDbvOctetString( psQuery->uQuery.psZ3958Query);
	 break;

      case QT_NaturalLanguage:
	 if ( psQuery->uQuery.psNaturalLanguageQuery )
	    DBVFreeDbvOctetString( psQuery->uQuery.psNaturalLanguageQuery);
	 break;

      case QT_Erpn:
         if (psQuery->uQuery.psErpnQuery != NULL)
                DBVFree_DbvRPNQuery(psQuery->uQuery.psErpnQuery);
         break;

      case QT_Rpn:
         if (psQuery->uQuery.psRpnQuery != NULL)
                DBVFree_DbvRPNQuery(psQuery->uQuery.psRpnQuery);
 
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQuery );
   } /* End DBVFree_DbvQuery() */ 

void DBVFree_DbvRange( DbvRange *psRange )
   {
   DbvFree( psRange );
   } /* End DBVFree_DbvRange() */ 

void DBVFree_DbvRecords( DbvRecords *psRecords )
   {

   if ( psRecords == NULL )
      return;

   switch( psRecords->eRecordType )
      {
      case RT_Response:
	 FreeN_DbvNamePlusRecord( psRecords->uRecord.ppsResponseRecord );
	 break;

      case RT_NonSurrDiagnostics:
	 DBVFree_DbvDefaultDiagFormat( psRecords->uRecord.psNonSurrogateDiag );
	 break;

      case RT_MultipleNonSurrDiagnostics:
	 FreeN_DbvDiagRec( psRecords->uRecord.ppsMultipleNonSurrogateDiag );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psRecords );
   } /* End DBVFree_DbvRecord() */ 

void DBVFree_DbvResourceReport1( DbvResourceReport1 *psResourceReport1 )
   {

   if ( psResourceReport1 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate1( psResourceReport1->ppsEstimate );

   /* sMessage */

   DBVFree_compulsory_DbvInternationalString( &psResourceReport1->sMessage );

   DbvFree( psResourceReport1 );
   } /* End DBVFree_DbvResourceReport1() */ 

void DBVFree_DbvResourceReport2( DbvResourceReport2 *psResourceReport2 )
   {

   if ( psResourceReport2 == NULL )
      return;

   /* ppsEstimate */

   FreeN_DbvEstimate2( psResourceReport2->ppsEstimate );

   /* psMessage */

   if ( psResourceReport2->psMessage != NULL )
      DBVFreeDbvInternationalString( psResourceReport2->psMessage );

   DbvFree( psResourceReport2 );
   } /* End DBVFree_DbvResourceReport2() */ 

void DBVFree_DbvResourceReportReq( DbvResourceReportReq *psResourceReportReq )
   {

   /* psReferenceId */

   if( psResourceReportReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psResourceReportReq->psReferenceId);

   /* psOperationId */

   if( psResourceReportReq->psOperationId != NULL)
      DBVFreeDbvOctetString(psResourceReportReq->psOperationId );

   /* pcPrefResourceReportFormat */

   if( psResourceReportReq->pcPrefResourceReportFormat != NULL)
      DBVFreeDbvObjectIdentifier(psResourceReportReq->pcPrefResourceReportFormat );

   /* ppsOtherInformation */

   if( psResourceReportReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportReq->ppsOtherInformation );

   DbvFree( psResourceReportReq );
   } /* End DBVFree_DbvResourceReportReq() */ 

void DBVFree_DbvResourceReportRsp( DbvResourceReportRsp *psResourceReportRsp )
   {

   /* psReferenceId */

   if( psResourceReportRsp->psReferenceId != NULL)
      DBVFreeDbvOctetString(psResourceReportRsp->psReferenceId );

   /* psResourceReport*/

   DBVFreeDbvASN1External( psResourceReportRsp->psResourceReport );

   /* ppsOtherInformation */

   if( psResourceReportRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psResourceReportRsp->ppsOtherInformation );

   DbvFree( psResourceReportRsp );
   } /* End DBVFree_DbvResourceReportRsp() */ 

void DBVFree_DbvResultSetStatus( DbvResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End DBVFree_DbvResultSetStatus() */ 

void DBVFree_DbvScanReq( DbvScanReq *psScanReq )
   {
   /* psReferenceId */
   if( psScanReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psScanReq->psReferenceId);
			 
   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psScanReq->ppsDatabaseNames );

   /* pcAttributeSetId  */

   if( psScanReq->pcAttributeSetId != NULL)
      DBVFreeDbvObjectIdentifier( psScanReq->pcAttributeSetId );

   /* pulStepSize */

   if( psScanReq->pulStepSize != NULL)
      DBVFree_Ulong( psScanReq->pulStepSize );

   /* pulPreferredPositionInResponse */

   if( psScanReq->pulPreferredPositionInResponse != NULL)
      DBVFree_Ulong( psScanReq->pulPreferredPositionInResponse );

   /* ppsOtherInformation */

   if( psScanReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanReq->ppsOtherInformation );

   DbvFree( psScanReq );
   } /* End DBVFree_DbvScanReq() */ 

void DBVFree_DbvScanRsp( DbvScanRsp *psScanRsp )
   {

   /* psReferenceId */

   if( psScanRsp->psReferenceId != NULL )
      DBVFreeDbvOctetString(psScanRsp->psReferenceId );

   /* pulStepSize */

   if ( psScanRsp->pulStepSize )
      DBVFree_Ulong( psScanRsp->pulStepSize );

   /* pulPositionOfTerm */

   if ( psScanRsp->pulPositionOfTerm )
      DBVFree_Ulong( psScanRsp->pulPositionOfTerm );

   /* psEntries */

   DBVFree_DbvListEntries( psScanRsp->psEntries );

   /* pcAttributeSetId */

   if( psScanRsp->pcAttributeSetId != NULL)
      DBVFreeDbvObjectIdentifier( psScanRsp->pcAttributeSetId );

   /* ppsOtherInformation */

   if( psScanRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psScanRsp->ppsOtherInformation );

   DbvFree( psScanRsp );
   } /* End DBVFree_DbvScanRsp() */ 

void DBVFree_DbvSearchReq( DbvSearchReq *psSearchReq )
   {

   /* psReferenceId */

   if( psSearchReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psSearchReq->psReferenceId);

   /* sResultSetName */

   DBVFree_compulsory_DbvInternationalString( &psSearchReq->sResultSetName );

   /* ppsDatabaseNames */

   FreeN_DbvInternationalString( psSearchReq->ppsDatabaseNames );

   /* psSmallSetElementNames */

   DBVFree_DbvElementSetNames( psSearchReq->psSmallSetElementNames );

   /* psMediumSetElementNames */

   DBVFree_DbvElementSetNames( psSearchReq->psMediumSetElementNames );

   /* pcPreferredRecordSyntax */

   if( psSearchReq->pcPreferredRecordSyntax != NULL)
      DBVFreeDbvObjectIdentifier( psSearchReq->pcPreferredRecordSyntax );

   /* psQuery */

   DBVFree_DbvQuery( psSearchReq->psQuery );

   /* psAdditionalSearchInfo */

   if( psSearchReq->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchReq->ppsOtherInformation );

   DbvFree( psSearchReq );
   } /* End DBVFree_DbvSearchReq() */ 

void DBVFree_DbvSearchRsp( DbvSearchRsp *psSearchRsp )
   {

   /* psReferenceId */

   if( psSearchRsp->psReferenceId != NULL )
      DBVFreeDbvOctetString(psSearchRsp->psReferenceId );

   /* peResultSetStatus */

   if ( psSearchRsp->peResultSetStatus )
      DBVFree_DbvResultSetStatus( psSearchRsp->peResultSetStatus );

   /* pePresentStatus */

   if ( psSearchRsp->pePresentStatus )
      DBVFree_DbvPresentStatus( psSearchRsp->pePresentStatus );

   /* psRecords */

   DBVFree_DbvRecords( psSearchRsp->psRecords );

   /* psAdditionalSearchInfo */

   if( psSearchRsp->ppsAdditionalSearchInfo != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsAdditionalSearchInfo );

   /* ppsOtherInformation */

   if( psSearchRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSearchRsp->ppsOtherInformation );

   DbvFree( psSearchRsp );
   } /* End DBVFree_DbvSearchRsp() */ 

void DBVFree_DbvSegmentReq( DbvSegmentReq *psSegmentReq )
   {

   /* psReferenceId */

   if( psSegmentReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psSegmentReq->psReferenceId);

   /* ppsSegmentRecords */

   FreeN_DbvNamePlusRecord( psSegmentReq->ppsSegmentRecords );

   /* ppsOtherInformation */

   if( psSegmentReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSegmentReq->ppsOtherInformation );

   DbvFree( psSegmentReq );
   } /* End DBVFree_DbvSegmentReq() */ 

void DBVFree_DbvSingleASN1Type( DbvSingleASN1Type *psSingleASN1Type )
   {

   if ( psSingleASN1Type == NULL )
      return;

   switch( psSingleASN1Type->eASN1Type )
      {
      case AT_UnstructuredText:
	 if ( psSingleASN1Type-> uASN1.psUnstructuredText != NULL )
	    DBVFreeDbvInternationalString( psSingleASN1Type->
				 uASN1.psUnstructuredText );
	 break;

      case AT_OpacRecord:
	 DBVFree_DbvOpacRecord( psSingleASN1Type->uASN1.psOpacRecord );
	 break;

      case AT_SummaryRecord:
	 DBVFree_DbvSummaryRecord( psSingleASN1Type->uASN1.psSummaryRecord );
	 break;

      case AT_ResourceReport1:
	 DBVFree_DbvResourceReport1( psSingleASN1Type->uASN1.psResourceReport1 );
	 break;

      case AT_ResourceReport2:
	 DBVFree_DbvResourceReport2( psSingleASN1Type->uASN1.psResourceReport2 );
	 break;

      case AT_EsTaskPackage:
	 DBVFree_DbvEsTaskPackage( psSingleASN1Type->uASN1.psEsTaskPackage );
	 break;

      case AT_ItemOrderExtendedService:
	 DBVFree_DbvItemOrderExtendedService( psSingleASN1Type->
					   uASN1.psItemOrderExtendedService );
	 break;

      case AT_UpdateExtendedService:
	 DBVFree_DbvUpdateExtendedService( psSingleASN1Type->
					uASN1.psUpdateExtendedService );
	 break;

      case AT_IllPdu:
	 DBVFree_DbvIllPdu( psSingleASN1Type->uASN1.psIllPdu );
	 break;

      case AT_UnknownType:
	 if ( psSingleASN1Type->uASN1.psUnknownType != NULL )
	    DBVFreeDbvOctetString( psSingleASN1Type->uASN1.psUnknownType );
         break;
	
      case AT_ExplainRecord:
	 DBVFree_DbvExplainRecord( psSingleASN1Type->uASN1.psExplainRecord );
         break;

      case AT_PromptObject:
         DBVFree_DbvPromptObject( psSingleASN1Type->uASN1.psPromptObject );
         break;


	#ifdef NOT_YET_IMPLEMENTED
      case AT_GenericRecord: 
	 DBVFree_DbvGenericRecord( psSingleASN1Type->uASN1.psGenericRecord );
	 break;

	case AT_AccessControlDes1:
	case AT_AccessControlKrb1:
	case AT_PerResSetExtendedService:
	case AT_PerQueryExtendedService:
	case AT_PerQuerySchExtendedService:
	case AT_ExpSpecExtendedService:
	case AT_ExpInvoExtendedService:
	case AT_UserInfoSearchResult:
	case AT_ElementSpec1:
		break;
	#endif


      default:
	 break;
      } /* End switch */

   DbvFree( psSingleASN1Type );
   } /* End DBVFree_DbvSingleASN1Type() */ 

void DBVFree_DbvSortKeySpec( DbvSortKeySpec *psSortKeySpec )
   {

   if ( psSortKeySpec->psMVActionType != NULL )
      DBVFree_DbvSortMVAction( psSortKeySpec->psMVActionType );

   DbvFree( psSortKeySpec );
   } /* End DBVFree_DbvSortKeySpec() */ 

void DBVFree_DbvSortMVAction( DbvSortMVAction *psSortMVAction )
   {

   if ( psSortMVAction->psOctetString!= NULL )
      DBVFreeDbvOctetString( psSortMVAction->psOctetString );

   DbvFree( psSortMVAction );
   } /* End DBVFree_DbvSortMVAction() */ 

void DBVFree_DbvSortReq( DbvSortReq *psSortReq )
   {
   /* psReferenceId */
   if( psSortReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psSortReq->psReferenceId);
			 
   /* ppsInputResultSetName */

   FreeN_DbvInternationalString( psSortReq->ppsInputResultSetName );

   /* sSortedResultSetName */

   DBVFree_compulsory_DbvInternationalString( &psSortReq->sSortedResultSetName );

   /* ppsSortSequence */

   FreeN_DbvSortKeySpec( psSortReq->ppsSortSequence );

   /* ppsOtherInformation */

   if( psSortReq->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortReq->ppsOtherInformation );

   DbvFree( psSortReq );
   } /* End DBVFree_DbvSortReq() */ 

void DBVFree_DbvSortResultSetStatus( DbvSortResultSetStatus *peResultSetStatus )
   {

   if ( peResultSetStatus == NULL )
      return;

   DbvFree( peResultSetStatus );
   } /* End DBVFree_DbvSortResultSetStatus() */ 

void DBVFree_DbvSortRsp( DbvSortRsp *psSortRsp )
   {

   /* psReferenceId */

   if( psSortRsp->psReferenceId != NULL )
      DBVFreeDbvOctetString(psSortRsp->psReferenceId );

   /* peResultSetStatus  */

   DBVFree_DbvSortResultSetStatus( psSortRsp->peResultSetStatus );

   /* ppsDiagnostics */

   FreeN_DbvDiagRec( psSortRsp->ppsDiagnostics );

   /* ppsOtherInformation */

   if( psSortRsp->ppsOtherInformation != NULL)
     FreeN_DbvOtherInformation( psSortRsp->ppsOtherInformation );

   DbvFree( psSortRsp );
   } /* End DBVFree_DbvSortRsp() */ 

void DBVFree_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      {
      do_DBVFree_DbvStringOrNumeric( psStringOrNumeric );
      DbvFree( psStringOrNumeric );
      } /* End if */

   } /* End DBVFreeDbvStringOrNumeric() */

void DBVFree_compulsory_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {

   if ( psStringOrNumeric != NULL )
      do_DBVFree_DbvStringOrNumeric( psStringOrNumeric );

   } /* End DBVFree_compulsory_DbvStringOrNumeric() */


void do_DBVFree_DbvStringOrNumeric( DbvStringOrNumeric *psStringOrNumeric )
   {


   switch( psStringOrNumeric->eStringOrNumeric )
      {
      case SNT_String:
	 DBVFreeDbvInternationalString( psStringOrNumeric->
				     uStringOrNumeric.psString);
	 break;

      case SNT_Numeric:
      default:
	 break;
      } /* End switch */

   } /* End do_DBVFree_DbvStringOrNumeric() */ 

void DBVFree_DbvSummaryRecord( DbvSummaryRecord *psSummaryRecord )
   {

   if ( psSummaryRecord == NULL )
      return;

   /* sTitle */

   DBVFree_compulsory_DbvInternationalString( &psSummaryRecord->sTitle );

   /* psAuthor */

   if ( psSummaryRecord->psAuthor != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psAuthor );

   /* psCallNumber */

   if ( psSummaryRecord->psCallNumber != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psCallNumber );

   /* psRecordType */

   if ( psSummaryRecord->psRecordType != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psRecordType );

   /* psBibliographicLevel */

   if ( psSummaryRecord->psBibliographicLevel != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psBibliographicLevel );

   /* ppsFormat */

   FreeN_DbvFormatSpec( psSummaryRecord->ppsFormat );

   /* psPublicationPlace */

   if ( psSummaryRecord->psPublicationPlace != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psPublicationPlace );

   /* psPublicationDate */

   if ( psSummaryRecord->psPublicationDate != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psPublicationDate );

   /* psTargetSystemKey */

   if ( psSummaryRecord->psTargetSystemKey != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psTargetSystemKey );

   /* psSatisfyingElement */

   if ( psSummaryRecord->psSatisfyingElement != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psSatisfyingElement );

   /* pulRank */

   if ( psSummaryRecord->pulRank )
      DBVFree_Ulong( psSummaryRecord->pulRank );

   /* psDocumentId */

   if ( psSummaryRecord->psDocumentId != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psDocumentId );

   /* psAbstract */

   if ( psSummaryRecord->psAbstract != NULL )
      DBVFreeDbvInternationalString( psSummaryRecord->psAbstract );

   /* ppsOtherInformation */

   if ( psSummaryRecord->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psSummaryRecord->ppsOtherInformation );

   DbvFree( psSummaryRecord );
   } /* End DBVFree_DbvSummaryRecord() */ 

void DBVFree_DbvSuppliedRecord( DbvSuppliedRecord *psSuppliedRecord )
   {

   if ( psSuppliedRecord == NULL )
      return;

   /* psUpdtRecordId  */

   DBVFree_DbvUpdtRecordId( psSuppliedRecord->psUpdtRecordId ); 

   /* psUpdtSupplementalId */

   DBVFree_DbvUpdtSupplementalId( psSuppliedRecord->psUpdtSupplementalId  );

   /* psCorrelationInfo */

   DBVFree_DbvCorrelationInfo( psSuppliedRecord->psCorrelationInfo  );

   /* sRecord */

   DBVFree_compulsory_DbvASN1External( &psSuppliedRecord->sRecord );

   DbvFree( psSuppliedRecord );

   } /* End DBVFree_compulsory_DbvSuppliedRecord() */ 

void DBVFree_DbvTaskPackageRecord( DbvTaskPackageRecord *psTaskPackageRecord )
   {

   if ( psTaskPackageRecord == NULL )
      return;

   /* psUpdtRecordOrDiag */

   DBVFree_DbvUpdtRecordOrDiag( psTaskPackageRecord->psUpdtRecordOrDiag );

   /* psCorrelationInfo */

   DBVFree_DbvCorrelationInfo( psTaskPackageRecord->psCorrelationInfo );

   DbvFree( psTaskPackageRecord );
   } /* End DBVFree_DbvTaskPackageRecord() */ 

void DBVFree_DbvTermInfo( DbvTermInfo *psTermInfo )
   {

   if ( psTermInfo == NULL )
      return;

   /* sTerm */

   DBVFree_compulsory_DbvTerm( &psTermInfo->sTerm  );

   /* psDisplayTerm */
   DBVFreeDbvInternationalString( psTermInfo->psDisplayTerm );

   /* ppsAttributeElement */

   FreeN_DbvAttributeElement( psTermInfo->ppsSuggestedAttribute );

   /* ppsAlternativeTerm */

   FreeN_DbvAttributesPlusTerm( psTermInfo->ppsAlternativeTerm );

   /* pulGlobalOccurrences */

   if ( psTermInfo->pulGlobalOccurrences != NULL )
      DBVFree_Ulong( psTermInfo->pulGlobalOccurrences );

   /* psByAttributes */

   FreeN_DbvOccurrenceByAttributes( psTermInfo->ppsByAttributes );

   /* ppsOtherInformation */

   if ( psTermInfo->ppsOtherInformation != NULL )
      FreeN_DbvOtherInformation( psTermInfo->ppsOtherInformation );

   DbvFree( psTermInfo );
   } /* End DBVFree_DbvTermInfo() */ 

void DBVFree_DbvUpdateExtendedService( DbvUpdateExtendedService 
						*psUpdateExtendedService )
   {

   if ( psUpdateExtendedService == NULL )
      return;

   switch( psUpdateExtendedService->eUpdateReqRsp )
      {
      case UROR_Request:
	 DBVFree_DbvUpdateRequestPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateRequest );
	 break;

      case UROR_Response:
	 DBVFree_DbvUpdateTaskPackage( psUpdateExtendedService->
				     uUpdateReqOrRsp.psUpdateResponse );
	 break;

      default:
	 break;

      } /* End switch */

   DbvFree( psUpdateExtendedService );
   } /* End DBVFree_DbvUpdateExtendedService() */

void DBVFree_DbvUpdateRequestPackage( DbvUpdateRequestPackage 
						*psUpdateRequestPackage )
   {

   if ( psUpdateRequestPackage == NULL )
      return;

   DBVFree_compulsory_DbvOriginPartToKeep( &psUpdateRequestPackage->
							sOriginPartToKeep );
   FreeN_DbvSuppliedRecord( psUpdateRequestPackage->ppsSuppliedRecord );

   DbvFree( psUpdateRequestPackage );
   } /* End DBVFree_DbvUpdateRequestPackage() */ 

void DBVFree_DbvUpdateTaskPackage( DbvUpdateTaskPackage 
						*psUpdateTaskPackage )
   {

   if ( psUpdateTaskPackage == NULL )
      return;

   DBVFree_compulsory_DbvOriginPartToKeep( &psUpdateTaskPackage->
							sOriginPartToKeep );
   DBVFree_compulsory_DbvUpdateTargetPart( &psUpdateTaskPackage->sTargetPart );

   DbvFree( psUpdateTaskPackage );
   } /* End DBVFree_DbvUpdateTaskPackage() */ 

void DBVFree_DbvUpdtRecordId( DbvUpdtRecordId *psUpdtRecordId )
   {

   if ( psUpdtRecordId == NULL )
      return;

   switch( psUpdtRecordId->eRecordIdType )
      {
      case URIT_Number:
	 /* No Space allocated */
	 break;

      case URIT_String:
	 if ( psUpdtRecordId->uUpdtRecordId.psString != NULL )
	    DBVFreeDbvInternationalString( psUpdtRecordId->
					uUpdtRecordId.psString );
	 break;

      case URIT_Opaque:
	 if ( psUpdtRecordId->uUpdtRecordId.psOpaque != NULL )
	    DBVFreeDbvOctetString( psUpdtRecordId-> uUpdtRecordId.psOpaque);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordId );
   } /* End DBVFree_DbvUpdtRecordId() */ 

void DBVFree_DbvUpdtRecordOrDiag( DbvUpdtRecordOrDiag *psUpdtRecordOrDiag )
   {

   if ( psUpdtRecordOrDiag == NULL )
      return;

   switch( psUpdtRecordOrDiag->eUpdtRecordOrDiag )
      {
      case URDT_Record:
	 DBVFreeDbvASN1External( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord );
	 break;

      case URDT_Diagnostic:
	 DBVFree_DbvDiagRec( psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtRecordOrDiag );
   } /* End DBVFree_DbvUpdtRecordOrDiag() */ 

void DBVFree_DbvUpdtSupplementalId( DbvUpdtSupplementalId *psUpdtSupplementalId )
   {

   if ( psUpdtSupplementalId == NULL )
      return;

   switch( psUpdtSupplementalId->eSupplementalIdType )
      {
      case USIT_Time:
	 DBVFree_DbvGeneralizedTime( psUpdtSupplementalId->
					uUpdtSupplementalId.pcTime );
	 break;

      case USIT_Version:
	 if ( psUpdtSupplementalId->uUpdtSupplementalId.psVersion != NULL )
	    DBVFreeDbvInternationalString( psUpdtSupplementalId->
					uUpdtSupplementalId.psVersion );
	 break;

      case USIT_PreviousVersion:
	 DBVFreeDbvASN1External( psUpdtSupplementalId->
					uUpdtSupplementalId.psPreviousVersion );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUpdtSupplementalId );
   } /* End DBVFree_DbvUpdtSupplementalId() */ 

void DBVFree_DbvVisibleString( DbvVisibleString psVisibleString )
   {

   if ( psVisibleString == NULL )
      return;

   DbvFree( psVisibleString );
   } /* End DBVFree_DbvVisibleString() */ 

void DBVFree_DbvVolume( DbvVolume *psVolume )
   {

   if ( psVolume == NULL )
	 return;

   /* psEnumeration */

   if ( psVolume->psEnumeration != NULL )
      DBVFreeDbvInternationalString( psVolume->psEnumeration );

   /* psChronology */

   if ( psVolume->psChronology != NULL )
      DBVFreeDbvInternationalString( psVolume->psChronology );

   /* psEnumAndChron */

   if ( psVolume->psEnumAndChron != NULL )
      DBVFreeDbvInternationalString( psVolume->psEnumAndChron );

   DbvFree( psVolume );

   } /* End DBVFree_DbvVolume() */ 

void DBVFree_Ulong( Ulong *pulong )
   {
   DbvFree( pulong );
   } /* End DBVFree_Ulong() */ 


void DBVFreeDbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      {
      do_DBVFreeDbvASN1External( psASN1External );
      DbvFree( psASN1External );
      } /* End if */

   } /* End DBVFreeDbvASN1External() */

void DBVFree_compulsory_DbvASN1External( DbvASN1External *psASN1External )
   {

   if ( psASN1External != NULL )
      do_DBVFreeDbvASN1External( psASN1External );

   } /* End DBVFree_compulsory_DbvASN1External() */


void do_DBVFreeDbvASN1External( DbvASN1External *psASN1External )
   {


   if ( psASN1External->pcObjectIdentifier != NULL )
      DBVFreeDbvObjectIdentifier( psASN1External->pcObjectIdentifier );

   switch( psASN1External->eValueType )
      {
      case EVT_SingleASN1Type:
	 DBVFree_DbvSingleASN1Type( psASN1External->
				 uExternalValue.psSingleASN1Type );
	 break;

      case EVT_OctetAligned:
	 DBVFreeDbvOctetString( psASN1External->
			     uExternalValue.psOctetAligned );
	 break;

      case EVT_Arbitrary:
	 DBVFreeDbvOctetString( psASN1External->
			     uExternalValue.psArbitrary );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVFreeDbvASN1External() */ 

void DBVFree_DbvRecordComposition(DbvRecordComposition *psRecordComposition)
   {


   if(psRecordComposition != NULL)
   {
	switch(psRecordComposition->eRecordCompositionType)
	{
	case RCT_ElementSetNames:
		DBVFree_DbvElementSetNames(psRecordComposition->uRecordComposition.
			psElementSetNames);
		break;
	case RCT_CompSpec:
		break;
	default:
		DBVFree_DbvCompSpec(psRecordComposition->uRecordComposition.
			psCompSpec);
		break;
	} /* End switch */
	DbvFree(psRecordComposition);
   } /* End if */

   } /* End DBVFree_DbvRecordComposition */

void DBVFree_DbvCompSpec(DbvCompSpec *psCompSpec)
   {
   if(psCompSpec != NULL)
	{
	DBVFree_DbvSpecification(psCompSpec->psGeneric);
	FreeN_DbvDbSpecificComp(psCompSpec->ppsDbSpecific);
	FreeN_DbvObjectIdentifier(psCompSpec->ppcRecordSyntax);
	DbvFree(psCompSpec);
	}
   } /* End DBVFree_DbvCompSpec */

void DBVFree_DbvSpecification(DbvSpecification *psSpecification)
{
	if(psSpecification != NULL)
	{
		DBVFree_compulsory_DbvSpecification(psSpecification);
		DbvFree(psSpecification);
	}
} /* End DBVFree_DbvSpecification */

void DBVFree_DbvDbSpecificComp(DbvDbSpecificComp *psDbSpecific)
{
	if(psDbSpecific != NULL)
	{
		DBVFree_compulsory_DbvInternationalString(&psDbSpecific->
			sDatabaseName);
		DBVFree_compulsory_DbvSpecification(&psDbSpecific->
			sSpecification);
	} /* End if */
} /* End DBVFree_DbvDbSpecificComp */

void DBVFree_DbvRPNQuery(DbvRPNQuery *psDbvRpnQuery)
{
	if(psDbvRpnQuery->pcAttributeSetId != NULL)
		DbvFree(psDbvRpnQuery->pcAttributeSetId);

        DBVFree_rpn_structure(&psDbvRpnQuery->sRpnStructure);
        DbvFree(psDbvRpnQuery);
}

void DBVFree_rpn_structure(DbvRPNStructure *psDbvRPNStructure)
{
 
        switch (psDbvRPNStructure->eRpnStructureType) {
        case RST_RpnLopRopOperator:
 
                if (psDbvRPNStructure->uRPNStructure.psRpnLopRopOperator != NULL)
                        DBVFree_rpnlopropoperator(psDbvRPNStructure->uRPNStructure.
				psRpnLopRopOperator);
                break;
        case RST_RpnOperand:
                if (psDbvRPNStructure->uRPNStructure.psRpnOperand != NULL)
 
                        DBVFree_rpnoperand(psDbvRPNStructure->uRPNStructure.
				psRpnOperand);
                break;
        }
 
}

void DBVFree_rpnlopropoperator(DbvRPNLopRopOperator *psDbvRPNLopRopOperator)
{
 
        if (psDbvRPNLopRopOperator->psLeftOperand != NULL)
                DBVFree_rpn_structure(psDbvRPNLopRopOperator->psLeftOperand);
 
        if (psDbvRPNLopRopOperator->psRightOperand != NULL)
                DBVFree_rpn_structure(psDbvRPNLopRopOperator->psRightOperand);
 
	DBVFree_DbvOperator(&psDbvRPNLopRopOperator->sOperator);
 
        DbvFree(psDbvRPNLopRopOperator);
}

void DBVFree_rpnoperand(DbvOperand *psRpnOperand)
{
 
        switch (psRpnOperand->eOperandType) {
        case OT_AttributePlusTerm:
 
                if (psRpnOperand->uOperand.psAttributesPlusTerm != NULL)
                        DBVFree_attsplusterm(psRpnOperand->uOperand.
				psAttributesPlusTerm);
                break;
        }
 
        DbvFree(psRpnOperand);
}

void DBVFree_attsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm)
{
        int index = 0;
 
 
        /* Print all attributes */
 
        while (psAttributesPlusTerm->ppsAttributeElement[index] != NULL) {
                DBVFree_att_element(psAttributesPlusTerm->
			ppsAttributeElement[index]);
                index++;
        }
 
        /* then the term */
        DBVFree_att_term(psAttributesPlusTerm->sTerm);
 
 
        DbvFree(psAttributesPlusTerm);
}

void DBVFree_att_element(DbvAttributeElement *psAttributeElement)
{
        if (psAttributeElement->pcAttributeSetId != NULL)
                DbvFree(psAttributeElement->pcAttributeSetId);
 
        DbvFree(psAttributeElement);
}
 
void DBVFree_att_term(DbvTerm sTerm)
{
        /* Only general catered for !! */
        switch (sTerm.eTermType) {
        case TT_General:
 
                if (sTerm.uTerm.psGeneral != NULL)
                        DbvFree(sTerm.uTerm.psGeneral);
                break;
        }
 
}

void DBVFree_DbvOperator(DbvOperator *psOperator)
{
	
	/* Operator is not dynamically allocated */
	if(psOperator->psProximityOperator != NULL)
	{
		if(psOperator->psProximityOperator->pfExclusion != NULL)
			DbvFree(psOperator->psProximityOperator->pfExclusion);
		DbvFree(psOperator->psProximityOperator);
	}
}

void DBVFree_DbvExtendedServicesReq(DbvExtendedServicesReq *psExtendedServicesReq)
{
   /* psReferenceId */

   if( psExtendedServicesReq->psReferenceId != NULL)
      DBVFreeDbvOctetString(psExtendedServicesReq->psReferenceId);

   /* pcPackageType */
   DBVFreeDbvObjectIdentifier( psExtendedServicesReq->pcPackageType);

   /* psPackageName */
   DBVFreeDbvInternationalString(psExtendedServicesReq->psPackageName);

   /* psUserId */
   DBVFreeDbvInternationalString(psExtendedServicesReq->psUserId);

   /* psRetentionTime */
   DBVFree_DbvIntUnit(psExtendedServicesReq->psRetentionTime);

   /* ppsPermissions */
   FreeN_DbvPermissions(psExtendedServicesReq->ppsPermissions);

   /* psDescription */
   DBVFreeDbvInternationalString(psExtendedServicesReq->psDescription);

   /* psOriginSpecificParameter */
   DBVFreeDbvASN1External(psExtendedServicesReq->psOriginSpecificParameter);

   /* psElements */
   DBVFreeDbvInternationalString(psExtendedServicesReq->psElements);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesReq->ppsOtherInformation);

   DbvFree(psExtendedServicesReq);
} /* End DBVFree_DbvExtendedServicesRsp */

void DBVFree_DbvExtendedServicesRsp(DbvExtendedServicesRsp *psExtendedServicesRsp)
{
   /* psReferenceId */
 
   DBVFreeDbvOctetString(psExtendedServicesRsp->psReferenceId);
 
   /* ppsDiagnostics */
   FreeN_DbvDiagRec(psExtendedServicesRsp->ppsDiagnostics);

   /* psTaskPackage */
   DBVFreeDbvASN1External(psExtendedServicesRsp->psTaskPackage);

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation(psExtendedServicesRsp->ppsOtherInformation);

} /* End DBVFree_DbvExtendedServicesRsp */

void DBVFree_DbvAssociateResult(DbvAssociateResult *puAssociateResult,
        DbvAssociateState eAssociateState)
{
        int place = 0;
        if(eAssociateState == Associate_Null || eAssociateState ==
          Associate_Pending)
                return;
 
        if(eAssociateState == Associate_Accepted)
        {
                if(puAssociateResult->psProtocolAndSyntaxes->
                  peApplicationProtocol != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peApplicationProtocol);
                if(puAssociateResult->psProtocolAndSyntaxes->peAbstractSyntax 
                  != NULL)
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          peAbstractSyntax);
                if(puAssociateResult->psProtocolAndSyntaxes->
			ppcCommonSyntaxes != NULL)
                {
                while(puAssociateResult->psProtocolAndSyntaxes->
                  ppcCommonSyntaxes[place] != NULL)
                {
                        DbvFree(puAssociateResult->psProtocolAndSyntaxes->
                          ppcCommonSyntaxes[place]);
                        place++;
                }
		}
                DbvFree(puAssociateResult->psProtocolAndSyntaxes);
        }
        else
        {       
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectSource != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectSource);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectResult != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectResult);
                if(puAssociateResult->psAssociateReject->
                  peAssociateRejectReason != NULL)
                        DbvFree(puAssociateResult->psAssociateReject->
                          peAssociateRejectReason);
                DbvFree(puAssociateResult->psAssociateReject);

        }
        DbvFree(puAssociateResult);
}

/* 12-2-96 The following function where added by Dan Maskell  */

void DBVFree_DbvGeneralString( DbvGeneralString *psGeneralString )
   {
   if( psGeneralString == NULL)
      return;

   if(psGeneralString->pcValue != NULL )
      DbvFree( psGeneralString->pcValue );

   DbvFree( psGeneralString );
   } /* End DBVFree_DbvGeneralString() */
 
 
void DBVFree_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   if ( psEsIoTargetPart != NULL )
      {
      do_DBVFree_DbvEsIoTargetPart( psEsIoTargetPart );
      DbvFree( psEsIoTargetPart );
      } /* End if */

   } /* End DBVFree_DbvEsIoTargetPart() */ 

void do_DBVFree_DbvEsIoTargetPart( DbvEsIoTargetPart *psEsIoTargetPart )
   {

   /* psItemRequest */
   DBVFreeDbvASN1External( psEsIoTargetPart->psItemRequest );

   /* psStatusOrErrorReport */
   DBVFreeDbvASN1External( psEsIoTargetPart->psStatusOrErrorReport );

   /* peAuxiliaryStatus */
   DBVFree_DbvEsIoAuxiliaryStatus( psEsIoTargetPart->peAuxiliaryStatus );

   } /* End do_DBVFree_DbvEsIoTargetPart() */ 

void DBVFree_DbvEsIoResultSetItem( DbvEsIoResultSetItem *psEsIoResultSetItem )
   {

   if ( psEsIoResultSetItem == NULL )
      return;

   /* sResultSetId */
   DBVFree_compulsory_DbvInternationalString( &psEsIoResultSetItem->sResultSetId);

   DbvFree( psEsIoResultSetItem );
   } /* End DBVFree_DbvEsIoResultSetItem() */ 

void DBVFree_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   if ( psEsIoOriginPartNotToKeep != NULL )
      {
      do_DBVFree_DbvEsIoOriginPartNotToKeep( psEsIoOriginPartNotToKeep );
      DbvFree( psEsIoOriginPartNotToKeep );
      } /* End if */

   } /* End DBVFree_DbvEsIoOriginPartNotToKeep() */ 

void DBVFree_DbvEsIoCreditCardInfo( DbvEsIoCreditCardInfo *psEsIoCreditCardInfo)
   {
   if( psEsIoCreditCardInfo == NULL)
      return;

   /* sNameOnCard */
   DBVFree_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sNameOnCard );

   /* sExpirationDate */
   DBVFree_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->
					   sExpirationDate );

   /* sCardNumber */
   DBVFree_compulsory_DbvInternationalString( &psEsIoCreditCardInfo->sCardNumber );

   DbvFree( psEsIoCreditCardInfo );
   } /* End DBVFree_DbvEsIoCreditCardInfo() */
 
 
void do_DBVFree_DbvEsIoOriginPartNotToKeep( 
		DbvEsIoOriginPartNotToKeep *psEsIoOriginPartNotToKeep )
   {

   /* psResultSetItem */
   DBVFree_DbvEsIoResultSetItem( psEsIoOriginPartNotToKeep->psResultSetItem );

   /* psItemRequest */
   DBVFreeDbvASN1External( psEsIoOriginPartNotToKeep->psItemRequest );

   } /* End do_DBVFree_DbvEsIoOriginPartNotToKeep() */ 

void DBVFree_DbvIllString( DbvIllString *psIllString )
   {

   if ( psIllString != NULL )
      {
      do_DBVFree_DbvIllString( psIllString );
      DbvFree( psIllString );
      } /* End if */

   } /* End DBVFree_DbvIllString() */ 


void do_DBVFree_DbvIllString( DbvIllString *psIllString )
   {

   switch( psIllString->eStringType)
      {
      case ST_GeneralString:
	 DBVFree_DbvGeneralString( psIllString->uString.psGeneralString);
	 break;

      case ST_EdifactString:
	 DBVFree_DbvVisibleString( psIllString->uString.pcVisibleString);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_DBVFree_DbvIllString() */ 

void DBVFree_DbvPersonOrInstitutionSymbol( DbvPersonOrInstitutionSymbol 
					*psPersonOrInstitutionSymbol )
   {
   if ( psPersonOrInstitutionSymbol == NULL )
      return;

   /* sSymbol */
   DBVFree_compulsory_DbvIllString( &psPersonOrInstitutionSymbol->sSymbol );

   DbvFree( psPersonOrInstitutionSymbol );
   } /* End DBVFree_DbvPersonOrInstitutionSymbol() */ 

void DBVFree_DbvNameOfPersonOrInstitution( DbvNameOfPersonOrInstitution 
					*psNameOfPersonOrInstitution )
   {
   if ( psNameOfPersonOrInstitution == NULL )
      return;

   /* sName */
   DBVFree_compulsory_DbvIllString( &psNameOfPersonOrInstitution->sName );

   DbvFree( psNameOfPersonOrInstitution );
   } /* End DBVFree_DbvNameOfPersonOrInstitution() */ 

void DBVFree_DbvSystemId( DbvSystemId *psSystemId )
   {
   if ( psSystemId != NULL )
      {
      do_DBVFree_DbvSystemId( psSystemId );
      DbvFree( psSystemId );
      } /* End if */

   } /* End DBVFree_DbvSystemId() */ 

void do_DBVFree_DbvSystemId( DbvSystemId *psSystemId )
   {
   /* psSymbol */
   DBVFree_DbvPersonOrInstitutionSymbol( psSystemId->psSymbol );

   /* psName */
   DBVFree_DbvNameOfPersonOrInstitution( psSystemId->psName );

   } /* End do_DBVFree_DbvSystemId() */ 

void DBVFree_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   if ( psTransactionId != NULL )
      {
      do_DBVFree_DbvTransactionId( psTransactionId );
      DbvFree( psTransactionId );
      } /* End if */

   } /* End DBVFree_DbvTransactionId() */ 

void do_DBVFree_DbvTransactionId( DbvTransactionId *psTransactionId )
   {

   /* psInitialRequesterId */
   DBVFree_DbvSystemId( psTransactionId->psInitialRequesterId );

   /* sTransactionGroupQualifier */
   DBVFree_compulsory_DbvIllString( &psTransactionId->sTransactionGroupQualifier );

   /* sTransactionQualifier */
   DBVFree_compulsory_DbvIllString( &psTransactionId->sTransactionQualifier );

   /* psSubTransactionQualifier */
   DBVFree_DbvIllString( psTransactionId->psSubTransactionQualifier );

   } /* End do_DBVFree_DbvTransactionId() */ 

void DBVFree_DbvDateTime( DbvDateTime *psDateTime )
   {

   if ( psDateTime != NULL )
      {
      do_DBVFree_DbvDateTime(  psDateTime );
      DbvFree( psDateTime );
      } /* End if */
   } /* End DBVFree_DbvDateTime() */ 

void do_DBVFree_DbvDateTime( DbvDateTime *psDateTime )
   {

   /* pcDate */
   DBVFree_DbvVisibleString( psDateTime->pcDate );

   /* pcTime */
   DBVFree_DbvVisibleString( psDateTime->pcTime );

   } /* End do_DBVFree_DbvDateTime() */ 
void DBVFree_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   if ( psServiceDateTime != NULL )
      {
      do_DBVFree_DbvServiceDateTime( psServiceDateTime );
      DbvFree( psServiceDateTime );
      } /* End if */

   } /* End DBVFree_DbvServiceDateTime() */ 

void do_DBVFree_DbvServiceDateTime( DbvServiceDateTime *psServiceDateTime )
   {

   /* sThisService */
   DBVFree_compulsory_DbvDateTime( &psServiceDateTime->sThisService );

   /* psOriginalService */
   DBVFree_DbvDateTime( psServiceDateTime->psOriginalService );

   } /* End do_DBVFree_DbvServiceDateTime() */ 


void DBVFree_DbvPostalAddress( DbvPostalAddress *psPostalAddress )
   {

   if ( psPostalAddress == NULL )
      return;

   /* psNameOfPersonOrInstitution */
   DBVFree_DbvNameOfPersonOrInstitution( psPostalAddress->
				      psNameOfPersonOrInstitution );

   /* psExtendedPostalDeliveryAddress */
   DBVFree_DbvIllString( psPostalAddress->psExtendedPostalDeliveryAddress );

   /* psStreetAndNumber */
   DBVFree_DbvIllString( psPostalAddress->psStreetAndNumber );

   /* psPostOfficeBox */
   DBVFree_DbvIllString( psPostalAddress->psPostOfficeBox );

   /* psCity */
   DBVFree_DbvIllString( psPostalAddress->psCity );

   /* psRegion */
   DBVFree_DbvIllString( psPostalAddress->psRegion );

   /* psCountry */
   DBVFree_DbvIllString( psPostalAddress->psCountry );

   /* psPostalCode */
   DBVFree_DbvIllString( psPostalAddress->psPostalCode );

   DbvFree( psPostalAddress );
   } /* End DBVFree_DbvPostalAddress() */ 

void DBVFree_DbvSystemAddress( DbvSystemAddress *psSystemAddress )
   {

   if ( psSystemAddress == NULL )
      return;

   /* psTelecomServiceIdentifier  */
   DBVFree_DbvIllString( psSystemAddress->psTelecomServiceIdentifier );

   /* psTelecomServiceAddress */
   DBVFree_DbvIllString( psSystemAddress->psTelecomServiceAddress );

   DbvFree( psSystemAddress );
   } /* End DBVFree_DbvSystemAddress() */ 

void DBVFree_DbvDeliveryAddress( DbvDeliveryAddress *psDeliveryAddress )
   {

   if ( psDeliveryAddress == NULL )
      return;

   /* psPostalAddress */
   DBVFree_DbvPostalAddress( psDeliveryAddress->psPostalAddress );

   /* psElectronicAddress */
   DBVFree_DbvSystemAddress( psDeliveryAddress->psElectronicAddress);

   DbvFree( psDeliveryAddress );
   } /* End DBVFree_DbvDeliveryAddress() */ 

void DBVFree_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   if ( psElDeliveryDetails != NULL )
      {
      do_DBVFree_DbvElDeliveryDetails( psElDeliveryDetails );
      DbvFree( psElDeliveryDetails );
      } /* End if */

   } /* End DBVFree_DbvElDeliveryDetails() */ 

void do_DBVFree_DbvElDeliveryDetails( DbvElDeliveryDetails *psElDeliveryDetails )
   {

   switch( psElDeliveryDetails->eDeliveryDetailsType )
      {
      case DDT_SystemAddress:
	 DBVFree_DbvSystemAddress( psElDeliveryDetails->
				uDeliveryDetails.psSystemAddress);
	 break;

      case DDT_SystemId:
	 DBVFree_DbvSystemId( psElDeliveryDetails->
			   uDeliveryDetails.psSystemId);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End do_DBVFree_DbvElDeliveryDetails() */ 

void DBVFree_DbvElDeliveryParameter( 
			DbvElDeliveryParameter *psElDeliveryParameter )
   {

   if ( psElDeliveryParameter == NULL )
      return;

   /* pcDeliveryMode */
   DBVFreeDbvObjectIdentifier( psElDeliveryParameter->pcDeliveryMode );

   /* psDummy */
   if ( psElDeliveryParameter->puDeliveryParameter )
      {
/*
      DBVFreeDbvASN1External( psElDeliveryParameter->
			   puDeliveryParameter->psDummy );
*/

      /* puDeliveryParameter */
      DbvFree( psElDeliveryParameter->puDeliveryParameter );
      } /* End if */

   DbvFree( psElDeliveryParameter );
   } /* End DBVFree_DbvElDeliveryParameter() */ 


void DBVFree_DbvDocumentTypeParameter( 
			DbvDocumentTypeParameter *psDocumentTypeParameter )
   {

   if ( psDocumentTypeParameter == NULL )
      return;

   /* pcDeliveryMode */
   DBVFreeDbvObjectIdentifier( psDocumentTypeParameter->pcDocumentTypeId );

   /* psDummy */
   if ( psDocumentTypeParameter->puDocumentTypeParameter)
      {
      DBVFreeDbvASN1External( psDocumentTypeParameter->
			   puDocumentTypeParameter->psDummy );

      /* puDeliveryParameter */
      DbvFree( psDocumentTypeParameter->puDocumentTypeParameter);
      } /* End if */

   DbvFree( psDocumentTypeParameter );
   } /* End DBVFree_DbvDocumentTypeParameter() */ 


void DBVFree_DbvElectronicDeliveryService( 
		DbvElectronicDeliveryService *psElectronicDeliveryService )
   {

   if ( psElectronicDeliveryService == NULL )
      return;

   /* psDeliveryParameter */
   DBVFree_DbvElDeliveryParameter( psElectronicDeliveryService->
				psDeliveryParameter);

   /* psDocumentTypeParameter */
   DBVFree_DbvDocumentTypeParameter( psElectronicDeliveryService->
				  psDocumentTypeParameter );
   /* psDeliveryDescription */
   DBVFree_DbvIllString( psElectronicDeliveryService->psDeliveryDescription );

   /* sDeliveryDetails */
   DBVFree_compulsory_DbvElDeliveryDetails( &psElectronicDeliveryService->
					 sDeliveryDetails );


   /* psNameOrCode */
   DBVFree_DbvIllString( psElectronicDeliveryService->psNameOrCode );

   /* pcDeliveryTime */
   DBVFree_DbvVisibleString( psElectronicDeliveryService->pcDeliveryTime );
   
   DbvFree( psElectronicDeliveryService );
   } /* End DBVFree_DbvElectronicDeliveryService() */ 


void DBVFree_DbvDeliveryService( DbvDeliveryService *psDeliveryService )
   {

   if ( psDeliveryService == NULL )
      return;

   switch( psDeliveryService->eDeliveryServiceType )
      {
      case DST_PhysicalDelivery:
	 DBVFree_DbvIllString( psDeliveryService->
			    uDeliveryService.psPhysicalDelivery);
	 break;

      case DST_ElectronicDelivery:
	 FreeN_DbvElectronicDeliveryService( psDeliveryService->
				uDeliveryService.ppsElectronicDelivery);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psDeliveryService );
   } /* End DBVFree_DbvDeliveryService() */ 

void DBVFree_DbvRequesterOptionalMessages( 
		DbvRequesterOptionalMessages *psRequesterOptionalMessages )
   {

   if ( psRequesterOptionalMessages == NULL )
      return;

   DbvFree( psRequesterOptionalMessages );
   } /* End DBVFree_DbvRequesterOptionalMessages() */ 

void DBVFree_DbvSearchType( DbvSearchType *psSearchType )
   {

   if ( psSearchType == NULL )
      return;

   /* psLevelOfService */
   DBVFree_DbvIllString( psSearchType->psLevelOfService );

   /* pcNeedBeforeDate */
   DBVFree_DbvVisibleString( psSearchType->pcNeedBeforeDate );

   /* peExpiryFlag */
   DBVFree_DbvExpiry( psSearchType->peExpiryFlag );

   /* pcExpiryDate */
   DBVFree_DbvVisibleString( psSearchType->pcExpiryDate );

   DbvFree( psSearchType );
   } /* End DBVFree_DbvSearchType() */ 

void DBVFree_DbvSupplyMediumInfo( DbvSupplyMediumInfo *psSupplyMediumInfo )
   {

   if ( psSupplyMediumInfo == NULL )
      return;

   /* psMediumCharacteristics */
   DBVFree_DbvIllString( psSupplyMediumInfo->psMediumCharacteristics );

   DbvFree( psSupplyMediumInfo );
   } /* End DBVFree_DbvSupplyMediumInfo() */ 

void DBVFree_DbvClientId( DbvClientId *psClientId )
   {

   if ( psClientId == NULL )
      return;

   /* psClientName */
   DBVFree_DbvIllString( psClientId->psClientName);

   /* psClientStatus */
   DBVFree_DbvIllString( psClientId->psClientStatus);

   /* psClientIdentifier */
   DBVFree_DbvIllString( psClientId->psClientIdentifier);

   DbvFree( psClientId );
   } /* End DBVFree_DbvClientId() */ 

void DBVFree_DbvItemId( DbvItemId *psItemId )
   {

   if ( psItemId != NULL )
      {

      /* Free the fields in psItemId */
      do_DBVFree_DbvItemId( psItemId );

      /* Dont attempt a DbvFree on psItemId as psItemId points to a structure
	 allocated at compile time, not one malloced at run time */

      } /* End if */

   } /* End DBVFree_DbvItemId() */ 

void do_DBVFree_DbvItemId( DbvItemId *psItemId )
   {

   /* peItemType */
   DBVFree_DbvItemType( psItemId->peItemType );

   /* peMediumType */
   DBVFree_DbvMediumType( psItemId->peMediumType );

   /* psCallNumber */
   DBVFree_DbvIllString( psItemId->psCallNumber );

   /* psAuthor */
   DBVFree_DbvIllString( psItemId->psAuthor );

   /* psTitle */
   DBVFree_DbvIllString( psItemId->psTitle );

   /* psSubTitle */
   DBVFree_DbvIllString( psItemId->psSubTitle );

   /* psSponsoringBody */
   DBVFree_DbvIllString( psItemId->psSponsoringBody );

   /* psPlaceOfPublication */
   DBVFree_DbvIllString( psItemId->psPlaceOfPublication );

   /* psPublisher */
   DBVFree_DbvIllString( psItemId->psPublisher);

   /* psSeriesTitleNumber */
   DBVFree_DbvIllString( psItemId->psSeriesTitleNumber);

   /* psVolumeIssue */
   DBVFree_DbvIllString( psItemId->psVolumeIssue );

   /* psEdition */
   DBVFree_DbvIllString( psItemId->psEdition );

   /* psPublicationDate */
   DBVFree_DbvIllString( psItemId->psPublicationDate );

   /* psPublicationDateOfComponent */
   DBVFree_DbvIllString( psItemId->psPublicationDateOfComponent );

   /* psAuthorOfArticle */
   DBVFree_DbvIllString( psItemId->psAuthorOfArticle );

   /* psTitleOfArticle */
   DBVFree_DbvIllString( psItemId->psTitleOfArticle );

   /* psPagination */
   DBVFree_DbvIllString( psItemId->psPagination );

   /* psNationalBibliographyNo */
   DBVFreeDbvASN1External( psItemId->psNationalBibliographyNo );

   /* psIsbn */
   DBVFree_DbvIllString( psItemId->psIsbn );

   /* psIssn */
   DBVFree_DbvIllString( psItemId->psIssn );

   /* psSystemNo */
   DBVFreeDbvASN1External( psItemId->psSystemNo );

   /* psAdditionalNoLetters */
   DBVFree_DbvIllString( psItemId->psAdditionalNoLetters );

   /* psVerificationReferenceSource */
   DBVFree_DbvIllString( psItemId->psVerificationReferenceSource );

   } /* End do_DBVFree_DbvItemId() */ 

void DBVFree_DbvAmount( DbvAmount *psAmount )
   {

   if ( psAmount == NULL )
      return;

   /* pcCurrencyCode */
   DBVFree_DbvPrintableString( psAmount->pcCurrencyCode );

   /* pcMonetaryValue */
   DBVFree_DbvPrintableString( psAmount->pcMonetaryValue );

   DbvFree( psAmount );
   } /* End DBVFree_DbvAmount() */ 

void DBVFree_DbvCostInfo( DbvCostInfo *psCostInfo )
   {

   if ( psCostInfo == NULL )
      return;

   /* psAccountNumber */
   DBVFree_DbvIllString( psCostInfo->psAccountNumber );

   /* psMaximumCost */
   DBVFree_DbvAmount( psCostInfo->psMaximumCost );

   /* pfReciprocalAgreement */
   DBVFree_DbvBool( psCostInfo->pfReciprocalAgreement );

   /* pfWillPayFee */
   DBVFree_DbvBool( psCostInfo->pfWillPayFee );

   /* pfPaymentProvided */
   DBVFree_DbvBool( psCostInfo->pfPaymentProvided );

   DbvFree( psCostInfo );
   } /* End DBVFree_DbvCostInfo() */ 

void DBVFree_DbvSendToList( DbvSendToList *psSendToList )
   {

   if ( psSendToList == NULL )
      return;

   /* sSystemId */
   DBVFree_compulsory_DbvSystemId( &psSendToList->sSystemId );

   /* psAccountNumber */
   DBVFree_DbvIllString( psSendToList->psAccountNumber );

   /* psSystemAddress */
   DBVFree_DbvSystemAddress( psSendToList->psSystemAddress );

   DbvFree( psSendToList );
   } /* End DBVFree_DbvSendToList() */ 

void DBVFree_DbvThirdPartyInfo( DbvThirdPartyInfo *psThirdPartyInfo )
   {

   if ( psThirdPartyInfo == NULL )
      return;

   /* pfPermissionToForward */
   DBVFree_DbvBool( psThirdPartyInfo->pfPermissionToForward );

   /* pfPermissionToChain */
   DBVFree_DbvBool( psThirdPartyInfo->pfPermissionToChain );

   /* pfPermissionToPartition */
   DBVFree_DbvBool( psThirdPartyInfo->pfPermissionToPartition );

   /* pfPermissionToChangeSendToList */
   DBVFree_DbvBool( psThirdPartyInfo->pfPermissionToChangeSendToList );

   /* psInitialRequesterAddress */
   DBVFree_DbvSystemAddress( psThirdPartyInfo->psInitialRequesterAddress );

   /* pePreference */
   DBVFree_DbvPreference( psThirdPartyInfo->pePreference );

   /* ppsSendToList */
   FreeN_DbvSendToList( psThirdPartyInfo->ppsSendToList );

   /* ppsAlreadyTriedList */
   FreeN_DbvSystemId( psThirdPartyInfo->ppsAlreadyTriedList );

   DbvFree( psThirdPartyInfo );
   } /* End DBVFree_DbvThirdPartyInfo() */ 


void DBVFree_DbvRequesterGroup( DbvRequesterGroup *peRequesterGroup )
{
   if ( peRequesterGroup == NULL )
      return;

   DbvFree( peRequesterGroup );

} /* End DBVFree_DbvRequesterGroup() */



void DBVFree_DbvAddRequesterInfo( DbvAddRequesterInfo *psAddRequesterInfo )
{
   if ( psAddRequesterInfo == NULL )
      return;

   /* peRequesterGroup */
   DBVFree_DbvRequesterGroup( psAddRequesterInfo->peRequesterGroup );

   /* pcRequesterEmailAddress */
   DBVFree_DbvVisibleString( psAddRequesterInfo->pcRequesterEmailAddress );

   DbvFree( psAddRequesterInfo );
 
} /* End DBVFree_DbvAddRequesterInfo() */


void DBVFree_DbvStatusLevelUser( DbvStatusLevelUser *peStatusLevelUser )
{
   if ( peStatusLevelUser == NULL )
      return;

   DbvFree( peStatusLevelUser );
 
} /* End DBVFree_DbvStatusLevelUser() */


 
void DBVFree_DbvStatusLevelRequester( DbvStatusLevelRequester *peStatusLevelOrigin )
{
   if ( peStatusLevelOrigin == NULL )
      return;

   DbvFree( peStatusLevelOrigin );
 
} /* End DBVFree_DbvStatusLevelRequester() */


 
void DBVFree_DbvAddDeliveryInfo( DbvAddDeliveryInfo *psAddDeliveryInfo )
{
   if ( psAddDeliveryInfo == NULL )
      return;

   /* peStatusLevelUser */
   DBVFree_DbvStatusLevelUser( psAddDeliveryInfo->peStatusLevelUser );

   /* peStatusLevelOrigin */
   DBVFree_DbvStatusLevelRequester( psAddDeliveryInfo->peStatusLevelOrigin );

   /* pcDeliveryNotes */
   DBVFree_DbvVisibleString( psAddDeliveryInfo->pcDeliveryNotes );

   /* pcCountryDeliveryTarget */
   DBVFree_DbvVisibleString( psAddDeliveryInfo->pcCountryDeliveryTarget );

   /* pfAlternativePostalDelivery */
   DBVFree_DbvBool( psAddDeliveryInfo->pfAlternativePostalDelivery );
    
   DbvFree( psAddDeliveryInfo );
 
} /* End DBVFree_DbvAddDeliveryInfo() */



void DBVFree_DbvBillingBankInfo( DbvBillingBankInfo *psBillingBankInfo)
{
   if ( psBillingBankInfo == NULL )
      return;

   /* pcBankAccountHouse */
   DBVFree_DbvVisibleString( psBillingBankInfo->pcBankAccountHouse );

   /* pcBankAccountRoutingSymbol */
   DBVFree_DbvVisibleString( psBillingBankInfo->pcBankAccountRoutingSymbol );

   /* pcBankAccountNo */
   DBVFree_DbvVisibleString( psBillingBankInfo->pcBankAccountNo );

   /* fBankAuthority ok */

   DbvFree( psBillingBankInfo );

} /* End DBVFree_DbvBillingBankInfo */



void DBVFree_DbvBillingCreditCardInfo( DbvBillingCreditCardInfo *psBillingCreditCardInfo)
{
   if ( psBillingCreditCardInfo == NULL )
      return;

   /* pcCardCompany */
   DBVFree_DbvVisibleString( psBillingCreditCardInfo->pcCardCompany );

   /* pcCardNo */
   DBVFree_DbvVisibleString( psBillingCreditCardInfo->pcCardNo );

   /* pcCardValidFrom */
   DBVFree_DbvVisibleString( psBillingCreditCardInfo->pcCardValidFrom );

   /* pcCardValidThru */
   DBVFree_DbvVisibleString( psBillingCreditCardInfo->pcCardValidThru );


   DbvFree( psBillingCreditCardInfo );

} /* End DBVFree_DbvBillingCreditCardInfo */



void DBVFree_DbvAddBillingInfo( DbvAddBillingInfo *psAddBillingInfo )
{
   if ( psAddBillingInfo == NULL )
      return;

   switch( psAddBillingInfo->eAddBillingType )
      {
      case ABT_BankInfo:
	 DBVFree_DbvBillingBankInfo( psAddBillingInfo->
			    uAddBillingInfo.psBillingBankInfo);
	 break;

      case ABT_CreditCardInfo:
	 DBVFree_DbvBillingCreditCardInfo( psAddBillingInfo->
				uAddBillingInfo.psBillingCreditCardInfo);
	 break;

      case ABT_InvoiceType:

         /* nothing required here because no pointer */
	 break;

      case ABT_PrepaidType:
         DBVFree_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcPrepaidId); 
	 break;

      case ABT_DepositType:
         DBVFree_DbvVisibleString( psAddBillingInfo->
                                uAddBillingInfo.pcDepositId); 
	 break;


      default:
	 break;
      } /* End switch */

   DbvFree( psAddBillingInfo );
 
} /* End DBVFree_DbvAddBillingInfo() */



 


void DBVFree_DbvSubitoAdditionalInfo( DbvSubitoAdditionalInfo *psAddIllReqInfo )
{
   if ( psAddIllReqInfo == NULL )
      return;

   /* pfCustomerInfoToKeep */
   DBVFree_DbvBool( psAddIllReqInfo->pfCustomerInfoToKeep );

   /* psAddRequesterInfo */
   DBVFree_DbvAddRequesterInfo( psAddIllReqInfo->psAddRequesterInfo );

 
   /* psAddDeliveryInfo */
   DBVFree_DbvAddDeliveryInfo( psAddIllReqInfo->psAddDeliveryInfo );

 
   /* psAddBillingInfo */
   DBVFree_DbvAddBillingInfo( psAddIllReqInfo->psAddBillingInfo );

 

   DbvFree( psAddIllReqInfo );
   } /* End DBVFree_DbvSubitoAdditionalInfo() */ 


void DBVFree_DbvExtension( DbvExtension *psExtension )
   {

   if ( psExtension == NULL )
      return;

   /* pfCritical */
   DBVFree_DbvBool( psExtension->pfCritical );

   /* uItem.psAddIllReqInfo */

   DBVFree_DbvSubitoAdditionalInfo( psExtension->uItem.psAddIllReqInfo );

   DbvFree( psExtension );
   } /* End DBVFree_DbvExtension() */ 


void DBVFree_DbvIllRequest( DbvIllRequest *psIllRequest )
   {

   if ( psIllRequest == NULL )
      return;

   /* eProtocolVersion : No Free required */
   /* sTransactionId */
   DBVFree_compulsory_DbvTransactionId( &psIllRequest->sTransactionId );

   /* sServiceDateTime */
   DBVFree_compulsory_DbvServiceDateTime( &psIllRequest->sServiceDateTime );

   /* psRequesterId */
   DBVFree_DbvSystemId( psIllRequest->psRequesterId );

   /* psResponderId */
   DBVFree_DbvSystemId( psIllRequest->psResponderId );

   /* peTransactionType */
   DBVFree_DbvTransactionType( psIllRequest->peTransactionType );

   /* psDeliveryAddress */
   DBVFree_DbvDeliveryAddress( psIllRequest->psDeliveryAddress );

   /* psDeliveryService */
   DBVFree_DbvDeliveryService( psIllRequest->psDeliveryService );

   /* psBillingAddress */
   DBVFree_DbvDeliveryAddress( psIllRequest->psBillingAddress );

   /* ppeServiceType */
   FreeN_DbvServiceType( psIllRequest->ppeServiceType );

   /* psResponderSpecificService */
   DBVFreeDbvASN1External( psIllRequest->psResponderSpecificService );

   /* sRequesterOptionalMessages : This is currently defined as a    */
   /* macro which converts to a null as the stucture has no pointers */
   DBVFree_compulsory_DbvRequesterOptionalMessages( 
			&psIllRequest->sRequesterOptionalMessages );

   /* psSearchType */
   DBVFree_DbvSearchType( psIllRequest->psSearchType );

   /* ppsSupplyMediumInfo */
   FreeN_DbvSupplyMediumInfo( psIllRequest->ppsSupplyMediumInfo );

   /* pePlaceOnHold */
   DBVFree_DbvPlaceOnHold( psIllRequest->pePlaceOnHold );

   /* psClientId */
   DBVFree_DbvClientId( psIllRequest->psClientId );

   /* sItemId */
   DBVFree_compulsory_DbvItemId( &psIllRequest->sItemId );

   /* ppsSupplementalItemDescription */
   FreeN_DbvASN1External( psIllRequest->ppsSupplementalItemDescription );

   /* psCostInfo */
   DBVFree_DbvCostInfo( psIllRequest->psCostInfo );

   /* psCopyrightCompliance */
   DBVFree_DbvIllString( psIllRequest->psCopyrightCompliance );

   /* psThirdPartyInfo */
   DBVFree_DbvThirdPartyInfo( psIllRequest->psThirdPartyInfo );

   /* pfRetry */
   DBVFree_DbvBool( psIllRequest->pfRetry );

   /* pfForward */
   DBVFree_DbvBool( psIllRequest->pfForward );

   /* psRequesterNote */
   DBVFree_DbvIllString( psIllRequest->psRequesterNote );

   /* psForwardNote */
   DBVFree_DbvIllString( psIllRequest->psForwardNote );

   /* ppsRequestExtension */
   FreeN_DbvExtension( psIllRequest->ppsRequestExtension );

   DbvFree( psIllRequest );
   } /* End DBVFree_DbvIllRequest() */ 


void DBVFree_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   if ( psHistoryReport != NULL )
      {
      do_DBVFree_DbvHistoryReport( psHistoryReport );
      DbvFree( psHistoryReport );
      } /* End if */

   } /* End DBVFree_DbvHistoryReport() */ 

void do_DBVFree_DbvHistoryReport( DbvHistoryReport *psHistoryReport )
   {

   /* pcDateRequested  */
   DBVFree_DbvVisibleString( psHistoryReport->pcDateRequested );

   /* psAuthor */
   DBVFree_DbvIllString( psHistoryReport->psAuthor );

   /* psTitle */
   DBVFree_DbvIllString( psHistoryReport->psTitle );

   /* psAuthorOfArticle */
   DBVFree_DbvIllString( psHistoryReport->psAuthorOfArticle );

   /* psTitleOfArticle */
   DBVFree_DbvIllString( psHistoryReport->psTitleOfArticle );

   /* pcDateOfLastTransition */
   DBVFree_DbvVisibleString( psHistoryReport->pcDateOfLastTransition );

   /* pcDateOfMostRecentService */
   DBVFree_DbvVisibleString( psHistoryReport->pcDateOfMostRecentService );

   /* sSystemId */
   DBVFree_compulsory_DbvSystemId( &psHistoryReport->sSystemId );

   /* peShippedServiceType */
   DBVFree_DbvShippedServiceType( psHistoryReport->peShippedServiceType );

   /* peTransactionResults */
   DBVFree_DbvTransactionResults( psHistoryReport->peTransactionResults );

   /* psMostRecentServiceNote */
   DBVFree_DbvIllString( psHistoryReport->psMostRecentServiceNote );

   } /* End do_DBVFree_DbvHistoryReport() */ 

void DBVFree_DbvStateTransitionProhibited( 
		DbvStateTransitionProhibited *psStateTransitionProhibited )
   {

   if ( psStateTransitionProhibited == NULL )
      return;

   DbvFree( psStateTransitionProhibited );
   } /* End DBVFree_DbvStateTransitionProhibited() */ 


void DBVFree_DbvAlreadyForwarded( DbvAlreadyForwarded *psAlreadyForwarded )
   {

   if ( psAlreadyForwarded == NULL )
      return;

   /* sResponderId */
   DBVFree_compulsory_DbvSystemId( &psAlreadyForwarded->sResponderId );

   /* psResponderAddress */
   DBVFree_DbvSystemAddress( psAlreadyForwarded->psResponderAddress );


   DbvFree( psAlreadyForwarded );
   } /* End DBVFree_DbvAlreadyForwarded() */ 


void DBVFree_DbvProviderErrorReport( 
			DbvProviderErrorReport *psProviderErrorReport )
   {

   if ( psProviderErrorReport == NULL )
      return;

   /* uProviderErrorReport */

   switch ( psProviderErrorReport->eReportType )
      {
      case PER_GeneralProblem:
      case PER_TransactionIdProblem:
	 break;

      case PER_StateTransitionProhibited:
	 DBVFree_DbvStateTransitionProhibited( psProviderErrorReport->
					    uProviderErrorReport.
					    psStateTransitionProblem );
         break;

      default:
	 break;

      } /* End if */
   DbvFree( psProviderErrorReport );
   } /* End DBVFree_DbvProviderErrorReport() */ 

void DBVFree_DbvUserErrorReport( DbvUserErrorReport *psUserErrorReport )
   {

   if ( psUserErrorReport == NULL )
      return;

   /* uUserErrorReport */

   switch ( psUserErrorReport->eReportType )
      {
      case UER_AlreadyForwarded:
	 DBVFree_DbvAlreadyForwarded( psUserErrorReport->
				   uUserErrorReport.
				   psAlreadyForwarded );
	 break;
      case UER_IntermediaryProblem:
	 break;

      case UER_SecurityProblem:
	 DBVFree_DbvIllString( psUserErrorReport->
			    uUserErrorReport.
			    psSecurityProblem );
	 break;

      case UER_UnableToPerform:
      default:
	 break;

      } /* End if */
   DbvFree( psUserErrorReport );
   } /* End DBVFree_DbvUserErrorReport() */ 

void DBVFree_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   if ( psUserOrProviderErrorReport != NULL )
      {
      do_DBVFree_DbvUserOrProviderErrorReport( psUserOrProviderErrorReport );
      DbvFree( psUserOrProviderErrorReport );
      } /* End if */

   } /* End DBVFree_DbvUserOrProviderErrorReport() */ 

void do_DBVFree_DbvUserOrProviderErrorReport( 
		DbvUserOrProviderErrorReport *psUserOrProviderErrorReport )
   {

   /* uReport */

   switch ( psUserOrProviderErrorReport->eReportSource )
      {
      case RS_Provider:
	 DBVFree_DbvProviderErrorReport( psUserOrProviderErrorReport->
				      uReport.psProviderErrorReport);
	 break;

      case RS_User:
	 DBVFree_DbvUserErrorReport( psUserOrProviderErrorReport->
			    uReport.psUserErrorReport );
	 break;

      default:
	 break;

      } /* End if */

   } /* End do_DBVFree_DbvUserOrProviderErrorReport() */ 

void DBVFree_DbvErrorReport( DbvErrorReport *psErrorReport )
   {

   if ( psErrorReport == NULL )
      return;

   /* sCorrelationInfo */
   DBVFree_compulsory_DbvIllString( &psErrorReport->sCorrelationInfo );

   /* sReport */
   DBVFree_compulsory_DbvUserOrProviderErrorReport(  &psErrorReport->sReport );

   DbvFree( psErrorReport );
   } /* End DBVFree_DbvErrorReport() */ 

void DBVFree_DbvStatusReport( DbvStatusReport *psStatusReport )
   {

   if ( psStatusReport == NULL )
      return;

   /* sUserStatusReport */
   DBVFree_compulsory_DbvHistoryReport( &psStatusReport->sUserStatusReport );

   DbvFree( psStatusReport );
   } /* End DBVFree_DbvStatusReport() */ 

void DBVFree_DbvStatusOrErrorReport( 
			DbvStatusOrErrorReport *psStatusOrErrorReport )
   {

   if ( psStatusOrErrorReport == NULL )
      return;

   /* eProtocolVersion */

   /* sTransactionId */
   DBVFree_compulsory_DbvTransactionId( &psStatusOrErrorReport->sTransactionId );

   /* sServiceDateTime */
   DBVFree_compulsory_DbvServiceDateTime( 
				&psStatusOrErrorReport->sServiceDateTime );
   /* psRequesterId */
   DBVFree_DbvSystemId( psStatusOrErrorReport->psRequesterId );

   /* psResponderId */
   DBVFree_DbvSystemId( psStatusOrErrorReport->psResponderId );

   /* peReasonNoReport */
   DBVFree_DbvReasonNoReport( psStatusOrErrorReport->peReasonNoReport );

   /* psStatusReport */
   DBVFree_DbvStatusReport( psStatusOrErrorReport->psStatusReport );

   /* psErrorReport */
   DBVFree_DbvErrorReport( psStatusOrErrorReport->psErrorReport );

   /* psNote */
   DBVFree_DbvIllString( psStatusOrErrorReport->psNote );

   /* ppsStOrErReportExtension */
   FreeN_DbvExtension( psStatusOrErrorReport->ppsStOrErReportExtension );

   DbvFree( psStatusOrErrorReport );
   } /* End DBVFree_DbvStatusOrErrorReport() */ 


void DBVFree_DbvIllPdu( DbvIllPdu *psIllPdu )
   {

   if ( psIllPdu == NULL )
      return;

   /* uIllPdu */

   switch( psIllPdu->eIllPduType )
      {
      case ILL_IllRequest:
	 DBVFree_DbvIllRequest( psIllPdu->uIllPdu.psIllRequest );
	 break;

      case ILL_StatusOrErrorReport:
	 DBVFree_DbvStatusOrErrorReport( psIllPdu->
				      uIllPdu.psStatusOrErrorReport);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psIllPdu );
   } /* End DBVFree_DbvIllPdu() */ 

void DBVFree_DbvBool( DbvBool *peBool )
   {

   if ( peBool == NULL )
      return;

   DbvFree( peBool );
   } /* End DBVFree_DbvBool() */ 

void DBVFree_DbvPreference( DbvPreference *pePreference )
   {

   if ( pePreference == NULL )
      return;

   DbvFree( pePreference );
   } /* End DBVFree_DbvPreference() */ 

void DBVFree_DbvTransactionType( DbvTransactionType *peTransactionType )
   {

   if ( peTransactionType == NULL )
      return;

   DbvFree( peTransactionType );
   } /* End DBVFree_DbvTransactionType() */ 

void DBVFree_DbvReasonNoReport( DbvReasonNoReport *peReasonNoReport )
   {

   if ( peReasonNoReport == NULL )
      return;

   DbvFree( peReasonNoReport );
   } /* End DBVFree_DbvReasonNoReport() */ 

void DBVFree_DbvPrintableString( DbvPrintableString pcPrintableString )
   {

   if ( pcPrintableString == NULL )
      return;

   DbvFree( pcPrintableString );
   } /* End DBVFree_DbvPrintableString() */ 

void DBVFree_DbvItemType( DbvItemType *peItemType )
   {

   if ( peItemType == NULL )
      return;

   DbvFree( peItemType );
   } /* End DBVFree_DbvItemType() */ 

void DBVFree_DbvExpiry( DbvExpiry *peExpiry )
   {

   if ( peExpiry == NULL )
      return;

   DbvFree( peExpiry );
   } /* End DBVFree_DbvExpiry() */ 

void DBVFree_DbvMediumType( DbvMediumType *peMediumType )
   {

   if ( peMediumType == NULL )
      return;

   DbvFree( peMediumType );
   } /* End DBVFree_DbvMediumType() */ 

void DBVFree_DbvServiceType( DbvServiceType *peServiceType )
   {

   if ( peServiceType == NULL )
      return;

   DbvFree( peServiceType );
   } /* End DBVFree_DbvServiceType() */ 

void DBVFree_DbvPlaceOnHold( DbvPlaceOnHold *pePlaceOnHold )
   {

   if ( pePlaceOnHold == NULL )
      return;

   DbvFree( pePlaceOnHold );
   } /* End DBVFree_DbvPlaceOnHold() */ 

void DBVFree_DbvShippedServiceType( DbvShippedServiceType *peShippedServiceType )
   {

   if ( peShippedServiceType == NULL )
      return;

   DbvFree( peShippedServiceType );
   } /* End DBVFree_DbvShippedServiceType() */ 

void DBVFree_DbvTransactionResults( DbvTransactionResults *peTransactionResults )
   {

   if ( peTransactionResults == NULL )
      return;

   DbvFree( peTransactionResults );
   } /* End DBVFree_DbvTransactionResults() */ 

void DBVFree_DbvEsIoAuxiliaryStatus( 
			DbvEsIoAuxiliaryStatus *peEsIoAuxiliaryStatus )
   {

   if ( peEsIoAuxiliaryStatus == NULL )
      return;

   DbvFree( peEsIoAuxiliaryStatus );
   } /* End DBVFree_DbvEsIoAuxiliaryStatus() */ 

void DBVFree_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod == NULL )
      {
      do_DBVFree_DbvEsIoPaymentMethod( psEsIoPaymentMethod );
      DbvFree( psEsIoPaymentMethod );
      } /* End if */

   } /* End DBVFree_DbvEsIoPaymentMethod() */ 


void do_DBVFree_DbvEsIoPaymentMethod( DbvEsIoPaymentMethod *psEsIoPaymentMethod )
   {

   if ( psEsIoPaymentMethod->puPaymentMethod )
      {
      switch( psEsIoPaymentMethod->ePaymentMethodType )
         {
         case IOPM_CreditCard:
	    DBVFree_DbvEsIoCreditCardInfo( psEsIoPaymentMethod->
				       puPaymentMethod->psCreditCardInfo );
	    break;
   
         case IOPM_PrivateNotKnown:
	    DBVFreeDbvASN1External( psEsIoPaymentMethod->
				 puPaymentMethod->psPrivateNotKnown );
	    break;
   
	 case IOPM_BillInvoice:
	 case IOPM_Prepay:
	 case IOPM_DepositAccount:
	 case IOPM_CardInfoPrevSupplied:
	 case IOPM_PrivateKnown:
	 case IOPM_Invalid:
         default:
	    break;
         } /* End switch */
      } /* End if */

   } /* End do_DBVFree_DbvEsIoPaymentMethod() */ 


void DBVFree_DbvEsIoAdditionalBilling( 
			DbvEsIoAdditionalBilling *psEsIoAdditionalBilling )
   {

   if ( psEsIoAdditionalBilling == NULL )
      return;

   /* sPaymentMethod */

   DBVFree_compulsory_DbvEsIoPaymentMethod( 
				&psEsIoAdditionalBilling->sPaymentMethod );
   /* psCustomerReference */
   if ( psEsIoAdditionalBilling->psCustomerReference != NULL )
      DBVFreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerReference);

   /* psCustomerPONumber */
   if ( psEsIoAdditionalBilling->psCustomerPONumber != NULL )
      DBVFreeDbvInternationalString( psEsIoAdditionalBilling->psCustomerPONumber);

   DbvFree( psEsIoAdditionalBilling );
   } /* End DBVFree_DbvEsIoAdditionalBilling() */ 


void DBVFree_DbvEsIoContact( DbvEsIoContact *psEsIoContact )
   {

   if ( psEsIoContact == NULL )
      return;

   /* psName */
   if ( psEsIoContact->psName != NULL )
      DBVFreeDbvInternationalString( psEsIoContact->psName );

   /* psPhone */
   if ( psEsIoContact->psPhone != NULL )
      DBVFreeDbvInternationalString( psEsIoContact->psPhone );

   /* psEmail */
   if ( psEsIoContact->psEmail != NULL )
      DBVFreeDbvInternationalString( psEsIoContact->psEmail );

   DbvFree( psEsIoContact );
   } /* End DBVFree_DbvEsIoContact() */ 

void DBVFree_DbvEsIoOriginPartToKeep( 
			DbvEsIoOriginPartToKeep *psEsIoOriginPartToKeep )
   {

   if ( psEsIoOriginPartToKeep == NULL )
      return;

   /* psSupplementalDescription  */
   DBVFreeDbvASN1External( psEsIoOriginPartToKeep->psSupplementalDescription );

   /* psContact */
   DBVFree_DbvEsIoContact( psEsIoOriginPartToKeep->psContact );

   /* psAdditionalBilling */
   DBVFree_DbvEsIoAdditionalBilling( psEsIoOriginPartToKeep->psAdditionalBilling );

   DbvFree( psEsIoOriginPartToKeep );
   } /* End DBVFree_DbvEsIoOriginPartToKeep() */ 

void DBVFree_DbvEsIoRequest( DbvEsIoRequest *psEsIoRequest )
   {

   if ( psEsIoRequest == NULL )
      return;

   /* psToKeep */
   DBVFree_DbvEsIoOriginPartToKeep( psEsIoRequest->psToKeep );

   /* sNotToKeep */
   DBVFree_compulsory_DbvEsIoOriginPartNotToKeep( &psEsIoRequest->sNotToKeep );

   DbvFree( psEsIoRequest );
   } /* End DBVFree_DbvEsIoRequest() */ 

void DBVFree_DbvEsIoTaskPackage( DbvEsIoTaskPackage *psEsIoTaskPackage )
   {

   if ( psEsIoTaskPackage == NULL )
      return;

   /* psOriginPart */
   DBVFree_DbvEsIoOriginPartToKeep( psEsIoTaskPackage->psOriginPart );

   /* sTargetPart */
   DBVFree_compulsory_DbvEsIoTargetPart( &psEsIoTaskPackage->sTargetPart );

   DbvFree( psEsIoTaskPackage );
   } /* End DBVFree_DbvEsIoTaskPackage() */ 

void DBVFree_DbvItemOrderExtendedService( 
		DbvItemOrderExtendedService *psItemOrderExtendedService )
   {

   if ( psItemOrderExtendedService == NULL )
      return;

   /* uItemOrderReqOrRsp */

   switch( psItemOrderExtendedService->eItemOrderReqRsp )
      {
      case IOROR_Request:
	 DBVFree_DbvEsIoRequest( psItemOrderExtendedService->
			      uItemOrderReqOrRsp.psItemOrderRequest);
	 break;

      case IOROR_Response:
	 DBVFree_DbvEsIoTaskPackage( psItemOrderExtendedService->
				  uItemOrderReqOrRsp.psItemOrderResponse);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psItemOrderExtendedService );
   } /* End DBVFree_DbvItemOrderExtendedService() */ 

 

void DBVFree_DbvCommonInfo( DbvCommonInfo *psCommonInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psCommonInfo == NULL )
      return;

   /* pcDateAdded */
   DBVFree_DbvGeneralizedTime( psCommonInfo->pcDateAdded );

   /* pcDateChanged */
   DBVFree_DbvGeneralizedTime( psCommonInfo->pcDateChanged );

   /* pcExpiry */
   DBVFree_DbvGeneralizedTime( psCommonInfo->pcExpiry );

   /* psHumanStringLanguage */
   DBVFree_DbvLanguageCode( psCommonInfo->psHumanStringLanguage );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psCommonInfo->ppsOtherInformation );

   DbvFree( psCommonInfo );
   } /* End DBVFree_DbvCommonInfo() */ 

void DBVFree_DbvHumanString( DbvHumanString *psHumanString )
   {
#ifdef DEBUGZ

#endif

   if ( psHumanString == NULL )
      return;

   /* psLanguage */
   DBVFree_DbvLanguageCode( psHumanString->psLanguage );

   /* sText */
   DBVFree_compulsory_DbvInternationalString( &psHumanString->sText );

   DbvFree( psHumanString );
   } /* End DBVFree_DbvHumanString() */ 

void DBVFree_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUGZ

#endif

   if ( psBodyType != NULL )
      {
      do_DBVFree_DbvBodyType( psBodyType );
      DbvFree( psBodyType );
      } /* End if */

   } /* End DBVFree_DbvBodyType() */ 

void do_DBVFree_DbvBodyType( DbvBodyType *psBodyType )
   {
#ifdef DEBUGZ

#endif

   if ( psBodyType == NULL )
      return;

   /* uBodyTypeType */

   switch( psBodyType->eBodyTypeType )
      {
      case BTT_IanaType:
	 DBVFreeDbvInternationalString( psBodyType->uBodyType.psIanaType );
	 break;

      case BTT_Z3950Type:
	 DBVFreeDbvInternationalString( psBodyType->uBodyType.psZ3950Type );
	 break;

      case BTT_OtherType:
	 DBVFreeDbvInternationalString( psBodyType->uBodyType.psOtherType );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVFree_DbvBodyType() */ 

void DBVFree_DbvIconObject( DbvIconObject *psIconObject )
   {
#ifdef DEBUGZ

#endif

   if ( psIconObject == NULL )
      return;

   /* sBodyType */
   DBVFree_compulsory_DbvBodyType( &psIconObject->sBodyType );

   /* sContent */
   DBVFree_compulsory_DbvOctetString( &psIconObject->sContent );

   DbvFree( psIconObject );
   } /* End DBVFree_DbvIconObject() */ 

void DBVFree_DbvContactInfo( DbvContactInfo *psContactInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psContactInfo == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psContactInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psContactInfo->ppsDescription );

   /* ppsAddress */
   FreeN_DbvHumanString( psContactInfo->ppsAddress );

   /* psEmail */
   DBVFreeDbvInternationalString( psContactInfo->psEmail );

   /* psPhone */
   DBVFreeDbvInternationalString( psContactInfo->psPhone );

   DbvFree( psContactInfo );
   } /* End DBVFree_DbvContactInfo() */ 

void DBVFree_DbvDatabaseList( DbvDatabaseList *psDatabaseList )
   {
#ifdef DEBUGZ

#endif

   if ( psDatabaseList == NULL )
      return;

   /* ppsInnerDatabases */
   FreeN_DbvInternationalString( psDatabaseList->ppsInnerDatabases );

   DbvFree( psDatabaseList );
   } /* End DBVFree_DbvDatabaseList() */ 

void DBVFree_DbvInterAddress( DbvInterAddress *psInterAddress )
   {
#ifdef DEBUGZ

#endif

   if ( psInterAddress == NULL )
      return;

   /* sHostAddress */
   DBVFree_compulsory_DbvInternationalString( &psInterAddress->sHostAddress );

   DbvFree( psInterAddress );
   } /* End DBVFree_DbvInterAddress() */ 

void DBVFree_DbvOsiPresentationAddress( 
		DbvOsiPresentationAddress *psOsiPresentationAddress )
   {
#ifdef DEBUGZ

#endif

   if ( psOsiPresentationAddress == NULL )
      return;

   /* sPsel */
   DBVFree_compulsory_DbvInternationalString( &psOsiPresentationAddress->sPsel );

   /* psSSel */
   DBVFreeDbvInternationalString( psOsiPresentationAddress->psSSel );

   /* psTSel */
   DBVFreeDbvInternationalString( psOsiPresentationAddress->psTSel );

   /* sNSap */
   DBVFree_compulsory_DbvInternationalString( &psOsiPresentationAddress->sNSap );

   DbvFree( psOsiPresentationAddress );
   } /* End DBVFree_DbvOsiPresentationAddress() */ 

void DBVFree_DbvOtherAddress( DbvOtherAddress *psOtherAddress )
   {
#ifdef DEBUGZ

#endif

   if ( psOtherAddress == NULL )
      return;

   /* sType */
   DBVFree_compulsory_DbvInternationalString( &psOtherAddress->sType );

   /* sAddress */
   DBVFree_compulsory_DbvInternationalString( &psOtherAddress->sAddress );

   DbvFree( psOtherAddress );
   } /* End DBVFree_DbvOtherAddress() */ 

void DBVFree_DbvNetworkAddress( DbvNetworkAddress *psNetworkAddress )
   {
#ifdef DEBUGZ

#endif

   if ( psNetworkAddress == NULL )
      return;

   /* uNetworkAddressType */

   switch( psNetworkAddress->eNetworkAddressType )
      {
      case NAT_InternetAddress:
	 DBVFree_DbvInterAddress( psNetworkAddress->
				uNetworkAddress.psInternetAddress );
	 break;

      case NAT_OsiPresentationAddress:
	 DBVFree_DbvOsiPresentationAddress( psNetworkAddress->
				uNetworkAddress.psOsiPresentationAddress );
	 break;

      case NAT_OtherAddress:
	 DBVFree_DbvOtherAddress( psNetworkAddress->
				uNetworkAddress.psOtherAddress );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psNetworkAddress );
   } /* End DBVFree_DbvNetworkAddress() */ 

void DBVFree_DbvOperators( DbvOperators *psOperators )
   {
#ifdef DEBUGZ

#endif

   if ( psOperators == NULL )
      return;

   /* sOperator */
   DBVFree_compulsory_DbvInternationalString( &psOperators->sOperator );

   /* ppsDescription */
   FreeN_DbvHumanString( psOperators->ppsDescription );

   DbvFree( psOperators );
   } /* End DBVFree_DbvOperators() */ 

void DBVFree_DbvSearchKey( DbvSearchKey *psSearchKey )
   {
#ifdef DEBUGZ

#endif

   if ( psSearchKey == NULL )
      return;

   /* sSearchKey */
   DBVFree_compulsory_DbvInternationalString( &psSearchKey->sSearchKey );

   /* ppsDescription */
   FreeN_DbvHumanString( psSearchKey->ppsDescription );

   DbvFree( psSearchKey );
   } /* End DBVFree_DbvSearchKey() */ 


void DBVFree_DbvHumanStringSeq( DbvHumanStringSeq *psDescription )
   {
#ifdef DEBUGZ

#endif

   if ( psDescription == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psDescription->ppsHumanString );

   DbvFree( psDescription );
   } /* End DBVFree_DbvHumanStringSeq() */ 



void DBVFree_DbvPrivateCapabilities( 
				DbvPrivateCapabilities *psPrivateCapabilities )
   {
#ifdef DEBUGZ

#endif

   if ( psPrivateCapabilities == NULL )
      return;

   /* ppsOperators */
   FreeN_DbvOperators( psPrivateCapabilities->ppsOperators );

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psPrivateCapabilities->ppsSearchKeys );

   /* ppsDescription */
   FreeN_DbvHumanStringSeq( psPrivateCapabilities->ppsDescription );

   DbvFree( psPrivateCapabilities );
   } /* End DBVFree_DbvPrivateCapabilities() */ 

void DBVFree_DbvPrivate( DbvPrivate *psPrivate )
   {
#ifdef DEBUGZ

#endif

   if ( psPrivate == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psPrivate->ppsDescription );

   DbvFree( psPrivate );
   } /* End DBVFree_DbvPrivate() */ 

void DBVFree_DbvUnitsSupported( DbvUnitsSupported *psUnitsSupported )
   {
#ifdef DEBUGZ

#endif

   if ( psUnitsSupported == NULL )
      return;

   /* uUnitsSupportedType */

   switch( psUnitsSupported->eUnitsSupportedType )
      {
      case UST_Private:
	 DBVFree_DbvPrivate( psUnitsSupported->uUnitsSupported.psPrivate );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psUnitsSupported );
   } /* End DBVFree_DbvUnitsSupported() */ 

void DBVFree_DbvProximitySupport( DbvProximitySupport *psProximitySupport )
   {
#ifdef DEBUGZ

#endif

   if ( psProximitySupport == NULL )
      return;

   FreeN_DbvUnitsSupported( psProximitySupport->ppsUnitsSupported );

   DbvFree( psProximitySupport );
   } /* End DBVFree_DbvProximitySupport() */ 

void DBVFree_DbvRpnCapabilities( DbvRpnCapabilities *psRpnCapabilities )
   {
#ifdef DEBUGZ

#endif

   if ( psRpnCapabilities == NULL )
      return;

   /* ppulOperators */
   FreeN_Ulong( psRpnCapabilities->ppulOperators );

   /* psProximity */
   DBVFree_DbvProximitySupport( psRpnCapabilities->psProximity );

   DbvFree( psRpnCapabilities );
   } /* End DBVFree_DbvRpnCapabilities() */ 

void DBVFree_DbvIso8777Capabilities(
			DbvIso8777Capabilities *psIso8777Capabilities )
   {
#ifdef DEBUGZ

#endif

   if ( psIso8777Capabilities == NULL )
      return;

   /* ppsSearchKeys */
   FreeN_DbvSearchKey( psIso8777Capabilities->ppsSearchKeys );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psIso8777Capabilities->ppsRestrictions );

   DbvFree( psIso8777Capabilities );
   } /* End DBVFree_DbvIso8777Capabilities() */ 

void DBVFree_DbvQueryTypeDetails( DbvQueryTypeDetails *psQueryTypeDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psQueryTypeDetails == NULL )
      return;

   /* uQueryTypeDetailsType */

   switch( psQueryTypeDetails->eQueryTypeDetailsType )
      {
      case QTDT_Private:
	 DBVFree_DbvPrivateCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psPrivate );
	 break;

      case QTDT_Rpn:
	 DBVFree_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psRpn );
	 break;

      case QTDT_Iso8777:
	 DBVFree_DbvIso8777Capabilities( psQueryTypeDetails->
				uQueryTypeDetails.psIso8777 );
	 break;

      case QTDT_Z39_58:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsZ39_58 );
	 break;

      case QTDT_Erpn:
	 DBVFree_DbvRpnCapabilities( psQueryTypeDetails->
				uQueryTypeDetails.psErpn );
	 break;

      case QTDT_RankedList:
	 FreeN_DbvHumanString( psQueryTypeDetails->
				uQueryTypeDetails.ppsRankedList );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psQueryTypeDetails );
   } /* End DBVFree_DbvQueryTypeDetails() */ 

void DBVFree_DbvAccessRestrictions( DbvAccessRestrictions *psAccessRestrictions )
   {
#ifdef DEBUGZ

#endif

   if ( psAccessRestrictions == NULL )
      return;

   /* ppsAccessText */
   FreeN_DbvHumanString( psAccessRestrictions->ppsAccessText );

   /* ppcAccessChallenges */
   FreeN_DbvObjectIdentifier( psAccessRestrictions->ppcAccessChallenges );

   DbvFree( psAccessRestrictions );
   } /* End DBVFree_DbvAccessRestrictions() */ 

void DBVFree_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUGZ

#endif

   if ( psCharge != NULL )
      {
      do_DBVFree_DbvCharge( psCharge );
      DbvFree( psCharge );
      } /* End if */

   } /* End DBVFree_DbvCharge() */ 

void do_DBVFree_DbvCharge( DbvCharge *psCharge )
   {
#ifdef DEBUGZ

#endif

   /* sCost */
   DBVFree_compulsory_DbvIntUnit( &psCharge->sCost );

   /* psPerWhat */
   DBVFree_DbvUnit( psCharge->psPerWhat );

   /* ppsText */
   FreeN_DbvHumanString( psCharge->ppsText );

   } /* End do_DBVFree_DbvCharge() */ 

void DBVFree_DbvOtherCharge( DbvOtherCharge *psOtherCharge )
   {
#ifdef DEBUGZ

#endif

   if ( psOtherCharge == NULL )
      return;

   /* ppsForWhat */
   FreeN_DbvHumanString( psOtherCharge->ppsForWhat );

   /* sCharge */
   DBVFree_compulsory_DbvCharge( &psOtherCharge->sCharge );

   DbvFree( psOtherCharge );
   } /* End DBVFree_DbvOtherCharge() */ 

void DBVFree_DbvCosts( DbvCosts *psCosts )
   {
#ifdef DEBUGZ

#endif

   if ( psCosts == NULL )
      return;

   /* ppsForWhat */
   DBVFree_DbvCharge( psCosts->psConnectCharge );

   /* psConnectTime */
   DBVFree_DbvCharge( psCosts->psConnectTime );

   /* psDisplayCharge */
   DBVFree_DbvCharge( psCosts->psDisplayCharge );

   /* psSearchCharge */
   DBVFree_DbvCharge( psCosts->psSearchCharge );

   /* psSubscriptCharge */
   DBVFree_DbvCharge( psCosts->psSubscriptCharge );

   /* ppsOtherCharge */
   FreeN_DbvOtherCharge( psCosts->ppsOtherCharge );

   DbvFree( psCosts );
   } /* End DBVFree_DbvCosts() */ 

void DBVFree_DbvAccessInfo( DbvAccessInfo *psAccessInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psAccessInfo == NULL )
      return;

   /* ppsQueryTypesSupported */
   FreeN_DbvQueryTypeDetails( psAccessInfo->ppsQueryTypesSupported );

   /* ppcDiagnosticsSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcDiagnosticsSets );

   /* ppcAttributeSetIds */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcAttributeSetIds );

   /* ppcSchemas */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcSchemas );

   /* ppcRecordSyntaxes */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcRecordSyntaxes );

   /* ppcResourceChallenges */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcResourceChallenges );

   /* ppsRestrictedAccess */
   FreeN_DbvAccessRestrictions( psAccessInfo->ppsRestrictedAccess );

   /* psCostInfo */
   DBVFree_DbvCosts( psAccessInfo->psCostInfo );

   /* ppcVariantSets */
   FreeN_DbvObjectIdentifier( psAccessInfo->ppcVariantSets );

   /* ppsElementSetNames */
   FreeN_DbvInternationalString( psAccessInfo->ppsElementSetNames );

   /* ppsUnitSystems */
   FreeN_DbvInternationalString( psAccessInfo->ppsUnitSystems );

   DbvFree( psAccessInfo );
   } /* End DBVFree_DbvAccessInfo() */ 

void DBVFree_DbvTargetInfo( DbvTargetInfo *psTargetInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psTargetInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psTargetInfo->psCommonInfo );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psTargetInfo->sName );

   /* ppsRecentNews */
   FreeN_DbvHumanString( psTargetInfo->ppsRecentNews );

   /* ppsIcon */
   FreeN_DbvIconObject( psTargetInfo->ppsIcon );

   /* pulMaxResultSets */
   DBVFree_Ulong( psTargetInfo->pulMaxResultSets );

   /* pulMaxResultSize */
   DBVFree_Ulong( psTargetInfo->pulMaxResultSize );

   /* pulMaxTerms */
   DBVFree_Ulong( psTargetInfo->pulMaxTerms );

   /* psTimeoutInterval */
   DBVFree_DbvIntUnit( psTargetInfo->psTimeoutInterval );

   /* ppsWelcomeMessage */
   FreeN_DbvHumanString( psTargetInfo->ppsWelcomeMessage );

   /* psContactInfo */
   DBVFree_DbvContactInfo( psTargetInfo->psContactInfo );

   /* ppsDescription */
   FreeN_DbvHumanString( psTargetInfo->ppsDescription );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psTargetInfo->ppsNicknames );

   /* ppsUsageRest */
   FreeN_DbvHumanString( psTargetInfo->ppsUsageRest );

   /* ppsPaymentAddr */
   FreeN_DbvHumanString( psTargetInfo->ppsPaymentAddr );

   /* ppsHours */
   FreeN_DbvHumanString( psTargetInfo->ppsHours  );

   /* ppsDbCombinations */
   FreeN_DbvDatabaseList( psTargetInfo->ppsDbCombinations );

   /* ppsAddresses */
   FreeN_DbvNetworkAddress( psTargetInfo->ppsAddresses );

   /* ppsLanguages */
   FreeN_DbvInternationalString( psTargetInfo->ppsLanguages );

   /* psCommonAccessInfo */
   DBVFree_DbvAccessInfo( psTargetInfo->psCommonAccessInfo );

   DbvFree( psTargetInfo );
   } /* End DBVFree_DbvTargetInfo() */ 

void DBVFree_DbvRecordCount( DbvRecordCount *psRecordCount )
   {
#ifdef DEBUGZ

#endif

   if ( psRecordCount == NULL )
      return;

   DbvFree( psRecordCount );
   } /* End DBVFree_DbvRecordCount() */ 

void DBVFree_DbvDatabaseInfo( DbvDatabaseInfo *psDatabaseInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psDatabaseInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psDatabaseInfo->psCommonInfo );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psDatabaseInfo->sName );

   /* pvExplainDatabase */
   DBVFree_DbvNull( psDatabaseInfo->pvExplainDatabase );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsNicknames );

   /* ppsIcon */
   FreeN_DbvIconObject( psDatabaseInfo->ppsIcon );

   /* ppsTitleString */
   FreeN_DbvHumanString( psDatabaseInfo->ppsTitleString );

   /* ppsKeywords */
   FreeN_DbvDataBaseInfoSeq( psDatabaseInfo->ppsKeywords );

   /* ppsDescription */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDescription );

   /* ppsAssociatedDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsAssociatedDbs );

   /* ppsSubDbs */
   FreeN_DbvInternationalString( psDatabaseInfo->ppsSubDbs );

   /* ppsDisclaimers */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDisclaimers );

   /* ppsNews */
   FreeN_DbvHumanString( psDatabaseInfo->ppsNews );

   /* psRecordCount */
   DBVFree_DbvRecordCount( psDatabaseInfo->psRecordCount );

   /* ppsDefaultOrder */
   FreeN_DbvHumanString( psDatabaseInfo->ppsDefaultOrder );

   /* pulAvRecordSize */
   DBVFree_Ulong( psDatabaseInfo->pulAvRecordSize );

   /* pulMaxRecordSize */
   DBVFree_Ulong( psDatabaseInfo->pulMaxRecordSize );

   /* ppsHours */
   FreeN_DbvHumanString( psDatabaseInfo->ppsHours );

   /* ppsBestTime */
   FreeN_DbvHumanString( psDatabaseInfo->ppsBestTime );

   /* pcLastUpdate */
   DBVFree_DbvGeneralizedTime( psDatabaseInfo->pcLastUpdate );

   /* psUpdateInterval */
   DBVFree_DbvIntUnit( psDatabaseInfo->psUpdateInterval );

   /* ppsCoverage */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCoverage );

   /* pfProprietary */
   DBVFree_DbvBool( psDatabaseInfo->pfProprietary );

   /* ppsCopyrightText */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightText );

   /* ppsCopyrightNotice */
   FreeN_DbvHumanString( psDatabaseInfo->ppsCopyrightNotice );

   /* psProducerContactInfo */
   DBVFree_DbvContactInfo( psDatabaseInfo->psProducerContactInfo );

   /* psSupplierContactInfo */
   DBVFree_DbvContactInfo( psDatabaseInfo->psSupplierContactInfo );

   /* psSubmissionContactInfo */
   DBVFree_DbvContactInfo( psDatabaseInfo->psSubmissionContactInfo );

   /* psAccessInfo */
   DBVFree_DbvAccessInfo( psDatabaseInfo->psAccessInfo );

   DbvFree( psDatabaseInfo );
   } /* End DBVFree_DbvDatabaseInfo() */ 

void DBVFree_DbvTagTypeMapping( DbvTagTypeMapping *psTagTypeMapping )
   {
#ifdef DEBUGZ

#endif

   if ( psTagTypeMapping == NULL )
      return;

   /* pcTagSet */
   DBVFreeDbvObjectIdentifier( psTagTypeMapping->pcTagSet );

   /* pvDefaultTagType */
   DBVFree_DbvNull( psTagTypeMapping->pvDefaultTagType );

   DbvFree( psTagTypeMapping );
   } /* End DBVFree_DbvTagTypeMapping() */ 

void DBVFree_DbvPath( DbvPath *psPath )
   {
#ifdef DEBUGZ

#endif

   if ( psPath == NULL )
      return;

   /* sTagValue */
   DBVFree_compulsory_DbvStringOrNumeric( &psPath->sTagValue );

   DbvFree( psPath );
   } /* End DBVFree_DbvPath() */ 

void DBVFree_DbvPathA( DbvPathA *psPathA )
   {
#ifdef DEBUGZ

#endif

   if ( psPathA == NULL )
      return;

   /* ppsSchemaTags */
   FreeN_DbvPath( psPathA->ppsSchemaTags );

   DbvFree( psPathA );
   } /* End DBVFree_DbvPathA() */ 

void DBVFree_DbvElementInfo( DbvElementInfo *psElementInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psElementInfo == NULL )
      return;

   /* sElementName */
   DBVFree_compulsory_DbvInternationalString( &psElementInfo->sElementName );

   /* ppsElementTagPath */
   FreeN_DbvPath( psElementInfo->ppsElementTagPath );

   /* psDataType */
   DBVFree_DbvElementDataType( psElementInfo->psDataType );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementInfo->ppsDescription );

   DbvFree( psElementInfo );
   } /* End DBVFree_DbvElementInfo() */ 

void DBVFree_DbvElementDataType( DbvElementDataType *psElementDataType )
   {
#ifdef DEBUGZ

#endif

   if ( psElementDataType == NULL )
      return;

   /* uElementDataTypeType */

   switch( psElementDataType->eElementDataTypeType )
      {
      case EDTT_Primitive:
	 break;

      case EDTT_Structured:
	 FreeN_DbvElementInfo( psElementDataType->
				uElementDataType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psElementDataType );
   } /* End DBVFree_DbvElementDataType() */ 

void DBVFree_DbvSchemaInfo( DbvSchemaInfo *psSchemaInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psSchemaInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psSchemaInfo->psCommonInfo );

   /* pcSchema */
   DBVFreeDbvObjectIdentifier( psSchemaInfo->pcSchema );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psSchemaInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psSchemaInfo->ppsDescription );

   /* ppsTagTypeMapping */
   FreeN_DbvTagTypeMapping( psSchemaInfo->ppsTagTypeMapping );

   /* ppsRecordStructure */
   FreeN_DbvElementInfo( psSchemaInfo->ppsRecordStructure );

   DbvFree( psSchemaInfo );
   } /* End DBVFree_DbvSchemaInfo() */ 

void DBVFree_DbvElements( DbvElements *psElements )
   {
#ifdef DEBUGZ

#endif

   if ( psElements == NULL )
      return;

   /* sElementName */
   DBVFree_compulsory_DbvInternationalString( &psElements->sElementName );

   /* ppsNicknames */
   FreeN_DbvInternationalString( psElements->ppsNicknames );

   /* sElementTag */
   DBVFree_compulsory_DbvStringOrNumeric( &psElements->sElementTag );

   /* ppsDescription */
   FreeN_DbvHumanString( psElements->ppsDescription );

   /* peDataType */
   DBVFree_DbvPrimitiveDataType( psElements->peDataType );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psElements->ppsOtherInformation );

   DbvFree( psElements );
   } /* End DBVFree_DbvElements() */ 

void DBVFree_DbvPrimitiveDataType( DbvPrimitiveDataType *pePrimitiveDataType )
   {
#ifdef DEBUGZ

#endif

   if ( pePrimitiveDataType == NULL )
      return;

   DbvFree( pePrimitiveDataType );
   } /* End DBVFree_DbvPrimitiveDataType() */ 

void DBVFree_DbvTagSetInfo( DbvTagSetInfo *psTagSetInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psTagSetInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psTagSetInfo->psCommonInfo );

   /* pcTagSet */
   DBVFreeDbvObjectIdentifier( psTagSetInfo->pcTagSet );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psTagSetInfo->sName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTagSetInfo->ppsDescription );

   /* ppsElements */
   FreeN_DbvElements( psTagSetInfo->ppsElements );

   DbvFree( psTagSetInfo );
   } /* End DBVFree_DbvTagSetInfo() */ 

void DBVFree_DbvRecordSyntaxInfo( DbvRecordSyntaxInfo *psRecordSyntaxInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psRecordSyntaxInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psRecordSyntaxInfo->psCommonInfo );

   /* pcRecordSyntax */
   DBVFreeDbvObjectIdentifier( psRecordSyntaxInfo->pcRecordSyntax );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psRecordSyntaxInfo->sName );

   /* ppcTransferSyntaxes */
   FreeN_DbvObjectIdentifier( psRecordSyntaxInfo->ppcTransferSyntaxes );

   /* ppsDescription */
   FreeN_DbvHumanString( psRecordSyntaxInfo->ppsDescription );

   /* psAsn1Module */
   DBVFreeDbvInternationalString( psRecordSyntaxInfo->psAsn1Module );

   /* ppsAbstractStructure */
   FreeN_DbvElementInfo( psRecordSyntaxInfo->ppsAbstractStructure );

   DbvFree( psRecordSyntaxInfo );
   } /* End DBVFree_DbvRecordSyntaxInfo() */ 

void DBVFree_DbvAttributeDescription( 
		DbvAttributeDescription *psAttributeDescription )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeDescription == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psAttributeDescription->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeDescription->ppsDescription );

   /* sAttributeValue */
   DBVFree_compulsory_DbvStringOrNumeric( 
				&psAttributeDescription->sAttributeValue );

   /* ppsEquivalentAttributes */
   FreeN_DbvStringOrNumeric( psAttributeDescription->ppsEquivalentAttributes );

   DbvFree( psAttributeDescription );
   } /* End DBVFree_DbvAttributeDescription() */ 

void DBVFree_DbvAttributeType( DbvAttributeType *psAttributeType )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeType == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psAttributeType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeType->ppsDescription );

   /* ppsAttributeValues */
   FreeN_DbvAttributeDescription( psAttributeType->ppsAttributeValues );

   DbvFree( psAttributeType );
   } /* End DBVFree_DbvAttributeType() */ 

void DBVFree_DbvAttributeSetInfo( DbvAttributeSetInfo *psAttributeSetInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeSetInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psAttributeSetInfo->psCommonInfo );

   /* pcAttributeSetId */
   DBVFreeDbvObjectIdentifier( psAttributeSetInfo->pcAttributeSetId );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psAttributeSetInfo->sName );

   /* ppsAttributes */
   FreeN_DbvAttributeType( psAttributeSetInfo->ppsAttributes );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeSetInfo->ppsDescription );

   DbvFree( psAttributeSetInfo );
   } /* End DBVFree_DbvAttributeSetInfo() */ 

void DBVFree_DbvTermList( DbvTermList *psTermList )
   {
#ifdef DEBUGZ

#endif

   if ( psTermList == NULL )
      return;

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psTermList->sName );

   /* ppsTitle */
   FreeN_DbvHumanString( psTermList->ppsTitle );

   /* peSearchCost */
   DBVFree_DbvSearchCost( psTermList->peSearchCost );

   /* ppsBroader */
   FreeN_DbvInternationalString( psTermList->ppsBroader );

   /* ppsNarrower */
   FreeN_DbvInternationalString( psTermList->ppsNarrower );

   DbvFree( psTermList );
   } /* End DBVFree_DbvTermList() */ 

void DBVFree_DbvTermListInfo( DbvTermListInfo *psTermListInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psTermListInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psTermListInfo->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( &psTermListInfo->sDatabaseName );

   /* ppsTermLists */
   FreeN_DbvTermList( psTermListInfo->ppsTermLists );

   DbvFree( psTermListInfo );
   } /* End DBVFree_DbvTermListInfo() */ 

void DBVFree_DbvSearchCost( DbvSearchCost *peSearchCost )
   {
#ifdef DEBUGZ

#endif

   if ( peSearchCost == NULL )
      return;

   DbvFree( peSearchCost );
   } /* End DBVFree_DbvSearchCost() */ 

void DBVFree_DbvExtendedServicesInfo( 
			DbvExtendedServicesInfo *psExtendedServicesInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psExtendedServicesInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psExtendedServicesInfo->psCommonInfo );

   /* pcType */
   DBVFreeDbvObjectIdentifier( psExtendedServicesInfo->pcType );

   /* psName */
   DBVFreeDbvInternationalString( psExtendedServicesInfo->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psExtendedServicesInfo->ppsDescription );

   /* psSpecificExplain */
   DBVFreeDbvASN1External( psExtendedServicesInfo->psSpecificExplain );

   /* psEsASN */
   DBVFreeDbvInternationalString( psExtendedServicesInfo->psEsASN );

   DbvFree( psExtendedServicesInfo );
   } /* End DBVFree_DbvExtendedServicesInfo() */ 

void DBVFree_DbvOmittedAttributeInterpretation( 
	DbvOmittedAttributeInterpretation *psOmittedAttributeInterpretation )
   {
#ifdef DEBUGZ

#endif

   if ( psOmittedAttributeInterpretation == NULL )
      return;

   /* psDefaultValue */
   DBVFree_DbvStringOrNumeric( psOmittedAttributeInterpretation->
			    psDefaultValue );

   /* ppsDefaultDescription */
   FreeN_DbvHumanString( psOmittedAttributeInterpretation->
			 ppsDefaultDescription );

   DbvFree( psOmittedAttributeInterpretation );
   } /* End DBVFree_DbvOmittedAttributeInterpretation() */ 

void DBVFree_DbvAttributeValues( DbvAttributeValues *psAttributeValues )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeValues == NULL )
      return;

   /* sValue */
   DBVFree_compulsory_DbvStringOrNumeric( &psAttributeValues->sValue );

   /* ppsDescription */
   FreeN_DbvHumanString( psAttributeValues->ppsDescription );

   /* ppsSubAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSubAttributes );

   /* ppsSuperAttributes */
   FreeN_DbvStringOrNumeric( psAttributeValues->ppsSuperAttributes );

   /* pvPartialSupport */
   DBVFree_DbvNull( psAttributeValues->pvPartialSupport );

   DbvFree( psAttributeValues );
   } /* End DBVFree_DbvAttributeValues() */ 

void DBVFree_DbvAttributesByType( DbvAttributesByType *psAttributesByType )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributesByType == NULL )
      return;

   /* psDefaultIfOmitted */
   DBVFree_DbvOmittedAttributeInterpretation( psAttributesByType->psDefaultIfOmitted );

   /* ppsAttributeValues */
   FreeN_DbvAttributeValues( psAttributesByType->ppsAttributeValues );

   DbvFree( psAttributesByType );
   } /* End DBVFree_DbvAttributesByType() */ 

void DBVFree_DbvAttributesBySet( DbvAttributesBySet *psAttributesBySet )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributesBySet == NULL )
      return;

   /* pcAttributeSet */
   DBVFreeDbvObjectIdentifier( psAttributesBySet->pcAttributeSet );

   /* ppsAttributesByType */
   FreeN_DbvAttributesByType( psAttributesBySet->ppsAttributesByType );

   DbvFree( psAttributesBySet );
   } /* End DBVFree_DbvAttributesBySet() */ 

void DBVFree_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeOccurrenceChoice != NULL )
      {
      do_DBVFree_DbvAttributeOccurrenceChoice( psAttributeOccurrenceChoice );
      DbvFree( psAttributeOccurrenceChoice );
      } /* End if */

   } /* End DBVFree_DbvAttributeOccurrenceChoice() */ 

void do_DBVFree_DbvAttributeOccurrenceChoice( 
		DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeOccurrenceChoice == NULL )
      return;

   /* uAttributeOccurrenceChoiceType */

   switch( psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType )
      {
      case AOT_AnyOrNone:
	 DBVFree_DbvNull( psAttributeOccurrenceChoice->
		       uAttributeOccurrenceChoiceType.
		       pvAnyOrNone );
	 break;

      case AOT_Specific:
	 FreeN_DbvStringOrNumeric( psAttributeOccurrenceChoice->
				       uAttributeOccurrenceChoiceType.
				       ppsSpecific );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVFree_DbvAttributeOccurrenceChoice() */ 

void DBVFree_DbvAttributeOccurrence(
			DbvAttributeOccurrence *psAttributeOccurrence )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeOccurrence == NULL )
      return;

   /* pcAttributeSet */
   DBVFreeDbvObjectIdentifier( psAttributeOccurrence->pcAttributeSet );

   /* pvMustBeSupplied */
   DBVFree_DbvNull( psAttributeOccurrence->pvMustBeSupplied );

   /* sAttributeValues */
   DBVFree_compulsory_DbvAttributeOccurrenceChoice(
				&psAttributeOccurrence->sAttributeValues );

   DbvFree( psAttributeOccurrence );
   } /* End DBVFree_DbvAttributeOccurrence() */ 

void DBVFree_DbvAttributeCombination( 
			DbvAttributeCombination *psAttributeCombination )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeCombination == NULL )
      return;

   /* ppsAttributeOccurrence */
   FreeN_DbvAttributeOccurrence( 
			psAttributeCombination->ppsAttributeOccurrence );

   DbvFree( psAttributeCombination );
   } /* End DBVFree_DbvAttributeCombination() */ 

void DBVFree_DbvAttributeCombinations( 
			DbvAttributeCombinations *psAttributeCombinations )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeCombinations == NULL )
      return;

   /* pcDefaultAttributeSet */
   DBVFreeDbvObjectIdentifier( psAttributeCombinations->pcDefaultAttributeSet );

   /* ppsLegalCombinations */
   FreeN_DbvAttributeCombination( psAttributeCombinations->
				   ppsLegalCombinations );

   DbvFree( psAttributeCombinations );
   } /* End DBVFree_DbvAttributeCombinations() */ 

void DBVFree_DbvAttributeDetails( DbvAttributeDetails *psAttributeDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psAttributeDetails == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psAttributeDetails->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( &psAttributeDetails->sDatabaseName );

   /* ppsAttributesBySet */
   FreeN_DbvAttributesBySet( psAttributeDetails->ppsAttributesBySet );

   /* psAttributeCombinations */
   DBVFree_DbvAttributeCombinations( psAttributeDetails->psAttributeCombinations );

   DbvFree( psAttributeDetails );
   } /* End DBVFree_DbvAttributeDetails() */ 

void DBVFree_DbvScanInfo( DbvScanInfo *psScanInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psScanInfo == NULL )
      return;

   /* pulMaxStepSize */
   DBVFree_Ulong( psScanInfo->pulMaxStepSize );

   /* ppsCollatingSequence */
   FreeN_DbvHumanString( psScanInfo->ppsCollatingSequence );

   /* pfIncreasing */
   DBVFree_DbvBool( psScanInfo->pfIncreasing );

   DbvFree( psScanInfo );
   } /* End DBVFree_DbvScanInfo() */ 

void DBVFree_DbvTermListDetails( DbvTermListDetails *psTermListDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psTermListDetails == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psTermListDetails->psCommonInfo );

   /* sTermListName */
   DBVFree_compulsory_DbvInternationalString( &psTermListDetails->sTermListName );

   /* ppsDescription */
   FreeN_DbvHumanString( psTermListDetails->ppsDescription );

   /* psAttributes */
   DBVFree_DbvAttributeCombinations( psTermListDetails->psAttributes );

   /* psScanInfo */
   DBVFree_DbvScanInfo( psTermListDetails->psScanInfo );

   /* pulEstNumberTerms */
   DBVFree_Ulong( psTermListDetails->pulEstNumberTerms );

   /* ppsSampleTerms */
   FreeN_DbvTerm( psTermListDetails->ppsSampleTerms );

   DbvFree( psTermListDetails );
   } /* End DBVFree_DbvTermListDetails() */ 

void DBVFree_DbvRecordTag( DbvRecordTag *psRecordTag )
   {
#ifdef DEBUGZ

#endif

   if ( psRecordTag == NULL )
      return;

   /* psQualifier */
   DBVFree_DbvStringOrNumeric( psRecordTag->psQualifier );

   /* sTagValue */
   DBVFree_compulsory_DbvStringOrNumeric( &psRecordTag->sTagValue );

   DbvFree( psRecordTag );
   } /* End DBVFree_DbvRecordTag() */ 

void DBVFree_DbvDetailsPerElement( DbvDetailsPerElement *psDetailsPerElement )
   {
#ifdef DEBUGZ

#endif

   if ( psDetailsPerElement == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psDetailsPerElement->psName );

   /* psRecordTag */
   DBVFree_DbvRecordTag( psDetailsPerElement->psRecordTag );

   /* ppsSchemaTags */
   FreeN_DbvPathA( psDetailsPerElement->ppsSchemaTags );

   /* pulMaxSize */
   DBVFree_Ulong(  psDetailsPerElement->pulMaxSize );

   /* pulMinSize */
   DBVFree_Ulong(  psDetailsPerElement->pulMinSize );

   /* pulAvgSize */
   DBVFree_Ulong(  psDetailsPerElement->pulAvgSize );

   /* pulFixedSize */
   DBVFree_Ulong(  psDetailsPerElement->pulFixedSize );

   /* ppsDescription*/
   FreeN_DbvHumanString( psDetailsPerElement->ppsDescription );

   /* ppsContents */
   FreeN_DbvHumanString( psDetailsPerElement->ppsContents );

   /* ppsBillingInfo */
   FreeN_DbvHumanString( psDetailsPerElement->ppsBillingInfo );

   /* ppsRestrictions */
   FreeN_DbvHumanString( psDetailsPerElement->ppsRestrictions );

   /* ppsAlternateNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsAlternateNames );

   /* ppsGenericNames */
   FreeN_DbvInternationalString( psDetailsPerElement->ppsGenericNames );

   /* psSearchAccess */
   DBVFree_DbvAttributeCombinations( psDetailsPerElement->psSearchAccess );

   DbvFree( psDetailsPerElement );
   } /* End DBVFree_DbvDetailsPerElement() */ 

void DBVFree_DbvElementSetDetails( DbvElementSetDetails *psElementSetDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psElementSetDetails == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psElementSetDetails->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( &psElementSetDetails->sDatabaseName);

   /* sElementSetName */
   DBVFree_compulsory_DbvInternationalString( 
					&psElementSetDetails->sElementSetName );

   /* pcRecordSyntax */
   DBVFreeDbvObjectIdentifier( psElementSetDetails->pcRecordSyntax );

   /* pcSchema */
   DBVFreeDbvObjectIdentifier( psElementSetDetails->pcSchema );

   /* ppsDescription */
   FreeN_DbvHumanString( psElementSetDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psElementSetDetails->ppsDetailsPerElement );

   DbvFree( psElementSetDetails );
   } /* End DBVFree_DbvElementSetDetails() */ 

void DBVFree_DbvRetrievalRecordDetails( 
			DbvRetrievalRecordDetails *psRetrievalRecordDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psRetrievalRecordDetails == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psRetrievalRecordDetails->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( &psRetrievalRecordDetails->
					   sDatabaseName);
   /* pcSchema */
   DBVFreeDbvObjectIdentifier( psRetrievalRecordDetails->pcSchema );

   /* pcRecordSyntax */
   DBVFreeDbvObjectIdentifier( psRetrievalRecordDetails->pcRecordSyntax );

   /* ppsDescription */
   FreeN_DbvHumanString( psRetrievalRecordDetails->ppsDescription );

   /* ppsDetailsPerElement */
   FreeN_DbvDetailsPerElement( psRetrievalRecordDetails->ppsDetailsPerElement );

   DbvFree( psRetrievalRecordDetails );
   } /* End DBVFree_DbvRetrievalRecordDetails() */ 

void DBVFree_DbvSortType( DbvSortType *psSortType )
   {
#ifdef DEBUGZ

#endif

   if ( psSortType == NULL )
      return;

   /* uSortTypeType */

   switch( psSortType->eSortTypeType )
      {
      case STT_Structured:
	 FreeN_DbvHumanString( psSortType->uSortType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSortType );
   } /* End DBVFree_DbvSortType() */ 

void DBVFree_DbvCaseSensitivity( DbvCaseSensitivity *peCaseSensitivity )
   {
#ifdef DEBUGZ

#endif

   if ( peCaseSensitivity == NULL )
      return;

   DbvFree( peCaseSensitivity );
   } /* End DBVFree_DbvCaseSensitivity() */ 

void DBVFree_DbvSortKeys( DbvSortKeys *psSortKeys )
   {
#ifdef DEBUGZ

#endif

   if ( psSortKeys == NULL )
      return;

   /* ppsDescription */
   FreeN_DbvHumanString( psSortKeys->ppsDescription );

   /* ppsElementSpecifications */
   FreeN_DbvSpecification( psSortKeys->ppsElementSpecifications );

   /* psAttributeSpecifications */
   DBVFree_DbvAttributeCombinations( psSortKeys->psAttributeSpecifications );

   /* psSortType */
   DBVFree_DbvSortType( psSortKeys->psSortType );

   /* peCaseSensitivity */
   DBVFree_DbvCaseSensitivity( psSortKeys->peCaseSensitivity );
   
   DbvFree( psSortKeys );
   } /* End DBVFree_DbvSortKeys() */ 

void DBVFree_DbvSortDetails( DbvSortDetails *psSortDetails )
   {
#ifdef DEBUGZ

#endif

   if ( psSortDetails == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psSortDetails->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( &psSortDetails->sDatabaseName );

   /* ppsSortKeys */
   FreeN_DbvSortKeys( psSortDetails->ppsSortKeys );

   DbvFree( psSortDetails );
   } /* End DBVFree_DbvSortDetails() */ 

void DBVFree_DbvProcessingInformation( 
			DbvProcessingInformation *psProcessingInformation )
   {
#ifdef DEBUGZ

#endif

   if ( psProcessingInformation == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psProcessingInformation->psCommonInfo );

   /* sDatabaseName */
   DBVFree_compulsory_DbvInternationalString( 
				&psProcessingInformation->sDatabaseName );
   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psProcessingInformation->sName );

   /* pcOid */
   DBVFreeDbvObjectIdentifier( psProcessingInformation->pcOid );

   /* ppsDescription */
   FreeN_DbvHumanString( psProcessingInformation->ppsDescription );

   /* psInstructions */
   DBVFreeDbvASN1External( psProcessingInformation->psInstructions );

   DbvFree( psProcessingInformation );
   } /* End DBVFree_DbvProcessingInformation() */ 

void DBVFree_DbvValueDescription( DbvValueDescription *psValueDescription )
   {
#ifdef DEBUGZ

#endif

   if ( psValueDescription == NULL )
      return;

   /* uValueDescriptionType */

   switch( psValueDescription->eValueDescriptionType )
      {
      case VDT_Integer:
	 break;

      case VDT_String:
	 DBVFreeDbvInternationalString( psValueDescription->
					uValueDescription.psString );
	 break;

      case VDT_Octets:
	 DBVFreeDbvOctetString( psValueDescription->
					uValueDescription.psOctets );
	 break;

      case VDT_Oid:
	 DBVFreeDbvObjectIdentifier( psValueDescription->
					uValueDescription.pcOid );
	 break;

      case VDT_Unit:
	 DBVFree_DbvUnit( psValueDescription->
					uValueDescription.psUnit );
	 break;

      case VDT_ValueAndUnit:
	 DBVFree_DbvIntUnit( psValueDescription->
					uValueDescription.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueDescription );
   } /* End DBVFree_DbvValueDescription() */ 

void DBVFree_DbvValueRange( DbvValueRange *psValueRange )
   {
#ifdef DEBUGZ

#endif

   if ( psValueRange == NULL )
      return;

   /* psLower */
   DBVFree_DbvValueDescription( psValueRange->psLower );

   /* psUpper */
   DBVFree_DbvValueDescription( psValueRange->psUpper );

   DbvFree( psValueRange );
   } /* End DBVFree_DbvValueRange() */ 

void DBVFree_DbvValueSet( DbvValueSet *psValueSet )
   {
#ifdef DEBUGZ

#endif

   if ( psValueSet == NULL )
      return;

   /* uValueSetType */

   switch( psValueSet->eValueSetType )
      {
      case VT_Range:
	 DBVFree_DbvValueRange( psValueSet->uValueSet.psRange );
	 break;

      case VT_Enumerated:
	 FreeN_DbvValueDescription( psValueSet->uValueSet.ppsEnumerated );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValueSet );
   } /* End DBVFree_DbvValueSet() */ 

void DBVFree_DbvVariantValue( DbvVariantValue *psVariantValue )
   {
#ifdef DEBUGZ

#endif

   if ( psVariantValue == NULL )
      return;

   /* psValues */
   DBVFree_DbvValueSet( psVariantValue->psValues );

   DbvFree( psVariantValue );
   } /* End DBVFree_DbvVariantValue() */ 

void DBVFree_DbvVariantTypes( DbvVariantTypes *psVariantTypes )
   {
#ifdef DEBUGZ

#endif

   if ( psVariantTypes == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psVariantTypes->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantTypes->ppsDescription );

   /* psVariantValue */
   DBVFree_DbvVariantValue( psVariantTypes->psVariantValue );

   DbvFree( psVariantTypes );
   } /* End DBVFree_DbvVariantTypes() */ 

void DBVFree_DbvVariantClass( DbvVariantClass *psVariantClass )
   {
#ifdef DEBUGZ

#endif

   if ( psVariantClass == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psVariantClass->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psVariantClass->ppsDescription );

   /* ppsVariantTypes */
   FreeN_DbvVariantTypes( psVariantClass->ppsVariantTypes );

   DbvFree( psVariantClass );
   } /* End DBVFree_DbvVariants() */ 

void DBVFree_DbvVariantSetInfo( DbvVariantSetInfo *psVariantSetInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psVariantSetInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psVariantSetInfo->psCommonInfo );

   /* pcVariantSet */
   DBVFreeDbvObjectIdentifier( psVariantSetInfo->pcVariantSet );

   /* sName */
   DBVFree_compulsory_DbvInternationalString( &psVariantSetInfo->sName );

   /* ppsVariants */
   FreeN_DbvVariantClass( psVariantSetInfo->ppsVariants );

   DbvFree( psVariantSetInfo );
   } /* End DBVFree_DbvVariantSetInfo() */ 

void DBVFree_DbvUnits( DbvUnits *psUnits )
   {
#ifdef DEBUGZ

#endif

   if ( psUnits == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psUnits->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnits->ppsDescription );

   /* sUnit */
   DBVFree_compulsory_DbvStringOrNumeric( &psUnits->sUnit );

   DbvFree( psUnits );
   } /* End DBVFree_DbvUnits() */ 

void DBVFree_DbvUnitType( DbvUnitType *psUnitType )
   {
#ifdef DEBUGZ

#endif

   if ( psUnitType == NULL )
      return;

   /* psName */
   DBVFreeDbvInternationalString( psUnitType->psName );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitType->ppsDescription );

   /* sUnitType */
   DBVFree_compulsory_DbvStringOrNumeric( &psUnitType->sUnitType );

   /* ppsUnits */
   FreeN_DbvUnits( psUnitType->ppsUnits );

   DbvFree( psUnitType );
   } /* End DBVFree_DbvUnitType() */ 

void DBVFree_DbvUnitInfo( DbvUnitInfo *psUnitInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psUnitInfo == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psUnitInfo->psCommonInfo );

   /* sUnitSystem */
   DBVFree_compulsory_DbvInternationalString( &psUnitInfo->sUnitSystem );

   /* ppsDescription */
   FreeN_DbvHumanString( psUnitInfo->ppsDescription );

   /* ppsUnitTypes */
   FreeN_DbvUnitType( psUnitInfo->ppsUnitTypes );

   DbvFree( psUnitInfo );
   } /* End DBVFree_DbvUnitInfo() */ 

void DBVFree_DbvCategoryInfo( DbvCategoryInfo *psCategoryInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psCategoryInfo == NULL )
      return;

   /* sCategory */
   DBVFree_compulsory_DbvInternationalString( &psCategoryInfo->sCategory );

   /* psOriginalCategory */
   DBVFreeDbvInternationalString( psCategoryInfo->psOriginalCategory );

   /* ppsDescription */
   FreeN_DbvHumanString( psCategoryInfo->ppsDescription );

   /* psAsn1Module */
   DBVFreeDbvInternationalString( psCategoryInfo->psAsn1Module );

   DbvFree( psCategoryInfo );
   } /* End DBVFree_DbvCategorInfo() */ 

void DBVFree_DbvCategoryList( DbvCategoryList *psCategoryList )
   {
#ifdef DEBUGZ

#endif

   if ( psCategoryList == NULL )
      return;

   /* psCommonInfo */
   DBVFree_DbvCommonInfo( psCategoryList->psCommonInfo );

   /* ppsCategories */
   FreeN_DbvCategoryInfo( psCategoryList->ppsCategories );

   DbvFree( psCategoryList );
   } /* End DBVFree_DbvCategoryList() */ 

void DBVFree_DbvExplainRecord( DbvExplainRecord *psExplainRecord )
   {
#ifdef DEBUGZ

#endif

   if ( psExplainRecord == NULL )
      return;

   /* uExplainRecordType */

   switch( psExplainRecord->eExplainRecordType )
      {
      case ERT_TargetInfo:
	 DBVFree_DbvTargetInfo( psExplainRecord->uExplainRecord.psTargetInfo);
	 break;

      case ERT_DatabaseInfo:
	 DBVFree_DbvDatabaseInfo( psExplainRecord->
					uExplainRecord.psDatabaseInfo);
	 break;

      case ERT_SchemaInfo:
	 DBVFree_DbvSchemaInfo( psExplainRecord->
					uExplainRecord.psSchemaInfo);
	 break;

      case ERT_TagSetInfo:
	 DBVFree_DbvTagSetInfo( psExplainRecord->
					uExplainRecord.psTagSetInfo);
	 break;

      case ERT_RecordSyntaxInfo:
	 DBVFree_DbvRecordSyntaxInfo( psExplainRecord->
					uExplainRecord.psRecordSyntaxInfo);
	 break;

      case ERT_AttributeSetInfo:
	 DBVFree_DbvAttributeSetInfo( psExplainRecord->
					uExplainRecord.psAttributeSetInfo);
	 break;

      case ERT_TermListInfo:
	 DBVFree_DbvTermListInfo( psExplainRecord->
					uExplainRecord.psTermListInfo);
	 break;

      case ERT_ExtendedServicesInfo:
	 DBVFree_DbvExtendedServicesInfo( psExplainRecord->
					uExplainRecord.psExtendedServicesInfo);
	 break;

      case ERT_AttributeDetails:
	 DBVFree_DbvAttributeDetails( psExplainRecord->
					uExplainRecord.psAttributeDetails);
	 break;

      case ERT_TermListDetails:
	 DBVFree_DbvTermListDetails( psExplainRecord->
					uExplainRecord.psTermListDetails);
	 break;

      case ERT_ElementSetDetails:
	 DBVFree_DbvElementSetDetails( psExplainRecord->
					uExplainRecord.psElementSetDetails);
	 break;

      case ERT_RetrievalRecordDetails:
	 DBVFree_DbvRetrievalRecordDetails( psExplainRecord->
			 	uExplainRecord.psRetrievalRecordDetails);
	 break;

      case ERT_SortDetails:
	 DBVFree_DbvSortDetails( psExplainRecord->
					uExplainRecord.psSortDetails);
	 break;

      case ERT_Processing:
	 DBVFree_DbvProcessingInformation( psExplainRecord->
					uExplainRecord.psProcessing);
	 break;

      case ERT_Variants:
	 DBVFree_DbvVariantSetInfo( psExplainRecord->
					uExplainRecord.psVariants);
	 break;

      case ERT_Units:
	 DBVFree_DbvUnitInfo( psExplainRecord->
					uExplainRecord.psUnits);
	 break;

      case ERT_CategoryList:
	 DBVFree_DbvCategoryList( psExplainRecord->
					uExplainRecord.psCategoryList);
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psExplainRecord );
   } /* End DBVFree_DbvExplainRecord() */ 

void DBVFree_DbvLanguageCode( DbvLanguageCode *psLanguageCode )
   {
#ifdef DEBUGZ

#endif

   DBVFreeDbvOctetString( (DbvOctetString*)psLanguageCode );

   } /* End DBVFree_DbvLanguageCode() */ 

void DBVFree_DbvNull( DbvNull *psNull )
   {
#ifdef DEBUGZ

#endif

   if ( psNull == NULL )
      return;

   DbvFree( psNull );

   } /* End DBVFree_DbvNull() */ 

void DBVFree_DbvUnit( DbvUnit *psUnit )
   {
#ifdef DEBUGZ

#endif

   if ( psUnit == NULL )
      return;

   /* psUnitSystem */
   DBVFreeDbvInternationalString( psUnit->psUnitSystem );

   /* psUnitType */
   DBVFree_DbvStringOrNumeric( psUnit->psUnitType );

   /* psUnit */
   DBVFree_DbvStringOrNumeric( psUnit->psUnit );

   /* pulScaleFactor */
   DBVFree_Ulong( psUnit->pulScaleFactor );

   DbvFree( psUnit );
   } /* End DBVFree_DbvUnit() */ 

void DBVFree_DbvTerm( DbvTerm *psTerm )
   {
#ifdef DEBUGZ

#endif

   if ( psTerm == NULL )
      return;

   /* uTermType */

   switch( psTerm->eTermType )
      {
      case TT_General:
	 DBVFreeDbvOctetString( psTerm->uTerm.psGeneral );
	 break;

      case TT_Numeric:
	 break;

      case TT_SimpleCharacter:
	 DBVFreeDbvInternationalString( psTerm->uTerm.psSimpleCharacter );
	 break;

      case TT_ObjectIdentifier:
	 DBVFreeDbvObjectIdentifier( psTerm->uTerm.pcObjectIdentifier );
	 break;

      case TT_DateTime:
	 DBVFree_DbvGeneralizedTime( psTerm->uTerm.pcDateTime );
	 break;

      case TT_External:
	 DBVFreeDbvASN1External( psTerm->uTerm.psExternal );
	 break;

      case TT_IntegerAndUnit:
	 DBVFree_DbvIntUnit( psTerm->uTerm.psIntegerAndUnit );
	 break;

      case TT_Null:
      default:
	 break;
      } /* End switch */

   DbvFree( psTerm );
   } /* End DBVFree_DbvTerm() */ 

void DBVFree_DbvDataBaseInfoSeq( DbvDataBaseInfoSeq *psDataBaseInfoSeq )
   {
#ifdef DEBUGZ

#endif

   if ( psDataBaseInfoSeq == NULL )
      return;

   /* ppsDataBaseInfo */
   FreeN_DbvHumanString( psDataBaseInfoSeq->ppsDataBaseInfo );

   DbvFree( psDataBaseInfoSeq );
   } /* End DBVFree_DbvDataBaseInfoSeq() */ 


#ifdef NOT_YET_IMPLEMENTED

void DBVFree_DbvOrder( DbvOrder *psOrder )
   {
#ifdef DEBUGZ

#endif

   if ( psOrder == NULL )
      return;

   DbvFree( psOrder );
   } /* End DBVFree_DbvOrder() */ 

void DBVFree_DbvUsage( DbvUsage *psUsage )
   {
#ifdef DEBUGZ

#endif

   if ( psUsage == NULL )
      return;

   /* psRestriction */
   DBVFreeDbvInternationalString( psUsage->psRestriction );

   DbvFree( psUsage );
   } /* End DBVFree_DbvUsage() */ 

void DBVFree_DbvHits( DbvHits *psHits )
   {
#ifdef DEBUGZ

#endif

   if ( psHits == NULL )
      return;

   /* psSatisfier */
   DBVFree_DbvTerm( psHits->psSatisfier );

   /* psOffsetIntoElement */
   DBVFree_DbvIntUnit( psHits->psOffsetIntoElement );

   /* psLength */
   DBVFree_DbvIntUnit( psHits->psLength );

   /* pulHitRank */
   DBVFree_Ulong( psHits->pulHitRank );

   /* psTargetToken */
   DBVFreeDbvOctetString( psHits->psTargetToken );

   DbvFree( psHits );
   } /* End DBVFree_DbvHits() */ 



void DBVFree_DbvValue( DbvValue *psValue )
   {
#ifdef DEBUGZ

#endif

   if ( psValue == NULL )
      return;

   /* uValueType */

   switch( psValue->eVariantValueType )
      {
      case VVT_Int:
	 break;

      case VVT_InternationalString:
	 DBVFreeDbvInternationalString( psValue->
				      uVariantValueType.psInternationalString );
	 break;

      case VVT_Octs:
	 DBVFreeDbvOctetString( psValue->uVariantValueType.psOcts );
	 break;

      case VVT_Oid:
	 DBVFreeDbvObjectIdentifier( psValue->uVariantValueType.pcOid );
	 break;

      case VVT_Bool:
      case VVT_Null:
	 break;

      case VVT_Unit:
	 DBVFree_DbvUnit( psValue->uVariantValueType.psUnit );
	 break;

      case VVT_ValueAndUnit:
	 DBVFree_DbvIntUnit( psValue->uVariantValueType.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psValue );
   } /* End DBVFree_DbvValue() */ 



void DBVFree_DbvTriples( DbvTriples *psTriples )
   {
#ifdef DEBUGZ

#endif

   if ( psTriples == NULL )
      return;

   /* pcVariantSetId */
   DBVFreeDbvObjectIdentifier( psTriples->pcVariantSetId );

   /* ppsValue */
   FreeN_DbvValue( psTriples->ppsValue );

   DbvFree( psTriples );
   } /* End DBVFree_DbvTriples() */ 

void DBVFree_DbvVariant( DbvVariant *psVariant )
   {
#ifdef DEBUGZ

#endif

   if ( psVariant == NULL )
      return;

   /* pcGlobalVariantSetId */
   DBVFreeDbvObjectIdentifier( psVariant->pcGlobalVariantSetId );

   /* ppsTriples */
   FreeN_DbvTriples( psVariant->ppsTriples );

   DbvFree( psVariant );
   } /* End DBVFree_DbvVariant() */ 

void DBVFree_DbvTagPath( DbvTagPath *psTagPath )
   {
#ifdef DEBUGZ

#endif

   if ( psTagPath == NULL )
      return;

   /* pulTagType */
   DBVFree_Ulong( psTagPath->pulTagType );

   /* sTagValue */
   DBVFree_compulsory_DbvStringOrNumeric( &psTagPath->sTagValue );

   /* pulTagOccurrence */
   DBVFree_Ulong( psTagPath->pulTagOccurrence );

   DbvFree( psTagPath );
   } /* End DBVFree_DbvTagPath() */ 

void DBVFree_DbvElementMetaData( DbvElementMetaData *psElementMetaData )
   {
#ifdef DEBUGZ

#endif

   if ( psElementMetaData == NULL )
      return;

   /* psSeriesOrder */
   DBVFree_DbvOrder( psElementMetaData->psSeriesOrder );

   /* psUsageRight */
   DBVFree_DbvUsage( psElementMetaData->psUsageRight );

   /* ppsHits */
   FreeN_DbvHits( psElementMetaData->ppsHits );

   /* psDisplayName */
   DBVFreeDbvInternationalString( psElementMetaData->psDisplayName );

   /* ppsSupportedVariants */
   FreeN_DbvVariant( psElementMetaData->ppsSupportedVariants );

   /* psMessage */
   DBVFreeDbvInternationalString( psElementMetaData->psMessage );

   /* psElementDescriptor */
   DBVFreeDbvOctetString( psElementMetaData->psElementDescriptor );

   /* ppsSurrogateFor */
   FreeN_DbvTagPath( psElementMetaData->ppsSurrogateFor );

   /* ppsSurrogateElement */
   FreeN_DbvTagPath( psElementMetaData->ppsSurrogateElement );

   /* psOther */
   DBVFreeDbvASN1External( psElementMetaData->psOther );

   DbvFree( psElementMetaData );
   } /* End DBVFree_DbvElementMetaData() */ 

void DBVFree_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

#ifdef DEBUGZ

#endif

   if ( psElementDataG == NULL )
      {
      do_DBVFree_DbvElementDataG( psElementDataG );
      DbvFree( psElementDataG );
      } /* End if */

   } /* End DBVFree_DbvElementData() */ 

void do_DBVFree_DbvElementDataG( DbvElementDataG *psElementDataG )
   {

#ifdef DEBUGZ

#endif

   /* uElementDataGType */

   switch( psElementDataG->eElementDataGType )
      {
      case EDT_Octets:
	 DBVFreeDbvOctetString( psElementDataG->uElementDataG.psOctets );
	 break;

      case EDT_Numeric:
	 break;

      case EDT_Date:
	 DBVFree_DbvGeneralizedTime( psElementDataG->uElementDataG.pcDate );
	 break;

      case EDT_Ext:
	 DBVFreeDbvASN1External( psElementDataG->uElementDataG.psExt );
	 break;

      case EDT_String:
	 DBVFreeDbvInternationalString( psElementDataG->uElementDataG.psString );
	 break;

      case EDT_TrueOrFalse:
	 break;

      case EDT_Oid:
	 DBVFreeDbvObjectIdentifier( psElementDataG->uElementDataG.pcOid );
	 break;

      case EDT_IntUnit:
	 DBVFree_DbvIntUnit( psElementDataG->uElementDataG.psIntUnit);
	 break;

      case EDT_ElementNotThere:
      case EDT_ElementEmpty:
      case EDT_NoDataRequested:
	 break;

      case EDT_Diagnostic:
	 DBVFreeDbvASN1External( psElementDataG->uElementDataG.psDiagnostic );
	 break;

      case EDT_Subtree:
	 FreeN_DbvTaggedElement( psElementDataG->uElementDataG.ppsSubTree);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVFree_DbvElementDataG() */ 

void DBVFree_DbvTaggedElement( DbvTaggedElement *psTaggedElement )
   {
#ifdef DEBUGZ

#endif

   if ( psTaggedElement == NULL )
      return;


   /* pulTagType */
   DBVFree_Ulong( psTaggedElement->pulTagType );

   /* sTagValue */
   DBVFree_compulsory_DbvStringOrNumeric( &psTaggedElement->sTagValue );

   /* pulTagOccurrence */
   DBVFree_Ulong( psTaggedElement->pulTagOccurrence );

   /* sContent */
   DBVFree_compulsory_DbvElementDataG( &psTaggedElement->sContent );

   /* psMetaData */
   DBVFree_DbvElementMetaData( psTaggedElement->psMetaData );

   /* psAppliedVariant */
   DBVFree_DbvVariant( psTaggedElement->psAppliedVariant );

   DbvFree( psTaggedElement );
   } /* End DBVFree_DbvTaggedElement() */ 

void DBVFree_DbvGenericRecord( DbvGenericRecord *psGenericRecord )
   {
#ifdef DEBUGZ

#endif

   if ( psGenericRecord == NULL )
      return;

   /* ppsTaggedElement */
   FreeN_DbvTaggedElement( psGenericRecord->ppsTaggedElement );

   DbvFree( psGenericRecord );
   } /* End DBVFree_DbvGenericRecord() */ 

#endif





/*-------------------- ACCESSCONTROL --------------------*/

void DBVFree_DbvSecurityChallengeRsp( DbvSecurityChallengeRsp *psSecurityChallengeRsp )
   {
#ifdef DEBUGZ

#endif

   if ( psSecurityChallengeRsp == NULL )
      return;

   /* uSecurityChallengeRsp */

   switch( psSecurityChallengeRsp->eSecurityChallengeRspType ) 
      {
      case SECT_SimpleForm:
   	 DBVFreeDbvOctetString( psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm );
	 break;

      case SECT_ExternallyDefined:
   	 DBVFreeDbvASN1External( psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psSecurityChallengeRsp );
   } /* End DBVFree_DbvSecurityChallengeRsp */



void DBVFree_DbvAccessControlReq( DbvAccessControlReq *psAccessControlReq )
   {
#ifdef DEBUGZ

#endif

   if ( psAccessControlReq == NULL )
      return;

   /* psReferenceId */  
   DBVFreeDbvOctetString( psAccessControlReq->psReferenceId );

   /* sSecurityChallengeReq */
   DBVFree_compulsory_DbvSecurityChallengeReq( &psAccessControlReq->sSecurityChallengeReq );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlReq->ppsOtherInformation );

   DbvFree( psAccessControlReq );
   } /* End DBVFree_DbvAccessControlReq() */ 



void DBVFree_DbvAccessControlRsp( DbvAccessControlRsp *psAccessControlRsp )
   {
#ifdef DEBUGZ

#endif

   if ( psAccessControlRsp == NULL )
      return;

   /* psReferenceId */
   DBVFreeDbvOctetString( psAccessControlRsp->psReferenceId );

   /* psSecurityChallengeRsp */
   DBVFree_DbvSecurityChallengeRsp( psAccessControlRsp->psSecurityChallengeRsp );

   /* psDiagnostic */
   DBVFree_DbvDiagRec( psAccessControlRsp->psDiagnostic );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psAccessControlRsp->ppsOtherInformation );

   DbvFree( psAccessControlRsp );
   } /* End DBVFree_DbvAccessControlRsp() */ 



/* -------------- Access Control Format "Prompt 1" ----------------*/


void DBVFree_DbvEnumeratedPrompt( DbvEnumeratedPrompt *psEnumeratedPrompt )
   {
#ifdef DEBUGZ

#endif

   if ( psEnumeratedPrompt == NULL )
      return;


   /* compulsory eType */

   /* psSuggestedString */
   DBVFreeDbvInternationalString( psEnumeratedPrompt->psSuggestedString );

   DbvFree( psEnumeratedPrompt );
   } /* End DBVFree_DbvEnumeratedPrompt() */ 



void DBVFree_DbvEncryption( DbvEncryption *psEncryption )
   {
#ifdef DEBUGZ

#endif

   if ( psEncryption == NULL )
      return;

   /* psCryptType */
   DBVFreeDbvOctetString( psEncryption->psCryptType );

   /* psCredential */
   DBVFreeDbvOctetString( psEncryption->psCredential );

   /* sData */
   DBVFree_compulsory_DbvOctetString( &psEncryption->sData);

   DbvFree( psEncryption );
   } /* End DBVFree_DbvEncryption() */ 



void DBVFree_DbvPromptInfo( DbvPromptInfo *psPromptInfo )
   {
#ifdef DEBUGZ

#endif

   if ( psPromptInfo == NULL )
      return;


   /* uPromptInfo */

   switch( psPromptInfo->ePromptInfoType ) 
      {
      case PIIT_Character:
   	 DBVFreeDbvInternationalString( psPromptInfo->uPromptInfo.psCharacter );
	 break;

      case PIIT_Encrypted:
         DBVFree_DbvEncryption( psPromptInfo->uPromptInfo.psEncrypted );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptInfo );
   } /* End DBVFree_DbvPromptInfo */



void DBVFree_DbvDataType( DbvDataType *peDataType )
   {
#ifdef DEBUGZ

#endif
     
   if ( peDataType == NULL )
      return;

   DbvFree( peDataType );
   } /* End DBVFree_DbvDataType() */



void DBVFree_DbvChallenge( DbvChallenge *psChallenge )
   {
#ifdef DEBUGZ

#endif

   if ( psChallenge == NULL )
      return;

   /* sPromptId */
   DBVFree_compulsory_DbvPromptId( &psChallenge->sPromptId );

   /* psDefaultResponse */
   DBVFreeDbvInternationalString( psChallenge->psDefaultResponse );

   /* psPromptInfo */
   DBVFree_DbvPromptInfo( psChallenge->psPromptInfo );

   /* psRegExpr */
   DBVFreeDbvInternationalString( psChallenge->psRegExpr );

   /* pvResponseRequired */
   DBVFree_DbvNull( psChallenge->pvResponseRequired );

   /* ppsAllowedValues */
   FreeN_DbvInternationalString( psChallenge->ppsAllowedValues );

   /* pvShouldSave */
   DBVFree_DbvNull( psChallenge->pvShouldSave );

   /* peDataType */
   DBVFree_DbvDataType( psChallenge->peDataType );

   /* psDiagnostic */
   DBVFreeDbvASN1External( psChallenge->psDiagnostic );

   DbvFree( psChallenge );
   } /* End DBVFree_DbvChallenge() */ 



void DBVFree_DbvResponse( DbvResponse *psResponse )
   {
#ifdef DEBUGZ

#endif

   if ( psResponse == NULL )
      return;

   /* sPromptId */
   DBVFree_compulsory_DbvPromptId( &psResponse->sPromptId );

   /* sPromptResponse */
   DBVFree_compulsory_DbvPromptResponse( &psResponse->sPromptResponse );

   DbvFree( psResponse );
   } /* DBVFree_DbvResponse() */



void DBVFree_DbvPromptObject( DbvPromptObject *psPromptObject )
   {
#ifdef DEBUGZ

#endif

   if ( psPromptObject == NULL )
      return;


   /* uPromptObject */

   switch( psPromptObject->ePromptObjectType ) 
      {
      case POT_Challenge:
   	 FreeN_DbvChallenge( psPromptObject->uPromptObject.ppsChallenge );
	 break;

      case POT_Response:
   	 FreeN_DbvResponse( psPromptObject->uPromptObject.ppsResponse );
	 break;

      default:
	 break;
      } /* End switch */

   DbvFree( psPromptObject );
   } /* End DBVFree_DbvPromptObject() */



/*-------------------- RESOURCECONTROL --------------------*/


void DBVFree_DbvPartialResultsAvailable( DbvPartialResultsAvailable *pePartialResultsAvailable )
   {
#ifdef DEBUGZ

#endif
     
   if ( pePartialResultsAvailable == NULL )
      return;

   DbvFree( pePartialResultsAvailable );
   } /* End DBVFree_DbvPartialResultsAvailable() */



void DBVFree_DbvResourceControlReq( DbvResourceControlReq *psResourceControlReq )
   {
#ifdef DEBUGZ

#endif

   if ( psResourceControlReq == NULL )
      return;

   /* psReferenceId */
   DBVFreeDbvOctetString( psResourceControlReq->psReferenceId );

   /* pfSuspendedFlag */
   DBVFree_DbvBool( psResourceControlReq->pfSuspendedFlag );

   /* psResourceReport */
   DBVFreeDbvASN1External( psResourceControlReq->psResourceReport );

   /* pePartialResultsAvailable */
   DBVFree_DbvPartialResultsAvailable( psResourceControlReq->pePartialResultsAvailable );

   /* compulsory fResponseRequired */

   /* pfTriggeredRequestFlag */
   DBVFree_DbvBool( psResourceControlReq->pfTriggeredRequestFlag );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlReq->ppsOtherInformation );

   DbvFree( psResourceControlReq );
   } /* End DBVFree_DbvResourceControlReq() */ 




void DBVFree_DbvResourceControlRsp( DbvResourceControlRsp *psResourceControlRsp )
   {
#ifdef DEBUGZ

#endif

   if ( psResourceControlRsp == NULL )
      return;

   /* psReferenceId */
   DBVFreeDbvOctetString( psResourceControlRsp->psReferenceId );

   /* compulsory fContinueFlag */

   /* pfResultSetWanted */
   DBVFree_DbvBool( psResourceControlRsp->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psResourceControlRsp->ppsOtherInformation );

   DbvFree( psResourceControlRsp );
   } /* End DBVFree_DbvResourceControlRsp() */ 




/*-------------------- TRIGGERRESOURCECONTROL --------------------*/


void DBVFree_DbvTriggerResourceControlReq( DbvTriggerResourceControlReq *psTriggerResourceControlReq )
   {
#ifdef DEBUGZ

#endif

   if ( psTriggerResourceControlReq == NULL )
      return;

   /* psReferenceId */
   DBVFreeDbvOctetString( psTriggerResourceControlReq->psReferenceId );

   /* compulsory eRequestedAction */

   /* pcPreferredResourceReportFormat */
   DBVFreeDbvObjectIdentifier( psTriggerResourceControlReq->pcPreferredResourceReportFormat );

   /* pfResultSetWanted */
   DBVFree_DbvBool( psTriggerResourceControlReq->pfResultSetWanted );

   /* ppsOtherInformation */
   FreeN_DbvOtherInformation( psTriggerResourceControlReq->ppsOtherInformation );

   DbvFree( psTriggerResourceControlReq );
   } /* End DBVFree_DbvTriggerResourceControlReq() */ 


/*------------------------------------*/


void DBVFree_DbvElementSpec(DbvElementSpec *psElementSpec)                 
{
#ifdef DEBUGZ

#endif

   if ( psElementSpec == NULL )
      return;

   switch(psElementSpec->eElementSpecType)
   {
   case EST_Primitive:
           DBVFreeDbvInternationalString(psElementSpec->
                   uElementSpec.psPrimitive);
           break;
   case EST_External:
           DBVFreeDbvASN1External(psElementSpec->
                   uElementSpec.psExternal);
           break;
   default:
           break;
   }
} /* DBVFree_DbvElementSpec() */




