/*  Module Name: @(#)z3950_externals.c	7.6
    Version: 12/4/96
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: DBV-OSI II externals setup/get code
*/


#include "z3950_externals.h"


/**********************************************************************
External implementation notes:


To test an unknown external:
1) #ifdef out relevant 'if' in DBVunknown_external() in this file
2) #ifdef out relevant 'if' in DBVsnacc_to_dbv_anytype() in this file

 
To test an unknown PDU:
1) #ifdef out relevant case statements in BDecPDUContent() in file z39_50_apdu_1995.c



Design structure for each external set up function:
(snacc ILL definition contains AnyDefinedBy types which are not 
EXTERNAL types - these map onto DBV-OSI II EXTERNAL always however)

dbv_to_snacc ANSI top routine
dbv_to_snacc ISO top routine
dbv_to_snacc support routines (as needed)
   
snacc_to_dbv ANSI top routine
snacc_to_dbv ISO top routine
snacc_to_dbv support routines (as needed)

Checking functions 



General schema for setting up and checking: 
1) check if structure is not null - if so call new function to set
   up/check function  

If a type of checking/set up is required more than once then null checking
should be done in the set up/check function

In some of the (older) structure checking functions, even though only 
used once, pointers are checked within structure 
checking functions rather than before the functions are called

**********************************************************************/

   
   
/* The following function (DBVinit_external_data()) only needs to be 
   called once at the start of the program to set up the hash tables 
   for the externals */
int DBVinit_external_data()
{

	/* Initialize the hash table for the
        * ANY DEFINED BY type mappings (used by NEWEXTERNAL).
        * This only needs to be done once per execution
        * (before any encoding or decoding is done)
        */
 
        InitAnyZ39_50_EXTERNALS_A_1995();
        InitAnyZ39_50_EXTERNALS_B_1995();
        InitAnyZ39_50_EXTERNALS_D_1995();
        InitAnyZ39_50_EXTERNALS_E_1995();
        InitAnyZ39_50_EXTERNALS_F_1995();
	InitAnyZ39_50_EXTERNALS_ISO_1995();
	InitAnyISO_10161_ILL_1();
	InitAnyIORDEXT();

	/* This contains the silly int->any mappings used by the Extension 
	   structure in ILL as well as the Unknown External definition */
	InitAnyZ39_50_EXTERNALS_EXTRA_1995();
	InitAnyZ39_50_EXTERNALS_SIL_1995();
	InitAnyZ39_50_EXTERNALS_CHARSET_1995();

}



/* The following function (DBV_reset_external_counters()) needs to be 
   called before each PDU is encoded */

/* Reset all the global SingleASN1Type external counters so that for 
   the next PDU to be encoded all external arrays start at 0.

   DBV_reset_external_counters is called by function DBVencode_and_send in file
   z3950_send_receive.c

   global are defined in z3950_globals.c
*/

int DBV_reset_external_counters(void)
{
	/* When changing the following remember to alter the
	   extern ints in header file z3950_includes.h */

	/* Following order should corresspond to order in z3950_globals.c */


/* Ansi */

        current_explain = 0;
        current_sutrs = 0;
        current_opac = 0;
        current_summary = 0;

        current_generic = 0;

        current_taskpack = 0;

        current_resrep1 = 0;
        current_resrep2 = 0;

        current_promptobj = 0;

        current_itemorder = 0;
        current_update = 0;

/* extra */
	current_silintmaptest = 0;


	current_print = 0;
	current_alert = 0;
	current_corder = 0;

        current_charsetandlang = 0;



/*-----------------------------*/
/* Iso */

        current_isosutrs = 0;
        current_isoopac = 0;
        current_isosummary = 0;
        current_isotaskpack = 0;
        current_isoresrep1 = 0;
        current_isoresrep2 = 0;
        current_isoitemorder = 0;
        current_isoupdate = 0;
        current_isoexplain = 0;

        current_isogeneric = 0;



/* iso_10161_ill_1.c */ 
        current_isoill = 0;

/* iordext.h */
	current_subito = 0;

        current_isopromptobj = 0;


        current_isocharsetandlang = 0;



	/* When changing the above remember to alter the
	   extern ints in header file z3950_includes.h */
 

}



/* Check to see if an external is unknown */
int DBVunknown_external(char *oid)
{
        if ( 
             (strcmp(oid, ISO_EXPLAIN_RECORD) != 0) &&
             (strcmp(oid, ANSI_EXPLAIN_RECORD) != 0) &&
             (strcmp(oid, ISO_UNSTRUCTURED_TEXT_RECORD) != 0) &&
	     (strcmp(oid, ANSI_UNSTRUCTURED_TEXT_RECORD) != 0) &&
             (strcmp(oid, ISO_OPAC_RECORD) != 0) &&
             (strcmp(oid, ANSI_OPAC_RECORD) != 0) &&
	     (strcmp(oid, ISO_SUMMARY_RECORD) != 0) &&
	     (strcmp(oid, ANSI_SUMMARY_RECORD) != 0) &&

             (strcmp(oid, ISO_GENERIC_RECORD) != 0) &&
             (strcmp(oid, ANSI_GENERIC_RECORD) != 0) &&

	     (strcmp(oid, ISO_ES_TASK_PACKAGE_RECORD) != 0) &&
	     (strcmp(oid, ANSI_ES_TASK_PACKAGE_RECORD) != 0) &&

	     (strcmp(oid, ISO_RESOURCE_REPORT_1) != 0) &&
	     (strcmp(oid, ANSI_RESOURCE_REPORT_1) != 0) &&
	     (strcmp(oid, ISO_RESOURCE_REPORT_2) != 0) && 
	     (strcmp(oid, ANSI_RESOURCE_REPORT_2) != 0) &&

	     (strcmp(oid, ISO_ITEMORDER_EXTENDED_SERVICE) != 0) && 
	     (strcmp(oid, ANSI_ITEMORDER_EXTENDED_SERVICE) != 0) &&
	     (strcmp(oid, ISO_ILL_PDU) != 0) &&

	     (strcmp(oid, ISO_UPDATE_EXTENDED_SERVICE) != 0) &&
	     (strcmp(oid, ANSI_UPDATE_EXTENDED_SERVICE) != 0) &&


             (strcmp(oid, ESRIN_PRINT) != 0) &&
             (strcmp(oid, ESRIN_ALERT) != 0) &&
             (strcmp(oid, ESRIN_ORDER) != 0) &&

             (strcmp(oid, ISO_CHARSET_AND_LANGUAGE_NEGOTIATION) != 0) &&
             (strcmp(oid, ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION) != 0) &&



             (strcmp(oid, ISO_ACCESS_PROMPT_1) != 0) &&
             (strcmp(oid, ANSI_ACCESS_PROMPT_1) != 0)
	) 

                /* then we have an unknown external */
                return 1;
 
	/* else we have found an external we can decode by snacc hash tables */
        return 0;
}



void DBVdbv_to_snacc_DbvASN1External_comp(NEWEXTERNAL **snacc_external, DbvASN1External *dbv_external, jmp_buf env)
{
	*snacc_external = (NEWEXTERNAL *) Asn1Alloc (sizeof (NEWEXTERNAL));
	CheckAsn1Alloc(*snacc_external, env);
	DBVdbv_to_snacc_DbvASN1External(*snacc_external, dbv_external, env);
}



void DBVdbv_to_snacc_DbvASN1External(NEWEXTERNAL *snacc_external, DbvASN1External *dbv_external, jmp_buf env)
{
	DBVdbv_to_snacc_external(snacc_external, dbv_external, env);
}



/* dbv -> snacc external */
int DBVdbv_to_snacc_external(NEWEXTERNAL *snacc_external, DbvASN1External *dbv_external, jmp_buf env)
{
	char *oid = NULL;

       	oid = dbv_external->pcObjectIdentifier;

	/* Check if the oid is present (it is optional) */
	if (oid != NULL)
       		ASN_MakeOid(oid, &snacc_external->direct_reference, env);
	switch (dbv_external->eValueType) {
	case EVT_SingleASN1Type:
		DBVdbv_to_snacc_anytype(oid, &snacc_external->single_ASN1_type, dbv_external, env);
		break;
	case EVT_OctetAligned:
               	DBVdbv_to_snacc_aligned(snacc_external, dbv_external, env);
		break;
	case EVT_Arbitrary:
               	DBVdbv_to_snacc_arbitrary(snacc_external, dbv_external, env);
		break;
	}
}



/* Always map from a DBV-OSI II external to a snacc any */
int DBVdbv_to_snacc_anytype(char *oid, AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	switch(dbv_external->uExternalValue.psSingleASN1Type->eASN1Type) {
       	case AT_UnstructuredText:
		if (strcmp(oid, ANSI_UNSTRUCTURED_TEXT_RECORD) == 0)
            		DBVdbv_to_snacc_sutrs_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_UNSTRUCTURED_TEXT_RECORD) == 0)
               		DBVdbv_to_snacc_isosutrs_any(any_type, dbv_external, env);
		break;
	case AT_OpacRecord:
		if (strcmp(oid, ANSI_OPAC_RECORD) == 0)
               		DBVdbv_to_snacc_opac_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_OPAC_RECORD) == 0)
               		DBVdbv_to_snacc_isoopac_any(any_type, dbv_external, env);
		break;
	case AT_SummaryRecord:
		if (strcmp(oid, ANSI_SUMMARY_RECORD) == 0)
               		DBVdbv_to_snacc_summary_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_SUMMARY_RECORD) == 0)
               		DBVdbv_to_snacc_isosummary_any(any_type, dbv_external, env);
		break;
        case AT_ResourceReport1:
		if (strcmp(oid, ANSI_RESOURCE_REPORT_1) == 0)
              		DBVdbv_to_snacc_resrep1_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_RESOURCE_REPORT_1) == 0)
               		DBVdbv_to_snacc_isoresrep1_any(any_type, dbv_external, env);
		break;
	case AT_ResourceReport2:
		if (strcmp(oid, ANSI_RESOURCE_REPORT_2) == 0)
               		DBVdbv_to_snacc_resrep2_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_RESOURCE_REPORT_2) == 0)
               		DBVdbv_to_snacc_isoresrep2_any(any_type, dbv_external, env);
		break;
	case AT_EsTaskPackage:
		if (strcmp(oid, ANSI_ES_TASK_PACKAGE_RECORD) == 0)
              		DBVdbv_to_snacc_estaskpack_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_ES_TASK_PACKAGE_RECORD) == 0)
               		DBVdbv_to_snacc_isoestaskpack_any(any_type, dbv_external, env);
		break;
	case AT_UpdateExtendedService:
		if (strcmp(oid, ANSI_UPDATE_EXTENDED_SERVICE) == 0)
               		DBVdbv_to_snacc_update_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_UPDATE_EXTENDED_SERVICE) == 0)
               		DBVdbv_to_snacc_isoupdate_any(any_type, dbv_external, env);
		break;

	case AT_ItemOrderExtendedService:
		if (strcmp(oid, ANSI_ITEMORDER_EXTENDED_SERVICE) == 0)
               		DBVdbv_to_snacc_itemorder_any(any_type, dbv_external, env);
		if (strcmp(oid, ISO_ITEMORDER_EXTENDED_SERVICE) == 0)
               		DBVdbv_to_snacc_isoitemorder_any(any_type, dbv_external, env);
		break;
	case AT_IllPdu:
		if (strcmp(oid, ISO_ILL_PDU) == 0)
               		DBVdbv_to_snacc_isoillrequest_any(any_type, dbv_external, env);
		break;

        case AT_ExplainRecord:
                if (strcmp(oid, ANSI_EXPLAIN_RECORD) == 0)
                        DBVdbv_to_snacc_explain_any(any_type, dbv_external, env);
                if (strcmp(oid, ISO_EXPLAIN_RECORD) == 0)
                        DBVdbv_to_snacc_isoexplain_any(any_type, dbv_external, env);

                break;

        case AT_GenericRecord:
                if (strcmp(oid, ANSI_GENERIC_RECORD) == 0)
                        DBVdbv_to_snacc_generic_any(any_type, dbv_external, env);
                if (strcmp(oid, ISO_GENERIC_RECORD) == 0)
                        DBVdbv_to_snacc_isogeneric_any(any_type, dbv_external, env);

                break;



        case AT_PromptObject:
                if (strcmp(oid, ANSI_ACCESS_PROMPT_1) == 0)
                        DBVdbv_to_snacc_DbvPromptObject_any(any_type, dbv_external, env);
                if (strcmp(oid, ISO_ACCESS_PROMPT_1) == 0)
                        DBVdbv_to_snacc_isoDbvPromptObject_any(any_type, dbv_external, env);

                break;



        case AT_Print:
                if (strcmp(oid, ESRIN_PRINT) == 0)
                        DBVdbv_to_snacc_DbvPrintExtendedService_any(any_type, dbv_external, env);
                break;

        case AT_Alert:
                if (strcmp(oid, ESRIN_ALERT) == 0)
                        DBVdbv_to_snacc_DbvAlertExtendedService_any(any_type, dbv_external, env);
                break;

        case AT_Corder:
                if (strcmp(oid, ESRIN_ORDER) == 0)
                        DBVdbv_to_snacc_DbvCOrder_any(any_type, dbv_external, env);
                break;




        case AT_CharSetAndLanguage:
                if (strcmp(oid, ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0)
                        DBVdbv_to_snacc_DbvCharSetAndLanguage_any(any_type, dbv_external, env);
                if (strcmp(oid, ISO_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0)
                        DBVdbv_to_snacc_isoDbvCharSetAndLanguage_any(any_type, dbv_external, env);

                break;


	case AT_UnknownType:
		break;
	}
}



void DBVsnacc_to_dbv_DbvASN1External_comp(DbvASN1External **dbv_external, NEWEXTERNAL *snacc_external, jmp_buf env)
{
	*dbv_external = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
	CheckDbvAlloc(*dbv_external, env);
	DBVsnacc_to_dbv_DbvASN1External(*dbv_external, snacc_external, env);
}



void DBVsnacc_to_dbv_DbvASN1External(DbvASN1External *dbv_external, NEWEXTERNAL *snacc_external, jmp_buf env)
{
	DBVsnacc_to_dbv_external(dbv_external, snacc_external, env);
}



/* snacc -> dbv external */
int DBVsnacc_to_dbv_external(DbvASN1External *dbv_external, NEWEXTERNAL *snacc_external, jmp_buf env)
{
	char direct_reference[SA_ARRAY_SIZE];
	char *string = NULL;
	unsigned long length;

	/* If an oid exists (it is optional) */
	if (snacc_external->direct_reference.octs != NULL) {
		memset(direct_reference, 0, sizeof(direct_reference));

		DBVsa_strcpy(direct_reference, (char *) ASN_EncodedToString(&snacc_external->direct_reference));

		/* setup dbv external oid from a char* */
		DBVsetup_dbv_externaloid(dbv_external, direct_reference, env);
	}


	/* Now extract stuff from the NEWEXTERNAL structure, it isnt
	   the standard choice but three optionals */

	/* If there is a single_ASN1_type */
	if (snacc_external->single_ASN1_type.value != NULL) {
		DBVsnacc_to_dbv_anytype(direct_reference, dbv_external, &snacc_external->single_ASN1_type, env);
	}

	/* If there is an octet_aligned (as used by V2 MARC implementations) */
	if (snacc_external->octet_aligned.octs != NULL) {
		DBVsnacc_to_dbv_aligned(dbv_external, snacc_external, env);

	}

	/* If there is a psArbitrary (unlikely as not used 
 	   by V2 or V3 implementations !) */
	if (snacc_external->arbitrary.bits != NULL) {
		DBVsnacc_to_dbv_arbitrary(dbv_external, snacc_external, env);
	}
}



int DBVsetup_dbv_externaloid(DbvASN1External *dbv_external, char *direct_reference, jmp_buf env)
{
	/* Add the oid to the dbv structure: +1 to copy the \0 */
	if (strlen(direct_reference) != 0) {

		dbv_external->pcObjectIdentifier = (char *) DbvAlloc (strlen(direct_reference) + 1);
		CheckDbvAlloc(dbv_external->pcObjectIdentifier, env);

		memcpy(dbv_external->pcObjectIdentifier, direct_reference, strlen(direct_reference));
	}
}



/* Convert snacc external to dbv external */

/* Always map from a snacc any to a DBV-OSI II external */
int DBVsnacc_to_dbv_anytype(char *direct_reference, DbvASN1External *dbv_external, AsnAnyDefinedBy *any_type, jmp_buf env)
{
        dbv_external->eValueType = EVT_SingleASN1Type;
        dbv_external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type *) DbvAlloc (sizeof (DbvSingleASN1Type));
	CheckDbvAlloc(dbv_external->uExternalValue.psSingleASN1Type, env);



	if (strcmp(direct_reference, ANSI_UNSTRUCTURED_TEXT_RECORD) == 0)
                DBVsnacc_to_dbv_sutrs_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);


	else if (strcmp(direct_reference, ISO_UNSTRUCTURED_TEXT_RECORD) == 0)
                DBVsnacc_to_dbv_isosutrs_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);


        else if (strcmp(direct_reference, ANSI_OPAC_RECORD) == 0) 
                DBVsnacc_to_dbv_opac_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);


        else if (strcmp(direct_reference, ISO_OPAC_RECORD) == 0) 
                DBVsnacc_to_dbv_isoopac_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_SUMMARY_RECORD) == 0) 
                DBVsnacc_to_dbv_summary_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_SUMMARY_RECORD) == 0) 
                DBVsnacc_to_dbv_isosummary_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_RESOURCE_REPORT_1) == 0) 
                DBVsnacc_to_dbv_resrep1_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_RESOURCE_REPORT_1) == 0) 
                DBVsnacc_to_dbv_isoresrep1_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_RESOURCE_REPORT_2) == 0) 
                DBVsnacc_to_dbv_resrep2_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_RESOURCE_REPORT_2) == 0) 
                DBVsnacc_to_dbv_isoresrep2_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_ES_TASK_PACKAGE_RECORD) == 0)
                DBVsnacc_to_dbv_estaskpack_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_ES_TASK_PACKAGE_RECORD) == 0)
                DBVsnacc_to_dbv_isoestaskpack_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);


        else if (strcmp(direct_reference, ANSI_UPDATE_EXTENDED_SERVICE) == 0) 
                DBVsnacc_to_dbv_update_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_UPDATE_EXTENDED_SERVICE) == 0) 
                DBVsnacc_to_dbv_isoupdate_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_ITEMORDER_EXTENDED_SERVICE) == 0)
                DBVsnacc_to_dbv_itemorder_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_ITEMORDER_EXTENDED_SERVICE) == 0)
                DBVsnacc_to_dbv_isoitemorder_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_ILL_PDU) == 0) 
                DBVsnacc_to_dbv_isoillrequest_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);




        else if (strcmp(direct_reference, ANSI_EXPLAIN_RECORD) == 0)
                DBVsnacc_to_dbv_explain_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_EXPLAIN_RECORD) == 0)
                DBVsnacc_to_dbv_isoexplain_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);



        else if (strcmp(direct_reference, ANSI_GENERIC_RECORD) == 0)
                DBVsnacc_to_dbv_generic_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_GENERIC_RECORD) == 0)
                DBVsnacc_to_dbv_isogeneric_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);




        else if (strcmp(direct_reference, ANSI_ACCESS_PROMPT_1) == 0)
                DBVsnacc_to_dbv_DbvPromptObject_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_ACCESS_PROMPT_1) == 0)
                DBVsnacc_to_dbv_isoDbvPromptObject_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);





       else if (strcmp(direct_reference, ESRIN_PRINT) == 0)
                DBVsnacc_to_dbv_DbvPrintExtendedService_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ESRIN_ALERT) == 0)
                DBVsnacc_to_dbv_DbvAlertExtendedService_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ESRIN_ORDER) == 0)
                DBVsnacc_to_dbv_DbvCOrder_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);






        else if (strcmp(direct_reference, ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0)
                DBVsnacc_to_dbv_DbvCharSetAndLanguage_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

        else if (strcmp(direct_reference, ISO_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0)
                DBVsnacc_to_dbv_isoDbvCharSetAndLanguage_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);





	/* else got an unknown type (one for which no hash table mapping
	   has been set up) */
	else 

		/* any_type is prepared as hex in the snacc decoding functions 
                   if it is unknown */
		DBVsnacc_to_dbv_unknown_any(dbv_external->uExternalValue.psSingleASN1Type, any_type, env);

	
}




/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/


/* sutrs */


/* dbv -> snacc ansi sutrs external */


int DBVdbv_to_snacc_sutrs_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{

	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static SutrsRecord sutrs_external[STRING_LEN];
 
        SutrsRecord *sutrs_ptr;

	/* Set up the Sutrs external */
        sutrs_ptr = &sutrs_external[current_sutrs];

	/* Clear the location (may have been used by the last PDU) */
	memset(sutrs_ptr, 0, sizeof(SutrsRecord));

	/* Convert a dbv international string into a snacc international 
	   string  */
	DBVdbv_to_snacc_inter_string(sutrs_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &sutrs_external[current_sutrs];


	/* Only if not at end of array */
	if (current_sutrs < (STRING_LEN - 1)) {

		/* Increment current_sutrs for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_sutrs++;
	}
}
 


/* dbv -> snacc iso sutrs external */

int DBVdbv_to_snacc_isosutrs_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{

	static IsoSutrsRecord isosutrs_external[STRING_LEN];
        IsoSutrsRecord *isosutrs_ptr;

        isosutrs_ptr = &isosutrs_external[current_isosutrs];
	memset(isosutrs_ptr, 0, sizeof(IsoSutrsRecord));
	DBVdbv_to_snacc_inter_string(isosutrs_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText, env);
	any_type->value = (void *) &isosutrs_external[current_isosutrs];


	/* Only if not at end of array */
	if (current_isosutrs < (STRING_LEN - 1)) {
		current_isosutrs++;
	}

}
 


/* snacc -> dbv ansi sutrs external */
int DBVsnacc_to_dbv_sutrs_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
 
	SutrsRecord *sutrs_ptr;
 
	sutrs_ptr = (SutrsRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_UnstructuredText;
        psSingleASN1Type->uASN1.psUnstructuredText = (struct DbvInternationalString *) DbvAlloc (sizeof(struct DbvInternationalString));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psUnstructuredText, env);
 

	/* snacc inter str to dbv inter str */
	DBVsnacc_to_dbv_inter_string(psSingleASN1Type->uASN1.psUnstructuredText, sutrs_ptr, env);

}



/* snacc -> dbv iso sutrs external */
int DBVsnacc_to_dbv_isosutrs_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
 
	IsoSutrsRecord *isosutrs_ptr;
 
	isosutrs_ptr = (IsoSutrsRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_UnstructuredText;
        psSingleASN1Type->uASN1.psUnstructuredText = (struct DbvInternationalString *) DbvAlloc (sizeof(struct DbvInternationalString));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psUnstructuredText, env);
 

	/* snacc inter str to dbv inter str */
	DBVsnacc_to_dbv_inter_string(psSingleASN1Type->uASN1.psUnstructuredText, isosutrs_ptr, env);

}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/



/* opac */


/* dbv -> snacc ansi opac external */

int DBVdbv_to_snacc_opac_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
 
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static OPACRecord opac_external[STRING_LEN];
 
        OPACRecord *opac_ptr;

	/* Set up the Opac external */
        opac_ptr = &opac_external[current_opac];

	/* Clear the location (may have been used by the last PDU) */
	memset(opac_ptr, 0, sizeof(OPACRecord));

        /* johnfix is necessary becuase when decoding an ANY/ANYDEFINEDBY
           type, snacc mallocs, not callocs, space for the top
           structure of the type (see asn_any.c).
           This means that if some optional fields do not exist in an
           ANY/ANYDEFINEDBY type, these fields may contain garbage
           after decoding, as left by the malloc
           (e.g. wierd octetlen values, strange octs pointer values etc).
 
           There are two solutions to this problem:
           a) change the malloc(size) statements in asn_any.c to calloc(1, size)
           b) Use a dummy choice within the ASN.1 code which tricks snacc
              into callocing space for the structure using Asn1Alloc (this
              will work if Asn1Alloc is #defined as calloc(1, size), as it
              is in DBV-OSI II).
              This dummy choice does not effect the BER encoding,
              only the structures snacc uses around it.
 
 
           I prefer the dummy choice rather than customizing snacc.
           For the Generic record type which has a list as its top
           structure, the dummy choice must be used as snacc get confused.
        */

	opac_ptr->choiceId = OPACRECORD_JOHNFIX;
 
        opac_ptr->a.johnfix = (struct OrigOPACRecord*) Asn1Alloc (sizeof(struct OrigOPACRecord));
        CheckAsn1Alloc(opac_ptr->a.johnfix, env);

	DBVdbv_to_snacc_opac_main(opac_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psOpacRecord, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &opac_external[current_opac];

	/* Only if not at end of array */
	if (current_opac < (STRING_LEN - 1)) {

		/* Increment current_sutrs for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_opac++;
	}

}
 
 
/* dbv -> snacc iso opac external */

int DBVdbv_to_snacc_isoopac_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoOPACRecord isoopac_external[STRING_LEN];
        IsoOPACRecord *isoopac_ptr;

        isoopac_ptr = &isoopac_external[current_isoopac];
	memset(isoopac_ptr, 0, sizeof(IsoOPACRecord));
	isoopac_ptr->choiceId = OPACRECORD_JOHNFIX;
        isoopac_ptr->a.johnfix = (struct OrigOPACRecord*) Asn1Alloc (sizeof(struct OrigOPACRecord));
        CheckAsn1Alloc(isoopac_ptr->a.johnfix, env);


	DBVdbv_to_snacc_opac_main(isoopac_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psOpacRecord, env);

	any_type->value = (void *) &isoopac_external[current_isoopac];

	if (current_isoopac < (STRING_LEN - 1)) {
		current_isoopac++;
	}

}


int DBVdbv_to_snacc_opac_main(OrigOPACRecord *opac_ptr, struct DbvOpacRecord *psOpacRecord, jmp_buf env)
{
	/* bibliographicRecord */
	if (psOpacRecord->psBibliographicRecord != NULL) {

		/* Alloc the snacc external */
		opac_ptr->bibliographicRecord = (NEWEXTERNAL *) Asn1Alloc(sizeof(NEWEXTERNAL));
		CheckAsn1Alloc(opac_ptr->bibliographicRecord, env);
		DBVdbv_to_snacc_external(opac_ptr->bibliographicRecord, psOpacRecord->psBibliographicRecord, env);
	}


	/* holdingsData */
	if (psOpacRecord->ppsHoldingsData != NULL) {
		opac_ptr->holdingsData = AsnListNew (sizeof(void *));	
		CheckAsn1Alloc(opac_ptr->holdingsData, env);
		DBVdbv_to_snacc_holdings(opac_ptr->holdingsData, psOpacRecord->ppsHoldingsData, env);
	}
}



int DBVdbv_to_snacc_holdings(OrigOPACRecordSeqOf *holdingsData, DbvHoldingsRecord **ppsHoldingsData, jmp_buf env)
{
	HoldingsRecord *data_ptr;	
	HoldingsRecord **data_ptr_address;	

	int current_listnode = 0;

	/* while not at end of dbv list */
	while (ppsHoldingsData[current_listnode] != NULL) {
		data_ptr_address = AsnListAppend(holdingsData);
		CheckAsn1Alloc(data_ptr_address, env);

		*data_ptr_address = Asn1Alloc(sizeof(HoldingsRecord));
		CheckAsn1Alloc(*data_ptr_address, env);
		
		data_ptr = *data_ptr_address;
 
		DBVdbv_to_snacc_holding_rec(data_ptr, ppsHoldingsData[current_listnode], env);
		current_listnode++;
	}
}



/* Set up each snacc HoldingsRecord */
int DBVdbv_to_snacc_holding_rec(HoldingsRecord *data_ptr, DbvHoldingsRecord *psHoldingsData, jmp_buf env)
{
	switch (psHoldingsData->eHoldingsRecType) {
	case HRT_MarcHoldingsRecord:
		data_ptr->choiceId = HOLDINGSRECORD_MARCHOLDINGSRECORD;
		data_ptr->a.marcHoldingsRecord = (struct NEWEXTERNAL*) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(data_ptr->a.marcHoldingsRecord, env);
		DBVdbv_to_snacc_external(data_ptr->a.marcHoldingsRecord, &psHoldingsData->uMarcOrHoldAndCirc.sMarcHoldingsRecord, env);
		break;
	case HRT_HoldingsAndCirc:
		data_ptr->choiceId = HOLDINGSRECORD_HOLDINGSANDCIRC;
		data_ptr->a.holdingsAndCirc = (struct HoldingsAndCircData*) Asn1Alloc (sizeof(struct HoldingsAndCircData));
		CheckAsn1Alloc(data_ptr->a.holdingsAndCirc, env);

		DBVdbv_to_snacc_holdings_bulk(data_ptr->a.holdingsAndCirc, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc, env);

		DBVdbv_to_snacc_hold_volume(data_ptr->a.holdingsAndCirc, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc, env);

		DBVdbv_to_snacc_hold_circ(data_ptr->a.holdingsAndCirc, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc, env);



		break;
	}
	
}



int DBVdbv_to_snacc_holdings_bulk(struct HoldingsAndCircData *holdingsAndCirc, DbvHoldingsAndCircData sHoldingsAndCirc, jmp_buf env)
{

	/* psTypeOfRecord */
	if (sHoldingsAndCirc.psTypeOfRecord != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->typeOfRecord), sHoldingsAndCirc.psTypeOfRecord, env);
			

	/* psEncodingLevel */
	if (sHoldingsAndCirc.psEncodingLevel != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->encodingLevel), sHoldingsAndCirc.psEncodingLevel, env);
			

	/* psFormat */
	if (sHoldingsAndCirc.psFormat != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->format), sHoldingsAndCirc.psFormat, env);
			

	/* psReceiptAcqStatus */
	if (sHoldingsAndCirc.psReceiptAcqStatus != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->receiptAcqStatus), sHoldingsAndCirc.psReceiptAcqStatus, env);
			
	/* psGeneralRetention */
	if (sHoldingsAndCirc.psGeneralRetention != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->generalRetention), sHoldingsAndCirc.psGeneralRetention, env);
			

	/* psCompleteness */
	if (sHoldingsAndCirc.psCompleteness != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->completeness), sHoldingsAndCirc.psCompleteness, env);
			

	/* psDateOfReport */
	if (sHoldingsAndCirc.psDateOfReport != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->dateOfReport), sHoldingsAndCirc.psDateOfReport, env);
			

	/* psNucCode */
	if (sHoldingsAndCirc.psNucCode != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->nucCode), sHoldingsAndCirc.psNucCode, env);
			

	/* psLocalLocation */
	if (sHoldingsAndCirc.psLocalLocation != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->localLocation), sHoldingsAndCirc.psLocalLocation, env);
			

	/* psShelvingLocation */
	if (sHoldingsAndCirc.psShelvingLocation != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->shelvingLocation), sHoldingsAndCirc.psShelvingLocation, env);
			

	/* psCallNumber */
	if (sHoldingsAndCirc.psCallNumber != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->callNumber), sHoldingsAndCirc.psCallNumber, env);
			
	/* psShelvingData */
	if (sHoldingsAndCirc.psShelvingData != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->shelvingData), sHoldingsAndCirc.psShelvingData, env);
			

	/* psCopyNumber */
	if (sHoldingsAndCirc.psCopyNumber != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->copyNumber), sHoldingsAndCirc.psCopyNumber, env);
			


	/* psPublicNote */
	if (sHoldingsAndCirc.psPublicNote != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->publicNote), sHoldingsAndCirc.psPublicNote, env);
			

	/* psReproductionNote */
	if (sHoldingsAndCirc.psReproductionNote != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->reproductionNote), sHoldingsAndCirc.psReproductionNote, env);
			

	/* psTermsUseRepro */
	if (sHoldingsAndCirc.psTermsUseRepro != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->termsUseRepro), sHoldingsAndCirc.psTermsUseRepro, env);
			
	/* psEnumAndChron */
	if (sHoldingsAndCirc.psEnumAndChron != NULL) 
		DBVdbv_to_snacc_inter_string(&(holdingsAndCirc->enumAndChron), sHoldingsAndCirc.psEnumAndChron, env);
			


}


int DBVdbv_to_snacc_hold_volume(struct HoldingsAndCircData *holdingsAndCirc, DbvHoldingsAndCircData sHoldingsAndCirc, jmp_buf env)
{
	
	Volume *data_ptr;
	Volume **data_ptr_address;

	int current_listnode = 0;

	if (sHoldingsAndCirc.ppsVolume != NULL) {

		/* Create the snacc list */
		holdingsAndCirc->volumes = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(holdingsAndCirc->volumes, env);

		while (sHoldingsAndCirc.ppsVolume[current_listnode] != NULL) {

			/* Create a list node */
	                data_ptr_address = AsnListAppend(holdingsAndCirc->volumes);
			CheckAsn1Alloc(data_ptr_address, env);

                	*data_ptr_address = Asn1Alloc(sizeof(Volume));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;

			DBVdbv_to_snacc_vol_struct(data_ptr, sHoldingsAndCirc.ppsVolume[current_listnode], env);
			current_listnode++;
		}
	}
}



int DBVdbv_to_snacc_vol_struct(Volume *data_ptr, DbvVolume *psVolume, jmp_buf env)
{
	/* Now the volune structure - all optional... */

	if (psVolume->psEnumeration != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->enumeration, psVolume->psEnumeration, env);
		
	if (psVolume->psChronology != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->chronology, psVolume->psChronology, env);
		
	if (psVolume->psEnumAndChron != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->enumAndChron, psVolume->psEnumAndChron, env);
		
}



int DBVdbv_to_snacc_hold_circ(struct HoldingsAndCircData *holdingsAndCirc, DbvHoldingsAndCircData sHoldingsAndCirc, jmp_buf env)
{
	CircRecord *data_ptr;
	CircRecord **data_ptr_address;

	int current_listnode = 0;

	if (sHoldingsAndCirc.ppsCircRecord != NULL) {

		/* Create the snacc list */
		holdingsAndCirc->circulationData = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(holdingsAndCirc->circulationData, env);

		while (sHoldingsAndCirc.ppsCircRecord[current_listnode] != NULL) {

			/* Create a list node */
	                data_ptr_address = AsnListAppend(holdingsAndCirc->circulationData);
			CheckAsn1Alloc(data_ptr_address, env);

                	*data_ptr_address = Asn1Alloc(sizeof(CircRecord));
			CheckAsn1Alloc(*data_ptr_address, env);

                	data_ptr = *data_ptr_address;

			DBVdbv_to_snacc_circ_struct(data_ptr, sHoldingsAndCirc.ppsCircRecord[current_listnode], env);
			current_listnode++;
		}
	}
}


int DBVdbv_to_snacc_circ_struct(CircRecord *data_ptr, DbvCircRecord *psCircRecord, jmp_buf env)
{
	if (psCircRecord->fAvailableNow == DBV_TRUE)
		data_ptr->availableNow = TRUE;
	else
		data_ptr->availableNow = FALSE;

	if (psCircRecord->psAvailabilityDate != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->availablityDate, psCircRecord->psAvailabilityDate, env);

	if (psCircRecord->psAvailableThru != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->availableThru, psCircRecord->psAvailableThru, env);

	if (psCircRecord->psRestrictions != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->restrictions, psCircRecord->psRestrictions, env);

	if (psCircRecord->psItemId != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->itemId, psCircRecord->psItemId, env);


	if (psCircRecord->fRenewable == DBV_TRUE)
		data_ptr->renewable = TRUE;
	else
		data_ptr->renewable = FALSE;


	if (psCircRecord->fOnHold == DBV_TRUE)
		data_ptr->onHold = TRUE;
	else
		data_ptr->onHold = FALSE;

	if (psCircRecord->psEnumAndChron != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->enumAndChron, psCircRecord->psEnumAndChron, env);

	if (psCircRecord->psMidspine != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->midspine, psCircRecord->psMidspine, env);

	if (psCircRecord->psTemporaryLocation != NULL)
		DBVdbv_to_snacc_inter_string(&data_ptr->temporaryLocation, psCircRecord->psTemporaryLocation, env);


}




/* snacc -> dbv ansi opac external */
void DBVsnacc_to_dbv_opac_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
 
	OPACRecord *opac_ptr;
 
	opac_ptr = (OPACRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_OpacRecord;
        psSingleASN1Type->uASN1.psOpacRecord = (struct DbvOpacRecord *) DbvAlloc (sizeof(struct DbvOpacRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psOpacRecord, env);

	DBVsnacc_to_dbv_opac_main(psSingleASN1Type->uASN1.psOpacRecord, opac_ptr->a.johnfix, env);
}



/* snacc -> dbv iso opac external */
int DBVsnacc_to_dbv_isoopac_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoOPACRecord *isoopac_ptr;
 
	isoopac_ptr = (IsoOPACRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_OpacRecord;
        psSingleASN1Type->uASN1.psOpacRecord = (struct DbvOpacRecord *) DbvAlloc (sizeof(struct DbvOpacRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psOpacRecord, env);

	DBVsnacc_to_dbv_opac_main(psSingleASN1Type->uASN1.psOpacRecord, isoopac_ptr->a.johnfix, env);
}



int DBVsnacc_to_dbv_opac_main(struct DbvOpacRecord *psOpacRecord, OrigOPACRecord *opac_ptr, jmp_buf env)
{
	
	int num_elements = 0;

	/* psBibliographicRecord */
	if (opac_ptr->bibliographicRecord != NULL)  {
		psOpacRecord->psBibliographicRecord = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		CheckDbvAlloc(psOpacRecord->psBibliographicRecord, env);

		DBVsnacc_to_dbv_external(psOpacRecord->psBibliographicRecord, opac_ptr->bibliographicRecord, env);
	}

	/* ppsHoldingsData */
	if (opac_ptr->holdingsData != NULL) {
			
		/* Get the number of elements in the list */
		num_elements = AsnListCount(opac_ptr->holdingsData);
		psOpacRecord->ppsHoldingsData = (DbvHoldingsRecord **) DbvAlloc (sizeof(DbvHoldingsRecord *) * (num_elements + 1));
		CheckDbvAlloc(psOpacRecord->ppsHoldingsData, env);

		DBVsnacc_to_dbv_holding(psOpacRecord->ppsHoldingsData, opac_ptr->holdingsData, env);
	}
		
}



int DBVsnacc_to_dbv_holding(DbvHoldingsRecord **ppsHoldingsData, OrigOPACRecordSeqOf *holdingsData, jmp_buf env)
{
	HoldingsRecord *data_ptr;
	int index = 0;
	 
	FOR_EACH_LIST_ELMT(data_ptr, holdingsData)
	{
        	ppsHoldingsData[index]= (DbvHoldingsRecord *) DbvAlloc (sizeof(DbvHoldingsRecord));
		CheckDbvAlloc(ppsHoldingsData[index], env);

		switch (data_ptr->choiceId) {
		case HOLDINGSRECORD_MARCHOLDINGSRECORD:
			ppsHoldingsData[index]->eHoldingsRecType = HRT_MarcHoldingsRecord;
			DBVsnacc_to_dbv_external( & (ppsHoldingsData[index]->uMarcOrHoldAndCirc.sMarcHoldingsRecord), data_ptr->a.marcHoldingsRecord, env);
			break;
		case HOLDINGSRECORD_HOLDINGSANDCIRC:
			ppsHoldingsData[index]->eHoldingsRecType = HRT_HoldingsAndCirc;
			DBVsnacc_to_dbv_holding_bulk(& (ppsHoldingsData[index]->uMarcOrHoldAndCirc.sHoldingsAndCirc), data_ptr->a.holdingsAndCirc, env);

			DBVsnacc_to_dbv_hold_volume(& (ppsHoldingsData[index]->uMarcOrHoldAndCirc.sHoldingsAndCirc), data_ptr->a.holdingsAndCirc, env);
			DBVsnacc_to_dbv_hold_circ(& (ppsHoldingsData[index]->uMarcOrHoldAndCirc.sHoldingsAndCirc), data_ptr->a.holdingsAndCirc, env);

			break;
		}
        	index++;
	}
	ppsHoldingsData[index]=NULL;

}


int DBVsnacc_to_dbv_holding_bulk(DbvHoldingsAndCircData *psHoldingsAndCirc, struct HoldingsAndCircData* holdingsAndCirc, jmp_buf env)
{
	/* If typeOfRecord exists */
	if (holdingsAndCirc->typeOfRecord.octetLen != 0) {
		psHoldingsAndCirc->psTypeOfRecord = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psTypeOfRecord, env);

		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psTypeOfRecord, &holdingsAndCirc->typeOfRecord, env);
	}
		
	/* If encodingLevel exists */
	if (holdingsAndCirc->encodingLevel.octetLen != 0) {
		psHoldingsAndCirc->psEncodingLevel = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psEncodingLevel, env);

		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psEncodingLevel, &holdingsAndCirc->encodingLevel, env);
	}
		
	/* If format exists */
	if (holdingsAndCirc->format.octetLen != 0) {
		psHoldingsAndCirc->psFormat = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psFormat, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psFormat, &holdingsAndCirc->format, env);
	}
		
	/* If receiptAcqStatus exists */
	if (holdingsAndCirc->receiptAcqStatus.octetLen != 0) {
		psHoldingsAndCirc->psReceiptAcqStatus = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psReceiptAcqStatus, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psReceiptAcqStatus, &holdingsAndCirc->receiptAcqStatus, env);
	}
		
	/* If generalRetention exists */
	if (holdingsAndCirc->generalRetention.octetLen != 0) {
		psHoldingsAndCirc->psGeneralRetention = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psGeneralRetention, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psGeneralRetention, &holdingsAndCirc->generalRetention, env);
	}
		
	/* If completeness exists */
	if (holdingsAndCirc->completeness.octetLen != 0) {
		psHoldingsAndCirc->psCompleteness = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psCompleteness, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psCompleteness, &holdingsAndCirc->completeness, env);
	}
		
	/* If dateOfReport exists */
	if (holdingsAndCirc->dateOfReport.octetLen != 0) {
		psHoldingsAndCirc->psDateOfReport = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psDateOfReport, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psDateOfReport, &holdingsAndCirc->dateOfReport, env);
	}
		
	/* If nucCode exists */
	if (holdingsAndCirc->nucCode.octetLen != 0) {
		psHoldingsAndCirc->psNucCode = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psNucCode, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psNucCode, &holdingsAndCirc->nucCode, env);
	}
		
	/* If localLocation exists */
	if (holdingsAndCirc->localLocation.octetLen != 0) {
		psHoldingsAndCirc->psLocalLocation = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psLocalLocation, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psLocalLocation, &holdingsAndCirc->localLocation, env);
	}
		
	/* If shelvingLocation exists */
	if (holdingsAndCirc->shelvingLocation.octetLen != 0) {
		psHoldingsAndCirc->psShelvingLocation = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psShelvingLocation, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psShelvingLocation, &holdingsAndCirc->shelvingLocation, env);
	}
		
	/* If callNumber exists */
	if (holdingsAndCirc->callNumber.octetLen != 0) {
		psHoldingsAndCirc->psCallNumber = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psCallNumber, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psCallNumber, &holdingsAndCirc->callNumber, env);
	}
		
	/* If shelvingData exists */
	if (holdingsAndCirc->shelvingData.octetLen != 0) {
		psHoldingsAndCirc->psShelvingData = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psShelvingData, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psShelvingData, &holdingsAndCirc->shelvingData, env);
	}
		
	/* If copyNumber exists */
	if (holdingsAndCirc->copyNumber.octetLen != 0) {
		psHoldingsAndCirc->psCopyNumber = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psCopyNumber, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psCopyNumber, &holdingsAndCirc->copyNumber, env);
	}
		
	/* If publicNote exists */
	if (holdingsAndCirc->publicNote.octetLen != 0) {
		psHoldingsAndCirc->psPublicNote = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psPublicNote, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psPublicNote, &holdingsAndCirc->publicNote, env);
	}
		

	/* If reproductionNote exists */
	if (holdingsAndCirc->reproductionNote.octetLen != 0) {
		psHoldingsAndCirc->psReproductionNote = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psReproductionNote, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psReproductionNote, &holdingsAndCirc->reproductionNote, env);
	}
		
	/* If termsUseRepro exists */
	if (holdingsAndCirc->termsUseRepro.octetLen != 0) {
		psHoldingsAndCirc->psTermsUseRepro = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psTermsUseRepro, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psTermsUseRepro, &holdingsAndCirc->termsUseRepro, env);
	}
		
	/* If enumAndChron exists */
	if (holdingsAndCirc->enumAndChron.octetLen != 0) {
		psHoldingsAndCirc->psEnumAndChron = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psHoldingsAndCirc->psEnumAndChron, env);
		DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->psEnumAndChron, &holdingsAndCirc->enumAndChron, env);
	}
		
}



int DBVsnacc_to_dbv_hold_volume(DbvHoldingsAndCircData *psHoldingsAndCirc, struct HoldingsAndCircData* holdingsAndCirc, jmp_buf env)
{
	Volume *data_ptr;
	int num_elements = 0;
	int index = 0;
 
	/* If a volumes structure exists (it is optional) */
	if (holdingsAndCirc->volumes != NULL) {
		num_elements = AsnListCount(holdingsAndCirc->volumes);
		psHoldingsAndCirc->ppsVolume = (DbvVolume **) DbvAlloc (sizeof (DbvVolume *) * (num_elements + 1));
		CheckDbvAlloc(psHoldingsAndCirc->ppsVolume, env);


		FOR_EACH_LIST_ELMT(data_ptr, holdingsAndCirc->volumes) {
        		psHoldingsAndCirc->ppsVolume[index] = (DbvVolume *) DbvAlloc (sizeof (DbvVolume));
			CheckDbvAlloc(psHoldingsAndCirc->ppsVolume[index], env);
	
			/* If enumeration exists (it is optional) */
			if (data_ptr->enumeration.octetLen != 0) {
				psHoldingsAndCirc->ppsVolume[index]->psEnumeration = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
				CheckDbvAlloc(psHoldingsAndCirc->ppsVolume[index]->psEnumeration, env);

        			DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->ppsVolume[index]->psEnumeration, & (data_ptr->enumeration), env);
			}


			/* If chronology exists (it is optional) */
			if (data_ptr->chronology.octetLen != 0) {
				psHoldingsAndCirc->ppsVolume[index]->psChronology = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
				CheckDbvAlloc(psHoldingsAndCirc->ppsVolume[index]->psChronology, env);

        			DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->ppsVolume[index]->psChronology, & (data_ptr->chronology), env);
			}
	

			/* If enumAndChron exists (it is optional) */
			if (data_ptr->enumAndChron.octetLen != 0) {
				psHoldingsAndCirc->ppsVolume[index]->psEnumAndChron = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
				CheckDbvAlloc(psHoldingsAndCirc->ppsVolume[index]->psEnumAndChron, env);

        			DBVsnacc_to_dbv_inter_string(psHoldingsAndCirc->ppsVolume[index]->psEnumAndChron, & (data_ptr->enumAndChron), env);
			}
        		index++;
		}
		psHoldingsAndCirc->ppsVolume[index]=NULL;
	}
	
}



int DBVsnacc_to_dbv_hold_circ(DbvHoldingsAndCircData *psHoldingsAndCirc, struct HoldingsAndCircData* holdingsAndCirc, jmp_buf env)
{
	CircRecord *data_ptr;
	int num_elements = 0;
	int index = 0;
 
	/* If a circulationData structure exists (it is optional) */
	if (holdingsAndCirc->circulationData != NULL) {
		num_elements = AsnListCount(holdingsAndCirc->circulationData);
		psHoldingsAndCirc->ppsCircRecord = (DbvCircRecord **) DbvAlloc (sizeof (DbvCircRecord *) * (num_elements + 1));
		CheckDbvAlloc(psHoldingsAndCirc->ppsCircRecord, env);


		FOR_EACH_LIST_ELMT(data_ptr, holdingsAndCirc->circulationData) {
        		psHoldingsAndCirc->ppsCircRecord[index] = (DbvCircRecord *) DbvAlloc (sizeof (DbvCircRecord));
			CheckDbvAlloc(psHoldingsAndCirc->ppsCircRecord[index], env);
	
			DBVsnacc_to_dbv_circrec(psHoldingsAndCirc->ppsCircRecord[index], data_ptr, env);
        		index++;
		}
		psHoldingsAndCirc->ppsCircRecord[index]=NULL;
	}
	
}



int DBVsnacc_to_dbv_circrec(DbvCircRecord *psCircRecord, CircRecord *data_ptr, jmp_buf env)
{
	if (data_ptr->availableNow == TRUE)
		psCircRecord->fAvailableNow = DBV_TRUE;
	else
		psCircRecord->fAvailableNow = DBV_FALSE;

	/* If availablityDate exists (it is optional) */
	if (data_ptr->availablityDate.octetLen != 0) {
		psCircRecord->psAvailabilityDate = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psAvailabilityDate, env);

       		DBVsnacc_to_dbv_inter_string(psCircRecord->psAvailabilityDate, & (data_ptr->availablityDate), env);
	}

	/* If availableThru exists (it is optional) */
	if (data_ptr->availableThru.octetLen != 0) {
		psCircRecord->psAvailableThru = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psAvailableThru, env);

       		DBVsnacc_to_dbv_inter_string(psCircRecord->psAvailableThru, & (data_ptr->availableThru), env);
	}

	/* If restrictions exists (it is optional) */
	if (data_ptr->restrictions.octetLen != 0) {
		psCircRecord->psRestrictions = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psRestrictions, env);

       		DBVsnacc_to_dbv_inter_string(psCircRecord->psRestrictions, & (data_ptr->restrictions), env);

	}

	/* If psItemId exists (it is optional) */
	if (data_ptr->itemId.octetLen != 0) {
		psCircRecord->psItemId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psItemId, env);

       		DBVsnacc_to_dbv_inter_string(psCircRecord->psItemId, & (data_ptr->itemId), env);
	}


	if (data_ptr->renewable == TRUE)
		psCircRecord->fRenewable = DBV_TRUE;
	else
		psCircRecord->fRenewable = DBV_FALSE;


	if (data_ptr->onHold == TRUE)
		psCircRecord->fOnHold = DBV_TRUE;
	else
		psCircRecord->fOnHold = DBV_FALSE;

	/* If enumAndChron exists (it is optional) */
	if (data_ptr->enumAndChron.octetLen != 0) {
		psCircRecord->psEnumAndChron = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psEnumAndChron, env);

       		DBVsnacc_to_dbv_inter_string(psCircRecord->psEnumAndChron, & (data_ptr->enumAndChron), env);
	}


	/* If psMidspine exists (it is optional) */
	if (data_ptr->midspine.octetLen != 0) {
		psCircRecord->psMidspine = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psMidspine, env);
       		DBVsnacc_to_dbv_inter_string(psCircRecord->psMidspine, & (data_ptr->midspine), env);
	}

	/* If temporaryLocation exists (it is optional) */
	if (data_ptr->temporaryLocation.octetLen != 0) {
		psCircRecord->psTemporaryLocation = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psCircRecord->psTemporaryLocation, env);
       		DBVsnacc_to_dbv_inter_string(psCircRecord->psTemporaryLocation, & (data_ptr->temporaryLocation), env);
	}



}


/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* summary */

/* dbv -> snacc ansi summary external */
int DBVdbv_to_snacc_summary_any(AsnAnyDefinedBy *asn_any, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */

	static SummaryRecord summary_external[STRING_LEN];
 
        SummaryRecord *summary_ptr;

	/* Set up the Summary external */
        summary_ptr = &summary_external[current_summary];

	/* Clear the location (may have been used by the last PDU) */
	memset(summary_ptr, 0, sizeof(SummaryRecord));


	/* johnfix is necessary becuase when decoding an ANY/ANYDEFINEDBY
	   type, snacc mallocs, not callocs, space for the top 
	   structure of the type (see asn_any.c). 
	   This means that if some optional fields do not exist in an
	   ANY/ANYDEFINEDBY type, these fields may contain garbage 
	   after decoding, as left by the malloc 
	   (e.g. wierd octetlen values, strange octs pointer values etc).

	   There are two solutions to this problem:
	   a) change the malloc(size) statements in asn_any.c to calloc(1, size)
	   b) Use a dummy choice within the ASN.1 code which tricks snacc 
	      into callocing space for the structure using Asn1Alloc (this
	      will work if Asn1Alloc is #defined as calloc(1, size), as it
	      is in DBVOSI).
	      This dummy choice does not effect the BER encoding, 
	      only the structures snacc uses around it.


	   I prefer the dummy choice rather than customizing snacc. 
	   For the Generic record type which has a list as its top
	   structure, the dummy choice must be used as snacc get confused.
	*/
	    

	 

 
	summary_ptr->choiceId = SUMMARYRECORD_JOHNFIX;

	summary_ptr->a.johnfix = (struct BriefBib*) Asn1Alloc (sizeof(struct BriefBib));
	CheckAsn1Alloc(summary_ptr->a.johnfix, env);

	/* Dbv to snacc */
	DBVdbv_to_snacc_summ_main(summary_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	asn_any->value = (void *) &summary_external[current_summary];


	/* Only if not at end of array */
	if (current_summary < (STRING_LEN - 1)) {

		/* Increment current_summary for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_summary++;
	}
}
 
 

/* dbv -> snacc iso summary external */

int DBVdbv_to_snacc_isosummary_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoSummaryRecord isosummary_external[STRING_LEN];
        IsoSummaryRecord *isosummary_ptr;

        isosummary_ptr = &isosummary_external[current_isosummary];
	memset(isosummary_ptr, 0, sizeof(IsoSummaryRecord));
	isosummary_ptr->choiceId = SUMMARYRECORD_JOHNFIX;
	isosummary_ptr->a.johnfix = (struct BriefBib*) Asn1Alloc (sizeof(struct BriefBib));
	CheckAsn1Alloc(isosummary_ptr->a.johnfix, env);

	DBVdbv_to_snacc_summ_main(isosummary_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord, env);

	any_type->value = (void *) &isosummary_external[current_isosummary];

	if (current_isosummary < (STRING_LEN - 1)) {
		current_isosummary++;
	}
}
  


int DBVdbv_to_snacc_summ_main(BriefBib *summary_ptr, struct DbvSummaryRecord *psSummaryRecord, jmp_buf env)
{
	/* title is compulsory */
	DBVdbv_to_snacc_inter_string(&(summary_ptr->title), &psSummaryRecord->sTitle, env);
			
	if (psSummaryRecord->psAuthor != NULL) {
		DBVdbv_to_snacc_inter_string(&summary_ptr->author, psSummaryRecord->psAuthor, env);
	}

			
	if (psSummaryRecord->psCallNumber != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->callNumber, psSummaryRecord->psCallNumber, env);
			
	if (psSummaryRecord->psRecordType != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->recordType, psSummaryRecord->psRecordType, env);
			
	if (psSummaryRecord->psBibliographicLevel != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->bibliographicLevel, psSummaryRecord->psBibliographicLevel, env);
			

	if (psSummaryRecord->ppsFormat != NULL) {

		/* Create them there snacc list */
		summary_ptr->format = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(summary_ptr->format, env);
		DBVdbv_to_snacc_summ_format(summary_ptr->format, psSummaryRecord->ppsFormat, env);
	}

	if (psSummaryRecord->psPublicationPlace != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->publicationPlace, psSummaryRecord->psPublicationPlace, env);
			
	if (psSummaryRecord->psPublicationDate != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->publicationDate, psSummaryRecord->psPublicationDate, env);
			
	if (psSummaryRecord->psTargetSystemKey != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->targetSystemKey, psSummaryRecord->psTargetSystemKey, env);
			
	if (psSummaryRecord->psSatisfyingElement != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->satisfyingElement, psSummaryRecord->psSatisfyingElement, env);
			
	if (psSummaryRecord->pulRank != NULL) {
		summary_ptr->rank = (AsnInt *) Asn1Alloc (sizeof(AsnInt));

		CheckAsn1Alloc(summary_ptr->rank, env);
		*summary_ptr->rank = *psSummaryRecord->pulRank;
	}
			
	if (psSummaryRecord->psDocumentId != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->documentId, psSummaryRecord->psDocumentId, env);
			
	if (psSummaryRecord->psAbstract != NULL) 
		DBVdbv_to_snacc_inter_string(&summary_ptr->abstract, psSummaryRecord->psAbstract, env);

	
	if (psSummaryRecord->ppsOtherInformation != NULL) {
		summary_ptr->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(summary_ptr->otherInfo, env);
		DBVdbv_to_snacc_otherinfo(summary_ptr->otherInfo, psSummaryRecord->ppsOtherInformation, env);
	}

}



int DBVdbv_to_snacc_summ_format(BriefBibSeqOf *format, DbvFormatSpec **ppsFormat, jmp_buf env)
{
	FormatSpec *data_ptr;
	FormatSpec **data_ptr_address;

	int current_listnode = 0;

	while (ppsFormat[current_listnode] != NULL) {

		/* Create a snacc list node */
                data_ptr_address = AsnListAppend(format);
		CheckAsn1Alloc(data_ptr_address, env);

               	*data_ptr_address = Asn1Alloc(sizeof(FormatSpec));
		CheckAsn1Alloc(*data_ptr_address, env);

               	data_ptr = *data_ptr_address;

		/* Fill them there list node */

		/* type - a compulsory */
		DBVdbv_to_snacc_inter_string(&data_ptr->type, & (ppsFormat[current_listnode]->sType), env);

		/* size - an optional */
		if (ppsFormat[current_listnode]->pulSize != NULL) {
			data_ptr->size = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
			CheckAsn1Alloc(data_ptr->size, env);
			*data_ptr->size = *ppsFormat[current_listnode]->pulSize;
		}
		
		/* bestPosn - an optional */
		if (ppsFormat[current_listnode]->pulBestPosn != NULL) {
			data_ptr->bestPosn = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
			CheckAsn1Alloc(data_ptr->bestPosn, env);
			*data_ptr->bestPosn = *ppsFormat[current_listnode]->pulBestPosn;
		}
		
			
		current_listnode++;
	}
}



/* snacc -> dbv ansi summary external */
int DBVsnacc_to_dbv_summary_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
 
	SummaryRecord *summary_ptr;
 
	summary_ptr = (SummaryRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_SummaryRecord;
        psSingleASN1Type->uASN1.psSummaryRecord = (struct DbvSummaryRecord *) DbvAlloc (sizeof(struct DbvSummaryRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psSummaryRecord, env);

	DBVsnacc_to_dbv_summ_main(psSingleASN1Type->uASN1.psSummaryRecord, summary_ptr->a.johnfix, env);
}


/* snacc -> dbv iso summary external */
int DBVsnacc_to_dbv_isosummary_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
 
	IsoSummaryRecord *isosummary_ptr;
	isosummary_ptr = (IsoSummaryRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_SummaryRecord;
        psSingleASN1Type->uASN1.psSummaryRecord = (struct DbvSummaryRecord *) DbvAlloc (sizeof(struct DbvSummaryRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psSummaryRecord, env);

	DBVsnacc_to_dbv_summ_main(psSingleASN1Type->uASN1.psSummaryRecord, isosummary_ptr->a.johnfix, env);
}



int DBVsnacc_to_dbv_summ_main(struct DbvSummaryRecord *psSummaryRecord, BriefBib *summary_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* title is compulsory */
	DBVsnacc_to_dbv_inter_string(&psSummaryRecord->sTitle, &(summary_ptr->title), env);
			
	/* If author exists (it is optional) */
	if (summary_ptr->author.octetLen != 0)  {
		psSummaryRecord->psAuthor = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psAuthor, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psAuthor, &summary_ptr->author, env);
	}

	/* If callNumber exists (it is optional) */
	if (summary_ptr->callNumber.octetLen != 0)  {
		psSummaryRecord->psCallNumber = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psCallNumber, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psCallNumber, &summary_ptr->callNumber, env);
	}

	/* If recordType exists (it is optional) */
	if (summary_ptr->recordType.octetLen != 0)  {
		psSummaryRecord->psRecordType = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psRecordType, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psRecordType, &summary_ptr->recordType, env);
	}

	/* If bibliographicLevel exists (it is optional) */
	if (summary_ptr->bibliographicLevel.octetLen != 0)  {
		psSummaryRecord->psBibliographicLevel = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psBibliographicLevel, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psBibliographicLevel, &summary_ptr->bibliographicLevel, env);
	}


	/* If format exists (it is optional) */
	if (summary_ptr->format != NULL)  {
		num_elements = AsnListCount(summary_ptr->format);
		psSummaryRecord->ppsFormat = (DbvFormatSpec **) DbvAlloc (sizeof(DbvFormatSpec *) * (num_elements + 1));
		CheckDbvAlloc(psSummaryRecord->ppsFormat, env);
		DBVsnacc_to_dbv_summ_format(psSummaryRecord->ppsFormat, summary_ptr->format, env);
	}



	/* If publicationPlace exists (it is optional) */
	if (summary_ptr->publicationPlace.octetLen != 0)  {
		psSummaryRecord->psPublicationPlace = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psPublicationPlace, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psPublicationPlace, &summary_ptr->publicationPlace, env);
	}

	/* If publicationDate exists (it is optional) */
	if (summary_ptr->publicationDate.octetLen != 0)  {
		psSummaryRecord->psPublicationDate = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psPublicationDate, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psPublicationDate, &summary_ptr->publicationDate, env);
	}

	/* If targetSystemKey exists (it is optional) */
	if (summary_ptr->targetSystemKey.octetLen != 0)  {
		psSummaryRecord->psTargetSystemKey = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psTargetSystemKey, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psTargetSystemKey, &summary_ptr->targetSystemKey, env);
	}

	/* If satisfyingElement exists (it is optional) */
	if (summary_ptr->satisfyingElement.octetLen != 0)  {
		psSummaryRecord->psSatisfyingElement = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psSatisfyingElement, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psSatisfyingElement, &summary_ptr->satisfyingElement, env);
	}

	/* If rank exists (it is optional) */
	if (summary_ptr->rank != NULL) {
		psSummaryRecord->pulRank = (Ulong *) DbvAlloc (sizeof (Ulong));
		CheckDbvAlloc(psSummaryRecord->pulRank, env);

		*psSummaryRecord->pulRank = *summary_ptr->rank;
	}

	/* If documentId exists (it is optional) */
	if (summary_ptr->documentId.octetLen != 0)  {
		psSummaryRecord->psDocumentId = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psDocumentId, env);

		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psDocumentId, &summary_ptr->documentId, env);
	}

	/* If psAbstract exists (it is optional) */
	if (summary_ptr->abstract.octetLen != 0)  {
		psSummaryRecord->psAbstract = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psSummaryRecord->psAbstract, env);
		DBVsnacc_to_dbv_inter_string(psSummaryRecord->psAbstract, &summary_ptr->abstract, env);
	}

	if (summary_ptr->otherInfo != NULL) {
		num_elements = AsnListCount(summary_ptr->otherInfo);
                psSummaryRecord->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(psSummaryRecord->ppsOtherInformation, env);
                DBVsnacc_to_dbv_otherinfo(psSummaryRecord->ppsOtherInformation, summary_ptr->otherInfo, env);
        }


}



int DBVsnacc_to_dbv_summ_format(DbvFormatSpec **ppsFormat, BriefBibSeqOf *format, jmp_buf env)
{
	FormatSpec *data_ptr;
	int num_elements = 0;
	int index = 0;
 
	FOR_EACH_LIST_ELMT(data_ptr, format) {
        	ppsFormat[index] = (DbvFormatSpec *) DbvAlloc (sizeof (DbvFormatSpec));
		CheckDbvAlloc(ppsFormat[index], env);

		DBVsnacc_to_dbv_inter_string(& (ppsFormat[index]->sType), &data_ptr->type, env);


		if (data_ptr->size != NULL) {
			ppsFormat[index]->pulSize = (Ulong *) DbvAlloc (sizeof(Ulong));
			CheckDbvAlloc(ppsFormat[index]->pulSize, env);

			*ppsFormat[index]->pulSize = *data_ptr->size;
		}


		if (data_ptr->bestPosn != NULL) {
			ppsFormat[index]->pulBestPosn = (Ulong *) DbvAlloc (sizeof(Ulong));
			CheckDbvAlloc(ppsFormat[index]->pulBestPosn, env);

			*ppsFormat[index]->pulBestPosn = *data_ptr->bestPosn;
		}


        	index++;
	}
	ppsFormat[index]=NULL;
}



/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* Resource Report 1 */

/* dbv -> snacc ansi Resource Report 1 external */

int DBVdbv_to_snacc_resrep1_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static ResourceReport1 resrep1_external[STRING_LEN];
 
        ResourceReport1 *resrep1_ptr;

	/* Set up the Resource Report 1 external */
        resrep1_ptr = &resrep1_external[current_resrep1];

	/* Clear the location (may have been used by the last PDU) */
	memset(resrep1_ptr, 0, sizeof(ResourceReport1));

	DBVdbv_to_snacc_resrep1_main(resrep1_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport1, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &resrep1_external[current_resrep1];

	/* Only if not at end of array */
	if (current_resrep1 < (STRING_LEN - 1)) {

		/* Increment current_resrep1 for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_resrep1++;
	}

}


/* dbv -> snacc iso Resource Report 1 external */

int DBVdbv_to_snacc_isoresrep1_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoResourceReport1 isoresrep1_external[STRING_LEN];
        IsoResourceReport1 *isoresrep1_ptr;

        isoresrep1_ptr = &isoresrep1_external[current_isoresrep1];
	memset(isoresrep1_ptr, 0, sizeof(IsoResourceReport1));
	DBVdbv_to_snacc_resrep1_main(isoresrep1_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport1, env);
	any_type->value = (void *) &isoresrep1_external[current_isoresrep1];

	if (current_isoresrep1 < (STRING_LEN - 1)) {
		current_isoresrep1++;
	}

}



int DBVdbv_to_snacc_resrep1_main(ResourceReport1 *resrep1_ptr, struct DbvResourceReport1 *psResourceReport1, jmp_buf env)
{
	Estimate *data_ptr;
	Estimate **data_ptr_address;

        int current_listnode = 0;
 
        resrep1_ptr->estimates = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(resrep1_ptr->estimates, env);
 
        while(psResourceReport1->ppsEstimate[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(resrep1_ptr->estimates);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Estimate));
		CheckAsn1Alloc(*data_ptr_address, env);
                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_estimate1(data_ptr, psResourceReport1->ppsEstimate[current_listnode], env);
                current_listnode++;
        }


	/* This field is compulsory */
	DBVdbv_to_snacc_inter_string(&resrep1_ptr->message, &psResourceReport1->sMessage, env);

}



int DBVdbv_to_snacc_estimate1(Estimate *data_ptr, DbvEstimate1 *psEstimate, jmp_buf env)
{
	/* First two compulsory, second optional */
	switch (psEstimate->eEstimateType) {
	case ET_CurrentSearchRecords:
		data_ptr->type = CURRENTSEARCHRECORDS;
		break;
	case ET_FinalSearchRecords:
		data_ptr->type = FINALSEARCHRECORDS;
		break;
	case ET_CurrentPresentRecords:
		data_ptr->type = CURRENTPRESENTRECORDS;
		break;
	case ET_FinalPresentRecords:
		data_ptr->type = FINALPRESENTRECORDS;
		break;
	case ET_CurrentOpTimeProcessing:
		data_ptr->type = CURRENTOPTIMEPROCESSING;
		break;
	case ET_FinalOpTimeProcessing:
		data_ptr->type = FINALOPTIMEPROCESSING;
		break;
	case ET_CurrentAssocTime:
		data_ptr->type = CURRENTASSOCTIME;
		break;
	case ET_CurrentOperationCost:
		data_ptr->type = CURRENTOPERATIONCOST;
		break;
	case ET_FinalOperationCost:
		data_ptr->type = FINALOPERATIONCOST;
		break;
	case ET_CurrentAssocCost:
		data_ptr->type = CURRENTASSOCCOST;
		break;
	case ET_FinalOpTimeElapsed:
		data_ptr->type = FINALOPTIMEELAPSED;
		break;
	case ET_PercentComplete:
		data_ptr->type = PERCENTCOMPLETE;
		break;
	case ET_CurrentSearchAssocCost:
		data_ptr->type = CURRENTSEARCHASSOCCOST;
		break;
	case ET_CurrentPresentAssocCost:
		data_ptr->type = CURRENTPRESENTASSOCCOST;
		break;
	case ET_CurrentConnectAssocCost:
		data_ptr->type = CURRENTCONNECTASSOCCOST;
		break;
	case ET_CurrentOtherAssocCost:
		data_ptr->type = CURRENTOTHERASSOCCOST;
		break; 
	}

	data_ptr->value = psEstimate->ulValue;

	/* currency_code is optional */
	if (psEstimate->pulCurrencyCode != NULL) {
		data_ptr->currency_code = (AsnInt *) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(data_ptr->currency_code, env);
		*data_ptr->currency_code = *psEstimate->pulCurrencyCode;
	}
}



/* snacc -> dbv ansi Resource Report 1 external */

int DBVsnacc_to_dbv_resrep1_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	ResourceReport1 *resrep1_ptr;
 
	resrep1_ptr = (ResourceReport1 *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ResourceReport1;
        psSingleASN1Type->uASN1.psResourceReport1 = (struct DbvResourceReport1 *) DbvAlloc (sizeof(struct DbvResourceReport1));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psResourceReport1, env);

	DBVsnacc_to_dbv_resrep1_main(psSingleASN1Type->uASN1.psResourceReport1, resrep1_ptr, env);
}



/* snacc -> dbv iso Resource Report 1 external */
int DBVsnacc_to_dbv_isoresrep1_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoResourceReport1 *isoresrep1_ptr;
 
	isoresrep1_ptr = (IsoResourceReport1 *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ResourceReport1;
        psSingleASN1Type->uASN1.psResourceReport1 = (struct DbvResourceReport1 *) DbvAlloc (sizeof(struct DbvResourceReport1));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psResourceReport1, env);

	DBVsnacc_to_dbv_resrep1_main(psSingleASN1Type->uASN1.psResourceReport1, isoresrep1_ptr, env);
}



int DBVsnacc_to_dbv_resrep1_main(struct DbvResourceReport1 *psResourceReport1, ResourceReport1 *resrep1_ptr, jmp_buf env)
{
	Estimate *data_ptr;
        int num_elements = 0;
        int index = 0;
 
	/* ppsEstimate */
        num_elements = AsnListCount(resrep1_ptr->estimates);
	psResourceReport1->ppsEstimate = (DbvEstimate1 **) DbvAlloc (sizeof(DbvEstimate1 *) * (num_elements + 1) );
	CheckDbvAlloc(psResourceReport1->ppsEstimate, env);

        FOR_EACH_LIST_ELMT(data_ptr, resrep1_ptr->estimates) {
                psResourceReport1->ppsEstimate[index] = (DbvEstimate1 *) DbvAlloc(sizeof(DbvEstimate1));
		CheckDbvAlloc(psResourceReport1->ppsEstimate[index], env);

                DBVsnacc_to_dbv_estimate1(psResourceReport1->ppsEstimate[index], data_ptr, env);
                index++;
        }
        psResourceReport1->ppsEstimate[index] = NULL;


	/* sMessage */
	DBVsnacc_to_dbv_inter_string(&psResourceReport1->sMessage, &resrep1_ptr->message, env);

}



int DBVsnacc_to_dbv_estimate1(DbvEstimate1 *psEstimate, Estimate *data_ptr, jmp_buf env)
{
	switch (data_ptr->type) {
	case CURRENTSEARCHRECORDS:
		psEstimate->eEstimateType = ET_CurrentSearchRecords;
		break;
	case FINALSEARCHRECORDS:
		psEstimate->eEstimateType = ET_FinalSearchRecords;
		break;
	case CURRENTPRESENTRECORDS:
		psEstimate->eEstimateType = ET_CurrentPresentRecords;
		break;
	case FINALPRESENTRECORDS:
		psEstimate->eEstimateType = ET_FinalPresentRecords;
		break;
	case CURRENTOPTIMEPROCESSING:
		psEstimate->eEstimateType = ET_CurrentOpTimeProcessing;
		break;
	case FINALOPTIMEPROCESSING:
		psEstimate->eEstimateType = ET_FinalOpTimeProcessing;
		break;
	case CURRENTASSOCTIME:
		psEstimate->eEstimateType = ET_CurrentAssocTime;
		break;
	case CURRENTOPERATIONCOST:
		psEstimate->eEstimateType = ET_CurrentOperationCost;
		break;
	case FINALOPERATIONCOST:
		psEstimate->eEstimateType = ET_FinalOperationCost;
		break;
	case CURRENTASSOCCOST:
		psEstimate->eEstimateType = ET_CurrentAssocCost;
		break;
	case FINALOPTIMEELAPSED:
		psEstimate->eEstimateType = ET_FinalOpTimeElapsed;
		break;
	case PERCENTCOMPLETE:
		psEstimate->eEstimateType = ET_PercentComplete;
		break;
	case CURRENTSEARCHASSOCCOST:
		psEstimate->eEstimateType = ET_CurrentSearchAssocCost;
		break;
	case CURRENTPRESENTASSOCCOST:
		psEstimate->eEstimateType = ET_CurrentPresentAssocCost;
		break;
	case CURRENTCONNECTASSOCCOST:
		psEstimate->eEstimateType = ET_CurrentConnectAssocCost;
		break;
	case CURRENTOTHERASSOCCOST:
		psEstimate->eEstimateType = ET_CurrentOtherAssocCost;
		break; 
	default:
		psEstimate->eEstimateType = ET_Invalid;
		break; 
	}


	psEstimate->ulValue = data_ptr->value;

	if (data_ptr->currency_code != NULL) {
		psEstimate->pulCurrencyCode = (Ulong *) DbvAlloc (sizeof(Ulong));
		CheckDbvAlloc(psEstimate->pulCurrencyCode, env);

		*psEstimate->pulCurrencyCode = *data_ptr->currency_code;
	}
}



/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* Resource Report 2 */

/* dbv -> snacc ansi Resource Report 2 external */
int DBVdbv_to_snacc_resrep2_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static ResourceReport2Record resrep2_external[STRING_LEN];
 
        ResourceReport2Record *resrep2_ptr;

	/* Set up the Resource Report 2 external */
        resrep2_ptr = &resrep2_external[current_resrep2];

	/* Clear the location (may have been used by the last PDU) */
	memset(resrep2_ptr, 0, sizeof(ResourceReport2Record));

	resrep2_ptr->choiceId = RESOURCEREPORT2RECORD_JOHNFIX;
	resrep2_ptr->a.johnfix = (struct ResourceReport2*) Asn1Alloc (sizeof(struct ResourceReport2));
	CheckAsn1Alloc(resrep2_ptr->a.johnfix, env);


	DBVdbv_to_snacc_resrep2_main(resrep2_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport2, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &resrep2_external[current_resrep2];

	/* Only if not at end of array */
	if (current_resrep2 < (STRING_LEN - 1)) {

		/* Increment current_resrep2 for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_resrep2++;
	}
}



/* dbv -> snacc iso Resource Report 2 external */

int DBVdbv_to_snacc_isoresrep2_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoResourceReport2Record isoresrep2_external[STRING_LEN];
        IsoResourceReport2Record *isoresrep2_ptr;

        isoresrep2_ptr = &isoresrep2_external[current_isoresrep2];
	memset(isoresrep2_ptr, 0, sizeof(IsoResourceReport2Record));
	isoresrep2_ptr->choiceId = RESOURCEREPORT2RECORD_JOHNFIX;
	isoresrep2_ptr->a.johnfix = (struct ResourceReport2*) Asn1Alloc (sizeof(struct ResourceReport2));
	CheckAsn1Alloc(isoresrep2_ptr->a.johnfix, env);

	DBVdbv_to_snacc_resrep2_main(isoresrep2_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport2, env);
	any_type->value = (void *) &isoresrep2_external[current_isoresrep2];

	if (current_isoresrep2 < (STRING_LEN - 1)) {

		current_isoresrep2++;
	}
}



int DBVdbv_to_snacc_resrep2_main(ResourceReport2 *resrep2_ptr, struct DbvResourceReport2 *psResourceReport2, jmp_buf env)
{
	/* Both the following are optional in snacc structures */

	if (psResourceReport2->ppsEstimate != NULL) {
        	resrep2_ptr->estimates = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(resrep2_ptr->estimates, env);

		DBVdbv_to_snacc_resrep2est(resrep2_ptr->estimates, psResourceReport2->ppsEstimate, env);
	}


	/* Optional */
	if (psResourceReport2->psMessage != 0)
		DBVdbv_to_snacc_inter_string(&resrep2_ptr->message, psResourceReport2->psMessage, env);
}

 


int DBVdbv_to_snacc_resrep2est(ResourceReport2SeqOf *estimates, DbvEstimate2 **ppsEstimate, jmp_buf env)
{
	Estimate2 *data_ptr;
	Estimate2 **data_ptr_address;

        int current_listnode = 0;

	/* ResourceReport2SeqOf is SEQUENCE OF Estimate2 */

        while(ppsEstimate[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(estimates);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Estimate2));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_estimate2(data_ptr, ppsEstimate[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_estimate2(Estimate2 *data_ptr, DbvEstimate2 *psEstimate, jmp_buf env)
{
	/* Both compulsory */
	data_ptr->type = (struct StringOrNumeric*) Asn1Alloc (sizeof(struct StringOrNumeric));
	CheckAsn1Alloc(data_ptr->type, env);

	DBVdbv_to_snacc_strornum(data_ptr->type, &psEstimate->sType, env);

	data_ptr->value = (struct IntUnit*) Asn1Alloc (sizeof(struct IntUnit));
	CheckAsn1Alloc(data_ptr->value, env);
	DBVdbv_to_snacc_intunit(data_ptr->value, &psEstimate->sValue, env);
}



/* snacc -> dbv ansi Resource Report 2 external */
int DBVsnacc_to_dbv_resrep2_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	ResourceReport2Record *resrep2_ptr;
 
	resrep2_ptr = (ResourceReport2Record *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ResourceReport2;
        psSingleASN1Type->uASN1.psResourceReport2 = (struct DbvResourceReport2 *) DbvAlloc (sizeof(struct DbvResourceReport2));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psResourceReport2, env);
	DBVsnacc_to_dbv_resrep2_main(psSingleASN1Type->uASN1.psResourceReport2, resrep2_ptr->a.johnfix, env);
}



/* snacc -> dbv iso Resource Report 2 external */
int DBVsnacc_to_dbv_isoresrep2_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoResourceReport2Record *isoresrep2_ptr;
	isoresrep2_ptr = (IsoResourceReport2Record *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ResourceReport2;
        psSingleASN1Type->uASN1.psResourceReport2 = (struct DbvResourceReport2 *) DbvAlloc (sizeof(struct DbvResourceReport2));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psResourceReport2, env);

	DBVsnacc_to_dbv_resrep2_main(psSingleASN1Type->uASN1.psResourceReport2, isoresrep2_ptr->a.johnfix, env);
}



int DBVsnacc_to_dbv_resrep2_main(struct DbvResourceReport2 *psResourceReport2, ResourceReport2 *resrep2_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* ppsEstimate */
	if (resrep2_ptr->estimates != NULL) {
        	num_elements = AsnListCount(resrep2_ptr->estimates);
		psResourceReport2->ppsEstimate = (DbvEstimate2 **) DbvAlloc (sizeof(DbvEstimate2 *) * (num_elements + 1) );
		CheckDbvAlloc(psResourceReport2->ppsEstimate, env);

		DBVsnacc_to_dbv_resrep2est(psResourceReport2->ppsEstimate, resrep2_ptr->estimates, env);
	}


	/* psMessage */
	if (resrep2_ptr->message.octetLen != 0) {
		psResourceReport2->psMessage = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psResourceReport2->psMessage , env);
		DBVsnacc_to_dbv_inter_string(psResourceReport2->psMessage, &resrep2_ptr->message, env);
	}
}



int DBVsnacc_to_dbv_resrep2est(DbvEstimate2 **ppsEstimate, ResourceReport2SeqOf *estimates, jmp_buf env)
{

	Estimate2 *data_ptr;
        int num_elements = 0;
        int index = 0;

        FOR_EACH_LIST_ELMT(data_ptr, estimates) {
                ppsEstimate[index] = (DbvEstimate2 *) DbvAlloc(sizeof(DbvEstimate2));
		CheckDbvAlloc(ppsEstimate[index], env);

                DBVsnacc_to_dbv_estimate2(ppsEstimate[index], data_ptr, env);
                index++;
        }
	ppsEstimate[index] = NULL;
}



int DBVsnacc_to_dbv_estimate2(DbvEstimate2 *psEstimate, Estimate2 *data_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_strornum(&psEstimate->sType, data_ptr->type, env);
	DBVsnacc_to_dbv_intunit(&psEstimate->sValue, data_ptr->value, env);

}



/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* Extended Service Task Package */

/* dbv -> snacc ansi Extended Service Task Package external */
int DBVdbv_to_snacc_estaskpack_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static TaskPackageRecord taskpack_external[STRING_LEN];
 
        TaskPackageRecord *taskpack_ptr;

	/* Set up the Task Package external */
        taskpack_ptr = &taskpack_external[current_taskpack];

	/* Clear the location (may have been used by the last PDU) */
	memset(taskpack_ptr, 0, sizeof(TaskPackageRecord));

	taskpack_ptr->choiceId = TASKPACKAGERECORD_JOHNFIX;

	taskpack_ptr->a.johnfix = (struct TaskPackage*) Asn1Alloc (sizeof(struct TaskPackage));
	CheckAsn1Alloc(taskpack_ptr->a.johnfix, env);

	DBVdbv_to_snacc_taskpack_main(taskpack_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psEsTaskPackage, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &taskpack_external[current_taskpack];

	/* Only if not at end of array */
	if (current_taskpack < (STRING_LEN - 1)) {

		/* Increment current_taskpack for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_taskpack++;
	}
}



/* dbv -> snacc iso Extended Service Task Package external */
int DBVdbv_to_snacc_isoestaskpack_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoTaskPackageRecord isotaskpack_external[STRING_LEN];
        IsoTaskPackageRecord *isotaskpack_ptr;

        isotaskpack_ptr = &isotaskpack_external[current_isotaskpack];
	memset(isotaskpack_ptr, 0, sizeof(IsoTaskPackageRecord));
	isotaskpack_ptr->choiceId = TASKPACKAGERECORD_JOHNFIX;
	isotaskpack_ptr->a.johnfix = (struct TaskPackage*) Asn1Alloc (sizeof(struct TaskPackage));
	CheckAsn1Alloc(isotaskpack_ptr->a.johnfix, env);

	DBVdbv_to_snacc_taskpack_main(isotaskpack_ptr->a.johnfix, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psEsTaskPackage, env);
	any_type->value = (void *) &isotaskpack_external[current_isotaskpack];

	if (current_isotaskpack < (STRING_LEN - 1)) {
		current_isotaskpack++;
	}
}



int DBVdbv_to_snacc_taskpack_main(TaskPackage *taskpack_ptr, struct DbvEsTaskPackage *psEsTaskPackage, jmp_buf env)
{

	char *string = NULL;
	unsigned long length = 0;

	ASN_MakeOid((char *) psEsTaskPackage->pcPackageType, &taskpack_ptr->packageType, env);
	
	/* All except taskStatus and taskSpecificParameters in 
	   following optional */
	if (psEsTaskPackage->psPackageName != NULL)
		DBVdbv_to_snacc_inter_string(&taskpack_ptr->packageName, psEsTaskPackage->psPackageName, env);
 
	if (psEsTaskPackage->psUserId != NULL)
		DBVdbv_to_snacc_inter_string(&taskpack_ptr->userId, psEsTaskPackage->psUserId, env);
 

	if (psEsTaskPackage->psRetentionTime != NULL) {
		taskpack_ptr->retentionTime = (struct IntUnit*) Asn1Alloc (sizeof(struct IntUnit));
		CheckAsn1Alloc(taskpack_ptr->retentionTime, env);

		DBVdbv_to_snacc_intunit(taskpack_ptr->retentionTime, psEsTaskPackage->psRetentionTime, env);
	}


	if (psEsTaskPackage->ppsPermissions != NULL) {
		taskpack_ptr->permissions = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(taskpack_ptr->permissions, env);

		DBVdbv_to_snacc_permissions(taskpack_ptr->permissions, psEsTaskPackage->ppsPermissions, env);
	}


	if (psEsTaskPackage->psDescription != NULL)
		DBVdbv_to_snacc_inter_string(&taskpack_ptr->description, psEsTaskPackage->psDescription, env);
	
	if (psEsTaskPackage->psTargetReference != NULL)
		DBVdbv_to_snacc_octet_string(&taskpack_ptr->targetReference, psEsTaskPackage->psTargetReference, env);
	

	/* GeneralizedTime is AsnOcts */
	if (psEsTaskPackage->pcCreationDateTime != NULL) {
		string = psEsTaskPackage->pcCreationDateTime;
		length = strlen(psEsTaskPackage->pcCreationDateTime);

		taskpack_ptr->creationDateTime.octs = (char *) Asn1Alloc (length);
		CheckAsn1Alloc(taskpack_ptr->creationDateTime.octs, env);

		memcpy(taskpack_ptr->creationDateTime.octs, string, length);
		taskpack_ptr->creationDateTime.octetLen = length;
	}
		

	
	switch (psEsTaskPackage->eExtServicesTaskStatus) {
	case ESTS_Pending:
		taskpack_ptr->taskStatus = PENDING;
		break;	
	case ESTS_Active:
		taskpack_ptr->taskStatus = ACTIVE;
		break;	
	case ESTS_Complete:
		taskpack_ptr->taskStatus = COMPLETE;
		break;	
	case ESTS_Aborted:
		taskpack_ptr->taskStatus = ABORTED;
		break;	
	}
	

	if (psEsTaskPackage->ppsPackageDiagnostics != NULL) {
		taskpack_ptr->packageDiagnostics = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(taskpack_ptr->packageDiagnostics, env);

		DBVdbv_to_snacc_diagrec_list(taskpack_ptr->packageDiagnostics, psEsTaskPackage->ppsPackageDiagnostics, env);
	}


	taskpack_ptr->taskSpecificParameters = (struct NEWEXTERNAL*) Asn1Alloc (sizeof (struct NEWEXTERNAL));
	CheckAsn1Alloc(taskpack_ptr->taskSpecificParameters, env);

	DBVdbv_to_snacc_external(taskpack_ptr->taskSpecificParameters, &psEsTaskPackage->sTaskSpecificParameters, env);
}



int DBVdbv_to_snacc_permissions(Permissions *permissions, DbvPermissions **ppsPermissions, jmp_buf env)
{
	PermissionsSeq *data_ptr;
	PermissionsSeq **data_ptr_address;

	int current_listnode = 0;
 
        while(ppsPermissions[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(permissions);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PermissionsSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_permisnode(data_ptr, ppsPermissions[current_listnode], env);
                current_listnode++;
        }
 
}



int DBVdbv_to_snacc_permisnode(PermissionsSeq *data_ptr, DbvPermissions *psPermissions, jmp_buf env)
{

	/* Both compulsory */
	DBVdbv_to_snacc_inter_string(&data_ptr->userId, &psPermissions->sUserId, env); 
	data_ptr->allowableFunctions = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(data_ptr->allowableFunctions, env);

	DBVdbv_to_snacc_allowfuncts(data_ptr->allowableFunctions, psPermissions->ppeAllowableFunctions, env);
}



int DBVdbv_to_snacc_allowfuncts(PermissionsSeqSeqOf *allowableFunctions, DbvPermAllowableFunctions **ppeAllowableFunctions, jmp_buf env)
{

	PermissionsSeqSeqOfInt *data_ptr;
	PermissionsSeqSeqOfInt **data_ptr_address;

	int current_listnode = 0;
 
        while(ppeAllowableFunctions[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(allowableFunctions);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PermissionsSeqSeqOfInt));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 

		/* ppeAllowableFunctions is an array of pointers to ints */
		switch (* (ppeAllowableFunctions[current_listnode]) ) {
		case PAF_Delete:
			*data_ptr = DELETE;
			break;
		case PAF_ModifyContents:
			*data_ptr = MODIFYCONTENTS;
			break;
		case PAF_ModifyPermissions:
			*data_ptr = MODIFYPERMISSIONS;
			break;
		case PAF_Present:
			*data_ptr = PRESENT1;
			break;
		case PAF_Invoke:
			*data_ptr = INVOKE;
			break;
		}
                current_listnode++;
        }
}



int DBVdbv_to_snacc_diagrec_list(AsnList *snacc_diaglist, DbvDiagRec **dbv_diaglist, jmp_buf env)
{
	DiagRec *data_ptr;
	DiagRec **data_ptr_address;

	int current_listnode = 0;
 
        while(dbv_diaglist[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(snacc_diaglist);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(DiagRec));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
		DBVdbv_to_snacc_diag_rec(data_ptr, dbv_diaglist[current_listnode], env);	
                current_listnode++;
        }
}



/* snacc -> dbv ansi Extended Service Task Package external */
int DBVsnacc_to_dbv_estaskpack_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	TaskPackageRecord *taskpack_ptr;
 
	taskpack_ptr = (TaskPackageRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_EsTaskPackage;
        psSingleASN1Type->uASN1.psEsTaskPackage = (struct DbvEsTaskPackage *) DbvAlloc (sizeof(struct DbvEsTaskPackage));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psEsTaskPackage, env);

	DBVsnacc_to_dbv_taskpack_main(psSingleASN1Type->uASN1.psEsTaskPackage, taskpack_ptr->a.johnfix, env);
}



/* snacc -> dbv iso Extended Service Task Package external */
int DBVsnacc_to_dbv_isoestaskpack_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoTaskPackageRecord *isotaskpack_ptr;
	isotaskpack_ptr = (IsoTaskPackageRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_EsTaskPackage;
        psSingleASN1Type->uASN1.psEsTaskPackage = (struct DbvEsTaskPackage *) DbvAlloc (sizeof(struct DbvEsTaskPackage));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psEsTaskPackage, env);

	DBVsnacc_to_dbv_taskpack_main(psSingleASN1Type->uASN1.psEsTaskPackage, isotaskpack_ptr->a.johnfix, env);
}



int DBVsnacc_to_dbv_taskpack_main(struct DbvEsTaskPackage *psEsTaskPackage, TaskPackage *taskpack_ptr, jmp_buf env)
{
	char *oid = NULL;
	int num_elements = 0;
	char *string = NULL;
	unsigned long length = 0;


	/* Compulsory */
	oid = (char *) ASN_EncodedToString(&taskpack_ptr->packageType);	
	psEsTaskPackage->pcPackageType = (char *) DbvAlloc (strlen(oid) + 1);
	CheckDbvAlloc(psEsTaskPackage->pcPackageType, env);

	memcpy(psEsTaskPackage->pcPackageType, oid, strlen(oid));


	if (taskpack_ptr->packageName.octetLen != 0) {
		psEsTaskPackage->psPackageName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psEsTaskPackage->psPackageName, env);
		DBVsnacc_to_dbv_inter_string(psEsTaskPackage->psPackageName, &taskpack_ptr->packageName,env);
	}


	if (taskpack_ptr->userId.octetLen != 0) {
		psEsTaskPackage->psUserId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psEsTaskPackage->psUserId, env);

		DBVsnacc_to_dbv_inter_string(psEsTaskPackage->psUserId, &taskpack_ptr->userId, env);
	}


	if (taskpack_ptr->retentionTime != NULL) {
		psEsTaskPackage->psRetentionTime = (DbvIntUnit *) DbvAlloc (sizeof(DbvIntUnit));
		CheckDbvAlloc(psEsTaskPackage->psRetentionTime, env);

		DBVsnacc_to_dbv_intunit(psEsTaskPackage->psRetentionTime, taskpack_ptr->retentionTime, env);
	}


	if (taskpack_ptr->permissions != NULL) {
		num_elements = AsnListCount(taskpack_ptr->permissions);
		psEsTaskPackage->ppsPermissions = (DbvPermissions **) DbvAlloc (sizeof(DbvPermissions *) * (num_elements + 1) );
		CheckDbvAlloc(psEsTaskPackage->ppsPermissions, env);

		DBVsnacc_to_dbv_permissions(psEsTaskPackage->ppsPermissions, taskpack_ptr->permissions, env);
	}


	if (taskpack_ptr->description.octetLen != 0) {
		psEsTaskPackage->psDescription = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psEsTaskPackage->psDescription, env);

		DBVsnacc_to_dbv_inter_string(psEsTaskPackage->psDescription, &taskpack_ptr->description, env);
	}


	if (taskpack_ptr->targetReference.octetLen != 0) {
		psEsTaskPackage->psTargetReference = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		CheckDbvAlloc(psEsTaskPackage->psTargetReference, env);

		DBVsnacc_to_dbv_octet_string(psEsTaskPackage->psTargetReference, &taskpack_ptr->targetReference, env);
	}


	if (taskpack_ptr->creationDateTime.octetLen != 0) {
		string = taskpack_ptr->creationDateTime.octs;
		length = taskpack_ptr->creationDateTime.octetLen;
		psEsTaskPackage->pcCreationDateTime = (char *) DbvAlloc (length + 1);
		CheckDbvAlloc(psEsTaskPackage->pcCreationDateTime, env);

		memcpy(psEsTaskPackage->pcCreationDateTime, string, length);
	}

	switch (taskpack_ptr->taskStatus) {
	case PENDING:
		psEsTaskPackage->eExtServicesTaskStatus = ESTS_Pending;
		break;
	case ACTIVE:
		psEsTaskPackage->eExtServicesTaskStatus = ESTS_Active;
		break;
	case COMPLETE:
		psEsTaskPackage->eExtServicesTaskStatus = ESTS_Complete;
		break;
	case ABORTED:
		psEsTaskPackage->eExtServicesTaskStatus = ESTS_Aborted;
		break;
	default:
		psEsTaskPackage->eExtServicesTaskStatus = ESTS_Invalid;
		break;
	}


	if (taskpack_ptr->packageDiagnostics != NULL) {
		num_elements = AsnListCount(taskpack_ptr->packageDiagnostics);
		psEsTaskPackage->ppsPackageDiagnostics = (DbvDiagRec **) DbvAlloc (sizeof(DbvDiagRec *) * (num_elements + 1) );
		CheckDbvAlloc(psEsTaskPackage->ppsPackageDiagnostics, env);

		Dbvsnacc_to_dbv_diagrec_list(psEsTaskPackage->ppsPackageDiagnostics, taskpack_ptr->packageDiagnostics, env);
	}

	if (taskpack_ptr->taskSpecificParameters != NULL)
		DBVsnacc_to_dbv_external(&psEsTaskPackage->sTaskSpecificParameters, taskpack_ptr->taskSpecificParameters, env);
	
}




int DBVsnacc_to_dbv_permissions(DbvPermissions **ppsPermissions, Permissions *permissions, jmp_buf env)
{
	PermissionsSeq *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, permissions) {
                ppsPermissions[index] = (DbvPermissions *) DbvAlloc(sizeof(DbvPermissions));
		CheckDbvAlloc(ppsPermissions[index], env);

                DBVsnacc_to_dbv_permisnode(ppsPermissions[index], data_ptr, env);
                index++;
        }
        ppsPermissions[index] = NULL;
}



int DBVsnacc_to_dbv_permisnode(DbvPermissions *psPermissions, PermissionsSeq *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	DBVsnacc_to_dbv_inter_string(&psPermissions->sUserId, &data_ptr->userId, env);
	num_elements = AsnListCount(data_ptr->allowableFunctions);
	psPermissions->ppeAllowableFunctions = (DbvPermAllowableFunctions **) DbvAlloc (sizeof(DbvPermAllowableFunctions *) * (num_elements + 1) );
	CheckDbvAlloc(psPermissions->ppeAllowableFunctions, env);

	Dbvsnacc_to_dbv_allowfuncts(psPermissions->ppeAllowableFunctions, data_ptr->allowableFunctions, env);
}



int Dbvsnacc_to_dbv_allowfuncts(DbvPermAllowableFunctions **ppeAllowableFunctions, PermissionsSeqSeqOf *allowableFunctions, jmp_buf env)
{
	PermissionsSeqSeqOfInt *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, allowableFunctions) {
                ppeAllowableFunctions[index] = (DbvPermAllowableFunctions *) DbvAlloc(sizeof(DbvPermAllowableFunctions));
		CheckDbvAlloc(ppeAllowableFunctions[index], env);

		switch (*data_ptr) {
		case DELETE:
			*ppeAllowableFunctions[index] = PAF_Delete;
			break;
		case MODIFYCONTENTS:
			*ppeAllowableFunctions[index] = PAF_ModifyContents;
			break;
		case MODIFYPERMISSIONS:
			*ppeAllowableFunctions[index] = PAF_ModifyPermissions;
			break;
		case PRESENT1:
			*ppeAllowableFunctions[index] = PAF_Present;
			break;
		case INVOKE:
			*ppeAllowableFunctions[index] = PAF_Invoke;
			break;
		default:
			*ppeAllowableFunctions[index] = PAF_Invalid;
			break;
		}
                index++;
        }
        ppeAllowableFunctions[index] = NULL;
}



int Dbvsnacc_to_dbv_diagrec_list(DbvDiagRec **dbv_diaglist, AsnList *snacc_diaglist, jmp_buf env)
{
	DiagRec *data_ptr;
        int num_elements = 0;
        int index = 0;
 
        FOR_EACH_LIST_ELMT(data_ptr, snacc_diaglist) {
                dbv_diaglist[index] = (DbvDiagRec *) DbvAlloc(sizeof(DbvDiagRec));
		CheckDbvAlloc(dbv_diaglist[index], env);

		DBVsnacc_to_dbv_diag_rec(dbv_diaglist[index], data_ptr, env);
                index++;
        }
        dbv_diaglist[index] = NULL;
}



/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* Update Extended Service */

/* dbv -> snacc ansi Update Extended Service external */
int DBVdbv_to_snacc_update_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static Update update_external[STRING_LEN];
 
        Update *update_ptr;

	/* Set up the Task Package external */
	update_ptr = &update_external[current_update];

	/* Clear the location (may have been used by the last PDU) */
	memset(update_ptr, 0, sizeof(Update));

	DBVdbv_to_snacc_update_main(update_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psUpdateExtendedService, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &update_external[current_update];

	/* Only if not at end of array */
	if (current_update < (STRING_LEN - 1)) {

		/* Increment current_update for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_update++;
	}
}



/* dbv -> snacc iso Update Extended Service external */
int DBVdbv_to_snacc_isoupdate_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoUpdate isoupdate_external[STRING_LEN];
        IsoUpdate *isoupdate_ptr;

	isoupdate_ptr = &isoupdate_external[current_isoupdate];
	memset(isoupdate_ptr, 0, sizeof(IsoUpdate));
	DBVdbv_to_snacc_update_main(isoupdate_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psUpdateExtendedService, env);
	any_type->value = (void *) &isoupdate_external[current_isoupdate];

	if (current_isoupdate < (STRING_LEN - 1)) {
		current_isoupdate++;
	}
}



int DBVdbv_to_snacc_update_main(Update *update_ptr, struct DbvUpdateExtendedService *psUpdateExtendedService, jmp_buf env)
{
	switch (psUpdateExtendedService->eUpdateReqRsp) {
	case UROR_Request:
		update_ptr->choiceId = UPDATE_ESREQUEST;
		update_ptr->a.esRequest = (struct UpdateSeq *) Asn1Alloc (sizeof(struct UpdateSeq));
		CheckAsn1Alloc(update_ptr->a.esRequest, env);

		DBVdbv_to_snacc_update_esreq(update_ptr->a.esRequest, psUpdateExtendedService->uUpdateReqOrRsp.psUpdateRequest, env);
		break;
	case UROR_Response:
		update_ptr->choiceId = UPDATE_TASKPACKAGE;
		update_ptr->a.taskPackage = (struct UpdateSeq1 *) Asn1Alloc (sizeof(struct UpdateSeq1));
		CheckAsn1Alloc(update_ptr->a.taskPackage, env);

		DBVdbv_to_snacc_update_taspack(update_ptr->a.taskPackage, psUpdateExtendedService->uUpdateReqOrRsp.psUpdateResponse, env);
		break;
	}
}



int DBVdbv_to_snacc_update_esreq(struct UpdateSeq* esRequest, DbvUpdateRequestPackage *psUpdateRequest, jmp_buf env)
{
	DatabaseName *data_ptr;
        int num_elements = 0;
        int index = 0;

	/* Both compulsory */
	esRequest->toKeep = (struct OriginPartToKeep5 *) Asn1Alloc (sizeof (struct OriginPartToKeep5));
	CheckAsn1Alloc(esRequest->toKeep, env);

	DBVdbv_to_snacc_parttokeep5(esRequest->toKeep, &psUpdateRequest->sOriginPartToKeep, env);


	/* OriginPartNotToKeep5 is a typdefed SuppliedRecords,
	   SuppliedRecords is a typdefed AsnList - 
	   a SEQUENCE OF SuppliedRecordsSeq  */

	esRequest->notToKeep = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(esRequest->notToKeep, env);

	DBVdbv_to_snacc_partnottokeep5(esRequest->notToKeep, psUpdateRequest->ppsSuppliedRecord, env);
	
}



int DBVdbv_to_snacc_parttokeep5(struct OriginPartToKeep5 *toKeep, DbvOriginPartToKeep *psOriginPartToKeep, jmp_buf env)
{
	/* Compulsory */
	switch (psOriginPartToKeep->eAction) {
	case UA_RecordInsert:
		toKeep->action = RECORDINSERT;
		break;
	case UA_RecordReplace:
		toKeep->action = RECORDREPLACE;
		break;
	case UA_RecordDelete:
		toKeep->action = RECORDDELETE;
		break;
	case UA_ElementUpdate:
		toKeep->action = ELEMENTUPDATE;
		break;
	}
	

	DBVdbv_to_snacc_inter_string(&toKeep->databaseName, &psOriginPartToKeep->sDatabaseName, env);


	/* Next two optional */
	if (psOriginPartToKeep->pcSchema != NULL)
		ASN_MakeOid((char *) psOriginPartToKeep->pcSchema, &toKeep->schema, env);


	if (psOriginPartToKeep->psElementSetName != NULL)
		DBVdbv_to_snacc_inter_string(&toKeep->elementSetName, psOriginPartToKeep->psElementSetName, env);
		
}



int DBVdbv_to_snacc_partnottokeep5(OriginPartNotToKeep5 *notToKeep, DbvSuppliedRecord **ppsSuppliedRecord, jmp_buf env)
{

	/* OriginPartNotToKeep5 is a typdefed SuppliedRecords,
	   SuppliedRecords is a typdefed AsnList - 
	   a SEQUENCE OF SuppliedRecordsSeq  */

	SuppliedRecordsSeq *data_ptr;
	SuppliedRecordsSeq **data_ptr_address;

	int current_listnode = 0;


        while(ppsSuppliedRecord[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(notToKeep);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(SuppliedRecordsSeq));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_supplied_rec(data_ptr, ppsSuppliedRecord[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_supplied_rec(SuppliedRecordsSeq *data_ptr, DbvSuppliedRecord *psSuppliedRecord, jmp_buf env)
{
	/* All except last optional */
	if (psSuppliedRecord->psUpdtRecordId != NULL) {
		data_ptr->recordId = (struct SuppliedRecordsSeqChoice*) Asn1Alloc (sizeof(struct SuppliedRecordsSeqChoice));
		CheckAsn1Alloc(data_ptr->recordId, env);

		DBVdbv_to_snacc_recordid(data_ptr->recordId, psSuppliedRecord->psUpdtRecordId, env);
	}

	if (psSuppliedRecord->psUpdtSupplementalId != NULL) {
		data_ptr->supplementalId = (struct SuppliedRecordsSeqChoice1*) Asn1Alloc (sizeof(struct SuppliedRecordsSeqChoice1));
		CheckAsn1Alloc(data_ptr->supplementalId, env);

		DBVdbv_to_snacc_supple_id(data_ptr->supplementalId, psSuppliedRecord->psUpdtSupplementalId, env);
	}

	if (psSuppliedRecord->psCorrelationInfo != NULL) {
		data_ptr->correlationInfo = (struct CorrelationInfo*) Asn1Alloc (sizeof(struct CorrelationInfo));
		CheckAsn1Alloc(data_ptr->correlationInfo, env);

		DBVdbv_to_snacc_correl_info(data_ptr->correlationInfo, psSuppliedRecord->psCorrelationInfo, env);
	}
	

	/* Following compulsory */
	data_ptr->record = (struct NEWEXTERNAL *) Asn1Alloc (sizeof(struct NEWEXTERNAL));
	CheckAsn1Alloc(data_ptr->record, env);
	DBVdbv_to_snacc_external(data_ptr->record, &psSuppliedRecord->sRecord, env);

	
}


int DBVdbv_to_snacc_recordid(struct SuppliedRecordsSeqChoice* recordId, DbvUpdtRecordId *psUpdtRecordId, jmp_buf env)
{
	switch (psUpdtRecordId->eRecordIdType) {
	case URIT_Number:
		recordId->choiceId = SUPPLIEDRECORDSSEQCHOICE_NUMBER;
		recordId->a.number = psUpdtRecordId->uUpdtRecordId.ulNumber;
		break;
	case URIT_String:
		recordId->choiceId = SUPPLIEDRECORDSSEQCHOICE_STRING;
		recordId->a.string = (InternationalString *) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(recordId->a.string, env);

		DBVdbv_to_snacc_inter_string(recordId->a.string, psUpdtRecordId->uUpdtRecordId.psString, env);
		break;
	case URIT_Opaque:
		recordId->choiceId = SUPPLIEDRECORDSSEQCHOICE_OPAQUE;
		recordId->a.opaque = (AsnOcts *) Asn1Alloc (sizeof(AsnOcts));
		CheckAsn1Alloc(recordId->a.opaque, env);

		DBVdbv_to_snacc_octet_string(recordId->a.opaque, psUpdtRecordId->uUpdtRecordId.psOpaque, env);

		break;
	}
	
}



int DBVdbv_to_snacc_supple_id(struct SuppliedRecordsSeqChoice1 *supplementalId, DbvUpdtSupplementalId *psUpdtSupplementalId, jmp_buf env)
{
	char *string = NULL;
	unsigned long length = 0;

	switch (psUpdtSupplementalId->eSupplementalIdType) {
	case USIT_Time:
		supplementalId->choiceId = SUPPLIEDRECORDSSEQCHOICE1_TIMESTAMP;

		string = psUpdtSupplementalId->uUpdtSupplementalId.pcTime;
		length = strlen(psUpdtSupplementalId->uUpdtSupplementalId.pcTime);
		supplementalId->a.timeStamp = (GeneralizedTime *) Asn1Alloc (sizeof(GeneralizedTime));
		CheckAsn1Alloc(supplementalId->a.timeStamp, env);

		supplementalId->a.timeStamp->octs = (char *) Asn1Alloc (length);
		CheckAsn1Alloc(supplementalId->a.timeStamp->octs, env);

		memcpy(supplementalId->a.timeStamp->octs, string, length);
		supplementalId->a.timeStamp->octetLen = length;

		break;
	case USIT_Version:
		supplementalId->choiceId = SUPPLIEDRECORDSSEQCHOICE1_VERSIONNUMBER;
		supplementalId->a.versionNumber = (InternationalString *) Asn1Alloc (sizeof(InternationalString));
		CheckAsn1Alloc(supplementalId->a.versionNumber, env);

		DBVdbv_to_snacc_inter_string(supplementalId->a.versionNumber, psUpdtSupplementalId->uUpdtSupplementalId.psVersion, env);
		break;
	case USIT_PreviousVersion:
		supplementalId->choiceId = SUPPLIEDRECORDSSEQCHOICE1_PREVIOUSVERSION;
		supplementalId->a.previousVersion = (struct NEWEXTERNAL*) DbvAlloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(supplementalId->a.previousVersion, env);

		DBVdbv_to_snacc_external(supplementalId->a.previousVersion, psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion, env);
		break;
	}
}



int DBVdbv_to_snacc_correl_info(struct CorrelationInfo* correlationInfo, DbvCorrelationInfo *psCorrelationInfo, jmp_buf env)
{
	/* Both optional */
	if (psCorrelationInfo->psNote != NULL)
		DBVdbv_to_snacc_inter_string(&correlationInfo->note, psCorrelationInfo->psNote, env);

	if (psCorrelationInfo->pulId != NULL) {
		correlationInfo->id = (AsnInt*) Asn1Alloc (sizeof(AsnInt));
		CheckAsn1Alloc(correlationInfo->id, env);

		*correlationInfo->id = *psCorrelationInfo->pulId;
	}
	
}



int DBVdbv_to_snacc_update_taspack(struct UpdateSeq1 *taskPackage, DbvUpdateTaskPackage *psUpdateResponse, jmp_buf env)
{
	/* Both compulsory */
	taskPackage->originPart = (struct OriginPartToKeep5*) Asn1Alloc (sizeof (struct OriginPartToKeep5));
	CheckAsn1Alloc(taskPackage->originPart, env);

	DBVdbv_to_snacc_parttokeep5(taskPackage->originPart, &psUpdateResponse->sOriginPartToKeep, env);
	
	taskPackage->targetPart = (struct TargetPart5*) Asn1Alloc (sizeof(struct TargetPart5));
	CheckAsn1Alloc(taskPackage->targetPart, env);

	DBVdbv_to_snacc_targetpart(taskPackage->targetPart, &psUpdateResponse->sTargetPart, env);
}



int DBVdbv_to_snacc_targetpart(struct TargetPart5* targetPart, DbvUpdateTargetPart *psTargetPart, jmp_buf env)
{
	switch (psTargetPart->eTaskStatus) {
	case UTS_Success:
		targetPart->updateStatus = SUCCESS6;
		break;
	case UTS_Partial:
		targetPart->updateStatus = PARTIAL1;
		break;
	case UTS_Failure:
		targetPart->updateStatus = FAILURE5;
		break;
	}


	if (psTargetPart->ppsGlobalDiagnostics != NULL) {
		targetPart->globalDiagnostics = AsnListNew(sizeof(void *));
		CheckAsn1Alloc(targetPart->globalDiagnostics, env);

		DBVdbv_to_snacc_diagrec_list(targetPart->globalDiagnostics, psTargetPart->ppsGlobalDiagnostics, env);
	}


	/* Following compulsory */
	targetPart->taskPackageRecords = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(targetPart->taskPackageRecords, env);

	DBVdbv_to_snacc_taskpackrecs(targetPart->taskPackageRecords, psTargetPart->ppsTaskPackageRecord, env);
}



int DBVdbv_to_snacc_taskpackrecs(TargetPart5SeqOf1 *taskPackageRecords, DbvTaskPackageRecord **ppsTaskPackageRecord, jmp_buf env)
{
	TaskPackageRecordStructure *data_ptr;
	TaskPackageRecordStructure **data_ptr_address;

        int current_listnode = 0;
 
        while(ppsTaskPackageRecord[current_listnode] != NULL)
        {
                data_ptr_address = AsnListAppend(taskPackageRecords);
		CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(TaskPackageRecordStructure));
		CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;
 
                DBVdbv_to_snacc_taskpackrec(data_ptr, ppsTaskPackageRecord[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_taskpackrec(TaskPackageRecordStructure *data_ptr, DbvTaskPackageRecord *psTaskPackageRecord, jmp_buf env)
{
	/* First two optional */
	if (psTaskPackageRecord->psUpdtRecordOrDiag != NULL) {
		data_ptr->recordOrSurDiag = (struct TaskPackageRecordStructureChoice*) Asn1Alloc (sizeof(struct TaskPackageRecordStructureChoice));
		CheckAsn1Alloc(data_ptr->recordOrSurDiag, env);

		DBVdbv_to_snacc_recorsur(data_ptr->recordOrSurDiag, psTaskPackageRecord->psUpdtRecordOrDiag, env);
	}


	if (psTaskPackageRecord->psCorrelationInfo != NULL) {
		data_ptr->correlationInfo = (struct CorrelationInfo* ) Asn1Alloc (sizeof(struct CorrelationInfo));
		CheckAsn1Alloc(data_ptr->correlationInfo, env);
		DBVdbv_to_snacc_correl_info(data_ptr->correlationInfo, psTaskPackageRecord->psCorrelationInfo, env);
	}


	/* recordStatus is compulsory */
	switch (psTaskPackageRecord->eUpdtRecordStatus) {
	case URST_Success:
		data_ptr->recordStatus = SUCCESS5;
		break;
	case URST_Queued:
		data_ptr->recordStatus = QUEUED;
		break;
	case URST_InProcess:
		data_ptr->recordStatus = INPROCESS;
		break;
	case URST_Failure:
		data_ptr->recordStatus = FAILURE4;
		break;
	}


	
}



int DBVdbv_to_snacc_recorsur(struct TaskPackageRecordStructureChoice* recordOrSurDiag, DbvUpdtRecordOrDiag *psUpdtRecordOrDiag, jmp_buf env)
{
	switch (psUpdtRecordOrDiag->eUpdtRecordOrDiag) {
	case URDT_Record:
		recordOrSurDiag->choiceId = TASKPACKAGERECORDSTRUCTURECHOICE_RECORD;
		recordOrSurDiag->a.record = (struct NEWEXTERNAL *) Asn1Alloc (sizeof(struct NEWEXTERNAL));
		CheckAsn1Alloc(recordOrSurDiag->a.record, env);

		DBVdbv_to_snacc_external(recordOrSurDiag->a.record, psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord, env);
		break;
	case URDT_Diagnostic:
		recordOrSurDiag->choiceId = TASKPACKAGERECORDSTRUCTURECHOICE_DIAGNOSTIC;
		recordOrSurDiag->a.diagnostic = (struct DiagRec*) Asn1Alloc (sizeof(struct DiagRec));
		CheckAsn1Alloc(recordOrSurDiag->a.diagnostic, env);
		DBVdbv_to_snacc_diag_rec(recordOrSurDiag->a.diagnostic,psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic, env);	

		break;
	}
}



/* snacc -> dbv ansi Update Extended Service external */
int DBVsnacc_to_dbv_update_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Update *update_ptr;
 
	update_ptr = (Update *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_UpdateExtendedService;
        psSingleASN1Type->uASN1.psUpdateExtendedService = (struct DbvUpdateExtendedService *) DbvAlloc (sizeof(struct DbvUpdateExtendedService));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psUpdateExtendedService, env);

	DBVsnacc_to_dbv_update_main(psSingleASN1Type->uASN1.psUpdateExtendedService, update_ptr, env);
}



/* snacc -> dbv iso Update Extended Service external */
int DBVsnacc_to_dbv_isoupdate_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoUpdate *isoupdate_ptr;
 
	isoupdate_ptr = (IsoUpdate *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_UpdateExtendedService;
        psSingleASN1Type->uASN1.psUpdateExtendedService = (struct DbvUpdateExtendedService *) DbvAlloc (sizeof(struct DbvUpdateExtendedService));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psUpdateExtendedService, env);

	DBVsnacc_to_dbv_update_main(psSingleASN1Type->uASN1.psUpdateExtendedService, isoupdate_ptr, env);
}



int DBVsnacc_to_dbv_update_main(struct DbvUpdateExtendedService *psUpdateExtendedService, Update *update_ptr, jmp_buf env)
{
	switch (update_ptr->choiceId) {
	case UPDATE_ESREQUEST:
		psUpdateExtendedService->eUpdateReqRsp = UROR_Request;
		psUpdateExtendedService->uUpdateReqOrRsp.psUpdateRequest = (DbvUpdateRequestPackage *) DbvAlloc (sizeof(DbvUpdateRequestPackage));
		CheckDbvAlloc(psUpdateExtendedService->uUpdateReqOrRsp.psUpdateRequest, env);

		DBVsnacc_to_dbv_updatereq(psUpdateExtendedService->uUpdateReqOrRsp.psUpdateRequest, update_ptr->a.esRequest, env);
		break;
	case UPDATE_TASKPACKAGE:
		psUpdateExtendedService->eUpdateReqRsp = UROR_Response;
		psUpdateExtendedService->uUpdateReqOrRsp.psUpdateResponse = (DbvUpdateTaskPackage *) DbvAlloc (sizeof(DbvUpdateTaskPackage));
		CheckDbvAlloc(psUpdateExtendedService->uUpdateReqOrRsp.psUpdateResponse, env);

		DBVsnacc_to_dbv_updatersp(psUpdateExtendedService->uUpdateReqOrRsp.psUpdateResponse, update_ptr->a.taskPackage, env);
		break;
	}
}



int DBVsnacc_to_dbv_updatereq(DbvUpdateRequestPackage *psUpdateRequest, struct UpdateSeq *esRequest, jmp_buf env)
{

	int num_elements = 0;

	/* Both compulsory */

	/* Part to keep */
	DBVsnacc_to_dbv_parttokeep5(&psUpdateRequest->sOriginPartToKeep, esRequest->toKeep, env);


	/* Part not to keep */
	num_elements = AsnListCount(esRequest->notToKeep);

       /* +1 to allocate space for the list terminator NULL */
        psUpdateRequest->ppsSuppliedRecord = (DbvSuppliedRecord **) DbvAlloc(sizeof(DbvSuppliedRecord *) * (num_elements + 1));
	CheckDbvAlloc(psUpdateRequest->ppsSuppliedRecord, env);
 
	DBVsnacc_to_dbv_notokeep(psUpdateRequest->ppsSuppliedRecord, esRequest->notToKeep, env);
}



int DBVsnacc_to_dbv_parttokeep5(DbvOriginPartToKeep *psOriginPartToKeep, struct OriginPartToKeep5* toKeep, jmp_buf env)
{
	char *oid = NULL;

	switch (toKeep->action) {
	case RECORDINSERT:
		psOriginPartToKeep->eAction = UA_RecordInsert;
		break;
	case RECORDREPLACE:
		psOriginPartToKeep->eAction = UA_RecordReplace;
		break;
	case RECORDDELETE:
		psOriginPartToKeep->eAction = UA_RecordDelete;
		break;
	case ELEMENTUPDATE:
		psOriginPartToKeep->eAction = UA_ElementUpdate;
		break;
	default:
		psOriginPartToKeep->eAction = UA_Invalid;
		break;
	}


	DBVsnacc_to_dbv_inter_string(&psOriginPartToKeep->sDatabaseName, &toKeep->databaseName, env);

	if (toKeep->schema.octetLen != 0) {
		oid = (char *) ASN_EncodedToString(&(toKeep->schema));
		psOriginPartToKeep->pcSchema = (char *) DbvAlloc (strlen(oid) + 1);
		CheckDbvAlloc(psOriginPartToKeep->pcSchema, env);

		memcpy(psOriginPartToKeep->pcSchema, oid, strlen(oid));
	}


	if (toKeep->elementSetName.octetLen != 0) {
		psOriginPartToKeep->psElementSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		CheckDbvAlloc(psOriginPartToKeep->psElementSetName, env);

		DBVsnacc_to_dbv_inter_string(psOriginPartToKeep->psElementSetName, &toKeep->elementSetName, env);
	}
}



int DBVsnacc_to_dbv_notokeep(DbvSuppliedRecord **ppsSuppliedRecord, OriginPartNotToKeep5 *notToKeep, jmp_buf env)
{

	SuppliedRecordsSeq *data_ptr = NULL;
	int index = 0;

        FOR_EACH_LIST_ELMT(data_ptr, notToKeep) {
                ppsSuppliedRecord[index] = (DbvSuppliedRecord *) DbvAlloc (sizeof(DbvSuppliedRecord));
		CheckDbvAlloc(ppsSuppliedRecord[index], env);

                DBVsnacc_to_dbv_supp_rec(ppsSuppliedRecord[index], data_ptr, env);
                index++;
        }
        ppsSuppliedRecord[index] = NULL;
}



int DBVsnacc_to_dbv_supp_rec(DbvSuppliedRecord *psSuppliedRecord, SuppliedRecordsSeq *data_ptr, jmp_buf env)
{
	/* All optional except last external */
	if (data_ptr->recordId != NULL) {
		psSuppliedRecord->psUpdtRecordId = (DbvUpdtRecordId *) DbvAlloc (sizeof (DbvUpdtRecordId));
		CheckDbvAlloc(psSuppliedRecord->psUpdtRecordId, env);

		DBVsnacc_to_dbv_recordid(psSuppliedRecord->psUpdtRecordId, data_ptr->recordId, env);
	}


	if (data_ptr->supplementalId != NULL) {
		psSuppliedRecord->psUpdtSupplementalId = (DbvUpdtSupplementalId *) DbvAlloc (sizeof (DbvUpdtSupplementalId));
		CheckDbvAlloc(psSuppliedRecord->psUpdtSupplementalId, env);

		DBVsnacc_to_dbv_suppid(psSuppliedRecord->psUpdtSupplementalId, data_ptr->supplementalId, env);
	}

	
	if (data_ptr->correlationInfo != NULL) {
		psSuppliedRecord->psCorrelationInfo = (DbvCorrelationInfo *) DbvAlloc (sizeof(DbvCorrelationInfo));
		CheckDbvAlloc(psSuppliedRecord->psCorrelationInfo, env);

		DBVsnacc_to_dbv_corr_info(psSuppliedRecord->psCorrelationInfo, data_ptr->correlationInfo, env);
	}



	/* This external is compulsory */
	DBVsnacc_to_dbv_external(&psSuppliedRecord->sRecord, data_ptr->record, env);

	
}



int DBVsnacc_to_dbv_recordid(DbvUpdtRecordId *psUpdtRecordId, struct SuppliedRecordsSeqChoice* recordId, jmp_buf env)
{
	switch (recordId->choiceId) {
	case SUPPLIEDRECORDSSEQCHOICE_NUMBER:
		psUpdtRecordId->eRecordIdType = URIT_Number;
		psUpdtRecordId->uUpdtRecordId.ulNumber = recordId->a.number;
		break;
	case SUPPLIEDRECORDSSEQCHOICE_STRING:
		psUpdtRecordId->eRecordIdType = URIT_String;
		psUpdtRecordId->uUpdtRecordId.psString = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psUpdtRecordId->uUpdtRecordId.psString, env);

		DBVsnacc_to_dbv_inter_string(psUpdtRecordId->uUpdtRecordId.psString, recordId->a.string, env);
		break;
	case SUPPLIEDRECORDSSEQCHOICE_OPAQUE:
		psUpdtRecordId->eRecordIdType = URIT_Opaque;
		psUpdtRecordId->uUpdtRecordId.psOpaque = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(psUpdtRecordId->uUpdtRecordId.psOpaque, env);

		DBVsnacc_to_dbv_octet_string(psUpdtRecordId->uUpdtRecordId.psOpaque, recordId->a.opaque, env);

		break;
	}
	
}



int DBVsnacc_to_dbv_suppid(DbvUpdtSupplementalId *psUpdtSupplementalId, struct SuppliedRecordsSeqChoice1* supplementalId, jmp_buf env)
{
	char *string = NULL;
	unsigned long length = 0;

	switch (supplementalId->choiceId) {
	case SUPPLIEDRECORDSSEQCHOICE1_TIMESTAMP:
		psUpdtSupplementalId->eSupplementalIdType = USIT_Time;
		string = supplementalId->a.timeStamp->octs;
		length = supplementalId->a.timeStamp->octetLen;
		psUpdtSupplementalId->uUpdtSupplementalId.pcTime = (char *)  DbvAlloc (length +1);
		CheckDbvAlloc(psUpdtSupplementalId->uUpdtSupplementalId.pcTime, env);

		memcpy(psUpdtSupplementalId->uUpdtSupplementalId.pcTime, string, length);
		break;
	case SUPPLIEDRECORDSSEQCHOICE1_VERSIONNUMBER:
		psUpdtSupplementalId->eSupplementalIdType = USIT_Version;
		psUpdtSupplementalId->uUpdtSupplementalId.psVersion = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psUpdtSupplementalId->uUpdtSupplementalId.psVersion, env);

		DBVsnacc_to_dbv_inter_string(psUpdtSupplementalId->uUpdtSupplementalId.psVersion, supplementalId->a.versionNumber, env);
		break;
	case SUPPLIEDRECORDSSEQCHOICE1_PREVIOUSVERSION:
		psUpdtSupplementalId->eSupplementalIdType = USIT_PreviousVersion;
		psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion, env);
		
		DBVsnacc_to_dbv_external(psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion, supplementalId->a.previousVersion, env);
		break;
	}
	
}



int DBVsnacc_to_dbv_corr_info(DbvCorrelationInfo *psCorrelationInfo, struct CorrelationInfo* correlationInfo, jmp_buf env)
{

	/* Both optionals */
	if (correlationInfo->note.octetLen != 0) {
		psCorrelationInfo->psNote = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		CheckDbvAlloc(psCorrelationInfo->psNote, env);

		DBVsnacc_to_dbv_inter_string(psCorrelationInfo->psNote, &correlationInfo->note, env);
	}

	if (correlationInfo->id != NULL) {
		psCorrelationInfo->pulId = (Ulong *) DbvAlloc (sizeof (Ulong));
		CheckDbvAlloc(psCorrelationInfo->pulId, env);

		*psCorrelationInfo->pulId = *correlationInfo->id;
	}
}
	



int DBVsnacc_to_dbv_updatersp(DbvUpdateTaskPackage *psUpdateResponse, struct UpdateSeq1 *taskPackage, jmp_buf env)
{
	/* Both these compulsory */
	DBVsnacc_to_dbv_parttokeep5(&psUpdateResponse->sOriginPartToKeep, taskPackage->originPart, env);

	DBVsnacc_to_dbv_targetPart(&psUpdateResponse->sTargetPart, taskPackage->targetPart, env); 
}



int DBVsnacc_to_dbv_targetPart(DbvUpdateTargetPart *psTargetPart, struct TargetPart5* targetPart, jmp_buf env) 
{
	int num_elements = 0;

	switch (targetPart->updateStatus) {
	case SUCCESS6:
		psTargetPart->eTaskStatus = UTS_Success;
		break;
	case PARTIAL1:
		psTargetPart->eTaskStatus = UTS_Partial;
		break;
	case FAILURE5:
		psTargetPart->eTaskStatus = UTS_Failure;
		break;
	default:
		psTargetPart->eTaskStatus = UTS_Invalid;
		break;
	}

	
	if (targetPart->globalDiagnostics != NULL) {
		num_elements = AsnListCount(targetPart->globalDiagnostics);

		/* +1 to allocate space for the list terminator NULL */
        	psTargetPart->ppsGlobalDiagnostics = (DbvDiagRec **) DbvAlloc(sizeof(DbvDiagRec *) * (num_elements + 1));
		CheckDbvAlloc(psTargetPart->ppsGlobalDiagnostics, env);

		Dbvsnacc_to_dbv_diagrec_list(psTargetPart->ppsGlobalDiagnostics,targetPart->globalDiagnostics, env); 
	}



	/* This is compulsory */
	num_elements = AsnListCount(targetPart->taskPackageRecords);
 
        /* +1 to allocate space for the list terminator NULL */
        psTargetPart->ppsTaskPackageRecord = (DbvTaskPackageRecord **) DbvAlloc(sizeof(DbvTaskPackageRecord *) * (num_elements + 1));
	CheckDbvAlloc(psTargetPart->ppsTaskPackageRecord, env);

        DBVsnacc_to_dbv_taskpack(psTargetPart->ppsTaskPackageRecord, targetPart->taskPackageRecords, env);

}



int DBVsnacc_to_dbv_taskpack(DbvTaskPackageRecord **ppsTaskPackageRecord, TargetPart5SeqOf1 *taskPackageRecords, jmp_buf env)
{

	TaskPackageRecordStructure *data_ptr = NULL;
	int index = 0;

        FOR_EACH_LIST_ELMT(data_ptr, taskPackageRecords) {
                ppsTaskPackageRecord[index] = (DbvTaskPackageRecord *) DbvAlloc (sizeof(DbvTaskPackageRecord));
		CheckDbvAlloc(ppsTaskPackageRecord[index] , env);

                DBVsnacc_to_dbv_taskpackrec(ppsTaskPackageRecord[index], data_ptr, env);
                index++;
        }
        ppsTaskPackageRecord[index] = NULL;
}



int DBVsnacc_to_dbv_taskpackrec(DbvTaskPackageRecord *psTaskPackageRecord, TaskPackageRecordStructure *data_ptr, jmp_buf env)
{
	if (data_ptr->recordOrSurDiag != NULL) {
		psTaskPackageRecord->psUpdtRecordOrDiag = (DbvUpdtRecordOrDiag *) DbvAlloc (sizeof (DbvUpdtRecordOrDiag));
		CheckDbvAlloc(psTaskPackageRecord->psUpdtRecordOrDiag, env);

		DBVsnacc_to_dbv_recordiag(psTaskPackageRecord->psUpdtRecordOrDiag, data_ptr->recordOrSurDiag, env);
	}


	if (data_ptr->correlationInfo != NULL) {
		psTaskPackageRecord->psCorrelationInfo = (DbvCorrelationInfo *) DbvAlloc (sizeof (DbvCorrelationInfo));
		CheckDbvAlloc(psTaskPackageRecord->psCorrelationInfo, env);

		DBVsnacc_to_dbv_corr_info(psTaskPackageRecord->psCorrelationInfo, data_ptr->correlationInfo, env);
	}


	/* This is compulsory */
	switch (data_ptr->recordStatus) {
	case SUCCESS5:
		psTaskPackageRecord->eUpdtRecordStatus = URST_Success;
		break;
	case QUEUED:
		psTaskPackageRecord->eUpdtRecordStatus = URST_Queued;
		break;
	case INPROCESS:
		psTaskPackageRecord->eUpdtRecordStatus = URST_InProcess;
		break;
	case FAILURE4:
		psTaskPackageRecord->eUpdtRecordStatus = URST_Failure;
		break;
	default:
		psTaskPackageRecord->eUpdtRecordStatus = URST_Invalid;
		break;
	}
				
	
}



int DBVsnacc_to_dbv_recordiag(DbvUpdtRecordOrDiag *psUpdtRecordOrDiag, struct TaskPackageRecordStructureChoice* recordOrSurDiag, jmp_buf env)
{
	switch (recordOrSurDiag->choiceId) {
	case TASKPACKAGERECORDSTRUCTURECHOICE_RECORD:
		psUpdtRecordOrDiag->eUpdtRecordOrDiag = URDT_Record;
		psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord, env);

		DBVsnacc_to_dbv_external(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord, recordOrSurDiag->a.record, env);
		break;
	case TASKPACKAGERECORDSTRUCTURECHOICE_DIAGNOSTIC:
		psUpdtRecordOrDiag->eUpdtRecordOrDiag = URDT_Diagnostic;
		psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
		CheckDbvAlloc(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic, env);

		DBVsnacc_to_dbv_diag_rec(psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic, recordOrSurDiag->a.diagnostic, env);
		break;
	}
}





/*******************************************************************/


/* snacc -> dbv unknown external */
int DBVsnacc_to_dbv_unknown_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Unknownexternal *unknown_ptr;
 
	unknown_ptr = (Unknownexternal *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_UnknownType;
        psSingleASN1Type->uASN1.psUnknownType = (struct DbvOctetString *) DbvAlloc (sizeof(struct DbvOctetString));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psUnknownType, env);

	DBVsnacc_to_dbv_octet_string(psSingleASN1Type->uASN1.psUnknownType, unknown_ptr, env);



	/* Because Unknown is a 'special' external with a 
	   single_ASN1_type.value and a NULL 
	   single_ASN1_type.ai (see z39_50_externals_a_1995.c function
	   BDecNEWEXTERNALContent), free the value field here because 
	   FreeAsnAnyDefinedBy called by FreeNEWEXTERNAL will do nothing 

	   (calling FreeAsnAnyDefinedBy actually calls FreeAsnAny because of 
	   the following #define in asn_any.h:

	   #define FreeAsnAnyDefinedBy FreeAsnAny
	   )
	*/

	if (unknown_ptr->octs != NULL)
		Asn1Free(unknown_ptr->octs);

	if (unknown_ptr != NULL)
		Asn1Free(unknown_ptr);
	
	  
}



/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* OctetAligned  - All V2 MARC targets use this (e.g. Cansearch) */
/* dbv -> snacc aligned external */
int DBVdbv_to_snacc_aligned(NEWEXTERNAL *snacc_external, DbvASN1External *dbv_external, jmp_buf env)
{

	/* Setting up a V2 octetaligned external is nice and easy */
	DBVdbv_to_snacc_octet_string(&(snacc_external->octet_aligned), dbv_external->uExternalValue.psOctetAligned, env);
}



/* snacc -> dbv aligned external */
int DBVsnacc_to_dbv_aligned(DbvASN1External *dbv_external, NEWEXTERNAL *snacc_external, jmp_buf env)
{
        dbv_external->eValueType = EVT_OctetAligned;
        dbv_external->uExternalValue.psOctetAligned = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
	CheckDbvAlloc(dbv_external->uExternalValue.psOctetAligned, env);

	DBVsnacc_to_dbv_octet_string(dbv_external->uExternalValue.psOctetAligned, &(snacc_external->octet_aligned), env);
}


/***********************************************************************/

/* Arbitrary  - I know of NO targets/origins in the world which use this */
/* dbv -> snacc arbitrary external */
int DBVdbv_to_snacc_arbitrary(NEWEXTERNAL *snacc_external, DbvASN1External *dbv_external, jmp_buf env)
{
	char *string = NULL;
	unsigned long length = 0;

        AsnBits *a1;
        unsigned long i;
	unsigned long bytes_to_alloc = 0;
	

	string = dbv_external->uExternalValue.psArbitrary->pcValue;
	length = dbv_external->uExternalValue.psArbitrary->ulLength;

	/* Default values */
	a1 = &snacc_external->arbitrary;
	a1->bits = Asn1Alloc(1);
	CheckAsn1Alloc(a1->bits, env);
	a1->bitLen = 0;

	/* If the psArbitrary structure contains data */
	if ((string != NULL) && (length != 0)) {
 
		bytes_to_alloc = (strlen(string) / 8) + 1;

		Asn1Free(a1->bits);
		a1->bits = Asn1Alloc(bytes_to_alloc);
		CheckAsn1Alloc(a1->bits, env);

		a1->bitLen = 8 * bytes_to_alloc;

        	for (i = 0; i < length; i++){
                	if (string[i] == '1')
                        	SetAsnBit(a1, i);
                	else
                        	ClrAsnBit(a1, i);
        	}

		a1->bitLen = i;
	}
 
}


int DBVsnacc_to_dbv_arbitrary(DbvASN1External *dbv_external, NEWEXTERNAL *snacc_external, jmp_buf env)
{
        char *string = NULL;
        unsigned long length;

	char *arb_string = NULL;
	unsigned long i;
	AsnBits *a1;
 
	a1 = &snacc_external->arbitrary;

	/* e.g. of following:
	 bitlen = 2 
	 arb_string = alloc (3) | | |
 	 < bitlen = 1           | | 
	 i = bitlen = 2             | 
	*/
	
        arb_string = (char *) DbvAlloc (a1->bitLen  + 1);
        CheckDbvAlloc(arb_string, env);


	for (i = 0; i < (a1->bitLen); i++){
		if (GetAsnBit(a1, i)) {
			arb_string[i] =  '1';
		} else {
			arb_string[i] =  '0';
		}
	}

	/* Terminate the string */
        arb_string[i] = 0;

	
	length = strlen(arb_string);

        dbv_external->eValueType = EVT_Arbitrary;
        dbv_external->uExternalValue.psArbitrary = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
	CheckDbvAlloc(dbv_external->uExternalValue.psArbitrary, env);
	
        dbv_external->uExternalValue.psArbitrary->ulLength = length;
        dbv_external->uExternalValue.psArbitrary->pcValue = (char *) DbvAlloc (length);
	CheckDbvAlloc(dbv_external->uExternalValue.psArbitrary->pcValue, env);

        memcpy(dbv_external->uExternalValue.psArbitrary->pcValue, arb_string, length);


	if (arb_string != NULL)
		DbvFree(arb_string);
}



/***************************************************************/
/* Structure checking routines */


int DBVcheck_DbvASN1External_opt(DbvAssocId lAssocId, DbvASN1External *psDbvASN1External, char *error_buffer, DbvError **ppsSIError)
{
	if (psDbvASN1External)
		return(DBVcheck_DbvASN1External(lAssocId, psDbvASN1External, error_buffer, ppsSIError));


	return SI_OK;
}


int DBVcheck_DbvASN1External(DbvAssocId lAssocId, DbvASN1External *psDbvASN1External, char *error_buffer, DbvError **ppsSIError)
{
	return(DBVcheck_dbv_external(lAssocId, psDbvASN1External, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_dbv_external(DbvAssocId lAssocId, DbvASN1External *psDbvASN1External, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);


	/* At this point a pointer to a DbvASN1External exists */
	

	/* if direct reference is not NULL check if it is a valid oid 
	   (it can be NULL because it is optional) */
	if (psDbvASN1External->pcObjectIdentifier != NULL)
		if ( (DBVcheck_dbv_recsyn(psDbvASN1External->pcObjectIdentifier) == SI_NOTOK) && (DBVcheck_dbv_allowed_external(psDbvASN1External->pcObjectIdentifier) == SI_NOTOK) ) {

			DBVsa_strcat(local_error_buffer, "pcObjectIdentifier");
                	return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
		}


	switch (psDbvASN1External->eValueType) {
	case EVT_SingleASN1Type:

		/* Although direct reference is optional, we need it if
		   we want to set up a SingleASN1Type external. If it's
		   not given for this type of external type then 
		   return an error */ 


		if (psDbvASN1External->pcObjectIdentifier == NULL) {
			DBVsa_strcat(local_error_buffer, "pcObjectIdentifier");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		DBVsa_strcat(local_error_buffer, "uExternalValue.psSingleASN1Type");

		if (psDbvASN1External->uExternalValue.psSingleASN1Type == NULL) 
		
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_singleasn1(lAssocId, psDbvASN1External->uExternalValue.psSingleASN1Type, psDbvASN1External->pcObjectIdentifier, local_error_buffer, ppsSIError));
		break;

	case EVT_OctetAligned:

		DBVsa_strcat(local_error_buffer, "uExternalValue.psOctetAligned");
		if (psDbvASN1External->uExternalValue.psOctetAligned == NULL) 
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		break;
	case EVT_Arbitrary:

		DBVsa_strcat(local_error_buffer, "uExternalValue.psArbitrary");

		if (psDbvASN1External->uExternalValue.psArbitrary == NULL) 
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_arbitrary(lAssocId, psDbvASN1External->uExternalValue.psArbitrary, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eValueType");
               	return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
        }
 
        return SI_OK;
}



int DBVcheck_dbv_singleasn1(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{

	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);
	
	switch (psSingleASN1Type->eASN1Type) {
	case AT_UnstructuredText:
		return(DBVcheck_at_unstructuredtext(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_OpacRecord:
		return(DBVcheck_at_opacrecord(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_SummaryRecord:
		return(DBVcheck_at_summaryrecord(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_ResourceReport1:
		return(DBVcheck_at_resourcereport1(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_ResourceReport2:
		return(DBVcheck_at_resourcereport2(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_EsTaskPackage:
		return(DBVcheck_at_estaskpackage(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_UpdateExtendedService:
		return(DBVcheck_at_updateextendedservice(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;

	case AT_ItemOrderExtendedService:
		return(DBVcheck_at_itemorderextendedservice(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;
	case AT_IllPdu:
		return(DBVcheck_at_isoill(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
		break;

        case AT_ExplainRecord:
                return(DBVcheck_at_explain(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;

        case AT_GenericRecord:
                return(DBVcheck_at_generic(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;

        case AT_PromptObject:
                return(DBVcheck_at_DbvPromptObject(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;



        case AT_Print:
                return(DBVcheck_at_DbvPrintExtendedService(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;

        case AT_Alert:
                return(DBVcheck_at_DbvAlertExtendedService(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;

        case AT_Corder:
                return(DBVcheck_at_DbvCOrder(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;




/****************
 No longer used

	case AT_IntMapTest:
		return(DBVcheck_at_intmaptest(lAssocId, psSingleASN1Type, local_error_buffer, ppsSIError));
		break;

****************/


        case AT_CharSetAndLanguage:
                return(DBVcheck_at_DbvCharSetAndLanguage(lAssocId, psSingleASN1Type, oid, local_error_buffer, ppsSIError));
                break;

	case AT_UnknownType:
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eASN1Type");

               	return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	
        return SI_OK;
}



int DBVcheck_at_unstructuredtext(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

	DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */

	if ( (strcmp(oid, ANSI_UNSTRUCTURED_TEXT_RECORD) != 0) &&
	     (strcmp(oid, ISO_UNSTRUCTURED_TEXT_RECORD) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
         	return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));

	}


	DBVsa_strcat(local_error_buffer, "uASN1.psUnstructuredText");
	if (psSingleASN1Type->uASN1.psUnstructuredText == NULL) 
              	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	return SI_OK;
}


			
int DBVcheck_at_opacrecord(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{

        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_OPAC_RECORD) != 0) &&
	     (strcmp(oid, ISO_OPAC_RECORD) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
              	return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
	}

	DBVsa_strcat(local_error_buffer, "uASN1.psOpacRecord");
	if (psSingleASN1Type->uASN1.psOpacRecord == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_opac(lAssocId, psSingleASN1Type->uASN1.psOpacRecord, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_opac(DbvAssocId lAssocId, struct DbvOpacRecord *psOpacRecord, char *error_buffer, DbvError **ppsSIError)
{
	
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* Both optional */

	if (psOpacRecord->psBibliographicRecord != NULL) {
		DBVsa_strcat(local_error_buffer, "psBibliographicRecord->");
		if (DBVcheck_dbv_external(lAssocId, psOpacRecord->psBibliographicRecord, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}


	/* If a list exists */
	if (psOpacRecord->ppsHoldingsData != NULL) {

 

        	/* Check all items in the list */
        	while (psOpacRecord->ppsHoldingsData[index] != NULL) {

			/* The following is a bit clumsy but safe (remember
			   that infinitley recursive externals are valid */
			DBVsa_strcpy(temp_error_buffer, local_error_buffer);
			DBVsa_strcat(temp_error_buffer, "ppsHoldingsData[");

			if (index < MAX_INDEX)
				sprintf(index_buffer, "%d", index);
			DBVsa_strcat(temp_error_buffer, index_buffer);
			DBVsa_strcat(temp_error_buffer, "]->");
			
 
                	if (DBVcheck_dbv_holdrec(lAssocId, psOpacRecord->ppsHoldingsData[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
				return SI_NOTOK;
 
        		index++;
        	}
	}
	
        return SI_OK;
}



int DBVcheck_dbv_holdrec(DbvAssocId lAssocId, DbvHoldingsRecord *psHoldingsData, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psHoldingsData->eHoldingsRecType) {
	case HRT_MarcHoldingsRecord:
		DBVsa_strcat(local_error_buffer, "uMarcOrHoldAndCirc.sMarcHoldingsRecord.");

		return(DBVcheck_dbv_external(lAssocId, &psHoldingsData->uMarcOrHoldAndCirc.sMarcHoldingsRecord, local_error_buffer, ppsSIError));
		break;
	case HRT_HoldingsAndCirc:
		DBVsa_strcat(local_error_buffer, "uMarcOrHoldAndCirc.sHoldingsAndCirc.");

		return(DBVcheck_dbv_holdandcirc(lAssocId, &psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc, local_error_buffer, ppsSIError));
		break;
	default:
		DBVsa_strcat(local_error_buffer, "eHoldingsRecType");
               	return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

        return SI_OK;
}



int DBVcheck_dbv_holdandcirc(DbvAssocId lAssocId, DbvHoldingsAndCircData *psHoldingsAndCirc, char *error_buffer, DbvError **ppsSIError)
{

        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcpy(temp_error_buffer, local_error_buffer);

	if (DBVcheck_dbv_volume(lAssocId, psHoldingsAndCirc->ppsVolume, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

        DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	return(DBV_check_dbv_circrec(lAssocId, psHoldingsAndCirc->ppsCircRecord, temp_error_buffer, ppsSIError));


	return SI_OK;

}




int DBVcheck_dbv_volume(DbvAssocId lAssocId, DbvVolume **ppsVolume, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If list is null then ok cos it is optional */
	if (ppsVolume == NULL) 
		return SI_OK;

	
	/* Right we know we have a list but does it just have
	   one NULL node ? */
	if (ppsVolume[0] ==  NULL) {
		DBVsa_strcat(local_error_buffer, "ppsVolume[0] (firstitem in a list must be non NULL)");

		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}
 

	return SI_OK;
}



int DBV_check_dbv_circrec(DbvAssocId lAssocId, DbvCircRecord **ppsCircRecord, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If list is null then ok cos it is optional */

	if (ppsCircRecord == NULL) 
		return SI_OK;

	
       	/* Check all items in the list */
       	while (ppsCircRecord[index] != NULL) {
 
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsCircRecord[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");

               	if (DBVcheck_circrec(lAssocId, ppsCircRecord[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

        	index++;
       	}
        return SI_OK;
}



int DBVcheck_circrec(DbvAssocId lAssocId, DbvCircRecord *psCircRecord, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* All optional except the booleans */
	
	if (DBVcheck_dbv_bool(psCircRecord->fAvailableNow) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fAvailableNow");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}



	if (DBVcheck_dbv_bool(psCircRecord->fRenewable) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRenewable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	if (DBVcheck_dbv_bool(psCircRecord->fOnHold) == SI_NOTOK)  {
		DBVsa_strcat(local_error_buffer, "fOnHold");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	return SI_OK;
}


int DBVcheck_at_summaryrecord(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_SUMMARY_RECORD) != 0) &&
	     (strcmp(oid, ISO_SUMMARY_RECORD) != 0) ) {

		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }


        DBVsa_strcat(local_error_buffer, "uASN1.psSummaryRecord");
	if (psSingleASN1Type->uASN1.psSummaryRecord == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_summary(lAssocId, psSingleASN1Type->uASN1.psSummaryRecord, local_error_buffer, ppsSIError));
}




int DBVcheck_dbv_summary(DbvAssocId lAssocId, struct DbvSummaryRecord *psSummaryRecord, char *error_buffer, DbvError **ppsSIError)
{

	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	
	/* no list is fine cos psBibliographicLevel is optional */
	if (psSummaryRecord->ppsFormat != NULL) {

       		/* No need to check items in DbvFormatSpec list */
       	}


	/* Dont have to check pulRank */
	
	
	if (psSummaryRecord->ppsOtherInformation != NULL) {
		DBVsa_strcat(local_error_buffer, "ppsOtherInformation->");
		return(DBVcheck_dbv_otherinfo(lAssocId, psSummaryRecord->ppsOtherInformation, local_error_buffer, ppsSIError));
	}


        return SI_OK;
}


int DBVcheck_at_resourcereport1(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_RESOURCE_REPORT_1) != 0) &&
	    (strcmp(oid, ISO_RESOURCE_REPORT_1) != 0) ) {

                DBVsa_strcat(local_error_buffer, "eASN1Type");
                return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }
 

        DBVsa_strcat(local_error_buffer, "uASN1.psResourceReport1");
	if (psSingleASN1Type->uASN1.psResourceReport1 == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_resrep1(lAssocId, psSingleASN1Type->uASN1.psResourceReport1, local_error_buffer, ppsSIError));

}




int DBVcheck_dbv_resrep1(DbvAssocId lAssocId, struct DbvResourceReport1 *psResourceReport1, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psResourceReport1->ppsEstimate == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsEstimate");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer,  ppsSIError));
	}

	
	/* While not an end of list */
	while (psResourceReport1->ppsEstimate[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsEstimate[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");

		if (DBVcheck_dbv_estimate1(lAssocId, psResourceReport1->ppsEstimate[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
        		return SI_NOTOK;
		index++;
	}

	return SI_OK;
}



int DBVcheck_dbv_estimate1(DbvAssocId lAssocId, DbvEstimate1 *psEstimate, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

       if (DBVcheck_dbv_enum(psEstimate->eEstimateType, ET_CurrentSearchRecords, ET_CurrentOtherAssocCost) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eEstimateType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}

 
        return SI_OK;
}



int DBVcheck_at_resourcereport2(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_RESOURCE_REPORT_2) != 0) &&
	     (strcmp(oid, ISO_RESOURCE_REPORT_2) != 0) ) {

                DBVsa_strcat(local_error_buffer, "eASN1Type");
                return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }
 

	DBVsa_strcat(local_error_buffer, "uASN1.psResourceReport2");
	if (psSingleASN1Type->uASN1.psResourceReport2 == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_resrep2(lAssocId, psSingleASN1Type->uASN1.psResourceReport2, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_resrep2(DbvAssocId lAssocId, struct DbvResourceReport2 *psResourceReport2, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* Can be NULL because optional */
	if (psResourceReport2->ppsEstimate != NULL) {

		/* While not an end of list */
		while (psResourceReport2->ppsEstimate[index] != NULL) {

			DBVsa_strcpy(temp_error_buffer, local_error_buffer);
			DBVsa_strcat(temp_error_buffer, "ppsEstimate[");

			if (index < MAX_INDEX)
				sprintf(index_buffer, "%d", index);
			DBVsa_strcat(temp_error_buffer, index_buffer);
			DBVsa_strcat(temp_error_buffer, "]->");

			if (DBVcheck_dbv_estimate2(lAssocId, psResourceReport2->ppsEstimate[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
       				return SI_NOTOK;
			index++;
		}
	}

       	return SI_OK;
}



int DBVcheck_dbv_estimate2(DbvAssocId lAssocId, DbvEstimate2 *psEstimate, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sType.");
	if (DBVcheck_DbvStringOrNumeric(lAssocId, &psEstimate->sType, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

        DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sValue.");
	if (DBVcheck_DbvIntUnit(lAssocId, &psEstimate->sValue, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;
 
        return SI_OK;
}



int DBVcheck_at_estaskpackage(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_ES_TASK_PACKAGE_RECORD) != 0) &&
	     (strcmp(oid, ISO_ES_TASK_PACKAGE_RECORD) != 0) ) {

                DBVsa_strcat(local_error_buffer, "eASN1Type");
                return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }
 

	DBVsa_strcat(local_error_buffer, "uASN1.psEsTaskPackage");
	if (psSingleASN1Type->uASN1.psEsTaskPackage == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_estask(lAssocId, psSingleASN1Type->uASN1.psEsTaskPackage, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_estask(DbvAssocId lAssocId, struct DbvEsTaskPackage *psEsTaskPackage, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psEsTaskPackage->pcPackageType != NULL)
		if (DBVcheck_dbv_validoid(psEsTaskPackage->pcPackageType) == SI_NOTOK) {
			DBVsa_strcat(local_error_buffer, "pcPackageType");
			return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
		}


	if (psEsTaskPackage->psRetentionTime != NULL) {

        	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
        	DBVsa_strcat(temp_error_buffer, "psRetentionTime");
		
		if (DBVcheck_DbvIntUnit(lAssocId, psEsTaskPackage->psRetentionTime, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
	

	if (psEsTaskPackage->ppsPermissions != NULL) {

 
		while (psEsTaskPackage->ppsPermissions[index] != NULL) {

			DBVsa_strcpy(temp_error_buffer, local_error_buffer);
			DBVsa_strcat(temp_error_buffer, "ppsPermissions[");

			if (index < MAX_INDEX)
				sprintf(index_buffer, "%d", index);
			DBVsa_strcat(temp_error_buffer, index_buffer);
			DBVsa_strcat(temp_error_buffer, "]->");


			if (DBVcheck_dbv_allowfuncs(lAssocId, psEsTaskPackage->ppsPermissions[index]->ppeAllowableFunctions, temp_error_buffer, ppsSIError) == SI_NOTOK)
				return SI_NOTOK;
			index++;
		}

	}
 

		
	if (DBVcheck_dbv_enum(psEsTaskPackage->eExtServicesTaskStatus, ESTS_Pending, ESTS_Aborted) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eExtServicesTaskStatus");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}


	if (psEsTaskPackage->ppsPackageDiagnostics != NULL) {
		DBVsa_strcat(local_error_buffer, "ppsPackageDiagnostics");
	        if (DBVcheck_dbv_multdiag(lAssocId, psEsTaskPackage->ppsPackageDiagnostics, local_error_buffer, ppsSIError) == SI_NOTOK)
                	return SI_NOTOK;
	}


        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* sTaskSpecificParameters compulsory */
	DBVsa_strcat(local_error_buffer, "sTaskSpecificParameters->");
	return(DBVcheck_dbv_external(lAssocId, &psEsTaskPackage->sTaskSpecificParameters, local_error_buffer, ppsSIError));
 
}



int DBVcheck_at_updateextendedservice(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 

	/* If external object idenifier is not valid for the external */
	if ( (strcmp(oid, ANSI_UPDATE_EXTENDED_SERVICE) != 0) &&
	     (strcmp(oid, ISO_UPDATE_EXTENDED_SERVICE) != 0) ) {
 
                DBVsa_strcat(local_error_buffer, "eASN1Type");
                return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
        }
 

	DBVsa_strcat(local_error_buffer, "uASN1.psUpdateExtendedService");
	if (psSingleASN1Type->uASN1.psUpdateExtendedService == NULL) 
               	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_dbv_update(lAssocId, psSingleASN1Type->uASN1.psUpdateExtendedService, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_update(DbvAssocId lAssocId, struct DbvUpdateExtendedService *psUpdateExtendedService, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
        switch (psUpdateExtendedService->eUpdateReqRsp) {
        case UROR_Request:

		DBVsa_strcat(local_error_buffer, "uUpdateReqOrRsp.psUpdateRequest");

                return(DBVcheck_dbv_updatereq(lAssocId, psUpdateExtendedService->uUpdateReqOrRsp.psUpdateRequest, local_error_buffer, ppsSIError));
                break;
        case UROR_Response:

		DBVsa_strcat(local_error_buffer, "uUpdateReqOrRsp.psUpdateResponse");
                return(DBVcheck_dbv_updatersp(lAssocId, psUpdateExtendedService->uUpdateReqOrRsp.psUpdateResponse, local_error_buffer, ppsSIError));
                break;
        default:
		DBVsa_strcat(local_error_buffer, "eUpdateReqRsp");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }

	
        return SI_OK;
}



int DBVcheck_dbv_updatereq(DbvAssocId lAssocId, DbvUpdateRequestPackage *psUpdateRequest, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
 
	if (psUpdateRequest == NULL) 
		return (DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");

	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sOriginPartToKeep.");
	if (DBVcheck_dbv_originpart(lAssocId, &psUpdateRequest->sOriginPartToKeep, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	if (DBVcheck_dbv_suppliedrecs(lAssocId, psUpdateRequest->ppsSuppliedRecord, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_originpart(DbvAssocId lAssocId, DbvOriginPartToKeep *psOriginPartToKeep, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);
	
        if (DBVcheck_dbv_enum(psOriginPartToKeep->eAction, UA_RecordInsert, UA_ElementUpdate) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eAction");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}


	
	/* pcSchema is optional so can be NULL */	
	if (psOriginPartToKeep->pcSchema == NULL)
		return SI_OK;


        if (DBVcheck_dbv_validoid(psOriginPartToKeep->pcSchema) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "pcSchema");
                return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));
	}

        return SI_OK;
}



int DBVcheck_dbv_suppliedrecs(DbvAssocId lAssocId, DbvSuppliedRecord **ppsSuppliedRecord, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* Cant be NULL because compulsory */
	if (ppsSuppliedRecord == NULL) {
		DBVsa_strcat(local_error_buffer, "ppsSuppliedRecord");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}



	while (ppsSuppliedRecord[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsSuppliedRecord[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");

		if (DBVcheck_dbv_supprec(lAssocId, ppsSuppliedRecord[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
		index++;
	}

	return SI_OK;
}



int DBVcheck_dbv_supprec(DbvAssocId lAssocId, DbvSuppliedRecord *psSuppliedRecord, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* In Z39.50 ASN, supprec is a list but in DBV it is just one
	   structure */

	/* Can be NULL cos optional */
	if (psSuppliedRecord->psUpdtRecordId != NULL) {
		DBVsa_strcat(local_error_buffer, "psUpdtRecordId->");
		if (DBVcheck_dbv_recid(lAssocId, psSuppliedRecord->psUpdtRecordId, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}

	
	if (psSuppliedRecord->psUpdtSupplementalId != NULL) {
		DBVsa_strcat(local_error_buffer, "psUpdtSupplementalId->");
		if (DBVcheck_dbv_suppid(lAssocId, psSuppliedRecord->psUpdtSupplementalId, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
	
	if (psSuppliedRecord->psCorrelationInfo != NULL) {
		DBVsa_strcat(local_error_buffer, "psCorrelationInfo->");
		if (DBVcheck_dbv_corrinfo(lAssocId, psSuppliedRecord->psCorrelationInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
	

	/* External is compulsory */
	DBVsa_strcat(local_error_buffer, "sRecord.");
	return(DBVcheck_dbv_external(lAssocId, &psSuppliedRecord->sRecord, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_recid(DbvAssocId lAssocId, DbvUpdtRecordId *psUpdtRecordId, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        switch (psUpdtRecordId->eRecordIdType) {
        case URIT_Number:
                break;
        case URIT_String:

		if (psUpdtRecordId->uUpdtRecordId.psString == NULL) {
			DBVsa_strcat(local_error_buffer, "uUpdtRecordId.psString");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

                break;
        case URIT_Opaque:

		if (psUpdtRecordId->uUpdtRecordId.psOpaque == NULL) {
			DBVsa_strcat(local_error_buffer, "uUpdtRecordId.psOpaque");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

                break;
        default:
		DBVsa_strcat(local_error_buffer, "eRecordIdType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }

	return SI_OK;
}



int DBVcheck_dbv_suppid(DbvAssocId lAssocId, DbvUpdtSupplementalId *psUpdtSupplementalId, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        switch (psUpdtSupplementalId->eSupplementalIdType) {
        case USIT_Time:

		if (psUpdtSupplementalId->uUpdtSupplementalId.pcTime == NULL) {
			DBVsa_strcat(local_error_buffer, "uUpdtSupplementalId.pcTime");
			
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

		break;
        case USIT_Version:

		if (psUpdtSupplementalId->uUpdtSupplementalId.psVersion == NULL) {
			DBVsa_strcat(local_error_buffer, "uUpdtSupplementalId.psVersion");
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		}

                break;
        case USIT_PreviousVersion:

		DBVsa_strcat(local_error_buffer, "uUpdtSupplementalId.psPreviousVersion");
		if (psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion == NULL) 
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_external(lAssocId, psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion, local_error_buffer, ppsSIError));
                break;
        default:
		DBVsa_strcat(local_error_buffer, "eRecordIdType");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }

	return SI_OK;
}



int DBVcheck_dbv_corrinfo(DbvAssocId lAssocId, DbvCorrelationInfo *psCorrelationInfo, char *error_buffer, DbvError **ppsSIError)
{
	return SI_OK;
}



int DBVcheck_dbv_updatersp(DbvAssocId lAssocId, DbvUpdateTaskPackage *psUpdateResponse, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psUpdateResponse == NULL)
		return (DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	DBVsa_strcat(local_error_buffer, "->");

	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sOriginPartToKeep.");
	if (DBVcheck_dbv_originpart(lAssocId, &psUpdateResponse->sOriginPartToKeep, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	
	DBVsa_strcpy(temp_error_buffer, local_error_buffer);
	DBVsa_strcat(temp_error_buffer, "sTargetPart.");
	if (DBVcheck_dbv_targetpart(lAssocId, &psUpdateResponse->sTargetPart, temp_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_dbv_targetpart(DbvAssocId lAssocId, DbvUpdateTargetPart *psTargetPart, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
        char index_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        if (DBVcheck_dbv_enum(psTargetPart->eTaskStatus, UTS_Success, UTS_Failure) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eTaskStatus");	
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* Can be NULL because optional */
	if (psTargetPart->ppsGlobalDiagnostics != NULL)  {


		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsGlobalDiagnostics");
		if (DBVcheck_dbv_multdiag(lAssocId, psTargetPart->ppsGlobalDiagnostics, temp_error_buffer, ppsSIError) == SI_NOTOK) 
			return SI_NOTOK;
	}


	/* Cant be NULL because compulsory */
	if (psTargetPart->ppsTaskPackageRecord == NULL)  {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsTaskPackageRecord");
		return(DBVdbv_field_missing_error(lAssocId, temp_error_buffer, ppsSIError));
	}

 


	while (psTargetPart->ppsTaskPackageRecord[index] != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "ppsTaskPackageRecord[");

		if (index < MAX_INDEX)
			sprintf(index_buffer, "%d", index);
		DBVsa_strcat(temp_error_buffer, index_buffer);
		DBVsa_strcat(temp_error_buffer, "]->");

		if (DBVcheck_dbv_taskpackrec(lAssocId, psTargetPart->ppsTaskPackageRecord[index], temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;	
		index++;
	}

	return SI_OK;
}



int DBVcheck_dbv_taskpackrec(DbvAssocId lAssocId, DbvTaskPackageRecord *psTaskPackageRecord, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* Can be NULL because optional */
	if (psTaskPackageRecord->psUpdtRecordOrDiag != NULL) {

		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "psUpdtRecordOrDiag->");

		if (DBVcheck_dbv_recordiag(lAssocId, psTaskPackageRecord->psUpdtRecordOrDiag, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
		
	if (psTaskPackageRecord->psCorrelationInfo != NULL) {
		
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "psCorrelationInfo->");
		if (DBVcheck_dbv_corrinfo(lAssocId, psTaskPackageRecord->psCorrelationInfo, temp_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	}
	

        if (DBVcheck_dbv_enum(psTaskPackageRecord->eUpdtRecordStatus, URST_Success, URST_Failure) == SI_NOTOK) {
		DBVsa_strcpy(temp_error_buffer, local_error_buffer);
		DBVsa_strcat(temp_error_buffer, "eUpdtRecordStatus");
		
                return(DBVdbv_invalid_enum_error(lAssocId, temp_error_buffer, ppsSIError));
	}


	return SI_OK;
}



int DBVcheck_dbv_recordiag(DbvAssocId lAssocId, DbvUpdtRecordOrDiag *psUpdtRecordOrDiag, char *error_buffer, DbvError **ppsSIError)
{
        char local_error_buffer[SA_ARRAY_SIZE];
        char temp_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);

        switch (psUpdtRecordOrDiag->eUpdtRecordOrDiag) {
        case URDT_Record:

		DBVsa_strcat(local_error_buffer, "uUpdtRecordOrDiag.psRecord");

		if (psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord == NULL) 
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
		

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_external(lAssocId, psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord, local_error_buffer, ppsSIError));

		break;
        case URDT_Diagnostic:

		DBVsa_strcat(local_error_buffer, "uUpdtRecordOrDiag.psDiagnostic");
		if (psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic == NULL) 
                	return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
			
		
		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_dbv_surrdiag(lAssocId, psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic, local_error_buffer, ppsSIError));

                
        default:
		DBVsa_strcat(local_error_buffer, "eUpdtRecordOrDiag");
                return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
                break;
        }

	return SI_OK;
}



int DBVcheck_dbv_arbitrary(DbvAssocId lAssocId, DbvOctetString *psArbitrary, char *error_buffer, DbvError **ppsSIError)
{
 
	char tempbuff[SA_ARRAY_SIZE];
        char local_error_buffer[SA_ARRAY_SIZE];
 
        DBVsa_strcpy(local_error_buffer, error_buffer);


	memset(tempbuff, 0, sizeof(tempbuff));

	/* Copy psArbitrary->pcValue to a zeroed temporary buffer before 
	   checking it because it is a char* in an octet string and so will
	   not be \0 terminated */

	if (psArbitrary->ulLength < SA_ARRAY_SIZE)
		memcpy(tempbuff, psArbitrary->pcValue, psArbitrary->ulLength);

	/* and must be a bit pattern (0s and 1s in a char* )  */
	if (DBVcheck_bit_string(tempbuff) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "pcValue");
		return(DBVdbv_bit_string_error(lAssocId, local_error_buffer, psArbitrary->pcValue, ppsSIError));
	}

        return SI_OK;
}


