/*  Module Name: @(#)z3950_externals3.c	7.3
    Version: 12/4/96
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description: More DBV-OSI II externals setup/get code
*/




/* New in here and also z3950_shared.c - optional field set up 
   wrapper functions to avoid duplicating pre-mapping
   initial allocs. The names of these functions end in _opt 

   These wrappers are ESSENTIAL now because of the massive duplication of 
   the structures involved.

   There are some _comp setups as well for enumeration and some other fields
   which require allocs. Cant really always use _opt functions for 
   _comp because there are cases where pointer parameters types dont match.

   These wrapper functions are only needed if a structure must be
   alloced before being mapped. Wrapper functions are only used if the
   alloc is required more than once or is generally useful to the rest 
   of the api.
   
   Q. Why not alloc opt syle in the main dbv_to_snacc/snacc_to_dbv function ? 
   A. Mustn't alloc/realloc for structures which are not pointers but are
      passed as pointers.

*/


#include "z3950_externals3.h"


/* Explain */

/* dbv -> snacc ansi Explain external */

int DBVdbv_to_snacc_explain_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static Explain_Record explain_external[STRING_LEN];
 
        Explain_Record *explain_ptr;

	/* Set up the Explain external */
	explain_ptr = &explain_external[current_explain];

	/* Clear the location (may have been used by the last PDU) */
	memset(explain_ptr, 0, sizeof(Explain_Record));

	DBVdbv_to_snacc_explain_main(explain_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &explain_external[current_explain];

	/* Only if not at end of array */
	if (current_explain < (STRING_LEN - 1)) {

		/* Increment current_explain for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_explain++;
	}
}



/* dbv -> snacc iso Explain external */

int DBVdbv_to_snacc_isoexplain_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoExplain_Record isoexplain_external[STRING_LEN];
        IsoExplain_Record *isoexplain_ptr;

	isoexplain_ptr = &isoexplain_external[current_isoexplain];
	memset(isoexplain_ptr, 0, sizeof(IsoExplain_Record));
	DBVdbv_to_snacc_explain_main(isoexplain_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord, env);
	any_type->value = (void *) &isoexplain_external[current_isoexplain];

	if (current_isoexplain < (STRING_LEN - 1)) {
		current_isoexplain++;
	}
}




int DBVdbv_to_snacc_explain_main(Explain_Record *explain_ptr, struct DbvExplainRecord *psExplainRecord, jmp_buf env)
{
	switch (psExplainRecord->eExplainRecordType) {
	case ERT_TargetInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_TARGETINFO;
		explain_ptr->a.targetInfo = (struct TargetInfo* ) Asn1Alloc (sizeof (struct TargetInfo));
		CheckAsn1Alloc(explain_ptr->a.targetInfo, env);
		DBVdbv_to_snacc_exp_targetinfo(explain_ptr->a.targetInfo, psExplainRecord->uExplainRecord.psTargetInfo, env);
		break;

	case ERT_DatabaseInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_DATABASEINFO;
		explain_ptr->a.databaseInfo = (struct DatabaseInfo* ) Asn1Alloc (sizeof (struct DatabaseInfo));
		CheckAsn1Alloc(explain_ptr->a.databaseInfo, env);
		DBVdbv_to_snacc_exp_databaseinfo(explain_ptr->a.databaseInfo, psExplainRecord->uExplainRecord.psDatabaseInfo, env);
		break;

	case ERT_SchemaInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_SCHEMAINFO;
		explain_ptr->a.schemaInfo = (struct SchemaInfo* ) Asn1Alloc (sizeof (struct SchemaInfo));
		CheckAsn1Alloc(explain_ptr->a.schemaInfo, env);
		DBVdbv_to_snacc_exp_schemainfo(explain_ptr->a.schemaInfo, psExplainRecord->uExplainRecord.psSchemaInfo, env);
		break;

	case ERT_TagSetInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_TAGSETINFO;
		explain_ptr->a.tagSetInfo = (struct TagSetInfo* ) Asn1Alloc (sizeof (struct TagSetInfo));
		CheckAsn1Alloc(explain_ptr->a.tagSetInfo, env);
		DBVdbv_to_snacc_exp_tagsetinfo(explain_ptr->a.tagSetInfo, psExplainRecord->uExplainRecord.psTagSetInfo, env);
		break;

	case ERT_RecordSyntaxInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_RECORDSYNTAXINFO;
		explain_ptr->a.recordSyntaxInfo = (struct RecordSyntaxInfo* ) Asn1Alloc (sizeof (struct RecordSyntaxInfo));
		CheckAsn1Alloc(explain_ptr->a.recordSyntaxInfo, env);
		DBVdbv_to_snacc_exp_recsyninfo(explain_ptr->a.recordSyntaxInfo, psExplainRecord->uExplainRecord.psRecordSyntaxInfo, env);
		break;

	case ERT_AttributeSetInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_ATTRIBUTESETINFO;
		explain_ptr->a.attributeSetInfo = (struct AttributeSetInfo* ) Asn1Alloc (sizeof (struct AttributeSetInfo));
		CheckAsn1Alloc(explain_ptr->a.attributeSetInfo, env);
		DBVdbv_to_snacc_exp_attsetinfo(explain_ptr->a.attributeSetInfo, psExplainRecord->uExplainRecord.psAttributeSetInfo, env);
		break;

	case ERT_TermListInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_TERMLISTINFO;
		explain_ptr->a.termListInfo = (struct TermListInfo* ) Asn1Alloc (sizeof (struct TermListInfo));
		CheckAsn1Alloc(explain_ptr->a.termListInfo, env);
		DBVdbv_to_snacc_exp_termlistinfo(explain_ptr->a.termListInfo, psExplainRecord->uExplainRecord.psTermListInfo, env);
		break;

	case ERT_ExtendedServicesInfo:
		explain_ptr->choiceId = EXPLAIN_RECORD_EXTENDEDSERVICESINFO;
		explain_ptr->a.extendedServicesInfo = (struct ExtendedServicesInfo* ) Asn1Alloc (sizeof (struct ExtendedServicesInfo));
		CheckAsn1Alloc(explain_ptr->a.extendedServicesInfo, env);
		DBVdbv_to_snacc_exp_extservinfo(explain_ptr->a.extendedServicesInfo, psExplainRecord->uExplainRecord.psExtendedServicesInfo, env);
		break;

	case ERT_AttributeDetails:
		explain_ptr->choiceId = EXPLAIN_RECORD_ATTRIBUTEDETAILS;
		explain_ptr->a.attributeDetails = (struct AttributeDetails* ) Asn1Alloc (sizeof (struct AttributeDetails));
		CheckAsn1Alloc(explain_ptr->a.attributeDetails, env);
		DBVdbv_to_snacc_exp_attdetails(explain_ptr->a.attributeDetails, psExplainRecord->uExplainRecord.psAttributeDetails, env);
		break;

	case ERT_TermListDetails:
		explain_ptr->choiceId = EXPLAIN_RECORD_TERMLISTDETAILS;
		explain_ptr->a.termListDetails = (struct TermListDetails* ) Asn1Alloc (sizeof (struct TermListDetails));
		CheckAsn1Alloc(explain_ptr->a.termListDetails, env);
		DBVdbv_to_snacc_exp_termlistdetails(explain_ptr->a.termListDetails, psExplainRecord->uExplainRecord.psTermListDetails, env);
		break;

	case ERT_ElementSetDetails:
		explain_ptr->choiceId = EXPLAIN_RECORD_ELEMENTSETDETAILS;
		explain_ptr->a.elementSetDetails = (struct ElementSetDetails* ) Asn1Alloc (sizeof (struct ElementSetDetails));
		CheckAsn1Alloc(explain_ptr->a.elementSetDetails, env);
			DBVdbv_to_snacc_exp_elsetdetails(explain_ptr->a.elementSetDetails, psExplainRecord->uExplainRecord.psElementSetDetails, env);
			break;

	case ERT_RetrievalRecordDetails:
		explain_ptr->choiceId = EXPLAIN_RECORD_RETRIEVALRECORDDETAILS;
		explain_ptr->a.retrievalRecordDetails = (struct RetrievalRecordDetails* ) Asn1Alloc (sizeof (struct RetrievalRecordDetails));
		CheckAsn1Alloc(explain_ptr->a.retrievalRecordDetails, env);
		DBVdbv_to_snacc_exp_retrecdetails(explain_ptr->a.retrievalRecordDetails, psExplainRecord->uExplainRecord.psRetrievalRecordDetails, env);
		break;

	case ERT_SortDetails:
		explain_ptr->choiceId = EXPLAIN_RECORD_SORTDETAILS;
		explain_ptr->a.sortDetails = (struct SortDetails* ) Asn1Alloc (sizeof (struct SortDetails));
		CheckAsn1Alloc(explain_ptr->a.sortDetails, env);
		DBVdbv_to_snacc_exp_sortdetails(explain_ptr->a.sortDetails, psExplainRecord->uExplainRecord.psSortDetails, env);
		break;

	case ERT_Processing:
		explain_ptr->choiceId = EXPLAIN_RECORD_PROCESSING;
		explain_ptr->a.processing = (struct ProcessingInformation* ) Asn1Alloc (sizeof (struct ProcessingInformation));
		CheckAsn1Alloc(explain_ptr->a.processing, env);
		DBVdbv_to_snacc_exp_processinginfo(explain_ptr->a.processing, psExplainRecord->uExplainRecord.psProcessing, env);
		break;

	case ERT_Variants:
		explain_ptr->choiceId = EXPLAIN_RECORD_VARIANTS;
		explain_ptr->a.variants = (struct VariantSetInfo* ) Asn1Alloc (sizeof (struct VariantSetInfo));
		CheckAsn1Alloc(explain_ptr->a.variants, env);
		DBVdbv_to_snacc_exp_variantsetinfo(explain_ptr->a.variants, psExplainRecord->uExplainRecord.psVariants, env);
		break;

	case ERT_Units:
		explain_ptr->choiceId = EXPLAIN_RECORD_UNITS;
		explain_ptr->a.units = (struct UnitInfo* ) Asn1Alloc (sizeof (struct UnitInfo));
		CheckAsn1Alloc(explain_ptr->a.units, env);
		DBVdbv_to_snacc_exp_unitinfo(explain_ptr->a.units, psExplainRecord->uExplainRecord.psUnits, env);
		break;

	case ERT_CategoryList:
		explain_ptr->choiceId = EXPLAIN_RECORD_CATEGORYLIST;
		explain_ptr->a.categoryList = (struct CategoryList* ) Asn1Alloc (sizeof (struct CategoryList));
		CheckAsn1Alloc(explain_ptr->a.categoryList, env);
		DBVdbv_to_snacc_exp_categorylist(explain_ptr->a.categoryList, psExplainRecord->uExplainRecord.psCategoryList, env);
		break;
	}
			
}


/* -------------------- DbvTargetInfo -------------------- */

int DBVdbv_to_snacc_exp_targetinfo(struct TargetInfo* targetInfo, DbvTargetInfo *psTargetInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&targetInfo->commonInfo, psTargetInfo->psCommonInfo, env);


	/* c */
	DBVdbv_to_snacc_inter_string(&targetInfo->name, &psTargetInfo->sName, env);
	

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->recent_news, psTargetInfo->ppsRecentNews, env);


	/* o */
	DBVdbv_to_snacc_exp_icon_list_opt(&targetInfo->icon, psTargetInfo->ppsIcon, env);


	/* c */
	DBVdbv_to_snacc_bool(&targetInfo->namedResultSets, &psTargetInfo->fNamedResultSets, env);
	
	
	/* c */
	DBVdbv_to_snacc_bool(&targetInfo->multipleDBsearch, &psTargetInfo->fMultipleDBSearch, env);

	/* o */
	DBVdbv_to_snacc_int_opt(&targetInfo->maxResultSets, psTargetInfo->pulMaxResultSets, env);

	/* o */
	DBVdbv_to_snacc_int_opt(&targetInfo->maxResultSize, psTargetInfo->pulMaxResultSize, env);


	/* o */
	DBVdbv_to_snacc_int_opt(&targetInfo->maxTerms, psTargetInfo->pulMaxTerms, env);

	/* o */
	DBVdbv_to_snacc_intunit_opt(&targetInfo->timeoutInterval, psTargetInfo->psTimeoutInterval, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->welcomeMessage, psTargetInfo->ppsWelcomeMessage, env);

	/* o */
	DBVdbv_to_snacc_exp_contactinfo_opt(&targetInfo->contactInfo, psTargetInfo->psContactInfo, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->description, psTargetInfo->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&targetInfo->nicknames, psTargetInfo->ppsNicknames, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->usage_restrictions, psTargetInfo->ppsUsageRest, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->paymentAddr, psTargetInfo->ppsPaymentAddr, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&targetInfo->hours, psTargetInfo->ppsHours, env);

	/* o */
	if (psTargetInfo->ppsDbCombinations != NULL) {
		targetInfo->dbCombinations = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(targetInfo->dbCombinations, env);
		DBVdbv_to_snacc_exp_databaselist_list(targetInfo->dbCombinations, psTargetInfo->ppsDbCombinations, env);
	}
		

	/* o */
	if (psTargetInfo->ppsAddresses != NULL) {
		targetInfo->addresses = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(targetInfo->addresses, env);
		DBVdbv_to_snacc_exp_addresses_list(targetInfo->addresses, psTargetInfo->ppsAddresses, env);
	}


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&targetInfo->languages, psTargetInfo->ppsLanguages, env);


	/* o */
	DBVdbv_to_snacc_exp_accessinfo_opt(&targetInfo->commonAccessInfo, psTargetInfo->psCommonAccessInfo, env);


}


int DBVdbv_to_snacc_exp_commoninfo_opt(struct CommonInfo** commonInfo, DbvCommonInfo *psCommonInfo, jmp_buf env)
{

	if (psCommonInfo != NULL) {
		*commonInfo = (struct CommonInfo*) Asn1Alloc(sizeof (struct CommonInfo));
		CheckAsn1Alloc(*commonInfo, env);
		DBVdbv_to_snacc_exp_commoninfo(*commonInfo, psCommonInfo, env);
	}
}

	

int DBVdbv_to_snacc_exp_commoninfo(struct CommonInfo* commonInfo, DbvCommonInfo *psCommonInfo, jmp_buf env)
{

	/* o */
	DBVdbv_to_snacc_gentime_opt(&commonInfo->dateAdded, psCommonInfo->pcDateAdded, env);

	/* o */
	DBVdbv_to_snacc_gentime_opt(&commonInfo->dateChanged, psCommonInfo->pcDateChanged, env);


	/* o */
	DBVdbv_to_snacc_gentime_opt(&commonInfo->expiry, psCommonInfo->pcExpiry, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&commonInfo->humanString_Language, psCommonInfo->psHumanStringLanguage, env);

	/* o */
	DBVdbv_to_snacc_otherinfo_opt(&commonInfo->otherInfo, psCommonInfo->ppsOtherInformation, env);
}



int DBVdbv_to_snacc_exp_humanstring_list_opt(HumanString **recent_news, DbvHumanString **ppsRecentNews, jmp_buf env)
{

	if (ppsRecentNews != NULL) {
		*recent_news = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*recent_news, env);
		DBVdbv_to_snacc_exp_humanstring_list(*recent_news, ppsRecentNews, env);
	}
}

int DBVdbv_to_snacc_exp_humanstring_list(HumanString *recent_news, DbvHumanString **ppsRecentNews, jmp_buf env)
{
        HumanStringSeq *data_ptr;
        HumanStringSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsRecentNews[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(recent_news);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(HumanStringSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_humanstring(data_ptr, ppsRecentNews[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_humanstring(HumanStringSeq *data_ptr, DbvHumanString *psRecentNews, jmp_buf env)
{
	/* o */

	/* DbvLanguageCode is a typedefed DbvInternationalString
	   LanguageCode is a typedefed InternationalString */

	if (psRecentNews->psLanguage != NULL) 
		DBVdbv_to_snacc_inter_string(&data_ptr->language, psRecentNews->psLanguage, env);
	

	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->text, &psRecentNews->sText, env);
	
}



int DBVdbv_to_snacc_exp_icon_list_opt(IconObject **icon, DbvIconObject **ppsIcon, jmp_buf env)
{

	if (ppsIcon != NULL) {
		*icon = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*icon, env);
		DBVdbv_to_snacc_exp_icon_list(*icon, ppsIcon, env);
	}
}
	


int DBVdbv_to_snacc_exp_icon_list(IconObject *icon, DbvIconObject **ppsIcon, jmp_buf env)
{
        IconObjectSeq *data_ptr;
        IconObjectSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsIcon[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(icon);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(IconObjectSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_icon(data_ptr, ppsIcon[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_icon(IconObjectSeq *data_ptr, DbvIconObject *psIcon, jmp_buf env)
{
	/* c */
	data_ptr->bodyType = (struct IconObjectSeqChoice*) Asn1Alloc (sizeof(struct IconObjectSeqChoice));
	CheckAsn1Alloc(data_ptr->bodyType, env);
	DBVdbv_to_snacc_exp_bodytype(data_ptr->bodyType, &psIcon->sBodyType, env);

	/* c */
	DBVdbv_to_snacc_octet_string(&data_ptr->content, &psIcon->sContent, env);

}


int DBVdbv_to_snacc_exp_bodytype(struct IconObjectSeqChoice* bodyType, DbvBodyType *psBodyType, jmp_buf env)
{
	switch (psBodyType->eBodyTypeType) {
	case BTT_IanaType:
		bodyType->choiceId = ICONOBJECTSEQCHOICE_IANATYPE;

		DBVdbv_to_snacc_interstring_comp(&bodyType->a.ianaType, psBodyType->uBodyType.psIanaType, env);

		break;
	case BTT_Z3950Type:
		bodyType->choiceId = ICONOBJECTSEQCHOICE_Z3950TYPE;

		DBVdbv_to_snacc_interstring_comp(&bodyType->a.z3950type, psBodyType->uBodyType.psZ3950Type, env);

		break;
	case BTT_OtherType:
		bodyType->choiceId = ICONOBJECTSEQCHOICE_OTHERTYPE;

		DBVdbv_to_snacc_interstring_comp(&bodyType->a.otherType, psBodyType->uBodyType.psOtherType, env);
		break;
	}
}


int DBVdbv_to_snacc_exp_contactinfo(struct ContactInfo* contactInfo, DbvContactInfo *psContactInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&contactInfo->name, psContactInfo->psName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&contactInfo->description, psContactInfo->ppsDescription, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&contactInfo->address, psContactInfo->ppsAddress, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&contactInfo->email, psContactInfo->psEmail, env);
	
	/* o */
	DBVdbv_to_snacc_inter_string(&contactInfo->phone, psContactInfo->psPhone, env);

}



int DBVdbv_to_snacc_exp_contactinfo_opt(struct ContactInfo** contactInfo, DbvContactInfo *psContactInfo, jmp_buf env)
{

	if (psContactInfo != NULL) {
		*contactInfo = (struct ContactInfo*) Asn1Alloc (sizeof (struct ContactInfo));
		CheckAsn1Alloc(*contactInfo, env);
		DBVdbv_to_snacc_exp_contactinfo(*contactInfo, psContactInfo, env);
	}
}



/********************************************************
typedef AsnList TargetInfoSeqOf1; // SEQUENCE OF DatabaseList //
typedef AsnList DatabaseList; // SEQUENCE OF DatabaseName //
typedef InternationalString DatabaseName;


so this is equivalent to 

typedef AsnList TargetInfoSeqOf1; // SEQUENCE OF DatabaseList //
typedef AsnList DatabaseList; // SEQUENCE OF InternationalString //


so TargetInfoSeqOf1 =


DatabaseList 0
	Internationalstring 0
	Internationalstring 1
	Internationalstring 2
		
DatabaseList 1
	Internationalstring 0
	Internationalstring 1

DatabaseList 2
	Internationalstring 0

********************************************************/



int DBVdbv_to_snacc_exp_databaselist_list(TargetInfoSeqOf1 *dbCombinations, DbvDatabaseList **ppsDbCombinations, jmp_buf env)
{
        DatabaseList *data_ptr;
        DatabaseList **data_ptr_address;

        int current_listnode = 0;

        while (ppsDbCombinations[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(dbCombinations);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(DatabaseList));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_dblist(data_ptr, ppsDbCombinations[current_listnode], env)
;
                current_listnode++;
        }
}



/* careful - no direct mapping here  - snacc doesnt have a structure but
   a single list, dbv has a structure to hold the single list */
int DBVdbv_to_snacc_dblist(DatabaseList *dblist, DbvDatabaseList *psDbCombinations, jmp_buf env)
{
        InternationalString *data_ptr;
        InternationalString **data_ptr_address;

        int current_listnode = 0;

        while (psDbCombinations->ppsInnerDatabases[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(dblist);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(InternationalString));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_inter_string(data_ptr, psDbCombinations->ppsInnerDatabases[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_databasename_list(DatabaseList *dblist, DbvDatabaseList *psDbCombinations, jmp_buf env)
{
        DatabaseName *data_ptr;
        DatabaseName **data_ptr_address;

        int current_listnode = 0;

        while (psDbCombinations->ppsInnerDatabases[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(dblist);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(DatabaseName));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

		/* database name is an inter string so blat it with inter */
                DBVdbv_to_snacc_inter_string(data_ptr, psDbCombinations->ppsInnerDatabases[current_listnode], env);
                current_listnode++;
        }
}


int DBVdbv_to_snacc_exp_addresses_list(TargetInfoSeqOf2 *addresses, DbvNetworkAddress **ppsAddresses, jmp_buf env)
{
        NetworkAddress *data_ptr;
        NetworkAddress **data_ptr_address;

        int current_listnode = 0;

        while (ppsAddresses[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(addresses);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(NetworkAddress));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_addresses(data_ptr, ppsAddresses[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_addresses(NetworkAddress *data_ptr, DbvNetworkAddress *psAddresses, jmp_buf env)
{
	switch (psAddresses->eNetworkAddressType) {
	case NAT_InternetAddress:
		data_ptr->choiceId = NETWORKADDRESS_INTERNETADDRESS;
		data_ptr->a.internetAddress = (struct NetworkAddressSeq*) Asn1Alloc (sizeof (struct NetworkAddressSeq));
		CheckAsn1Alloc(data_ptr->a.internetAddress, env);
		DBVdbv_to_snacc_exp_interaddr(data_ptr->a.internetAddress, psAddresses->uNetworkAddress.psInternetAddress, env);
		break;

	case NAT_OsiPresentationAddress:
		data_ptr->choiceId = NETWORKADDRESS_OSIPRESENTATIONADDRESS;
		data_ptr->a.osiPresentationAddress = (struct NetworkAddressSeq1*) Asn1Alloc (sizeof (struct NetworkAddressSeq1));
		CheckAsn1Alloc(data_ptr->a.internetAddress, env);
		DBVdbv_to_snacc_exp_osiaddr(data_ptr->a.osiPresentationAddress, psAddresses->uNetworkAddress.psOsiPresentationAddress, env);
		break;

	case NAT_OtherAddress:
		data_ptr->choiceId = NETWORKADDRESS_OTHER;
		data_ptr->a.other = (struct NetworkAddressSeq2*) Asn1Alloc (sizeof (struct NetworkAddressSeq2));
		CheckAsn1Alloc(data_ptr->a.other, env);
		DBVdbv_to_snacc_exp_otheraddr(data_ptr->a.other, psAddresses->uNetworkAddress.psOtherAddress, env);
		break;
	}
	
}



int DBVdbv_to_snacc_exp_interaddr(struct NetworkAddressSeq* internetAddress, DbvInterAddress *psInternetAddress, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&internetAddress->hostAddress, &psInternetAddress->sHostAddress, env);
	
	/* c */
	internetAddress->port = psInternetAddress->ulPort;
}



int DBVdbv_to_snacc_exp_osiaddr(struct NetworkAddressSeq1* osiPresentationAddress, DbvOsiPresentationAddress *psOsiPresentationAddress, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&osiPresentationAddress->pSel, &psOsiPresentationAddress->sPsel, env);
	
	/* o  - note that no malloc needed for snacc structure, 
	   snaccy, snaccy, snucky */
	DBVdbv_to_snacc_inter_string(&osiPresentationAddress->sSel, psOsiPresentationAddress->psSSel, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&osiPresentationAddress->tSel, psOsiPresentationAddress->psTSel, env);
	
	/* c */
	DBVdbv_to_snacc_inter_string(&osiPresentationAddress->nSap, &psOsiPresentationAddress->sNSap, env);

}


int DBVdbv_to_snacc_exp_otheraddr(struct NetworkAddressSeq2* other, DbvOtherAddress *psOtherAddress, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&other->type, &psOtherAddress->sType, env);
	
	/* c */
	DBVdbv_to_snacc_inter_string(&other->address, &psOtherAddress->sAddress, env);
	
}



int DBVdbv_to_snacc_exp_accessinfo_opt(struct AccessInfo** commonAccessInfo, DbvAccessInfo *psCommonAccessInfo, jmp_buf env)
{

	if (psCommonAccessInfo != NULL) {
		*commonAccessInfo = (struct AccessInfo*) Asn1Alloc (sizeof(struct AccessInfo));
		CheckAsn1Alloc(*commonAccessInfo, env);
		DBVdbv_to_snacc_exp_accessinfo(*commonAccessInfo, psCommonAccessInfo, env);
	}
}



int DBVdbv_to_snacc_exp_accessinfo(struct AccessInfo* commonAccessInfo, DbvAccessInfo *psCommonAccessInfo, jmp_buf env)
{
	/* o */
	if (psCommonAccessInfo->ppsQueryTypesSupported != NULL) {
		commonAccessInfo->queryTypesSupported = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(commonAccessInfo->queryTypesSupported, env);
		DBVdbv_to_snacc_exp_querytypedet_list(commonAccessInfo->queryTypesSupported, psCommonAccessInfo->ppsQueryTypesSupported, env);
	}



	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->diagnosticsSets, psCommonAccessInfo->ppcDiagnosticsSets, env);


	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->attributeSetIds, psCommonAccessInfo->ppcAttributeSetIds, env);


	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->schemas, psCommonAccessInfo->ppcSchemas, env);


	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->recordSyntaxes, psCommonAccessInfo->ppcRecordSyntaxes, env);


	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->resourceChallenges, psCommonAccessInfo->ppcResourceChallenges, env);


	/* o */
	if (psCommonAccessInfo->ppsRestrictedAccess != NULL) {
		commonAccessInfo->restrictedAccess = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(commonAccessInfo->restrictedAccess, env);
		DBVdbv_to_snacc_exp_restrictacces_list(commonAccessInfo->restrictedAccess, psCommonAccessInfo->ppsRestrictedAccess, env);
	}
	

	/* o */
	if (psCommonAccessInfo->psCostInfo != NULL) {
		commonAccessInfo->costInfo = (struct Costs*) Asn1Alloc(sizeof (struct Costs));
		CheckAsn1Alloc(commonAccessInfo->costInfo, env);
		DBVdbv_to_snacc_exp_costs(commonAccessInfo->costInfo, psCommonAccessInfo->psCostInfo, env);
	}
	
	
	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&commonAccessInfo->variantSets, psCommonAccessInfo->ppcVariantSets, env);


	/* o */

	/* LOOK OUT - THERE IS AN ElementsetNames STRUCTURE AND AN
	   ElementSetName (an InternationalString) STRUCTURE WHICH ARE 
    	   BOTH DIFFERENT */
	DBVdbv_to_snacc_interstring_list_opt(&commonAccessInfo->elementSetNames, psCommonAccessInfo->ppsElementSetNames, env);

		
	
	/* c */
	DBVdbv_to_snacc_interstring_list_opt(&commonAccessInfo->unitSystems, psCommonAccessInfo->ppsUnitSystems, env);

}



int DBVdbv_to_snacc_exp_querytypedet_list(AccessInfoSeqOf *queryTypesSupported, DbvQueryTypeDetails **ppsQueryTypesSupported, jmp_buf env)
{
        QueryTypeDetails *data_ptr;
        QueryTypeDetails **data_ptr_address;

        int current_listnode = 0;

        while (ppsQueryTypesSupported[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(queryTypesSupported);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(QueryTypeDetails));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_querytypedet(data_ptr, ppsQueryTypesSupported[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_querytypedet(QueryTypeDetails *data_ptr, DbvQueryTypeDetails *psQueryTypesSupported, jmp_buf env)
{
	switch (psQueryTypesSupported->eQueryTypeDetailsType) {
	case QTDT_Private:
		data_ptr->choiceId = QUERYTYPEDETAILS_PRIVATE;
		data_ptr->a.private = (struct PrivateCapabilities*) Asn1Alloc (sizeof (struct PrivateCapabilities));
		CheckAsn1Alloc(data_ptr->a.private, env);
		DBVdbv_to_snacc_exp_private(data_ptr->a.private, psQueryTypesSupported->uQueryTypeDetails.psPrivate, env);
		break;

	case QTDT_Rpn:
		data_ptr->choiceId = QUERYTYPEDETAILS_RPN;
		data_ptr->a.rpn = (struct RpnCapabilities*) Asn1Alloc (sizeof (struct RpnCapabilities));
		CheckAsn1Alloc(data_ptr->a.rpn, env);
		DBVdbv_to_snacc_exp_rpn(data_ptr->a.rpn, psQueryTypesSupported->uQueryTypeDetails.psRpn, env);
		break;

	case QTDT_Iso8777:
		data_ptr->choiceId = QUERYTYPEDETAILS_ISO8777;
		data_ptr->a.iso8777 = (struct Iso8777Capabilities*) Asn1Alloc (sizeof (struct Iso8777Capabilities));
		CheckAsn1Alloc(data_ptr->a.iso8777, env);
		DBVdbv_to_snacc_exp_iso8777(data_ptr->a.iso8777, psQueryTypesSupported->uQueryTypeDetails.psIso8777, env);
		break;

	case QTDT_Z39_58:
		data_ptr->choiceId = QUERYTYPEDETAILS_Z39_58;
		DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->a.z39_58, psQueryTypesSupported->uQueryTypeDetails.ppsZ39_58, env);
		break;

	case QTDT_Erpn:
		data_ptr->choiceId = QUERYTYPEDETAILS_ERPN;
		data_ptr->a.erpn = (struct RpnCapabilities*) Asn1Alloc (sizeof (struct PrivateCapabilities));
		CheckAsn1Alloc(data_ptr->a.erpn, env);
		DBVdbv_to_snacc_exp_rpn(data_ptr->a.erpn, psQueryTypesSupported->uQueryTypeDetails.psErpn, env);
		break;

	case QTDT_RankedList:
		data_ptr->choiceId = QUERYTYPEDETAILS_RANKEDLIST;
		DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->a.rankedList, psQueryTypesSupported->uQueryTypeDetails.ppsRankedList, env);
		break;
	}
}


int DBVdbv_to_snacc_exp_private(struct PrivateCapabilities* private, DbvPrivateCapabilities *psPrivate, jmp_buf env)
{
	/* o */
	if (psPrivate->ppsOperators != NULL) {
		private->operators = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(private->operators, env);
		DBVdbv_to_snacc_exp_operators_list(private->operators, psPrivate->ppsOperators, env);
	}

	/* o */
	DBVdbv_to_snacc_exp_searchkeys_list_opt(&private->searchKeys, psPrivate->ppsSearchKeys, env);


	/* o */
	if (psPrivate->ppsDescription != NULL) {
		private->description = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(private->description, env);
		DBVdbv_to_snacc_exp_humanstringseq_list_opt(&private->description, psPrivate->ppsDescription, env);
	}
	

}



int DBVdbv_to_snacc_exp_operators_list(PrivateCapabilitiesSeqOf *operators, DbvOperators **ppsOperators, jmp_buf env)
{
        PrivateCapabilitiesSeqOfSeq *data_ptr;
        PrivateCapabilitiesSeqOfSeq **data_ptr_address;
        

        int current_listnode = 0;

        while (ppsOperators[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(operators);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PrivateCapabilitiesSeqOfSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_operators(data_ptr, ppsOperators[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_operators(PrivateCapabilitiesSeqOfSeq *data_ptr, DbvOperators *psOperators, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->operator, &psOperators->sOperator, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psOperators->ppsDescription, env);
}


int DBVdbv_to_snacc_exp_searchkeys_list_opt(PrivateCapabilitiesSeqOf1 **searchKeys, DbvSearchKey **ppsSearchKey, jmp_buf env)
{

	if (ppsSearchKey != NULL) {
		*searchKeys = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*searchKeys, env);
		DBVdbv_to_snacc_exp_searchkeys_list(*searchKeys, ppsSearchKey, env);
	}
}



int DBVdbv_to_snacc_exp_searchkeys_list(PrivateCapabilitiesSeqOf1 *searchKeys, DbvSearchKey **ppsSearchKey, jmp_buf env)
{
        SearchKey *data_ptr;
        SearchKey **data_ptr_address;
        

        int current_listnode = 0;

        while (ppsSearchKey[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(searchKeys);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PrivateCapabilitiesSeqOf1));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_searchkey(data_ptr, ppsSearchKey[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_searchkey(SearchKey *data_ptr, DbvSearchKey *psSearchKey, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->searchKey, &psSearchKey->sSearchKey, env);
	
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psSearchKey->ppsDescription, env);
	
}



int DBVdbv_to_snacc_exp_humanstringseq_list_opt(PrivateCapabilitiesSeqOf2 **description, DbvHumanStringSeq **ppsDescription, jmp_buf env)
{

	if (ppsDescription != NULL) {
		*description = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*description, env);
		DBVdbv_to_snacc_exp_humanstringseq_list(*description, ppsDescription, env);
	}
}



int DBVdbv_to_snacc_exp_humanstringseq_list(PrivateCapabilitiesSeqOf2 *description, DbvHumanStringSeq **ppsDescription, jmp_buf env)
{
        HumanString *data_ptr;
        HumanString **data_ptr_address;
        

        int current_listnode = 0;

        while (ppsDescription[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(description);
                CheckAsn1Alloc(data_ptr_address, env);

		/* Because its another list */
		*data_ptr_address = AsnListNew (sizeof(void *));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_humanstring_list(data_ptr, ppsDescription[current_listnode]->ppsHumanString, env);
                current_listnode++;
	}
}



int DBVdbv_to_snacc_exp_rpn(struct RpnCapabilities* rpn, DbvRpnCapabilities *psRpn, jmp_buf env)
{
	/* o */
	if (psRpn->ppulOperators != NULL) {
		rpn->operators = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(rpn->operators, env);
		DBVdbv_to_snacc_exp_rpnoperators_list(rpn->operators, psRpn->ppulOperators, env);
	}

	/* c */
	DBVdbv_to_snacc_bool(&rpn->resultSetAsOperandSupported, &psRpn->fResultSetAsOperandSupported, env);
	

	/* c */
	DBVdbv_to_snacc_bool(&rpn->restrictionOperandSupported, &psRpn->fRestrictionOperandSupported, env);
	

	/* o */
	if (psRpn->psProximity != NULL) {
		rpn->proximity = (struct ProximitySupport*) Asn1Alloc (sizeof (struct ProximitySupport));
		CheckAsn1Alloc(rpn->proximity, env);
		DBVdbv_to_snacc_proxsupport(rpn->proximity, psRpn->psProximity, env);
	}

}




int DBVdbv_to_snacc_exp_rpnoperators_list(RpnCapabilitiesSeqOf *operators, Ulong **ppulOperators, jmp_buf env)
{
        AsnInt *data_ptr;
        AsnInt **data_ptr_address;
        

        int current_listnode = 0;

        while (ppulOperators[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(operators);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AsnInt));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

		*data_ptr = *ppulOperators[current_listnode];
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_proxsupport(struct ProximitySupport *proximity, DbvProximitySupport *psProximity, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_bool(&proximity->anySupport, &psProximity->fAnySupport, env);

	/* o */
	if (psProximity->ppsUnitsSupported != NULL) {
		proximity->unitsSupported = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(proximity->unitsSupported, env);
		DBVdbv_to_snacc_exp_unitssupported_list(proximity->unitsSupported, psProximity->ppsUnitsSupported, env);
	}
}



int DBVdbv_to_snacc_exp_unitssupported_list(ProximitySupportSeqOf *unitsSupported, DbvUnitsSupported **ppsUnitsSupported, jmp_buf env)
{
        ProximitySupportSeqOfChoice *data_ptr;
        ProximitySupportSeqOfChoice **data_ptr_address;
        
        int current_listnode = 0;

        while (ppsUnitsSupported[current_listnode] != NULL) {

                data_ptr_address = AsnListAppend(unitsSupported);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(ProximitySupportSeqOfChoice));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_unitssupported(data_ptr, ppsUnitsSupported[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_unitssupported(ProximitySupportSeqOfChoice *data_ptr, DbvUnitsSupported *psUnitsSupported, jmp_buf env)
{
	switch (psUnitsSupported->eUnitsSupportedType) {
	case UST_Known:
		data_ptr->choiceId = PROXIMITYSUPPORTSEQOFCHOICE_KNOWN;
		data_ptr->a.known = psUnitsSupported->uUnitsSupported.ulKnown;
		break;
	case UST_Private:
		data_ptr->choiceId = PROXIMITYSUPPORTSEQOFCHOICE_PRIVATE;
		data_ptr->a.private = (struct ProximitySupportSeqOfChoiceSeq*) Asn1Alloc (sizeof (struct ProximitySupportSeqOfChoiceSeq));
		CheckAsn1Alloc(data_ptr->a.private, env);
		DBVdbv_to_snacc_exp_privateunit(data_ptr->a.private, psUnitsSupported->uUnitsSupported.psPrivate, env);
		break;
	}
}



int DBVdbv_to_snacc_exp_privateunit(struct ProximitySupportSeqOfChoiceSeq* private, DbvPrivate *psPrivate, jmp_buf env)
{
	/* c */
	private->unit = psPrivate->ulUnit;

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&private->description, psPrivate->ppsDescription, env);
	
}



int DBVdbv_to_snacc_exp_iso8777(struct Iso8777Capabilities* iso8777, DbvIso8777Capabilities *psIso8777, jmp_buf env)
{
	/* c - but use optional set up wrapper cos need the malloc */
	DBVdbv_to_snacc_exp_searchkeys_list_opt(&iso8777->searchKeys, psIso8777->ppsSearchKeys, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&iso8777->restrictions, psIso8777->ppsRestrictions, env);
}



int DBVdbv_to_snacc_exp_objectident_list_opt(AccessInfoSeqOf1 **diagnosticsSets, DbvObjectIdentifier *pcDiagnosticsSets, jmp_buf env)
{

	if (pcDiagnosticsSets != NULL) {
		*diagnosticsSets = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*diagnosticsSets, env);
		DBVdbv_to_snacc_exp_objectident_list(*diagnosticsSets, pcDiagnosticsSets, env);
	}

}



int DBVdbv_to_snacc_exp_objectident_list(AccessInfoSeqOf1 *diagnosticsSets, DbvObjectIdentifier *dbv, jmp_buf env)
{
        AsnOid *data_ptr;
        AsnOid **data_ptr_address;

        int current_listnode = 0;

        while (dbv[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(diagnosticsSets);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AsnOid));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

		DBVdbv_to_snacc_objectident(data_ptr, dbv[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_restrictacces_list(AccessRestrictions *restrictedAccess, DbvAccessRestrictions **ppsRestrictedAccess, jmp_buf env)
{
        AccessRestrictionsSeq *data_ptr;
        AccessRestrictionsSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsRestrictedAccess[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(restrictedAccess);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AccessRestrictionsSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_restrictacces(data_ptr, ppsRestrictedAccess[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_restrictacces(AccessRestrictionsSeq *data_ptr, DbvAccessRestrictions *psRestrictedAccess, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_exp_accesstype(&data_ptr->accessType, &psRestrictedAccess->eAccessType, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->accessText, psRestrictedAccess->ppsAccessText, env);
	
	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&data_ptr->accessChallenges, psRestrictedAccess->ppcAccessChallenges, env);
		
}


int DBVdbv_to_snacc_exp_accesstype(AccessRestrictionsSeqInt *accessType, DbvAccessType *peAccessType, jmp_buf env)
{
	switch (*peAccessType) {
	case AT_Any:
		*accessType = ANY;
		break;
	case AT_Search:
		*accessType = SEARCH1;
		break;
	case AT_Present:
		*accessType = PRESENT2;
		break;
	case AT_SpecificElements:
		*accessType = SPECIFIC_ELEMENTS;
		break;
	case AT_ExtendedServices:
		*accessType = EXTENDED_SERVICES;
		break;
	case AT_ByDatabase:
		*accessType = BY_DATABASE;
		break;
	}
}


int DBVdbv_to_snacc_exp_costs(struct Costs *costInfo, DbvCosts *psCostInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_charge_opt(&costInfo->connectCharge, psCostInfo->psConnectCharge, env);

	/* o */
	DBVdbv_to_snacc_exp_charge_opt(&costInfo->connectTime, psCostInfo->psConnectTime, env);

	/* o */
	DBVdbv_to_snacc_exp_charge_opt(&costInfo->displayCharge, psCostInfo->psDisplayCharge, env);


	/* o */
	DBVdbv_to_snacc_exp_charge_opt(&costInfo->searchCharge, psCostInfo->psSearchCharge, env);


	/* o */
	DBVdbv_to_snacc_exp_charge_opt(&costInfo->subscriptCharge, psCostInfo->psSubscriptCharge, env);


	/* o */
	if (psCostInfo->ppsOtherCharge != NULL) {
		costInfo->otherCharges = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(costInfo->otherCharges, env);
		DBVdbv_to_snacc_exp_othercharge_list(costInfo->otherCharges, psCostInfo->ppsOtherCharge, env);

	}
}



int DBVdbv_to_snacc_exp_charge_opt(struct Charge **connectCharge, DbvCharge *psConnectCharge, jmp_buf env)
{
	if (psConnectCharge != NULL) {
		*connectCharge = (struct Charge *) Asn1Alloc (sizeof (struct Charge));
		CheckAsn1Alloc(*connectCharge, env);
		DBVdbv_to_snacc_exp_charge(*connectCharge, psConnectCharge, env);
	}
}



int DBVdbv_to_snacc_exp_charge(struct Charge* connectCharge, DbvCharge *psConnectCharge, jmp_buf env)
{
	/* c - needs wrapper */
	connectCharge->cost = (struct IntUnit*) Asn1Alloc(sizeof (struct IntUnit*));
	CheckAsn1Alloc(connectCharge->cost, env);
	DBVdbv_to_snacc_intunit(connectCharge->cost, &psConnectCharge->sCost, env);
	
	/* o */
	DBVdbv_to_snacc_unit_opt(&connectCharge->perWhat, psConnectCharge->psPerWhat, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&connectCharge->text, psConnectCharge->ppsText, env);
	
}


int DBVdbv_to_snacc_unit_opt(struct Unit **perWhat, DbvUnit *psPerWhat, jmp_buf env)
{

	if (psPerWhat != NULL) {
		*perWhat = (struct Unit *) Asn1Alloc (sizeof (struct Unit));
		CheckAsn1Alloc(*perWhat, env);
		DBVdbv_to_snacc_unit(*perWhat, psPerWhat, env);
	}
}



int DBVdbv_to_snacc_exp_othercharge_list(CostsSeqOf *otherCharges, DbvOtherCharge **ppsOtherCharge, jmp_buf env)
{
        CostsSeqOfSeq *data_ptr;
        CostsSeqOfSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsOtherCharge[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(otherCharges);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(CostsSeqOfSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_othercharge(data_ptr, ppsOtherCharge[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_othercharge(CostsSeqOfSeq *data_ptr, DbvOtherCharge *psOtherCharge, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->forWhat, psOtherCharge->ppsForWhat, env);

	/* c */
	data_ptr->charge = (struct Charge*) Asn1Alloc (sizeof (struct Charge));
	CheckAsn1Alloc(data_ptr->charge, env);
	DBVdbv_to_snacc_exp_charge(data_ptr->charge, &psOtherCharge->sCharge, env);
	
}



/* -------------------- DbvDatabaseInfo -------------------- */

int DBVdbv_to_snacc_exp_databaseinfo(struct DatabaseInfo* databaseInfo, DbvDatabaseInfo *psDatabaseInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&databaseInfo->commonInfo, psDatabaseInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&databaseInfo->name, &psDatabaseInfo->sName, env);
	/* o pointer to AsnNull  - weird man weird */
	DBVdbv_to_snacc_exp_nullpointer_opt(&databaseInfo->explainDatabase, psDatabaseInfo->pvExplainDatabase, env);


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&databaseInfo->nicknames, psDatabaseInfo->ppsNicknames, env);


	/* o */
	DBVdbv_to_snacc_exp_icon_list_opt(&databaseInfo->icon, psDatabaseInfo->ppsIcon, env);

	/* c */
	DBVdbv_to_snacc_bool(&databaseInfo->user_fee, &psDatabaseInfo->fUserFee, env);

	/* c */
	DBVdbv_to_snacc_bool(&databaseInfo->available, &psDatabaseInfo->fAvailable, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->titleString, psDatabaseInfo->ppsTitleString, env);

	/* o */
	if (psDatabaseInfo->ppsKeywords) {
		databaseInfo->keywords = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(databaseInfo->keywords, env);
		DBVdbv_to_snacc_exp_databaseinfo_list(databaseInfo->keywords, psDatabaseInfo->ppsKeywords, env);
	}



	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->description, psDatabaseInfo->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&databaseInfo->associatedDbs, psDatabaseInfo->ppsAssociatedDbs, env);


	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&databaseInfo->subDbs, psDatabaseInfo->ppsSubDbs, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->disclaimers, psDatabaseInfo->ppsDisclaimers, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->news, psDatabaseInfo->ppsNews, env);


	/* o */
	if (psDatabaseInfo->psRecordCount != NULL) {
		databaseInfo->recordCount = (struct DatabaseInfoChoice*) Asn1Alloc (sizeof (struct DatabaseInfoChoice));
		CheckAsn1Alloc(databaseInfo->recordCount, env);
		DBVdbv_to_snacc_exp_recordcount(databaseInfo->recordCount, psDatabaseInfo->psRecordCount, env);
	}
		
		
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->defaultOrder, psDatabaseInfo->ppsDefaultOrder, env);


	/* o */
	DBVdbv_to_snacc_int_opt(&databaseInfo->avRecordSize, psDatabaseInfo->pulAvRecordSize, env);
	

	/* o */
	DBVdbv_to_snacc_int_opt(&databaseInfo->maxRecordSize, psDatabaseInfo->pulMaxRecordSize, env);
	

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->hours, psDatabaseInfo->ppsHours, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->bestTime, psDatabaseInfo->ppsBestTime, env);

	/* o */
	DBVdbv_to_snacc_gentime_opt(&databaseInfo->lastUpdate, psDatabaseInfo->pcLastUpdate, env);

	/* o */
	DBVdbv_to_snacc_intunit_opt(&databaseInfo->updateInterval, psDatabaseInfo->psUpdateInterval, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->coverage, psDatabaseInfo->ppsCoverage, env);

	/* o */
	if (psDatabaseInfo->pfProprietary != NULL) {
		databaseInfo->proprietary = (AsnBool*) Asn1Alloc (sizeof (AsnBool));
		CheckAsn1Alloc(databaseInfo->proprietary, env);
		DBVdbv_to_snacc_bool(databaseInfo->proprietary, psDatabaseInfo->pfProprietary, env);
	}

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->copyrightText, psDatabaseInfo->ppsCopyrightText, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&databaseInfo->copyrightNotice, psDatabaseInfo->ppsCopyrightNotice, env);

	/* o */
	DBVdbv_to_snacc_exp_contactinfo_opt(&databaseInfo->producerContactInfo, psDatabaseInfo->psProducerContactInfo, env);

	/* o */
	DBVdbv_to_snacc_exp_contactinfo_opt(&databaseInfo->supplierContactInfo, psDatabaseInfo->psSupplierContactInfo, env);

	/* o */
	DBVdbv_to_snacc_exp_contactinfo_opt(&databaseInfo->submissionContactInfo, psDatabaseInfo->psSubmissionContactInfo, env);

	/* o */
	DBVdbv_to_snacc_exp_accessinfo_opt(&databaseInfo->accessInfo, psDatabaseInfo->psAccessInfo, env);

}


int DBVdbv_to_snacc_exp_nullpointer_opt(AsnNull** explainDatabase, DbvNull pvExplainDatabase, jmp_buf env)
{
	if (pvExplainDatabase != NULL) {
		*explainDatabase = (AsnNull*) Asn1Alloc (sizeof(AsnNull));
		CheckAsn1Alloc(*explainDatabase, env);

		/* dont have to do anything else ! */
	}
}
	

int DBVdbv_to_snacc_exp_databaseinfo_list(DatabaseInfoSeqOf1 *keywords, DbvDataBaseInfoSeq **ppsKeywords, jmp_buf env)
{
        HumanString *data_ptr;
        HumanString **data_ptr_address;

        int current_listnode = 0;

        while (ppsKeywords[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(keywords);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = AsnListNew (sizeof(void *));
                CheckAsn1Alloc(*data_ptr_address, env);
                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_humanstring_list(data_ptr, ppsKeywords[current_listnode]->ppsDataBaseInfo, env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_recordcount(struct DatabaseInfoChoice* recordCount, DbvRecordCount *psRecordCount, jmp_buf env)
{
	switch (psRecordCount->eRecordCountType) {
	case RCT_ActualNumber:

		recordCount->choiceId = DATABASEINFOCHOICE_ACTUALNUMBER;
		recordCount->a.actualNumber = psRecordCount->uRecordCount.ulActualNumber;
		break;
	case RCT_ApproxNumber:
		recordCount->choiceId = DATABASEINFOCHOICE_APPROXNUMBER;
		recordCount->a.approxNumber = psRecordCount->uRecordCount.ulApproxNumber;
		break;
	}
	
}



/* -------------------- DbvSchemaInfo -------------------- */

int DBVdbv_to_snacc_exp_schemainfo(struct SchemaInfo* schemaInfo, DbvSchemaInfo *psSchemaInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&schemaInfo->commonInfo, psSchemaInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&schemaInfo->schema, psSchemaInfo->pcSchema, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&schemaInfo->name, &psSchemaInfo->sName, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&schemaInfo->description, psSchemaInfo->ppsDescription, env);


	/* o */
	if (psSchemaInfo->ppsTagTypeMapping != NULL) {
		schemaInfo->tagTypeMapping = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(schemaInfo->tagTypeMapping, env);
		DBVdbv_to_snacc_exp_tagtypemapp_list(schemaInfo->tagTypeMapping, psSchemaInfo->ppsTagTypeMapping, env);

	}


	/* o */
	DBVdbv_to_snacc_exp_elementinfo_list_opt(&schemaInfo->recordStructure, psSchemaInfo->ppsRecordStructure, env);


}


int DBVdbv_to_snacc_exp_tagtypemapp_list(SchemaInfoSeqOf *tagTypeMapping, DbvTagTypeMapping **ppsTagTypeMapping, jmp_buf env)
{
        SchemaInfoSeqOfSeq *data_ptr;
        SchemaInfoSeqOfSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsTagTypeMapping[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(tagTypeMapping);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(SchemaInfoSeqOfSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_tagtypermapp(data_ptr, ppsTagTypeMapping[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_tagtypermapp(SchemaInfoSeqOfSeq *data_ptr, DbvTagTypeMapping *psTagTypeMapping, jmp_buf env)
{
	/* c */
	data_ptr->tagType = psTagTypeMapping->ulTagType;
	
	/* o */
	DBVdbv_to_snacc_objectident_opt(&data_ptr->tagSet, psTagTypeMapping->pcTagSet, env);

	/* o */
	DBVdbv_to_snacc_exp_nullpointer_opt(&data_ptr->defaultTagType, psTagTypeMapping->pvDefaultTagType, env);
	
}



int DBVdbv_to_snacc_exp_elementinfo_list_opt(SchemaInfoSeqOf1** recordStructure, DbvElementInfo **ppsRecordStructure, jmp_buf env)
{

	if (ppsRecordStructure != NULL) {
		*recordStructure = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*recordStructure, env);
		DBVdbv_to_snacc_exp_elementinfo_list(*recordStructure, ppsRecordStructure, env);

	}
}



int DBVdbv_to_snacc_exp_elementinfo_list(SchemaInfoSeqOf1 *recordStructure, DbvElementInfo **ppsRecordStructure, jmp_buf env)
{
        ElementInfo *data_ptr;
        ElementInfo **data_ptr_address;

        int current_listnode = 0;

        while (ppsRecordStructure[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(recordStructure);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(ElementInfo));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_elementinfo(data_ptr, ppsRecordStructure[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_elementinfo(ElementInfo *data_ptr, DbvElementInfo *psRecordStructure, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->elementName, &psRecordStructure->sElementName, env);
	
	/* c */
	data_ptr->elementTagPath = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->elementTagPath, env);
	DBVdbv_to_snacc_exp_path_list(data_ptr->elementTagPath, psRecordStructure->ppsElementTagPath, env);

	
	/* o */
	if (psRecordStructure->psDataType != NULL) {
		data_ptr->dataType = (struct ElementDataType *) DbvAlloc (sizeof (struct ElementDataType));
		CheckDbvAlloc(data_ptr->dataType, env);
		DBVdbv_to_snacc_exp_eldatatype(data_ptr->dataType, psRecordStructure->psDataType, env);
	}

	/* c */
	DBVdbv_to_snacc_bool(&data_ptr->required, &psRecordStructure->fRequired, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&data_ptr->repeatable, &psRecordStructure->fRepeatable, env);
	


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psRecordStructure->ppsDescription, env);
}



int DBVdbv_to_snacc_exp_path_list(Path *elementTagPath, DbvPath **ppsElementTagPath, jmp_buf env)
{
        PathSeq *data_ptr;
        PathSeq **data_ptr_address;

        int current_listnode = 0;

        while (ppsElementTagPath[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(elementTagPath);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PathSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_path(data_ptr, ppsElementTagPath[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_path(PathSeq *data_ptr, DbvPath *psElementTagPath, jmp_buf env)
{
	/* c */
	data_ptr->tagType = psElementTagPath->ulTagType;
	
	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->tagValue, &psElementTagPath->sTagValue, env);

}


int DBVdbv_to_snacc_exp_eldatatype(ElementDataType *dataType, struct DbvElementDataType *psDataType, jmp_buf env)
{
	switch (psDataType->eElementDataTypeType) {
	case EDTT_Primitive:
		dataType->choiceId = ELEMENTDATATYPE_PRIMITIVE;
		DBVdbv_to_snacc_primitive(&dataType->a.primitive, &psDataType->uElementDataType.ePrimitive, env);
		break;
		
	case EDTT_Structured:
		dataType->choiceId = ELEMENTDATATYPE_STRUCTURED;
		
		/* recursion here  - use the o set up function */
		DBVdbv_to_snacc_exp_elementinfo_list_opt(&dataType->a.structured, psDataType->uElementDataType.ppsStructured, env);
		break;
	}
}



int DBVdbv_to_snacc_primitive(PrimitiveDataType *primitive, DbvPrimitiveDataType *pePrimitive, jmp_buf env)
{
	switch (*pePrimitive) {
	case PDT_OctetString:
		*primitive = OCTETSTRING;
		break;
	case PDT_Numeric:
		*primitive = NUMERIC;
		break;
	case PDT_Date:
		*primitive = DATE;
		break;
	case PDT_External:
		*primitive = EXTERNAL1;
		break;
	case PDT_String:
		*primitive = STRING;
		break;
	case PDT_TrueOrFalse:
		*primitive = TRUEORFALSE;
		break;
	case PDT_Oid:
		*primitive = OID_SNACC;
		break;
	case PDT_IntUnit:
		*primitive = INTUNIT;
		break;
	case PDT_Empty:
		*primitive = EMPTY1;
		break;
	case PDT_NoneOfTheAbove:
		*primitive = NONEOFTHEABOVE;
		break;
	}
	
}



/* -------------------- DbvTagSetInfo -------------------- */

int DBVdbv_to_snacc_exp_tagsetinfo(struct TagSetInfo* tagSetInfo, DbvTagSetInfo *psTagSetInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&tagSetInfo->commonInfo, psTagSetInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&tagSetInfo->tagSet, psTagSetInfo->pcTagSet, env);
	
	/* c */
	DBVdbv_to_snacc_inter_string(&tagSetInfo->name, &psTagSetInfo->sName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&tagSetInfo->description, psTagSetInfo->ppsDescription, env);

	/* o */
	if (psTagSetInfo->ppsElements != NULL) {
		tagSetInfo->elements = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(tagSetInfo->elements, env);
		DBVdbv_to_snacc_exp_elements_list(tagSetInfo->elements, psTagSetInfo->ppsElements, env);

	}
}



int DBVdbv_to_snacc_exp_elements_list(TagSetInfoSeqOf *elements, DbvElements **ppsElements, jmp_buf env)
{
        TagSetInfoSeqOfSeq *data_ptr;
        TagSetInfoSeqOfSeq **data_ptr_address;

	DbvElements *psElements = NULL;

	/* 0 if NULL assigned */
        while (psElements = *ppsElements) {
                data_ptr_address = AsnListAppend(elements);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(TagSetInfoSeqOfSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_elements(data_ptr, psElements, env)
;

		/* Increment position in array of pointers */
		ppsElements++;
        }
}



int DBVdbv_to_snacc_exp_elements(TagSetInfoSeqOfSeq *data_ptr, DbvElements *psElements, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->elementname, &psElements->sElementName, env);
	
	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&data_ptr->nicknames, psElements->ppsNicknames, env);

	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->elementTag, &psElements->sElementTag, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psElements->ppsDescription, env);

	
	/* o */
	if (psElements->peDataType != NULL) {
		data_ptr->dataType = (PrimitiveDataType *) Asn1Alloc (sizeof (PrimitiveDataType));
		CheckAsn1Alloc(data_ptr->dataType, env);
		DBVdbv_to_snacc_primitive(data_ptr->dataType, psElements->peDataType, env);
	}


	/* o */
	DBVdbv_to_snacc_otherinfo_opt(&data_ptr->otherTagInfo, psElements->ppsOtherInformation, env);

}



/* -------------------- RecordSyntaxInfo -------------------- */

int DBVdbv_to_snacc_exp_recsyninfo(struct RecordSyntaxInfo* recordSyntaxInfo, DbvRecordSyntaxInfo *psRecordSyntaxInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&recordSyntaxInfo->commonInfo, psRecordSyntaxInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&recordSyntaxInfo->recordSyntax, psRecordSyntaxInfo->pcRecordSyntax, env);
	
	/* c */
	DBVdbv_to_snacc_inter_string(&recordSyntaxInfo->name, &psRecordSyntaxInfo->sName, env);
	

	/* o */
	DBVdbv_to_snacc_exp_objectident_list_opt(&recordSyntaxInfo->transferSyntaxes, psRecordSyntaxInfo->ppcTransferSyntaxes, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&recordSyntaxInfo->description, psRecordSyntaxInfo->ppsDescription, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&recordSyntaxInfo->asn1Module, psRecordSyntaxInfo->psAsn1Module, env);

	/* o */
	DBVdbv_to_snacc_exp_elementinfo_list_opt(&recordSyntaxInfo->abstractStructure, psRecordSyntaxInfo->ppsAbstractStructure, env);

}



/* -------------------- DbvAttributeSetInfo -------------------- */

int DBVdbv_to_snacc_exp_attsetinfo(struct AttributeSetInfo* attributeSetInfo, DbvAttributeSetInfo *psAttributeSetInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&attributeSetInfo->commonInfo, psAttributeSetInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&attributeSetInfo->attributeSet, psAttributeSetInfo->pcAttributeSetId, env);
	
	/* c */
	DBVdbv_to_snacc_inter_string(&attributeSetInfo->name, &psAttributeSetInfo->sName, env);
	

	/* o */
	if (psAttributeSetInfo->ppsAttributes != NULL) {
		attributeSetInfo->attributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(attributeSetInfo->attributes, env);
		DBVdbv_to_snacc_exp_attributetype_list(attributeSetInfo->attributes, psAttributeSetInfo->ppsAttributes, env);
	}
		

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&attributeSetInfo->description, psAttributeSetInfo->ppsDescription, env);

}



int DBVdbv_to_snacc_exp_attributetype_list(AttributeSetInfoSeqOf *attributes, DbvAttributeType **ppsAttributes, jmp_buf env)
{
        AttributeType *data_ptr;
        AttributeType **data_ptr_address;

	DbvAttributeType *psAttributes = NULL;

	/* 0 if NULL assigned */
        while (psAttributes = *ppsAttributes) {
                data_ptr_address = AsnListAppend(attributes);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeType));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attributetype(data_ptr, psAttributes, env);

		/* Increment position in array of pointers */
		ppsAttributes++;
        }
}



int DBVdbv_to_snacc_exp_attributetype(AttributeType *data_ptr, DbvAttributeType *psAttributes, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psAttributes->psName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psAttributes->ppsDescription, env);

	/* c */
	data_ptr->attributeType = psAttributes->ulAttributeType;


	/* AttributeType, AttributeTypeSeqOf: DbvAttributeType, DbvAttributeDescription */
	/* c */
	data_ptr->attributeValues = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->attributeValues, env);
	DBVdbv_to_snacc_exp_attributevalues_list(data_ptr->attributeValues, psAttributes->ppsAttributeValues, env);
		
}



int DBVdbv_to_snacc_exp_attributevalues_list(AttributeTypeSeqOf *attributeValues, DbvAttributeDescription **ppsAttributeValues, jmp_buf env)
{
        AttributeDescription *data_ptr;
        AttributeDescription **data_ptr_address;

	DbvAttributeDescription *psAttributeValues = NULL;

	/* 0 if NULL assigned */
        while (psAttributeValues = *ppsAttributeValues) {
                data_ptr_address = AsnListAppend(attributeValues);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeDescription));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attributevalues(data_ptr, psAttributeValues, env);

		/* Increment position in array of pointers */
		ppsAttributeValues++;
        }
}



int DBVdbv_to_snacc_exp_attributevalues(AttributeDescription *data_ptr, DbvAttributeDescription *psAttributeValues, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psAttributeValues->psName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psAttributeValues->ppsDescription, env);


	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->attributeValue, &psAttributeValues->sAttributeValue, env);

	/* o */
	if (psAttributeValues->ppsEquivalentAttributes != NULL) {
		data_ptr->equivalentAttributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->equivalentAttributes, env);
		DBVdbv_to_snacc_attcomplist(data_ptr->equivalentAttributes, psAttributeValues->ppsEquivalentAttributes, env);
	}
			
}



/* -------------------- DbvTermListInfo -------------------- */

int DBVdbv_to_snacc_exp_termlistinfo(struct TermListInfo* termListInfo, DbvTermListInfo *psTermListInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&termListInfo->commonInfo, psTermListInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&termListInfo->databaseName, &psTermListInfo->sDatabaseName, env);
	

	/* c */
	if (psTermListInfo->ppsTermLists != NULL) {
		termListInfo->termLists = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(termListInfo->termLists, env);
		DBVdbv_to_snacc_exp_termlist_list(termListInfo->termLists, psTermListInfo->ppsTermLists, env);
	}
		
}


int DBVdbv_to_snacc_exp_termlist_list(TermListInfoSeqOf *termLists, DbvTermList **ppsTermLists, jmp_buf env)
{
        TermListInfoSeqOfSeq *data_ptr;
        TermListInfoSeqOfSeq **data_ptr_address;

	DbvTermList *psTermLists = NULL;

	/* 0 if NULL assigned */
        while (psTermLists = *ppsTermLists) {
                data_ptr_address = AsnListAppend(termLists);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(TermListInfoSeqOfSeq));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_termlist(data_ptr, psTermLists, env);

		/* Increment position in array of pointers */
		ppsTermLists++;
        }
}



int DBVdbv_to_snacc_exp_termlist(TermListInfoSeqOfSeq *data_ptr, DbvTermList *psTermLists, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, &psTermLists->sName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->title, psTermLists->ppsTitle, env);

	/* o */
	if (psTermLists->peSearchCost != NULL) {
		data_ptr->searchCost = (TermListInfoSeqOfSeqInt *) Asn1Alloc (sizeof (TermListInfoSeqOfSeqInt));
		CheckAsn1Alloc(data_ptr->searchCost, env);
		DBVdbv_to_snacc_searchcost(data_ptr->searchCost, psTermLists->peSearchCost, env);
	}

	/* c */
	DBVdbv_to_snacc_bool(&data_ptr->scanable, &psTermLists->fScanable, env);

	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&data_ptr->broader, psTermLists->ppsBroader, env);
	

	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&data_ptr->narrower, psTermLists->ppsNarrower, env);
	

}


int DBVdbv_to_snacc_searchcost(TermListInfoSeqOfSeqInt *searchCost, DbvSearchCost *peSearchCost, jmp_buf env)
{
	switch (*peSearchCost) {
	case SC_Optimized:
		*searchCost = OPTIMIZED;
		break;
	case SC_Normal:
		*searchCost = NORMAL;
		break;
	case SC_Expensive:
		*searchCost = EXPENSIVE;
		break;
	case SC_Filter:
		*searchCost = FILTER;
		break;
	}
	
}



/* -------------------- DbvExtendedServicesInfo -------------------- */

int DBVdbv_to_snacc_exp_extservinfo(struct ExtendedServicesInfo* extendedServicesInfo, DbvExtendedServicesInfo *psExtendedServicesInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&extendedServicesInfo->commonInfo, psExtendedServicesInfo->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&extendedServicesInfo->type, psExtendedServicesInfo->pcType, env);
	
	/* o */
	DBVdbv_to_snacc_inter_string(&extendedServicesInfo->name, psExtendedServicesInfo->psName, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&extendedServicesInfo->privateType, &psExtendedServicesInfo->fPrivateType, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&extendedServicesInfo->restrictionsApply, &psExtendedServicesInfo->fRestrictionsApply, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&extendedServicesInfo->feeApply, &psExtendedServicesInfo->fFeeApply, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&extendedServicesInfo->available, &psExtendedServicesInfo->fAvailable, env);
	
	/* c */
	DBVdbv_to_snacc_bool(&extendedServicesInfo->retentionSupported, &psExtendedServicesInfo->fRetentionSupported, env);
	
	/* c */
	DBVdbv_to_snacc_exp_infoint(&extendedServicesInfo->waitAction, &psExtendedServicesInfo->eWaitAction, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&extendedServicesInfo->description, psExtendedServicesInfo->ppsDescription, env);
	
	/* o */
	DBVdbv_to_snacc_external_opt(&extendedServicesInfo->specificExplain, psExtendedServicesInfo->psSpecificExplain, env);


	/* o */
	DBVdbv_to_snacc_inter_string(&extendedServicesInfo->esASN, psExtendedServicesInfo->psEsASN, env);
	
}



int DBVdbv_to_snacc_exp_infoint(ExtendedServicesInfoInt *waitAction, DbvExtendedServicesInfoInt *peWaitAction, jmp_buf env)
{
	switch (*peWaitAction) {
	case ESII_WaitSupported:
		*waitAction = WAITSUPPORTED;
		break;
	case ESII_WaitAlways:
		*waitAction = WAITALWAYS;
		break;
	case ESII_WaitNotSupported:
		*waitAction = WAITNOTSUPPORTED;
		break;
	case ESII_Depends:
		*waitAction = DEPENDS;
		break;
	case ESII_NotSaying:
		*waitAction = NOTSAYING;
		break;
	}
}



int DBVdbv_to_snacc_external_opt(struct NEWEXTERNAL **specificExplain, DbvASN1External *psSpecificExplain, jmp_buf env)
{

	if (psSpecificExplain != NULL) {
		*specificExplain = (struct NEWEXTERNAL*) Asn1Alloc (sizeof (struct NEWEXTERNAL));
		CheckAsn1Alloc(*specificExplain, env);
		DBVdbv_to_snacc_external(*specificExplain, psSpecificExplain, env);
	}
}


/* -------------------- DbvAttributeDetails -------------------- */

int DBVdbv_to_snacc_exp_attdetails(struct AttributeDetails* attributeDetails, DbvAttributeDetails *psAttributeDetails, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&attributeDetails->commonInfo, psAttributeDetails->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&attributeDetails->databaseName, &psAttributeDetails->sDatabaseName, env);


	/* o */
	if (psAttributeDetails->ppsAttributesBySet != NULL) {
		attributeDetails->attributesBySet = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(attributeDetails->attributesBySet, env);
		DBVdbv_to_snacc_exp_attsbyset_list(attributeDetails->attributesBySet, psAttributeDetails->ppsAttributesBySet, env);
	}


	/* o */
	DBVdbv_to_snacc_attcombs_opt(&attributeDetails->attributeCombinations, psAttributeDetails->psAttributeCombinations, env);

}



int DBVdbv_to_snacc_exp_attsbyset_list(AttributeDetailsSeqOf *attributesBySet, DbvAttributesBySet **ppsAttributesBySet, jmp_buf env)
{
        AttributeSetDetails *data_ptr;
        AttributeSetDetails **data_ptr_address;

	DbvAttributesBySet *psAttributesBySet = NULL;

	/* 0 if NULL assigned */
        while (psAttributesBySet = *ppsAttributesBySet) {
                data_ptr_address = AsnListAppend(attributesBySet);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeSetDetails));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attsbyset(data_ptr, psAttributesBySet, env);

		/* Increment position in array of pointers */
		ppsAttributesBySet++;
        }
}



int DBVdbv_to_snacc_exp_attsbyset(AttributeSetDetails *data_ptr, DbvAttributesBySet *psAttributesBySet, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_objectident(&data_ptr->attributeSet, psAttributesBySet->pcAttributeSet, env);

	/* c */
	data_ptr->attributesByType = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->attributesByType, env);
	DBVdbv_to_snacc_exp_attsbytype_list(data_ptr->attributesByType, psAttributesBySet->ppsAttributesByType, env);
}



int DBVdbv_to_snacc_exp_attsbytype_list(AttributeSetDetailsSeqOf *attributesByType, DbvAttributesByType **ppsAttributesByType, jmp_buf env)
{
        AttributeTypeDetails *data_ptr;
        AttributeTypeDetails **data_ptr_address;

	DbvAttributesByType *psAttributesByType = NULL;

	/* 0 if NULL assigned */
        while (psAttributesByType = *ppsAttributesByType) {
                data_ptr_address = AsnListAppend(attributesByType);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeTypeDetails));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attsbytype(data_ptr, psAttributesByType, env);

		/* Increment position in array of pointers */
		ppsAttributesByType++;
        }
}



int DBVdbv_to_snacc_exp_attsbytype(AttributeTypeDetails *data_ptr, DbvAttributesByType *psAttributesByType, jmp_buf env)
{
	/* c */
	data_ptr->attributeType = psAttributesByType->ulAttributeType;
	
	/* o */
	if (psAttributesByType->psDefaultIfOmitted != NULL) {
		data_ptr->defaultIfOmitted = (struct OmittedAttributeInterpretation*) Asn1Alloc (sizeof (struct OmittedAttributeInterpretation));
		CheckAsn1Alloc(data_ptr->defaultIfOmitted, env);
		DBVdbv_to_snacc_exp_omitattint(data_ptr->defaultIfOmitted, psAttributesByType->psDefaultIfOmitted, env);
	}
	
	/* o */
	if (psAttributesByType->ppsAttributeValues != NULL) {
		data_ptr->attributeValues = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->attributeValues, env);
		DBVdbv_to_snacc_exp_attvalues_list(data_ptr->attributeValues, psAttributesByType->ppsAttributeValues, env);
	}
}



int DBVdbv_to_snacc_exp_omitattint(struct OmittedAttributeInterpretation* optionalType, DbvOmittedAttributeInterpretation *psOptionalType, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_strornum_opt(&optionalType->defaultValue, psOptionalType->psDefaultValue, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&optionalType->defaultDescription, psOptionalType->ppsDefaultDescription, env);
}


int DBVdbv_to_snacc_exp_attvalues_list(AttributeTypeDetailsSeqOf *attributeValues, DbvAttributeValues **ppsAttributeValues, jmp_buf env)
{
        AttributeValue *data_ptr;
        AttributeValue **data_ptr_address;

	DbvAttributeValues *psAttributeValues = NULL;

	/* 0 if NULL assigned */
        while (psAttributeValues = *ppsAttributeValues) {
                data_ptr_address = AsnListAppend(attributeValues);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeValue));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attvalues(data_ptr, psAttributeValues, env);

		/* Increment position in array of pointers */
		ppsAttributeValues++;
        }
}



int DBVdbv_to_snacc_exp_attvalues(AttributeValue *data_ptr, DbvAttributeValues *psAttributeValues, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_strornum_opt(&data_ptr->value, &psAttributeValues->sValue, env);
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psAttributeValues->ppsDescription, env);
	
	/* o */
	DBVdbv_to_snacc_strornum_list_opt(&data_ptr->subAttributes, psAttributeValues->ppsSubAttributes, env);

	/* o */
	DBVdbv_to_snacc_strornum_list_opt(&data_ptr->superAttributes, psAttributeValues->ppsSuperAttributes, env);

	/* o */
	DBVdbv_to_snacc_exp_nullpointer_opt(&data_ptr->partialSupport, psAttributeValues->pvPartialSupport, env);
}



int DBVdbv_to_snacc_strornum_list_opt(AttributeValueSeqOf **subAttributes, DbvStringOrNumeric **ppsSubAttributes, jmp_buf env)
{
	if (ppsSubAttributes != NULL) {
		*subAttributes = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*subAttributes, env);
		DBVdbv_to_snacc_exp_strornum_list(*subAttributes, ppsSubAttributes, env);
	}
}



int DBVdbv_to_snacc_exp_strornum_list(AttributeValueSeqOf *subAttributes, DbvStringOrNumeric **ppsSubAttributes, jmp_buf env)
{
        StringOrNumeric *data_ptr;
        StringOrNumeric **data_ptr_address;

	DbvStringOrNumeric *psSubAttributes = NULL;

	/* 0 if NULL assigned */
        while (psSubAttributes = *ppsSubAttributes) {
                data_ptr_address = AsnListAppend(subAttributes);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(StringOrNumeric));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_strornum(data_ptr, psSubAttributes, env);

		/* Increment position in array of pointers */
		ppsSubAttributes++;
        }
}



int DBVdbv_to_snacc_attcombs_opt(struct AttributeCombinations** attributeCombinations, DbvAttributeCombinations *psAttributeCombinations, jmp_buf env)
{

	if (psAttributeCombinations != NULL) {
		*attributeCombinations = (struct AttributeCombinations *) Asn1Alloc (sizeof (struct AttributeCombinations));
		CheckAsn1Alloc(*attributeCombinations, env);
		DBVdbv_to_snacc_exp_attcombs(*attributeCombinations, psAttributeCombinations, env);
	}
}



int DBVdbv_to_snacc_exp_attcombs(struct AttributeCombinations* attributeCombinations, DbvAttributeCombinations *psAttributeCombinations, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_objectident(&attributeCombinations->defaultAttributeSet, psAttributeCombinations->pcDefaultAttributeSet, env);

	/* c */
	attributeCombinations->legalCombinations = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(attributeCombinations->legalCombinations, env);
	DBVdbv_to_snacc_exp_attcomb_nos_list(attributeCombinations->legalCombinations, psAttributeCombinations->ppsLegalCombinations, env);

}



int DBVdbv_to_snacc_exp_attcomb_nos_list(AttributeCombinationsSeqOf *legalCombinations, DbvAttributeCombination **ppsLegalCombinations, jmp_buf env)
{
        AttributeCombination *data_ptr;
        AttributeCombination **data_ptr_address;

	struct DbvAttributeCombination *psLegalCombinations = NULL;

	/* 0 if NULL assigned */
        while (psLegalCombinations = *ppsLegalCombinations) {
                data_ptr_address = AsnListAppend(legalCombinations);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeCombination));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_attcomb_nos(data_ptr, psLegalCombinations, env);

		/* Increment position in array of pointers */
		ppsLegalCombinations++;
        }
}



int DBVdbv_to_snacc_exp_attcomb_nos(AttributeCombination *attcomblist, DbvAttributeCombination *psLegalCombinations, jmp_buf env)
{
	/* c */
        AttributeOccurrence *data_ptr;
        AttributeOccurrence **data_ptr_address;

        int current_listnode = 0;

        while (psLegalCombinations->ppsAttributeOccurrence[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(attcomblist);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(AttributeOccurrence));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_attoccur(data_ptr, psLegalCombinations->ppsAttributeOccurrence[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_attoccur(AttributeOccurrence *data_ptr, DbvAttributeOccurrence *psAttributeOccurrence, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_objectident_opt(&data_ptr->attributeSet, psAttributeOccurrence->pcAttributeSet, env);

	/* c */
	data_ptr->attributeType = psAttributeOccurrence->ulAttributeType;

	/* o */
	DBVdbv_to_snacc_exp_nullpointer_opt(&data_ptr->mustBeSupplied, psAttributeOccurrence->pvMustBeSupplied, env);


	/* c */
	data_ptr->attributeValues = (struct AttributeOccurrenceChoice *) Asn1Alloc (sizeof (struct AttributeOccurrenceChoice));
	CheckAsn1Alloc(data_ptr->attributeValues, env);
	DBVdbv_to_snacc_exp_attoccur(data_ptr->attributeValues, &psAttributeOccurrence->sAttributeValues, env);

}



int DBVdbv_to_snacc_exp_attoccur(struct AttributeOccurrenceChoice* attributeValues, DbvAttributeOccurrenceChoice *psAttributeValues, jmp_buf env)
{
	switch (psAttributeValues->eAttributeOccurrenceChoiceType) {
	case AOT_AnyOrNone:
		attributeValues->choiceId = ATTRIBUTEOCCURRENCECHOICE_ANY_OR_NONE;
		
		/* Dont have to set snacc AsnNull 
		(AsnNull is a typedef char * in asn_null.h)
		*/

		break;
	case AOT_Specific:
		attributeValues->choiceId = ATTRIBUTEOCCURRENCECHOICE_SPECIFIC;

	
		/* Use the optional set up function */
		DBVdbv_to_snacc_strornum_list_opt(&attributeValues->a.specific, psAttributeValues->uAttributeOccurrenceChoiceType.ppsSpecific, env);
		break;
	}
}



/* -------------------- DbvTermListDetails -------------------- */

int DBVdbv_to_snacc_exp_termlistdetails(struct TermListDetails* termListDetails, DbvTermListDetails *psTermListDetails, jmp_buf env)
{
	
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&termListDetails->commonInfo, psTermListDetails->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&termListDetails->termListName, &psTermListDetails->sTermListName, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&termListDetails->description, psTermListDetails->ppsDescription, env);

	/* o */
	DBVdbv_to_snacc_attcombs_opt(&termListDetails->attributes, psTermListDetails->psAttributes, env);

	/* o */
	if (psTermListDetails->psScanInfo != NULL) {
		termListDetails->scanInfo = (struct TermListDetailsSeq*) Asn1Alloc (sizeof (struct TermListDetailsSeq));
		CheckAsn1Alloc(termListDetails->scanInfo, env);
		DBVdbv_to_snacc_exp_scaninfo(termListDetails->scanInfo, psTermListDetails->psScanInfo, env);
	}

	/* o */
	DBVdbv_to_snacc_int_opt(&termListDetails->estNumberTerms, psTermListDetails->pulEstNumberTerms, env);


	/* o */
	if (psTermListDetails->ppsSampleTerms != NULL) {
		termListDetails->sampleTerms = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(termListDetails->sampleTerms, env);
		DBVdbv_to_snacc_exp_term_list(termListDetails->sampleTerms, psTermListDetails->ppsSampleTerms, env);
	}
}



int DBVdbv_to_snacc_exp_scaninfo(struct TermListDetailsSeq* scanInfo, DbvScanInfo *psScanInfo, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_int_opt(&scanInfo->maxStepSize, psScanInfo->pulMaxStepSize, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&scanInfo->collatingSequence, psScanInfo->ppsCollatingSequence, env);

	/* o */
	DBVdbv_to_snacc_bool_opt(&scanInfo->increasing, psScanInfo->pfIncreasing, env);

}


int DBVdbv_to_snacc_exp_term_list(TermListDetailsSeqOf *sampleTerms, DbvTerm **ppsSampleTerms, jmp_buf env)
{
        Term *data_ptr;
        Term **data_ptr_address;

        int current_listnode = 0;

        while (ppsSampleTerms[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(sampleTerms);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Term));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_term(data_ptr, ppsSampleTerms[current_listnode], env);
                current_listnode++;
        }
}



/* -------------------- DbvElementSetDetails -------------------- */

int DBVdbv_to_snacc_exp_elsetdetails(struct ElementSetDetails* elementSetDetails, DbvElementSetDetails *psElementSetDetails, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&elementSetDetails->commonInfo, psElementSetDetails->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&elementSetDetails->databaseName, &psElementSetDetails->sDatabaseName, env);


	/* c */
	DBVdbv_to_snacc_inter_string(&elementSetDetails->elementSetName, &psElementSetDetails->sElementSetName, env);

	/* c */
	DBVdbv_to_snacc_objectident(&elementSetDetails->recordSyntax, psElementSetDetails->pcRecordSyntax, env);

	/* c */
	DBVdbv_to_snacc_objectident(&elementSetDetails->schema, psElementSetDetails->pcSchema, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&elementSetDetails->description, psElementSetDetails->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_elsetdet_list_opt(&elementSetDetails->detailsPerElement, psElementSetDetails->ppsDetailsPerElement, env);

}



int DBVdbv_to_snacc_elsetdet_list_opt(RetrievalRecordDetailsSeqOf **detailsPerElement, DbvDetailsPerElement **ppsDetailsPerElement, jmp_buf env)
{

	if (ppsDetailsPerElement != NULL) {
		*detailsPerElement = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*detailsPerElement, env);
		DBVdbv_to_snacc_exp_elsetdet_list(*detailsPerElement, ppsDetailsPerElement, env);
	}
}



int DBVdbv_to_snacc_exp_elsetdet_list(ElementSetDetailsSeqOf *detailsPerElement, DbvDetailsPerElement **ppsDetailsPerElement, jmp_buf env)
{
        PerElementDetails *data_ptr;
        PerElementDetails **data_ptr_address;

        int current_listnode = 0;

        while (ppsDetailsPerElement[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(detailsPerElement);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(PerElementDetails));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_elsetdat(data_ptr, ppsDetailsPerElement[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_elsetdat(PerElementDetails *data_ptr, DbvDetailsPerElement *psDetailsPerElement, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psDetailsPerElement->psName, env);


	/* o */
	if (psDetailsPerElement->psRecordTag != NULL) {
		data_ptr->recordTag = (struct RecordTag*) Asn1Alloc (sizeof (struct RecordTag));
		CheckAsn1Alloc(data_ptr->recordTag, env);
		DBVdbv_to_snacc_exp_rectag(data_ptr->recordTag, psDetailsPerElement->psRecordTag, env);
	}

	/* o */
	/* Path is sequnce of PathSeq ! */
	if (psDetailsPerElement->ppsSchemaTags != NULL) {
		data_ptr->schemaTags = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->schemaTags, env);
		DBVdbv_to_snacc_exp_patha_list(data_ptr->schemaTags, psDetailsPerElement->ppsSchemaTags, env);
	}

	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->maxSize, psDetailsPerElement->pulMaxSize, env);


	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->minSize, psDetailsPerElement->pulMinSize, env);


	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->avgSize, psDetailsPerElement->pulAvgSize, env);


	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->fixedSize, psDetailsPerElement->pulFixedSize, env);


	/* c */
	DBVdbv_to_snacc_bool(&data_ptr->repeatable, &psDetailsPerElement->fRepeatable, env);


	/* c */
	DBVdbv_to_snacc_bool(&data_ptr->required, &psDetailsPerElement->fRequired, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psDetailsPerElement->ppsDescription, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->contents, psDetailsPerElement->ppsContents, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->billingInfo, psDetailsPerElement->ppsBillingInfo, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->restrictions, psDetailsPerElement->ppsRestrictions, env);

	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&data_ptr->alternateNames, psDetailsPerElement->ppsAlternateNames, env);

	/* o */
	DBVdbv_to_snacc_interstring_list_opt(&data_ptr->genericNames, psDetailsPerElement->ppsGenericNames, env);

	/* o */
	DBVdbv_to_snacc_attcombs_opt(&data_ptr->searchAccess, psDetailsPerElement->psSearchAccess, env);


	
}


int DBVdbv_to_snacc_exp_rectag(struct RecordTag* recordTag, DbvRecordTag *psRecordTag, jmp_buf env)
{
	/* o */	
	DBVdbv_to_snacc_strornum_opt(&recordTag->qualifier, psRecordTag->psQualifier, env);
	
	/* c */
	recordTag->tagValue = (struct StringOrNumeric*) Asn1Alloc(sizeof (struct StringOrNumeric));
	CheckAsn1Alloc(recordTag->tagValue, env);
	DBVdbv_to_snacc_strornum(recordTag->tagValue, &psRecordTag->sTagValue, env);
}



int DBVdbv_to_snacc_exp_patha_list(PerElementDetailsSeqOf *schemaTags, DbvPathA **ppsSchemaTags, jmp_buf env)
{
        Path *data_ptr;
        Path **data_ptr_address;

        int current_listnode = 0;

        while (ppsSchemaTags[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(schemaTags);
                CheckAsn1Alloc(data_ptr_address, env);

		*data_ptr_address = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(*data_ptr_address, env);
                data_ptr = *data_ptr_address;

		DBVdbv_to_snacc_exp_path_list(data_ptr, ppsSchemaTags[current_listnode]->ppsSchemaTags, env);
                current_listnode++;
        }
}



/* -------------------- DbvRetrievalRecordDetails -------------------- */

int DBVdbv_to_snacc_exp_retrecdetails(struct RetrievalRecordDetails* retrievalRecordDetails, DbvRetrievalRecordDetails *psRetrievalRecordDetails, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&retrievalRecordDetails->commonInfo, psRetrievalRecordDetails->psCommonInfo, env);


	/* c */
	/* DatabaseName is a typedefed InternationalString in z39_50_apdu_1994.h */
	DBVdbv_to_snacc_inter_string(&retrievalRecordDetails->databaseName, &psRetrievalRecordDetails->sDatabaseName, env);

	/* c */
	DBVdbv_to_snacc_objectident(&retrievalRecordDetails->schema, psRetrievalRecordDetails->pcSchema, env);

	/* c */
	DBVdbv_to_snacc_objectident(&retrievalRecordDetails->recordSyntax, psRetrievalRecordDetails->pcRecordSyntax, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&retrievalRecordDetails->description, psRetrievalRecordDetails->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_elsetdet_list_opt(&retrievalRecordDetails->detailsPerElement, psRetrievalRecordDetails->ppsDetailsPerElement, env);

}



/* -------------------- DbvSortDetails -------------------- */

int DBVdbv_to_snacc_exp_sortdetails(struct SortDetails* sortDetails, DbvSortDetails *psSortDetails, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&sortDetails->commonInfo, psSortDetails->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&sortDetails->databaseName, &psSortDetails->sDatabaseName, env);


	/* o */
	if (psSortDetails->ppsSortKeys != NULL) {
		sortDetails->sortKeys = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(sortDetails->sortKeys, env);
		DBVdbv_to_snacc_exp_sortkeys_list(sortDetails->sortKeys, psSortDetails->ppsSortKeys, env);
	}

}



int DBVdbv_to_snacc_exp_sortkeys_list(SortDetailsSeqOf *sortKeys, DbvSortKeys **ppsSortKeys, jmp_buf env)
{
        SortKeyDetails *data_ptr;
        SortKeyDetails **data_ptr_address;

        int current_listnode = 0;

        while (ppsSortKeys[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(sortKeys);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(SortKeyDetails));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_sortkeys(data_ptr, ppsSortKeys[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_sortkeys(SortKeyDetails *data_ptr, DbvSortKeys *psSortKeys, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psSortKeys->ppsDescription, env);

	/* o */
	if (psSortKeys->ppsElementSpecifications != NULL) {
		data_ptr->elementSpecifications = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(data_ptr->elementSpecifications, env);
		DBVdbv_to_snacc_exp_specification_list(data_ptr->elementSpecifications, psSortKeys->ppsElementSpecifications, env);
	}


	/* o */
	DBVdbv_to_snacc_attcombs_opt(&data_ptr->attributeSpecifications, psSortKeys->psAttributeSpecifications, env);


	/* o */
	if (psSortKeys->psSortType != NULL) {
		data_ptr->sortType = (struct SortKeyDetailsChoice*) Asn1Alloc (sizeof (DbvSortType));
		CheckAsn1Alloc(data_ptr->sortType, env);
		DBVdbv_to_snacc_sorttype(data_ptr->sortType, psSortKeys->psSortType, env);
	}

	/* o */
	if (psSortKeys->peCaseSensitivity != NULL) {
		data_ptr->caseSensitivity = (SortKeyDetailsInt*) Asn1Alloc (sizeof (SortKeyDetailsInt));
		CheckAsn1Alloc(data_ptr->caseSensitivity, env);
		DBVdbv_to_snacc_sortdetailsint(data_ptr->caseSensitivity, psSortKeys->peCaseSensitivity, env);
	}
}



int DBVdbv_to_snacc_exp_specification_list(SortKeyDetailsSeqOf *elementSpecifications, DbvSpecification **ppsElementSpecifications, jmp_buf env)
{
        Specification *data_ptr;
        Specification **data_ptr_address;

        int current_listnode = 0;

        while (ppsElementSpecifications[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(elementSpecifications);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Specification));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_specification(data_ptr, ppsElementSpecifications[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_sorttype(struct SortKeyDetailsChoice* sortType, DbvSortType *psSortType, jmp_buf env)
{
	/* In snacc, both character and numeric are AsnNull */

	switch (psSortType->eSortTypeType) {
	case STT_Character:
		sortType->choiceId = SORTKEYDETAILSCHOICE_CHARACTER;
		break;
		
	case STT_Numeric:
		sortType->choiceId = SORTKEYDETAILSCHOICE_NUMERIC;
		break;

	case STT_Structured:
		sortType->choiceId = SORTKEYDETAILSCHOICE_STRUCTURED;

		/* Use optional set up */
		DBVdbv_to_snacc_exp_humanstring_list_opt(&sortType->a.structured, psSortType->uSortType.ppsStructured, env);
		break;
	}
}



int DBVdbv_to_snacc_sortdetailsint(SortKeyDetailsInt *caseSensitivity, DbvCaseSensitivity *peCaseSensitivity, jmp_buf env)
{
	switch (*peCaseSensitivity) {
	case CSE_Always:
		*caseSensitivity = ALWAYS;
		break;

	case CSE_Never:
		*caseSensitivity = NEVER;
		break;

	case CSE_DefaultYes:
		*caseSensitivity = DEFAULT_YES;
		break;

	case CSE_DefaultNo:
		*caseSensitivity = DEFAULT_NO;
		break;

	}
}



/* -------------------- DbvProcessingInformation -------------------- */

int DBVdbv_to_snacc_exp_processinginfo(struct ProcessingInformation* processing, DbvProcessingInformation *psProcessing, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&processing->commonInfo, psProcessing->psCommonInfo, env);


	/* c */
	/* DatabaseName is a typedefed InternationalString in z39_50_apdu_1994.h */
	DBVdbv_to_snacc_inter_string(&processing->databaseName, &psProcessing->sDatabaseName, env);


	/* c */
	DBVdbv_to_snacc_exp_processcontext(&processing->processingContext, &psProcessing->eProcessingContext, env);


	/* c */
	DBVdbv_to_snacc_inter_string(&processing->name, &psProcessing->sName, env);

	/* c */
	DBVdbv_to_snacc_objectident(&processing->oid, psProcessing->pcOid, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&processing->description, psProcessing->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_external_opt(&processing->instructions, psProcessing->psInstructions, env);


}



int DBVdbv_to_snacc_exp_processcontext(ProcessingInformationInt *processingContext, DbvProcessingContext *peProcessingContext, jmp_buf env)
{
	switch (*peProcessingContext) {
	case PC_Access:
		*processingContext = ACCESS;
		break;

	case PC_Search:
		*processingContext = SEARCH2;
		break;

	case PC_Retrieval:
		*processingContext = RETRIEVAL;
		break;

	case PC_RecordPresentation:
		*processingContext = RECORD_PRESENTATION;
		break;

	case PC_RecordHandling:
		*processingContext = RECORD_HANDLING;
		break;
	}
}



/* -------------------- DbvVariantSetInfo -------------------- */

int DBVdbv_to_snacc_exp_variantsetinfo(struct VariantSetInfo* variants, DbvVariantSetInfo *psVariants, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&variants->commonInfo, psVariants->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_objectident(&variants->variantSet, psVariants->pcVariantSet, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&variants->name, &psVariants->sName, env);

	/* o */
	if (psVariants->ppsVariants != NULL) {
		variants->variants = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(variants->variants, env);
		DBVdbv_to_snacc_exp_variantclass_list(variants->variants, psVariants->ppsVariants, env);
	}
}


int DBVdbv_to_snacc_exp_variantclass_list(VariantSetInfoSeqOf *variants, DbvVariantClass **ppsVariants, jmp_buf env)
{
        VariantClass *data_ptr;
        VariantClass **data_ptr_address;

        int current_listnode = 0;

        while (ppsVariants[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(variants);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(VariantClass));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_variantclass(data_ptr, ppsVariants[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_variantclass(VariantClass *data_ptr, DbvVariantClass *psVariants, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psVariants->psName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psVariants->ppsDescription, env);

	/* c */
	data_ptr->variantClass = psVariants->ulVariantClass;
	
	/* c */
	data_ptr->variantTypes = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->variantTypes, env);
	DBVdbv_to_snacc_exp_varianttypes_list(data_ptr->variantTypes, psVariants->ppsVariantTypes, env);
	

}



int DBVdbv_to_snacc_exp_varianttypes_list(VariantClassSeqOf *variantTypes, DbvVariantTypes **ppsVariantTypes, jmp_buf env)
{
        VariantType *data_ptr;
        VariantType **data_ptr_address;

        int current_listnode = 0;

        while (ppsVariantTypes[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(variantTypes);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(VariantType));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_varianttype(data_ptr, ppsVariantTypes[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_varianttype(VariantType *data_ptr, DbvVariantTypes *psVariantTypes, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psVariantTypes->psName, env);
	
	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psVariantTypes->ppsDescription, env);

	/* c */
	data_ptr->variantType = psVariantTypes->ulVariantType;
	

	/* o */
	if (psVariantTypes->psVariantValue != NULL) {
		data_ptr->variantValue = (struct VariantValue*) Asn1Alloc (sizeof (struct VariantValue));
		CheckAsn1Alloc(data_ptr->variantValue, env);
		DBVdbv_to_snacc_exp_variantvalue(data_ptr->variantValue, psVariantTypes->psVariantValue, env);
	}


}



int DBVdbv_to_snacc_exp_variantvalue(struct VariantValue* variantValue, DbvVariantValue *psVariantValue, jmp_buf env)
{
	/* c */
	DBVdbv_to_snacc_primitive(&variantValue->dataType, &psVariantValue->eDataType, env);
	
	/* o */
	if (psVariantValue->psValues != NULL) {
		variantValue->values = (struct ValueSet*) Asn1Alloc (sizeof (struct ValueSet));
		CheckAsn1Alloc(variantValue->values, env);
		DBVdbv_to_snacc_exp_valueset(variantValue->values, psVariantValue->psValues, env);
	}

}



int DBVdbv_to_snacc_exp_valueset(struct ValueSet* values, DbvValueSet *psValues, jmp_buf env)
{
	switch (psValues->eValueSetType) {
	case VT_Range:
		values->choiceId = VALUESET_RANGE;
		values->a.range = (struct ValueRange*) Asn1Alloc (sizeof (struct ValueRange));
		CheckAsn1Alloc(values->a.range, env);
		DBVdbv_to_snacc_exp_valuerange(values->a.range, psValues->uValueSet.psRange, env);
                
		break;

	case VT_Enumerated:
		values->choiceId = VALUESET_ENUMERATED;
		values->a.enumerated = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(values->a.enumerated, env);
		DBVdbv_to_snacc_exp_enumerated_list(values->a.enumerated, psValues->uValueSet.ppsEnumerated, env);
		
		break;
	}
	
}


int DBVdbv_to_snacc_exp_valuerange(struct ValueRange* range, DbvValueRange *psValueRange, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_valdesc_opt(&range->lower, psValueRange->psLower, env);

	/* o */
	DBVdbv_to_snacc_exp_valdesc_opt(&range->upper, psValueRange->psUpper, env);
}




int DBVdbv_to_snacc_exp_valdesc_opt(struct ValueDescription** lower, DbvValueDescription *psLower, jmp_buf env)
{

	if (psLower != NULL) {
		*lower = (struct ValueDescription*) Asn1Alloc (sizeof (struct ValueDescription));
		CheckAsn1Alloc(*lower, env);
		DBVdbv_to_snacc_exp_valdesc(*lower, psLower, env);
	}
}



int DBVdbv_to_snacc_exp_valdesc(struct ValueDescription* lower, DbvValueDescription *psLower, jmp_buf env)
{
	switch (psLower->eValueDescriptionType) {
	case VDT_Integer:
		lower->choiceId = VALUEDESCRIPTION_INTEGER;
		lower->a.integer = psLower->uValueDescription.ulInteger;
		break;

	case VDT_String:
		lower->choiceId = VALUEDESCRIPTION_STRING;
		DBVdbv_to_snacc_interstring_comp(&lower->a.string, psLower->uValueDescription.psString, env);
		break;

	case VDT_Octets:
		lower->choiceId = VALUEDESCRIPTION_OCTETS;
		DBVdbv_to_snacc_octetstring_comp(&lower->a.octets, psLower->uValueDescription.psOctets, env);
		break;

	case VDT_Oid:
		lower->choiceId = VALUEDESCRIPTION_OID_SNACC;
		DBVdbv_to_snacc_objectident_comp(&lower->a.octets, psLower->uValueDescription.pcOid, env);
		
		break;

	case VDT_Unit:
		lower->choiceId = VALUEDESCRIPTION_UNIT;
		DBVdbv_to_snacc_unit_opt(&lower->a.unit, psLower->uValueDescription.psUnit, env);
		break;

	case VDT_ValueAndUnit:
		lower->choiceId = VALUEDESCRIPTION_VALUEANDUNIT;
		DBVdbv_to_snacc_intunit_opt(&lower->a.valueAndUnit, psLower->uValueDescription.psValueAndUnit, env);
		break;
	}
	
}



int DBVdbv_to_snacc_exp_enumerated_list(ValueSetSeqOf *enumerated, DbvValueDescription **ppsEnumerated, jmp_buf env)
{
        ValueDescription *data_ptr;
        ValueDescription **data_ptr_address;

        int current_listnode = 0;

        while (ppsEnumerated[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(enumerated);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(ValueDescription));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_valdesc(data_ptr, ppsEnumerated[current_listnode], env);
                current_listnode++;
        }
	
}


/* -------------------- DbvUnitInfo -------------------- */

int DBVdbv_to_snacc_exp_unitinfo(struct UnitInfo* units, DbvUnitInfo *psUnits, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&units->commonInfo, psUnits->psCommonInfo, env);

	/* c */
	DBVdbv_to_snacc_inter_string(&units->unitSystem, &psUnits->sUnitSystem, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&units->description, psUnits->ppsDescription, env);

	/* o */
	if (psUnits->ppsUnitTypes != NULL)  {
		units->units = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(units->units, env);
		DBVdbv_to_snacc_exp_unittype_list(units->units, psUnits->ppsUnitTypes, env);
	}
		
}



int DBVdbv_to_snacc_exp_unittype_list(UnitInfoSeqOf *units, DbvUnitType **ppsUnitTypes, jmp_buf env)
{
        UnitType *data_ptr;
        UnitType **data_ptr_address;

        int current_listnode = 0;

        while (ppsUnitTypes[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(units);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(UnitType));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_unittype(data_ptr, ppsUnitTypes[current_listnode], env);
                current_listnode++;
        }
	
}



int DBVdbv_to_snacc_exp_unittype(UnitType *data_ptr, DbvUnitType *psUnitTypes, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psUnitTypes->psName, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psUnitTypes->ppsDescription, env);

	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->unitType, &psUnitTypes->sUnitType, env);

	/* c */
	data_ptr->units = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(data_ptr->units, env);
	DBVdbv_to_snacc_exp_units_list(data_ptr->units, psUnitTypes->ppsUnits, env);
	
}



int DBVdbv_to_snacc_exp_units_list(UnitTypeSeqOf *units, DbvUnits **ppsUnits, jmp_buf env)
{
        Units *data_ptr;
        Units **data_ptr_address;

        int current_listnode = 0;

        while (ppsUnits[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(units);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(Units));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_units(data_ptr, ppsUnits[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_units(Units *data_ptr, DbvUnits *psUnits, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->name, psUnits->psName, env);

	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psUnits->ppsDescription, env);

	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->unit, &psUnits->sUnit, env);

}


/* -------------------- DbvCategoryList -------------------- */

int DBVdbv_to_snacc_exp_categorylist(struct CategoryList* categoryList, DbvCategoryList *psCategoryList, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_exp_commoninfo_opt(&categoryList->commonInfo, psCategoryList->psCommonInfo, env);

	/* c */
	categoryList->categories = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(categoryList->categories, env);
	DBVdbv_to_snacc_exp_categoryinfo_list(categoryList->categories, psCategoryList->ppsCategories, env);
}


int DBVdbv_to_snacc_exp_categoryinfo_list(CategoryListSeqOf *categories, DbvCategoryInfo **ppsCategories, jmp_buf env)
{
        CategoryInfo *data_ptr;
        CategoryInfo **data_ptr_address;

        int current_listnode = 0;

        while (ppsCategories[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(categories);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(CategoryInfo));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_exp_categoryinfo(data_ptr, ppsCategories[current_listnode], env);
                current_listnode++;
        }
}



int DBVdbv_to_snacc_exp_categoryinfo(CategoryInfo *data_ptr, DbvCategoryInfo *psCategories, jmp_buf env)
{

	/* c */
	DBVdbv_to_snacc_inter_string(&data_ptr->category, &psCategories->sCategory, env);

	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->originalCategory, psCategories->psOriginalCategory, env);


	/* o */
	DBVdbv_to_snacc_exp_humanstring_list_opt(&data_ptr->description, psCategories->ppsDescription, env);


	/* o */
	DBVdbv_to_snacc_inter_string(&data_ptr->asn1Module, psCategories->psAsn1Module, env);
}


/***********************************************************************/
/***********************************************************************/

/* snacc -> dbv ansi Explain external */
int DBVsnacc_to_dbv_explain_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	Explain_Record *explain_ptr;
 
	explain_ptr = (Explain_Record *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ExplainRecord;
        psSingleASN1Type->uASN1.psExplainRecord = (struct DbvExplainRecord *) DbvAlloc (sizeof(struct DbvExplainRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psExplainRecord, env);

	DBVsnacc_to_dbv_explain_main(psSingleASN1Type->uASN1.psExplainRecord, explain_ptr, env);
}



int DBVsnacc_to_dbv_isoexplain_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoExplain_Record *isoexplain_ptr;
 
	isoexplain_ptr = (Explain_Record *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_ExplainRecord;
        psSingleASN1Type->uASN1.psExplainRecord = (struct DbvExplainRecord *) DbvAlloc (sizeof(struct DbvExplainRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psExplainRecord, env);

	DBVsnacc_to_dbv_explain_main(psSingleASN1Type->uASN1.psExplainRecord, isoexplain_ptr, env);
}





int DBVsnacc_to_dbv_explain_main(struct DbvExplainRecord *psExplainRecord, Explain_Record *explain_ptr, jmp_buf env)
{
	switch (explain_ptr->choiceId) {
	case EXPLAIN_RECORD_TARGETINFO:
		psExplainRecord->eExplainRecordType = ERT_TargetInfo;
		psExplainRecord->uExplainRecord.psTargetInfo = (DbvTargetInfo *) DbvAlloc (sizeof (DbvTargetInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psTargetInfo, env);
		DBVsnacc_to_dbv_exp_targetinfo(psExplainRecord->uExplainRecord.psTargetInfo, explain_ptr->a.targetInfo, env);
		break;

	case EXPLAIN_RECORD_DATABASEINFO:
		psExplainRecord->eExplainRecordType = ERT_DatabaseInfo;
		psExplainRecord->uExplainRecord.psDatabaseInfo = (DbvDatabaseInfo *) DbvAlloc (sizeof (DbvDatabaseInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psDatabaseInfo, env);
		DBVsnacc_to_dbv_exp_dbinfo(psExplainRecord->uExplainRecord.psDatabaseInfo, explain_ptr->a.databaseInfo, env);
		break;

	case EXPLAIN_RECORD_SCHEMAINFO:
		psExplainRecord->eExplainRecordType = ERT_SchemaInfo;
		psExplainRecord->uExplainRecord.psSchemaInfo = (DbvSchemaInfo *) DbvAlloc (sizeof (DbvSchemaInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psSchemaInfo, env);
		DBVsnacc_to_dbv_exp_schemainfo(psExplainRecord->uExplainRecord.psSchemaInfo, explain_ptr->a.schemaInfo, env);
		break;

	case EXPLAIN_RECORD_TAGSETINFO:
		psExplainRecord->eExplainRecordType = ERT_TagSetInfo;
		psExplainRecord->uExplainRecord.psTagSetInfo = (DbvTagSetInfo *) DbvAlloc (sizeof (DbvTagSetInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psTagSetInfo, env);
		DBVsnacc_to_dbv_exp_tagsetinfo(psExplainRecord->uExplainRecord.psTagSetInfo, explain_ptr->a.tagSetInfo, env);
		break;

	case EXPLAIN_RECORD_RECORDSYNTAXINFO:
		psExplainRecord->eExplainRecordType = ERT_RecordSyntaxInfo;
		psExplainRecord->uExplainRecord.psRecordSyntaxInfo = (DbvRecordSyntaxInfo *) DbvAlloc (sizeof (DbvRecordSyntaxInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psRecordSyntaxInfo, env);
		DBVsnacc_to_dbv_exp_recsyninfo(psExplainRecord->uExplainRecord.psRecordSyntaxInfo, explain_ptr->a.recordSyntaxInfo, env);
		break;

	case EXPLAIN_RECORD_ATTRIBUTESETINFO:
		psExplainRecord->eExplainRecordType = ERT_AttributeSetInfo;
		psExplainRecord->uExplainRecord.psAttributeSetInfo = (DbvAttributeSetInfo *) DbvAlloc (sizeof (DbvAttributeSetInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psAttributeSetInfo, env);
		DBVsnacc_to_dbv_exp_attsetinfo(psExplainRecord->uExplainRecord.psAttributeSetInfo, explain_ptr->a.attributeSetInfo, env);
		break;

	case EXPLAIN_RECORD_TERMLISTINFO:
		psExplainRecord->eExplainRecordType = ERT_TermListInfo;
		psExplainRecord->uExplainRecord.psTermListInfo = (DbvTermListInfo *) DbvAlloc (sizeof (DbvTermListInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psTermListInfo, env);
		DBVsnacc_to_dbv_exp_termlistinfo(psExplainRecord->uExplainRecord.psTermListInfo, explain_ptr->a.termListInfo, env);
		break;

	case EXPLAIN_RECORD_EXTENDEDSERVICESINFO:
		psExplainRecord->eExplainRecordType = ERT_ExtendedServicesInfo;
		psExplainRecord->uExplainRecord.psExtendedServicesInfo = (DbvExtendedServicesInfo *) DbvAlloc (sizeof (DbvExtendedServicesInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psExtendedServicesInfo, env);
		DBVsnacc_to_dbv_exp_extservinfo(psExplainRecord->uExplainRecord.psExtendedServicesInfo, explain_ptr->a.extendedServicesInfo, env);
		break;

	case EXPLAIN_RECORD_ATTRIBUTEDETAILS:
		psExplainRecord->eExplainRecordType = ERT_AttributeDetails;
		psExplainRecord->uExplainRecord.psAttributeDetails = (DbvAttributeDetails *) DbvAlloc (sizeof (DbvAttributeDetails));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psAttributeDetails, env);
		DBVsnacc_to_dbv_exp_attdetails(psExplainRecord->uExplainRecord.psAttributeDetails, explain_ptr->a.attributeDetails, env);
		break;

	case EXPLAIN_RECORD_TERMLISTDETAILS:
		psExplainRecord->eExplainRecordType = ERT_TermListDetails;
		psExplainRecord->uExplainRecord.psTermListDetails = (DbvTermListDetails *) DbvAlloc (sizeof (DbvTermListDetails));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psTermListDetails, env);
		DBVsnacc_to_dbv_exp_termlistdetails(psExplainRecord->uExplainRecord.psTermListDetails, explain_ptr->a.termListDetails, env);
		break;

	case EXPLAIN_RECORD_ELEMENTSETDETAILS:
		psExplainRecord->eExplainRecordType = ERT_ElementSetDetails;
		psExplainRecord->uExplainRecord.psElementSetDetails = (DbvElementSetDetails *) DbvAlloc (sizeof (DbvElementSetDetails));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psElementSetDetails, env);
		DBVsnacc_to_dbv_exp_elsetdetails(psExplainRecord->uExplainRecord.psElementSetDetails, explain_ptr->a.elementSetDetails, env);
		break;

	case EXPLAIN_RECORD_RETRIEVALRECORDDETAILS:
		psExplainRecord->eExplainRecordType = ERT_RetrievalRecordDetails;
		psExplainRecord->uExplainRecord.psRetrievalRecordDetails = (DbvRetrievalRecordDetails *) DbvAlloc (sizeof (DbvRetrievalRecordDetails));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psRetrievalRecordDetails, env);
		DBVsnacc_to_dbv_exp_retrecdetails(psExplainRecord->uExplainRecord.psRetrievalRecordDetails, explain_ptr->a.retrievalRecordDetails, env);
		break;

	case EXPLAIN_RECORD_SORTDETAILS:
		psExplainRecord->eExplainRecordType = ERT_SortDetails;
		psExplainRecord->uExplainRecord.psSortDetails = (DbvSortDetails *) DbvAlloc (sizeof (DbvSortDetails));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psSortDetails, env);
		DBVsnacc_to_dbv_exp_sortdetails(psExplainRecord->uExplainRecord.psSortDetails, explain_ptr->a.sortDetails, env);
		break;

	case EXPLAIN_RECORD_PROCESSING:
		psExplainRecord->eExplainRecordType = ERT_Processing;
		psExplainRecord->uExplainRecord.psProcessing = (DbvProcessingInformation *) DbvAlloc (sizeof (DbvProcessingInformation));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psProcessing, env);
		DBVsnacc_to_dbv_exp_processinginfo(psExplainRecord->uExplainRecord.psProcessing, explain_ptr->a.processing, env);
		break;

	case EXPLAIN_RECORD_VARIANTS:
		psExplainRecord->eExplainRecordType = ERT_Variants;
		psExplainRecord->uExplainRecord.psVariants = (DbvVariantSetInfo *) DbvAlloc (sizeof (DbvVariantSetInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psVariants, env);
		DBVsnacc_to_dbv_exp_variantsetinfo(psExplainRecord->uExplainRecord.psVariants, explain_ptr->a.variants, env);
		break;

	case EXPLAIN_RECORD_UNITS:
		psExplainRecord->eExplainRecordType = ERT_Units;
		psExplainRecord->uExplainRecord.psUnits = (DbvUnitInfo *) DbvAlloc (sizeof (DbvUnitInfo));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psUnits, env);
		DBVsnacc_to_dbv_exp_unitinfo(psExplainRecord->uExplainRecord.psUnits, explain_ptr->a.units, env);
		break;

	case EXPLAIN_RECORD_CATEGORYLIST:
		psExplainRecord->eExplainRecordType = ERT_CategoryList;
		psExplainRecord->uExplainRecord.psCategoryList = (DbvCategoryList *) DbvAlloc (sizeof (DbvCategoryList));
		CheckDbvAlloc(psExplainRecord->uExplainRecord.psCategoryList, env);
		DBVsnacc_to_dbv_exp_categorylist(psExplainRecord->uExplainRecord.psCategoryList, explain_ptr->a.categoryList, env);
		break;
	default:
		psExplainRecord->eExplainRecordType = ERT_Invalid;
		break;
	}
}



/* ---------------------------- TargetInfo ------------------------------ */

int DBVsnacc_to_dbv_exp_targetinfo(DbvTargetInfo *psTargetInfo, struct TargetInfo* targetInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psTargetInfo->psCommonInfo, targetInfo->commonInfo, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psTargetInfo->sName, &targetInfo->name, env);
	

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsRecentNews, targetInfo->recent_news, env);


	/* o */
	DBVsnacc_to_dbv_icon_list_opt(&psTargetInfo->ppsIcon, targetInfo->icon, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psTargetInfo->fNamedResultSets, &targetInfo->namedResultSets, env);

	
	/* c */
	DBVsnacc_to_dbv_bool(&psTargetInfo->fMultipleDBSearch, &targetInfo->multipleDBsearch, env);


	/* o */
	DBVsnacc_to_dbv_int_opt(&psTargetInfo->pulMaxResultSets, targetInfo->maxResultSets, env);


	/* o */
	DBVsnacc_to_dbv_int_opt(&psTargetInfo->pulMaxResultSize, targetInfo->maxResultSize, env);


	/* o */
	DBVsnacc_to_dbv_int_opt(&psTargetInfo->pulMaxTerms, targetInfo->maxTerms, env);


	/* o */
	DBVsnacc_to_dbv_intunit_opt(&psTargetInfo->psTimeoutInterval, targetInfo->timeoutInterval, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsWelcomeMessage, targetInfo->welcomeMessage, env);


	/* o */
	DBVsnacc_to_dbv_exp_contact_info_opt(&psTargetInfo->psContactInfo, targetInfo->contactInfo, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsDescription, targetInfo->description, env);


	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psTargetInfo->ppsNicknames, targetInfo->nicknames, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsUsageRest, targetInfo->usage_restrictions, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsPaymentAddr, targetInfo->paymentAddr, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTargetInfo->ppsHours, targetInfo->hours, env);


	/* o */
	if (targetInfo->dbCombinations != NULL) {
		num_elements = AsnListCount(targetInfo->dbCombinations);
                psTargetInfo->ppsDbCombinations = (DbvDatabaseList **) DbvAlloc (sizeof (DbvDatabaseList *) * (num_elements + 1));
                CheckDbvAlloc(psTargetInfo->ppsDbCombinations, env);
		DBVsnacc_to_dbv_exp_databaselist_list(psTargetInfo->ppsDbCombinations, targetInfo->dbCombinations, env);
	}
		


	/* o */
	if (targetInfo->addresses != NULL) {
		num_elements = AsnListCount(targetInfo->addresses);
                psTargetInfo->ppsAddresses = (DbvNetworkAddress **) DbvAlloc (sizeof (DbvNetworkAddress *) * (num_elements + 1));
                CheckDbvAlloc(psTargetInfo->ppsAddresses, env);
		DBVsnacc_to_dbv_exp_netaddr_list(psTargetInfo->ppsAddresses, targetInfo->addresses, env);
	}


	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psTargetInfo->ppsLanguages, targetInfo->languages, env);


	/* o */
	DBVsnacc_to_dbv_exp_accessinfo_opt(&psTargetInfo->psCommonAccessInfo, targetInfo->commonAccessInfo, env);


}



int DBVsnacc_to_dbv_exp_commoninfo_opt(DbvCommonInfo **ppsCommonInfo, struct CommonInfo *commonInfo, jmp_buf env)
{
	if (commonInfo != NULL) {
		*ppsCommonInfo = (DbvCommonInfo *) DbvAlloc (sizeof (DbvCommonInfo));
		CheckDbvAlloc(*ppsCommonInfo, env);
		DBVsnacc_to_dbv_commoninfo(*ppsCommonInfo, commonInfo, env);
	}
}



int DBVsnacc_to_dbv_commoninfo(DbvCommonInfo *psCommonInfo, struct CommonInfo* commonInfo, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_gentime_opt(&psCommonInfo->pcDateAdded, commonInfo->dateAdded, env);


	/* o */
	DBVsnacc_to_dbv_gentime_opt(&psCommonInfo->pcDateChanged, commonInfo->dateChanged, env);

	/* o */
	DBVsnacc_to_dbv_gentime_opt(&psCommonInfo->pcExpiry, commonInfo->expiry, env);

	/* o */
	DBVsnacc_to_dbv_languagecode_opt(&psCommonInfo->psHumanStringLanguage, commonInfo->humanString_Language, env);


	/* o */
	DBVsnacc_to_dbv_otherinfo_opt(&psCommonInfo->ppsOtherInformation, commonInfo->otherInfo, env);

}







int DBVsnacc_to_dbv_languagecode_opt(DbvLanguageCode **ppsHumanStringLanguage, LanguageCode humanString_Language, jmp_buf env)
{
	if (humanString_Language.octetLen != 0) {
		*ppsHumanStringLanguage = (DbvLanguageCode *)DbvAlloc(sizeof(DbvLanguageCode));
		CheckDbvAlloc(*ppsHumanStringLanguage, env);
		DBVsnacc_to_dbv_inter_string((DbvInternationalString *) *ppsHumanStringLanguage, &humanString_Language, env);
	}
}



/* triple pointer babes */
int DBVsnacc_to_dbv_humanstring_list_opt(DbvHumanString ***ppsRecentNews, HumanString *recent_news, jmp_buf env)
{
	int num_elements = 0;

	if (recent_news != NULL) {
		num_elements = AsnListCount(recent_news);
                *ppsRecentNews = (DbvHumanString **) DbvAlloc (sizeof (DbvHumanString *) * (num_elements + 1));
                CheckDbvAlloc(*ppsRecentNews, env);
		DBVsnacc_to_dbv_humanstring_list(*ppsRecentNews, recent_news, env);
	}
}



int DBVsnacc_to_dbv_humanstring_list(DbvHumanString **ppsRecentNews, HumanString *recent_news, jmp_buf env)
{
	HumanStringSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, recent_news) {
		ppsRecentNews[index] = (DbvHumanString*) DbvAlloc (sizeof (DbvHumanString));
		CheckDbvAlloc(ppsRecentNews[index], env);
		DBVsnacc_to_dbv_humanstring(ppsRecentNews[index], data_ptr, env);
		index++;
	}
	ppsRecentNews[index] = NULL;
}



int DBVsnacc_to_dbv_humanstring(DbvHumanString *psRecentNews, HumanStringSeq *data_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_languagecode_opt(&psRecentNews->psLanguage, data_ptr->language, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psRecentNews->sText, &data_ptr->text, env);
}



int DBVsnacc_to_dbv_icon_list_opt(DbvIconObject ***pppsIcon, IconObject *icon, jmp_buf env)
{
	int num_elements = 0;

	if (icon != NULL) {
		num_elements = AsnListCount(icon);
                *pppsIcon = (DbvIconObject **) DbvAlloc (sizeof (DbvIconObject *) * (num_elements + 1));
                CheckDbvAlloc(*pppsIcon, env);
		DBVsnacc_to_dbv_icon_list(*pppsIcon, icon, env);
	}
}


int DBVsnacc_to_dbv_icon_list(DbvIconObject **ppsIcon, IconObject *icon, jmp_buf env)
{
	IconObjectSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, icon) {
		ppsIcon[index] = (DbvIconObject*) DbvAlloc (sizeof (DbvIconObject));
		CheckDbvAlloc(ppsIcon[index], env);
		DBVsnacc_to_dbv_icon(ppsIcon[index], data_ptr, env);
		index++;
	}
	ppsIcon[index] = NULL;
}



int DBVsnacc_to_dbv_icon(DbvIconObject *psIcon, IconObjectSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_bodytype(&psIcon->sBodyType, data_ptr->bodyType, env);
	
	/* c */
	DBVsnacc_to_dbv_octet_string(&psIcon->sContent, &data_ptr->content, env);
	
}



int DBVsnacc_to_dbv_bodytype(DbvBodyType *psBodyType, struct IconObjectSeqChoice* bodyType, jmp_buf env)
{
	switch (bodyType->choiceId) {
	case ICONOBJECTSEQCHOICE_IANATYPE:
		psBodyType->eBodyTypeType = BTT_IanaType;

		DBVsnacc_to_dbv_interstring_comp(&psBodyType->uBodyType.psIanaType, bodyType->a.ianaType, env);

		break;
	case ICONOBJECTSEQCHOICE_Z3950TYPE:
		psBodyType->eBodyTypeType = BTT_Z3950Type;

		DBVsnacc_to_dbv_interstring_comp(&psBodyType->uBodyType.psZ3950Type, bodyType->a.z3950type, env);
		break;
	case ICONOBJECTSEQCHOICE_OTHERTYPE:
		psBodyType->eBodyTypeType = BTT_OtherType;
		DBVsnacc_to_dbv_interstring_comp(&psBodyType->uBodyType.psOtherType, bodyType->a.otherType, env);

		break;
	default:
		psBodyType->eBodyTypeType = BTT_Invalid;
		break;
	}
}



int DBVsnacc_to_dbv_exp_contact_info_opt(DbvContactInfo **ppsContactInfo, struct ContactInfo *contactInfo, jmp_buf env)
{

	if (contactInfo != NULL) {
		*ppsContactInfo = (DbvContactInfo *) DbvAlloc (sizeof (DbvContactInfo));
		CheckDbvAlloc (*ppsContactInfo, env);
		DBVsnacc_to_dbv_contact_info(*ppsContactInfo, contactInfo, env);
	}
}



/* On the snacc side, struct ContactInfo is typdefed in z39_50_externals_c_1994.h */
int DBVsnacc_to_dbv_contact_info(DbvContactInfo *psContactInfo, struct ContactInfo* contactInfo, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psContactInfo->psName, &contactInfo->name, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psContactInfo->ppsDescription, contactInfo->description, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psContactInfo->ppsAddress, contactInfo->address, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psContactInfo->psEmail, &contactInfo->email, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psContactInfo->psPhone, &contactInfo->phone, env);
}



int DBVsnacc_to_dbv_exp_databaselist_list(DbvDatabaseList **ppsDbCombinations, TargetInfoSeqOf1 *dbCombinations, jmp_buf env)
{
	DatabaseList *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, dbCombinations) {
		ppsDbCombinations[index] = (DbvDatabaseList*) DbvAlloc (sizeof (DbvDatabaseList));
		CheckDbvAlloc(ppsDbCombinations[index], env);
		DBVsnacc_to_dbv_exp_databaselist(ppsDbCombinations[index], data_ptr, env);
		index++;
	}
	ppsDbCombinations[index] = NULL;
}



 
int DBVsnacc_to_dbv_exp_databaselist(DbvDatabaseList *psDbCombinations, DatabaseList *data_ptr, jmp_buf env)
{
	/* use optional setup for now */
	DBVsnacc_to_dbv_interstring_list_opt(&psDbCombinations->ppsInnerDatabases, data_ptr, env);
}



int DBVsnacc_to_dbv_exp_netaddr_list(DbvNetworkAddress **ppsAddresses, TargetInfoSeqOf2 *addresses, jmp_buf env)
{
	NetworkAddress *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, addresses) {
		ppsAddresses[index] = (DbvNetworkAddress*) DbvAlloc (sizeof (DbvNetworkAddress));
		CheckDbvAlloc(ppsAddresses[index], env);
		DBVsnacc_to_dbv_exp_netaddr(ppsAddresses[index], data_ptr, env);
		index++;
	}
	ppsAddresses[index] = NULL;
}



int DBVsnacc_to_dbv_exp_netaddr(DbvNetworkAddress *psAddresses, NetworkAddress *data_ptr, jmp_buf env)
{
	switch (data_ptr->choiceId) {
	case NETWORKADDRESS_INTERNETADDRESS:
		psAddresses->eNetworkAddressType = NAT_InternetAddress;
		psAddresses->uNetworkAddress.psInternetAddress = (DbvInterAddress *) DbvAlloc (sizeof (DbvInterAddress));
		CheckDbvAlloc(psAddresses->uNetworkAddress.psInternetAddress, env);
		DBVsnacc_to_dbv_exp_interaddress(psAddresses->uNetworkAddress.psInternetAddress, data_ptr->a.internetAddress, env);
		break;
	case NETWORKADDRESS_OSIPRESENTATIONADDRESS:
		psAddresses->eNetworkAddressType = NAT_OsiPresentationAddress;
		psAddresses->uNetworkAddress.psOsiPresentationAddress = (DbvOsiPresentationAddress *) DbvAlloc (sizeof (DbvOsiPresentationAddress));
		CheckDbvAlloc(psAddresses->uNetworkAddress.psOsiPresentationAddress, env);
		DBVsnacc_to_dbv_exp_osipresaddr(psAddresses->uNetworkAddress.psOsiPresentationAddress, data_ptr->a.osiPresentationAddress, env);
		break;
	case NETWORKADDRESS_OTHER:
		psAddresses->eNetworkAddressType = NAT_OtherAddress;
		psAddresses->uNetworkAddress.psOtherAddress = (DbvOtherAddress *) DbvAlloc (sizeof (DbvOtherAddress));
		CheckDbvAlloc(psAddresses->uNetworkAddress.psOtherAddress, env);
		DBVsnacc_to_dbv_exp_otheradder(psAddresses->uNetworkAddress.psOtherAddress, data_ptr->a.other, env);
		break;
	default:
		psAddresses->eNetworkAddressType = NAT_Invalid;
		break;
	}
}



int DBVsnacc_to_dbv_exp_interaddress(DbvInterAddress *psInternetAddress,  struct NetworkAddressSeq* internetAddress, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psInternetAddress->sHostAddress, &internetAddress->hostAddress, env);

	/* c */
	psInternetAddress->ulPort = internetAddress->port;
}



int DBVsnacc_to_dbv_exp_osipresaddr(DbvOsiPresentationAddress *psOsiPresentationAddress, struct NetworkAddressSeq1* osiPresentationAddress, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psOsiPresentationAddress->sPsel, &osiPresentationAddress->pSel, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psOsiPresentationAddress->psSSel, &osiPresentationAddress->sSel, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psOsiPresentationAddress->psTSel, &osiPresentationAddress->tSel, env);
	

	/* c */
	DBVsnacc_to_dbv_inter_string(&psOsiPresentationAddress->sNSap, &osiPresentationAddress->nSap, env);
}


int DBVsnacc_to_dbv_exp_otheradder(DbvOtherAddress *psOtherAddress, struct NetworkAddressSeq2* other, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psOtherAddress->sType, &other->type, env);
	

	/* c */
	DBVsnacc_to_dbv_inter_string(&psOtherAddress->sAddress, &other->address, env);
	
}



int DBVsnacc_to_dbv_exp_accessinfo_opt(DbvAccessInfo **ppsCommonAccessInfo, struct AccessInfo *commonAccessInfo, jmp_buf env)
{

	if (commonAccessInfo != NULL) {
		*ppsCommonAccessInfo = (DbvAccessInfo *) DbvAlloc (sizeof (DbvAccessInfo));
		CheckDbvAlloc(*ppsCommonAccessInfo, env);
		DBVsnacc_to_dbv_exp_accessinfo(*ppsCommonAccessInfo, commonAccessInfo, env);
	}
}



int DBVsnacc_to_dbv_exp_accessinfo(DbvAccessInfo *psCommonAccessInfo, struct AccessInfo* commonAccessInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	if (commonAccessInfo->queryTypesSupported != NULL) {
		num_elements = AsnListCount(commonAccessInfo->queryTypesSupported);
                psCommonAccessInfo->ppsQueryTypesSupported = (DbvQueryTypeDetails **) DbvAlloc (sizeof (DbvQueryTypeDetails *) * (num_elements + 1));
                CheckDbvAlloc(psCommonAccessInfo->ppsQueryTypesSupported, env);
		DBVsnacc_to_dbv_exp_quertypede_list(psCommonAccessInfo->ppsQueryTypesSupported, commonAccessInfo->queryTypesSupported, env);
	}

	
	/* o */

	/* Remember that DbvObjectIdentifier is a typedefed char * */

	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcDiagnosticsSets, commonAccessInfo->diagnosticsSets, env);


	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcAttributeSetIds, commonAccessInfo->attributeSetIds, env);

	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcSchemas, commonAccessInfo->schemas, env);


	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcRecordSyntaxes, commonAccessInfo->recordSyntaxes, env);


	
	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcResourceChallenges, commonAccessInfo->resourceChallenges, env);


	/* o */
	if (commonAccessInfo->restrictedAccess != NULL) {
		num_elements = AsnListCount(commonAccessInfo->restrictedAccess);
                psCommonAccessInfo->ppsRestrictedAccess = (DbvAccessRestrictions **) DbvAlloc (sizeof (DbvAccessRestrictions *) * (num_elements + 1));
                CheckDbvAlloc(psCommonAccessInfo->ppsRestrictedAccess, env);
		DBVsnacc_to_dbv_exp_accessrestrict(psCommonAccessInfo->ppsRestrictedAccess, commonAccessInfo->restrictedAccess, env);
	}
	

	/* o */
	if (commonAccessInfo->costInfo != NULL) {
		psCommonAccessInfo->psCostInfo = (DbvCosts *) DbvAlloc (sizeof (DbvCosts));
		CheckDbvAlloc(psCommonAccessInfo->psCostInfo, env);
		DBVsnacc_to_dbv_exp_costs(psCommonAccessInfo->psCostInfo, commonAccessInfo->costInfo, env);
	}
	
	
	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psCommonAccessInfo->ppcVariantSets, commonAccessInfo->variantSets, env);
	

	
	/* o */
	/* LOOK OUT - THERE IS AN ElementsetNames STRUCTURE AND AN
	   ElementSetName (an InternationalString) STRUCTURE WHICH ARE 
    	   BOTH DIFFERENT */
	DBVsnacc_to_dbv_interstring_list_opt(&psCommonAccessInfo->ppsElementSetNames, commonAccessInfo->elementSetNames, env);


	/* c */
	DBVsnacc_to_dbv_interstring_list_opt(&psCommonAccessInfo->ppsUnitSystems, commonAccessInfo->unitSystems, env);

}




int DBVsnacc_to_dbv_exp_quertypede_list(DbvQueryTypeDetails **ppsQueryTypesSupported, AccessInfoSeqOf *queryTypesSupported, jmp_buf env)
{
	QueryTypeDetails *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, queryTypesSupported) {
		ppsQueryTypesSupported[index] = (DbvQueryTypeDetails*) DbvAlloc (sizeof (DbvQueryTypeDetails));
		CheckDbvAlloc(ppsQueryTypesSupported[index], env);
		DBVsnacc_to_dbv_exp_quertypede(ppsQueryTypesSupported[index], data_ptr, env);
		index++;
	}
	ppsQueryTypesSupported[index] = NULL;
}




/* Dont be fooled - this looks like code in target_z3950_search.c but 
   is very different */


int DBVsnacc_to_dbv_exp_quertypede(DbvQueryTypeDetails *psQueryTypesSupported, QueryTypeDetails *data_ptr, jmp_buf env)
{
	switch (data_ptr->choiceId) {
	case QUERYTYPEDETAILS_PRIVATE:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Private;
		psQueryTypesSupported->uQueryTypeDetails.psPrivate = (DbvPrivateCapabilities *) DbvAlloc (sizeof (DbvPrivateCapabilities));
		CheckDbvAlloc(psQueryTypesSupported->uQueryTypeDetails.psPrivate, env);
		DBVsnacc_to_dbv_privatecap(psQueryTypesSupported->uQueryTypeDetails.psPrivate, data_ptr->a.private, env);

		break;
	case QUERYTYPEDETAILS_RPN:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Rpn;
		DBVsnacc_to_dbv_rpncapab_comp(&psQueryTypesSupported->uQueryTypeDetails.psRpn, data_ptr->a.rpn, env);

		break;	
	case QUERYTYPEDETAILS_ISO8777:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Iso8777;
		psQueryTypesSupported->uQueryTypeDetails.psIso8777 = (DbvIso8777Capabilities *) DbvAlloc (sizeof (DbvIso8777Capabilities));
		CheckDbvAlloc(psQueryTypesSupported->uQueryTypeDetails.psIso8777, env);
		DBVsnacc_to_dbv_iso8777cap(psQueryTypesSupported->uQueryTypeDetails.psIso8777, data_ptr->a.iso8777, env);

		break;
	case QUERYTYPEDETAILS_Z39_58:

		/* Its DbvHumanString - compulsory but pointers */
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Z39_58;

		/* Although compulsory use the optional set up function */
		DBVsnacc_to_dbv_humanstring_list_opt(&psQueryTypesSupported->uQueryTypeDetails.ppsZ39_58, data_ptr->a.z39_58, env);

		break;
	case QUERYTYPEDETAILS_ERPN:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Erpn;
		DBVsnacc_to_dbv_rpncapab_comp(&psQueryTypesSupported->uQueryTypeDetails.psErpn, data_ptr->a.erpn, env);
		break;
	case QUERYTYPEDETAILS_RANKEDLIST:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_RankedList;
		DBVsnacc_to_dbv_humanstring_list_opt(&psQueryTypesSupported->uQueryTypeDetails.ppsRankedList, data_ptr->a.rankedList, env);
		break;
	default:
		psQueryTypesSupported->eQueryTypeDetailsType = QTDT_Invalid;
		break;
		
	}
}



int DBVsnacc_to_dbv_rpncapab_comp(DbvRpnCapabilities **psRpn, struct RpnCapabilities *rpn, jmp_buf env)
{
	*psRpn = (DbvRpnCapabilities *) DbvAlloc (sizeof (DbvRpnCapabilities));
	CheckDbvAlloc(*psRpn, env);
	DBVsnacc_to_dbv_rpncap(*psRpn, rpn, env);
}



int DBVsnacc_to_dbv_privatecap(DbvPrivateCapabilities *psPrivate, struct PrivateCapabilities* private, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	if (private->operators != NULL) {
		num_elements = AsnListCount(private->operators);
       		psPrivate->ppsOperators = (DbvOperators **) DbvAlloc (sizeof (DbvOperators *) * (num_elements + 1));
		CheckDbvAlloc(psPrivate->ppsOperators, env);
		DBVsnacc_to_dbv_exp_ops_list(psPrivate->ppsOperators, private->operators, env);
	}



	/* o */
	DBVsnacc_to_dbv_exp_srchket_opt(&psPrivate->ppsSearchKeys, private->searchKeys, env);


	/* o */
	if (private->description != NULL) {
		num_elements = AsnListCount(private->description);
       		psPrivate->ppsDescription = (DbvHumanStringSeq **) DbvAlloc (sizeof (DbvHumanStringSeq *) * (num_elements + 1));
		CheckDbvAlloc(psPrivate->ppsDescription, env);
		DBVsnacc_to_dbv_humanstringseq_list(psPrivate->ppsDescription, private->description, env);
	}

}



int DBVsnacc_to_dbv_exp_ops_list(DbvOperators **ppsOperators, PrivateCapabilitiesSeqOf *operators, jmp_buf env)
{
	PrivateCapabilitiesSeqOfSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, operators) {
		ppsOperators[index] = (DbvOperators*) DbvAlloc (sizeof (DbvOperators));
		CheckDbvAlloc(ppsOperators[index], env);
		DBVsnacc_to_dbv_exp_operators(ppsOperators[index], data_ptr, env);
		index++;
	}
	ppsOperators[index] = NULL;
}



int DBVsnacc_to_dbv_exp_operators(DbvOperators *psOperators, PrivateCapabilitiesSeqOfSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psOperators->sOperator, &data_ptr->operator, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psOperators->ppsDescription, data_ptr->description, env);
	
	
}



int DBVsnacc_to_dbv_exp_srchket_opt(DbvSearchKey ***pppsSearchKey, PrivateCapabilitiesSeqOf1 *searchKeys, jmp_buf env)
{
	int num_elements = 0;
	
	if (searchKeys != NULL) {
		num_elements = AsnListCount(searchKeys);
       		*pppsSearchKey = (DbvSearchKey **) DbvAlloc (sizeof (DbvSearchKey *) * (num_elements + 1));
		CheckDbvAlloc(*pppsSearchKey, env);
		DBVsnacc_to_dbv_exp_srchkey_list(*pppsSearchKey, searchKeys, env);
	}
}



int DBVsnacc_to_dbv_exp_srchkey_list(DbvSearchKey **ppsSearchKey, PrivateCapabilitiesSeqOf1* searchKeys, jmp_buf env)
{
	SearchKey *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, searchKeys) {
		ppsSearchKey[index] = (DbvSearchKey*) DbvAlloc (sizeof (DbvSearchKey));
		CheckDbvAlloc(ppsSearchKey[index], env);
		DBVsnacc_to_dbv_exp_srchkey(ppsSearchKey[index], data_ptr, env);
		index++;
	}
	ppsSearchKey[index] = NULL;
	
}



int DBVsnacc_to_dbv_exp_srchkey(DbvSearchKey *psSearchKey, SearchKey *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psSearchKey->sSearchKey, &data_ptr->searchKey, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psSearchKey->ppsDescription, data_ptr->description, env);
	
}



int DBVsnacc_to_dbv_humanstringseq_list(DbvHumanStringSeq **ppsDescription, PrivateCapabilitiesSeqOf2 *description, jmp_buf env)
{
	HumanString *data_ptr;
        int index = 0;
	int num_elements = 0;

	FOR_EACH_LIST_ELMT(data_ptr, description) {
		ppsDescription[index] = (DbvHumanStringSeq*) DbvAlloc (sizeof (DbvHumanStringSeq));
		CheckDbvAlloc(ppsDescription[index], env);

		num_elements = AsnListCount(data_ptr);
       		ppsDescription[index]->ppsHumanString = (DbvHumanString **) DbvAlloc (sizeof (DbvHumanString *) * (num_elements + 1));
		CheckDbvAlloc(ppsDescription[index]->ppsHumanString, env);
		DBVsnacc_to_dbv_humanstring_list(ppsDescription[index]->ppsHumanString, data_ptr, env);
		index++;
	}
	ppsDescription[index] = NULL;
}





int DBVsnacc_to_dbv_rpncap(DbvRpnCapabilities *psRpn, struct RpnCapabilities* rpn, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_ulong_list_opt(&psRpn->ppulOperators, rpn->operators, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psRpn->fResultSetAsOperandSupported, &rpn->resultSetAsOperandSupported, env);

	/* c */
	DBVsnacc_to_dbv_bool(&psRpn->fRestrictionOperandSupported, &rpn->restrictionOperandSupported, env);

	/* o */
	if (rpn->proximity != NULL) {
		psRpn->psProximity = (DbvProximitySupport *) DbvAlloc (sizeof (DbvProximitySupport));
		CheckDbvAlloc(psRpn->psProximity, env);
		DBVsnacc_to_dbv_exp_proximity(psRpn->psProximity, rpn->proximity, env);
	}
	
}



int DBVsnacc_to_dbv_exp_proximity(DbvProximitySupport *psProximity, struct ProximitySupport* proximity, jmp_buf env)
{
	int num_elements = 0;

	/* c */
	DBVsnacc_to_dbv_bool(&psProximity->fAnySupport, &proximity->anySupport, env);
	
	/* o */
	if (proximity->unitsSupported != NULL) {
		num_elements = AsnListCount(proximity->unitsSupported);
       		psProximity->ppsUnitsSupported = (DbvUnitsSupported **) DbvAlloc (sizeof (DbvUnitsSupported *) * (num_elements + 1));
		CheckDbvAlloc(psProximity->ppsUnitsSupported, env);
		DBVsnacc_to_dbv_exp_unitssupp_list(psProximity->ppsUnitsSupported, proximity->unitsSupported, env);
	}
		

}


int DBVsnacc_to_dbv_exp_unitssupp_list(DbvUnitsSupported **ppsUnitsSupported, ProximitySupportSeqOf *unitsSupported, jmp_buf env)
{
	ProximitySupportSeqOfChoice *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, unitsSupported) {
		ppsUnitsSupported[index] = (DbvUnitsSupported*) DbvAlloc (sizeof (DbvUnitsSupported));
		CheckDbvAlloc(ppsUnitsSupported[index], env);
		DBVsnacc_to_dbv_exp_unitssupp(ppsUnitsSupported[index], data_ptr, env);
		index++;
	}
	ppsUnitsSupported[index] = NULL;
}



int DBVsnacc_to_dbv_exp_unitssupp(DbvUnitsSupported *psUnitsSupported, ProximitySupportSeqOfChoice *data_ptr, jmp_buf env)
{
	switch (data_ptr->choiceId) {
	case PROXIMITYSUPPORTSEQOFCHOICE_KNOWN:
		psUnitsSupported->eUnitsSupportedType = UST_Known;
		psUnitsSupported->uUnitsSupported.ulKnown = data_ptr->a.known;
		break;
	case PROXIMITYSUPPORTSEQOFCHOICE_PRIVATE:
		psUnitsSupported->eUnitsSupportedType = UST_Private;
		psUnitsSupported->uUnitsSupported.psPrivate = (DbvPrivate *) DbvAlloc (sizeof (DbvPrivate));
		CheckDbvAlloc(psUnitsSupported->uUnitsSupported.psPrivate, env);
		DBVsnacc_to_dbv_exp_private(psUnitsSupported->uUnitsSupported.psPrivate, data_ptr->a.private, env);

		break;
	default:
		psUnitsSupported->eUnitsSupportedType = UST_Invalid;
		break;
	}
}


int DBVsnacc_to_dbv_exp_private(DbvPrivate *psPrivate, struct ProximitySupportSeqOfChoiceSeq* private, jmp_buf env)
{
	/* c */
	psPrivate->ulUnit = private->unit;
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psPrivate->ppsDescription, private->description, env);
}


int DBVsnacc_to_dbv_iso8777cap(DbvIso8777Capabilities *psIso8777, struct Iso8777Capabilities* iso8777, jmp_buf env)
{
	/* c  - although compulsory we can use the optional set up function
	   because we still have to malloc the space */
	DBVsnacc_to_dbv_exp_srchket_opt(&psIso8777->ppsSearchKeys, iso8777->searchKeys, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psIso8777->ppsRestrictions, iso8777->restrictions, env);
}



int DBVsnacc_to_dbv_exp_objectident_list(DbvObjectIdentifier *ppcDiagnosticsSets, AccessInfoSeqOf1 *diagnosticsSets, jmp_buf env)
{
	AsnOid *data_ptr;

        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, diagnosticsSets) {
		DBVsnacc_to_dbv_objectident_comp(&ppcDiagnosticsSets[index], data_ptr, env);
		index++;
	}
	ppcDiagnosticsSets[index] = NULL;
}



int DBVsnacc_to_dbv_exp_accessrestrict(DbvAccessRestrictions **ppsRestrictedAccess, AccessRestrictions *restrictedAccess, jmp_buf env)
{
	AccessRestrictionsSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, restrictedAccess) {
		
		ppsRestrictedAccess[index] = (DbvAccessRestrictions *) DbvAlloc (sizeof (DbvAccessRestrictions));
		CheckDbvAlloc(ppsRestrictedAccess[index], env);
		DBVsnacc_to_dbv_exp_accessrest(ppsRestrictedAccess[index], data_ptr, env);
		index++;
	}
	ppsRestrictedAccess[index] = NULL;
}



int DBVsnacc_to_dbv_exp_accessrest(DbvAccessRestrictions *psRestrictedAccess, AccessRestrictionsSeq *data_ptr, jmp_buf env)
{

	/* c */
	DBVsnacc_to_dbv_exp_accesstype(&psRestrictedAccess->eAccessType, &data_ptr->accessType, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psRestrictedAccess->ppsAccessText, data_ptr->accessText, env);

	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psRestrictedAccess->ppcAccessChallenges, data_ptr->accessChallenges, env);
	
}



int DBVsnacc_to_dbv_exp_accesstype(DbvAccessType *peAccessType, AccessRestrictionsSeqInt *accessType, jmp_buf env)
{
	switch (*accessType) {
	case ANY:
		*peAccessType = AT_Any;
		break;
	case SEARCH1:
		*peAccessType = AT_Search;
		break;
	case PRESENT2:
		*peAccessType = AT_Present;
		break;
	case SPECIFIC_ELEMENTS:
		*peAccessType = AT_SpecificElements;
		break;
	case EXTENDED_SERVICES:
		*peAccessType = AT_ExtendedServices;
		break;
	case BY_DATABASE:
		*peAccessType = AT_ByDatabase;
		break;
	default:
		*peAccessType = AT_Invalid;
		break;

	}
}



int DBVsnacc_to_dbv_exp_costs(DbvCosts *psCostInfo, struct Costs* costInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_charge_opt(&psCostInfo->psConnectCharge, costInfo->connectCharge, env);

	/* o */
	DBVsnacc_to_dbv_exp_charge_opt(&psCostInfo->psConnectTime, costInfo->connectTime, env);

	/* o */
	DBVsnacc_to_dbv_exp_charge_opt(&psCostInfo->psDisplayCharge, costInfo->displayCharge, env);

	/* o */
	DBVsnacc_to_dbv_exp_charge_opt(&psCostInfo->psSearchCharge, costInfo->searchCharge, env);

	/* o */
	DBVsnacc_to_dbv_exp_charge_opt(&psCostInfo->psSubscriptCharge, costInfo->subscriptCharge, env);

	/* o */
	if (costInfo->otherCharges != NULL) {
		num_elements = AsnListCount(costInfo->otherCharges);
                psCostInfo->ppsOtherCharge = (DbvOtherCharge **) DbvAlloc (sizeof (DbvOtherCharge *) * (num_elements + 1));
                CheckDbvAlloc(psCostInfo->ppsOtherCharge, env);
		DBVsnacc_to_dbv_exp_othercharge_list(psCostInfo->ppsOtherCharge,costInfo->otherCharges, env);
	}
}
		




int DBVsnacc_to_dbv_exp_charge_opt(DbvCharge **ppsConnectCharge, struct Charge *connectCharge, jmp_buf env)
{

	if (connectCharge != NULL) {
		*ppsConnectCharge = (DbvCharge *) DbvAlloc (sizeof (DbvCharge));
		CheckDbvAlloc(*ppsConnectCharge, env);
		DBVsnacc_to_dbv_exp_charge(*ppsConnectCharge, connectCharge, env);

	}
	
}



int DBVsnacc_to_dbv_exp_charge(DbvCharge *psConnectCharge, struct Charge* connectCharge, jmp_buf env)
{
	/* c */
	if (connectCharge->cost != NULL) 
		DBVsnacc_to_dbv_intunit(&psConnectCharge->sCost, connectCharge->cost, env);


	/* o */
	DBVsnacc_to_dbv_unit_opt(&psConnectCharge->psPerWhat, connectCharge->perWhat, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psConnectCharge->ppsText, connectCharge->text, env);
	
}



int DBVsnacc_to_dbv_unit_opt(DbvUnit **ppsPerWhat, struct Unit *perWhat, jmp_buf env)
{

	if (perWhat != NULL) {
		*ppsPerWhat = (DbvUnit *) DbvAlloc (sizeof (DbvUnit));
		CheckDbvAlloc(*ppsPerWhat, env);
		DBVsnacc_to_dbv_unit(*ppsPerWhat, perWhat, env);
	}
}

int DBVsnacc_to_dbv_exp_othercharge_list(DbvOtherCharge **ppsOtherCharge, CostsSeqOf *otherCharges, jmp_buf env)
{
	CostsSeqOfSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, otherCharges) {
		
		ppsOtherCharge[index] = (DbvOtherCharge *) DbvAlloc (sizeof (DbvOtherCharge));
		CheckDbvAlloc(ppsOtherCharge[index], env);
		DBVsnacc_to_dbv_exp_othercharge(ppsOtherCharge[index], data_ptr, env);
		index++;
	}
	ppsOtherCharge[index] = NULL;
}



int DBVsnacc_to_dbv_exp_othercharge(DbvOtherCharge *psOtherCharge, CostsSeqOfSeq *data_ptr, jmp_buf env)
{
	/* c  - use the _opt set up */
	DBVsnacc_to_dbv_humanstring_list_opt(&psOtherCharge->ppsForWhat, data_ptr->forWhat, env);

	/* c */
	DBVsnacc_to_dbv_exp_charge(&psOtherCharge->sCharge, data_ptr->charge, env);
}



/* -------------------- DbvDatabaseInfo -------------------- */


int DBVsnacc_to_dbv_exp_dbinfo(DbvDatabaseInfo *psDatabaseInfo, struct DatabaseInfo* databaseInfo, jmp_buf env)
{
	int num_elements = 0;

	 /* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psDatabaseInfo->psCommonInfo, databaseInfo->commonInfo, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psDatabaseInfo->sName, &databaseInfo->name, env);
	

	/* o */
	if (databaseInfo->explainDatabase != NULL) {

		/* Just alloc a byte */
		psDatabaseInfo->pvExplainDatabase = (char *) DbvAlloc (1);
		CheckDbvAlloc(psDatabaseInfo->pvExplainDatabase, env);
	}


	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psDatabaseInfo->ppsNicknames, databaseInfo->nicknames, env);


	/* o */
	DBVsnacc_to_dbv_icon_list_opt(&psDatabaseInfo->ppsIcon, databaseInfo->icon, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psDatabaseInfo->fUserFee, &databaseInfo->user_fee, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psDatabaseInfo->fAvailable, &databaseInfo->available, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsTitleString, databaseInfo->titleString, env);


	/* o */
	if (databaseInfo->keywords) {
		num_elements = AsnListCount(databaseInfo->keywords);
                psDatabaseInfo->ppsKeywords = (DbvDataBaseInfoSeq **) DbvAlloc (sizeof (DbvDataBaseInfoSeq *) * (num_elements + 1));
                CheckDbvAlloc(psDatabaseInfo->ppsKeywords, env);
		DBVsnacc_to_dbv_exp_databaseinfo_list(psDatabaseInfo->ppsKeywords,databaseInfo->keywords, env);
	}
	

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsDescription, databaseInfo->description, env);


	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psDatabaseInfo->ppsAssociatedDbs, databaseInfo->associatedDbs, env);


	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psDatabaseInfo->ppsSubDbs, databaseInfo->subDbs, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsDisclaimers, databaseInfo->disclaimers, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsNews, databaseInfo->news, env);


	/* o */
	if (databaseInfo->recordCount != NULL) {
		psDatabaseInfo->psRecordCount = (DbvRecordCount *) DbvAlloc (sizeof (DbvRecordCount));
		CheckDbvAlloc(psDatabaseInfo->psRecordCount, env);
		DBVsnacc_to_dbv_exp_recordcount(psDatabaseInfo->psRecordCount, databaseInfo->recordCount, env);
	}
		
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsDefaultOrder, databaseInfo->defaultOrder, env);


	/* o */
	DBVsnacc_to_dbv_int_opt(&psDatabaseInfo->pulAvRecordSize, databaseInfo->avRecordSize, env);


	/* o */
	DBVsnacc_to_dbv_int_opt(&psDatabaseInfo->pulMaxRecordSize, databaseInfo->maxRecordSize, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsHours, databaseInfo->hours, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsBestTime, databaseInfo->bestTime, env);

	/* o */
	DBVsnacc_to_dbv_gentime_opt(&psDatabaseInfo->pcLastUpdate, databaseInfo->lastUpdate, env);

		
	/* o */ 
	DBVsnacc_to_dbv_intunit_opt(&psDatabaseInfo->psUpdateInterval, databaseInfo->updateInterval, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsCoverage, databaseInfo->coverage, env);

	/* o */	
	if (databaseInfo->proprietary != NULL) {
		psDatabaseInfo->pfProprietary = (DbvBool *) DbvAlloc (sizeof (DbvBool));
		CheckDbvAlloc(psDatabaseInfo->pfProprietary, env);
		DBVsnacc_to_dbv_bool(psDatabaseInfo->pfProprietary, databaseInfo->proprietary, env);
	}

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsCopyrightText, databaseInfo->copyrightText, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDatabaseInfo->ppsCopyrightNotice, databaseInfo->copyrightNotice, env);


	/* o */
	DBVsnacc_to_dbv_exp_contact_info_opt(&psDatabaseInfo->psProducerContactInfo, databaseInfo->producerContactInfo, env);


	/* o */
	DBVsnacc_to_dbv_exp_contact_info_opt(&psDatabaseInfo->psSupplierContactInfo, databaseInfo->supplierContactInfo, env);


	/* o */
	DBVsnacc_to_dbv_exp_contact_info_opt(&psDatabaseInfo->psSubmissionContactInfo, databaseInfo->submissionContactInfo, env);


	/* o */
	DBVsnacc_to_dbv_exp_accessinfo_opt(&psDatabaseInfo->psAccessInfo, databaseInfo->accessInfo, env);

}



int DBVsnacc_to_dbv_exp_databaseinfo_list(DbvDataBaseInfoSeq **ppsKeywords, DatabaseInfoSeqOf1 *keywords, jmp_buf env)
{
	HumanString *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, keywords) {
		
		ppsKeywords[index] = (DbvDataBaseInfoSeq *) DbvAlloc (sizeof (DbvDataBaseInfoSeq));
		CheckDbvAlloc(ppsKeywords[index], env);
		DBVsnacc_to_dbv_humanstring_list_opt(&ppsKeywords[index]->ppsDataBaseInfo, data_ptr, env);
		index++;
	}
	ppsKeywords[index] = NULL;
}



int DBVsnacc_to_dbv_exp_recordcount(DbvRecordCount *psRecordCount, struct DatabaseInfoChoice* recordCount, jmp_buf env)
{
	switch (recordCount->choiceId) {
	case DATABASEINFOCHOICE_ACTUALNUMBER:
		psRecordCount->eRecordCountType = RCT_ActualNumber;
		psRecordCount->uRecordCount.ulActualNumber = recordCount->a.actualNumber;
		break;
	case DATABASEINFOCHOICE_APPROXNUMBER:
		psRecordCount->eRecordCountType = RCT_ApproxNumber;
		psRecordCount->uRecordCount.ulApproxNumber = recordCount->a.approxNumber;
		break;
	default:
		psRecordCount->eRecordCountType = RCT_Invalid;
		break;
	}
}



/* -------------------- DbvSchemaInfo -------------------- */

int DBVsnacc_to_dbv_exp_schemainfo(DbvSchemaInfo *psSchemaInfo, struct SchemaInfo* schemaInfo, jmp_buf env)
{
	int num_elements = 0;

	 /* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psSchemaInfo->psCommonInfo, schemaInfo->commonInfo, env);

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psSchemaInfo->pcSchema, &schemaInfo->schema, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psSchemaInfo->sName, &schemaInfo->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psSchemaInfo->ppsDescription, schemaInfo->description, env);

	/* o */
	if (schemaInfo->tagTypeMapping != NULL) {
		num_elements = AsnListCount(schemaInfo->tagTypeMapping);
       		psSchemaInfo->ppsTagTypeMapping = (DbvTagTypeMapping **) DbvAlloc (sizeof (DbvTagTypeMapping *) * (num_elements + 1));
		CheckDbvAlloc(psSchemaInfo->ppsTagTypeMapping, env);
		DBVsnacc_to_dbv_exp_tagtypemapp_list(psSchemaInfo->ppsTagTypeMapping, schemaInfo->tagTypeMapping, env);
	}
		

	/* o */
	DBVsnacc_to_dbv_exp_elementinfo_list_opt(&psSchemaInfo->ppsRecordStructure, schemaInfo->recordStructure, env);



}



int DBVsnacc_to_dbv_exp_tagtypemapp_list(DbvTagTypeMapping **ppsTagTypeMapping, SchemaInfoSeqOf *tagTypeMapping, jmp_buf env)
{
	SchemaInfoSeqOfSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, tagTypeMapping) {
		
		ppsTagTypeMapping[index] = (DbvTagTypeMapping *) DbvAlloc (sizeof (DbvTagTypeMapping));
		CheckDbvAlloc(ppsTagTypeMapping[index], env);
		DBVsnacc_to_dbv_exp_tagtypemapp(ppsTagTypeMapping[index], data_ptr, env);
		index++;
	}
	ppsTagTypeMapping[index] = NULL;
}



int DBVsnacc_to_dbv_exp_tagtypemapp(DbvTagTypeMapping *psTagTypeMapping, SchemaInfoSeqOfSeq *data_ptr, jmp_buf env)
{
	/* c */
	psTagTypeMapping->ulTagType = data_ptr->tagType;

	/* o */
	DBVsnacc_to_dbv_objectident_opt(&psTagTypeMapping->pcTagSet, &data_ptr->tagSet, env);
	
	/* o */
	DBVsnacc_to_dbv_nullpointer_opt(&psTagTypeMapping->pvDefaultTagType, data_ptr->defaultTagType, env);

}



int DBVsnacc_to_dbv_exp_elementinfo_list_opt(DbvElementInfo ***pppsRecordStructure, SchemaInfoSeqOf1 *recordStructure, jmp_buf env)
{
	int num_elements = 0;

	if (recordStructure != NULL) {
		num_elements = AsnListCount(recordStructure);
       		*pppsRecordStructure = (DbvElementInfo **) DbvAlloc (sizeof (DbvElementInfo *) * (num_elements + 1));
		CheckDbvAlloc(*pppsRecordStructure, env);
		DBVsnacc_to_dbv_exp_elementinfo_list(*pppsRecordStructure, recordStructure, env);
	}
}



int DBVsnacc_to_dbv_exp_elementinfo_list(DbvElementInfo **ppsRecordStructure, SchemaInfoSeqOf1 *recordStructure, jmp_buf env)
{
	ElementInfo *data_ptr;
	int num_elements = 0;

        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, recordStructure) {
		
		ppsRecordStructure[index] = (DbvElementInfo *) DbvAlloc (sizeof (DbvElementInfo));
		CheckDbvAlloc(ppsRecordStructure[index], env);
		DBVsnacc_to_dbv_exp_elementinfo(ppsRecordStructure[index], data_ptr, env);
		index++;
	}
	ppsRecordStructure[index] = NULL;

}


int DBVsnacc_to_dbv_exp_elementinfo(DbvElementInfo *psRecordStructure, ElementInfo *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* c */
	DBVsnacc_to_dbv_inter_string(&psRecordStructure->sElementName, &data_ptr->elementName, env);
	
	/* c */
	num_elements = AsnListCount(data_ptr->elementTagPath);
       	psRecordStructure->ppsElementTagPath = (DbvPath **) DbvAlloc (sizeof (DbvPath *) * (num_elements + 1));
	CheckDbvAlloc(psRecordStructure->ppsElementTagPath, env);
	DBVsnacc_to_dbv_exp_path_list(psRecordStructure->ppsElementTagPath, data_ptr->elementTagPath, env);
	
	
	/* o */
	if (data_ptr->dataType != NULL) {
		psRecordStructure->psDataType = (struct DbvElementDataType *) DbvAlloc (sizeof (struct DbvElementDataType));
		CheckDbvAlloc(psRecordStructure->psDataType, env);
		DBVsnacc_to_dbv_eldatatype(psRecordStructure->psDataType, data_ptr->dataType, env);
	}


	/* c */
	DBVsnacc_to_dbv_bool(&psRecordStructure->fRequired, &data_ptr->required, env);

	/* c */
	DBVsnacc_to_dbv_bool(&psRecordStructure->fRepeatable, &data_ptr->repeatable, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psRecordStructure->ppsDescription, data_ptr->description, env);

}



int DBVsnacc_to_dbv_exp_path_list(DbvPath **ppsElementTagPath, Path *elementTagPath, jmp_buf env)
{
	PathSeq *data_ptr;
        int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, elementTagPath) {
		
		ppsElementTagPath[index] = (DbvPath *) DbvAlloc (sizeof (DbvPath));
		CheckDbvAlloc(ppsElementTagPath[index], env);
		DBVsnacc_to_dbv_exp_path(ppsElementTagPath[index], data_ptr, env);
		index++;
	}
	ppsElementTagPath[index] = NULL;
}



int DBVsnacc_to_dbv_exp_path(DbvPath *psElementTagPath, PathSeq *data_ptr, jmp_buf env)
{
	/* c */
	psElementTagPath->ulTagType = data_ptr->tagType;
	
	/* c */
	DBVsnacc_to_dbv_strornum(&psElementTagPath->sTagValue, data_ptr->tagValue, env);
}


int DBVsnacc_to_dbv_eldatatype(struct DbvElementDataType *psDataType, struct ElementDataType* dataType, jmp_buf env)
{
	switch (dataType->choiceId) {
	case ELEMENTDATATYPE_PRIMITIVE:
		psDataType->eElementDataTypeType = EDTT_Primitive;
		DBVsnacc_to_dbv_exp_primitive(&psDataType->uElementDataType.ePrimitive, &dataType->a.primitive, env);
		break;
	case ELEMENTDATATYPE_STRUCTURED:
		psDataType->eElementDataTypeType = EDTT_Structured;
		DBVsnacc_to_dbv_exp_elementinfo_list_opt(&psDataType->uElementDataType.ppsStructured, dataType->a.structured, env);
		break;
	default:
		psDataType->eElementDataTypeType = EDTT_Invalid;
		break;
	}
}


int DBVsnacc_to_dbv_exp_primitive(DbvPrimitiveDataType *pePrimitive, PrimitiveDataType *primitive, jmp_buf env)
{
	switch (*primitive) {
	case OCTETSTRING:
		*pePrimitive = PDT_OctetString;
		break;

	case NUMERIC:
		*pePrimitive = PDT_Numeric;
		break;

	case DATE:
		*pePrimitive = PDT_Date;
		break;

	case EXTERNAL1:
		*pePrimitive = PDT_External;
		break;

	case STRING:
		*pePrimitive = PDT_String;
		break;

	case TRUEORFALSE:
		*pePrimitive = PDT_TrueOrFalse;
		break;

	case OID_SNACC:
		*pePrimitive = PDT_Oid;
		break;

	case INTUNIT:
		*pePrimitive = PDT_IntUnit;
		break;

	case EMPTY1:
		*pePrimitive = PDT_Empty;
		break;

	case NONEOFTHEABOVE:
		*pePrimitive = PDT_NoneOfTheAbove;
		break;

	default:
		*pePrimitive = PDT_Invalid;
		break;


	}
}



/* -------------------- DbvTagSetInfo -------------------- */

int DBVsnacc_to_dbv_exp_tagsetinfo(DbvTagSetInfo *psTagSetInfo, struct TagSetInfo* tagSetInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psTagSetInfo->psCommonInfo, tagSetInfo->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psTagSetInfo->pcTagSet, &tagSetInfo->tagSet, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psTagSetInfo->sName, &tagSetInfo->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTagSetInfo->ppsDescription, tagSetInfo->description, env);


	/* o */
	if (tagSetInfo->elements != NULL) {
		num_elements = AsnListCount(tagSetInfo->elements);
       		psTagSetInfo->ppsElements = (DbvElements **) DbvAlloc (sizeof (DbvElements *) * (num_elements + 1));
		CheckDbvAlloc(psTagSetInfo->ppsElements, env);
		DBVsnacc_to_dbv_exp_elements_list(psTagSetInfo->ppsElements, tagSetInfo->elements, env);
	}
}



int DBVsnacc_to_dbv_exp_elements_list(DbvElements **ppsElements, TagSetInfoSeqOf *elements, jmp_buf env)
{
	/* clever, but difficult to read .. */

	TagSetInfoSeqOfSeq *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, elements) {
		
		*ppsElements = (DbvElements *) DbvAlloc (sizeof (DbvElements));
		CheckDbvAlloc(*ppsElements, env);
		DBVsnacc_to_dbv_exp_elements(*ppsElements, data_ptr, env);
		ppsElements++;
	}
	*ppsElements = NULL;
}



int DBVsnacc_to_dbv_exp_elements(DbvElements *psElements, TagSetInfoSeqOfSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psElements->sElementName, &data_ptr->elementname, env);
	
	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psElements->ppsNicknames, data_ptr->nicknames, env);

	/* c */
	DBVsnacc_to_dbv_strornum(&psElements->sElementTag, data_ptr->elementTag, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psElements->ppsDescription, data_ptr->description, env);

	/* o */
	if (data_ptr->dataType != NULL) {
		psElements->peDataType = (DbvPrimitiveDataType *) DbvAlloc (sizeof (DbvPrimitiveDataType));
		CheckDbvAlloc(psElements->peDataType, env);
		DBVsnacc_to_dbv_exp_primitive(psElements->peDataType, data_ptr->dataType, env);
	}


	/* o */
	DBVsnacc_to_dbv_otherinfo_opt(&psElements->ppsOtherInformation, data_ptr->otherTagInfo, env);

}


	


/* -------------------- RecordSyntaxInfo -------------------- */

int DBVsnacc_to_dbv_exp_recsyninfo(DbvRecordSyntaxInfo *psRecordSyntaxInfo, struct RecordSyntaxInfo* recordSyntaxInfo, jmp_buf env)
{

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psRecordSyntaxInfo->psCommonInfo, recordSyntaxInfo->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psRecordSyntaxInfo->pcRecordSyntax, &recordSyntaxInfo->recordSyntax, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psRecordSyntaxInfo->sName, &recordSyntaxInfo->name, env);

	/* o */
	DBVsnacc_to_dbv_objectident_list_opt(&psRecordSyntaxInfo->ppcTransferSyntaxes, recordSyntaxInfo->transferSyntaxes, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psRecordSyntaxInfo->ppsDescription, recordSyntaxInfo->description, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psRecordSyntaxInfo->psAsn1Module, &recordSyntaxInfo->asn1Module, env);

	/* o */
	DBVsnacc_to_dbv_exp_elementinfo_list_opt(&psRecordSyntaxInfo->ppsAbstractStructure, recordSyntaxInfo->abstractStructure, env);
}



/* -------------------- DbvAttributeSetInfo -------------------- */

int DBVsnacc_to_dbv_exp_attsetinfo(DbvAttributeSetInfo *psAttributeSetInfo, struct AttributeSetInfo* attributeSetInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psAttributeSetInfo->psCommonInfo, attributeSetInfo->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psAttributeSetInfo->pcAttributeSetId, &attributeSetInfo->attributeSet, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psAttributeSetInfo->sName, &attributeSetInfo->name, env);


	/* o */
	if (attributeSetInfo->attributes != NULL) {
		num_elements = AsnListCount(attributeSetInfo->attributes);
       		psAttributeSetInfo->ppsAttributes = (DbvAttributeType **) DbvAlloc (sizeof (DbvAttributeType *) * (num_elements + 1));
		CheckDbvAlloc(psAttributeSetInfo->ppsAttributes, env);
		DBVsnacc_to_dbv_exp_attributetype_list(psAttributeSetInfo->ppsAttributes, attributeSetInfo->attributes, env);
	}
		

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psAttributeSetInfo->ppsDescription, attributeSetInfo->description, env);

}



int DBVsnacc_to_dbv_exp_attributetype_list(DbvAttributeType **ppsAttributes, AttributeSetInfoSeqOf *attributes, jmp_buf env)
{
	/* clever, but difficult to read .. */

	AttributeType *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, attributes) {
		
		*ppsAttributes = (DbvAttributeType *) DbvAlloc (sizeof (DbvAttributeType));
		CheckDbvAlloc(*ppsAttributes, env);
		DBVsnacc_to_dbv_exp_attributetype(*ppsAttributes, data_ptr, env);
		ppsAttributes++;
	}
	*ppsAttributes = NULL;
}



int DBVsnacc_to_dbv_exp_attributetype(DbvAttributeType *psAttributes, AttributeType *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psAttributes->psName, &data_ptr->name, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psAttributes->ppsDescription, data_ptr->description, env);

	/* c */
	psAttributes->ulAttributeType = data_ptr->attributeType;



	/* AttributeType, AttributeTypeSeqOf: DbvAttributeType, DbvAttributeDescription */
	/* c */
	num_elements = AsnListCount(data_ptr->attributeValues);
       	psAttributes->ppsAttributeValues = (DbvAttributeDescription **) DbvAlloc (sizeof (DbvAttributeDescription *) * (num_elements + 1));
	CheckDbvAlloc(psAttributes->ppsAttributeValues, env);
	DBVsnacc_to_dbv_exp_attributevalues_list(psAttributes->ppsAttributeValues, data_ptr->attributeValues, env);
}



int DBVsnacc_to_dbv_exp_attributevalues_list(DbvAttributeDescription **ppsAttributeValues, AttributeTypeSeqOf *attributeValues, jmp_buf env)
{
	AttributeDescription *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, attributeValues) {
		
		*ppsAttributeValues = (DbvAttributeDescription *) DbvAlloc (sizeof (DbvAttributeDescription));
		CheckDbvAlloc(*ppsAttributeValues, env);
		DBVsnacc_to_dbv_exp_attributevalues(*ppsAttributeValues, data_ptr, env);
		ppsAttributeValues++;
	}
	*ppsAttributeValues = NULL;
}



int DBVsnacc_to_dbv_exp_attributevalues(DbvAttributeDescription *psAttributeValues, AttributeDescription *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psAttributeValues->psName, &data_ptr->name, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psAttributeValues->ppsDescription, data_ptr->description, env);

	/* c */
	DBVsnacc_to_dbv_strornum(&psAttributeValues->sAttributeValue, data_ptr->attributeValue, env);

	/* o */
	if (data_ptr->equivalentAttributes != NULL) {
		num_elements = AsnListCount(data_ptr->equivalentAttributes);
       		psAttributeValues->ppsEquivalentAttributes = (DbvStringOrNumeric **) DbvAlloc (sizeof (DbvStringOrNumeric *) * (num_elements + 1));
		CheckDbvAlloc(psAttributeValues->ppsEquivalentAttributes, env);
		DBVsnacc_to_dbv_DbvStringOrNumeric_list(psAttributeValues->ppsEquivalentAttributes, data_ptr->equivalentAttributes, env);
	}
}



/* -------------------- DbvTermListInfo -------------------- */

int DBVsnacc_to_dbv_exp_termlistinfo(DbvTermListInfo *psTermListInfo, struct TermListInfo* termListInfo, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psTermListInfo->psCommonInfo, termListInfo->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psTermListInfo->sDatabaseName, &termListInfo->databaseName, env);


	/* c */
	if (termListInfo->termLists != NULL) {
		num_elements = AsnListCount(termListInfo->termLists);
       		psTermListInfo->ppsTermLists = (DbvTermList **) DbvAlloc (sizeof (DbvTermList *) * (num_elements + 1));
		CheckDbvAlloc(psTermListInfo->ppsTermLists, env);
		DBVsnacc_to_dbv_exp_termlist_list(psTermListInfo->ppsTermLists, termListInfo->termLists, env);
	}
		
}



int DBVsnacc_to_dbv_exp_termlist_list(DbvTermList **ppsTermLists, TermListInfoSeqOf *termLists, jmp_buf env)
{
	TermListInfoSeqOfSeq *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, termLists) {
		
		*ppsTermLists = (DbvTermList *) DbvAlloc (sizeof (DbvTermList));
		CheckDbvAlloc(*ppsTermLists, env);
		DBVsnacc_to_dbv_exp_termlist(*ppsTermLists, data_ptr, env);
		ppsTermLists++;
	}
	*ppsTermLists = NULL;
}



int DBVsnacc_to_dbv_exp_termlist(DbvTermList *psTermLists, TermListInfoSeqOfSeq *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psTermLists->sName, &data_ptr->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTermLists->ppsTitle, data_ptr->title, env);

	/* o */
	if (data_ptr->searchCost != NULL) {
		psTermLists->peSearchCost = (DbvSearchCost *) DbvAlloc (sizeof (DbvSearchCost));
		CheckDbvAlloc(psTermLists->peSearchCost, env);
		DBVsnacc_to_dbv_exp_searchcost(psTermLists->peSearchCost, data_ptr->searchCost, env);
	}

	/* c */
	DBVsnacc_to_dbv_bool(&psTermLists->fScanable, &data_ptr->scanable, env);

	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psTermLists->ppsBroader, data_ptr->broader, env);
	
	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psTermLists->ppsNarrower, data_ptr->narrower, env);
	
}



int DBVsnacc_to_dbv_exp_searchcost(DbvSearchCost *peSearchCost, TermListInfoSeqOfSeqInt *searchCost, jmp_buf env)
{
	switch (*searchCost) {
	case OPTIMIZED:
		*peSearchCost = SC_Optimized;
		break;	
	case NORMAL:
		*peSearchCost = SC_Normal;
		break;	
	case EXPENSIVE:
		*peSearchCost = SC_Expensive;
		break;	
	case FILTER:
		*peSearchCost = SC_Filter;
		break;	
	default:
		*peSearchCost = SC_Invalid;
		break;	
	}
	
}



/* -------------------- DbvExtendedServicesInfo -------------------- */

int DBVsnacc_to_dbv_exp_extservinfo(DbvExtendedServicesInfo *psExtendedServicesInfo, struct ExtendedServicesInfo* extendedServicesInfo, jmp_buf env)
{
	
	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psExtendedServicesInfo->psCommonInfo, extendedServicesInfo->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psExtendedServicesInfo->pcType, &extendedServicesInfo->type, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psExtendedServicesInfo->psName, &extendedServicesInfo->name, env);

	/* c */
	DBVsnacc_to_dbv_bool(&psExtendedServicesInfo->fPrivateType, &extendedServicesInfo->privateType, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psExtendedServicesInfo->fRestrictionsApply, &extendedServicesInfo->restrictionsApply, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psExtendedServicesInfo->fFeeApply, &extendedServicesInfo->feeApply, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psExtendedServicesInfo->fAvailable, &extendedServicesInfo->available, env);

	/* c */
	DBVsnacc_to_dbv_bool(&psExtendedServicesInfo->fRetentionSupported, &extendedServicesInfo->retentionSupported, env);

	/* c */
	DBVsnacc_to_dbv_infoint(&psExtendedServicesInfo->eWaitAction, &extendedServicesInfo->waitAction, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psExtendedServicesInfo->ppsDescription, extendedServicesInfo->description, env);

	/* o */
	DBVsnacc_to_dbv_external_opt(&psExtendedServicesInfo->psSpecificExplain, extendedServicesInfo->specificExplain, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psExtendedServicesInfo->psEsASN, &extendedServicesInfo->esASN, env);


}


int DBVsnacc_to_dbv_infoint(DbvExtendedServicesInfoInt *peWaitAction, ExtendedServicesInfoInt *waitAction, jmp_buf env)
{
	switch (*waitAction) {
	case WAITSUPPORTED:
		*peWaitAction = ESII_WaitSupported;
		break;

	case WAITALWAYS:
		*peWaitAction = ESII_WaitAlways;
		break;

	case WAITNOTSUPPORTED:
		*peWaitAction = ESII_WaitNotSupported;
		break;

	case DEPENDS:
		*peWaitAction = ESII_Depends;
		break;

	case NOTSAYING:
		*peWaitAction = ESII_NotSaying;
		break;
	default:
		*peWaitAction = ESII_Invalid;
		break;

	}
}



int DBVsnacc_to_dbv_external_opt(DbvASN1External **ppsSpecificExplain, struct NEWEXTERNAL *specificExplain, jmp_buf env)
{

	if (specificExplain != NULL) {
		*ppsSpecificExplain = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		CheckDbvAlloc(*ppsSpecificExplain, env);
		DBVsnacc_to_dbv_external(*ppsSpecificExplain, specificExplain, env);
	}
}

/* -------------------- DbvAttributeDetails -------------------- */

int DBVsnacc_to_dbv_exp_attdetails(DbvAttributeDetails *psAttributeDetails, struct AttributeDetails* attributeDetails, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psAttributeDetails->psCommonInfo, attributeDetails->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psAttributeDetails->sDatabaseName, &attributeDetails->databaseName, env);

	/* o */
	if (attributeDetails->attributesBySet != NULL) {
		num_elements = AsnListCount(attributeDetails->attributesBySet);
       		psAttributeDetails->ppsAttributesBySet = (DbvAttributesBySet **) DbvAlloc (sizeof (DbvAttributesBySet *) * (num_elements + 1));
		CheckDbvAlloc(psAttributeDetails->ppsAttributesBySet, env);
		DBVsnacc_to_dbv_exp_attsbyset_list(psAttributeDetails->ppsAttributesBySet, attributeDetails->attributesBySet, env);
	}

	/* o */
	DBVsnacc_to_dbv_attcombs_opt(&psAttributeDetails->psAttributeCombinations, attributeDetails->attributeCombinations, env);

	
}



int DBVsnacc_to_dbv_exp_attsbyset_list(DbvAttributesBySet **ppsAttributesBySet, AttributeDetailsSeqOf *attributesBySet, jmp_buf env)
{
	AttributeSetDetails *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, attributesBySet) {
		
		*ppsAttributesBySet = (DbvAttributesBySet *) DbvAlloc (sizeof (DbvAttributesBySet));
		CheckDbvAlloc(*ppsAttributesBySet, env);
		DBVsnacc_to_dbv_exp_attributebyset(*ppsAttributesBySet, data_ptr, env);
		ppsAttributesBySet++;
	}
	*ppsAttributesBySet = NULL;
}



int DBVsnacc_to_dbv_exp_attributebyset(DbvAttributesBySet *psAttributesBySet, AttributeSetDetails *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psAttributesBySet->pcAttributeSet, &data_ptr->attributeSet, env);

	/* c */
	num_elements = AsnListCount(data_ptr->attributesByType);
       	psAttributesBySet->ppsAttributesByType = (DbvAttributesByType **) DbvAlloc (sizeof (DbvAttributesByType *) * (num_elements + 1));
	CheckDbvAlloc(psAttributesBySet->ppsAttributesByType, env);
	DBVsnacc_to_dbv_exp_attsbytype_list(psAttributesBySet->ppsAttributesByType, data_ptr->attributesByType, env);

}



int DBVsnacc_to_dbv_exp_attsbytype_list(DbvAttributesByType **ppsAttributesByType, AttributeSetDetailsSeqOf *attributesByType, jmp_buf env)
{
	AttributeTypeDetails *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, attributesByType) {
		
		*ppsAttributesByType = (DbvAttributesByType *) DbvAlloc (sizeof (DbvAttributesByType));
		CheckDbvAlloc(*ppsAttributesByType, env);
		DBVsnacc_to_dbv_exp_attsbytype(*ppsAttributesByType, data_ptr, env);
		ppsAttributesByType++;
	}
	*ppsAttributesByType = NULL;
}



int DBVsnacc_to_dbv_exp_attsbytype(DbvAttributesByType *psAttributesByType, AttributeTypeDetails *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* c */
	psAttributesByType->ulAttributeType = data_ptr->attributeType;

	/* o */
	if (data_ptr->defaultIfOmitted != NULL) {
		psAttributesByType->psDefaultIfOmitted = (DbvOmittedAttributeInterpretation *) DbvAlloc (sizeof(DbvOmittedAttributeInterpretation));
		CheckDbvAlloc(psAttributesByType->psDefaultIfOmitted, env);
		DBVsnacc_to_dbv_exp_ommitattint(psAttributesByType->psDefaultIfOmitted, data_ptr->defaultIfOmitted, env);
	}
	
	/* o */
	if (data_ptr->attributeValues != NULL) {
		num_elements = AsnListCount(data_ptr->attributeValues);
       		psAttributesByType->ppsAttributeValues = (DbvAttributeValues **) DbvAlloc (sizeof (DbvAttributeValues *) * (num_elements + 1));
		CheckDbvAlloc(psAttributesByType->ppsAttributeValues, env);
		DBVsnacc_to_dbv_exp_attsvals_list(psAttributesByType->ppsAttributeValues, data_ptr->attributeValues, env);
	}
	
}



int DBVsnacc_to_dbv_exp_ommitattint(DbvOmittedAttributeInterpretation *psOptionalType, struct OmittedAttributeInterpretation* optionalType, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_strornum_opt(&psOptionalType->psDefaultValue, optionalType->defaultValue, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psOptionalType->ppsDefaultDescription, optionalType->defaultDescription, env);

}



int DBVsnacc_to_dbv_exp_attsvals_list(DbvAttributeValues **ppsAttributeValues, AttributeTypeDetailsSeqOf *attributeValues, jmp_buf env)
{
	AttributeValue *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, attributeValues) {
		
		*ppsAttributeValues = (DbvAttributeValues *) DbvAlloc (sizeof (DbvAttributeValues));
		CheckDbvAlloc(*ppsAttributeValues, env);
		DBVsnacc_to_dbv_exp_attsvals(*ppsAttributeValues, data_ptr, env);
		ppsAttributeValues++;
	}
	*ppsAttributeValues = NULL;
}



int DBVsnacc_to_dbv_exp_attsvals(DbvAttributeValues *psAttributeValues, AttributeValue *data_ptr, jmp_buf env)
{

	/* c */
	DBVsnacc_to_dbv_strornum(&psAttributeValues->sValue, data_ptr->value, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psAttributeValues->ppsDescription, data_ptr->description, env);

	/* o */
	DBVsnacc_to_dbv_strornum_list_opt(&psAttributeValues->ppsSubAttributes, data_ptr->subAttributes, env);

	/* o */
	DBVsnacc_to_dbv_strornum_list_opt(&psAttributeValues->ppsSuperAttributes, data_ptr->superAttributes, env);


	/* o */
	DBVsnacc_to_dbv_nullpointer_opt(&psAttributeValues->pvPartialSupport, data_ptr->partialSupport, env);


}



int DBVsnacc_to_dbv_strornum_list_opt(DbvStringOrNumeric ***pppsSubAttributes, AttributeValueSeqOf *subAttributes, jmp_buf env)
{
	int num_elements = 0;


	if  (subAttributes != NULL) {

		num_elements = AsnListCount(subAttributes);
       		*pppsSubAttributes = (DbvStringOrNumeric **) DbvAlloc (sizeof (DbvStringOrNumeric *) * (num_elements + 1));
		CheckDbvAlloc(*pppsSubAttributes, env);
		DBVsnacc_to_dbv_exp_strornum_list(*pppsSubAttributes, subAttributes, env);
	}
}



int DBVsnacc_to_dbv_exp_strornum_list(DbvStringOrNumeric **ppsSubAttributes, AttributeValueSeqOf *subAttributes, jmp_buf env)
{
	StringOrNumeric *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, subAttributes) {
		
		*ppsSubAttributes = (DbvStringOrNumeric *) DbvAlloc (sizeof (DbvStringOrNumeric));
		CheckDbvAlloc(*ppsSubAttributes, env);
		DBVsnacc_to_dbv_strornum(*ppsSubAttributes, data_ptr, env);
		ppsSubAttributes++;
	}
	*ppsSubAttributes = NULL;
}



int DBVsnacc_to_dbv_attcombs_opt(DbvAttributeCombinations **ppsAttributeCombinations, struct AttributeCombinations *attributeCombinations, jmp_buf env)
{


	if (attributeCombinations != NULL) {
       		*ppsAttributeCombinations = (DbvAttributeCombinations *) DbvAlloc (sizeof (DbvAttributeCombinations));
		CheckDbvAlloc(*ppsAttributeCombinations, env);
		DBVsnacc_to_dbv_exp_attcombs(*ppsAttributeCombinations, attributeCombinations, env);
	}
}



int DBVsnacc_to_dbv_exp_attcombs(DbvAttributeCombinations *psAttributeCombinations, struct AttributeCombinations* attributeCombinations, jmp_buf env)
{
	int num_elements = 0;

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psAttributeCombinations->pcDefaultAttributeSet, &attributeCombinations->defaultAttributeSet, env);
	
	
	/* c */
	num_elements = AsnListCount(attributeCombinations->legalCombinations);
       	psAttributeCombinations->ppsLegalCombinations = (DbvAttributeCombination **) DbvAlloc (sizeof (DbvAttributeCombination *) * (num_elements + 1));
	CheckDbvAlloc(psAttributeCombinations->ppsLegalCombinations, env);
	DBVsnacc_to_dbv_exp_attcomb_nos_list(psAttributeCombinations->ppsLegalCombinations, attributeCombinations->legalCombinations, env);
	
}



int DBVsnacc_to_dbv_exp_attcomb_nos_list(DbvAttributeCombination **ppsLegalCombinations, AttributeCombinationsSeqOf *legalCombinations, jmp_buf env)
{
	AttributeCombination *data_ptr;
	int num_elements = 0;

	FOR_EACH_LIST_ELMT(data_ptr, legalCombinations) {
		
		*ppsLegalCombinations = (DbvAttributeCombination *) DbvAlloc (sizeof (DbvAttributeCombination));
		CheckDbvAlloc(*ppsLegalCombinations, env);

		DBVsnacc_to_dbv_attoccur_list(*ppsLegalCombinations, data_ptr, env);
		ppsLegalCombinations++;
	}
	*ppsLegalCombinations = NULL;
}


/* No direct mapping here so be careful */
int DBVsnacc_to_dbv_attoccur_list(DbvAttributeCombination *psLegalCombinations, AttributeCombination *attcomlist, jmp_buf env)
{
	int index = 0;
	int num_elements = 0;

	AttributeOccurrence *data_ptr;

	num_elements = AsnListCount(attcomlist);
       	psLegalCombinations->ppsAttributeOccurrence = (DbvAttributeOccurrence **) DbvAlloc (sizeof (DbvAttributeOccurrence *) * (num_elements + 1));
	CheckDbvAlloc(psLegalCombinations->ppsAttributeOccurrence, env);

	FOR_EACH_LIST_ELMT(data_ptr, attcomlist) {
		
		psLegalCombinations->ppsAttributeOccurrence[index] = (DbvAttributeOccurrence *) DbvAlloc (sizeof (DbvAttributeOccurrence));
		CheckDbvAlloc(psLegalCombinations->ppsAttributeOccurrence[index], env);
		DBVsnacc_to_dbv_attoccur(psLegalCombinations->ppsAttributeOccurrence[index], data_ptr, env);
		index++;
	}
	psLegalCombinations->ppsAttributeOccurrence[index] = NULL;

}



int DBVsnacc_to_dbv_attoccur(DbvAttributeOccurrence *psAttributeOccurrence, AttributeOccurrence *data_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_objectident_opt(&psAttributeOccurrence->pcAttributeSet, &data_ptr->attributeSet, env);

	/* c */
	psAttributeOccurrence->ulAttributeType = data_ptr->attributeType;

	/* o */
	DBVsnacc_to_dbv_nullpointer_opt(&psAttributeOccurrence->pvMustBeSupplied, data_ptr->mustBeSupplied, env);

	/* c */
	DBVsnacc_to_dbv_attoccurchoice(&psAttributeOccurrence->sAttributeValues, data_ptr->attributeValues, env);
	
}



int DBVsnacc_to_dbv_attoccurchoice(DbvAttributeOccurrenceChoice *psAttributeValues, struct AttributeOccurrenceChoice* attributeValues, jmp_buf env)
{
	switch (attributeValues->choiceId) {
	case ATTRIBUTEOCCURRENCECHOICE_ANY_OR_NONE:
		psAttributeValues->eAttributeOccurrenceChoiceType = AOT_AnyOrNone;

		psAttributeValues->uAttributeOccurrenceChoiceType.pvAnyOrNone = (DbvNull) DbvAlloc (1);
		CheckDbvAlloc(psAttributeValues->uAttributeOccurrenceChoiceType.pvAnyOrNone, env);
		break;

	case ATTRIBUTEOCCURRENCECHOICE_SPECIFIC:
		psAttributeValues->eAttributeOccurrenceChoiceType = AOT_Specific;

		/* Just use opty */
		DBVsnacc_to_dbv_strornum_list_opt(&psAttributeValues->uAttributeOccurrenceChoiceType.ppsSpecific, attributeValues->a.specific, env);
		break;
	default:
		psAttributeValues->eAttributeOccurrenceChoiceType = AOT_Invalid;
		break;

	}
}



/* -------------------- DbvTermListDetails -------------------- */

int DBVsnacc_to_dbv_exp_termlistdetails(DbvTermListDetails *psTermListDetails, struct TermListDetails* termListDetails, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psTermListDetails->psCommonInfo, termListDetails->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psTermListDetails->sTermListName, &termListDetails->termListName, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psTermListDetails->ppsDescription, termListDetails->description, env);

	/* o */
	DBVsnacc_to_dbv_attcombs_opt(&psTermListDetails->psAttributes, termListDetails->attributes, env);

	/* o */
	if (termListDetails->scanInfo != NULL) {
		psTermListDetails->psScanInfo = (DbvScanInfo *) DbvAlloc (sizeof (DbvScanInfo));
		CheckDbvAlloc(psTermListDetails->psScanInfo, env);
		DBVsnacc_to_dbv_scaninfo(psTermListDetails->psScanInfo, termListDetails->scanInfo, env);
	}


	/* o */
	DBVsnacc_to_dbv_int_opt(&psTermListDetails->pulEstNumberTerms, termListDetails->estNumberTerms, env);

	/* o */
	if (termListDetails->sampleTerms != NULL) {
		num_elements = AsnListCount(termListDetails->sampleTerms);
       		psTermListDetails->ppsSampleTerms = (DbvTerm **) DbvAlloc (sizeof (DbvTerm *) * (num_elements + 1));
		CheckDbvAlloc(psTermListDetails->ppsSampleTerms, env);
		DBVsnacc_to_dbv_exp_term_list(psTermListDetails->ppsSampleTerms, termListDetails->sampleTerms, env);
	}

		
}


int DBVsnacc_to_dbv_scaninfo(DbvScanInfo *psScanInfo, struct TermListDetailsSeq* scanInfo, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_int_opt(&psScanInfo->pulMaxStepSize, scanInfo->maxStepSize, env);
	
	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psScanInfo->ppsCollatingSequence, scanInfo->collatingSequence, env);

	/* o */
	DBVsnacc_to_dbv_bool_opt(&psScanInfo->pfIncreasing, scanInfo->increasing, env);

}


int DBVsnacc_to_dbv_exp_term_list(DbvTerm **ppsSampleTerms, TermListDetailsSeqOf *sampleTerms, jmp_buf env)
{
	int index = 0;

	Term *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, sampleTerms) {
		
		ppsSampleTerms[index] = (DbvTerm *) DbvAlloc (sizeof (DbvTerm));
		CheckDbvAlloc(ppsSampleTerms[index], env);
		DBVsnacc_to_dbv_term(ppsSampleTerms[index], data_ptr, env);
		index++;
	}
	ppsSampleTerms[index] = NULL;
}



/* -------------------- DbvElementSetDetails -------------------- */

int DBVsnacc_to_dbv_exp_elsetdetails(DbvElementSetDetails *psElementSetDetails, struct ElementSetDetails* elementSetDetails, jmp_buf env)
{

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psElementSetDetails->psCommonInfo, elementSetDetails->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psElementSetDetails->sDatabaseName, &elementSetDetails->databaseName, env);


	/* c */
	DBVsnacc_to_dbv_inter_string(&psElementSetDetails->sElementSetName, &elementSetDetails->elementSetName, env);

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psElementSetDetails->pcRecordSyntax, &elementSetDetails->recordSyntax, env);

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psElementSetDetails->pcSchema, &elementSetDetails->schema, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psElementSetDetails->ppsDescription, elementSetDetails->description, env);

	/* o */
	DBVsnacc_to_dbv_elsetde_list_opt(&psElementSetDetails->ppsDetailsPerElement, elementSetDetails->detailsPerElement, env);
}


int DBVsnacc_to_dbv_elsetde_list_opt(DbvDetailsPerElement ***pppsDetailsPerElement, RetrievalRecordDetailsSeqOf *detailsPerElement, jmp_buf env)
{
	int num_elements = 0;

	if (detailsPerElement != NULL) {
		num_elements = AsnListCount(detailsPerElement);
       		*pppsDetailsPerElement = (DbvDetailsPerElement **) DbvAlloc (sizeof (DbvDetailsPerElement *) * (num_elements + 1));
		CheckDbvAlloc(*pppsDetailsPerElement, env);
		DBVsnacc_to_dbv_exp_elsetde_list(*pppsDetailsPerElement, detailsPerElement, env);
	}
}


int DBVsnacc_to_dbv_exp_elsetde_list(DbvDetailsPerElement **ppsDetailsPerElement, ElementSetDetailsSeqOf *detailsPerElement, jmp_buf env)
{
	int index = 0;

	PerElementDetails *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, detailsPerElement) {
		
		ppsDetailsPerElement[index] = (DbvDetailsPerElement *) DbvAlloc (sizeof (DbvDetailsPerElement));
		CheckDbvAlloc(ppsDetailsPerElement[index], env);
		DBVsnacc_to_dbv_exp_elsetde(ppsDetailsPerElement[index], data_ptr, env);
		index++;
	}
	ppsDetailsPerElement[index] = NULL;
}



int DBVsnacc_to_dbv_exp_elsetde(DbvDetailsPerElement *psDetailsPerElement, PerElementDetails *data_ptr, jmp_buf env)
{
	int num_elements = 0;


	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psDetailsPerElement->psName, &data_ptr->name, env);

	/* o */	
	if (data_ptr->recordTag != NULL) {
		psDetailsPerElement->psRecordTag = (DbvRecordTag *) DbvAlloc (sizeof (DbvRecordTag));
		CheckDbvAlloc(psDetailsPerElement->psRecordTag, env);
		DBVsnacc_to_dbv_exp_rectag(psDetailsPerElement->psRecordTag, data_ptr->recordTag, env);
	}
		

	/* o */
	if (data_ptr->schemaTags != NULL) {
		num_elements = AsnListCount(data_ptr->schemaTags);
       		psDetailsPerElement->ppsSchemaTags = (DbvPathA **) DbvAlloc (sizeof (DbvPathA *) * (num_elements + 1));
		CheckDbvAlloc(psDetailsPerElement->ppsSchemaTags, env);
		DBVsnacc_to_dbv_exp_patha_list(psDetailsPerElement->ppsSchemaTags, data_ptr->schemaTags, env);
	}

	
	/* o */
	DBVsnacc_to_dbv_int_opt(&psDetailsPerElement->pulMaxSize, data_ptr->maxSize, env);

	/* o */
	DBVsnacc_to_dbv_int_opt(&psDetailsPerElement->pulMinSize, data_ptr->minSize, env);

	/* o */
	DBVsnacc_to_dbv_int_opt(&psDetailsPerElement->pulAvgSize, data_ptr->avgSize, env);

	/* o */
	DBVsnacc_to_dbv_int_opt(&psDetailsPerElement->pulFixedSize, data_ptr->fixedSize, env);


	/* c */
	DBVsnacc_to_dbv_bool(&psDetailsPerElement->fRepeatable, &data_ptr->repeatable, env);

	/* c */
	DBVsnacc_to_dbv_bool(&psDetailsPerElement->fRequired, &data_ptr->required, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDetailsPerElement->ppsDescription, data_ptr->description, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDetailsPerElement->ppsContents, data_ptr->contents, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDetailsPerElement->ppsBillingInfo, data_ptr->billingInfo, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psDetailsPerElement->ppsRestrictions, data_ptr->restrictions, env);

	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psDetailsPerElement->ppsAlternateNames, data_ptr->alternateNames, env);

	/* o */
	DBVsnacc_to_dbv_interstring_list_opt(&psDetailsPerElement->ppsGenericNames, data_ptr->genericNames, env);

	/* o */
	DBVsnacc_to_dbv_attcombs_opt(&psDetailsPerElement->psSearchAccess, data_ptr->searchAccess, env);

}



int DBVsnacc_to_dbv_exp_rectag(DbvRecordTag *psRecordTag, struct RecordTag* recordTag, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_strornum_opt(&psRecordTag->psQualifier, recordTag->qualifier, env);
	
	/* c */
	DBVsnacc_to_dbv_strornum(&psRecordTag->sTagValue, recordTag->tagValue, env);
}


int DBVsnacc_to_dbv_exp_patha_list(DbvPathA **ppsSchemaTags, PerElementDetailsSeqOf *schemaTags, jmp_buf env)
{
	Path *data_ptr;
	int num_elements = 0;
	int index = 0;

	FOR_EACH_LIST_ELMT(data_ptr, schemaTags) {
		
		ppsSchemaTags[index] = (DbvPathA *) DbvAlloc (sizeof (DbvPathA));
		CheckDbvAlloc(ppsSchemaTags[index], env);

		num_elements = AsnListCount(data_ptr);
       		ppsSchemaTags[index]->ppsSchemaTags = (DbvPath **) DbvAlloc (sizeof (DbvPath *) * (num_elements + 1));
		CheckDbvAlloc(ppsSchemaTags[index]->ppsSchemaTags, env);
		DBVsnacc_to_dbv_exp_path_list(ppsSchemaTags[index]->ppsSchemaTags, data_ptr, env);
		index++;
	}
	ppsSchemaTags[index] = NULL;
}


/* -------------------- DbvRetrievalRecordDetails -------------------- */

int DBVsnacc_to_dbv_exp_retrecdetails(DbvRetrievalRecordDetails *psRetrievalRecordDetails, struct RetrievalRecordDetails* retrievalRecordDetails, jmp_buf env)
{

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psRetrievalRecordDetails->psCommonInfo, retrievalRecordDetails->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psRetrievalRecordDetails->sDatabaseName, &retrievalRecordDetails->databaseName, env);

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psRetrievalRecordDetails->pcSchema, &retrievalRecordDetails->schema, env);

	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psRetrievalRecordDetails->pcRecordSyntax, &retrievalRecordDetails->recordSyntax, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psRetrievalRecordDetails->ppsDescription, retrievalRecordDetails->description, env);

	/* o */
	DBVsnacc_to_dbv_elsetde_list_opt(&psRetrievalRecordDetails->ppsDetailsPerElement, retrievalRecordDetails->detailsPerElement, env);

}



/* -------------------- DbvSortDetails -------------------- */

int DBVsnacc_to_dbv_exp_sortdetails(DbvSortDetails *psSortDetails, struct SortDetails* sortDetails, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psSortDetails->psCommonInfo, sortDetails->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psSortDetails->sDatabaseName, &sortDetails->databaseName, env);

	/* o */
	if (sortDetails->sortKeys != NULL) {
		num_elements = AsnListCount(sortDetails->sortKeys);
       		psSortDetails->ppsSortKeys = (DbvSortKeys **) DbvAlloc (sizeof (DbvSortKeys *) * (num_elements + 1));
		CheckDbvAlloc(psSortDetails->ppsSortKeys, env);
		DBVsnacc_to_dbv_exp_sortkeys_list(psSortDetails->ppsSortKeys, sortDetails->sortKeys, env);
	}
}



int DBVsnacc_to_dbv_exp_sortkeys_list(DbvSortKeys **ppsSortKeys, SortDetailsSeqOf *sortKeys, jmp_buf env)
{
	int index = 0;

	SortKeyDetails *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, sortKeys) {
		
		ppsSortKeys[index] = (DbvSortKeys *) DbvAlloc (sizeof (DbvSortKeys));
		CheckDbvAlloc(ppsSortKeys[index], env);
		DBVsnacc_to_dbv_exp_sortkeys(ppsSortKeys[index], data_ptr, env);
		index++;
	}
	ppsSortKeys[index] = NULL;
}



int DBVsnacc_to_dbv_exp_sortkeys(DbvSortKeys *psSortKeys, SortKeyDetails *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psSortKeys->ppsDescription, data_ptr->description, env);

	/* o */
	if (data_ptr->elementSpecifications != NULL) {
		num_elements = AsnListCount(data_ptr->elementSpecifications);
       		psSortKeys->ppsElementSpecifications = (DbvSpecification **) DbvAlloc (sizeof (DbvSpecification *) * (num_elements + 1));
		CheckDbvAlloc(psSortKeys->ppsElementSpecifications, env);
		DBVsnacc_to_dbv_exp_specification_list(psSortKeys->ppsElementSpecifications, data_ptr->elementSpecifications, env);
	}
	
	/* o */
	DBVsnacc_to_dbv_attcombs_opt(&psSortKeys->psAttributeSpecifications, data_ptr->attributeSpecifications, env);

	/* o */
	if (data_ptr->sortType != NULL) {
		psSortKeys->psSortType = (DbvSortType *) DbvAlloc (sizeof (DbvSortType));
		CheckDbvAlloc(psSortKeys->psSortType, env);
		DBVsnacc_to_dbv_exp_sorttype(psSortKeys->psSortType, data_ptr->sortType, env);
	}


	/* o */
	if (data_ptr->caseSensitivity != NULL) {
		psSortKeys->peCaseSensitivity = (DbvCaseSensitivity *) DbvAlloc (sizeof (DbvCaseSensitivity));
		CheckDbvAlloc(psSortKeys->peCaseSensitivity, env);
		DBVsnacc_to_dbv_exp_casesen(psSortKeys->peCaseSensitivity, data_ptr->caseSensitivity, env);
	}
		

}



int  DBVsnacc_to_dbv_exp_specification_list(DbvSpecification **ppsElementSpecifications, SortKeyDetailsSeqOf *elementSpecifications, jmp_buf env)
{
	int index = 0;

	Specification *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, elementSpecifications) {
		
		ppsElementSpecifications[index] = (DbvSpecification *) DbvAlloc (sizeof (DbvSpecification));
		CheckDbvAlloc(ppsElementSpecifications[index], env);
		DBVsnacc_to_dbv_specification(ppsElementSpecifications[index], data_ptr, env);
		index++;
	}
	ppsElementSpecifications[index] = NULL;

}


int DBVsnacc_to_dbv_exp_sorttype(DbvSortType *psSortType, struct SortKeyDetailsChoice* sortType, jmp_buf env)
{
	switch (sortType->choiceId) {
	case SORTKEYDETAILSCHOICE_CHARACTER:
		psSortType->eSortTypeType = STT_Character;
		break;

	case SORTKEYDETAILSCHOICE_NUMERIC:
		psSortType->eSortTypeType = STT_Numeric;
		break;

	case SORTKEYDETAILSCHOICE_STRUCTURED:
		psSortType->eSortTypeType = STT_Structured;

		/* Use o set up */
		DBVsnacc_to_dbv_humanstring_list_opt(&psSortType->uSortType.ppsStructured, sortType->a.structured, env);
	
		break;

	default:
		psSortType->eSortTypeType = STT_Invalid;
		break;

	}
}



int DBVsnacc_to_dbv_exp_casesen(DbvCaseSensitivity *peCaseSensitivity, SortKeyDetailsInt *caseSensitivity, jmp_buf env)
{
	switch (*caseSensitivity) {
	case ALWAYS:
		*peCaseSensitivity = CSE_Always;
		break;
	case NEVER:
		*peCaseSensitivity = CSE_Never;
		break;
	case DEFAULT_YES:
		*peCaseSensitivity = CSE_DefaultYes;
		break;
	case DEFAULT_NO:
		*peCaseSensitivity = CSE_DefaultNo;
		break;
	case CSE_Invalid:
		*peCaseSensitivity = CSE_Invalid;
		break;
	}
}



/* -------------------- DbvProcessingInformation -------------------- */

int DBVsnacc_to_dbv_exp_processinginfo(DbvProcessingInformation *psProcessing, struct ProcessingInformation* processing, jmp_buf env)
{

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psProcessing->psCommonInfo, processing->commonInfo, env);
	
	/* c */
	DBVsnacc_to_dbv_inter_string(&psProcessing->sDatabaseName, &processing->databaseName, env);

	/* c */
	DBVsnacc_to_dbv_exp_processcontext(&psProcessing->eProcessingContext, &processing->processingContext, env);


	/* c */
	DBVsnacc_to_dbv_inter_string(&psProcessing->sName, &processing->name, env);


	/* c */
	DBVsnacc_to_dbv_objectident_comp(&psProcessing->pcOid, &processing->oid, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psProcessing->ppsDescription, processing->description, env);


	/* o */
	DBVsnacc_to_dbv_external_opt(&psProcessing->psInstructions, processing->instructions, env);
}



int DBVsnacc_to_dbv_exp_processcontext(DbvProcessingContext *peProcessingContext, ProcessingInformationInt *processingContext, jmp_buf env)
{
	switch (*processingContext) {
	case ACCESS:
		*peProcessingContext = PC_Access;
		break;

	case SEARCH2:
		*peProcessingContext = PC_Search;
		break;

	case RETRIEVAL:
		*peProcessingContext = PC_Retrieval;
		break;

	case RECORD_PRESENTATION:
		*peProcessingContext = PC_RecordPresentation;
		break;

	case RECORD_HANDLING:
		*peProcessingContext = PC_RecordHandling;
		break;

	default:
		*peProcessingContext = PC_Invalid;
		break;

	}
}



/* -------------------- DbvVariantSetInfo -------------------- */

int DBVsnacc_to_dbv_exp_variantsetinfo(DbvVariantSetInfo *psVariants, struct VariantSetInfo* variants, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psVariants->psCommonInfo, variants->commonInfo, env);
	
	/* c */ 
	DBVsnacc_to_dbv_objectident_comp(&psVariants->pcVariantSet, &variants->variantSet, env);

	/* c */
	DBVsnacc_to_dbv_inter_string(&psVariants->sName, &variants->name, env);

	/* o */
	if (variants->variants != NULL) {
		num_elements = AsnListCount(variants->variants);
       		psVariants->ppsVariants = (DbvVariantClass **) DbvAlloc (sizeof (DbvVariantClass *) * (num_elements + 1));
		CheckDbvAlloc(psVariants->ppsVariants, env);
		DBVsnacc_to_dbv_exp_variantclass_list(psVariants->ppsVariants, variants->variants, env);
	}

}


int DBVsnacc_to_dbv_exp_variantclass_list(DbvVariantClass **ppsVariants, VariantSetInfoSeqOf *variants, jmp_buf env)
{
	int index = 0;

	VariantClass *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, variants) {
		
		ppsVariants[index] = (DbvVariantClass *) DbvAlloc (sizeof (DbvVariantClass));
		CheckDbvAlloc(ppsVariants[index], env);
		DBVsnacc_to_dbv_variantclass(ppsVariants[index], data_ptr, env);
		index++;
	}
	ppsVariants[index] = NULL;

}



int DBVsnacc_to_dbv_variantclass(DbvVariantClass *psVariants, VariantClass *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psVariants->psName, &data_ptr->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psVariants->ppsDescription, data_ptr->description, env);

	/* c */
	psVariants->ulVariantClass = data_ptr->variantClass;

	/* c */
	num_elements = AsnListCount(data_ptr->variantTypes);
       	psVariants->ppsVariantTypes = (DbvVariantTypes **) DbvAlloc (sizeof (DbvVariantTypes *) * (num_elements + 1));
	CheckDbvAlloc(psVariants->ppsVariantTypes, env);
	DBVsnacc_to_dbv_exp_varianttypes_list(psVariants->ppsVariantTypes, data_ptr->variantTypes, env);
	

}


int DBVsnacc_to_dbv_exp_varianttypes_list(DbvVariantTypes **ppsVariantTypes, VariantClassSeqOf *variantTypes, jmp_buf env)
{
	int index = 0;

	VariantType *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, variantTypes) {
		
		ppsVariantTypes[index] = (DbvVariantTypes *) DbvAlloc (sizeof (DbvVariantTypes));
		CheckDbvAlloc(ppsVariantTypes[index], env);
		DBVsnacc_to_dbv_varianttypes(ppsVariantTypes[index], data_ptr, env);
		index++;
	}
	ppsVariantTypes[index] = NULL;

}



int DBVsnacc_to_dbv_varianttypes(DbvVariantTypes *psVariantTypes, VariantType *data_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psVariantTypes->psName, &data_ptr->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psVariantTypes->ppsDescription, data_ptr->description, env);

	/* c */
	psVariantTypes->ulVariantType = data_ptr->variantType;

	/* o */
	if (data_ptr->variantValue != NULL) {
		psVariantTypes->psVariantValue = (DbvVariantValue *) DbvAlloc (sizeof (DbvVariantValue));
		CheckDbvAlloc(psVariantTypes->psVariantValue, env);
		DBVsnacc_to_dbv_exp_variantvalue(psVariantTypes->psVariantValue, data_ptr->variantValue, env);
	}

}



int DBVsnacc_to_dbv_exp_variantvalue(DbvVariantValue *psVariantValue, struct VariantValue* variantValue, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_exp_primitive(&psVariantValue->eDataType, &variantValue->dataType, env);
	
	/* o */	
	if (variantValue->values != NULL) {
		psVariantValue->psValues = (DbvValueSet *) DbvAlloc (sizeof (DbvValueSet));
		CheckDbvAlloc(psVariantValue->psValues, env);
		DBVsnacc_to_dbv_exp_valueset(psVariantValue->psValues, variantValue->values, env);
	}
}



int DBVsnacc_to_dbv_exp_valueset(DbvValueSet *psValues, struct ValueSet* values, jmp_buf env)
{
	int num_elements = 0;

	switch (values->choiceId) {
	case VALUESET_RANGE:
		psValues->eValueSetType = VT_Range;
		psValues->uValueSet.psRange = (DbvValueRange *) DbvAlloc (sizeof (DbvValueRange));
		CheckDbvAlloc(psValues->uValueSet.psRange, env);
		DBVsnacc_to_dbv_exp_valuerange(psValues->uValueSet.psRange, values->a.range, env);

		break;
	case VALUESET_ENUMERATED:
		psValues->eValueSetType = VT_Enumerated;

		num_elements = AsnListCount(values->a.enumerated);
       		psValues->uValueSet.ppsEnumerated = (DbvValueDescription **) DbvAlloc (sizeof (DbvValueDescription *) * (num_elements + 1));
		CheckDbvAlloc(psValues->uValueSet.ppsEnumerated, env);
		DBVsnacc_to_dbv_exp_valuedescript_list(psValues->uValueSet.ppsEnumerated, values->a.enumerated, env);
	
		break;
	default:
		psValues->eValueSetType = VT_Invalid;
		break;
	}
}



int DBVsnacc_to_dbv_exp_valuerange(DbvValueRange *psRange, struct ValueRange *range, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_exp_valuedescript_opt(&psRange->psLower, range->lower, env);


	/* o */
	DBVsnacc_to_dbv_exp_valuedescript_opt(&psRange->psUpper, range->upper, env);
	
}



int DBVsnacc_to_dbv_exp_valuedescript_opt(DbvValueDescription **ppsLower, struct ValueDescription *lower, jmp_buf env)
{
	
	if (lower != NULL) {
		*ppsLower = (DbvValueDescription *) DbvAlloc (sizeof (DbvValueDescription));
		CheckDbvAlloc(*ppsLower, env);
		DBVsnacc_to_dbv_exp_valuedescript(*ppsLower, lower, env);

	}
}




int DBVsnacc_to_dbv_exp_valuedescript(DbvValueDescription *psLower, struct ValueDescription *lower, jmp_buf env)
{
	switch (lower->choiceId) {
	case VALUEDESCRIPTION_INTEGER:
		psLower->eValueDescriptionType = VDT_Integer;
		psLower->uValueDescription.ulInteger = lower->a.integer;
		break;
	case VALUEDESCRIPTION_STRING:
		psLower->eValueDescriptionType = VDT_String;

		/* Why write a new _comp if an existing _opt will do ? */
		DBVsnacc_to_dbv_interstring_opt(&psLower->uValueDescription.psString, lower->a.string, env);
		break;
	case VALUEDESCRIPTION_OCTETS:
		psLower->eValueDescriptionType = VDT_Octets;
		DBVsnacc_to_dbv_octetstring_comp(&psLower->uValueDescription.psOctets, lower->a.octets, env);
		break;
	case VALUEDESCRIPTION_OID_SNACC:
		psLower->eValueDescriptionType = VDT_Oid;
		DBVsnacc_to_dbv_objectident_comp(&psLower->uValueDescription.pcOid, lower->a.oid, env);
		break;
	case VALUEDESCRIPTION_UNIT:
		psLower->eValueDescriptionType = VDT_Unit;
		DBVsnacc_to_dbv_unit_opt(&psLower->uValueDescription.psUnit, lower->a.unit, env);
		break;
	case VALUEDESCRIPTION_VALUEANDUNIT:
		psLower->eValueDescriptionType = VDT_ValueAndUnit;
		DBVsnacc_to_dbv_intunit_opt(&psLower->uValueDescription.psValueAndUnit, lower->a.valueAndUnit, env);
		break;
	default:
		psLower->eValueDescriptionType = VDT_Invalid;
		break;
	}
}



int DBVsnacc_to_dbv_exp_valuedescript_list(DbvValueDescription **ppsEnumerated, ValueSetSeqOf *enumerated, jmp_buf env)
{
	int index = 0;

	ValueDescription *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, enumerated) {
		
		ppsEnumerated[index] = (DbvValueDescription *) DbvAlloc (sizeof (DbvValueDescription));
		CheckDbvAlloc(ppsEnumerated[index], env);
		DBVsnacc_to_dbv_exp_valuedescript(ppsEnumerated[index], data_ptr, env);
		index++;
	}
	ppsEnumerated[index] = NULL;

}



/* -------------------- DbvUnitInfo -------------------- */

int DBVsnacc_to_dbv_exp_unitinfo(DbvUnitInfo *psUnits, struct UnitInfo* units, jmp_buf env)
{
	int num_elements = 0;

	
	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psUnits->psCommonInfo, units->commonInfo, env);
	
	/* c */ 
	DBVsnacc_to_dbv_inter_string(&psUnits->sUnitSystem, &units->unitSystem, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psUnits->ppsDescription, units->description, env);

	/* o */
	if (units->units != NULL) {
		num_elements = AsnListCount(units->units);
       		psUnits->ppsUnitTypes = (DbvUnitType **) DbvAlloc (sizeof (DbvUnitType *) * (num_elements + 1));
		CheckDbvAlloc(psUnits->ppsUnitTypes, env);
		DBVsnacc_to_dbv_exp_unittypes_list(psUnits->ppsUnitTypes, units->units, env);
	}
}
	



int DBVsnacc_to_dbv_exp_unittypes_list(DbvUnitType **ppsUnitTypes, UnitInfoSeqOf *units, jmp_buf env)
{
	int index = 0;

	UnitType *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, units) {
		
		ppsUnitTypes[index] = (DbvUnitType *) DbvAlloc (sizeof (DbvUnitType));
		CheckDbvAlloc(ppsUnitTypes[index], env);
		DBVsnacc_to_dbv_exp_unittypes(ppsUnitTypes[index], data_ptr, env);
		index++;
	}
	ppsUnitTypes[index] = NULL;

}



int DBVsnacc_to_dbv_exp_unittypes(DbvUnitType *psUnitTypes, UnitType *data_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* o */ 
	DBVsnacc_to_dbv_interstring_opt(&psUnitTypes->psName, &data_ptr->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psUnitTypes->ppsDescription, data_ptr->description, env);

	/* o */
	DBVsnacc_to_dbv_strornum(&psUnitTypes->sUnitType, data_ptr->unitType, env);

	/* c */
	num_elements = AsnListCount(data_ptr->units);
       	psUnitTypes->ppsUnits = (DbvUnits **) DbvAlloc (sizeof (DbvUnits *) * (num_elements + 1));
	CheckDbvAlloc(psUnitTypes->ppsUnits, env);
	DBVsnacc_to_dbv_exp_units_list(psUnitTypes->ppsUnits, data_ptr->units, env);
}


int DBVsnacc_to_dbv_exp_units_list(DbvUnits **ppsUnits, UnitTypeSeqOf *units, jmp_buf env)
{
	int index = 0;

	Units *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, units) {
		
		ppsUnits[index] = (DbvUnits *) DbvAlloc (sizeof (DbvUnits));
		CheckDbvAlloc(ppsUnits[index], env);
		DBVsnacc_to_dbv_exp_units(ppsUnits[index], data_ptr, env);
		index++;
	}
	ppsUnits[index] = NULL;
}


int DBVsnacc_to_dbv_exp_units(DbvUnits *psUnits, Units *data_ptr, jmp_buf env)
{
	/* o */ 
	DBVsnacc_to_dbv_interstring_opt(&psUnits->psName, &data_ptr->name, env);

	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psUnits->ppsDescription, data_ptr->description, env);

	/* c */
	DBVsnacc_to_dbv_strornum(&psUnits->sUnit, data_ptr->unit, env);

}



/* -------------------- DbvCategoryList -------------------- */

int DBVsnacc_to_dbv_exp_categorylist(DbvCategoryList *psCategoryList, struct CategoryList* categoryList, jmp_buf env)
{
	int num_elements = 0;

	/* o */
	DBVsnacc_to_dbv_exp_commoninfo_opt(&psCategoryList->psCommonInfo, categoryList->commonInfo, env);
	
	/* c */ 
	num_elements = AsnListCount(categoryList->categories);
       	psCategoryList->ppsCategories = (DbvCategoryInfo **) DbvAlloc (sizeof (DbvCategoryInfo *) * (num_elements + 1));
	CheckDbvAlloc(psCategoryList->ppsCategories, env);
	DBVsnacc_to_dbv_exp_categoryinfo_list(psCategoryList->ppsCategories, categoryList->categories, env);
}



int DBVsnacc_to_dbv_exp_categoryinfo_list(DbvCategoryInfo **ppsCategories, CategoryListSeqOf *categories, jmp_buf env)
{
	int index = 0;

	CategoryInfo *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, categories) {
		
		ppsCategories[index] = (DbvCategoryInfo *) DbvAlloc (sizeof (DbvCategoryInfo));
		CheckDbvAlloc(ppsCategories[index], env);
		DBVsnacc_to_dbv_exp_categoryinfo(ppsCategories[index], data_ptr, env);
		index++;
	}
	ppsCategories[index] = NULL;
}



int DBVsnacc_to_dbv_exp_categoryinfo(DbvCategoryInfo *psCategories, CategoryInfo *data_ptr, jmp_buf env)
{
	/* c */
	DBVsnacc_to_dbv_inter_string(&psCategories->sCategory, &data_ptr->category, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psCategories->psOriginalCategory, &data_ptr->originalCategory, env);


	/* o */
	DBVsnacc_to_dbv_humanstring_list_opt(&psCategories->ppsDescription, data_ptr->description, env);

	/* o */
	DBVsnacc_to_dbv_interstring_opt(&psCategories->psAsn1Module, &data_ptr->asn1Module, env);
}




/*
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################
*/





/* Generic Record */


/* dbv -> snacc ansi GenericRecord external */

int DBVdbv_to_snacc_generic_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	/* Arrays of externals are used because snacc only generates code
	to free the contents of AnyDefinedBy types, not the types themselves */
	static GenericRecord generic_external[STRING_LEN];
 
        GenericRecord *generic_ptr;

	/* Set up the Generic external */
	generic_ptr = &generic_external[current_generic];

	/* Clear the location (may have been used by the last PDU) */
	memset(generic_ptr, 0, sizeof(GenericRecord));

	DBVdbv_to_snacc_generic_main(generic_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, env);

	/* Now add the external we have just defined to the external field 
	   in the PDU. single_ASN1_type is used for all V3 externals */
	any_type->value = (void *) &generic_external[current_generic];

	/* Only if not at end of array */
	if (current_generic < (STRING_LEN - 1)) {

		/* Increment current_generic for the next external 
		  (remember that there can be more than one external 
		  of the same type in searchResponse PDUs) */ 
		current_generic++;
	}
}



/* dbv -> snacc iso GenericRecord external */

int DBVdbv_to_snacc_isogeneric_any(AsnAnyDefinedBy *any_type, DbvASN1External *dbv_external, jmp_buf env)
{
	static IsoGenericRecord isogeneric_external[STRING_LEN];
        IsoGenericRecord *isogeneric_ptr;

	isogeneric_ptr = &isogeneric_external[current_isogeneric];
	memset(isogeneric_ptr, 0, sizeof(IsoGenericRecord));
	DBVdbv_to_snacc_generic_main(isogeneric_ptr, dbv_external->uExternalValue.psSingleASN1Type->uASN1.psGenericRecord, env);
	any_type->value = (void *) &isogeneric_external[current_isogeneric];

	if (current_isogeneric < (STRING_LEN - 1)) {
		current_isogeneric++;
	}
}



/* 'Generic Lite' image transfer:

   1) Image returned using level 2 segmentation - one record split amongst
      several segment requests followed by a terminating present response.

      A fragment is a proper substring of a record.
      A starting fragment is defined to be a fragment that starts at the
      beginning of a record.
      An intermediare fragment is a fragment that neither starts at the 
      beginning nor ends at the end of a record.
      A final fragment is a fragment that ends at the end of a record.

      Starting fragment in a SegReq (number of records set to 1 when 
      this is received)
      Intermediate fragment(s) each in a SeqReq
      Final Fragment sent in a PresentResponse.

   2) Name of image in tagValue (as a string)

   3) Fragment of image (binary data) in content->octets 
  
   4) Only one image to be retrieved at a time.

*/


int DBVdbv_to_snacc_generic_main(GenericRecord *generic_ptr, struct DbvGenericRecord *psGenericRecord, jmp_buf env)
{
	generic_ptr->choiceId = GENERICRECORD_JOHNFIX;

	DBVdbv_to_snacc_gen_taggedelement_list_comp(&generic_ptr->a.johnfix, psGenericRecord->ppsTaggedElement, env);
}


int DBVdbv_to_snacc_gen_taggedelement_list_comp(GenericRecordSeqOf **johnfix, DbvTaggedElement **ppsTaggedElement, jmp_buf env)
{
	*johnfix = AsnListNew (sizeof(void *));
	CheckAsn1Alloc(*johnfix, env);
	DBVdbv_to_snacc_gen_taggedelement_list(*johnfix, ppsTaggedElement, env);
}



int DBVdbv_to_snacc_gen_taggedelement_list(GenericRecordSeqOf *johnfix, DbvTaggedElement **ppsTaggedElement, jmp_buf env)
{
        TaggedElement  *data_ptr;
        TaggedElement **data_ptr_address;

        int current_listnode = 0;

        while (ppsTaggedElement[current_listnode] != NULL) {
                data_ptr_address = AsnListAppend(johnfix);
                CheckAsn1Alloc(data_ptr_address, env);

                *data_ptr_address = Asn1Alloc(sizeof(TaggedElement));
                CheckAsn1Alloc(*data_ptr_address, env);

                data_ptr = *data_ptr_address;

                DBVdbv_to_snacc_gen_taggedelement(data_ptr, ppsTaggedElement[current_listnode], env)
;
                current_listnode++;
        }
}



int DBVdbv_to_snacc_gen_taggedelement(TaggedElement *data_ptr, DbvTaggedElement *psTaggedElement, jmp_buf env)
{
	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->tagType, psTaggedElement->pulTagType, env);
	
	/* c */
	DBVdbv_to_snacc_strornum_comp(&data_ptr->tagValue, &psTaggedElement->sTagValue, env);

	/* o */
	DBVdbv_to_snacc_int_opt(&data_ptr->tagOccurrence, psTaggedElement->pulTagOccurrence, env);

	/* c */
	data_ptr->content = (struct ElementData*) Asn1Alloc (sizeof (struct ElementData));
	CheckAsn1Alloc(data_ptr->content, env);
	DBVdbv_to_snacc_gen_elementdata(data_ptr->content, &psTaggedElement->sContent, env);

	/* o */
	if (psTaggedElement->psMetaData) {
		data_ptr->metaData = (struct ElementMetaData*) Asn1Alloc (sizeof (struct ElementMetaData));
		CheckAsn1Alloc(data_ptr->metaData, env);
		DBVdbv_to_snacc_gen_metadata(data_ptr->metaData, psTaggedElement->psMetaData, env);
	}

	if (psTaggedElement->psAppliedVariant) {
		data_ptr->appliedVariant = (struct Variant*) Asn1Alloc (sizeof (data_ptr->appliedVariant));
		CheckAsn1Alloc(data_ptr->appliedVariant, env);
		DBVdbv_to_snacc_gen_variant(data_ptr->appliedVariant, psTaggedElement->psAppliedVariant, env);
	}
	

}


int DBVdbv_to_snacc_gen_elementdata(struct ElementData* content, DbvElementDataG *psContent, jmp_buf env)
{
	switch (psContent->eElementDataGType) {
	case EDT_Octets:
		content->choiceId = ELEMENTDATA_OCTETS;
		DBVdbv_to_snacc_octetstring_comp(&content->a.octets, psContent->uElementDataG.psOctets, env);
		break;
	case EDT_Numeric:
		content->choiceId = ELEMENTDATA_NUMERIC;
		content->a.numeric = psContent->uElementDataG.ulNumeric;
		break;
	case EDT_Date:
		content->choiceId = ELEMENTDATA_DATE;
		content->a.date = (GeneralizedTime *) Asn1Alloc (sizeof (GeneralizedTime));
		CheckAsn1Alloc(content->a.date, env);
		DBVdbv_to_snacc_gentime(content->a.date, psContent->uElementDataG.pcDate, env);
		break;
	case EDT_Ext:
		content->choiceId = ELEMENTDATA_EXT;
		DBVdbv_to_snacc_external_opt(&content->a.ext, psContent->uElementDataG.psExt, env);
		break;
	case EDT_String:
		content->choiceId = ELEMENTDATA_STRING;
		DBVdbv_to_snacc_interstring_comp(&content->a.string, psContent->uElementDataG.psString, env);
		break;
	case EDT_TrueOrFalse:
		content->choiceId = ELEMENTDATA_TRUEORFALSE;
		DBVdbv_to_snacc_bool(&content->a.trueOrFalse, &psContent->uElementDataG.fTrueOrFalse, env);
		break;
	case EDT_Oid:
		content->choiceId = ELEMENTDATA_OID_SNACC;
		DBVdbv_to_snacc_objectident_comp(&content->a.oid, psContent->uElementDataG.pcOid, env);
		break;
	case EDT_IntUnit:
		content->choiceId = ELEMENTDATA_INTUNIT;
		DBVdbv_to_snacc_intunit_opt(&content->a.intUnit, psContent->uElementDataG.psIntUnit, env);
		break;
	case EDT_ElementNotThere:
		content->choiceId = ELEMENTDATA_ELEMENTNOTTHERE;
		break;
	case EDT_ElementEmpty:
		content->choiceId = ELEMENTDATA_ELEMENTEMPTY;
		break;
	case EDT_NoDataRequested:
		content->choiceId = ELEMENTDATA_NODATAREQUESTED;
		break;
	case EDT_Diagnostic:
		content->choiceId = ELEMENTDATA_DIAGNOSTIC;
		DBVdbv_to_snacc_external_opt(&content->a.diagnostic, psContent->uElementDataG.psDiagnostic, env);
		break;
	case EDT_Subtree:
		content->choiceId = ELEMENTDATA_SUBTREE;
		DBVdbv_to_snacc_gen_taggedelement_list_comp(&content->a.subtree, psContent->uElementDataG.ppsSubTree, env);
	
		break;
	}
}



/* TO BE COMPLETED !!

     |
     |
     |
    \|/

*/

int DBVdbv_to_snacc_gen_metadata(struct ElementMetaData* metaData, DbvElementMetaData *psMetaData, jmp_buf env)
{
}



int DBVdbv_to_snacc_gen_variant(struct Variant* appliedVariant, DbvVariant *psAppliedVariant, jmp_buf env)
{
}


/***************************************************/


/* snacc -> dbv ansi GenericRecord external */
int DBVsnacc_to_dbv_generic_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	GenericRecord *generic_ptr;
 
	generic_ptr = (GenericRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_GenericRecord;
        psSingleASN1Type->uASN1.psGenericRecord = (struct DbvGenericRecord *) DbvAlloc (sizeof(struct DbvGenericRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psGenericRecord, env);

	DBVsnacc_to_dbv_generic_main(psSingleASN1Type->uASN1.psGenericRecord, generic_ptr, env);
}



int DBVsnacc_to_dbv_isogeneric_any(DbvSingleASN1Type *psSingleASN1Type, AsnAnyDefinedBy *any_type, jmp_buf env)
{
	IsoGenericRecord *isogeneric_ptr;
 
	isogeneric_ptr = (GenericRecord *) any_type->value;

        psSingleASN1Type->eASN1Type = AT_GenericRecord;
        psSingleASN1Type->uASN1.psGenericRecord = (struct DbvGenericRecord *) DbvAlloc (sizeof(struct DbvGenericRecord));
	CheckDbvAlloc(psSingleASN1Type->uASN1.psGenericRecord, env);

	DBVsnacc_to_dbv_generic_main(psSingleASN1Type->uASN1.psGenericRecord, isogeneric_ptr, env);
}





int DBVsnacc_to_dbv_generic_main(struct DbvGenericRecord *psGenericRecord, GenericRecord *generic_ptr, jmp_buf env)
{
	DBVsnacc_to_dbv_taggedelement_list_comp(&psGenericRecord->ppsTaggedElement, generic_ptr->a.johnfix, env);

}


int DBVsnacc_to_dbv_taggedelement_list_comp(DbvTaggedElement ***pppsTaggedElement, GenericRecordSeqOf *johnfix, jmp_buf env)
{
	int num_elements = 0;

	/* c */ 
	num_elements = AsnListCount(johnfix);
       	*pppsTaggedElement = (DbvTaggedElement **) DbvAlloc (sizeof (DbvTaggedElement *) * (num_elements + 1));
	CheckDbvAlloc(*pppsTaggedElement, env);
	DBVsnacc_to_dbv_gen_taggedelement_list(*pppsTaggedElement, johnfix, env);
}



int DBVsnacc_to_dbv_gen_taggedelement_list(DbvTaggedElement **ppsTaggedElement, GenericRecordSeqOf *johnfix, jmp_buf env)
{
	int index = 0;

	TaggedElement *data_ptr;

	FOR_EACH_LIST_ELMT(data_ptr, johnfix) {
		
		ppsTaggedElement[index] = (DbvTaggedElement *) DbvAlloc (sizeof (DbvTaggedElement));
		CheckDbvAlloc(ppsTaggedElement[index], env);
		DBVsnacc_to_dbv_gen_taggedelement(ppsTaggedElement[index], data_ptr, env);
		index++;
	}
	ppsTaggedElement[index] = NULL;
}



int DBVsnacc_to_dbv_gen_taggedelement(DbvTaggedElement *psTaggedElement, TaggedElement *data_ptr, jmp_buf env)
{
	/* o */
	DBVsnacc_to_dbv_int_opt(&psTaggedElement->pulTagType, data_ptr->tagType, env);

	/* c */
	DBVsnacc_to_dbv_strornum(&psTaggedElement->sTagValue, data_ptr->tagValue, env);

	/* o */
	DBVsnacc_to_dbv_int_opt(&psTaggedElement->pulTagOccurrence, data_ptr->tagOccurrence, env);


	/* c */
	DBVsnacc_to_dbv_gen_elementdata(&psTaggedElement->sContent, data_ptr->content, env);
	

	/* o */
	if (data_ptr->metaData) {
		psTaggedElement->psMetaData = (DbvElementMetaData *) DbvAlloc (sizeof (DbvElementMetaData));
		CheckDbvAlloc(psTaggedElement->psMetaData, env);
		DBVsnacc_to_dbv_gen_metadata(psTaggedElement->psMetaData, data_ptr->metaData, env);
	}
		
	/* o */
	if (data_ptr->appliedVariant) {
		psTaggedElement->psAppliedVariant = (DbvVariant *) DbvAlloc (sizeof (DbvVariant));
		CheckDbvAlloc(psTaggedElement->psAppliedVariant, env);
		DBVsnacc_to_dbv_gen_variant(psTaggedElement->psAppliedVariant, data_ptr->appliedVariant, env);
	}
}



int DBVsnacc_to_dbv_gen_elementdata(DbvElementDataG *psContent, struct ElementData *content, jmp_buf env)
{
	switch (content->choiceId) {
	case ELEMENTDATA_OCTETS:
		psContent->eElementDataGType = EDT_Octets;
		DBVsnacc_to_dbv_octetstring_comp(&psContent->uElementDataG.psOctets, content->a.octets, env);
		break;
	case ELEMENTDATA_NUMERIC:
		psContent->eElementDataGType = EDT_Numeric;
		psContent->uElementDataG.ulNumeric = content->a.numeric;
		break;
	case ELEMENTDATA_DATE:
		psContent->eElementDataGType = EDT_Date;

	
		/* Use the opt set up !! */
		DBVsnacc_to_dbv_gentime_opt(&psContent->uElementDataG.pcDate, *content->a.date, env);
		break;
	case ELEMENTDATA_EXT:
		psContent->eElementDataGType = EDT_Ext;
		DBVsnacc_to_dbv_external_opt(&psContent->uElementDataG.psExt, content->a.ext, env);
		break;
	case ELEMENTDATA_STRING:
		psContent->eElementDataGType = EDT_String;
		DBVsnacc_to_dbv_interstring_comp(&psContent->uElementDataG.psString, content->a.string, env);
		break;
	case ELEMENTDATA_TRUEORFALSE:
		psContent->eElementDataGType = EDT_TrueOrFalse;
		DBVsnacc_to_dbv_bool(&psContent->uElementDataG.fTrueOrFalse, &content->a.trueOrFalse, env);
		break;
	case ELEMENTDATA_OID_SNACC:
		psContent->eElementDataGType = EDT_Oid;
		DBVsnacc_to_dbv_objectident_comp(&psContent->uElementDataG.pcOid, content->a.oid, env);
		break;
	case ELEMENTDATA_INTUNIT:
		psContent->eElementDataGType = EDT_IntUnit;
		DBVsnacc_to_dbv_intunit_opt(&psContent->uElementDataG.psIntUnit, content->a.intUnit, env);
		break;
	case ELEMENTDATA_ELEMENTNOTTHERE:
		psContent->eElementDataGType = EDT_ElementNotThere;
		break;
	case ELEMENTDATA_ELEMENTEMPTY:
		psContent->eElementDataGType = EDT_ElementEmpty;
		break;
	case ELEMENTDATA_NODATAREQUESTED:
		psContent->eElementDataGType = EDT_NoDataRequested;
		break;
	case ELEMENTDATA_DIAGNOSTIC:
		psContent->eElementDataGType = EDT_Diagnostic;
		DBVsnacc_to_dbv_external_opt(&psContent->uElementDataG.psDiagnostic, content->a.diagnostic, env);
		break;
	case ELEMENTDATA_SUBTREE:
		psContent->eElementDataGType = EDT_Subtree;
		DBVsnacc_to_dbv_taggedelement_list_comp(&psContent->uElementDataG.ppsSubTree, content->a.subtree, env);
		break;
	default:
		psContent->eElementDataGType = EDT_Invalid;
		break;
	}
}



/* TO BE COMPLETED !!

     |
     |
     |
    \|/

*/


int DBVsnacc_to_dbv_gen_metadata(DbvElementMetaData *psMetaData, struct ElementMetaData *metaData, jmp_buf env)
{
}



int DBVsnacc_to_dbv_gen_variant(DbvVariant *psAppliedVariant, struct Variant *appliedVariant, jmp_buf env)
{
}




/*******************************************************************/
/*******************************************************************/
/* Structure checking routines */



/* Check the DBV explain external structure for validity */

int DBVcheck_at_explain(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{


	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* If external object identifier is not valid for the external */
	if ( (strcmp(oid, ANSI_EXPLAIN_RECORD) != 0) && (strcmp(oid, ISO_EXPLAIN_RECORD) != 0) ) {
		DBVsa_strcat(local_error_buffer, "eASN1Type");
		return(DBVdbv_external_oid_error(lAssocId, oid, local_error_buffer, ppsSIError));
	}

    
	DBVsa_strcat(local_error_buffer, "uASN1.psExplainRecord");
	if (psSingleASN1Type->uASN1.psExplainRecord == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

    
    
	DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvExplainRecord(lAssocId, psSingleASN1Type->uASN1.psExplainRecord, local_error_buffer, ppsSIError));


	
	return SI_OK;
}




int DBVcheck_DbvExplainRecord(DbvAssocId lAssocId, DbvExplainRecord *psExplainRecord, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psExplainRecord->eExplainRecordType) {
	case ERT_TargetInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psTargetInfo");
		if (psExplainRecord->uExplainRecord.psTargetInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvTargetInfo(lAssocId, psExplainRecord->uExplainRecord.psTargetInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_DatabaseInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psDatabaseInfo");
		if (psExplainRecord->uExplainRecord.psDatabaseInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvDatabaseInfo(lAssocId, psExplainRecord->uExplainRecord.psDatabaseInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_SchemaInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psSchemaInfo");
		if (psExplainRecord->uExplainRecord.psSchemaInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvSchemaInfo(lAssocId, psExplainRecord->uExplainRecord.psSchemaInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_TagSetInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psTagSetInfo");
		if (psExplainRecord->uExplainRecord.psTagSetInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvTagSetInfo(lAssocId, psExplainRecord->uExplainRecord.psTagSetInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_RecordSyntaxInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psRecordSyntaxInfo");
		if (psExplainRecord->uExplainRecord.psRecordSyntaxInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvRecordSyntaxInfo(lAssocId, psExplainRecord->uExplainRecord.psRecordSyntaxInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_AttributeSetInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psAttributeSetInfo");
		if (psExplainRecord->uExplainRecord.psAttributeSetInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvAttributeSetInfo(lAssocId, psExplainRecord->uExplainRecord.psAttributeSetInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_TermListInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psTermListInfo");
		if (psExplainRecord->uExplainRecord.psTermListInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvTermListInfo(lAssocId, psExplainRecord->uExplainRecord.psTermListInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_ExtendedServicesInfo:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psExtendedServicesInfo");
		if (psExplainRecord->uExplainRecord.psExtendedServicesInfo == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvExtendedServicesInfo(lAssocId, psExplainRecord->uExplainRecord.psExtendedServicesInfo, local_error_buffer, ppsSIError));
		break;




	case ERT_AttributeDetails:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psAttributeDetails");
		if (psExplainRecord->uExplainRecord.psAttributeDetails == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvAttributeDetails(lAssocId, psExplainRecord->uExplainRecord.psAttributeDetails, local_error_buffer, ppsSIError));
		break;




	case ERT_TermListDetails:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psTermListDetails");
		if (psExplainRecord->uExplainRecord.psTermListDetails == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvTermListDetails(lAssocId, psExplainRecord->uExplainRecord.psTermListDetails, local_error_buffer, ppsSIError));
		break;




	case ERT_ElementSetDetails:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psElementSetDetails");
		if (psExplainRecord->uExplainRecord.psElementSetDetails == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvElementSetDetails(lAssocId, psExplainRecord->uExplainRecord.psElementSetDetails, local_error_buffer, ppsSIError));
		break;




	case ERT_RetrievalRecordDetails:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psRetrievalRecordDetails");
		if (psExplainRecord->uExplainRecord.psRetrievalRecordDetails == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvRetrievalRecordDetails(lAssocId, psExplainRecord->uExplainRecord.psRetrievalRecordDetails, local_error_buffer, ppsSIError));
		break;




	case ERT_SortDetails:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psSortDetails");
		if (psExplainRecord->uExplainRecord.psSortDetails == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvSortDetails(lAssocId, psExplainRecord->uExplainRecord.psSortDetails, local_error_buffer, ppsSIError));
		break;



	case ERT_Processing:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psProcessing");
		if (psExplainRecord->uExplainRecord.psProcessing == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvProcessingInformation(lAssocId, psExplainRecord->uExplainRecord.psProcessing, local_error_buffer, ppsSIError));
		break;



	case ERT_Variants:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psVariants");
		if (psExplainRecord->uExplainRecord.psVariants == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvVariantSetInfo(lAssocId, psExplainRecord->uExplainRecord.psVariants, local_error_buffer, ppsSIError));
		break;



	case ERT_Units:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psUnits");
		if (psExplainRecord->uExplainRecord.psUnits == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvUnitInfo(lAssocId, psExplainRecord->uExplainRecord.psUnits, local_error_buffer, ppsSIError));
		break;




	case ERT_CategoryList:
		DBVsa_strcat(local_error_buffer, "uExplainRecord.psCategoryList");
		if (psExplainRecord->uExplainRecord.psCategoryList == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvCategoryList(lAssocId, psExplainRecord->uExplainRecord.psCategoryList, local_error_buffer, ppsSIError));
		break;



	default:
		DBVsa_strcat(local_error_buffer, "eExplainRecordType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}



	return SI_OK;
}



/* -------------------- DbvTargetInfo -------------------- */

int DBVcheck_DbvTargetInfo(DbvAssocId lAssocId, DbvTargetInfo *psTargetInfo, char *error_buffer, DbvError **ppsSIError)
{
	int index = 0;
	char local_error_buffer[SA_ARRAY_SIZE];
	char temp_error_buffer[SA_ARRAY_SIZE];
	char index_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psTargetInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */

	/* o ppsRecentNews */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (DBVcheck_list_opt(lAssocId, (void **) psTargetInfo->ppsIcon, local_error_buffer, "ppsIcon", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvIconObject)) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
	if (DBVcheck_dbv_bool(psTargetInfo->fNamedResultSets) == SI_NOTOK) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "fNamedResultSets");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}

	/* c */
	if (DBVcheck_dbv_bool(psTargetInfo->fMultipleDBSearch) == SI_NOTOK) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "fMultipleDBSearch");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}



	/* o  pulMaxResultSets */

	/* o  pulMaxResultSize */

	/* o  pulMaxTerms */


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psTimeoutInterval->");
	if (DBVcheck_DbvIntUnit_opt(lAssocId, psTargetInfo->psTimeoutInterval, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsWelcomeMessage */


	/* o psContactInfo */

	/* o ppsDescription */

	/* o ppsNicknames */

	/* o ppsUsageRest */

	/* o  ppsPaymentAddr */

	/* o ppsHours */
	
	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psTargetInfo->ppsDbCombinations, local_error_buffer, "ppsDbCombinations", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvDatabaseList)))
		return SI_NOTOK;



	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psTargetInfo->ppsAddresses, local_error_buffer, "ppsAddresses", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvNetworkAddress)))
		return SI_NOTOK;


	/* o ppsLanguages */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psCommonAccessInfo->");
	if (DBVcheck_DbvAccessInfo_opt(lAssocId, psTargetInfo->psCommonAccessInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	return SI_OK;
}




int DBVcheck_DbvCommonInfo_opt(DbvAssocId lAssocId, DbvCommonInfo *psCommonInfo, char *error_buffer, DbvError **ppsSIError)
{
	if (psCommonInfo) 
		return(DBVcheck_DbvCommonInfo(lAssocId, psCommonInfo, error_buffer, ppsSIError));
	

	return SI_OK;
}



int DBVcheck_DbvCommonInfo(DbvAssocId lAssocId, DbvCommonInfo *psCommonInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);
	

	/* pcDateAdded o - dont have to check */
	/* pcDateChanged o - dont have to check */
	/* pcExpiry o - dont have to check */
	/* psHumanStringLanguage o - dont have to check */

	/* o */
	DBVsa_strcat(local_error_buffer, "ppsOtherInformation->");
        if (DBVcheck_DbvOtherInformation_opt(lAssocId, psCommonInfo->ppsOtherInformation, local_error_buffer, ppsSIError))
		return SI_NOTOK;


	return SI_OK;
}






int DBVcheck_DbvIconObject(DbvAssocId lAssocId, DbvIconObject *psIcon, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */
        DBVsa_strcat(local_error_buffer, "sBodyType.");
	if (DBVcheck_DbvBodyType(lAssocId, &psIcon->sBodyType, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	/* c - no need to check */
	
	return SI_OK;
}



int DBVcheck_DbvBodyType(DbvAssocId lAssocId, DbvBodyType *psBodyType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psBodyType->eBodyTypeType) {
	case BTT_IanaType:
		DBVsa_strcat(local_error_buffer, "uBodyType.psIanaType");
		if (psBodyType->uBodyType.psIanaType == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* Dont have to check inter string */
		break;



	case BTT_Z3950Type:
		DBVsa_strcat(local_error_buffer, "uBodyType.psZ3950Type");
		if (psBodyType->uBodyType.psZ3950Type == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* Dont have to check inter string */
		break;


	case BTT_OtherType:
		DBVsa_strcat(local_error_buffer, "uBodyType.psOtherType");
		if (psBodyType->uBodyType.psOtherType == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* Dont have to check inter string */
		break;


	/* anything else including BTT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eBodyTypeType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;
}



int DBVcheck_DbvDatabaseList(DbvAssocId lAssocId, DbvDatabaseList *psDbCombinations, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

        DBVsa_strcat(local_error_buffer, "ppsInnerDatabases");
	if (psDbCombinations->ppsInnerDatabases == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvNetworkAddress(DbvAssocId lAssocId, DbvNetworkAddress *psAddresses, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);



	switch (psAddresses->eNetworkAddressType) {
	case NAT_InternetAddress:
		DBVsa_strcat(local_error_buffer, "uNetworkAddress.psInternetAddress");
		if (psAddresses->uNetworkAddress.psInternetAddress == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check structure */
		break;



	case NAT_OsiPresentationAddress:
		DBVsa_strcat(local_error_buffer, "uNetworkAddress.psOsiPresentationAddress");
		if (psAddresses->uNetworkAddress.psOsiPresentationAddress == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check structure */
		break;




	case NAT_OtherAddress:
		DBVsa_strcat(local_error_buffer, "uNetworkAddress.psOtherAddress");
		if (psAddresses->uNetworkAddress.psOtherAddress == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check structure */
		break;




	/* anything else including NAT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eNetworkAddressType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;

}



int DBVcheck_DbvAccessInfo_opt(DbvAssocId lAssocId, DbvAccessInfo *psCommonAccessInfo, char *error_buffer, DbvError **ppsSIError) 
{
	if (psCommonAccessInfo) 
		return(DBVcheck_DbvAccessInfo(lAssocId, psCommonAccessInfo, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvAccessInfo(DbvAssocId lAssocId, DbvAccessInfo *psCommonAccessInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	if (DBVcheck_list_opt(lAssocId, (void **) psCommonAccessInfo->ppsQueryTypesSupported, local_error_buffer, "ppsQueryTypesSupported", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvQueryTypeDetails)))
		return SI_NOTOK;


	/* o ppcDiagnosticsSets */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcDiagnosticsSets, local_error_buffer, "ppcDiagnosticsSets", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;
	
	
	/* o ppcAttributeSetIds */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcAttributeSetIds, local_error_buffer, "ppcAttributeSetIds", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	/* o ppcSchemas */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcSchemas, local_error_buffer, "ppcSchemas", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	/* o ppcRecordSyntaxes */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcRecordSyntaxes, local_error_buffer, "ppcRecordSyntaxes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	/* o ppcResourceChallenges */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcResourceChallenges, local_error_buffer, "ppcResourceChallenges", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;


	/* o ppsRestrictedAccess */
	if (DBVcheck_list_opt(lAssocId, (void **) psCommonAccessInfo->ppsRestrictedAccess, local_error_buffer, "ppsRestrictedAccess", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAccessRestrictions)))
		return SI_NOTOK;



	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_DbvCosts_opt(lAssocId, psCommonAccessInfo->psCostInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppcVariantSets */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psCommonAccessInfo->ppcVariantSets, local_error_buffer, "ppcVariantSets", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	/* o ppsElementSetNames */


	/* c ! */
	if (psCommonAccessInfo->ppsUnitSystems == NULL) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
        	DBVsa_strcat(local_error_buffer, "ppsUnitSystems");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}
	

	return SI_OK;
}



int DBVcheck_DbvObjectIdentifier(DbvAssocId lAssocId, DbvObjectIdentifier pcDiagnosticsSets, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* let any style DbvObjectIdentifier oid through (numbers and periods) */
	if (DBVcheck_oidformat(pcDiagnosticsSets) == SI_NOTOK) 
		return(DBVdbv_oid_error(lAssocId, local_error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvAccessRestrictions(DbvAssocId lAssocId, DbvAccessRestrictions *psRestrictedAccess, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_dbv_enum(psRestrictedAccess->eAccessType, AT_Any, AT_ByDatabase) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eAccessType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));     
	}


	/* o ppsAccessText */

	/* o ppcAccessChallenges */
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psRestrictedAccess->ppcAccessChallenges, local_error_buffer, "ppcAccessChallenges", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvQueryTypeDetails(DbvAssocId lAssocId, DbvQueryTypeDetails *psQueryTypesSupported, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psQueryTypesSupported->eQueryTypeDetailsType) {
	case QTDT_Private:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.psPrivate");
		if (psQueryTypesSupported->uQueryTypeDetails.psPrivate == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		return(DBVcheck_DbvPrivateCapabilities(lAssocId, psQueryTypesSupported->uQueryTypeDetails.psPrivate, local_error_buffer, ppsSIError));


		break;



	case QTDT_Rpn:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.psRpn");
		if (psQueryTypesSupported->uQueryTypeDetails.psRpn == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvRpnCapabilities(lAssocId, psQueryTypesSupported->uQueryTypeDetails.psRpn, local_error_buffer, ppsSIError));
	
		break;



	case QTDT_Iso8777:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.psIso8777");
		if (psQueryTypesSupported->uQueryTypeDetails.psIso8777 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		/* no need to check rest ! */
		break;



	case QTDT_Z39_58:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.ppsZ39_58");
		if (psQueryTypesSupported->uQueryTypeDetails.ppsZ39_58 == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check rest ! */
	
		break;



	case QTDT_Erpn:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.psErpn");
		if (psQueryTypesSupported->uQueryTypeDetails.psErpn == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvRpnCapabilities(lAssocId, psQueryTypesSupported->uQueryTypeDetails.psErpn, local_error_buffer, ppsSIError));
	
		break;



	case QTDT_RankedList:
		DBVsa_strcat(local_error_buffer, "uQueryTypeDetails.ppsRankedList");
		if (psQueryTypesSupported->uQueryTypeDetails.ppsRankedList == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check rest ! */
		break;



	/* anything else including QTDT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eQueryTypeDetailsType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}



	return SI_OK;
}



int DBVcheck_DbvPrivateCapabilities(DbvAssocId lAssocId, DbvPrivateCapabilities *psPrivate, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "->");
	if (DBVcheck_list_opt(lAssocId, (void **) psPrivate->ppsDescription, local_error_buffer, "ppsDescription", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvHumanStringSeq)))
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvHumanStringSeq(DbvAssocId lAssocId, DbvHumanStringSeq *psDescription, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	if (psDescription->ppsHumanString == NULL) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
        	DBVsa_strcat(local_error_buffer, "ppsHumanString");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}


	return SI_OK;
}



int DBVcheck_DbvRpnCapabilities(DbvAssocId lAssocId, DbvRpnCapabilities *psRpn, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o ppulOperators */

	/* c */
	if (DBVcheck_dbv_bool(psRpn->fResultSetAsOperandSupported) == SI_NOTOK)  {
	        DBVsa_strcat(local_error_buffer, "fResultSetAsOperandSupported");
                return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));    
        }



	/* c */
	if (DBVcheck_dbv_bool(psRpn->fRestrictionOperandSupported) == SI_NOTOK)  {
	        DBVsa_strcat(local_error_buffer, "fRestrictionOperandSupported");
                return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));    
        }


	/* o */
        DBVsa_strcat(local_error_buffer, "psProximity->");
	if (DBVcheck_DbvProximitySupport_opt(lAssocId, psRpn->psProximity, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    


	return SI_OK;
}



int DBVcheck_DbvProximitySupport_opt(DbvAssocId lAssocId, DbvProximitySupport *psProximity, char *error_buffer, DbvError **ppsSIError)
{
	if (psProximity)
		return(DBVcheck_DbvProximitySupport(lAssocId, psProximity, error_buffer, ppsSIError));
	
	return SI_OK;
}



int DBVcheck_DbvProximitySupport(DbvAssocId lAssocId, DbvProximitySupport *psProximity, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */
	if (DBVcheck_dbv_bool(psProximity->fAnySupport) == SI_NOTOK)  {
	        DBVsa_strcat(local_error_buffer, "fAnySupport");
                return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));    
        }


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psProximity->ppsUnitsSupported, local_error_buffer, "ppsUnitsSupported", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvUnitsSupported)))
		return SI_NOTOK;


	return SI_OK;
}




int DBVcheck_DbvUnitsSupported(DbvAssocId lAssocId, DbvUnitsSupported *psQueryTypesSupported, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psQueryTypesSupported->eUnitsSupportedType) {
	case UST_Known:

		/* no need to check rest ! */

		break;

	case UST_Private:
		DBVsa_strcat(local_error_buffer, "uUnitsSupported.psPrivate");
		if (psQueryTypesSupported->uUnitsSupported.psPrivate == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* no need to check rest ! */
		break;



	/* anything else including UST_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eQueryTypeDetailsType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;
}



int DBVcheck_DbvCosts_opt(DbvAssocId lAssocId, DbvCosts *psCostInfo, char *error_buffer, DbvError **ppsSIError)
{
	if (psCostInfo)
		return(DBVcheck_DbvCosts(lAssocId, psCostInfo, error_buffer, ppsSIError));
	
	return SI_OK;
}



int DBVcheck_DbvCosts(DbvAssocId lAssocId, DbvCosts *psCostInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "psConnectCharge->");
	if (DBVcheck_DbvCharge_opt(lAssocId, psCostInfo->psConnectCharge, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    

	/* o */
        DBVsa_strcat(local_error_buffer, "psConnectTime->");
	if (DBVcheck_DbvCharge_opt(lAssocId, psCostInfo->psConnectTime, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    

	/* o */
        DBVsa_strcat(local_error_buffer, "psDisplayCharge->");
	if (DBVcheck_DbvCharge_opt(lAssocId, psCostInfo->psDisplayCharge, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    

	/* o */
        DBVsa_strcat(local_error_buffer, "psSearchCharge->");
	if (DBVcheck_DbvCharge_opt(lAssocId, psCostInfo->psSearchCharge, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    

	/* o */
        DBVsa_strcat(local_error_buffer, "psSubscriptCharge->");
	if (DBVcheck_DbvCharge_opt(lAssocId, psCostInfo->psSubscriptCharge, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psCostInfo->ppsOtherCharge, local_error_buffer, "ppsOtherCharge", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvOtherCharge)))
		return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvCharge_opt(DbvAssocId lAssocId, DbvCharge *psConnectCharge, char *error_buffer, DbvError **ppsSIError)
{
	if (psConnectCharge)
		return(DBVcheck_DbvCharge(lAssocId, psConnectCharge, error_buffer, ppsSIError));
	
	return SI_OK;
}



int DBVcheck_DbvCharge(DbvAssocId lAssocId, DbvCharge *psConnectCharge, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */
	DBVsa_strcat(local_error_buffer, "sCost.");
	if (DBVcheck_DbvIntUnit(lAssocId, &psConnectCharge->sCost, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psPerWhat->");
	if (DBVcheck_DbvUnit_opt(lAssocId, psConnectCharge->psPerWhat, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsText */

	return SI_OK;
}



int DBVcheck_DbvOtherCharge(DbvAssocId lAssocId, DbvOtherCharge *psUnitsSupported, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c ppsForWhat */
	DBVsa_strcat(local_error_buffer, "ppsForWhat");
	if (psUnitsSupported->ppsForWhat == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	/* c */
        DBVsa_strcat(local_error_buffer, "sCharge.");
	if (DBVcheck_DbvCharge(lAssocId, &psUnitsSupported->sCharge, local_error_buffer, ppsSIError) == SI_NOTOK)  
                return SI_NOTOK;    

	return SI_OK;
}





/* -------------------- DbvDatabaseInfo -------------------- */


int DBVcheck_DbvDatabaseInfo(DbvAssocId lAssocId, DbvDatabaseInfo *psDatabaseInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psDatabaseInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */

	/* o pvExplainDatabase */

	/* o ppsNicknames */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psDatabaseInfo->ppsIcon, local_error_buffer, "ppsIcon", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvIconObject)) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
	if (DBVcheck_dbv_bool(psDatabaseInfo->fUserFee) == SI_NOTOK) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "fUserFee");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psDatabaseInfo->fAvailable) == SI_NOTOK) {
        	DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "fAvailable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* o ppsTitleString */

	/* o ppsKeywords */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psDatabaseInfo->ppsKeywords, local_error_buffer, "ppsKeywords", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvDataBaseInfoSeq)) == SI_NOTOK)
		return SI_NOTOK;



	/* o ppsDescription */

	/* o ppsAssociatedDbs */
	
	/* o ppsSubDbs */

	/* o ppsDisclaimers */

	/* o ppsNews */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psRecordCount->");
	if (DBVcheck_DbvRecordCount_opt(lAssocId, psDatabaseInfo->psRecordCount, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsDefaultOrder */

	/* o pulAvRecordSize */
	/* o pulMaxRecordSize */
	
	/* o ppsHours */
	/* o ppsBestTime */

	/* o pcLastUpdate */

	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psUpdateInterval->");
	if (DBVcheck_DbvIntUnit_opt(lAssocId, psDatabaseInfo->psUpdateInterval, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsCoverage */

	/* o */
	DBVsa_strcpy(local_error_buffer, "*");
	DBVsa_strcat(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "pfProprietary");
	if (DBVcheck_DbvBool_opt(lAssocId, psDatabaseInfo->pfProprietary, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	

	/* o ppsCopyrightText */

	/* o ppsCopyrightNotice */


	/* o psProducerContactInfo */

	/* o psSupplierContactInfo */

	/* o psSubmissionContactInfo */


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psAccessInfo->");
	if (DBVcheck_DbvAccessInfo_opt(lAssocId, psDatabaseInfo->psAccessInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	
	return SI_OK;
}



int DBVcheck_DbvDataBaseInfoSeq(DbvAssocId lAssocId, DbvDataBaseInfoSeq *psKeywords, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
	DBVsa_strcat(local_error_buffer, "ppsDataBaseInfo");
	if (psKeywords->ppsDataBaseInfo == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvRecordCount_opt(DbvAssocId lAssocId, DbvRecordCount *psRecordCount, char *error_buffer, DbvError **ppsSIError)
{
	if (psRecordCount)
		return(DBVcheck_DbvRecordCount(lAssocId, psRecordCount, error_buffer, ppsSIError));

	return SI_OK;
}
	

	
int DBVcheck_DbvRecordCount(DbvAssocId lAssocId, DbvRecordCount *psRecordCount, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psRecordCount->eRecordCountType) {
	case RCT_ActualNumber:

		/* no need to check rest ! */

		break;

	case RCT_ApproxNumber:

		/* no need to check rest ! */
		break;



	/* anything else including RCT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eRecordCountType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;
}



/* -------------------- DbvSchemaInfo -------------------- */

int DBVcheck_DbvSchemaInfo(DbvAssocId lAssocId, DbvSchemaInfo *psSchemaInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psSchemaInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcSchema");
	if (psSchemaInfo->pcSchema == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psSchemaInfo->pcSchema, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */

	/* o ppsDescription */

	/* o ppsTagTypeMapping */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psSchemaInfo->ppsRecordStructure, local_error_buffer, "ppsRecordStructure", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvElementInfo)) == SI_NOTOK)
		return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvElementInfo(DbvAssocId lAssocId, DbvElementInfo *psRecordStructure, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c sElementName */

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psRecordStructure->ppsElementTagPath == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsElementTagPath");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	if(DBVcheck_list(lAssocId, (void **) psRecordStructure->ppsElementTagPath, local_error_buffer, "ppsElementTagPath", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) DBVcheck_DbvPath))
		return SI_NOTOK;
	


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_DbvElementDataType_opt(lAssocId, psRecordStructure->psDataType, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	
	/* c */
	if (DBVcheck_dbv_bool(psRecordStructure->fRequired) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRequired");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psRecordStructure->fRepeatable) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRepeatable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* o ppsDescription */

	return SI_OK;
}




int DBVcheck_DbvPath(DbvAssocId lAssocId, DbvPath *psElementTagPath, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c ulTagType */

	/* c */
        DBVsa_strcat(local_error_buffer, "sTagValue.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psElementTagPath->sTagValue, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvElementDataType_opt(DbvAssocId lAssocId, DbvElementDataType *psDataType, char *error_buffer, DbvError **ppsSIError)
{
	if (psDataType)
		return(DBVcheck_DbvElementDataType(lAssocId, psDataType, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvElementDataType(DbvAssocId lAssocId, DbvElementDataType *psDataType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psDataType->eElementDataTypeType) {
	case EDTT_Primitive:

               	DBVsa_strcat(local_error_buffer, "uElementDataType.ePrimitive");
		return(DBVcheck_DbvPrimitiveDataType(lAssocId, &psDataType->uElementDataType.ePrimitive, local_error_buffer, ppsSIError));

		break;


	case EDTT_Structured:
		DBVsa_strcat(local_error_buffer, "uElementDataType.ppsStructured");
		if (psDataType->uElementDataType.ppsStructured == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uElementDataType.");
		return(DBVcheck_list_opt(lAssocId, (void **) psDataType->uElementDataType.ppsStructured, local_error_buffer, "ppsStructured", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvElementInfo)));


		break;

	/* anything else including EDTT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eElementDataTypeType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}
	return SI_OK;
}



int DBVcheck_DbvPrimitiveDataType(DbvAssocId lAssocId, DbvPrimitiveDataType *peDataType, char *error_buffer, DbvError **ppsSIError)
{
	if (DBVcheck_dbv_enum(*peDataType, PDT_OctetString, PDT_NoneOfTheAbove) == SI_NOTOK) 
               	return(DBVdbv_invalid_enum_error(lAssocId, error_buffer, ppsSIError));     

	
	return SI_OK;
}




/* -------------------- DbvTagSetInfo -------------------- */

int DBVcheck_DbvTagSetInfo(DbvAssocId lAssocId, DbvTagSetInfo *psTagSetInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psTagSetInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcTagSet");
	if (psTagSetInfo->pcTagSet == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psTagSetInfo->pcTagSet, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	/* c sName */

	/* o ppsDescription */

	/* o ppsTagTypeMapping */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psTagSetInfo->ppsElements, local_error_buffer, "ppsElements", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvElements)) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvElements(DbvAssocId lAssocId, DbvElements *psElements, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c sElementName */

	/* o ppsNicknames */

	/* c */
        DBVsa_strcat(local_error_buffer, "sElementTag.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psElements->sElementTag, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	/* o ppsDescription */

	/* o */
        DBVsa_strcpy(local_error_buffer, "*");
        DBVsa_strcat(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "peDataType");
        if (DBVcheck_DbvPrimitiveDataType_opt(lAssocId, psElements->peDataType, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvPrimitiveDataType_opt(DbvAssocId lAssocId, DbvPrimitiveDataType *peDataType, char *error_buffer, DbvError **ppsSIError)
{
	if (peDataType)
		DBVcheck_DbvPrimitiveDataType(lAssocId, peDataType, error_buffer, ppsSIError);

	return SI_OK;

}



/* -------------------- DbvRecordSyntaxInfo -------------------- */

int DBVcheck_DbvRecordSyntaxInfo(DbvAssocId lAssocId, DbvRecordSyntaxInfo *psRecordSyntaxInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psRecordSyntaxInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcRecordSyntax");
	if (psRecordSyntaxInfo->pcRecordSyntax == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psRecordSyntaxInfo->pcRecordSyntax, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */

	/* o ppcTransferSyntaxes */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt_nosub(lAssocId, (void **) psRecordSyntaxInfo->ppcTransferSyntaxes, local_error_buffer, "ppcTransferSyntaxes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvObjectIdentifier)))
		return SI_NOTOK;

	/* o ppsDescription */

	/* o  psAsn1Module */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psRecordSyntaxInfo->ppsAbstractStructure, local_error_buffer, "ppsAbstractStructure", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvElementInfo)) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}




/* -------------------- DbvAttributeSetInfo -------------------- */

int DBVcheck_DbvAttributeSetInfo(DbvAssocId lAssocId, DbvAttributeSetInfo *psAttributeSetInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psAttributeSetInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcAttributeSetId");
	if (psAttributeSetInfo->pcAttributeSetId == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psAttributeSetInfo->pcAttributeSetId, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributeSetInfo->ppsAttributes, local_error_buffer, "ppsAttributes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributeType)) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsDescription */

	return SI_OK;
}



int DBVcheck_DbvAttributeType(DbvAssocId lAssocId, DbvAttributeType *psAttributes, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o psName */

	/* o ppsDescription */

	/* c ulAttributeType */


	/* AttributeType, AttributeTypeSeqOf: DbvAttributeType, DbvAttributeDescription */
	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psAttributes->ppsAttributeValues == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsAttributeValues");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributes->ppsAttributeValues, local_error_buffer, "ppsAttributeValues", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributeDescription)) == SI_NOTOK)
		return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvAttributeDescription(DbvAssocId lAssocId, DbvAttributeDescription *psAttributeValues, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o psName */

	/* o ppsDescription */


	/* c */
        DBVsa_strcat(local_error_buffer, "sAttributeValue.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psAttributeValues->sAttributeValue, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributeValues->ppsEquivalentAttributes, local_error_buffer, "ppsEquivalentAttributes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvStringOrNumeric)) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}




/* -------------------- DbvTermListInfo -------------------- */

int DBVcheck_DbvTermListInfo(DbvAssocId lAssocId, DbvTermListInfo *psTermListInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psTermListInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sDatabaseName */


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psTermListInfo->ppsTermLists == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsTermLists");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	if(DBVcheck_list(lAssocId, (void **) psTermListInfo->ppsTermLists, local_error_buffer, "ppsTermLists", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) DBVcheck_DbvTermList))
		return SI_NOTOK;
	

	return SI_OK;
}



int DBVcheck_DbvTermList(DbvAssocId lAssocId, DbvTermList *psTermLists, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c sName */

	/* o ppsTitle */

	/* o */
	if (psTermLists->peSearchCost) 
		if (DBVcheck_dbv_enum(*psTermLists->peSearchCost, SC_Optimized, SC_Filter) == SI_NOTOK) {
        		DBVsa_strcpy(local_error_buffer, "*");
        		DBVsa_strcat(local_error_buffer, error_buffer);
        		DBVsa_strcat(local_error_buffer, "peSearchCost");
               		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));     
		}


	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_dbv_bool(psTermLists->fScanable) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fScanable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}
	

	/* o ppsBroader */

	/* o ppsNarrower */

	return SI_OK;
}



/* -------------------- DbvExtendedServicesInfo -------------------- */

int DBVcheck_DbvExtendedServicesInfo(DbvAssocId lAssocId, DbvExtendedServicesInfo *psExtendedServicesInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psExtendedServicesInfo->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c pcType */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcType");
	if (psExtendedServicesInfo->pcType == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psExtendedServicesInfo->pcType, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o psName */


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_dbv_bool(psExtendedServicesInfo->fPrivateType) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fPrivateType");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psExtendedServicesInfo->fRestrictionsApply) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRestrictionsApply");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psExtendedServicesInfo->fFeeApply) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fFeeApply");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psExtendedServicesInfo->fAvailable) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fAvailable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psExtendedServicesInfo->fRetentionSupported) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRetentionSupported");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_enum(psExtendedServicesInfo->eWaitAction, ESII_WaitSupported, ESII_NotSaying) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eWaitAction");
               	return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));     
	}


	/* o ppsDescription */


	/* o */
   	DBVsa_strcat(local_error_buffer, "psSpecificExplain->");
 	if (DBVcheck_DbvASN1External_opt(lAssocId, psExtendedServicesInfo->psSpecificExplain, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o psEsASN */
	return SI_OK;
}




/* -------------------- DbvAttributeDetails -------------------- */

int DBVcheck_DbvAttributeDetails(DbvAssocId lAssocId, DbvAttributeDetails *psAttributeDetails, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psAttributeDetails->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sDatabaseName */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributeDetails->ppsAttributesBySet, local_error_buffer, "ppsAttributesBySet", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributesBySet)) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
        DBVsa_strcat(local_error_buffer, "psAttributeCombinations->");
	if (DBVcheck_DbvAttributeCombinations_opt(lAssocId, psAttributeDetails->psAttributeCombinations, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvAttributesBySet(DbvAssocId lAssocId, DbvAttributesBySet *psAttributesBySet, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcAttributeSet");
	if (psAttributesBySet->pcAttributeSet == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psAttributesBySet->pcAttributeSet, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psAttributesBySet->ppsAttributesByType == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsAttributesByType");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list(lAssocId, (void **) psAttributesBySet->ppsAttributesByType, local_error_buffer, "ppsAttributesByType", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributesByType)) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvAttributesByType(DbvAssocId lAssocId, DbvAttributesByType *psAttributesByType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c ulAttributeType */

	/* o */
        DBVsa_strcat(local_error_buffer, "psDefaultIfOmitted->");
	if (DBVcheck_DbvOmittedAttributeInterpretation_opt(lAssocId, psAttributesByType->psDefaultIfOmitted, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributesByType->ppsAttributeValues, local_error_buffer, "ppsAttributeValues", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributeValues)) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvOmittedAttributeInterpretation_opt(DbvAssocId lAssocId, DbvOmittedAttributeInterpretation *psOptionalType, char *error_buffer, DbvError **ppsSIError)
{
	if (psOptionalType)
		return(DBVcheck_DbvOmittedAttributeInterpretation(lAssocId, psOptionalType, error_buffer, ppsSIError));
	
	return SI_OK;

}



int DBVcheck_DbvOmittedAttributeInterpretation(DbvAssocId lAssocId, DbvOmittedAttributeInterpretation *psOptionalType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "psDefaultValue->");
        if (DBVcheck_DbvStringOrNumeric_opt(lAssocId, psOptionalType->psDefaultValue, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;
	
	/* o ppsDefaultDescription */
	return SI_OK;
}



int DBVcheck_DbvAttributeValues(DbvAssocId lAssocId, DbvAttributeValues *psAttributeValues, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
        DBVsa_strcat(local_error_buffer, "sValue.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psAttributeValues->sValue, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;
	
	/* o ppsDescription */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributeValues->ppsSubAttributes, local_error_buffer, "ppsSubAttributes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvStringOrNumeric)) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psAttributeValues->ppsSuperAttributes, local_error_buffer, "ppsSuperAttributes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvStringOrNumeric)) == SI_NOTOK)
		return SI_NOTOK;


	/* o pvPartialSupport */

	return SI_OK;
}



int DBVcheck_DbvAttributeCombinations_opt(DbvAssocId lAssocId, DbvAttributeCombinations *psAttributeCombinations, char *error_buffer, DbvError **ppsSIError)
{
	if (psAttributeCombinations)
		return(DBVcheck_DbvAttributeCombinations(lAssocId, psAttributeCombinations, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvAttributeCombinations(DbvAssocId lAssocId, DbvAttributeCombinations *psAttributeCombinations, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */ 
       	DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcDefaultAttributeSet");
	if (psAttributeCombinations->pcDefaultAttributeSet == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psAttributeCombinations->pcDefaultAttributeSet, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psAttributeCombinations->ppsLegalCombinations == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsLegalCombinations");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list(lAssocId, (void **) psAttributeCombinations->ppsLegalCombinations, local_error_buffer, "ppsLegalCombinations", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributeCombination)) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvAttributeCombination(DbvAssocId lAssocId, DbvAttributeCombination *psLegalCombinations, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psLegalCombinations->ppsAttributeOccurrence == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsAttributeOccurrence");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}


        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list(lAssocId, (void **) psLegalCombinations->ppsAttributeOccurrence, local_error_buffer, "ppsAttributeOccurrence", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvAttributeOccurrence)) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvAttributeOccurrence(DbvAssocId lAssocId, DbvAttributeOccurrence *psAttributeOccurrence, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o pcAttributeSet */
        DBVsa_strcat(local_error_buffer, "pcAttributeSet");

	if (psAttributeOccurrence->pcAttributeSet)
		if (DBVcheck_DbvObjectIdentifier(lAssocId, psAttributeOccurrence->pcAttributeSet, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;
	

	/* c ulAttributeType */

	/* o pvMustBeSupplied */

	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "sAttributeValues.");
	if (DBVcheck_DbvAttributeOccurrenceChoice(lAssocId, &psAttributeOccurrence->sAttributeValues, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;
	
	return SI_OK;
}



int DBVcheck_DbvAttributeOccurrenceChoice(DbvAssocId lAssocId, DbvAttributeOccurrenceChoice *psAttributeValues, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psAttributeValues->eAttributeOccurrenceChoiceType) {
	case AOT_AnyOrNone:

               	DBVsa_strcat(local_error_buffer, "uAttributeOccurrenceChoiceType.pvAnyOrNone");

		if (psAttributeValues->uAttributeOccurrenceChoiceType.pvAnyOrNone == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		/* nothing else to check */

		break;


	case AOT_Specific:
		DBVsa_strcat(local_error_buffer, "uAttributeOccurrenceChoiceType.ppsSpecific");
		if (psAttributeValues->uAttributeOccurrenceChoiceType.ppsSpecific == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uAttributeOccurrenceChoiceType.");
		return(DBVcheck_list_opt(lAssocId, (void **) psAttributeValues->uAttributeOccurrenceChoiceType.ppsSpecific, local_error_buffer, "ppsSpecific", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvStringOrNumeric)));


		break;

	/* anything else including AOT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eAttributeOccurrenceChoiceType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



/* -------------------- DbvTermListDetails -------------------- */

int DBVcheck_DbvTermListDetails(DbvAssocId lAssocId, DbvTermListDetails *psTermListDetails, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psTermListDetails->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sTermListName */

	/* o ppsDescription */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psAttributes->");
	if (DBVcheck_DbvAttributeCombinations_opt(lAssocId, psTermListDetails->psAttributes, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psScanInfo->");
	if (DBVcheck_DbvScanInfo_opt(lAssocId, psTermListDetails->psScanInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o pulEstNumberTerms */


	/* o */
	DBVsa_strcat(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psTermListDetails->ppsSampleTerms, local_error_buffer, "ppsSampleTerms", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_dbv_term)))
		return SI_NOTOK;




	return SI_OK;
}



int DBVcheck_DbvScanInfo_opt(DbvAssocId lAssocId, DbvScanInfo *psScanInfo, char *error_buffer, DbvError **ppsSIError)
{
	if (psScanInfo)
		return(DBVcheck_DbvScanInfo(lAssocId, psScanInfo, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvScanInfo(DbvAssocId lAssocId, DbvScanInfo *psScanInfo, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o pulMaxStepSize */

	/* o ppsCollatingSequence */

	/* o */
	DBVsa_strcpy(local_error_buffer, "*");
	DBVsa_strcat(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "pfIncreasing");
	if (DBVcheck_DbvBool_opt(lAssocId, psScanInfo->pfIncreasing, local_error_buffer, ppsSIError) == SI_NOTOK) 
		return SI_NOTOK;


	return SI_OK;
}



/* -------------------- DbvElementSetDetails -------------------- */

int DBVcheck_DbvElementSetDetails(DbvAssocId lAssocId, DbvElementSetDetails *psElementSetDetails, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psElementSetDetails->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sDatabaseName  */

	/* c sElementSetName */

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcRecordSyntax");
	if (psElementSetDetails->pcRecordSyntax == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psElementSetDetails->pcRecordSyntax, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcSchema");
	if (psElementSetDetails->pcSchema == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psElementSetDetails->pcSchema, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	/* o ppsDescription */

	/* o */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psElementSetDetails->ppsDetailsPerElement, local_error_buffer, "ppsDetailsPerElement", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvDetailsPerElement)))
		return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvDetailsPerElement(DbvAssocId lAssocId, DbvDetailsPerElement *psDetailsPerElement, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o psName */

	/* o */
	DBVsa_strcat(local_error_buffer, "psRecordTag->");
	if (DBVcheck_DbvRecordTag_opt(lAssocId, psDetailsPerElement->psRecordTag, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psDetailsPerElement->ppsSchemaTags, local_error_buffer, "ppsSchemaTags", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvPathA)))
		return SI_NOTOK;


	/* o pulMaxSize */

	/* o pulMinSize */

	/* o pulAvgSize */

	/* o pulFixedSize */

	/* c */
	if (DBVcheck_dbv_bool(psDetailsPerElement->fRepeatable) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRepeatable");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* c */
	if (DBVcheck_dbv_bool(psDetailsPerElement->fRequired) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "fRequired");
		return(DBVdbv_bool_error(lAssocId, local_error_buffer, ppsSIError));
	}


	/* o ppsDescription */

	/* o ppsContents */

	/* o ppsBillingInfo */

	/* o psRestrictions */

	/* o ppsAlternateNames */

	/* o ppsGenericNames */

	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "psSearchAccess->");
	if (DBVcheck_DbvAttributeCombinations_opt(lAssocId, psDetailsPerElement->psSearchAccess, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvRecordTag_opt(DbvAssocId lAssocId, DbvRecordTag *psRecordTag, char *error_buffer, DbvError **ppsSIError)
{
	if (psRecordTag)
		return(DBVcheck_DbvRecordTag(lAssocId, psRecordTag, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvRecordTag(DbvAssocId lAssocId, DbvRecordTag *psRecordTag, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
        DBVsa_strcat(local_error_buffer, "psQualifier->");
        if (DBVcheck_DbvStringOrNumeric_opt(lAssocId, psRecordTag->psQualifier, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;
	

	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "sTagValue.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psRecordTag->sTagValue, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;
	

	return SI_OK;
}



int DBVcheck_DbvPathA(DbvAssocId lAssocId, DbvPathA *psSchemaTags, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psSchemaTags->ppsSchemaTags == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsSchemaTags");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	if (DBVcheck_list(lAssocId, (void **) psSchemaTags->ppsSchemaTags, local_error_buffer, "ppsSchemaTags", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvPath)))
		return SI_NOTOK;

	return SI_OK;
}



/* -------------------- DbvRetrievalRecordDetails -------------------- */

int DBVcheck_DbvRetrievalRecordDetails(DbvAssocId lAssocId, DbvRetrievalRecordDetails *psRetrievalRecordDetails, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psRetrievalRecordDetails->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sDatabaseName */

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "pcSchema");
	if (psRetrievalRecordDetails->pcSchema == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psRetrievalRecordDetails->pcSchema, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcRecordSyntax");
	if (psRetrievalRecordDetails->pcRecordSyntax == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	

	if (DBVcheck_DbvObjectIdentifier(lAssocId, psRetrievalRecordDetails->pcRecordSyntax, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* o ppsDescription */



	/* o */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psRetrievalRecordDetails->ppsDetailsPerElement, local_error_buffer, "ppsDetailsPerElement", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvDetailsPerElement)))
		return SI_NOTOK;



	return SI_OK;
}



/* -------------------- DbvSortDetails -------------------- */

int DBVcheck_DbvSortDetails(DbvAssocId lAssocId, DbvSortDetails *psSortDetails, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psSortDetails->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c sDatabaseName */

	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psSortDetails->ppsSortKeys, local_error_buffer, "ppsSortKeys", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvSortKeys)))
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvSortKeys(DbvAssocId lAssocId, DbvSortKeys *psSortKeys, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	
	/* o ppsDescription */

	/* o */
	if (DBVcheck_list_opt(lAssocId, (void **) psSortKeys->ppsElementSpecifications, local_error_buffer, "ppsElementSpecifications", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvSpecification)))
		return SI_NOTOK;


	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psAttributeSpecifications->");
	if (DBVcheck_DbvAttributeCombinations_opt(lAssocId, psSortKeys->psAttributeSpecifications, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psSortType->");
	if (DBVcheck_DbvSortType_opt(lAssocId, psSortKeys->psSortType, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
	if (psSortKeys->peCaseSensitivity)
		if (DBVcheck_dbv_enum(*psSortKeys->peCaseSensitivity, CSE_Always, CSE_DefaultNo) == SI_NOTOK) {
			DBVsa_strcpy(local_error_buffer, "*");
			DBVsa_strcat(local_error_buffer, error_buffer);
			DBVsa_strcat(local_error_buffer, "peCaseSensitivity");
               		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));     
	}


	return SI_OK;
}



int DBVcheck_DbvSortType_opt(DbvAssocId lAssocId, DbvSortType *psSortType, char *error_buffer, DbvError **ppsSIError)
{
	if (psSortType)
		return(DBVcheck_DbvSortType(lAssocId, psSortType, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvSortType(DbvAssocId lAssocId, DbvSortType *psSortType, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	switch (psSortType->eSortTypeType) {
	case STT_Character:

		/* nothing else to check */
		break;


	case STT_Numeric:

		/* nothing else to check */
		break;

	case STT_Structured:

		/* c ppsStructured */
       		DBVsa_strcat(local_error_buffer, "ppsStructured");
		if (psSortType->uSortType.ppsStructured == NULL) 
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
		break;



	/* anything else including STT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eSortTypeType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;

}



/* -------------------- DbvProcessingInformation -------------------- */

int DBVcheck_DbvProcessingInformation(DbvAssocId lAssocId, DbvProcessingInformation *psProcessing, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psProcessing->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c sDatabaseName */

	/* c */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_dbv_enum(psProcessing->eProcessingContext, PC_Access, PC_RecordHandling) == SI_NOTOK) {
		DBVsa_strcat(local_error_buffer, "eProcessingContext");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));     
	}


	/* c sName */

	/* c pcOid */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcOid");
	if (psProcessing->pcOid == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	
	if (DBVcheck_DbvObjectIdentifier(lAssocId, psProcessing->pcOid, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o ppsDescription */


	/* o */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "psInstructions->");
	if (DBVcheck_DbvASN1External_opt(lAssocId, psProcessing->psInstructions, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	return SI_OK;
}




/* -------------------- DbvVariantSetInfo -------------------- */

int DBVcheck_DbvVariantSetInfo(DbvAssocId lAssocId, DbvVariantSetInfo *psVariants, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psVariants->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c pcVariantSet */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
       	DBVsa_strcat(local_error_buffer, "pcVariantSet");
	if (psVariants->pcVariantSet == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


	if (DBVcheck_DbvObjectIdentifier(lAssocId, psVariants->pcVariantSet, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sName */


	/* o */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psVariants->ppsVariants, local_error_buffer, "ppsVariants", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvVariantClass)))
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvVariantClass(DbvAssocId lAssocId, DbvVariantClass *psVariants, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o psName */

	/* o ppsDescription */

	/* c ulVariantClass */

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psVariants->ppsVariantTypes == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsVariantTypes");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	if (DBVcheck_list(lAssocId, (void **) psVariants->ppsVariantTypes, local_error_buffer, "ppsVariantTypes", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvVariantTypes)))
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvVariantTypes(DbvAssocId lAssocId, DbvVariantTypes *psVariantTypes, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o psName */

	/* o ppsDescription */

	/* c ulVariantType */

	/* o */
	DBVsa_strcat(local_error_buffer, "psVariantValue->");
	if (DBVcheck_DbvVariantValue_opt(lAssocId, psVariantTypes->psVariantValue, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;



	return SI_OK;
}



int DBVcheck_DbvVariantValue_opt(DbvAssocId lAssocId, DbvVariantValue *psVariantValue, char *error_buffer, DbvError **ppsSIError)
{
	if (psVariantValue)
		return(DBVcheck_DbvVariantValue(lAssocId, psVariantValue, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvVariantValue(DbvAssocId lAssocId, DbvVariantValue *psVariantValue, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* c */
        DBVsa_strcat(local_error_buffer, "eDataType");
	if (DBVcheck_DbvPrimitiveDataType(lAssocId, &psVariantValue->eDataType, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psValues->");
	if (DBVcheck_DbvValueSet_opt(lAssocId, psVariantValue->psValues, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvValueSet_opt(DbvAssocId lAssocId, DbvValueSet *psValues, char *error_buffer, DbvError **ppsSIError)
{
	if (psValues)
		return(DBVcheck_DbvValueSet(lAssocId, psValues, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvValueSet(DbvAssocId lAssocId, DbvValueSet *psValues, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psValues->eValueSetType) {
	case VT_Range:
		DBVsa_strcat(local_error_buffer, "uValueSet.psRange");
		if (psValues->uValueSet.psRange == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return(DBVcheck_DbvValueRange(lAssocId, psValues->uValueSet.psRange, local_error_buffer, ppsSIError));

		/* nothing else to check */
		break;


	case VT_Enumerated:
		DBVsa_strcat(local_error_buffer, "uValueSet.ppsEnumerated");
		if (psValues->uValueSet.ppsEnumerated == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcpy(local_error_buffer, error_buffer);
		DBVsa_strcat(local_error_buffer, "uValueSet.");
		return(DBVcheck_list(lAssocId, (void **) psValues->uValueSet.ppsEnumerated, local_error_buffer, "ppsEnumerated", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvValueDescription)));


		/* nothing else to check */
		break;

	/* anything else including VT_Invalid */
	default:
		DBVsa_strcat(local_error_buffer, "eValueSetType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_DbvValueRange(DbvAssocId lAssocId, DbvValueRange *psRange, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	/* o */
	DBVsa_strcat(local_error_buffer, "psLower->");
	if (DBVcheck_DbvValueDescription_opt(lAssocId, psRange->psLower, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* o */
        DBVsa_strcpy(local_error_buffer, error_buffer);
	DBVsa_strcat(local_error_buffer, "psUpper->");
	if (DBVcheck_DbvValueDescription_opt(lAssocId, psRange->psUpper, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	return SI_OK;
}



int DBVcheck_DbvValueDescription_opt(DbvAssocId lAssocId, DbvValueDescription *psEnumerated, char *error_buffer, DbvError **ppsSIError)
{
	if (psEnumerated)
		return(DBVcheck_DbvValueDescription(lAssocId, psEnumerated, error_buffer, ppsSIError));

	return SI_OK;
}



int DBVcheck_DbvValueDescription(DbvAssocId lAssocId, DbvValueDescription *psEnumerated, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);


	switch (psEnumerated->eValueDescriptionType) {
	case VDT_Integer:

		/* nothing else to check */
		break;


	case VDT_String:

		DBVsa_strcat(local_error_buffer, "uValueDescription.psString");
		if (psEnumerated->uValueDescription.psString == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* nothing else to check */
		break;



	case VDT_Octets:

		DBVsa_strcat(local_error_buffer, "uValueDescription.psOctets");
		if (psEnumerated->uValueDescription.psOctets == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		/* nothing else to check */

		break;



	case VDT_Oid:

		DBVsa_strcat(local_error_buffer, "uValueDescription.pcOid");
		if (psEnumerated->uValueDescription.pcOid == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		if (DBVcheck_DbvObjectIdentifier(lAssocId, psEnumerated->uValueDescription.pcOid, local_error_buffer, ppsSIError) == SI_NOTOK)
			return SI_NOTOK;

		/* nothing else to check */
		break;



	case VDT_Unit:

		DBVsa_strcat(local_error_buffer, "uValueDescription.psUnit");
		if (psEnumerated->uValueDescription.psUnit == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));


		DBVsa_strcat(local_error_buffer, "->");
		return (DBVcheck_DbvUnit(lAssocId, psEnumerated->uValueDescription.psUnit, local_error_buffer, ppsSIError));

		break;

	case VDT_ValueAndUnit:

		DBVsa_strcat(local_error_buffer, "uValueDescription.psValueAndUnit");
		if (psEnumerated->uValueDescription.psValueAndUnit == NULL)
			return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

		DBVsa_strcat(local_error_buffer, "->");
		return (DBVcheck_DbvIntUnit(lAssocId, psEnumerated->uValueDescription.psValueAndUnit, local_error_buffer, ppsSIError));


		break;



	/* anything else including VDT_Invalid  */
	default:
		DBVsa_strcat(local_error_buffer, "eValueDescriptionType");
		return(DBVdbv_invalid_enum_error(lAssocId, local_error_buffer, ppsSIError));
		break;
	}


	return SI_OK;
}



/* -------------------- DbvUnitInfo -------------------- */

int DBVcheck_DbvUnitInfo(DbvAssocId lAssocId, DbvUnitInfo *psUnits, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psUnits->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;


	/* c sUnitSystem */

	/* o ppsDescription */


	/* o */
	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list_opt(lAssocId, (void **) psUnits->ppsUnitTypes, local_error_buffer, "ppsUnitTypes", ppsSIError, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvUnitType)))
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvUnitType(DbvAssocId lAssocId, DbvUnitType *psUnitTypes, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o psName */

	/* o ppsDescription */

	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "sUnitType.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psUnitTypes->sUnitType, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psUnitTypes->ppsUnits == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsUnits");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (DBVcheck_list(lAssocId, (void **) psUnitTypes->ppsUnits, local_error_buffer, "ppsUnits", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvUnits)))
		return SI_NOTOK;


	return SI_OK;
}




int DBVcheck_DbvUnits(DbvAssocId lAssocId, DbvUnits *psUnits, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o psName */

	/* o ppsDescription */

	/* c */
        DBVsa_strcpy(local_error_buffer, error_buffer);
        DBVsa_strcat(local_error_buffer, "sUnit.");
        if (DBVcheck_DbvStringOrNumeric(lAssocId, &psUnits->sUnit, local_error_buffer, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;


	return SI_OK;
}



/* -------------------- DbvCategoryList -------------------- */

int DBVcheck_DbvCategoryList(DbvAssocId lAssocId, DbvCategoryList *psCategoryList, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* o */
	DBVsa_strcat(local_error_buffer, "psCommonInfo->");
	if (DBVcheck_DbvCommonInfo_opt(lAssocId, psCategoryList->psCommonInfo, local_error_buffer, ppsSIError) == SI_NOTOK)
		return SI_NOTOK;

	/* c */
       	DBVsa_strcpy(local_error_buffer, error_buffer);
	if (psCategoryList->ppsCategories == NULL) {
        	DBVsa_strcat(local_error_buffer, "ppsCategories");
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));
	}

	DBVsa_strcat(local_error_buffer, error_buffer);
	if (DBVcheck_list(lAssocId, (void **) psCategoryList->ppsCategories, local_error_buffer, "ppsCategories", ppsSIError, DBV_TRUE, (int (*) (DbvAssocId, void *, char *, DbvError **)) (DBVcheck_DbvCategoryInfo)))
		return SI_NOTOK;


	return SI_OK;
}



int DBVcheck_DbvCategoryInfo(DbvAssocId lAssocId, DbvCategoryInfo *psCategories, char *error_buffer, DbvError **ppsSIError)
{
	char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, error_buffer);

	/* c sCategory */

	/* o psOriginalCategory */

	/* o ppsDescription */

	/* o psAsn1Module */

	return SI_OK;
}




/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/




/* Check the DBV generic external structure for validity */

int DBVcheck_at_generic(DbvAssocId lAssocId, DbvSingleASN1Type *psSingleASN1Type, char *oid, char *error_buffer, DbvError **ppsSIError)
{
	return SI_OK;
}







