/*  Module Name: 12/6/96
    Version: @(#)z3950_pdu_print.c	7.11
    Author: John Hough
 
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ
 
    Date of Issue:
 
    QA Acceptance:
    Date of Acceptance:
 
    Description:
        
 
*/


#include "z3950_pdu_print.h"


/*******************************************************/
/* A couple of trace message functions */

/* Output a message before outputting a PDU to be sent */ 
int DBVSendingTrace(DbvAssocId lAssocId)
{
	DBVTrace(lAssocId, Trace_High, "\nSending the following PDU:\n");
}



/* Output a message before outputting a PDU received */ 
int DBVReceivingTrace(DbvAssocId lAssocId)
{
	DBVTrace(lAssocId, Trace_High, "\nReceived the following PDU:\n");
}



/*******************************************************/
/* PDU trace routines */

/* InitRequest */

int DBVInitReqTrace(DbvAssocId lAssocId, DbvInitializeReq *psInitializeReq)
{
 
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## InitializeReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {

		/* Access the filename and write to it directly rather than 
		   using DBVTrace - because fprintf (rather than sprintf,
		   strcat etc) will be used, there will never be any 
		   chance of core dumps etc. due to buffer overwrites. */

		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_initreq_fields(trace_file, psInitializeReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}



void DBVget_trace_file(DbvAssocId lAssocId, FILE **trace_file)
{
        DBVTraceDetails *psTraceInformation = NULL;
 
        if(DBVTraceList == (struct SList *)NULL)
                return;
 
        /* Get the Association from the list of configured trace files*/
        psTraceInformation = SL_Locate(DBVTraceList,
                (SL_LocateFunc)DBVLocateAssociateTrace, &lAssocId);

        if(psTraceInformation == NULL)
                return;
 
	*trace_file = psTraceInformation->DBVTraceFile;
}





void DBVnewline(FILE *f)
{
	fprintf(f, "\n");
}





/*******************************************************/
/* InitResponse */


int DBVInitRspTrace(DbvAssocId lAssocId, DbvInitializeRsp *psInitializeRsp)
{
 
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## InitializeRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_initrsp_fields(trace_file, psInitializeRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}





/*******************************************************/
/* SearchRequest */


int DBVSrchReqTrace(DbvAssocId lAssocId, DbvSearchReq *psSearchReq)
{
 
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## SearchReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_searchreq_fields(trace_file, psSearchReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 






/*******************************************************/
/* SearchResponse */


int DBVSrchRspTrace(DbvAssocId lAssocId, DbvSearchRsp *psSearchRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## SearchRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_searchrsp_fields(trace_file, psSearchRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* PresentRequest */


int DBVPresReqTrace(DbvAssocId lAssocId, DbvPresentReq *psPresentReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## PresentReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_presreq_fields(trace_file, psPresentReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* PresentResponse */


int DBVPresRspTrace(DbvAssocId lAssocId, DbvPresentRsp *psPresentRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## PresentRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_presrsp_fields(trace_file, psPresentRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* DeleteResultSetRequest */


int DBVDelReqTrace(DbvAssocId lAssocId, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## DeleteResultSetReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_delreq_fields(trace_file, psDeleteResultSetReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* DeleteResultSetRsp */


int DBVDelRspTrace(DbvAssocId lAssocId, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## DeleteResultSetRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL) 
			DBVprint_delrsp_fields(trace_file, psDeleteResultSetRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* ResourceReportReq */


int DBVResRepReqTrace(DbvAssocId lAssocId, DbvResourceReportReq *psResourceReportReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ResourceReportReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_resrepreq_fields(trace_file, psResourceReportReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* DbvResourceReportResponse */


int DBVResRepRspTrace(DbvAssocId lAssocId, DbvResourceReportRsp *psResourceReportRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ResourceReportRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_resreprsp_fields(trace_file, psResourceReportRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* ScanRequest */


int DBVScanReqTrace(DbvAssocId lAssocId, DbvScanReq *psScanReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ScanReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_scanreq_fields(trace_file, psScanReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* ScanResponse */


int DBVScanRspTrace(DbvAssocId lAssocId, DbvScanRsp *psScanRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ScanRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_scanrsp_fields(trace_file, psScanRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* SortRequest */


int DBVSortReqTrace(DbvAssocId lAssocId, DbvSortReq *psSortReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## SortReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_sortreq_fields(trace_file, psSortReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* SortResponse */


int DBVSortRspTrace(DbvAssocId lAssocId, DbvSortRsp *psSortRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## SortRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_sortrsp_fields(trace_file, psSortRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 




/*******************************************************/
/* SegmentRequest  - target to origin */


int DBVSegReqTrace(DbvAssocId lAssocId, DbvSegmentReq *psSegmentReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## SegmentReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_segreq_fields(trace_file, psSegmentReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* ExtendedServicesRequest */


int DBVExtendReqTrace(DbvAssocId lAssocId, DbvExtendedServicesReq *psExtendedServicesReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ExtendedServicesReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_extendreq_fields(trace_file, psExtendedServicesReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 





/*******************************************************/
/* ExtendedServicesResponse */


int DBVExtendRspTrace(DbvAssocId lAssocId, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ExtendedServicesRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_extendrsp_fields(trace_file, psExtendedServicesRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 


 



/*******************************************************/
/* Close 'CloseRequest' */


int DBVCloseReqTrace(DbvAssocId lAssocId, DbvCloseReq *psCloseReq)

{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## Close 'CloseReq' PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_closereq_fields(trace_file, psCloseReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 


/*******************************************************/
/* Close 'CloseResponse' */

int DBVCloseRspTrace(DbvAssocId lAssocId, DbvCloseReq *psCloseRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## Close 'CloseRsp' PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_closereq_fields(trace_file, psCloseRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}


/*******************************************************/
/* Unknown PDU */


int DBVUnknownPDUTrace(DbvAssocId lAssocId, DbvOctetString *psInvalidAPDU)

{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## psInvalidAPDU PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_invalidapdu(trace_file, psInvalidAPDU);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}
 






/*******************************************************/
/* AccessControlRequest  - target to origin */


int DBVAccReqTrace(DbvAssocId lAssocId, DbvAccessControlReq *psAccessControlReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## AccessControlReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_accessreq_fields(trace_file, psAccessControlReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}



/*******************************************************/
/* AccessControlResponse - origin to target */


int DBVAccRspTrace(DbvAssocId lAssocId, DbvAccessControlRsp *psAccessControlRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## AccessControlRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_accessrsp_fields(trace_file, psAccessControlRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}



/*******************************************************/
/* ResourceControlRequest  - target to origin */


int DBVResCntrlReqTrace(DbvAssocId lAssocId, DbvResourceControlReq *psResourceControlReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ResourceControlReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_rescntrlreq_fields(trace_file, psResourceControlReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}



/*******************************************************/
/* ResourceControlResponse - origin to target */

int DBVResCntrlRspTrace(DbvAssocId lAssocId, DbvResourceControlRsp *psResourceControlRsp)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## ResourceControlRsp PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_rescntrlrsp_fields(trace_file, psResourceControlRsp);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}




/*******************************************************/
/* TriggerResourceControlRequest - origin to target */


int DBVTrigResCntrlReqTrace(DbvAssocId lAssocId, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	FILE *trace_file = NULL;

	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");
	DBVTrace(lAssocId, Trace_High, "## TriggerResourceControlReq PDU ##\n");
	DBVTrace(lAssocId, Trace_High, "\n");

	if (DBVTraceLevel(lAssocId) == Trace_High) {
		DBVget_trace_file(lAssocId, &trace_file);
		if (trace_file != NULL)
			DBVprint_trigrescntrlreq_fields(trace_file, psTriggerResourceControlReq);
		else
			DBVTrace(lAssocId, Trace_High, "Error with trace file\n");
	} else
		DBVTrace(lAssocId, Trace_High, "PDU fields are only displayed for trace level Trace_High\n");


	DBVTrace(lAssocId, Trace_High, "----------------------------------------\n");

}





/*#########################################################################*/

/*
Dont change the code below. Change api_pdu_print.c and import....
Change z3950_pdu_print.h headers also (use indent on a copy
before get_headers)

api_pdu_print1.c:
api_pdu_print2.c:

(delete #includes in imports)

<CNTRL+G>
<current line>,$s/print_/DBVprint_/g
<current line>,$s/printN_/DBVprintN_/g
<current line>,$s/to_hex/DBVto_hex/g

/aligned_external(FILE

Delete code under line containing
 _octet_string(f, external_ptr->uExternalValue.psOctetAligned);

*/



/***************************************************************/ 
/* InitResponse */

int DBVprint_initrsp_fields(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
	fprintf(f,"InitResponse PDU\n");

	DBVprint_init_response_refid(f, psInitializeRsp);
	DBVprint_init_response_prover(f, psInitializeRsp);
	DBVprint_init_response_options(f, psInitializeRsp);
	DBVprint_init_response_prefmess(f, psInitializeRsp);
	DBVprint_init_response_maxmess(f, psInitializeRsp);
	DBVprint_init_response_result(f, psInitializeRsp);
	DBVprint_init_response_impid(f, psInitializeRsp);
	DBVprint_init_response_impname(f, psInitializeRsp);
	DBVprint_init_response_impver(f, psInitializeRsp);
	DBVprint_init_response_userinfo(f, psInitializeRsp);
	DBVprint_init_response_otherinfo(f, psInitializeRsp);
}



int DBVprint_init_response_refid(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psInitializeRsp->psReferenceId);
}


int DBVprint_init_response_prover(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
        fprintf(f,"pcProtocolVersion: %s\n", psInitializeRsp->pcProtocolVersion);
}


int DBVprint_init_response_options(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
        fprintf(f,"pcOptions: %s\n", psInitializeRsp->pcOptions);
}



int DBVprint_init_response_prefmess(FILE *f, DbvInitializeRsp *psInitializeRsp)
{

        fprintf(f,"ulPreferredMessageSize: %ld\n", psInitializeRsp->ulPreferredMessageSize);
}



int DBVprint_init_response_maxmess(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
        fprintf(f,"ulMaxmiumMessageSize: %ld\n", psInitializeRsp->ulMaximumMessageSize);
}




int DBVprint_init_response_result(FILE *f, DbvInitializeRsp *psInitializeRsp)
{

	/* result */
	if (psInitializeRsp->fResult == DBV_TRUE)
		fprintf(f,"fResult: TRUE\n");
	else
	if (psInitializeRsp->fResult == DBV_FALSE)
		fprintf(f,"fResult: FALSE\n");
	else
		fprintf(f,"fResult: %d - invalid\n", psInitializeRsp->fResult);
}



int DBVprint_init_response_impid(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
       	fprintf(f,"psImplementationId: ");
	DBVprint_inter_string(f, psInitializeRsp->psImplementationId);
}



int DBVprint_init_response_impname(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
        fprintf(f,"psImplementationName: ");
	DBVprint_inter_string(f, psInitializeRsp->psImplementationName);
}


int DBVprint_init_response_impver(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
        fprintf(f,"psImplementationVersion: ");
	DBVprint_inter_string(f, psInitializeRsp->psImplementationVersion);
}
 
 
int DBVprint_init_response_userinfo(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
       	fprintf(f,"psUserInformationField: ");
	DBVprint_external(f, psInitializeRsp->psUserInformationField);
}



int DBVprint_init_response_otherinfo(FILE *f, DbvInitializeRsp *psInitializeRsp)
{
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psInitializeRsp->ppsOtherInformation);
}



/***************************************************************/ 
/* SearchResponse */


int DBVprint_searchrsp_fields(FILE *f, DbvSearchRsp *psSearchRsp)
{
        fprintf(f,"SearchResponse PDU\n");

	fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psSearchRsp->psReferenceId);

	fprintf(f,"ulResultCount: %d\n", psSearchRsp->ulResultCount);
	fprintf(f,"ulNumberOfRecordsReturned: %d\n", psSearchRsp->ulNumberOfRecordsReturned);
	fprintf(f,"ulNextResultSetPosition: %d\n", psSearchRsp->ulNextResultSetPosition);

	fprintf(f,"fSearchStatus: ");
        DBVprint_bool(f, &psSearchRsp->fSearchStatus);

	DBVprint_DbvResultSetStatus(f, psSearchRsp->peResultSetStatus);

	fprintf(f,"pePresentStatus: ");
	if (psSearchRsp->pePresentStatus != NULL)
		DBVprint_DbvPresentStatus(f, psSearchRsp->pePresentStatus);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"psRecords: \n");
	DBVprint_records(f, psSearchRsp->psRecords);
	
        fprintf(f,"ppsAdditionalSearchInfo: ");
        DBVprint_otherinfo(f, psSearchRsp->ppsAdditionalSearchInfo);
 
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psSearchRsp->ppsOtherInformation);

}



int DBVprint_DbvResultSetStatus(FILE *f, DbvResultSetStatus *peResultSetStatus)
{
	fprintf(f,"peResultSetStatus: ");
	if(peResultSetStatus != NULL)
	{
		switch(*peResultSetStatus)
		{
		case RSS_Subset:
			fprintf(f,"RSS_Subset\n");
			break;
		case RSS_Interim:
			fprintf(f,"RSS_Interim\n");
			break;
		case RSS_None:
			fprintf(f,"RSS_None\n");
			break;
		case RSS_Invalid:
			fprintf(f,"RSS_Invalid\n");
			break;
		}
	}
	else
		fprintf(f,"NULL\n");
}



/***************************************************************/ 
/* PresentResponse */


int DBVprint_presrsp_fields(FILE *f, DbvPresentRsp *psPresentRsp)
{
        fprintf(f,"PresentResponse PDU\n");
 
        DBVprint_presrsp_refid(f, psPresentRsp);
        DBVprint_presrsp_numrecs(f, psPresentRsp);
        DBVprint_presrsp_nxtresset(f, psPresentRsp);
        DBVprint_presrsp_presstat(f, psPresentRsp);
        DBVprint_presrsp_records(f, psPresentRsp);
	DBVprint_presrsp_otherinfo(f, psPresentRsp);
}



int DBVprint_presrsp_refid(FILE *f, DbvPresentRsp *psPresentRsp)
{
 
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psPresentRsp->psReferenceId);
}




int DBVprint_presrsp_numrecs(FILE *f, DbvPresentRsp *psPresentRsp)
{
	fprintf(f,"ulNumberOfRecordsReturned: %d\n", psPresentRsp->ulNumberOfRecordsReturned);

}



int DBVprint_presrsp_nxtresset(FILE *f, DbvPresentRsp *psPresentRsp)
{
	fprintf(f,"ulNextResultSetPosition: %d\n", psPresentRsp->ulNextResultSetPosition);
}



int DBVprint_presrsp_presstat(FILE *f, DbvPresentRsp *psPresentRsp)
{
        fprintf(f,"ePresentStatus: ");
			
	DBVprint_DbvPresentStatus(f, &psPresentRsp->ePresentStatus);
}



int DBVprint_presrsp_records(FILE *f, DbvPresentRsp *psPresentRsp)
{
	fprintf(f,"psRecords: ");
	DBVprint_records(f, psPresentRsp->psRecords);
}




int DBVprint_presrsp_otherinfo(FILE *f, DbvPresentRsp *psPresentRsp)
{
 
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psPresentRsp->ppsOtherInformation);
}
 
 

/***************************************************************/ 
/* DeleteResultSetResponse */


int DBVprint_delrsp_fields(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
        fprintf(f,"DeleteResultSetResponse PDU\n");

        DBVprint_delete_resp_refid(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_opstat(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_dellistat(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_numnotdel(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_bulkstat(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_delmess(f, psDeleteResultSetRsp);
        DBVprint_delete_resp_otherinfo(f, psDeleteResultSetRsp);
 
}


int DBVprint_delete_resp_refid(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psDeleteResultSetRsp->psReferenceId);

}



int DBVprint_delete_resp_opstat(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	fprintf(f,"eDeleteOperationStatus: ");
	DBVprint_opstat(f, psDeleteResultSetRsp->eDeleteOperationStatus);
}



int DBVprint_opstat(FILE *f, DbvDeleteSetStatus eDeleteOperationStatus)
{
        switch(eDeleteOperationStatus) {
	case DSS_Success:
		fprintf(f,"DSS_Success\n");
		break;
	case DSS_ResultSetDidNotExist:
		fprintf(f,"DSS_ResultSetDidNotExist\n");
		break;
	case DSS_PreviouslyDeletedByTarget:
		fprintf(f,"DSS_PreviouslyDeletedByTarget\n");
		break;
	case DSS_SystemProblemAtTarget:
		fprintf(f,"DSS_SystemProblemAtTarget\n");
		break;
	case DSS_AccessNotAllowed:
		fprintf(f,"DSS_AccessNotAllowed\n");
		break;
	case DSS_ResourceControlAtOrigin:
		fprintf(f,"DSS_ResourceControlAtOrigin\n");
		break;
	case DSS_ResourceControlAtTarget:
		fprintf(f,"DSS_ResourceControlAtTarget\n");
		break;
	case DSS_BulkDeleteNotSupported:
		fprintf(f,"DSS_BulkDeleteNotSupported\n");
		break;
	case DSS_NotAllSetsDeletedOnBulkDlt:
		fprintf(f,"DSS_NotAllSetsDeletedOnBulkDlt\n");
		break;
	case DSS_NotAllRequestedSetsDeleted:
		fprintf(f,"DSS_NotAllRequestedSetsDeleted\n");
		break;
	case DSS_ResultSetInUse:
		fprintf(f,"DSS_ResultSetInUse\n");
		break;
	case DSS_Invalid:
		fprintf(f,"DSS_Invalid\n");
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_delete_resp_dellistat(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	int index = 0;

	fprintf(f,"ppsDeleteListStatuses: ");
	if (psDeleteResultSetRsp->ppsDeleteListStatuses != NULL) {
		fprintf(f,"\n");

		while( psDeleteResultSetRsp->ppsDeleteListStatuses[index] != NULL) {
			DBVprint_liststat(f, psDeleteResultSetRsp->ppsDeleteListStatuses[index]);
			index++;
		}
		fprintf(f,"\n");
		
	} else
		fprintf(f,"NULL\n");
				
}



int DBVprint_liststat(FILE *f, DbvListStatus *psDeleteListStatuses)
{
	/* Both the following are compulsory */
	fprintf(f,"sResultSetId: ");
	DBVprint_inter_string(f, &psDeleteListStatuses->sResultSetId);

	fprintf(f,"eDeleteSetStatus: ");
	DBVprint_opstat(f, psDeleteListStatuses->eDeleteSetStatus);
}



int DBVprint_delete_resp_numnotdel(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	fprintf(f,"pulNumberNotDeleted: ");
	if (psDeleteResultSetRsp->pulNumberNotDeleted != NULL) 
		fprintf(f,"%lu\n", *psDeleteResultSetRsp->pulNumberNotDeleted); 
	else
		fprintf(f,"NULL\n");
}



int DBVprint_delete_resp_bulkstat(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	int index = 0;

	fprintf(f,"ppsBulkStatuses: ");
	if (psDeleteResultSetRsp->ppsBulkStatuses != NULL) {
		fprintf(f,"\n");

		while( psDeleteResultSetRsp->ppsBulkStatuses[index] != NULL) {
			DBVprint_liststat(f, psDeleteResultSetRsp->ppsBulkStatuses[index]);
			index++;
		}
		fprintf(f,"\n");
		
	} else
		fprintf(f,"NULL\n");
}



int DBVprint_delete_resp_delmess(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
	fprintf(f,"psDeleteMessage: ");
	DBVprint_inter_string(f, psDeleteResultSetRsp->psDeleteMessage);
}



int DBVprint_delete_resp_otherinfo(FILE *f, DbvDeleteResultSetRsp *psDeleteResultSetRsp)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psDeleteResultSetRsp->ppsOtherInformation);

}



/***************************************************************/ 
/* ResourceReportResponse */


int DBVprint_resreprsp_fields(FILE *f, DbvResourceReportRsp *psResourceReportRsp)
{
        fprintf(f,"ResourceReportResponse PDU\n");
 
        DBVprint_resrep_resp_refid(f, psResourceReportRsp);
        DBVprint_resrep_resp_repstat(f, psResourceReportRsp);
        DBVprint_resrep_resp_resrep(f, psResourceReportRsp);
        DBVprint_resrep_resp_otherinfo(f, psResourceReportRsp);
}



int DBVprint_resrep_resp_refid(FILE *f, DbvResourceReportRsp *psResourceReportRsp)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psResourceReportRsp->psReferenceId);
}



int DBVprint_resrep_resp_repstat(FILE *f, DbvResourceReportRsp *psResourceReportRsp)
{
	fprintf(f,"eResourceReportStatus: ");
	switch( psResourceReportRsp->eResourceReportStatus) {
	case RRS_Success:
		fprintf(f,"RRS_Success\n");
		break;
	case RRS_Partial:
		fprintf(f,"RRS_Partial\n");
		break;
	
		fprintf(f,"RRS_Failure_1\n");
		break;
	case RRS_Failure_2:
		fprintf(f,"RRS_Failure_2\n");
		break;
	case RRS_Failure_3:
		fprintf(f,"RRS_Failure_3\n");
		break;
	case RRS_Failure_4:
		fprintf(f,"RRS_Failure_4\n");
		break;
	case RRS_Failure_5:
		fprintf(f,"RRS_Failure_5\n");
		break;
	case RRS_Failure_6:
		fprintf(f,"RRS_Failure_6\n");
		break;
	case RRS_Invalid:
		fprintf(f,"RRS_Invalid\n");
		break;
	}
}



int DBVprint_resrep_resp_resrep(FILE *f, DbvResourceReportRsp *psResourceReportRsp)
{
	fprintf(f,"psResourceReport: ");
	DBVprint_external(f, psResourceReportRsp->psResourceReport);
}



int DBVprint_resrep_resp_otherinfo(FILE *f, DbvResourceReportRsp *psResourceReportRsp)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psResourceReportRsp->ppsOtherInformation);

}



/***************************************************************/ 
/* ScanResponse */


int DBVprint_scanrsp_fields(FILE *f, DbvScanRsp *psScanRsp)
{
        fprintf(f,"ScanResponse PDU\n");
 
        DBVprint_scan_resp_refid(f, psScanRsp);
        DBVprint_scan_resp_stepsz(f, psScanRsp);
        DBVprint_scan_resp_scanstat(f, psScanRsp);
        DBVprint_scan_resp_nument(f, psScanRsp);
        DBVprint_scan_resp_posterm(f, psScanRsp);
        DBVprint_scan_resp_entries(f, psScanRsp);
        DBVprint_scan_resp_attsetid(f, psScanRsp);
        DBVprint_scan_resp_otherinfo(f, psScanRsp);
}



int DBVprint_scan_resp_refid(FILE *f, DbvScanRsp *psScanRsp)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psScanRsp->psReferenceId);
}
 
 
int DBVprint_scan_resp_stepsz(FILE *f, DbvScanRsp *psScanRsp)
{
	fprintf(f,"pulStepSize: ");
	if (psScanRsp->pulStepSize != NULL)
		fprintf(f,"%lu\n", *psScanRsp->pulStepSize);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_scan_resp_scanstat(FILE *f, DbvScanRsp *psScanRsp)
{
	/* Compulsory */
	fprintf(f,"eScanStatus: ");
	switch( psScanRsp->eScanStatus) {
	case SCS_Success:
		fprintf(f,"SCS_Success\n");
		break;
	case SCS_Partial_1:
		fprintf(f,"SCS_Partial_1\n");
		break;
	case SCS_Partial_2:
		fprintf(f,"SCS_Partial_2\n");
		break;
	case SCS_Partial_3:
		fprintf(f,"SCS_Partial_3\n");
		break;
	case SCS_Partial_4:
		fprintf(f,"SCS_Partial_4\n");
		break;
	case SCS_Partial_5:
		fprintf(f,"SCS_Partial_5\n");
		break;
	case SCS_Failure:
		fprintf(f,"SCS_Failure\n");
		break;
	case SCS_Invalid:
		fprintf(f,"SCS_Invalid\n");
		break;
	}
}



int DBVprint_scan_resp_nument(FILE *f, DbvScanRsp *psScanRsp)
{
	fprintf(f,"ulNumberOfEntriesReturned: ");
	fprintf(f,"%lu\n", psScanRsp->ulNumberOfEntriesReturned);
}



int DBVprint_scan_resp_posterm(FILE *f, DbvScanRsp *psScanRsp)
{
	fprintf(f,"pulPositionOfTerm: ");
	if (psScanRsp->pulPositionOfTerm != NULL)
		fprintf(f,"%lu\n", *psScanRsp->pulPositionOfTerm);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_scan_resp_entries(FILE *f, DbvScanRsp *psScanRsp)
{
	fprintf(f,"psEntries: ");
	if (psScanRsp->psEntries != NULL) 
		DBVprint_listentries(f, psScanRsp->psEntries);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_listentries(FILE *f, DbvListEntries *psEntries)
{
	fprintf(f,"\n");

	fprintf(f,"ppsEntry: ");
	if (psEntries->ppsEntry != NULL) 
		DBVprint_entry_list(f, psEntries->ppsEntry);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"ppsNonSurrogateDiagnostics: ");
	if (psEntries->ppsNonSurrogateDiagnostics != NULL) 
		DBVprint_diagrec_list(f, psEntries->ppsNonSurrogateDiagnostics);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_entry_list(FILE *f, DbvEntry **ppsEntry)
{
	int index = 0;

	if (ppsEntry != NULL) {
		while( ppsEntry[index] != NULL) {
			switch( ppsEntry[index]->eEntryType) {
			case ET_TermInfo:
				fprintf(f,"ET_TermInfo:\n");
				DBVprint_terminfo(f, ppsEntry[index]->uEntry.psTermInfo);
				break;
			case ET_SurrogateDiagnostic:
				fprintf(f,"ET_SurrogateDiagnostic:\n");
				DBVprint_diag_rec(f, ppsEntry[index]->uEntry.psSurrogateDiagnostic);
				break;
			}
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_terminfo(FILE *f, DbvTermInfo *psTermInfo)
{
	/* Term is compulsory */
	fprintf(f,"sTerm: ");
	DBVprint_att_term(f, psTermInfo->sTerm);

	fprintf(f,"psDisplayTerm: ");
	if (psTermInfo->psDisplayTerm != NULL)
		DBVprint_inter_string(f, psTermInfo->psDisplayTerm);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"ppsSuggestedAttribute: ");
	if (psTermInfo->ppsSuggestedAttribute != NULL)
		DBVprint_attelmnt_list(f, psTermInfo->ppsSuggestedAttribute);
	else 
		fprintf(f,"NULL\n");
	

	fprintf(f,"\nppsAlternativeTerm: ");
	if (psTermInfo->ppsAlternativeTerm != NULL)
		DBVprint_attsplusterm_list(f, psTermInfo->ppsAlternativeTerm);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"pulGlobalOccurrences: ");
	if (psTermInfo->pulGlobalOccurrences != NULL)
		fprintf(f,"%lu\n", *psTermInfo->pulGlobalOccurrences);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"ppsByAttributes: ");
	if (psTermInfo->ppsByAttributes != NULL)
		DBVprint_occurrbyatt_list(f, psTermInfo->ppsByAttributes);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"ppsOtherInformation: ");
	if (psTermInfo->ppsOtherInformation != NULL)
		DBVprint_otherinfo(f, psTermInfo->ppsOtherInformation);
	else
		fprintf(f,"NULL\n");

}



int DBVprint_attsplusterm_list(FILE *f, DbvAttributesPlusTerm **ppsAlternativeTerm)
{
	int index = 0;

	if (ppsAlternativeTerm != NULL) {
		while( ppsAlternativeTerm[index] != NULL) {
			DBVprint_attsplusterm(f, ppsAlternativeTerm[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_occurrbyatt_list(FILE *f, DbvOccurrenceByAttributes **ppsByAttributes)
{
	int index = 0;

	if (ppsByAttributes != NULL) {
		while( ppsByAttributes[index] != NULL) {
			DBVprint_occurrbyatt(f, ppsByAttributes[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}




int DBVprint_occurrbyatt(FILE *f, DbvOccurrenceByAttributes *psByAttributes)
{
	/* ppsAttributeElement is compulsory */
	fprintf(f,"\nppsAttributeElement: ");
	DBVprint_attelmnt_list(f, psByAttributes->ppsAttributeElement);

	fprintf(f,"\npsOccurrence: ");
	if (psByAttributes->psOccurrence != NULL)
		DBVprint_occurrence(f, psByAttributes->psOccurrence);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"ppsOtherInformation: ");
	if (psByAttributes->ppsOtherInformation != NULL)
		DBVprint_otherinfo(f, psByAttributes->ppsOtherInformation);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_occurrence(FILE *f, DbvOccurrence *psOccurrence)
{
	switch( psOccurrence->eOccurrenceType) {
	case OT_Global:
		fprintf(f,"OT_Global: ");
		fprintf(f,"%lu\n", psOccurrence->uOccurence.ulGlobal);
		break;
	case OT_ByDatabase:
		fprintf(f,"OT_ByDatabase:\n");
		DBVprint_occurbydb_list(f, psOccurrence->uOccurence.ppsOccurrenceByDb);
		break;
	}
}



int DBVprint_occurbydb_list(FILE *f, DbvOccurrenceByDb **ppsOccurrenceByDb)
{
	int index = 0;

	if (ppsOccurrenceByDb != NULL) {
		while( ppsOccurrenceByDb[index] != NULL) {
			DBVprint_occurbydb(f, ppsOccurrenceByDb[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}


int DBVprint_occurbydb(FILE *f, DbvOccurrenceByDb *psOccurrenceByDb)
{
	/* sDatabaseName is compulsory */
	fprintf(f,"sDatabaseName:");
	DBVprint_inter_string(f, &psOccurrenceByDb->sDatabaseName);

	fprintf(f,"pulNumber: ");
	if (psOccurrenceByDb->pulNumber != NULL)
		fprintf(f,"%lu\n", *psOccurrenceByDb->pulNumber);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"ppsOtherInformation: ");
	if (psOccurrenceByDb->ppsOtherInformation != NULL)
		DBVprint_otherinfo(f, psOccurrenceByDb->ppsOtherInformation);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_scan_resp_attsetid(FILE *f, DbvScanRsp *psScanRsp)
{
	fprintf(f,"pcAttributeSetId: ");
	if (psScanRsp->pcAttributeSetId != NULL)
		fprintf(f,"%s\n", psScanRsp->pcAttributeSetId);
	else
		fprintf(f,"NULL\n");
}




int DBVprint_scan_resp_otherinfo(FILE *f, DbvScanRsp *psScanRsp)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psScanRsp->ppsOtherInformation);
 
}



/***************************************************************/ 
/* SortResponse */


int DBVprint_sortrsp_fields(FILE *f, DbvSortRsp *psSortRsp)
{
        fprintf(f,"SortResponse PDU\n");
 
        DBVprint_sort_resp_refid(f, psSortRsp);
        DBVprint_sort_resp_status(f, psSortRsp);
        DBVprint_sort_resp_ressetstat(f, psSortRsp);
        DBVprint_sort_resp_diags(f, psSortRsp);
        DBVprint_sort_resp_otherinfo(f, psSortRsp);
}



int DBVprint_sort_resp_refid(FILE *f, DbvSortRsp *psSortRsp)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psSortRsp->psReferenceId);
}
 

 
int DBVprint_sort_resp_status(FILE *f, DbvSortRsp *psSortRsp)
{
	fprintf(f,"eSortStatus: ");
	switch( psSortRsp->eSortStatus) {
	case SOS_Success:
		fprintf(f,"SOS_Success\n");
		break;
	case SOS_Partial_1:
		fprintf(f,"SOS_Success\n");
		break;
	case SOS_Failure:
		fprintf(f,"SOS_Success\n");
		break;
	case SOS_Invalid:
		fprintf(f,"SOS_Invalid\n");
		break;
	}
	
}



int DBVprint_sort_resp_ressetstat(FILE *f, DbvSortRsp *psSortRsp)
{
	fprintf(f,"peResultSetStatus: ");
	if (psSortRsp->peResultSetStatus != NULL) {
		switch (*psSortRsp->peResultSetStatus) {
		case SRSS_Empty:
			fprintf(f,"SRSS_Empty\n");
			break;
		case SRSS_Interim:
			fprintf(f,"SRSS_Interim\n");
			break;
		case SRSS_Unchanged:
			fprintf(f,"SRSS_Unchanged\n");
			break;
		case SRSS_None:
			fprintf(f,"SRSS_None\n");
			break;
		case SRSS_Invalid:
			fprintf(f,"SRSS_Invalid\n");
			break;
		}
	} else
		fprintf(f,"NULL\n");
}



int DBVprint_sort_resp_diags(FILE *f, DbvSortRsp *psSortRsp)
{
	fprintf(f,"ppsDiagnostics: ");

	if (psSortRsp->ppsDiagnostics != NULL) 
		DBVprint_diagrec_list(f, psSortRsp->ppsDiagnostics);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_sort_resp_otherinfo(FILE *f, DbvSortRsp *psSortRsp)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psSortRsp->ppsOtherInformation);
 
}
 


/***************************************************************/ 
/* ExtendedServicesResponse */

 
int DBVprint_extendrsp_fields(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
        fprintf(f,"ExtendServicesResponse PDU\n");

        DBVprint_extend_resp_refid(f, psExtendedServicesRsp);
        DBVprint_extend_resp_opstat(f, psExtendedServicesRsp);
        DBVprint_extend_resp_diags(f, psExtendedServicesRsp);
        DBVprint_extend_resp_taskpack(f, psExtendedServicesRsp);
        DBVprint_extend_resp_otherinfo(f, psExtendedServicesRsp);
}



int DBVprint_extend_resp_refid(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psExtendedServicesRsp->psReferenceId);
}
 
 
int DBVprint_extend_resp_opstat(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
	fprintf(f,"eOperationStatus: ");
	switch( psExtendedServicesRsp->eOperationStatus) {
	case ESOS_Done:
		fprintf(f,"ESOS_Done\n");
		break;
	case ESOS_Queued:
		fprintf(f,"ESOS_Queued\n");
		break;
	case ESOS_Failure:
		fprintf(f,"ESOS_Failure\n");
		break;
	case ESOS_Invalid:
		fprintf(f,"ESOS_Invalid\n");
		break;
	}
}



int DBVprint_extend_resp_diags(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
	fprintf(f,"ppsDiagnostics: ");
	if (psExtendedServicesRsp->ppsDiagnostics != NULL)
		DBVprint_diagrec_list(f, psExtendedServicesRsp->ppsDiagnostics);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_extend_resp_taskpack(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
	fprintf(f,"psTaskPackage: ");
	DBVprint_external(f, psExtendedServicesRsp->psTaskPackage);
}


 
int DBVprint_extend_resp_otherinfo(FILE *f, DbvExtendedServicesRsp *psExtendedServicesRsp)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psExtendedServicesRsp->ppsOtherInformation)
;
 
}
 



/***************************************************************/ 
/* Close (CloseRequest/CloseResponse) */


int DBVprint_closereq_fields(FILE *f, DbvCloseReq  *psCloseReq)
{
        fprintf(f,"Close PDU\n");
 
        DBVprint_close_request_refid(f, psCloseReq);
        DBVprint_close_request_reason(f, psCloseReq);
        DBVprint_close_request_diaginfo(f, psCloseReq);
        DBVprint_close_request_resrepfmt(f, psCloseReq);
        DBVprint_close_request_resrep(f, psCloseReq);
        DBVprint_close_request_otherinfo(f, psCloseReq);
 
}



int DBVprint_close_request_refid(FILE *f, DbvCloseReq *psCloseReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psCloseReq->psReferenceId);
}
 


int DBVprint_close_request_reason(FILE *f, DbvCloseReq *psCloseReq)
{
	fprintf(f,"eCloseReason: ");
	switch( psCloseReq->eCloseReason) {
	case CR_Finished:
		fprintf(f,"CR_Finished\n");
		break;
	case CR_ShutDown:
		fprintf(f,"CR_ShutDown\n");
		break;
	case CR_SystemProblem:
		fprintf(f,"CR_SystemProblem\n");
		break;
	case CR_CostLimit:
		fprintf(f,"CR_CostLimit\n");
		break;
	case CR_Resources:
		fprintf(f,"CR_Resources\n");
		break;
	case CR_SecurityViolation:
		fprintf(f,"CR_SecurityViolation\n");
		break;
	case CR_ProtocolError:
		fprintf(f,"CR_ProtocolError\n");
		break;
	case CR_LackOfActivity:
		fprintf(f,"CR_LackOfActivity\n");
		break;
	case CR_PeerAbort:
		fprintf(f,"CR_PeerAbort\n");
		break;
	case CR_Unspecified:
		fprintf(f,"CR_Unspecified\n");
		break;
	case CR_Invalid:
		fprintf(f,"CR_Invalid\n");
		break;
	}
}



int DBVprint_close_request_diaginfo(FILE *f, DbvCloseReq *psCloseReq)
{
	fprintf(f,"psDiagnosticInformation: ");
	DBVprint_inter_string(f, psCloseReq->psDiagnosticInformation);
}



int DBVprint_close_request_resrepfmt(FILE *f, DbvCloseReq *psCloseReq)
{
	fprintf(f,"pcResourceReportFormat: ");

	if (psCloseReq->pcResourceReportFormat != NULL) 
		fprintf(f,"%s\n", psCloseReq->pcResourceReportFormat);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_close_request_resrep(FILE *f, DbvCloseReq *psCloseReq)
{
	fprintf(f,"psResourceReport: ");
	DBVprint_external(f, psCloseReq->psResourceReport);
}



int DBVprint_close_request_otherinfo(FILE *f, DbvCloseReq *psCloseReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psCloseReq->ppsOtherInformation);
 
}
 
 

/***************************************************************/ 
/* InitializeRequest */
 

int DBVprint_initreq_fields(FILE *f, DbvInitializeReq *psInitializeReq)
{
        fprintf(f,"InitRequest PDU\n");
 
	DBVprint_init_request_refid(f, psInitializeReq);
	DBVprint_init_request_prover(f, psInitializeReq);
	DBVprint_init_request_options(f, psInitializeReq);
	DBVprint_init_request_prefmess(f, psInitializeReq);
	DBVprint_init_request_maxmess(f, psInitializeReq);
	DBVprint_init_request_idauth(f, psInitializeReq);
	DBVprint_init_request_impid(f, psInitializeReq);
	DBVprint_init_request_impname(f, psInitializeReq);
	DBVprint_init_request_impver(f, psInitializeReq);
	DBVprint_init_request_userinfo(f, psInitializeReq);
	DBVprint_init_request_otherinfo(f, psInitializeReq);
}



int DBVprint_init_request_refid(FILE *f, DbvInitializeReq *psInitializeReq)
{

        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psInitializeReq->psReferenceId); 
}



int DBVprint_init_request_prover(FILE *f, DbvInitializeReq *psInitializeReq)
{
 
        fprintf(f,"pcProtocolVersion: %s\n", psInitializeReq->pcProtocolVersion);
}



int DBVprint_init_request_options(FILE *f, DbvInitializeReq *psInitializeReq)
{
        fprintf(f,"pcOptions: %s\n", psInitializeReq->pcOptions);
}



int DBVprint_init_request_prefmess(FILE *f, DbvInitializeReq *psInitializeReq)
{
        fprintf(f,"ulPreferredMessageSize: %ld\n", psInitializeReq->ulPreferredMessageSize);
}


int DBVprint_init_request_maxmess(FILE *f, DbvInitializeReq *psInitializeReq)
{
        fprintf(f,"ulMaxmiumMessageSize: %ld\n", psInitializeReq->ulMaximumMessageSize);
}


int DBVprint_init_request_idauth(FILE *f, DbvInitializeReq *psInitializeReq)
{

	/* Id Authentication */
	fprintf(f,"psAuthentication: ");
	if (psInitializeReq->psAuthentication != NULL) {
		switch( psInitializeReq->psAuthentication->eAuthenticationType)
		{
		case AT_Open:
			DBVprint_initreq_idauth_open(f, psInitializeReq);
			break;
		case AT_Anonymous:
			fprintf(f,"Anonymous\n");
			break;
		case AT_IdPassword:
			DBVprint_initreq_idauth_pass(f, psInitializeReq);
			break;
		case AT_Other:
			DBVprint_initreq_idauth_other(f, psInitializeReq);
			break;
		default:
			fprintf(f,"Unknown id auth\n");
			break;
		}
	} else {
		fprintf(f,"NULL\n");
	}
}



int DBVprint_initreq_idauth_open(FILE *f, DbvInitializeReq *psInitializeReq)
{
	fprintf(f,"Open\n");
	fprintf(f,"\tpsOpen: %s\n", psInitializeReq->psAuthentication->uAuthentication.psOpen);
}



int DBVprint_initreq_idauth_pass(FILE *f, DbvInitializeReq *psInitializeReq)
{

	fprintf(f,"IdPassword\n");

	/* Group id */
	fprintf(f,"\tpsGroupId: ");
	if (psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psGroupId != NULL)
		DBVprint_inter_string(f, psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psGroupId);
	else
		fprintf(f,"NULL\n");
	

	/* user id  */
	fprintf(f,"\tpsUserId: ");
	if (psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psUserId != NULL)
		DBVprint_inter_string(f, psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psUserId);
	else
		fprintf(f,"NULL\n");


	/* password */
	fprintf(f,"\tpsPassword: ");
	if (psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psPassword != NULL)
		DBVprint_inter_string(f, psInitializeReq->psAuthentication->uAuthentication.psIdPassword->psPassword);
	else
		fprintf(f,"NULL\n");
}





int DBVprint_initreq_idauth_other(FILE *f, DbvInitializeReq *psInitializeReq)
{
	/* Dont need a \n at the end of the folllowing because a \n
	   is output by DBVprint_external */
	fprintf(f,"psOther");
	DBVprint_external(f, psInitializeReq->psAuthentication->uAuthentication.psOther);
}



int DBVprint_init_request_impid(FILE *f, DbvInitializeReq *psInitializeReq)
{
	fprintf(f,"psImplementationId: ");
        DBVprint_inter_string(f, psInitializeReq->psImplementationId);
}



int DBVprint_init_request_impname(FILE *f, DbvInitializeReq *psInitializeReq)
{
	fprintf(f,"psImplementationName: ");
        DBVprint_inter_string(f, psInitializeReq->psImplementationName);
}



int DBVprint_init_request_impver(FILE *f, DbvInitializeReq *psInitializeReq)
{
	fprintf(f,"psImplementationVersion: ");
        DBVprint_inter_string(f, psInitializeReq->psImplementationVersion);

}
 
 

int DBVprint_init_request_userinfo(FILE *f, DbvInitializeReq *psInitializeReq)
{
	fprintf(f,"psUserInformationField: ");
	DBVprint_external(f, psInitializeReq->psUserInformationField);

}
 


int DBVprint_init_request_otherinfo(FILE *f, DbvInitializeReq *psInitializeReq)
{

	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psInitializeReq->ppsOtherInformation);
}



/***************************************************************/ 
/* SearchRequest */


int DBVprint_searchreq_fields(FILE *f, DbvSearchReq *psSearchReq)
{
	int index = 0;

        fprintf(f,"SearchRequest PDU\n");

	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psSearchReq->psReferenceId);
	fprintf(f,"ulSmallSetUpperBound: %d\n",psSearchReq->ulSmallSetUpperBound);

	fprintf(f,"ulLargeSetLowerBound: %d\n",psSearchReq->ulLargeSetLowerBound);

	fprintf(f,"ulMediumSetPresentNumber: %d\n",psSearchReq->ulMediumSetPresentNumber);

	fprintf(f,"fReplaceIndicator: ");
	DBVprint_bool(f, &psSearchReq->fReplaceIndicator);

	fprintf(f,"sResultSetName: ");
	DBVprint_inter_string(f, &psSearchReq->sResultSetName);

	fprintf(f,"ppsDatabaseNames:\n");

	if (psSearchReq->ppsDatabaseNames != NULL) {
		while( psSearchReq->ppsDatabaseNames[index] != NULL) {
			fprintf(f,"\t");
			DBVprint_inter_string(f, psSearchReq->ppsDatabaseNames[index]);
			index++;
		}
	}

	fprintf(f,"\n");

	fprintf(f,"psSmallSetElementNames:");
	DBVprint_DbvElementSetNames(f, psSearchReq->psSmallSetElementNames);

	fprintf(f,"psMediumSetElementNames:");
	DBVprint_DbvElementSetNames(f, psSearchReq->psMediumSetElementNames);

	fprintf(f,"pcPreferredRecordSyntax: ");
	if (psSearchReq->pcPreferredRecordSyntax == NULL)
		fprintf(f,"NULL\n");
	else
		fprintf(f,"%s\n",psSearchReq->pcPreferredRecordSyntax);

	fprintf(f,"psQuery: ");
	DBVprint_DbvQuery(f, psSearchReq->psQuery);

	fprintf(f,"ppsAdditionalSearchInfo: ");
	DBVprint_otherinfo(f, psSearchReq->ppsAdditionalSearchInfo);

	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psSearchReq->ppsOtherInformation);

}



int	DBVprint_DbvQuery(FILE *f, DbvQuery *psQuery)
{
	if (psQuery != NULL)
	{
		fprintf(f,"\n\teQueryType: ");
		switch( psQuery->eQueryType)
		{
		case QT_Private:
			fprintf(f,"QT_Private\n");
			fprintf(f,"\tpsPrivateQuery: ");
			DBVprint_octet_string(f, psQuery->uQuery.psPrivateQuery);
			break;
		case QT_Rpn:
			fprintf(f,"QT_Rpn\n");
			DBVprint_rpnquery(f, psQuery->uQuery.psRpnQuery);
			break;
		case QT_Iso8777:
			fprintf(f,"QT_Iso8777\n");
			fprintf(f,"\tpsIso8777Query: ");
			DBVprint_octet_string(f, psQuery->uQuery.psIso8777Query);
			break;
		case QT_Z3958:
			fprintf(f,"QT_Z3958\n");
			fprintf(f,"\tpsZ3958Query: ");
			DBVprint_octet_string(f, psQuery->uQuery.psZ3958Query);
			break;
		case QT_Erpn:
			fprintf(f,"QT_Erpn\n");
			DBVprint_rpnquery(f, psQuery->uQuery.psErpnQuery);
			break;
		case QT_NaturalLanguage:
			fprintf(f,"QT_NaturalLanguage\n");
			fprintf(f,"\tpsNaturalLanguageQuery: ");
			DBVprint_octet_string(f, psQuery->uQuery.psNaturalLanguageQuery);
			break;
		}
		fprintf(f,"\n");
	}
	else
		fprintf(f,"NULL\n");
}



int DBVprint_rpnquery(FILE *f, DbvRPNQuery *psRpnQuery)
{
	fprintf(f,"DbvRPNQuery pcAttributeSetId: %s\n", psRpnQuery->pcAttributeSetId);
	DBVprint_rpn_structure(f, &psRpnQuery->sRpnStructure);
}



int DBVprint_rpn_structure(FILE *f, DbvRPNStructure *psDbvRPNStructure)
{
	
	switch( psDbvRPNStructure->eRpnStructureType) {
	case RST_RpnLopRopOperator:
		fprintf(f,"\nRST_RpnLopRopOperator:\n");
		DBVprint_rpnlopropoperator(f, psDbvRPNStructure->uRPNStructure.psRpnLopRopOperator);
		break;
	case RST_RpnOperand:
		fprintf(f,"\nRST_RpnOperand:\n");
		DBVprint_rpnoperand(f, psDbvRPNStructure->uRPNStructure.psRpnOperand);
		break;
	}
}




int DBVprint_rpnlopropoperator(FILE *f, DbvRPNLopRopOperator *psDbvRPNLopRopOperator)
{

	if (psDbvRPNLopRopOperator->psLeftOperand != NULL) 
		DBVprint_rpn_structure(f, psDbvRPNLopRopOperator->psLeftOperand);

	if (psDbvRPNLopRopOperator->psRightOperand != NULL) 
		DBVprint_rpn_structure(f, psDbvRPNLopRopOperator->psRightOperand);

	DBVprint_operator(f, psDbvRPNLopRopOperator->sOperator);
}



int DBVprint_operator(FILE *f, DbvOperator sOperator)
{
	fprintf(f,"sOperator: ");
	switch (sOperator.eOperatorType) {
	case OT_Or:
		fprintf(f,"OR\n");
		break;
	case OT_And:
		fprintf(f,"AND\n");
		break;
	case OT_AndNot:
		fprintf(f,"ANDNOT\n");
		break;
	case OT_Proximity:
		fprintf(f,"PROX\n");
		DBVprint_proxoperator(f, sOperator.psProximityOperator);
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_proxoperator(FILE *f, DbvProximityOperator *psProximityOperator)
{
	
	fprintf(f,"pfExclusion: ");
	if (psProximityOperator->pfExclusion != NULL)
		DBVprint_bool(f, psProximityOperator->pfExclusion);
	else
		fprintf(f,"NULL\n");
		

	/* The rest of these are compulsory */
	fprintf(f,"ulDistance: %ld\n", psProximityOperator->ulDistance);

	fprintf(f,"fOrdered: ");
	DBVprint_bool(f, &psProximityOperator->fOrdered);


	fprintf(f,"eProximityRelationType: ");
	switch( psProximityOperator->eProximityRelationType) {
	case PRT_LessThan:
		fprintf(f,"PRT_LessThan\n");
		break;
	case PRT_LessThanOrEqual:
		fprintf(f,"PRT_LessThanOrEqual\n");
		break;
	case PRT_Equal:
		fprintf(f,"PRT_Equal\n");
		break;
	case PRT_GreaterThanOrEqual:
		fprintf(f,"PRT_GreaterThanOrEqual\n");
		break;
	case PRT_GreaterThan:
		fprintf(f,"PRT_GreaterThan\n");
		break;
	case PRT_NotEqual:
		fprintf(f,"PRT_NotEqual\n");
		break;
	case PRT_Invalid:
		fprintf(f,"PRT_Invalid\n");
		break;
	}

	DBVprint_proxunitcode(f, psProximityOperator->sProximityUnitCode);
	
}



int DBVprint_proxunitcode(FILE *f, DbvProximityUnitCode sProximityUnitCode)
{

	fprintf(f,"sProximityUnitCode:\n");

	switch (sProximityUnitCode.eProximityUnitType) {
	case PUT_Known:
		fprintf(f,"eProximityUnitType: PUT_Known\n");		
		
		switch (sProximityUnitCode.uProximityUnit.eKnownProximityUnit) {
		case KPU_Character:
			fprintf(f,"eKnownProximityUnit: KPU_Character\n");
			break;
		case KPU_Word:
			fprintf(f,"eKnownProximityUnit: KPU_Word\n");
			break;
		case KPU_Sentence:
			fprintf(f,"eKnownProximityUnit: KPU_Sentence\n");
			break;
		case KPU_Paragraph:
			fprintf(f,"eKnownProximityUnit: KPU_Paragraph\n");
			break;
		case KPU_Section:
			fprintf(f,"eKnownProximityUnit: KPU_Section\n");
			break;
		case KPU_Chapter:
			fprintf(f,"eKnownProximityUnit: KPU_Chapter\n");
			break;
		case KPU_Document:
			fprintf(f,"eKnownProximityUnit: KPU_Document\n");
			break;
		case KPU_Element:
			fprintf(f,"eKnownProximityUnit: KPU_Element\n");
			break;
		case KPU_SubElement:
			fprintf(f,"eKnownProximityUnit: KPU_SubElement\n");
			break;
		case KPU_ElementType:
			fprintf(f,"eKnownProximityUnit: KPU_ElementType\n");
			break;
		case KPU_Byte:
			fprintf(f,"eKnownProximityUnit: KPU_Byte\n");
			break;
		case KPU_Invalid:
			fprintf(f,"eKnownProximityUnit: KPU_Invalid\n");
			break;
		}

		break;
	case PUT_Private:
		fprintf(f,"eProximityUnitType: PUT_Private\n");		
		fprintf(f,"ulPrivate: %ld\n", sProximityUnitCode.uProximityUnit.ulPrivate);
		break;
	}
}



int DBVprint_rpnoperand(FILE *f, DbvOperand *psRpnOperand)
{

	switch( psRpnOperand->eOperandType) {
	case OT_AttributePlusTerm:
		DBVprint_attsplusterm(f, psRpnOperand->uOperand.psAttributesPlusTerm);
		break;
	case OT_ResultSetId:
		fprintf(f,"psResultSetId: ");
		DBVprint_inter_string(f, psRpnOperand->uOperand.psResultSetId);
		break;
	case OT_ResultSetPlusAttributes:
		DBVprint_resplusatts(f, psRpnOperand->uOperand.psResultSetPlusAttributes);
		break;
	}	

}






int DBVprint_resplusatts(FILE *f, DbvResultSetPlusAttributes *psResultSetPlusAttributes)
{
	int index = 0;

	/* Both compulsory */
	fprintf(f,"sResultSetId: \n");
	DBVprint_inter_string(f, &psResultSetPlusAttributes->sResultSetId);
	
	/* Print all attributes */

	if (psResultSetPlusAttributes->ppsAttributeElement != NULL) {
		while( psResultSetPlusAttributes->ppsAttributeElement[index] != NULL) {

			/* print the attribute element */
			DBVprint_att_element(f, psResultSetPlusAttributes->ppsAttributeElement[index]);
			index++;
		}
	}

	fprintf(f,"\n");

}



/***************************************************************/ 
/* PresentRequest */


int DBVprint_presreq_fields(FILE *f, DbvPresentReq *psPresentReq)
{
        fprintf(f,"PresentRequest PDU\n");
 
	DBVprint_pres_request_refid(f, psPresentReq);
	DBVprint_pres_request_ressetid(f, psPresentReq);
	DBVprint_pres_request_ressetstart(f, psPresentReq);
	DBVprint_pres_request_numrecsreq(f, psPresentReq);
	DBVprint_pres_request_addranges(f, psPresentReq);
	DBVprint_pres_request_reccomp(f, psPresentReq);
	DBVprint_pres_request_prefrecsyn(f, psPresentReq);
	DBVprint_pres_request_maxsegcnt(f, psPresentReq);
	DBVprint_pres_request_maxrecsze(f, psPresentReq);
	DBVprint_pres_request_maxsegsze(f, psPresentReq);
	DBVprint_pres_request_otherinfo(f, psPresentReq);
}



int DBVprint_pres_request_refid(FILE *f, DbvPresentReq *psPresentReq)
{

        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psPresentReq->psReferenceId); 
	
}


int DBVprint_pres_request_ressetid(FILE *f, DbvPresentReq *psPresentReq)
{
        fprintf(f,"sResultSetId: ");
        DBVprint_inter_string(f, &psPresentReq->sResultSetId); 
}



int DBVprint_pres_request_ressetstart(FILE *f, DbvPresentReq *psPresentReq)
{
       fprintf(f,"ulResultSetStartPoint: %d\n",psPresentReq->ulResultSetStartPoint);
}



int DBVprint_pres_request_numrecsreq(FILE *f, DbvPresentReq *psPresentReq)
{
       fprintf(f,"ulNumberOfRecordsRequested: %d\n",psPresentReq->ulNumberOfRecordsRequested);

}



int DBVprint_pres_request_addranges(FILE *f, DbvPresentReq *psPresentReq)
{
	/* o */
	DBVprintN_DbvRange(f, "ppsAdditionalRanges", psPresentReq->ppsAdditionalRanges);
}



/* Print out the record composition */
int DBVprint_pres_request_reccomp(FILE *f, DbvPresentReq *psPresentReq)
{
	/* o */
	DBVprint_DbvRecordComposition(f, psPresentReq->psRecordComposition);
}



int DBVprint_pres_request_prefrecsyn(FILE *f, DbvPresentReq *psPresentReq)
{
	fprintf(f,"pcPreferredRecordSyntax: ");
	if (psPresentReq->pcPreferredRecordSyntax != NULL)
		fprintf(f,"%s\n",psPresentReq->pcPreferredRecordSyntax);
	else
		fprintf(f,"NULL\n");

}



int DBVprint_pres_request_maxsegcnt(FILE *f, DbvPresentReq *psPresentReq)
{
	fprintf(f,"pulMaxSegmentCount: ");
	if (psPresentReq->pulMaxSegmentCount != NULL)
		fprintf(f,"%d\n", *psPresentReq->pulMaxSegmentCount);
	else
		fprintf(f,"NULL\n");

}



int DBVprint_pres_request_maxrecsze(FILE *f, DbvPresentReq *psPresentReq)
{
	fprintf(f,"pulMaxRecordSize: ");
	if (psPresentReq->pulMaxRecordSize != NULL)
		fprintf(f,"%d\n", *psPresentReq->pulMaxRecordSize);
	else
		fprintf(f,"NULL\n");

}



int DBVprint_pres_request_maxsegsze(FILE *f, DbvPresentReq *psPresentReq)
{

	fprintf(f,"pulMaxSegmentSize: ");
	if (psPresentReq->pulMaxSegmentSize != NULL)
		fprintf(f,"%d\n", *psPresentReq->pulMaxSegmentSize);
	else
		fprintf(f,"NULL\n");

}


int DBVprint_pres_request_otherinfo(FILE *f, DbvPresentReq *psPresentReq)
{

	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psPresentReq->ppsOtherInformation);
}



/***************************************************************/ 
/* DeleteResultSetRequest */
	
	
int DBVprint_delreq_fields(FILE *f, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
        fprintf(f,"DeleteResultSetRequest PDU\n");
 
        DBVprint_delete_request_refid(f, psDeleteResultSetReq);
        DBVprint_delete_request_delfunc(f, psDeleteResultSetReq);
        DBVprint_delete_request_ressetid(f, psDeleteResultSetReq);
        DBVprint_delete_request_otherinfo(f, psDeleteResultSetReq);
 
}



int DBVprint_delete_request_refid(FILE *f, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psDeleteResultSetReq->psReferenceId);
}



int DBVprint_delete_request_delfunc(FILE *f, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
	fprintf(f,"eDeleteFunction: ");
	switch( psDeleteResultSetReq->eDeleteFunction) {
	case DF_List:
		fprintf(f,"DF_List\n");
		break;
	case DF_All:
		fprintf(f,"DF_All\n");
		break;
	case DF_Invalid:
		fprintf(f,"DF_Invalid\n");
		break;
	}
		
}



int DBVprint_delete_request_ressetid(FILE *f, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
	int index = 0;

	fprintf(f,"ppsResultSetId: ");

	if (psDeleteResultSetReq->ppsResultSetId != NULL) {
		fprintf(f,"\n");

		while( psDeleteResultSetReq->ppsResultSetId[index] != NULL) {
			DBVprint_inter_string(f, psDeleteResultSetReq->ppsResultSetId[index]);
			index++;
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");

}



int DBVprint_delete_request_otherinfo(FILE *f, DbvDeleteResultSetReq *psDeleteResultSetReq)
{
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psDeleteResultSetReq->ppsOtherInformation);

}


	
/***************************************************************/ 
/* ResourceReportRequest */
	
int DBVprint_resrepreq_fields(FILE *f, DbvResourceReportReq *psResourceReportReq)
{
        fprintf(f,"ResourceReportRequest PDU\n");
 
        DBVprint_resrep_request_refid(f, psResourceReportReq);
        DBVprint_resrep_request_opid(f, psResourceReportReq);
        DBVprint_resrep_request_prefrepfmt(f, psResourceReportReq);
        DBVprint_resrep_request_otherinfo(f, psResourceReportReq);
}
	
	
int DBVprint_resrep_request_refid(FILE *f, DbvResourceReportReq *psResourceReportReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psResourceReportReq->psReferenceId);
 
}



int DBVprint_resrep_request_opid(FILE *f, DbvResourceReportReq *psResourceReportReq)
{
        fprintf(f,"psOperationId: ");
        DBVprint_octet_string(f, psResourceReportReq->psOperationId);
 
}



int DBVprint_resrep_request_prefrepfmt(FILE *f, DbvResourceReportReq *psResourceReportReq)
{
	fprintf(f,"pcPrefResourceReportFormat: ");
	if (psResourceReportReq->pcPrefResourceReportFormat != NULL)
		fprintf(f,"%s\n", psResourceReportReq->pcPrefResourceReportFormat);
	else
		fprintf(f,"NULL\n");	
}



int DBVprint_resrep_request_otherinfo(FILE *f, DbvResourceReportReq *psResourceReportReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psResourceReportReq->ppsOtherInformation);

}

 
/***************************************************************/ 
/* ScanRequest */

int DBVprint_scanreq_fields(FILE *f, DbvScanReq *psScanReq)
{
        fprintf(f,"ScanRequest PDU\n");
 
        DBVprint_scan_request_refid(f, psScanReq);
        DBVprint_scan_request_dbnames(f, psScanReq);
        DBVprint_scan_request_attsetid(f, psScanReq);
        DBVprint_scan_request_attplustrm(f, psScanReq);
        DBVprint_scan_request_stepsize(f, psScanReq);
        DBVprint_scan_request_numterms(f, psScanReq);
        DBVprint_scan_request_prefpos(f, psScanReq);
        DBVprint_scan_request_otherinfo(f, psScanReq);
 
}
	


int DBVprint_scan_request_refid(FILE *f, DbvScanReq *psScanReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psScanReq->psReferenceId);
}


int DBVprint_scan_request_dbnames(FILE *f, DbvScanReq *psScanReq)
{
	int index = 0;

	/* Compulsory */
	fprintf(f,"ppsDatabaseNames:\n");

	if (psScanReq->ppsDatabaseNames != NULL) {
        	while( psScanReq->ppsDatabaseNames[index] != NULL) {
                	DBVprint_inter_string(f, psScanReq->ppsDatabaseNames[index]);
                	index++;
        	}
	}
	fprintf(f,"\n");

}



int DBVprint_scan_request_attsetid(FILE *f, DbvScanReq *psScanReq)
{
	fprintf(f,"pcAttributeSetId: ");
	if (psScanReq->pcAttributeSetId != NULL) 
		fprintf(f,"%s\n", psScanReq->pcAttributeSetId);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_scan_request_attplustrm(FILE *f, DbvScanReq *psScanReq)
{
	/* Compulsory */
	fprintf(f,"sAttributesPlusTerm:\n");
	DBVprint_attsplusterm(f, &psScanReq->sAttributesPlusTerm);
	
}



int DBVprint_scan_request_stepsize(FILE *f, DbvScanReq *psScanReq)
{
	fprintf(f,"pulStepSize: ");
	if (psScanReq->pulStepSize != NULL)
		fprintf(f,"%lu\n", *psScanReq->pulStepSize);
	else 
		fprintf(f,"NULL\n");
}



int DBVprint_scan_request_numterms(FILE *f, DbvScanReq *psScanReq)
{
	/* Compulsory */
	fprintf(f,"ulNumberOfTermsRequested: ");
	fprintf(f,"%lu\n", psScanReq->ulNumberOfTermsRequested);
}



int DBVprint_scan_request_prefpos(FILE *f, DbvScanReq *psScanReq)
{
	fprintf(f,"pulPreferredPositionInResponse: ");
	if (psScanReq->pulPreferredPositionInResponse != NULL)
		fprintf(f,"%lu\n", *psScanReq->pulPreferredPositionInResponse);
	else 
		fprintf(f,"NULL\n");
}



int DBVprint_scan_request_otherinfo(FILE *f, DbvScanReq *psScanReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psScanReq->ppsOtherInformation);
 
}
 

	
/***************************************************************/ 
/* SortRequest */

 

int DBVprint_sortreq_fields(FILE *f, DbvSortReq *psSortReq)
{
        fprintf(f,"SortRequest PDU\n");
 
        DBVprint_sort_request_refid(f, psSortReq);
        DBVprint_sort_request_inresset(f, psSortReq);
        DBVprint_sort_request_sortedres(f, psSortReq);
        DBVprint_sort_request_sortseq(f, psSortReq);
        DBVprint_sort_request_otherinfo(f, psSortReq);
 
}


 
int DBVprint_sort_request_refid(FILE *f, DbvSortReq *psSortReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psSortReq->psReferenceId);
}
 
 

int DBVprint_sort_request_inresset(FILE *f, DbvSortReq *psSortReq)
{
	int index = 0;

	/* Compulsory */
	fprintf(f,"ppsInputResultSetName:\n");

	if (psSortReq->ppsInputResultSetName != NULL) {
        	while( psSortReq->ppsInputResultSetName[index] != NULL) {
			DBVprint_inter_string(f, psSortReq->ppsInputResultSetName[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_sort_request_sortedres(FILE *f, DbvSortReq *psSortReq)
{
	/* Compulsory */
	fprintf(f,"sSortedResultSetName: ");
	DBVprint_inter_string(f, &psSortReq->sSortedResultSetName);
}



int DBVprint_sort_request_sortseq(FILE *f, DbvSortReq *psSortReq)
{
	int index = 0;

	/* Compulsory */
	fprintf(f,"ppsSortSequence:\n");

	if (psSortReq->ppsSortSequence != NULL) {
		while( psSortReq->ppsSortSequence[index] != NULL) {
			DBVprint_sortkeyspec(f, psSortReq->ppsSortSequence[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_sortkeyspec(FILE *f, DbvSortKeySpec *psSortSequence)
{
	fprintf(f,"sSortElement: ");
	DBVprint_sortelement(f, &psSortSequence->sSortElement);
	
	fprintf(f,"\neSortRelationType: ");
	DBVprint_sortrelation(f, psSortSequence->eSortRelationType);

	fprintf(f,"eSortCaseType: ");
	DBVprint_sortcase(f, psSortSequence->eSortCaseType);

	fprintf(f,"psMVActionType: ");
	if (psSortSequence->psMVActionType != NULL)
		DBVprint_mvaction(f, psSortSequence->psMVActionType);
	else
		fprintf(f,"NULL\n");
	
}



int DBVprint_sortelement(FILE *f, DbvSortElement *psSortElement)
{
	switch( psSortElement->eSortElementType) {
	case SET_Generic:
		fprintf(f,"SET_Generic\n");
		DBVprint_sortkey(f, psSortElement->uSortElement.psGeneric);
		break;
	case SET_DbSpecific:
		fprintf(f,"SET_DbSpecific\n");
		DBVprint_dbspecific(f, psSortElement->uSortElement.ppsDatabaseSpecific);
		break;
	}
}



int DBVprint_sortkey(FILE *f, DbvSortKey *psGeneric)
{
	fprintf(f,"eSortKeyType: ");
	switch( psGeneric->eSortKeyType) {
	case SKT_SortField:
		fprintf(f,"SKT_SortField: ");
		DBVprint_inter_string(f, psGeneric->uSortElement.psSortField);
		break;
	case SKT_ElementSpec:
		fprintf(f,"SKT_ElementSpec:\n");
		DBVprint_DbvSpecification(f, "psElementSpec", psGeneric->uSortElement.psElementSpec);
		break;
	case SKT_SortAttributes:
		fprintf(f,"SKT_SortAttributes:\n");
		DBVprint_sortatts(f, psGeneric->uSortElement.psSortAttributes);
		break;
	}
}



int DBVprint_sortatts(FILE *f, DbvSortAttributes *psSortAttributes)
{
	fprintf(f,"pcAttributeSetId: %s\n", psSortAttributes->pcAttributeSetId);

	fprintf(f,"ppsAttributeElement:\n");
	DBVprint_attelmnt_list(f, psSortAttributes->ppsAttributeElement);
}



int DBVprint_dbspecific(FILE *f, DbvDbSpecificSortKey **ppsDatabaseSpecific)
{
	int index = 0;

	if (ppsDatabaseSpecific != NULL) {
		while( ppsDatabaseSpecific[index] != NULL) {
			DBVprint_specsortkey(f, ppsDatabaseSpecific[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_specsortkey(FILE *f, DbvDbSpecificSortKey *psDatabaseSpecific)
{
	fprintf(f,"sDatabaseName: ");
	DBVprint_inter_string(f, &psDatabaseSpecific->sDatabaseName);

	fprintf(f,"sDbSort: ");
	DBVprint_sortkey(f, &psDatabaseSpecific->sDbSort);
}



int DBVprint_sortrelation(FILE *f, DbvSortRelationType eSortRelationType)
{
	switch (eSortRelationType) {
	case SRT_Ascending:
		fprintf(f,"SRT_Ascending\n");
		break;
	case SRT_Descending:
		fprintf(f,"SRT_Descending\n");
		break;
	case SRT_FrequencyAsc:
		fprintf(f,"SRT_FrequencyAsc\n");
		break;
	case SRT_FrequencyDesc:
		fprintf(f,"SRT_FrequencyDesc\n");
		break;
	case SRT_Invalid:
		fprintf(f,"SRT_Invalid\n");
		break;
	}
}



int DBVprint_sortcase(FILE *f, DbvSortCaseType eSortCaseType)
{
	switch (eSortCaseType) {
	case SCT_CaseSensitive:
		fprintf(f,"SCT_CaseSensitive\n");
		break;
	case SCT_CaseInsensitive:
		fprintf(f,"SCT_CaseInsensitive\n");
		break;
	case SCT_Invalid:
		fprintf(f,"SCT_Invalid\n");
		break;
	}
}



int DBVprint_mvaction(FILE *f, DbvSortMVAction *psMVActionType)
{
	switch( psMVActionType->eSortMVActionType) {
	case SMT_Abort:
		fprintf(f,"SMT_Abort\n");
		break;
	case SMT_Null:
		fprintf(f,"SMT_Null\n");
		break;
	case SMT_Constant:
		fprintf(f,"SMT_Constant: ");
		DBVprint_octet_string(f, psMVActionType->psOctetString);
		break;
	}
	
}



int DBVprint_sort_request_otherinfo(FILE *f, DbvSortReq *psSortReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psSortReq->ppsOtherInformation);
 
}
 

/***************************************************************/ 
/* SegmentRequest  - sent from target to origin */
 

int DBVprint_segreq_fields(FILE *f, DbvSegmentReq *psSegmentReq)
{
        fprintf(f,"SegmentRequest PDU\n");
 
        DBVprint_segreq_refid(f, psSegmentReq);
        DBVprint_segreq_numrecs(f, psSegmentReq);
        DBVprint_segreq_segrecs(f, psSegmentReq);
        DBVprint_segreq_otherinfo(f, psSegmentReq);
}



int DBVprint_segreq_refid(FILE *f, DbvSegmentReq *psSegmentReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psSegmentReq->psReferenceId);
}
 
 

int DBVprint_segreq_numrecs(FILE *f, DbvSegmentReq *psSegmentReq)
{
	fprintf(f,"ulNumberOfRecordsReturned: %lu\n", psSegmentReq->ulNumberOfRecordsReturned);
}



int DBVprint_segreq_segrecs(FILE *f, DbvSegmentReq *psSegmentReq)
{
	fprintf(f,"ppsSegmentRecords:\n");
	DBVprint_nameplusrec(f, psSegmentReq->ppsSegmentRecords);
}



int DBVprint_segreq_otherinfo(FILE *f, DbvSegmentReq *psSegmentReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psSegmentReq->ppsOtherInformation);
}
 


/***************************************************************/ 
/* ExtendedServicesRequest */


int DBVprint_extendreq_fields(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
        fprintf(f,"ExtendedServicesRequest PDU\n");
 
        DBVprint_extend_request_refid(f, psExtendedServicesReq);
        DBVprint_extend_request_function(f, psExtendedServicesReq);
        DBVprint_extend_request_packtype(f, psExtendedServicesReq);
        DBVprint_extend_request_packname(f, psExtendedServicesReq);
        DBVprint_extend_request_userid(f, psExtendedServicesReq);
        DBVprint_extend_request_rettime(f, psExtendedServicesReq);
        DBVprint_extend_request_permiss(f, psExtendedServicesReq);
        DBVprint_extend_request_descrp(f, psExtendedServicesReq);
        DBVprint_extend_request_orspec(f, psExtendedServicesReq);
        DBVprint_extend_request_waction(f, psExtendedServicesReq);
        DBVprint_extend_request_elements(f, psExtendedServicesReq);
        DBVprint_extend_request_otherinfo(f, psExtendedServicesReq);
 
}



int DBVprint_extend_request_refid(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
        fprintf(f,"psReferenceId: ");
        DBVprint_octet_string(f, psExtendedServicesReq->psReferenceId);
}
 
 

int DBVprint_extend_request_function(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
        fprintf(f,"eFunction: ");
	switch( psExtendedServicesReq->eFunction) {
	case ESF_Create:
		fprintf(f,"ESF_Create\n");
		break;
	case ESF_Delete:
		fprintf(f,"ESF_Delete\n");
		break;
	case ESF_Modify:
		fprintf(f,"ESF_Modify\n");
		break;
	case ESF_Invalid:
		fprintf(f,"ESF_Invalid\n");
		break;
	}
}



int DBVprint_extend_request_packtype(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"pcPackageType: ");
	fprintf(f,"%s\n", psExtendedServicesReq->pcPackageType);
}



int DBVprint_extend_request_packname(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psPackageName: ");
	DBVprint_inter_string(f, psExtendedServicesReq->psPackageName);
}



int DBVprint_extend_request_userid(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psUserId: ");
	DBVprint_inter_string(f, psExtendedServicesReq->psUserId);
}



int DBVprint_extend_request_rettime(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psRetentionTime: ");
	if (psExtendedServicesReq->psRetentionTime != NULL) {
		fprintf(f,"\n");

		DBVprint_intunit(f, psExtendedServicesReq->psRetentionTime);
	} else
		fprintf(f,"NULL\n");
}



int DBVprint_extend_request_permiss(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"ppsPermissions: ");
	if (psExtendedServicesReq->ppsPermissions != NULL)
		DBVprint_permissions(f, psExtendedServicesReq->ppsPermissions);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_extend_request_descrp(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psDescription: ");
	DBVprint_inter_string(f, psExtendedServicesReq->psDescription);
}



int DBVprint_extend_request_orspec(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psOriginSpecificParameter: ");
	DBVprint_external(f, psExtendedServicesReq->psOriginSpecificParameter);
}



int DBVprint_extend_request_waction(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"eWaitAction: ");
	switch( psExtendedServicesReq->eWaitAction) {
	case ESWA_Wait:
		fprintf(f,"ESWA_Wait\n");
		break;
	case ESWA_WaitIfPossible:
		fprintf(f,"ESWA_WaitIfPossible\n");
		break;
	case ESWA_DontWait:
		fprintf(f,"ESWA_DontWait\n");
		break;
	case ESWA_DontReturnPackage:
		fprintf(f,"ESWA_DontReturnPackage\n");
		break;
	case ESWA_Invalid:
		fprintf(f,"ESWA_Invalid\n");
		break;
	}
}



int DBVprint_extend_request_elements(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
	fprintf(f,"psElements: ");
	DBVprint_inter_string(f, psExtendedServicesReq->psElements);
}



int DBVprint_extend_request_otherinfo(FILE *f, DbvExtendedServicesReq *psExtendedServicesReq)
{
        fprintf(f,"ppsOtherInformation: ");
        DBVprint_otherinfo(f, psExtendedServicesReq->ppsOtherInformation);
 
}



 


/***********************************************/
/* shared printing routines */

void DBVprint_DbvASN1External(FILE *f, DbvASN1External *psASN1External)
{
	DBVprint_external(f, psASN1External);
}



int	DBVprint_external(FILE *f, DbvASN1External *external_ptr)
{
	char	*oid = NULL;
	char	temp[500];
	char	*string = NULL;
	unsigned long	length = 0;

	memset(temp, 0, sizeof(temp));


	if (external_ptr != NULL) {

		fprintf(f,"\n");

		oid = external_ptr->pcObjectIdentifier;

		/* Print the oid of the external (a char* in dbv) */
	
		if (oid != NULL)
			fprintf(f,"\tObjectIdentifier: %s\n", oid);
		else
			fprintf(f,"\tObjectIdentifier: NULL\n");

		/* Print the type of the external (eValueType) */
		DBVprint_external_evaluetype(f, external_ptr->eValueType);
	
		switch (external_ptr->eValueType) {
		case EVT_SingleASN1Type:

			if (external_ptr->uExternalValue.psSingleASN1Type != NULL) {
				DBVprint_singleasn1type_easn1type(f, external_ptr->uExternalValue.psSingleASN1Type->eASN1Type);

				DBVprint_singleasn1type_external(f, oid, external_ptr);
			}
			break;
		case EVT_OctetAligned:
			DBVprint_aligned_external(f, external_ptr);
			break;
		case EVT_Arbitrary:
			DBVprint_arbitrary_external(f, external_ptr);
		default:
			break;
		}
	} else
		fprintf(f,"NULL\n");

}


int	DBVprint_external_evaluetype(FILE *f, DbvExternalValueType eValueType)
{
	switch (eValueType) {
	case EVT_SingleASN1Type:
		fprintf(f,"\teValueType: EVT_SingleASN1Type\n");
		break;
	case EVT_OctetAligned:
		fprintf(f,"\teValueType: EVT_OctetAligned\n");
		break;
	case EVT_Arbitrary:
		fprintf(f,"\teValueType: EVT_Arbitrary\n");
		break;
	default:
		fprintf(f,"\teValueType: Unknown\n");
		break;
	}
}



int	DBVprint_singleasn1type_easn1type(FILE *f, DbvExternalValueType eASN1Type)
{
	switch (eASN1Type) {
	case AT_UnstructuredText:
		fprintf(f,"\teASN1Type: AT_UnstructuredText\n");
		break;
	case AT_OpacRecord:
		fprintf(f,"\teASN1Type: AT_OpacRecord\n");
		break;
	case AT_SummaryRecord:
		fprintf(f,"\teASN1Type: AT_SummaryRecord\n");
		break;
	case AT_ResourceReport1:
		fprintf(f,"\teASN1Type: AT_ResourceReport1\n");
		break;
	case AT_ResourceReport2:
		fprintf(f,"\teASN1Type: AT_ResourceReport2\n");
		break;
	case AT_EsTaskPackage:
		fprintf(f,"\teASN1Type: AT_EsTaskPackage\n");
		break;
	case AT_UpdateExtendedService:
		fprintf(f,"\teASN1Type: AT_UpdateExtendedService\n");
		break;

	case AT_ItemOrderExtendedService:
		fprintf(f,"\teASN1Type: AT_ItemOrderExtendedService\n");
		break;
	case AT_IllPdu:
		fprintf(f,"\teASN1Type: AT_IllPdu\n");
		break;


        case AT_ExplainRecord:
                fprintf(f,"\teASN1Type: AT_ExplainRecord\n");
                break;

        case AT_GenericRecord:
                fprintf(f,"\teASN1Type: AT_GenericRecord\n");
                break;

        case AT_PromptObject:
                fprintf(f,"\teASN1Type: AT_PromptObject\n");
                break;
	


        case AT_Print:
                fprintf(f,"\teASN1Type: AT_Print\n");
                break;
        case AT_Alert:
                fprintf(f,"\teASN1Type: AT_Alert\n");
                break;
        case AT_Corder:
                fprintf(f,"\teASN1Type: AT_Corder\n");
                break;
        case AT_CharSetAndLanguage:
                fprintf(f,"\teASN1Type: AT_CharSetAndLanguage\n");
                break;




/*****************
 No longer used

	case AT_IntMapTest:
		fprintf(f,"\teASN1Type: AT_IntMapTest\n");
		break;

*****************/


	default:
		fprintf(f,"\teASN1Type: Unknown\n");
		break;
	}
}



int DBVprint_singleasn1type_external(FILE *f, char *oid, DbvASN1External *external_ptr)
{
	/* If oid is not null (it will be null for
	   ILL intmappers after decoding) */

	if (oid != NULL) 
		DBVprint_external_with_oid(f, oid, external_ptr);

	else
		DBVprint_external_with_int(f, external_ptr);
}



void DBVprint_external_with_oid(FILE *f, char *oid, DbvASN1External *external_ptr)
{
	if ( (strcmp(oid, ANSI_UNSTRUCTURED_TEXT_RECORD) == 0)
	|| (strcmp(oid, ISO_UNSTRUCTURED_TEXT_RECORD) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_UnstructuredText) {
			DBVprint_sutrs_external(f, external_ptr);
			return;
		}


	}

	if ( (strcmp(oid, ANSI_OPAC_RECORD) == 0) 
       	|| (strcmp(oid, ISO_OPAC_RECORD) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_OpacRecord) {
		 	DBVprint_opac_external(f, external_ptr);
			return;
		}


	} 

	if ( (strcmp(oid, ANSI_SUMMARY_RECORD) == 0) 
	|| (strcmp(oid, ISO_SUMMARY_RECORD) == 0) ) {	
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_SummaryRecord) { 
			DBVprint_summary_external(f, external_ptr);
			return;
		}


	} 

	if ( (strcmp(oid, ANSI_RESOURCE_REPORT_1) == 0) 
	|| (strcmp(oid, ISO_RESOURCE_REPORT_1) == 0) )  {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_ResourceReport1) {
			DBVprint_resrep1_external(f, external_ptr);
			return;
		}

	}

	if ( (strcmp(oid, ANSI_RESOURCE_REPORT_2) == 0) 
		|| (strcmp(oid, ISO_RESOURCE_REPORT_2) == 0) ) {	
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_ResourceReport2) {
			DBVprint_resrep2_external(f, external_ptr);
			return;
		}

	}

	if ( (strcmp(oid, ANSI_ES_TASK_PACKAGE_RECORD) == 0) 
	|| (strcmp(oid, ISO_ES_TASK_PACKAGE_RECORD) == 0)) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_EsTaskPackage) {
			DBVprint_estask_external(f, external_ptr);
			return;
		}

	}

	if ( (strcmp(oid, ANSI_UPDATE_EXTENDED_SERVICE) == 0) 
	|| (strcmp(oid, ISO_UPDATE_EXTENDED_SERVICE) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_UpdateExtendedService) {
			DBVprint_update_external(f, external_ptr);
			return;
		}
	} 


	if ( (strcmp(oid, ANSI_ITEMORDER_EXTENDED_SERVICE) == 0) 
	|| (strcmp(oid, ISO_ITEMORDER_EXTENDED_SERVICE) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_ItemOrderExtendedService) {
			DBVprint_itemorder_external(f, external_ptr);
			return;
		}

	} 


	if ( (strcmp(oid, ISO_ILL_PDU) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_IllPdu) {
			DBVprint_ill_external(f, external_ptr);
			return;
		}
	} 



	if ( (strcmp(oid, ANSI_EXPLAIN_RECORD) == 0)
	|| (strcmp(oid, ISO_EXPLAIN_RECORD) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_ExplainRecord) {
			DBVprint_explain_external(f, external_ptr);
			return;
		}
	}



	if ( (strcmp(oid, ANSI_GENERIC_RECORD) == 0)
	|| (strcmp(oid, ISO_GENERIC_RECORD) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_GenericRecord) {
			DBVprint_generic_external(f, external_ptr);
			return;
		}
	}



	
	if ( (strcmp(oid, ANSI_ACCESS_PROMPT_1) == 0) 
	|| (strcmp(oid, ISO_ACCESS_PROMPT_1) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_PromptObject) {
			DBVprint_DbvPromptObject_external(f, external_ptr);
			return;
		}
	} 





	if (strcmp(oid, ESRIN_PRINT) == 0) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_Print) {
			DBVprint_DbvPrintExtendedService_external(f, external_ptr);
			return;
		}
	} 



	if (strcmp(oid, ESRIN_ALERT) == 0) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_Alert) {
			DBVprint_DbvAlertExtendedService_external(f, external_ptr);
			return;
		}
	} 



	if (strcmp(oid, ESRIN_ORDER) == 0) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_Corder) {
			DBVprint_DbvCOrder_external(f, external_ptr);
			return;
		}
	} 



	if ( (strcmp(oid, ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0) 
	|| (strcmp(oid, ISO_CHARSET_AND_LANGUAGE_NEGOTIATION) == 0) ) {
		if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_CharSetAndLanguage) {
			DBVprint_DbvCharSetAndLanguage_external(f, external_ptr);
			return;
		}
	} 








	/* Any other combination */
	DBVprint_unknown_external(f, external_ptr);	
}



int DBVprint_external_with_int(FILE *f, DbvASN1External *external_ptr)
{

/********************
 No longer used

	if (external_ptr->uExternalValue.psSingleASN1Type->eASN1Type == AT_IntMapTest) {
		DBVprint_intmaptest_external(f, external_ptr);
		return;
	}

********************/


	/* Any other combination */
	DBVprint_unknown_external(f, external_ptr);	
}



int	DBVprint_sutrs_external(FILE *f, DbvASN1External *external_ptr)
{
	struct DbvInternationalString *sutrs_ptr = NULL;

	sutrs_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText;
	
	/* Will always be non NULL after decoding but check just in
	   case function has been called before PDU sending */
	if (sutrs_ptr != NULL) {
        	fprintf(f,"\tpsUnstructuredText: ");
		DBVprint_inter_string(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText);
	}

}



int	DBVprint_opac_external(FILE *f, DbvASN1External *external_ptr)
{
	int index = 0;
	struct DbvOpacRecord *local_ptr = NULL;

	local_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psOpacRecord;

	if (local_ptr != NULL) {
		fprintf(f,"psOpacRecord:\n");	

		/* If psBibliographicRecord is given */
		fprintf(f,"psBibliographicRecord: ");
		if (local_ptr->psBibliographicRecord != NULL) {

			/* It an external so print it as one */
			DBVprint_external(f, local_ptr->psBibliographicRecord);
		} else {
			fprintf(f,"NULL\n");
		}

		/* If ppsHoldingsData is given */
		fprintf(f,"ppsHoldingsData: ");
		if (local_ptr->ppsHoldingsData != NULL) {
			while (local_ptr->ppsHoldingsData[index] != NULL) {
				DBVprint_holdings_data(f, local_ptr->ppsHoldingsData[index]);
				index++;
			}
			fprintf(f,"\n");
		} else {
			fprintf(f,"NULL\n");
		}
	}

}



int DBVprint_holdings_data(FILE *f, DbvHoldingsRecord *psHoldingsData)
{
	/* HoldingsData is a choice according to V3 ballot 2 */
	switch( psHoldingsData->eHoldingsRecType) {
	case HRT_MarcHoldingsRecord:
		fprintf(f,"\nHRT_MarcHoldingsRecord\n");
		DBVprint_external(f, &psHoldingsData->uMarcOrHoldAndCirc.sMarcHoldingsRecord);
		break;
	case HRT_HoldingsAndCirc:
		fprintf(f,"\nHRT_HoldingsAndCirc\n");
		DBVprint_holdings_bulk(f, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc);
		DBVprint_hold_volume(f, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc);
		DBVprint_hold_circ(f, psHoldingsData->uMarcOrHoldAndCirc.sHoldingsAndCirc);
		break;
	}

}



int DBVprint_holdings_bulk(FILE *f, DbvHoldingsAndCircData sHoldingsAndCirc)
{
	/* psTypeOfRecord */
	fprintf(f,"psTypeOfRecord: ");
	if (sHoldingsAndCirc.psTypeOfRecord != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psTypeOfRecord);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psEncodingLevel: ");
	if (sHoldingsAndCirc.psEncodingLevel != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psEncodingLevel);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psFormat: ");
	if (sHoldingsAndCirc.psFormat != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psFormat);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psReceiptAcqStatus: ");
	if (sHoldingsAndCirc.psReceiptAcqStatus != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psReceiptAcqStatus);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psGeneralRetention: ");
	if (sHoldingsAndCirc.psGeneralRetention != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psGeneralRetention);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psCompleteness: ");
	if (sHoldingsAndCirc.psCompleteness != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psCompleteness);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psDateOfReport: ");
	if (sHoldingsAndCirc.psDateOfReport != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psDateOfReport);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psNucCode: ");
	if (sHoldingsAndCirc.psNucCode != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psNucCode);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psLocalLocation: ");
	if (sHoldingsAndCirc.psLocalLocation != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psLocalLocation);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psShelvingLocation: ");
	if (sHoldingsAndCirc.psShelvingLocation != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psShelvingLocation);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psCallNumber: ");
	if (sHoldingsAndCirc.psCallNumber != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psCallNumber);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psShelvingData: ");
	if (sHoldingsAndCirc.psShelvingData != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psShelvingData);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psCopyNumber: ");
	if (sHoldingsAndCirc.psCopyNumber != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psCopyNumber);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psPublicNote: ");
	if (sHoldingsAndCirc.psPublicNote != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psPublicNote);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psReproductionNote: ");
	if (sHoldingsAndCirc.psReproductionNote != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psReproductionNote);
	else
		fprintf(f,"NULL\n");
		

	fprintf(f,"psTermsUseRepro: ");
	if (sHoldingsAndCirc.psTermsUseRepro != NULL)
		DBVprint_inter_string(f, sHoldingsAndCirc.psTermsUseRepro);
	else
		fprintf(f,"NULL\n");
		


}



int DBVprint_hold_volume(FILE *f, DbvHoldingsAndCircData sHoldingsAndCirc)
{
	int index = 0;

	fprintf(f,"ppsVolume: ");
	if (sHoldingsAndCirc.ppsVolume != NULL) {
		while (sHoldingsAndCirc.ppsVolume[index] != NULL) {	

			fprintf(f,"\npsEnumeration: ");
			if (sHoldingsAndCirc.ppsVolume[index]->psEnumeration != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsVolume[index]->psEnumeration);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psChronology: ");
			if (sHoldingsAndCirc.ppsVolume[index]->psChronology != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsVolume[index]->psChronology);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psEnumAndChron: ");
			if (sHoldingsAndCirc.ppsVolume[index]->psEnumAndChron != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsVolume[index]->psEnumAndChron);
			else
				fprintf(f,"NULL\n");

			index++;
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");
}



int DBVprint_hold_circ(FILE *f, DbvHoldingsAndCircData sHoldingsAndCirc)
{
	int index = 0;

	fprintf(f,"ppsCircRecord: ");
	if (sHoldingsAndCirc.ppsCircRecord != NULL) {
		while (sHoldingsAndCirc.ppsCircRecord[index] != NULL) {	

			fprintf(f,"\nfAvailableNow: ");
			DBVprint_bool(f, &sHoldingsAndCirc.ppsCircRecord[index]->fAvailableNow);
			fprintf(f,"psAvailabilityDate: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psAvailabilityDate != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psAvailabilityDate);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psAvailableThru: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psAvailableThru != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psAvailableThru);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psRestrictions: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psRestrictions != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psRestrictions);
			else
				fprintf(f,"NULL\n");


			fprintf(f,"psItemId: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psItemId != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psItemId);
			else
				fprintf(f,"NULL\n");


			fprintf(f,"fRenewable: ");
			DBVprint_bool(f, &sHoldingsAndCirc.ppsCircRecord[index]->fRenewable);

			fprintf(f,"fOnHold: ");
			DBVprint_bool(f, &sHoldingsAndCirc.ppsCircRecord[index]->fOnHold);

			fprintf(f,"psEnumAndChron: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psEnumAndChron != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psEnumAndChron);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psMidspine: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psMidspine != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psMidspine);
			else
				fprintf(f,"NULL\n");

			fprintf(f,"psTemporaryLocation: ");
			if (sHoldingsAndCirc.ppsCircRecord[index]->psTemporaryLocation != NULL) 
				DBVprint_inter_string(f, sHoldingsAndCirc.ppsCircRecord[index]->psTemporaryLocation);
			else
				fprintf(f,"NULL\n");


			index++;
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");
}



int	DBVprint_summary_external(FILE *f, DbvASN1External *external_ptr)
{
	
	DbvSummaryRecord *summary_ptr;
	int index = 0;

	summary_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord;

	if (summary_ptr != NULL) {

		fprintf(f,"psSummaryRecord: \n");

		/* sTitle is compulsory */
		fprintf(f,"sTitle: ");
		DBVprint_inter_string(f, &summary_ptr->sTitle);

		/* psAuthor is optional */
		fprintf(f,"psAuthor: ");
		if (summary_ptr->psAuthor != NULL) 
			DBVprint_inter_string(f, summary_ptr->psAuthor);
		else
			fprintf(f,"NULL\n");
	

		/* psCallNumber is optional */
		fprintf(f,"psCallNumber: ");
		if (summary_ptr->psCallNumber != NULL) 
			DBVprint_inter_string(f, summary_ptr->psCallNumber);
		else
			fprintf(f,"NULL\n");
	

		/* psRecordType is optional */
		fprintf(f,"psRecordType: ");
		if (summary_ptr->psRecordType != NULL) 
			DBVprint_inter_string(f, summary_ptr->psRecordType);
		else
			fprintf(f,"NULL\n");
	

		/* psBibliographicLevel is optional */
		fprintf(f,"psBibliographicLevel: ");
		if (summary_ptr->psBibliographicLevel != NULL) 
			DBVprint_inter_string(f, summary_ptr->psBibliographicLevel);
		else
			fprintf(f,"NULL\n");

	
		/* ppsFormat is optional */
		fprintf(f,"ppsFormat: ");
		if (summary_ptr->ppsFormat != NULL) {
			while (summary_ptr->ppsFormat[index] != NULL) {	

				fprintf(f,"\nsType: ");
				DBVprint_inter_string(f, &(summary_ptr->ppsFormat[index]->sType));
				fprintf(f,"pulSize: ");
				if (summary_ptr->ppsFormat[index]->pulSize != NULL) 
					fprintf(f,"%ld\n", *(summary_ptr->ppsFormat[index]->pulSize));
				else
					fprintf(f,"NULL\n");

				fprintf(f,"pulBestPosn: ");
				if (summary_ptr->ppsFormat[index]->pulBestPosn != NULL) 
					fprintf(f,"%ld\n", *(summary_ptr->ppsFormat[index]->pulBestPosn));
				else
					fprintf(f,"NULL\n");
	
				index++;
			}
			fprintf(f,"\n");
		} else
			fprintf(f,"NULL\n");

		/* psPublicationPlace is optional */
		fprintf(f,"psPublicationPlace: ");
		if (summary_ptr->psPublicationPlace != NULL) 
			DBVprint_inter_string(f, summary_ptr->psPublicationPlace);
		else
			fprintf(f,"NULL\n");
	

		/* psPublicationDate is optional */
		fprintf(f,"psPublicationDate: ");
		if (summary_ptr->psPublicationDate != NULL) 
			DBVprint_inter_string(f, summary_ptr->psPublicationDate);
		else
			fprintf(f,"NULL\n");
	

		/* psTargetSystemKey is optional */
		fprintf(f,"psTargetSystemKey: ");
		if (summary_ptr->psTargetSystemKey != NULL) 
			DBVprint_inter_string(f, summary_ptr->psTargetSystemKey);
		else
			fprintf(f,"NULL\n");
	

		/* psSatisfyingElement is optional */
		fprintf(f,"psSatisfyingElement: ");
		if (summary_ptr->psSatisfyingElement != NULL) 
			DBVprint_inter_string(f, summary_ptr->psSatisfyingElement);
		else
			fprintf(f,"NULL\n");
	
		/* pulRank is optional */
		fprintf(f,"pulRank: ");
		if (summary_ptr->pulRank != NULL) 
			fprintf(f,"%ld\n", *summary_ptr->pulRank);
		else
			fprintf(f,"NULL\n");
	

		/* psDocumentId is optional */
		fprintf(f,"psDocumentId: ");
		if (summary_ptr->psDocumentId != NULL) 
			DBVprint_inter_string(f, summary_ptr->psDocumentId);
		else
			fprintf(f,"NULL\n");
	

		/* psAbstract is optional */
		fprintf(f,"psAbstract: ");
		if (summary_ptr->psAbstract != NULL) 
			DBVprint_inter_string(f, summary_ptr->psAbstract);
		else
			fprintf(f,"NULL\n");
	
		/* psOtherInformation is optional */
		fprintf(f,"ppsOtherInformation: ");
		DBVprint_otherinfo(f, summary_ptr->ppsOtherInformation);
	}
}



int DBVprint_resrep1_external(FILE *f, DbvASN1External *external_ptr)
{
	int index = 0;
	struct DbvResourceReport1 *local_ptr = NULL;

	local_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psResourceReport1;

	if (local_ptr != NULL) {
		fprintf(f,"psResourceReport1:\n");	

		/* Print out the ppsEstimate list */

		if (local_ptr->ppsEstimate != NULL) {
			while (local_ptr->ppsEstimate[index] != NULL) {
		
				fprintf(f,"ppsEstimate[%d]->eEstimateType: ", index);
				DBVprint_estimate_type(f, local_ptr->ppsEstimate[index]->eEstimateType);

				fprintf(f,"ppsEstimate[%d]->ulValue: %lu\n", index, local_ptr->ppsEstimate[index]->ulValue);

				fprintf(f,"*ppsEstimate[%d]->pulCurrencyCode: ", index);
				if (local_ptr->ppsEstimate[index]->pulCurrencyCode != NULL) 
					fprintf(f,"%lu\n", *local_ptr->ppsEstimate[index]->pulCurrencyCode);
				else
					fprintf(f,"NULL\n");
	
				index++;
			}
		}

		fprintf(f,"\n");


		/* Print out the sMessage string */
		fprintf(f,"sMessage: ");	
		DBVprint_inter_string(f, &local_ptr->sMessage);
	}

}



int DBVprint_estimate_type(FILE *f, DbvEstimateType eEstimateType)
{
	switch (eEstimateType) {
	case ET_CurrentSearchRecords:
		fprintf(f,"ET_CurrentSearchRecords\n");
		break;
	case ET_FinalSearchRecords:
		fprintf(f,"ET_FinalSearchRecords\n");
		break;
	case ET_CurrentPresentRecords:
		fprintf(f,"ET_CurrentPresentRecords\n");
		break;
	case ET_FinalPresentRecords:
		fprintf(f,"ET_FinalPresentRecords\n");
		break;
	case ET_CurrentOpTimeProcessing:
		fprintf(f,"ET_CurrentOpTimeProcessing\n");
		break;
	case ET_FinalOpTimeProcessing:
		fprintf(f,"ET_FinalOpTimeProcessing\n");
		break;
	case ET_CurrentAssocTime:
		fprintf(f,"ET_CurrentAssocTime\n");
		break;
	case ET_CurrentOperationCost:
		fprintf(f,"ET_CurrentOperationCost\n");
		break;
	case ET_FinalOperationCost:
		fprintf(f,"ET_FinalOperationCost\n");
		break;
	case ET_CurrentAssocCost:
		fprintf(f,"ET_CurrentAssocCost\n");
		break;
	case ET_FinalOpTimeElapsed:
		fprintf(f,"ET_FinalOpTimeElapsed\n");
		break;
	case ET_PercentComplete:
		fprintf(f,"ET_PercentComplete\n");
		break;
	case ET_CurrentSearchAssocCost:
		fprintf(f,"ET_CurrentSearchAssocCost\n");
		break;
	case ET_CurrentPresentAssocCost:
		fprintf(f,"ET_CurrentPresentAssocCost\n");
		break;
	case ET_CurrentConnectAssocCost:
		fprintf(f,"ET_CurrentConnectAssocCost\n");
		break;
	case ET_CurrentOtherAssocCost:
		fprintf(f,"ET_CurrentOtherAssocCost\n");
		break;
	case ET_Invalid:
		fprintf(f,"ET_Invalid\n");
		break;
	}
}



int DBVprint_resrep2_external(FILE *f, DbvASN1External *external_ptr)
{
	int index = 0;
	struct DbvResourceReport2 *local_ptr = NULL;

	local_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psResourceReport2;

	if (local_ptr != NULL) {
		fprintf(f,"psResourceReport2:\n");	


		/* Both the following are optional */

		/* Print out the ppsEstimate list */
		fprintf(f,"ppsEstimate: ");
		if (local_ptr->ppsEstimate != NULL) {

			fprintf(f,"\n");

			while (local_ptr->ppsEstimate[index] != NULL) {
		
				fprintf(f,"ppsEstimate[%d]->sType:\n", index);
				DBVprint_strornum(f, &(local_ptr->ppsEstimate[index]->sType));
				DBVprint_intunit(f, &(local_ptr->ppsEstimate[index]->sValue));
	
				index++;
			}
			fprintf(f,"\n");
		} else
			fprintf(f,"NULL\n");
	


		/* Print out the psMessage string */
		fprintf(f,"psMessage: ");	
		if (local_ptr->psMessage != NULL)
			DBVprint_inter_string(f, local_ptr->psMessage);
		else
			fprintf(f,"NULL\n");
	}
}



int DBVprint_estask_external(FILE *f, DbvASN1External *external_ptr)
{
	int index = 0;
	struct DbvEsTaskPackage *local_ptr = NULL;

	local_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psEsTaskPackage;

	if (local_ptr != NULL) {
		fprintf(f,"psEsTaskPackage:\n");	
		fprintf(f,"pcPackageType: %s\n", local_ptr->pcPackageType);

		fprintf(f,"psPackageName: ");
		if (local_ptr->psPackageName != NULL)
			DBVprint_inter_string(f, local_ptr->psPackageName);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"psUserId: ");
		if (local_ptr->psUserId != NULL)
			DBVprint_inter_string(f, local_ptr->psUserId);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"psRetentionTime:\n");
		if (local_ptr->psRetentionTime != NULL)
			DBVprint_intunit(f, local_ptr->psRetentionTime);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"ppsPermissions: ");
		if (local_ptr->ppsPermissions != NULL)
			DBVprint_permissions(f, local_ptr->ppsPermissions);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"psDescription: ");
		if (local_ptr->psDescription != NULL)
			DBVprint_inter_string(f, local_ptr->psDescription);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"psTargetReference: ");
		if (local_ptr->psTargetReference != NULL)
			DBVprint_octet_string(f, local_ptr->psTargetReference);
		else
			fprintf(f,"NULL\n");

		fprintf(f,"pcCreationDateTime: ");
		if (local_ptr->pcCreationDateTime != NULL)
			fprintf(f,"%s\n", local_ptr->pcCreationDateTime);
		else
			fprintf(f,"NULL\n");


		fprintf(f,"eExtServicesTaskStatus: ");
		switch (local_ptr->eExtServicesTaskStatus) {
		case ESTS_Pending:
			fprintf(f,"ESTS_Pending\n");
			break;
		case ESTS_Active:
			fprintf(f,"ESTS_Active\n");
			break;
		case ESTS_Complete:
			fprintf(f,"ESTS_Complete\n");
			break;
		case ESTS_Aborted:
			fprintf(f,"ESTS_Aborted\n");
			break;
		case ESTS_Invalid:
			fprintf(f,"ESTS_Invalid\n");
			break;
		}
	

		fprintf(f,"ppsPackageDiagnostics:\n");
		if (local_ptr->ppsPackageDiagnostics != NULL)
			DBVprint_diagrec_list(f, local_ptr->ppsPackageDiagnostics);
		else
			fprintf(f,"NULL\n");

		fprintf(f,"sTaskSpecificParameters:\n");	
		DBVprint_external(f, &local_ptr->sTaskSpecificParameters);
	}
}



int DBVprint_permissions(FILE *f, DbvPermissions **ppsPermissions)
{
	int index = 0;

	if (ppsPermissions != NULL) {
		while( ppsPermissions[index] != NULL) {
			fprintf(f,"\nuserId: ");
			DBVprint_inter_string(f, &ppsPermissions[index]->sUserId);
			DBVprint_allowfuncts(f, ppsPermissions[index]->ppeAllowableFunctions);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_allowfuncts(FILE *f, DbvPermAllowableFunctions **ppeAllowableFunctions)
{
	int index = 0;

	fprintf(f,"ppeAllowableFunctions:\n");

	if (ppeAllowableFunctions != NULL) {
		while (ppeAllowableFunctions[index] != NULL) {
			switch (*ppeAllowableFunctions[index]) {
			case PAF_Delete:
				fprintf(f,"PAF_Delete\n");
				break;
			case PAF_ModifyContents:
				fprintf(f,"PAF_ModifyContents\n");
				break;
			case PAF_ModifyPermissions:
				fprintf(f,"PAF_ModifyPermissions\n");
				break;
			case PAF_Present:
				fprintf(f,"PAF_Present\n");
				break;
			case PAF_Invoke:
				fprintf(f,"PAF_Invoke\n");
				break;
			case PAF_Invalid:
				fprintf(f,"PAF_Invalid\n");
				break;
			}
			index++;
		}
	}
	fprintf(f,"\n");

}



int DBVprint_update_external(FILE *f, DbvASN1External *external_ptr)
{
	DbvUpdateExtendedService *update_ptr;

	update_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psUpdateExtendedService;

	if (update_ptr != NULL) {

		fprintf(f,"psUpdateExtendedService: \n");
		fprintf(f,"eUpdateReqRsp:\n");
		switch (update_ptr->eUpdateReqRsp) {
		case UROR_Request:
			fprintf(f,"UROR_Request\n");
			DBVprint_update_request(f, update_ptr->uUpdateReqOrRsp.psUpdateRequest);
			break;
		case UROR_Response:
			fprintf(f,"UROR_Response\n");
			DBVprint_update_response(f, update_ptr->uUpdateReqOrRsp.psUpdateResponse);
			break;
		default:
			fprintf(f,"Unknown\n");
			break;
		}
	}
}



int DBVprint_update_request(FILE *f, DbvUpdateRequestPackage *psUpdateRequest)
{
	/* Both compulsory */
	DBVprint_originparttokeep(f, &psUpdateRequest->sOriginPartToKeep);
	DBVprint_suppliedrecs(f, psUpdateRequest->ppsSuppliedRecord);
	
}



int DBVprint_originparttokeep(FILE *f, DbvOriginPartToKeep *psOriginPartToKeep)
{
	fprintf(f,"psOriginPartToKeep:\n");
	DBVprint_action(f, psOriginPartToKeep->eAction);
	
	fprintf(f,"sDatabaseName: ");
	DBVprint_inter_string(f, &psOriginPartToKeep->sDatabaseName);

	fprintf(f,"pcSchema: ");
	if (psOriginPartToKeep->pcSchema != NULL)
		fprintf(f,"%s\n", psOriginPartToKeep->pcSchema);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"psElementSetName: ");
	if (psOriginPartToKeep->psElementSetName != NULL)
		DBVprint_inter_string(f, psOriginPartToKeep->psElementSetName);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_action(FILE *f, DbvUpdtAction eAction)
{
	fprintf(f,"eAction: ");
	switch (eAction) {
	case UA_RecordInsert:
		fprintf(f,"UA_RecordInsert\n");
		break;
	case UA_RecordReplace:
		fprintf(f,"UA_RecordReplace\n");
		break;
	case UA_RecordDelete:
		fprintf(f,"UA_RecordDelete\n");
		break;
	case UA_ElementUpdate:
		fprintf(f,"UA_ElementUpdate\n");
		break;
	case UA_Invalid:
		fprintf(f,"UA_Invalid\n");
		break;
	default:
		fprintf(f,"unknown\n");
		break;
	}
}



int DBVprint_suppliedrecs(FILE *f, DbvSuppliedRecord **ppsSuppliedRecord)
{
	int index = 0;

	fprintf(f,"ppsSuppliedRecord:\n");

	if (ppsSuppliedRecord != NULL) {
		while( ppsSuppliedRecord[index] != NULL) {
			DBVprint_suppliedrec(f, ppsSuppliedRecord[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_suppliedrec(FILE *f, DbvSuppliedRecord *psSuppliedRecord)
{
	fprintf(f,"\n\npsSuppliedRecord:\n");

	fprintf(f,"psUpdtRecordId: ");
	if (psSuppliedRecord->psUpdtRecordId != NULL)
		DBVprint_recordid(f, psSuppliedRecord->psUpdtRecordId);
	else
		fprintf(f,"NULL\n");


	fprintf(f,"psUpdtSupplementalId: ");
	if (psSuppliedRecord->psUpdtSupplementalId != NULL)
		DBVprint_suppid(f, psSuppliedRecord->psUpdtSupplementalId);
	else
		fprintf(f,"NULL\n");
	

	fprintf(f,"psCorrelationInfo: ");
	if (psSuppliedRecord->psCorrelationInfo != NULL)
		DBVprint_correl_info(f, psSuppliedRecord->psCorrelationInfo);
	else
		fprintf(f,"NULL\n");
	

	/* The following external is compulsory */
	fprintf(f,"sRecord:\n");
	DBVprint_external(f, &psSuppliedRecord->sRecord);
}



int DBVprint_recordid(FILE *f, DbvUpdtRecordId *psUpdtRecordId)
{
	fprintf(f,"\neRecordIdType: ");
	switch( psUpdtRecordId->eRecordIdType) {
	case URIT_Number:
		fprintf(f,"URIT_Number\n");
		fprintf(f,"ulNumber: %lu\n", psUpdtRecordId->uUpdtRecordId.ulNumber);
		break;
	case URIT_String:
		fprintf(f,"URIT_String\n");
		fprintf(f,"psString: ");
		DBVprint_inter_string(f, psUpdtRecordId->uUpdtRecordId.psString);
		break;
	case URIT_Opaque:
		fprintf(f,"URIT_Opaque\n");
		fprintf(f,"psOpaque: ");
		DBVprint_octet_string(f, psUpdtRecordId->uUpdtRecordId.psOpaque);
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_suppid(FILE *f, DbvUpdtSupplementalId *psUpdtSupplementalId)
{
	fprintf(f,"\neSupplementalIdType: ");
	switch( psUpdtSupplementalId->eSupplementalIdType) {
	case USIT_Time:
		fprintf(f,"USIT_Time\n");
		fprintf(f,"pcTime: %s\n", psUpdtSupplementalId->uUpdtSupplementalId.pcTime);
		break;
	case USIT_Version:
		fprintf(f,"USIT_Version\n");
		fprintf(f,"psVersion: ");
		DBVprint_inter_string(f, psUpdtSupplementalId->uUpdtSupplementalId.psVersion);
		break;
	case USIT_PreviousVersion:
		fprintf(f,"USIT_PreviousVersion\n");
		DBVprint_external(f, psUpdtSupplementalId->uUpdtSupplementalId.psPreviousVersion);
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_correl_info(FILE *f, DbvCorrelationInfo *psCorrelationInfo)
{
	/* Both optional */
	fprintf(f,"\npsNote: ");
	if (psCorrelationInfo->psNote != NULL)
		DBVprint_inter_string(f, psCorrelationInfo->psNote);
	else
		fprintf(f,"NULL\n");
		
	fprintf(f,"pulId: ");
	if (psCorrelationInfo->pulId != NULL)
		fprintf(f,"%lu\n", *psCorrelationInfo->pulId);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_update_response(FILE *f, DbvUpdateTaskPackage *psUpdateResponse)
{
	/* Both compulsory */
	DBVprint_originparttokeep(f, &psUpdateResponse->sOriginPartToKeep);
	DBVprint_targetpart(f, &psUpdateResponse->sTargetPart);
}



int DBVprint_targetpart(FILE *f, DbvUpdateTargetPart *psTargetPart)
{
	fprintf(f,"psTargetPart:\n");
	DBVprint_taskstatus(f, psTargetPart->eTaskStatus);

	/* Following two optional */

	fprintf(f,"ppsGlobalDiagnostics: ");
	if (psTargetPart->ppsGlobalDiagnostics != NULL)
		DBVprint_diagrec_list(f, psTargetPart->ppsGlobalDiagnostics);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"ppsTaskPackageRecord: ");
	if (psTargetPart->ppsTaskPackageRecord != NULL)
		DBVprint_taskpack_recs(f, psTargetPart->ppsTaskPackageRecord);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_taskstatus(FILE *f, DbvUpdtTaskStatus eTaskStatus)
{
	fprintf(f,"eTaskStatus: ");
	switch (eTaskStatus) {
	case UTS_Success:
		fprintf(f,"UTS_Success\n");
		break;
	case UTS_Partial:
		fprintf(f,"UTS_Partial\n");
		break;
	case UTS_Failure:
		fprintf(f,"UTS_Failure\n");
		break;
	case UTS_Invalid:
		fprintf(f,"UTS_Invalid\n");
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_taskpack_recs(FILE *f, DbvTaskPackageRecord **ppsTaskPackageRecord)
{
	int index = 0;

	if (ppsTaskPackageRecord != NULL) {
		while( ppsTaskPackageRecord[index] != NULL) {
			fprintf(f,"\n");
			DBVprint_taskpack_rec(f, ppsTaskPackageRecord[index]);
			index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_taskpack_rec(FILE *f, DbvTaskPackageRecord *psTaskPackageRecord)
{
	fprintf(f,"psUpdtRecordOrDiag: ");
	if (psTaskPackageRecord->psUpdtRecordOrDiag != NULL)
		DBVprint_recordiag(f, psTaskPackageRecord->psUpdtRecordOrDiag);
	else
		fprintf(f,"NULL\n");

	
	fprintf(f,"psCorrelationInfo: ");
	if (psTaskPackageRecord->psCorrelationInfo != NULL)
		DBVprint_correl_info(f, psTaskPackageRecord->psCorrelationInfo);
	else
		fprintf(f,"NULL\n");

	
	DBVprint_recstatus(f, psTaskPackageRecord->eUpdtRecordStatus);
	
}



int DBVprint_recordiag(FILE *f, DbvUpdtRecordOrDiag *psUpdtRecordOrDiag)
{
	fprintf(f,"\neUpdtRecordOrDiag: ");
	switch( psUpdtRecordOrDiag->eUpdtRecordOrDiag) {
	case URDT_Record:
		fprintf(f,"URDT_Record\n");
		DBVprint_external(f, psUpdtRecordOrDiag->uUpdtRecordOrDiag.psRecord);
		break;
	case URDT_Diagnostic:
		fprintf(f,"URDT_Diagnostic\n");
		DBVprint_diag_rec(f, psUpdtRecordOrDiag->uUpdtRecordOrDiag.psDiagnostic);
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}

}



int DBVprint_recstatus(FILE *f, DbvUpdtRecordStatus eUpdtRecordStatus)
{
	fprintf(f,"eUpdtRecordStatus: ");
	switch (eUpdtRecordStatus) {
	case URST_Success:
		fprintf(f,"URST_Success\n");
		break;
	case URST_Queued:
		fprintf(f,"URST_Queued\n");
		break;
	case URST_InProcess:
		fprintf(f,"URST_InProcess\n");
		break;
	case URST_Failure:
		fprintf(f,"URST_Failure\n");
		break;
	case URST_Invalid:
		fprintf(f,"URST_Invalid\n");
		break;
	default:
		fprintf(f,"Unknown\n");
		break;
	}
}



int DBVprint_unknown_external(FILE *f, DbvASN1External *external_ptr)
{
	char *ber_data = NULL;
	struct DbvOctetString *pdu_ptr = NULL;
	unsigned long length = 0;

	pdu_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psUnknownType;

	if (pdu_ptr != NULL) {
		ber_data = pdu_ptr->pcValue;
		length = pdu_ptr->ulLength;
		fprintf(f,"\tpsUnknownType:");

		DBVprint_as_hex(f, ber_data, length);
	}
}



int	DBVprint_arbitrary_external(FILE *f, DbvASN1External *external_ptr)
{
        fprintf(f,"\tpsArbitrary:");
	DBVprint_octet_string(f, external_ptr->uExternalValue.psArbitrary);

}


/* Routines to print out psOtherInformation Fields */
int	DBVprint_otherinfo(FILE *f, DbvOtherInformation **ppsOtherInformation)
{
	int index = 0;

	if (ppsOtherInformation != NULL) {
		while( ppsOtherInformation[index] != NULL) {
			DBVprint_otherinfo_node(f, ppsOtherInformation[index]);
			index++;
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");

	fprintf(f,"\n");
}



int DBVprint_otherinfo_node(FILE *f, DbvOtherInformation *psOtherInformation)
{

	char	*oid = NULL;


	if (psOtherInformation != NULL) {

		fprintf(f,"\n");

		fprintf(f,"\tpsInfoCategory: ");

		/* If the psInfoCategory is given (it is an optional field) */
		if (psOtherInformation->psInfoCategory != NULL) {
	
			fprintf(f,"\n\tpcCategoryTypeId: ");
			/* If pcCategoryTypeId is given (it is an optional field) */
			if (psOtherInformation->psInfoCategory->pcCategoryTypeId != NULL) {
				oid = psOtherInformation->psInfoCategory->pcCategoryTypeId;
	
				/* Print the oid (a char* in dbv) */
				fprintf(f,"%s\n", oid);
			} else
				fprintf(f,"NULL\n");
	
			/* ulCategoryValue must be given in a psInfoCategory */
			fprintf(f,"\tulCategoryValue: %ld\n", psOtherInformation->psInfoCategory->ulCategoryValue);
		} else
			fprintf(f,"NULL\n");
	

		/* Print out the type of otherinformation */
		DBVprint_otherinfo_infotype(f, psOtherInformation->eInformationType);
	
	
		/* Now print out the actual otherinfo data */
		switch( psOtherInformation->eInformationType) {
		case IT_CharacterInfo:
			DBVprint_otherinfo_char(f, psOtherInformation->uInformation.psCharacterInfo);
			break;
		case IT_BinaryInfo:
			DBVprint_otherinfo_bin(f, psOtherInformation->uInformation.psBinaryInfo);
			break;
		case IT_ExternalInfo:
			DBVprint_otherinfo_ext(f, psOtherInformation->uInformation.psExternallyDefinedInfo);
			break;
		case IT_ObjectIdentifier:
			DBVprint_otherinfo_obj(f, psOtherInformation->uInformation.pcObjectIdentifier);
			break;
		}
	} else
		fprintf(f,"NULL\n");
}
	




int DBVprint_otherinfo_char(FILE *f, DbvInternationalString *psCharacterInfo)
{
	fprintf(f,"\tpsCharacterInfo: ");
        DBVprint_inter_string(f, psCharacterInfo);
}



int DBVprint_otherinfo_bin(FILE *f, DbvOctetString *psBinaryInfo)
{
	fprintf(f,"\tpsBinaryInfo: ");
        DBVprint_octet_string(f, psBinaryInfo);
}



int DBVprint_otherinfo_ext(FILE *f, DbvASN1External *psExternallyDefinedInfo)
{
	fprintf(f,"\tpsExternallyDefinedInfo:\n");
	DBVprint_external(f, psExternallyDefinedInfo);
}



int	DBVprint_otherinfo_obj(FILE *f, DbvObjectIdentifier pcObjectIdentifier)
{
	/* Print the oid of the external (a char* in dbv) */
	fprintf(f,"\tpcObjectIdentifier:%s\n", pcObjectIdentifier);

}


int	DBVprint_otherinfo_infotype(FILE *f, DbvInformationType eInformationType)
{
	switch (eInformationType) {
	case IT_CharacterInfo:
		fprintf(f,"\teInformationType: IT_CharacterInfo\n");
		break;
	case IT_BinaryInfo:
		fprintf(f,"\teInformationType: IT_BinaryInfo\n");
		break;
	case IT_ExternalInfo:
		fprintf(f,"\teInformationType: IT_ExternalInfo\n");
		break;
	case IT_ObjectIdentifier:
		fprintf(f,"\teInformationType: IT_ObjectIdentifier\n");
		break;
	default:
		fprintf(f,"\teInformationType: Unknown\n");
		break;
	}
}



int DBVprint_octet_string(FILE *f, DbvOctetString *octet_string)
{
        char *temp = NULL;
        char *string = NULL;
        unsigned long length = 0;
 
 
        if (octet_string != NULL) {
                if (octet_string->pcValue != NULL) {
 
                        string = octet_string->pcValue;
                        length = octet_string->ulLength;
 
                        temp = (char *) DbvAlloc (length + 1);
                        if (temp != NULL) {
                                memcpy(temp, string, length);
                                fprintf(f,"%s\n", temp);
                                DbvFree(temp);
                        } else
				fprintf(f,"NOT ENOUGH MEMORY FOR PRINT OPERATION!\n");
                } else
                        fprintf(f,"\n");
 
        } else
                fprintf(f,"NULL\n");
}



void DBVprint_DbvInternationalString(FILE *f, char *name, DbvInternationalString *psInternationalString)
{
   /* Write the title out */ 
   if (psInternationalString == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s: ", name);


	DBVprint_inter_string(f, psInternationalString);
}



int DBVprint_inter_string(FILE *f, DbvInternationalString *inter_string)
{
        char *temp = NULL;
        char *string = NULL;
        unsigned long length = 0;
 
        if (inter_string != NULL) {
                if (inter_string->pcValue != NULL) {
 
                        string = inter_string->pcValue;
                        length = inter_string->ulLength;
 
                        temp = (char *) DbvAlloc (length + 1);
                        if (temp != NULL) {
                                memcpy(temp, string, length);
                                fprintf(f,"%s\n", temp);
                                DbvFree(temp);

				DBVprint_as_hex(f, string, length);
                        } else
				fprintf(f,"NOT ENOUGH MEMORY FOR PRINT OPERATION!\n");
                } else
                        fprintf(f,"\n");
 
        } else
                fprintf(f,"NULL\n");

}



int 	DBVprint_bool(FILE *f, DbvBool *bool)
{
	if(bool != NULL)
	{
		if(*bool == DBV_FALSE)
			fprintf(f,"DBV_FALSE\n");
		else
			fprintf(f,"DBV_TRUE\n");
	}
	else
		fprintf(f,"NULL\n");
		
}



int DBVprint_DbvPresentStatus(FILE *f, DbvPresentStatus *pePresentStatus)
{
	switch (*pePresentStatus) {
		case PS_Success:
			fprintf(f,"PS_Success\n");
			break;
		case PS_Partial_1:
			fprintf(f,"PS_Partial_1\n");
			break;
		case PS_Partial_2:
			fprintf(f,"PS_Partial_2\n");
			break;
		case PS_Partial_3:
			fprintf(f,"PS_Partial_3\n");
			break;
		case PS_Partial_4:
			fprintf(f,"PS_Partial_4\n");
			break;
		case PS_Failure:
			fprintf(f,"PS_Failure\n");
			break;
		case PS_Invalid:
			fprintf(f,"PS_Invalid\n");
			break;
	}

}


int DBVprint_DbvElementSetNames(FILE *f, DbvElementSetNames *psElementSetNames)
{

	fprintf(f,"\npsElementSetNames: ");
	if (psElementSetNames != NULL)
		DBVprint_el_set_names(f, psElementSetNames);
	else
		fprintf(f,"NULL\n");

}




int DBVprint_el_set_names(FILE *f, DbvElementSetNames *psElementSetNames)
{

	switch( psElementSetNames->eElementSetType) {
	case EST_Generic:
		fprintf(f,"EST_Generic:\n");
        	DBVprint_inter_string(f, psElementSetNames->uElementSet.psGeneric); 
		break;
	case EST_DatabaseSpecific:
		fprintf(f,"EST_DatabaseSpecific:\n");
		DBVprint_el_set_specific(f, psElementSetNames->uElementSet.ppsElementSetsDbSpecific);
		break;
	}
}



int DBVprint_el_set_specific(FILE *f, DbvElementSetsDbSpecific **ppsElementSetsDbSpecific)
{
	int index = 0;

	if (ppsElementSetsDbSpecific != NULL){
		while(ppsElementSetsDbSpecific[index] != NULL) {
       			fprintf(f,"\tpsDatabaseName: ");
			DBVprint_inter_string(f, ppsElementSetsDbSpecific[index]->psDatabaseName);
       			fprintf(f,"\tpspsElementSetName: ");
			DBVprint_inter_string(f, ppsElementSetsDbSpecific[index]->psElementSetName);
			index++;
			fprintf(f,"\n");
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");
}




int	DBVprint_records(FILE *f, DbvRecords *psRecords)
{
	if (psRecords != NULL) {
		fprintf(f,"\n\n");
		switch( psRecords->eRecordType) {
		case RT_Response:
			fprintf(f,"RT_Response:\n");
			DBVprint_nameplusrec(f, psRecords->uRecord.ppsResponseRecord);
			break;
		case RT_NonSurrDiagnostics:
			fprintf(f,"RT_NonSurrDiagnostics:\n");
			if (psRecords->uRecord.psNonSurrogateDiag != NULL)
				DBVprint_default_diag(f, psRecords->uRecord.psNonSurrogateDiag);
			break;
		case RT_MultipleNonSurrDiagnostics:
			fprintf(f,"RT_MultipleNonSurrDiagnostics:\n");
			if (psRecords->uRecord.ppsMultipleNonSurrogateDiag != NULL) 
				DBVprint_diagrec_list(f, psRecords->uRecord.ppsMultipleNonSurrogateDiag);
			break;
		}
		fprintf(f,"\n\n");
	
		
	} else
		fprintf(f,"NULL\n");

}



int DBVprint_nameplusrec(FILE *f, DbvNamePlusRecord **ppsResponseRecord)
{
	int index = 0;

	if (ppsResponseRecord != NULL) {
		while( ppsResponseRecord[index] != NULL) {
			fprintf(f,"\npsDatabaseName: ");
			if (ppsResponseRecord[index]->psDatabaseName != NULL)
				DBVprint_inter_string(f, ppsResponseRecord[index]->psDatabaseName);
			else
				fprintf(f,"NULL\n");
	
			switch( ppsResponseRecord[index]->eDbRecordType) {
			case DRT_Retrieval:
				fprintf(f,"DRT_Retrieval:\n");
				DBVprint_external(f, ppsResponseRecord[index]->uDbRecord.psRetrievalRecord);
				break;
			case DRT_SurrogateDiagnostics:
				fprintf(f,"DRT_SurrogateDiagnostics:\n");
				DBVprint_diag_rec(f, ppsResponseRecord[index]->uDbRecord.psSurrogateDiagnostic);
				break;
			case DRT_StartingFragment:
				fprintf(f,"DRT_StartingFragment:\n");
				DBVprint_fragment(f, ppsResponseRecord[index]->uDbRecord.psStartingFragment);
				break;
			case DRT_IntermediateFragment:
				fprintf(f,"DRT_IntermediateFragment:\n");
				DBVprint_fragment(f, ppsResponseRecord[index]->uDbRecord.psIntermediateFragment);
				break;
			case DRT_FinalFragment:
				fprintf(f,"DRT_FinalFragment:\n");
				DBVprint_fragment(f, ppsResponseRecord[index]->uDbRecord.psFinalFragment);
				break;
			}
	
		index++;
		}
	}
	fprintf(f,"\n");
}



int DBVprint_diag_rec(FILE *f, DbvDiagRec *psSurrogateDiagnostic)
{
	if (psSurrogateDiagnostic == NULL) {
		fprintf(f, "NULL\n");
		return 1;
	}

	switch( psSurrogateDiagnostic->eDiagnosticsType) {
	case DT_DefaultFormat:
		fprintf(f,"DT_DefaultFormat:\n");
		DBVprint_default_diag(f, psSurrogateDiagnostic->uDiagnostics.psDefault);
		break;
	case DT_ExternallyDefined:
		fprintf(f,"DT_ExternallyDefined:\n");
		DBVprint_external(f, psSurrogateDiagnostic->uDiagnostics.psExternallyDefined);
		break;
	}

	return 1;
}



int DBVprint_default_diag(FILE *f, DbvDefaultDiagFormat *psDefault)
{
	/* All compulsory fields */
	fprintf(f,"pcDiagnosticSetId: %s\n", psDefault->pcDiagnosticSetId);
	fprintf(f,"ulCondition: %ld\n", psDefault->ulCondition);

	DBVprint_additional_info(f, psDefault->sAdditionalInfo);
}



int DBVprint_fragment(FILE *f, DbvFragmentSyntax *psStartingFragment)
{
	fprintf(f,"eFragmentSyntaxType: ");
	switch( psStartingFragment->eFragmentSyntaxType) {
	case FST_ExternallyTagged:
		fprintf(f,"FST_ExternallyTagged\n");
		DBVprint_external(f, psStartingFragment->uFragmentSyntax.psExternallyTagged);
		break;
	case FST_NotExternallyTagged:
		fprintf(f,"FST_NotExternallyTagged\n");
		DBVprint_octet_string(f, psStartingFragment->uFragmentSyntax.psNotExternallyTagged);
		break;
	}
}



int DBVprint_additional_info(FILE *f, DbvAdditionalInfo sAdditionalInfo)
{

	switch(sAdditionalInfo.eProtocolVersionInForce) {
	case PVIF_Version2:
		fprintf(f,"PVIF_Version2: %s\n", sAdditionalInfo.uAdditionalInfo.pcV2);	
		break;
	case PVIF_Version3:
		fprintf(f,"PVIF_Version3: ");	
		DBVprint_inter_string(f, sAdditionalInfo.uAdditionalInfo.psV3);	
		break;
	}

}



int DBVprint_diagrec_list(FILE *f, DbvDiagRec **diagrec_list)
{
	int index = 0;

	if (diagrec_list != NULL) {
		while (diagrec_list[index] != NULL) {
			fprintf(f,"\n");
			DBVprint_diag_rec(f, diagrec_list[index]);
			index++;
		}
	}

	fprintf(f,"\n");
		
}


 
 

int DBVprint_strornum(FILE *f, DbvStringOrNumeric *psList)
{
	if (psList == NULL) {
		fprintf(f, "NULL\n");
		return 1; 
	} 
   
        switch( psList->eStringOrNumeric) {
        case SNT_String:
                fprintf(f,"psString: ");
                DBVprint_inter_string(f, psList->uStringOrNumeric.psString);
                break;
        case SNT_Numeric:
                fprintf(f,"ulNumeric: %ld\n", psList->uStringOrNumeric.ulNumeric);
                break;
        }

	return 1;
}



int DBVprint_intunit(FILE *f, DbvIntUnit *psIntegerAndUnit)
{
	if (psIntegerAndUnit == NULL) {
		fprintf(f, "NULL\n");
		return 1;
	}

        /* Both compulsory */
        fprintf(f,"value: %ld\n", psIntegerAndUnit->ulValue);
 
        DBVprint_unitused(f, psIntegerAndUnit->sUnitUsed);

	return 1;
}
 
 
 
int DBVprint_unitused(FILE *f, DbvUnit sUnitUsed)
{
 
        /* All optional */
 
        fprintf(f,"psUnitSystem: ");
        if (sUnitUsed.psUnitSystem != NULL)
                DBVprint_inter_string(f, sUnitUsed.psUnitSystem);
        else
                fprintf(f,"NULL\n");
 
 
        fprintf(f,"psUnitType: ");
        if (sUnitUsed.psUnitType != NULL)
                DBVprint_strornum(f, sUnitUsed.psUnitType);
        else
                fprintf(f,"NULL\n");
 
 
        fprintf(f,"psUnit: ");
        if (sUnitUsed.psUnit != NULL)
                DBVprint_strornum(f, sUnitUsed.psUnit);
        else
                fprintf(f,"NULL\n");
 
 
        fprintf(f,"pulScaleFactor: ");
        if (sUnitUsed.pulScaleFactor != NULL)
                fprintf(f,"%ld\n", *sUnitUsed.pulScaleFactor);
        else
                fprintf(f,"NULL\n");
}






int DBVprint_attsplusterm(FILE *f, DbvAttributesPlusTerm *psAttributesPlusTerm)
{
	int index = 0;


	/* Print all attributes */
	
	if (psAttributesPlusTerm->ppsAttributeElement != NULL) {
		while( psAttributesPlusTerm->ppsAttributeElement[index] != NULL) {
			DBVprint_att_element(f, psAttributesPlusTerm->ppsAttributeElement[index]);
			index++;
		}
	}

	fprintf(f,"\n");

	/* then the term */
	DBVprint_att_term(f, psAttributesPlusTerm->sTerm);

	fprintf(f,"\n\n");
}




int DBVprint_att_element(FILE *f, DbvAttributeElement *psAttributeElement)
{
	fprintf(f,"\npcAttributeSetId: ");
	if (psAttributeElement->pcAttributeSetId != NULL)
		fprintf(f,"%s ", psAttributeElement->pcAttributeSetId);
	else
		fprintf(f,"NULL ");


	fprintf(f,"attype: %ld ", psAttributeElement->ulAttributeType);
	DBVprint_att_value(f, psAttributeElement->sAttributeValue);
}



int DBVprint_att_value(FILE *f, DbvAttributeValue sAttributeValue)
{
	switch (sAttributeValue.eAttributeValueType) {
	case AVT_Numeric:
		fprintf(f,"attval: %ld   ", sAttributeValue.uAttributeValue.ulNumeric);
		break;
	case AVT_Complex:
		fprintf(f,"\nComplex attval: ");
		DBVprint_complexval(f, sAttributeValue.uAttributeValue.psComplexValue);
		break;
	}
}



int DBVprint_complexval(FILE *f, DbvComplexValue *psComplexValue)
{
	/* First compulsory */
	int index = 0;

	
	fprintf(f,"\n#####################\n");
	fprintf(f,"Start of complexval: \n");

	/* Print all attributes */

	if (psComplexValue->ppsList != NULL) {
		while( psComplexValue->ppsList[index] != NULL) {
			fprintf(f,"\npsStringOrNumeric: \n");
			DBVprint_strornum(f, psComplexValue->ppsList[index]);
			index++;
		}
	}
	fprintf(f,"\n");
	


	/* Second optional */
	index = 0;

	/* Print all attributes */

	fprintf(f,"\nppulSemanticAction list: ");
	if (psComplexValue->ppulSemanticAction != NULL) {
		while( psComplexValue->ppulSemanticAction[index] != NULL) {
			fprintf(f,"\npulSemanticAction: %ld", *psComplexValue->ppulSemanticAction[index]);
			index++;

		}
		fprintf(f,"\n\n");
	} else
		fprintf(f,"NULL\n\n");


	fprintf(f,"End of complexval: \n");
	fprintf(f,"#####################\n");
}



int DBVprint_att_term(FILE *f, DbvTerm sTerm)
{
	switch (sTerm.eTermType) {
	case TT_General:
		fprintf(f,"\npsGeneral: ");
		DBVprint_octet_string(f, sTerm.uTerm.psGeneral);
		break;
	case TT_Numeric:
		fprintf(f,"\nulInteger: %ld\n", sTerm.uTerm.ulInteger);
		break;
	case TT_SimpleCharacter:
		fprintf(f,"\npsSimpleCharacter: ");
		DBVprint_inter_string(f, sTerm.uTerm.psSimpleCharacter);
		break;
	case TT_ObjectIdentifier:
		fprintf(f,"\npcObjectIdentifier: %s\n", sTerm.uTerm.pcObjectIdentifier );
		break;
	case TT_DateTime:

		/* DateTime is just a char* */
		fprintf(f,"\npcDateTime: %s\n", sTerm.uTerm.pcObjectIdentifier);
		break;
	case TT_External:
		fprintf(f,"\npsExternal: ");
		DBVprint_external(f, sTerm.uTerm.psExternal);
		break;
	case TT_IntegerAndUnit:
		fprintf(f,"\nTT_IntegerAndUnit:\n");
		DBVprint_intunit(f, sTerm.uTerm.psIntegerAndUnit);
		break;
	case TT_Null:
		fprintf(f,"\nTT_Null: ");
		break;
	}

}



int DBVprint_DbvSpecification(FILE *f, char *name, DbvSpecification *psSpecification)
{
	if (psSpecification == NULL) {
           fprintf(f, "%s: NULL\n", name);
           return 1;
	}
   
	fprintf(f, "%s:\n", name);

	DBVprint_DbvObjectIdentifier(f, "pcSchema", psSpecification->pcSchema);
	DBVprint_DbvElementSpec(f, "psElementSpec", psSpecification->psElementSpec);

	return 1;
}



int DBVprint_DbvElementSpec(FILE *f, char *name, DbvElementSpec *psElementSpec)
{
	if (psElementSpec == NULL) {
		fprintf(f, "%s: NULL\n", name);
		return 1;
	} 
   
	fprintf(f, "%s:\n", name);

        switch( psElementSpec->eElementSpecType) {
        case EST_Primitive:
                fprintf(f,"EST_Primitive: ");
                DBVprint_inter_string(f, psElementSpec->uElementSpec.psPrimitive);
                break;
        case EST_External:
                fprintf(f,"EST_External:\n");
                DBVprint_external(f, psElementSpec->uElementSpec.psExternal);
                break;
        }

	return 1;
}
 


int DBVprint_attelmnt_list(FILE *f, DbvAttributeElement **ppsAttributeElement)
{
        int index = 0;
 
	if (ppsAttributeElement != NULL) {
        	while( ppsAttributeElement[index] != NULL) {
                	DBVprint_att_element(f, ppsAttributeElement[index]);
                	index++;
        	}
	}
	fprintf(f,"\n");
}
 
 
 
 


int DBVprint_invalidapdu(FILE *f, DbvOctetString *psInvalidAPDU)
{
	char	*ber_data = NULL;
	long	length = 0;

	fprintf(f,"Invalid PDU:\n");

	ber_data = psInvalidAPDU->pcValue;
	length = psInvalidAPDU->ulLength;

	DBVprint_as_hex(f, ber_data, length);
}



int DBVprint_explain_external(FILE *f, DbvASN1External *external_ptr)
{
        fprintf(f,"\tpsExplainRecord:\n");

	fprintf(f,"eExplainRecordType: ");


	/* If not NULL */
	if (external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord) {

		switch(external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->eExplainRecordType) {
		case ERT_TargetInfo:
			fprintf(f,"ERT_TargetInfo\n");
			DBVprint_DbvTargetInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psTargetInfo);
			break;

		case ERT_DatabaseInfo:
			fprintf(f,"ERT_DatabaseInfo\n");
			DBVprint_DbvDatabaseInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psDatabaseInfo);

			break;
	
		case ERT_SchemaInfo:
			fprintf(f,"ERT_SchemaInfo\n");
			DBVprint_DbvSchemaInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psSchemaInfo);
			break;

		case ERT_TagSetInfo:
			fprintf(f,"ERT_TagSetInfo\n");
			DBVprint_DbvTagSetInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psTagSetInfo);
			break;

		case ERT_RecordSyntaxInfo:
			fprintf(f,"ERT_RecordSyntaxInfo\n");
			DBVprint_DbvRecordSyntaxInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psRecordSyntaxInfo);
			break;
	
		case ERT_AttributeSetInfo:
			fprintf(f,"ERT_AttributeSetInfo\n");
			DBVprint_DbvAttributeSetInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psAttributeSetInfo);
			break;

		case ERT_TermListInfo:
			fprintf(f,"ERT_TermListInfo\n");
			DBVprint_DbvTermListInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psTermListInfo);
			break;

		case ERT_ExtendedServicesInfo:
			fprintf(f,"ERT_ExtendedServicesInfo\n");
			DBVprint_DbvExtendedServicesInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psExtendedServicesInfo);

			break;

		case ERT_AttributeDetails:
			fprintf(f,"ERT_AttributeDetails\n");
			DBVprint_DbvAttributeDetails(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psAttributeDetails);

			break;

		case ERT_TermListDetails:
			fprintf(f,"ERT_TermListDetails\n");
			DBVprint_DbvTermListDetails(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psTermListDetails);

			break;

		case ERT_ElementSetDetails:
			fprintf(f,"ERT_ElementSetDetails\n");
			DBVprint_DbvElementSetDetails(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psElementSetDetails);

			break;

		case ERT_RetrievalRecordDetails:
			fprintf(f,"ERT_RetrievalRecordDetails\n");
			DBVprint_DbvRetrievalRecordDetails(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psRetrievalRecordDetails);

			break;

		case ERT_SortDetails:
			fprintf(f,"ERT_SortDetails\n");
			DBVprint_DbvSortDetails(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psSortDetails);

			break;

		case ERT_Processing:
			fprintf(f,"ERT_Processing\n");
			DBVprint_DbvProcessingInformation(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psProcessing);

			break;

		case ERT_Variants:
			fprintf(f,"ERT_Variants\n");
			DBVprint_DbvVariantSetInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psVariants);

			break;

		case ERT_Units:
			fprintf(f,"ERT_Units\n");
			DBVprint_DbvUnitInfo(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psUnits);

			break;
	
		case ERT_CategoryList:
			fprintf(f,"ERT_CategoryList\n");
			DBVprint_DbvCategoryList(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord->uExplainRecord.psCategoryList);

			break;
		}
	}
}




/* -------------------- DbvTargetInfo -------------------- */

void DBVprint_DbvTargetInfo(FILE *f, DbvTargetInfo *psTargetInfo)
   {


   /* Write the title out */ 
   if (psTargetInfo == NULL)
      {
      fprintf(f, "psTargetInfo: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psTargetInfo:\n");
   

   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f,  "psCommonInfo", psTargetInfo->psCommonInfo);

   /* sName */
   fprintf(f,  "sName: ");
   DBVprint_inter_string(f,  &psTargetInfo->sName);

   /* ppsRecentNews */
   DBVprintN_DbvHumanString(f, "ppsRecentNews", psTargetInfo->ppsRecentNews);

   /* ppsIcon */
   DBVprintN_DbvIconObject(f, "ppsIcon", psTargetInfo->ppsIcon);

   /* fNamedResultSets */
   fprintf(f,"fNamedResultSets: ");
   DBVprint_bool(f, &psTargetInfo->fNamedResultSets);
   

   /* fMultipleDBSearch */
   fprintf(f,"fMultipleDBSearch:" );
   DBVprint_bool(f, &psTargetInfo->fMultipleDBSearch);
   

   /* pulMaxResultSets */
   fprintf(f, "pulMaxResultSets: ");
   DBVprint_Ulong(f, psTargetInfo->pulMaxResultSets);



   /* pulMaxResultSize */
   fprintf(f, "pulMaxResultSize: ");
   DBVprint_Ulong(f, psTargetInfo->pulMaxResultSize);
 

   /* pulMaxTerms */
   fprintf(f, "pulMaxTerms: ");
   DBVprint_Ulong(f, psTargetInfo->pulMaxTerms);
 

   /* psTimeoutInterval */
   fprintf(f, "psTimeoutInterval: ");
   DBVprint_intunit(f,  psTargetInfo->psTimeoutInterval);

   /* ppsWelcomeMessage */
   DBVprintN_DbvHumanString(f, "ppsWelcomeMessage", 
			  psTargetInfo->ppsWelcomeMessage);

   /* psContactInfo */
   DBVprint_DbvContactInfo(f, "psContactInfo", psTargetInfo->psContactInfo);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psTargetInfo->ppsDescription);

   /* ppsNicknames */
   DBVprintN_DbvInternationalString(f, "ppsNicknames",
			  psTargetInfo->ppsNicknames);

   /* ppsUsageRest */
   DBVprintN_DbvHumanString(f, "ppsUsageRest", psTargetInfo->ppsUsageRest);

   /* ppsPaymentAddr */
   DBVprintN_DbvHumanString(f, "ppsPaymentAddr", psTargetInfo->ppsPaymentAddr);

   /* ppsHours */
   DBVprintN_DbvHumanString(f, "ppsHours", psTargetInfo->ppsHours);

   /* ppsDbCombinations */
   DBVprintN_DbvDatabaseList(f, "ppsDbCombinations",
			  psTargetInfo->ppsDbCombinations);

   /* ppsAddresses */
   DBVprintN_DbvNetworkAddress(f, "ppsAddresses", psTargetInfo->ppsAddresses);

   /* ppsLanguages */
   DBVprintN_DbvInternationalString(f, "ppsLanguages",
			  psTargetInfo->ppsLanguages);

   /* psCommonAccessInfo */
   DBVprint_DbvAccessInfo(f, "psCommonAccessInfo", psTargetInfo->psCommonAccessInfo);

   } /* End DBVprint_DbvTargetInfo() */



void 
DBVprint_Ulong(FILE *f, Ulong *pulMaxResultSets)
{
	if (pulMaxResultSets == NULL) {
		fprintf(f, "NULL\n");
		return;
	}

	fprintf(f, "%ld\n", *pulMaxResultSets);
}
   


void DBVprint_DbvCommonInfo(FILE *f, char *name, DbvCommonInfo *psCommonInfo)
   {

   /* Write the title out */ 
   if (psCommonInfo == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   


   /* pcDateAdded */
   DBVprint_DbvGeneralizedTime(f,  "pcDateAdded", psCommonInfo->pcDateAdded);

   /* pcDateChanged */
   DBVprint_DbvGeneralizedTime(f,  "pcDateChanged", psCommonInfo->pcDateChanged);

   /* pcExpiry */
   DBVprint_DbvGeneralizedTime(f,  "pcExpiry", psCommonInfo->pcExpiry);


   /* psHumanStringLanguage */
   DBVprint_DbvLanguageCode(f,  "psHumanStringLanguage", 
			   psCommonInfo->psHumanStringLanguage);


   /* ppsOtherInformation */
   fprintf(f, "ppsOtherInformation: ");
   DBVprint_otherinfo(f, psCommonInfo->ppsOtherInformation);
   
   } /* End DBVprint_DbvCommonInfo() */



void DBVprint_DbvGeneralizedTime(FILE *f, char *name, DbvGeneralizedTime psGeneralizedTime)
   {

   if ( psGeneralizedTime != NULL )
      {
      fprintf(f, "%s: %s\n", name, psGeneralizedTime );
      } /* End if */
   else
      fprintf(f, "%s : NULL\n", name);


   } /* End DBVprint_DbvGeneralizedTime() */





void DBVprint_DbvLanguageCode(FILE *f, char *name,DbvLanguageCode *psLanguageCode )
   {

   fprintf(f, "%s: ", name);
   DBVprint_octet_string(f, (DbvOctetString *)psLanguageCode);

   } /* End DBVprint_DbvLanguageCode() */



void DBVprintN_DbvHumanString(FILE *f, char *name, 
			    DbvHumanString **ppsHumanString )
   {

   if( ppsHumanString != NULL)
      {
      fprintf(f, "%s: \n", name);
      

      while( ppsHumanString && *ppsHumanString )
         {
         DBVprint_DbvHumanString(f, "psHumanString", *ppsHumanString);
         ppsHumanString++;
	 fprintf(f, "\n");
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */
     
   } /* End DBVprintN_DbvHumanString() */



void DBVprint_DbvHumanString(FILE *f, char *name, DbvHumanString *psHumanString)
   {

   /* Write the title out */ 
   if (psHumanString == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

   /* psLanguage */
   DBVprint_DbvLanguageCode(f, "psLanguage", psHumanString->psLanguage);

   /* psText */
   fprintf(f, "sText: ");
   DBVprint_inter_string(f, &psHumanString->sText);

   } /* End DBVprint_DbvHumanString() */



void DBVprintN_DbvIconObject(FILE *f, char *name, DbvIconObject **ppsIconObject)
   {

   if( ppsIconObject != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsIconObject && *ppsIconObject )
         {
         DBVprint_DbvIconObject(f,  *ppsIconObject);
         ppsIconObject++;
	 fprintf(f, "\n");
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvIconObject() */



void DBVprint_DbvIconObject(FILE *f, DbvIconObject *psIconObject)
   {


   /* Write the title out */ 
   if (psIconObject == NULL)
      {
      fprintf(f, "psIconObject: NULL\n");
      
      return;
      } /* End if */

   fprintf(f, "psIconObject:\n");

   /* psBodyType */
   DBVprint_DbvBodyType(f, &psIconObject->sBodyType);
				
   /* sContent */
   fprintf(f, "sContent: ");
   DBVprint_octet_string(f, &psIconObject->sContent);

   } /* End DBVprint_DbvIconObject() */



void DBVprint_DbvBodyType(FILE *f, DbvBodyType *psBodyType)
   {

   /* Write the title out */ 
   if (psBodyType == NULL)
      {
      fprintf(f, "psBodyType: NULL\n");
      
      return;
      } /* End if */

   fprintf(f, "psBodyType:\n");
   


   /* eBodyTypeType */
   fprintf(f, "eBodyTypeType: ");
   DBVprint_DbvBodyTypeType(f, &psBodyType->eBodyTypeType);

   /* uBodyType */
   switch( psBodyType->eBodyTypeType )
      {
      case BTT_IanaType:
	 fprintf( f, "psIanaType: ");
	 DBVprint_inter_string(f, psBodyType->uBodyType.psIanaType);
				 
	 break;

      case BTT_Z3950Type:
	 fprintf( f, "psZ3950Type: ");
	 DBVprint_inter_string(f, psBodyType->uBodyType.psZ3950Type);
	 break;

      case BTT_OtherType:
	 fprintf( f, "psOtherType: ");
	 DBVprint_inter_string(f, psBodyType->uBodyType.psOtherType);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvBodyType() */



void DBVprint_DbvBodyTypeType(FILE *f, DbvBodyTypeType *peBodyTypeType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peBodyTypeType == BTT_IanaType)
         strcpy(temp, "BTT_IanaType");

      if( *peBodyTypeType == BTT_Z3950Type)
         strcpy(temp, "BTT_Z3950Type");

      if( *peBodyTypeType == BTT_OtherType)
         strcpy(temp, "BTT_OtherType");

      if( *peBodyTypeType == BTT_Invalid)
         strcpy(temp, "BTT_Invalid");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvBodyTypeType() */



void DBVprint_DbvContactInfo(FILE *f, char *name, DbvContactInfo *psContactInfo)
   {

   /* Write the title out */ 
   if (psContactInfo == NULL)
      {
      fprintf(f, "psContactInfo: NULL\n");
      
      return;
      } /* End if */

   fprintf(f, "psContactInfo:\n");
   


   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psContactInfo->psName);
				
   /* ppsDescription */
   DBVprintN_DbvHumanString(f,  "ppsDescription",
			  psContactInfo->ppsDescription);

   /* ppsAddress */
   DBVprintN_DbvHumanString(f,  "ppsAddress",
			  psContactInfo->ppsAddress);

   /* psEmail */
   fprintf(f,  "psEmail: ");
   DBVprint_inter_string(f,  psContactInfo->psEmail);

   /* psPhone */
   fprintf(f,  "psPhone: ");
   DBVprint_inter_string(f, psContactInfo->psPhone);

   } /* End DBVprint_DbvContactInfo() */




void DBVprintN_DbvInternationalString(FILE *f, char *name, DbvInternationalString **ppsInternationalString)
   {

   if( ppsInternationalString != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsInternationalString && *ppsInternationalString )
         {
         fprintf(f,  "psInternationalString: ");
         DBVprint_inter_string(f,  *ppsInternationalString);
         ppsInternationalString++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */
     
   } /* End DBVprintN_DbvInternationalString() */



void DBVprintN_DbvDatabaseList(FILE *f, char *name, DbvDatabaseList **ppsDbCombinations)
   {

   if( ppsDbCombinations != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsDbCombinations && *ppsDbCombinations )
         {
         DBVprint_DbvDatabaseList(f, "psDbCombinations", *ppsDbCombinations);
         ppsDbCombinations++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvDatabaseList() */



void DBVprint_DbvDatabaseList(FILE *f, char *name, DbvDatabaseList *psDbCombinations)
   {
   DBVprintN_DbvInternationalString(f, name, psDbCombinations->ppsInnerDatabases);

   } /* End DBVprint_DbvDatabaseList() */



void DBVprintN_DbvNetworkAddress(FILE *f, char *name, DbvNetworkAddress **ppsNetworkAddress )
   {

   if( ppsNetworkAddress != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsNetworkAddress && *ppsNetworkAddress )
         {
         DBVprint_DbvNetworkAddress(f, "psNetworkAddress", *ppsNetworkAddress);
         ppsNetworkAddress++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvNetworkAddress() */



void DBVprint_DbvNetworkAddress(FILE *f, char *name, DbvNetworkAddress *psNetworkAddress)
   {
   char temp[ STRING_LEN  ];

   /* Write the title out */ 
   if (psNetworkAddress == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

   /* eNetworkAddressType */
   fprintf(f, "eNetworkAddress: ");
   DBVprint_DbvNetworkAddressType(f, &psNetworkAddress->eNetworkAddressType);

   /* uNetworkAddress */
   switch( psNetworkAddress->eNetworkAddressType )
      {
      case NAT_InternetAddress:
	 DBVprint_DbvInterAddress(f, "psInternetAddress", psNetworkAddress->uNetworkAddress.psInternetAddress);
	 break;

      case NAT_OsiPresentationAddress:
	 DBVprint_DbvOsiPresentationAddress(f, "psOsiPresentationAddress", psNetworkAddress->uNetworkAddress.psOsiPresentationAddress);
	 break;

      case NAT_OtherAddress:
	 DBVprint_DbvOtherAddress(f, "psOtherAddress", psNetworkAddress->uNetworkAddress.psOtherAddress);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvNetworkAddress() */



void DBVprint_DbvNetworkAddressType(FILE *f, DbvNetworkAddressType *peNetworkAddressType)
   {
   char temp[ SML_BUFFER ];

   if (peNetworkAddressType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */


      temp[ 0 ] = '\0';

      if( *peNetworkAddressType == NAT_InternetAddress)
         strcpy(temp, "NAT_InternetAddress");

      if( *peNetworkAddressType == NAT_OsiPresentationAddress)
         strcpy(temp, "NAT_OsiPresentationAddress");

      if( *peNetworkAddressType == NAT_OtherAddress)
         strcpy(temp, "NAT_OtherAddress");

      if( *peNetworkAddressType == NAT_Invalid)
         strcpy(temp, "NAT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvNetworkAddressType() */



void DBVprint_DbvInterAddress(FILE *f, char *name, DbvInterAddress *psInterAddress)
   {
   char temp[ SML_BUFFER ];


   /* Write the title out */ 
   if (psInterAddress == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

   /* sHostAddress */
   fprintf(f, "sHostAddress: ");
   DBVprint_inter_string(f, &psInterAddress->sHostAddress);

   /* ulPort */
   fprintf(f, "ulPort: ");
   DBVprint_Ulong(f, &psInterAddress->ulPort);
   

   } /* End DBVprint_DbvInterAddress() */



void DBVprint_DbvOsiPresentationAddress(FILE *f, char *name, DbvOsiPresentationAddress *psOsiPresentationAddress)
   {

   /* Write the title out */ 
   if (psOsiPresentationAddress == NULL)
      {
      fprintf(f, "%s: NULL\n",name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   


   /* sPsel */
   fprintf(f, "sPsel: ");
   DBVprint_inter_string(f, &psOsiPresentationAddress->sPsel);

   /* psSSel */
   fprintf(f, "psSSel: ");
   DBVprint_inter_string(f, psOsiPresentationAddress->psSSel);

   /* psTSel */
   fprintf(f, "psTSel: ");
   DBVprint_inter_string(f, psOsiPresentationAddress->psTSel);

   /* sNSap */
   fprintf(f, "sNSap: ");
   DBVprint_inter_string(f, &psOsiPresentationAddress->sNSap);

   } /* End DBVprint_DbvOsiPresentationAddress() */



void DBVprint_DbvOtherAddress(FILE *f, char *name, DbvOtherAddress *psOtherAddress)
   {

   /* Write the title out */ 
   if (psOtherAddress == NULL)
      {
      fprintf(f, "%s: NULL\n",name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   


   /* sType */
   fprintf(f, "sType: ");
   DBVprint_inter_string(f, &psOtherAddress->sType);

   /* sAddress */
   fprintf(f, "sAddress: ");
   DBVprint_inter_string(f, &psOtherAddress->sAddress);

   } /* End DBVprint_DbvOtherAddress() */




void DBVprint_DbvAccessInfo(FILE *f, char *name, DbvAccessInfo *psAccessInfo)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psAccessInfo == NULL)
      {
      fprintf(f, "%s: NULL\n",name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

   /* ppsQueryTypesSupported */
   DBVprintN_DbvQueryTypeDetails(f, "ppsQueryTypesSupported", psAccessInfo->ppsQueryTypesSupported);

   /* ppcDiagnosticsSets */
   DBVprintN_DbvObjectIdentifier(f, "ppcDiagnosticsSets", 
				psAccessInfo->ppcDiagnosticsSets);

   /* ppcAttributeSetIds */
   DBVprintN_DbvObjectIdentifier(f, "ppcAttributeSetIds",
				psAccessInfo->ppcAttributeSetIds);

   /* ppcSchemas */
   DBVprintN_DbvObjectIdentifier(f, "ppcSchemas", psAccessInfo->ppcSchemas);

   /* ppcRecordSyntaxes */
   DBVprintN_DbvObjectIdentifier(f, "ppcRecordSyntaxes", 
				psAccessInfo->ppcRecordSyntaxes);

   /* ppcResourceChallenges */
   DBVprintN_DbvObjectIdentifier(f, "ppcResourceChallenges",
				psAccessInfo->ppcResourceChallenges);

   /* ppsRestrictedAccess */
   DBVprintN_DbvAccessRestrictions(f, psAccessInfo->ppsRestrictedAccess);

   /* psCostInfo */
   DBVprint_DbvCosts(f, psAccessInfo->psCostInfo);

   /* ppcVariantSets */
   DBVprintN_DbvObjectIdentifier(f, "ppcVariantSets", 
				psAccessInfo->ppcVariantSets);

   /* ppsElementSetNames */
   DBVprintN_DbvInternationalString(f, "ppsElementSetNames",
				 psAccessInfo->ppsElementSetNames);
   /* ppsUnitSystems */
   DBVprintN_DbvInternationalString(f, "ppsUnitSystems", 
				  psAccessInfo->ppsUnitSystems);

   } /* End DBVprint_DbvAccessInfo() */



void DBVprintN_DbvQueryTypeDetails(FILE *f, char *name, DbvQueryTypeDetails **ppsQueryTypeDetails )
   {

   if( ppsQueryTypeDetails != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsQueryTypeDetails && *ppsQueryTypeDetails )
         {
         DBVprint_DbvQueryTypeDetails(f, "psQueryTypeDetails",  *ppsQueryTypeDetails);
         ppsQueryTypeDetails++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */
     
   } /* End DBVprintN_DbvQueryTypeDetails() */



void DBVprint_DbvQueryTypeDetails(FILE *f, char *name, 
		DbvQueryTypeDetails *psQueryTypeDetails)
   {

   /* Write the title out */ 
   if (psQueryTypeDetails == NULL)
      {
      fprintf(f, "%s: NULL\n",name);
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

   /* eQueryTypeDetailsType */
   fprintf(f, "eQueryTypeDetailsType: "); 
   DBVprint_DbvQueryTypeDetailsType(f, &psQueryTypeDetails->eQueryTypeDetailsType);

   /* uQueryTypeDetails */
   switch( psQueryTypeDetails->eQueryTypeDetailsType )
      {
      case QTDT_Private:
	 DBVprint_DbvPrivateCapabilities(f, psQueryTypeDetails->
				       uQueryTypeDetails.psPrivate );
	 break;

      case QTDT_Rpn:
	 DBVprint_DbvRpnCapabilities( f, psQueryTypeDetails->
				   uQueryTypeDetails.psRpn);
	 break;

      case QTDT_Iso8777:
	 DBVprint_DbvIso8777Capabilities(f,  psQueryTypeDetails->
				       uQueryTypeDetails.psIso8777);
	 break;

      case QTDT_Z39_58:
	 DBVprintN_DbvHumanString( f, "ppsZ39_58",
				psQueryTypeDetails->uQueryTypeDetails.ppsZ39_58);
	 break;

      case QTDT_Erpn:
	 DBVprint_DbvRpnCapabilities(f,  psQueryTypeDetails->
				   uQueryTypeDetails.psErpn);
	 break;

      case QTDT_RankedList:
	 DBVprintN_DbvHumanString( f, "ppsRankedList",
			       psQueryTypeDetails->
			       uQueryTypeDetails.ppsRankedList);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvQueryTypeDetails() */



void DBVprint_DbvQueryTypeDetailsType(FILE *f, DbvQueryTypeDetailsType *peQueryTypeDetailsType)
   {
   char temp[ SML_BUFFER ];

   if (peQueryTypeDetailsType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */


      temp[ 0 ] = '\0';
      if( *peQueryTypeDetailsType == QTDT_Private)
         strcpy(temp, "QTDT_Private");

      if( *peQueryTypeDetailsType == QTDT_Rpn)
         strcpy(temp, "QTDT_Rpn");

      if( *peQueryTypeDetailsType == QTDT_Iso8777)
         strcpy(temp, "QTDT_Iso8777");

      if( *peQueryTypeDetailsType == QTDT_Z39_58)
         strcpy(temp, "QTDT_Z39_58");

      if( *peQueryTypeDetailsType == QTDT_Erpn)
         strcpy(temp, "QTDT_Erpn");

      if( *peQueryTypeDetailsType == QTDT_RankedList)
         strcpy(temp, "QTDT_RankedList");

      if( *peQueryTypeDetailsType == QTDT_Invalid)
         strcpy(temp, "QTDT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvQueryTypeDetailsType() */



void DBVprint_DbvPrivateCapabilities(FILE *f, DbvPrivateCapabilities *psPrivateCapabilities)
   {

   /* Write the title out */ 
   if (psPrivateCapabilities == NULL)
      {
      fprintf(f, "%s: NULL\n","psPrivateCapabilities");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psPrivateCapabilities");


   /* ppsOperators */
   DBVprintN_DbvOperators(f,  psPrivateCapabilities->ppsOperators);

   /* ppsSearchKey */
   DBVprintN_DbvSearchKey(f,  psPrivateCapabilities->ppsSearchKeys);

   /* ppsDescription */
   DBVprintN_DbvHumanStringSeq(f, "ppsDescription", 
			  psPrivateCapabilities->ppsDescription);

   } /* End DBVprint_DbvPrivateCapabilities() */



void DBVprintN_DbvOperators(FILE *f, DbvOperators **ppsOperators)
   {

   if( ppsOperators != NULL)
      {
      fprintf(f, "%s:\n", "ppsOperators");

      while( ppsOperators && *ppsOperators )
         {
         DBVprint_DbvOperators(f, *ppsOperators);
         ppsOperators++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsOperators");
      
      } /* End else */

     
   } /* End DBVprintN_DbvOperators() */



void DBVprint_DbvOperators(FILE *f,  DbvOperators *psOperators)
   {

   /* Write the title out */ 
   if (psOperators == NULL)
      {
      fprintf(f, "%s: NULL\n","psOperators");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psOperators");
   


   /* sOperator */
   fprintf(f, "sOperator: ");
   DBVprint_inter_string(f, &psOperators->sOperator);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psOperators->ppsDescription);

   } /* End DBVprint_DbvOperators() */



void DBVprintN_DbvSearchKey(FILE *f, DbvSearchKey **ppsSearchKey)
   {

   if( ppsSearchKey != NULL)
      {
      fprintf(f, "%s:\n", "ppsSearchKey");

      while( ppsSearchKey && *ppsSearchKey )
         {
         DBVprint_DbvSearchKey(f, *ppsSearchKey);
         ppsSearchKey++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsSearchKey");
      
      } /* End else */

     
   } /* End DBVprintN_DbvSearchKey() */



void DBVprint_DbvSearchKey(FILE *f, DbvSearchKey *psSearchKey)
   {

   /* Write the title out */ 
   if (psSearchKey == NULL)
      {
      fprintf(f, "%s: NULL\n","psSearchKey");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psSearchKey");
   


   /* sSearchKey */
   fprintf(f, "sSearchKey: ");
   DBVprint_inter_string(f, &psSearchKey->sSearchKey);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psSearchKey->ppsDescription);

   } /* End DBVprint_DbvSearchKey() */



void DBVprintN_DbvHumanStringSeq(FILE *f, char *name, DbvHumanStringSeq **ppsDescription)
   {

   if( ppsDescription != NULL)
      {
      fprintf(f, "%s:\n", name);
      
      while( ppsDescription && *ppsDescription )
         {
         DBVprint_DbvHumanStringSeq(f, *ppsDescription);
         ppsDescription++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvHumanStringSeq() */



void DBVprint_DbvHumanStringSeq(FILE *f,  DbvHumanStringSeq *psHumanStringSeq)
   {

   /* Write the title out */ 
   if (psHumanStringSeq == NULL)
      {
      fprintf(f, "%s: NULL\n","psHumanStringSeq");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psHumanStringSeq");
   

   DBVprintN_DbvHumanString(f, "ppsHumanString", psHumanStringSeq->ppsHumanString);

   } /* End DBVprint_DbvHumanStringSeq() */



void DBVprint_DbvRpnCapabilities(FILE *f, DbvRpnCapabilities *psRpnCapabilities)
   {

   /* Write the title out */ 
   if (psRpnCapabilities == NULL)
      {
      fprintf(f, "%s: NULL\n","psRpnCapabilities");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psRpnCapabilities");
   


   /* ppulOperators */
   DBVprintN_Ulong(f, "ppulOperators", psRpnCapabilities->ppulOperators);

   /* fResultSetAsOperandSupported */
   fprintf(f, "fResultSetAsOperandSupported: ");
   DBVprint_bool(f, &psRpnCapabilities->fResultSetAsOperandSupported);
   

   /* fRestrictionOperandSupported */
   fprintf(f, "fRestrictionOperandSupported: ");
   DBVprint_bool(f, &psRpnCapabilities->fRestrictionOperandSupported);
   

   /* psProximity */
   DBVprint_DbvProximitySupport(f, psRpnCapabilities->psProximity);

   } /* End DBVprint_DbvRpnCapabilities() */



void DBVprintN_Ulong(FILE *f, char *name, Ulong **ppsUlong)
   {

   if( ppsUlong != NULL)
      {
      fprintf(f, "%s:\n", name);
      

      while( ppsUlong && *ppsUlong )
         {
         fprintf(f, "psUlong: ");
         DBVprint_Ulong(f, *ppsUlong);
	 
         ppsUlong++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_Ulong() */


void DBVprint_DbvProximitySupport(FILE *f, DbvProximitySupport *psProximitySupport)
   {


   /* Write the title out */ 
   if (psProximitySupport == NULL)
      {
      fprintf(f, "%s: NULL\n","psProximitySupport");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psProximitySupport");
   


   /* fAnySupport */
   fprintf(f, "fAnySupport: ");
   DBVprint_bool(f, &psProximitySupport->fAnySupport);
   

   /* ppsUnitsSupported */
   DBVprintN_DbvUnitsSupported(f, psProximitySupport->ppsUnitsSupported);

   } /* End DBVprint_DbvProximitySupport() */



void DBVprintN_DbvUnitsSupported(FILE *f, DbvUnitsSupported **ppsUnitsSupported )
   {

   if( ppsUnitsSupported != NULL)
      {
      fprintf(f, "%s:\n", "ppsUnitsSupported");

      while( ppsUnitsSupported && *ppsUnitsSupported )
         {
         DBVprint_DbvUnitsSupported(f, *ppsUnitsSupported);
         ppsUnitsSupported++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsUnitsSupported");
      
      } /* End else */

     
   } /* End DBVprintN_DbvUnitsSupported() */



void DBVprint_DbvUnitsSupported(FILE *f,  DbvUnitsSupported *psUnitsSupported)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psUnitsSupported == NULL)
      {
      fprintf(f, "%s: NULL\n","psUnitsSupported");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psUnitsSupported");
   

   /* eUnitsSupportedType */
   fprintf(f, "eUnitsSupportedType: ");
   DBVprint_DbvUnitsSupportedType(f, &psUnitsSupported->eUnitsSupportedType);

   /* uUnitsSupported */
   switch( psUnitsSupported->eUnitsSupportedType )
      {
      case UST_Known:
         fprintf(f, "ulKnown: ");
         DBVprint_Ulong(f, &psUnitsSupported->uUnitsSupported.ulKnown );
	 break;

      case UST_Private:
	 DBVprint_DbvPrivate(f, psUnitsSupported->uUnitsSupported.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvUnitsSupported() */



void DBVprint_DbvUnitsSupportedType(FILE *f, DbvUnitsSupportedType *peUnitsSupportedType)
   {
   char temp[ SML_BUFFER ];

   if (peUnitsSupportedType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peUnitsSupportedType == UST_Known)
         strcpy(temp, "UST_Known");

      if( *peUnitsSupportedType == UST_Private)
         strcpy(temp, "UST_Private");

      if( *peUnitsSupportedType == UST_Invalid)
         strcpy(temp, "UST_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvUnitsSupportedType() */



void DBVprint_DbvPrivate(FILE *f,  DbvPrivate *psPrivate)
   {
   char temp[ STRING_LEN  ];

   /* Write the title out */ 
   if (psPrivate == NULL)
      {
      fprintf(f, "%s: NULL\n","psPrivate");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psPrivate");
   


   /* ulUnit */
   fprintf(f, "ulUnit: ");
   DBVprint_Ulong(f, &psPrivate->ulUnit );
   

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psPrivate->ppsDescription);


   } /* End DBVprint_DbvPrivate() */



void DBVprint_DbvIso8777Capabilities(FILE *f, DbvIso8777Capabilities *psIso8777Capabilities)
   {

   /* Write the title out */ 
   if (psIso8777Capabilities == NULL)
      {
      fprintf(f, "%s: NULL\n","psIso8777Capabilities");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psIso8777Capabilities");
   

   /* ppsSearchKeys */
   DBVprintN_DbvSearchKey(f, psIso8777Capabilities->ppsSearchKeys);

   /* ppsRestrictions */
   DBVprintN_DbvHumanString(f, "ppsRestrictions", 
			  psIso8777Capabilities->ppsRestrictions);

   } /* End DBVprint_DbvIso8777Capabilities() */



void DBVprintN_DbvObjectIdentifier(FILE *f, char *name, DbvObjectIdentifier *ppcObjectIdentifier )
   {

   if( ppcObjectIdentifier != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppcObjectIdentifier && *ppcObjectIdentifier )
         {
         DBVprint_DbvObjectIdentifier(f, "pcObjectIdentifier", *ppcObjectIdentifier);
         ppcObjectIdentifier++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvObjectIdentifier() */




void DBVprintN_DbvAccessRestrictions(FILE *f, DbvAccessRestrictions **ppsAccessRestrictions)
   {

   if( ppsAccessRestrictions != NULL)
      {
      fprintf(f, "%s:\n", "ppsAccessRestrictions");

      while( ppsAccessRestrictions && *ppsAccessRestrictions )
         {
         DBVprint_DbvAccessRestrictions(f, *ppsAccessRestrictions);
         ppsAccessRestrictions++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAccessRestrictions");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAccessRestrictions() */



void DBVprint_DbvAccessRestrictions(FILE *f,  DbvAccessRestrictions *psAccessRestrictions)
   {
   char temp[ STRING_LEN ];


   /* Write the title out */ 
   if (psAccessRestrictions == NULL)
      {
      fprintf(f, "%s: NULL\n","psAccessRestrictions");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAccessRestrictions");
   


   /* eAccessType */
   fprintf(f, "eAccessType: ");
   DBVprint_DbvAccessType( f, &psAccessRestrictions->eAccessType);

   /* ppsAccessText */
   DBVprintN_DbvHumanString(f, "ppsAccessText", psAccessRestrictions->ppsAccessText);

   /* ppcAccessChallenges */
   DBVprintN_DbvObjectIdentifier(f, "ppcAccessChallenges", 
				psAccessRestrictions->ppcAccessChallenges);

   } /* End DBVprint_DbvAccessRestrictions() */




void DBVprint_DbvAccessType(FILE *f, DbvAccessType *peAccessType)
   {
   char temp[ SML_BUFFER ];

   if (peAccessType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peAccessType == AT_Any)
         strcpy(temp, "AT_Any");

      if( *peAccessType == AT_Search)
         strcpy(temp, "AT_Search");

      if( *peAccessType == AT_Present)
         strcpy(temp, "AT_Present");

      if( *peAccessType == AT_SpecificElements)
         strcpy(temp, "AT_SpecificElements");

      if( *peAccessType == AT_ExtendedServices)
         strcpy(temp, "AT_ExtendedServices");

      if( *peAccessType == AT_ByDatabase)
         strcpy(temp, "AT_ByDatabase");

      if( *peAccessType == AT_Invalid)
         strcpy(temp, "AT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvAccessType() */





void DBVprint_DbvCosts(FILE *f, DbvCosts *psCosts)
   {

   /* Write the title out */ 
   if(psCosts == NULL)
      {
      fprintf(f, "%s: NULL\n", "psCosts");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psCosts");


   /* psConnectCharge */
   DBVprint_DbvCharge(f, psCosts->psConnectCharge);

   /* psConnectTime */
   DBVprint_DbvCharge(f, psCosts->psConnectTime);

   /* psDisplayCharge */
   DBVprint_DbvCharge(f, psCosts->psDisplayCharge);

   /* psSearchCharge */
   DBVprint_DbvCharge(f, psCosts->psSearchCharge);

   /* ppsOtherCharge */
   DBVprintN_DbvOtherCharge(f, psCosts->ppsOtherCharge);

   } /* End DBVprint_DbvCosts() */



void DBVprint_DbvCharge( FILE *f, DbvCharge *psCharge)
   {


   /* Write the title out */ 
   if(psCharge == NULL)
      {
      fprintf(f, "%s: NULL\n", "psCharge");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psCharge");


   /* sCost */
   DBVprint_intunit(f, &psCharge->sCost);

   /* psPerWhat */
   DBVprint_DbvUnit(f, psCharge->psPerWhat);

   /* ppsText */
   DBVprintN_DbvHumanString(f, "ppsText", psCharge->ppsText);

   } /* End DBVprint_DbvCharge() */



void DBVprint_DbvUnit( FILE *f, DbvUnit *psPerWhat)
   {

   /* Write the title out */ 
   if(psPerWhat == NULL)
      {
      fprintf(f, "%s: NULL\n", "psPerWhat");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psPerWhat");

   DBVprint_unitused(f, *psPerWhat);
} /* End DBVprint_DbvUnit() */



void DBVprintN_DbvOtherCharge(FILE *f, DbvOtherCharge **ppsOtherCharge)
   {

   if( ppsOtherCharge != NULL)
      {
      fprintf(f, "%s:\n", "psOtherCharge");

      while( ppsOtherCharge && *ppsOtherCharge )
         {
         DBVprint_DbvOtherCharge(f, *ppsOtherCharge);
         ppsOtherCharge++;
         } /* End while */
      } /* End if */
   else
      fprintf(f, "%s: NULL\n", "psOtherCharge");

     
   } /* End DBVprintN_DbvOtherCharge() */



void DBVprint_DbvOtherCharge(FILE *f, DbvOtherCharge *psOtherCharge)
   {

   /* Write the title out */ 
   if(psOtherCharge == NULL)
      {
      fprintf(f, "%s: NULL\n", "psOtherCharge");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psOtherCharge");


   /* ppsForWhat */
   DBVprintN_DbvHumanString(f, "ppsForWhat", psOtherCharge->ppsForWhat);

   /* sCharge */
   DBVprint_DbvCharge(f, &psOtherCharge->sCharge);

   } /* End DBVprint_DbvOtherCharge() */



/* -------------------- DbvDatabaseInfo -------------------- */

void DBVprint_DbvDatabaseInfo(FILE *f, DbvDatabaseInfo *psDatabaseInfo)
   {


   /* Write the title out */ 
   if (psDatabaseInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psDatabaseInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psDatabaseInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psDatabaseInfo->psCommonInfo);

   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psDatabaseInfo->sName);

   /* pvExplainDatabase */
   DBVprint_DbvNull(f, "pvExplainDatabase",
		  psDatabaseInfo->pvExplainDatabase);

   /* ppsNicknames */
   DBVprintN_DbvInternationalString(f, "ppsNicknames",
				psDatabaseInfo->ppsNicknames);

   /* ppsIcon */
   DBVprintN_DbvIconObject(f, "ppsIcon", psDatabaseInfo->ppsIcon);

   /* fUserFee */
   fprintf(f,"fUserFee: ");
   DBVprint_bool(f, &psDatabaseInfo->fUserFee);
   

   /* fAvailable */
   fprintf(f, "fAvailable: ");
   DBVprint_bool(f, &psDatabaseInfo->fAvailable);
   

   /* ppsTitleString */
   DBVprintN_DbvHumanString(f, "ppsTitleString", psDatabaseInfo->ppsTitleString);
   /* ppsKeywords */
   DBVprintN_DbvDataBaseInfoSeq(f, "ppsKeywords", psDatabaseInfo->ppsKeywords);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psDatabaseInfo->ppsDescription);

   /* ppsAssociatedDbs */
   DBVprintN_DbvInternationalString(f, "ppsAssociatedDbs",
				psDatabaseInfo->ppsAssociatedDbs);

   /* ppsSubDbs */
   DBVprintN_DbvInternationalString(f, "ppsSubDbs",
				psDatabaseInfo->ppsSubDbs);
   /* ppsDisclaimers */
   DBVprintN_DbvHumanString(f, "ppsDisclaimers", psDatabaseInfo->ppsDisclaimers);

   /* ppsNews */
   DBVprintN_DbvHumanString(f, "ppsNews", psDatabaseInfo->ppsNews);

   /* psRecordCount */
   DBVprint_DbvRecordCount(f, psDatabaseInfo->psRecordCount);

   /* ppsDefaultOrder */
   DBVprintN_DbvHumanString(f, "ppsDefaultOrder", psDatabaseInfo->ppsDefaultOrder);

   /* pulAvRecordSize */
   fprintf(f, "pulAvRecordSize: ");
   DBVprint_Ulong(f, psDatabaseInfo->pulAvRecordSize);
   

   /* pulMaxRecordSize */
   fprintf(f, "pulMaxRecordSize: ");
   DBVprint_Ulong(f, psDatabaseInfo->pulMaxRecordSize);
   

   /* ppsHours */
   DBVprintN_DbvHumanString(f, "ppsHours", psDatabaseInfo->ppsHours);

   /* ppsBestTime */
   DBVprintN_DbvHumanString(f, "ppsBestTime",psDatabaseInfo->ppsBestTime);

   /* pcLastUpdate */
   DBVprint_DbvGeneralizedTime(f, "pcLastUpdate", psDatabaseInfo->pcLastUpdate);

   /* psUpdateInterval */
   fprintf(f, "psUpdateInterval: ");
   DBVprint_intunit(f, psDatabaseInfo->psUpdateInterval);

   /* ppsCoverage */
   DBVprintN_DbvHumanString(f, "ppsCoverage",psDatabaseInfo->ppsCoverage);

   /* pfProprietary */
      fprintf(f, "pfProprietary: ");
      DBVprint_bool(f, psDatabaseInfo->pfProprietary);
   

   /* ppsCopyrightText */
   DBVprintN_DbvHumanString(f, "ppsCopyrightText", psDatabaseInfo->ppsCopyrightText);
   /* ppsCopyrightNotice */
   DBVprintN_DbvHumanString(f, "ppsCopyrightNotice",
				psDatabaseInfo->ppsCopyrightNotice);
   /* psProducerContactInfo */
   DBVprint_DbvContactInfo(f, "psProducerContactInfo", psDatabaseInfo->psProducerContactInfo);

   /* psSupplierContactInfo */
   DBVprint_DbvContactInfo(f, "psSupplierContactInfo", psDatabaseInfo->psSupplierContactInfo);

   /* psSubmissionContactInfo */
   DBVprint_DbvContactInfo(f, "psSubmissionContactInfo", psDatabaseInfo->psSubmissionContactInfo);

   /* psAccessInfo */
   DBVprint_DbvAccessInfo(f, "psAccessInfo", psDatabaseInfo->psAccessInfo);

   } /* End DBVprint_DbvDatabaseInfo() */



void DBVprint_DbvNull(FILE *f, char *name, DbvNull pvNull)
   {
                                
   if(pvNull == NULL)
      fprintf(f, "%s: NULL\n",name);
   else 
      fprintf(f, "%s: Value Defined\n",name);

   return;

   } /* End DBVprint_DbvNull() */



void DBVprintN_DbvDataBaseInfoSeq(FILE *f, char *name, DbvDataBaseInfoSeq **ppsKeywords)
   {
   if( ppsKeywords != NULL)
      {

      fprintf(f, "%s: \n", name);
      
      while( ppsKeywords && *ppsKeywords )
         {
	 fprintf(f, "%s:\n", name);

         DBVprint_DbvDataBaseInfoSeq(f, "psKeywords", *ppsKeywords);
         ppsKeywords++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvDataBaseInfoSeq() */



void DBVprint_DbvDataBaseInfoSeq(FILE *f, char *name, DbvDataBaseInfoSeq *psKeywords)
{
	DBVprintN_DbvHumanString(f, name, psKeywords->ppsDataBaseInfo);
} /* End DBVprint_DbvDataBaseInfoSeq */



void DBVprint_DbvRecordCount(FILE *f, DbvRecordCount *psRecordCount)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psRecordCount == NULL)
      {
      fprintf(f, "%s: NULL\n","psRecordCount");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psRecordCount");
   

   /* eRecordCountType */
   fprintf(f, "eRecordCountType: ");
   DBVprint_DbvRecordCountType(f, &psRecordCount->eRecordCountType);

   /* uRecordCount */

   switch( psRecordCount->eRecordCountType )
      {
      case RCT_ActualNumber:
         fprintf(f, "ulActualNumber: ");
         DBVprint_Ulong(f, &psRecordCount->uRecordCount.ulActualNumber  );
         
	 break;

      case RCT_ApproxNumber:
         fprintf(f, "ulApproxNumber: ");
         DBVprint_Ulong(f, &psRecordCount->uRecordCount.ulApproxNumber  );
         
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvRecordCount() */



void DBVprint_DbvRecordCountType(FILE *f, DbvRecordCountType *peRecordCountType)
   {
   char temp[ SML_BUFFER ];

   if (peRecordCountType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peRecordCountType == RCT_ActualNumber)
         strcpy(temp, "RCT_ActualNumber");

      if( *peRecordCountType == RCT_ApproxNumber)
         strcpy(temp, "RCT_ApproxNumber");

      if( *peRecordCountType == RCT_Invalid)
         strcpy(temp, "RCT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvRecordCountType() */



/* -------------------- DbvSchemaInfo -------------------- */

void DBVprint_DbvSchemaInfo(FILE *f, DbvSchemaInfo *psSchemaInfo)
   {


   /* Write the title out */ 
   if (psSchemaInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psSchemaInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psSchemaInfo");
   

   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psSchemaInfo->psCommonInfo);

   /* pcSchema */
   DBVprint_DbvObjectIdentifier(f, "pcSchema", psSchemaInfo->pcSchema);

   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psSchemaInfo->sName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psSchemaInfo->ppsDescription);

   /* ppsTagTypeMapping */
   DBVprintN_DbvTagTypeMapping(f, "ppsTagTypeMapping", psSchemaInfo->ppsTagTypeMapping);

   /* ppsRecordStructure */
   DBVprintN_DbvElementInfo(f, "ppsRecordStructure", psSchemaInfo->ppsRecordStructure);

   } /* End DBVprint_DbvSchemaInfo() */



void DBVprint_DbvObjectIdentifier(FILE *f, char *name, DbvObjectIdentifier psObjectIdentifier)
   {

   if ( psObjectIdentifier != NULL )
      {
      fprintf(f, "%s: %s\n", name, psObjectIdentifier );
      } /* End if */
   else
      fprintf(f, "%s: NULL\n", name);

   } /* End DBVprint_DbvObjectIdentifier() */



void DBVprintN_DbvTagTypeMapping(FILE *f, char *name, DbvTagTypeMapping **ppsTagTypeMapping )
   {
   if( ppsTagTypeMapping != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsTagTypeMapping && *ppsTagTypeMapping )
         {
         DBVprint_DbvTagTypeMapping(f, *ppsTagTypeMapping);
         ppsTagTypeMapping++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      } /* End else */
     
   } /* End DBVprintN_DbvTagTypeMapping() */



void DBVprint_DbvTagTypeMapping(FILE *f, DbvTagTypeMapping *psTagTypeMapping)
   {


   /* Write the title out */ 
   if (psTagTypeMapping == NULL)
      {
      fprintf(f, "%s: NULL\n","psTagTypeMapping");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psTagTypeMapping");
   


   /* ulTagType */
   fprintf(f, "ulTagType: ");
   DBVprint_Ulong(f, &psTagTypeMapping->ulTagType);
   

   /* pcTagSet */
   DBVprint_DbvObjectIdentifier(f, "pcTagSet", psTagTypeMapping->pcTagSet);

   /* pvDefaultTagType */
   DBVprint_DbvNull(f, "pvDefaultTagType",
		  psTagTypeMapping->pvDefaultTagType);

   } /* End DBVprint_DbvTagTypeMapping() */



void DBVprintN_DbvElementInfo(FILE *f, char *name, DbvElementInfo **ppsElementInfo)
   {

   if( ppsElementInfo != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsElementInfo && *ppsElementInfo )
         {
         DBVprint_DbvElementInfo(f, *ppsElementInfo);
         ppsElementInfo++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */
     
   } /* End DBVprintN_DbvElementInfo() */



void DBVprint_DbvElementInfo(FILE *f, DbvElementInfo *psElementInfo)
   {

   /* Write the title out */ 
   if (psElementInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psElementInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psElementInfo");
   


   /* sElementName */
   fprintf(f, "sElementName: ");
   DBVprint_inter_string(f, &psElementInfo->sElementName);

   /* ppsElementTagPath */
   DBVprintN_DbvPath(f, psElementInfo->ppsElementTagPath);

   /* psDataType */
   DBVprint_DbvElementDataType(f, psElementInfo->psDataType);

   /* fRequired */
   fprintf(f, "fRequired: ");
   DBVprint_bool(f, &psElementInfo->fRequired);
   

   /* fRepeatable */
   fprintf(f, "fRepeatable: ");
   DBVprint_bool(f, &psElementInfo->fRepeatable);
   

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psElementInfo->ppsDescription);

   } /* End DBVprint_DbvElementInfo() */



void DBVprintN_DbvPath(FILE *f, DbvPath **ppsPath)
   {

   if( ppsPath != NULL)
      {
      fprintf(f, "%s:\n", "ppsPath");

      while( ppsPath && *ppsPath )
         {
         DBVprint_DbvPath(f, *ppsPath);
         ppsPath++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsPath");
      
      } /* End else */

     
   } /* End DBVprintN_DbvPath() */


void DBVprint_DbvPath(FILE *f, DbvPath *psPath)
   {

   /* Write the title out */ 
   if (psPath == NULL)
      {
      fprintf(f, "%s: NULL\n","psPath");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psPath");
   


   /* ulTagType */
   fprintf(f, "ulTagType: ");
   DBVprint_Ulong(f, &psPath->ulTagType);
   

   /* sTagValue */
   fprintf(f, "sTagValue: ");
   DBVprint_strornum(f, &psPath->sTagValue);

   } /* End DBVprint_DbvPath() */



void DBVprint_DbvElementDataType(FILE *f, DbvElementDataType *psElementDataType)
   {
   char temp[ STRING_LEN  ];

   /* Write the title out */ 
   if (psElementDataType == NULL)
      {
      fprintf(f, "%s: NULL\n","psElementDataType");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psElementDataType");
   


   /* eElementDataTypeType */
   fprintf(f, "eElementDataType: ");
   DBVprint_DbvElementDataTypeType(f, &psElementDataType->eElementDataTypeType);

   /* uElementDataType */
   switch( psElementDataType->eElementDataTypeType )
      {
      case EDTT_Primitive:
	 DBVprint_DbvPrimitiveDataType(f, "ePrimitive", 
				     &psElementDataType->
				     uElementDataType.ePrimitive);
	 break;

      case EDTT_Structured:
	 DBVprintN_DbvElementInfo(f, "ppsStructured", psElementDataType->
				uElementDataType.ppsStructured);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvElementDataType() */



void DBVprint_DbvElementDataTypeType(FILE *f, DbvElementDataTypeType *peElementDataTypeType)
   {
   char temp[ SML_BUFFER ];

   if (peElementDataTypeType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peElementDataTypeType == EDTT_Primitive)
         strcpy(temp, "EDTT_Primitive");

      if( *peElementDataTypeType == EDTT_Structured)
         strcpy(temp, "EDTT_Structured");

      if( *peElementDataTypeType == EDTT_Invalid)
         strcpy(temp, "EDTT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvElementDataTypeType() */



void DBVprint_DbvPrimitiveDataType(FILE *f, char *name, DbvPrimitiveDataType *pePrimitiveDataType)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if(pePrimitiveDataType == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      
      return;
      } /* End if */

   fprintf(f, "%s: ", name);

   /* ePrimitiveDataType */
   DBVprint_DbvPrimitiveDataTypeA(f, pePrimitiveDataType );
  
   } /* End DBVprint_DbvPrimitiveDataType() */



void DBVprint_DbvPrimitiveDataTypeA(FILE *f, DbvPrimitiveDataType *pePrimitiveDataType)
   {
   char temp[ SML_BUFFER ];

   if (pePrimitiveDataType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';


	switch (*pePrimitiveDataType) {
	case PDT_OctetString:
		fprintf(f, "PDT_OctetString\n");
		break;  
	case PDT_Numeric:
		fprintf(f, "PDT_Numeric\n");
		break;  
	case PDT_Date:
		fprintf(f, "PDT_Date\n");
		break;  
	case PDT_External:
		fprintf(f, "PDT_External\n");
		break;  
	case PDT_String:
		fprintf(f, "PDT_String\n");
		break;  
	case PDT_TrueOrFalse:
		fprintf(f, "PDT_TrueOrFalse\n");
		break;
	case PDT_Oid:
		fprintf(f, "PDT_Oid\n");
		break;
	case PDT_IntUnit:
		fprintf(f, "PDT_IntUnit\n");
		break;
	case PDT_Empty:
		fprintf(f, "PDT_Empty\n");
		break;
	case PDT_NoneOfTheAbove:
		fprintf(f, "PDT_NoneOfTheAbove\n");
		break;
	case PDT_Invalid:
		fprintf(f, "PDT_Invalid\n");
		break;
	}

/*
      if( *pePrimitiveDataType == PDT_OctetString)
         strcpy(temp, "PDT_OctetString");

      if( *pePrimitiveDataType == PDT_Numeric)
         strcpy(temp, "PDT_Numeric");

      if( *pePrimitiveDataType == PDT_Date)
         strcpy(temp, "PDT_Date");

      if( *pePrimitiveDataType == PDT_External)
         strcpy(temp, "PDT_External");

      if( *pePrimitiveDataType == PDT_String)
         strcpy(temp, "PDT_String");

      if( *pePrimitiveDataType == PDT_TrueOrFalse)
         strcpy(temp, "PDT_TrueOrFalse");

      if( *pePrimitiveDataType == PDT_Oid)
         strcpy(temp, "PDT_Oid");

      if( *pePrimitiveDataType == PDT_IntUnit)
         strcpy(temp, "PDT_IntUnit");

      if( *pePrimitiveDataType == PDT_Empty)
         strcpy(temp, "PDT_Empty");

      if( *pePrimitiveDataType == PDT_NoneOfTheAbove)
         strcpy(temp, "PDT_NoneOfTheAbove");

      if( *pePrimitiveDataType == PDT_Invalid)
         strcpy(temp, "PDT_Invalid");

         fprintf(f, temp);
*/

   } /* End DBVprint_DbvPrimitiveDataTypeA() */


/* -------------------- DbvTagSetInfo -------------------- */

void DBVprint_DbvTagSetInfo(FILE *f,  DbvTagSetInfo *psTagSetInfo)
   {


   /* Write the title out */ 
   if (psTagSetInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psTagSetInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psTagSetInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psTagSetInfo->psCommonInfo);

   /* pcTagSet */
   DBVprint_DbvObjectIdentifier(f, "pcTagSet", psTagSetInfo->pcTagSet);

   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psTagSetInfo->sName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psTagSetInfo->ppsDescription);

   /* ppsElements */
   DBVprintN_DbvElements(f, psTagSetInfo->ppsElements);

   } /* End DBVprint_DbvTagSetInfo() */


void DBVprintN_DbvElements(FILE *f, DbvElements **ppsElements)
   {

   if( ppsElements != NULL)
      {
      fprintf(f, "%s:\n", "ppsElements");
      while( ppsElements && *ppsElements )
         {
         DBVprint_DbvElements(f, *ppsElements);
         ppsElements++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsElements");
      
      } /* End else */

     
   } /* End DBVprintN_DbvElements() */



void DBVprint_DbvElements(FILE *f,  DbvElements *psElements)
   {


   /* Write the title out */ 
   if (psElements == NULL)
      {
      fprintf(f, "%s: NULL\n","psElements");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psElements");
   

   /* sElementName */
   fprintf(f, "sElementName: ");
   DBVprint_inter_string(f, &psElements->sElementName);

   /* ppsNicknames */
   DBVprintN_DbvInternationalString(f, "ppsNicknames", 
				  psElements->ppsNicknames);

   /* sElementTag */
   fprintf(f, "sElementTag: ");
   DBVprint_strornum(f, &psElements->sElementTag);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psElements->ppsDescription);

   /* peDataType */
   DBVprint_DbvPrimitiveDataType(f, "peDataType",psElements->peDataType);

   /* ppsOtherInformation */
   fprintf(f, "ppsOtherInformation: ");
   DBVprint_otherinfo(f, psElements->ppsOtherInformation);

   } /* End DBVprint_DbvElements() */




/* -------------------- DbvRecordSyntaxInfo -------------------- */

void DBVprint_DbvRecordSyntaxInfo(FILE *f,  DbvRecordSyntaxInfo *psRecordSyntaxInfo)
   {


   /* Write the title out */ 
   if (psRecordSyntaxInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psRecordSyntaxInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psRecordSyntaxInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psRecordSyntaxInfo->psCommonInfo);

   /* pcRecordSyntax */
   DBVprint_DbvObjectIdentifier(f, "pcRecordSyntax",
			      psRecordSyntaxInfo->pcRecordSyntax);

   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psRecordSyntaxInfo->sName);

   /* ppcTransferSyntaxes */
   DBVprintN_DbvObjectIdentifier(f, "ppcTransferSyntaxes", 
			       psRecordSyntaxInfo->ppcTransferSyntaxes);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psRecordSyntaxInfo->ppsDescription);

   /* psAsn1Module */
   fprintf(f, "psAsn1Module: ");
   DBVprint_inter_string(f, psRecordSyntaxInfo->psAsn1Module);

   /* ppsAbstractStructure */
   DBVprintN_DbvElementInfo(f, "ppsAbstractStructure", psRecordSyntaxInfo->ppsAbstractStructure);

   } /* End DBVprint_DbvRecordSyntaxInfo() */






/* -------------------- DbvAttributeSetInfo -------------------- */

void DBVprint_DbvAttributeSetInfo(FILE *f,  DbvAttributeSetInfo *psAttributeSetInfo )
   {


   /* Write the title out */ 
   if (psAttributeSetInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributeSetInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeSetInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psAttributeSetInfo->psCommonInfo);

   /* pcAttributeSetId */
   DBVprint_DbvObjectIdentifier(f, "pcAttributeSetId", 
				psAttributeSetInfo->pcAttributeSetId);
   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psAttributeSetInfo->sName );

   /* ppsAttributes */
   DBVprintN_DbvAttributeType(f, psAttributeSetInfo->ppsAttributes);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psAttributeSetInfo->ppsDescription );

   } /* End DBVprint_DbvAttributeSetInfo() */



void DBVprintN_DbvAttributeType(FILE *f,  DbvAttributeType **ppsAttributeType )
   {

   if( ppsAttributeType != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributeType");

      while( ppsAttributeType && *ppsAttributeType )
         {
         DBVprint_DbvAttributeType(f, *ppsAttributeType);
         ppsAttributeType++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributeType");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributeType() */



void DBVprint_DbvAttributeType(FILE *f, DbvAttributeType *psAttributeType)
   {


   /* Write the title out */ 
   if (psAttributeType == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributeType");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeType");
   


   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f,  psAttributeType->psName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psAttributeType->ppsDescription);
   /* UlAttributeType */

   fprintf(f,  "ulAttributeType: ");
   DBVprint_Ulong(f,  &psAttributeType->ulAttributeType);
   

   /* ppsAttributeValues */
   DBVprintN_DbvAttributeDescription(f, psAttributeType->ppsAttributeValues);

   } /* End DBVprint_DbvAttributeType() */




void DBVprintN_DbvAttributeDescription(FILE *f, DbvAttributeDescription **ppsAttributeDescription )
   {

   if( ppsAttributeDescription != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributeDescription");

      while( ppsAttributeDescription && *ppsAttributeDescription )
         {
         DBVprint_DbvAttributeDescription(f,  *ppsAttributeDescription);
         ppsAttributeDescription++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributeDescription");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributeDescription() */



void DBVprint_DbvAttributeDescription(FILE *f, DbvAttributeDescription *psAttributeDescription)
   {


   /* Write the title out */ 
   if (psAttributeDescription == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributeDescription");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeDescription");
   


   /* psName */
   fprintf(f,  "psName: ");
   DBVprint_inter_string(f,  psAttributeDescription->psName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", 
			  psAttributeDescription->ppsDescription);

   /* psAttributeValue */
   fprintf(f, "sAttributeValue: ");
   DBVprint_strornum(f, &psAttributeDescription->sAttributeValue);

   /* ppsEquivalentAttributes */
   DBVprintN_DbvStringOrNumeric(f, "ppsEquivalentAttributes", psAttributeDescription->ppsEquivalentAttributes);

   } /* End DBVprint_DbvAttributeDescription() */



void DBVprintN_DbvStringOrNumeric(FILE *f, char *name, DbvStringOrNumeric **ppsStringOrNumeric)
   {

   if( ppsStringOrNumeric != NULL)
      {
      fprintf(f, "%s:\n", name);

      while( ppsStringOrNumeric && *ppsStringOrNumeric )
         {
         fprintf(f, "psStringOrNumeric: ");
         DBVprint_strornum(f, *ppsStringOrNumeric);
         ppsStringOrNumeric++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvStringOrNumeric() */



/* -------------------- DbvTermListInfo -------------------- */

void DBVprint_DbvTermListInfo(FILE *f, DbvTermListInfo *psTermListInfo)
   {


   /* Write the title out */ 
   if (psTermListInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psTermListInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psTermListInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psTermListInfo->psCommonInfo);

   /* psDatabaseName */
   fprintf(f, "psDatabaseName: ");
   DBVprint_inter_string(f, &psTermListInfo->sDatabaseName );

   /* ppsTermLists */
   DBVprintN_DbvTermList(f, psTermListInfo->ppsTermLists);

   } /* End DBVprint_DbvTermListInfo() */



void DBVprintN_DbvTermList(FILE *f, DbvTermList **ppsTermList)
   {

   if( ppsTermList != NULL)
      {
      fprintf(f, "%s:\n", "ppsTermList");

      while( ppsTermList && *ppsTermList )
         {
         DBVprint_DbvTermList(f, *ppsTermList);
         ppsTermList++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsTermList");
      
      } /* End else */

     
   } /* End DBVprintN_DbvTermList() */




void DBVprint_DbvTermList(FILE *f,  DbvTermList *psTermList)
   {


   /* Write the title out */ 
   if (psTermList == NULL)
      {
      fprintf(f, "%s: NULL\n","psTermList");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psTermList");
   


   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psTermList->sName);

   /* ppsTitle */
   DBVprintN_DbvHumanString(f, "ppsTitle", psTermList->ppsTitle);

   /* peSearchCost */
   DBVprint_DbvSearchCost(f, "peSearchCost", psTermList->peSearchCost);

   /* fScanable */
   fprintf(f, "fScanable: ");
   DBVprint_bool(f, &psTermList->fScanable);
   

   /* ppsBroader */
   DBVprintN_DbvInternationalString(f, "ppsBroader", psTermList->ppsBroader );

   /* ppsNarrower */
   DBVprintN_DbvInternationalString(f, "ppsNarrower", psTermList->ppsNarrower);

   } /* End DBVprint_DbvTermList() */


void DBVprint_DbvSearchCost(FILE *f,  char* name, DbvSearchCost *peSearchCost)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if(peSearchCost == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      
      return;
      } /* End if */

   fprintf(f, "%s: ", name);

   /* eSearchCost */
   DBVprint_DbvSearchCostA(f, peSearchCost );

   } /* End DBVprint_DbvSearchCost() */



void DBVprint_DbvSearchCostA(FILE *f, DbvSearchCost *peSearchCost)
   {
   char temp[ SML_BUFFER ];

   if (peSearchCost == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peSearchCost == SC_Optimized)
         strcpy(temp, "SC_Optimized");

      if( *peSearchCost == SC_Normal)
         strcpy(temp, "SC_Normal");

      if( *peSearchCost == SC_Expensive)
         strcpy(temp, "SC_Expensive");

      if( *peSearchCost == SC_Filter)
         strcpy(temp, "SC_Filter");

      if( *peSearchCost == SC_Invalid)
         strcpy(temp, "SC_Invalid");

         fprintf(f, "%s\n", temp);	
   } /* End DBVprint_DbvSearchCostA() */



/* -------------------- DbvExtendedServicesInfo -------------------- */

void DBVprint_DbvExtendedServicesInfo(FILE *f, DbvExtendedServicesInfo *psExtendedServicesInfo )
   {


   /* Write the title out */ 
   if (psExtendedServicesInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psExtendedServicesInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psExtendedServicesInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psExtendedServicesInfo->psCommonInfo);

   /* pcType */
    DBVprint_DbvObjectIdentifier(f, "pcType", psExtendedServicesInfo->pcType );

   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psExtendedServicesInfo->psName );

   /* fPrivateType */
   fprintf(f,"fPrivateType: ");
   DBVprint_bool(f, &psExtendedServicesInfo->fPrivateType );
   

   /* fRestrictionsApply */
   fprintf(f,"fRestrictionsApply: ");
   DBVprint_bool(f, &psExtendedServicesInfo->fRestrictionsApply );
   

   /* fFeeApply */
   fprintf(f,"fFeeApply: ");
   DBVprint_bool(f, &psExtendedServicesInfo->fFeeApply);
   

   /* fAvailable */
   fprintf(f,"fAvailable: ");
   DBVprint_bool(f, &psExtendedServicesInfo->fAvailable);
   

   /* fRetentionSupported */
   fprintf(f,"fRetentionSupported: ");
   DBVprint_bool(f, &psExtendedServicesInfo->fRetentionSupported);
   

   /* ewaitAction */
   DBVprint_DbvExtendedServicesInfoInt(f, &psExtendedServicesInfo->eWaitAction );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", 
			  psExtendedServicesInfo->ppsDescription );

   /* psSpecificExplain */
   fprintf(f, "psSpecificExplain: ");
   DBVprint_external(f, psExtendedServicesInfo->psSpecificExplain);

   /* psEsASN */
   fprintf(f, "psEsASN: ");
   DBVprint_inter_string(f, psExtendedServicesInfo->psEsASN );

   } /* End DBVprint_DbvExtendedServicesInfo() */



void DBVprint_DbvExtendedServicesInfoInt( FILE *f,
	DbvExtendedServicesInfoInt *peExtendedServicesInfoInt)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if(peExtendedServicesInfoInt == NULL)
      {
      fprintf(f, "%s: NULL\n","peExtendedServicesInfoInt");
      
      return;
      } /* End if */

   fprintf(f, "%s: ","peExtendedServicesInfoInt");

   /* eExtendedServicesInfoInt */
   DBVprint_DbvExtendedServicesInfoIntA(f, peExtendedServicesInfoInt );

   } /* End DBVprint_DbvExtendedServicesInfoInt() */



void DBVprint_DbvExtendedServicesInfoIntA(FILE *f, DbvExtendedServicesInfoInt *peExtendedServicesInfoInt)
   {
   char temp[ SML_BUFFER ];

   if (peExtendedServicesInfoInt == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peExtendedServicesInfoInt == ESII_WaitSupported)
         strcpy(temp, "ESII_WaitSupported");

      if( *peExtendedServicesInfoInt == ESII_WaitAlways)
         strcpy(temp, "ESII_WaitAlways");

      if( *peExtendedServicesInfoInt == ESII_WaitNotSupported)
         strcpy(temp, "ESII_WaitNotSupported");

      if( *peExtendedServicesInfoInt == ESII_Depends)
         strcpy(temp, "ESII_Depends");

      if( *peExtendedServicesInfoInt == ESII_NotSaying)
         strcpy(temp, "ESII_NotSaying");

      if( *peExtendedServicesInfoInt == ESII_Invalid)
         strcpy(temp, "ESII_Invalid");

         fprintf(f, "%s\n", temp);	
   } /* End DBVprint_DbvExtendedServicesInfoIntA() */


/* -------------------- DbvAttributeDetails -------------------- */

void DBVprint_DbvAttributeDetails(FILE *f, DbvAttributeDetails *psAttributeDetails )
   {


   /* Write the title out */ 
   if (psAttributeDetails == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributeDetails");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeDetails");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psAttributeDetails->psCommonInfo);


   /* sDatabaseName */
   fprintf( f, "sDatabaseName: ");
   DBVprint_inter_string(f, &psAttributeDetails->sDatabaseName );

   /* ppsAttributesBySet */
   DBVprintN_DbvAttributesBySet(f, psAttributeDetails->ppsAttributesBySet );

   /* psAttributeCombinations */
   DBVprint_DbvAttributeCombinations(f, psAttributeDetails->psAttributeCombinations );

   } /* End DBVprint_DbvAttributeDetails() */



void DBVprintN_DbvAttributesBySet(FILE *f, DbvAttributesBySet **ppsAttributesBySet )
   {

   if( ppsAttributesBySet != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributesBySet");
      while( ppsAttributesBySet && *ppsAttributesBySet )
         {
         DBVprint_DbvAttributesBySet(f, *ppsAttributesBySet);
         ppsAttributesBySet++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributesBySet");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributesBySet() */



void DBVprint_DbvAttributesBySet(FILE *f, DbvAttributesBySet *psAttributesBySet )
   {


   /* Write the title out */ 
   if (psAttributesBySet == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributesBySet");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributesBySet");
   


   /* pcAttributeSet */
   DBVprint_DbvObjectIdentifier(f, "pcAttributeSet", 
			      psAttributesBySet->pcAttributeSet );

   /* ppsAttributesByType */
   DBVprintN_DbvAttributesByType(f, psAttributesBySet->ppsAttributesByType );

   } /* End DBVprint_DbvAttributesBySet() */



void DBVprintN_DbvAttributesByType(FILE *f, DbvAttributesByType **ppsAttributesByType )
   {

   if( ppsAttributesByType != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributesByType");
      while( ppsAttributesByType && *ppsAttributesByType )
         {
         DBVprint_DbvAttributesByType(f, *ppsAttributesByType);
         ppsAttributesByType++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributesByType");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributesByType() */


void DBVprint_DbvAttributesByType(FILE *f,  DbvAttributesByType *psAttributesByType )
   {

   /* Write the title out */ 
   if (psAttributesByType == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributesByType");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributesByType");
   


   /* ulAttributeType */
   fprintf(f, "ulAttributeType: ");
   DBVprint_Ulong(f, &psAttributesByType->ulAttributeType );
   

   /* psDefaultIfOmitted */
   DBVprint_DbvOmittedAttributeInterpretation(f, psAttributesByType->psDefaultIfOmitted );

   /* ppsAttributeValues */
   DBVprintN_DbvAttributeValues(f, psAttributesByType->ppsAttributeValues);

   } /* End DBVprint_DbvAttributesByType() */



void DBVprint_DbvOmittedAttributeInterpretation(FILE *f, DbvOmittedAttributeInterpretation *psOmittedAttributeInterpretation )
   {


   /* Write the title out */ 
   if (psOmittedAttributeInterpretation == NULL)
      {
      fprintf(f, "%s: NULL\n","psOmittedAttributeInterpretation");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psOmittedAttributeInterpretation");
   

   /* psDefaultValue */
   fprintf(f, "psDefaultValue");
   DBVprint_strornum(f, psOmittedAttributeInterpretation->psDefaultValue );

   /* ppsDefaultDescription */
   DBVprintN_DbvHumanString(f, "ppsDefaultDescription", psOmittedAttributeInterpretation->ppsDefaultDescription );

   } /* End DBVprint_DbvOmittedAttributeInterpretation() */



void DBVprintN_DbvAttributeValues(FILE *f, DbvAttributeValues **ppsAttributeValues )
   {

   if( ppsAttributeValues != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributeValues");
      while( ppsAttributeValues && *ppsAttributeValues )
         {
         DBVprint_DbvAttributeValues(f, *ppsAttributeValues);
         ppsAttributeValues++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributeValues");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributeValues() */



void DBVprint_DbvAttributeValues(FILE *f, DbvAttributeValues *psAttributeValues )
   {


   /* Write the title out */ 
   if (psAttributeValues == NULL)
      {
      fprintf(f, "%s: NULL\n","psAttributeValues");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeValues");
   


   /* sValue */
   fprintf(f, "sValue: ");
   DBVprint_strornum(f, &psAttributeValues->sValue);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psAttributeValues->ppsDescription );

   /* ppsSubAttributes */
   DBVprintN_DbvStringOrNumeric(f, "ppsSubAttributes", psAttributeValues->ppsSubAttributes);

   /* ppsSuperAttributes */
   DBVprintN_DbvStringOrNumeric(f, "ppsSuperAttributes", psAttributeValues->ppsSuperAttributes);

   /* pvPartialSupport */
   DBVprint_DbvNull(f, "pvPartialSupport",
		  psAttributeValues->pvPartialSupport);

   } /* End DBVprint_DbvAttributeValues() */





/* -------------------- DbvTermListDetails -------------------- */

void DBVprint_DbvTermListDetails(FILE *f,  DbvTermListDetails *psTermListDetails )
   {


   /* Write the title out */ 
   if (psTermListDetails == NULL)
      {
      fprintf(f, "%s: NULL\n","psTermListDetails");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psTermListDetails");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psTermListDetails->psCommonInfo);

   /* sTermListName */
   fprintf(f, "sTermListName: ");
   DBVprint_inter_string(f, &psTermListDetails->sTermListName );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psTermListDetails->ppsDescription );

   /* psAttributes */
   DBVprint_DbvAttributeCombinations(f, psTermListDetails->psAttributes);

   /* psScanInfo */
   DBVprint_DbvScanInfo(f, psTermListDetails->psScanInfo);

   /* pulEstNumberTerms */
   fprintf(f, "pulEstNumberTerms: ");
   DBVprint_Ulong(f, psTermListDetails->pulEstNumberTerms );
   

   /* ppsSampleTerms */
   DBVprintN_DbvTerm(f, "ppsSampleTerms", psTermListDetails->ppsSampleTerms);

   } /* End DBVprint_DbvTermListDetails() */



void DBVprint_DbvAttributeCombinations(FILE *f, DbvAttributeCombinations *psAttributeCombinations )
   {


   /* Write the title out */ 
   if (psAttributeCombinations == NULL)
      {
      fprintf(f, "%s: NULL\n", "psAttributeCombinations");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psAttributeCombinations");
   


   /* pcDefaultAttributeSet */
   DBVprint_DbvObjectIdentifier(f, "pcDefaultAttributeSet", 
				psAttributeCombinations->pcDefaultAttributeSet );


   /* ppsLegalCombinations */
   DBVprintN_DbvAttributeCombination(f, psAttributeCombinations->
				ppsLegalCombinations );

   } /* End DBVprint_DbvAttributeCombinations() */



void DBVprintN_DbvAttributeCombination(FILE *f,  DbvAttributeCombination **ppsLegalCombinations)
   {

   if( ppsLegalCombinations != NULL)
      {
      fprintf(f, "%s:\n", "ppsLegalCombinations");
      while( ppsLegalCombinations && *ppsLegalCombinations )
         {
         DBVprint_DbvAttributeCombination(f, *ppsLegalCombinations);
         ppsLegalCombinations++;
	 fprintf(f, "\n");
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsLegalCombinations");
      
      } /* End else */

     
   } /* End DBVprintN_DbvAttributeCombination() */


void DBVprint_DbvAttributeCombination(FILE *f, DbvAttributeCombination *psLegalCombinations)
{
	/* Write the title out */ 
	if (psLegalCombinations == NULL) {
		fprintf(f, "%s: NULL\n","psLegalCombinations");
		
		return;
	} /* End if */

	fprintf(f, "%s:\n", "psLegalCombinations");
	

	DBVprintN_DbvDbvAttributeOccurrence(f, psLegalCombinations->ppsAttributeOccurrence);
	

} /* End DBVprint_DbvAttributeCombination */



void DBVprintN_DbvDbvAttributeOccurrence(FILE *f, DbvAttributeOccurrence **ppsAttributeOccurrence)
{
   if( ppsAttributeOccurrence != NULL)
      {
      fprintf(f, "%s:\n", "ppsAttributeCombinations");
      while( ppsAttributeOccurrence && *ppsAttributeOccurrence )
         {
         DBVprint_DbvDbvAttributeOccurrence(f, *ppsAttributeOccurrence);
         ppsAttributeOccurrence++;
         } 
      } 
   else
      {
      fprintf(f, "%s: NULL\n", "ppsAttributeCombinations");
      
      } 

} /* End DBVprintN_DbvDbvAttributeOccurrence */



void DBVprint_DbvDbvAttributeOccurrence(FILE *f, DbvAttributeOccurrence *psAttributeOccurrence)
{

	/* Write the title out */ 
	if (psAttributeOccurrence == NULL) {
		fprintf(f, "%s: NULL\n","psAttributeOccurrence");
		
		return;
	} /* End if */

	fprintf(f, "%s:\n", "psAttributeOccurrence");
	

	DBVprint_DbvObjectIdentifier(f, "pcAttributeSet", psAttributeOccurrence->pcAttributeSet);

         fprintf(f, "ulAttributeType: ");
         DBVprint_Ulong(f, &psAttributeOccurrence->ulAttributeType);

	DBVprint_DbvNull(f, "pvMustBeSupplied",
		  psAttributeOccurrence->pvMustBeSupplied);

	DBVprint_DbvAttributeOccurrenceChoice(f, &psAttributeOccurrence->sAttributeValues);
	
} /* End DBVprint_DbvDbvAttributeOccurrence */



void DBVprint_DbvAttributeOccurrenceChoice(FILE *f, DbvAttributeOccurrenceChoice *psAttributeOccurrenceChoice)
{

	/* Write the title out */ 
	if (psAttributeOccurrenceChoice == NULL) {
		fprintf(f, "%s: NULL\n","psAttributeOccurrenceChoice");
		
		return;
	} /* End if */
	
	fprintf(f, "%s:\n", "psAttributeOccurrenceChoice");
	
	fprintf(f, "eAttributeOccurrenceChoiceType: ");
	DBVprint_DbvAttributeOccurrenceChoiceType(f, &psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType);

	switch (psAttributeOccurrenceChoice->eAttributeOccurrenceChoiceType) {
	case AOT_AnyOrNone:
		DBVprint_DbvNull(f, "pvAnyOrNone",
		  psAttributeOccurrenceChoice->uAttributeOccurrenceChoiceType.pvAnyOrNone);
		break;
	case AOT_Specific:
   		DBVprintN_DbvStringOrNumeric(f, "ppsSpecific", psAttributeOccurrenceChoice->uAttributeOccurrenceChoiceType.ppsSpecific);
		break;
	}

} /* End DBVprint_DbvAttributeOccurrenceChoice */



void DBVprint_DbvAttributeOccurrenceChoiceType(FILE *f, DbvAttributeOccurrenceChoiceType *peAttributeOccurrenceChoiceType)
   {
   char temp[SML_BUFFER];

   if (peAttributeOccurrenceChoiceType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peAttributeOccurrenceChoiceType == AOT_AnyOrNone )
         strcpy(temp, "AOT_AnyOrNone");

      if( *peAttributeOccurrenceChoiceType == AOT_Specific  )
         strcpy(temp, "AOT_Specific");

      if( *peAttributeOccurrenceChoiceType == AOT_Invalid  )
         strcpy(temp, "AOT_Invalid");


	fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvAttributeOccurrenceChoiceType() */



void DBVprint_DbvScanInfo(FILE *f,  DbvScanInfo *psScanInfo)
   {


   /* Write the title out */ 
   if (psScanInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psScanInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psScanInfo");
   


   /* pulMaxStepSize */
   fprintf(f, "pulMaxStepSize: ");
   DBVprint_Ulong(f, psScanInfo->pulMaxStepSize );
   

   /* ppsCollatingSequence */
   DBVprintN_DbvHumanString(f, "ppsCollatingSequence",
			  psScanInfo->ppsCollatingSequence );

   /* pfIncreasing */
      fprintf(f, "pfIncreasing: ");
      DBVprint_bool(f,  psScanInfo->pfIncreasing);
   

   } /* End DBVprint_DbvScanInfo() */



void DBVprintN_DbvTerm(FILE *f,  char *name, DbvTerm **ppsTerm)
   {

   if( ppsTerm != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsTerm && *ppsTerm )
         {
         DBVprint_DbvTerm(f, *ppsTerm);
         ppsTerm++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvTerm() */


void DBVprint_DbvTerm( FILE *f, DbvTerm *psTerm)
   {
   char *pc_string = NULL;

   /* Write the title out */

   if (psTerm == NULL)
   {
           fprintf(f, "%s: NULL\n","psTerm");
           
           return;
   }

   fprintf(f, "%s:\n", "psTerm");
   

   /* Term type */
   fprintf(f, "eTermType: ");
   DBVprint_DbvTermType(f, &psTerm->eTermType );


   /* DbvTerm */

   switch( psTerm->eTermType )
      {
      case TT_General:
         fprintf(f,  "psGeneral: " );
         DBVprint_octet_string(f,  psTerm->uTerm.psGeneral);
         break;

      case TT_Numeric:
         fprintf(f, "ulInteger: ");
         DBVprint_Ulong(f, &psTerm->uTerm.ulInteger );
         break;

      case TT_SimpleCharacter:
         fprintf(f,  "psSimpleCharacter: " );
         DBVprint_inter_string(f,  psTerm->uTerm.psSimpleCharacter);
         break;

      case TT_ObjectIdentifier:
         DBVprint_DbvObjectIdentifier(f,  "pcObjectIdentifier",
                                psTerm->uTerm.pcObjectIdentifier);
         break;

      case TT_DateTime:
         DBVprint_DbvGeneralizedTime(f,  "pcDateTime",
                                   psTerm->uTerm.pcDateTime);
         break;

      case TT_External:
         fprintf(f,  "psExternal: " );
         DBVprint_external(f, psTerm->uTerm.psExternal);
         break;

      case TT_IntegerAndUnit:
         fprintf(f, "psIntegerAndUnit: ");
         DBVprint_intunit(f, psTerm->uTerm.psIntegerAndUnit);
         break;

      case TT_Null:
         break;

      default:
         break;
      } /* End switch */

   } /* End DBVprint_DbvTerm() */



void DBVprint_DbvTermType(FILE *f, DbvTermType *peTermType)
   {
   char temp[SML_BUFFER];

   if (peTermType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peTermType == TT_General )

         strcpy(temp, "TT_General");
      if( *peTermType == TT_Numeric  )
         strcpy(temp, "TT_Numeric");
      if( *peTermType == TT_SimpleCharacter )
         strcpy(temp, "TT_SimpleCharacter");
      if( *peTermType ==  TT_ObjectIdentifier)
         strcpy(temp, "TT_ObjectIdentifier");
      if( *peTermType == TT_DateTime )
         strcpy(temp, "TT_DateTime");
      if( *peTermType == TT_External )
         strcpy(temp, "TT_External");
      if( *peTermType == TT_IntegerAndUnit )
         strcpy(temp, "TT_IntegerAndUnit");
      if( *peTermType == TT_Null)
         strcpy(temp, "TT_Null");

	fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvTermType() */



/* -------------------- DbvElementSetDetails -------------------- */


void DBVprint_DbvElementSetDetails(FILE *f, DbvElementSetDetails *psElementSetDetails )
   {


   /* Write the title out */ 
   if (psElementSetDetails == NULL)
      {
      fprintf(f, "%s: NULL\n","psElementSetDetails");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psElementSetDetails");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psElementSetDetails->psCommonInfo);

   /* sDatabaseName */
   fprintf(f, "sDatabaseName: ");
   DBVprint_inter_string(f, &psElementSetDetails->sDatabaseName );

   /* sElementSetName */
   fprintf(f, "sElementSetName: ");
   DBVprint_inter_string(f, &psElementSetDetails->sElementSetName );

   /* pcRecordSyntax */
   DBVprint_DbvObjectIdentifier(f, "pcRecordSyntax", 
				psElementSetDetails->pcRecordSyntax );
   /* pcSchema */
   DBVprint_DbvObjectIdentifier(f, "pcSchema", psElementSetDetails->pcSchema );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psElementSetDetails->ppsDescription );

   /* ppsDetailsPerElement */
   DBVprintN_DbvDetailsPerElement(f, psElementSetDetails->ppsDetailsPerElement);

   } /* End DBVprint_DbvElementSetDetails() */


void DBVprintN_DbvDetailsPerElement(FILE *f, DbvDetailsPerElement **ppsDetailsPerElement )
   {

   if( ppsDetailsPerElement != NULL)
      {
      fprintf(f, "%s:\n", "ppsDetailsPerElement");
      while( ppsDetailsPerElement && *ppsDetailsPerElement )
         {
         DBVprint_DbvDetailsPerElement(f, *ppsDetailsPerElement);
         ppsDetailsPerElement++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsDetailsPerElement");
      
      } /* End else */

     
   } /* End DBVprintN_DbvDetailsPerElement() */


void DBVprint_DbvDetailsPerElement(FILE *f, DbvDetailsPerElement *psDetailsPerElement )
   {


   /* Write the title out */ 
   if (psDetailsPerElement == NULL)
      {
      fprintf(f, "%s: NULL\n","psDetailsPerElement");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psDetailsPerElement");
   


   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psDetailsPerElement->psName );

   /* psRecordTag */
   DBVprint_DbvRecordTag(f, psDetailsPerElement->psRecordTag);

   /* ppsSchemaTags */
   DBVprintN_DbvPathA(f, psDetailsPerElement->ppsSchemaTags);

   /* pulMaxSize */
   fprintf(f,"pulMaxSize: ");
   DBVprint_Ulong(f, psDetailsPerElement->pulMaxSize );
   

   /* pulMinSize */
   fprintf(f,"pulMinSize: ");
   DBVprint_Ulong(f, psDetailsPerElement->pulMinSize );
   

   /* pulAvgSize */
   fprintf(f,"pulAvgSize: ");
   DBVprint_Ulong(f, psDetailsPerElement->pulAvgSize );
   

   /* pulFixedSize */
   fprintf(f,"pulFixedSize: ");
   DBVprint_Ulong(f, psDetailsPerElement->pulFixedSize );
   

   /* fRepeatable */
   fprintf(f,"fRepeatable: ");
   DBVprint_bool(f, &psDetailsPerElement->fRepeatable);
   

   /* fRequired */
   fprintf(f,"fRequired: ");
   DBVprint_bool(f, &psDetailsPerElement->fRequired);
   

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psDetailsPerElement->ppsDescription );

   /* ppsContents */
   DBVprintN_DbvHumanString(f, "ppsContents", psDetailsPerElement->ppsContents );

   /* ppsBillingInfo */
   DBVprintN_DbvHumanString(f, "ppsBillingInfo", psDetailsPerElement->ppsBillingInfo );

   /* ppsRestrictions */
   DBVprintN_DbvHumanString(f, "ppsRestrictions",
			  psDetailsPerElement->ppsRestrictions );

   /* ppsAlternateNames */
   DBVprintN_DbvInternationalString(f, "ppsAlternateNames",
			  psDetailsPerElement->ppsAlternateNames );

   /* ppsGenericNames */
   DBVprintN_DbvInternationalString(f, "ppsGenericNames",
			  psDetailsPerElement->ppsGenericNames );

   /* psSearchAccess */
   DBVprint_DbvAttributeCombinations(f, psDetailsPerElement->psSearchAccess );

   } /* End DBVprint_DbvDetailsPerElement() */



void DBVprint_DbvRecordTag(FILE *f,  DbvRecordTag *psRecordTag)
   {


   /* Write the title out */ 
   if (psRecordTag == NULL)
      {
      fprintf(f, "%s: NULL\n","psRecordTag");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psRecordTag");
   


   /* psQualifier */
   fprintf(f, "psQualifier: ");
   DBVprint_strornum(f,  psRecordTag->psQualifier);

   /* sTagValue */
   fprintf(f, "sTagValue: ");
   DBVprint_strornum(f,  &psRecordTag->sTagValue);

   } /* End DBVprint_DbvRecordTag() */


void DBVprintN_DbvPathA(FILE *f, DbvPathA **ppsPathA)
   {

   if( ppsPathA != NULL)
      {
      fprintf(f, "%s:\n", "ppsPathA");
      while( ppsPathA && *ppsPathA )
         {
         DBVprintN_DbvPath(f,  (*ppsPathA)->ppsSchemaTags);
         ppsPathA++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsPathA");
      
      } /* End else */

     
   } /* End DBVprintN_DbvPathA() */




/* -------------------- DbvRetrievalRecordDetails -------------------- */

void DBVprint_DbvRetrievalRecordDetails(FILE *f,  DbvRetrievalRecordDetails *psRetrievalRecordDetails )
   {


   /* Write the title out */ 
   if (psRetrievalRecordDetails == NULL)
      {
      fprintf(f, "%s: NULL\n","psRetrievalRecordDetails");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psRetrievalRecordDetails");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psRetrievalRecordDetails->psCommonInfo);

   /* sDatabaseName */
   fprintf(f, "sDatabaseName: ");
   DBVprint_inter_string(f, &psRetrievalRecordDetails->sDatabaseName );

   /* pcSchema */
   DBVprint_DbvObjectIdentifier(f, "pcSchema",
			      psRetrievalRecordDetails->pcSchema );

   /* pcRecordSyntax */
   DBVprint_DbvObjectIdentifier(f, "pcRecordSyntax",
			      psRetrievalRecordDetails->pcRecordSyntax);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", 
			  psRetrievalRecordDetails->ppsDescription );

   /* ppsDetailsPerElement */
   DBVprintN_DbvDetailsPerElement(f, psRetrievalRecordDetails->ppsDetailsPerElement );

   } /* End DBVprint_DbvRetrievalRecordDetails() */



/* -------------------- DbvSortDetails -------------------- */

void DBVprint_DbvSortDetails( FILE *f, DbvSortDetails *psSortDetails)
   {


   /* Write the title out */ 
   if (psSortDetails == NULL)
      {
      fprintf(f, "%s: NULL\n","psSortDetails");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psSortDetails");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psSortDetails->psCommonInfo);

   /* sDatabaseName */
   fprintf(f, "sDatabaseName: ");
   DBVprint_inter_string(f, &psSortDetails->sDatabaseName );

   /* ppsSortKeys */
   DBVprintN_DbvSortKeys(f, psSortDetails->ppsSortKeys);

   } /* End DBVprint_DbvSortDetails() */



void DBVprintN_DbvSortKeys(FILE *f, DbvSortKeys **ppsSortKeys)
   {

   if( ppsSortKeys != NULL)
      {
      fprintf(f, "%s:\n", "ppsSortKeys");
      while( ppsSortKeys && *ppsSortKeys )
         {
         DBVprint_DbvSortKeys(f, *ppsSortKeys);
         ppsSortKeys++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsSortKeys");
      
      } /* End else */

     
   } /* End DBVprintN_DbvSortKeys() */



void DBVprint_DbvSortKeys(FILE *f, DbvSortKeys *psSortKeys)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psSortKeys == NULL)
      {
      fprintf(f, "%s: NULL\n","psSortKeys");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psSortKeys");
   


   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psSortKeys->ppsDescription );

   /* ppsElementSpecifications */
   DBVprintN_DbvSpecification(f, "ppsElementSpecifications", psSortKeys->ppsElementSpecifications);

   /* psAttributeSpecifications */
   DBVprint_DbvAttributeCombinations(f, psSortKeys->psAttributeSpecifications );

   /* psSortType */
   DBVprint_DbvSortType(f, psSortKeys->psSortType);

   /* peCaseSensitivity */
   fprintf(f, "peCaseSensitivity: ");
   DBVprint_DbvCaseSensitivity( f, psSortKeys->peCaseSensitivity );
  
   } /* End DBVprint_DbvSortKeys() */



void DBVprintN_DbvSpecification(FILE *f, char *name, DbvSpecification **ppsSpecification)
   {

   if( ppsSpecification != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsSpecification && *ppsSpecification )
         {
         DBVprint_DbvSpecification(f, "psSpecification", *ppsSpecification);
         ppsSpecification++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvSpecification() */



void DBVprint_DbvSortType(FILE *f,  DbvSortType *psSortType)
   {
   char temp[ STRING_LEN  ];

   /* Write the title out */ 
   if (psSortType == NULL)
      {
      fprintf(f, "%s: NULL\n","psSortType");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psSortType");
   

   /* eSortTypeType */
   fprintf(f, "eSortTypeType: ");
   DBVprint_DbvSortTypeType(f, &psSortType->eSortTypeType);


   /* uSortType */

   switch( psSortType->eSortTypeType )
      {
      case STT_Structured:
	 DBVprintN_DbvHumanString(f, "ppsStructured", 
				psSortType->uSortType.ppsStructured );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvSortType() */



void DBVprint_DbvSortTypeType(FILE *f, DbvSortTypeType *peSortTypeType)
   {
   char temp[ SML_BUFFER ];

   if (peSortTypeType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peSortTypeType == STT_Character)
         strcpy(temp, "STT_Character");

      if( *peSortTypeType == STT_Numeric)
         strcpy(temp, "STT_Numeric");

      if( *peSortTypeType == STT_Structured)
         strcpy(temp, "STT_Structured");

      if( *peSortTypeType == STT_Invalid)
         strcpy(temp, "STT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvSortTypeType() */




void DBVprint_DbvCaseSensitivity(FILE *f, DbvCaseSensitivity *peCaseSensitivity)
   {
   char temp[ SML_BUFFER ];

   if (peCaseSensitivity == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peCaseSensitivity == CSE_Always)
         strcpy(temp, "CSE_Always");

      if( *peCaseSensitivity == CSE_Never)
         strcpy(temp, "CSE_Never");

      if( *peCaseSensitivity == CSE_DefaultYes)
         strcpy(temp, "CSE_DefaultYes");

      if( *peCaseSensitivity == CSE_DefaultNo)
         strcpy(temp, "CSE_DefaultNo");

      if( *peCaseSensitivity == CSE_Invalid)
         strcpy(temp, "CSE_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvCaseSensitivity() */



/* -------------------- DbvProcessingInformation -------------------- */

void DBVprint_DbvProcessingInformation(FILE *f, DbvProcessingInformation *psProcessingInformation )
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psProcessingInformation == NULL)
      {
      fprintf(f, "%s: NULL\n","psProcessingInformation");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psProcessingInformation");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psProcessingInformation->psCommonInfo);

   /* sDatabaseName */
   fprintf(f, "sDatabaseName: ");
   DBVprint_inter_string(f, &psProcessingInformation->sDatabaseName );


   /* eProcessingContext */
   fprintf(f, "eProcessingContext: ");
   DBVprint_DbvProcessingContext(f, &psProcessingInformation->eProcessingContext);

   /* psName */
   fprintf( f, "psName: ");
   DBVprint_inter_string(f,  &psProcessingInformation->sName );

   /* pcOid */
   DBVprint_DbvObjectIdentifier(f, "pcOid", psProcessingInformation->pcOid );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", 
			  psProcessingInformation->ppsDescription );

   /* psInstructions */
   fprintf(f, "psInstructions: ");
   DBVprint_external(f, psProcessingInformation->psInstructions);

   } /* End DBVprint_DbvProcessingInformation() */



void DBVprint_DbvProcessingContext(FILE *f, DbvProcessingContext *peProcessingContext)
   {
   char temp[ SML_BUFFER ];

   if (peProcessingContext == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peProcessingContext == PC_Access)
         strcpy(temp, "PC_Access");

      if( *peProcessingContext == PC_Search)
         strcpy(temp, "PC_Search");

      if( *peProcessingContext == PC_Retrieval)
         strcpy(temp, "PC_Retrieval");

      if( *peProcessingContext == PC_RecordPresentation)
         strcpy(temp, "PC_RecordPresentation");

      if( *peProcessingContext == PC_RecordHandling)
         strcpy(temp, "PC_RecordHandling");

      if( *peProcessingContext == PC_Invalid)
         strcpy(temp, "PC_Invalid");

         fprintf(f, "%s\n", temp);	
   } /* End DBVprint_DbvProcessingContext() */



/* -------------------- DbvVariantSetInfo -------------------- */


void DBVprint_DbvVariantSetInfo(FILE *f,  DbvVariantSetInfo *psVariantSetInfo)
   {

   /* Write the title out */ 
   if (psVariantSetInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psVariantSetInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psVariantSetInfo");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psVariantSetInfo->psCommonInfo);

   /* pcVariantSet */
   DBVprint_DbvObjectIdentifier(f, "pcVariantSet", psVariantSetInfo->pcVariantSet );

   /* sName */
   fprintf(f, "sName: ");
   DBVprint_inter_string(f, &psVariantSetInfo->sName);

   /* ppsVariants */
   DBVprintN_DbvVariantClass(f, psVariantSetInfo->ppsVariants);

   } /* End DBVprint_DbvVariantSetInfo() */



void DBVprintN_DbvVariantClass(FILE *f,  DbvVariantClass **ppsVariants)
   {

   if( ppsVariants != NULL)
      {
      fprintf(f, "%s:\n", "ppsVariants");
      while( ppsVariants && *ppsVariants )
         {
         DBVprint_DbvVariantClass(f, *ppsVariants);
         ppsVariants++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsVariants");
      
      } /* End else */

     
   } /* End DBVprintN_DbvVariantClass() */



void DBVprint_DbvVariantClass(FILE *f,  DbvVariantClass *psVariants)
   {

   /* Write the title out */ 
   if (psVariants == NULL)
      {
      fprintf(f, "%s: NULL\n","psVariants");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psVariants");
   

   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psVariants->psName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psVariants->ppsDescription );

   /* ulVariantClass */
   fprintf(f, "ulVariantClass: ");
   DBVprint_Ulong(f, &psVariants->ulVariantClass );
   

   /* ppsVariantTypes */
   DBVprintN_DbvVariantTypes(f, psVariants->ppsVariantTypes);

   } /* End DBVprint_DbvVariantClass() */



void DBVprintN_DbvVariantTypes(FILE *f,  DbvVariantTypes **ppsVariantTypes)
   {

   if( ppsVariantTypes != NULL)
      {
      fprintf(f, "%s:\n", "ppsVariantTypes");
      while( ppsVariantTypes && *ppsVariantTypes )
         {
         DBVprint_DbvVariantTypes(f,  *ppsVariantTypes);
         ppsVariantTypes++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsVariantTypes");
      
      } /* End else */

     
   } /* End DBVprintN_DbvVariantTypes() */



void DBVprint_DbvVariantTypes(FILE *f,  DbvVariantTypes *psVariantTypes)
   {


   /* Write the title out */ 
   if (psVariantTypes == NULL)
      {
      fprintf(f, "%s: NULL\n","psVariantTypes");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psVariantTypes");
   


   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psVariantTypes->psName );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psVariantTypes->ppsDescription );

   /* ulVariantType */
   fprintf(f, "ulVariantType: ");
   DBVprint_Ulong(f, &psVariantTypes->ulVariantType );
   
   /* psVariantValue */
   DBVprint_DbvVariantValue(f,  psVariantTypes->psVariantValue);

   } /* End DBVprint_DbvVariantTypes() */



void DBVprint_DbvVariantValue(FILE *f,  DbvVariantValue *psVariantValue)
   {


   /* Write the title out */ 
   if (psVariantValue == NULL)
      {
      fprintf(f, "%s: NULL\n","psVariantValue");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psVariantValue");
   


   /* eDataType */
   DBVprint_DbvPrimitiveDataType(f, "eDataType", 
				&psVariantValue->eDataType);

   /* psValues */
   DBVprint_DbvValueSet(f, psVariantValue->psValues);

   } /* End DBVprint_DbvVariantValue() */



void DBVprint_DbvValueSet(FILE *f,  DbvValueSet *psValues)
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psValues == NULL)
      {
      fprintf(f, "%s: NULL\n","psValues");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psValues");
   


   /* eValuesType */
   fprintf(f, "eValueSetType: ");
   DBVprint_DbvValueSetType(f, &psValues->eValueSetType );

   /* uValues */
   switch( psValues->eValueSetType )
      {
      case VT_Range:
	 DBVprint_DbvValueRange(f, psValues->uValueSet.psRange);
	 break;

      case VT_Enumerated:
	DBVprintN_DbvValueDescription(f, psValues->uValueSet.ppsEnumerated);

	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvValueSet() */



void DBVprint_DbvValueSetType(FILE *f, DbvValueSetType *peValueSetType)
   {
   char temp[ SML_BUFFER ];

   if (peValueSetType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peValueSetType == VT_Range)
         strcpy(temp, "VT_Range");

      if( *peValueSetType == VT_Enumerated)
         strcpy(temp, "VT_Enumerated");

      if( *peValueSetType == VT_Invalid)
         strcpy(temp, "VT_Invalid");

         fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvValueSetType() */



void DBVprint_DbvValueRange(FILE *f, DbvValueRange *psValueRange)
   {


   /* Write the title out */ 
   if (psValueRange == NULL)
      {
      fprintf(f, "%s: NULL\n","psValueRange");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psValueRange");
   


   /* psLower */
   DBVprint_DbvValueDescription(f, psValueRange->psLower);

   /* psUpper */
   DBVprint_DbvValueDescription(f, psValueRange->psUpper);

   } /* End DBVprint_DbvValueRange() */




void DBVprint_DbvValueDescription(FILE *f, DbvValueDescription *psValueDescription )
   {
   char temp[ STRING_LEN  ];


   /* Write the title out */ 
   if (psValueDescription == NULL)
      {
      fprintf(f, "%s: NULL\n","psValueDescription");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psValueDescription");
   

   /* eValueDescriptionType */
   fprintf(f, "eValueDescriptiptionType: ");
   DBVprint_DbvValueDescriptionType(f, &psValueDescription->eValueDescriptionType);
 
   /* uValueDescription */
   switch( psValueDescription->eValueDescriptionType )
      {
      case VDT_Integer:
	 fprintf(f, "ulInteger: ");
	 DBVprint_Ulong(f, &psValueDescription-> uValueDescription.ulInteger);
         
	 break;

      case VDT_String:
	 fprintf(f, "psString: ");
	 DBVprint_inter_string(f, psValueDescription->
				uValueDescription.psString);
	 break;

      case VDT_Octets:
	 fprintf(f, "psOctets: ");
	 DBVprint_octet_string(f,  psValueDescription->
				uValueDescription.psOctets);
	 break;

      case VDT_Oid:
	 DBVprint_DbvObjectIdentifier(f, "pcOid", psValueDescription->
				uValueDescription.pcOid);
	 break;

      case VDT_Unit:
	 fprintf(f, "psUnit: ");
	 DBVprint_DbvUnit(f, psValueDescription->uValueDescription.psUnit );
	 break;

      case VDT_ValueAndUnit:
	 fprintf(f, "psValueAndUnit: ");
	 DBVprint_intunit(f, psValueDescription->uValueDescription.psValueAndUnit );
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvValueDescription() */



void DBVprint_DbvValueDescriptionType(FILE *f, DbvValueDescriptionType *peValueDescriptionType)
   {
   char temp[ SML_BUFFER ];

   if (peValueDescriptionType == NULL)
      {
      fprintf(f, "NULL\n");
      return;
      } /* End if */

      temp[ 0 ] = '\0';
      if( *peValueDescriptionType == VDT_Integer)
         strcpy(temp, "VDT_Integer");

      if( *peValueDescriptionType == VDT_String)
         strcpy(temp, "VDT_String");

      if( *peValueDescriptionType == VDT_Octets)
         strcpy(temp, "VDT_Octets");

      if( *peValueDescriptionType == VDT_Oid)
         strcpy(temp, "VDT_Oid");

      if( *peValueDescriptionType == VDT_Unit)
         strcpy(temp, "VDT_Unit");

      if( *peValueDescriptionType == VDT_ValueAndUnit)
         strcpy(temp, "VDT_ValueAndUnit");

      if( *peValueDescriptionType == VDT_Invalid)
         strcpy(temp, "VDT_Invalid");

    fprintf(f, "%s\n", temp);
   } /* End DBVprint_DbvValueDescriptionType() */



void DBVprintN_DbvValueDescription(FILE *f, DbvValueDescription **ppsEnumerated)
{
   if( ppsEnumerated != NULL)
      {
      fprintf(f, "%s:\n", "ppsEnumerated");
      while( ppsEnumerated && *ppsEnumerated )
         {
         DBVprint_DbvValueDescription(f, *ppsEnumerated);
         ppsEnumerated++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsEnumerated");
      
      } /* End else */


   } /* End DBVprintN_DbvValueDescription() */



/* -------------------- DbvUnitInfo -------------------- */


void DBVprint_DbvUnitInfo( FILE *f, DbvUnitInfo *psUnitInfo)
   {

   /* Write the title out */ 
   if (psUnitInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psUnitInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psUnitInfo");
   
   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psUnitInfo->psCommonInfo);

   /* sUnitSystem */
   fprintf(f, "sUnitSystem: ");
   DBVprint_inter_string(f, &psUnitInfo->sUnitSystem );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psUnitInfo->ppsDescription );
   /* ppsUnitTypes */
   DBVprintN_DbvUnitType(f, "ppsUnitTypes", psUnitInfo->ppsUnitTypes );

   } /* End DBVprint_DbvUnitInfo() */



void DBVprintN_DbvUnitType(FILE *f, char *name, DbvUnitType **ppsUnitTypes)
{
   if( ppsUnitTypes != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsUnitTypes && *ppsUnitTypes )
         {
         DBVprint_DbvUnitType(f, *ppsUnitTypes);
         ppsUnitTypes++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */


   } /* End DBVprintN_DbvUnitType() */



void DBVprint_DbvUnitType( FILE *f, DbvUnitType *psUnitTypes)
   {

   /* Write the title out */ 
   if (psUnitTypes == NULL)
      {
      fprintf(f, "%s: NULL\n","psUnitTypes");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psUnitTypes");
   
   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psUnitTypes->psName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psUnitTypes->ppsDescription );
   /* sUnitType */
   fprintf(f, "sUnitType: ");
   DBVprint_strornum(f, &psUnitTypes->sUnitType);

   /* ppsUnitTypes */
   DBVprintN_DbvUnits(f, "ppsUnits", psUnitTypes->ppsUnits );

   } /* End DBVprint_DbvUnitType() */



void DBVprintN_DbvUnits(FILE *f, char *name, DbvUnits **ppsUnits)
{
   if( ppsUnits != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsUnits && *ppsUnits )
         {
         DBVprint_DbvUnits(f, *ppsUnits);
         ppsUnits++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */
   } /* End DBVprintN_DbvUnits() */



void DBVprint_DbvUnits( FILE *f, DbvUnits *psUnits)
   {

   /* Write the title out */ 
   if (psUnits == NULL)
      {
      fprintf(f, "%s: NULL\n","psUnits");
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psUnits");
   
   /* psName */
   fprintf(f, "psName: ");
   DBVprint_inter_string(f, psUnits->psName);

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psUnits->ppsDescription );

   /* sUnit */
   fprintf(f, "sUnit: ");
   DBVprint_strornum(f, &psUnits->sUnit);

   } /* End DBVprint_DbvUnits() */



/* -------------------- DbvCategoryList -------------------- */

void DBVprint_DbvCategoryList( FILE *f, DbvCategoryList *psCategoryList)
   {

   /* Write the title out */ 
   if (psCategoryList == NULL)
      {
      fprintf(f, "%s: NULL\n","psCategoryList");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psCategoryList");
   


   /* psCommonInfo */
   DBVprint_DbvCommonInfo(f, "psCommonInfo", psCategoryList->psCommonInfo);

   /* ppsCategories */
   DBVprintN_DbvCategoryInfo(f, psCategoryList->ppsCategories);

   } /* End DBVprint_DbvCategoryList() */



void DBVprintN_DbvCategoryInfo(FILE *f,  DbvCategoryInfo **ppsCategories)
   {

   if( ppsCategories != NULL)
      {
      fprintf(f, "%s:\n", "ppsCategories");
      while( ppsCategories && *ppsCategories )
         {
         DBVprint_DbvCategoryInfo(f, *ppsCategories);
         ppsCategories++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", "ppsCategories");
      
      } /* End else */

     
   } /* End DBVprintN_DbvCategoryInfo() */



void DBVprint_DbvCategoryInfo(FILE *f,  DbvCategoryInfo *psCategoryInfo)
   {

   /* Write the title out */ 
   if (psCategoryInfo == NULL)
      {
      fprintf(f, "%s: NULL\n","psCategoryInfo");
      
      return;
      } /* End if */

   fprintf(f, "%s:\n", "psCategoryInfo");
   


   /* sCategory */
   fprintf(f, "sCategory: ");
   DBVprint_inter_string(f, &psCategoryInfo->sCategory );

   /* psOriginalCategory */
   fprintf(f, "psOriginalCategory: ");
   DBVprint_inter_string(f, psCategoryInfo->psOriginalCategory );

   /* ppsDescription */
   DBVprintN_DbvHumanString(f, "ppsDescription", psCategoryInfo->ppsDescription );
			
   /* psAsn1Module */
   fprintf(f, "psAsn1Module: ");
   DBVprint_inter_string(f, psCategoryInfo->psAsn1Module );

   } /* End DBVprint_DbvCategoryInfo() */


/*************************************/





int DBVprint_generic_external(FILE *f, DbvASN1External *external_ptr)
{
        fprintf(f,"\tpsGenericRecord:\n");

	fprintf(f,"Printing of this record type is not yet supported\n");
}








/******************************************************************/
/******************************************************************/
/* Before printing compulsory fields which are pointers, check that they 
   are not NULL to prevent nasty test program crashes */
/*******************************************************/
/* ItemOrder external */ 

int DBVprint_itemorder_external(FILE *f, DbvASN1External *external_ptr)
{
	DbvItemOrderExtendedService *io_ptr;

	fprintf(f,"psItemOrderExtendedService:\n");
	io_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psItemOrderExtendedService;

	if (io_ptr != NULL) {
		fprintf(f,"eItemOrderReqRsp: ");
		switch (io_ptr->eItemOrderReqRsp) {
		case IOROR_Request:
			fprintf(f,"IOROR_Request: ");
			if (io_ptr->uItemOrderReqOrRsp.psItemOrderRequest != NULL)
				DBVprint_io_request(f, io_ptr->uItemOrderReqOrRsp.psItemOrderRequest);
			break;
		case IOROR_Response:
			fprintf(f,"IOROR_Response: ");
			if (io_ptr->uItemOrderReqOrRsp.psItemOrderResponse != NULL)
				DBVprint_io_response(f, io_ptr->uItemOrderReqOrRsp.psItemOrderResponse);
			break;
		default:
			fprintf(f,"Unknown\n");
			break;
		}
	}
}



int DBVprint_io_request(FILE *f, DbvEsIoRequest *psItemOrderRequest)
{
	fprintf(f,"\n");

	/* o */
        fprintf(f,"psToKeep: ");
        if (psItemOrderRequest->psToKeep != NULL)
                DBVprint_io_originpartkeep(f, psItemOrderRequest->psToKeep);
        else
                fprintf(f,"NULL\n");


	/* c */
	fprintf(f,"sNotToKeep: ");
	DBVprint_io_originpartnotkeep(f, &psItemOrderRequest->sNotToKeep);
	
}



int DBVprint_io_originpartkeep(FILE *f, DbvEsIoOriginPartToKeep *psToKeep)
{
	
	/* o */
        fprintf(f,"psSupplementalDescription: ");
        if (psToKeep->psSupplementalDescription != NULL)
		DBVprint_external(f, psToKeep->psSupplementalDescription);
        else
                fprintf(f,"NULL\n");


	/* o */
        fprintf(f,"psContact: ");
        if (psToKeep->psContact != NULL)
		DBVprint_io_contact(f, psToKeep->psContact);
        else
                fprintf(f,"NULL\n");

	/* o */
        fprintf(f,"psAdditionalBilling:\n");
        if (psToKeep->psAdditionalBilling != NULL)
		DBVprint_io_addbilling(f, psToKeep->psAdditionalBilling);
        else
                fprintf(f,"NULL\n");


}


int DBVprint_io_contact(FILE *f, DbvEsIoContact *psContact)
{
	/* o */
        fprintf(f,"psName: ");
	if (psContact->psName != NULL)
		DBVprint_inter_string(f, psContact->psName);
        else
                fprintf(f,"NULL\n");


	/* o */
        fprintf(f,"psPhone: ");
	if (psContact->psPhone != NULL)
		DBVprint_inter_string(f, psContact->psPhone);
        else
                fprintf(f,"NULL\n");



	/* o */
        fprintf(f,"psEmail: ");
	if (psContact->psEmail != NULL)
		DBVprint_inter_string(f, psContact->psEmail);
        else
                fprintf(f,"NULL\n");


	
}



int DBVprint_io_addbilling(FILE *f, DbvEsIoAdditionalBilling *psAdditionalBilling)
{
	
	/* c */
        fprintf(f,"sPaymentMethod: ");
	DBVprint_io_paymethod(f, &psAdditionalBilling->sPaymentMethod);

	/* o */
        fprintf(f,"psCustomerReference: ");
	if (psAdditionalBilling->psCustomerReference != NULL)
		DBVprint_inter_string(f, psAdditionalBilling->psCustomerReference);
        else
                fprintf(f,"NULL\n");

	/* o */
        fprintf(f,"psCustomerPONumber: ");
	if (psAdditionalBilling->psCustomerPONumber != NULL)
		DBVprint_inter_string(f, psAdditionalBilling->psCustomerPONumber);
        else
                fprintf(f,"NULL\n");
	
}



int DBVprint_io_paymethod(FILE *f, DbvEsIoPaymentMethod *psPaymentMethod)
{
	fprintf(f,"\n");

        fprintf(f,"ePaymentMethodType: ");
        switch (psPaymentMethod->ePaymentMethodType) {
        case IOPM_BillInvoice:
                fprintf(f,"IOPM_BillInvoice\n");
                break;
        case IOPM_Prepay:
                fprintf(f,"IOPM_Prepay\n");
                break;
        case IOPM_DepositAccount:
                fprintf(f,"IOPM_DepositAccount\n");
                break;
        case IOPM_CreditCard:
                fprintf(f,"IOPM_CreditCard\n");

		if (psPaymentMethod->puPaymentMethod->psCreditCardInfo != NULL)
			DBVprint_io_creditcardinfo(f, psPaymentMethod->puPaymentMethod->psCreditCardInfo);
                break;
        case IOPM_CardInfoPrevSupplied:
                fprintf(f,"IOPM_CardInfoPrevSupplied\n");
                break;
        case IOPM_PrivateKnown:
                fprintf(f,"IOPM_PrivateKnown\n");
                break;
        case IOPM_PrivateNotKnown:
                fprintf(f,"IOPM_PrivateNotKnown\n");

		if (psPaymentMethod->puPaymentMethod->psPrivateNotKnown != NULL)
			DBVprint_external(f, psPaymentMethod->puPaymentMethod->psPrivateNotKnown);
                break;
        case IOPM_Invalid:
                fprintf(f,"IOPM_Invalid\n");
                break;
        }
	
}


int DBVprint_io_creditcardinfo(FILE *f, DbvEsIoCreditCardInfo *psCreditCardInfo)
{
	/* c */
        fprintf(f,"sNameOnCard: ");
	DBVprint_inter_string(f, &psCreditCardInfo->sNameOnCard);

	/* c */
        fprintf(f,"sExpirationDate: ");
	DBVprint_inter_string(f, &psCreditCardInfo->sExpirationDate);

	/* c */
        fprintf(f,"sCardNumber: ");
	DBVprint_inter_string(f, &psCreditCardInfo->sCardNumber);
	
}



int DBVprint_io_originpartnotkeep(FILE *f, DbvEsIoOriginPartNotToKeep *psNotToKeep)
{

	/* o */
	fprintf(f,"psResultSetItem: ");
	if (psNotToKeep->psResultSetItem != NULL)
		DBVprint_io_ressetitem(f, psNotToKeep->psResultSetItem);
	else
                fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psItemRequest: ");
	if (psNotToKeep->psItemRequest != NULL)
		DBVprint_external(f, psNotToKeep->psItemRequest);
	else
                fprintf(f,"NULL\n");

		
	
}


int DBVprint_io_ressetitem(FILE *f, DbvEsIoResultSetItem *psResultSetItem)
{
	/* c */
	fprintf(f,"sResultSetId: ");
	DBVprint_inter_string(f, &psResultSetItem->sResultSetId);

	/* c */
	fprintf(f,"sResultSetId: ");
	fprintf(f,"%lu\n", psResultSetItem->ulItem);
	
}



int DBVprint_io_response(FILE *f, DbvEsIoTaskPackage *psItemOrderResponse)
{
	fprintf(f,"\n");

	/* o */
        fprintf(f,"psOriginPart: ");
        if (psItemOrderResponse->psOriginPart != NULL)
		DBVprint_io_originpartkeep(f, psItemOrderResponse->psOriginPart);
        else
                fprintf(f,"NULL\n");

	/* c */
        fprintf(f,"sTargetPart: ");
	DBVprint_io_targetpart(f, &psItemOrderResponse->sTargetPart);

}




int DBVprint_io_targetpart(FILE *f, DbvEsIoTargetPart *psTargetPart)
{

	/* o */

	/* print out ILL - could legally be any other external however ! */
        fprintf(f,"psItemRequest: ");
        if (psTargetPart->psItemRequest != NULL)
		DBVprint_external(f, psTargetPart->psItemRequest);
        else
                fprintf(f,"NULL\n");


	/* c */
        fprintf(f,"psStatusOrErrorReport: ");
	if (psTargetPart->psStatusOrErrorReport != NULL)
		DBVprint_external(f, psTargetPart->psStatusOrErrorReport);

	/* o */
        fprintf(f,"peAuxiliaryStatus: ");
        if (psTargetPart->peAuxiliaryStatus != NULL)
		DBVprint_io_auxstat(f, psTargetPart->peAuxiliaryStatus);
        else
                fprintf(f,"NULL\n");

	
}



int DBVprint_io_auxstat(FILE *f, DbvEsIoAuxiliaryStatus *peAuxiliaryStatus)
{
	switch (*peAuxiliaryStatus) {
	case IOAS_NotReceived:
		fprintf(f,"IOAS_NotReceived\n");
		break;
	case IOAS_LoanQueue:
		fprintf(f,"IOAS_LoanQueue\n");
		break;
	case IOAS_Forwarded:
		fprintf(f,"IOAS_Forwarded\n");
		break;
	case IOAS_UnfilledCopyright:
		fprintf(f,"IOAS_UnfilledCopyright\n");
		break;
	case IOAS_FilledCopyright:
		fprintf(f,"IOAS_FilledCopyright\n");
		break;
	case IOAS_Invalid:
		fprintf(f,"IOAS_Invalid\n");
		break;
	}
}



/*******************************************************/
/* Ill external */ 
int DBVprint_ill_external(FILE *f, DbvASN1External *external_ptr)
{
	DbvIllPdu *ill_ptr;

	ill_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psIllPdu;

	if (ill_ptr != NULL) {

		fprintf(f,"eIllPduType: ");
		switch (ill_ptr->eIllPduType) {
		case ILL_IllRequest:
			fprintf(f,"ILL_IllRequest: ");
			if (ill_ptr->uIllPdu.psIllRequest != NULL)
				DBVprint_ill_request(f, ill_ptr->uIllPdu.psIllRequest);
			break;
		case ILL_StatusOrErrorReport:
			fprintf(f,"ILL_StatusOrErrorReport: ");
			if (ill_ptr->uIllPdu.psStatusOrErrorReport != NULL)
				DBVprint_ill_statorerr(f, ill_ptr->uIllPdu.psStatusOrErrorReport);
			break;
		case ILL_Invalid:
			fprintf(f,"ILL_Invalid\n");
			break;
		}
	}
}


int DBVprint_ill_request(FILE *f, struct DbvIllRequest *psIllRequest)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eProtocolVersion: ");
	DBVprint_ill_protocolver(f, &psIllRequest->eProtocolVersion);

	/* c */
	fprintf(f,"sTransactionId: ");
	DBVprint_ill_transid(f, &psIllRequest->sTransactionId);

	/* c */
	fprintf(f,"sServiceDateTime: ");
	DBVprint_ill_servdatetime(f, &psIllRequest->sServiceDateTime);
	
	/* o */
	fprintf(f,"psRequesterId: ");
	if (psIllRequest->psRequesterId != NULL) 
		DBVprint_ill_systemid(f, psIllRequest->psRequesterId);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psResponderId: ");
	if (psIllRequest->psResponderId != NULL) 
		DBVprint_ill_systemid(f, psIllRequest->psResponderId);
	else
		fprintf(f,"NULL\n");
	

	/* c */
	fprintf(f,"peTransactionType: ");
	if (psIllRequest->peTransactionType != NULL) 
		DBVprint_ill_transtype(f, psIllRequest->peTransactionType);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psDeliveryAddress: ");
	if (psIllRequest->psDeliveryAddress != NULL) 
		DBVprint_ill_delivaddr(f, psIllRequest->psDeliveryAddress);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psDeliveryService: ");
	if (psIllRequest->psDeliveryService != NULL) 
		DBVprint_ill_delivserv(f, psIllRequest->psDeliveryService);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psBillingAddress: ");
	if (psIllRequest->psBillingAddress != NULL) 
		DBVprint_ill_delivaddr(f, psIllRequest->psBillingAddress);
	else
		fprintf(f,"NULL\n");
	


	/* iLL_service_type */
	/* c */
	fprintf(f,"ppeServiceType: ");
	if (psIllRequest->ppeServiceType != NULL)
		DBVprint_ill_servtypelist(f, psIllRequest->ppeServiceType);


	/* o */
	fprintf(f,"psResponderSpecificService: ");
	if (psIllRequest->psResponderSpecificService != NULL) 
		DBVprint_external(f, psIllRequest->psResponderSpecificService);
	else
		fprintf(f,"NULL\n");
	

	
	/* c */
	fprintf(f,"sRequesterOptionalMessages: ");
	DBVprint_ill_reqoptmsg(f, &psIllRequest->sRequesterOptionalMessages);


	/* o */
	fprintf(f,"psSearchType: ");
	if (psIllRequest->psSearchType != NULL) 
		DBVprint_ill_searchtype(f, psIllRequest->psSearchType);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"ppsSupplyMediumInfo: ");
	if (psIllRequest->ppsSupplyMediumInfo != NULL)
		DBVprint_ill_suppmedlist(f, psIllRequest->ppsSupplyMediumInfo);
	else
		fprintf(f,"NULL\n");
	
	


	/* place_on_hold */
	/* default */
	fprintf(f,"pePlaceOnHold: ");
	if (psIllRequest->pePlaceOnHold)
		DBVprint_ill_placeonhold(f, psIllRequest->pePlaceOnHold);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psClientId: ");
	if (psIllRequest->psClientId != NULL)
		DBVprint_ill_clientid(f, psIllRequest->psClientId);
	else
		fprintf(f,"NULL\n");
	

	/* c */
	fprintf(f,"sItemId: ");
	DBVprint_ill_itemid(f, &psIllRequest->sItemId);
	


	/* o */
	fprintf(f,"ppsSupplementalItemDescription: ");
	if (psIllRequest->ppsSupplementalItemDescription != NULL)
		DBVprint_ill_suppdesc(f, psIllRequest->ppsSupplementalItemDescription);
	else
		fprintf(f,"NULL\n");
	
		

	/* o */
	fprintf(f,"psCostInfo: ");
	if (psIllRequest->psCostInfo != NULL)
		DBVprint_ill_costinfo(f, psIllRequest->psCostInfo);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psCopyrightCompliance: ");
	if (psIllRequest->psCopyrightCompliance != NULL)
		DBVprint_ill_illstring(f, psIllRequest->psCopyrightCompliance);
	else
		fprintf(f,"NULL\n");
	


	/* o */
	fprintf(f,"psThirdPartyInfo: ");
	if (psIllRequest->psThirdPartyInfo != NULL)
		DBVprint_ill_thirdparty(f, psIllRequest->psThirdPartyInfo);
	else
		fprintf(f,"NULL\n");


	/* retry_flag */
	/* default */
	fprintf(f,"pfRetry: ");
	if (psIllRequest->pfRetry)
		DBVprint_bool(f, psIllRequest->pfRetry);
	else
		fprintf(f,"NULL\n");


	/* default */
	fprintf(f,"pfForward: ");
	if (psIllRequest->pfForward)
		DBVprint_bool(f, psIllRequest->pfForward);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psRequesterNote: ");
	if (psIllRequest->psRequesterNote != NULL)
		DBVprint_ill_illstring(f, psIllRequest->psRequesterNote);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psForwardNote: ");
	if (psIllRequest->psForwardNote != NULL)
		DBVprint_ill_illstring(f, psIllRequest->psForwardNote);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"ppsRequestExtension: ");
	if (psIllRequest->ppsRequestExtension != NULL)
		DBVprint_ill_extensionlist(f, psIllRequest->ppsRequestExtension);
	else
		fprintf(f,"NULL\n");

}



int DBVprint_ill_protocolver(FILE *f, DbvIllProtocolVersion *peProtocolVersion)
{
	switch (*peProtocolVersion) {
	case PV_Version1:
		fprintf(f,"PV_Version1\n");
		break;
	case PV_Version2:
		fprintf(f,"PV_Version2\n");
		break;
	case PV_Invalid:
		fprintf(f,"PV_Invalid\n");
		break;
	}

	
}



int DBVprint_ill_transid(FILE *f, DbvTransactionId *psTransactionId)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psInitialRequesterId: ");
	if (psTransactionId->psInitialRequesterId != NULL)
		DBVprint_ill_systemid(f, psTransactionId->psInitialRequesterId);
	else
		fprintf(f,"NULL\n");
	

	/* c */
	fprintf(f,"sTransactionGroupQualifier: ");
	DBVprint_ill_illstring(f, &psTransactionId->sTransactionGroupQualifier);


	/* c */
	fprintf(f,"sTransactionQualifier: ");
	DBVprint_ill_illstring(f, &psTransactionId->sTransactionQualifier);


	/* o */
	fprintf(f,"psSubTransactionQualifier: ");
	if (psTransactionId->psSubTransactionQualifier != NULL)
		DBVprint_ill_illstring(f, psTransactionId->psSubTransactionQualifier);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_systemid(FILE *f, DbvSystemId *psInitialRequesterId)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psSymbol: ");
	if (psInitialRequesterId->psSymbol != NULL)
		DBVprint_ill_symbol(f, psInitialRequesterId->psSymbol);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psName: ");
	if (psInitialRequesterId->psName != NULL)
		DBVprint_ill_name(f, psInitialRequesterId->psName);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_symbol(FILE *f, DbvPersonOrInstitutionSymbol *psSymbol)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eSymbolType: ");
	DBVprint_ill_perorinstsymtype(f, &psSymbol->eSymbolType);

	/* c */
	fprintf(f,"sSymbol: ");
	DBVprint_ill_illstring(f, &psSymbol->sSymbol);

}



int DBVprint_ill_perorinstsymtype(FILE *f, DbvPersOrInstSymbolType *peSymbolType)
{
	switch (*peSymbolType) {
	case SPI_PersonSymbol:
		fprintf(f,"SPI_PersonSymbol\n");
		break;
	case SPI_InstitutionSymbol:
		fprintf(f,"SPI_InstitutionSymbol\n");
		break;
	}
	
}


int DBVprint_ill_illstring(FILE *f, DbvIllString *psSymbol)
{

	switch (psSymbol->eStringType) {
	case ST_GeneralString:
		fprintf(f,"ST_GeneralString: ");
		if (psSymbol->uString.psGeneralString != NULL)
			DBVprint_DbvGeneralString(f, psSymbol->uString.psGeneralString);
		break;
	case ST_EdifactString:
		fprintf(f,"ST_EdifactString: ");
		if (psSymbol->uString.pcVisibleString != NULL)
			fprintf(f,"%s\n", psSymbol->uString.pcVisibleString);
		break;
	}
	
}


/* DbvGeneralString is defined in sitypes.h - it has the same 
structure as DbvInternationalString so print as such */
int DBVprint_DbvGeneralString(FILE *f, DbvGeneralString *psGeneralString)
{
	DBVprint_inter_string(f, (DbvInternationalString *) psGeneralString);
}



int DBVprint_ill_name(FILE *f, DbvNameOfPersonOrInstitution *psName)
{
	/* c */
	fprintf(f,"eNameType: ");
	DBVprint_ill_nameofperorinst_type(f, &psName->eNameType);

	/* c */
	fprintf(f,"sName: ");
	DBVprint_ill_illstring(f, &psName->sName);
}


int DBVprint_ill_nameofperorinst_type(FILE *f, DbvNameOfPersOrInstType *peNameType)
{
	switch (*peNameType) {
	case NPI_NameOfPerson:
		fprintf(f,"NPI_NameOfPerson\n");
		break;
	case NPI_NameOfInstitution:
		fprintf(f,"NPI_NameOfInstitution\n");
		break;
	}
	
}


int DBVprint_ill_servdatetime(FILE *f, DbvServiceDateTime *psServiceDateTime)
{
	/* c */
	fprintf(f,"sThisService: ");
	DBVprint_ill_datetime(f, &psServiceDateTime->sThisService);

	/* o */
	fprintf(f,"psOriginalService: ");
	if (psServiceDateTime->psOriginalService != NULL)
		DBVprint_ill_datetime(f, psServiceDateTime->psOriginalService);
	else
		fprintf(f,"NULL\n");
}


int DBVprint_ill_datetime(FILE *f, DbvDateTime *psThisService)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"pcDate: %s\n", psThisService->pcDate);

	/* o */
	fprintf(f,"pcTime: ");
	if (psThisService->pcTime != NULL)
		fprintf(f,"%s\n", psThisService->pcTime);
	else
		fprintf(f,"NULL\n");
}


int DBVprint_ill_transtype(FILE *f, DbvTransactionType *peTransactionType)
{
	switch (*peTransactionType) {
	case TT_Simple:
		fprintf(f,"TT_Simple\n");
		break;
	case TT_Chained:
		fprintf(f,"TT_Chained\n");
		break;
	case TT_Partitioned:
		fprintf(f,"TT_Partitioned\n");
		break;
	case TT_Invalid:
		fprintf(f,"TT_Invalid\n");
		break;
	}
}



int DBVprint_ill_delivaddr(FILE *f, DbvDeliveryAddress *psDeliveryAddress)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psPostalAddress: ");
	if (psDeliveryAddress->psPostalAddress != NULL)
		DBVprint_ill_postaladdr(f, psDeliveryAddress->psPostalAddress);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psElectronicAddress: ");
	if (psDeliveryAddress->psElectronicAddress != NULL)
		DBVprint_ill_sysaddr(f, psDeliveryAddress->psElectronicAddress);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_postaladdr(FILE *f, DbvPostalAddress *psPostalAddress)
{
	/* o */
	fprintf(f,"psNameOfPersonOrInstitution: ");
	if (psPostalAddress->psNameOfPersonOrInstitution != NULL)
		DBVprint_ill_name(f, psPostalAddress->psNameOfPersonOrInstitution);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psExtendedPostalDeliveryAddress: ");
	if (psPostalAddress->psExtendedPostalDeliveryAddress != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psExtendedPostalDeliveryAddress);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psStreetAndNumber: ");
	if (psPostalAddress->psStreetAndNumber != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psStreetAndNumber);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psPostOfficeBox: ");
	if (psPostalAddress->psPostOfficeBox != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psPostOfficeBox);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psCity: ");
	if (psPostalAddress->psCity != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psCity);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psRegion: ");
	if (psPostalAddress->psRegion != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psRegion);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psCountry: ");
	if (psPostalAddress->psCountry != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psCountry);
	else
		fprintf(f,"NULL\n");
	

	/* o */	
	fprintf(f,"psPostalCode: ");
	if (psPostalAddress->psPostalCode != NULL)
		DBVprint_ill_illstring(f, psPostalAddress->psPostalCode);
	else
		fprintf(f,"NULL\n");
	
	
}



int DBVprint_ill_sysaddr(FILE *f, DbvSystemAddress *psElectronicAddress)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psTelecomServiceIdentifier: ");
	if (psElectronicAddress->psTelecomServiceIdentifier != NULL)
		DBVprint_ill_illstring(f, psElectronicAddress->psTelecomServiceIdentifier);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psTelecomServiceAddress: ");
	if (psElectronicAddress->psTelecomServiceAddress != NULL)
		DBVprint_ill_illstring(f, psElectronicAddress->psTelecomServiceAddress);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_delivserv(FILE *f, DbvDeliveryService *psDeliveryService)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eDeliveryServiceType: ");
	switch (psDeliveryService->eDeliveryServiceType) {
	case DST_PhysicalDelivery:
		fprintf(f,"DST_PhysicalDelivery: ");

		if (psDeliveryService->uDeliveryService.psPhysicalDelivery != NULL) 
			DBVprint_ill_illstring(f, psDeliveryService->uDeliveryService.psPhysicalDelivery);
		
		break;

	case DST_ElectronicDelivery:
		fprintf(f,"DST_ElectronicDelivery: ");

		if (psDeliveryService->uDeliveryService.ppsElectronicDelivery != NULL) 
			DBVprint_ill_elecdelivservlist(f, psDeliveryService->uDeliveryService.ppsElectronicDelivery);
		break;
	case DST_Invalid:
		fprintf(f,"DST_Invalid\n");
		break;
	}

}


int DBVprint_ill_elecdelivservlist(FILE *f, DbvElectronicDeliveryService **ppsElectronicDelivery)
{
	int index = 0;

	fprintf(f,"\n");

	while (ppsElectronicDelivery[index] != NULL) {
       		fprintf(f,"\tpsElectronicDelivery[%d]\n", index);
		DBVprint_ill_elecdelivserv(f, ppsElectronicDelivery[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_elecdelivserv(FILE *f, DbvElectronicDeliveryService *psElectronicDelivery)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psDeliveryParameter: ");
	if (psElectronicDelivery->psDeliveryParameter != NULL)
		DBVprint_ill_eldelivparam(f, psElectronicDelivery->psDeliveryParameter);
	else
		fprintf(f,"NULL\n");

	fprintf(f,"\n");

	/* o */
	fprintf(f,"psDocumentTypeParameter: ");
	if (psElectronicDelivery->psDocumentTypeParameter != NULL)
		DBVprint_ill_doctypeparam(f, psElectronicDelivery->psDocumentTypeParameter);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psDeliveryDescription: ");
	if (psElectronicDelivery->psDeliveryDescription != NULL)
		DBVprint_ill_illstring(f, psElectronicDelivery->psDeliveryDescription);
	else
		fprintf(f,"NULL\n");


	/* c */
	fprintf(f,"sDeliveryDetails: ");
	DBVprint_ill_eldelivdetails(f, &psElectronicDelivery->sDeliveryDetails);
	
	/* o */
	fprintf(f,"psNameOrCode: ");
	if (psElectronicDelivery->psNameOrCode != NULL)
		DBVprint_ill_illstring(f, psElectronicDelivery->psNameOrCode);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"pcDeliveryTime: ");
	if (psElectronicDelivery->pcDeliveryTime != NULL)
		fprintf(f,"%s\n", psElectronicDelivery->pcDeliveryTime);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_eldelivparam(FILE *f, DbvElDeliveryParameter *psDeliveryParameter)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"pcDeliveryMode: ");
	if (psDeliveryParameter->pcDeliveryMode != NULL) 
		fprintf(f,"%s\n", psDeliveryParameter->pcDeliveryMode);
	else 
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psDummy: ");
	if (psDeliveryParameter->puDeliveryParameter != NULL) 
		if (psDeliveryParameter->puDeliveryParameter->psDummy != NULL)
			   DBVprint_external(f, psDeliveryParameter->puDeliveryParameter->psDummy);
		else
			fprintf(f,"NULL\n");
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_doctypeparam(FILE *f, DbvDocumentTypeParameter *psDocumentTypeParameter)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"pcDocumentTypeId: ");
	if (psDocumentTypeParameter->pcDocumentTypeId != NULL) 
		fprintf(f,"%s\n", psDocumentTypeParameter->pcDocumentTypeId);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psDummy: ");
	if (psDocumentTypeParameter->puDocumentTypeParameter != NULL) 
		if (psDocumentTypeParameter->puDocumentTypeParameter->psDummy != NULL)
			DBVprint_external(f, psDocumentTypeParameter->puDocumentTypeParameter->psDummy);
		else
			fprintf(f,"NULL\n");
	else
		fprintf(f,"NULL\n");
}


int DBVprint_ill_eldelivdetails(FILE *f, DbvElDeliveryDetails *psDeliveryDetails)
{
	switch (psDeliveryDetails->eDeliveryDetailsType) {
	case DDT_SystemAddress:
		fprintf(f,"DDT_SystemAddress\n");
		if (psDeliveryDetails->uDeliveryDetails.psSystemAddress != NULL)
			DBVprint_ill_sysaddr(f, psDeliveryDetails->uDeliveryDetails.psSystemAddress);
		
		break;
	case DDT_SystemId:
		fprintf(f,"DDT_SystemId\n");
		if (psDeliveryDetails->uDeliveryDetails.psSystemId != NULL)
			DBVprint_ill_systemid(f, psDeliveryDetails->uDeliveryDetails.psSystemId);
		break;
	}
	
}


int DBVprint_ill_servtypelist(FILE *f, DbvServiceType **ppeServiceType)
{
	int index = 0;

	fprintf(f,"\n");

	while (ppeServiceType[index] != NULL) {
       		fprintf(f,"\tpeServiceType[%d]: ", index);
		DBVprint_ill_servtype(f, ppeServiceType[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_servtype(FILE *f, DbvServiceType *peServiceType)
{
	fprintf(f,"\n");

	switch (*peServiceType) {
	case ST_Loan:
		fprintf(f,"ST_Loan\n");
		break;
	case ST_CopyNonReturnable:
		fprintf(f,"ST_CopyNonReturnable\n");
		break;
	case ST_Locations:
		fprintf(f,"ST_Locations\n");
		break;
	case ST_Estimate:
		fprintf(f,"ST_Estimate\n");
		break;
	case ST_ResponderSpecific:
		fprintf(f,"ST_ResponderSpecific\n");
		break;
	case ST_Invalid:
		fprintf(f,"ST_Invalid\n");
		break;
	}
}



int DBVprint_ill_reqoptmsg(FILE *f, DbvRequesterOptionalMessages *psRequesterOptionalMessages)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"fCanSendReceived: ");
	DBVprint_bool(f, &psRequesterOptionalMessages->fCanSendReceived);

	/* c */
	fprintf(f,"fCanSendReturned: ");
	DBVprint_bool(f, &psRequesterOptionalMessages->fCanSendReturned);

	/* c */
	fprintf(f,"eRequesterShipped: ");
	DBVprint_ill_optmsgreq(f, &psRequesterOptionalMessages->eRequesterShipped);

	/* c */
	fprintf(f,"eRequesterCheckedIn: ");
	DBVprint_ill_optmsgreq(f, &psRequesterOptionalMessages->eRequesterCheckedIn);
}



int DBVprint_ill_optmsgreq(FILE *f, DbvOptionalMsgRequirements *peRequesterCheckedIn)
{
	switch (*peRequesterCheckedIn) {
	case OMR_Requires:
		fprintf(f,"OMR_Requires\n");	
		break;
	case OMR_Desires:
		fprintf(f,"OMR_Desires\n");	
		break;
	case OMR_Neither:
		fprintf(f,"OMR_Neither\n");	
		break;
	case OMR_Invalid:
		fprintf(f,"OMR_Invalid\n");	
		break;
	}
}


int DBVprint_ill_searchtype(FILE *f, DbvSearchType *psSearchType)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psLevelOfService: ");
	if (psSearchType->psLevelOfService != NULL)
		DBVprint_ill_illstring(f, psSearchType->psLevelOfService);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"pcNeedBeforeDate: ");
	if (psSearchType->pcNeedBeforeDate != NULL)
		fprintf(f,"%s\n", psSearchType->pcNeedBeforeDate);
	else
		fprintf(f,"NULL\n");

	/* c */
	fprintf(f,"peExpiryFlag: ");
	if (psSearchType->peExpiryFlag != NULL)
		DBVprint_ill_expflag(f, psSearchType->peExpiryFlag);
	else
		fprintf(f,"NULL\n");
		

	/* o */
	fprintf(f,"pcExpiryDate: ");
	if (psSearchType->pcExpiryDate != NULL)
		fprintf(f,"%s\n", psSearchType->pcExpiryDate);
	else
		fprintf(f,"NULL\n");

	
}


int DBVprint_ill_expflag(FILE *f, DbvExpiry *peExpiryFlag)
{
	switch (*peExpiryFlag) {
	case EXP_NeedBeforeDate:
		fprintf(f,"EXP_NeedBeforeDate\n");
		break;
	case EXP_OtherDate:
		fprintf(f,"EXP_OtherDate\n");
		break;
	case EXP_NoExpiry:
		fprintf(f,"EXP_NoExpiry\n");
		break;
	case EXP_Invalid:
		fprintf(f,"EXP_Invalid\n");
		break;
	}
	
}


int DBVprint_ill_suppmedlist(FILE *f, DbvSupplyMediumInfo **ppsSupplyMediumInfo)
{
	int index = 0;

	fprintf(f,"\n");

	while (ppsSupplyMediumInfo[index] != NULL) {
       		fprintf(f,"\tpsSupplyMediumInfo[%d]: ", index);
		DBVprint_ill_suppmed(f, ppsSupplyMediumInfo[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_suppmed(FILE *f, DbvSupplyMediumInfo *psSupplyMediumInfo)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eSupplyMediumType: ");
	DBVprint_ill_suppmedinfo(f, &psSupplyMediumInfo->eSupplyMediumType);	

	/* o */
	fprintf(f,"psMediumCharacteristics: ");
	if (psSupplyMediumInfo->psMediumCharacteristics != NULL)
		DBVprint_ill_illstring(f, psSupplyMediumInfo->psMediumCharacteristics);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_suppmedinfo(FILE *f, DbvSupplyMediumType *peSupplyMediumType)
{
	switch (*peSupplyMediumType) {
	case SMT_Printed:
		fprintf(f,"SMT_Printed\n");
		break;
	case SMT_Photocopy:
		fprintf(f,"SMT_Photocopy\n");
		break;
	case SMT_Microform:
		fprintf(f,"SMT_Microform\n");
		break;
	case SMT_FilmOrVideoRecording:
		fprintf(f,"SMT_FilmOrVideoRecording\n");
		break;
	case SMT_AudioRecording:
		fprintf(f,"SMT_AudioRecording\n");
		break;
	case SMT_MachineReadable:
		fprintf(f,"SMT_MachineReadable\n");
		break;
	case SMT_Other:
		fprintf(f,"SMT_Other\n");
		break;
	case SMT_Invalid:
		fprintf(f,"SMT_Invalid\n");
		break;
	}
}



int DBVprint_ill_placeonhold(FILE *f, DbvPlaceOnHold *pePlaceOnHold)
{
	switch (*pePlaceOnHold) {
	case POH_Yes:
		fprintf(f,"POH_Yes\n");
		break;
	case POH_No:
		fprintf(f,"POH_Yes\n");
		break;
	case POH_ResponderPolicy:
		fprintf(f,"POH_ResponderPolicy\n");
		break;
	case POH_Invalid:
		fprintf(f,"POH_Invalid\n");
		break;
	}
}


int DBVprint_ill_clientid(FILE *f, DbvClientId *psClientId)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psClientName: ");
	if (psClientId->psClientName != NULL)
		DBVprint_ill_illstring(f, psClientId->psClientName);
	else
		fprintf(f,"NULL\n");
	

	/* o */
	fprintf(f,"psClientStatus: ");
	if (psClientId->psClientStatus != NULL)
		DBVprint_ill_illstring(f, psClientId->psClientStatus);
	else
		fprintf(f,"NULL\n");
	



	/* o */
	fprintf(f,"psClientIdentifier: ");
	if (psClientId->psClientIdentifier != NULL)
		DBVprint_ill_illstring(f, psClientId->psClientIdentifier);
	else
		fprintf(f,"NULL\n");
	

	
}



int DBVprint_ill_itemid(FILE *f, DbvItemId *psItemId)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"peItemType: ");
	if (psItemId->peItemType != NULL)
		DBVprint_ill_itemtype(f, psItemId->peItemType);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"peMediumType: ");
	if (psItemId->peMediumType != NULL)
		DBVprint_ill_mediumtype(f, psItemId->peMediumType);
	else
		fprintf(f,"NULL\n");

	
	/* o */
	fprintf(f,"psCallNumber: ");
	if (psItemId->psCallNumber != NULL)
		DBVprint_ill_illstring(f, psItemId->psCallNumber);
	else
		fprintf(f,"NULL\n");

	
	/* o */
	fprintf(f,"psAuthor: ");
	if (psItemId->psAuthor != NULL)
		DBVprint_ill_illstring(f, psItemId->psAuthor);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psTitle: ");
	if (psItemId->psTitle != NULL)
		DBVprint_ill_illstring(f, psItemId->psTitle);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psSubTitle: ");
	if (psItemId->psSubTitle != NULL)
		DBVprint_ill_illstring(f, psItemId->psSubTitle);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psSponsoringBody: ");
	if (psItemId->psSponsoringBody != NULL)
		DBVprint_ill_illstring(f, psItemId->psSponsoringBody);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psPlaceOfPublication: ");
	if (psItemId->psPlaceOfPublication != NULL)
		DBVprint_ill_illstring(f, psItemId->psPlaceOfPublication);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psPublisher: ");
	if (psItemId->psPublisher != NULL)
		DBVprint_ill_illstring(f, psItemId->psPublisher);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psSeriesTitleNumber: ");
	if (psItemId->psSeriesTitleNumber != NULL)
		DBVprint_ill_illstring(f, psItemId->psSeriesTitleNumber);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psVolumeIssue: ");
	if (psItemId->psVolumeIssue != NULL)
		DBVprint_ill_illstring(f, psItemId->psVolumeIssue);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psEdition: ");
	if (psItemId->psEdition != NULL)
		DBVprint_ill_illstring(f, psItemId->psEdition);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psPublicationDate: ");
	if (psItemId->psPublicationDate != NULL)
		DBVprint_ill_illstring(f, psItemId->psPublicationDate);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psPublicationDateOfComponent: ");
	if (psItemId->psPublicationDateOfComponent != NULL)
		DBVprint_ill_illstring(f, psItemId->psPublicationDateOfComponent);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psAuthorOfArticle: ");
	if (psItemId->psAuthorOfArticle != NULL)
		DBVprint_ill_illstring(f, psItemId->psAuthorOfArticle);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psTitleOfArticle: ");
	if (psItemId->psTitleOfArticle != NULL)
		DBVprint_ill_illstring(f, psItemId->psTitleOfArticle);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psPagination: ");
	if (psItemId->psPagination != NULL)
		DBVprint_ill_illstring(f, psItemId->psPagination);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psNationalBibliographyNo: ");
	if (psItemId->psNationalBibliographyNo != NULL)
		DBVprint_external(f, psItemId->psNationalBibliographyNo);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psIsbn: ");
	if (psItemId->psIsbn != NULL)
		DBVprint_ill_illstring(f, psItemId->psIsbn);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psIssn: ");
	if (psItemId->psIssn != NULL)
		DBVprint_ill_illstring(f, psItemId->psIssn);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psSystemNo: ");
	if (psItemId->psSystemNo != NULL)
		DBVprint_external(f, psItemId->psSystemNo);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psAdditionalNoLetters: ");
	if (psItemId->psAdditionalNoLetters != NULL)
		DBVprint_ill_illstring(f, psItemId->psAdditionalNoLetters);
	else
		fprintf(f,"NULL\n");
	
	/* o */
	fprintf(f,"psVerificationReferenceSource: ");
	if (psItemId->psVerificationReferenceSource != NULL)
		DBVprint_ill_illstring(f, psItemId->psVerificationReferenceSource);
	else
		fprintf(f,"NULL\n");
	
}



int DBVprint_ill_itemtype(FILE *f, DbvItemType *peItemType)
{
	switch (*peItemType) {
	case IT_Monograph:
		fprintf(f,"IT_Monograph\n");
		break;
	case IT_Serial:
		fprintf(f,"IT_Serial\n");
		break;
	case IT_Other:
		fprintf(f,"IT_Other\n");
		break;
	case IT_Invalid:
		fprintf(f,"IT_Invalid\n");
		break;
	}
}



int DBVprint_ill_mediumtype(FILE *f, DbvMediumType *peMediumType)
{
	switch (*peMediumType) {
	case MT_Printed:
		fprintf(f,"MT_Printed\n");
		break;
	case MT_Microform:
		fprintf(f,"MT_Microform\n");
		break;
	case MT_FilmOrVideoRecording:
		fprintf(f,"MT_FilmOrVideoRecording\n");
		break;
	case MT_AudioRecording:
		fprintf(f,"MT_AudioRecording\n");
		break;
	case MT_MachineReadable:
		fprintf(f,"MT_MachineReadable\n");
		break;
	case MT_Other:
		fprintf(f,"MT_Other\n");
		break;
	case MT_Invalid:
		fprintf(f,"MT_Invalid\n");
		break;
	}
	
}



int DBVprint_ill_suppdesc(FILE *f, DbvASN1External **ppsSupplementalItemDescription)
{

	int index = 0;

	fprintf(f,"\n");

	while (ppsSupplementalItemDescription[index] != NULL) {
       		fprintf(f,"\tpsSupplementalItemDescription[%d]: ", index);
		DBVprint_external(f, ppsSupplementalItemDescription[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_costinfo(FILE *f, DbvCostInfo *psCostInfo)
{
	
	fprintf(f,"\n");

	/* o */
	fprintf(f,"psAccountNumber: ");
	if (psCostInfo->psAccountNumber != NULL)
		DBVprint_ill_illstring(f, psCostInfo->psAccountNumber);
	else
		fprintf(f,"NULL\n");
	
	
	/* o */
	fprintf(f,"psMaximumCost: ");
	if (psCostInfo->psMaximumCost != NULL)
		DBVprint_ill_amount(f, psCostInfo->psMaximumCost);
	else
		fprintf(f,"NULL\n");
	

	/* default */
	fprintf(f,"pfReciprocalAgreement: ");
	if (psCostInfo->pfReciprocalAgreement)
		DBVprint_bool(f, psCostInfo->pfReciprocalAgreement);
	else
		fprintf(f,"NULL\n");
	

	/* default */
	fprintf(f,"pfWillPayFee: ");
	if (psCostInfo->pfWillPayFee)
		DBVprint_bool(f, psCostInfo->pfWillPayFee);
	else
		fprintf(f,"NULL\n");

	/* default */
	fprintf(f,"pfPaymentProvided: ");
	if (psCostInfo->pfWillPayFee)
		DBVprint_bool(f, psCostInfo->pfPaymentProvided);
	else
		fprintf(f,"NULL\n");
	
}



int DBVprint_ill_amount(FILE *f, DbvAmount *psMaximumCost)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"pcCurrencyCode: ");
	if (psMaximumCost->pcCurrencyCode != NULL)
		fprintf(f,"%s\n", psMaximumCost->pcCurrencyCode);
	else
		fprintf(f,"NULL\n");
	

	/* c */
	fprintf(f,"pcMonetaryValue: ");
	if (psMaximumCost->pcMonetaryValue != NULL)
		fprintf(f,"%s\n", psMaximumCost->pcMonetaryValue);
	
}



int DBVprint_ill_thirdparty(FILE *f, DbvThirdPartyInfo *psThirdPartyInfo)
{
	fprintf(f,"\n");

	/* default so optional */ 
	fprintf(f,"pfPermissionToForward: ");
	if (psThirdPartyInfo->pfPermissionToForward)
		DBVprint_bool(f, psThirdPartyInfo->pfPermissionToForward);
	else
		fprintf(f,"NULL\n");


	/* default so optional */
	fprintf(f,"pfPermissionToChain: ");
	if (psThirdPartyInfo->pfPermissionToChain)
		DBVprint_bool(f, psThirdPartyInfo->pfPermissionToChain);
	else
		fprintf(f,"NULL\n");


	/* default so optional */
	fprintf(f,"pfPermissionToPartition: ");
	if (psThirdPartyInfo->pfPermissionToPartition)
		DBVprint_bool(f, psThirdPartyInfo->pfPermissionToPartition);
	else
		fprintf(f,"NULL\n");


	/* default so optional */
	fprintf(f,"pfPermissionToChangeSendToList: ");
	if (psThirdPartyInfo->pfPermissionToChangeSendToList)
		DBVprint_bool(f, psThirdPartyInfo->pfPermissionToChangeSendToList);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psInitialRequesterAddress: ");
	if (psThirdPartyInfo->psInitialRequesterAddress != NULL)
		DBVprint_ill_sysaddr(f, psThirdPartyInfo->psInitialRequesterAddress);
	else
		fprintf(f,"NULL\n");
	

	/* default so optional */
	fprintf(f,"pePreference: ");
	if (psThirdPartyInfo->pePreference)
		DBVprint_ill_preference(f, psThirdPartyInfo->pePreference);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"ppsSendToList: ");
	if (psThirdPartyInfo->ppsSendToList != NULL)
		DBVprint_ill_sendtolistlist(f, psThirdPartyInfo->ppsSendToList);
	else
		fprintf(f,"NULL\n");

	

	/* o */
	fprintf(f,"ppsAlreadyTriedList: ");
	if (psThirdPartyInfo->ppsAlreadyTriedList != NULL)
		DBVprint_ill_systemidlist(f, psThirdPartyInfo->ppsAlreadyTriedList);
	else
		fprintf(f,"NULL\n");

	
	
}



int DBVprint_ill_preference(FILE *f, DbvPreference *pePreference)
{
	switch (*pePreference) {
	case PRE_Ordered:
		fprintf(f,"PRE_Ordered\n");
		break;
	case PRE_Unordered:
		fprintf(f,"PRE_Unordered\n");
		break;
	case PRE_Invalid:
		fprintf(f,"PRE_Invalid\n");
		break;
	}
}



int DBVprint_ill_sendtolistlist(FILE *f, DbvSendToList **ppsSendToList)
{

	int index = 0;

	fprintf(f,"\n");

	while (ppsSendToList[index] != NULL) {
       		fprintf(f,"\tpsSendToList[%d]: ", index);
		DBVprint_ill_sendtolist(f, ppsSendToList[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_sendtolist(FILE *f, DbvSendToList *psSendToList)
{
	/* c */
	fprintf(f,"sSystemId: ");
	DBVprint_ill_systemid(f, &psSendToList->sSystemId);

	/* o */
	fprintf(f,"psAccountNumber: ");
	if (psSendToList->psAccountNumber != NULL) 
		DBVprint_ill_illstring(f, psSendToList->psAccountNumber);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psSystemAddress: ");
	if (psSendToList->psSystemAddress != NULL) 
		DBVprint_ill_sysaddr(f, psSendToList->psSystemAddress);
	else
		fprintf(f,"NULL\n");
}


int DBVprint_ill_systemidlist(FILE *f, DbvSystemId **ppsAlreadyTriedList)
{
	int index = 0;

	fprintf(f,"\n");

	while (ppsAlreadyTriedList[index] != NULL) {
       		fprintf(f,"\tpsAlreadyTriedList[%d]: ", index);
		DBVprint_ill_systemid(f, ppsAlreadyTriedList[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_extensionlist(FILE *f, DbvExtension **ppsRequestExtension)
{
	int index = 0;

	fprintf(f,"\n");

	while (ppsRequestExtension[index] != NULL) {
       		fprintf(f,"\tpsRequestExtension[%d]\n", index);
		DBVprint_ill_extension(f, ppsRequestExtension[index]);
		index++;
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}



int DBVprint_ill_extension(FILE *f, DbvExtension *psRequestExtension)
{
	/* c */
	fprintf(f,"ulIdentifier: %lu\n", psRequestExtension->ulIdentifier);

	/* default */
	fprintf(f,"pfCritical: ");
	if (psRequestExtension->pfCritical)
		DBVprint_bool(f, psRequestExtension->pfCritical);	
	else
		fprintf(f,"NULL\n");


	/* c */
	switch (psRequestExtension->ulIdentifier) {

	case SUBITO_ADDITIONAL_INFO:
                DBVprint_illext_subito(f, psRequestExtension->uItem.psAddIllReqInfo);
                break;
        }


}



int DBVprint_illext_subito(FILE *f, DbvSubitoAdditionalInfo *psAddIllReqInfo)
{
        fprintf(f,"psAddIllReqInfo:\n");
        if (psAddIllReqInfo) {

                /* c */
                fprintf(f,"pfCustomerInfoToKeep: ");
               	DBVprint_bool(f, psAddIllReqInfo->pfCustomerInfoToKeep);

                /* o */
                DBVprint_illext_subito_request(f, psAddIllReqInfo->psAddRequesterInfo);

                /* o */
                DBVprint_illext_subito_deliver(f, psAddIllReqInfo->psAddDeliveryInfo);

                /* o */
                DBVprint_illext_subito_billing(f, psAddIllReqInfo->psAddBillingInfo);

        } else
                fprintf(f,"NULL\n");

}



int DBVprint_illext_subito_request(FILE *f, DbvAddRequesterInfo *psAddRequesterInfo)
{
        fprintf(f,"psAddRequesterInfo: ");
        if (psAddRequesterInfo) {

		fprintf(f,"\n");
	
		/* c */
		DBVprint_illext_subito_requestergroup(f, psAddRequesterInfo->peRequesterGroup);

		/* o */
		/* DbVisibleString is a typedefed char* */
		fprintf(f,"pcRequesterEmailAddress: ");
		DBVprint_visible_string(f, psAddRequesterInfo->pcRequesterEmailAddress);
		
        } else
                fprintf(f,"NULL\n");

}


int DBVprint_visible_string(FILE *f, DbvVisibleString pcRequesterEmailAddress)
{
	if (pcRequesterEmailAddress)
		fprintf(f,"%s\n", pcRequesterEmailAddress);
	else
		fprintf(f,"NULL\n");
}


int DBVprint_illext_subito_requestergroup(FILE *f, DbvRequesterGroup *peRequesterGroup)
{
	fprintf(f,"peRequesterGroup: ");
	if (peRequesterGroup) {
		switch (*peRequesterGroup) {
		case RG_UserGroup1:
			fprintf(f,"RG_UserGroup1\n");
			break;
		case RG_UserGroup2:
			fprintf(f,"RG_UserGroup2\n");
			break;
		}
	} else
                fprintf(f,"NULL\n");
}



int DBVprint_illext_subito_deliver(FILE *f, DbvAddDeliveryInfo *psAddDeliveryInfo)
{
        fprintf(f,"psAddDeliveryInfo: ");
        if (psAddDeliveryInfo) {

		fprintf(f,"\n");

		/* c */
		DBVprint_illext_subito_statusleveluser(f, psAddDeliveryInfo->peStatusLevelUser);
		
		/* c */
		DBVprint_illext_subito_statuslevelerequester(f, psAddDeliveryInfo->peStatusLevelOrigin);

		/* o */
		fprintf(f,"pcDeliveryNotes: ");
		DBVprint_visible_string(f, psAddDeliveryInfo->pcDeliveryNotes);

		/* c */
		fprintf(f,"pcCountryDeliveryTarget: ");
		DBVprint_visible_string(f, psAddDeliveryInfo->pcCountryDeliveryTarget);
		/* c */
                fprintf(f,"pfAlternativePostalDelivery: ");
               	DBVprint_bool(f, psAddDeliveryInfo->pfAlternativePostalDelivery);
		

        } else
                fprintf(f,"NULL\n");
}



int DBVprint_unsigned_long(FILE *f, Ulong *pulNumberOfCopies) 
{
	if (pulNumberOfCopies) 
		fprintf(f,"%lu\n", *pulNumberOfCopies);
	else
		fprintf(f,"NULL\n");
}




int DBVprint_illext_subito_statusleveluser(FILE *f, DbvStatusLevelUser *peStatusLevelUser)
{
	fprintf(f,"peStatusLevelUser: ");
	if (peStatusLevelUser) {
		switch (*peStatusLevelUser) {
		case SLU_All:
			fprintf(f,"SLU_All\n");
			break;
		case SLU_Negative:
			fprintf(f,"SLU_Negative\n");
			break;
		case SLU_None:
			fprintf(f,"SLU_None\n");
			break;	
		}
	} else
                fprintf(f,"NULL\n");
		
}



int DBVprint_illext_subito_statuslevelerequester(FILE *f, DbvStatusLevelRequester *peStatusLevelOrigin)
{
	fprintf(f,"peStatusLevelOrigin: ");
	if (peStatusLevelOrigin) {
		switch (*peStatusLevelOrigin) {
		case SLO_All:
			fprintf(f,"SLO_All\n");
			break;
		case SLO_Negative:
			fprintf(f,"SLO_Negative\n");
			break;
		case SLO_None:
			fprintf(f,"SLO_None\n");
			break;
		}
	} else
                fprintf(f,"NULL\n");
}



int DBVprint_illext_subito_billing(FILE *f, DbvAddBillingInfo *psAddBillingInfo)
{
        fprintf(f,"psAddBillingInfo: ");

        if (psAddBillingInfo) {

		fprintf(f, "\n");

		fprintf(f,"eAddBillingType: ");

		switch (psAddBillingInfo->eAddBillingType) {
		case ABT_BankInfo:
			fprintf(f,"ABT_BankInfo\n");
			DBVprint_illext_subito_billingbankinfo(f, psAddBillingInfo->uAddBillingInfo.psBillingBankInfo);
			break;
		case ABT_CreditCardInfo:
			fprintf(f,"ABT_CreditCardInfo\n");
			DBVprint_illext_subito_billingcreditcardinfo(f, psAddBillingInfo->uAddBillingInfo.psBillingCreditCardInfo);
			break;
		case ABT_InvoiceType:
			fprintf(f,"ABT_InvoiceType\n");
			DBVprint_illext_subito_billinginvoicetype(f, psAddBillingInfo->uAddBillingInfo.eBillingInvoiceType);
			break;

		case ABT_PrepaidType:
			fprintf(f,"ABT_PrepaidType\n");
			fprintf(f,"pcPrepaidId: ");
			DBVprint_visible_string(f, psAddBillingInfo->uAddBillingInfo.pcPrepaidId);
			break;

		case ABT_DepositType:
			fprintf(f,"ABT_DepositType\n");
			fprintf(f,"pcDepositId: ");
			DBVprint_visible_string(f, psAddBillingInfo->uAddBillingInfo.pcDepositId);
			break;
		default:
			fprintf(f,"\n");
			break;

		}
        } else
                fprintf(f,"NULL\n");
		
}



int DBVprint_illext_subito_billingbankinfo(FILE *f, DbvBillingBankInfo *psBillingBankInfo)
{
	fprintf(f,"psBillingBankInfo: ");
	if (psBillingBankInfo) {

		fprintf(f,"\n");

		/* c */
		fprintf(f,"pcBankAccountHouse: ");
		DBVprint_visible_string(f, psBillingBankInfo->pcBankAccountHouse);
		
		/* c */
		fprintf(f,"pcBankAccountRoutingSymbol: ");
		DBVprint_visible_string(f, psBillingBankInfo->pcBankAccountRoutingSymbol);
		
		/* c */
		fprintf(f,"pcBankAccountNo: ");
		DBVprint_visible_string(f, psBillingBankInfo->pcBankAccountNo);
		
		/* c */
                fprintf(f,"fBankAuthority: ");
               	DBVprint_bool(f, &psBillingBankInfo->fBankAuthority);
		

	} else
                fprintf(f,"NULL\n");

}



int DBVprint_illext_subito_billingcreditcardinfo(FILE *f, DbvBillingCreditCardInfo *psBillingCreditCardInfo)
{
	fprintf(f,"psBillingCreditCardInfo: ");
	if (psBillingCreditCardInfo) {

		fprintf(f,"\n");

		/* c */
		fprintf(f,"pcCardCompany: ");
		DBVprint_visible_string(f, psBillingCreditCardInfo->pcCardCompany);
		
		/* c */
		fprintf(f,"pcCardNo: ");
		DBVprint_visible_string(f, psBillingCreditCardInfo->pcCardNo);
		
		/* ISO_Date are VisibleString */

		/* o */
		fprintf(f,"pcCardValidFrom: ");
		DBVprint_visible_string(f, psBillingCreditCardInfo->pcCardValidFrom);
		
		/* c */
		fprintf(f,"pcCardValidThru: ");
		DBVprint_visible_string(f, psBillingCreditCardInfo->pcCardValidThru);

		
	} else
                fprintf(f,"NULL\n");

}



int DBVprint_illext_subito_billinginvoicetype(FILE *f, DbvBillingInvoiceType eBillingInvoiceType)
{
	fprintf(f,"eBillingInvoiceType: ");
	switch (eBillingInvoiceType) {
	case BIT_Single:
		fprintf(f,"BIT_Single\n");
		break;
	case BIT_Collective:
		fprintf(f,"BIT_Collective\n");
		break;
	}
}



int DBVprint_ill_statorerr(FILE *f, struct DbvStatusOrErrorReport *psStatusOrErrorReport)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eProtocolVersion: ");
	DBVprint_ill_protocolver(f, &psStatusOrErrorReport->eProtocolVersion);

	/* c */
	fprintf(f,"sTransactionId: ");
	DBVprint_ill_transid(f, &psStatusOrErrorReport->sTransactionId);
	
	/* c */
	fprintf(f,"sServiceDateTime: ");	
	DBVprint_ill_servdatetime(f, &psStatusOrErrorReport->sServiceDateTime);

	/* o */
	fprintf(f,"psRequesterId: ");
	if (psStatusOrErrorReport->psRequesterId != NULL) 
		DBVprint_ill_systemid(f, psStatusOrErrorReport->psRequesterId);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psResponderId: ");
	if (psStatusOrErrorReport->psResponderId != NULL) 
		DBVprint_ill_systemid(f, psStatusOrErrorReport->psResponderId);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"peReasonNoReport: ");
	if (psStatusOrErrorReport->peReasonNoReport != NULL) 
		DBVprint_ill_reasnorep(f, psStatusOrErrorReport->peReasonNoReport);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psStatusReport: ");
	if (psStatusOrErrorReport->psStatusReport != NULL) 
		DBVprint_ill_statrep(f, psStatusOrErrorReport->psStatusReport);
	else
		fprintf(f,"NULL\n");

	/* o */
	fprintf(f,"psErrorReport: ");
	if (psStatusOrErrorReport->psErrorReport != NULL) 
		DBVprint_ill_errrep(f, psStatusOrErrorReport->psErrorReport);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psNote: ");
	if (psStatusOrErrorReport->psNote != NULL) 
		DBVprint_ill_illstring(f, psStatusOrErrorReport->psNote);
	else
		fprintf(f,"NULL\n");


	
	/* o */
	fprintf(f,"ppsStOrErReportExtension: ");
	if (psStatusOrErrorReport->ppsStOrErReportExtension != NULL) 
		DBVprint_ill_extensionlist(f, psStatusOrErrorReport->ppsStOrErReportExtension);
	else
		fprintf(f,"NULL\n");

	
}



int DBVprint_ill_reasnorep(FILE *f, DbvReasonNoReport *peReasonNoReport)
{
	switch (*peReasonNoReport) {
	case RNR_Temporary:
		fprintf(f,"RNR_Temporary\n");
		break;
	case RNR_Permanent:
		fprintf(f,"RNR_Permanent\n");
		break;
	case RNR_Invalid:
		fprintf(f,"RNR_Invalid\n");
		break;
	}
	
}


int DBVprint_ill_statrep(FILE *f, DbvStatusReport *psStatusReport)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"sUserStatusReport: ");
	DBVprint_ill_histrep(f, &psStatusReport->sUserStatusReport);


	/* c */
	fprintf(f,"eProviderstatusReport: ");
	DBVprint_ill_provstat(f, &psStatusReport->eProviderStatusReport);
	
}



int DBVprint_ill_histrep(FILE *f, DbvHistoryReport *psUserStatusReport)
{
	fprintf(f,"\n");

	/* o */
	fprintf(f,"pcDateRequested: ");
	if (psUserStatusReport->pcDateRequested != NULL) 
		fprintf(f,"%s\n", psUserStatusReport->pcDateRequested);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psAuthor: ");
	if (psUserStatusReport->psAuthor != NULL) 
		DBVprint_ill_illstring(f, psUserStatusReport->psAuthor);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psTitle: ");
	if (psUserStatusReport->psTitle != NULL) 
		DBVprint_ill_illstring(f, psUserStatusReport->psTitle);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psAuthorOfArticle: ");
	if (psUserStatusReport->psAuthorOfArticle != NULL) 
		DBVprint_ill_illstring(f, psUserStatusReport->psAuthorOfArticle);
	else
		fprintf(f,"NULL\n");

	
	/* o */
	fprintf(f,"psTitleOfArticle: ");
	if (psUserStatusReport->psTitleOfArticle != NULL) 
		DBVprint_ill_illstring(f, psUserStatusReport->psTitleOfArticle);
	else
		fprintf(f,"NULL\n");


	/* c */
	fprintf(f,"pcDateOfLastTransition: ");
	if (psUserStatusReport->pcDateOfLastTransition != NULL) 
		fprintf(f,"%s\n", psUserStatusReport->pcDateOfLastTransition);
	else
		fprintf(f,"NULL\n");


	/* c */
	fprintf(f,"eMostRecentService: ");
	DBVprint_ill_mostrecserv(f, &psUserStatusReport->eMostRecentService);

	/* c */
	fprintf(f,"pcDateOfMostRecentService: ");
	if (psUserStatusReport->pcDateOfMostRecentService != NULL) 
		fprintf(f,"%s\n", psUserStatusReport->pcDateOfMostRecentService);
	else
		fprintf(f,"NULL\n");


	/* c */
	fprintf(f,"sSystemId: ");
	DBVprint_ill_systemid(f, &psUserStatusReport->sSystemId);
	
	/* o */
	fprintf(f,"peShippedServiceType: ");
	if (psUserStatusReport->peShippedServiceType != NULL) 
		DBVprint_ill_shippedservtype(f, psUserStatusReport->peShippedServiceType);
	else
		fprintf(f,"NULL\n");

	
	/* o */
	fprintf(f,"peTransactionResults: ");
	if (psUserStatusReport->peTransactionResults != NULL) 
		DBVprint_ill_transres(f, psUserStatusReport->peTransactionResults);
	else
		fprintf(f,"NULL\n");


	/* o */
	fprintf(f,"psMostRecentServiceNote: ");
	if (psUserStatusReport->psMostRecentServiceNote != NULL) 
		DBVprint_ill_illstring(f, psUserStatusReport->psMostRecentServiceNote);
	else
		fprintf(f,"NULL\n");
}



int DBVprint_ill_mostrecserv(FILE *f, DbvMostRecentService *peMostRecentService)
{
	switch (*peMostRecentService) {
	case MRS_IllRequest:
		fprintf(f,"MRS_IllRequest\n");
		break;
	case MRS_Forward:
		fprintf(f,"MRS_Forward\n");
		break;
	case MRS_ForwardNotification:
		fprintf(f,"MRS_ForwardNotification\n");
		break;
	case MRS_Shipped:
		fprintf(f,"MRS_Shipped\n");
		break;
	case MRS_IllAnswer:
		fprintf(f,"MRS_IllAnswer\n");
		break;
	case MRS_ConditionalReply:
		fprintf(f,"MRS_ConditionalReply\n");
		break;
	case MRS_Cancel:
		fprintf(f,"MRS_Cancel\n");
		break;
	case MRS_CancelReply:
		fprintf(f,"MRS_CancelReply\n");
		break;
	case MRS_Received:
		fprintf(f,"MRS_Received\n");
		break;
	case MRS_Recall:
		fprintf(f,"MRS_Recall\n");
		break;
	case MRS_Returned:
		fprintf(f,"MRS_Returned\n");
		break;
	case MRS_CheckedIn:
		fprintf(f,"MRS_CheckedIn\n");
		break;
	case MRS_Overdue:
		fprintf(f,"MRS_Overdue\n");
		break;
	case MRS_Renew:
		fprintf(f,"MRS_Renew\n");
		break;
	case MRS_RenewAnswer:
		fprintf(f,"MRS_RenewAnswer\n");
		break;
	case MRS_Lost:
		fprintf(f,"MRS_Lost\n");
		break;
	case MRS_Damaged:
		fprintf(f,"MRS_Damaged\n");
		break;
	case MRS_Message:
		fprintf(f,"MRS_Message\n");
		break;
	case MRS_StatusQuery:
		fprintf(f,"MRS_StatusQuery\n");
		break;
	case MRS_StatusOrErrorReport:
		fprintf(f,"MRS_StatusOrErrorReport\n");
		break;
	case MRS_Expired:
		fprintf(f,"MRS_Expired\n");
		break;
	case MRS_Invalid:
		fprintf(f,"MRS_Invalid\n");
		break;
	}
}



int DBVprint_ill_shippedservtype(FILE *f, DbvShippedServiceType *peShippedServiceType)
{
	switch (*peShippedServiceType) {
	case SST_Loan:
		fprintf(f,"SST_Loan\n");
		break;
	case SST_CopyNonReturnable:
		fprintf(f,"SST_CopyNonReturnable\n");
		break;
	case SST_Invalid:
		fprintf(f,"SST_Invalid\n");
		break;
	}
	
}


int DBVprint_ill_transres(FILE *f, DbvTransactionResults *peTransactionResults)
{
	switch (*peTransactionResults) {
	case TR_Conditional:
		fprintf(f,"TR_Conditional\n");
		break;
	case TR_Retry:
		fprintf(f,"TR_Retry\n");
		break;
	case TR_Unfilled:
		fprintf(f,"TR_Unfilled\n");
		break;
	case TR_LocationsProvided:
		fprintf(f,"TR_LocationsProvided\n");
		break;
	case TR_WillSupply:
		fprintf(f,"TR_WillSupply\n");
		break;
	case TR_HoldPlaced:
		fprintf(f,"TR_HoldPlaced\n");
		break;
	case TR_Estimate:
		fprintf(f,"TR_Estimate\n");
		break;
	case TR_Invalid:
		fprintf(f,"TR_Invalid\n");
		break;
	}
}



int DBVprint_ill_provstat(FILE *f, DbvCurrentState *peProviderStatusReport)
{
	switch (*peProviderStatusReport) {
	case CS_NotSupplied:
		fprintf(f,"CS_NotSupplied\n");
		break;
	case CS_Pending:
		fprintf(f,"CS_Pending\n");
		break;
	case CS_InProcess:
		fprintf(f,"CS_InProcess\n");
		break;
	case CS_Forward:
		fprintf(f,"CS_Forward\n");
		break;
	case CS_Conditional:
		fprintf(f,"CS_Conditional\n");
		break;
	case CS_CancelPending:
		fprintf(f,"CS_CancelPending\n");
		break;
	case CS_Cancelled:
		fprintf(f,"CS_Cancelled\n");
		break;
	case CS_Shipped:
		fprintf(f,"CS_Shipped\n");
		break;
	case CS_Received:
		fprintf(f,"CS_Received\n");
		break;
	case CS_RenewPending:
		fprintf(f,"CS_RenewPending\n");
		break;
	case CS_NotReceivedOverdue:
		fprintf(f,"CS_NotReceivedOverdue\n");
		break;
	case CS_RenewOverdue:
		fprintf(f,"CS_RenewOverdue\n");
		break;
	case CS_Overdue:
		fprintf(f,"CS_Overdue\n");
		break;
	case CS_Returned:
		fprintf(f,"CS_Returned\n");
		break;
	case CS_CheckedIn:
		fprintf(f,"CS_CheckedIn\n");
		break;
	case CS_Recall:
		fprintf(f,"CS_Recall\n");
		break;
	case CS_Lost:
		fprintf(f,"CS_Lost\n");
		break;
	case CS_Unknown:
		fprintf(f,"CS_Unknown\n");
		break;
	case CS_Invalid:
		fprintf(f,"CS_Invalid\n");
		break;
	}
	
}



int DBVprint_ill_errrep(FILE *f, DbvErrorReport *psErrorReport)
{

	/* c */
	fprintf(f,"sCorrelationInfo: ");
	DBVprint_ill_illstring(f, &psErrorReport->sCorrelationInfo);
	

	/* c */
	fprintf(f,"sReport: ");
	DBVprint_ill_userorprovrrrep(f, &psErrorReport->sReport);
}



int DBVprint_ill_userorprovrrrep(FILE *f, DbvUserOrProviderErrorReport *psReport)
{
	fprintf(f,"\n");

	fprintf(f,"eReportSource: ");
	switch (psReport->eReportSource) {
	case RS_User:
		fprintf(f,"RS_User: ");
		if (psReport->uReport.psUserErrorReport != NULL)
			DBVprint_ill_usererrrep(f, psReport->uReport.psUserErrorReport);
		break;
	case RS_Provider:
		fprintf(f,"RS_Provider: ");
		if (psReport->uReport.psProviderErrorReport != NULL)
			DBVprint_ill_proverrrep(f, psReport->uReport.psProviderErrorReport);

		break;
	}
}



int DBVprint_ill_usererrrep(FILE *f, DbvUserErrorReport *psUserErrorReport)
{
	fprintf(f,"\n");

	fprintf(f,"eReportType: ");
	switch (psUserErrorReport->eReportType) {
	case UER_AlreadyForwarded:
		fprintf(f,"UER_AlreadyForwarded: ");
		if (psUserErrorReport->uUserErrorReport.psAlreadyForwarded != NULL) 
			DBVprint_ill_alreadyforw(f, psUserErrorReport->uUserErrorReport.psAlreadyForwarded);
		break;
	case UER_IntermediaryProblem:
		fprintf(f,"UER_IntermediaryProblem: ");
		DBVprint_ill_intermed(f, &psUserErrorReport->uUserErrorReport.eIntermediaryProblem);
		break;
	case UER_SecurityProblem:
		fprintf(f,"UER_SecurityProblem: ");
		if (psUserErrorReport->uUserErrorReport.psSecurityProblem != NULL)
			DBVprint_ill_illstring(f, psUserErrorReport->uUserErrorReport.psSecurityProblem);
		break;
	case UER_UnableToPerform:
		fprintf(f,"UER_UnableToPerform: ");
		DBVprint_ill_unabletoper(f, &psUserErrorReport->uUserErrorReport.eUnableToPerform);
		break;
	}
	
}



int DBVprint_ill_alreadyforw(FILE *f, DbvAlreadyForwarded *psAlreadyForwarded)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"sResponderId: ");
	DBVprint_ill_systemid(f, &psAlreadyForwarded->sResponderId);

	/* o */
	fprintf(f,"psResponderAddress: ");
	if (psAlreadyForwarded->psResponderAddress != NULL)
		DBVprint_ill_sysaddr(f, psAlreadyForwarded->psResponderAddress);
	else
		fprintf(f,"NULL\n");
	
}


int DBVprint_ill_intermed(FILE *f, DbvIntermediaryProblem *peIntermediaryProblem)
{
	switch (*peIntermediaryProblem) {
	case IP_CannotSendOnward:
		fprintf(f,"IP_CannotSendOnward\n");
		break;
	case IP_Invalid:
		fprintf(f,"IP_Invalid\n");
		break;
	}
}



int DBVprint_ill_unabletoper(FILE *f, DbvUnableToPerform *peUnableToPerform)
{
	switch (*peUnableToPerform) {
	case UTP_NotAvailable:
		fprintf(f,"UTP_NotAvailable\n");
		break;
	case UTP_ResourceLimitation:
		fprintf(f,"UTP_ResourceLimitation\n");
		break;
	case UTP_Other:
		fprintf(f,"UTP_Other\n");
		break;
	case UTP_Invalid:
		fprintf(f,"UTP_Invalid\n");
		break;
	}
	
}



int DBVprint_ill_proverrrep(FILE *f, DbvProviderErrorReport *psProviderErrorReport)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eReportType: ");
	switch (psProviderErrorReport->eReportType) {
	case PER_GeneralProblem:
		fprintf(f,"PER_GeneralProblem: ");
		DBVprint_ill_genprob(f, &psProviderErrorReport->uProviderErrorReport.eGeneralProblem);
		break;
	case PER_TransactionIdProblem:
		fprintf(f,"PER_TransactionIdProblem: ");
		DBVprint_ill_transidprob(f, &psProviderErrorReport->uProviderErrorReport.eTransactionIdProblem);
		break;
	case PER_StateTransitionProhibited:
		fprintf(f,"PER_StateTransitionProhibited: ");
		if (psProviderErrorReport->uProviderErrorReport.psStateTransitionProblem != NULL)
			DBVprint_ill_statetransprohib(f, psProviderErrorReport->uProviderErrorReport.psStateTransitionProblem);
		break;
	}
}



int DBVprint_ill_genprob(FILE *f, DbvGeneralProblem *peGeneralProblem)
{
	switch (*peGeneralProblem) {
	case GP_UnrecognizedAPDU:
		fprintf(f,"GP_UnrecognizedAPDU\n");
		break;
	case GP_MistypedAPDU:
		fprintf(f,"GP_MistypedAPDU\n");
		break;
	case GP_BadlyStructuredAPDU:
		fprintf(f,"GP_BadlyStructuredAPDU\n");
		break;
	case GP_ProtocolVersionNotSupported:
		fprintf(f,"GP_ProtocolVersionNotSupported\n");
		break;
	case GP_Other:
		fprintf(f,"GP_Other\n");
		break;
	case GP_Invalid:
		fprintf(f,"GP_Invalid\n");
		break;
	}
	
}



int DBVprint_ill_transidprob(FILE *f, DbvTransactionIdProblem *peTransactionIdProblem)
{
	switch (*peTransactionIdProblem) {
	case TIP_DuplicateTransactionId:
		fprintf(f,"TIP_DuplicateTransactionId\n");
		break;
	case TIP_InvalidTransactionId:
		fprintf(f,"TIP_InvalidTransactionId\n");
		break;
	case TIP_UnknownTransactionId:
		fprintf(f,"TIP_UnknownTransactionId\n");
		break;
	case TIP_Invalid:
		fprintf(f,"TIP_Invalid\n");
		break;
	}

}

int DBVprint_ill_statetransprohib(FILE *f, DbvStateTransitionProhibited *psStateTransitionProblem)
{
	fprintf(f,"\n");

	/* c */
	fprintf(f,"eAPDUType: ");
	DBVprint_ill_apdutype(f, &psStateTransitionProblem->eAPDUType);
	
	/* c */
	fprintf(f,"eCurrentState: ");
	DBVprint_ill_provstat(f, &psStateTransitionProblem->eCurrentState);
	
}


int DBVprint_ill_apdutype(FILE *f, DbvIllAPDUType *peAPDUType)
{
	switch (*peAPDUType) {
	case IAT_IllRequest:
		fprintf(f,"IAT_IllRequest\n");
		break;
	case IAT_ForwardNotification:
		fprintf(f,"IAT_ForwardNotification\n");
		break;
	case IAT_Shipped:
		fprintf(f,"IAT_Shipped\n");
		break;
	case IAT_IllAnswer:
		fprintf(f,"IAT_IllAnswer\n");
		break;
	case IAT_ConditionalReply:
		fprintf(f,"IAT_ConditionalReply\n");
		break;
	case IAT_Cancel:
		fprintf(f,"IAT_Cancel\n");
		break;
	case IAT_CancelReply:
		fprintf(f,"IAT_CancelReply\n");
		break;
	case IAT_Received:
		fprintf(f,"IAT_Received\n");
		break;
	case IAT_Recall:
		fprintf(f,"IAT_Recall\n");
		break;
	case IAT_Returned:
		fprintf(f,"IAT_Returned\n");
		break;
	case IAT_CheckedIn:
		fprintf(f,"IAT_CheckedIn\n");
		break;
	case IAT_Overdue:
		fprintf(f,"IAT_Overdue\n");
		break;
	case IAT_Renew:
		fprintf(f,"IAT_Renew\n");
		break;
	case IAT_RenewAnswer:
		fprintf(f,"IAT_RenewAnswer\n");
		break;
	case IAT_Lost:
		fprintf(f,"IAT_Lost\n");
		break;
	case IAT_Damaged:
		fprintf(f,"IAT_Damaged\n");
		break;
	case IAT_Message:
		fprintf(f,"IAT_Message\n");
		break;
	case IAT_StatusQuery:
		fprintf(f,"IAT_StatusQuery\n");
		break;
	case IAT_StatusOrErrorReport:
		fprintf(f,"IAT_StatusOrErrorReport\n");
		break;
	case IAT_Expired:
		fprintf(f,"IAT_Expired\n");
		break;
	case IAT_Invalid:
		fprintf(f,"IAT_Invalid\n");
		break;
	}
}





int DBVprint_aligned_external(FILE *f, DbvASN1External *external_ptr)
{
        fprintf(f,"\tpsOctetAligned:");
        DBVprint_octet_string(f, external_ptr->uExternalValue.psOctetAligned);



}



int DBVprint_intmaptest_external(FILE *f, DbvASN1External *external_ptr)
{

/********************
 No longer used

	DbvIntMapTest *intmaptest_ptr;

	fprintf(f,"psIntMapTest: \n");
	intmaptest_ptr = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psIntMapTest;

	fprintf(f,"eIntMapTestType: ");

	if (intmaptest_ptr != NULL) {
		switch (intmaptest_ptr->eIntMapTestType) {
		case IMT_FirstString:
			fprintf(f,"IMT_FirstString: ");
			if (intmaptest_ptr->uIntMapTest.psFirstString != NULL)
				DBVprint_inter_string(f, intmaptest_ptr->uIntMapTest.psFirstString);
			break;
		case IMT_SecondString:
			fprintf(f,"IMT_SecondString: ");
			if (intmaptest_ptr->uIntMapTest.psSecondString != NULL)
				DBVprint_inter_string(f, intmaptest_ptr->uIntMapTest.psSecondString);
			break;
		}
	}
	
********************/

}







void DBVprint_accessrsp_fields(FILE *f, DbvAccessControlRsp *psAccessControlRsp)
{

	fprintf(f,"AccessControlResponse PDU\n");
	
	/* o, o, o, o */
	DBVprint_DbvAccessControlRsp_psReferenceId(f, psAccessControlRsp);
	DBVprint_DbvAccessControlRsp_psSecurityChallengeRsp(f, psAccessControlRsp);
	DBVprint_DbvAccessControlRsp_psDiagnostic(f, psAccessControlRsp);
	DBVprint_DbvAccessControlRsp_ppsOtherInformation(f, psAccessControlRsp);
}



void DBVprint_DbvAccessControlRsp_psReferenceId(FILE *f, DbvAccessControlRsp *psAccessControlRsp)
{
	/* o */
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psAccessControlRsp->psReferenceId);
}



void DBVprint_DbvAccessControlRsp_psSecurityChallengeRsp(FILE *f, DbvAccessControlRsp *psAccessControlRsp)
{
	/* o */
	fprintf(f,"psSecurityChallengeRsp: ");

	if (psAccessControlRsp->psSecurityChallengeRsp == NULL) {
		fprintf(f,"NULL\n");
		return;
	}


	fprintf(f,"\neSecurityChallengeRspType: ");

	switch (psAccessControlRsp->psSecurityChallengeRsp->eSecurityChallengeRspType) {
	case SECT_SimpleForm:
		fprintf(f,"SECT_SimpleForm\n");
		fprintf(f,"psSimpleForm: ");
		DBVprint_octet_string(f, psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psSimpleForm);
		break;
	case SECT_ExternallyDefined:
		fprintf(f,"SECT_ExternallyDefined\n");
		fprintf(f,"psExternallyDefinedInfo:");
		DBVprint_external(f, psAccessControlRsp->psSecurityChallengeRsp->uSecurityChallengeRsp.psExternallyDefinedInfo);
		break;
	case SECT_Invalid:
		fprintf(f,"SECT_Invalid\n");
		break;
	}
	
}



void DBVprint_DbvAccessControlRsp_psDiagnostic(FILE *f, DbvAccessControlRsp *psAccessControlRsp)
{
	/* o */
	fprintf(f,"psDiagnostic:\n");
	DBVprint_diag_rec(f, psAccessControlRsp->psDiagnostic);
}



void DBVprint_DbvAccessControlRsp_ppsOtherInformation(FILE *f, DbvAccessControlRsp *psAccessControlRsp)
{
	/* o */
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psAccessControlRsp->ppsOtherInformation);
}




void DBVprint_rescntrlrsp_fields(FILE *f, DbvResourceControlRsp *psResourceControlRsp)
{
	fprintf(f,"ResourceControlResponse PDU\n");

	/* o, c, o, o */
	DBVprint_DbvResourceControlRsp_psReferenceId(f, psResourceControlRsp);
	DBVprint_DbvResourceControlRsp_fContinueFlag(f, psResourceControlRsp);
	DBVprint_DbvResourceControlRsp_pfResultSetWanted(f, psResourceControlRsp);
	DBVprint_DbvResourceControlRsp_ppsOtherInformation(f, psResourceControlRsp);
}


void DBVprint_DbvResourceControlRsp_psReferenceId(FILE *f, DbvResourceControlRsp *psResourceControlRsp)
{
	/* o */
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psResourceControlRsp->psReferenceId);
}



void DBVprint_DbvResourceControlRsp_fContinueFlag(FILE *f, DbvResourceControlRsp *psResourceControlRsp)
{
	/* c */
       	fprintf(f,"fContinueFlag: ");
	DBVprint_bool(f, &psResourceControlRsp->fContinueFlag);
}



void DBVprint_DbvResourceControlRsp_pfResultSetWanted(FILE *f, DbvResourceControlRsp *psResourceControlRsp)
{
	/* o */
       	fprintf(f,"pfResultSetWanted: ");
	DBVprint_bool(f, psResourceControlRsp->pfResultSetWanted);
}



void DBVprint_DbvResourceControlRsp_ppsOtherInformation(FILE *f, DbvResourceControlRsp *psResourceControlRsp)
{
	/* o */
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psResourceControlRsp->ppsOtherInformation);
}



void DBVprint_trigrescntrlreq_fields(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	fprintf(f,"TriggerResourceControlRequest PDU\n");

	/* o, c, o, o, o */
	DBVprint_DbvTriggerResourceControlReq_psReferenceId(f, psTriggerResourceControlReq);
	DBVprint_DbvTriggerResourceControlReq_eRequestedAction(f, psTriggerResourceControlReq);
	DBVprint_DbvTriggerResourceControlReq_pcPreferredResourceReportFormat(f, psTriggerResourceControlReq);
	DBVprint_DbvTriggerResourceControlReq_pfResultSetWanted(f, psTriggerResourceControlReq);
	DBVprint_DbvTriggerResourceControlReq_ppsOtherInformation(f, psTriggerResourceControlReq);
}


void DBVprint_DbvTriggerResourceControlReq_psReferenceId(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	/* o */
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psTriggerResourceControlReq->psReferenceId);
}



void DBVprint_DbvTriggerResourceControlReq_eRequestedAction(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	/* c */

	fprintf(f,"eRequestedAction: ");

	switch (psTriggerResourceControlReq->eRequestedAction) {
	case RA_ResourceReport1:
		fprintf(f,"RA_ResourceReport1\n");
		break;
		
	case RA_ResourceControl:
		fprintf(f,"RA_ResourceControl\n");
		break;

	case RA_Cancel:
		fprintf(f,"RA_Cancel\n");
		break;

	case RA_Invalid:
		fprintf(f,"RA_Invalid\n");
		break;
	}
}



void DBVprint_DbvTriggerResourceControlReq_pcPreferredResourceReportFormat(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	/* o */
	fprintf(f,"pcPreferredResourceReportFormat: ");
	DBVprint_DbvObjectIdentifierA(f, psTriggerResourceControlReq->pcPreferredResourceReportFormat);
}



void DBVprint_DbvObjectIdentifierA(FILE *f, DbvObjectIdentifier pcPreferredResourceReportFormat)
{	
	if (pcPreferredResourceReportFormat == NULL) {
		fprintf(f,"NULL\n");
		return;
	}


	fprintf(f,pcPreferredResourceReportFormat);
	fprintf(f,"\n");
}



void DBVprint_DbvTriggerResourceControlReq_pfResultSetWanted(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	/* o */
       	fprintf(f,"pfResultSetWanted: ");
	DBVprint_bool(f, psTriggerResourceControlReq->pfResultSetWanted);
}



void DBVprint_DbvTriggerResourceControlReq_ppsOtherInformation(FILE *f, DbvTriggerResourceControlReq *psTriggerResourceControlReq)
{
	/* o */
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psTriggerResourceControlReq->ppsOtherInformation);
}



void DBVprint_accessreq_fields(FILE *f, DbvAccessControlReq *psAccessControlReq)
{
	fprintf(f,"AccessControlRequest PDU\n");
	
	/* o, c, o */
	DBVprint_DbvAccessControlReq_psReferenceId(f, psAccessControlReq);
	DBVprint_DbvAccessControlReq_psSecurityChallengeReq(f, psAccessControlReq);
	DBVprint_DbvAccessControlReq_ppsOtherInformation(f, psAccessControlReq);
}



void DBVprint_DbvAccessControlReq_psReferenceId(FILE *f, DbvAccessControlReq *psAccessControlReq)
{
	/* o */
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psAccessControlReq->psReferenceId);
}



void DBVprint_DbvAccessControlReq_psSecurityChallengeReq(FILE *f, DbvAccessControlReq *psAccessControlReq)
{
	/* c */
	fprintf(f,"sSecurityChallengeReq:\n");

	fprintf(f,"eSecurityChallengeReqType: ");

	switch (psAccessControlReq->sSecurityChallengeReq.eSecurityChallengeReqType) {
	case SECT_SimpleForm:
		fprintf(f,"SECT_SimpleForm\n");
		fprintf(f,"psSimpleForm: ");
		DBVprint_octet_string(f, psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psSimpleForm);
		break;
	case SECT_ExternallyDefined:
		fprintf(f,"SECT_ExternallyDefined\n");
		fprintf(f,"psExternallyDefinedInfo:");
		DBVprint_external(f, psAccessControlReq->sSecurityChallengeReq.uSecurityChallengeReq.psExternallyDefinedInfo);
		break;
	case SECT_Invalid:
		fprintf(f,"SECT_Invalid\n");
		break;
	}
}



void DBVprint_DbvAccessControlReq_ppsOtherInformation(FILE *f, DbvAccessControlReq *psAccessControlReq)
{
	/* o */
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psAccessControlReq->ppsOtherInformation);
}


void DBVprint_rescntrlreq_fields(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	fprintf(f,"ResourceControlRequest PDU\n");
	
	/* o, o, o, o, c, o, o */
	DBVprint_DbvResourceControlReq_psReferenceId(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_pfSuspendedFlag(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_psResourceReport(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_pePartialResultsAvailable(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_fResponseRequired(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_pfTriggeredRequestFlag(f, psResourceControlReq);
	DBVprint_DbvResourceControlReq_ppsOtherInformation(f, psResourceControlReq);
}



void DBVprint_DbvResourceControlReq_psReferenceId(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
	/* o */
       	fprintf(f,"psReferenceId: ");
	DBVprint_octet_string(f, psResourceControlReq->psReferenceId);
}


void DBVprint_DbvResourceControlReq_pfSuspendedFlag(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
       	fprintf(f,"pfSuspendedFlag: ");
	DBVprint_bool(f, psResourceControlReq->pfSuspendedFlag);
}


void DBVprint_DbvResourceControlReq_psResourceReport(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
	fprintf(f,"psResourceReport: ");
	DBVprint_external(f, psResourceControlReq->psResourceReport);
}


void DBVprint_DbvResourceControlReq_pePartialResultsAvailable(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
	fprintf(f,"pePartialResultsAvailable: ");

	if (psResourceControlReq->pePartialResultsAvailable == NULL) {
		fprintf(f,"NULL\n");
		return;
	}


	switch (*psResourceControlReq->pePartialResultsAvailable) {
	case PRA_Subset:
		fprintf(f,"PRA_Subset\n");
		break;
	case PRA_Interim2:
		fprintf(f,"PRA_Interim2\n");
		break;
	case PRA_None2:
		fprintf(f,"PRA_None2\n");
		break;
	case PRA_Invalid:
		fprintf(f,"PRA_Invalid\n");
		break;
	}
}



void DBVprint_DbvResourceControlReq_fResponseRequired(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* c */
       	fprintf(f,"fResponseRequired: ");
	DBVprint_bool(f, &psResourceControlReq->fResponseRequired);
}



void DBVprint_DbvResourceControlReq_pfTriggeredRequestFlag(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
       	fprintf(f,"pfTriggeredRequestFlag: ");
	DBVprint_bool(f, psResourceControlReq->pfTriggeredRequestFlag);
}



void DBVprint_DbvResourceControlReq_ppsOtherInformation(FILE *f, DbvResourceControlReq *psResourceControlReq)
{
	/* o */
	/* o */
	fprintf(f,"ppsOtherInformation: ");
	DBVprint_otherinfo(f, psResourceControlReq->ppsOtherInformation);
}



void DBVprint_DbvPromptObject_external(FILE *f, DbvASN1External *external_ptr)
{
	struct DbvPromptObject *psPromptObject = NULL;

	psPromptObject = external_ptr->uExternalValue.psSingleASN1Type->uASN1.psPromptObject;
	
	/* Will always be non NULL after decoding but check just in
	   case function has been called before PDU sending */
	if (psPromptObject != NULL) {
        	fprintf(f,"\tpsPromptObject: ");
		DBVprint_DbvPromptObject(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psPromptObject);
	}
}



void DBVprint_DbvPromptObject(FILE *f, struct DbvPromptObject *psPromptObject)
{
	if (psPromptObject == NULL) {
		fprintf(f,"NULL\n");
		return;
	}
	
	fprintf(f,"\n");

	fprintf(f,"ePromptObjectType: ");
	switch (psPromptObject->ePromptObjectType) {
	case POT_Challenge:
		fprintf(f,"POT_Challenge\n");
		fprintf(f,"ppsChallenge:\n");
		DBVprint_DbvChallenge_list(f, psPromptObject->uPromptObject.ppsChallenge);
		break;
	case POT_Response:
		fprintf(f,"POT_Response\n");
		fprintf(f,"ppsResponse:\n");
		DBVprint_DbvResponse_list(f, psPromptObject->uPromptObject.ppsResponse);
		break;
	case POT_Invalid:
		fprintf(f,"POT_Invalid\n");
		break;
	}
}



void DBVprint_DbvChallenge_list(FILE *f, DbvChallenge **ppsChallenge)
{
	if (ppsChallenge == NULL) {
		fprintf(f, "NULL\n");
		return;
	}

	while (*ppsChallenge) {
		DBVprint_DbvChallenge(f, *ppsChallenge);
		ppsChallenge++;
		fprintf(f,"\n");
	}
}



void DBVprint_DbvChallenge(FILE *f, DbvChallenge *psChallenge)
{
	/* c */
	fprintf(f,"sPromptId: ");
	DBVprint_DbvPromptId(f, &psChallenge->sPromptId);
	
	/* o */
	fprintf(f,"psDefaultResponse: ");
	DBVprint_inter_string(f, psChallenge->psDefaultResponse);
	
	/* o */
	fprintf(f,"psPromptInfo: ");
	DBVprint_DbvPromptInfo(f, psChallenge->psPromptInfo);
	
	/* o */
	fprintf(f,"psRegExpr: ");
	DBVprint_inter_string(f, psChallenge->psRegExpr);
	
	/* o */
	DBVprint_DbvNull(f, "pvResponseRequired", psChallenge->pvResponseRequired);
	
	/* o */
	DBVprintN_DbvInternationalString(f, "ppsAllowedValues", psChallenge->ppsAllowedValues);
	
	/* o */
	DBVprint_DbvNull(f, "pvShouldSave", psChallenge->pvShouldSave);
	
	/* o */
	fprintf(f,"peDataType: ");
	DBVprint_DbvDataType(f, psChallenge->peDataType);
	
	/* o */
	fprintf(f,"psDiagnostic: ");
	DBVprint_external(f, psChallenge->psDiagnostic);
	
	
}


void DBVprint_DbvPromptId(FILE *f, DbvPromptId *psPromptId)
{
	if (psPromptId == NULL) {
		fprintf(f,"NULL\n");
		return;
	}
	
	fprintf(f,"\n");

	fprintf(f,"ePromptIdType: ");
	switch (psPromptId->ePromptIdType) {
	case PIT_EnumeratedPrompt:
		fprintf(f,"PIT_EnumeratedPrompt\n");
		fprintf(f,"psEnumeratedPrompt:\n");
		DBVprint_DbvEnumeratedPrompt(f, psPromptId->uPromptId.psEnumeratedPrompt);
		break;
	case PIT_NoneEnumeratedPrompt:
		fprintf(f,"PIT_NoneEnumeratedPrompt\n");
		fprintf(f,"psNonEnumeratedPrompt:\n");
		DBVprint_inter_string(f, psPromptId->uPromptId.psNonEnumeratedPrompt);
		break;
	case PIT_Invalid:
		fprintf(f,"PIT_Invalid\n");
		break;
	}
}



void DBVprint_DbvEnumeratedPrompt(FILE *f, DbvEnumeratedPrompt *psEnumeratedPrompt)
{
	if (psEnumeratedPrompt == NULL) {
		fprintf(f, "NULL\n");
		return;
	}

	/* c */
	fprintf(f,"eType: ");
	DBVprint_DbvEPromptIdType(f, &psEnumeratedPrompt->eType);

	/* o */
	fprintf(f,"psSuggestedString: ");
	DBVprint_inter_string(f, psEnumeratedPrompt->psSuggestedString);
}



void DBVprint_DbvEPromptIdType(FILE *f, DbvEPromptIdType *peType)
{
	switch (*peType) {
	case EPIT_GroupId:
		fprintf(f,"EPIT_GroupId\n");
		break;
	case EPIT_UserId:
		fprintf(f,"EPIT_UserId\n");
		break;
	case EPIT_Password:
		fprintf(f,"EPIT_Password\n");
		break;
	case EPIT_NewPassword:
		fprintf(f,"EPIT_NewPassword\n");
		break;
	case EPIT_Copyright:
		fprintf(f,"EPIT_Copyright\n");
		break;
	case EPIT_SessionId:
		fprintf(f,"EPIT_SessionId\n");
		break;
	case EPIT_Invalid:
		fprintf(f,"EPIT_Invalid\n");
		break;	
	}
}



void DBVprint_DbvPromptInfo(FILE *f, DbvPromptInfo *psPromptInfo)
{
	if (psPromptInfo == NULL) {
		fprintf(f,"NULL\n");
		return;
	}
	
	fprintf(f,"\n");

	fprintf(f,"ePromptInfoType: ");
	switch (psPromptInfo->ePromptInfoType) {
	case PIIT_Character:
		fprintf(f,"PIIT_Character\n");
		fprintf(f,"psCharacter: ");
		DBVprint_inter_string(f, psPromptInfo->uPromptInfo.psCharacter);
		break;

	case PIIT_Encrypted:
		fprintf(f,"PIT_NoneEnumeratedPrompt\n");
		fprintf(f,"psEncrypted: ");
		DBVprint_DbvEncryption(f, psPromptInfo->uPromptInfo.psEncrypted);
		break;

	case PIIT_Invalid:
		fprintf(f,"PIIT_Invalid\n");
		break;
	}
}



void DBVprint_DbvEncryption(FILE *f, DbvEncryption *psEncrypted)
{
	if (psEncrypted == NULL) {
		fprintf(f,"NULL\n");
		return;
	}

	fprintf(f,"\n");

	/* o */
	fprintf(f,"psCryptType: ");
	DBVprint_octet_string(f, psEncrypted->psCryptType);

	/* o */
	fprintf(f,"psCredential: ");
	DBVprint_octet_string(f, psEncrypted->psCredential);

	/* o */
	fprintf(f,"sData: ");
	DBVprint_octet_string(f, &psEncrypted->sData);
}



void DBVprint_DbvDataType(FILE *f, DbvDataType *peDataType)
{
	if (peDataType == NULL) {
		fprintf(f,"NULL\n");
		return;
	}
	
	fprintf(f,"\n");

	fprintf(f,"peDataType: ");

	switch (*peDataType) {
	case DTTT_Integer:
		fprintf(f,"DTTT_Integer\n");
		break;
	case DTTT_Date1:
		fprintf(f,"DTTT_Date1\n");
		break;
	case DTTT_Float:
		fprintf(f,"DTTT_Float\n");
		break;
	case DTTT_AlphaNumeric:
		fprintf(f,"DTTT_AlphaNumeric\n");
		break;
	case DTTT_UrlUrn:
		fprintf(f,"DTTT_UrlUrn\n");
		break;
	case DTTT_Boolean:
		fprintf(f,"DTTT_Boolean\n");
		break;
	case DTTT_Invalid:
		fprintf(f,"DTTT_Invalid\n");
		break;
	}
}



void DBVprint_DbvResponse_list(FILE *f, DbvResponse **ppsResponse)
{
	if (ppsResponse == NULL) {
		fprintf(f, "NULL\n");
		return;
	}


	while (*ppsResponse) {
		DBVprint_DbvResponse(f, *ppsResponse);
		ppsResponse++;
		fprintf(f,"\n");
	}
}


void DBVprint_DbvResponse(FILE *f, DbvResponse *psResponse)
{
	/* c */
	fprintf(f,"sPromptId: ");
	DBVprint_DbvPromptId(f, &psResponse->sPromptId);

	/* c */
	fprintf(f,"sPromptResponse: ");
	DBVprint_DbvPromptResponse(f, &psResponse->sPromptResponse);
}



void DBVprint_DbvPromptResponse(FILE *f, DbvPromptResponse *psPromptResponse)
{
	if (psPromptResponse == NULL) {
		fprintf(f,"NULL\n");
		return;
	}
	
	fprintf(f,"\n");

	fprintf(f,"eResponseType: ");
	switch (psPromptResponse->eResponseType) {
	case RT_String:
		fprintf(f,"RT_String\n");
		fprintf(f,"psString: ");
		DBVprint_inter_string(f, psPromptResponse->uResponse.psString);
		break;

	case RT_Accept:
		fprintf(f,"RT_Accept\n");
		fprintf(f,"fAccept: ");
		DBVprint_bool(f, &psPromptResponse->uResponse.fAccept);
		break;

	case RT_Acknowledge:
		fprintf(f,"RT_Acknowledge\n");
		break;

	case RT_Diagnostic:
		fprintf(f,"RT_Diagnostic\n");
		fprintf(f,"psDiagnostic: ");
		DBVprint_diag_rec(f, psPromptResponse->uResponse.psDiagnostic);
		break;

	case RT_Encrypted:
		fprintf(f,"RT_Encrypted\n");
		fprintf(f,"psEncryption: ");
		DBVprint_DbvEncryption(f, psPromptResponse->uResponse.psEncryption);
		break;
	}
}





/*******************************************************************/






/* A char is 8 bits (0 - 7) and can hold BOTH possitive and negative values,
   depending upon how the char is interpreted and used.
   Note that the C language does not specify whether variables of
   type char without pre specification are signed or unsigned 
   (K & R p. 43).

   Z39.50 has a variety of types including integers which are always in twos 
   complement form.
   snacc takes a char* array and decodes/encodes the array, casting/
   converting the array elements as necessary, it doesnt care if the
   array is signed or unsigned. Note that function DBVencode_pdu_to_bytes()
   in z3950_enc_dec.c returns a char** and function DBVdecode_bytes_to_pdu
   in the same file accepts a char* .

   If an unknown PDU/external is received, the data bytes making up the 
   unknown, a sequence of chars (remember that when received over a socket data
   is just received into a char* 'array'), which may represent possitive or 
   negative values of various structures/types are entered into the
   pcValue field of a DbvOctetString structure rather then being decoded


   The DbvOctetString structure in sitypes.h:

   typedef struct DbvOctetString {
   Ulong      ulLength;
   char      *pcValue;
   } DbvOctetString;


  Function DBVprint_as_hex() is used by the API test programs only to 
  print received but undecoded data. This function takes  
  takes a pcValue string and prints it as meaningless hex, the
  hex printed may represented multiple different structures/types which
  have not been decoded.

  DBVprint_as_hex() prints each 8 bit char as two ascii digits.

  Assuming ber_data[i] (a character without any signed or unsigned 
  specification) held the following
  10001111

  First charater print:
  ber_data[i] >> 4 = 00001000 on some machines (unsigned quantitity 
  or signed quantitiy logical ) 11111000 on others (signed quantity
  arithmetic ). Note that we dont know precisely which  will
  be done because the C language does not specify whether variables of
  type char are signed or unsigned quantities (K & R p. 43) and when
  they are signed if an arithmatic or logical  is done (K & R p.49).

  The exact  chosen doesnt matter however as the &0x0f in sub function 
  DBVto_hex() rips away the top 4 bits ( & 15) to leave 1000 = 8


  Second character print:
  ber_data[i] = 10001111
  The &0x0f in DBVto_hex() rips away the top 4 bits ( & 15) to leave 1111 = F 

  so the char holding the 8 bit value 10001111, which may represent
  143 or -112 is always printed as 8F
  
  Note that the (ber_data[i] >> 4) does not permanently modify ber_data[i].

  Although theoretically possible, negative data will be rare for strings
  as all ASCII set characters are possitive. 

*/


int DBVprint_as_hex(FILE *f, char *ber_data, unsigned long length)
{
	unsigned long i = 0;

	fprintf(f, "Hex: ");
	for(i = 0; i < length; i++)
		fprintf(f,"%c%c", DBVto_hex(ber_data[i] >> 4), DBVto_hex(ber_data[i]));

	/* Print a newline */
	fprintf(f, "\n");
}



/* ascii char set goes from 0 to 127 inclusive */
char DBVto_hex(char eightBits) 
{
 	char numToHexCharTblG[16] =
	{ '0', '1', '2', '3', '4', '5', '6', '7',
	  '8', '9', 'a', 'b', 'c', 'd' ,'e', 'f'};

	return numToHexCharTblG[(eightBits) & 0x0f];
}




void DBVprint_DbvPrintExtendedService_external(FILE *f, DbvASN1External *external_ptr)
{
	DBVprint_DbvPrintExtendedService(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psPrintExtendedService);
}



void DBVprint_DbvPrintExtendedService(FILE *f, DbvPrintExtendedService *psPrintExtendedService)
{

   /* Write the title out */ 
   if (psPrintExtendedService == NULL)
      {
      fprintf(f, "psPrintExtendedService: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psPrintExtendedService:\n");
   

   /* ePrintReqRsp */
   fprintf(f, "ePrintReqRsp: ");
   DBVprint_DbvPrintReqOrRsp(f, &psPrintExtendedService->ePrintReqRsp);


   /* uPrintReqOrRsp */
   switch( psPrintExtendedService->ePrintReqRsp )
      {
      case PROR_Request:
	 DBVprint_DbvPrintRequestPackage(f, psPrintExtendedService->uPrintReqOrRsp.psPrintRequest);
	 break;

      case PROR_Response:
	DBVprint_DbvPrintTaskPackage(f, psPrintExtendedService->uPrintReqOrRsp.psPrintResponse);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvPrintExtendedService() */



void DBVprint_DbvPrintReqOrRsp(FILE *f, DbvPrintReqOrRsp *pePrintReqRsp)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *pePrintReqRsp == PROR_Request)
         strcpy(temp, "PROR_Request");

      if( *pePrintReqRsp == PROR_Response)
         strcpy(temp, "PROR_Response");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvPrintReqOrRsp() */




void DBVprint_DbvPrintRequestPackage(FILE *f, DbvPrintRequestPackage *psPrintRequest)
{
	if (psPrintRequest == NULL) {
		fprintf(f,"psPrintRequest: NULL\n");
		return;
	}

	fprintf(f,"psPrintRequest:\n");

	/* o */
	DBVprint_DbvPrintOriginPartToKeep(f, psPrintRequest->psOriginPartToKeep);
}



void DBVprint_DbvPrintOriginPartToKeep(FILE *f, DbvPrintOriginPartToKeep *psOriginPartToKeep)
{

   /* Write the title out */ 
   if (psOriginPartToKeep == NULL)
      {
      fprintf(f, "psOriginPartToKeep: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psOriginPartToKeep:\n");
   

   /* ePrintOriginPartToKeepType */
   fprintf(f, "ePrintOriginPartToKeepType: ");
   DBVprint_DbvPrintOriginPartToKeepType(f, &psOriginPartToKeep->ePrintOriginPartToKeepType);


   /* uPrintOriginPartToKeep */
   switch( psOriginPartToKeep->ePrintOriginPartToKeepType )
      {
      case POPKT_Create:
	 DBVprint_DbvCreateOption(f, psOriginPartToKeep->uPrintOriginPartToKeep.psCreateOption);
				 
	 break;

      case POPKT_Delete:
	 DBVprint_DbvDeleteOption(f, psOriginPartToKeep->uPrintOriginPartToKeep.psDeleteOption);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvPrintOriginPartToKeep() */



void DBVprint_DbvPrintOriginPartToKeepType(FILE *f, DbvPrintOriginPartToKeepType *pePrintOriginPartToKeepType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *pePrintOriginPartToKeepType == POPKT_Create)
         strcpy(temp, "POPKT_Create");

      if( *pePrintOriginPartToKeepType == POPKT_Delete)
         strcpy(temp, "POPKT_Delete");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvPrintOriginPartToKeepType() */



void DBVprint_DbvCreateOption(FILE *f, DbvCreateOption *psCreateOption)
{
	if (psCreateOption == NULL) {
		fprintf(f, "psCreateOption: NULL\n");
		return;
	}

	fprintf(f, "psCreateOption:\n");


   /* eCreateOptionType */
   fprintf(f, "eCreateOptionType: ");
   DBVprint_DbvCreateOptionType(f, &psCreateOption->eCreateOptionType);


   /* uCreateOption */
   switch( psCreateOption->eCreateOptionType )
      {
      case COPT_ResultSetReference:
	 DBVprint_DbvResultSetReference(f, psCreateOption->uCreateOption.psResultSetReference);
				 
	 break;

      case COPT_AccessionReference:
	 DBVprint_DbvAccessionReference(f, psCreateOption->uCreateOption.psAccessionReference);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvCreateOption() */



void DBVprint_DbvCreateOptionType(FILE *f, DbvCreateOptionType *peCreateOptionType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peCreateOptionType == COPT_ResultSetReference)
         strcpy(temp, "COPT_ResultSetReference");

      if( *peCreateOptionType == COPT_AccessionReference)
         strcpy(temp, "COPT_AccessionReference");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvCreateOptionType() */



void DBVprint_DbvResultSetReference(FILE *f, DbvResultSetReference *psResultSetReference)
{
	if (psResultSetReference == NULL) {
		fprintf(f, "psResultSetReference: NULL\n");
		return;
	}

	fprintf(f, "psResultSetReference:\n");


	/* c */
   	fprintf(f, "sResultSetId: ");
   	DBVprint_inter_string(f, &psResultSetReference->sResultSetId);
	
	/* c */
	fprintf(f,"ulResultSetStartPoint: ");
	fprintf(f,"%lu\n", psResultSetReference->ulResultSetStartPoint);

	/* c */
	fprintf(f,"ulNumberOfRecordsRequested: ");
	fprintf(f,"%lu\n", psResultSetReference->ulNumberOfRecordsRequested);

	/* o */
	DBVprintN_DbvRange(f, "ppsAdditionalRanges", psResultSetReference->ppsAdditionalRanges);

	/* o */
	DBVprint_DbvRecordComposition(f, psResultSetReference->psRecordComposition);

	/* c */
	DBVprint_DbvAddress(f, "sAddress", &psResultSetReference->sAddress);
	
}



void DBVprint_DbvAddress(FILE *f, char *name, DbvAddress *psAddress)
{
	if (psAddress == NULL) {
		fprintf(f, "%s: NULL\n", name);
		return;
	}

	fprintf(f, "%s:\n", name);


	/* c */
   	fprintf(f, "sPostalAddress: ");
   	DBVprint_inter_string(f, &psAddress->sPostalAddress);
	

	/* o */
   	fprintf(f, "psEmailAddress: ");
   	DBVprint_inter_string(f, psAddress->psEmailAddress);

}




void DBVprint_DbvAccessionReference(FILE *f, DbvAccessionReference *psAccessionReference)
{
	if (psAccessionReference == NULL) {
		fprintf(f, "psAccessionReference: NULL\n");
		return;
	}

	fprintf(f, "psAccessionReference:\n");


	/* c */
   	fprintf(f, "sReferenceId: ");
   	DBVprint_inter_string(f, &psAccessionReference->sReferenceId);
	
	/* o */
	DBVprint_DbvRecordComposition(f, psAccessionReference->psRecordComposition);
	/* c */
   	DBVprint_DbvAddress(f, "sAddress", &psAccessionReference->sAddress);
}





void DBVprintN_DbvRange(FILE *f, char *name, DbvRange **ppsAdditionalRanges)
   {

   if( ppsAdditionalRanges != NULL)
      {
      fprintf(f, "%s:\n", name);
      while( ppsAdditionalRanges && *ppsAdditionalRanges )
         {
         DBVprint_DbvRange(f, "psDbvRange", *ppsAdditionalRanges);
         ppsAdditionalRanges++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "%s: NULL\n", name);
      
      } /* End else */

     
   } /* End DBVprintN_DbvRange() */




void DBVprint_DbvRange(FILE *f, char *name, DbvRange *psAdditionalRanges)
{

   /* Write the title out */ 
   if (psAdditionalRanges == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);
   

	/* c */
	fprintf(f,"ulStartingPosition: ");
	fprintf(f,"%lu\n", psAdditionalRanges->ulStartingPosition);

	/* c */
	fprintf(f,"ulNumberOfRecords: ");
	fprintf(f,"%lu\n", psAdditionalRanges->ulNumberOfRecords);
}




void DBVprint_DbvRecordComposition(FILE *f, DbvRecordComposition *psRecordComposition)
{

   /* Write the title out */ 
   if (psRecordComposition == NULL)
      {
      fprintf(f, "psRecordComposition: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psRecordComposition:\n");
   
	switch( psRecordComposition->eRecordCompositionType) {
		case RCT_ElementSetNames:
			DBVprint_el_set_names(f, psRecordComposition->uRecordComposition.psElementSetNames);
			break;
		case RCT_CompSpec:
			DBVprint_pres_comp_spec(f, psRecordComposition->uRecordComposition.psCompSpec);
			break;
	}
	
}



int DBVprint_pres_comp_spec(FILE *f, DbvCompSpec *psCompSpec)
{
      	fprintf(f,"psCompSpec:\n");

      	fprintf(f,"fSelectAlternativeSyntax: ");
	DBVprint_bool(f, &psCompSpec->fSelectAlternativeSyntax);

	DBVprint_DbvSpecification(f, "psGeneric", psCompSpec->psGeneric);

      	fprintf(f,"ppsDbSpecific:\n");
	if (psCompSpec->ppsDbSpecific != NULL)
		DBVprint_pres_dbspecific(f, psCompSpec->ppsDbSpecific);
	else
		fprintf(f,"NULL\n");


      	fprintf(f,"ppcRecordSyntax:\n");
	if (psCompSpec->ppcRecordSyntax != NULL)
		DBVprint_pres_recsyntax(f, psCompSpec->ppcRecordSyntax);
	else
		fprintf(f,"NULL\n");
}




int DBVprint_pres_dbspecific(FILE *f, DbvDbSpecificComp **ppsDbSpecific)
{
	int index = 0;

	if (ppsDbSpecific != NULL){
		while(ppsDbSpecific[index] != NULL) {
       			fprintf(f,"sDatabaseName: ");
			DBVprint_inter_string(f, &ppsDbSpecific[index]->sDatabaseName);
			DBVprint_DbvSpecification(f, "sSpecification", &ppsDbSpecific[index]->sSpecification);
			index++;
			fprintf(f,"\n");
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");
}



int DBVprint_pres_recsyntax(FILE *f, DbvObjectIdentifier *ppcRecordSyntax)
{
	int index = 0;

	if (ppcRecordSyntax != NULL) {
		while(ppcRecordSyntax[index] != NULL) {
       			fprintf(f,"\tpcRecordSyntax: ");
			fprintf(f,"%s\n", ppcRecordSyntax[index]);
			index++;
		}
		fprintf(f,"\n");
	} else
		fprintf(f,"NULL\n");
}





void DBVprint_DbvDeleteOption(FILE *f, DbvDeleteOption *psDeleteOption)
{
	if (psDeleteOption == NULL) {
		fprintf(f, "psDeleteOption: NULL\n");
		return;
	}

	fprintf(f, "psDeleteOption:\n");

	/* c */
   	fprintf(f, "sJobReference: ");
   	DBVprint_inter_string(f, &psDeleteOption->sJobReference);
	
}



void DBVprint_DbvPrintTaskPackage(FILE *f, DbvPrintTaskPackage *psPrintResponse)
{
	if (psPrintResponse == NULL) {
		fprintf(f,"psPrintResponse: NULL\n");
		return;
	}

	fprintf(f,"psPrintResponse:\n");

	/* o */
	DBVprint_DbvPrintOriginPartToKeep(f, psPrintResponse->psOriginPartToKeep);

	/* c */
	DBVprint_DbvPrintTargetPartToKeep(f, "sTargetPart", &psPrintResponse->sTargetPart);
}



void DBVprint_DbvPrintTargetPartToKeep(FILE *f, char *name, DbvPrintTargetPartToKeep *psTargetPart)
{
	if (psTargetPart == NULL) {
		fprintf(f, "%s: NULL\n", name);
		return;
	}

	fprintf(f, "%s:\n", name);

	/* c */
   	fprintf(f, "sJobReference: ");
   	DBVprint_inter_string(f, &psTargetPart->sJobReference);
	
	
}


void DBVprint_DbvAlertExtendedService_external(FILE *f, DbvASN1External *external_ptr)
{
	DBVprint_DbvAlertExtendedService(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psAlertExtendedService);
}



void DBVprint_DbvAlertExtendedService(FILE *f, DbvAlertExtendedService *psAlertExtendedService)
{

   /* Write the title out */ 
   if (psAlertExtendedService == NULL)
      {
      fprintf(f, "psAlertExtendedService: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psAlertExtendedService:\n");
   

   /* eAlertReqRsp */
   fprintf(f, "eAlertReqRsp: ");
   DBVprint_DbvAlertReqOrRsp(f, &psAlertExtendedService->eAlertReqRsp);


   /* uAlertReqOrRsp */
   switch( psAlertExtendedService->eAlertReqRsp )
      {
      case AROR_Request:
	 DBVprint_DbvAlertRequestPackage(f, psAlertExtendedService->uAlertReqOrRsp.psAlertRequest);
	 break;

      case AROR_Response:
	DBVprint_DbvAlertTaskPackage(f, psAlertExtendedService->uAlertReqOrRsp.psAlertResponse);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvAlertExtendedService() */



void DBVprint_DbvAlertReqOrRsp(FILE *f, DbvAlertReqOrRsp *peAlertReqRsp)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peAlertReqRsp == AROR_Request)
         strcpy(temp, "AROR_Request");

      if( *peAlertReqRsp == AROR_Response)
         strcpy(temp, "AROR_Response");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvAlertReqOrRsp() */



void DBVprint_DbvAlertRequestPackage(FILE *f, DbvAlertRequestPackage *psAlertRequest)
{
	if (psAlertRequest == NULL) {
		fprintf(f, "psAlertRequest: NULL\n");
		return;
	}

	fprintf(f, "psAlertRequest:\n");

	/* o */
	DBVprint_DbvAlertOriginPartToKeep(f, psAlertRequest->psOriginPartToKeep);

}



void DBVprint_DbvAlertOriginPartToKeep(FILE *f, DbvAlertOriginPartToKeep *psOriginPartToKeep)
{
	if (psOriginPartToKeep == NULL) {
		fprintf(f, "psOriginPartToKeep: NULL\n");
		return;
	}

	fprintf(f, "psOriginPartToKeep:\n");

	/* c */
	DBVprint_DbvInternationalString(f, "sAlertName", &psOriginPartToKeep->sAlertName);

	/* o */
	DBVprint_DbvAddress(f, "psAddress", psOriginPartToKeep->psAddress);
}



void DBVprint_DbvAlertTaskPackage(FILE *f, DbvAlertTaskPackage *psAlertResponse)
{
	if (psAlertResponse == NULL) {
		fprintf(f, "psAlertResponse: NULL\n");
		return;
	}

	fprintf(f, "psAlertResponse:\n");

	 /* o */
	DBVprint_DbvAlertOriginPartToKeep(f, psAlertResponse->psOriginPartToKeep);
	
}


void DBVprint_DbvCOrder_external(FILE *f, DbvASN1External *external_ptr)
{
	DBVprint_DbvCOrder(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psCOrder);
}



void DBVprint_DbvCOrder(FILE *f, DbvCOrder *psCOrder)
{

   /* Write the title out */ 
   if (psCOrder == NULL)
      {
      fprintf(f, "psCOrder: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psCOrder:\n");
   

   /* eCOrderType */
   fprintf(f, "eCOrderType: ");
   DBVprint_DbvCOrderType(f, &psCOrder->eCOrderType);


   /* uCOrder */
   switch( psCOrder->eCOrderType )
      {
      case COT_OrderRequest:
	 DBVprint_DbvOrderRequest(f, psCOrder->uCOrder.psOrderRequest);
	 break;

      case COT_OrderResponse:
	DBVprintN_DbvInternationalString(f, "ppsOrderResponse", psCOrder->uCOrder.ppsOrderResponse);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvCOrder() */



void DBVprint_DbvCOrderType(FILE *f, DbvCOrderType *peCOrderType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peCOrderType == COT_OrderRequest)
         strcpy(temp, "COT_OrderRequest");

      if( *peCOrderType == COT_OrderResponse)
         strcpy(temp, "COT_OrderResponse");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvCOrderType() */



void DBVprint_DbvOrderRequest(FILE *f, DbvOrderRequest *psOrderRequest)
{

   /* Write the title out */ 
   if (psOrderRequest == NULL)
      {
      fprintf(f, "psOrderRequest: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psOrderRequest:\n");
   

   /* eOrderRequestType */
   fprintf(f, "eOrderRequestType: ");
   DBVprint_DbvOrderRequestType(f, &psOrderRequest->eOrderRequestType);


   /* uOrderRequest */
   switch( psOrderRequest->eOrderRequestType )
      {
      case ORT_NormalOrder:
	 DBVprint_DbvNormalOrder(f, psOrderRequest->uOrderRequest.psNormalOrder);
	 break;

      case ORT_OrderRefs:
	DBVprintN_DbvInternationalString(f, "ppsOrderRefs", psOrderRequest->uOrderRequest.ppsOrderRefs);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvOrderRequest() */



void DBVprint_DbvOrderRequestType(FILE *f, DbvOrderRequestType *peOrderRequestType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peOrderRequestType == ORT_NormalOrder)
         strcpy(temp, "ORT_NormalOrder");

      if( *peOrderRequestType == ORT_OrderRefs)
         strcpy(temp, "ORT_OrderRefs");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvOrderRequestType() */



void DBVprint_DbvNormalOrder(FILE *f, DbvNormalOrder *psNormalOrder)
{

   /* Write the title out */ 
   if (psNormalOrder == NULL)
      {
      fprintf(f, "psNormalOrder: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psNormalOrder:\n");
   


	/* c */
	DBVprintN_DbvRange(f, "ppsOrderRanges", psNormalOrder->ppsOrderRanges);


	/* c */
	fprintf(f,"ulQuantity: ");
	fprintf(f,"%lu\n", psNormalOrder->ulQuantity);

}




void DBVprint_DbvCharSetAndLanguage_external(FILE *f, DbvASN1External *external_ptr)
{
        fprintf(f,"\tpsCharSetAndLanguage:\n");

	/* If not NULL */
	if (external_ptr->uExternalValue.psSingleASN1Type->uASN1.psExplainRecord) 
		DBVprint_DbvCharSetAndLanguage(f, external_ptr->uExternalValue.psSingleASN1Type->uASN1.psCharSetAndLanguage);
	
}



void DBVprint_DbvCharSetAndLanguage(FILE *f, DbvCharSetAndLanguage *psCharSetAndLanguage)
{

   /* Write the title out */ 
   if (psCharSetAndLanguage == NULL)
      {
      fprintf(f, "psCharSetAndLanguage: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psCharSetAndLanguage:\n");
   

   /* eCharSetAndLanguageType */
   fprintf(f, "eCharSetAndLanguageType: ");
   DBVprint_DbvCharSetAndLanguageType(f, &psCharSetAndLanguage->eCharSetAndLanguageType);


   /* uCharSetAndLanguage */
   switch( psCharSetAndLanguage->eCharSetAndLanguageType )
      {
      case CSALT_Proposal:
	 DBVprint_DbvOriginProposal(f, psCharSetAndLanguage->uCharSetAndLanguage.psProposal);
	 break;

      case CSALT_Response:
	DBVprint_DbvTargetResponse(f, psCharSetAndLanguage->uCharSetAndLanguage.psResponse);
	 break;

      default:
	 break;
      } /* End switch */

   } /* End DBVprint_DbvCharSetAndLanguage() */



void DBVprint_DbvCharSetAndLanguageType(FILE *f, DbvCharSetAndLanguageType *peCharSetAndLanguageType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peCharSetAndLanguageType == CSALT_Proposal)
         strcpy(temp, "CSALT_Proposal");

      if( *peCharSetAndLanguageType == CSALT_Response)
         strcpy(temp, "CSALT_Response");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvCharSetAndLanguageType() */



void DBVprint_DbvOriginProposal(FILE *f, DbvOriginProposal *psProposal)
{
   /* Write the title out */ 
   if (psProposal == NULL)
      {
      fprintf(f, "psProposal: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psProposal:\n");
   

	/* o */
	DBVprintN_DbvProposedCharSet(f, psProposal->ppsProposedCharSets);

	/* o */
	DBVprintN_DbvLanguageCode(f, psProposal->ppsProposedLanguages);
}



void DBVprintN_DbvProposedCharSet(FILE *f, DbvProposedCharSet **ppsProposedCharSets)
   {

   if( ppsProposedCharSets != NULL)
      {
      fprintf(f, "ppsProposedCharSets:\n");
      while( ppsProposedCharSets && *ppsProposedCharSets )
         {
         DBVprint_DbvProposedCharSet(f, *ppsProposedCharSets);
         ppsProposedCharSets++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "ppsProposedCharSets: NULL\n");
      
      } /* End else */
   } /* End DBVprintN_DbvProposedCharSet() */



void DBVprint_DbvProposedCharSet(FILE *f, DbvProposedCharSet *psProposedCharSet)
{
   /* Write the title out */ 
   if (psProposedCharSet == NULL)
      {
      fprintf(f, "psProposedCharSet: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psProposedCharSet:\n");
   
   /* eProposedCharSetType */
   fprintf(f, "eProposedCharSetType: ");
   DBVprint_DbvProposedCharSetType(f, &psProposedCharSet->eProposedCharSetType);

   /* uProposedCharSet */
   switch( psProposedCharSet->eProposedCharSetType )
      {
      case PRCST_Iso2022:
	 DBVprint_DbvIso2022(f, psProposedCharSet->uProposedCharSet.psIso2022);
	 break;

      case PRCST_Iso10646:
	DBVprint_DbvIso10646(f, psProposedCharSet->uProposedCharSet.psIso10646);
	 break;

      case PRCST_Private:
	DBVprint_DbvPrivateCharacterSet(f, psProposedCharSet->uProposedCharSet.psPrivate);
	 break;
      default:
	 break;
      } /* End switch */
   } /* End DBVprint_DbvProposedCharSet() */



void DBVprint_DbvProposedCharSetType(FILE *f, DbvProposedCharSetType *peProposedCharSetType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peProposedCharSetType == PRCST_Iso2022)
         strcpy(temp, "PRCST_Iso2022");

      if( *peProposedCharSetType == PRCST_Iso10646)
         strcpy(temp, "PRCST_Iso10646");

      if( *peProposedCharSetType == PRCST_Private)
         strcpy(temp, "PRCST_Private");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvProposedCharSetType() */



void DBVprint_DbvIso2022(FILE *f, DbvIso2022 *psIso2022)
{
   /* Write the title out */ 
   if (psIso2022 == NULL)
      {
      fprintf(f, "psIso2022: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psIso2022:\n");
   
   /* eIso2022Type */
   fprintf(f, "eIso2022Type: ");
   DBVprint_DbvIso2022Type(f, &psIso2022->eIso2022Type);

   /* uIso2022Type */
   switch( psIso2022->eIso2022Type )
      {
      case I2022T_OriginProposal:
	 DBVprint_DbvIso2022Proposal(f, psIso2022->uIso2022Type.psOriginProposal);
	 break;

      case I2022T_TargetResponse:
	DBVprint_DbvIso2022Response(f, psIso2022->uIso2022Type.psTargetResponse);
	 break;
      default:
	 break;
      } /* End switch */
   } /* End DBVprint_DbvIso2022() */



void DBVprint_DbvIso2022Type(FILE *f, DbvIso2022Type *peIso2022Type)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peIso2022Type == I2022T_OriginProposal)
         strcpy(temp, "I2022T_OriginProposal");

      if( *peIso2022Type == I2022T_TargetResponse)
         strcpy(temp, "I2022T_TargetResponse");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvIso2022Type() */



void DBVprint_DbvIso2022Proposal(FILE *f, DbvIso2022Proposal *psOriginProposal)
{
   /* Write the title out */ 
   if (psOriginProposal == NULL)
      {
      fprintf(f, "psOriginProposal: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psOriginProposal:\n");
   
	/* o */
	DBVprint_DbvEnvironment(f, "psProposedEnvironment", psOriginProposal->psProposedEnvironment);

	/* c */
	DBVprintN_Ulong(f, "ppulProposedSets", psOriginProposal->ppulProposedSets);

	/* c */
	DBVprintN_DbvInitialSet(f, psOriginProposal->ppsProposedInitialSets);

	/* c */
	DBVprint_DbvLeftAndRight(f, "psProposedLeftAndRight", psOriginProposal->psProposedLeftAndRight);
}



void DBVprint_DbvEnvironment(FILE *f, char *name, DbvEnvironment *psProposedEnvironment)
{
   if (psProposedEnvironment == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);

   /* eEnvironmentType */
   fprintf(f, "eEnvironmentType: ");
   DBVprint_DbvEnvironmentType(f, &psProposedEnvironment->eEnvironmentType);

}
   


void DBVprint_DbvEnvironmentType(FILE *f, DbvEnvironmentType *peEnvironmentType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peEnvironmentType == ENT_SevenBit)
         strcpy(temp, "ENT_SevenBit");

      if( *peEnvironmentType == ENT_EightBit)
         strcpy(temp, "ENT_EightBit");

         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvEnvironmentType() */



void DBVprint_DbvLeftAndRight(FILE *f, char *name, DbvLeftAndRight *psProposedLeftAndRight)
{

   if (psProposedLeftAndRight == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);

   /* eEnvironmentType */
	/* c */
	fprintf(f, "ulGLeft: ");
	DBVprint_Ulong(f, &psProposedLeftAndRight->ulGLeft);

	/* c */
	fprintf(f, "ulGRight: ");
	DBVprint_Ulong(f, &psProposedLeftAndRight->ulGRight);
}



void DBVprintN_DbvInitialSet(FILE *f, DbvInitialSet **ppsProposedInitialSets)
{

   if( ppsProposedInitialSets != NULL)
      {
      fprintf(f, "ppsProposedInitialSets:\n");

      while( ppsProposedInitialSets && *ppsProposedInitialSets )
         {
         DBVprint_DbvInitialSet(f, "psProposedInitialSets", *ppsProposedInitialSets);
         ppsProposedInitialSets++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "ppsProposedInitialSets: NULL\n");
      
      } /* End else */
   } /* End DBVprintN_DbvInitialSet() */



void DBVprint_DbvInitialSet(FILE *f, char *name, DbvInitialSet *psProposedInitialSets)
{

   if (psProposedInitialSets == NULL)
      {
      fprintf(f, "%s: NULL\n", name);
      return;
      } /* End if */

   fprintf(f, "%s:\n", name);

	/* c */
	fprintf(f, "ulG0: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulG0);
	
	/* c */
	fprintf(f, "ulG1: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulG1);

	/* c */
	fprintf(f, "ulG2: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulG2);

	/* c */
	fprintf(f, "ulG3: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulG3);

	/* c */
	fprintf(f, "ulC0: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulC0);

	/* c */	
	fprintf(f, "ulC1: ");
	DBVprint_Ulong(f, &psProposedInitialSets->ulC1);
}



void DBVprint_DbvIso2022Response(FILE *f, DbvIso2022Response *psTargetResponse)
{
   /* Write the title out */ 
   if (psTargetResponse == NULL)
      {
      fprintf(f, "psTargetResponse: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psTargetResponse:\n");
   
	/* o */
	DBVprint_DbvEnvironment(f, "psSelectedEnvironment", psTargetResponse->psSelectedEnvironment);

	/* c */
	DBVprintN_Ulong(f, "ppulSelectedSets", psTargetResponse->ppulSelectedSets);

	/* c */
	DBVprint_DbvInitialSet(f, "psSelectedInitialSet", psTargetResponse->psSelectedInitialSet);

	/* c */
	DBVprint_DbvLeftAndRight(f, "psSelectedLeftAndRight", psTargetResponse->psSelectedLeftAndRight);
}



void DBVprint_DbvIso10646(FILE *f, DbvIso10646 *psIso10646)
{

   /* Write the title out */ 
   if (psIso10646 == NULL)
      {
      fprintf(f, "psIso10646: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psIso10646:\n");
   
	/* c */
	DBVprint_DbvObjectIdentifier(f, "pcCollections", psIso10646->pcCollections);
	
	/* c */
	DBVprint_DbvObjectIdentifier(f, "pcEncodingLevel", psIso10646->pcEncodingLevel);
}



void DBVprint_DbvPrivateCharacterSet(FILE *f, DbvPrivateCharacterSet *psPrivate)
{
   /* Write the title out */ 
   if (psPrivate == NULL)
      {
      fprintf(f, "psPrivate: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psPrivate:\n");
   
   /* ePrivateCharacterSetType */
   fprintf(f, "ePrivateCharacterSetType: ");
   DBVprint_DbvPrivateCharacterSetType(f, &psPrivate->ePrivateCharacterSetType);

   /* uPrivateCharacterSet */
   switch( psPrivate->ePrivateCharacterSetType )
      {
      case PCST_ViaOid:
	 DBVprintN_DbvObjectIdentifier(f, "ppcViaOid", psPrivate->uPrivateCharacterSet.ppcViaOid);
	 break;

      case PCST_ExternallySpecified:
	DBVprint_DbvASN1External(f, psPrivate->uPrivateCharacterSet.psExternallySpecified);
	 break;
      default:
	 break;
      } /* End switch */
   } /* End DBVprint_DbvPrivateCharacterSet() */



void DBVprint_DbvPrivateCharacterSetType(FILE *f, DbvPrivateCharacterSetType *pePrivateCharacterSetType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *pePrivateCharacterSetType == PCST_ViaOid)
         strcpy(temp, "PCST_ViaOid");

      if( *pePrivateCharacterSetType == PCST_ExternallySpecified)
         strcpy(temp, "PCST_ExternallySpecified");

      if( *pePrivateCharacterSetType == PCST_PreviouslyAgreedUpon)
         strcpy(temp, "PCST_PreviouslyAgreedUpon");


         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvPrivateCharacterSetType() */




void DBVprintN_DbvLanguageCode(FILE *f, DbvLanguageCode **ppsProposedLangauages)
   {

   if( ppsProposedLangauages != NULL)
      {
      fprintf(f, "ppsProposedLanguages:\n");
      while( ppsProposedLangauages && *ppsProposedLangauages )
         {
         DBVprint_DbvLanguageCode(f, "psProposedLanguages", *ppsProposedLangauages);
         ppsProposedLangauages++;
         } /* End while */
      } /* End if */
   else
      {
      fprintf(f, "ppsProposedLanguages: NULL\n");
      
      } /* End else */
   } /* End DBVprintN_DbvLanguageCode() */



void DBVprint_DbvTargetResponse(FILE *f, DbvTargetResponse *psResponse)
{
   if (psResponse == NULL)
      {
      fprintf(f, "psResponse: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psResponse:\n");

	/* o */
	DBVprint_DbvSelectedCharSet(f, psResponse->psSelectedCharSets);

	/* o */
	fprintf(f, "psSelectedLanguage: ");
	DBVprint_DbvGeneralString(f, psResponse->psSelectedLanguage);
}



void DBVprint_DbvSelectedCharSet(FILE *f, DbvSelectedCharSet *psSelectedCharSets)
{

   /* Write the title out */ 
   if (psSelectedCharSets == NULL)
      {
      fprintf(f, "psSelectedCharSets: NULL\n");
      return;
      } /* End if */

   fprintf(f, "psSelectedCharSets:\n");
   
   /* eSelectedCharSetType */
   fprintf(f, "eSelectedCharSetType: ");
   DBVprint_DbvSelectedCharSetType(f, &psSelectedCharSets->eSelectedCharSetType);

   /* uSelectedCharSet */
   switch( psSelectedCharSets->eSelectedCharSetType )
      {
      case SCST_Iso2022:
	 DBVprint_DbvIso2022(f, psSelectedCharSets->uSelectedCharSet.psIso2022);
	 break;

      case SCST_Iso10646:
	DBVprint_DbvIso10646(f, psSelectedCharSets->uSelectedCharSet.psIso10646);
	 break;

      case SCST_Private:
	DBVprint_DbvPrivateCharacterSet(f, psSelectedCharSets->uSelectedCharSet.psPrivate);
	 break;

      default:
	 break;
      } /* End switch */
   } /* End DBVprint_DbvPrivateCharacterSet() */



void DBVprint_DbvSelectedCharSetType(FILE *f, DbvSelectedCharSetType *peSelectedCharSetType)
   {

   char temp[ STRING_LEN ];
      temp[ 0 ] = '\0';

      if( *peSelectedCharSetType == SCST_Iso2022)
         strcpy(temp, "SCST_Iso2022");

      if( *peSelectedCharSetType == SCST_Iso10646)
         strcpy(temp, "SCST_Iso10646");

      if( *peSelectedCharSetType == SCST_Private)
         strcpy(temp, "SCST_Private");

      if( *peSelectedCharSetType == SCST_None)
         strcpy(temp, "SCST_None");


         fprintf(f, "%s\n", temp);

   } /* End DBVprint_DbvSelectedCharSetType() */



